define([
		"utils/ShapeUtils",
		"utils/Observable",
		"underscore",
		"jquery",
		"jqueryUI"
	],
	function (
		ShapeUtils,
		Observable,
		_,
		$) {
		"use strict";

		/**
		 * Component for the WorkflowEditor, popup with a header, close button and a content
		 * area. The popup is draggable. Needs to be extended by a specific implementation.
		 * The popup is hidden by default.
		 *
		 * @exports DraggablePopup
		 * @class
		 *
		 * @param {JQueryObj} context - elements it adds itself to
		 */
		var DraggablePopup = function (context) {
			Observable.mixin(this, "draggablePopup");

			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._context = context;

			/**
			 * @member {Boolean} indicates if the popup is currently being hovered
			 * @private
			 */
			this._isHovered = false;

			/**
			 * @member {Boolean} indicates if the popup is being moved from its start position
			 * 						by JQuery UI draggable.
			 * @private
			 */
			this._isMoved = false;

			/**
			 * @member {Boolean} if a popup is sticky it doesn't hide on mouseout.
			 * The popup becomes sticky by clicking on it.
			 * @private
			 */
			this._isSticky = false;

			/**
			 * @member {Boolean} is the popup visible at this moment.
			 * @private
			 */
			this._isVisible = false;

			/**
			 * @member {JQueryObj} main element for the popup
			 * @private
			 */
			this._element = $("<div>").addClass("draggablePopup");

			/**
			 * @member {JQueryObj} header element, contains the title and the close button
			 * @private
			 */
			this._header = $("<div>").addClass("popupHeader");

			/**
			 * @member {JQueryObj} content of the popup
			 * @private
			 */
			this._content = $("<div>").addClass("popupContent");

			/**
			 * @member {JQueryObj} inline title element, is added to the header
			 * @private
			 */
			this._title = $("<span>");

			/**
			 * @member {JQueryObj} close button, used to close the popup
			 * @private
			 */
			this._closeBtn = $("<span>").text("X").addClass("closeBtn");
			this._closeBtn.hide(); //hide button by default, only visible when the popup has been moved

			//construct the correct DOM structure for the popup
			this._header.append(this._title);
			this._header.append(this._closeBtn);
			this._element.append(this._header);
			this._element.append(this._content);
			this._context.append(this._element);

			//make the element draggable, JQuery UI draggable is used
			this._element.draggable({
				handle: this._header,
				start: _.bind(this._draggableStart, this),
				stop: _.bind(this._draggableStop, this),
				containment: this._context
			});
			//hide the popup by default
			this.hide();
			//register the correct mouse listeners to the popup
			this._registerListeners();


			this._deferedCloseTimer = null;
		};

		/**
		 * Executed when dragging started
		 */
		DraggablePopup.prototype._draggableStart = function () {
			this._setMoved(true);
			this._closeBtn.show();
			this._startedMoving();
			this._emit("startMoving");
		};

		/**
		 * Executed when dragging stopped
		 */
		DraggablePopup.prototype._draggableStop = function () {
			this._stoppedMoving();
			this._emit("stopMoving");
		};

		/**
		 * Registers mouselisteners for the close button and the element
		 * @private
		 */
		DraggablePopup.prototype._registerListeners = function () {
			this._closeBtn.on("click", _.bind(function (e) {
				this._setHovered(false);
				this._setMoved(false);
				this._setSticky(false);
				this._closeBtn.hide();
				this._closed();
				this._emit("closed");
				this.hide(true);
				e.stopPropagation();
			}, this));

			this._element.on("click", _.bind(function (e) {
				this.setSticky(true);
				this._closeBtn.show();
			}, this));

			this._element.on("mouseenter", _.bind(function () {
				this._setHovered(true);
				this.cancelDeferedClose();
			}, this));
			this._element.on("mouseleave", _.bind(function () {
				this._setHovered(false);
			}, this));
			this._element.on("mousedown", function (e) {
				e.stopPropagation();
			});

		};
		/**
		 * Internal setter for hovered
		 * @param {Boolean} isHovered
		 */
		DraggablePopup.prototype._setHovered = function (isHovered) {
			if (this._isHovered !== isHovered) {
				this._isHovered = isHovered;
				this._emit("changed.hover", isHovered);
			}
		};

		/**
		 * Internal setter for moved
		 * @param {Boolean} isMoved
		 */
		DraggablePopup.prototype._setMoved = function (isMoved) {
			if (this._isMoved !== isMoved) {
				this._isMoved = isMoved;
				this._emit("changed.moved", isMoved);
			}
		};

		/**
		 * Internal setter for sticky
		 * @param {Boolean} isSticky
		 */
		DraggablePopup.prototype._setSticky = function (isSticky) {
			if (this._isSticky !== isSticky) {

				this._isSticky = isSticky;
				this._emit("changed.sticky", isSticky);
			}
		};

		/**
		 * Internal setter for visible
		 * @param {Boolean} isVisible
		 */
		DraggablePopup.prototype._setVisible = function (isVisible) {
			if (this._isVisible !== isVisible) {
				this._isVisible = isVisible;
				this._emit("changed.visible", isVisible);
			}
		};


		/**
		 * Is the popup sticky?
		 * @returns {Boolean}
		 */
		DraggablePopup.prototype.isSticky = function () {
			return this._isSticky;
		};

		/**
		 * Make the popup sticky or not
		 * @param {Boolean} sticky
		 */
		DraggablePopup.prototype.setSticky = function (sticky) {
			this._setSticky(sticky);
		};

		/**
		 * Is the popup beign hovered at this moment
		 * @returns {Boolean}
		 */
		DraggablePopup.prototype.isHovered = function () {
			return this._isHovered;
		};

		/**
		 * Show the popup.
		 */
		DraggablePopup.prototype.show = function () {
			this.cancelDeferedClose();
			this._element.show();
			this._setVisible(true);

		};

		/**
		 * Hide the popup, does not hide it if the popup is moved or hovered or sticky.
		 * Use parameter to force hide the popup.
		 * @param {Boolean} forceHide
		 */
		DraggablePopup.prototype.hide = function (forceHide) {
			if ((!this._isHovered && !this._isMoved && !this._isSticky) || forceHide) {
				this._element.hide();
				this._setVisible(false);
			}
		};

		/**
		 * Set the title in the popup.
		 * @param {String} title
		 */
		DraggablePopup.prototype.setTitle = function (title) {
			this._title.text(title);
		};

		/**
		 * Is the popup moved.
		 * @returns {Boolean}
		 */
		DraggablePopup.prototype.isMoved = function () {
			return this._isMoved;
		};

		/**
		 * Is the popup visible.
		 * @returns {Boolean}
		 */
		DraggablePopup.prototype.isVisible = function () {
			return this._isVisible;
		};

		/**
		 * Get the location of the popup, returns the distance from the top left corner of the context element.
		 * @returns {ShapeUtils.Point}
		 */
		DraggablePopup.prototype.getLocation = function () {
			var offset = this._element.offset(); //distance from the top left of the page
			var parentOffset = this._context.offset(); //distance of the context top left to the page top left corner
			var width = this._element.width();
			var height = this._element.height();
			return new ShapeUtils.Point(offset.left - parentOffset.left + width / 2, offset.top - parentOffset.top + height / 2);
		};

		/**
		 * The popup started moving around by user interaction.
		 * @private
		 */
		DraggablePopup.prototype._startedMoving = function () {
			//implement by specific popup to handle moving start	
		};

		/**
		 * The popup stopped to move around by user interaction.
		 * @private
		 */
		DraggablePopup.prototype._stoppedMoving = function () {
			//implement by specific popup to handle moving stop	
		};

		/**
		 * The popup was closed by using the close button on the popup.
		 * @private
		 */
		DraggablePopup.prototype._closed = function () {
			//implement by specific popup to hanl	
		};

		/**
		 * Destroys all resources of this popup
		 */
		DraggablePopup.prototype.destroyPopup = function () {
			this._element.remove();
		};

		/**
		 * Close the popup in after a delay
		 * cancelable by calling cancelDeferedClose()
		 */
		DraggablePopup.prototype.deferredClose = function () {
			this.cancelDeferedClose();
			this._deferedCloseTimer = setTimeout(_.bind(function () {
				this.hide();
			}, this), 500);
		};

		/**
		 * Cancel the ongoing defered close operation
		 */
		DraggablePopup.prototype.cancelDeferedClose = function () {
			clearTimeout(this._deferedCloseTimer);
			this._deferedCloseTimer = null;
		};


		return DraggablePopup;

	});