define(
	[
		"jquery",
		"underscore"
	],
	function (
		$,
		_
	) {


		"use strict";

		/**
		 * An inputfield that resizes itself according to its contents.
		 * @class
		 * @exports InputField
		 * 
		 * @param {JQueryObj} context - element to which to inputfield will be appended
		 * @param {Number} maxWidth - The maximum width of the input field
		 *                          @param {String} textAlign - text alignment of the inputfield
		 */
		var InputField = function (context, maxWidth, textAlign) {

			/**
			 * @member {String}
			 * @private
			 */
			this._textAlign = textAlign;

			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._context = context;

			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._element = $("<div></div>");

			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._inputElement = $("<textarea />");

			this._element.addClass("inputField");

			this._context.append(this._element);
			this._element.append(this._inputElement);

			/**
			 * @member {Number}
			 * @private
			 */
			this._maxWidth = maxWidth;

			this._registerListeners();

			// Make sure no scroll bar is shown
			this._inputElement.css({
				resize: "none",
				"overflow": "hidden",
				"text-align": textAlign
			});

			// resize the field on input
			this._inputElement.on("input", _.bind(function (e) {
				this._resize();
			}, this));


		};

		/**
		 * show the input
		 */
		InputField.prototype.show = function () {
			this._element.show();
			this._resize();
		};

		/**
		 * hide the input
		 */
		InputField.prototype.hide = function () {
			this._element.hide();
		};

		/**
		 * adds a function to the change event
		 * @param {Function} callback
		 */
		InputField.prototype.onChange = function (callback) {
			this._inputElement.on("change", callback);
		};

		/**
		 * returns the value of the input field
		 * @returns {String}
		 */
		InputField.prototype.getText = function () {
			return this._inputElement.val();
		};

		/**
		 * sets the value of the input field
		 * @param {String} text
		 */
		InputField.prototype.setText = function (text) {
			this._inputElement.val(text);
			this._resize();
		};

		/**
		 * register all listeners on the input element
		 */
		InputField.prototype._registerListeners = function () {
			this._inputElement.on("mousedown mouseup mousemove click dblclick mouseenter", _.bind(function (e) {
				e.stopPropagation();
			}, this));
		};

		/**
		 * resize the input field
		 */
		InputField.prototype._resize = function () {
			// To get the width of the text, we fill in a div with the text and get the width of the div
			var testDiv = $('<div></div>');
			testDiv.hide();
			testDiv.text(this._inputElement.val());
			$("body").append(testDiv);
			var elementWidth = Math.max(3, Math.min(testDiv.width(), this._maxWidth)) + 2;

			this._inputElement.width(elementWidth + "px");
			this._inputElement.height("1px");

			// set the height of the input field
			this._inputElement.height((this._inputElement.get(0).scrollHeight) + "px");

			var location = this._getLocation();

			if (this._textAlign === "center") {
				this._element.css({
					left: location.x - elementWidth / 2,
					top: location.y
				});
			} else if (this._textAlign === "right") {
				this._element.css({
					left: location.x - elementWidth,
					top: location.y
				});
			}

			testDiv.remove();
		};

		/**
		 * gets the location of the input field
		 */
		InputField.prototype._getLocation = function () {
			// should be implemented by a specific input field
		};

		/**
		 * destroys the element
		 */
		InputField.prototype.destroy = function () {
			this._element.remove();
		};

		/**
		 * gives focus to the input element
		 */
		InputField.prototype.focus = function () {
			this._inputElement.focus();
			this._inputElement.select();
		};

		/**
		 * adds event handler to the input element
		 * @param {String} eventName 
		 * @param {Function} callback  
		 */
		InputField.prototype.on = function (eventName, callback) {
			this._inputElement.on(eventName, callback);
		};

		/**
		 * removes event handler from the input element
		 * @param {String} eventName 
		 * @param {Function} callback  
		 */
		InputField.prototype.off = function (eventName, callback) {
			this._inputElement.off(eventName, callback);
		};



		return InputField;

	});
