define([
	"utils/Observable",
	"component/button/Button",
	"underscore",
	"utils/ShapeUtils",
	"jquery",
	"jqueryUI"],
	function (
		Observable,
		Button,
		_,
		ShapeUtils,
		$) {
		"use strict";

		/**
		 * Popup component for Workflow Editor
		 * @exports Popup
		 * @class
		 *            
		 * @mixes {Observable}
		 * @param {String}     name       - Name of the popup
		 * @param {JQueryObj} containment - Restrict (and add) the popup to this html element	
 		 * @param {String}     [cssClass]   - CSS Class to be added to the element
		 */
		var Popup = function (name, containment, cssClass) {
			Observable.mixin(this, "popup" + name);

			/**
			 * @member {JQueryObj} containment of the popup
			 */
			this._containment = containment;

			/**
			 * @member {JQueryObj} popup main element
			 * @private
			 */
			this.element = $("<div/>").addClass("wfeditor-popup hidden").addClass(cssClass);

			/**
			 * @member {JQueryObj} popup header element
			 * @private
			 */
			this._header = $("<div/>").addClass("wfeditor-popup-header").appendTo(this.element);

			/**
			 * @member {Button} popup close button
			 * @private
			 */
			this._closeBtn = new Button(null, null, false, "closePopup", "", false, _.bind(this._closeButtonPressed, this));
			this._closeBtn.addClass("wfeditor-popup-close");
			this._closeBtn.setContent($("<i>").addClass("icon icon-WCR_Close12"));
			this._header.append(this._closeBtn.element);

			/**
			 * @member {JQueryObj} popup title element
			 * @private
			 */
			this._headerTitle = $("<h3>").appendTo(this._header);

			/**
			 * @member {JQueryObj} popup content element, in here all content should be put!
			 */
			this.content = $("<div/>").addClass("wfeditor-popup-content").appendTo(this.element);

			/**
			 * @member {JQueryObj} popup footer element
			 * @private
			 */
			this._footer = $("<div/>").addClass("wfeditor-popup-footer").appendTo(this.element);


			/**
			 * @member {Boolean} was the popup resized by the user
			 */
			this._isResizedByUser = false;


			// Add popup to DOM and initialized draggable/resizable functionality
			this._containment.append(this.element);
			this._initializeResizableDraggable();

			var fitContentBound = _.bind(function (e, scrollDown) {
				this.fitContent(scrollDown);
			}, this);
			this.content.on("fitcontent", _.debounce(fitContentBound, 10));

		};

		/**
		 * Initialize the Resizable and draggable functionality
		 * @param {Integer} [minWidth = 200] - the minimum width of the popup
		 * @param {Integer} [minHeight = 150] - the minimum height of the popup
		 */
		Popup.prototype._initializeResizableDraggable = function (minWidth, minHeight) {
			// Resizable
			this.element.resizable({
				handles: "all",
				autoHide: true,
				minWidth: minWidth || 200,
				minHeight: minHeight || 150,
				containment: this._containment,
				start: _.bind(function (event, ui) {
					this._emit("resize.start");
					this._isResizedByUser = true;
					event.stopPropagation();
				}, this),
				resize: _.throttle(_.bind(function (event, ui) {
					this._emit("resize.change");
					event.stopPropagation();
				}, this), 10),
				stop: _.bind(function (event, ui) {
					this._emit("resize.stop");
					event.stopPropagation();
				}, this)
			});

			// Draggable
			this.element.draggable({
				containment: this._containment,
				handle: this._headerTitle,
				start: _.bind(function (event, ui) {
					this._emit("drag.start");
				}, this),
				stop: _.bind(function (event, ui) {
					this._emit("drag.stop");
					event.stopPropagation();
				}, this),
				drag: _.bind(function (event, ui) {
					this._emit("drag.move");
				}, this)
			});
		};

		/**
		 * Destroy the popup and its associated DOM elements
		 */
		Popup.prototype.destroy = function () {
			this.element.remove();
			this._destroyObservable();
		};

		/**
		 * Show the popup
		 * 
		 * @param {ShapeUtils.Point} [location] - location of the popup (relative to (0,0) of containment)
		 * @param {Boolean}          [centered=false] - should the popup be centered on the given location or not?
		 */
		Popup.prototype.show = function (location, centered) {
			this.element.removeClass("hidden");


			this.fitContent();

			if (location) {
				this.setLocation(location, centered, true);
			}


			this._setVisible(true);
		};
		/**
		 * Set the popup visibility flag
		 * @private
		 * @param {Boolean} isVisible
		 */
		Popup.prototype._setVisible = function (isVisible) {
			if (this._isVisible !== isVisible) {
				this._isVisible = isVisible;
				this._emit("changed.visible", isVisible);
			}
		};

		/**
		 * Is the popup visible?
		 * @returns {Boolean} 
		 */
		Popup.prototype.isVisible = function () {
			return this._isVisible;
		};

		/**
		 * Hide the popup
		 * (note, this doesn't destroy the popup DOM elements)
		 */
		Popup.prototype.hide = function () {
			this.element.addClass("hidden");
			this._setVisible(false);
		};

		/**
		 * Focus the popup
		 */
		Popup.prototype.focus = function () {
			// do nothing
		};

		/**
		 * Change the location of the popup
		 * @param {ShapeUtils.Point} location              - location of the popup (relative to (0,0) of containment)
		 * @param {Boolean}          [centered=false]        - should the popup be centered on the given location or not
		 * @param {Boolean}          [ignoreFitConten=false] - if false (or undefined) fit content is executed
		 */
		Popup.prototype.setLocation = function (location, centered, ignoreFitContent) {
			var top = location.y;
			var left = location.x;

			if (!ignoreFitContent) {
				this.doFitContent();
			}

			if (centered) {
				top = Math.max(0, location.y - (this.element.height() / 2));
				left = Math.max(0, location.x - (this.element.width() / 2));
			}

			if (top + this.element.height() > this._containment.innerHeight()) {
				top -= top + this.element.height() - this._containment.innerHeight();
			}

			if (left + this.element.width() > this._containment.innerWidth()) {
				left -= left + this.element.width() - this._containment.innerWidth();
			}

			this.element.css({
				top: Math.round(Math.max(top, 0)) + "px",
				left: Math.round(Math.max(left, 0)) + "px"
			});

			this._emit("changed.location");
		};

		/**
		 * Change the title of the popup
		 * @param {String} title - title of the popup
		 */
		Popup.prototype.setTitle = function (title) {
			this._headerTitle.text(title);
		};

		/**
		 * The close button was pressed
		 * @protected
		 */
		Popup.prototype._closeButtonPressed = function () {
			this.hide();
			this._emit("closed");
		};

		/**
		 * Call this method to make the popup fit its contents
		 */
		Popup.prototype.fitContent = function (scrollDown) {
			if (!this._isResizedByUser) {
				this._doFitContent(scrollDown);
				_.defer(_.bind(this._doFitContent, this, scrollDown));
			}
		};

		/**
		 * Do the exact calculations to fit the popup around its contents
		 */
		Popup.prototype._doFitContent = function (scrollDown) {
			// note there is a problem when elements do word wrap when the window is popup/contentdiv is small
			// the calculation of the popup size will be too big in this case we might need a fix for this in the future
			// for now borders are taken into account in this method, this is the easiest/fastest solution at the moment...
			// it might need to change in the future

			var height = Math.min(this._containment.innerHeight(), this.content.get(0).scrollHeight + this._header.outerHeight(true) + this._footer.outerHeight(true) + 4);
			var width = Math.min(this._containment.innerWidth(), this.content.get(0).scrollWidth + 2);
			this.element.css({
				width: width + "px",
				height: height + "px"
			});
			
			if(scrollDown){
				this.content[0].scrollTop = this.content[0].scrollHeight;
			}
		};

		/**
		 * Get the center of the popup
		 * @returns {ShapeUtils.Point} Center of the popup relative to (0,0) of containment element
		 */
		Popup.prototype.getCenter = function () {
			var position = this.element.position();
			return new ShapeUtils.Point(position.left + this.element.width() / 2, position.top + this.element.height() / 2);
		};

		return Popup;
	});