define(["underscore",
		"WFEditor/toolbar/Toolbar",
		"component/dropdownmenu/DropdownMenu",
		"utils/webi18n!WFEditorWCR",
		"utils/Observer"
	],
	function (_,
		Toolbar,
		DropdownMenu,
		i18n,
		Observer) {

		"use strict";

		/**
		 * View of the context menu used in the workflow editor
		 * 
		 * @class
		 * @exports ContextMenu
		 * @augments Toolbar
		 * 
		 * @param {ContextMenuModel} contextMenuModel 
		 * @param {Scene}            scene            
		 * @param {Boolean}          isMobile         
		 */
		var ContextMenu = function (contextMenuModel, scene, isMobile) {
			Toolbar.call(this, contextMenuModel, scene.getContext());
			Observer.mixin(this);

			/**
			 * @member {DropdownMenu}
			 * @private
			 */
			this._dropdown = new DropdownMenu(i18n, scene.getContext(), isMobile, "bottomLeft", "wf-context-menu wcr-dropdown", undefined, ["bottomLeft", "right", "bottom", "left", "top"]);
			this._dropdown.disableAutoHide();
			this._observe(this._dropdown, "initialized", function () {
				this._dropdown.popover.getElement().on("contextmenu", function (e) {
					e.preventDefault();
				});
				
				// prevent scene from losing focus on mousedown
				this._dropdown.popover.getElement().on("mousedown", function (e) {
					e.preventDefault();
				});
			});

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = scene;

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._contextMenuDiv = $("<div/>").css({
				position: "relative",
				width: 0,
				height: 0
			});


		};

		ContextMenu.prototype = Object.create(Toolbar.prototype);
		ContextMenu.prototype.constructor = ContextMenu;


		/**
		 * @override
		 */
		ContextMenu.prototype._addComponent = function (componentId) {
			var dropdownMenuGroup = {
				name: componentId
			};
			this._dropdown.addGroup(dropdownMenuGroup);
		};

		/**
		 * Add a context menu item to the view
		 * @override
		 */
		ContextMenu.prototype._addButton = function (componentId, buttonId) {
			var buttonData = this._model.getButtonData(buttonId);

			// action object for the dropdown menu
			var action = {};
			action.action = buttonData.action;
			action.name = buttonData.id;
			action.icon = buttonData.icon;
			action.content = buttonData.name;
			action.enabled = buttonData.enabled;
			action.isVisible = buttonData.visible;
			action.group = componentId;
			this._dropdown.addAction(action);
		};

		/**
		 * show context menu
		 */
		ContextMenu.prototype.show = function () {
			// Check if there are buttons visible.
			if (_.some(this._model.getAllComponents(), function (component) {
					return _.some(component.buttons, function (button) {
						var buttonData = this._model.getButtonData(button);
						return buttonData.visible;
					}, this);
				}, this)) {
				var location = this._model.getContextMenuSceneLocation();
				this._scene.getContext().append(this._contextMenuDiv.css({
					top: location.y,
					left: location.x
				}));
				this._dropdown.show(this._contextMenuDiv);
				this._dropdown.menu.find(".last-element").removeClass("last-element");
				// :last psuedoselector doesn't exist in css but it does work in jquery, so use it to find the last element and add a class
				this._dropdown.menu.find(".group li:not([style='display:none']):last").addClass("last-element");

			}

		};

		/** 
		 * hide context menu
		 */
		ContextMenu.prototype.hide = function () {
			this._dropdown.hide();
		};

		/**
		 * update the state of a menu item
		 * @param {String} buttonId - id of the menu item
		 */
		ContextMenu.prototype._updateButton = function (buttonId) {
			var buttonData = this._model.getButtonData(buttonId);
			var componentData = this._model.getComponent(buttonData.componentId);
			this._dropdown.setActionEnabled(buttonData.id, buttonData.enabled);
			if (!buttonData.visible) {
				this._dropdown.hideElement(buttonData.id);
			} else {
				this._dropdown.showElement(buttonData.id);

			}
		};

		return ContextMenu;

	});
