define(["underscore", "WFEditor/toolbar/ToolbarModel"], function (_, ToolbarModel) {

	"use strict";

	/**
	 * @class
	 * @augments ToolbarModel
	 * @exports ContextMenuModel
	 */
	var ContextMenuModel = function () {
		ToolbarModel.call(this);

		/**
		 * @member {HitArea} the hitArea underneath the contextmenu
		 * @private
		 */
		this._hitArea = null;

		/**
		 * @member {ShapeUitls.Point} the location of the context menu in scene coordinates
		 * @private 
		 */
		this._sceneLocation = null;

		/**
		 * @member {ShapeUtils.Point}the location of the contetx menu in world coordinates
		 * @private
		 */
		this._worldLocation = null;

	};

	ContextMenuModel.prototype = Object.create(ToolbarModel.prototype);
	ContextMenuModel.prototype.constructor = ContextMenuModel;


	/**
	 * Add a button to a toolbar component, adds the toolbar component if it doesn't exist yet
	 * @param {String} componentId
	 * @param {String} buttonId
	 * @param {String} name
	 * @param {String} icon
	 * @param {Function} action
	 * @param {Boolean} isEnabled
	 * @param {Boolean} isActive
	 */
	ContextMenuModel.prototype.addButton = function (componentId, buttonId, name, icon, action, isEnabled, isActive, isVisible) {
		var component = this._toolbarItems.getItemByKey(componentId);
		if (component) {
			var buttonData = {
				id: buttonId,
				icon: icon,
				action: action,
				enabled: isEnabled,
				active: isActive,
				visible: typeof isVisible === "undefined" ? true : isVisible,
				name: name,
				componentId: componentId
			};
			component.buttons.push(buttonId);
			this._buttons[buttonId] = buttonData;
		}
		this._emit("buttonAdded", componentId, buttonId);
	};

	/**
	 * sets the scene location of the context menu
	 * @param {ShapeUtils.Point} sceneLocation 
	 */
	ContextMenuModel.prototype.setContextMenuSceneLocation = function (sceneLocation) {
		this._sceneLocation = sceneLocation;
		this._emit("changed.location.scene");
	};

	/**
	 * sets the world location of the context menu
	 * @param {ShapeUtils.Point} worldLocation 
	 */
	ContextMenuModel.prototype.setContextMenuWorldLocation = function (worldLocation) {
		this._worldLocation = worldLocation;
		this._emit("changed.location.world");

	};

	/**
	 * gets the scene location of the context menu
	 */
	ContextMenuModel.prototype.getContextMenuSceneLocation = function () {
		return this._sceneLocation;
	};

	/**
	 * gets the world location of the context menu
	 */
	ContextMenuModel.prototype.getContextMenuWorldLocation = function () {
		return this._worldLocation;
	};

	/**
	 * gets the hitArea underneath the context menu
	 */
	ContextMenuModel.prototype.getHitArea = function () {
		return this._hitArea;
	};

	/**
	 * sets the hitArea
	 * @param {HitArea} hitArea 
	 */
	ContextMenuModel.prototype.setHitArea = function (hitArea) {
		if (this._hitArea !== hitArea) {
			this._hitArea = hitArea;
			this._emit("changed.hitArea", hitArea);
		}
	};


	/**
	 * Change the visible state of this button.
	 * @param {String} buttonId
	 * @param {Boolean} isVisible - should the button be visible
	 */
	ContextMenuModel.prototype.setButtonVisible = function (buttonId, isVisible) {
		var button = this._buttons[buttonId];
		if (button && button.visible !== isVisible) {
			button.visible = isVisible;
			this._emit("buttonStateChanged", buttonId);
		}
	};

	return ContextMenuModel;
});
