define(["utils/UndoRedoController",
		"WFEditor/tools/ToolConstants",
		"utils/Command",
		"utils/Observer",
		"underscore"
	],
	function (UndoRedoController,
		ToolConstants,
		Command,
		Observer,
		_) {
		"use strict";

		/**
		 * This is the UNDO/REDO controller for the WFEditor.
		 * It adds undo/redo buttons to the topbar of the editor
		 * and listens to Ctrl+z, Ctrl+Shift+Z and Ctrl+y keyboard short cuts
		 * @exports WFEditorUndoRedoController
		 * @class
		 *
		 * @param {WFEditorContext} wFEditorContext
		 * @pram {Function} [onWorkflowModifiedFlagChanged] - called whenever the isworkflowmodified was changed
		 */
		var WFEditorUndoRedoController = function (wFEditorContext, onWorkflowModifiedFlagChanged) {
			Observer.mixin(this);

			/**
			 * @member {UndoRedoController}
			 * @private
			 */
			this._controller = new UndoRedoController();

			/**
			 * @member {I18n}
			 */
			this._i18n = wFEditorContext.get("i18n");

			/**
			 * @member {ToolbarModel}
			 * @private
			 */
			this._toolbarModel = wFEditorContext.get("toolbarModel");

			/**
			 * @member {Command} used to mark last save operation
			 * @private
			 */
			this._saveMarker = null;

			/**
			 * @member {Function}
			 * @private
			 */
			this._onWorkflowModifiedFlagChanged = onWorkflowModifiedFlagChanged;

			/**
			 * @member {Boolean} flag that forces isWorfklowModified to return true even if the command stack is empty
			 * @private
			 */
			this._forceWorfklowModifiedFlag = false;

			// add toolbar component for undo/redo actions
			this._toolbarModel.addComponent(ToolConstants.ButtonGroups.UNDOREDO, this._i18n.t("UndoRedoTitle"));
			this._toolbarModel.addButton(ToolConstants.ButtonGroups.UNDOREDO, ToolConstants.Buttons.UNDO, "icon-WCR_WF-Undo", _.bind(function () {
				this._controller.undo();
			}, this), this._i18n.t("Undo"));
			this._toolbarModel.addButton(ToolConstants.ButtonGroups.UNDOREDO, ToolConstants.Buttons.REDO, "icon-WCR_WF-Redo", _.bind(function () {
				this._controller.redo();
			}, this), this._i18n.t("Redo"));
			this._updateButtons();

			// listen to keyboard short cuts
			wFEditorContext.get("keyboardHandler").bind(["mod+z"], _.bind(function (e) {
				e.preventDefault();
				e.stopPropagation();
				this._controller.undo();
			}, this), "keydown");


			wFEditorContext.get("keyboardHandler").bind(["mod+y", "mod+shift+z"], _.bind(function (e) {
				e.preventDefault();
				e.stopPropagation();
				this._controller.redo();
			}, this), "keydown");

			// update buttons 
			this._observe(this._controller, "changed", this._updateButtons);

		};

		/**
		 * Execute a command on the UndoRedo Controller
		 * The doCallback of the command is executed and the command is added to the undo stack
		 * @param {Command} command
		 * @param {Boolean} [skipDo=false] - if true, the command.executeDo() is not called automatically
		 */
		WFEditorUndoRedoController.prototype.execute = function (command, skipDo) {
			this._controller.execute(command, skipDo);
		};


		/**
		 * Clear the is modified flag of the workflow (can be used when workflow is successfully saved)
		 * whenever the undo/redocontroller comes in the same state as the moment addSaveMarker is called, the workflow is marked as not modified
		 */
		WFEditorUndoRedoController.prototype.addSaveMarker = function () {
			// the SaveMarker is a command that just instructs the controller to do an extra redo() or undo()
			// the command should be "invisible" to the users
			this._saveMarker = new Command(_.bind(function () {
					this._controller.redo();
				}, this),
				_.bind(function () {
					this._controller.undo();
				}, this),
				"");

			// we mark the markercommand for later usage (to filter them out in counts and such)
			this._saveMarker.__isWFEditorSaveMarker = true;

			// execute the "save marker command", nothing will happen, it will just be added to the undo stack
			this._controller.execute(this._saveMarker, true, true);
			
			this._forceWorfklowModifiedFlag = false;
		};
	
		/**
		 * Sets the forceModifiedFlag to true,
		 * indicating that the workflow has been modified regardless of the undo and redo stacks and the save marker
		 */
		WFEditorUndoRedoController.prototype.addForceModifiedFlag = function () {
			this._forceWorfklowModifiedFlag = true;
			this._updateButtons();
		}

		/**
		 * Check if the workflow is modified since initialisation or last addSaveMarker() call
		 * @returns {Boolean}
		 */
		WFEditorUndoRedoController.prototype.isWorkflowModified = function () {
			if(this._forceWorfklowModifiedFlag) {
				return true;
			}
			if (this._saveMarker) {
				return !(this._controller.peekUndo() === this._saveMarker || this._controller.peekRedo() === this._saveMarker);
			} else {
				return this._controller.undoCount() > 0 || this._controller.didReachStackSizeLimit();
			}
		};

		/**
		 * Change the workflow modified callback
		 * @param {Function} callback - called when the isModified status might have changed. first parameter of callback function is a boolean isWorkflowModified
		 */
		WFEditorUndoRedoController.prototype.setWorkflowModifiedCallback = function (callback) {
			this._onWorkflowModifiedFlagChanged = callback;
		};

		/**
		 * This mimics the peekUndo method of UndoRedoController, but filters out the savemarker commands
		 * @returns {Command}  command on top of the undo stack
		 */
		WFEditorUndoRedoController.prototype.peekUndo = function () {
			var i = 0;
			var undo;

			do {
				undo = this._controller.peekUndo(i++);
			} while (undo && undo.__isWFEditorSaveMarker);

			return undo;
		};

		/**
		 * This mimics the peekRedo method of UndoRedoController, but filters out the savemarker commands
		 * @returns {Command}  command on top of the redo stack
		 */
		WFEditorUndoRedoController.prototype.peekRedo = function () {
			var i = 0;
			var redo;

			do {
				redo = this._controller.peekRedo(i++);
			} while (redo && redo.__isWFEditorSaveMarker);

			return redo;
		};
	
		/**
		 * Get the number of undo actions on the command, savemarker commands are ignored
		 * @returns {Number}
		 */
		WFEditorUndoRedoController.prototype.undoCount = function () {
			// count all undo commands that are not a savemarker
			return _.reduce(this._controller.getUndoStack(), function (memo, command) {
				if (!command.__isWFEditorSaveMarker) {
					memo++;
				}
				return memo;
			}, 0);
		};

		/**
		 * Get the number of redo actions on the command, savemarker commands are ignored
		 * @returns {Number}
		 */
		WFEditorUndoRedoController.prototype.redoCount = function () {
			// cound all redo commands that are not a savemarker
			return _.reduce(this._controller.getRedoStack(), function (memo, command) {
				if (!command.__isWFEditorSaveMarker) {
					memo++;
				}
				return memo;
			}, 0);
		};

		/**
		 * Update the state of the buttons when undo or redo stack has changed
		 * @protected
		 */
		WFEditorUndoRedoController.prototype._updateButtons = function () {
			this._toolbarModel.setButtonEnabled(ToolConstants.Buttons.UNDO, this.undoCount() > 0);
			var undoCommand = this.peekUndo();
			if (undoCommand) {
				this._toolbarModel.setButtonTooltip(ToolConstants.Buttons.UNDO, this._i18n.t("UndoAction", undoCommand.getDescription()));
			} else {
				this._toolbarModel.setButtonTooltip(ToolConstants.Buttons.UNDO, this._i18n.t("Undo"));
			}


			this._toolbarModel.setButtonEnabled(ToolConstants.Buttons.REDO, this.redoCount() > 0);
			var redoCommand = this.peekRedo();
			if (redoCommand) {
				this._toolbarModel.setButtonTooltip(ToolConstants.Buttons.REDO, this._i18n.t("RedoAction", redoCommand.getDescription()));
			} else {
				this._toolbarModel.setButtonTooltip(ToolConstants.Buttons.REDO, this._i18n.t("Redo"));
			}

			if (this._onWorkflowModifiedFlagChanged) {
				this._onWorkflowModifiedFlagChanged(this.isWorkflowModified());
			}
		};


		return WFEditorUndoRedoController;
	});