define(["underscore",
		"WFEditor/nodeConfiguration/NodeConfigurationPopup",
		"WFEditor/workflow/node/NodeConfigurationPanel",
		"WFEditor/components/Popup",
		"utils/Command",
		"utils/Observer",
		"utils/Observable",
		"WFEditor/utils/WFI18nConstants"
	],
	function (_,
		NodeConfigurationPopup,
		NodeConfigurationPanel,
		Popup,
		Command,
		Observer,
		Observable,
		WFI18nConstants) {
		"use strict";

		/**
		 * This controller manages the Node Configuration Popups
		 * @exports NodeConfigurationController
		 * @class
		 * @mixes Observable
		 * 
		 * @param {Scene}                scene        - Scene 
		 * @param {UndoRedoController}   undoRedo    
		 */
		var NodeConfigurationController = function (scene, i18n, undoRedo, context) {
			Observer.mixin(this);
			Observable.mixin(this, ["Scene", "NodeConfigurationController"], "WFEditor." + scene.getWorkflowModel().getID());

			/**
			 * @member {Object} set of currently active popups            
			 * @private
			 */
			this._popups = {};
            
            /**
			 * @member {WFEditorContext} 
			 * @private
			 */
            this._wfEditorContext = context;
            
			/**
			 * @member {Scene} 
			 * @private
			 */
			this._scene = scene;

			/**
			 * @member {I18n}
			 * @private
			 */
			this._i18n = i18n;

			/**
			 * @member {UndoRedoController}
			 * @private
			 */
			this._undoRedo = undoRedo;

			this._observe(this._scene.getWorkflowModel(), "Node.removed", function (node) {
				if (this._popups[node.getId()]) {
					this._destroyPopup(this._popups[node.getId()]);
				}
			});
			
			this._observe(this._scene, "destroyed", function(){
				_.invoke(this._popups, "destroy");
				this._popups = {};
			});

		};

		/**
		 * Open the Node Configuration Panel
		 * @param {Node} node - node for which the configuration panel should be opened
		 * @param {String} configurationPanelType - identifier of the type of the configuration panel, there can only be one of each type
		 */
		NodeConfigurationController.prototype.openNodeConfiguration = function (node, configurationPanelType) {
			var configurationPanelId = node.getId() + (configurationPanelType || "");
			var popup = this._popups[configurationPanelId];
			this._emit("open.configurationPanel");
			if (!popup) {

				var temporaryPanel = new NodeConfigurationPanel(node);
				temporaryPanel.getContext().append(
					$("<p>").text(this._i18n.t("loading")).addClass("panel-loading"));

				popup = new NodeConfigurationPopup(temporaryPanel, this._scene.getContext(), this._scene, this._i18n, configurationPanelId);
				this._popups[configurationPanelId] = popup;

				// destroy the popup when it is closed
				this._observe(popup, "changed.visible", function (isVisible) {
					if (!isVisible) {
						this._destroyPopup(popup);
					}
				});


				// popup doesn't exist yet
				// generate the panel
				var panelPromise = node.getConfigurationPanel(this._wfEditorContext);
				if (panelPromise) {
					panelPromise.done(_.bind(function (panel) {
						var configuration = node.getConfiguration();
						if (configuration) {
							panel.setParameters(configuration);
						}

						var popup = this._popups[configurationPanelId];
						if (popup) {
							var oldPanel = popup.setPanel(panel);
							oldPanel.destroy();

							// Disable after setting the panel if needed
							if (panel.isReadOnly()) {
								panel.disable();
							}

							// undo redo save
							this._observe(popup, "saved", function (newConfig) {
								var oldConfig = node.getConfiguration();
								var equal = true;
								if (oldConfig.length === newConfig.length) {
									for (var i = 0; i < oldConfig.length; i++) {
										if (!oldConfig[i].equals(newConfig[i])) {
											equal = false;
											break;
										}
									}
								} else {
									equal = false;
								}

								if (!equal) {
									this._undoRedo.execute(new Command(
										function () {
											node.setConfiguration(newConfig);
										},
										function () {
											node.setConfiguration(oldConfig);
										},
										this._i18n.t(WFI18nConstants.saveNodeConfigurationDescription)
									));
								}

							});
							var footerContent = panel.getFooterContent();
							if (footerContent) {
								popup.addFooterContent(footerContent);
							}


							popup.show(this._scene.getLocationForNode(node).round(), true);
							popup.focus();
						}
					}, this));
				}
			}

			// show the popup and focus
			if (popup) {
				popup.show(this._scene.getLocationForNode(node).round(), true);
				popup.resize();
				popup.focus();
			}
		};

		/**
		 * Destroy a popup and remove its reference
		 * @private
		 * @param {NodeConfigurationPopup} popup 
		 */
		NodeConfigurationController.prototype._destroyPopup = function (popup) {
			delete this._popups[popup.getId()];

			popup.destroy();
		};

		return NodeConfigurationController;
	});