define(["underscore",
		"WFEditor/components/Popup",
		"WFEditor/nodeConfiguration/NodeConfigurationLifelineHelper",
		"component/button/Button",
		"utils/Observer"
	],
	function (_,
		Popup,
		NodeConfigurationLifelineHelper,
		Button,
		Observer) {

		"use strict";

		/**
		 * Popup used to show Node Configuration Panels
		 * @exports NodeConfigurationPopup
		 * @class
		 * @augments Popup	
		 * @mixes Observer
		 *             
		 * @param {NodeConfigurationPanel}    nodeConfigurationPanel - Configuration panel for the node
		 * @param {JqueryObj} containment            - containment for the popup
		 * @param {Scene}     scene                  - Scene
		 * @param {I18n} i18n
		 * @param {id} id
		 */
		var NodeConfigurationPopup = function (nodeConfigurationPanel, containment, scene, i18n, id) {
			Popup.call(this, nodeConfigurationPanel.getTitle(), containment);

			Observer.mixin(this);

			/**
			 * @member {String}
			 * @private
			 */
			this._id = id;

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = scene;

			/**
			 * @member {I18n}
			 * @private
			 */
			this._i18n = i18n;

			/**
			 * @member {NodeConfigurationPanel}
			 * @private
			 */
			this._nodeConfigurationPanel = nodeConfigurationPanel;


			// add the panel to the content of the popup
			this.content.append(nodeConfigurationPanel.getContext());
			this._nodeConfigurationPanel.init();

			/**
			 * @member {Button} Button to cancel the changes
			 * @private
			 */
			this._cancelButton = null;

			/**
			 * @member {Button} Button to save the changes
			 * @private
			 */
			this._saveButton = null;

			this._initializeButtons();

			// set the title (in the future we might need to listen to title changes)
			this.setTitle(nodeConfigurationPanel.getTitle());

			// initialize the lifeline
			this._initializeLifeline();

			/**
			 * @member {Number} scroll bar margin that needs to be taken in to account (this is used to avoid unnecessary scrollbars)
			 * @private
			 */
			this._scrollbarMargin = 18;

			this._observe(this, "resize.stop", this._resized);
			this._observe(this, "resize.change", _.throttle(_.bind(this._resized, this), 40));

			this._observe(this._scene, "beforeRemoveFromDom", function () {
				this._nodeConfigurationPanel.beforeRemoveFromDom();
			});

			this._observe(this._scene, "afterAddToDom", function () {
				this._nodeConfigurationPanel.afterAddToDom();
			});
		};

		NodeConfigurationPopup.prototype = Object.create(Popup.prototype);
		NodeConfigurationPopup.prototype.constructor = NodeConfigurationPopup;


		/**
		 * Set the panel of the popup
		 * Note this doesn't destroy the current panel
		 * @param {NodeConfigurationPanel} panel -  new panel
		 * @returns {NodeConfigurationPanel} oldPanel
		 */
		NodeConfigurationPopup.prototype.setPanel = function (panel) {
			this.content.empty(); // add the panel to the content of the popup
			var oldPanel = this._nodeConfigurationPanel;
			this._nodeConfigurationPanel = panel;
			this.content.append(panel.getContext());
			this._nodeConfigurationPanel.init();

			// set the title of new panel
			this.setTitle(panel.getTitle());

			return oldPanel;
		};

		/**
		 * Get the id of the popup
		 * @returns {String} id
		 */
		NodeConfigurationPopup.prototype.getId = function () {
			return this._id;
		};

		/**
		 * The close button was pressed, cancel the changes
		 * @override
		 */
		NodeConfigurationPopup.prototype._closeButtonPressed = function () {
			this._nodeConfigurationPanel.cancel();
			Popup.prototype._closeButtonPressed.call(this);
		};

		/**
		 * Initialize the lifeline of the popup
		 */
		NodeConfigurationPopup.prototype._initializeLifeline = function () {
			var node = this._nodeConfigurationPanel.getNode();
			var workflow = this._nodeConfigurationPanel.getWorkflow();

			this._lifeline = new NodeConfigurationLifelineHelper(workflow, this._id + "detailsLifeline", this._scene, this);
			workflow.addHelper(this._lifeline);


			// make sure to make the lifeline interactive when needed
			// update the lifeline when popup is moved/resized or node has moved
			var startInteractive = function () {
				this._scene.setInteractive(this._lifeline.getId(), true);
				this._nodeConfigurationPanel.startInteractive();
			};

			var stopInteractive = function () {
				this._scene.setInteractive(this._lifeline.getId(), false);
				this.resize();
			};

			var updateEndPoint = function () {
				this._lifeline.setEndPointModified();
			};

			var nodeLocationChanged = function () {
				var nodeView = this._scene.getView(node.getId());
				this._lifeline.setStartPoint(nodeView.getCenter());
			};

			this._observe(this, "drag.start", startInteractive);
			this._observe(this, "resize.start", startInteractive);
			this._observe(this, "drag.move", updateEndPoint);
			this._observe(this, "changed.location", updateEndPoint);
			this._observe(this, "resize.change", updateEndPoint);
			this._observe(this, "drag.stop", stopInteractive);
			this._observe(this, "resize.stop", stopInteractive);
			this._observe(this._scene, "resized", function () {
				this.resize();
			});
			this._observe(this._scene, "destroyed", function () {
				this._nodeConfigurationPanel.destroy();
			});
			this._observe(node, "changed.location", nodeLocationChanged);

			nodeLocationChanged.call(this);
		};

		/**
		 * Remove all observers when being destroyed
		 * @override
		 */
		NodeConfigurationPopup.prototype.destroy = function () {
			this._stopObserveAll();
			this._nodeConfigurationPanel.destroy();
			this._nodeConfigurationPanel.getWorkflow().removeHelper(this._lifeline);
			Popup.prototype.destroy.call(this);
		};

		/**
		 * Make sure the configuration panel is focused
		 * @override
		 */
		NodeConfigurationPopup.prototype.focus = function () {
			this._nodeConfigurationPanel.focus();
		};

		/**
		 * Initialize the buttons for the popup
		 */
		NodeConfigurationPopup.prototype._initializeButtons = function () {
			this._cancelButton = new Button(null, null, false, "Cancel", this._i18n.t("configurationCancel"), true, _.bind(function () {
				this._nodeConfigurationPanel.cancel();
				this.hide();
			}, this));
			this._cancelButton.addClass("btn btn-default btn-sm");


			this._saveButton = new Button(null, null, false, "OK", this._i18n.t("configurationOK"), true, _.bind(function () {
				if (this._nodeConfigurationPanel.validate()) {
					this._emit("saved", this._nodeConfigurationPanel.getParameters());
					this._nodeConfigurationPanel.save();
					this.hide();
				}
			}, this));
			this._saveButton.addClass("btn btn-primary btn-sm");


			var buttonGroup = $("<div>").addClass("btn-group pull-right");
			if (this._nodeConfigurationPanel.isReadOnly()) {
				this._cancelButton.setContent(this._i18n.t("configurationOK"));
				buttonGroup.append(this._cancelButton.element);
			} else {
				buttonGroup.append(this._cancelButton.element).append(this._saveButton.element);
			}

			// add buttons to the footer
			this._footer.append(buttonGroup);

		};


		/**
		 * Measure the panel desired width, and set the size of the popup
		 * @Override
		 */
		NodeConfigurationPopup.prototype._doFitContent = function (scrollDown) {
			this.content.css("bottom", "auto");
			var innerWidth = this._containment.innerWidth();


			this._applyContainerClass(innerWidth);

			// The node configuration popup should be contained in height by the containment element
			var height = Math.min(this._containment.innerHeight(), this.content.get(0).scrollHeight + this._header.outerHeight(true) + this._footer.outerHeight(true) + 4);

			// If possible the width of the content should be at least 404px and there should be no horizontal scroll bar.
			// If the containment is too smal, just set the widht of the contents to that width
			var width = Math.min(innerWidth, Math.max(Math.max(404, this._nodeConfigurationPanel.getPreferredWidth() + this._scrollbarMargin), this.content.get(0).scrollWidth + 2));

			// to avoid width inflation, this scrollbar margin should only be taken in to account once.
			this._scrollbarMargin = 0;

			this.element.css({
				width: width + "px",
				height: height + "px"
			});
			this.content.css("bottom", "");
			if (this._lifeline) {
				this._lifeline.setEndPointModified();
			}
			this._resized();

			// If the size has changed, make sure it is still contained in the containment element
			var offsetTop = this.element.get()[0].offsetTop;
			var containmentHeight = this._containment.height();
			if (offsetTop + height > containmentHeight) {
				this.element.css({
					top: (containmentHeight - height) + "px"
				});
			}

			if (scrollDown) {
				this.content[0].scrollTop = this.content[0].scrollHeight;
			}
		};

		/**
		 * Make the popup contents responsive :)
		 */
		NodeConfigurationPopup.prototype._resized = function () {
			this._applyContainerClass(this.element.width());
		};

		/** 
		 * Apply the correct container class for responsiveness
		 * @param {Number} width - width of the popup
		 */
		NodeConfigurationPopup.prototype._applyContainerClass = function (width) {
			this.content.toggleClass("container-sm", width < 400)
				.toggleClass("container-lg", width >= 400);
		};

		/**
		 * Adds content to the footer
		 * @param {JQueryObject}
		 */
		NodeConfigurationPopup.prototype.addFooterContent = function (footerContent) {
			this._footer.append(footerContent);
		};

		/**
		 * Call when the node confirationpopup is resized
		 */
		NodeConfigurationPopup.prototype.resize = function () {
			this._nodeConfigurationPanel.resize();
		};

		return NodeConfigurationPopup;
	});