define([
		"jquery",
		"underscore",
        "utils/Observable",
        "./view/NodeGalleryList",
        "./view/NodeGallerySelection",
        "utils/webcss!./NodeGallery-style.css"
    ],
	function (
        $,
        _,
        Observable,
        NodeGalleryList,
        NodeGallerySelection,
        NodeGalleryStyle
      ) {
		"use strict";

		/**
		 * NodeGallery is an overview of all the possible nodes for the workflow editor. The user
		 * can add nodes by dragging them from the gallery onto the scene.
		 *
		 * @exports NodeGallery
		 * @class
		 */
		var NodeGallery = function () {
            Observable.mixin(this, "nodeGallery");
            
            /**
			 *  @member {Array.<GalleryNode>} contains all nodes of the nodegallery
			 *  @private
			 */
            this._allNodes = [];
            
            /**
			 *  @member {Map.<String, Array.<GalleryNode>>} maps the node category on the list of nodes belonging to that category
			 *  @private
			 */
            this._nodesPerCategory = {};
            
            /**
			 *  @member {JQueryObj} context for the selection part of the gallery
			 *  @private
			 */
            this._selectionContext = $("<div>").addClass('gallerySelection');
            
            /**
			 *  @member {JQueryObj} context for the node list part of the gallery
			 *  @private
			 */
            this._listContext = $("<div>").addClass('galleryList');
            
            /**
			 *  @member {NodeGalleryList} the NodeGalleryList that belongs to this NodeGallery
			 *  @private
			 */
            this._gallerylist = null;
            
            /**
			 *  @member {NodeGallerySelection} the NodeGallerySelection that belongs to this NodeGallery
			 *  @private
			 */
            this._selection = null;
            
            // initialize NodeGalleryList and NodeGallerySelection
            this._initializeContent();
		};
    
        /**
		 * Set NodeGalleryList and NodeGallerySelection
		 * Override to have specific NodeGalleryList and NodeGallerySelection
		 */
		NodeGallery.prototype._initializeContent = function () {
            this._gallerylist = new NodeGalleryList(this, {}, {});
            this._selection = new NodeGallerySelection(this, {});
        };

		/**
		 * Set the context for the gallery and constructs the DOM structure.
		 * @param {JQueryObj} context
		 */
        NodeGallery.prototype.setContext = function (context) {
			this.context = context;
            
            var borderdiv = $("<div>").addClass("galleryBorder");
            var divtable = $("<div>").addClass("galleryTable");
			divtable.append(this._selectionContext);
            
            var divList = $("<div>").addClass("galleryListRow");
			divList.append(this._listContext);
            divtable.append(divList);
            
            borderdiv.append(divtable);
            this.context.append(borderdiv);
        };

		/**
		 * Sets the nodes to be shown in the gallery and initializes the category dropdown, 
		 * node filter and shown nodes.
		 * @param {Array.<GalleryNode>} nodes
		 */
		NodeGallery.prototype.setNodeDefinitions = function (nodes) {
            this._allNodes = nodes;
            this._nodesPerCategory = {};
            _.each(nodes, function(node){
                if (this._nodesPerCategory[node.getCategory()]){
                    this._nodesPerCategory[node.getCategory()].push(node);
                } else {
                    this._nodesPerCategory[node.getCategory()] = [node];
                }
            }, this);
            
            var dropdownitems = ['all'];
            $.each(this._nodesPerCategory, function(index, value){
                dropdownitems.push(index);
            });
            this._selection.render(this._selectionContext, dropdownitems);
            this._gallerylist.render(this._listContext, this._allNodes);
        };

        /**
         * Updates the galleryList to show nodes of the selected category.
         * @param {String} category - what category should be shown
         * @param {String} [filter] - show only items of the category matching the filter
         */
        NodeGallery.prototype.selectList = function (category, filter) {
            if (category == 'all'){
                this._gallerylist.update(this._allNodes, filter);
            }
            else {
                this._gallerylist.update(this._nodesPerCategory[category], filter);
            }
        };
    
        /**
         * Select a node from the gallery and emit event
         * @param {String} node - selected node
         * @param {Event} e - mouse event that triggeredt this selection                     
         */
        NodeGallery.prototype.selectNode = function (node, e) {
            this._emit("gallery.selectNode", node, e);
        };

		return NodeGallery;

	});