define([
		"Template",
		"tpl!WFEditor/nodeGallery/view/NodeGalleryList",
		"utils/FilterUtils",
		"jquery",
		"underscore",
		"jqueryUI",
		"utils/webcss!./NodeGalleryList-style.css"
	],
	function (
		Template,
		tpl,
		FilterUtils,
		$,
		_
	) {
		"use strict";

		/**
		 * NodeGalleryList is a view object for rendering the list part of a NodeGallery. 
		 * For specific NodeGalleryLists, the render and update methods need to be implemented.
		 * 
		 * @exports NodeGalleryList
		 * @class
		 * 
		 * @param {NodeGallery} nodegallery - the nodegallery to which the NodeGalleryList belongs.
		 * @param {I18n} i18n - instance of i18n
		 */
		var NodeGalleryList = function (nodegallery, i18n) {
			this._nodegallery = nodegallery;
			this._i18n = i18n;

			this._context = null;

			this.t = new Template(this._i18n, tpl);

			this.t.bind("mousedown", "tr", this._handleGalleryNodeActivation, this);
		};

		/**
		 * Render the gallery list
		 * @param   {JQueryObj} context
		 * @param   {Array.<GalleryNode>} nodes - array of nodes to display
		 */
		NodeGalleryList.prototype.render = function (context, nodes) {
			if (context) {
				this._context = context;
				this.t.setContext(this._context);
				this._context.disableSelection();
				this._context.css("cursor", "default");
			}

			if (this._context) {
				this.t.d.nodes = _.sortBy(nodes, _.bind(function (node) {
					return this._i18n.t(node.getLabel()).toLowerCase();
				}, this));

				this.t.render();
			}
		};

		/**
		 * Update the nodes displayed in the gallery
		 * @param   {Array.<GalleryNode>} nodes - array of nodes to display
		 * @param   {String} [filter] - show only nodes whose name match the filter
		 */
		NodeGalleryList.prototype.update = function (nodes, filter) {
			var filteredNodes;
			if (filter) {
				var filterRegExp = FilterUtils.generateRegex(filter, true);
				filteredNodes = _.filter(nodes, _.bind(function (node) {
					return filterRegExp.test(this._i18n.t(node.getLabel()));
				}, this));
			} else {
				filteredNodes = nodes;
			}

			this.render(null, filteredNodes);
		};

		/**
		 * Register listeners on the gallery list context
		 * @param {DOMElement} elem - selected element
		 * @param {Event} e - mouse event                         
		 */
		NodeGalleryList.prototype._handleGalleryNodeActivation = function (elem, e) {
			var idx = $(elem).attr("data-bind");
			if (e.button === 0 && idx) {
				var selectedNode = this.t.d.nodes[idx];
				this._nodegallery.selectNode(selectedNode, e);
			}
		};


		return NodeGalleryList;

	});