define([
		"../NodeGallery",
        "Template",
		"tpl!WFEditor/nodeGallery/view/NodeGallerySelection",
		"jquery",
        "underscore",
        "utils/webcss!./NodeGallerySelection-style.css"
	],
	function (
        NodeGallery,
        Template,
        tpl,
        $,
        _
	) {
		"use strict";

		/**
		 * NodeGallerySelection is a view object for rendering the selection part of a NodeGallery. 
		 * For specific NodeGallerySelections, the render method needs to be implemented.
		 * 
		 * @exports NodeGallerySelection
		 * @class
		 * 
		 * @param {NodeGallery} nodegallery - the nodegallery to which the NodeGallerySelection belongs.
		 * @param {Map.<String,String>} filterIcons - maps css icon classes to icon label name
		 * @param {I18n} i18n - instance of i18n
		 */
		var NodeGallerySelection = function (nodegallery, filterIcons, i18n) {
            this._nodegallery = nodegallery;
            this._i18n = i18n;
            this._filterIcons = filterIcons;
            
            this._selected = "";
            this._filtertext = "";
            
            this._titleElem = undefined;
            this._filterIconElem = undefined;
            this._filterInputElem = undefined;
            
            this.t = new Template(this._i18n, tpl);
            this.t.bind("click", "li", this._selectList, this);
            this.t.bind("keyup", "input", this._filterList, this);
            this.t.bind("click", "button", this._clearFilter, this);
		};

        /**
         * Render the items displayed in the dropdownlist and initialize the filter input field.
         * @param   {JQueryObj} context
         * @param   {Array.<String>} dropdownlist - items for the dropdown
         */
        NodeGallerySelection.prototype.render = function(context, dropdownlist) {
            this.t.setContext(context);
            //init the dropdown
            this.t.d.dropdownlist = _.sortBy(dropdownlist, _.bind(function(item){
                return this._i18n.t(item);
            },this));
            this.t.d.dropdowntitle = this._i18n.t(this.t.d.dropdownlist[0]);
            this._selected = this.t.d.dropdownlist[0];
            
            //init the filter
            this.t.d.filtericon = this._filterIcons.SEARCH;
            this.t.render();
            
            this._titleElem = context.find('.dropdownTitle');
            this._filterIconElem = context.find(".filterIcon");
            this._filterInputElem = context.find(".filterInput");
        };
    
        /**
         * Called when an item is clicked in the dropdown. 
         * Updates the category of nodes that are shown in the gallery.
         * @param el         -     Element that was clicked on in the dropdown.
         * @param event      -     Event
         */
        NodeGallerySelection.prototype._selectList = function(el, event){
            this._selected = $(el).attr("data-bind");
            this._nodegallery.selectList(this._selected, this._filtertext);
            this._titleElem.text($(el).text());
			event.preventDefault();
        };
    
        /**
         * Called when something is typed in the inputfield.
         * Filters the nodes that are shown in the gallery.
         * @param el - The inputfield.
         * @param event - Event
         */
        NodeGallerySelection.prototype._filterList = function(el, event){
            this._filtertext = $(el).val();
            
            if(this._filtertext){
                this._filterIconElem.removeClass(this._filterIcons.SEARCH).addClass(this._filterIcons.CLEAR);
                this.t.d.filtericon = this._filterIcons.CLEAR;
            } else {
                this._filterIconElem.removeClass(this._filterIcons.CLEAR).addClass(this._filterIcons.SEARCH);
                this.t.d.filtericon = this._filterIcons.SEARCH;
            }
            
            this._nodegallery.selectList(this._selected, this._filtertext);
        };
    
        /**
         * Called when the icon next to the inputfield is clicked.
         * Clears the filter.
         * @param el - The button that was clicked
         * @param event - Event
         */
        NodeGallerySelection.prototype._clearFilter = function(el, event){
            //only clear the filter if something was in it, else do nothing
            if (this._filterIconElem.hasClass(this._filterIcons.CLEAR)){
                this._filterInputElem.val("");
                this._filtertext = "";
                this._nodegallery.selectList(this._selected);
                this._filterIconElem.removeClass(this._filterIcons.CLEAR).addClass(this._filterIcons.SEARCH);
            }
        };

		return NodeGallerySelection;

	});