define([
		"utils/ShapeUtils",
		"utils/Observer",
		"utils/Observable"
	],
	function (
		ShapeUtils,
		Observer,
		Observable
	) {
		"use strict";

		/**
		 * Generic View object for a workflow scene.
		 *
		 * @exports AbstractView
		 * @class
		 *
		 * @param {WorkflowItem} model - model for the view
		 */
		var AbstractView = function (model) {
			Observable.mixin(this, ["viewObject", model.getId()]);
			Observer.mixin(this);
			/**
			 * @member {WorkflowItem} the model for the view
			 * @private
			 */
			this._model = model;

			/**
			 * @member {Boolean} is the view object interactive (moving, animation,...)
			 * @private
			 */
			this._isInteractive = false;


			/**
			 * @member {NormalizedRectArray} bounding rect of the view (world coordinates) [x1, y1, x2, y2]
			 * @private NEVER CHANGE DIRECTLY (override updateBoundingRect() and use _setBoundingRect to change the bounding rect)
			 */
			this._boundingRect = null;
			/**
			 * @member {Array.<HitArea>} set of hit areas
			 * @private NEVER CHANGE DIRECTLY (override updateHitAreas() and use _setHitAreas() to change the hit areas)
			 */
			this._hitAreas = [];

			/**
			 * @member {CanvasRenderingContext2D} context for the view
			 * NEVER DRAW ON THIS CONTEXT!
			 * @private
			 */
			this._context = null;

		};

		/**
		 * Renders the view on canvas context.
		 * @param {CanvasRenderingContext2D} context
		 * @param {Layer} layer
		 * @param {Boolean} quickRender - if true, only high priority render actions should be executed (i.e. during panning)
		 */
		AbstractView.prototype.render = function (context, layer, quickRender) {
			// by default nothing is rendered
		};


		/**
		 * Get the ID of the AbstractVIew
		 * @returns {String}
		 */
		AbstractView.prototype.getId = function () {
			return this._model.getId();
		};


		/**
		 * Returns the z-index of the view
		 * A view with a higher z-index covers elements of a lower z-index
		 * @returns {Number}
		 */
		AbstractView.prototype.getZIndex = function () {
			return 0;
		};


		/**
		 * Get the Model of the view
		 * @returns {WorkflowItem}
		 */
		AbstractView.prototype.getModel = function () {
			return this._model;
		};

		/**
		 * Set the view object to interactive, means the object is being moved, animation,...
		 * @param {Boolean} isInteractive
		 */
		AbstractView.prototype.setInteractive = function (isInteractive) {
			this._isInteractive = isInteractive;
		};

		/**
		 * Is the view object interactive
		 * @returns {Boolean}
		 */
		AbstractView.prototype.isInteractive = function () {
			return this._isInteractive;
		};

		/**
		 * Destroys the view
		 */
		AbstractView.prototype.destroyView = function () {
			this._stopObserveAll();
		};

		/**
		 * Mark the view dirty, this should trigger a render of the layer
		 */
		AbstractView.prototype.makeDirty = function () {
			this._emit("isDirty", this);
		};

		/**
		 * This method is called when a view is added to a layer the first time
		 * The context of that layer is provided and can be used to calculate things
		 * Whenever a render method is called, the correct context is provided in that render call
		 * never use the context variable for rendering
		 * @param {CanvasRenderingContext2D} context
		 */
		AbstractView.prototype.initialize = function (context) {
			this._context = context;
			this._updateBoundingRect();
			this._updateHitAreas();
		};

		/**
		 * Update the current bounding rect.
		 * This method must be implemented by your view
		 */
		AbstractView.prototype._updateBoundingRect = function () {
			throw new Error("_updateBoundingRect is not implemented");
		};

		/**
		 * Update the hit areas for the view
		 */
		AbstractView.prototype._updateHitAreas = function () {
			// by default no hit areas are set, please override
		};

		/**
		 * Set the bounding rect of the view
		 * @param {NormalizedRectArray} boundingRect - [x1, y1, x2, y2]
		 */
		AbstractView.prototype._setBoundingRect = function (boundingRect) {
			var oldRect = this._boundingRect;
			if (boundingRect != oldRect ||
				(boundingRect && oldRect &&
					!ShapeUtils.normalizedRectEquals(boundingRect, oldRect))) {
				// only send out event when bounding rect really changed
				this._boundingRect = boundingRect;
				this._emit("changed.boundingRect", this);
			}
		};

		/**
		 * Set the hit areas of the view
		 * when having multiple areas, please always set them in the same order, this will prevent unnecessary tree updates when they are still equal
		 * @param {Array.<HitArea>} hitAreaList - this can be empty or a list of hitareas
		 */
		AbstractView.prototype._setHitAreas = function (hitAreaList) {
			var equal = false;

			if (this._hitAreas.length === hitAreaList.length) {
				equal = true;
				for (var i = 0; equal && i < this._hitAreas.length; i++) {
					if (!this._hitAreas[i].equals(hitAreaList[i])) {
						equal = false;
					}
				}
			}
			// the hitAreaList differs from the original 
			if (!equal) {
				this._hitAreas = hitAreaList;
				this._emit("changed.hitAreas", this);
			}
		};

		/**
		 * Get the bounding rect of the view (in world coordinates)
		 * @returns {NormalizedRectArray} [x1, y1, x2, y2]
		 */
		AbstractView.prototype.getBoundingRect = function () {
			return this._boundingRect;
		};

		/**
		 * Get the list of hit areas for the view
		 * @returns {Array.<NormalizedRectArray>} hitAreaList
		 */
		AbstractView.prototype.getHitAreas = function () {
			return this._hitAreas;
		};
		
		/**
		 * Fill the current path in the context with a white opaque background
		 * @param {CanvasRenderingContext2D} context
		 */
		AbstractView.prototype._renderOpaqueBackground = function (context) {
			var fillStyle = context.fillStyle;
			var globalAlpha = context.globalAlpha;
			context.globalAlpha = 1;
			context.fillStyle = "white";
			context.fill();
			context.globalAlpha = globalAlpha;
			context.fillStyle = fillStyle;
		};

		return AbstractView;
	});