define(["WFEditor/workflow/node/Node",
		"WFEditor/workflow/pin/Pin",
		"WFEditor/tools/drawEdge/MouseNode",
		"WFEditor/tools/drawEdge/MouseNodeView",
		"WFEditor/tools/drawEdge/MousePin",
		"WFEditor/tools/drawEdge/MousePinView",
		"WFEditor/tools/selection/SelectionHelper",
		"WFEditor/tools/selection/SelectionView",
		"WFEditor/tools/moveNodes/GridHelper",
		"WFEditor/tools/moveNodes/GridView",
		"WFEditor/workflow/helper/LifelineHelper",
		"WFEditor/workflow/helper/LifelineHelperView",
		"utils/ColorUtils",
		"jquery"
	],
	function (
		Node,
		Pin,
		MouseNode,
		MouseNodeView,
		MousePin,
		MousePinView,
		SelectionHelper,
		SelectionView,
		GridHelper,
		GridView,
		LifelineHelper,
		LifelineHelperView,
		ColorUtils,
		$
	) {
		"use strict";

		/**
		 * Abstract definition of the workflow view factory. This class specifies which methods a
		 * specific implementation of the factory has to define to be able to work in the framework.
		 *
		 * @exports AbstractViewFactory
		 * @class
		 */
		var AbstractViewFactory = function () {

		};

		/**
		 * Create all abstractView objects for representing a workflow
		 * @param {Workflow} workflow - the workflow for which the views should be generated
		 * @param {Scene} scene - scene that contains the layers the view object are rendered on
		 * @returns {$.Deferred} promise object, which is resolved when all views are created (and returns a Map.<id,AbstractView>)
		 */
		AbstractViewFactory.prototype.createViewsForWorkflow = function (workflow, scene) {
			var deferred = $.Deferred;
			deferred.resolve({});
			return deferred.promise();
		};


		/**
		 * Create all abstractView objects for representing a certain workflow item
		 * @param {WorkflowItem} workflowItem - the workflowItem for which the views should be generated
		 * @param {Map.<String, AbstractView>} existingViews - a mapping of viewID to AbstractView for all existing views
		 * @param {Scene} scene - scene that contains the layers the view object are rendered on
		 * @param {Map.<String, AbstractView>} viewsDestination - newly created views should be added to this map<ViewID,AbstractView>
		 */
		AbstractViewFactory.prototype.createViewsForWorkflowItem = function (workflowItem, existingViews, scene, viewsDestination) {};

		/**
		 * Creates a view object for a node.
		 *
		 * @abstract
		 * @param {Node} node - node model
		 * @param {Map.<String, AbstractView>} existingViews - a mapping of viewID to AbstractView for all existing views
		 * @param {Scene} scene - scene that contains the layers the view object are rendered on
		 * @param {Map.<String, AbstractView>} viewsDestination - newly created views should be added to this map<ViewID,AbstractView>
		 * @returns {Boolean} True if it could create a view for the given model
		 */
		AbstractViewFactory.prototype._createNode = function (node, existingViews, scene, viewsDestination) {
			if (node instanceof MouseNode) {
				var nodeView = new MouseNodeView(node);
				viewsDestination[nodeView.getId()] = nodeView;
				existingViews[nodeView.getId()] = nodeView;

				_.each(node.getPins(), function (pin) {
					this._createPin(pin, existingViews, scene, viewsDestination);
				}, this);
				return true;
			}
			return false;
		};

		/**
		 * Creates a view object for a pin.
		 *
		 * @abstract
		 * @param {Pin} pin - pin model
		 * @param {Map.<String, AbstractView>} existingViews - a mapping of viewID to AbstractView for all existing views
		 * @param {Scene} scene - scene that contains the layers the view object are rendered on
		 * @param {Map.<String, AbstractView>} viewsDestination - newly created views should be added to this map<ViewID,AbstractView>
		 * @returns {Boolean} True if it could create a view for the given model
		 */
		AbstractViewFactory.prototype._createPin = function (pin, existingViews, scene, viewsDestination) {
			if (pin instanceof MousePin) {
				var nodeView = existingViews[pin.getNode().getId()];
				if (nodeView) {
					var pinView = new MousePinView(pin, nodeView, scene);
					viewsDestination[pinView.getId()] = pinView;
					return true;
				}
			}
			return false;
		};

		/**
		 * Creates a view object for an edge.
		 *
		 * @abstract
		 * @param   {Edge} edge - edge model
		 * @param {Map.<String, AbstractView>} existingViews - a mapping of viewID to AbstractView for all existing views
		 * @param {Scene} scene - scene that contains the layers the view object are rendered on
		 * @param {Map.<String, AbstractView>} viewsDestination - newly created views should be added to this map<ViewID,AbstractView>
		 * @returns {Boolean} True if it could create a view for the given model
		 */
		AbstractViewFactory.prototype._createEdge = function (edge, existingViews, scene, viewsDestination) {
			return false;
		};

		/**
		 * Creates a view object for an Token.
		 *
		 * @abstract
		 * @param {Token} token - token model
		 * @param {Map.<String, AbstractView>} existingViews - a mapping of viewID to AbstractView for all existing views
		 * @param {Scene} scene - scene that contains the layers the view object are rendered on
		 * @param {Map.<String, AbstractView>} viewsDestination - newly created views should be added to this map<ViewID,AbstractView>
		 * @returns {Boolean} True if it could create a view for the given model
		 */
		AbstractViewFactory.prototype._createToken = function (token, existingViews, scene, viewsDestination) {
			return false;
		};


		/**
		 * Create a view object for Helper items
		 * @param {Map.<String, AbstractView>} existingViews - a mapping of viewID to AbstractView for all existing views
		 * @param {Scene} scene - scene that contains the layers the view object are rendered on
		 * @param {Map.<String, AbstractView>} viewsDestination - newly created views should be added to this map<ViewID,AbstractView>
		 * @returns {Boolean} True if it could create a view for the given model
		 */
		AbstractViewFactory.prototype._createHelper = function (helper, existingViews, scene, viewsDestination) {
			if (helper instanceof SelectionHelper) {
				var selectionView = new SelectionView(helper);
				viewsDestination[helper.getId()] = selectionView;
				return true;
			} else if (helper instanceof LifelineHelper) {
				var view = new LifelineHelperView(helper);
				viewsDestination[helper.getId()] = view;
				return true;

			} else if (helper instanceof GridHelper) {
				var gridView = new GridView(helper);
				viewsDestination[helper.getId()] = gridView;
				return true;
			}
			return false;
		};



		return AbstractViewFactory;
	});