define([
		"utils/ShapeUtils",
		"utils/Transformation",
		"utils/Observer",
		"utils/Observable",
		"utils/ExecutionQueue",
		"WFEditor/scene/layer/LayerSet",
		"WFEditor/scene/hitarea/HitAreaTree",
		"WFEditor/scene/ViewConstants",
		"WFEditor/workflow/ModelConstants",
		"component/scrollbar/Scrollbar",
		"rbush",
		"underscore",
		"utils/webcss!WFEditor/core/wfeditor-style.css"
	],
	function (
		ShapeUtils,
		Transformation,
		Observer,
		Observable,
		ExecutionQueue,
		LayerSet,
		HitAreaTree,
		ViewConstants,
		ModelConstants,
		Scrollbar,
		Rbush,
		_
	) {
		"use strict";

		// install requestAnimationFrame Polyfill
		if (!window.requestAnimationFrame) { //not supported by the browser, define a polyfill method 
			var lastTime = 0;
			//register the polyfill to requestAnimationFrame so we don't have to constuct this function again
			window.requestAnimationFrame = function (callback, element) {
				var currTime = new Date().getTime();
				//16 makes it possible to render up to 60fps
				var timeToCall = Math.max(0, 16 - (currTime - lastTime));
				var id = window.setTimeout(function () {
						callback(currTime + timeToCall);
					},
					timeToCall);
				lastTime = currTime + timeToCall;
				return id;
			};
		}

		/**
		 * The Scene is responsible for the layers and registers listeners on the model.
		 * It uses the viewFactory to create the correct view components and adds these to
		 * the correct layer.
		 *
		 * @exports Scene
		 * @class
		 *
		 * @param {JQueryObj}           context          - the context in which the workflow scene is drawn
		 * @param {SceneModel}	        sceneModel	     - the model of the scene
		 * @param {Workflow}            workflowModel    - model of the workflow which contains all the workflow items
		 * @param {AbstractViewFacotry} viewFactory      - factory used to generate all the view objects
		 * @param {WFEditorContext}		WFEditorContext
		 */
		var Scene = function (context, sceneModel, workflowModel, viewFactory, WFEditorContext) {
			Observable.mixin(this, "Scene", "WFEditor." + workflowModel.getID());
			Observer.mixin(this);
			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._context = context;


			/**
			 * @member {JQueryObj} This div is used to put all layers in
			 * @private
			 */
			this._layerContext = $("<div>").addClass("layerContext").appendTo(this._context);

			/**
			 * @member {SceneModel}
			 * @private
			 */
			this._sceneModel = sceneModel;

			/**
			 * @member {Workflow}
			 * @private
			 */
			this._workflowModel = workflowModel;

			/**
			 * @member {AbstractViewFactory}
			 * @private
			 */
			this._viewFactory = viewFactory;

			/**
			 * @member {Map.<String,AbstractView}
			 * @private
			 */
			this._viewObjects = {};


			/** 
			 * @member {Array.<Number>} list of ZIndexes for which a layer is initialized
			 * @private
			 */
			this._initializedLayers = [];

			/**
			 * @member {Array.<LayerSet>} all the initialized layers, this is a sparse array! (ZIndex -> LayerSet}
			 * @private
			 */
			this._layers = [];

			/**
			 * @member {Array.<LayerObj>} list of layers that need animationframe, each layerObj has a renderduration property and a reference to the layer
			 * @private
			 */
			this._layersWaitingForAnimationFrame = null;


			/**
			 * @member {HitAreaTree} manages the hit area tree
			 * @private
			 */
			this._hitAreaTree = new HitAreaTree();

			/**
			 * @member {Function} bound version of the handle animation frame function
			 * @private
			 */
			this._boundHandleAnimationFrame = _.bind(this._handleAnimationFrame, this);

			/**
			 * @member {Boolean} true if all views are set up for the workflow
			 * @private
			 */
			this._isWorkflowLoaded = false;

			/**
			 * @member {I18n} i18n for scene
			 */
			this._i18n = WFEditorContext.get("i18n");


			// we need to add the ZIndex=0 layer first as an opaque layer (opaque mode of render context can't be changed after first render)
			this._addLayerSet(0);

			// initalize scrollbars
			var boundingRect = this.getBoundingRect();
			var visibleRect = this.getVisibleRect();
			this._horizontalScroll = new Scrollbar("horizontal", boundingRect, visibleRect);
			this._verticalScroll = new Scrollbar("vertical", boundingRect, visibleRect);
			this._context.append(this._horizontalScroll.element).append(this._verticalScroll.element);

			// set the insets correctly, we want 5px padding + scrollbars shouldn't overlap canvas content
			this._sceneModel.setInsets(5, this._verticalScroll.element.width() + 5, this._horizontalScroll.element.height() + 5, 5);

			$(window).resize(_.bind(this.resized, this));


			// Init scene with view objects
			this._initializeViewObjects();

			this._observe(this._workflowModel, "*.added", this._workflowItemAdded);
			this._observe(this._workflowModel, "*.removed", this._workflowItemRemoved);
			this._observe(this._workflowModel, "Node.*.Pin.added", this._workflowItemAdded);
			this._observe(this._workflowModel, "Node.*.Pin.removed", this._workflowItemRemoved);
			this._observe(this._workflowModel, "Node.*.Pin.*.Token.added", this._workflowItemAdded);

			this._observe(this._sceneModel, "zoomChanged", this._renderAllLayers);
			this._observe(this._sceneModel, "panningOffsetChanged", this._renderAllLayers);
			this._observe(this._sceneModel, "beingModifiedFrequently", this._updateBeingModifiedFrequently);
			this._observe(this._sceneModel, "cursorChanged", this._changeCursor);
			this._observe(this._sceneModel, "highlightModeChanged", this._renderAllLayers);

			this._observe(this._sceneModel, "*", this._updateScrollbars);

			/**
			 * This non-visible input is used to mimick the focus on the scene. If this scene has focus, the scene has focus.
			 * Furthermore it is used to make sure copy/paste events get generated on all browsers.
			 * The mousetrap class is needed for mousetrap to enable key events if this input is focused
			 * @member {JQueryObject}
			 * @private
			 */
			this._inputForSceneFocus = $('<input type="text" />').addClass("input-for-copy mousetrap");
			this._context.prepend(this._inputForSceneFocus);
			this._inputForSceneFocus.on("blur", _.bind(function () {
				this._context.removeClass("focused");
				this._emit("blur");
			}, this));
			this._inputForSceneFocus.on("focus", _.bind(function (e) {
				this._context.addClass("focused");
				this._emit("focus");
				e.preventDefault();
			}, this));
			// Prevent typing in the scene input
			this._inputForSceneFocus.on("keydown", function (e) {
				var keyPressed = _.bind(e.originalEvent.getModifierState, e.originalEvent);
				// We cannot prevent typing if these are pressed because otherwise the copy/paste event won't be supported
				if (!(keyPressed("Meta") ||
						keyPressed("Control") ||
					 e.originalEvent.keyCode === 123 /*F12*/ || 
					 e.originalEvent.keyCode === 122 /*F11*/ || 
					 e.originalEvent.keyCode === 9 /*TAB*/ )) {   
					e.preventDefault();
				}
			});

		};

		/**
		 * Get the scene model
		 * @returns {SceneModel}
		 */
		Scene.prototype.getSceneModel = function () {
			return this._sceneModel;
		};

		/**
		 * Get the correct LayerSet for a specific view.
		 * if the LayerSet doesn't exist yet, it is created
		 * @param {AbstractView} viewObject
		 */
		Scene.prototype._getLayerSetForViewObject = function (viewObject) {
			var layer = this._layers[viewObject.getZIndex()];
			if (layer) {
				return layer;
			} else {
				// layer doesn't exist yet
				return this._addLayerSet(viewObject.getZIndex());
			}
		};

		/** 
		 * Add a LayerSet for a certain ZIndex
		 * @param {Number} ZIndex
		 * @returns {LayerSet}
		 */
		Scene.prototype._addLayerSet = function (ZIndex) {
			// search the next ZIndex which is already initialized
			var j = 0;
			while (j < this._initializedLayers.length && ZIndex > this._initializedLayers[j]) {
				j++;
			}
			var nextLayerZIndex = this._initializedLayers[j];

			var layerSet = this._layers[ZIndex] = new LayerSet(ZIndex, this, this._sceneModel, "ZIndex_" + ZIndex, j === 0);
			if (typeof nextLayerZIndex !== "undefined") {
				// there is already a layer initialized with a higer ZIndex
				var nextLayerSet = this._layers[nextLayerZIndex];
				// insert the layerSet before that existing LayerSet
				layerSet.getLayerCanvas().insertBefore(nextLayerSet.getLayerCanvas());
			} else {
				// insert the layerSet at the end of the scene context
				this._layerContext.append(layerSet.getLayerCanvas());
			}
			// insert the interactive canvas just behind the layer canvas
			layerSet.getInteractiveLayerCanvas().insertAfter(layerSet.getLayerCanvas());

			// mark the ZIndex as initialized
			this._initializedLayers.splice(j, 0, ZIndex);

			return layerSet;
		};

		/**
		 * Uses the view factory to create view objects from the workflow model
		 */
		Scene.prototype._initializeViewObjects = function () {
			var creation = this._viewFactory.createViewsForWorkflow(this._workflowModel, this);
			creation.done(_.bind(function (views) {
				this._addViews(views).done(_.bind(function () {
					this._isWorkflowLoaded = true;
					this._emit("workflowLoaded");
				}, this));
			}, this));

		};

		/**
		 * Add the view for WorkflowItem to the Scene
		 * @param {WorkflowItem} workflowItem
		 */
		Scene.prototype._workflowItemAdded = function (workflowItem) {
			if (!this._viewObjects[workflowItem.getId()]) {
				var views = {};
				this._viewFactory.createViewsForWorkflowItem(workflowItem, this._viewObjects, this, views);
				this._addViews(views, true);
			}
		};

		/**
		 * Fetch a list of nodes that interset with a certain world rect
		 * @param {ShapeUtils.Rect} selectionRect - rect in World Coordinates
		 * @returns {Map.<NodeId, Node>}
		 */
		Scene.prototype.getNodesInRect = function (selectionRect) {

			var nodesInRect = {};

			this._hitAreaTree.getObjectsIntersectingRect(selectionRect, function (view) {
				if (view.getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Node) {
					nodesInRect[view.getModel().getId()] = view.getModel();
				}
			});

			return nodesInRect;
		};

		/**
		 * Request an animation frame. All requests will be fired synchronously when the animation frame is given by the browser
		 * This is used by layers to enable synchronised rendering (in sync with the browser render speed
		 * @param {Layer} layer - the layer that wants to be rerendered
		 */
		Scene.prototype.requestAnimationFrame = function (layer) {
			var renderObject = {
				layer: layer,
				renderDuration: layer.getRenderDuration()
			};

			if (this._layersWaitingForAnimationFrame == null) {
				// didn't request an animation frame yet	
				this._layersWaitingForAnimationFrame = [renderObject];

				window.requestAnimationFrame(this._boundHandleAnimationFrame);
			} else {
				this._layersWaitingForAnimationFrame.push(renderObject);
			}
		};

		/**
		 * The browser has given us an animation frame. Render all layers that want to be rendered
		 */
		Scene.prototype._handleAnimationFrame = function () {

			var layersToBeRendered = this._layersWaitingForAnimationFrame;
			this._layersWaitingForAnimationFrame = null;



			// calculate how much time we will need to render all layers
			// TODO this can still be finetuned, now all layers that are slower that the slot per layer are buffered
			// however it could be enough to only buffer the slowest layer

			var totalRenderDuration = 0;
			var maxRenderDurationPerLayer = 20 / layersToBeRendered.length;
			if (this._sceneModel.isBeingModifiedFrequently()) {
				totalRenderDuration = _.reduce(layersToBeRendered, function (memo, layerObject) {
					return memo + layerObject.renderDuration;
				}, 0);
			}


			// render dirty layers
			_.each(layersToBeRendered, function (obj) {
				if (totalRenderDuration > 20 && obj.renderDuration > maxRenderDurationPerLayer) {
					// we don't have enough time to render everything
					// this layer will render to slow... use its buffer
					obj.layer.renderBuffer();
				} else {
					// layer will render fast enough, render it completely
					obj.layer.render();
				}
			}, this);

		};


		/**
		 * Removes the view for WorkflowItem from the Scene
		 * @param {WorkflowItem} workflowItem
		 */
		Scene.prototype._workflowItemRemoved = function (workflowItem) {
			var view = this._viewObjects[workflowItem.getId()];
			if (view) {
				//remove the view from the hit area tree
				this._hitAreaTree.removeView(view);

				// remove the view from its layerset
				var layerSet = this._layers[view.getZIndex()];
				if (layerSet) {
					layerSet.removeViewObject(view.getId());
				}

				// destroy the view
				view.destroyView();

				// forget the view
				delete this._viewObjects[workflowItem.getId()];
			}
		};

		/**
		 * Requests the scene to remove views for a workflow item.
		 * @param {WorkflowItem} workflowItem
		 */
		Scene.prototype.removeViewsForWorkflowItem = function (workflowItem) {
			this._workflowItemRemoved(workflowItem);
		};

		/**
		 * Add a list of view objects to the Scene
		 * @param {Map.<String,View>,Array.<View>} views - the views to be added to the Scene
		 * @param {Boolean} synchronized - if true all operations are executed synchronously
		 * @returns {Promise} fired when all operations are finished
		 */
		Scene.prototype._addViews = function (views, synchronized) {
			var queue = new ExecutionQueue(synchronized);
			queue.forEach(views, function (view) {
				this._viewObjects[view.getId()] = view;
				var layer = this._getLayerSetForViewObject(view);
				layer.addViewObject(view.getId(), view);

				// whenever a view starts an animation, make it interactive
				this._observe(view, "animationStarted", function () {
					this._updateInteractive(view);
				});
				this._observe(view, "animationFinished", function () {
					this._updateInteractive(view);
				});

			}, this, "SceneAddViews", this._i18n.t("initializingView"));

			queue.run(function () {
				// register all views in the hitarea tree
				this._hitAreaTree.registerViews(views);
			}, this, false, "SceneAddViews", this._i18n.t("initializingView"));

			var deferred = $.Deferred();

			queue.whenFinished(function () {
				deferred.resolve();
			});

			return deferred.promise();
		};




		/**
		 * Recalculate te bounding rect of the scene (result is stored in scenemodel)
		 */
		Scene.prototype.getBoundingRect = function () {
			var sceneBoundingRect = null;
			_.each(this._initializedLayers, function (ZIndex) {
				var layer = this._layers[ZIndex];

				var layerBoundingRect = layer.getBoundingRect();

				if (layerBoundingRect) {
					if (sceneBoundingRect) {
						sceneBoundingRect = ShapeUtils.calculateBoundingBoxRectArrays(sceneBoundingRect, layerBoundingRect);

					} else {
						sceneBoundingRect = layerBoundingRect;
					}
				}
			}, this);
			if (!sceneBoundingRect) {
				return new ShapeUtils.Rect();
			} else {
				return ShapeUtils.rectFromNormalizedRectArray(sceneBoundingRect);
			}
		};


		/**
		 * Get the view object belonging to the given workflow item model
		 * @param {WorkflowItem} workflowItem
		 * @returns {AbstractView} the view or null if no view exists
		 */
		Scene.prototype.getViewByWorkflowItem = function (workflowItem) {
			return this._viewObjects[workflowItem.getId()];
		};

		/**
		 * Get the transformation for the current state of the scene
		 * @returns {Tranformation}
		 */
		Scene.prototype.getTransformation = function () {
			var t = new Transformation();
			var offset = this._sceneModel.getOffset();
			var zoom = this._sceneModel.getZoom();

			t.scale(zoom, zoom);

			t.translate(Math.round(offset.width), Math.round(offset.height));

			if (this._sceneModel.isPanning()) {
				var panningOffset = this._sceneModel.getPanningOffset();
				t.translate(panningOffset.width, panningOffset.height);
			}


			return t;
		};

		/**
		 * Get the visible rectangle in world coordinates.
		 * @param {Boolean} includeInsets - should the scene insets be taken into acount?
		 * @returns {ShapeUtils.Rect} in world coordinates
		 */
		Scene.prototype.getVisibleRect = function (includeInsets) {
			var visibleRect;
			if (!includeInsets) {
				visibleRect = new ShapeUtils.Rect(new ShapeUtils.Point(),
					new ShapeUtils.Size(this._context.width(), this._context.height()));
			} else {
				var insets = this._sceneModel.getInsets();
				visibleRect = new ShapeUtils.Rect(new ShapeUtils.Point(insets.left, insets.top),
					new ShapeUtils.Size(this._context.width() - insets.right, this._context.height() - insets.bottom));
			}
			var transformation = this.getTransformation().createInverse();
			return transformation.transformedRect(visibleRect);
		};

		/**
		 * Get the given scene point in world coordinates
		 * @param   {ShapeUtils.Point} scenePoint - The Scene point to convert to world point
		 * @returns {ShapeUtils.Point} The world point from the scene point
		 */
		Scene.prototype.getWorldPointFromScenePoint = function (scenePoint) {
			var t = this.getTransformation().createInverse();
			return t.transformedPoint(scenePoint);
		};

		/**
		 * Get the given world point in scene coordinates
		 * @param   {ShapeUtils.Point} worldPoint - The world point to convert to scene point
		 * @returns {ShapeUtils.Point} The scene point from the world point
		 */
		Scene.prototype.getScenePointFromWorldPoint = function (worldPoint) {
			var t = this.getTransformation();
			return t.transformedPoint(worldPoint);
		};

		/**
		 * Returns the context in which the scene is rendered.
		 * @returns {JQueryObj}
		 */
		Scene.prototype.getContext = function () {
			return this._context;
		};

		/**
		 * Returns the context in which the layers sit, this is the preferred div to listen to events for tools
		 * @returns {JQueryObj} 
		 */
		Scene.prototype.getLayerContext = function () {
			return this._layerContext;
		};

		/**
		 * Get the object at a specific position in the workflow
		 * @param   {ShapeUtils.Point} location - Scene coordinates
		 * @returns {AbstractView|null}
		 */
		Scene.prototype.getObjectAtPosition = function (location) {

			var worldPoint = this.convertScenePointToWorld(location);
			var view = _.first(this._hitAreaTree.getTopLevelObjectAtPosition(worldPoint));

			if (view) {
				return view;
			} else {
				return null;
			}
		};

		/**
		 * returns all abstract views at a position
		 * @param   {ShapeUtils.Point} position - position in scene coordinates
		 * @param   {Function} filter -  function accepting an AbstractView as argument, return null to skip, return a view to add the view
		 * @returns {Array.<AbstractView>} 
		 */
		Scene.prototype.getObjectsAtPosition = function (location, filter) {
			var worldPoint = this.convertScenePointToWorld(location);
			return this._hitAreaTree.getObjectsAtPosition(worldPoint, filter);
		};

		/**
		 * returns the top hit area at a position
		 * @param   {ShapeUtils.Point} position - position in scene coordinates
		 * @returns {Array.<HitArea>}
		 */
		Scene.prototype.getHitAreaAtPosition = function (position) {
			var worldPoint = this.convertScenePointToWorld(position);
			var topLevelHitAreas = this._hitAreaTree.getTopLevelHitAreaAtPosition(worldPoint);

			// if there are multiple top level hit areas, we want the one that is rendered on top -> find highest z-index
			var highestViewZIndex = _.reduce(topLevelHitAreas, function (currentHighest, hitArea) {
				if (hitArea.getView().getZIndex() > currentHighest) {
					return hitArea.getView().getZIndex();
				} else {
					return currentHighest;
				}
			}, 0);
			var hitAreasWithHighestView = _.filter(topLevelHitAreas, function (hitArea) {
				return hitArea.getView().getZIndex() === highestViewZIndex;
			});

			if (hitAreasWithHighestView.length === 1) {
				return hitAreasWithHighestView[0];
			} else if (hitAreasWithHighestView.length === 0) {
				return null;
			}

			// There are multiple hit areas that have views on the highest zIndex, choose the one that got rendered last
			var viewsInLayer = this._layers[highestViewZIndex].getLayer().getVisibleViews();

			// get view with highest index
			var lastRenderedHitArea = null;
			_.reduce(hitAreasWithHighestView, function (currentHighest, hitArea) {
				var index = viewsInLayer.indexOf(hitArea.getView());
				if (index > currentHighest) {
					lastRenderedHitArea = hitArea;
					return index;
				} else {
					return currentHighest;
				}
			}, 0);

			if (lastRenderedHitArea) {
				return lastRenderedHitArea;
			} else {
				return null;
			}

		};


		/**
		 * Convert a rectangle in Scene coordinates to world coordinates
		 * @param {ShapeUtils.Rect} rect - Scene Coordindates
		 * @returns {ShapeUtils.Rect} rect in world coordinates
		 */
		Scene.prototype.convertSceneRectToWorld = function (rect) {
			return this.getTransformation().createInverse().transformedRect(rect);
		};

		/**
		 * Convert a point in Scene coordinates to world coordinates
		 * @param {ShapeUtils.Point} rect - Scene Coordindates
		 * @returns {ShapeUtils.Point} rect in world coordinates
		 */
		Scene.prototype.convertScenePointToWorld = function (point) {
			return this.getTransformation().createInverse().transformedPoint(point);
		};

		/**
		 * Get view objects near the given position within the given rectangular marge.
		 * For each found view object the filter will be called with the view object as argument to further filter down the result set.
		 * @param {ShapeUtils.Point} position - The position to check in Scene Coordinates
		 * @param {Number} marge - The marge in scene pixels
		 * @param {Function} filter - Function returning true if the given viewObject argument passes the filter
		 * @returns {Array.<AbstractView>} array containing the view objects
		 */
		Scene.prototype.getObjectsNearPosition = function (position, marge, filter) {
			var worldPoint = this.convertScenePointToWorld(position);
			return this._hitAreaTree.getObjectsNearPosition(worldPoint, marge * this._sceneModel.getZoom(), filter);
		};


		/**
		 * Set one of the viewobjects as interactive, this causes them to be rendered in
		 * the interactive layer.
		 * @param {String} id - Id of the view object
		 * @param {Boolean} isInteractive
		 */
		Scene.prototype.setInteractive = function (id, isInteractive) {
			var view = this._viewObjects[id];
			if (view) {
				view.setInteractive(isInteractive);
				this._updateInteractive(view);
			}
		};

		/**
		 * Move the view to the correct layer if needed
		 * when a view isInteractive or animating it should be in the interactive layer
		 * @param {AbstractView} view
		 */
		Scene.prototype._updateInteractive = function (view) {
			var isInteractive = view.isInteractive() || (view.isAnimating && view.isAnimating());

			this._layers[view.getZIndex()].setInteractive(view.getId(), isInteractive);

		};

		/**
		 * @private
		 * Make the default layer dirty
		 */
		Scene.prototype._makeDefaultLayerDirty = function () {
			this._renderAllLayers();
		};

		/**
		 * Get the workflow model
		 * @returns {Workflow}
		 */
		Scene.prototype.getWorkflowModel = function () {
			return this._workflowModel;
		};

		/**
		 * Renders all the layers of the scene
		 * @private
		 */
		Scene.prototype._renderAllLayers = function () {
			_.each(this._initializedLayers, function (ZIndex) {
				this._layers[ZIndex].makeDirty();
			}, this);
		};

		/**
		 * The workflow might be modified frequently or not...
		 * Clear buffers and trigger rerender of all layers if it was being modified requently and no longer is
		 */
		Scene.prototype._updateBeingModifiedFrequently = function (isBeingModifiedFrequently) {

			if (!isBeingModifiedFrequently) {
				_.each(this._initializedLayers, function (ZIndex) {
					this._layers[ZIndex].resetBuffer();
				}, this);

				this._renderAllLayers();
			}
		};

		/**
		 * Change the cursor in the scene to the one changed in the model.
		 * @param {String} oldCursor css class of the old cursor
		 * @param {String} newCursor css class of the new cursor
		 */
		Scene.prototype._changeCursor = function (oldCursor, newCursor) {
			this._layerContext.removeClass(oldCursor);
			this._layerContext.addClass(newCursor);
		};


		/**
		 * Get the scroll bars of the Scene
		 * @returns {Object} with horizontal and vertical scrollbar
		 */
		Scene.prototype.getScrollbars = function () {
			return {
				horizontal: this._horizontalScroll,
				vertical: this._verticalScroll
			};
		};

		/**
		 * Execute this method when the Scene was resized.
		 * This sets the dimensions of the canvases correctly and updates the scrollbars
		 */
		Scene.prototype.resized = function () {
			this._renderAllLayers();
			this._emit("resized");
			_.defer(_.bind(this._updateScrollbars, this));
		};
	
		/**
		 * indicates that the scene should be destroyed
		 */
		Scene.prototype.destroy = function () {
			this._emit("destroyed");
		};

		/**
		 * Update the scrollbars
		 */
		Scene.prototype._updateScrollbars = function () {
			var boundingRect = this.getBoundingRect();
			var visibleRect = this.getVisibleRect();
			this._horizontalScroll.setWorld(boundingRect, visibleRect);
			this._verticalScroll.setWorld(boundingRect, visibleRect);
		};

		/**
		 * Emits an event to requests a new workflow with given id to be loaded.
		 * @param {String} taskId
		 */
		Scene.prototype.requestNewWorkflow = function (taskId) {
			this._emit("requestNewWorkflow", taskId);
		};

		/**
		 * A token got selected in the view, emit an event where the controller can listen to.
		 * @param {Token} token
		 */
		Scene.prototype.tokenItemSelected = function (token) {
			this._emit("tokenItemSelected", token);
		};

		/**
		 * @returns {ShapeUtils.Size} the size of the scene
		 */
		Scene.prototype.getSceneSize = function () {
			return new ShapeUtils.Size(this._context.width(), this._context.height());
		};

		/**
		 * Get the view with given ID
		 * @param   {String} viewId
		 * @returns {AbstractView}
		 */
		Scene.prototype.getView = function (viewId) {
			return this._viewObjects[viewId];
		};

		/**
		 * Be notified when the workflow is initialized completely
		 * @param {Function} callback - called when the workflow is loaded
		 */
		Scene.prototype.whenWorkflowLoaded = function (callback) {
			if (this._isWorkflowLoaded) {
				callback();
			} else {
				this.once("workflowLoaded", callback);
			}
		};

		/**
		 * Get the location for a node (in Scene Pixels)
		 * @param   {Node}             node - node
		 * @returns {ShapeUtils.Point} Location in Scene Pixel
		 */
		Scene.prototype.getLocationForNode = function (node) {
			return this.getTransformation().transformedPoint(node.getLocation());
		};


		/**
		 * Notify the Scene that a tool is active. This should be used when the mouse events should not be interupted by other DOM elements in the Scene.
		 * Popups that are in the scene, will block pointer-events (thus the events will go through the popup, they won't break other events (i.e. for panning)
		 * @param {Boolean} isToolActive - true if pointerevents should not be blocked by another DOM element in the scene
		 */
		Scene.prototype.blockPointerEventsForDOMElements = function (blockPointerEvents) {
			this._context.toggleClass("blockPointerEvents", blockPointerEvents);
		};



		/**
		 * sets the focus on the scene
		 */
		Scene.prototype.focus = function () {
			this._inputForSceneFocus.val(" ").focus().select();
		};

		/** 
		 * @return {Boolean} is the scene focused?
		 */
		Scene.prototype.hasFocus = function () {
			return this._context.hasClass("focused");
		};

		/**
		 * Trigger a "beforeRemoveFromDom" event for when the wf editor is removed from dom.
		 */
		Scene.prototype.triggerRemoveFromDomEvent = function () {
			this._emit("beforeRemoveFromDom");
		};

		/**
		 * Trigger a "afterAddToDom" event for when the wf editor is (re-)added to the dom.
		 */
		Scene.prototype.triggerAddToDomEvent = function () {
			this._emit("afterAddToDom");
		};

		return Scene;

	});