define([
		"WFEditor/tools/ToolManager",
		"WFEditor/utils/WFI18nConstants",
		"utils/Observer",
		"utils/Command",
		"utils/ShapeUtils",
		"WFEditor/tools/ToolConstants",
		"WFEditor/tools/ZoomTool",
		"WFEditor/tools/PanTool",
		"WFEditor/tools/MouseInteractionTool",
		"WFEditor/tools/moveNodes/MoveNodesTool",
		"WFEditor/tools/ContextMenuTool",
		"WFEditor/tools/drawEdge/DrawEdgeTool",
		"WFEditor/tools/selection/SelectionTool",
		"WFEditor/tools/TooltipTool",
		"WFEditor/workflow/ModelConstants",
		"WFEditor/nodeConfiguration/NodeConfigurationController",
		"component/loadingindicator/LoadingIndicatorSingleton"
	],
	function (
		ToolManager,
		WFI18nConstants,
		Observer,
		Command,
		ShapeUtils,
		ToolConstants,
		ZoomTool,
		PanTool,
		MouseInteractionTool,
		MoveNodesTool,
		ContextMenuTool,
		DrawEdgeTool,
		SelectionTool,
		TooltipTool,
		ModelConstants,
		NodeConfigurationController,
		LoadingIndicator
	) {
		"use strict";

		/**
		 * Controller for the Scene of the workflow. The SceneController is resposible
		 * initializing all the interactive tools for the scene.
		 *
		 * @exports SceneController
		 * @class
		 *
		 * @param {AbstractWorkflowFactory} workflowFactory - the workflow factory to create objects
		 * @param {SceneModel} sceneModel
		 * @param {Workflow} workflowModel
		 * @param {Scene} scene
		 * @param {WFEditorContext} WFEditorContext
		 */
		var SceneController = function (workflowFactory, sceneModel, workflowModel, scene, WFEditorContext) {
			Observer.mixin(this);

			/**
			 * @member {AbstractWorkflowFactory} the workflow factory to create objects
			 * @private
			 */
			this._workflowFactory = workflowFactory;

			/**
			 * @member {SceneModel} model used to render the scene
			 * @private
			 */
			this._sceneModel = sceneModel;

			/**
			 *  @member {Workflow}
			 *  @private
			 */
			this._workflowModel = workflowModel;

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = scene;

			/** 
			 * @member {ToolbarModel}
			 */
			this._toolbarModel = WFEditorContext.get("toolbarModel");


			/**
			 * @member {ContextMenuModel}
			 */
			this._contextMenuModel = WFEditorContext.get("contextMenuModel");

			/**
			 * @member {I18n}
			 * @private
			 */
			this._i18n = WFEditorContext.get("i18n");

			/**
			 * @member {WFEditorUndoRedoController}
			 * @protected
			 */
			this._undoRedo = WFEditorContext.get("undoRedo");

			/**
			 * @member {ToolManager} main controller for all the interactive tools that use the scene
			 * @private
			 */
			this._toolManager = WFEditorContext.get("toolManager");

			/**
			 * @member {ErrorHandler}
			 * @private
			 */
			this._errorHandler = WFEditorContext.get("errorHandler");

			// init mouse interaction tool
			this._mouseInteractionTool = new MouseInteractionTool(this, WFEditorContext);
			this._toolManager.setDefaultTool(this._mouseInteractionTool);

			// init selection tool
			this._selectionTool = new SelectionTool(this, this._mouseInteractionTool, WFEditorContext);

			// init draw edge tool
			this._drawEdgeTool = new DrawEdgeTool(this, this._mouseInteractionTool, WFEditorContext);

			// init move nodes tool
			this._moveNodesTool = new MoveNodesTool(this, this._mouseInteractionTool, WFEditorContext);

			// init context menu tool
			this._contextMenuTool = new ContextMenuTool(this._mouseInteractionTool, WFEditorContext);

			// init tooltiptool
			new TooltipTool(this._mouseInteractionTool, WFEditorContext);

			// init zoom tool
			new ZoomTool(this, WFEditorContext);

			// init pan tool
			new PanTool(WFEditorContext);
            
            /**
			 * @member {WFEditorContext} 
			 * @private
			 */
            this._wfEditorContext = WFEditorContext;

			this._observe(this._scene, "tokenItemSelected", _.bind(this._tokenItemSelected, this));

			/**
			 * @member {Map.<String, Node>} a map of the selected nodes, used to temporary save the selected files when selecting aditional nodes by pressing CTRL and dragging
			 * @private
			 */
			this._tempNodesSelection = {};


			/**
			 * @member {Boolean} When selecting, should the current selection be cleared
			 */
			this._clearNodeSelection = true;

			// Add the Remove Nodes button
			// add delete button to the model
			if (this._workflowModel.getPrivilege(ModelConstants.Privileges.removeNodes)) {
				this._toolbarModel.addComponent(ToolConstants.ButtonGroups.REMOVENODES, "removeNodes");
				var removeSelectedNodesFn = _.bind(this._removeSelectedNodes, this);
				this._toolbarModel.addButton(ToolConstants.ButtonGroups.REMOVENODES, ToolConstants.Buttons.REMOVENODES, "icon-WCR_Trash", removeSelectedNodesFn, this._i18n.t("removeNodesTooltip"));
				this._contextMenuModel.addComponent(ToolConstants.ContextMenuGroups.NODE);
				this._contextMenuModel.addButton(ToolConstants.ContextMenuGroups.NODE, ToolConstants.Buttons.REMOVENODES, this._i18n.t("removeNodesContextMenuTitle"), "icon-WCR_Trash12 icon-normal", removeSelectedNodesFn, true, undefined, false);
				// delete button
				WFEditorContext.get("keyboardHandler").bind(["del", "backspace"], _.bind(this._removeSelectedNodes, this), "keydown");
				// updates of toolbar button
				this._updateRemoveNodesButton();
				this._observe(this._workflowModel, "nodeSelectionChanged", this._updateRemoveNodesButton);
			}

			/**
			 * @member {NodeConfigurationController}
			 * @private
			 */
			this._nodeConfigurationController = this._initializeNodeConfigurationController();

			this._listenForNodeDetails();
			if (this._workflowModel.getPrivilege(ModelConstants.Privileges.copyNodes)) {
				this._registerCopyPasteEvents();
			}


		};

		/**
		 * Update the state of the remove nodes button. Called when the selection changed
		 */
		SceneController.prototype._updateRemoveNodesButton = function () {
			this._toolbarModel.setButtonEnabled(ToolConstants.Buttons.REMOVENODES, this.canRemoveSelectedNodes());
			this._contextMenuModel.setButtonVisible(ToolConstants.Buttons.REMOVENODES, this.canRemoveSelectedNodes());
		};

		/**
		 * Update the state of the copy nodes context menu items. Called when the selection changed
		 */
		SceneController.prototype._updateCopyPasteButton = function () {
			this._contextMenuModel.setButtonEnabled(ToolConstants.Buttons.COPYNODES, this._canCopySelectedNodes() && document.queryCommandSupported('copy'));
			this._contextMenuModel.setButtonEnabled(ToolConstants.Buttons.CUTNODES, this._canCopySelectedNodes() && document.queryCommandSupported('cut'));
		};

		/**
		 * Can the selected nodes be copied?
		 * @returns {Boolean} true if some can be coppied
		 */
		SceneController.prototype._canCopySelectedNodes = function () {
			var selectedNodes = this._workflowModel.getSelectedNodes();
			return this._workflowModel.getPrivilege(ModelConstants.Privileges.copyNodes) && _.some(selectedNodes, function (node) {
				return node.isCopyable();
			});
		};

		/**
		 * Can the selected nodes be removed?
		 * @returns {Boolean} true if the selected nodes can be removed
		 */
		SceneController.prototype.canRemoveSelectedNodes = function () {
			var selectedNodes = this._workflowModel.getSelectedNodes();
			return this._workflowModel.getPrivilege(ModelConstants.Privileges.removeNodes) && _.some(selectedNodes, function (node) {
				return node.isRemovable();
			});
		};

		/**
		 * Remove all selected nodes
		 */
		SceneController.prototype._removeSelectedNodes = function (e) {
			if (this.canRemoveSelectedNodes()) {
				var selectedNodes = this._workflowModel.getSelectedNodes();
				if (selectedNodes) {
					this.removeNodes(this._workflowModel, selectedNodes);
					this.clearSelection();
				}
				if (e) {
					e.preventDefault();
					e.stopPropagation();
				}
			}
		};

		/**
		 * A token got selected in the view, change the correct settings in the model.
		 * @param {Token} token
		 */
		SceneController.prototype._tokenItemSelected = function (token) {
			this._workflowModel.setHighlightedToken(token);
			this.resetHighlights();
		};

		/**
		 * Highlight the entire path the token has traveled through the workflow.
		 *
		 * @param {Token} token      Start token to reconstruct the path from (with its parent tokens)
		 * @param {Boolean} highlight  should the path be highlighted or not
		 */
		SceneController.prototype._highlightTokenPath = function (token, highlight) {
			var tokens = [token]; //make it an array to get started.
			var nextTokens;

			var runOverTokens = function (nextToken) { //runs over all tokens
				var parentPin = null;
				var pin = nextToken.getPin(); //get the pin the token belongs to
				//get all the parent tokens (can be multiple)
				_.each(nextToken.getParents(), function (parentToken) {
					parentPin = parentToken.getPin(); //get the pin
					parentToken.setHighlighted(highlight);
					if (parentPin) {
						parentPin.setHighlighted(highlight); //indicate that the pin belongs to the path 
						parentPin.getNode().setHighlighted(highlight);
						//get the edge that connects two pins (if there is one)
						var edge = parentPin.getEdgeTo(pin);
						if (edge) { //indicate that the edge is part of the path, it needs to be highlighted
							edge.setHighlighted(highlight);
						}
					}
					nextTokens.push(parentToken); //the parent token is part of the next iteration
				}, this);
			};

			while (tokens.length > 0) { //as long as there are tokens on the path, keep going
				nextTokens = []; //make sure the next tokens collection is empty, constructs all the tokens of the next iteration
				_.each(tokens, runOverTokens, this);
				//set new collection of tokens ready for the next iteration
				tokens = nextTokens;
			}
		};


		/**
		 * Selects nodes and lets it be undo/redoable
		 * @param {Map.<String, Node>} oldSelection - a map of the nodes that were selected
		 * @param {Map.<String, Node>} newSelection - a map of the nodes that should be selected
		 * @param {Boolean} skipDo
		 */
		SceneController.prototype._executeNodeSelectionCommand = function (oldSelection, newSelection, skipDo) {
			// make copies of the maps (so other people won't change them anymore)
			oldSelection = _.clone(oldSelection);
			newSelection = _.clone(newSelection);
			var command = new Command(
				_.bind(function () {
					this._workflowModel.setSelectedNodes(newSelection);
				}, this),
				_.bind(function () {
					this._workflowModel.setSelectedNodes(oldSelection);
				}, this),
				this._i18n.t(WFI18nConstants.selectNodesDescription)

			);
			this._undoRedo.execute(command, skipDo);
		};

		/**
		 * Start a node selection action
		 * this will make a copy of the current selection
		 * clear the current selection
		 * @param {Boolean} clear the current selection?
		 */
		SceneController.prototype.startSelectionSequence = function (clearSelection) {
			this._tempNodesSelection = _.clone(this._workflowModel.getSelectedNodes());
			this._clearNodeSelection = clearSelection;
		};

		/**
		 * Clear the current selection (this action is added to undo/redo stack, no need for start/stopSelectionSequence)
		 */
		SceneController.prototype.clearSelection = function () {
			this.startSelectionSequence();
			this._workflowModel.setSelectedNodes({});
			this.stopSelectionSequence();
		};

		/**
		 * This will finish a node selection sequence
		 * it will execute a command on the undoredocontroller with correct selection information for do and undo actions
		 */
		SceneController.prototype.stopSelectionSequence = function () {
			this._tempNodesSelection = {};
			this._clearNodeSelection = true;
		};


		/**
		 * Select a specific node. (this action is added to undo/redo stack, no need for start/stopSelectionSequence)
		 * @param {Node} node - the node to select
		 * @param {Boolean} clearPreviouslySelected - if true, all currently selected nodes are deselected
		 */
		SceneController.prototype.selectNode = function (node, clearPreviouslySelected) {
			if (clearPreviouslySelected) {
				var selection = {};
				selection[node.getId()] = node;
				this._workflowModel.setSelectedNodes(selection);
			} else {
				this._workflowModel.selectNode(node);
			}
		};

		/**
		 * Select all nodes in a certain rect (note: to make the action undoable use start/stopSelectingSequence)
		 * @param {ShapeUtils.Rect} selectionRect - Selection rectangle in world coordinates
		 */
		SceneController.prototype.selectNodesInRect = function (selectionRect) {
			var nodesInRect = this._scene.getNodesInRect(selectionRect);
			if (this._clearNodeSelection) {
				this._workflowModel.setSelectedNodes(nodesInRect);
			} else {
				this._workflowModel.setSelectedNodes(_.extend(nodesInRect, this._tempNodesSelection));
			}
		};


		/**
		 * Deselect a node (this action is added to undo/redo stack, no need for start/stopSelectionSequence)
		 * @param {Node} node - node to deselect
		 */
		SceneController.prototype.deselectNode = function (node) {
			this._workflowModel.deselectNode(node);
		};


		/**
		 * Create an edge in workflow from fromPin to toPin
		 * @param {Workflow} workflow
		 * @param {Pin} fromPin
		 * @param {Pin} toPin
		 * @param {Boolean} undoable - True if the action should be undoable
		 * @param {Boolean} [skipDo = false] - True if it should not be executed, but only pushed to the undo/redo stack
		 */
		SceneController.prototype.createEdge = function (workflow, fromPin, toPin, undoable, skipDo) {
			if (undoable) {
				this._undoRedo.execute(new Command(
					_.bind(function () { // do
						this._doCreateEdge(workflow, fromPin, toPin);
					}, this),
					_.bind(function () { // undo
						var edge = fromPin.getEdgeTo(toPin);
						this._doRemoveEdge(workflow, edge);
					}, this),
					this._i18n.t(WFI18nConstants.addEdgeDescription)), skipDo);
			} else {
				if (!skipDo) {
					this._doCreateEdge(workflow, fromPin, toPin);
				}
			}
		};

		/**
		 * @private
		 * Do create an edge
		 * @param {Workflow} workflow
		 * @param {Pin} fromPin
		 * @param {Pin} toPin
		 */
		SceneController.prototype._doCreateEdge = function (workflow, fromPin, toPin) {
			// accepts the edge and the edge does not yet exist
			if (toPin.acceptsIncomingEdges(fromPin) && !fromPin.getEdgeTo(toPin)) {
				// find next possible id
				var i = 0;
				var checkExistingEdgeId = function (edge) {
					return edge.getId() === fromPin.getId() + "/" + i;
				};
				while (_.find(fromPin.getEdges(), checkExistingEdgeId)) {
					i++;
				}

				var edge = this._workflowFactory.createEdge(workflow, fromPin.getId() + "/" + i, fromPin, toPin);
				fromPin.addEdge(edge);
				toPin.addEdge(edge);
				workflow.addEdge(edge);
			}
		};

		/**
		 * Remove edges from the workflow
		 * @param {Workflow} workflow
		 * @param {Array.<Edge>} edges
		 * @param {Boolean} undoable - True if the action should be undoable
		 * @param {Boolean} [skipDo = false] - True if it should not be executed, but only pushed to the undo/redo stack
		 */
		SceneController.prototype.removeEdges = function (workflow, edges, undoable, skipDo) {
			if (undoable) {
				this._undoRedo.execute(new Command(
					_.bind(function () { // do
						_.each(edges, function (edge) {
							var edgeToRemove = edge.getFromPin().getEdgeTo(edge.getToPin());
							this._doRemoveEdge(workflow, edgeToRemove);
						}, this);
					}, this),
					_.bind(function () { // undo
						_.each(edges, function (edge) {
							this._doCreateEdge(workflow, edge.getFromPin(), edge.getToPin());
						}, this);
					}, this),
					this._i18n.t(WFI18nConstants.removeEdgeDescription)), skipDo);
			} else {
				if (!skipDo) {
					_.each(edges, function (edge) {
						var edgeToRemove = edge.getFromPin().getEdgeTo(edge.getToPin());
						this._doRemoveEdge(workflow, edgeToRemove);
					}, this);
				}
			}
		};

		/**
		 * @private
		 * Do the removing
		 * @param {Workflow} workflow
		 * @param {Edge} edge
		 */
		SceneController.prototype._doRemoveEdge = function (workflow, edge) {
			if (edge) {
				var fromPin = edge.getFromPin();
				var toPin = edge.getToPin();
				fromPin.removeEdge(edge);
				toPin.removeEdge(edge);
				workflow.removeEdge(edge);
			}
		};

		/**
		 * Move edges to the newToPin
		 * @param {Workflow} workflow
		 * @param {Array.<Edge>} edges
		 * @param {Pin} newToPin
		 * @param {Boolean} [skipDo = false]
		 */
		SceneController.prototype.moveEdges = function (workflow, edges, newToPin, skipDo) {
			this._undoRedo.execute(new Command(
				_.bind(function () { // do
					_.each(edges, function (edge) {
						var edgeToRemove = edge.getFromPin().getEdgeTo(edge.getToPin());
						this._doRemoveEdge(workflow, edgeToRemove);
						this._doCreateEdge(workflow, edge.getFromPin(), newToPin);
					}, this);
				}, this),
				_.bind(function () { // undo
					_.each(edges, function (edge) {
						var edgeToRemove = edge.getFromPin().getEdgeTo(newToPin);
						this._doRemoveEdge(workflow, edgeToRemove);
						this._doCreateEdge(workflow, edge.getFromPin(), edge.getToPin());
					}, this);
				}, this),
				this._i18n.t(WFI18nConstants.moveEdgeDescription)), skipDo);
		};

		/**
		 * Move one or more workflow nodes
		 * @param {Array.<Node>} nodes
		 * @param {Number} deltaX - movement on X-axis (in world coordinates)
		 * @param {Number} deltaY - movement on Y-axis (in world coordinates)
		 * @param {Boolean} undoable - True if the action should be undoable
		 * @param {Boolean} [skipDo = false] - True if it should not be executed, but only pushed to the undo/redo stack
		 */
		SceneController.prototype.moveNodes = function (nodes, deltaX, deltaY, undoable, skipDo) {
			if (undoable) {
				this._undoRedo.execute(new Command(
					_.bind(function () { // do
						_.each(nodes, function (node) {
							var newLocation = node.getLocation().copy();
							newLocation.offsetXY(deltaX, deltaY);
							node.setLocation(newLocation.round());
						}, this);
					}, this),
					_.bind(function () { // undo
						_.each(nodes, function (node) {
							var newLocation = node.getLocation().copy();
							newLocation.offsetXY(-deltaX, -deltaY);
							node.setLocation(newLocation.round());
						}, this);
					}, this),
					this._i18n.t(WFI18nConstants.moveNodesDescription)), skipDo);
			} else {
				if (!skipDo) {
					_.each(nodes, _.bind(function (node) {
						var newLocation = node.getLocation().copy();
						newLocation.offsetXY(deltaX, deltaY);
						node.setLocation(newLocation.round());
					}, this));
				}
			}
		};

		/**
		 * Remove one or more workflow nodes (undoable)
		 * @param {Workflow} workflow
		 * @param {Array.<Node>} nodes
		 */
		SceneController.prototype.removeNodes = function (workflow, nodes) {
			this._undoRedo.execute(new Command(
				_.partial(function (sceneController) {
					// Create state object each time as edges get recreated when redo function is called to make sure there is no Id duplication
					this.state = sceneController._createNodesStateObject(_.filter(nodes, function (node) {
						return node.isRemovable();
					}));
					LoadingIndicator.start("removeNodesDo", sceneController._i18n.t(WFI18nConstants.removeNodesDo));
					_.defer(_.bind(function () {
						sceneController._doRemoveNodes(workflow, this.state);
						LoadingIndicator.stop("removeNodesDo");
					}, this));
				}, this),
				_.partial(function (sceneController) {
					LoadingIndicator.start("removeNodesUndo", sceneController._i18n.t(WFI18nConstants.removeNodesUndo));
					_.defer(_.bind(function () {
						sceneController._doAddNodes(workflow, this.state);
						LoadingIndicator.stop("removeNodesUndo");
					}, this));
				}, this),
				this._i18n.t(WFI18nConstants.removeNodesDescription)), false);
		};

		/**
		 * Add one or more workflow nodes (undoable)
		 * @param {Workflow}     workflow
		 * @param {Array.<Node> | Object.<Node>} nodes
		 * @param {Boolean} [skipDo = false]
		 */
		SceneController.prototype.addNodes = function (workflow, nodes, skipDo) {
			this._scene.focus();
			var command = new Command(
				_.partial(function (sceneController) {
					_.defer(_.bind(function () {
						sceneController._doAddNodes(workflow, this.state);
						// Create state object each time as edges get recreated adding nodes to make sure there is no Id duplication
						this.state = sceneController._createNodesStateObject(nodes);
					}, this));
				}, this),
				_.partial(function (sceneController) {
					_.defer(_.bind(function () {
						sceneController._doRemoveNodes(workflow, this.state);
					}, this));
				}, this), this._i18n.t(WFI18nConstants.addNodeDescription));
			command.state = this._createNodesStateObject(nodes);
			this._undoRedo.execute(command, skipDo);
		};

		/**
		 * Removes the nodes given in stateObject from workflow
		 * @param {Workflow} workflow    - workflow
		 * @param {Object}   stateObject - object with all nodes, pins and edges that should be removed, can be created with _createNodesStateObject
		 */
		SceneController.prototype._doRemoveNodes = function (workflow, stateObject) {
			_.each(stateObject.edges, function (edge) {
					this._doRemoveEdge(workflow, edge);
				},
				this);
			_.each(stateObject.nodes, function (nodeObject) {
				nodeObject.node.removeAllPins();
				workflow.removeNode(nodeObject.node);
			});
		};

		/**
		 * adds the nodes given in stateObject to workflow
		 * @param {Workflow} workflow    - workflow
		 * @param {Object}   stateObject - object with all nodes, pins and edges that should be removed, can be created with _createNodesStateObject
		 */
		SceneController.prototype._doAddNodes = function (workflow, stateObject) {
			if (stateObject.nodes) {
				_.each(stateObject.nodes, function (nodeObject) {
					// add node
					var node = nodeObject.node;
					workflow.addNode(node); // if we implement add nodes use that method instead of re-adding nodes with the same ID
					// add pins
					_.each(nodeObject.pins, function (pin) {
						if (!node.getPin(pin.getId())) {
							node.addPin(pin);
						}
					});
				});
			}
			if (stateObject.edges) {
				// add edges
				_.each(stateObject.edges, function (edge) {
					this._doCreateEdge(workflow, edge.getFromPin(), edge.getToPin());
				}, this);
			}
		};

		/**
		 * creates a state object that can be used to remove nodes from or add nodes to the workflow
		 * @param   {Array.<Node> | Object.<Node>} nodes  - the nodes of which a state object should be made
		 * @returns {Object} 
		 */
		SceneController.prototype._createNodesStateObject = function (nodes, filter) {
			var stateObject = {};
			stateObject.nodes = {};
			stateObject.edges = [];

			_.each(nodes, function (node) {
				// node
				stateObject.nodes[node.getId()] = {
					node: node
				};
				// pins
				stateObject.nodes[node.getId()].pins = node.getPins();
				// edges
				_.each(node.getPins(), function (pin) {
					_.each(pin.getEdges(), function (edge) {
						stateObject.edges.push(edge);
					});
				});
			});
			stateObject.edges = _.uniq(stateObject.edges);
			return stateObject;
		};

		/**
		 * Make sure node details are opened when needed
		 */
		SceneController.prototype._listenForNodeDetails = function () {
			if (this._workflowModel.getPrivilege(ModelConstants.Privileges.viewNodeConfiguration)) {
				this._observe(this._mouseInteractionTool, "dblclick", function (details) {
					if (details.hitArea && details.hitArea.hasClass(ModelConstants.HitAreaClass.NodeSelect)) {
						var node = details.hitArea.getView().getModel();
						if (node.isConfigurable()) {
							this.openNodeConfiguration(details.hitArea.getView().getModel());
						}
					}
				});
			}
		};

		/**
		 * Initialize the controller responsible for configuring nodes
		 * @returns {NodeConfigurationController} 
		 */
		SceneController.prototype._initializeNodeConfigurationController = function () {
			return new NodeConfigurationController(this._scene, this._i18n, this._undoRedo, this._wfEditorContext);
		};

		/**
		 * Open the details for a node
		 * @param {Node} node - the node for which the configuration should be opened
		 */
		SceneController.prototype.openNodeConfiguration = function (node) {
			this._nodeConfigurationController.openNodeConfiguration(node);
		};

		/**
		 * Resets highlights: active nodes are selected, if there is a selected token, 
		 * path will also be shown
		 */
		SceneController.prototype.resetHighlights = function () {
			this.removeAllHighlights();
			this.highlightActiveNodes();
			this.highlightErrorNodes();
			var highlightedToken = this._workflowModel.getHightLightedToken();
			if (highlightedToken) {
				highlightedToken.setSelected(true);
				this._highlightTokenPath(highlightedToken, true);
			}
		};

		/**
		 * Highlight active nodes
		 */
		SceneController.prototype.highlightActiveNodes = function () {
			_.each(this._workflowModel.getNodes(), function (node) {
				var inputPin = node.getInputPin();
				if (inputPin) {
					var tokens = inputPin.getTokens();
					if (tokens) {
						for (var i = 0; i < tokens.length; i++) {
							var active = false;
							if (!tokens[i].isProcessed()) {
								tokens[i].setHighlighted(true);
								active = true;
							}
							if (active) {
								node.setHighlighted(true);
							}
						}
					}
				}
			});
		};

		/**
		 * Highlight nodes in error state
		 */
		SceneController.prototype.highlightErrorNodes = function () {
			_.each(this._workflowModel.getTokens(), function (token) {
				if (token.hasErrors()) {
					var pin = token.getPin();
					if (pin) {
						var node = pin.getNode();
						if (node) {
							node.setHighlighted(true);
						}
					}
				}
			});
		};

		/**
		 * Remove highlight from all nodes
		 */
		SceneController.prototype.removeAllHighlights = function () {
			_.each(this._workflowModel.getNodes(), function (node) {
				node.setHighlighted(false);
			});
			_.each(this._workflowModel.getEdges(), function (edge) {
				edge.setHighlighted(false);
			});
			_.each(this._workflowModel.getTokens(), function (token) {
				token.setHighlighted(false);
				token.setSelected(false);
			});
		};


		/**
		 * This method sets the zoom and offset of the seen so that the given parameter will become the visible rect
		 * @param {ShapeUtils.Rect}
		 */
		SceneController.prototype.zoomToRect = function (boundingRect, maxZoomLevel) {
			var contextRect = this._scene.getContext()[0].getBoundingClientRect();

			var insets = this._sceneModel.getInsets();

			// calculate the real width/height we have for the canvas
			var contextRectHeight = contextRect.height - insets.top - insets.bottom;
			var contextRectWidth = contextRect.width - insets.left - insets.right;

			//use the height and width to calculate which zoom is needed to fit everything horizontal and vertical
			var verticalZoom = boundingRect.size.height / contextRectHeight;
			var horizontalZoom = boundingRect.size.width / contextRectWidth;

			//take zoom that is highest to be able to fit everything
			var zoom = 1 / Math.max(verticalZoom, horizontalZoom);
			if (maxZoomLevel && zoom > ToolConstants.ZoomLevel[maxZoomLevel]) {
				zoom = maxZoomLevel;
			}

			var bboxCenterX = (boundingRect.origin.x + (boundingRect.size.width / 2));
			var bboxCenterY = (boundingRect.origin.y + (boundingRect.size.height / 2));

			// insets.top and insets.left need to be taken in to account:
			// we want to use the center of the rectangle defined by the insets
			// contextrectWidth and height already have the insets subtracted from them
			var sceneCenterX = (contextRectWidth / 2 + insets.left) / zoom;
			var sceneCenterY = (contextRectHeight / 2 + insets.top) / zoom;

			//get the offsets to place the workflow in the middle of the scene 
			var widthOffset = Math.round(bboxCenterX - sceneCenterX);
			var heightOffset = Math.round(bboxCenterY - sceneCenterY);


			// center the workflow in the canvas
			this._sceneModel.setOffset(new ShapeUtils.Size(-widthOffset, -heightOffset), true);
			this._sceneModel.setZoom(zoom);
		};


		/**
		 * registers all relevant copy/cut/paste events and context menu items
		 */
		SceneController.prototype._registerCopyPasteEvents = function () {

			// copy event
			window.document.addEventListener('copy', _.bind(function (e) {
				if (this._scene.hasFocus()) {
					var selectedNodes = _.filter(this._workflowModel.getSelectedNodes(), function (node) {
						return node.isCopyable();
					});
					if (selectedNodes.length > 0) {
						// Clipboard is available for chrome & ff using the e.clipboardData, and for IE by using window.clipboardData
						this._copyNodesToClipboard(selectedNodes, (e && e.clipboardData) || window.clipboardData);
					}
					e.preventDefault();

				}
			}, this));
			// cut event
			window.document.addEventListener('cut', _.bind(function (e) {
				if (this._scene.hasFocus()) {
					var selectedNodes = _.filter(this._workflowModel.getSelectedNodes(), function (node) {
						return node.isCopyable();
					});
					if (selectedNodes.length > 0) {
						this._copyNodesToClipboard(selectedNodes, (e && e.clipboardData) || window.clipboardData);
						this.removeNodes(this._workflowModel, selectedNodes);
					}
					e.preventDefault();

				}
			}, this));

			window.document.addEventListener('paste', _.bind(function (e) {
				if (this._scene.hasFocus()) {
					try {
						var clipboardData = e.clipboardData || window.clipboardData;
						if (clipboardData) {
							// Text is the only type that is supported in Interne..., all browsers.
							var json = clipboardData.getData("Text");
							var parsedJson = JSON.parse(json);
							this._createNodesFromPaste(parsedJson);
						}
					} catch (event) {
						// this wasn't a paste which contained workflow data
					}
					e.preventDefault();
				}

			}, this));

			// Add copy and cut buttons to context menu, There currently is no browser support for paste on non user initiated paste events
			this._contextMenuModel.addComponent(ToolConstants.ContextMenuGroups.NODE);
			this._contextMenuModel.addButton(ToolConstants.ContextMenuGroups.NODE, ToolConstants.Buttons.COPYNODES, document.queryCommandSupported('copy') ? this._i18n.t(WFI18nConstants.copyNodesContextMenuTitle) : this._i18n.t(WFI18nConstants.copyNotSupportedShort), "icon-WCR_Actions-Copy12 icon-normal", _.bind(function () {
				this._scene.focus();
				try {
					document.execCommand("copy", false, null);
				} catch (e) {
					this._errorHandler.warning(this._i18n.t(WFI18nConstants.copyNotSupported));
				}
			}, this), false);

			this._contextMenuModel.addButton(ToolConstants.ContextMenuGroups.NODE, ToolConstants.Buttons.CUTNODES, document.queryCommandSupported('cut') ? this._i18n.t(WFI18nConstants.cutNodesContextMenuTitle) : this._i18n.t(WFI18nConstants.cutNotSupportedShort), "icon-WCR_WF-Cut12 icon-normal", _.bind(function () {
				this._scene.focus();
				try {
					document.execCommand("cut", false, null);
				} catch (e) {
					this._errorHandler.warning(this._i18n.t(WFI18nConstants.cutNotSupported));
				}
			}, this), false);
			// Add a disabled button for pasting, this context menu button should inform the user to use ctrl+v or cmd+v to paste
			this._contextMenuModel.addButton(ToolConstants.ContextMenuGroups.NODE, ToolConstants.Buttons.PASTENODES, this._i18n.t(WFI18nConstants.pasteNodesContextMenuTitle), "icon-WCR_WF-Paste12 icon-normal", function () {}, false);
			this._observe(this._workflowModel, "nodeSelectionChanged", this._updateCopyPasteButton);
		};

		/**
		 * copies nodes to the clipboard
		 * @param {Array.<Node>} nodes
		 * @param {ClipboardData} clipboardData
		 */
		SceneController.prototype._copyNodesToClipboard = function (selectedNodes, clipboardData) {
			var length = _.size(selectedNodes);

			if (length > 0) {
				var json = JSON.stringify(this._getObjectForCopy(selectedNodes));
				if (clipboardData) {
					// Text is the only type that is supported in Interne..., all browsers.
					clipboardData.setData("Text", json);
				}
			}
		};

		/**
		 * transforms the nodes into json by calling getStateObjectForCopy() on each node in selectedNodes. This object is serialized by using JSON.stringify
		 * @param {Array.<Node>} selectedNodes
		 * @return {String}
		 */
		SceneController.prototype._getObjectForCopy = function (selectedNodes) {
			var json = {};

			// locations for the nodes
			var locations = [];

			// state objects for copied nodes
			var nodes = [];

			// edges between two nodes that are in selectedNodes
			var edges = [];
			_.each(selectedNodes, function (node) {
				nodes.push(node.getStateObjectForCopy());
				locations.push({
					x: node.getLocation().x,
					y: node.getLocation().y
				});
				// Add all edges that are contained within the selected nodes (so to and from node should be in selected nodes);
				_.each(node.getPins(), function (pin) {
					edges.push.apply(edges, _.filter(pin.getEdges(), function (edge) {
						if (edge.getToPin().getNode() === node) {
							var fromNode = edge.getFromPin().getNode();
							return _.find(selectedNodes, function (node) {
								return node === fromNode;
							});
						}
					}));
				});
			});

			// Map the edges to a state object containing the indexes of the from and to node in the nodes array and index of the to and from pin in the node
			edges = _.map(edges, function (edge) {
				var toPin = edge.getToPin();
				var toNode = toPin.getNode();
				var fromPin = edge.getFromPin();
				var fromNode = fromPin.getNode();
				return {
					to: {
						nodeIndex: _.indexOf(selectedNodes, toNode),
						pinIndex: toNode.getPinIndex(toPin.getId())
					},
					from: {
						nodeIndex: _.indexOf(selectedNodes, fromNode),
						pinIndex: fromNode.getPinIndex(fromPin.getId())
					}
				};
			});
			json.nodes = nodes;
			json.locations = locations;
			json.edges = edges;
			return json;
		};

		/**
		 * creates nodes and edges from the json returned by _getObjectForCopy. 
		 * Nodes will be created by using the AbstractWorkflowFactory's createNode method, giving as parameters the workflow to which it will be added and the stateObject that was
		 * created by getStateObjectForCopy()
		 * @param {Object} nodesObject - as created by _getObjectForCopy
		 */
		SceneController.prototype._createNodesFromPaste = function (nodesObject) {

			// The state objects created from the original nodes
			var originalNodes = nodesObject.nodes;

			// the locations of the original nodes
			var locations = nodesObject.locations;

			// the edges between the original nodes
			var edges = nodesObject.edges;

			if (_.size(originalNodes) > 0) {
				var promises = [];
				_.each(originalNodes, function (node) {
					var promise = this._workflowFactory.createNode(this._workflowModel, node);
					if (promise) {
						promises.push(promise);
					}
				}, this);

				// Check if the center of the original locations is within the current visible rect
				var visibleRect = this._scene.getVisibleRect();
				var nodesMap = {};

				var left = Infinity,
					right = -Infinity,
					top = Infinity,
					bottom = -Infinity,
					offsetX = 0,
					offsetY = 0;
				_.each(locations, function (location) {
					left = location.x < left ? location.x : left;
					right = location.x > right ? location.x : right;
					top = location.y < top ? location.y : top;
					bottom = location.y > bottom ? location.y : bottom;
				});
				var centerOfCopiedNodes = new ShapeUtils.Point((left + right) / 2, (top + bottom) / 2);

				// If the center of the locations is in the visible rect, get add a small offset to the original locations, 
				//otherwise, put the center of the copied nodes to the center of the visible rect
				if (visibleRect.containsPoint(centerOfCopiedNodes)) {
					offsetX = 25;
					offsetY = 25;
				} else {
					var centerOfVisibleRect = visibleRect.center();
					offsetX = centerOfVisibleRect.x - centerOfCopiedNodes.x;
					offsetY = centerOfVisibleRect.y - centerOfCopiedNodes.y;
				}

				// counter used to see if all promises were resolved/rejected
				var nodePromisesHandled = 0;

				// deferred that will be resolved if all promises are handled
				var allNodesHandledDeferred = $.Deferred();

				// array of all newly created nodes
				var newNodes = [];

				_.each(promises, function (promise, i) {
					// create the nodes
					var location = locations[i];
					promise.done(_.bind(function (newNode) {
						// the node was successfully created
						nodesMap[newNode.getId()] = newNode;
						location = new ShapeUtils.Point(location.x + offsetX, location.y + offsetY);
						newNode.setLocation(location);
						this._workflowModel.addNode(newNode);
						newNodes[i] = newNode;
					}, this)).fail(_.bind(function (errorMsg) {
						// If the promise gets rejected and the node cannot be created, show Error
						this._errorHandler.warning.apply(this._errorHandler, arguments);

						// Set newNodes[i] to undefined to make sure that the edges get created correctly 
						// as the edges state object references the nodes through index in the nodes array
						newNodes[i] = undefined;
					}, this)).always(function () {
						// resolve the allNodesHandledDeferred if all nodes were handled
						nodePromisesHandled++;
						if (nodePromisesHandled === originalNodes.length) {
							allNodesHandledDeferred.resolve();
						}
					});
				}, this);

				allNodesHandledDeferred.done(_.bind(function () {
					_.each(edges, function (edgeObject) {
						// If the original nodes between the edges were copied, we can create the copied edge
						if (newNodes[edgeObject.from.nodeIndex] && newNodes[edgeObject.to.nodeIndex]) {
							var fromPin = newNodes[edgeObject.from.nodeIndex].getPins()[edgeObject.from.pinIndex];
							var toPin = newNodes[edgeObject.to.nodeIndex].getPins()[edgeObject.to.pinIndex];
							var edge = this._workflowFactory.createEdge(this._workflowModel, fromPin.getId() + "/" + _.size(fromPin.getEdges()), fromPin, toPin);
							fromPin.addEdge(edge);
							toPin.addEdge(edge);
							this._workflowModel.addEdge(edge);
						}
					}, this);

					// create the undo/redo command, but don't execute it as the nodes were already added to the workflow
					this.addNodes(this._workflowModel, nodesMap, true);
					this._workflowModel.setSelectedNodes(nodesMap);


				}, this));
			}
		};



		return SceneController;

	});
