define([
		"utils/Observable",
		"utils/ShapeUtils",
		"utils/StringSet",
		"WFEditor/tools/ToolConstants",
		"underscore"
	],
	function (
		Observable,
		ShapeUtils,
		StringSet,
		ToolConstants,
		_
	) {
		"use strict";

		/**
		 * Represents the current state of the scene.
		 *
		 * @exports SceneModel
		 * @class
		 *
		 * @param {String} sceneId - id for this scene model, should be unique for the obversable
		 */
		var SceneModel = function (sceneId, WFContext) {
			Observable.mixin(this, sceneId, "WFEditor.Scene");

			/**
			 * @member {float} current zoom level
			 * @private
			 */
			this._zoom = 1;


			/**
			 * @member {Boolean} is the scene being panned at the moment
			 * @private
			 */
			this._isPanning = false;

			/**
			 * @member {ShapeUtils.Size} the offset (world distance) used while panning (will be added to general offset when panning is done)
			 * @private
			 */
			this._panningOffset = new ShapeUtils.Size();

			/**
			 * @member {ShapeUtils.Size} general offset for the scene (comes from panning)
			 * @private
			 */
			this._offset = new ShapeUtils.Size();

			/**
			 * @member {ToolConstants.Cursors} the currently used cursor in the scene
			 * @private
			 */
			this._cursor = ToolConstants.Cursors.DEFAULT;


			/**
			 * @member {StringSet} true if the transformation is being interactively changed by the user and quick rendering is required
			 * @private
			 */
			this._frequentModifiers = new StringSet();

			/**
			 * @member {Boolean} is the scene in highlight mode at the moment
			 * @private
			 */
			this._isInHighlightMode = false;

			/**
			 * @Member {Object} insets
			 * @private
			 */
			this._insets = {
				top: 0,
				right: 0,
				bottom: 0,
				left: 0
			};

			/**
			 * @member {ShapeUtils.Rect} bounding box of the world
			 * @private
			 */
			this._worldBounds = null;

			/**
			 * @member {WFEditorContext} 
			 * @private
			 */
			this._WFContext = WFContext;

		};


		/**
		 * Set the new zoom level in the model.
		 * @param {Number} zoom
		 */
		SceneModel.prototype.setZoom = function (zoom) {
			this._zoom = zoom;
			this._emit("zoomChanged");
		};

		/**
		 * Set the zoom level for the scene correctly
		 * @param {Integer} zoomLevel
		 */
		SceneModel.prototype.setZoomLevel = function (zoomLevel) {
			this.setZoom(ToolConstants.ZoomLevel[zoomLevel]);
		};

		/**
		 * Get the zoom (number between 0 and "Infinity") which can be used as scale factor
		 * @returns {float}
		 */
		SceneModel.prototype.getZoom = function () {
			return this._zoom;
		};

		/**
		 * Get the zoom level (number between minimum and maximum level)
		 * @returns {number}
		 */
		SceneModel.prototype.getZoomLevel = function () {
			var zoomLevel = _.invert(ToolConstants.ZoomLevel)[this._zoom];
			if (zoomLevel) {
				return zoomLevel;
			} else {
				var closest = null;
				var smallestDiff = Infinity;
				_.each(ToolConstants.ZoomLevel, function (zoomLevel) {
					var difference = Math.abs(zoomLevel - this._zoom);
					if (difference < smallestDiff) {
						smallestDiff = difference;
						closest = zoomLevel;
					}
				}, this);
				return _.invert(ToolConstants.ZoomLevel)[closest];
			}
		};


		/**
		 * Set if the user is panning at this moment
		 * @param {Boolean} isPanning
		 */
		SceneModel.prototype.setPanning = function (isPanning) {
			if (this._isPanning !== isPanning) {
				this._isPanning = isPanning;
				if (isPanning) {
					this._panningOffset = new ShapeUtils.Size();
					this.startModifyFrequently("PANNING");
				} else {
					this._offset.add(this._panningOffset);
					this._panningOffset = new ShapeUtils.Size();
					this.stopModifyFrequently("PANNING");
				}
			}
		};

		/**
		 * Is the user panning at this moment.
		 * @returns {Boolean}
		 */
		SceneModel.prototype.isPanning = function () {
			return this._isPanning;
		};

		/**
		 * Set the panning offset, it's the size the user has panned around since pan start was called
		 * @param {ShapeUtils.Size} panningOffset - offset in scene coordinates
		 */
		SceneModel.prototype.setPanningOffsetScene = function (panningOffset) {
			this._panningOffset = this._bindPanningOffsetToWorldBoundingBox(new ShapeUtils.Size(panningOffset.width / this._zoom, panningOffset.height / this._zoom));
			this._emit("panningOffsetChanged");
		};

		/**
		 * Set the panning offset, it's the size the user has panned around since pan start was called
		 * @param {ShapeUtils.Size} panningOffset - offset in world coordinates
		 */
		SceneModel.prototype.setPanningOffsetWorld = function (panningOffset) {
			this._panningOffset = this._bindPanningOffsetToWorldBoundingBox(new ShapeUtils.Size(panningOffset.width, panningOffset.height));
			this._emit("panningOffsetChanged");
		};

		/**
		 * Add more offset to the current panning offset
		 * @param {ShapeUtils.Size} panningOffset - offset in World coordinates which should be added to the current panning offset
		 */
		SceneModel.prototype.panWorld = function (panningOffset) {
			var tempOffest = this._panningOffset.copy();
			tempOffest.add(panningOffset);
			this._panningOffset = this._bindPanningOffsetToWorldBoundingBox(tempOffest);
			this._emit("panningOffsetChanged");
		};

		/**
		 * Binds the panning offset to world bounding box
		 * @param {ShapeUtils.Rect} panningOffset
		 */
		SceneModel.prototype._bindPanningOffsetToWorldBoundingBox = function (panningOffset) {
			var scene = this._WFContext.get("scene");
			if (this._worldBounds && scene) {
				var visibleRect = scene.getVisibleRect();
				var sceneSize = scene.getSceneSize();

				var boundingRect = this._worldBounds.calculateArea(visibleRect);
				if (panningOffset.width !== 0) {
					if (-(this._offset.width + panningOffset.width) + (sceneSize.width / this._zoom) > boundingRect.size.width + boundingRect.origin.x) {
						panningOffset.width = -boundingRect.size.width - this._offset.width + sceneSize.width / this._zoom - boundingRect.origin.x;
					} else if (-(this._offset.width + panningOffset.width) < boundingRect.origin.x) {
						panningOffset.width = -boundingRect.origin.x - this._offset.width;
					}
				}
				if (panningOffset.height !== 0) {
					if (-(this._offset.height + panningOffset.height) + sceneSize.height / this._zoom > boundingRect.size.height + boundingRect.origin.y) {
						panningOffset.height = -boundingRect.size.height - this._offset.height + sceneSize.height / this._zoom - boundingRect.origin.y;

					} else if (-(this._offset.height + panningOffset.height) < boundingRect.origin.y) {
						panningOffset.height = -boundingRect.origin.y - this._offset.height;
					}
				}
			}
			return panningOffset;
		};

		/**
		 * sets bounding box of scene
		 * @param {ShapeUtils.Rect}
		 */
		SceneModel.prototype.setWorldBoundingBox = function (rect) {
			this._worldBounds = rect;
		};

		/**
		 * gets the bounding box of the world and takes insets into account
		 * @returns {ShapeUtils.Rect}
		 */
		SceneModel.prototype.getWorldBoundingBoxWithInsets = function () {
			var bbox = this._worldBounds.copy();
			bbox.origin.x += this._insets.left / this._zoom;
			bbox.origin.y += this._insets.top / this._zoom;
			bbox.size.height -= (this._insets.top + this._insets.bottom) / this._zoom;
			bbox.size.width -= (this._insets.left + this._insets.right) / this._zoom;
			return bbox;
		};

		/**
		 * Get the offset the user has panned in world coordinates.
		 * @returns {ShapeUtils.Size}
		 */
		SceneModel.prototype.getPanningOffset = function () {
			return this._panningOffset;
		};

		/**
		 * Set the general offset for the scene
		 * @param {ShapeUtils.Size} offset in world coordinates
		 */
		SceneModel.prototype.setOffset = function (offset, ignoreBounds) {
			this._offset = this._bindOffsetToWorldBoundingBox(offset, ignoreBounds);
		};

		/**
		 * Binds the panning offset to world bounding box
		 * @param {ShapeUtils.Rect} panningOffset
		 */
		SceneModel.prototype._bindOffsetToWorldBoundingBox = function (offset, ignoreBounds) {
			if (this._worldBounds && !ignoreBounds) {
				var scene = this._WFContext.get("scene");
				if (scene) {
					var sceneSize = scene.getSceneSize();
					if (-offset.width < this._worldBounds.origin.x) {
						offset.width = -this._worldBounds.origin.x;
					}
					if (- offset.width > this._worldBounds.size.width + this._worldBounds.origin.x - sceneSize.width / this._zoom) {
						offset.width = -(this._worldBounds.size.width + this._worldBounds.origin.x - sceneSize.width / this._zoom);
					}
					if (-offset.height < this._worldBounds.origin.y) {
						offset.height = -this._worldBounds.origin.y;
					}
					if (-offset.height > this._worldBounds.size.height + this._worldBounds.origin.y - sceneSize.height / this._zoom) {
						offset.height = -(this._worldBounds.size.height + this._worldBounds.origin.y - sceneSize.height / this._zoom);
					}
				}
			}
			return offset;
		};

		/**
		 * Get the general offset used in the scene
		 * @returns {ShapeUtils.Size}
		 */
		SceneModel.prototype.getOffset = function () {
			return this._offset;
		};

		/**
		 * Set the cursor(-class) that is used in the scene
		 * @param {ToolConstants.Cursors} cursor
		 */
		SceneModel.prototype.setCursor = function (cursor) {
			if (this._cursor !== cursor) {
				var oldCursor = this._cursor;
				this._cursor = cursor;
				this._emit("cursorChanged", oldCursor, cursor);
			}
		};

		/**
		 * Set insets of scene in Scene pixels
		 * @param {Number} top - inset for top
		 * @param {Number} right - inset for right
		 * @param {Number} bottom - inset for bottom
		 * @param {Number} left - inset for left
		 */
		SceneModel.prototype.setInsets = function (top, right, bottom, left) {
			this._insets.top = top;
			this._insets.right = right;
			this._insets.bottom = bottom;
			this._insets.left = left;
		};

		/**
		 * @returns {Object} with properties for top, right, bottom and left inset
		 */
		SceneModel.prototype.getInsets = function () {
			return this._insets;
		};


		/**
		 * Start modifying the workflow frequently
		 * From now on buffers might be used in non-interactive layers
		 * @param {String} source - unique identifier, must be specified to stop the frequent modification
		 */
		SceneModel.prototype.startModifyFrequently = function (source) {
			var wasBeingModifiedFrequently = this.isBeingModifiedFrequently();

			this._frequentModifiers.add(source);

			var isBeingModifiedFrequently = this.isBeingModifiedFrequently();

			if (wasBeingModifiedFrequently !== isBeingModifiedFrequently) {
				this._emit("beingModifiedFrequently", isBeingModifiedFrequently);
			}
		};

		/**
		 * Stop modifying the workflow frequently
		 * From now on buffers will no longer be used (if there are no other frequent modifiers)
		 * @param {String} source - unique identifier, same as was used for startModifyFrequently
		 */
		SceneModel.prototype.stopModifyFrequently = function (source) {
			var wasBeingModifiedFrequently = this.isBeingModifiedFrequently();

			this._frequentModifiers.remove(source);

			var isBeingModifiedFrequently = this.isBeingModifiedFrequently();

			if (wasBeingModifiedFrequently !== isBeingModifiedFrequently) {
				this._emit("beingModifiedFrequently", isBeingModifiedFrequently);
			}
		};

		/**
		 * Is the workflow being modified frequently?
		 * @returns {Boolean}
		 */
		SceneModel.prototype.isBeingModifiedFrequently = function () {
			return this._frequentModifiers.size() > 0;
		};

		/**
		 * Set if the scene should be shown in highlight mode
		 * @param {Boolean} isInHighlightMode
		 */
		SceneModel.prototype.setHighlightMode = function (isInHighlightMode) {
			if (isInHighlightMode !== this._isInHighlightMode) {
				var oldHighlightMode = this._isInHighlightMode;
				this._isInHighlightMode = isInHighlightMode;
				this._emit("highlightModeChanged", oldHighlightMode, isInHighlightMode);
			}
		};

		/**
		 * Is the scene in highlight at this moment.
		 * @returns {Boolean}
		 */
		SceneModel.prototype.isInHighlightMode = function () {
			return this._isInHighlightMode;
		};

		return SceneModel;
	});
