define(
	[
		"utils/ShapeUtils"
	],
	function (
		ShapeUtils
	) {

		"use strict";

		/**
		 * A class defining a hit area for a view object. The hitarea should define a rectangle in world coordinates. 
		 * This way the view object can be detectable on the canvas.
		 * An optional class or array of classes can be given to distinguish between different hit areas of the same view.
		 * 
		 * @exports HitArea
		 * @class
		 * 
		 * @param {ShapeUtils.Rect} rect - the rectangle on the scene that should trigger this hit area
		 * @param {AbstractView} view - the view that defined this hit area                                 
		 * @param {String | Array.<String>} [classes = []] - list of classes used for this hitarea
		 * @param {Number} [zIndex = view.getZIndex()] - hit z index of the hit area, defaults to the zindex of the view, not used for rendering                                    
		 * @param {Object} [options] - optional options object
		 *                           {
		 *                             cursor: <class that will be added to scene on hover>
		 *                             tooltip: <tooltip to be displayed on hover>
		 *                           }
		 */
		var HitArea = function (rect, view, classes, zIndex, options) {

			var normalizedRect = rect.getNormalizedRectArray();

			/**
			 * @member {Number} x coordinate of top left most point of the hitarea
			 */
			this.minX = normalizedRect[0];

			/**
			 * @member {Number} y coordinate of top left most point of the hitarea
			 */
			this.minY = normalizedRect[1];

			/**
			 * @member {Number} x coordinate of bottom right most point of the hitarea
			 */
			this.maxX = normalizedRect[2];

			/**
			 * @member {Number} y coordinate of bottom right most point of the hitarea
			 */
			this.maxY = normalizedRect[3];

			/**
			 * @member {AbstractView}
			 * @private
			 */
			this._view = view;

			/**
			 * @member {Array.<String>} classes of this hit area
			 * @private
			 */
			this._classes = _.isString(classes) ? [classes] : classes || [];

			/**
			 * @member {Number} the hit Z-Index is used to specify which hit area should be selected when clicked by the mouse
			 *                  It is not used for rendering. Defaults to the Z-index of the view.
			 * @private
			 */
			this._hitZIndex = typeof zIndex === "undefined" ? view.getZIndex() : zIndex;

			/**
			 * @member {Object} options object
			 * @private 
			 */
			this._options = options || {};

		};

		/**
		 * Check if the hitareas are equal
		 * @param {HitArea} hitArea
		 */
		HitArea.prototype.equals = function (hitArea) {
			var rectArr = ["minX", "minY", "maxX", "maxY"];
			for (var i = 0; i < 4; i++) {
				if (this[rectArr[i]] !== hitArea[rectArr[i]]) {
					return false;
				}
			}
			if (this._classes.length !== hitArea.getClasses().length) {
				return false;
			}
			for (i = 0; i < this._classes.length; i++) {
				if (this._classes[i] !== hitArea.getClasses()[i]) {
					return false;
				}
			}

			if (this._hitZIndex !== hitArea.getZIndex()) {
				return false;
			}
			_.each(this._options, function(option, key){
				if(hitArea.getOption[key] !== option) {
					return false;
				}
			});
			return true;
		};

		/**
		 * returns the classes of the hit area
		 * @returns {Array.<String>}
		 */
		HitArea.prototype.getClasses = function () {
			return this._classes;
		};

		/**
		 * returns the view of the hitarea
		 * @returns {AbstractView}
		 */
		HitArea.prototype.getView = function () {
			return this._view;
		};

		/**
		 * returns true if the hit area has the given class
		 * @param   {String} clazz
		 * @returns {Boolean} 
		 */
		HitArea.prototype.hasClass = function (clazz) {
			return _.contains(this._classes, clazz);
		};

		/**
		 * returns the z index of the hit area
		 * @returns {Number}
		 */
		HitArea.prototype.getZIndex = function () {
			return this._hitZIndex;
		};

		/**
		 * returns an option of the hitArea by option name. Currently "cursor" and "tooltip"
		 * @returns {String} 
		 */
		HitArea.prototype.getOption = function (option) {
			return this._options[option];
		};

		return HitArea;

	});
