define(["utils/Observer",
		"rbush",
		"underscore"
	],
	function (
		Observer,
		Rbush,
		_) {
		"use strict";

		/**
		 * The HitAreaTree is used to detect which hit areas for which views are on a certain location
		 * a tree Element is a array [x1,y1,x2,y2] with a property view which points to the view object
		 * @exports HitAreaTree
		 * @class
		 * @mixes Observer
		 */
		var HitAreaTree = function () {
			Observer.mixin(this);

			/**
			 * @member {Rbush} RD*-tree, a spatial map of all hit areas
			 * @private
			 */
			this._tree = new Rbush(15);

			/**
			 * @member {Map<viewId, Array.<TreeElement>} mapping of a view to its tree entries
			 * @private
			 */
			this._viewToTree = {};
		};

		/**
		 * Register a set of views in the hit area tree
		 * @param {Array<AbstractView>|Map<String,AbstractView> viewObjects
		 */

		HitAreaTree.prototype.registerViews = function (viewObjects) {
			var treeElements = [];
			_.each(viewObjects, function (viewObject) {
				var treeElementsForView = this._generateTreeElements(viewObject);
				_.each(treeElementsForView, function (treeEl) {
					treeElements.push(treeEl);
				});
				this._viewToTree[viewObject.getId()] = treeElementsForView;
				this._registerListenersOnView(viewObject);
			}, this);

			if (treeElements.length > 1) {
				this._tree.load(treeElements);
			} else if (treeElements.length === 1) {
				this._tree.insert(treeElements[0]);
			}
		};

		/**
		 * Remove a view from the hit area tree
		 * @param {AbstractView} viewObject
		 */
		HitAreaTree.prototype.removeView = function (viewObject) {
			var treeElements = this._viewToTree[viewObject.getId()];
			// remove all hit areas from hit tree
			_.each(treeElements, function(treeEl){
				this._tree.remove(treeEl);
			}, this);

			this._stopObserveObject(viewObject);
			delete this._viewToTree[viewObject.getId()];
		};

		/**
		 * Register asingle view in the hit area tree
		 * @param {AbstractView> viewObject
		 */
		HitAreaTree.prototype.registerView = function (viewObject) {
			this.registerViews([viewObject]);
		};


		/**
		 * Generate tree entry elements for a certain view object
		 * @param {AbstractView} viewObject
		 * @returns {Array.<HitArea>} list of tree entries
		 */
		HitAreaTree.prototype._generateTreeElements = function (viewObject) {
			return viewObject.getHitAreas();
		};


		/**
		 * Listen to the necessary events on a view
		 * @param {AbstractView} viewObject
		 */
		HitAreaTree.prototype._registerListenersOnView = function (viewObject) {
			this._observe(viewObject, "changed.hitAreas", this._updateHitAreas);
		};


		/**
		 * Update the tree whenever a view object changed its hit areas
		 * @param {AbstractView} viewObject - the view object that modified its hit areas
		 */
		HitAreaTree.prototype._updateHitAreas = function (viewObject) {
			var treeElements = this._viewToTree[viewObject.getId()];
			// remove all hit areas from hit tree
			_.each(treeElements, function(element){
				this._tree.remove(element);
			}, this);

			treeElements = this._generateTreeElements(viewObject);
			if (treeElements.length > 1) {
				this._tree.load(treeElements);
			} else if (treeElements.length === 1) {
				this._tree.insert(treeElements[0]);
			}
			this._viewToTree[viewObject.getId()] = treeElements;
		};

		/**
		 * Get all view objects that intersect with position. Only objects with the highest ZIndex are returned
		 *
		 * @param {ShapeUtils.Point} position - World Coordinates
		 * @param {Function} [filter] - Function returning the view (or another one) if the given viewObject argument passes the filter. Return null to skip it.
		 * @returns Array.<AbstractView> matched view objects
		 */
		HitAreaTree.prototype.getTopLevelObjectAtPosition = function (position, filter) {

			if (!filter) {
				filter = function (view) {
					return view;
				};
			}

			// first get all the hit areas at the position, then use the filter to filter the views
			return _.map(
				_.filter(this.getTopLevelHitAreaAtPosition(position), function (hitArea) {
					return filter(hitArea.getView()) !== null;
				}),
				function (hitArea) {
					return hitArea.getView();
				});

		};

		/**
		 * Get all hitarea objects that intersect with position. Only hitareas with the highest ZIndex are returned
		 *
		 * @param {ShapeUtils.Point} position - World Coordinates
		 * @param {Function} [filter] - Function returning the hitarea (or another one) if the given hitArea argument passes the filter. Return null to skip it.
		 * @returns Array.<HitArea> matched hitarea objects
		 */
		HitAreaTree.prototype.getTopLevelHitAreaAtPosition = function (position, filter) {
			var highestZIndex = -1;
			var result = [];

			// allow all view objects when no filter is provided
			if (!filter) {
				filter = function (hitArea) {
					return hitArea;
				};
			}

			var searchResults = this._tree.search({minX: position.x, minY: position.y, maxX: position.x, maxY: position.y});

			// filter results
			_.each(searchResults, function (hitArea) {
				if (hitArea.getView() && hitArea.getZIndex() >= highestZIndex) {
					hitArea = filter(hitArea);
					if (hitArea) {
						if (hitArea.getZIndex() === highestZIndex) {
							result.push(hitArea);
						} else {
							// the view is on a higher ZIndex than previously found views, reset the result set
							result = [hitArea];
						}
					}

					highestZIndex = hitArea.getZIndex();
				}
			});

			return result;
		};

		/**
		 * Get all view objects at a certain position
		 * @param {ShapeUtils.Point} position - World Coordinates
		 * @param {Function} [filter] - Function returning the view (or another one) if the given viewObject argument passes the filter. Return null to skip it.
		 * @returns Array.<AbstractView> matched view objects
		 */
		HitAreaTree.prototype.getObjectsAtPosition = function (position, filter) {
			return this._searchAndFilterViews({minX: position.x, minY: position.y, maxX: position.x, maxY: position.y}, filter);
		};


		/**
		 * Get view objects near the given position within the given rectangular marge.
		 * For each "hit" view object the filter will be called with the view object as argument to further filter down the result set.
		 * @param {ShapeUtils.Point} position - World Coordinates, the position to check
		 * @param {Number} marge - World distance
		 * @param {Function} filter - Function returning the view (or another one) if the given viewObject argument passes the filter. Return null to skip it.
		 * @returns Array.<AbstractView> matched view objects
		 */
		HitAreaTree.prototype.getObjectsNearPosition = function (position, marge, filter) {
			var x = Math.round(position.x - marge / 2);
			var y = Math.round(position.y - marge / 2);
			return this._searchAndFilterViews({minX: x, minY: y, maxX: x + marge, maxY: y + marge}, filter);

		};

		/**
		 * Get all views intersecting a given rectangle
		 * @param {ShapeUtils.Rect} rect - rectangle to search
		 * @param {Function} filter - Function returning the view (or another one) if the given viewObject argument passes the filter. Return null to skip it.
		 * @returns Array.<AbstractView> matched view objects
		 */
		HitAreaTree.prototype.getObjectsIntersectingRect = function (rect, filter) {
			var normalizedRect = rect.getNormalizedRectArray();
			return this._searchAndFilterViews({minX: normalizedRect[0], minY: normalizedRect[1], maxX: normalizedRect[2], maxY: normalizedRect[3]}, filter);
		};

		/**
		 * Internal function to search and filter hitareas a certain area
		 * @param {Object} searchArea - area to search in, object should have minX, minY, maxX, maxY, corresponding to top left and bottom right corner of search box
		 * @param {Function} [filter] - Function returning the hitArea (or another one) if the given hitArea argument passes the filter. Return null to skip it.
		 * @returns Array.<HitArea> matched hitArea objects
		 */
		HitAreaTree.prototype._searchAndFilter = function (searchArea, filter) {
			if (!filter) {
				filter = function (view) {
					return view;
				};
			}

			var result = [];
			var searchResults = this._tree.search(searchArea);
			_.each(searchResults, function (hitArea) {
				var view = filter(hitArea);
				if (view) {
					result.push(view);
				}
			});

			return result;
		};

		/**
		 * Internal function to search and filter views in a certain area
		 * @param {Object} searchArea - area to search in, object should have minX, minY, maxX, maxY, corresponding to top left and bottom right corner of search box
		 * @param {Function} [filter] - Function returning the view (or another one) if the given viewObject argument passes the filter. Return null to skip it.
		 * @returns Array.<AbstractView> matched view objects
		 */
		HitAreaTree.prototype._searchAndFilterViews = function (searchArea, filter) {
			return this._searchAndFilter(searchArea, function (hitArea) {
				return filter(hitArea.getView());
			});
		};

		/**
		 * returns the hit area opbjects at this position
		 * @param   {ShapeUtils.Point}   position 
		 * @param   {Function} filter   Function returning the hit area if the given HitArea argument passes the filter. Return null to skip it.
		 * @returns {Array.<HitArea>}
		 */
		HitAreaTree.prototype.getHitAreasAtPosition = function (position, filter) {
			return this._searchAndFilter({minX: position.x, minY: position.y, maxX: position.x, maxY:position.y}, filter);
		};


		return HitAreaTree;

	});
