define([
		"WFEditor/scene/layer/Layer",
		"utils/ShapeUtils"
	],
	function (
		Layer,
		ShapeUtils
	) {
		"use strict";

		/**
		 * Layer to Render all interactive view object. These are all objects that are being
		 * moved, animated,... Whenever the layer contains any view objects, the layer constantly
		 * renders all those view objects by calling requestAnimationFrame.
		 *
		 * @exports InteractiveLayer
		 * @class
		 * @augments Layer
		 *
		 * @param {JQueryObj} context
		 * @param {Scene} scene
		 * @param {String} id
		 */
		var InteractiveLayer = function (scene, sceneModel, id) {
			Layer.call(this, scene, sceneModel, id, false);

			/**
			 * @member {Boolean} indicates if the layer is currently rendering, if there are any view objects
			 * @private
			 */
			this._isRendering = false;


			/**
			 * @member{Number} how many views are in the interactive layer?
			 * @private
			 */
			this._viewsCount = 0;
		};

		InteractiveLayer.prototype = Object.create(Layer.prototype);
		InteractiveLayer.prototype.constructor = InteractiveLayer;

		/**
		 * Make sure to update the viewcount
		 * @Override
		 */
		InteractiveLayer.prototype.addViewObject = function (id, viewObject) {
			if (!this._viewObjects[id]) {
				Layer.prototype.addViewObject.call(this, id, viewObject);
				this._startRendering();
				this._viewsCount++;
			}
		};

		/**
		 * Interactive layers should never be rendering using a buffer
		 * @Override
		 */
		InteractiveLayer.prototype.renderBuffer = function (context) {
			this.render();
		};

		/**
		 * never store a buffer
		 * @Override
		 */
		InteractiveLayer.prototype._storeBufferIfNeeded = function () {
			// no buffering in interactive layer
		};

		/**
		 * Removes a view object from this layer, indicates this layer needs to stop rendering.
		 * @param   {String} id
		 */
		InteractiveLayer.prototype.removeViewObject = function (id) {
			var removedView = Layer.prototype.removeViewObject.call(this, id);
			if (removedView) {
				this._viewsCount--;

				if (this._viewsCount === 0) {
					this._stopRendering();
				}
			}

			return removedView;
		};

		/**
		 * Start rendering on this layer, this starts the continious rendering of this layer
		 * @private
		 */
		InteractiveLayer.prototype._startRendering = function () {
			if (!this._isRendering) {
				this._isRendering = true;

				this.makeDirty();
			}
		};

		/**
		 * Stop the rendering of this layer.
		 * @private
		 */
		InteractiveLayer.prototype._stopRendering = function () {
			this._isRendering = false;
		};


		/**
		 * @override
		 * after rendering is complete, rerender
		 */
		InteractiveLayer.prototype._postRender = function () {
			if (this._isRendering) {
				this.makeDirty();
			}
		};

		return InteractiveLayer;
	});
