define([
		"utils/ShapeUtils",
		"utils/Observer",
		"utils/ColorUtils",
		"WFEditor/scene/ViewConstants",
		"rbush",
		"underscore",
		"jquery"
	],
	function (
		ShapeUtils,
		Observer,
		ColorUtils,
		ViewConstants,
		Rbush,
		_,
		$
	) {
		"use strict";
		/**
		 * Layer is the layer used to draw the workflow. It adds a canvas to the context.
		 *
		 * @exports Layer
		 * @class
		 *
		 * @param {Scene} scene
		 * @param {SceneModel} sceneModel
		 * @param {String} id
		 * @param {Boolean} isOpaque
		 */
		var Layer = function (scene, sceneModel, id, isOpaque) {
			Observer.mixin(this);

			/**
			 * @member {Object} All the view objects have a render method which renders itself on this layer.
			 * @private
			 */
			this._viewObjects = {};

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = scene;

			/**
			 * @member {Boolean} indicates if the layer needs to be redrawn
			 * @private
			 */
			this._isDirty = false;

			/**
			 * @member {ViewConstants.Layer} the layer type, unique id to identify this layer
			 * @private
			 */
			this._layerId = id;

			/**
			 * @member {Boolean} is this layer opaque, if true layer behind this one are not visible
			 * @private
			 */
			this._isOpaque = isOpaque;

			/**
			 * @member {SceneModel} the model used by the scene
			 * @private
			 */
			this._sceneModel = sceneModel;

			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._canvas = $("<canvas>").css({
				'top': 0,
				'left': 0,
				'bottom': 0,
				'right': 0,
				'width': "100%",
				'height': "100%"
			}).addClass(id);


			/**
			 * @Member {Number} Sliding average of render durations
			 */
			this._renderDuration = 0;

			/**
			 * @member {Canvas} the buffer used to render the workflow while panning
			 * @private
			 */
			this._buffer = null;

			/**
			 * @member{Boolean} is the buffer stored?
			 */
			this._bufferStored = false;

			/**
			 * @member {ShapeUtils.Rect} the buffer rect in world coordinates
			 * @private
			 */
			this._bufferRect = new ShapeUtils.Rect();

			/**
			 * @Member {CanvasRenderingContext2D} the context used for rendering
			 * @private
			 */
			this._renderContext = null;

			// initalize the correct context (based on opaqueness of the layer)f
			if (this._isOpaque) {
				this._renderContext = this._canvas[0].getContext('2d', {
					alpha: false // non-opaque used for crisp text rendering
				});
			} else {
				this._renderContext = this._canvas[0].getContext('2d');
			}

			/**
			 * @member {Rbush} RD*-tree, spatial tree of bounding boxes of the view
			 * @private
			 */
			this._tree = new Rbush(10);

			/**
			 * @member {Map<viewId,TreeElement>} mapping of view objects to its tree entries
			 */
			this._treeObjects = {};


		};

		/**
		 * Get access to the canvas element of the layer
		 * @returns {JQueryObj}
		 */
		Layer.prototype.getCanvas = function () {
			return this._canvas;
		};

		/**
		 * Get access to the render context of the layer
		 * Do not use this to render somethingon the layer!!
		 * @returns {CanvasRenderingContext2D}
		 */
		Layer.prototype.getRenderContext = function () {
			return this._renderContext;
		};

		/**
		 * Make this layer dirty, it will request a rerender of the layer.
		 */
		Layer.prototype.makeDirty = function () {
			if (!this._isDirty) {
				this._isDirty = true;

				// ask the scene for a animation frame
				this._scene.requestAnimationFrame(this);
			}
		};

		/**
		 * Add a view object ot the layer. Indicates this layer needs to be redrawn.
		 * @param {String} id
		 * @param {AbstractView} viewObject
		 */
		Layer.prototype.addViewObject = function (id, viewObject) {
			// if a viewobject is already registered, first remove it
			if (this._viewObjects[id]) {
				if (this._viewObjects[id] != viewObject) {
					this.removeViewObject(id);
				}
			}
			// add the view object
			this._viewObjects[id] = viewObject;
			this._registerListenersOnView(viewObject);
			this._addViewObjectToTree(viewObject);

			this.makeDirty();
		};

		/**
		 * Insert the bounding rect of a view in the tree
		 * @param {AbstractView} viewObject
		 */
		Layer.prototype._addViewObjectToTree = function (viewObject) {
			var treeEl = this._generateTreeElementForView(viewObject);
			if (treeEl) {
				this._tree.insert(treeEl);
				this._treeObjects[viewObject.getId()] = treeEl;
			}
		};


		/**
		 * Generate Tree Element for view
		 * @param {AbstractView} viewObject - the view to generate the tree element for
		 * @returns {NormalizedRectArray} or null when no bounding rect defined
		 */
		Layer.prototype._generateTreeElementForView = function (viewObject) {
			var boundingRect = viewObject.getBoundingRect(this._renderContext);
			if (boundingRect) {
				// make a copy, so the tree entry is never changed externally
				var treeEl = {minX: boundingRect[0], minY: boundingRect[1], maxX: boundingRect[2], maxY: boundingRect[3]};
				treeEl.view = viewObject;
				return treeEl;
			} else {
				return null;
			}
		};

		/**
		 * Remove a view from the tree
		 * @param {AbstractView} viewObject
		 */
		Layer.prototype._removeViewObjectFromTree = function (viewObject) {
			var treeElement = this._treeObjects[viewObject.getId()];
			if (treeElement) {
				this._tree.remove(treeElement);
			}

		};

		/**
		 * Update the tree with the new bounding box of a viewobject
		 * @param {AbstractView} viewObject
		 */
		Layer.prototype._updateTreeWithViewObject = function (viewObject) {
			var newTreeEl = this._generateTreeElementForView(viewObject);
			var treeEl = this._treeObjects[viewObject.getId()];


			if (newTreeEl != treeEl ||
				(newTreeEl && treeEl &&
					newTreeEl.minX !== treeEl.minX ||
					newTreeEl.minY !== treeEl.minY ||
					newTreeEl.maxX !== treeEl.maxX ||
					newTreeEl.maxY !== treeEl.maxY)) {
				// bounding box has changed...

				if (treeEl) {
					this._tree.remove(treeEl);
				}

				if (newTreeEl) {
					this._tree.insert(newTreeEl);
					this._treeObjects[viewObject.getId()] = newTreeEl;
				} else {
					delete this._treeObjects[viewObject.getId()];
				}
			}
		};

		/**
		 * Removes a view object from this layer, indicates this layer needs to be redrawn.
		 * @param   {String} id
		 */
		Layer.prototype.removeViewObject = function (id) {
			var viewObject = this._viewObjects[id];

			if (viewObject) {
				this._removeListentersOnView(viewObject);
				this._removeViewObjectFromTree(viewObject);


				delete this._viewObjects[id];

				this.makeDirty();
			}

			return viewObject;
		};


		/**
		 * Register the correct listeners on the model of a view object
		 * @param {AbstractView} abstractView
		 */
		Layer.prototype._registerListenersOnView = function (abstractView) {
			this._observe(abstractView, "isDirty", this._viewIsDirty);
			this._observe(abstractView, "changed.boundingRect", this._updateTreeWithViewObject);

		};

		/**
		 * Remove all the listeners that were registered by the Layer on the specified view object
		 * @param {AbstractView} abstractView
		 */
		Layer.prototype._removeListentersOnView = function (abstractView) {
			this._stopObserveObject(abstractView);
		};



		/**
		 * Called before rendering the views
		 */
		Layer.prototype._preRender = function () {

		};

		/**
		 * Render the layer. Should only be called in an animation frame (and by the Scene)
		 * @param {Boolean} fullRender - if true all views (in the bounding rect should be rendered)
		 */
		Layer.prototype.render = function () {
			this._isDirty = false;

			this._preRender();

			// make sure the buffer is cleared when a new render happens... otherwise bufferlocation will be out of sync...
			this.resetBuffer();

			// do render
			var renderStart = Date.now();
			this._prepareCanvas(true);
			this._renderRect(this.getVisibleRect().getNormalizedRectArray(), false);


			// update render duration (sliding average)
			var renderDuration = Date.now() - renderStart;
			this._renderDuration = 0.2 * this._renderDuration + 0.8 * renderDuration;

			var t = this._scene.getTransformation().createInverse();
			this._bufferRect.origin = t.transformedPoint(new ShapeUtils.Point());
			this._bufferRect.size = t.transformedSize(this._scene.getSceneSize());



			this._postRender();
		};


		/**
		 * Render all objects in a certain rectangle
		 * @param {NormalizedRectArray} rect - [x1, y1, x2, y2] rectangle to render
		 * @param {Boolean} quickRender - if true only quick render methods are used
		 */
		Layer.prototype._renderRect = function (rect, quickRender) {
			var treeElements = this._tree.search({minX: rect[0], minY: rect[1], maxX: rect[2], maxY: rect[3]});

			_.each(treeElements, function (treeElement) {
				var view = treeElement.view;
				if(this._sceneModel.isInHighlightMode() && !view.getModel().isHighlighted() && !view.getModel().isHovered()){
					this._renderContext.globalAlpha = .2;
				} else {
					this._renderContext.globalAlpha = 1;
				}
				view.render(this._renderContext, this, quickRender);

				/* DEBUG RENDER BOUNDING RECTANGLES*
				this._renderContext.save();
				this._renderContext.beginPath();
				this._renderContext.strokeStyle = "#cccccc";
				var rect = view.getBoundingRect(this._renderContext);
				if (rect){
					this._renderContext.fillText(view.getId(), rect[0] + 4, rect[3]);
					this._renderContext.strokeRect(rect[0], rect[1], rect[2] - rect[0], rect[3] - rect[1]);
				}

				/**/

				/* RENDER HIT AREAS *
				this._renderContext.strokeStyle = "#008800";
				_.each(view.getHitAreas(), function(rect){
					this._renderContext.strokeRect(rect[0], rect[1], rect[2]-rect[0], rect[3]-rect[1]);
				},this);
				this._renderContext.restore();
				/**/
			}, this);

		};

		/**
		 * Called when the buffer should be used, the scene will make sure _storeBuffer is called at least once
		 * Priority objects can also be rendered
		 */
		Layer.prototype.renderBuffer = function () {
			// make the buffer if it doesn't exist yet
			this._storeBufferIfNeeded();

			this._isDirty = false;

			this._preRender();

			this._prepareCanvas(true);

			// quick render all views
			this._renderRect(this.getVisibleRect().getNormalizedRectArray(), true);

			// rende rthe buffer
			this._renderContext.drawImage(this._buffer, this._bufferRect.origin.x, this._bufferRect.origin.y, this._bufferRect.size.width, this._bufferRect.size.height);

			this._postRender();
		};

		/**
		 * Get the transformation of the layer
		 * @returns {Transformation}
		 */
		Layer.prototype.getTransformation = function () {
			return this._scene.getTransformation();
		};

		/**
		 * Get the sliding average render duration of the layer
		 * @returns {Number} sliding average in ms
		 */
		Layer.prototype.getRenderDuration = function () {
			return this._renderDuration;
		};

		/**
		 * Store the nbuffer of the layer if needed
		 */
		Layer.prototype._storeBufferIfNeeded = function () {
			if (!this._buffer) {
				this._buffer = document.createElement("canvas");
			}
			if (!this._bufferStored) {
				this._prepareCanvas(false);
				var context = this._buffer.getContext("2d");
				var sceneSize = this._scene.getSceneSize();
				this._buffer.width = sceneSize.width;
				this._buffer.height = sceneSize.height;
				context.clearRect(0, 0, sceneSize.width, sceneSize.height);

				context.drawImage(this._canvas[0], 0, 0);

				this._bufferStored = true;
			}
		};

		/**
		 * Clear the buffer data
		 */
		Layer.prototype.resetBuffer = function () {
			this._bufferStored = false;
		};

		/**
		 * Called after render is completed
		 */
		Layer.prototype._postRender = function () {

		};


		/**
		 * Returns the size of the layer (the bounding box of all its view items)
		 * @returns {NormalizedRectArray} bounding box of all view items
		 */
		Layer.prototype.getBoundingRect = function () {
			return [this._tree.data.minX, this._tree.data.minY, this._tree.data.maxX, this._tree.data.maxY];
		};

		/**
		 * Get the visible rectangle in world coordinates.
		 * @returns {ShapeUtils.Rect}
		 */
		Layer.prototype.getVisibleRect = function () {
			return this._scene.getVisibleRect();
		};


		/**
		 * Initialize the canvas with the correct settings and transformation
		 * @param {Boolean} clearCanvas - clearthe complete canvas
		 */
		Layer.prototype._prepareCanvas = function (clearCanvas) {
			this._renderContext.setTransform(1, 0, 0, 1, 0, 0);

			if (clearCanvas) {
				// clear canvas
				var sceneSize = this._scene.getSceneSize();
				this._canvas[0].width = sceneSize.width;
				this._canvas[0].height = sceneSize.height;

				this._renderContext.clearRect(0, 0, sceneSize.width, sceneSize.height);
				if (this._isOpaque) {
					// draw background due to non-opaque
					this._renderContext.save();
					this._renderContext.fillStyle = "white";
					this._renderContext.fillRect(0, 0, sceneSize.width, sceneSize.height);
					this._renderContext.restore();
				}
			}
			// apply transform
			var t = this._scene.getTransformation();
			this._renderContext.transform(t.a, t.b, t.c, t.d, t.e, t.f);
		};

		/**
		 * React to a dirty view
		 * @param {AbstractView} view
		 */
		Layer.prototype._viewIsDirty = function (view) {
			this.makeDirty();
		};

		/**
		 * returns views for this layer
		 * @returns {Array.<AbstractView>} 
		 */
		Layer.prototype.getVisibleViews = function () {
			var normalizedRect = this.getVisibleRect().getNormalizedRectArray();
			var treeElements = this._tree.search({minX: normalizedRect[0], minY: normalizedRect[1], maxX: normalizedRect[2], maxY: normalizedRect[3]});
			var views = [];
			_.each(treeElements, function (treeElement) {
				views.push(treeElement.view);
			});
			return views;
		};

		return Layer;
	});
