define(["WFEditor/scene/layer/Layer",
		"WFEditor/scene/layer/InteractiveLayer",
		"utils/ShapeUtils",
		"underscore"
	],
	function (Layer,
		InteractiveLayer,
		ShapeUtils,
		_) {

		"use strict";

		/**
		 * A LayerSet is a set of a normal Layer and an InterActiveLayer
		 * it has methods for adding/removing views and making views interactive or not
		 * @exports LayerSet
		 * @class
		 *
		 * @param {Number} ZIndex - ZIndex of the layer
		 * @param {Scene} scene
		 * @param {SceneModel sceneModel
		 * @param {String} id - id for the normal layer (the interactiveLayer name is derived from this name)
		 * @param {Boolean} isOpaque - needed for the bottom layer for crisp rendering
		 */
		var LayerSet = function (ZIndex, scene, sceneModel, id, isOpaque) {
			/**
			 * @member {Number} ZIndex of the layer set
			 * @private
			 */
			this._ZIndex = ZIndex;
			/**
			 * @member {Layer} Normal rendering layer
			 * @private
			 */
			this._layer = new Layer(scene, sceneModel, id, isOpaque);

			/**
			 * @member {InteractiveLayer} Layer for interactive rendering
			 * @private
			 */
			this._interactiveLayer = new InteractiveLayer(scene, sceneModel, id + "_interactive");

		};

		/**
		 * Add a ViewObject to the LayerSet
		 * @param {String} id - id of the viewobject
		 * @param {AbstractView} viewObjct
		 */
		LayerSet.prototype.addViewObject = function (id, viewObject) {
			var layer;

			// decide to which layer to add the view
			if (viewObject.isInteractive()) {
				layer = this._interactiveLayer;
			} else {
				layer = this._layer;
			}

			// initialize the viewObject
			viewObject.initialize(layer.getRenderContext());

			layer.addViewObject(id, viewObject);
		};

		/**
		 * Remove a ViewObject from the LayerSet
		 * @param {String} id - id of the viewobjec to remove
		 */
		LayerSet.prototype.removeViewObject = function (id) {
			this._interactiveLayer.removeViewObject(id);
			this._layer.removeViewObject(id);
		};

		/**
		 * Make a view interactive
		 * @param {String} id - id of the view to make interactive
		 * @param {Boolean} isInteractive - should the view be interactive or not
		 */
		LayerSet.prototype.setInteractive = function (id, isInteractive) {
			var fromLayer, toLayer;
			if (isInteractive) {
				fromLayer = this._layer;
				toLayer = this._interactiveLayer;
			} else {
				fromLayer = this._interactiveLayer;
				toLayer = this._layer;
			}

			// remove view object from layer first layer
			var viewObject = fromLayer.removeViewObject(id);
			if (viewObject) {
				// add view object to the correct layer
				toLayer.addViewObject(id, viewObject);
			}
		};

		/**
		 * Get the Canvas Object of the normal layer
		 * @returns {JQueryObj}
		 */
		LayerSet.prototype.getLayerCanvas = function () {
			return this._layer.getCanvas();
		};

		/**
		 * Get the Canvas Object of the interactive layer
		 * @param {JQueryObj}
		 */
		LayerSet.prototype.getInteractiveLayerCanvas = function () {
			return this._interactiveLayer.getCanvas();
		};

		/**
		 * Trigger a render operation of the LayerSet
		 */
		LayerSet.prototype.makeDirty = function () {
			this._layer.makeDirty();
			// we can ignore the interactive layer, since that layer is continiously rendering if there is a view in it
		};

		/**
		 * Get the boundingrect of the layerSet. (note: elements in the interactive layer are excluded!)
		 * @returens {NormalizedRectArray}
		 */
		LayerSet.prototype.getBoundingRect = function () {
			return ShapeUtils.calculateBoundingBoxRectArrays(this._layer.getBoundingRect(), this._interactiveLayer.getBoundingRect());
		};

		/**
		 * Reset render buffers
		 */
		LayerSet.prototype.resetBuffer = function () {
			this._layer.resetBuffer();
		};
	
		/**
		 * get layer
		 * @returns {Layer} 
		 */
		LayerSet.prototype.getLayer = function () {
			return this._layer;
		};

		return LayerSet;
	});