define([
		"utils/Observer"
	],
	function (
		Observer
	) {
		"use strict";

		/**
		 * Toolbar is the view for all the toolbar components in the workflow editor.
		 *
		 * @exports Toolbar
		 * @class
		 *
		 * @param {ToolbarModel} toolbarModel
		 * @param {JQueryObj} context
		 */
		var Toolbar = function (toolbarModel, context) {
			Observer.mixin(this);

			/**
			 * @member {ToolbarModel}
			 * @private
			 */
			this._model = toolbarModel;

			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._context = context;

			/**
			 * @member {Map.<String,Object>} Maps the component Id to the view implementation
			 * 									of the toolbar component
			 * @private
			 */
			this._toolbarComponents = {};

			/**
			 * @member {Map.<String,Object} Maps the button id to the view of the button.
			 * @private
			 */
			this._toolbarButtons = {};
			//register all the listeners to add new components or buttons to the toolbar
			this._registerListeners();
			
			// add existing items from model
			_.each(toolbarModel.getAllComponents(), function(componentData){
				this._addComponent(componentData.id);
				_.each(componentData.buttons, function(buttonID){
					this._addButton(componentData.id, buttonID);
				},this);
			}, this);
		};

		/**
		 * Registers the correct listeners to the model
		 * @private
		 */
		Toolbar.prototype._registerListeners = function () {
			this._observe(this._model, "componentAdded", this._addComponent);
			this._observe(this._model, "buttonAdded", this._addButton);
			this._observe(this._model, "buttonStateChanged", this._updateButton);
			this._observe(this._model, "componentDataChanged", this._updateComponent);
		};

		/**
		 * Add a toolbar component to the view.
		 * @private
		 */
		Toolbar.prototype._addComponent = function () {
			//to be implemented by specific implementation of the toolbar
		};

		/**
		 * Add a toolbar button to the view
		 * @private
		 */
		Toolbar.prototype._addButton = function () {
			//to be implemented by specific implementation of the toolbar
		};

		/**
		 * Update the button, properties are changed in the model, update the view.
		 * @private
		 */
		Toolbar.prototype._updateButton = function () {
			//to be implemented by specific implementation of the toolbar
		};

		/**
		 * Update the component, properties are changed in the model, update the view.
		 * @private
		 */
		Toolbar.prototype._updateComponent = function () {
			//to be implemented by specific implementation of the toolbar
		};

		return Toolbar;

	});