define([
		"utils/Observable",
		"utils/OrderedObject",
		"underscore"
	],
	function (
		Observable,
		OrderedObject,
		_
	) {
		"use strict";

		/**
		 * Model used for the toolbar. To add new toolbar components or buttons, add them to the model.
		 *
		 * @exports ToolbarModel
		 * @class
		 */
		var ToolbarModel = function () {
			Observable.mixin(this, "toolBarModel");

			/**
			 * @member {OrderedObject} saves all the toolbar items in an ordered object to
			 *  						save the order of the toolbar components
			 * @private
			 */
			this._toolbarItems = new OrderedObject();

			/**
			 * @member {Map.<String,Object} map of the button id on the button data
			 * @private
			 */
			this._buttons = {};
		};

		/**
		 * Add a component to the toolbar.
		 * @param {String} componentId
		 * @param {String} [componentTitle]
		 * @param {Object} [data] data object for the specific component
		 */
		ToolbarModel.prototype.addComponent = function (componentId, componentTitle, data) {
			if (!this._toolbarItems.containsKey(componentId)) {
				var componentData = {
					id: componentId,
					title: componentTitle,
					data: data,
					buttons: []
				};

				this._toolbarItems.addItem(componentId, componentData);
				this._emit("componentAdded", componentId);
			}
		};

		/**
		 * Add a button to a toolbar component, adds the toolbar component if it doesn't exist yet
		 * @param {String} componentId
		 * @param {String} buttonId
		 * @param {String} icon
		 * @param {Function} action
		 * @param {String} tooltipText
		 * @param {Object} options - extra options that can be defined
		 */
		ToolbarModel.prototype.addButton = function (componentId, buttonId, icon, action, tooltipText, options) {
			var component = this._toolbarItems.getItemByKey(componentId);
			if (component) {
				var buttonData = {
					id: buttonId,
					icon: icon,
					action: action,
					enabled: true,
					active: false,
					tooltipText: tooltipText,
					componentId: componentId
				};
				if (options) {
					buttonData = _.extend(buttonData, options);
				}
				component.buttons.push(buttonId);
				this._buttons[buttonId] = buttonData;
			}
			this._emit("buttonAdded", componentId, buttonId);
		};

		/**
		 * Get a list of all toolbar components
		 * @returns {Array.<Object>}
		 */
		ToolbarModel.prototype.getAllComponents = function () {
			return this._toolbarItems.getOrderedObjectList();
		};

		/**
		 * Returns all the data for a toolbar component.
		 *
		 * @param   {String} componentId
		 * @returns {Object}
		 * @example
		 *  {
		 * 		id: componentId,
		 *   	title: title,
		 *    	buttons: []
		 *	}
		 */
		ToolbarModel.prototype.getComponent = function (componentId) {
			return this._toolbarItems.getItemByKey(componentId);
		};

		/**
		 * Returns the data for a toolbar button.
		 * @param   {String} buttonId
		 * @returns {Object}
		 * @example
		 *  {
		 * 		id: buttonId,
		 * 		componentId: componentId,
		 *   	icon: iconString,
		 *    	action: actionFunction,
		 *     	enabled: isButtonEnabled,
		 *      active: isButtonActive
		 *      tooltipText: tooltipText
		 *	}
		 */
		ToolbarModel.prototype.getButtonData = function (buttonId) {
			return this._buttons[buttonId];
		};

		/**
		 * Change the active state of a button.
		 * @param {String} buttonId
		 * @param {Boolean} isActive - should be button be active
		 */
		ToolbarModel.prototype.setButtonActive = function (buttonId, isActive) {
			var button = this._buttons[buttonId];
			if (button && button.active !== isActive) {
				button.active = isActive;
				this._emit("buttonStateChanged", buttonId);
			}
		};

		/**
		 * Change the enabled state of a button.
		 * @param {String} buttonId
		 * @param {Boolean} isEnabled - should the button be enabled
		 */
		ToolbarModel.prototype.setButtonEnabled = function (buttonId, isEnabled) {
			var button = this._buttons[buttonId];
			if (button && button.enabled !== isEnabled) {
				button.enabled = isEnabled;
				this._emit("buttonStateChanged", buttonId);
			}
		};

		/**
		 * Change the icon of a button.
		 * @param {String} buttonId
		 * @param {String} icon - class of icon
		 */
		ToolbarModel.prototype.setButtonIcon = function (buttonId, icon) {
			var button = this._buttons[buttonId];
			if (button) {
				button.icon = icon;
				this._emit("buttonStateChanged", buttonId);
			}
		};

		/**
		 * Does the toolbar model contain the component with given id
		 * @param   {String} componentId
		 * @returns {Boolean}
		 */
		ToolbarModel.prototype.hasComponent = function (componentId) {
			return this._toolbarItems.getItemByKey(componentId) !== undefined;
		};

		/**
		 * Update the component with given id and data
		 * @param {String} componentId
		 * @param {Object} componentData
		 */
		ToolbarModel.prototype.updateComponent = function (componentId, componentData) {
			var component = this._toolbarItems.getItemByKey(componentId);
			if (component) {
				component.data = componentData;
				this._emit("componentDataChanged", componentId);
			}
		};

		/**
		 * Set the tooltip for a certain button
		 * @param {String} buttonId
		 * @param {String} tooltipText
		 */
		ToolbarModel.prototype.setButtonTooltip = function (buttonId, tooltipText) {
			var button = this._buttons[buttonId];
			if (button && button.tooltipText !== tooltipText) {
				button.tooltipText = tooltipText;
				this._emit("buttonStateChanged", buttonId);
			}
		};


		return ToolbarModel;
	});
