define([
		"underscore",
		"WFEditor/tools/AbstractTool",
		"WFEditor/tools/ToolConstants",
		"utils/ShapeUtils"
	],
	function (
		_,
		AbstractTool,
		ToolConstants,
		ShapeUtils
	) {
		"use strict";

		/**
		 * Abstract implementation of interactive tools in the Workflow Editor. Extends the AbstactTool
		 * class with extra features for interactivgity, e.g. pan when dragging towards the edges
		 *
		 * @exports AbstractInteractiveTool
		 * @augments AbstractTool
		 * @class
		 *
		 * @param {ToolConstants.Tools} toolId - ID for this tool
		 * @param {SceneController} sceneController
		 * @param {WFEditorContext} WFEditorContext
		 */
		var AbstractInteractiveTool = function (toolID, sceneController, WFEditorContext) {
			AbstractTool.call(this, WFEditorContext.get("scene").getLayerContext(), toolID, WFEditorContext);

			/**
			 * @member {SceneController}
			 * @private
			 */
			this._sceneController = sceneController;

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = WFEditorContext.get("scene");

			/**
			 * @member {SceneModel} model used by the scene
			 * @private
			 */
			this._sceneModel = WFEditorContext.get("sceneModel");

			/**
			 * @member {ToolManager}
			 * @private
			 */
			this._toolManager = WFEditorContext.get("toolManager");

			/**
			 * @member {ShapeUtils.Rect} the bounding rect of all views. Is renewed everytime the tool enters the default state. The bounding rect
			 * is cached because some tools may influence the bounding rect
			 * @private
			 */
			this._cachedBoundingRect = this._scene.getBoundingRect();

			/**
			 * @member {Number} id of the interval used for periodical panning
			 * @private
			 */
			this._intervalPanning = -1;

			/**
			 * @member {ShapeUtils.Point} This mouse position is used for automatic panning when the mouse is outside of the canvas
			 * @private
			 */
			this._currentMousePosition = null;

			/**
			 * @member {Function} The function called on mouse up when _continuePanningWhileMouseOutsideCanvas has been called
			 * @private
			 */
			this._boundedMouseUpBody = _.bind(this._mouseUpBody, this);

			/**
			 * @member {Function} Function called after this._boundedMouseUpBody has been dealt with. Should not be set directly, instead use continuePanningWhileMouseOutsideCanvas function
			 * @private
			 */
			this._mouseUpCallback = null;

			/**
			 * @member {Function} The function called on mousemove when _continuePanningWhileMouseOutsideCanvas has been called
			 * @private
			 */
			this._boundedAdjustMousePosition = _.bind(this._adjustMousePosition, this);

			/**
			 * @member {ShapeUtils.Point} start point for drawing edge or selection
			 * @private
			 */
			this._startPoint = null;

			/**
			 * @member {ShapeUtils.Point} the start point in world coordinates
			 * @private
			 */
			this._startPointWorld = null;
		};

		// extend from the AbstractTool
		AbstractInteractiveTool.prototype = Object.create(AbstractTool.prototype);
		AbstractInteractiveTool.prototype.constructor = AbstractInteractiveTool;

		/**
		 * Calculates the panning when close to the border
		 * @param {ShapeUtils.Point} position - The mouse position
		 */
		AbstractInteractiveTool.prototype._calculatePanning = function (position, stopAtBorder) {
			var canvasSize = this._scene.getSceneSize();
			var xOffset = 0;
			var yOffset = 0;
			var offset = ToolConstants.BorderOffsetForPanning;
			var worldPosition = this._scene.getWorldPointFromScenePoint(position);
			var visibleRect = this._scene.getVisibleRect();
			worldPosition.containPointInRect(visibleRect);
			var boundingRect = this._cachedBoundingRect;

			// Offsets should only be calculated if the cursor is close to/over the border and - if we should stop at workflow border - we're not there yet 
			if (canvasSize.width - position.x < offset && (worldPosition.x < boundingRect.normRight() || !stopAtBorder)) {
				xOffset = Math.max(-position.x + canvasSize.width - offset, -2 * offset);
			} else if (position.x < offset && (worldPosition.x > boundingRect.normLeft() || !stopAtBorder)) {
				xOffset = Math.min(offset - position.x, 2 * offset);
			}
			if (canvasSize.height - position.y < offset && (worldPosition.y < boundingRect.normBottom() || !stopAtBorder)) {
				yOffset = Math.max(-position.y + canvasSize.height - offset, -2 * offset);
			} else if (position.y < offset && (worldPosition.y > boundingRect.normTop() || !stopAtBorder)) {
				yOffset = Math.min(offset - position.y, 2 * offset);
			}
			var panningOffset = new ShapeUtils.Size(xOffset, yOffset);
			var currentPanningOffset = this._sceneModel.getPanningOffset();
			panningOffset.add(currentPanningOffset);
			this._sceneModel.setPanning(true);
			this._sceneModel.setPanningOffsetScene(panningOffset);
			this._sceneModel.setPanning(false);

		};

		/**
		 * Should the selection tool pan? ie. is the mouse close enough to the canvas border
		 * @param   {ShapeUtils.Point} position - The mouse position
		 * @returns {Boolean}
		 */
		AbstractInteractiveTool.prototype._shouldPan = function (position, stopAtBorder) {
			var canvasSize = this._scene.getSceneSize();
			var offset = ToolConstants.BorderOffsetForPanning;
			var worldPosition = this._scene.getWorldPointFromScenePoint(position);
			var visibleRect = this._scene.getVisibleRect();
			worldPosition.containPointInRect(visibleRect);
			var boundingRect = this._cachedBoundingRect;
			// We only want to pan if position is close to a border and - if we wish to stop at the border of the workflow - the corresponding world position is still in the bounding rect
			// Each line corresponds to a border of the canvas (right, left, bottom, top)
			return (canvasSize.width - position.x < offset && (worldPosition.x < boundingRect.normRight() || !stopAtBorder) ||
				(position.x < offset && (worldPosition.x > boundingRect.normLeft() || !stopAtBorder)) ||
				(canvasSize.height - position.y < offset && (worldPosition.y < boundingRect.normBottom() || !stopAtBorder)) ||
				(position.y < offset && (worldPosition.y > boundingRect.normTop() || !stopAtBorder)));
		};

		/**
		 * Starts the automatic panning;
		 * @param {ShapeUtils.Point} mousePosition
		 */
		AbstractInteractiveTool.prototype._startIntervalPanning = function (position, callback, stopAtBorder) {
			this._currentMousePosition = position;
			if (this._intervalPanning === -1) {
				this._intervalPanning = setInterval(_.bind(function () {
					if (this._intervalPanning !== -1) {
						if (this._shouldPan(this._currentMousePosition, stopAtBorder)) {
							this._calculatePanning(this._currentMousePosition, stopAtBorder);
						}
						callback.call(this, this._currentMousePosition);
					}
				}, this), 16);
			}
		};

		/**
		 * Stops the automatic panning
		 */
		AbstractInteractiveTool.prototype._stopIntervalPanning = function () {
			if (this._intervalPanning !== -1) {
				clearInterval(this._intervalPanning);
				this._intervalPanning = -1;
			}
		};

		/**
		 * Sets mouse listeners on <body>. An interval is started that will pan the scene if the mouse is outside the canvas. The interval
		 * will call intervalCallback each time it's fired.
		 * @param {Function} intervalCallback - called when moving the mouse, it has one parameter ie. ShapeUtils.Point the position of the mouse
		 * @param {Function} mouseupCallback - called once when the user releases the mouse on the body.
		 *                                   @param {Boolean} stopAtBorder    should the panning stop at the border of the bounding rect?
		 */
		AbstractInteractiveTool.prototype._continuePanningWhileMouseOutsideCanvas = function (intervalCallback, mouseupCallback, stopAtBorder) {
			this._mouseUpCallback = mouseupCallback;
			this._setMouseListenersOnWindow();
			this._startIntervalPanning(this._currentMousePosition, intervalCallback, stopAtBorder);

		};

		/**
		 * Removes all the mouselisteners on body and stops intervalpanning
		 */
		AbstractInteractiveTool.prototype._stopPanningWhileMouseOutsideCanvas = function () {
			this._removeMouseListenersOnWindow();
			this._stopIntervalPanning();

		};

		/**
		 * Sets the listeners on the window for panning when mouse outisde canvas
		 */
		AbstractInteractiveTool.prototype._setMouseListenersOnWindow = function () {
			this._removeMouseListenersOnWindow();
			$(window).on('mousemove', this._boundedAdjustMousePosition);
			$(window).on('mouseup', this._boundedMouseUpBody);
		};

		/**
		 * @override
		 */
		AbstractInteractiveTool.prototype.deActivate = function () {
			this._stopIntervalPanning();
			this._removeMouseListenersOnWindow();
			AbstractTool.prototype.deActivate.call(this);
		};

		/**
		 * removes the listeners on the window for panning when mouse outside canvas
		 */
		AbstractInteractiveTool.prototype._removeMouseListenersOnWindow = function () {
			$(window).off('mousemove', this._boundedAdjustMousePosition);
			$(window).off('mouseup', this._boundedMouseUpBody);
		};

		/**
		 * Adjusts the current mouse position, used for automatic panning
		 * @param {EventOb} e 
		 */
		AbstractInteractiveTool.prototype._adjustMousePosition = function (e) {
			var mousePosition = this._getMousePosition(e);
			this._currentMousePosition = mousePosition;
		};

		/**
		 * Called on mouseup when panning with mouse outside canvas
		 */
		AbstractInteractiveTool.prototype._mouseUpBody = function () {
			this._stopIntervalPanning();
			this._mouseUpCallback.call(this, this._currentMousePosition);
			this._removeMouseListenersOnWindow();
		};

		/**
		 * Registers an empty default state for this tool.
		 */
		AbstractInteractiveTool.prototype._registerDefaultState = function () {
			this.registerState(ToolConstants.State.DEFAULTSTATE, {
				"enterState": _.bind(function (e) {}, this),
				"leaveState": _.bind(function (e) {}, this),
				"mousemove": _.bind(function (e) {}, this),
				"mouseup": _.bind(function (e) {}, this),
				"mousedown": _.bind(function (e) {}, this),
				"mouseleave": _.bind(function (e) {}, this),
				"mouseenter": _.bind(function (e) {}, this),
				"contextmenu": function (e) {
					e.preventDefault();
				}
			});
		};



		return AbstractInteractiveTool;

	});