define([
		"underscore",
		"utils/ShapeUtils",
		"WFEditor/tools/ToolConstants",
		"utils/Observable"
	],
	function (
		_,
		ShapeUtils,
		ToolConstants,
		Observable
	) {
		"use strict";

		/**
		 * Abstract implementation of tools in the Workflow Editor. Defines a state pattern
		 * for tools and activate/deactivate functionality of the tool. Active and passive
		 * events can be registered.
		 *
		 * @exports AbstractTool
		 * @class
		 *
		 * @param {JqueryObj} context - object all the listeners will be registered to.
		 * @param {ToolConstants.Tools} toolID - ID for this tool
		 * @param {WFEditorContext} WFEditorContext - ID for this tool
		 */
		var AbstractTool = function AbstractTool(context, toolID, WFEditorContext) {
			Observable.mixin(this, ["WFEditor", "Tool"], toolID);

			/**
			 * @member {ToolConstants.Tools}
			 * @private
			 */
			this._toolID = toolID;

			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._context = context;
			
			/**
			 * @member {Mousetrap}
			 * @private
			 */
			this._keyboardHandler = WFEditorContext.get("keyboardHandler");

			/**
			 * @member {Map.<String,Function>} map of event name (active events) to function to be executed
			 * @private
			 */
			this._activeEvents = {};

			/**
			 * @member {Map.<String,Function>} map of event name (passive events) to function to be executed
			 * @private
			 */
			this._passiveEvents = {};

			/**
			 * @member {Map.<String,Map.<String, Function>>} maps state name on the map of events (event name to function)
			 * @private
			 */
			this._states = {};

			/**
			 * @member {String} currently active state name
			 * @private
			 */
			this._currentState = null;

			/**
			 * @member {Boolean} is the tool currently active
			 * @private
			 */
			this._isActive = false;

		};

		/**
		 * Get the id of this tool
		 * @returns {ToolConstants.Tools}
		 */
		AbstractTool.prototype.getID = function () {
			return this._toolID;
		};

		/**
		 * Is the tool currently active.
		 * @returns {Boolean}
		 */
		AbstractTool.prototype.isActive = function () {
			return this._isActive;
		};

		/**
		 * Register active events where the tool will listen to in one or multiple states.
		 * @param {Array.<String>} events -  list of all the events the tool will listen to.
		 */
		AbstractTool.prototype.registerActiveEvents = function (events) {
			_.each(events, function (event) {
				this._activeEvents[event] = _.bind(function (e) {
					this.getState(this._currentState)[event](e);
				}, this);
			}, this);
		};

		/**
		 * Register passive events where the tool will listen to in one or multiple states.
		 * @param {Array.<String>} events -  list of all the events the tool will listen to.
		 */
		AbstractTool.prototype.registerPassiveEvents = function (events) {
			if (!this.isActive() && this._currentState !== null) {
				this._context.off(this._passiveEvents);
			}
			_.each(events, function (event) {
				this._passiveEvents[event] = _.bind(function (e) {
					this.getState(this._currentState)[event](e);
				}, this);
			}, this);
			if (!this.isActive() && this._currentState !== null) {
				this._context.on(this._passiveEvents);
			}
		};

		/**
		 * Register a new state for the tool.
		 * @param {String} stateName
		 * @param {Map.<String,Function>} stateMappedListeners
		 */
		AbstractTool.prototype.registerState = function (stateName, stateMappedListeners) {
			this._states[stateName] = stateMappedListeners;
		};

		/**
		 * triggers the event on the current state
		 * @param {String} name
		 */
		AbstractTool.prototype.triggerEvent = function (name) {
			var eventName = arguments[0];
			Array.prototype.shift.apply(arguments);
			this._states[this._currentState][eventName].apply(this._states[this._currentState], arguments);
		};

		/**
		 * Activates the tool on the context.
		 * @param {EventObj} event
		 * @param {Boolean} silent - should the activation of the tool fire events
		 */
		AbstractTool.prototype.activate = function (event, silent) {
			var canActivate = true;
			if (!silent) {
				this._emit("activating", canActivate);
			}
			if (canActivate && !this._isActive) {
				if (!this._currentState) {
					this.changeState(ToolConstants.State.DEFAULTSTATE);
				}
				this._context.off(this._passiveEvents);
				this._context.on(this._activeEvents);
				this._isActive = true;
			}
		};

		/**
		 * Deactivate the tool. The passive events are now registered.
		 * @param {EventObj} event
		 * @param {Boolean} silent - should the activation of the tool fire events
		 */
		AbstractTool.prototype.deActivate = function (event, silent) {
			var canDeActivate = true;
			if (!silent) {
				this._emit("deActivating", canDeActivate);
			}
			if (canDeActivate && this._isActive) {
				this.changeState(ToolConstants.State.DEFAULTSTATE);
				this._context.off(this._activeEvents);
				this._context.on(this._passiveEvents);
				this._isActive = false;
			}
		};

		/**
		 * Get the state with a given name.
		 * @param   {String} stateName
		 * @returns {Map.<String, Function>}
		 */
		AbstractTool.prototype.getState = function (stateName) {
			return this._states[stateName];
		};

		/**
		 * Changes the state of the tool.
		 * @param {String} stateName
		 */
		AbstractTool.prototype.changeState = function (stateName) {
			if (this._currentState !== stateName) {
				// execute leave state if set
				if (this._states[this._currentState] && this._states[this._currentState].leaveState) {
					this._states[this._currentState].leaveState.apply(this._states[this._currentState], arguments);
				}

				var previousState = this._currentState;
				this._currentState = stateName;
				arguments[0] = previousState;
				if (this._states[this._currentState] && this._states[this._currentState].enterState) {
					this._states[this._currentState].enterState.apply(this._states[this._currentState], arguments);
				}
			}
		};

		/**
		 * Bind key events to an action of the tool
		 * @param {Array.<String>} keys - array with all the key identifiers to listen to
		 * @param {Function} callback - callback to execute
		 * @param {String} keyEvent - the event fired for the key (keyup, keydown,...)
		 */
		AbstractTool.prototype.bindKeyEvents = function (keys, callback, keyEvent) {
			this._keyboardHandler.bind(keys, callback, keyEvent);
		};

		/**
		 * Get the current position of the mouse in scene coordinates
		 * @param   {Event}            event
		 * @returns {ShapeUtils.Point}
		 */
		AbstractTool.prototype._getMousePosition = function (event) {
			var position = new ShapeUtils.Point();
			var contextRect = this._context[0].getBoundingClientRect();
			if ((event.pageX || (event.pageX === 0)) && (event.pageY || event.pageY === 0)) {
				position = new ShapeUtils.Point(event.pageX - contextRect.left - window.pageXOffset, event.pageY - contextRect.top - window.pageYOffset);
			} else if ((event.clientX || event.clientX === 0) && (event.clientY || event.clientY === 0)) {
				position = new ShapeUtils.Point(event.clientX - contextRect.left, event.clientY - contextRect.top);
			}
			return position;
		};

		/**
		 * Returns the default cursor for tools, needs to be implemented by specific tool
		 * @returns {ToolConstants.Cursors}
		 */
		AbstractTool.prototype.getDefaultCursor = function () {
			return ToolConstants.Cursors.DEFAULT;
		};

		/**
		 * can the tool be deactivated? Should be implemented by specific tools
		 * @returns {Boolean} 
		 */
		AbstractTool.prototype.allowDeactivate = function () {
			return true;
		};

		/**
		 * can the tool be activated? Should be implemented by specific tools
		 * @returns Boolean
		 */
		AbstractTool.prototype.allowActivate = function () {
			return true;
		};

		return AbstractTool;

	});
