define(
	[
		"WFEditor/tools/AbstractTool",
		"WFEditor/tools/ToolConstants",
		"utils/Command",
		"utils/Observer",
		"utils/ShapeUtils",
		"jquery",
		"underscore",
		"utils/webcss!WFEditor/nodeGallery/view/NodeGalleryList-style.css"
	],
	function (
		AbstractTool,
		ToolConstants,
		Command,
		Observer,
		ShapeUtils,
		$,
		_
	) {

		"use strict";

		/**
		 * The tool that lets the user select a node from the gallery and drop it into the workflow
		 * @class
		 * @augments AbstractTool
		 * @exports GalleryNodeSelectorTool
		 * @mixes Observer
		 * @param {NodeGallery} nodeGallery - the nodegallery the tool listens to for selecting of nodes
		 * @param {SceneController} sceneController
		 * @param {WFEditorContext}
		 */
		var GalleryNodeSelectorTool = function (nodeGallery, sceneController, WFEditorContext) {
			AbstractTool.call(this, $(window), "addNodeTool", WFEditorContext);
			Observer.mixin(this);

			/**
			 * @member {NodeGallery}
			 * @private
			 */
			this._nodeGallery = nodeGallery;

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = WFEditorContext.get("scene");

			/**			
			 * @member {SceneController}
			 * @private
			 */
			this._sceneController = sceneController;

			/**
			 * @member {ErrorHandler}
			 * @private
			 */
			this._errorHandler = WFEditorContext.get("errorHandler");

			/**
			 * @member {JQueryPromise} - the node that was selected in the gallery
			 * @private
			 */
			this._createdNodePromise = null;

			/**
			 * @member {Workflow}
			 * @private
			 */
			this._workflow = WFEditorContext.get("workflow");

			/**
			 * @member {AbstractWorkflowFactory}
			 * @private
			 */
			this._workflowFactory = WFEditorContext.get("workflowFactory");

			/**
			 * @member {ToolManager}
			 * @private
			 */
			this._toolManager = WFEditorContext.get("toolManager");

			/**
			 * @member {ToolbarModel}
			 * @private
			 */
			this._toolbarModel = WFEditorContext.get("toolbarModel");
			/**
			 * @member {UndoRedoController}
			 * @private
			 */
			this._undoredo = WFEditorContext.get("undoRedo");

			/**
			 * @member {JQueryObject} - the image object used to display the selected node while dragging
			 * @private
			 */
			this._img = $("<img/>").addClass("galleryIconImg helper").hide();
			WFEditorContext.get("scene").getContext().append(this._img);

			this._registerListeners();
			this.registerActiveEvents(["mousemove", "mouseup"]);

			/**
			 * @member {Number} - x offset used to display the node icon under the mouse pointer
			 * @private
			 */
			this._offsetX = 0;

			/**
			 * @member {Number} - y offset used to display the node icon under the mouse pointer
			 * @private
			 */
			this._offsetY = 0;

		};

		GalleryNodeSelectorTool.prototype = Object.create(AbstractTool.prototype);
		GalleryNodeSelectorTool.prototype.constructor = GalleryNodeSelectorTool;

		/**
		 * Registers the listeners for the Tool
		 */
		GalleryNodeSelectorTool.prototype._registerListeners = function () {

			// Listen to the gallery for when a node is selected
			this._observe(this._nodeGallery, "gallery.selectNode", function (galleryNode, e) {
				// Create the node on select so that the factory may already start getting data from server
				this._createdNodePromise = this._workflowFactory.createNode(this._workflow, galleryNode.getOptions());
				this._img.attr('src', galleryNode.getIconUrl());
				this._scene.focus();
				if ($(e.target).is('img')) {
					// If the image was selected, offset the image so that is nicely placed under the mouse
					this._offsetX = e.offsetX;
					this._offsetY = e.offsetY;
				} else {
					// Else set the middle of the image to the mouse pointer
					this._offsetX = this._img.width() / 2;
					this._offsetY = this._img.height() / 2;
				}
				this._moveImage(e);
				this._img.show();
				this._toolManager.activateTool(this, true);
				this.changeState(ToolConstants.State.DRAGGINGNEWNODESTATE);
			});



			this.registerState(ToolConstants.State.DRAGGINGNEWNODESTATE, {
				"leaveState": _.bind(function (e) {
					this._createdNodePromise = null;
					if (this.isActive()) {
						this._toolManager.deactivateCurrentTool();
					}
				}, this),
				"enterState": _.bind(function (e) {
					this._createdNodePromise.fail(_.bind(function () {
						this._errorHandler.warning.apply(this._errorHandler, arguments);
					}, this));
				}, this),
				// create a new node on mouseup. Bind mouseup on window because event doesn't fire on the scene context element
				'mouseup': _.bind(function (e) {
					this._img.hide();
					if (this._createdNodePromise !== null && this._inVisibleRect(e)) {
						// The mouse was released inside the scene, create a new node.
						var mousePos = this._scene.getWorldPointFromScenePoint(this._getMousePositionRelativeToScene(e));
						mousePos.offsetXY(-this._offsetX, -this._offsetY);
						this._createdNodePromise.done(_.bind(function (newNode) {
							newNode.setLocation(mousePos);
							this._sceneController.addNodes(this._workflow, [newNode]);
							this._sceneController.selectNode(newNode, true);
						}, this));
					}
					this.changeState(ToolConstants.State.DEFAULTSTATE);

				}, this),
				"mousemove": _.bind(function (e) {
					this._moveImage(e);
				}, this)

			});
		};

		/**
		 * Move the node icon along with the mouse
		 * @param {Event} event - the mouse event to move along with
		 */
		GalleryNodeSelectorTool.prototype._moveImage = function (event) {
			var x = 0;
			var y = 0;

			// set the top left of the image to be under the mouse
			if ((event.pageX || (event.pageX === 0)) && (event.pageY || event.pageY === 0)) {
				x = event.pageX - window.pageXOffset;
				y = event.pageY - window.pageYOffset;
			} else if ((event.clientX || event.clientX === 0) && (event.clientY || event.clientY === 0)) {
				x = event.clientX - window.pageXOffset;
				y = event.clientY - window.pageYOffset;
			}

			// Offset image to be at the right position under the mouse
			if (this._offsetX) {
				x -= this._offsetX;
			}
			if (this._offsetY) {
				y -= this._offsetY;
			}
			this._img.css({
				left: x,
				top: y
			});
			event.preventDefault();
		};



		/**
		 * checks if the mouse event is occuring in the scene
		 * @param   {Event}   e 
		 * @returns {Boolean} 
		 */
		GalleryNodeSelectorTool.prototype._inVisibleRect = function (e) {
			var mousePosition = this._getMousePositionRelativeToScene(e);
			var sceneSize = this._scene.getSceneSize();
			return !(mousePosition.x < 0 || mousePosition.x > sceneSize.width || mousePosition.y < 0 || mousePosition.y > sceneSize.height);
		};

		/**
		 * Gets the mouse position relative to the Scene
		 * @param   {Event}   e
		 * @returns {ShapeUtils.Point}
		 */
		GalleryNodeSelectorTool.prototype._getMousePositionRelativeToScene = function (event) {
			// can't use getMousePosition of abstract tool when context is window
			var position = new ShapeUtils.Point();
			var contextRect = this._scene.getContext()[0].getBoundingClientRect();
			if ((event.pageX || (event.pageX === 0)) && (event.pageY || event.pageY === 0)) {
				position = new ShapeUtils.Point(event.pageX - contextRect.left  - window.pageXOffset, event.pageY - contextRect.top - window.pageYOffset);
			} else if ((event.clientX || event.clientX === 0) && (event.clientY || event.clientY === 0)) {
				position = new ShapeUtils.Point(event.clientX - contextRect.left - window.pageXOffset, event.clientY - contextRect.top - window.pageYOffset);
			}
			return position;
		};

		/**
		 * @override
		 */
		GalleryNodeSelectorTool.prototype.activate = function (event, silent) {
			AbstractTool.prototype.activate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, true);
		};

		/**
		 * @override
		 */
		GalleryNodeSelectorTool.prototype.deActivate = function (event, silent) {
			AbstractTool.prototype.deActivate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, false);
		};

		/**
		 * @override
		 */
		GalleryNodeSelectorTool.prototype.allowDeactivate = function () {
			return this._createdNodePromise === null;
		};

		return GalleryNodeSelectorTool;

	});
