define([
		"underscore",
		"jquery",
		"WFEditor/tools/AbstractInteractiveTool",
		"WFEditor/tools/ToolConstants",
		"WFEditor/utils/WFI18nConstants",
		"WFEditor/workflow/ModelConstants",
		"WFEditor/tools/selection/SelectionHelper",
		"utils/ShapeUtils",
		"utils/Observable"
	],
	function (
		_,
		$,
		AbstractInteractiveTool,
		ToolConstants,
		WFI18nConstants,
		ModelConstants,
		SelectionHelper,
		ShapeUtils,
		Observable
	) {
		"use strict";

		/**
		 * Tool that handles all the mouse interaction on the scene.
		 *
		 * @exports MouseInteractionTool
		 * @augments Viewport.AbstractInteractiveTool
		 * @class
		 *
		 * @param {SceneController} sceneController - The scene controller
		 * @param {WFEditorContext} WFEditorContext
		 */
		var MouseInteractionTool = function (sceneController, WFEditorContext) {
			AbstractInteractiveTool.call(this, ToolConstants.Tools.MOUSEINTERACTION, sceneController, WFEditorContext);
			Observable.mixin(this, "mouseInteraction");

			/**
			 * @member {ToolbarModel} the model for the toolbar, used to add toolbar buttons
			 * @private
			 */
			this._toolbarModel = WFEditorContext.get("toolbarModel");

			/**
			 * @member {ToolManager}
			 * @private
			 */
			this._toolManager = WFEditorContext.get("toolManager");

			/**
			 * @member {AbstractView} object that is currently hovered, saved to set hovered to false when needed
			 * @private
			 */
			this._currentHoverObj = null;

			//register the default state to the tool
			this._registerDefaultState();
			//register all the events that this tool listens to
			this.registerActiveEvents(["mousemove", "mousedown", "mouseup", "mouseleave", "mouseenter", "contextmenu", "dblclick", "click"]);

			//add toolbar buttons for the mouse interaction tool
			this._toolbarModel.addComponent(ToolConstants.ButtonGroups.NAVIGATION, "navigationTools");
			this._toolbarModel.addButton(ToolConstants.ButtonGroups.NAVIGATION, ToolConstants.Buttons.SELECTION, "icon-WCR_WF-Select", _.bind(function () {
				this._toolManager.resetToDefaultTool(true);
				this._toolManager.activateTool(this, true);

			}, this), WFEditorContext.get("i18n").t(WFI18nConstants.pointerToolTooltip));


			// Control
			this.bindKeyEvents("mod", _.bind(function (e) {
				this._emit("keyevent.keydown.ctrl");
				e.preventDefault();
				e.stopPropagation();
			}, this), "keydown");
			this.bindKeyEvents("mod", _.bind(function (e) {
				this._emit("keyevent.keyup.ctrl");
				e.preventDefault();
				e.stopPropagation();
			}, this), "keyup");

			// Shift
			this.bindKeyEvents("shift", _.bind(function (e) {
				this._emit("keyevent.keydown.shift");
				e.preventDefault();
				e.stopPropagation();
			}, this), "keydown");
			this.bindKeyEvents("shift", _.bind(function (e) {
				this._emit("keyevent.keyup.shift");
				e.preventDefault();
				e.stopPropagation();
			}, this), "keyup");
		};

		// extend from the AbstractInteractiveTool
		MouseInteractionTool.prototype = Object.create(AbstractInteractiveTool.prototype);
		MouseInteractionTool.prototype.constructor = MouseInteractionTool;

		/**
		 * Activate the mouse interaction tool.
		 * @param {EventObj} event
		 * @param {Boolean} silent - should the tool fire events
		 */
		MouseInteractionTool.prototype.activate = function (event, silent) {
			AbstractInteractiveTool.prototype.activate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, true);
		};

		/**
		 * deActivate the mouse interaction tool.
		 * @param {EventObj} event
		 * @param {Boolean} silent - should the tool fire events
		 */
		MouseInteractionTool.prototype.deActivate = function (event, silent) {
			AbstractInteractiveTool.prototype.deActivate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, false);
		};

		/**
		 * Registers the default state of this tool.
		 * @override
		 */
		MouseInteractionTool.prototype._registerDefaultState = function () {
			this.registerState(ToolConstants.State.DEFAULTSTATE, {
				"enterState": _.bind(function () {
					this._stopIntervalPanning();
				}, this),
				"mousemove": _.bind(function (e) {
					// send hover event, but only 
					var position = this._getMousePosition(e);
					var hitArea = this._scene.getHitAreaAtPosition(position);
					if(hitArea && hitArea.getOption(ModelConstants.HitAreaOptions.CURSOR)) {
						this._sceneModel.setCursor(hitArea.getOption(ModelConstants.HitAreaOptions.CURSOR));
					} else {
						this._sceneModel.setCursor(this.getDefaultCursor());
					}
					this._checkHovering(hitArea ? hitArea.getView() : null);
					if (hitArea) {
						this._sendEvent("hover", e);
					}
					this._sendEvent("mousemove", e);
				}, this),
				"mousedown": _.bind(function (e) {
						this._scene.focus();
						this._sendEvent("mousedown", e);
					},
					this),
				"mouseup": _.bind(function (e) {
						this._sendEvent("mouseup", e);
					},
					this),
				"mouseenter": function (e) {},
				"mouseleave": function (e) {},
				"contextmenu": function (e) {
					e.preventDefault();
					e.stopPropagation();
				},
				"dblclick": _.bind(function (e) {
					this._sendEvent("dblclick", e);

				}, this),
				"click": _.bind(function (e) {
					this._sendEvent("click", e);

				}, this)
			});
		};

		/**
		 * Check the hovering state for the given viewObject
		 * @param {AbstractView} viewObject - The view object under the current mouse position
		 */
		MouseInteractionTool.prototype._checkHovering = function (viewObject) {
			// if the (previously) hovered object is not the current view object under the mouse position, set it to no longer hovered
			if (this._currentHoverObj && viewObject !== this._currentHoverObj) {
				this._currentHoverObj.getModel().setHovered(false);
			}
			if (viewObject && viewObject !== this._currentHoverObj) {
				viewObject.getModel().setHovered(true);
				this._currentHoverObj = viewObject;
			}
			if (this._currentHoverObj && !viewObject) {
				this._currentHoverObj = null;
			}
		};

		/**
		 * Sends an object containing hitArea, mouse position, event name and event to all listeners
		 * @param {String} eventName 
		 * @param {EventObject}  e
		 */
		MouseInteractionTool.prototype._sendEvent = function (eventName, e) {
			var position = this._getMousePosition(e);
			e.preventDefault();
			var hitArea = this._scene.getHitAreaAtPosition(position);
			this._emit(eventName, {
				eventName: eventName,
				e: e,
				hitArea: hitArea,
				position: position
			});
		};

		return MouseInteractionTool;

	});