define([
		"underscore",
		"WFEditor/tools/AbstractTool",
		"WFEditor/tools/ToolConstants",
		"utils/Mousewheel",
		"utils/ShapeUtils",
		"utils/Observer",
		"WFEditor/utils/WFI18nConstants"



	],
	function (
		_,
		AbstractTool,
		ToolConstants,
		Mousewheel,
		ShapeUtils,
		Observer,
		WFI18nConstants

	) {
		"use strict";

		/**
		 * Tool that handles panning in the workflow editor
		 *
		 * @exports PanTool
		 * @augments AbstractTool
		 * @class
		 *
		 * @param {WFEditorContext} WFEditorContext 
		 */
		var PanTool = function (WFEditorContext) {
			AbstractTool.call(this, WFEditorContext.get("scene").getLayerContext(), ToolConstants.Tools.PANTOOL, WFEditorContext);

			Observer.mixin(this);

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = WFEditorContext.get("scene");

			/**
			 * @member {SceneModel} model used by the scene
			 * @private
			 */
			this._sceneModel = WFEditorContext.get("sceneModel");

			/**
			 * @member {ToolbarModel} the model for the toolbar, used to add toolbar buttons
			 * @private
			 */
			this._toolbarModel = WFEditorContext.get("toolbarModel");

			/**
			 * @member {ToolManager}
			 * @private
			 */
			this._toolManager = WFEditorContext.get("toolManager");

			/**
			 * @member {ShapeUtils.Point} the start point for when panning started
			 * @private
			 */
			this._startPoint = null;

			/**
			 * @member {Boolean} has the pan tool been activated by the space bar
			 * @private
			 */
			this._isSpaceActivated = false;

			//register the default the states to the tool
			this._registerDefaultState();
			this._registerPanningState();
			//register all the events that this tool listens to
			this.registerActiveEvents(["mousedown", "mousemove", "mouseup", "mouseleave"]);

			//activate pan tool on space down
			this.bindKeyEvents("space", _.bind(function (e) {
				if (!this.isActive()) {
					this._toolManager.activateTool(this);
					this._isSpaceActivated = true;
				}
			}, this), "keydown");

			// on space up de-activate the pan tool
			this.bindKeyEvents("space", _.bind(function (e) {
				if (this._isSpaceActivated) {
					this._isSpaceActivated = false;
					this._toolManager.deactivateCurrentTool();
				}
			}, this), "keyup");

			this.bindKeyEvents("up", _.bind(function (e) {
				var contextRect = this._context[0].getBoundingClientRect();
				var panOffset = new ShapeUtils.Size(0, contextRect.height / 16);

				this._panSceneOffset(panOffset);
			}, this), "keydown");

			this.bindKeyEvents("down", _.bind(function (e) {
				var contextRect = this._context[0].getBoundingClientRect();
				var panOffset = new ShapeUtils.Size(0, -contextRect.height / 16);

				this._panSceneOffset(panOffset);
			}, this), "keydown");

			this.bindKeyEvents("left", _.bind(function (e) {
				var contextRect = this._context[0].getBoundingClientRect();
				var panOffset = new ShapeUtils.Size(contextRect.width / 16, 0);

				this._panSceneOffset(panOffset);
			}, this), "keydown");

			this.bindKeyEvents("right", _.bind(function (e) {
				var contextRect = this._context[0].getBoundingClientRect();
				var panOffset = new ShapeUtils.Size(-contextRect.width / 16, 0);

				this._panSceneOffset(panOffset);
			}, this), "keydown");

			var tooltip = WFEditorContext.get("i18n").t(WFI18nConstants.panToolTooltip);

			this._toolbarModel.addComponent(ToolConstants.ButtonGroups.NAVIGATION, "navigationTools");
			this._toolbarModel.addButton(ToolConstants.ButtonGroups.NAVIGATION, ToolConstants.Buttons.PAN, "icon-WCR_WF-Pan", _.bind(function () {
				this._toolManager.activateTool(this, true);
			}, this), tooltip);

			this._intializeScrollbarBehavior();
		};

		// extend from the AbstractTool
		PanTool.prototype = Object.create(AbstractTool.prototype);
		PanTool.prototype.constructor = PanTool;

		/**
		 * Activate the pan tool.
		 * @param {EventObj} event
		 * @param {Boolean} silent - should the tool fire events
		 */
		PanTool.prototype.activate = function (event, silent) {
			AbstractTool.prototype.activate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.PAN, true);
		};

		/**
		 * deActivate the pan tool.
		 * @param {EventObj} event
		 * @param {Boolean} silent - should the tool fire events
		 */
		PanTool.prototype.deActivate = function (event, silent) {
			AbstractTool.prototype.deActivate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.PAN, false);
		};

		/**
		 * Registers the default state of this tool.
		 */
		PanTool.prototype._registerDefaultState = function () {
			this.registerState(ToolConstants.State.DEFAULTSTATE, {

				"mousedown": _.bind(function (e) {
					if (e.which === 1) {
						this._scene.focus();
						e.preventDefault();
						this._startPoint = this._getMousePosition(e);
						this.changeState(ToolConstants.State.PANNINGSTATE);
					}
				}, this),
				"mousemove": _.bind(function (e) {

				}, this),
				"mouseup": _.bind(function (e) {

				}, this),
				"mouseleave": _.bind(function (e) {

				}, this)
			});
		};

		/**
		 * Register the tool state for panning.
		 */
		PanTool.prototype._registerPanningState = function () {
			this.registerState(ToolConstants.State.PANNINGSTATE, {
				"enterState": _.bind(function () {
					this._sceneModel.setPanning(true);
					this._scene.blockPointerEventsForDOMElements(true);
				}, this),
				"leaveState": _.bind(function () {
					this._sceneModel.setPanning(false);
					this._scene.blockPointerEventsForDOMElements(false);
				}, this),
				"mousemove": _.bind(function (e) {
					this._calculateOffset(e);
				}, this),
				"mousedown": _.bind(function (e) {
					e.preventDefault();
					this._calculateOffset(e);
					this.changeState(ToolConstants.State.DEFAULTSTATE);
				}, this),
				"mouseleave": _.bind(function (e) {
					e.preventDefault();
					this._calculateOffset(e);
					this.changeState(ToolConstants.State.DEFAULTSTATE);
				}, this),
				"mouseup": _.bind(function (e) {
					e.preventDefault();
					this._calculateOffset(e);
					this.changeState(ToolConstants.State.DEFAULTSTATE);
				}, this)
			});
		};



		/**
		 * Calculate the correct offset for panning.
		 * @param {EventObj} event - the event object from which the current mouseposition can be calculated
		 */
		PanTool.prototype._calculateOffset = function (event) {
			var mousePosition = this._getMousePosition(event);
			var offset = this._startPoint.sizeTo(mousePosition);
			if (Math.abs(offset.width) > 0 || Math.abs(offset.height) > 0) {
				this._sceneModel.setPanningOffsetScene(offset);
			}
		};

		/**
		 * Pan with a given scene offset.
		 * @param {ShapeUtils.Size} offset - offset in scene coordinates.
		 */
		PanTool.prototype._panSceneOffset = function (offset) {
			this._sceneModel.setPanning(true);
			this._sceneModel.setPanningOffsetScene(offset);
			this._sceneModel.setPanning(false);
		};

		/**
		 * Returns the default cursor for the pan tool
		 * @returns {ToolConstants.Cursors}
		 */
		PanTool.prototype.getDefaultCursor = function () {
			return ToolConstants.Cursors.HAND;
		};


		/**
		 * Initialize the behavior for the scrollbars.
		 * When user uses scrollbar: activate pan tool, execute pan movements
		 */
		PanTool.prototype._intializeScrollbarBehavior = function () {
			this._registerScrollbarState();

			_.each(this._scene.getScrollbars(), function (scrollbar) {
				this._observe(scrollbar, "start", function (interactively) {
					this._wasActiveBeforeScrollbar = this.isActive();
					if (!this._wasActiveBeforeScrollbar) {
						this._toolManager.activateTool(this);
					}
					this.changeState(ToolConstants.State.PANNINGSCROLLBARSTATE);
				});

				this._observe(scrollbar, "change", function (interactively, offset) {
					if (interactively) {
						this._sceneModel.panWorld(offset);
					}
				});
				this._observe(scrollbar, "stop", function (interactively) {
					this.changeState(ToolConstants.State.DEFAULTSTATE);

					if (!this._wasActiveBeforeScrollbar) {
						this._toolManager.deactivateCurrentTool();
					}

				});

			}, this);

		};

		/**
		 * Create the Scrollbar state, this state is used when the user is interacting with the scrollbars
		 */
		PanTool.prototype._registerScrollbarState = function () {
			this.registerState(ToolConstants.State.PANNINGSCROLLBARSTATE, {
				"enterState": _.bind(function () {
					this._sceneModel.setPanning(true);
				}, this),
				"leaveState": _.bind(function () {
					this._sceneModel.setPanning(false);
				}, this),
				"mousemove": function () {},
				"mousedown": function () {},
				"mouseleave": function () {},
				"mouseup": function () {}
			});
		};

		return PanTool;

	});
