define([
		"underscore"
	],
	function (
		_

	) {
		"use strict";

		/**
		 * ToolManager is the main controller for all the tools and is responsible for
		 * (de)activating tools when needed.
		 *
		 * @exports ToolManager
		 * @class
		 *
		 * @param {SceneModel} sceneModel
		 * @param {AbstractTool} defaultTool
		 */
		var ToolManager = function (sceneModel, defaultTool) {

			/**
			 * @member {AbstractTool} the default tool, the toolmanager will revert to this tool if no tools on the stack
			 * @private
			 */
			this._defaultTool = defaultTool;

			/**
			 * @member {SceneModel} the toolmanager will use the scene model to set the cursor
			 * @private
			 */
			this._sceneModel = sceneModel;

			/**
			 * @member {Array.<AbstractTool>} the stack of tools in use
			 * @private
			 */
			this._toolStack = [];

			if (typeof defaultTool !== "undefined") {
				defaultTool.activate();
			}



		};

		/**
		 * activate the tool if the current tool can be deactivated 
		 * @param {AbstractTool} tool - the tool to be activated
		 * @param {Boolean} [forceActivate = false] - if true, doesn't ask current tool if it's ok to deactivate it
		 */
		ToolManager.prototype.activateTool = function (tool, forceActivate) {
			if (typeof forceActivate === "undefined") {
				forceActivate = false;
			}
			var currentTool = this._toolStack.length === 0 ? this._defaultTool : this._toolStack[this._toolStack.length - 1];
			if (tool === currentTool) {
				// The tool is already active
				return;
			}
			if (forceActivate || (currentTool.allowDeactivate() && tool.allowActivate())) {
				currentTool.deActivate();
				this._toolStack.push(tool);
				tool.activate();
			}
			this.updateCursor();

		};

		/**
		 * deactivate the current tool if it can be deactivated 
		 * @param {Boolean} [forceActivate = false] - if true, doesn't ask current tool if it's ok to deactivate it
		 */
		ToolManager.prototype.deactivateCurrentTool = function (forceDeactivate) {
			if (typeof forceDeactivate === "undefined") {
				forceDeactivate = false;
			}
			if (this._toolStack.length > 0 && (forceDeactivate || this._toolStack[this._toolStack.length - 1].allowDeactivate())) {
				// The toolmanager is allowed to deactivate the tool
				this._toolStack.pop().deActivate();
				if (this._toolStack.length === 0) {
					// We popped the last tool on the stack
					this._defaultTool.activate();
				} else {
					var toolActivated = false;
					while (!toolActivated && this._toolStack.length > 0) {
						if (this._toolStack[this._toolStack.length - 1].allowActivate()) {
							this._toolStack[this._toolStack.length - 1].activate();
							toolActivated = true;
						} else {
							this._toolStack.pop();
						}
					}
					if (this._toolStack.length === 0) {
						this._defaultTool.activate();
					}
				}
			}
			this.updateCursor();

		};

		/**
		 * Resets to the default tool if the current tool can be deactivated
		 * @param {Boolean} force - force the reset, even if current tool doesn't want to be deactivated.
		 */
		ToolManager.prototype.resetToDefaultTool = function (force) {
			if (this._toolStack.length > 0 && (force || this._toolStack[this._toolStack.length - 1].allowDeactivate())) {
				// deactivate current tool, reset the stack and activate default tool
				this._toolStack[this._toolStack.length - 1].deActivate();
				this._toolStack = [];
				this._defaultTool.activate();
			}
			this.updateCursor();

		};

		/**
		 * sets the default tool
		 * @param {AbstractTool} tool 
		 */
		ToolManager.prototype.setDefaultTool = function (tool) {
			if (tool !== this._defaultTool) {
				if (this._defaultTool && this._defaultTool.isActive()) {
					this._defaultTool.deActivate();
				}
				this._defaultTool = tool;
				this._defaultTool.activate();
			}
			this.updateCursor();
		};

		/**
		 * Update the cursor for the  tools in the scene model
		 */
		ToolManager.prototype.updateCursor = function () {
			var currentTool = this._toolStack.length === 0 ? this._defaultTool : this._toolStack[this._toolStack.length - 1];

			this._sceneModel.setCursor(currentTool.getDefaultCursor());
		};

		return ToolManager;

	});
