define(["underscore",
	"jquery",
	"WFEditor/tools/AbstractTool",
	"component/tooltip/Tooltip",
	"WFEditor/tools/ToolConstants",
	"WFEditor/workflow/ModelConstants",
	"utils/Observer",
	"utils/ShapeUtils"
], function (_,
	$,
	AbstractTool,
	Tooltip,
	ToolConstants,
	ModelConstants,
	Observer,
	ShapeUtils) {

	"use strict";

	/**
	 * A class that handles tooltips on the canvas
	 * 
	 * @class
	 * @exports TooltipTool
	 * @augments AbstractTool
	 * @mixes Observer
	 * @param {Observable} mouseInteractionTool 
	 * @param {WFEditorContext}       WFEditorContext                 
	 */
	var TooltipTool = function (mouseInteractionTool, WFEditorContext) {
		
		/**
		 * @member {Scene}
		 * @private
		 */ 
		this._scene = WFEditorContext.get("scene");
		
		var sceneContext = this._scene.getLayerContext();
		Observer.mixin(this);
		AbstractTool.call(this, sceneContext, "TooltipTool", WFEditorContext);

		/**
		 * @member {JQueryObject} div to which the tooltip will be attached
		 * @private
		 */ 
		this._tooltipContextDiv = $("<div/>").css({
			position: "relative"
		});

		/**
		 * @member {Tooltip}
		 * @private
		 */
		this._tooltip = new Tooltip(WFEditorContext.get("i18n"), this._tooltipContextDiv, sceneContext, sceneContext, false, "");

		
		// Forward all events on the tooltipContextDiv to the scene 
		this._tooltipContextDiv.on("mousedown mousemove mouseup click dblclick", _.bind(function (e) {
			this._scene.getLayerContext().trigger(e);
		}, this));

		
		this._observe(mouseInteractionTool, "hover", this._showOrHideTooltip);
		this._observe(mouseInteractionTool, "mousemove", this._showOrHideTooltip);
		this._observe(mouseInteractionTool, "mousedown", this._hideTooltip);
		this._observe(mouseInteractionTool, "deActivating", this._hideTooltip);


	};

	TooltipTool.prototype = Object.create(AbstractTool.prototype);
	TooltipTool.prototype.constructor = TooltipTool;


	/**
	 * check if the tooltip should be shown or hidden
	 * @param {Object} options as sent by MouseInteractionTool
	 */
	TooltipTool.prototype._showOrHideTooltip = function (options) {
		if (options.hitArea && options.hitArea.getOption(ModelConstants.HitAreaOptions.TOOLTIP)) {
			this._showTooltip(options.hitArea);
		} else {
			this._hideTooltip();
		}
	};

	/**
	 * hide the tooltip
	 */
	TooltipTool.prototype._hideTooltip = function () {
		this._tooltip.disableTooltip();
		this._tooltip.hide();
		this._tooltipContextDiv.detach();
	};

	/**
	 * Show tooltip 
	 * @param {HitArea} hitArea - the hitarea under which the tooltip should be shown
	 */
	TooltipTool.prototype._showTooltip = function (hitArea) {
		var t = this._scene.getTransformation();
		var sceneHitAreaOrigin = t.transformedPoint(

			new ShapeUtils.Point(hitArea[0], hitArea[1]));
		var sceneHitAreaSize = t.transformedSize(
			new ShapeUtils.Size(hitArea[2] - hitArea[0], hitArea[3] - hitArea[1]));
		this._tooltipContextDiv.css({
			left: sceneHitAreaOrigin.x,
			top: sceneHitAreaOrigin.y,
			width: sceneHitAreaSize.width,
			height: sceneHitAreaSize.height
		});
		this._tooltip.enableTooltip();
		this._scene.getLayerContext().append(this._tooltipContextDiv);
		this._tooltip.setTooltipContent(hitArea.getOption(ModelConstants.HitAreaOptions.TOOLTIP));

	};


	return TooltipTool;
});
