define([
		"underscore",
		"WFEditor/tools/AbstractTool",
		"WFEditor/tools/ToolConstants",
		"utils/Mousewheel",
		"utils/ShapeUtils",
		"utils/Transformation",
		"utils/Observer",
		"utils/Animatable",
		"WFEditor/utils/WFI18nConstants",
		"WFEditor/workflow/ModelConstants"
	],
	function (
		_,
		AbstractTool,
		ToolConstants,
		Mousewheel,
		ShapeUtils,
		Transformation,
		Observer,
		Animatable,
		WFI18nConstants,
		ModelConstants
	) {
		"use strict";

		/**
		 * Tool that handles all the zoom events on the canvas.
		 *
		 * @exports ZoomTool
		 * @augments AbstractTool
		 * @class
		 *
		 * @param {SceneController} sceneController
		 * @param {WFEditorContext} WFEditorContext
		 */
		var ZoomTool = function (sceneController, WFEditorContext) {
			AbstractTool.call(this, WFEditorContext.get("scene").getLayerContext(), ToolConstants.Tools.ZOOMTOOL, WFEditorContext);
			Observer.mixin(this);
			Animatable.mixin(this);

			var zoomInTooltip, zoomOutTooltip, zoomToFitTooltip;
			zoomInTooltip = WFEditorContext.get("i18n").t(WFI18nConstants.zoomInTooltip);
			zoomOutTooltip = WFEditorContext.get("i18n").t(WFI18nConstants.zoomOutTooltip);
			zoomToFitTooltip = WFEditorContext.get("i18n").t(WFI18nConstants.zoomToFitTooltip);



			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = WFEditorContext.get("scene");

			/**
			 * @member {SceneModel} model used by the scene
			 * @private
			 */
			this._sceneModel = WFEditorContext.get("sceneModel");

			/**
			 * @member {SceneController}
			 * @private
			 */
			this._sceneController = sceneController;

			/**
			 * @member {ToolbarModel} the model for the toolbar, used to add toolbar buttons
			 * @private
			 */
			this._toolbarModel = WFEditorContext.get("toolbarModel");

			/**
			 * @member {Workflow} model of the workflow
			 * @private
			 */
			this._workflowModel = WFEditorContext.get("workflow");


			/**
			 *  @member {Function} a debounced version to register that the user stopped zooming
			 */
			this._stopScrollZoom = _.debounce(_.bind(function () {
				this._sceneModel.stopModifyFrequently("ZOOM");
			}, this), 500);

			/**
			 * @member {I18n}
			 * @private
			 */
			this._i18n = WFEditorContext.get("i18n");

			//register the default state to the tool
			this._registerDefaultState();

			//register listener to mousewheel actions

			var throttledAndBoundHandleScrollWheelFn = _.throttle(_.bind(this._handleScrollWheel, this), 150);

			Mousewheel.addListener(_.bind(function (event, delta) {
				//first check if the scroll action happened on the the scene context and if the zoom tool isn't animating a zoom already
				if (this._scene.getLayerContext().has(event.target).length > 0 && !this.isAnimating()) {
					throttledAndBoundHandleScrollWheelFn(event, delta);
					event.preventDefault();
				}
			}, this));
			//add a toolbar component for the zoom actions.
			this._toolbarModel.addComponent(ToolConstants.ButtonGroups.ZOOM, WFI18nConstants.zoomTools);
			this._toolbarModel.addButton(ToolConstants.ButtonGroups.ZOOM, ToolConstants.Buttons.ZOOMIN, "icon-WCR_WF-ZoomIn", _.bind(this._zoomIn, this), zoomInTooltip);
			this._toolbarModel.addButton(ToolConstants.ButtonGroups.ZOOM, ToolConstants.Buttons.ZOOMOUT, "icon-WCR_WF-ZoomOut", _.bind(this._zoomOut, this), zoomOutTooltip);
			this._toolbarModel.addButton(ToolConstants.ButtonGroups.ZOOM, ToolConstants.Buttons.ZOOMTOFIT, "icon-WCR_WF-ZoomFit", _.bind(this._zoomToFit, this), zoomToFitTooltip);

			if (this._workflowModel.getPrivilege(ModelConstants.Privileges.zoomToActiveNode)) {
				this._initializeStepThroughButtons();

			}



			//add keyboard shortcuts for zooming
			this.bindKeyEvents(["mod+plus", "f5", "mod+="], _.bind(function (e) {
				e.preventDefault();
				e.returnValue = false; // preventDefault() for IE
				this._zoomIn();
			}, this));

			this.bindKeyEvents(["mod+-", "f6"], _.bind(function (e) {
				e.preventDefault();
				e.returnValue = false; // preventDefault() for IE
				this._zoomOut();
			}, this));

			this.bindKeyEvents(["mod+0", "f9"], _.bind(function (e) {
				e.preventDefault();
				e.returnValue = false; // preventDefault() for IE
				this._zoomToFit();
			}, this));

			//do an initial zoom to fit, the workflow is already correctly created
			this._zoomToFit();
			this._observe(this._workflowModel, "tokensLoaded", this._zoomToFit);
			this._observe(this._scene, "workflowLoaded", this._zoomToFit);
		};

		// extend from the AbstractTool
		ZoomTool.prototype = Object.create(AbstractTool.prototype);
		ZoomTool.prototype.constructor = ZoomTool;

		/**
		 * Registers the default state of this tool.
		 */
		ZoomTool.prototype._registerDefaultState = function () {

		};

		/**
		 * Handles the scroll wheel action. It only handles scroll actions on the scene context.
		 * @param {Object}   event
		 * @param {float} delta - at what rate did the scroll wheel change
		 */
		ZoomTool.prototype._handleScrollWheel = function (event, delta) {
			this._sceneModel.startModifyFrequently("ZOOM");
			var position = new ShapeUtils.Point(); //calculate the current position of the mouse cursor
			var contextRect = this._context[0].getBoundingClientRect();
			if (event.pageX && event.pageY) {
				position = new ShapeUtils.Point(event.pageX - window.pageXOffset - contextRect.left, event.pageY - window.pageYOffset - contextRect.top);
			} else if (event.clientX && event.clientY) {
				position = new ShapeUtils.Point(event.clientX - contextRect.left, event.clientY - contextRect.top);
			}
			var zoomLevel = delta < 0 ? -1 : 1;
			if (zoomLevel !== -1 || this._sceneModel.getZoom() > ToolConstants.ZoomLevel[ToolConstants.zoomLimit.MIN]) {
				this._zoom(zoomLevel, position);
			}
			this._stopScrollZoom();
		};

		/**
		 * Zoom to fit, fit the entire workflow on the visible canvas. Zoom out so all parts of the
		 * workflow are visible.
		 */
		ZoomTool.prototype._zoomToFit = function () {
			var boundingRect = this._scene.getBoundingRect();
			this._sceneController.zoomToRect(boundingRect, 1);
		};

		/**
		 * Zoom out with one zoom level, uses the middle of the canvas as zoom point.
		 */
		ZoomTool.prototype._zoomOut = function () {
			if (this._sceneModel.getZoom() > ToolConstants.ZoomLevel[ToolConstants.zoomLimit.MIN]) {
				var contextRect = this._context[0].getBoundingClientRect();
				var position = new ShapeUtils.Point(contextRect.width / 2, contextRect.height / 2);
				this._zoom(-1, position);
			}
		};

		/**
		 * Zoom in with one zoom level, uses the middle of the canvas as zoom point.
		 */
		ZoomTool.prototype._zoomIn = function () {
			if (this._sceneModel.getZoom() < ToolConstants.ZoomLevel[ToolConstants.zoomLimit.MAX]) {
				var contextRect = this._context[0].getBoundingClientRect();
				var position = new ShapeUtils.Point(contextRect.width / 2, contextRect.height / 2);
				this._zoom(1, position);
			}
		};

		/**
		 * Zooms with given zoom level and zoom point.
		 *
		 * @param {Integer} zoomLevel - negative for zooming out, positive for zooming in
		 * @param {ShapeUtils.Point} scenePosition  - scene point used as zoom point
		 */
		ZoomTool.prototype._zoom = function (zoomLevel, scenePosition) {

			//calculate the new zoom level and limit it to the max and min zoom levels
			var newZoom = parseInt(this._sceneModel.getZoomLevel()) + zoomLevel; //zoom levels are between -5 and 2
			newZoom = newZoom > ToolConstants.zoomLimit.MAX ? ToolConstants.zoomLimit.MAX : newZoom; //keep max and min zoom level into account
			newZoom = newZoom < ToolConstants.zoomLimit.MIN ? ToolConstants.zoomLimit.MIN : newZoom;

			var zoomFactor = ToolConstants.ZoomLevel[newZoom]; //get the zoom factor for the zoom level

			// we want to zoom to this point
			var worldPosition = this._scene.convertScenePointToWorld(scenePosition);

			// calculate the new offset
			var newOffset = new ShapeUtils.Size(-Math.round(worldPosition.x - scenePosition.x / zoomFactor), -Math.round(worldPosition.y - scenePosition.y / zoomFactor));


			this._sceneModel.setZoomLevel(newZoom);
			this._sceneModel.setOffset(newOffset);
		};

		/**
		 * Zooms and pans to current active node
		 */
		ZoomTool.prototype._zoomToActive = function () {
			var activeNodes = this._workflowModel.getActiveNodes();
			if (activeNodes.length > 0) {
				var zoomFactor, startZoom, currentZoom;

				zoomFactor = 1.5; // zoom in
				startZoom = parseFloat(this._sceneModel.getZoom());

				this.addAnimatableProperties({
					zoom: function () {
						return zoomFactor; // Should this be constant or dependant on the size of the scene?
					},
					// Used to ease in the node to the center of the scene
					sceneOffsetModifier: function () {
						return 1;
					}
				});
				var isAlreadyAnimating = this.isAnimating();
				var duration = 500;
				this.animate("sceneOffsetModifier", 0, duration, "easeInQuad");
				this.animate("zoom", startZoom, duration, "easeInQuad");

				if (!isAlreadyAnimating) {
					var intervalFn = setInterval(_.bind(function () {
						currentZoom = this.getValue("zoom");
						var offsetModifier = this.getValue("sceneOffsetModifier");
						var worldPosition;
						var node = this._workflowModel.getFocusedActiveNode();
						if (node) {
							var nodeView = this._scene.getViewByWorkflowItem(node);
							// If there is a node view we want the center of the node
							if (nodeView) {
								worldPosition = nodeView.getCenter();
							} else {
								worldPosition = node.getLocation();
							}

							// Get the position of the node in scene coordinates
							var scenePositionOfNode = this._scene.getScenePointFromWorldPoint(worldPosition);

							// We wish to get the node in the middle of the screen at the end of the animation, get the size to the center
							var contextRect = this._context[0].getBoundingClientRect();
							var centerOfScene = new ShapeUtils.Point(contextRect.width / 2, contextRect.height / 2);
							var sizeToCenterOfScene = scenePositionOfNode.sizeTo(centerOfScene);

							// The new offset should let the node be a bit more in the center of the scene
							scenePositionOfNode.offsetXY(sizeToCenterOfScene.width * offsetModifier, sizeToCenterOfScene.height * offsetModifier);

							var offsetX = -Math.round(worldPosition.x - scenePositionOfNode.x / currentZoom);
							var offsetY = -Math.round(worldPosition.y - scenePositionOfNode.y / currentZoom);
							this._sceneModel.setOffset(new ShapeUtils.Size(offsetX, offsetY));
							this._sceneModel.setZoom(currentZoom);
						}
						if (!this.isAnimating()) {
							clearInterval(intervalFn);
						}
					}, this), 17);
				}
			}
		};

		/**
		 * initialize the buttons for stepping through the active nodes
		 */
		ZoomTool.prototype._initializeStepThroughButtons = function () {
			var zoomToNextActive = this._i18n.t(WFI18nConstants.zoomToNextActiveTooltip);
			var zoomToPreviousActive = this._i18n.t(WFI18nConstants.zoomToPreviousActiveTooltip);
			var zoomToFocusedActive = this._i18n.t(WFI18nConstants.zoomToFocusedActive);

			this._toolbarModel.addComponent(ToolConstants.ButtonGroups.STEPTHROUGH, WFI18nConstants.stepThroughNodesTools);

			// Step to previous
			this._toolbarModel.addButton(ToolConstants.ButtonGroups.STEPTHROUGH, ToolConstants.Buttons.ZOOMTOPREVIOUSACTIVE, "icon-WCR_Table-Page-Previous12", _.bind(function () {
				this._workflowModel.shiftFocusedActiveNodeIndex(-1);
			}, this), zoomToPreviousActive, {
				enabled: false
			});

			// Div for the "1 of 5" type of text
			var stepThroughActiveDiv = $("<div/>").text(this._i18n.t(ModelConstants.noActiveNodes));

			// Zoom to focused active node
			this._toolbarModel.addButton(ToolConstants.ButtonGroups.STEPTHROUGH, ToolConstants.ZOOMTOACTIVENODE, "", _.bind(function () {
				this._workflowModel.shiftFocusedActiveNodeIndex(0);
			}, this), zoomToFocusedActive, {
				content: stepThroughActiveDiv,
				enabled: false
			});

			// Fn that updates the "1 of 5" type of text
			var updateStepThroughText = function () {
				if (this._workflowModel.getActiveNodes().length === 0) {
					stepThroughActiveDiv.text(this._i18n.t(WFI18nConstants.noActiveNodes));
					this._toolbarModel.setButtonEnabled(ToolConstants.ZOOMTOACTIVENODE, false);
				} else {
					stepThroughActiveDiv.text((this._workflowModel.getFocusedActiveNodeIndex() + 1) + " " + this._i18n.t(WFI18nConstants.activeNodeOutOf) + " " + this._workflowModel.getActiveNodes().length);
					this._toolbarModel.setButtonEnabled(ToolConstants.ZOOMTOACTIVENODE, true);
				}
			};

			this._observe(this._workflowModel, "tokensChanged", updateStepThroughText);
			this._observe(this._workflowModel, "tokensLoaded", updateStepThroughText);
			this._observe(this._workflowModel, "focusedActiveNodeChanged", updateStepThroughText);
			this._observe(this._workflowModel, "workflowStateChanged", updateStepThroughText);

			// Zoom to next active node
			this._toolbarModel.addButton(ToolConstants.ButtonGroups.STEPTHROUGH, ToolConstants.Buttons.ZOOMTONEXTACTIVE, "icon-WCR_Table-Page-Next12", _.bind(function () {
				this._workflowModel.shiftFocusedActiveNodeIndex(1);
			}, this), zoomToNextActive, {
				enabled: false
			});
			this._observe(this._workflowModel, "tokensChanged", this._updateStepThroughButtons);
			this._observe(this._workflowModel, "focusedActiveNodeChanged", this._zoomToActive);
		};

		/**
		 * Updates the step through buttons
		 */
		ZoomTool.prototype._updateStepThroughButtons = function () {
			var isEnabled = this._workflowModel.getActiveNodes().length > 1;
			this._toolbarModel.setButtonEnabled(ToolConstants.Buttons.ZOOMTOPREVIOUSACTIVE, isEnabled);
			this._toolbarModel.setButtonEnabled(ToolConstants.Buttons.ZOOMTONEXTACTIVE, isEnabled);

		};


		return ZoomTool;

	});
