define([
		"underscore",
		"WFEditor/tools/AbstractInteractiveTool",
		"WFEditor/tools/ToolConstants",
		"WFEditor/workflow/ModelConstants",
		"WFEditor/tools/drawEdge/MouseNode",
		"utils/Observer"
	],
	function (
		_,
		AbstractInteractiveTool,
		ToolConstants,
		ModelConstants,
		MouseNode,
		Observer
	) {
		"use strict";

		/**
		 * Tool that handles hover effects and drawing of edges
		 *
		 * @exports DrawEdgeTool
		 * @augments AbstractInteractiveTool
		 * @class
		 *
		 * @param {SceneController} sceneController - The scene controller
		 * @param {Observable} interactionEvents - the object that is observed to determine activation of this tool
		 * @param {WFEditorContext} WFEditorContext 
		 */
		var DrawEdgeTool = function (sceneController, interactionEvents, WFEditorContext) {
			AbstractInteractiveTool.call(this, ToolConstants.Tools.DRAWEDGETOOL, sceneController, WFEditorContext);
			Observer.mixin(this);

			/**
			 * @member {Array.<Edge> list of edges if we are moving edges
			 * @private
			 */
			this._movingEdges = null;

			/**
			 * @member {Edge}
			 * @private
			 */
			this._hoveringEdge = null;

			/**
			 * @member {Array.<AbstractView>} views of the from pins while drawing edges
			 * @private
			 */
			this._fromPinViews = null;

			/**
			 * @member {ToolbarModel}
			 * @private
			 */
			this._toolbarModel = WFEditorContext.get("toolbarModel");

			/**
			 * @member {Boolean} is there an edge being drawn right now?
			 * @private
			 */
			this._isDrawingEdge = false;

			//register the default state to the tool
			this._registerDefaultState();
			this._registerDrawingEdgeState();
			this._registerHoveringEdgeState();
			//register all the events that this tool listens to
			this.registerActiveEvents(["mousemove", "mousedown", "mouseup", "mouseleave", "mouseenter", "contextmenu"]);

			/**
			 * @member {Boolean} is the alt key pressed or not
			 * @private
			 */
			this._altKeyPressed = false;
			this.bindKeyEvents("alt", _.bind(function (e) {
				this._altKeyPressed = true;
				if (this._hoveringEdge) {
					_.each(this._hoveringEdge.getToPin().getEdges(), _.bind(function (edge) {
						edge.setHovered(true);
					}, this));
					this._hoveringEdge.getToPin().setSplitEdges(false);
				}
				e.preventDefault();
				e.stopPropagation();
			}, this), "keydown");

			this.bindKeyEvents("alt", _.bind(function (e) {
				this._altKeyPressed = false;
				if (this._hoveringEdge) {
					_.each(this._hoveringEdge.getToPin().getEdges(), _.bind(function (edge) {
						edge.setHovered(false);
					}, this));
					this._hoveringEdge.setHovered(true);
					this._hoveringEdge.getToPin().setSplitEdges(true);
				}
				e.preventDefault();
				e.stopPropagation();
			}, this), "keyup");

			this._observe(interactionEvents, "hover", this._checkHoveringEdge);
			this._observe(interactionEvents, "mousedown", this._checkDrawEdge);
		};

		// extend from the AbstractInteractiveTool
		DrawEdgeTool.prototype = Object.create(AbstractInteractiveTool.prototype);
		DrawEdgeTool.prototype.constructor = DrawEdgeTool;


		/**
		 * @override
		 */
		DrawEdgeTool.prototype.activate = function (event, silent) {
			AbstractInteractiveTool.prototype.activate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, true);
		};

		/**
		 * @override
		 */
		DrawEdgeTool.prototype.deActivate = function (event, silent) {
			this._isDrawingEdge = false;
			AbstractInteractiveTool.prototype.deActivate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, false);
		};
		/**
		 * Check if hovering over an edge
		 */
		DrawEdgeTool.prototype._checkHoveringEdge = function (options) {
			var hitArea = options.hitArea;
			if (hitArea) {
				var viewObject = hitArea.getView();
				if (viewObject && viewObject.getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Edge) {
					this._hoveringEdge = viewObject.getModel();
					this._toolManager.activateTool(this);
					this.changeState(ToolConstants.State.HOVERINGEDGE);

				}
			}
		};

		/**
		 * Check if hovering over an edge
		 */
		DrawEdgeTool.prototype._checkDrawEdge = function (options) {
			var hitArea = options.hitArea;
			if (options.e.button === 0 && hitArea && hitArea.getView()) {
				var viewObject = hitArea.getView();
				this._currentWorkflow = viewObject.getModel().getWorkflow();
				if (viewObject.getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Pin && viewObject.getModel().acceptsOutgoingEdges() && this._currentWorkflow.getPrivilege(ModelConstants.Privileges.addEdges)) {
					// we clicked on an output pin where an edge can be added from
					this._fromPinViews = [viewObject];
					this._startPoint = viewObject.getLocation();
					this._toolManager.activateTool(this);
					this.changeState(ToolConstants.State.DRAWINGEDGE);
				}
			}
		};

		/**
		 * Edge drawing has finished, set it to the right pin or remove edge
		 * @param {ShapeUtils.Point} position
		 */
		DrawEdgeTool.prototype._drawEdgeMouseUpFunction = function (position) {
			this._isDrawingEdge = false;
			var toPinView = this._getSnapPinView(this._fromPinViews, position);

			if (toPinView) {
				var toPin = toPinView.getModel();
				// over an input pin, create all permanent edges
				if (this._currentWorkflow.getPrivilege(ModelConstants.Privileges.moveEdges) || this._currentWorkflow.getPrivilege(ModelConstants.Privileges.addEdges)) {
					_.each(this._fromPinViews, _.bind(function (fromPinView) {
						var fromPin = fromPinView.getModel();
						if (toPin.acceptsIncomingEdges(fromPin)) {
							this._sceneController.createEdge(this._currentWorkflow, fromPin, toPin, false);
						}
					}, this));
				}

				// Now push it on the undo/redo stack
				if (this._movingEdges && this._currentWorkflow.getPrivilege(ModelConstants.Privileges.moveEdges)) {
					// moving edges
					this._sceneController.moveEdges(this._currentWorkflow, this._movingEdges, toPin, true, true);
				} else if (!this._movingEdges && this._currentWorkflow.getPrivilege(ModelConstants.Privileges.addEdges)) {
					// adding edges, this is always one at a time
					this._sceneController.createEdge(this._currentWorkflow, this._fromPinViews[0].getModel(), toPin, true, true);
				}
			} else {
				if (this._movingEdges) {
					// not over an input pin, remove all if we were moving
					if (this._currentWorkflow.getPrivilege(ModelConstants.Privileges.removeEdges)) {
						this._sceneController.removeEdges(this._currentWorkflow, this._movingEdges, true, true);
					}
				}
				// We tried to create a new Edge, but we didn't end on a Pin or Node, cancel the operation, go back to the default state
			}
		};

		/**
		 * An Edge is being drawn
		 * @param {ShapeUitls.Point} position
		 */
		DrawEdgeTool.prototype._drawEdgeMouseMoveFunction = function (position) {
			this._isDrawingEdge = true;
			// Update location of mouse node
			var mouseNode = this._currentWorkflow.getNode(MouseNode.ID);

			// Snap to pins
			var view = this._getSnapPinView(this._fromPinViews, position);
			if (view) {
				position = view.getLocation();
				mouseNode.setLocation(position);
			} else {
				var worldPoint = this._scene.getWorldPointFromScenePoint(position);
				mouseNode.setLocation(worldPoint);
			}
		};

		/**
		 * Get a "snapped" pin view that accepts edges from at least one fromPinView close to the position.
		 * @param {Array.<PinView>} fromPinViews
		 * @param {ShapeUtils.Point} position
		 * @returns {AbstractView} null if not found
		 */
		DrawEdgeTool.prototype._getSnapPinView = function (fromPinViews, position) {
			var views = this._scene.getObjectsNearPosition(position, 10, _.bind(function (view) {
				// if you are near the node, also snap to the first input pin
				if (view.getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Node) {
					var inputPins = view.getModel().getPins();
					var firstInputPin = _.find(inputPins, function (pin) {
						return pin.acceptsIncomingEdges();
					});
					if (firstInputPin) {
						var newView = this._scene.getViewByWorkflowItem(firstInputPin);
						if (newView) {
							view = newView;
						}
					}
				} else if (view.getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Edge) {
					// over an edge -> take the input pin where it is going to
					view = this._scene.getViewByWorkflowItem(view.getModel().getToPin());
				}

				// Check the pin
				if (view.getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Pin // pin
					&& _.some(fromPinViews, function (fromPinView) {
						//accepts edges from at least one of the pins (the others will get removed) and the edge is not existing yet
						return view.getModel().acceptsIncomingEdges(fromPinView.getModel()) && !fromPinView.getModel().getEdgeTo(view.getModel());
					})) {
					return view;
				}
				return null;
			}, this));

			if (_.size(views) > 0) {
				// Now we just return the first hit, perhaps we should pick the closest view if there are multiple
				return views[0];
			}
			return null;
		};

		/**
		 * Register the hovering state while hovering over one of the edges
		 */
		DrawEdgeTool.prototype._registerHoveringEdgeState = function () {
			this.registerState(ToolConstants.State.HOVERINGEDGE, {
				"enterState": _.bind(function () {
					_.each(this._hoveringEdge.getToPin().getEdges(), _.bind(function (edge) {
						edge.setHovered(this._altKeyPressed);
					}, this));
					this._hoveringEdge.setHovered(true);
					this._hoveringEdge.getToPin().setSplitEdges(!this._altKeyPressed);
				}, this),
				"leaveState": _.bind(function () {
					_.each(this._hoveringEdge.getToPin().getEdges(), _.bind(function (edge) {
						edge.setHovered(false);
					}, this));
					this._hoveringEdge.getToPin().setSplitEdges(false);
					this._hoveringEdge = null;
				}, this),
				"mousemove": _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					var position = this._getMousePosition(e);
					// get a edge view at this position
					var viewObject = this._scene.getObjectsAtPosition(position, function (view) {
						if (view.getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Edge) {
							return view;
						}
					})[0];

					if (viewObject) {
						if (this._hoveringEdge !== viewObject.getModel()) {
							// we hovered over an edge of another node -> switch back
							if (this._hoveringEdge.getToPin().getNode() !== viewObject.getModel().getToPin().getNode()) {
								this._toolManager.deactivateCurrentTool();
								return;
							} else {
								// We hovered over another edge of the same node > check alt key and set right one hovered if needed
								_.each(this._hoveringEdge.getToPin().getEdges(), _.bind(function (edge) {
									edge.setHovered(this._altKeyPressed);
								}, this));
								this._hoveringEdge = viewObject.getModel();
								this._hoveringEdge.setHovered(true);
							}
						}
						this._hoveringEdge.getToPin().setSplitEdges(!this._altKeyPressed);
					} else {
						this._toolManager.deactivateCurrentTool();
					}
				}, this),
				"mousedown": _.bind(function (e) {
						e.preventDefault();
						e.stopPropagation();
						var position = this._getMousePosition(e);
						var viewObject = this._scene.getObjectsAtPosition(position, function (view) {
							if (view.getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Edge) {
								return view;
							}
						})[0];

						if (viewObject) {
							this._currentWorkflow = viewObject.getModel().getWorkflow();
							if (viewObject.getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Edge &&
								(this._currentWorkflow.getPrivilege(ModelConstants.Privileges.moveEdges) || this._currentWorkflow.getPrivilege(ModelConstants.Privileges.removeEdges))) {
								// we hovered over an edge at an input pin
								var toPin = viewObject.getModel().getToPin();
								var toPinView = viewObject.getToPinView();
								if (this._altKeyPressed) {
									// pick them all
									this._movingEdges = toPin.getEdges();
								} else {
									// pick the selected one
									this._movingEdges = [viewObject.getModel()];
								}
								this._fromPinViews = _.map(this._movingEdges, _.bind(function (edge) {
									return this._scene.getViewByWorkflowItem(edge.getFromPin());
								}, this));

								this._movingExistingEdges = true;
								this._startPoint = toPinView.getEdgeLocation(viewObject); // start on the selected edge
								this._sceneController.removeEdges(this._currentWorkflow, this._movingEdges, false); // remove the permanent ones
								this.changeState(ToolConstants.State.DRAWINGEDGE);
								return;
							}
						}
						this._toolManager.deactivateCurrentTool();
					},
					this),
				"mouseup": function (e) {
					e.preventDefault();
					e.stopPropagation();
				},
				"mouseenter": function (e) {
					e.preventDefault();
					e.stopPropagation();
				},
				"mouseleave": function (e) {
					e.preventDefault();
					e.stopPropagation();
				},
				"contextmenu": function (e) {
					e.preventDefault();
					e.stopPropagation();
				}
			});
		};

		/**
		 * Register the drawing an edge state
		 */
		DrawEdgeTool.prototype._registerDrawingEdgeState = function () {
			this.registerState(ToolConstants.State.DRAWINGEDGE, {
				"enterState": _.bind(function (previousState) {
					this._toolManager.updateCursor();
					// Create tmp mouse node
					var mouseNode = new MouseNode(this._currentWorkflow);
					mouseNode.setLocation(this._startPoint);
					this._currentWorkflow.addNode(mouseNode);
					// The mouse node is interactive
					this._scene.setInteractive(mouseNode.getId(), true);

					_.each(this._fromPinViews, function (fromPinView) {
						// Create tmp edge
						var fromPin = fromPinView.getModel();
						fromPin.setHovered(true);
						var toPin = mouseNode.getMousePin();
						this._sceneController.createEdge(this._currentWorkflow, fromPin, toPin, false);
						var createdEdgeId = fromPin.getEdgeTo(toPin).getId();
						this._scene.setInteractive(fromPin.getId(), true);
						this._scene.setInteractive(createdEdgeId, true);
					}, this);
				}, this),
				"leaveState": _.bind(function (newStateName) {
					var mouseNode = this._currentWorkflow.getNode(MouseNode.ID);

					var edgesToRemove = [];
					_.each(this._fromPinViews, function (fromPinView) {
						// remove tmp edges
						var fromPin = fromPinView.getModel();
						fromPin.setHovered(false);
						this._scene.setInteractive(fromPin.getId(), false);
						var toPin = mouseNode.getMousePin();
						edgesToRemove.push(fromPin.getEdgeTo(toPin));
					}, this);
					this._sceneController.removeEdges(this._currentWorkflow, edgesToRemove, false);

					// remove tmp node
					this._currentWorkflow.removeNode(mouseNode);

					// reset vars
					this._fromPinViews = null;
					this._movingEdges = null;
					this._currentWorkflow = null;
				}, this),
				"mousemove": _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					var position = this._getMousePosition(e);
					this._stopIntervalPanning();
					// Panning close to borders
					if (this._shouldPan(position)) {
						this._calculatePanning(position);
						this._startIntervalPanning(position, this._drawEdgeMouseMoveFunction);
					}
					this._drawEdgeMouseMoveFunction(position);
				}, this),
				"mousedown": function (e) {
					e.preventDefault();
					e.stopPropagation();
				},
				"mouseenter": _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					this._stopPanningWhileMouseOutsideCanvas();
				}, this),
				"mouseup": _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					var position = this._getMousePosition(e);
					this._drawEdgeMouseUpFunction(position);
					this._toolManager.deactivateCurrentTool();
				}, this),
				"mouseleave": _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					this._continuePanningWhileMouseOutsideCanvas(this._drawEdgeMouseMoveFunction, _.bind(function (mousePosition) {
						this._drawEdgeMouseUpFunction(mousePosition);
						this.changeState(ToolConstants.State.DEFAULTSTATE);
						if (this.isActive()) {
							this._toolManager.deactivateCurrentTool();
						}
					}, this));
				}, this),
				"contextmenu": function (e) {
					e.preventDefault();
					e.stopPropagation();
				}
			});
		};

		/**
		 * @override
		 * @returns {Boolean}
		 */
		DrawEdgeTool.prototype.allowDeactivate = function () {
			return !this._isDrawingEdge;
		};

		/**
		 * @override
		 */
		DrawEdgeTool.prototype.getDefaultCursor = function () {
			if (this._currentState === ToolConstants.State.DRAWINGEDGE) {
				return "grabbing-cursor";
			} else {
				return "pointer-cursor";
			}
		};

		return DrawEdgeTool;

	});