define(["WFEditor/workflow/helper/Helper",
		"underscore"
	],
	function (Helper,
		underscore) {

		"use strict";

		/**
		 * A Grid Helper is used to indicate a grid in the workflow
		 * @exports GridHelper
		 * @class
		 * @augments Helper
		 * @param {Workflow} workflow for which the helper is used
		 * @parma {String} unique id for the GridHelper
		 */
		var GridHelper = function (workflow, id) {
			Helper.call(this, workflow, id);

			/**
			 * @member {Boolean} is show grid
			 * @private
			 */
			this._showGrid = false;

			/**
			 * @member {Array.<Object>} Object describing vertical lines. 
			 *                             x is the world x point where this vertical line has to come.
			 *                             offset is the offset from mouse position when started moving (x) to desired location (x) (e.g. center of moving node).
			 * @example [{ x: 50
			 * 			  offset: 10}]
			 * @private
			 */
			this._verticalGridLines = [];
			/**
			 * @member {Array.<Object>} Object describing horizontal lines. 
			 *                             y is the world y point where this horizontal line has to come.
			 *                             offset is the offset from mouse position when started moving (y) to desired location (y) (e.g. location of input/output pin of connected edge).
			 * @example [{ y: 50
			 * 			  offset: 10}]
			 * @private
			 */
			this._horizontalGridLines = [];
		};

		GridHelper.prototype = Object.create(Helper.prototype);
		GridHelper.prototype.constructor = GridHelper;

		/**
		 * Should the grid be shown?
		 * @returns {Boolean}
		 */
		GridHelper.prototype.isShowGrid = function () {
			return this._showGrid;
		};

		/**
		 * Show/Hide the grid
		 * @param {Boolean} showGrid
		 */
		GridHelper.prototype.setShowGrid = function (showGrid) {
			if (this._showGrid !== showGrid) {
				this._showGrid = showGrid;
				this._emit("changed.showGrid", showGrid);
			}
		};

		/**
		 * Get the vertical grid lines
		 * @returns {Array.<Object>}
		 */
		GridHelper.prototype.getVerticalGridLines = function () {
			return this._verticalGridLines;
		};


		/**
		 * Set the vertical grid lines
		 * @param {Array.<Object>} verticalGridLines - New vertical grid lines
		 */
		GridHelper.prototype.setVerticalGridLines = function (verticalGridLines) {
			if (this._verticalGridLines !== verticalGridLines) {
				this._verticalGridLines = verticalGridLines;
				this._emit("changed.verticalGridLines", verticalGridLines);
			}
		};

		/**
		 * Set the horizontal grid lines
		 * @param {Array.<Object>} horizontalGridLines - New horizontal grid lines
		 */
		GridHelper.prototype.setHorizontalGridLines = function (horizontalGridLines) {
			if (this._horizontalGridLines !== horizontalGridLines) {
				this._horizontalGridLines = horizontalGridLines;
				this._emit("changed.horizontalGridLines", horizontalGridLines);
			}
		};

		/**
		 * Get the horizontal grid lines
		 * @returns {Array.<Object>}
		 */
		GridHelper.prototype.getHorizontalGridLines = function () {
			return this._horizontalGridLines;
		};

		return GridHelper;
	});