define(["WFEditor/scene/AbstractView",
		"utils/ShapeUtils",
		"underscore"
	],
	function (AbstractView,
		ShapeUtils,
		_) {
		"use strict";

		/**
		 * A view for a GridHelper
		 * @exports GridView
		 * @class
		 * @augments {AbstractView}
		 *
		 * @param {GridHelper} model
		 */
		var GridView = function (model) {
			AbstractView.call(this, model);

			this._observe(model, "changed.*", function () {
				this._updateBoundingRect();
				this.makeDirty();
			});
		};

		GridView.prototype = Object.create(AbstractView.prototype);
		GridView.prototype.constructor = GridView;

		/**
		 * Renders the view on canvas context.
		 * @Override
		 * @param {CanvasRenderingContext2D} context
		 * @param {Layer} layer
		 * @param {Boolean} quickRender - if true, only high priority render actions should be executed (i.e. during panning)
		 */
		GridView.prototype.render = function (ctx, layer, quickRender) {
			if (this._model.isShowGrid()) {
				ctx.save();
				ctx.strokeStyle = "rgba(0,0,0,0.1)";
				ctx.lineWidth = 1;

				// To render a crisp line, the scene coordinates have to be rounded at 0.5
				var t = layer.getTransformation();
				var i = t.createInverse();
				var visibleRect = layer.getVisibleRect();

				// Vertical lines
				ctx.beginPath();
				_.each(this._model.getVerticalGridLines(), function (verticalGridLine) {
					var p = new ShapeUtils.Point(verticalGridLine.x, 0);

					var tp = t.transformedPoint(p);
					tp.x = Math.floor(tp.x + 1) - 0.5;
					p = i.transformedPoint(tp);

					var p1 = new ShapeUtils.Point(p.x, visibleRect.origin.y);
					var p2 = new ShapeUtils.Point(p.x, visibleRect.origin.y + visibleRect.size.height);

					ctx.moveTo(p1.x, p1.y);
					ctx.lineTo(p2.x, p2.y);
				});
				ctx.stroke();

				// Horizontal lines
				ctx.beginPath();
				_.each(this._model.getHorizontalGridLines(), function (horizontalGridLine) {
					var p = new ShapeUtils.Point(0, horizontalGridLine.y);
					var tp = t.transformedPoint(p);
					tp.y = Math.floor(tp.y + 1) - 0.5;
					p = i.transformedPoint(tp);

					var p1 = new ShapeUtils.Point(visibleRect.origin.x, p.y);
					var p2 = new ShapeUtils.Point(visibleRect.origin.x + visibleRect.size.width, p.y);

					ctx.moveTo(p1.x, p1.y);
					ctx.lineTo(p2.x, p2.y);
				});
				ctx.stroke();


				ctx.restore();
			}
		};

		/**
		 * @Override
		 */
		GridView.prototype.getZIndex = function () {
			return 110;
		};

		/**
		 * @Override
		 */
		GridView.prototype._updateBoundingRect = function () {
			var vLines = this._model.getVerticalGridLines();
			var hLines = this._model.getHorizontalGridLines();

			if (!this._model.isShowGrid() || (vLines.length === 0 && hLines.length === 0)) {
				this._setBoundingRect(null);
				return;
			}

			var minX = 0;
			var maxX = Number.MAX_VALUE;

			var minY = 0;
			var maxY = Number.MAX_VALUE;


			if (vLines.length > 0) {
				minX = _.min(vLines, function (vLine) {
					return vLine.x;
				}).x;
				maxX = _.max(vLines, function (vLine) {
					return vLine.x;
				}).x;
			}

			if (hLines.length > 0) {
				minY = _.min(hLines, function (hLine) {
					return hLine.y;
				}).y;
				maxY = _.max(hLines, function (hLine) {
					return hLine.y;
				}).y;
			}

			var bbox = ShapeUtils.rectMake(Math.min(minX, maxX), Math.min(minY, maxY), Math.abs(maxX - minX), Math.abs(maxY - minY)).getNormalizedRectArray();
			this._setBoundingRect(bbox);
		};

		return GridView;
	});