define([
		"underscore",
		"WFEditor/tools/AbstractInteractiveTool",
		"WFEditor/tools/ToolConstants",
		"WFEditor/tools/moveNodes/GridHelper",
		"WFEditor/workflow/ModelConstants",
		"utils/Observer",
		"utils/ShapeUtils"
	],
	function (
		_,
		AbstractInteractiveTool,
		ToolConstants,
		GridHelper,
		ModelConstants,
		Observer,
		ShapeUtils
	) {
		"use strict";

		/**
		 * Tool that handles moving the Nodes
		 *
		 * @exports MoveNodesTool
		 * @augments AbstractInteractiveTool
		 * @class
		 *
		 * @param {SceneController} sceneController - The scene controller
		 * @param {SceneController} sceneController - The scene controller
		 * @param {WFEditorContext} WFEditorContext
		 */
		var MoveNodesTool = function (sceneController, interactionEvents, WFEditorContext) {
			AbstractInteractiveTool.call(this, ToolConstants.Tools.MOVENODESTOOL, sceneController, WFEditorContext);
			Observer.mixin(this);

			/**
			 * @member {Boolean} keep track of whether ctrl is pressed or not
			 * @private
			 */
			this._ctrlKeyPressed = false;

			/**
			 * @member {Boolean} keep track of whether shift is pressed or not
			 * @private
			 */
			this._shiftKeyPressed = false;

			/**
			 * @member {ToolbarModel}
			 * @private
			 */
			this._toolbarModel = WFEditorContext.get("toolbarModel");

			/**
			 * @member {Boolean}
			 * @private
			 */
			this._isMovingNodes = false;

			/**
			 * @member {ShapeUtils.Point} start point for moving nodes
			 * @private
			 */
			this._previousWorldPoint = null;

			/**
			 * @member {Map.<String, Node>} - keep track of members that were set interactive, to be able to remove them 
			 * from the interactive layer again in case selection changes
			 * @private
			 */
			this._selectedNodes = {};

			/**
			 * @member {Node} - the node being moved
			 * @private
			 */
			this._movingNodeView = null;

			/**
			 * @member {Number} offset from the mousepoint to the top left point of the moving node
			 * @private
			 */
			this._offsetX = 0;
			this._offsetY = 0;

			//register the default state to the tool
			this._registerMovingNodesState();
			//register all the events that this tool listens to
			this.registerActiveEvents(["mousemove", "mousedown", "mouseup", "mouseleave", "mouseenter", "contextmenu"]);

			this._observe(interactionEvents, "mousedown", _.bind(this._handleMouseInteractionEvent, this));
			this._observe(interactionEvents, "mouseup", _.bind(this._handleMouseInteractionEvent, this));
			this._observe(interactionEvents, "mousemove", _.bind(this._handleMouseInteractionEvent, this));
			this._observe(interactionEvents, "keyevent.keydown.shift", _.bind(this._shiftDown, this));
			this._observe(interactionEvents, "keyevent.keyup.shift", _.bind(this._shiftUp, this));
			this._observe(interactionEvents, "activating", _.bind(function () {
				this.changeState(ToolConstants.State.DEFAULTSTATE);
			}, this));

			this.changeState(ToolConstants.State.DEFAULTSTATE);
		};

		// extend from the AbstractTool
		MoveNodesTool.prototype = Object.create(AbstractInteractiveTool.prototype);
		MoveNodesTool.prototype.constructor = MoveNodesTool;

		/**
		 * Set shift key to released
		 */
		MoveNodesTool.prototype._shiftUp = function () {
			this._shiftKeyPressed = false;
			this._checkGrid();
		};

		/**
		 * Set shift key to pressed
		 */
		MoveNodesTool.prototype._shiftDown = function () {
			this._shiftKeyPressed = true;
			this._checkGrid();
		};

		/**
		 * Handles the mouse interaction events by triggering the current state.
		 * @param {Object} options
		 */
		MoveNodesTool.prototype._handleMouseInteractionEvent = function (options) {
			this._ctrlKeyPressed = options.e.ctrlKey;
			this.triggerEvent(options.eventName, options);
		};

		/**
		 * @override
		 */
		MoveNodesTool.prototype.activate = function (event, silent) {
			AbstractInteractiveTool.prototype.activate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, true);
		};

		/**
		 * @override
		 */
		MoveNodesTool.prototype.deActivate = function (event, silent) {
			this._isMovingNodes = false;
			AbstractInteractiveTool.prototype.deActivate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, false);
		};

		/** 
		 * State while moving nodes
		 */
		MoveNodesTool.prototype._registerMovingNodesState = function () {

			// The default state will check if there is a node on mouse down
			this.registerState(ToolConstants.State.DEFAULTSTATE, {
				"enterState": _.bind(function () {
					if (this.isActive()) {
						this._toolManager.deactivateCurrentTool();
					}
					this._resetNodeMoveChecks();
				}, this),
				"mousedown": _.bind(this._checkNodeUnderMouseDown, this),
				"mouseup": function () {},
				"mousemove": function () {},
				"contextmenu": function () {},
				"click": function () {},
				"dblclick": function () {},
				"mouseenter": function () {},
				"mouseleave": function () {}
			});

			// This state will check if we should activate this tool to move nodes
			this.registerState(ToolConstants.State.SHOULDMOVE, {
				"enterState": _.bind(function (previousState, options) {
					this._startPoint = options.position;
					this._movingNodeView = options.hitArea.getView();
				}, this),
				"mousemove": _.bind(this._checkMoveNodes, this),
				"mouseup": _.bind(function () {
					this.changeState(ToolConstants.State.DEFAULTSTATE);
				}, this),
				"contextmenu": function () {},
				"click": function () {},
				"dblclick": function () {},
				"mouseenter": function () {},
				"mouseleave": function () {},
				"mousedown": function () {}

			});

			// In this state the tool is actually active and moves nodes
			this.registerState(ToolConstants.State.MOVINGNODES, {
				"enterState": _.bind(function () {
					var workflow;
					if (!this._gridHelper) {
						workflow = this._scene.getWorkflowModel();
						this._gridHelper = new GridHelper(workflow, "MoveGrid");
					}

					this._checkGrid();
					this._calculateMoveGrid();

					if (workflow) {
						// if workflow is defined, we didn't add the grid helper yet to the workflow
						workflow.addHelper(this._gridHelper);
					}

					this._isMovingNodes = true;
					if (!this.isActive()) {
						this._toolManager.activateTool(this);
					}
					this._previousWorldPoint = this._startPointWorld;
					this._selectedNodes = this._currentWorkflow.getSelectedNodes();
					_.each(this._selectedNodes, function (selectedNode) {
						this._scene.setInteractive(selectedNode.getId(), true);
						_.each(selectedNode.getPins(), function (pin) {
							this._scene.setInteractive(pin.getId(), true);
							_.each(pin.getEdges(), function (edge) {
								this._scene.setInteractive(edge.getId(), true);
							}, this);
						}, this);
					}, this);
				}, this),
				"leaveState": _.bind(function () {
					this._isMovingNodes = false;
					this._stopIntervalPanning();
					this._previousWorldPoint = null;
					this._startPointWorld = null;
					_.each(this._selectedNodes, function (selectedNode) {
						this._scene.setInteractive(selectedNode.getId(), false);
						_.each(selectedNode.getPins(), function (pin) {
							this._scene.setInteractive(pin.getId(), false);
							_.each(pin.getEdges(), function (edge) {
								this._scene.setInteractive(edge.getId(), false);
							}, this);
						}, this);
					}, this);
					this._gridHelper.setShowGrid(false); // always disable
					this._selectedNodes = {};
				}, this),
				"mousemove": _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					var position = this._getSnappedMousePosition(e);
					// Panning close to borders
					this._stopIntervalPanning();
					if (this._shouldPan(position)) {
						this._calculatePanning(position);
						this._startIntervalPanning(position, this._calculateNodesMove);
					}
					this._calculateNodesMove(position);
				}, this),
				"mousedown": function (e) {
					e.preventDefault();
					e.stopPropagation();
				},
				"mouseup": _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					var mousePosition = this._getSnappedMousePosition(e);
					var newWorldPoint = this._scene.getWorldPointFromScenePoint(mousePosition);
					var offsetX = newWorldPoint.x - this._startPointWorld.x;
					var offsetY = newWorldPoint.y - this._startPointWorld.y;
					if (offsetX !== 0 || offsetY !== 0) {
						this._setNodesMove(mousePosition);
					}
					this.changeState(ToolConstants.State.DEFAULTSTATE);
				}, this),
				"mouseleave": _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					this._continuePanningWhileMouseOutsideCanvas(this._calculateNodesMove, _.bind(function (mousePosition) {
						this._setNodesMove(mousePosition);
						this.changeState(ToolConstants.State.DEFAULTSTATE);
					}, this));
				}, this),
				"mouseenter": _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					this._stopPanningWhileMouseOutsideCanvas();
				}, this),
				"contextmenu": function (e) {
					e.preventDefault();
					e.stopPropagation();
				}
			});
		};

		/**
		 * Get the snapped mouse position if needed
		 * @param {EventObject} e
		 */
		MoveNodesTool.prototype._getSnappedMousePosition = function (e) {
			var mousePosition = this._getMousePosition(e);
			if (this._gridHelper && this._shouldShowGrid()) {
				var snapMarge = 20;

				var worldMouse = this._scene.getWorldPointFromScenePoint(mousePosition);

				// Search closest Snap X
				var snapV = null;
				var minDx = Number.MAX_VALUE;
				var verticalLines = this._gridHelper.getVerticalGridLines();
				_.each(verticalLines, function (verticalLine) {
					var dx = Math.abs(verticalLine.x - (worldMouse.x + verticalLine.offset));
					if (dx < minDx) {
						snapV = verticalLine;
						minDx = dx;
					}
				});

				// Search closest Snap Y
				var snapH = null;
				var minDy = Number.MAX_VALUE;
				var horizontalLines = this._gridHelper.getHorizontalGridLines();
				_.each(horizontalLines, function (horizontalLine) {
					var dy = Math.abs(horizontalLine.y - (worldMouse.y + horizontalLine.offset));
					if (dy < minDy) {
						snapH = horizontalLine;
						minDy = dy;
					}
				});

				var snapPoint = new ShapeUtils.Point(worldMouse.x, worldMouse.y);
				if (minDx < snapMarge && snapV !== null) {
					snapPoint.x = snapV.x - snapV.offset;
				}
				if (minDy < snapMarge && snapH !== null) {
					snapPoint.y = snapH.y - snapH.offset;
				}

				return this._scene.getScenePointFromWorldPoint(snapPoint);
			} else {
				return mousePosition;
			}
		};

		/**
		 * Function to check if the move grid should be shown
		 * @returns {Boolean} True if show, false otherwise
		 */
		MoveNodesTool.prototype._shouldShowGrid = function () {
			return this._shiftKeyPressed && this._currentState === ToolConstants.State.MOVINGNODES;
		};

		/**
		 * Check if grid should be activated
		 */
		MoveNodesTool.prototype._checkGrid = function () {
			if (this._gridHelper) {
				// Moving and shift key pressed
				this._gridHelper.setShowGrid(this._shouldShowGrid());
			}
		};

		/**
		 * check if the mousedown object received from the mouse interaction tool is on a node's NodeSelect hit area
		 */
		MoveNodesTool.prototype._checkNodeUnderMouseDown = function (options) {
			var hitArea = options.hitArea;
			if (options.e.button === 0 && hitArea && hitArea.hasClass(ModelConstants.HitAreaClass.NodeSelect) &&
				hitArea.getView() && hitArea.getView().getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Node &&
				hitArea.getView().getModel().getWorkflow().getPrivilege(ModelConstants.Privileges.moveNodes)) {

				// There is a node hitarea with the NodeSelectclass under the mouse and the user has the privilege to moven nodes

				this.changeState(ToolConstants.State.SHOULDMOVE, options);
			}
		};

		/**
		 * resets the checks needed to activate the move node tool
		 */
		MoveNodesTool.prototype._resetNodeMoveChecks = function () {
			this._startPoint = null;
		};

		/**
		 * Check whether we should move nodes and enable tool if we must.
		 * @param {Object} options - 	{
		 * 								e: mouseEvent
		 * 								hitArea: HitArea object under mouse pointer
		 * 								position: mouse position
		 * 								}
		 */
		MoveNodesTool.prototype._checkMoveNodes = function (options) {
			var hitArea = options.hitArea;
			if (!hitArea || !hitArea.hasClass(ModelConstants.HitAreaClass.NodeSelect) || this._startPoint === null) {
				// not a hit area for node selecting or no start point yet
				return;
			}
			var viewObject = hitArea.getView();

			if (viewObject && viewObject.getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Node) {
				this._currentWorkflow = viewObject.getModel().getWorkflow();
				this._startPointWorld = this._scene.getWorldPointFromScenePoint(this._startPoint);
				this._offsetX = this._startPointWorld.x - this._movingNodeView.getModel().getLocation().x;
				this._offsetY = this._startPointWorld.y - this._movingNodeView.getModel().getLocation().y;
				var selectedNodes = this._currentWorkflow.getSelectedNodes();

				var currentMousePoint = this._getMousePosition(options.e);
				var mouseOffsetX = this._startPoint.x - currentMousePoint.x;
				var mouseOffsetY = this._startPoint.y - currentMousePoint.y;

				if (_.contains(selectedNodes, viewObject.getModel()) && (mouseOffsetX !== 0 || mouseOffsetY !== 0)) {
					// node was selected -> start moving
					if (!this._ctrlKeyPressed) {
						this.changeState(ToolConstants.State.MOVINGNODES);

					} else {
						this.changeState(ToolConstants.State.DEFAULTSTATE);
					}
				}
			}
		};



		/**
		 * Calculates the new position of nodes whilst moving
		 * @param {ShapeUtils.Point} position
		 */
		MoveNodesTool.prototype._calculateNodesMove = function (position) {
			if (this._previousWorldPoint === null) {
				this.changeState(ToolConstants.State.DEFAULTSTATE);
				return;
			}
			var newWorldPoint = this._scene.getWorldPointFromScenePoint(position);
			var offsetX = newWorldPoint.x - this._previousWorldPoint.x;
			var offsetY = newWorldPoint.y - this._previousWorldPoint.y;

			// Check if the scene has bounds and don't let the nodes move outside of the bounds
			var worldBoundingBox = this._sceneModel.getWorldBoundingBoxWithInsets();
			if (worldBoundingBox) {
				_.each(this._currentWorkflow.getSelectedNodes(), function (node) {
					var loc = node.getLocation();
					if (loc.x + offsetX  > worldBoundingBox.size.width + worldBoundingBox.origin.x) {
						offsetX = worldBoundingBox.size.width - loc.x + worldBoundingBox.origin.x;
					} else if (loc.x + offsetX < worldBoundingBox.origin.x) {
						offsetX = worldBoundingBox.origin.x - loc.x;
					}
					if (loc.y + offsetY > worldBoundingBox.size.height + worldBoundingBox.origin.y) {
						offsetY = worldBoundingBox.size.height - loc.y + worldBoundingBox.origin.y;

					} else if (loc.y + offsetY < worldBoundingBox.origin.y) {
						offsetY = worldBoundingBox.origin.y - loc.y;
					}

				});
			}


			this._sceneController.moveNodes(this._currentWorkflow.getSelectedNodes(), offsetX, offsetY, false);


			// don't set 'newWorldPoint' as previousWorldPoint as there might be rounding errors when setting the new location
			this._previousWorldPoint = this._movingNodeView.getModel().getLocation().copy();
			this._previousWorldPoint.offsetXY(this._offsetX, this._offsetY);
		};

		/**
		 * Sets the new position of nodes
		 * @param {ShapeUtils.Point} mousePosition
		 */
		MoveNodesTool.prototype._setNodesMove = function (mousePosition) {
			var newWorldPoint = this._scene.getWorldPointFromScenePoint(mousePosition);
			var offsetX = newWorldPoint.x - this._startPointWorld.x;
			var offsetY = newWorldPoint.y - this._startPointWorld.y;
			if (offsetX !== 0 || offsetY !== 0) {
				this._sceneController.moveNodes(this._currentWorkflow.getSelectedNodes(), offsetX, offsetY, true, true);
			}
		};

		MoveNodesTool.prototype.allowDeactivate = function () {
			return !this._isMovingNodes;
		};

		/**
		 * Calculate the grid to move on and set them in the helper
		 */
		MoveNodesTool.prototype._calculateMoveGrid = function () {
			var visibleNodes = this._scene.getNodesInRect(this._scene.getVisibleRect());
			var selectedNodes = this._currentWorkflow.getSelectedNodes();

			// Vertical grid (center of nodes)
			var verticalGridLines = [];
			var horizontalGridLines = [];

			if (this._movingNodeView) {
				var offsetX = this._movingNodeView.getCenter().x - this._startPointWorld.x;
				_.each(visibleNodes, function (node) {
					if (selectedNodes[node.getId()]) {
						// node is in selected nodes -> skip
						return;
					}
					var nodeView = this._scene.getViewByWorkflowItem(node);
					if (nodeView) {
						var nodeNormRect = nodeView.getBoundingRect();
						var nodeRect = ShapeUtils.rectFromNormalizedRectArray(nodeNormRect);
						var x = nodeRect.center().x;
						verticalGridLines.push({
							x: x,
							offset: offsetX
						});
					}
				}, this);

				var closestFromPinLocation = null;
				var closestToPinLocation = null;

				_.each(this._movingNodeView.getModel().getPins(), function (pin) {
					var pinLocation = this._movingNodeView.getLocationForPin(pin);
					var offsetY = pinLocation.y - this._startPointWorld.y;
					if (pin.acceptsIncomingEdges()) {
						// Horizontal grid (fromPin center of incoming edges)
						_.each(pin.getEdges(), function (edge) {
							var fromPin = edge.getFromPin();
							var fromPinNode = edge.getFromPin().getNode();
							if (selectedNodes[fromPinNode.getId()]) {
								// the "from" node is in the selected nodes -> skip
								return;
							}
							var fromPinNodeView = this._scene.getViewByWorkflowItem(fromPinNode);
							if (fromPinNodeView) {
								var fromPinViewLocation = fromPinNodeView.getLocationForPin(fromPin);
								var y = fromPinViewLocation.y;
								horizontalGridLines.push({
									y: y,
									offset: offsetY
								});

								if (closestFromPinLocation == null || fromPinViewLocation.x > closestFromPinLocation.x) {
									// first one or more to the right (closer) connected "From" pin Location
									closestFromPinLocation = fromPinViewLocation;
								}
							}
						}, this);
					} else if (pin.acceptsOutgoingEdges()) {
						// Horizontal grid (toPin center of outgoing edges)
						_.each(pin.getEdges(), function (edge) {
							var toPin = edge.getToPin();
							var toPinNode = edge.getToPin().getNode();
							if (selectedNodes[toPinNode.getId()]) {
								// the "to" node is in the selected nodes -> skip
								return;
							}
							var toPinNodeView = this._scene.getViewByWorkflowItem(toPinNode);
							if (toPinNodeView) {
								var toPinViewLocation = toPinNodeView.getLocationForPin(toPin);
								var y = toPinViewLocation.y;
								horizontalGridLines.push({
									y: y,
									offset: offsetY
								});

								if (closestToPinLocation == null || toPinViewLocation.x < closestToPinLocation.x) {
									// first one or more to the left (closer) connected "To" pin Location
									closestToPinLocation = toPinViewLocation;
								}
							}
						}, this);
					}
				}, this);

				// Vertical edge in the middle of the shortest path (middle of closest connected fromPin and closest connected toPin)
				if (closestFromPinLocation !== null && closestToPinLocation !== null) {
					var x = (closestFromPinLocation.x + closestToPinLocation.x) / 2;
					verticalGridLines.push({
						x: x,
						offset: offsetX
					});
				}
			}

			this._gridHelper.setVerticalGridLines(verticalGridLines);
			this._gridHelper.setHorizontalGridLines(horizontalGridLines);
		};

		/**
		 * @override
		 */
		MoveNodesTool.prototype.getDefaultCursor = function () {
			return "grabbing-cursor";
		};

		return MoveNodesTool;

	});
