define([
		"underscore",
		"WFEditor/tools/AbstractInteractiveTool",
		"WFEditor/tools/ToolConstants",
		"WFEditor/workflow/ModelConstants",
		"utils/ShapeUtils",
		"utils/Observer",
		"WFEditor/tools/selection/SelectionHelper"
	],
	function (
		_,
		AbstractInteractiveTool,
		ToolConstants,
		ModelConstants,
		ShapeUtils,
		Observer,
		SelectionHelper
	) {
		"use strict";

		/**
		 * Tool that handles all the selection of Nodes
		 *
		 * @exports SelectionTool
		 * @augments AbstractInteractiveTool
		 * @class
		 *
		 * @param {SceneController} sceneController - The scene controller
		 * @param {Observable} interactionEvents - the object that is observed to determine activation of this tool
		 * @param {WFEditorContext} WFEditorContext
		 */
		var SelectionTool = function (sceneController, interactionEvents, WFEditorContext) {
			AbstractInteractiveTool.call(this, ToolConstants.Tools.SELECTIONTOOL, sceneController, WFEditorContext);
			Observer.mixin(this);

			/**
			 * @member {Workflow} The current workflow model
			 * @private
			 */
			this._currentWorkflow = null;

			/**
			 * @member {Boolean} keep track of whether ctrl is pressed or not
			 * @private
			 */
			this._ctrlKeyPressed = false;

			/**
			 * @member {ToolbarModel}
			 * @private
			 */
			this._toolbarModel = WFEditorContext.get("toolbarModel");

			//register the default state to the tool
			this._registerDefaultState();
			this._registerSelectionStates();
			//register all the events that this tool listens to
			this.registerActiveEvents(["mousemove", "mousedown", "mouseup", "mouseleave", "mouseenter", "contextmenu"]);

			this._observe(interactionEvents, "mousedown", _.bind(this._handleMouseInteractionEvent, this));
			this._observe(interactionEvents, "mouseup", _.bind(this._handleMouseInteractionEvent, this));
			this._observe(interactionEvents, "mousemove", _.bind(this._handleMouseInteractionEvent, this));

			// reset selection checks if the mouse interaction tool is (re)activated
			this._observe(interactionEvents, "activating", _.bind(function () {
				this.changeState(ToolConstants.State.DEFAULTSTATE);
			}, this));
			this.changeState(ToolConstants.State.DEFAULTSTATE);

			this.bindKeyEvents(["esc"], _.bind(this._escapePressed, this), "keydown");
		};

		// extend from the AbstractTool
		SelectionTool.prototype = Object.create(AbstractInteractiveTool.prototype);
		SelectionTool.prototype.constructor = SelectionTool;

		/**
		 * @override
		 */
		SelectionTool.prototype.activate = function (event, silent) {
			AbstractInteractiveTool.prototype.activate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, true);
		};

		/**
		 * @override
		 */
		SelectionTool.prototype.deActivate = function (event, silent) {
			AbstractInteractiveTool.prototype.deActivate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, false);
		};

		/**
		 * Handles the mouse interaction events by triggering the current state.
		 * @param {Object} options
		 */
		SelectionTool.prototype._handleMouseInteractionEvent = function (options) {
			this._ctrlKeyPressed = options.e.ctrlKey;
			this.triggerEvent(options.eventName, options);
		};

		/**
		 * Handle mouse down from mouse interaction tool
		 * @param {Object} options - 	{
		 * 								e: mouseEvent
		 * 								hitArea: HitArea under mouse pointer
		 * 								position: mouse position
		 * 								}
		 */
		SelectionTool.prototype._checkForSelect = function (options) {
			var hitArea = options.hitArea;
			if (hitArea && !hitArea.hasClass(ModelConstants.HitAreaClass.NodeSelect)) {
				// Not a hit area we should react to
				return;
			}
			if (hitArea && hitArea.getView() && hitArea.getView().getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Node) {
				// On a Node with hit arae NodeSelect
				var viewObject = hitArea.getView();
				this._currentWorkflow = viewObject.getModel().getWorkflow();
				this._startPointWorld = this._scene.getWorldPointFromScenePoint(options.position);
				var selectedNodes = this._currentWorkflow.getSelectedNodes();

				if (_.contains(selectedNodes, viewObject.getModel()) && options.e.button === 0) {
					// mouse down on a selected node, possibly a select or a start of a selection with ctrl enabled
					this.changeState(ToolConstants.State.SHOULDSELECTSTATE, options);
					return;
				} else if (this._ctrlKeyPressed) {
					// Mouse down on a node that wasn't selected and ctrl key was pressed -> add it to the selection
					this._sceneController.selectNode(viewObject.getModel(), false);
				} else {
					// Mouse down on a not selected node -> select that
					this._sceneController.selectNode(viewObject.getModel(), true);
				}
			} else if (!hitArea && options.e.button === 0) {
				if (!this._ctrlKeyPressed) {
					// Clear selection if ctrl wasn't pressed
					this._sceneController.clearSelection();
				}
				this.changeState(ToolConstants.State.SHOULDSELECTSTATE, options);
			}
		};

		/**
		 * If no selection occured, reset the start point
		 */
		SelectionTool.prototype._resetSelectionChecks = function () {
			this._startPoint = null;
		};

		/**
		 * Handle mouse up from mouse interaction tool
		 * @param {Object} options - 	{
		 * 								e: mouseEvent
		 * 								hitArea: HitArea under mouse pointer
		 * 								position: mouse position
		 * 								}
		 */
		SelectionTool.prototype._checkForDeselect = function (options) {
			var hitArea = options.hitArea;

			// No selection occured
			this.changeState(ToolConstants.State.DEFAULTSTATE);
			if ((hitArea && !hitArea.hasClass(ModelConstants.HitAreaClass.NodeSelect)) || !hitArea) {
				return;
			}
			var view = hitArea.getView();
			if (view && view.getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Node && view.getModel().isSelected()) {
				if (this._ctrlKeyPressed) {
					this._sceneController.deselectNode(view.getModel());

				} else {
					this._sceneController.selectNode(view.getModel(), true);
				}
			}

		};

		/**
		 * Handle mouse move from mouse interaction tool
		 * @param {Object} options - 	{
		 * 								e: mouseEvent
		 * 								hitArea: HitArea under mouse pointer
		 * 								position: mouse position
		 * 								}
		 */
		SelectionTool.prototype._checkForSelection = function (options) {
			if (options.e.button !== 0) {
				this.changeState(ToolConstants.State.DEFAULTSTATE);
			}
			if (this._shouldSelect(options.position)) {

				this._cachedBoundingRect = this._scene.getBoundingRect();
				this._startPointWorld = this._scene.getWorldPointFromScenePoint(this._startPoint);
				this._toolManager.activateTool(this);
				this.changeState(ToolConstants.State.SELECTIONSTATE, this._startPoint);
			}
		};



		/**
		 * Should the selection tool start selecting?
		 * @param   {EventObj} event
		 * @returns {Boolean}
		 */
		SelectionTool.prototype._shouldSelect = function (mousePosition) {
			if (this._startPoint === null) {
				return false;
			}
			var viewObject = this._scene.getObjectAtPosition(mousePosition);
			var offset = this._startPoint.sizeTo(mousePosition);

			// If the mouse is on top of a selected node, the threshold will be a bit higher in case the user actually wishes to deselect the node
			var mouseOnSelectedNodeAndSmallOffset = viewObject && viewObject.getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Node && (Math.abs(offset.width) < 3 || Math.abs(offset.height) < 3);
			return (Math.abs(offset.width) > 0 || Math.abs(offset.height) > 0) && !mouseOnSelectedNodeAndSmallOffset;
		};

		/**
		 * Calculates the selection rectangle and sets it in the scene model
		 * @param {ShapeUtils.Point} position - The mouse position
		 */
		SelectionTool.prototype._calculateSelection = function (position) {
			var worldPosition = this._scene.getWorldPointFromScenePoint(position);
			var visibleRect = this._scene.getVisibleRect();
			worldPosition.containPointInRect(visibleRect);
			var offset = this._startPointWorld.sizeTo(worldPosition);
			var rect = new ShapeUtils.Rect(this._startPointWorld, offset);
			this._selectionHelper.setSelectionRect(rect);
			this._sceneController.selectNodesInRect(rect);
		};

		/**
		 * Registers the states of this tool
		 */
		SelectionTool.prototype._registerSelectionStates = function () {

			// Default state listens to  mouse downs of the mouse interaction tool. 
			this.registerState(ToolConstants.State.DEFAULTSTATE, {
				"enterState": _.bind(this._resetSelectionChecks, this),
				"mousedown": _.bind(this._checkForSelect, this),
				"mousemove": function () {},
				"mouseup": function () {},
				"mouseleave": function () {},
				"mouseenter": function () {},
				"contextmenu": function () {}
			});

			// This state decides if a selection with a selection rectangle has to start or not
			this.registerState(ToolConstants.State.SHOULDSELECTSTATE, {
				"enterState": _.bind(function (previousState, options) {
					this._startPoint = options.position;
				}, this),
				"mousemove": _.bind(this._checkForSelection, this),
				"mouseup": _.bind(this._checkForDeselect, this),
				"mousedown": function () {},
				"mouseleave": function () {},
				"mouseenter": function () {},
				"contextmenu": function () {}
			});


			// This state is the active state for selecting with a selection rectangle
			this.registerState(ToolConstants.State.SELECTIONSTATE, {
				"enterState": _.bind(function (previousState, mousePosition) {
					var workflow;
					if (!this._selectionHelper) {
						workflow = this._scene.getWorkflowModel();
						this._selectionHelper = new SelectionHelper(workflow, "MouseSelection");
					}
					this._sceneController.startSelectionSequence(!this._ctrlKeyPressed);

					this._selectionHelper.setIsSelecting(true);
					this._calculateSelection(mousePosition);

					if (workflow) {
						// if workflow is defined, we didn't add the selectionhelper yet to the workflow
						workflow.addHelper(this._selectionHelper);
					}

					this._scene.setInteractive(this._selectionHelper.getId(), true);
				}, this),
				"leaveState": _.bind(function (e) {
					this._selectionHelper.setIsSelecting(false);
					this._sceneController.stopSelectionSequence();
					this._scene.setInteractive(this._selectionHelper.getId(), false);
					if (this.isActive) {
						this._toolManager.deactivateCurrentTool();
					}
				}, this),
				"mousemove": _.bind(function (e) {
					if (e.which === 1) {
						e.preventDefault();
						var position = this._getMousePosition(e);
						this._stopIntervalPanning();
						if (this._shouldPan(position, true)) {
							this._calculatePanning(position, true);
							// When holding the mouse near the border without moving, the panning should continue
							this._startIntervalPanning(position, this._calculateSelection, true);
						}
						this._calculateSelection(position);

					}
				}, this),
				"mouseup": _.bind(function (e) {
					e.preventDefault();
					if (e.which === 1) {
						this._stopIntervalPanning();
						this.changeState(ToolConstants.State.DEFAULTSTATE);
					}
				}, this),
				"mousedown": _.bind(function (e) {}, this),
				"mouseleave": _.bind(function (e) {
					var mousePosition = this._getMousePosition(e);
					this._calculateSelection(mousePosition);
					this._continuePanningWhileMouseOutsideCanvas(this._calculateSelection, _.bind(function () {
						this.changeState(ToolConstants.State.DEFAULTSTATE);
					}, this), true);

				}, this),
				"mouseenter": _.bind(function (e) {
					this._stopPanningWhileMouseOutsideCanvas();

				}, this),
				"contextmenu": function (e) {
					e.preventDefault();
				}

			});

		};

		/**
		 * @override
		 */
		SelectionTool.prototype.allowDeactivate = function () {
			return !this._selectionHelper || !this._selectionHelper.isSelecting();
		};


		/**
		 * Clear selection when escape was pressed
		 * @param {Event} e - keypress event
		 */
		SelectionTool.prototype._escapePressed = function (e) {
			e.stopPropagation();
			e.preventDefault();

			this._sceneController.clearSelection();

			// if the selection tool is active, go to default state and deactivate
			if (this.isActive()) {
				this.changeState(ToolConstants.State.DEFAULTSTATE);
				this._toolManager.deactivateCurrentTool();
			}
		};

		return SelectionTool;

	});
