define(["WFEditor/scene/AbstractView",
		"utils/ShapeUtils",
		"utils/DashUtils"
	],
	function (AbstractView,
		ShapeUtils,
		DashUtils) {
		"use strict";

		/**
		 * A view for a SelectionHelper
		 * @exports SelectionView
		 * @class
		 * @augments {AbstractView}
		 *
		 * @param {SelectionHelper} model
		 */
		var SelectionView = function (model) {
			AbstractView.call(this, model);

			this._observe(model, "changed.*", this._updateBoundingRect);

		};

		SelectionView.prototype = Object.create(AbstractView.prototype);
		SelectionView.prototype.constructor = SelectionView;

		/**
		 * Renders the view on canvas context.
		 * @Override
		 * @param {CanvasRenderingContext2D} context
		 * @param {Layer} layer
		 * @param {Boolean} quickRender - if true, only high priority render actions should be executed (i.e. during panning)
		 */
		SelectionView.prototype.render = function (ctx, layer, quickRender) {
			if (this._model.isSelecting()) {
				ctx.save();
				ctx.beginPath();
				ctx.strokeStyle = "rgba(120,120,120,1)";
				var t = layer.getTransformation();
				var i = t.createInverse();

				// To render a crisp rectangle, the scene coordinates have to be rounded at 0.5
				var sceneRectCrisp = t.transformedRect(this._model.getSelectionRect());
				sceneRectCrisp.origin.x = Math.ceil(sceneRectCrisp.origin.x-1) + 0.5;
				sceneRectCrisp.origin.y = Math.ceil(sceneRectCrisp.origin.y-1) + 0.5;
				sceneRectCrisp.size.width = Math.round(sceneRectCrisp.size.width);
				sceneRectCrisp.size.height = Math.round(sceneRectCrisp.size.height);

				// Transform back to world coordinates
				sceneRectCrisp = i.transformedRect(sceneRectCrisp);

				ctx.lineWidth = 1;
				var dashes = i.transformedSize(new ShapeUtils.Size(4, 3));
				var dashArray = [dashes.width, dashes.height];
				var left = sceneRectCrisp.left();
				var right = sceneRectCrisp.right();
				var top = sceneRectCrisp.top();
				var bottom = sceneRectCrisp.bottom();
				// top
				DashUtils.drawDashedLine(ctx, left, top, right, top, dashArray);
				// bottom
				DashUtils.drawDashedLine(ctx, left, bottom, right, bottom, dashArray);
				// left
				DashUtils.drawDashedLine(ctx, left, top, left, bottom, dashArray);
				// right
				DashUtils.drawDashedLine(ctx, right, top, right, bottom, dashArray);

				ctx.stroke();
				ctx.restore();
			}
		};

		/**
		 * @Override
		 */
		SelectionView.prototype.getZIndex = function () {
			return 100;
		};

		/**
		 * @Override
		 */
		SelectionView.prototype._updateBoundingRect = function () {
			var selectionRect = this._model.getSelectionRect();
			if (selectionRect && this._model.isSelecting()) {
				this._setBoundingRect(selectionRect.getNormalizedRectArray());
			} else {
				this._setBoundingRect(null);
			}

		};

		return SelectionView;
	});
