define([],
	function () {
		"use strict";

		/**
		 * Abstract definition of the workflow factory. This class specifies which methods a
		 * specific implementation of the factory has to define to be able to work in the framework.
		 *
		 * @exports AbstractWorkflowFactory
		 * @class
		 */
		var AbstractWorkflowFactory = function () {};

		/**
		 * Creates the entire workflow model.
		 *
		 * @abstract
		 * @param {Function} callback - this function needs to be called when the workflow was successfully created.
		 *                            the first parameter is an instance of Workflow with the entire constructed workflow.
		 */
		AbstractWorkflowFactory.prototype.createWorkflow = function (callback) {};
    
		/**
		 * Returns the definitions of all the nodes that should be shown in the NodeGallery.
		 * 
		 * @abstract
		 * @return {Array.<GalleryNode>}
		 */
        AbstractWorkflowFactory.prototype.getNodeDefinitions = function () {
        	return [];
        };

		/**
		 * Creates an edge
		 *
		 * @abstract
		 * @param {Workflow} workflow
		 * @param {String} id - edge id
		 * @param {Pin} fromPin
		 * @param {Pin} toPin
		 */
		AbstractWorkflowFactory.prototype.createEdge = function (workflow, id, fromPin, toPin) {};
	
		/**
		 * creates a promise for a node. If the node can be created the promise should be resolved with the corresponding node.
		 * If this isn't the case, the promise can be rejected with a message containing the reason why it cannot be created.
		 * @param {Workflow} workflow
		 * @param {Object} options - additional options to create the node  
		 * @return {$.Deferred.<Node>} - this method should return a jquery promise of a Node                        
		 */
		AbstractWorkflowFactory.prototype.createNode = function (workflow, options) {};

		return AbstractWorkflowFactory;
	});