define([
		"WFEditor/workflow/ModelConstants",
		"utils/Observable",
		"utils/ShapeUtils",
		"underscore"
	],
	function (
		ModelConstants,
		Observable,
		ShapeUtils,
		_
	) {
		"use strict";

		/**
		 * Workflow contains all models of its components: Nodes, Pins and Edges
		 *
		 * @exports Workflow
		 * @class
		 *
		 * @param {String} ID
		 */
		var Workflow = function (ID) {
			Observable.mixin(this, ["workflow", ID], "WFEditor." + ID);

			/**
			 * @member {String}
			 * @private
			 */
			this._ID = ID;

			/**
			 * @member {Object.<String, Node>} maps nodeID to the node model
			 * @private
			 */
			this._nodes = {};

			/**
			 * @member {Object.<String, Edge>} maps edgeID to the edge model
			 * @private
			 */
			this._edges = {};

			/**
			 * @member {Object.<String, Token>} maps tokenId to the Token model
			 * @private
			 */
			this._tokens = {};

			/**
			 * @member {Object.<String, Helper} maps helperID to the Helper Model
			 * @private
			 */
			this._helpers = {};
			/**
			 * @member {Token}
			 * @private
			 */
			this._highlightedToken = null;

			/**
			 * @member {Object.<String, Node>} map of all the selected nodes
			 * @private
			 */
			this._selectedNodes = {};

			/**
			 * @member {Object.<String,Any>} privileges to do certain actions with the workflow
			 */
			this._privileges = {};

			/**
			 * @member {Number} this index is used to hightlight one particular active node. 
			 * eg. To zoom in to that node. 
			 * The number is the index of the node in the array returned by getActiveNodes()
			 *
			 * @private
			 */
			this._focusedActiveNodeIndex = 0;
		};


		/**
		 * Get the ID of the workflow
		 * @returns {String} ID of the workflow
		 */
		Workflow.prototype.getID = function () {
			return this._ID;
		};


		/**
		 * Add a node to the workflow
		 * @param {Node} node
		 */
		Workflow.prototype.addNode = function (node) {
			this._nodes[node.getId()] = node;
			this._emit("Node.added", node);
		};

		/**
		 * Remove a node from the workflow
		 * @param {Node} node
		 */
		Workflow.prototype.removeNode = function (node) {
			delete this._nodes[node.getId()];
			this._emit("Node.removed", node);
		};

		/**
		 * Get a node form the workflow with id.
		 * @param   {String} id
		 * @returns {Node}
		 */
		Workflow.prototype.getNode = function (id) {
			return this._nodes[id];
		};

		/**
		 * Add an edge to the workflow.
		 * @param {Edge} edge
		 */
		Workflow.prototype.addEdge = function (edge) {
			this._edges[edge.getId()] = edge;
			this._emit("Edge.added", edge);
		};

		/**
		 * remove an edge from the workflow
		 * @param {Edge} edge
		 */
		Workflow.prototype.removeEdge = function (edge) {
			delete this._edges[edge.getId()];
			this._emit("Edge.removed", edge);
		};

		/**
		 * Add a helper to the workflow
		 * @param {Helper} helper
		 */
		Workflow.prototype.addHelper = function (helper) {
			if (!this._helpers[helper.getId()]) {
				this._helpers[helper.getId()] = helper;
				this._emit("Helper.added", helper);
			}
		};

		/**
		 * Remove a helper form the workflow
		 * @param {Helper} helper
		 */
		Workflow.prototype.removeHelper = function (helper) {
			if (this._helpers[helper.getId()]) {
				delete this._helpers[helper.getId()];
				this._emit("Helper.removed", helper);
			}
		};

		/**
		 * Get an added edge by id
		 * @param   {String} edgeId
		 * @returns {Edge}
		 */
		Workflow.prototype.getEdgeById = function (edgeId) {
			return this._edges[edgeId];
		};

		/**
		 * Does the workflow have any items in it.
		 * @returns {Boolean}
		 */
		Workflow.prototype.hasWorkflowItems = function () {
			return _.size(this._nodes) > 0;
		};

		/**
		 * return all the workflow items of this workflow
		 * @returns {Object.<String, WorkflowItem>}
		 */
		Workflow.prototype.getWorkflowItems = function () {
			return _.flatten(this._nodes, this._edges);
		};

		/**
		 * Returns all the nodes of this workflow
		 * @returns {Object.<String, Node>}
		 */
		Workflow.prototype.getNodes = function () {
			return this._nodes;
		};

		/**
		 * Returns all the edges of this workflow
		 * @returns {Object.<String, Edge>}
		 */
		Workflow.prototype.getEdges = function () {
			return this._edges;
		};

		/**
		 * Returns all the helpers of this workflow
		 * @returns {Object.<String, Edge>}
		 */
		Workflow.prototype.getHelpers = function () {
			return this._helpers;
		};

		/**
		 * Returns the currently highlighted token
		 * @returns {Token}
		 */
		Workflow.prototype.getHightLightedToken = function () {
			return this._highlightedToken;
		};

		/**
		 * Set the highlighted token in the workflow
		 * @param {Token} highlightedToken
		 */
		Workflow.prototype.setHighlightedToken = function (highlightedToken) {
			this._highlightedToken = highlightedToken;
			this._emit("highlightedPathChanged");
		};

		/**
		 * Set the tokens in the workflow
		 * @param {Object.<String,Token>} tokens
		 */
		Workflow.prototype.setTokens = function (tokens) {
			var tokensLoaded = this._tokens.length === 0 && tokens;
			this._tokens = tokens;
			this._emit("tokensChanged");
			if (tokensLoaded) {
				this._emit("tokensLoaded");
			}
		};

		/**
		 * Get all tokens of this workflow
		 * @returns {Object.<String,Token>}
		 */
		Workflow.prototype.getTokens = function () {
			return this._tokens;
		};

		/**
		 * Get the token with specified ID
		 * @param   {String} tokenId
		 * @returns {Token}
		 */
		Workflow.prototype.getToken = function (tokenId) {
			return this._tokens[tokenId];
		};

		/**
		 * Get the privilege belonging to the key
		 * @param   {String} key - The key of the privilege
		 * @returns {Any} the privilege belonging to the key or undefined if not found
		 */
		Workflow.prototype.getPrivilege = function (key) {
			return this._privileges[key];
		};

		/**
		 * Set a privilege for this workflow
		 * @param {String} key - The key of the privilege
		 * @param {Any} value - The value of the privilege
		 */
		Workflow.prototype.setPrivilege = function (key, value) {
			this._privileges[key] = value;
		};

		/**
		 * Set privileges for this workflow.
		 * See ModelConstants.Privileges for framework defines ones. If not overwritten by the given privileges the defaults will be used.
		 * @param {Object<String, Any>} privileges - The privileges for this workflow
		 */
		Workflow.prototype.setPrivileges = function (privileges) {
			var defaults = _.clone(ModelConstants.DefaultPrivileges);
			this._privileges = _.extend(defaults, privileges);
		};


		/**
		 * Sets the nodes that are selected
		 * @param {Object.<String, Nodes>} the nodes that are selected
		 */
		Workflow.prototype.setSelectedNodes = function (nodes) {
			var newlySelected = _.omit(nodes, _.keys(this._selectedNodes));
			var noLongerSelected = _.omit(this._selectedNodes, _.keys(nodes));
			if (!_.isEqual(nodes, this._selectedNodes)) {

				_.each(newlySelected, function (node) {
					node.setSelected(true);
				});
				_.each(noLongerSelected, function (node) {
					node.setSelected(false);
				});
				this._selectedNodes = nodes;
				this._emit("nodeSelectionChanged", noLongerSelected, newlySelected);
			}

		};

		/**
		 * Select a node
		 * @param {Node} node to select
		 */
		Workflow.prototype.selectNode = function (node) {
			if (!this._selectedNodes[node.getId()]) {
				node.setSelected(true);
				this._selectedNodes[node.getId()] = node;
				var newlySelectedNodes = {};
				newlySelectedNodes[node.getId()] = node;

				this._emit("nodeSelectionChanged", {}, newlySelectedNodes);
			}
		};


		/**
		 * Deselect a node
		 * @param {Node} node to deselect
		 */
		Workflow.prototype.deselectNode = function (node) {
			if (this._selectedNodes[node.getId()]) {
				node.setSelected(false);
				delete this._selectedNodes[node.getId()];
				var deselectedNodes = {};
				deselectedNodes[node.getId()] = node;

				this._emit("nodeSelectionChanged", deselectedNodes, {});
			}
		};

		/**
		 * Gets the nodes that are selected
		 * @returns {Object.<String, Nodes>}
		 */
		Workflow.prototype.getSelectedNodes = function () {
			return this._selectedNodes;
		};

		/**
		 * Gets the active nodes. ie. the nodes that have unprocessed tokens
		 * @returns {Array.<Node>}
		 */
		Workflow.prototype.getActiveNodes = function () {
			return _.uniq(
				_.map(
					_.filter(this._tokens, function (token) { // Map all processed tokens
						return token.getPin() && !token.isProcessed() && token.getPin().getNode() && token.getPin().getNode().isVisible();
					}),
					function (token) { // To their nodes
						return token.getPin().getNode();
					}
				)
			);
		};

		/**
		 * gets the active node that is currently 'focused'
		 * @return {Node}
		 */
		Workflow.prototype.getFocusedActiveNode = function () {
			var activeNodes = this.getActiveNodes();
			return this.getActiveNodes()[(this._focusedActiveNodeIndex + activeNodes.length) % activeNodes.length];
		};

		Workflow.prototype.shiftFocusedActiveNodeIndex = function (dx) {
			var activeNodes = this.getActiveNodes();
			this._focusedActiveNodeIndex = (this._focusedActiveNodeIndex + dx + activeNodes.length) % activeNodes.length;
			this._emit("focusedActiveNodeChanged");
		};


		Workflow.prototype.getFocusedActiveNodeIndex = function () {
			var activeNodes = this.getActiveNodes();
			return (this._focusedActiveNodeIndex + activeNodes.length) % activeNodes.length;
		};

		return Workflow;
	});
