define([
		"utils/Observable"
	],
	function (
		Observable
	) {
		"use strict";

		/**
		 * WorkflowItem represents the model for a workflow item.
		 *
		 * @exports WorkflowItem
		 * @class
		 *
		 * @param {Workflow} workflow - the workflow to which this item belongs
		 * @param {String} type - type of the WorkflowItem
		 * @param {String} id - unique ID for the WorkflowItem
		 * @param {WorkflowItem} [parentItem] - Optional Parent Item
		 */
		var WorkflowItem = function (workflow, type, id, parentItem) {
			var observableID;
			if (typeof parentItem === "undefined") {
				observableID = ["workflow", workflow.getID(), type, id];
			} else {
				// the item has a parent Item
				observableID = [parentItem.getObservableID(), type, id];
			}

			Observable.mixin(this, observableID, "WFEditor." + workflow.getID());

			/**
			 * @member {String} id for the workflow item
			 * @private
			 */
			this._id = id;

			/**
			 * @member {ModelConstants.WorkflowItemType} type of the workflow item
			 * @private
			 */
			this._workflowItemType = type;

			/**
			 * @member {Workflow}
			 * @private
			 */
			this._workflow = workflow;

			/**
			 * @member {Boolean} is mouse currently above the workflow item
			 * @private
			 */
			this._isHovered = false;

			/**
			 * @member {Boolean} is the item highlighted in the workflow
			 * @private
			 */
			this._isHighlighted = false;

			/**
			 * @member {Boolean} By default all workflowItems are visible
			 * @private
			 */
			this._isVisible = true;

			/**
			 * @member{Boolean} is the Workflow item selected?
			 * @private
			 */
			this._isSelected = false;
		};

		/**
		 * Returns the id of the workflow item.
		 * @returns {String}
		 */
		WorkflowItem.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Set the hovered state of the workflow item.
		 * @param {Boolean} isHovered
		 */
		WorkflowItem.prototype.setHovered = function (isHovered) {
			if (this._isHovered !== isHovered) {
				this._isHovered = isHovered;
				this._emit("changed.hover");
			}
		};

		/**
		 * Is the mouse currently above the workflow item.
		 * @returns {Boolean}
		 */
		WorkflowItem.prototype.isHovered = function () {
			return this._isHovered;
		};

		/**
		 * Set the highlighted status of the workflow item
		 * @param {Boolean} isHighlighted
		 */
		WorkflowItem.prototype.setHighlighted = function (isHighlighted) {
			if (this._isHighlighted !== isHighlighted){
				this._isHighlighted = isHighlighted;
				this._emit("changed.highlight");
			}
		};

		/**
		 * Is the workflow item highlighted
		 * @returns {Boolean}
		 */
		WorkflowItem.prototype.isHighlighted = function () {
			return this._isHighlighted;
		};

		/**
		 * Return the workflow model this item belongs to
		 * @returns {Workflow}
		 */
		WorkflowItem.prototype.getWorkflow = function () {
			return this._workflow;
		};

		/** 
		 * Return the type of the workflowitem
		 * @returns {ModelConstants.WorkflowItemType}
		 */
		WorkflowItem.prototype.getWorkflowItemType = function () {
			return this._workflowItemType;
		};

		/**
		 * Is the workflow item visible or not
		 * @returns {Boolean}
		 */
		WorkflowItem.prototype.isVisible = function () {
			return this._isVisible;
		};

		/**
		 * Is the workflow item selected
		 * @returns {Boolean}
		 */
		WorkflowItem.prototype.isSelected = function () {
			return this._isSelected;
		};

		/**
		 * sets the selected state of this item
		 * @param {Boolean} isSelected 
		 */
		WorkflowItem.prototype.setSelected = function (isSelected) {
			if (this._isSelected !== isSelected) {
				this._isSelected = isSelected;
				this._emit("changed.select", this);
			}
		};

		return WorkflowItem;
	});
