define([
		"WFEditor/workflow/WorkflowItem",
		"WFEditor/workflow/ModelConstants",
		"WFEditor/workflow/node/NodeConfigurationPanel",
		"utils/OrderedObject",
		"underscore"
	],
	function (
		WorkflowItem,
		ModelConstants,
		NodeConfigurationPanel,
		OrderedObject,
		_
	) {
		"use strict";

		/**
		 * Node represents the model for a node in the Workflow Editor. Node consists of
		 * several pins which connect the node with other nodes (using edges).
		 *
		 * @exports Node
		 * @class
		 * @augments WorkflowItem
		 *
		 * @param {Workflow} workflow - the workflow to which the Node belongs
		 * @param {String} id - node id
		 * @param {ShapeUtils.Point} [location] - location of the node
		 */
		var Node = function (workflow, id, location) {
			WorkflowItem.call(this, workflow, ModelConstants.WorkflowItemType.Node, id);

			/**
			 *  @member {OrderedObject.<Pin>}
			 *  @private
			 */
			this._pins = new OrderedObject();

			/**
			 *  @member {ShapeUtils.Point}
			 *  @private
			 */
			this._location = location;

			/**
			 *  @member {Boolean}
			 *  @private
			 */
			this._isEnabled = true;

			/**
			 * @member {Array.<Parameter>}
			 * @private
			 */
			this._configuration = [];
		};

		Node.prototype = Object.create(WorkflowItem.prototype);
		Node.prototype.constructor = Node;

		/**
		 * Add an input pin to the node
		 * @param {Pin} pin
		 * @param {Number} [index]
		 */
		Node.prototype.addPin = function (pin, index) {
			if (index !== undefined) {
				this._pins.addItemAtIndex(pin.getId(), pin, index);
			} else {
				this._pins.addItem(pin.getId(), pin);
			}

			this._emit("Pin.added", pin);
		};

		/**
		 * Remove the given pin from the node
		 * @param {Pin} pin - The pin to remove
		 */
		Node.prototype.removePin = function (pin) {
			var removedPin = this._pins.removeByKey(pin.getId());
			if (removedPin) {
				this._emit("Pin.removed", removedPin);
			}
		};

		/**
		 * Remove all pins from a node
		 */
		Node.prototype.removeAllPins = function () {
			_.each(this.getPins(), function (pin) {
				this.removePin(pin);
			}, this);
		};

		/**
		 * Set the location of the node
		 * @param {ShapeUtils.Point} location
		 */
		Node.prototype.setLocation = function (location) {
			if (!this._location || !this._location.equals(location)) {
				this._location = location;
				this._emit("changed.location", location);
			}
		};

		/**
		 * Get the list of Pins registered on this node
		 * @returns {Array.<Pin>}
		 */
		Node.prototype.getPins = function () {
			return this._pins.getOrderedObjectList();
		};

		/**
		 * Get the pin from the collection of pins.
		 * @param   {String} pinId
		 * @returns {Pin}
		 */
		Node.prototype.getPin = function (pinId) {
			return this._pins.getItemByKey(pinId);
		};

		/**
		 * Get the number of pins
		 * @returns {Number}
		 */
		Node.prototype.getPinCount = function () {
			return this._pins.getLength();
		};

		/**
		 * Get the index of a Pin
		 * @param {String} pinID - ID of the pin
		 * @returns {Number} index of pin, -1 if pin not found
		 */
		Node.prototype.getPinIndex = function (pinID) {
			return this._pins.getIndexByKey(pinID);
		};

		/**
		 * Get a pin by its ID
		 * @returns {Pin}
		 */
		Node.prototype.getPinById = function (pinID) {
			return this._pins.getItemByKey(pinID);
		};

		/**
		 * Returns the location of the node
		 * !! IMMUTABLE !!
		 * @returns {ShapeUtils.Point}
		 */
		Node.prototype.getLocation = function () {
			return this._location;
		};

		/**
		 * Is this node removable?
		 * This can be overriden by specific nodes
		 * @returns {Boolean} true
		 */
		Node.prototype.isRemovable = function () {
			return true;
		};
	
		/**
		 * Is this node copyable?
		 * This can be overriden by specific nodes
		 * @returns {Boolean} true
		 */
		Node.prototype.isCopyable = function () {
			return true;
		};
	
		/** 
	     * return a string representation of this node for copying it
		 * @return {String}
		 */
		Node.prototype.getStateObjectForCopy = function () {
			// Override me
		};

		/**
		 * Is it possible to open a details panel for the node
		 * @returns {Boolean} false
		 */
		Node.prototype.isConfigurable = function () {
			return false;
		};

		/**
		 * Is the node enabled?
		 * @returns {Boolean} True if the node is enabled, false otherwise
		 */
		Node.prototype.isEnabled = function () {
			return this._isEnabled;
		};

		/**
		 * Set the node enabled or not
		 * @param {Boolean} enabled - True if it should become enabled, false for disabled.
		 */
		Node.prototype.setEnabled = function (enabled) {
			if (this._isEnabled !== enabled) {
				this._isEnabled = enabled;
				this._emit("changed.enabled", enabled);
			}
		};

		/**
		 * returns a copy of the configuration of this node
		 * @returns {Array.<Parameter>} 
		 */
		Node.prototype.getConfiguration = function () {
			var config = [];
			_.each(this._configuration, function (param) {
				config.push(param.copy());
			});
			return config;
		};

		/**
		 * Find a configuration parameter by name. This the original parameter and not a copy!
		 * @param {String} name - Name of the parameter to find in the configuration
		 * @returns {Parameter} The parameter with the given name from the configuration or undefined if not found
		 */
		Node.prototype.getConfigurationParameterByName = function (name) {
			return _.find(this._configuration, function (param) {
				return param.getName() === name;
			}, this);
		};

		/**
		 * sets the configuration of this node
		 * @param {Array.<Parameter>} config
		 */
		Node.prototype.setConfiguration = function (config) {
			var equal = true;
			if (config.length !== this._configuration.length) {
				equal = false;
			} else {
				for (var i = 0; i < config.length; i++) {
					if (!this._configuration[i].equals(config[i])) {
						equal = false;
						break;
					}
				}
			}
			if (!equal) {
				this._configuration = config;
				this._emit('changed.configuration', this.getConfiguration());
			}
		};

		/**
		 * returns the node configuration panel for this node
		 * @returns {NodeConfigurationPanel}
		 */
		Node.prototype.getConfigurationPanel = function () {
			return new NodeConfigurationPanel(this);
		};

		return Node;
	});