define(["underscore",
		"WFEditor/workflow/panel/CompositePanelComponent",
		"WFEditor/workflow/ModelConstants",
		"utils/Observable",
		"utils/Observer",
	],
	function (_,
		CompositePanelComponent,
		ModelConstants,
		Observable,
		Observer) {
		"use strict";

		/**
		 * A Configuration Panel for a node
		 * @class
		 * @exports NodeConfigurationPanel
		 * @mixes Observable
		 * @param {Node} node - node for which the panel is needed
		 */
		var NodeConfigurationPanel = function (node) {
			CompositePanelComponent.call(this, []);
			Observable.mixin(this, "ConfigurationPanel" + node.getId());
			Observer.mixin(this);

			/**
			 * @member {Node}
			 * @private
			 */
			this._node = node;

			/**
			 * @member {Boolean} true if there are non-persistent changes
			 * @private
			 */
			this._isDirty = false;

			this._observe(node, "changed.configuration", function (config) {
				// Check if the node's configuration is different from the configuration that is currently in the panel
				var currentConfig = this.getParameters();
				var isEqual = true;
				for (var i = 0; i < config.length; i++) {
					if (!config[i].equals(currentConfig[i])) {
						isEqual = false;
						break;
					}
				}
				if (!isEqual || config.length !== currentConfig.length) {
					this.setParameters(config);
				}
			});

		};

		NodeConfigurationPanel.prototype = Object.create(CompositePanelComponent.prototype);
		NodeConfigurationPanel.prototype.constructor = NodeConfigurationPanel;


		/**
		 * Mark the configuration as dirty
		 * @param {Boolean} isDirty
		 */
		NodeConfigurationPanel.prototype._setDirty = function (isDirty) {
			if (this._isDirty !== isDirty) {
				this._isDirty = isDirty;

				this._emit("changed.isDirty", isDirty);
			}
		};

		/**
		 * Save the changes made in the configuration panel
		 */
		NodeConfigurationPanel.prototype.save = function () {
			this._setDirty(false);
			this._emit("save");
		};

		/**
		 * Cancel the changes made in the configuration panel
		 */
		NodeConfigurationPanel.prototype.cancel = function () {
			this._emit("cancel");
		};

		/**
		 * Get the title of the panel
		 * @returns {String} title
		 */
		NodeConfigurationPanel.prototype.getTitle = function () {
			return "";
		};

		/**
		 * Get the node of the panel
		 * @returns {Node} 
		 */
		NodeConfigurationPanel.prototype.getNode = function () {
			return this._node;
		};

		/**
		 * Get the workflow of the panel
		 * @returns {Workflow} 
		 */
		NodeConfigurationPanel.prototype.getWorkflow = function () {
			return this._node.getWorkflow();
		};

		/**
		 * Is a panel readonly?
		 * @returns {Boolean} 
		 */
		NodeConfigurationPanel.prototype.isReadOnly = function () {
			return !this.getWorkflow().getPrivilege(ModelConstants.Privileges.editNodeConfiguration);
		};

		/**
		 * @override
		 */
		NodeConfigurationPanel.prototype.destroy = function () {
			CompositePanelComponent.prototype.destroy.call(this);
			this._stopObserveAll();
		};

		/**
		 * Focus the panel
		 */
		NodeConfigurationPanel.prototype.focus = function () {};

		/**
		 * With this method implementations can add extra data (eg. an info icon) to the footer
		 * @returns {JQueryObject}
		 */
		NodeConfigurationPanel.prototype.getFooterContent = function () {
			// Implement me
		};

		return NodeConfigurationPanel;
	});