define(["jquery", "underscore"],
	function ($, _) {

		"use strict";

		/**
		 * @class
		 * @exports AbstractPanelComponent
		 * 
		 * A panel component that can be placed inside a node configuration panel
		 * 
		 * @param {Array.<String>} parameters - list of parameter types the abstract panel should get
		 */
		var AbstractPanelComponent = function (parameters) {

			/**
			 * @member {Array.<String>} the names of the parameters this panel should get
			 * @private
			 */
			this._parameterWishList = parameters;

			/**
			 * @member {Array.<Parameter>} the parameter objects for this panel
			 * @private
			 */
			this._parameters = [];

			/**
			 * @member {JQueryObject} the div this panel will render in
			 * @private
			 */
			this._element = $('<div/>');

			/**
			 * @member {Boolean} is the panel visible or not
			 * @private
			 */
			this._isVisible = true;

			/**
			 * @member {Boolean} is the panel enabled (modifiable) or not
			 * @private
			 */
			this._isEnabled = true;

			/**
			 * @member {Boolean} is the panel applicable or not - not applicable panels have parameters that can be ignored;
			 *                   for example when the panel is a sub panel of a non checked radio button
			 * @private
			 */
			this._isApplicable = true;



		};


		/**
		 * sets the parameters for this panel
		 * @param {Array.<Parameter>} parameters
		 */
		AbstractPanelComponent.prototype.setParameters = function (parameters) {
			this._parameters = parameters;
		};

		/**
		 * returns the parameters for this panel
		 * @return {Array.<Parameter>}
		 */
		AbstractPanelComponent.prototype.getParameters = function () {
			return this._parameters;
		};

		/**
		 * Returns the parameter with the given name or undefined if no such parameter is present
		 * @param   {String} name
		 * @returns {Parameter}
		 */
		AbstractPanelComponent.prototype.getParameterByName = function (name) {
			return _.find(this._parameters, function (param) {
				return param.getName() === name;
			});
		};

		/**
		 * returns the types of the parameters this panel wants
		 * @returns {Array.<String>}
		 */
		AbstractPanelComponent.prototype.getParameterWishList = function () {
			return this._parameterWishList;
		};

		/**
		 * returns the parmeters that this panel wants for itself. 
		 * It is different from getParameterWishList because composite components won't ask child panels for their wishlists
		 * @returns {Array.<String>}
		 */
		AbstractPanelComponent.prototype.getOwnParameterWishList = function () {
			return this._parameterWishList;
		};

		/**
		 * returns the context element the panel will render in;
		 * @returns {JQueryObject}
		 */
		AbstractPanelComponent.prototype.getContext = function () {
			return this._element;
		};

		/**
		 * shows the panel component
		 */
		AbstractPanelComponent.prototype.show = function () {
			this._element.show();
			this._isVisible = true;
		};

		/**
		 * indicates that the component should hide
		 */
		AbstractPanelComponent.prototype.hide = function () {
			this._element.hide();
			this._isVisible = false;
		};

		/**
		 * this method is called right before the panel is removed from the dom
		 */
		AbstractPanelComponent.prototype.destroy = function () {

		};

		/**
		 * This method is called when ok is clicked. Each panel component can choose how to indicate what is wrong when false is returned.
		 * @returns {Boolean} True if everything is ok, false if not
		 */
		AbstractPanelComponent.prototype.validate = function () {
			return true;
		};

		/**
		 * this method is called after the panel is added to the dom
		 */
		AbstractPanelComponent.prototype.init = function () {

		};
		/**
		 * gets the preferred width of this panel component
		 */
		AbstractPanelComponent.prototype.getPreferredWidth = function () {
			return this._element.width();
		};

		/**
		 * Handle resize
		 */
		AbstractPanelComponent.prototype.resize = function () {
			// override me
		};

		/**
		 * Popup is being resized or dragged
		 */
		AbstractPanelComponent.prototype.startInteractive = function () {
			// override me*/
		};

		/**
		 * When this method is called, the panel should enable itself
		 */
		AbstractPanelComponent.prototype.enable = function () {
			this._isEnabled = true;
		};


		/**
		 * when this method is called, the panel should disable itself
		 */
		AbstractPanelComponent.prototype.disable = function () {
			this._isEnabled = false;

		};

		/**
		 * Indicates that the field is currently enabled or disabled
		 */
		AbstractPanelComponent.prototype.isEnabled = function () {
			return this._isEnabled;
		};

		/**
		 * Indicate that the elements in the panel component is applicable 
		 */
		AbstractPanelComponent.prototype.setApplicable = function () {
			this._isApplicable = true;
		};

		/**
		 * Indicate that the elements in the panel component aren't applicable - eg. if they are sub panels of a nonchecked checkbox
		 */
		AbstractPanelComponent.prototype.setNotApplicable = function () {
			this._isApplicable = false;
			_.each(this._parameters, function (param) {
				param.setValue(param.getDefaultValue());
			});
		};

		/**
		 * Request the container element to fit the panel
		 */
		AbstractPanelComponent.prototype._fitContent = function (scrollDown) {
			this._element.trigger("fitcontent", scrollDown);
		};

		/**
		 * called when the wf editor will be removed from the dom
		 */
		AbstractPanelComponent.prototype.beforeRemoveFromDom = function () {
			// Implement me
		};

		/**
		 * called when the wf editor has been re-added to the dom
		 */
		AbstractPanelComponent.prototype.afterAddToDom = function () {
			// Implement me
		};

		return AbstractPanelComponent;

	});