define(["WFEditor/workflow/panel/AbstractPanelComponent",
		"WFEditor/workflow/panel/PanelConstants",
		"underscore"
	],
	function (AbstractPanelComponent,
		PanelConstants,
		_) {

		"use strict";

		/**
		 * @class
		 * @exports CompositePanelComponent
		 * @param {Array.<String>} parameters - list of parameters names this panel should get
		 */
		var CompositePanelComponent = function (parameters) {
			AbstractPanelComponent.call(this, parameters);

			/**
			 * @member {Array.<AbstractPanelComponent>}
			 * @private
			 */
			this._subPanels = [];


		};

		CompositePanelComponent.prototype = Object.create(AbstractPanelComponent.prototype);
		CompositePanelComponent.prototype.constructor = CompositePanelComponent;

		/**
		 * adds  the abstractPanelComponent to this composite panel
		 * @param {AbstractPanelComponel} abstractPanelComponent
		 */
		CompositePanelComponent.prototype.add = function (abstractPanelComponent) {
			this._subPanels.push(abstractPanelComponent);
			this._element.append(abstractPanelComponent.getContext());
		};

		/**
		 * sets the parameters for this panel
		 * @param {Array.<Parameter>} parameters
		 */
		CompositePanelComponent.prototype.setParameters = function (parameters) {
			this._parameters = [];
			_.each(parameters, function (parameter) {
				if (_.contains(this._parameterWishList, parameter.getName()) || _.contains(this._parameterWishList, PanelConstants.WILDCARD)) {
					this._parameters.push(parameter);
				}

			}, this);
			_.each(this._subPanels, function (panel) {
				var parameterList = [];
				var wishList = panel.getParameterWishList();
				_.each(parameters, function (parameter) {
					if (_.contains(wishList, parameter.getName()) || _.contains(wishList, PanelConstants.WILDCARD)) {
						parameterList.push(parameter);
					}

				});
				panel.setParameters(parameterList);

			}, this);
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.getParameterWishList = function () {
			var parameterWishList = this._parameterWishList || [];
			_.each(this._subPanels, function (panelComponent) {
				parameterWishList = _.union(parameterWishList, panelComponent.getParameterWishList());
			});
			return parameterWishList;
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.getParameters = function () {
			var parameters = this._parameters || [];
			_.each(this._subPanels, function (panelComponent) {
				parameters = _.union(parameters, panelComponent.getParameters());
			});
			return parameters;
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.getPreferredWidth = function () {
			var max = 0;
			_.each(this._subPanels, function (panel) {
				var width = panel.getPreferredWidth();
				if (width > max) {
					max = width;
				}
			});
			return max;
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.hide = function () {
			AbstractPanelComponent.prototype.hide.call(this);
			_.each(this._subPanels, function (subPanel) {
				subPanel.hide();
			});
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.show = function () {
			AbstractPanelComponent.prototype.show.call(this);
			_.each(this._subPanels, function (subPanel) {
				subPanel.show();
			});
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.init = function () {
			_.each(this._subPanels, function (subPanel) {
				subPanel.init();
			});
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.destroy = function () {
			_.each(this._subPanels, function (subPanel) {
				subPanel.destroy();
			});
			AbstractPanelComponent.prototype.destroy.call(this);
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.validate = function () {
			return _.every(this._subPanels, function (subPanel) {
				return subPanel.validate();
			}) && AbstractPanelComponent.prototype.validate.call(this);
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.resize = function () {
			_.each(this._subPanels, function (subPanel) {
				subPanel.resize();
			});
		};


		/**
		 * @override
		 */
		CompositePanelComponent.prototype.startInteractive = function () {
			_.each(this._subPanels, function (subPanel) {
				subPanel.startInteractive();
			});
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.disable = function () {
			AbstractPanelComponent.prototype.disable.call(this);
			_.each(this._subPanels, function (subPanel) {
				subPanel.disable();
			});
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.enable = function () {
			AbstractPanelComponent.prototype.enable.call(this);
			_.each(this._subPanels, function (subPanel) {
				subPanel.enable();
			});
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.setApplicable = function () {
			AbstractPanelComponent.prototype.setApplicable.call(this);
			_.each(this._subPanels, function (subPanel) {
				subPanel.setApplicable();
			});
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.setNotApplicable = function () {
			AbstractPanelComponent.prototype.setNotApplicable.call(this);
			_.each(this._subPanels, function (subPanel) {
				subPanel.setNotApplicable();
			});
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.beforeRemoveFromDom = function () {
			_.each(this._subPanels, function (subPanel) {
				subPanel.beforeRemoveFromDom();
			});
		};

		/**
		 * @override
		 */
		CompositePanelComponent.prototype.afterAddToDom = function () {
			_.each(this._subPanels, function (subPanel) {
				subPanel.afterAddToDom();
			});
		};

		return CompositePanelComponent;

	});