define(["tpl!./Parameter",
        "underscore"],
	function (parameter,
		_) {

		"use strict";

		/**
		 * This class represents a parameter which can be used for the AbstractPanelComponents
		 * @class
		 * @exports Parameter
		 * @param {String} name                - name of the parameter
		 * @param {String} [defaultValue = ""] - default value of the parameter
		 * @param {String} [value]
		 * @param {Array.<Parameter>} [subparameters]
		 */
		var Parameter = function (name, defaultValue, value, subparameters) {

			/**
			 * @member {String}
			 * @private
			 */
			this._name = name;

			/**
			 * @member {String}
			 * @private
			 */
			this._defaultValue = defaultValue || "";

			/**
			 * @member {String}
			 * @private
			 */
			this._value = value;

			/**
			 * @member {Array.<Parameter>}
			 * @private
			 */
			this._subparameters = subparameters || [];
		};

		/**
		 * gets the name of the parameter
		 * @returns {String}
		 */
		Parameter.prototype.getName = function () {
			return this._name;
		};

		/**
		 * gets the value of the parameter or the default value if the value is not defined
		 * @returns {String}
		 */
		Parameter.prototype.getValue = function () {
			return this._value || this._defaultValue;
		};

		/**
		 * gets the default value of the parameter
		 * @returns {String}
		 */
		Parameter.prototype.getDefaultValue = function () {
			return this._defaultValue;
		};

		/**
		 * Set the default value of the parameter
		 * @param {String} defaultValue - The new default value
		 */
		Parameter.prototype.setDefaultValue = function (defaultValue) {
			this._defaultValue = defaultValue;
		};

		/**
		 * gets the subparameters of the parameter
		 * @returns {Array.<Parameter>}
		 */
		Parameter.prototype.getSubparameters = function () {
			return this._subparameters;
		};

		/**
		 * returns the string representation of this parameter
		 * @returns {String}
		 */
		Parameter.prototype.toString = function () {
			return parameter(this);
		};

		/**
		 * returns a string representation of the sub parameters
		 * @returns {String}
		 */
		Parameter.prototype.getSubparameterXML = function () {
			var xml = "";
			_.each(this._subparameters, function (param) {
				xml += param.toString();
			});
			return xml;
		};

		/**
		 * sets the name of this parameter
		 * @param {String} name
		 */
		Parameter.prototype.setName = function (name) {
			this._name = name;
		};

		/**
		 * sets the value of this parameter
		 * @param {String} val
		 */
		Parameter.prototype.setValue = function (val) {
			this._value = val;
		};

		/**
		 * sets the sub parameters of this parameter
		 * @param {Array.<Parameter>} params 
		 */
		Parameter.prototype.setSubParameters = function (params) {
			this._subparameters = params;
		};

		/**
		 * creates a deep copy of this parameter
		 * @returns {Parameter}
		 */
		Parameter.prototype.copy = function () {
			var copiedSubparams = [];
			_.each(this._subparameters, function (subParam) {
				copiedSubparams.push(subParam.copy());
			});
			return new Parameter(this._name, this._defaultValue, this._value, copiedSubparams);
		};

		/**
		 * two params are equal if their names are equal
		 * @param   {Parameter} param
		 * @returns {Boolean}
		 */
		Parameter.prototype.equals = function (param) {
			var subparams1 = this.getSubparameters();
			var subparams2 = param.getSubparameters();
			if (subparams1.length != subparams2.length) {
				return false;
			} else {
				for (var i = 0; i < subparams1.length; i++) {
					if (!subparams1[i].equals(subparams2[i])) {
						return false;
					}
				}
			}

			return (this.getName() === param.getName() && this.getValue() === param.getValue());
		};



		return Parameter;

	});