define([
		"WFEditor/workflow/WorkflowItem",
		"WFEditor/workflow/ModelConstants"
	],
	function (
		WorkflowItem,
		ModelConstants
	) {
		"use strict";

		/**
		 * Pin represents the model for a pin in the workflow. A pin belongs to a node.
		 *
		 * @exports Pin
		 * @class
		 * @augments WorkflowItem
		 *
		 * @param {Workflow} workflow
		 * @param {Node} node
		 * @param {String} id - pin id
		 */
		var Pin = function (workflow, node, id) {
			WorkflowItem.call(this, workflow, ModelConstants.WorkflowItemType.Pin, id, node);

			/**
			 *  @member {Array.<Edge>}
			 *  @private
			 */
			this._edges = [];

			/**
			 * @member {Map.<String,Token>} maps token id to the TOken object
			 * @private
			 */
			this._tokens = {};

			/**
			 *  @member {Node}
			 *  @private
			 */
			this._node = node;

			/**
			 *  @member {Boolean} Should edges from this node be split
			 *  @private
			 */
			this._splitEdges = false;
		};

		Pin.prototype = Object.create(WorkflowItem.prototype);
		Pin.prototype.constructor = Pin;

		/**
		 * Can an edge arrive in this pin from the given fromPin
		 * @param {Pin} fromPin - The from pin of the edge
		 * @returns {Boolean} True if edges can arrive in this pin from the given fromPin, false otherwise. Default: true
		 */
		Pin.prototype.acceptsIncomingEdges = function (fromPin) {
			return true;
		};

		/**
		 * Can an edge leave from this pin to the given toPin?
		 * @param {Pin} toPin - The to pin of the edge
		 * @returns {Boolean} True if edges can leave from this pin to the given toPin, false otherwise. Default: true
		 */
		Pin.prototype.acceptsOutgoingEdges = function (toPin) {
			return true;
		};

		/**
		 * Adds an edge to the pin, depending on the accept methods.
		 * @param {Edge} edge
		 */
		Pin.prototype.addEdge = function (edge) {
			if (edge.getToPin() === this && !this.acceptsIncomingEdges(edge.getFromPin())) {
				return;
			}
			if (edge.getFromPin() === this && !this.acceptsOutgoingEdges(edge.getToPin())) {
				return;
			}
			
			this._edges.push(edge);			
			this._emit("Edge.added", edge);
		};

		/**
		 * Remove the given edge from the pin
		 * @param {Edge} edge
		 */
		Pin.prototype.removeEdge = function (edge) {
			this._edges = _.without(this._edges, edge);			
			this._emit("Edge.removed", edge);
		};

		/**
		 * Get all the edges that are connected to this pin.
		 */
		Pin.prototype.getEdges = function () {
			return this._edges;
		};

		/**
		 * Get the number of edges of this pin
		 * @returns {Number}
		 */
		Pin.prototype.getNumberOfEdges = function () {
			return _.size(this._edges);
		};

		/**
		 * Get the edge that moves to the specified pin
		 * @param   {Pin} pin
		 * @returns {Edge}
		 */
		Pin.prototype.getEdgeTo = function (pin) {
			if (this === pin) {
				return;
			}
			return _.find(this._edges, function (edge) {
				return edge.getToPin() === pin;
			}, this);
		};

		/**
		 * Add a token to this pin.
		 * @param {Token} token
		 */
		Pin.prototype.addToken = function (token) {
			if (!this._tokens[token.getId()]) {
				this._tokens[token.getId()] = token;
				this._emit("Token.added", token);
			}
		};

		/**
		 * Get the index of an edge on this pin
		 * @return {Number} index of the edge, -1 if not found
		 */
		Pin.prototype.getEdgeIndex = function (edge) {
			return _.indexOf(this._edges, edge);
		};

		/**
		 * Returns all the tokens that are connected to this pin.
		 * @returns {Array.<Token>}
		 */
		Pin.prototype.getTokens = function () {
			return this._tokens;
		};

		/**
		 * Get the token by tokenId
		 * @param   {String} tokenId
		 * @returns {Token}
		 */
		Pin.prototype.getToken = function (tokenId) {
			return this._tokens[tokenId];
		};

		/**
		 * Does the pin have any tokens
		 * @returns {Boolean}
		 */
		Pin.prototype.hasTokens = function () {
			return _.size(this._tokens) > 0;
		};

		/**
		 * Get the number that needs to be displayed in the pin whenever there are tokens.
		 * @returns {Number}
		 */
		Pin.prototype.getNumberForPin = function () {
			var number = 0;
			_.each(this._tokens, function (token) {
				number += token.getWeight();
			}, this);
			return number;
		};

		/**
		 * Get the node for which the pin is defined
		 * @returns {Node}
		 */
		Pin.prototype.getNode = function () {
			return this._node;
		};

		/**
		 * Is one of the edges that connect this pin hovered.
		 * @returns {Boolean}
		 */
		Pin.prototype.isConnectedEdgeHovered = function () {
			return _.some(this._edges, function (edge) {
				return edge.isHovered();
			}, this);
		};

		/**
		 * Should edges be split
		 * @returns {Boolean} True if edges should be split, false otherwise
		 */
		Pin.prototype.isSplitEdges = function () {
			return this._splitEdges;
		};

		/**
		 * Set the split edges property
		 * @param {Boolean} splitEdges - True to split edges, false otherwise
		 */
		Pin.prototype.setSplitEdges = function (splitEdges) {
			if (this.isSplitEdges() !== splitEdges) {
				this._splitEdges = splitEdges;
				this._emit("changed.splitEdges", splitEdges);
			}
		};

		return Pin;
	});