define([
		"WFEditor/workflow/WorkflowItem",
		"WFEditor/workflow/ModelConstants",
		"underscore"
	],
	function (
		WorkflowItem,
		ModelConstants,
		_
	) {
		"use strict";

		/**
		 * Token represents a collection of data that moves in the workflow and is shown in
		 * the feedback version of a workflow.
		 *
		 * @exports Token
		 * @class
		 * @augments WorkflowItem
		 *
		 * @param {Workflow} workflow
		 * @param {Pin} [Pin] - The pin where this token belongs to. Can be undefined.
		 * @param {String} id - token id
		 * @param {Boolean} isProcessed - has this string been processed yet?                   
		 */
		var Token = function (workflow, pin, id, isProcessed) {
			WorkflowItem.call(this, workflow, ModelConstants.WorkflowItemType.Token, id, pin);

			/**
			 *  @member {Array.<Token>}
			 *  @private
			 */
			this._parentTokens = [];

			/**
			 *  @member {Pin}
			 *  @private
			 */
			this._pin = pin;

			/**
			 *  @member {Boolean}
			 *  @private
			 */
			this._isSelected = false;

			/**
			 *  @member {Boolean}
			 *  @private
			 */
			this._isProcessed = isProcessed;

		};

		Token.prototype = Object.create(WorkflowItem.prototype);
		Token.prototype.constructor = Token;

		/**
		 * Returns the parent tokens
		 * @returns {Array.<Token>}
		 */
		Token.prototype.getParents = function () {
			return this._parentTokens;
		};

		/**
		 * Add a parent token to this token.
		 * @param {Token} newParent
		 */
		Token.prototype.addParent = function (newParent) {
			this._parentTokens.push(newParent);
		};

		/**
		 * Remove the parent tokens for this token
		 */
		Token.prototype.removeParentTokens = function () {
			this._parentTokens = [];
		};

		/**
		 * Check if a token with the given ID is a parent token of this token
		 * @param {String} parentId
		 * @returns {Boolean} True if the token with the given parent ID is a parent of this token, false otherwise
		 */
		Token.prototype.hasParent = function (parentId) {
			return _.some(this._parentTokens, function (parentToken) {
				return parentToken.getId() === parentId;
			}, this);
		};


		/**
		 * Check if this token has future tokens
		 * @returns {Boolean} True if this token is a parent of another token, false otherwise
		 */
		Token.prototype.hasFutureTokens = function () {
			return _.some(this.getWorkflow().getTokens(), function (token) {
				return token.hasParent(this.getId());
			}, this);
		};

		/**
		 * get the Pin this token is connected to or undefined if not attached to a pin
		 * @returns {Pin}
		 */
		Token.prototype.getPin = function () {
			return this._pin;
		};

		/**
		 * Get the number that is used for this token (weight of the token). by default this is 1.
		 * @returns {Number} [[Description]]
		 */
		Token.prototype.getWeight = function () {
			return 1;
		};

		/**
		 * Set the selected state of the token
		 * @param {Boolean} isSelected
		 */
		Token.prototype.setSelected = function (isSelected) {
			this._isSelected = isSelected;
		};

		/**
		 * Is the token selected or not.
		 * @returns {Boolean}
		 */
		Token.prototype.isSelected = function () {
			return this._isSelected;
		};

		/**
		 * has the token been processed yet
		 * @returns {Boolean}
		 */
		Token.prototype.isProcessed = function () {
			return this._isProcessed;
		};

		/**
		 * Sets the processed state of the Token
		 */
		Token.prototype.setProcessed = function (isProcessed) {
			if (isProcessed !== this._isProcessed) {
				this._isProcessed = isProcessed;
				this._emit("tokenChanged", this);
			}

		};

		return Token;
	});