define(["webcenterapi/AbstractAPI",
		"webcenterapi/AttributesAPI",
		"webcenterapi/TaskTypeAPI",
		"WFEditorWCR/core/WebCenterWFEditorConstants",
		"WFEditorWCR/workflow/WebCenterWorkflowState",
		"WFEditorWCR/data/WebCenterAttributeCategory",
		"webcenterapi/data/WebCenterUser",
		"webcenterapi/data/WebCenterGroup",
		"webcenterapi/data/WebCenterRole",
		"WFEditorWCR/data/WebCenterTextContentElementType",
		"WFEditorWCR/data/WebCenterTextContentLanguage",
		"packaging-content-management/text-content-services/languageService",
		"packaging-content-management/text-content-utils/LanguageUtils",
		"packaging-content-management/ContentConstants",
		"utils/Cache",
		"utils/HTTPUtils",
		"utils/XMLUtils",
		"utils/Map",
		"component/loadingindicator/LoadingIndicatorSingleton",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery"
	],
	function (
		AbstractAPI,
		AttributesAPI,
		TaskTypeAPI,
		WebCenterWFEditorConstants,
		WebCenterWorkflowState,
		WebCenterAttributeCategory,
		WebCenterUser,
		WebCenterGroup,
		WebCenterRole,
		WebCenterTextContentElementType,
		WebCenterTextContentLanguage,
		languageService,
		LanguageUtils,
		ContentConstants,
		Cache,
		HTTPUtils,
		XMLUtils,
		Map,
		LoadingIndicator,
		i18n, 
		_,
		$) {
		"use strict";

		/**
		 * Utility class used for all communication with WebCenter
		 * @exports WebCenterAPI
		 * @class
		 * 
		 * @param {Object} wcrConfig - webcenter configuration object
		 * @param {ErrorHandler} errorHandler
		 */
		var WebCenterAPI = function (wcrConfig, errorHandler) {
			AbstractAPI.call(this, wcrConfig, errorHandler);

			/**
			 * @member {Number} interval ID (value used by setInterval to pauze/stop polling
			 * @private
			 */
			this._keepSessionAlive = null;

			/**
			 * @member {Promise.<XMLDocument>} Promise for the WorkflowDefinition call
			 * @private
			 */
			this._workflowDefinitionsPromise = null;

			/**
			 * @member {Promise.<Map.<WebCenterAttributeCategory>>} Promise for the WebCenterAttributeCategories
			 * @private
			 */
			this._deferredAttributeCategoriesPromise = null;

			/**
			 * @member {Promise.<Map.<XMLDocument>>} Promise for the call for fetching attribute categories
			 * @private
			 */
			this._attributeCategoriesPromise = null;

			/**
			 * @member {AttributesAPI} API for fetching attributes stuff
			 * @private
			 */
			this._attributesAPI = new AttributesAPI(wcrConfig, errorHandler);

			/**
			 * @member {TaskTypeAPI} API for fetching task type stuff
			 * @private
			 */
			this._taskTypeAPI = new TaskTypeAPI(wcrConfig, errorHandler);

			/**
			 * @member {Promise.<Map.<WebCenterUser>, Map.<WebCenterGroup>, Map.<WebCenterRole>>} Promise for users, groups and roles
			 * @private
			 */
			this._deferredUsersGroupsRolesPromise = null;
			/**
			 * @member {Promise.<XMLDocument>} Promise for the call for fetching users, groups and roles
			 * @private
			 */
			this._usersGroupsRolesPromise = null;

			/**
			 * @member {Boolean} is a save operation ongoing?
			 * @private 
			 */
			this._saveOngoing = false;
			/**
			 * @member {DocVersionViewableCache} cache for checking whether a document is viewable
			 * @private
			 */
			this._docVersionViewableCache = null;
			/**
			 * @member {Promise.<Map.<WebCenterProjectStatus>>} promise for the list of WebCenter statuses
			 * @private 
			 */
			this._projectStatusListPromise = null;

			/**
			 * @member {Promise.<Map.<WebCenterWorkflowConnection>> promise for the list of WebCenter workflow connections
			 * @private
			 */
			this._workflowConnectionsPromise = null;
			/**
			 * @member {Promise.<Map.<WebCenterTextContentElementType>>} promise for the list of WebCenter text content element types
			 * @private 
			 */
			this._textContentElementTypesPromise = null;

			/**
			 * @member {Promise.<Map.<WebCenterTextContentElementType>>} promise for the list of WebCenter table content element types
			 * @private 
			 */
			this._tableContentElementTypesPromise = null;

			/**
			 * @member {Promise.<Map.<WebCenterTextContentLanguage>>} promise for the list of WebCenter text content languages
			 * @private 
			 */
			this._textContentLangListPromise = null;

			/**
			 * @member {Cache} promised cache for task type definitions
			 * @private
			 */
			this._taskTypeDefinitionCache = new Cache(30, false);
			this._taskTypeDefinitionCache._loadItem = _.bind(this._loadTaskTypeDefinition, this);

			/**
			 * @member {Cache} promised cache for task type specifications
			 * @private
			 */
			this._taskTypeSpecificationsCache = new Cache(30, false);
			this._taskTypeSpecificationsCache._loadItem = _.bind(this._loadTaskTypeSpecifications, this);
		};

		WebCenterAPI.prototype = Object.create(AbstractAPI.prototype);
		WebCenterAPI.prototype.constructor = WebCenterAPI;

		/**
		 * Get the xml description of the workflow of a task type
		 * @param {String} taskTypeID 
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise for the workflowXML
		 */
		WebCenterAPI.prototype.getTaskTypeWorkflowXML = function (taskTypeID) {
			var deferred = $.Deferred();
			this.callSDK("DownloadWorkflowXML.jsp", {
				tasktypeid: taskTypeID
			}).fail(_.bind(function (jqXHR, textStatus, errorThrown, codeString) {
				if (codeString === "151") { // empty task type -> resolve with an empty xml
					var xml = document.implementation.createDocument("", "", null);
					var root = xml.createElement("root");
					xml.appendChild(root);
					deferred.resolve(xml, textStatus, jqXHR);
				} else {
					deferred.reject(jqXHR, textStatus, errorThrown);
					this._errorHandler.severe(i18n.t("LoadWorkflowFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				}
			}, this)).done(function (xml, textStatus, jqXHR) {
				deferred.resolve(xml, textStatus, jqXHR);
			});
			return deferred.promise();
		};


		/**
		 * Get the xml description of the workflow of a task
		 * @param {String} taskID 
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR> Promise for the workflowXML
		 */
		WebCenterAPI.prototype.getTaskWorkflowXML = function (taskID) {
			return this.callSDK("DownloadWorkflowXML.jsp", {
				taskid: taskID
			}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._errorHandler.severe(i18n.t("LoadWorkflowFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
			}, this));
		};

		/**
		 * Get the WorkflowDefinitions XML
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise for the WorkflowDefinitions
		 */
		WebCenterAPI.prototype.getWorkflowDefinitions = function () {
			if (!this._workflowDefinitionsPromise) {

				this._workflowDefinitionsPromise = this.callSDK("GetWFActions.jsp", {}, {
					cache: true
				}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
					this._errorHandler.severe(i18n.t("LoadWorkflowDefinitionsFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				}, this));
			}

			return this._workflowDefinitionsPromise;
		};

		/**
		 * Get the TaskTypeDefinitions XML
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise for the TaskTypeDefinitions
		 */
		WebCenterAPI.prototype.getTaskTypeDefinitions = function () {
			return this._taskTypeAPI.getTaskTypeDefinitions();
		};

		/**
		 * Used by the TaskTypeDEfinition Cache to fecth information on a task type
		 * @param {String} taskTypeID - id of the TaskType
		 */
		WebCenterAPI.prototype._loadTaskTypeDefinition = function (taskTypeID) {

			var promise = this.callSDK("GetTaskTypes.jsp", {
				verbose: "1",
				loadicons: "1",
				tasktypeid: taskTypeID
			}, {
				cache: true
			});

			promise.fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._errorHandler.severe(i18n.t("LoadTaskTypesFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				this._taskTypeDefinitionCache.remove(taskTypeID);
			}, this));

			this._taskTypeDefinitionCache._registerItem(taskTypeID, promise);
		};

		/**
		 * Get the promise for fetching the data of the current task type, includes the task statuses
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise for tasktype details
		 */
		WebCenterAPI.prototype.getTaskTypeDefinition = function (taskTypeID) {
			return this._taskTypeDefinitionCache.get(taskTypeID);
		};

		/**
		 * Get the latest version of the WorkflowReport
		 * @param {String} taskID - id of the task for which the report should be fetched
		 *                        @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise for the workflow report
		 */
		WebCenterAPI.prototype.getWorkflowReport = function (taskID) {
			if (taskID) {
				var deferred = $.Deferred();
				this.callSDK("DownloadWorkflowReportXML.jsp", {
					taskid: taskID,
				}).fail(_.bind(function (jqXHR, textStatus, errorThrown, codeString, xml) {
					if (codeString === "154") {
						// there is just no report (e.g. still in Created status), let the feedback handle it
						deferred.resolve(xml, textStatus, jqXHR);
					} else {
						deferred.reject(jqXHR, textStatus, errorThrown);
						this._errorHandler.severe(i18n.t("LoadWorkflowReportFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
					}
				}, this)).done(function (xml, textStatus, jqXHR) {
					deferred.resolve(xml, textStatus, jqXHR);
				});
				return deferred.promise();
			} else {
				return null;
			}
		};

		/**
		 * Keep the session alive in WebCenter
		 * @param {Boolean} keepAlive - true: start keeping session alive, false: stop polling for session alive
		 */
		WebCenterAPI.prototype.keepSessionAlive = function (keepAlive) {
			if (!keepAlive && this._keepSessionAlive !== null) {
				clearInterval(this._keepSessionAlive);
				this._keepSessionAlive = null;
			} else if (keepAlive && this._keepSessionAlive === null) {
				var sessionAliveUrl = this.constructURL("sessionalive.jsp", {});


				this._keepSessionAlive = setInterval(_.bind(function () {
					$.ajax(sessionAliveUrl, {
						dataType: "xml",
						cache: true
					}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
						this._errorHandler.severe(i18n.t("WCSessionLostError") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
					}, this));
				}, this), 300000); // every 5 minutes
			}
		};


		/**
		 * Save a workflow
		 * @param {workflow} the workflow to save
		 * @returns {$.Deferred.<>} resolves when workflow is succesfully saved
		 */
		WebCenterAPI.prototype.saveWorkflow = function (workflow) {
			var deferred = $.Deferred();

			if (!this._saveOngoing) {
				this._saveOngoing = true;
				LoadingIndicator.start("WFEditorWCRSaveWorkflow", i18n.t("savingWorkflowDefinition"));

				var workflowXML = workflow.toXML();

				// generate workflow xml and submit it
				$.ajax(HTTPUtils.constructURL("UploadWorkflowXML.jsp", {}), {
					method: "POST",
					data: {
						tasktypeid: workflow.getTaskTypeID(),
						workflowxmlstring: workflowXML,
						dataType: "xml"
					}
				}).done(_.bind(function (xml) {
					var message;

					// request was succesful, check if the save was successful
					var result = XMLUtils.xml2obj(xml);
					if (result && result.task_type_id === workflow.getTaskTypeID()) {
						//workflow correctly saved
						deferred.resolve();
					} else {
						// error while saving... Concatenate all error information
						if (result.error) {
							var errors;
							if (_.isArray(result.error)) {
								errors = result.error;
							} else {
								errors = [result.error];
							}

							var errorCodes = [];
							var errorMessages = [];
							_.each(errors, function (error) {
								errorCodes.push(error.code);
								errorMessages.push(error.message);
							});

							message = "(" + errorCodes.join(", ") + ")";
							message += " " + errorMessages.join(", ");
						}

						// trigger error
						this._errorHandler.severe(i18n.t("workflowSaveError", message), result);

						deferred.reject();
					}
				}, this)).fail(_.bind(function (xml, textStatus, errorThrown) {
					// error while saving
					this._errorHandler.severe(i18n.t("workflowSaveError", textStatus), errorThrown);

					deferred.reject();
				}, this)).always(_.bind(function () {
					// update save button
					this._saveOngoing = false;
					LoadingIndicator.stop("WFEditorWCRSaveWorkflow");
				}, this));
			} else {
				deferred.reject();
			}

			return deferred.promise();
		};

		/**
		 * Is a save operation ongoing?
		 * @returns {Boolean} 
		 */
		WebCenterAPI.prototype.isSaveOngoing = function () {
			return this._saveOngoing;
		};

		/**
		 * Hold the workflow, this pauses the workflow. Only possible if the workflow is running.
		 * @returns {$.Deferred.<>} resolves when workflow is held
		 */
		WebCenterAPI.prototype.holdWorkflow = function (workflow) {
			var result = $.Deferred();

			var promise = this._getWFControlsPromise(WebCenterWFEditorConstants.WorkflowControlActions.HOLD, workflow.getTaskID());

			promise.fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._errorHandler.severe(i18n.t("holdWorkflowFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				result.reject();
			}, this)).done(_.bind(function (xmlResponse) {
				//first check if the hold call was successful
				var response = XMLUtils.xml2obj(xmlResponse);
				if (response && response.success == "1") {
					result.resolve();
				} else {
					this._errorHandler.severe(i18n.t("holdWorkflowFailed"));
					result.reject();
				}
			}, this));

			return result.promise();
		};

		/**
		 * Resume the workflow, only possible if the workflow is paused
		 * @returns {$.Deferred.<>} resolves when workflow is resumed
		 */
		WebCenterAPI.prototype.resumeWorkflow = function (workflow) {
			var result = $.Deferred();
			var promise = this._getWFControlsPromise(WebCenterWFEditorConstants.WorkflowControlActions.RESUME, workflow.getTaskID());

			promise.fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._errorHandler.severe(i18n.t("resumeWorkflowFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				result.reject();
			}, this)).done(_.bind(function (xmlResponse) {
				var response = XMLUtils.xml2obj(xmlResponse);
				if (response && response.success == "1") {
					result.resolve();
				} else {
					this._errorHandler.severe(i18n.t("resumeWorkflowFailed"));
					result.reject();
				}
			}, this));

			return result;
		};

		/**
		 * Cancel the workflow, only possible if the workflow is paused or in progress.
		 * @returns {$.Deferred.<>} resolves when workflow is cancelled
		 */
		WebCenterAPI.prototype.cancelWorkflow = function (workflow) {
			var result = $.Deferred();
			var promise = this._getWFControlsPromise(WebCenterWFEditorConstants.WorkflowControlActions.CANCEL, workflow.getTaskID());
			workflow.setWorkflowState(WebCenterWorkflowState.STOPPING);
			workflow.setTaskStatus({
				status: i18n.t("workflowStatusStopping")
			});

			promise.fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._errorHandler.severe(i18n.t("cancelWorkflowFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				result.reject();
			}, this)).done(_.bind(function (xmlResponse) {
				var response = XMLUtils.xml2obj(xmlResponse);
				if (response && response.success == "1") {
					result.resolve();
				} else {
					this._errorHandler.severe(i18n.t("cancelWorkflowFailed"));
					result.reject();
				}
			}, this));

			return result.promise();
		};

		/**
		 * Get the WF control action promise for the given action
		 * @param   {WebCenterWFEditorConstants.WorkflowControlActions} action
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>}
		 */
		WebCenterAPI.prototype._getWFControlsPromise = function (action, taskID) {
			return this.callSDK("GetWFActions.jsp", {
				taskid: taskID,
				action: "updatewfstate",
				state: action
			});
		};

		/**
		 * Get the URL to open the viewer for a document
		 * @param   {String} documentVersionID - document version ID of the document to view
		 * @returns {String} URL to open viewer
		 */
		WebCenterAPI.prototype.getViewDocumentURL = function (documentVersionID) {
			return this.constructURL("viewdocument.jsp", {
				docVerID: documentVersionID
			});
		};

		/**
		 * Retry a certain node
		 * @param {Workflow} workflow - the workflow
		 * @param {WebCenterToken}    token    - token which should be retried
		 */
		WebCenterAPI.prototype.retryToken = function (workflow, token) {

			var retryPromise = this.callSDK("RelaunchWorkflow.jsp", {
				taskid: workflow.getTaskID(),
				token: token.getErrorHandlingId(),
				type: "retry"
			}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._errorHandler.severe(i18n.t("wfretry.failed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
			}, this));

			$.when(retryPromise).done(_.bind(function (result) {
				var resultObj = XMLUtils.xml2obj(result);

				if (resultObj && resultObj.taskid === "") {
					this._errorHandler.info(i18n.t("wfretry.success"));
				} else {
					this._errorHandler.warning("Unexpected returned result");
				}
			}, this));

		};


		/**
		 * Relaunch the owrkflow on a certain token
		 * @param   {Workflow}       workflow - workflow to relaunch
		 * @param   {WebCenterToken} token    - token on which the workflow should be relaunched
		 * @returns {$.Deferred.<String>}  resolved with new TaskID when workflow is relaunch
		 */
		WebCenterAPI.prototype.relaunchWorkflow = function (workflow, token) {
			var deferred = $.Deferred();


			if (token && _.size(token.getParents()) === 1 && token.getParents()[0].getId()) {
				var retryPromise = this.callSDK("RelaunchWorkflow.jsp", {
					taskid: workflow.getTaskID(),
					token: token.getParents()[0].getId(),
					type: "relaunch"
				}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
					this._errorHandler.severe(i18n.t("wfrelaunch.failed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
					deferred.reject();
				}, this));

				$.when(retryPromise).done(_.bind(function (result) {
					var resultObj = XMLUtils.xml2obj(result);

					if (resultObj && resultObj.taskid) {
						deferred.resolve(resultObj.taskid);
					} else {
						this._errorHandler.warning("Unexpected returned result");
						deferred.reject();
					}
				}, this));

			} else {
				this._errorHandler.severe(i18n.t("wfrelaunch.failed") + " Token not found.");
				deferred.reject();
			}

			return deferred.promise();
		};

		/**
		 * Check if a document is viewable
		 * @param {String}   documentVersionID - document version ID
		 * @param {Function} callback          - callback called when document viewable status is known
		 */
		WebCenterAPI.prototype.isDocumentViewable = function (documentVersionID, callback) {
			// only load the DocVersionViewableCache when needed
			requirejs(["WFEditorWCR/data/DocVersionViewableCache"], _.bind(function (DocVersionViewableCache) {
				if (!this._docVersionViewableCache) {
					this._docVersionViewableCache = new DocVersionViewableCache(this, this._errorHandler);
				}

				this._docVersionViewableCache.get(documentVersionID, callback);
			}, this));
		};

		/**
		 * Get all WebCenter attribute category definitions
		 * @returns {$.Deferred.<Map.<WebCenterAttributeCategory>} Promise for the webcenter attribute categories
		 */
		WebCenterAPI.prototype.getAttributeCategoryList = function () {
			if (!this._deferredAttributeCategoriesPromise) {
				var deferred = $.Deferred();
				this._deferredAttributeCategoriesPromise = deferred.promise();

				if (!this._attributeCategoriesPromise) {

					this._attributeCategoriesPromise = this.callSDK("GetAttributeCategoryList.jsp", {}, {
						cache: true
					}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
						this._errorHandler.severe(i18n.t("LoadAttributeCategoriesFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
						deferred.reject();
					}, this));
				}

				$.when(this._attributeCategoriesPromise).done(function (result) {
					var attributeCategoriesMap = new Map();
					//Add an option for "No Attribute Category" in the list
					attributeCategoriesMap.add("", new WebCenterAttributeCategory("", i18n.t('wfui.noattcat'), ""));

					if (result && result.documentElement) {
						var rootNode = result.documentElement;
						var id, name, description;

						var attCats = rootNode.getElementsByTagName("category");
						for (var i = 0; i < attCats.length; i++) {
							var attCat = attCats.item(i);

							id = attCat.getAttribute("id");
							name = attCat.getElementsByTagName("name")[0];
							description = attCat.getElementsByTagName("description")[0];

							if (id && name) {
								attributeCategoriesMap.add(id, new WebCenterAttributeCategory(id, name.textContent, description.textContent));
							}
						}
					}

					deferred.resolve(attributeCategoriesMap);
				});
			}
			return this._deferredAttributeCategoriesPromise;
		};

		/**
		 * Get all WebCenter attribute definitions
		 * @returns {$.Deferred.<Map.<WebCenterAttribute>} Promise for the webcenter attributes
		 */
		WebCenterAPI.prototype.getAttributes = function () {
			return this._attributesAPI.getAttributes();
		};


		/**
		 * Get all WebCenter specification definitions
		 * @returns {$.Deferred.<Map.<WebCenterAttribute>} Promise for the webcenter specifiations
		 */
		WebCenterAPI.prototype.getSpecifications = function () {
			return this._attributesAPI.getSpecifications();
		};

		/**
		 * Get all document reference attributes
		 * @returns {$.Deferred.<Map.<id, WebCenterAttribute>>}
		 */
		WebCenterAPI.prototype.getDocRefAttributes = function () {
			return this._attributesAPI.getDocRefAttributes();
		};

		/**
		 * Get task specifications
		 * @param {String} taskTypeID - The task type ID to get the specifications from
		 * @returns {$.Deferred.<Map.<WebCenterAttribute>>} Promise for the specifications
		 */
		WebCenterAPI.prototype.getTaskSpecifications = function (taskTypeID) {
			return this._taskTypeSpecificationsCache.get(taskTypeID);
		};

		/**
		 * Load specifications of a task type
		 * @param {String} taskTypeID - Id of the task type
		 */
		WebCenterAPI.prototype._loadTaskTypeSpecifications = function (taskTypeID) {
			var deferred = $.Deferred();

			$.when(this.getTaskTypeDefinition(taskTypeID), this.getSpecifications()).done(function (taskTypeDefinition, attributes) {
				var taskTypeDefinitionXML = taskTypeDefinition[0];
				var specifications = new Map();
				var specificationViews = taskTypeDefinitionXML.querySelectorAll("wctask_type_view[id='" + taskTypeID + "'] specification_list task_type_specification_view");
				for (var i = 0; i < specificationViews.length; i++) {
					var spec = specificationViews[i].querySelector("task_type_specification");
					var attrid = spec.querySelector("attr_id").textContent;
					var attribute = attributes.get(attrid);
					specifications.add(attrid, attribute);
				}
				deferred.resolve(specifications);
			});

			var promise = deferred.promise();

			this._taskTypeSpecificationsCache._registerItem(taskTypeID, promise);
		};

		/**
		 * Get the users groups and roles
		 * @returns {$.Deferred.<Map.<WebCenterUser>, Map.<WebCenterGroup>, Map.<WebCenterRole>>} Promise for users, groups and roles.
		 */
		WebCenterAPI.prototype.getUsersGroupsRoles = function () {
			if (!this._deferredUsersGroupsRolesPromise) {
				var deferred = $.Deferred();
				this._deferredUsersGroupsRolesPromise = deferred.promise();

				if (!this._usersGroupsRolesPromise) {

					this._usersGroupsRolesPromise = this.callSDK("GetPotentialProjectMembers.jsp", {
						fullname: 0
					}, {
						cache: true
					}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
						this._errorHandler.severe(i18n.t("LoadUsersGroupsRolesFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
						deferred.reject();
					}, this));
				}


				$.when(this._usersGroupsRolesPromise).done(function (result) {
					var users = new Map();
					var groups = new Map();
					var roles = new Map();

					if (result && result.documentElement) {
						var rootNode = result.documentElement;

						var entries = rootNode.getElementsByTagName("entry");
						for (var i = 0; i < entries.length; i++) {
							var entry = entries.item(i);

							var sort = entry.getElementsByTagName("sort")[0];
							var id = entry.getElementsByTagName("id")[0];
							var name = entry.getElementsByTagName("name")[0];

							if (sort && id && name) {
								id = id.textContent.substring(2); // skip u_, g_
								sort = sort.textContent;
								name = name.textContent;

								if (sort.indexOf("1") === 0) {
									// users
									users.add(id, new WebCenterUser(id, name));
								} else if (sort.indexOf("2") === 0) {
									// roles
									roles.add(id, new WebCenterRole(id, name));
								} else if (sort.indexOf("3") === 0) {
									// groups
									groups.add(id, new WebCenterGroup(id, name));
								}
							}
						}

					}
					deferred.resolve(users, groups, roles);
				});
			}

			return this._deferredUsersGroupsRolesPromise;
		};

		/**
		 * Get the list of Project Statuses installed on the system
		 * @returns {$.Deferred.<Map.<WebCenterProjectStatus>>} promise for the project status list
		 */
		WebCenterAPI.prototype.getProjectStatusList = function () {
			var promise = this._projectStatusListPromise;
			if (!promise) {
				var statusListDeferred = $.Deferred();

				this.getWorkflowDefinitions().done(function (wfactions) {
					requirejs(["WFEditorWCR/data/WebCenterProjectStatus"], function (WebCenterProjectStatus) {
						// parse xml for statuses and resolve the result promise
						var projectStatusXML = wfactions.getElementsByTagName("ProjectStatuses");
						var statusList = new Map();
						if (projectStatusXML.length > 0) {
							_.each(projectStatusXML[0].getElementsByTagName("ProjectStatus"), function (statusXML) {
								var status = WebCenterProjectStatus.fromXML(statusXML);
								statusList.add(status.getId(), status);
							});

							statusListDeferred.resolve(statusList);
						}

						statusListDeferred.fail();
					});
				}).fail(function () {
					statusListDeferred.fail.apply(statusListDeferred, arguments);
				});

				promise = statusListDeferred.promise();
				this._projectStatusListPromise = promise;

				statusListDeferred.fail(_.bind(function () {
					this._projectStatusListPromise = null;
				}, this));
			}

			return promise;
		};


		/**
		 * Get the list of available Workflow Connections
		 * @returns {$.Deferred.<Map.<WebCenterWorkflowConnection>>} list of workflow connections
		 */
		WebCenterAPI.prototype.getWorkflowConnections = function () {
			var promise = this._workflowConnectionsPromise;
			if (!promise) {
				var workflowConnectionsDeferred = $.Deferred();

				this.getWorkflowDefinitions().done(function (wfactions) {
					requirejs(["WFEditorWCR/data/WebCenterWorkflowConnection"], function (WebCenterWorkflowConnection) {
						// parse xml for statuses and resolve the result promise
						var workflowsXML = wfactions.getElementsByTagName("Workflows");
						var connectionList = new Map();
						if (workflowsXML.length > 0) {
							_.each(workflowsXML[0].getElementsByTagName("Workflow"), function (workflowXML) {
								var connection = WebCenterWorkflowConnection.fromXML(workflowXML);
								connectionList.add(connection.getId(), connection);
							});

							workflowConnectionsDeferred.resolve(connectionList);
						}

						workflowConnectionsDeferred.fail();
					});
				}).fail(function () {
					workflowConnectionsDeferred.fail.apply(workflowConnectionsDeferred, arguments);
				});

				promise = workflowConnectionsDeferred.promise();
				this._workflowConnectionsPromise = promise;

				workflowConnectionsDeferred.fail(_.bind(function () {
					this._workflowConnectionsPromise = null;
				}, this));
			}

			return promise;
		};

		/**
		 * Get the list of JMF Workflow Connections
		 * @returns {$.Deferred.<Map.<WebcenterWorkflowConnection>>} 
		 */

		WebCenterAPI.prototype.getWorkflowConnectionsJMF = function () {
			var deferred = $.Deferred();

			this.getWorkflowConnections().done(function (list) {
				var map = new Map();
				_.each(list.asObject(), function (connection, id) {
					if (connection.isJMF()) {
						map.add(id, connection);
					}
				});

				deferred.resolve(map);

			}).fail(function () {
				deferred.reject();
			});

			return deferred.promise();
		};

		/**
		 * Get the list of status for a certain task type
		 * @param   {String} taskTypeID - task type ID for which statuses should be fetched
		 * @returns {$.Deferred.<Map.<WebCenterTaskStatus>>} 
		 */
		WebCenterAPI.prototype.getWorkflowTaskStatusList = function (taskTypeID) {
			var statusListDeferred = $.Deferred();
			this.getTaskTypeDefinition(taskTypeID).done(function (xml) {

				requirejs(["WFEditorWCR/data/WebCenterTaskStatus"], function (WebCenterTaskStatus) {
					var statusList = new Map();

					var statusListXML = xml.getElementsByTagName("status_list");
					if (statusListXML.length > 0) {
						_.each(statusListXML[0].getElementsByTagName("task_type_status_view"), function (statusXML) {
							var status = WebCenterTaskStatus.fromXML(statusXML);
							statusList.add(status.getId(), status);
						});
					}

					statusListDeferred.resolve(statusList);
				});
			}).fail(function () {
				statusListDeferred.fail();
			});

			return statusListDeferred.promise();

		};

		/**
		 * Clear the workflow with the given task type id
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise of the call to clear workflow
		 */
		WebCenterAPI.prototype.clearWorkflow = function (taskTypeID) {
			return this.callSDK("UploadWorkflowXML.jsp", {
				tasktypeid: taskTypeID,
				clear: 1
			}).fail(_.bind(function (jqXHR, textStatus, errorThrown, codeString) {
				this._errorHandler.severe(i18n.t("workflowClearFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
			}, this));
		};

		/**
		 * Get the text content element type list
		 * @returns {$.Deferred.<Map.<WebCenterTextContentElementType>} Promise for text content element types
		 */
		WebCenterAPI.prototype.getTextContentElementTypeList = function () {
			if (!this._textContentElementTypesPromise) {
				var deferredTCElementTypeRequest = $.Deferred();
				$.ajax(this.constructURL("GetRowsFromList.jsp", {
					listType: "WCR_DB_List",
					indexed: "1"
				}), {
					dataType: "text",
					type: "GET",
					data: {
						outputType: "JSON",
						list: "ELEMENT_TYPES",
						maxrows: 200,
						maxcolrows: 200
					}
				}).done(_.bind(function (jsonContent, textStatus, jqXHR) {
					this._parseContentElementResponse(jsonContent, deferredTCElementTypeRequest, false);
				}, this)).fail(function (jqXHR, textStatus, errorThrown) {
					deferredTCElementTypeRequest.reject(jqXHR, textStatus, errorThrown);
				});

				this._textContentElementTypesPromise = deferredTCElementTypeRequest.promise();
			}

			return this._textContentElementTypesPromise;
		};

		/**
		 * Get the table content element type list
		 * @returns {$.Deferred.<Map.<WebCenterTextContentElementType>} Promise for table content element types
		 */
		WebCenterAPI.prototype.getTableContentElementTypeList = function () {
			if (!this._tableContentElementTypesPromise) {
				var deferredTCElementTypeRequest = $.Deferred();
				$.ajax(this.constructURL("GetRowsFromList.jsp", {
					listType: "WCR_DB_List",
					indexed: "1"
				}), {
					dataType: "text",
					type: "GET",
					data: {
						outputType: "JSON",
						list: "TABLE_ELEMENT_TYPES",
						maxrows: 200,
						maxcolrows: 200
					}
				}).done(_.bind(function (jsonContent, textStatus, jqXHR) {
					this._parseContentElementResponse(jsonContent, deferredTCElementTypeRequest, true);
				}, this)).fail(function (jqXHR, textStatus, errorThrown) {
					deferredTCElementTypeRequest.reject(jqXHR, textStatus, errorThrown);
				});

				this._tableContentElementTypesPromise = deferredTCElementTypeRequest.promise();
			}

			return this._tableContentElementTypesPromise;
		};

		/**
		 * Parses the reponse received for content element type request and sets the result in a deffered Object
		 * 
		 * @param {JSON} jsonContent - JSON response received from element type list request
		 * @param {$.Deferred} deferredTCElementTypeRequest - deferred JQueryObj to resolve the result list
		 * @param {Boolean} isTableContent - Flag to specify whether the request is for TextContent or TableContent
		 */
		WebCenterAPI.prototype._parseContentElementResponse = function (jsonContent, deferredTCElementTypeRequest, isTableContent) {
			var cachedElementTypes = new Map();

			if (!isTableContent) {
				//Add the default element type (--- All Element Types --)
				var defElementType = new WebCenterTextContentElementType('ALL_ELEMENT_TYPES', i18n.t('wfui.multidynamiccontentmap.elementtypedefaultlabel'));
				cachedElementTypes.add('ALL_ELEMENT_TYPES', defElementType);
			}

			var jsonObj = JSON.parse(jsonContent);
			if (!jsonObj.success || jsonObj.success == "0") {
				deferredTCElementTypeRequest.fail();
			} else {
				var elementTypesValues = jsonObj.columns[0].values;
				var humanReadableValues = jsonObj.columns[1].values;
				var elementCategoryValues = jsonObj.columns[2].values;
				var combinations = jsonObj.combinations;

				_.each(combinations, _.bind(function (value) {
					var combinationsArr = value.split('~');
					var indexElemType = combinationsArr[0];
					var indexHumanElem = combinationsArr[1];
					var indexElemCat = combinationsArr[2];

					var elementType = new WebCenterTextContentElementType(elementTypesValues[indexElemType], humanReadableValues[indexHumanElem], elementCategoryValues[indexElemCat]);
					cachedElementTypes.add(elementTypesValues[indexElemType], elementType);
				}, this));
				deferredTCElementTypeRequest.resolve(cachedElementTypes);
			}
		};

		/**
		 * Get the text content language list
		 * @returns {$.Deferred.<Map.<WebCenterTextContentLanguage>} Promise for text content element types
		 */
		WebCenterAPI.prototype.getTextContentLanguageList = function () {

			if (!this._textContentLangListPromise) {
				var deferredTCLangListRequest = $.Deferred();

				//Need to fetch both master and translation languages
				languageService.getBothAllLanguages(_.bind(function (masterLangs, transLangs) {
					var masterLangMap = new Map();
					var transLangMap = new Map();
					var combinedLangMap = new Map();
					var langMap = new Map();

					//Master Languages
					_.each(masterLangs, function (langObj) {
						masterLangMap.add(langObj.langCode, new WebCenterTextContentLanguage(langObj.langCode, langObj.langDesc, langObj.langFlag, langObj.langIndex));
					});
					//Translation Languages
					_.each(transLangs, function (langObj) {
						transLangMap.add(langObj.langCode, new WebCenterTextContentLanguage(langObj.langCode, langObj.langDesc, langObj.langFlag, langObj.langIndex));
					});

					// Merge master and translation languages together.
					var combinedLangs = LanguageUtils.mergeBothLanguagesTogether(masterLangs, transLangs);
					_.each(combinedLangs.values(), function (langObj) {
						combinedLangMap.add(langObj.langCode, new WebCenterTextContentLanguage(langObj.langCode, langObj.langDesc, langObj.langFlag, langObj.langIndex));
					});

					langMap.add(ContentConstants.LanguageType.MASTER, masterLangMap);
					langMap.add(ContentConstants.LanguageType.TRANSLATION, transLangMap);
					langMap.add(ContentConstants.LanguageType.ALL, combinedLangMap);
					deferredTCLangListRequest.resolve(langMap);
				}, this));

				this._textContentLangListPromise = deferredTCLangListRequest.promise();
			}

			return this._textContentLangListPromise;

		};

		return WebCenterAPI;
	});