define(["component/dropdownmenu/DropdownMenu",
		"WFEditorWCR/workflow/WebCenterWorkflowState",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"utils/webi18n!WFEditorWCR",
		"utils/HTTPUtils",
		"utils/XMLUtils",
		"jquery",
		"underscore"
	],
	function (DropdownMenu,
		WebCenterWorkflowState,
		WebCenterPinConstants,
		WebCenterNodeConstants,
		i18n,
		HTTPUtils,
		XMLUtils,
		$,
		_) {
		"use strict";

		/**
		 * This class is a wrapper around a dropdown menu to show a custom context menu of a document in a token
		 *
		 * @class
		 * @exports WebCenterDocumentContextMenu
		 *
		 * @param {Scene} scene - used to get the controller and to get the context element to attach the anchor to
		 */
		var WebCenterDocumentContextMenu = function (scene) {
			// will be filled in by show
			this._documentVersionID = null;
			this._tokenID = null;
			this._pinModel = null;

			this._scene = scene;

			this._webCenterAPI = scene.getWebCenterAPI();

			this._anchorDiv = $("<div />").addClass("documentContextMenuAnchor"); // the context menu is attached to this element
			scene.getContext().append(this._anchorDiv);

			// note i18n only supports "t" here, not the loadSingleBundle like it should
			this._dropdown = new DropdownMenu(i18n, scene.getContext(), this._webCenterAPI.getWebCenterConfig().isMobile, "right", "wcr-dropdown");

			// View option
			this._dropdown.addAction({
				name: "view",
				content: i18n.t("wfview"),
				enabled: false,
				isVisible: false,
				action: _.bind(function () {
					var viewURL = this._webCenterAPI.getViewDocumentURL(this._documentVersionID);
					window.open(viewURL, "_blank");
				}, this)
			});

			// Retry option
			this._dropdown.addAction({
				name: "retry",
				content: i18n.t("wfretry"),
				enabled: false,
				isVisible: false,
				action: _.bind(function () {
					this._webCenterAPI.retryToken(this._pinModel.getWorkflow(), this._pinModel.getToken(this._tokenID));
				}, this)
			});

			// Relaunch option
			this._dropdown.addAction({
				name: "relaunch",
				content: i18n.t("wfrelaunch"),
				enabled: false,
				isVisible: false,
				action: _.bind(function () {
					var token = this._pinModel.getWorkflow().getToken(this._tokenID);
					this._webCenterAPI.relaunchWorkflow(this._pinModel.getWorkflow(), token).done(_.bind(function (taskID) {
						this._scene.requestNewWorkflow(taskID);
					}, this));
				}, this)
			});
		};

		/**
		 * Show the context menu
		 * @param {WebCenterPin} pinModel - The pin model
		 * @param {String} documentVersionId - The version ID of the document
		 * @param {String} tokenId - The token ID
		 * @param {ShapeUtils.Point} location - Location of the mouseclick
		 */
		WebCenterDocumentContextMenu.prototype.show = function (pinModel, documentVersionId, tokenId, location) {
			if (!tokenId) {
				return;
			}

			this._anchorDiv.css({
				left: location.x,
				top: location.y
			});

			this._documentVersionID = documentVersionId;
			this._tokenID = tokenId;
			this._pinModel = pinModel;

			// View
			if (this._documentVersionID) {
				this._webCenterAPI.isDocumentViewable(this._documentVersionID, _.bind(function (viewableObj) {
					if (viewableObj !== undefined && !viewableObj.supported && !viewableObj.viewable) {
						this._dropdown.hideElement("view");
					} else {
						this._dropdown.showElement("view");
						this._dropdown.setActionEnabled("view", viewableObj && viewableObj.viewable);
						this._dropdown.show(this._anchorDiv);
					}
				}, this));
			} else {
				this._dropdown.hideElement("view");
			}

			var workflow = this._pinModel.getWorkflow();
			var token = workflow.getToken(this._tokenID);

			// Retry
			if (token && (this._pinModel.getPinType() === WebCenterPinConstants.Type.OutError ||
					this._pinModel.getPinType() === WebCenterPinConstants.Type.WorkflowConstructError) && workflow.getPrivilege(WebCenterNodeConstants.Privileges.showWorkflowControls) && workflow.getWorkflowState() == WebCenterWorkflowState.RUNNING) {
				this._dropdown.showElement("retry");
				var enableRetry = false;
				var nextToken = workflow.getToken(token.getErrorHandlingId());
				if (nextToken && !nextToken.hasFutureTokens()) {
					enableRetry = true;
				}
				this._dropdown.setActionEnabled("retry", enableRetry); // retry only enabled if no future tokens 
				this._dropdown.show(this._anchorDiv);
			} else {
				this._dropdown.hideElement("retry");
			}


			// Relaunch
			if (token && workflow.getWorkflowState() == WebCenterWorkflowState.FINISHED && workflow.getPrivilege(WebCenterNodeConstants.Privileges.showWorkflowControls) && workflow.getPrivilege("isMasterTask")) {
				this._dropdown.showElement("relaunch");
				this._dropdown.setActionEnabled("relaunch", _.size(token.getParents()) === 1); // only if there is 1 parent token enable it
				this._dropdown.show(this._anchorDiv);
			} else {
				this._dropdown.hideElement("relaunch");
			}

			// If nothing to show, hide it
			if (!this._dropdown.isElementVisible("view") && !this._dropdown.isElementVisible("retry") && !this._dropdown.isElementVisible("relaunch")) {
				this._dropdown.hide();
			}
		};

		return WebCenterDocumentContextMenu;
	});