define([
		"WFEditor/scene/SceneController",
		"WFEditorWCR/tools/labelEditor/LabelEditorTool",
		"WFEditorWCR/tools/stickyNoteTools/MoveStickyNoteTool",
		"WFEditorWCR/tools/stickyNoteTools/ResizeStickyNoteTool",
		"WFEditorWCR/tools/stickyNoteTools/StickyNoteTextEditorTool",
		"WFEditorWCR/workflow/helper/stickyNote/StickyNoteHelper",
		"WFEditorWCR/tools/WebCenterToolConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditor/workflow/ModelConstants",
		"component/helpIcon/HelpIcon",
		"utils/Command",
		"utils/Observer",
		"underscore"
	],
	function (
		SceneController,
		LabelEditorTool,
		MoveStickyNoteTool,
		ResizeStickyNoteTool,
		StickyNoteTextEditorTool,
		StickyNoteHelper,
		WebCenterToolConstants,
		WebCenterNodeConstants,
		ModelConstants,
		HelpIcon,
		Command,
		Observer,
		_

	) {
		"use strict";

		/**
		 * Controller for the Scene of a webcenter workflow. Extends the SceneController
		 * and adds specific WebCenter functionality to the contoller.
		 *
		 * @exports WebCenterSceneController
		 * @class
		 * @augments SceneController
		 *
		 * @param {AbstractWorkflowFactory} workflowFactory - the workflow factory to create objects
		 * @param {SceneModel} sceneModel
		 * @param {Workflow} workflowModel
		 * @param {Scene} scene
		 * @param {WFEditorContext} WFEditorContext
		 */
		var WebCenterSceneController = function (workflowFactory, sceneModel, workflowModel, scene, WFEditorContext) {
			SceneController.call(this, workflowFactory, sceneModel, workflowModel, scene, WFEditorContext);
			Observer.mixin(this);

			// init label editor tool
			new LabelEditorTool(this._mouseInteractionTool, WFEditorContext);
			if (this._workflowModel.getPrivilege(WebCenterNodeConstants.Privileges.editStickyNotes)) {
				new StickyNoteTextEditorTool(this._mouseInteractionTool, WFEditorContext);
				this._contextMenuModel.addComponent(WebCenterToolConstants.ButtonGroups.MISC);
				this._contextMenuModel.addButton(WebCenterToolConstants.ButtonGroups.MISC,
					WebCenterToolConstants.Tools.STICKYNOTE.CREATE,
					WFEditorContext.get("i18n").t("createStickyNote"), "icon-WCR_WF-AddSticky12 icon-normal", _.bind(function (e) {
						this._createStickyNote(this._contextMenuModel.getContextMenuWorldLocation());
					}, this), true);
				this._contextMenuModel.addButton(WebCenterToolConstants.ButtonGroups.MISC,
					WebCenterToolConstants.Buttons.OPENEDITORHELP,
					WFEditorContext.get("i18n").t("openEditorHelp"), "icon-WCR_Info-Balloon icon-normal", _.bind(function (e) {
						var help = HelpIcon.getHelpURL(WebCenterNodeConstants.HelpTopics.EDITOR);
						help.onClickFunction();
					}, this), true);
			}
			new MoveStickyNoteTool(this, this._mouseInteractionTool, WFEditorContext);
			new ResizeStickyNoteTool(this, this._mouseInteractionTool, WFEditorContext);

			if (this._workflowModel.getPrivilege(ModelConstants.Privileges.editNodeConfiguration)) {

				this._initializeNodeConfigurationContextMenuButtons();
			}


			this._observe(this._workflowModel, "Node.*.Pin.*.Token.*.tokenChanged", this._tokenChanged);
		};

		WebCenterSceneController.prototype = Object.create(SceneController.prototype);
		WebCenterSceneController.prototype.constructor = WebCenterSceneController;

		/**
		 * A token got selected in the view, change the correct settings in the model.
		 *
		 * @param {Token} token - the token that got selected
		 * @param {String} documentId - the id of the document in the token that got selected
		 */
		WebCenterSceneController.prototype._tokenItemSelected = function (token, documentId) {
			if (token) {
				token.setSelected(true, documentId); //make sure to add the selected document to the selected state
			}
			SceneController.prototype._tokenItemSelected.call(this, token, documentId);
		};

		/**
		 * A token's status has changed, if it is now processed, remove the view
		 * @param {WebCenterToken} token
		 */
		WebCenterSceneController.prototype._tokenChanged = function (token) {
			if (token.isProcessed()) {
				// Check if this token was highlighted or if it is an operation token, one of its subtokens has been highlighted
				var highlightedToken = this._workflowModel.getHightLightedToken();
				if (highlightedToken && (token === highlightedToken || (token.hasSubToken && token.hasSubToken(highlightedToken)))) {
					// remove the highlights
					this._highlightTokenPath(highlightedToken, false);
					highlightedToken.setSelected(false);
				}
				this._scene.removeViewsForWorkflowItem(token);
			}
		};

		/**
		 * @override
		 * Same as super function, but checks for implicit edges
		 */
		WebCenterSceneController.prototype._doCreateEdge = function (workflow, fromPin, toPin) {
			// accepts the edge and the edge does not yet exist
			if (toPin.acceptsIncomingEdges(fromPin) && !fromPin.getEdgeTo(toPin)) {
				// find next possible id
				var i = 0;
				var checkExistingEdgeId = function (edge) {
					return edge.getId() === fromPin.getId() + "/" + i;
				};
				while (_.find(fromPin.getEdges(), checkExistingEdgeId)) {
					i++;
				}

				var edge = this._workflowFactory.createEdge(workflow, fromPin.getId() + "/" + i, fromPin, toPin);

				var toNode = toPin.getNode();
				if (toNode.isImplicit && toNode.isImplicit()) {
					edge.setImplicit(true);
				}
				fromPin.addEdge(edge);
				toPin.addEdge(edge);
				workflow.addEdge(edge);
			}
		};

		/**
		 * create a sticky note
		 * @param {ShapeUtils.Position} mousePosition 
		 */
		WebCenterSceneController.prototype._createStickyNote = function (mousePosition) {
			var location = mousePosition.copy();
			location.offsetXY(-WebCenterNodeConstants.StickyNote.DEFAULT_WIDTH / 2, 0);
			var stickyNote = new StickyNoteHelper(this._workflowModel, location);
			this._undoRedo.execute(new Command(_.bind(function () {
				this._workflowModel.addHelper(stickyNote);
			}, this), _.bind(function () {
				this._workflowModel.removeHelper(stickyNote);
			}, this), this._i18n.t("createStickyNote")));
		};

		/**
		 * @override
		 * add hostname to object to check for cross server pasting
		 */
		WebCenterSceneController.prototype._getObjectForCopy = function (selectedNodes) {
			var json = SceneController.prototype._getObjectForCopy.call(this, selectedNodes);
			json.wcrHost = location.hostname;
			return json;
		};

		/**
		 * @override
		 * don't allow cross server pasting, there are import/export tools for this
		 */
		WebCenterSceneController.prototype._createNodesFromPaste = function (nodesObject) {
			if (nodesObject.wcrHost === location.hostname) {
				SceneController.prototype._createNodesFromPaste.call(this, nodesObject);
			} else {
				this._errorHandler.warning(this._i18n.t("noCrossServerPaste"));
			}
		};

		/**
		 * opens a warning policy panel for node
		 * @param {WebCenterNode}
		 */
		WebCenterSceneController.prototype._openWarningPolicyPanel = function (node) {
			// Create a proxy that redirects the parameters configuration methods to the warning policy configuration
			var nodeProxy = Object.create(node);
			nodeProxy.getConfigurationPanel = function () {
				var panelPromise = $.Deferred();
				requirejs(["WFEditorWCR/workflow/panel/WarningPanel"], _.bind(function (WarningPanel) {
					var nodeConfigPanel = new WarningPanel(node);
					panelPromise.resolve(nodeConfigPanel);
				}, this));
				return panelPromise.promise();
			};
			nodeProxy.setConfiguration = function (configuration) {
				node.setWarningPolicy(configuration);
			};
			nodeProxy.getConfiguration = function () {
				return node.getWarningPolicy();
			};
			this._nodeConfigurationController.openNodeConfiguration(nodeProxy, "warningpolicy");
		};


		/**
		 * adds the node configuration buttons to the context menu
		 */
		WebCenterSceneController.prototype._initializeNodeConfigurationContextMenuButtons = function () {
			this._contextMenuModel.addComponent(WebCenterToolConstants.ButtonGroups.WEBCENTERNODES);

			// edit parameters
			this._contextMenuModel.addButton(WebCenterToolConstants.ButtonGroups.WEBCENTERNODES,
				WebCenterToolConstants.Buttons.EDITCONFIGURATION,
				this._i18n.t("editParametersContextMenuItem"), "icon-normal icon-WCR_Actions-Edit12",
				_.bind(function (e) {
					this.openNodeConfiguration(this._contextMenuModel.getHitArea().getView().getModel());
				}, this), false, false, false);

			// configure outputs
			this._contextMenuModel.addButton(WebCenterToolConstants.ButtonGroups.WEBCENTERNODES,
				WebCenterToolConstants.Buttons.EDITOUTPUTS,
				this._i18n.t("configureOutputsContextMenuItem"), "icon-normal icon-WCR_Configuration",
				_.bind(function (e) {
					this._openWarningPolicyPanel(this._contextMenuModel.getHitArea().getView().getModel());
				}, this), false, false, false);

			// open node help
			this._contextMenuModel.addButton(WebCenterToolConstants.ButtonGroups.WEBCENTERNODES,
				WebCenterToolConstants.Buttons.OPENNODEHELP,
				this._i18n.t("openNodeHelpContextMenuItem"), "icon-normal icon-WCR_Info-Balloon",
				_.bind(function (e) {
					var node = this._contextMenuModel.getHitArea().getView().getModel();
					var help = HelpIcon.getHelpURL(node.getHelpTopic());
					help.onClickFunction();
				}, this), false, false, false);

			this._observe(this._contextMenuModel, "changed.hitArea", function (hitArea) {
				if (hitArea) {
					var model = hitArea.getView().getModel();
					var isNode = model.getWorkflowItemType() === ModelConstants.WorkflowItemType.Node;
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.EDITCONFIGURATION, isNode);
					this._contextMenuModel.setButtonEnabled(WebCenterToolConstants.Buttons.EDITCONFIGURATION, isNode && model.isConfigurable());
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.EDITOUTPUTS, isNode);
					this._contextMenuModel.setButtonEnabled(WebCenterToolConstants.Buttons.EDITOUTPUTS, isNode && model.canEditWarningPolicy());
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.OPENNODEHELP, isNode);
					this._contextMenuModel.setButtonEnabled(WebCenterToolConstants.Buttons.OPENNODEHELP, isNode && model.getHelpTopic());
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.OPENEDITORHELP, !isNode); // only node help is enough
				} else {
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.EDITOUTPUTS, false);
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.EDITCONFIGURATION, false);
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.OPENNODEHELP, false);
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.OPENEDITORHELP, true);
				}
			});
		};

		/** 
		 * @override
		 */
		WebCenterSceneController.prototype.removeEdges = function (workflow, edges, undoable, skipDo) {
			if (!undoable) {
				SceneController.prototype.removeEdges.call(this, workflow, edges, undoable, skipDo);
			} else {
				// It is possible that some edges cannot be redone, eg. an edge between an output pin and an input pin
				// of the same node. These types of edges can be made with the java editor but are no longer supported in the
				// HTML5 Version.
				var edgesThatCanBeUndone = _.filter(edges, function (edge) {
					return edge.getToPin().acceptsIncomingEdges(edge.getFromPin());
				});
				var edgesThatCannotBeUndone = _.without(edges, edgesThatCanBeUndone);

				if (edgesThatCanBeUndone.length > 0) {
					SceneController.prototype.removeEdges.call(this, workflow, edgesThatCanBeUndone, true, skipDo);
				}
				
				if (edgesThatCannotBeUndone.length > 0) {
					SceneController.prototype.removeEdges.call(this, workflow, edgesThatCannotBeUndone, false, skipDo);
					this._undoRedo.addForceModifiedFlag();
				}
			}

		};



		return WebCenterSceneController;

	});
