define(["WFEditor/scene/AbstractViewFactory",
		"WFEditor/workflow/ModelConstants",
		"utils/webi18n!WFEditorWCR",
		"WFEditorWCR/workflow/node/WebCenterNodeView",
		"WFEditorWCR/workflow/pin/WebCenterPinView",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/edge/WebCenterEdgeView",
		"WFEditorWCR/workflow/token/WebCenterTokenView",
		"WFEditorWCR/workflow/helper/stickyNote/StickyNoteHelper",
		"WFEditorWCR/workflow/helper/stickyNote/StickyNoteView",
		"utils/ExecutionQueue",
		"underscore",
		"jquery"
	],
	function (
		AbstractViewFactory,
		ModelConstants,
		i18n,
		WebCenterNodeView,
		WebCenterPinView,
		WebCenterPinConstants,
		WebCenterEdgeView,
		WebCenterTokenView,
		StickyNoteHelper,
		StickyNoteView,
		ExecutionQueue,
		_,
		$
	) {
		"use strict";

		/**
		 * Implementation of the view factory for WebCenter. It creates all view objects needed
		 * to render the WebCenter workflow editor.
		 *
		 * @exports WebCenterViewFactory
		 * @class
		 * @augments AbstractViewFactory
		 * @param {ErrorHandler} errorHandler
		 */
		var WebCenterViewFactory = function (errorHandler) {
			AbstractViewFactory.call(this);
			
			/**
			 * @member {ErrorHandler}
			 * @private
			 */
			this._errorHandler = errorHandler;
		};

		WebCenterViewFactory.prototype = Object.create(AbstractViewFactory.prototype);
		WebCenterViewFactory.prototype.constructor = WebCenterViewFactory;

		/**
		 * @Override
		 */
		WebCenterViewFactory.prototype.createViewsForWorkflow = function (workflow, scene) {
			var result = $.Deferred();
			var viewsDestination = {};


			var queue = new ExecutionQueue();

			queue.forEach(workflow.getNodes(), function (node) {
				this._createNode(node, viewsDestination, scene, viewsDestination);
			}, this, "ViewFactoryCreateNodes", i18n.t("initializingNodeViews"));

			queue.forEach(workflow.getEdges(), function (edge) {
				this._createEdge(edge, viewsDestination, scene, viewsDestination);
			}, this, "ViewFactoryCreateEdges", i18n.t("initializingConnectionViews"));

			queue.forEach(workflow.getTokens(), function (token) {
				this._createToken(token, viewsDestination, scene, viewsDestination);
			}, this, "ViewFactoryCreateTokens", i18n.t("initializingTokenViews"));

			queue.forEach(workflow.getHelpers(), function (helper) {
				this._createHelper(helper, viewsDestination, scene, viewsDestination);
			}, this, "ViewFactoryCreateHelper");

			queue.whenFinished(function () {
				result.resolve(viewsDestination);
			});

			return result;
		};

		/**
		 * @Override
		 */
		WebCenterViewFactory.prototype.createViewsForWorkflowItem = function (workflowItem, existingViews, scene, viewsDestination) {
			switch (workflowItem.getWorkflowItemType()) {
			case ModelConstants.WorkflowItemType.Edge:
				this._createEdge(workflowItem, existingViews, scene, viewsDestination);
				break;
			case ModelConstants.WorkflowItemType.Node:
				this._createNode(workflowItem, existingViews, scene, viewsDestination);
				break;
			case ModelConstants.WorkflowItemType.Pin:
				this._createPin(workflowItem, existingViews, scene, viewsDestination);
				break;
			case ModelConstants.WorkflowItemType.Token:
				this._createToken(workflowItem, existingViews, scene, viewsDestination);
				break;
			case ModelConstants.WorkflowItemType.Helper:
				this._createHelper(workflowItem, existingViews, scene, viewsDestination);
				break;
			}
		};

		/**
		 *  @Override
		 */
		WebCenterViewFactory.prototype._createNode = function (node, existingViews, scene, viewsDestination) {
			if (!AbstractViewFactory.prototype._createNode.call(this, node, existingViews, scene, viewsDestination)) { // first try super
				if (node.isVisible()) {
					var nodeView = new WebCenterNodeView(node, scene);
					viewsDestination[nodeView.getId()] = nodeView;
					existingViews[nodeView.getId()] = nodeView;

					_.each(node.getPins(), function (pin) {
						this._createPin(pin, existingViews, scene, viewsDestination);
					}, this);
					return true;
				}
				return false;
			}
			return true;
		};

		/**
		 * @Override
		 */
		WebCenterViewFactory.prototype._createPin = function (pin, existingViews, scene, viewsDestination) {
			if (!AbstractViewFactory.prototype._createPin.call(this, pin, existingViews, scene, viewsDestination)) { // first try super
				if (pin.isVisible()) {
					var nodeView = existingViews[pin.getNode().getId()];
					if (nodeView) {
						var pinView = new WebCenterPinView(pin, nodeView, scene);
						viewsDestination[pinView.getId()] = pinView;
						return true;
					}
				}
				return false;
			}
			return true;
		};

		/**
		 * @Override
		 */
		WebCenterViewFactory.prototype._createEdge = function (edge, existingViews, scene, viewsDestination) {
			// super always returns false
			if (edge.isVisible()) {
				var fromPinView = existingViews[edge.getFromPin().getId()];
				var toPinView = existingViews[edge.getToPin().getId()];

				if (fromPinView && toPinView) {
					var edgeView = new WebCenterEdgeView(edge, fromPinView, toPinView);

					viewsDestination[edgeView.getId()] = edgeView;
					return true;
				} else {
					this._errorHandler.severe(i18n.t("WorkflowEdgeViewError"));
					return false;
				}
			}
		};

		/**
		 * @Override
		 */
		WebCenterViewFactory.prototype._createToken = function (token, existingViews, scene, viewsDestination) {
			// super always returns false

			var pinView = existingViews[token.getPin().getId()];
			if (pinView && !token.isProcessed() && pinView.getModel().getPinType() === WebCenterPinConstants.Type.In) {
				var tokenView = new WebCenterTokenView(token, pinView, scene);
				viewsDestination[tokenView.getId()] = tokenView;
				return true;
			}
			return false;
		};

		/**
		 * @override
		 */
		WebCenterViewFactory.prototype._createHelper = function (helper, existingViews, scene, viewsDestination) {
			if (!AbstractViewFactory.prototype._createHelper.call(this, helper, existingViews, scene, viewsDestination)) {
				if (helper instanceof StickyNoteHelper) {
					var stickyNoteView = new StickyNoteView(helper, scene);
					viewsDestination[helper.getId()] = stickyNoteView;
					return true;
				}
				return false;
			}
			return true;
		};

		return WebCenterViewFactory;
	});
