define([
		"WFEditor/toolbar/Toolbar",
		"component/toolbar/ButtonGroup",
		"component/toolbar/ToolbarButton",
		"component/toolbar/ToolbarComponent",
		"WFEditor/tools/ToolConstants",
		"WFEditorWCR/core/WebCenterWFEditorConstants",
		"WFEditorWCR/toolbar/StatusToolbarComponent",
		"utils/webi18n!WFEditorWCR"
	],
	function (
		Toolbar,
		ButtonGroup,
		ToolbarButton,
		ToolbarComponent,
		ToolConstants,
		WebCenterWFEditorConstants,
		StatusToolbarComponent,
		i18n //only t is supported never provide langBundle from here!
	) {
		"use strict";

		/**
		 * View for the toolbar with all its components for WebCenter.
		 *
		 * @exports WebCenterToolbar
		 * @class
		 * @augments Toolbar
		 *
		 * @param {WebCenterToolbarModel} toolbarModel
		 * @param {JQueryObj} context
		 * @param {JQueryObj} globalContext - used to create buttons in
		 */
		var WebCenterToolbar = function (toolbarModel, context, globalContext) {
			/**
			 * @member {ToolbarComponent} the toolbar compoment where all the button groupd with buttons are added to (left)
			 * @private
			 */
			this._leftToolBar = new ToolbarComponent("WFEditorToolbarLeft");

			/**
			 * @member {ToolbarComponent} the toolbar compoment where all the button groupd with buttons are added to (right)
			 * @private
			 */
			this._rightToolBar = new ToolbarComponent("WFEditorToolbarRight");

			/**
			 * @member {JQueryObj}
			 * @private
			 * The global context to which the toolbar button tooltips will be added
			 */
			this._globalContext = globalContext;

			/**
			 * @member {Array.<String>} order of toolbar groups that are added to the left toolbar
			 * @private
			 */
			this._leftOrder = [ToolConstants.ButtonGroups.ZOOM,
				ToolConstants.ButtonGroups.STEPTHROUGH,
				ToolConstants.ButtonGroups.NAVIGATION,
				WebCenterWFEditorConstants.WorkflowControlToolbar,
				ToolConstants.ButtonGroups.UNDOREDO,
				WebCenterWFEditorConstants.WorkflowStatusToolbar,
				ToolConstants.ButtonGroups.REMOVENODES
			];

			/**
			 * @member {Array.<String>} order of toolbar groups that are added to the right toolbar
			 * @private
			 */
			this._rightOrder = [
				WebCenterWFEditorConstants.TaskNameToolbarComponent,
				WebCenterWFEditorConstants.WorkflowActionsToolbar,
				WebCenterWFEditorConstants.WorkflowWFModesToolbar
			];



			// call the constructor of Toolbar
			Toolbar.call(this, toolbarModel, context);


			//append the toolbar to the toolbar context
			this._context.append(this._leftToolBar.element);
			this._context.append(this._rightToolBar.element.addClass("toolbarRight"));

		};

		WebCenterToolbar.prototype = Object.create(Toolbar.prototype);
		WebCenterToolbar.prototype.constructor = WebCenterToolbar;

		/**
		 * Add a toolbar component to the toolbar.
		 * @param {String} componentId -  id of the component
		 */
		WebCenterToolbar.prototype._addComponent = function (componentId) {
			if (this._toolbarComponents[componentId] === undefined) {

				var right = true;
				var order = _.indexOf(this._leftOrder, componentId);
				if (order < 0) {
					order = _.indexOf(this._rightOrder, componentId);
				} else {
					right = false;
				}
				var componentData;
				var component;
				if (componentId === WebCenterWFEditorConstants.WorkflowStatusToolbar) {
					componentData = this._model.getComponent(componentId);
					var statusData = componentData.data;
					if (statusData) {
						component = new StatusToolbarComponent(statusData.icon, statusData.status, this._globalContext);
						this._toolbarComponents[componentId] = component;
					}

				} else if (componentId === WebCenterWFEditorConstants.TaskNameToolbarComponent) {
					var element = $("<div/>").addClass("taskNameToolbarComponent enableTextSelection").append($("<span/>").text(this._model.getComponent(componentId).data));
					component = {
						element: element
					};
					
					this._toolbarComponents[componentId] = component;

				} else {
					componentData = this._model.getComponent(componentId);
					component = new ButtonGroup(componentId);
					if (componentData && componentData.title) { //only add title if it is defined in the component data
						component.setTitle(i18n, undefined, componentData.title);
					}

					this._toolbarComponents[componentId] = component;
				}
				if (right) {
					this._rightToolBar.addObject(component, order);
				} else {
					this._leftToolBar.addObject(component, order);
				}

			}
		};

		/**
		 * Add a button to the given toolbar component, the component is added if it doesn't exist yet.
		 * @param {String} componentId
		 * @param {String} buttonId
		 */
		WebCenterToolbar.prototype._addButton = function (componentId, buttonId) {
			if (this._toolbarButtons[buttonId] === undefined) {
				if (this._toolbarComponents[componentId] === undefined) {
					this._addComponent(componentId);
				}
				var buttonData = this._model.getButtonData(buttonId);
				if (buttonData) {
					var icon = buttonData.icon;
					if (icon && !_.isString(buttonData.icon) && buttonData.icon.icon && buttonData.icon.name) {
						icon = $("<div>").text(buttonData.icon.name);
					}
					if (buttonData.content) {
						// custom content for the button
						icon = buttonData.content;
					}
					var button = new ToolbarButton(i18n, this._globalContext, false, buttonData.id, icon, false, buttonData.action, buttonData.tooltipText);

					// Prevent the workflow editor to lose focus on mousedown
					button.element.on("mousedown", function (e) {
						e.preventDefault();
					});

					if (buttonData.cssClass) {
						button.addClass(buttonData.cssClass);
					}
					this._toolbarButtons[buttonId] = button;
					button.setEnabled(buttonData.enabled);
					this._toolbarComponents[componentId].addButton(button);

				}
			}
		};

		/**
		 * Update the button with given id.
		 * @param {String} buttonId
		 */
		WebCenterToolbar.prototype._updateButton = function (buttonId) {
			if (this._toolbarButtons[buttonId]) {
				var buttonModel = this._model.getButtonData(buttonId);
				this._toolbarButtons[buttonId].setActive(buttonModel.active);
				this._toolbarButtons[buttonId].setEnabled(buttonModel.enabled);
				this._toolbarButtons[buttonId].setTooltip(buttonModel.tooltipText);
				if (!buttonModel.content) {
					this._toolbarButtons[buttonId].setIcon(buttonModel.icon + " icon-normal");
				}
			}
		};

		/**
		 * Update a component of the toolbar
		 * @param {String} componentId
		 */
		WebCenterToolbar.prototype._updateComponent = function (componentId) {
			if (this._toolbarComponents[componentId]) {
				var componentData = this._model.getComponent(componentId);
				var data = componentData.data;
				if (data && this._toolbarComponents[componentId].updateView) {
					this._toolbarComponents[componentId].updateView(data);
				}

			}
		};

		return WebCenterToolbar;
	});
