define(
	[
		"WFEditor/tools/AbstractTool",
		"utils/Observer",
		"WFEditor/workflow/ModelConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/tools/WebCenterToolConstants",
		"WFEditorWCR/tools/labelEditor/NodeLabelInputField",
		"WFEditor/tools/ToolConstants",
		"underscore"
	],
	function (
		AbstractTool,
		Observer,
		ModelConstants,
		WebCenterNodeConstants,
		WebCenterToolConstants,
		NodeLabelInputField,
		ToolConstants,
		_
	) {

		"use strict";

		/**
		 * @exports LabelEditorTool
		 * @class
		 * @augments AbstractTool
		 * The tool for editing node labels
		 * @param {Observable} mouseInteractionTool 
		 * @param {WFEditorContext} WFEditorContext
		 */
		var LabelEditorTool = function (mouseInteractionTool, WFEditorContext) {
			AbstractTool.call(this, WFEditorContext.get("scene").getContext(), WebCenterToolConstants.Tools.LABELEDITORTOOL, WFEditorContext);
			Observer.mixin(this);

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = WFEditorContext.get("scene");

			/**
			 * @member {ToolManager}
			 * @private
			 */
			this._toolManager = WFEditorContext.get("toolManager");

			/**
			 * @member {ToolbarModel}
			 * @private
			 */
			this._toolbarModel = WFEditorContext.get("toolbarModel");

			/**
			 * @member {NodeLabelInputField}
			 * @private
			 */
			this._nodeLabelInputField = null;

			/**
			 * @member {UndoRedoController}
			 * @private
			 */
			this._undoRedo = WFEditorContext.get("undoRedo");

			/**
			 * @member {I18n}
			 * @private
			 */
			this._i18n = WFEditorContext.get("i18n");

			// Listen to mouse downs
			this._observe(mouseInteractionTool, "mousedown", this._checkMouseDownOnLabel);

			// rerender the input box if the zoom has changed
			this._observe(WFEditorContext.get("sceneModel"), "zoomChanged", _.bind(function () {
				if (this._nodeLabelInputField !== null) {
					// Defer to ensure the transformation has been done
					_.defer(_.bind(function () {
						if (this._nodeLabelInputField) {
							this._nodeLabelInputField.show();
						}
					}, this));
				}
			}, this));
		};

		LabelEditorTool.prototype = Object.create(AbstractTool.prototype);
		LabelEditorTool.prototype.constructor = LabelEditorTool;


		/**
		 * check if the mouse down event received from the mouse interaction tool is on a Label, if so, ask tool manager to become active
		 * @param {Object} options
		 */
		LabelEditorTool.prototype._checkMouseDownOnLabel = function (options) {
			var hitArea = options.hitArea;
			this._doneEditing();
			var isLabelEditingHitArea = hitArea && hitArea.hasClass(WebCenterNodeConstants.HitAreas.LABEL);

			if (isLabelEditingHitArea && hitArea.getView() &&
				hitArea.getView().getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Node) {
				if (hitArea.getView().getModel().getWorkflow().getPrivilege(WebCenterNodeConstants.Privileges.editLabels)) {
					// There was a double click on a NodeLabel hitArea, the tool can be activated
					this._nodeLabelInputField = new NodeLabelInputField(this._scene, hitArea.getView(), _.bind(this._doneEditing, this), _.bind(this._stopWithoutSaving, this), this._undoRedo, this._i18n);
					this._nodeLabelInputField.on("blur", _.bind(this._doneEditing, this));
					this._nodeLabelInputField.show();
				}
			}
		};


		/**
		 * indicates that the editing has been finished, set the node's label if it isn't empty and hide the input field.
		 */
		LabelEditorTool.prototype._doneEditing = function () {
			if (this._nodeLabelInputField !== null) {
				this._nodeLabelInputField.hide();
				if (this._nodeLabelInputField.getText() !== "") {
					this._nodeLabelInputField.commit();
				}
				this._nodeLabelInputField.destroy();
				this._nodeLabelInputField = null;
			}

		};

		/**
		 * Stops the editing without saving it to the node's label
		 */
		LabelEditorTool.prototype._stopWithoutSaving = function () {
			if (this._nodeLabelInputField !== null) {
				this._nodeLabelInputField.hide();
				this._nodeLabelInputField.destroy();
				this._nodeLabelInputField = null;
			}
		};

		return LabelEditorTool;

	}
);
