define([
		"WFEditor/components/InputField",
		"utils/ShapeUtils",
		"WFEditorWCR/scene/WebCenterViewConstants",
		"utils/Command",
		"underscore"
	],
	function (
		InputField,
		ShapeUtils,
		WebCenterViewConstants,
		Command,
		_
	) {
		"use strict";

		/**
		 * an input field to edit the label of a node
		 * 
		 * @class
		 * @exports NodeLabelInputField
		 * 
		 * @param {Scene} scene
		 * @param {WebCenterNodeView} nodeView
		 * @param {Function} onEnter  what to do on enter?
		 * @param {Function} onEsc  what to do on escape?
		 * @param {UndoRedoController} undoManager
		 * @param {I18n} i18n
		 */
		var NodeLabelInputField = function (scene, nodeView, onEnter, onEsc, undoManager, i18n) {
			InputField.call(this, scene.getContext(), WebCenterViewConstants.NodeLabelMaxWidth, nodeView.getTextAlignment());

			/**
			 * @member {WebCenterNodeView}
			 * @private
			 */
			this._nodeView = nodeView;

			this._inputElement.val(nodeView.getModel().getLabel());

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = scene;

			/**
			 * @member {UndoRedoController}
			 * @private
			 */
			this._undoManager = undoManager;

			/**
			 * @member {I18n}
			 * @private
			 */
			this._i18n = i18n;

			// bind the enter key
			this._inputElement.keydown(_.bind(function (e) {
				if (e.keyCode == 13) {
					onEnter();
					e.preventDefault();
				} else if (e.keyCode == 27) {
					onEsc();
					e.preventDefault();
				}
			}, this));

		};

		NodeLabelInputField.prototype = Object.create(InputField.prototype);
		NodeLabelInputField.prototype.constructor = NodeLabelInputField;

		/**
		 * @override
		 */
		NodeLabelInputField.prototype._getLocation = function () {
			var t = this._scene.getTransformation();
			var location = this._nodeView.getLabelStartPoint();
			location.y -= WebCenterViewConstants.NodeLabelLineHeight;
			return t.transformedPoint(location);
		};

		/**
		 * @override
		 */
		NodeLabelInputField.prototype.show = function () {
			InputField.prototype.show.call(this);
			this._nodeView.getModel().setEditingLabel(true);
			this.focus();

		};

		/**
		 * @override
		 */
		NodeLabelInputField.prototype.hide = function () {
			this._nodeView.getModel().setEditingLabel(false);
			InputField.prototype.hide.call(this);
		};

		/**
		 * set the node's label to the content of the input field
		 */
		NodeLabelInputField.prototype.commit = function () {
			var currentLabel = this._nodeView.getModel().getLabel();
			var newLabel = this._inputElement.val().trim();
			if (currentLabel !== newLabel) {
				var command = new Command(
					_.bind(function () {
						this._nodeView.getModel().setLabel(newLabel);
					}, this),
					_.bind(function () {
						this._nodeView.getModel().setLabel(currentLabel);

					}, this),
					this._i18n.t("editLabelDescription")
				);
				this._nodeView.getModel().setLabel(this._inputElement.val());
				this._undoManager.execute(command);
			}
		};


		return NodeLabelInputField;
	}
);
