define(
	[
		"underscore",
		"WFEditor/tools/AbstractInteractiveTool",
		"WFEditorWCR/tools/WebCenterToolConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditor/tools/ToolConstants",
		"utils/Observer",
		"utils/Command"
	],
	function (_, AbstractInteractiveTool, WebCenterToolConstants, WebCenterNodeConstants, ToolConstants, Observer, Command) {

		"use strict";

		/**
		 * @class
		 * @exports MoveStickyNoteTool
		 * @augments AbstractInteractiveTool
		 * @mixes Observer
		 * 
		 * @param {SceneController}    sceneController        
		 * @param {Observable}         interactionEvents 
		 * @param {WFEditorContext}    WFEditorContext          
		 */
		var MoveStickyNoteTool = function (sceneController, interactionEvents, WFEditorContext) {
			AbstractInteractiveTool.call(this, WebCenterToolConstants.Tools.STICKYNOTE.MOVE, sceneController, WFEditorContext);
			Observer.mixin(this);

			this._observe(interactionEvents, "mousedown", this._handleMouseInteractionEvent);
			this._observe(interactionEvents, "mousemove", this._handleMouseInteractionEvent);
			this._observe(interactionEvents, "mouseup", this._handleMouseInteractionEvent);

			/**
			 * @member {StickyNoteView}
			 * @private
			 */ 
			this._stickyNoteView = null;

			/**
			 * @member {Scene}
			 * @private
			 */ 
			this._scene = WFEditorContext.get("scene");

			/**
			 * @member {ShapeUtils.Location}
			 * @private
			 */ 
			this._previousWorldPoint = null;

			/**
			 * @member {UndoRedoController}
			 * @private
			 */ 
			this._undoRedo = WFEditorContext.get("undoRedo");

			/**
			 * @member {Boolean}
			 * @private
			 */ 
			this._isMoving = false;

			/**
			 * @member {ToolbarModel}
			 * @private
			 */ 
			this._toolbarModel = WFEditorContext.get("toolbarModel");
			
			this.changeState(ToolConstants.State.DEFAULTSTATE);

			this.registerActiveEvents(["mousemove", "mouseup", "mousedown", "mouseleave", "mouseenter"]);

			this._registerStates();
		};

		MoveStickyNoteTool.prototype = Object.create(AbstractInteractiveTool.prototype);
		MoveStickyNoteTool.prototype.constructor = MoveStickyNoteTool;

		MoveStickyNoteTool.prototype._handleMouseInteractionEvent = function (options) {
			this.triggerEvent(options.eventName, options);
		};


		/**
		 * registers all states to handle the mouse interaction
		 */
		MoveStickyNoteTool.prototype._registerStates = function () {

			this.registerState(ToolConstants.State.DEFAULTSTATE, {
				"mousedown": _.bind(function (options) {
					var hitArea = options.hitArea;
					if (options.e.button !== 0 || !hitArea || (hitArea && !hitArea.hasClass(WebCenterNodeConstants.HitAreas.STICKYNOTE.MOVE))) {
						// Not a hit area we should react to
						return;
					}
					this.changeState(WebCenterToolConstants.State.STICKYNOTE.SHOULDMOVE, options);
				}, this),
				"mousemove": function () {},
				"mouseup": function () {},


			});

			this.registerState(WebCenterToolConstants.State.STICKYNOTE.SHOULDMOVE, {
				"enterState": _.bind(function (fromState, options) {
					this._stickyNoteView = options.hitArea.getView();
					this._startLocation = this._stickyNoteView.getModel().getLocation().copy();
				}, this),
				"mousemove": _.bind(function (options) {
					this.changeState(WebCenterToolConstants.State.STICKYNOTE.MOVE, options);
				}, this),
				"mouseup": _.bind(function (options) {
					this.changeState(ToolConstants.State.DEFAULTSTATE);
				}, this),
				"mousedown": function () {}
			});

			this.registerState(WebCenterToolConstants.State.STICKYNOTE.MOVE, {
				"enterState": _.bind(function (fromState, options) {
					this._scene.setInteractive(this._stickyNoteView.getId(), true);
					this._toolManager.activateTool(this);
					this._previousWorldPoint = this._scene.getWorldPointFromScenePoint(this._getMousePosition(options.e));
					this._isMoving = true;
				}, this),
				"leaveState": _.bind(function (fromState) {
					this._scene.setInteractive(this._stickyNoteView.getId(), false);
					var stickyNote = this._stickyNoteView.getModel();
					var oldLocation = this._startLocation;
					var newLocation = this._stickyNoteView.getModel().getLocation().copy();
					this._isMoving = false;
					this._undoRedo.execute(new Command(
						function () {
							stickyNote.setLocation(newLocation);
						},
						function () {
							stickyNote.setLocation(oldLocation);
						}, "moveStickyNoteDescription"
					), true);

				}, this),
				"mousemove": _.bind(function (e) {
					var currentMousePos = this._getMousePosition(e);
					e.preventDefault();
					e.stopPropagation();
					// Panning close to borders
					this._stopIntervalPanning();
					if (this._shouldPan(currentMousePos)) {
						this._calculatePanning(currentMousePos);
						this._startIntervalPanning(currentMousePos, this._moveStickyNote);
					}
					this._moveStickyNote(currentMousePos);
				}, this),
				"mouseup": _.bind(function (e) {
					this._toolManager.deactivateCurrentTool(true);
				}, this),
				"mouseleave": _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					this._continuePanningWhileMouseOutsideCanvas(this._moveStickyNote, _.bind(function (mousePosition) {
						this._moveStickyNote(mousePosition);
						this._toolManager.deactivateCurrentTool(true);
					}, this));
				}, this),
				"mouseenter": _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					this._stopPanningWhileMouseOutsideCanvas();
				}, this)
			});
		};

		/**
		 * @override
		 */
		MoveStickyNoteTool.prototype.activate = function (event, silent) {
			AbstractInteractiveTool.prototype.activate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, true);
		};

		/**
		 * moves the sticky note to current mouse position
		 * @param {ShapeUtils.Location} currentMousePos 
		 */
		MoveStickyNoteTool.prototype._moveStickyNote = function (currentMousePos) {
			var newWorldPoint = this._scene.getWorldPointFromScenePoint(currentMousePos);
			var offsetX = newWorldPoint.x - this._previousWorldPoint.x;
			var offsetY = newWorldPoint.y - this._previousWorldPoint.y;
			this._stickyNoteView.getModel().offsetXY(offsetX, offsetY);
			this._previousWorldPoint = newWorldPoint;

		};

		/**
		 * @override
		 */
		MoveStickyNoteTool.prototype.allowDeactivate = function () {
			return !this._isMoving;
		};

		/**
		 * @override
		 */
		MoveStickyNoteTool.prototype.getDefaultCursor = function () {
			return "grabbing-cursor";
		};

		return MoveStickyNoteTool;

	});
