define(
	[
		"underscore",
		"WFEditor/tools/AbstractTool",
		"WFEditorWCR/tools/WebCenterToolConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditor/tools/ToolConstants",
		"utils/Observer",
		"utils/Command"
	],
	function (_, AbstractTool, WebCenterToolConstants, WebCenterNodeConstants, ToolConstants, Observer, Command) {

		"use strict";

		/**
		 * Tool for resizing a sticky note
		 * @class
		 * @augments AbstractTool
		 * @mixes Observer
		 * @exports ResizeStickyNoteTool
		 * 
		 * @param {SceneController}      sceneController      
		 * @param {Observable} interactionEvents 
		 * @param {WFEditorContext}   WFEditorContext          
		 */
		var ResizeStickyNoteTool = function (sceneController, interactionEvents, WFEditorContext) {
			AbstractTool.call(this, WFEditorContext.get("scene").getContext(), WebCenterToolConstants.Tools.STICKYNOTE.RESIZE, WFEditorContext);
			Observer.mixin(this);

			this._observe(interactionEvents, "mousedown", this._handleMouseInteractionEvent);

			/**
			 * @member {StickyNoteView}
			 * @private
			 */
			this._stickyNoteView = null;

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = WFEditorContext.get("scene");

			/**
			 * @member {ToolManager}
			 * @private
			 */
			this._toolManager = WFEditorContext.get("toolManager");

			/**
			 * @member {ToolbarModel}
			 * @private
			 */
			this._toolbarModel = WFEditorContext.get("toolbarModel");

			this.changeState(ToolConstants.State.DEFAULTSTATE);

			this.registerActiveEvents(["mousemove", "mouseup"]);

			this._registerStates();

			/**
			 * @member {ShapeUtils.Location}
			 * @private
			 */
			this._previousWorldPoint = null;
			
			/**
			 * @member {ShapeUtils.Location}
			 * @private
			 */
			this._startSize = null;

			/**
			 * @member {UndoRedoController}
			 * @private
			 */
			this._undoRedo = WFEditorContext.get("undoRedo");

			/**
			 * @member {Boolean}
			 * @private
			 */
			this._isResizing = false;

		};

		ResizeStickyNoteTool.prototype = Object.create(AbstractTool.prototype);
		ResizeStickyNoteTool.prototype.constructor = ResizeStickyNoteTool;

		/**
		 * handle the events received from the mouse interaction tool
		 * @param {Object} options 
		 */
		ResizeStickyNoteTool.prototype._handleMouseInteractionEvent = function (options) {
			this.triggerEvent(options.eventName, options);
		};


		/**
		 * @override
		 */
		ResizeStickyNoteTool.prototype.activate = function (event, silent) {
			AbstractTool.prototype.activate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, true);
		};

		/**
		 * register states for sticky note resizing
		 */
		ResizeStickyNoteTool.prototype._registerStates = function () {

			this.registerState(ToolConstants.State.DEFAULTSTATE, {
				"mousedown": _.bind(function (options) {
					var hitArea = options.hitArea;
					if (!hitArea || (hitArea && !hitArea.hasClass(WebCenterNodeConstants.HitAreas.STICKYNOTE.RESIZE))) {
						// Not a hit area we should react to
						return;
					}
					this._toolManager.activateTool(this);
					this.changeState(WebCenterToolConstants.State.STICKYNOTE.RESIZE, options);
				}, this),
				"mousemove": function () {},
				"mouseup": function () {},


			});

			this.registerState(WebCenterToolConstants.State.STICKYNOTE.RESIZE, {
				"enterState": _.bind(function (fromState, options) {
					this._stickyNoteView = options.hitArea.getView();
					this._scene.setInteractive(this._stickyNoteView.getId(), true);
					this._previousWorldPoint = this._scene.getWorldPointFromScenePoint(this._getMousePosition(options.e));
					this._startSize = this._stickyNoteView.getModel().getSize().copy();
					this._isResizing = true;
				}, this),
				"mousemove": _.bind(function (e) {
					var currentMousePos = this._getMousePosition(e);
					e.preventDefault();
					e.stopPropagation();
					this._resizeStickyNote(currentMousePos);
				}, this),
				"mouseup": _.bind(function (e) {

					var oldSize = this._startSize;
					var newSize = this._stickyNoteView.getModel().getSize().copy();
					var stickyNote = this._stickyNoteView.getModel();
					this._undoRedo.execute(new Command(function () {
							stickyNote.setSize(newSize.width, newSize.height);
						},
						function () {
							stickyNote.setSize(oldSize.width, oldSize.height);
						},
						"resizeStickyNoteDescription"), true);
					this._isResizing = false;
					this._scene.setInteractive(this._stickyNoteView.getId(), false);
					this._toolManager.deactivateCurrentTool();
				}, this)
			});
		};

		/**
		 * resize a sticky note
		 * @param {ShapeUtils.Point} currentMousePos 
		 */
		ResizeStickyNoteTool.prototype._resizeStickyNote = function (currentMousePos) {
			var newWorldPoint = this._scene.getWorldPointFromScenePoint(currentMousePos);
			var offsetX = newWorldPoint.x - this._previousWorldPoint.x;
			var offsetY = newWorldPoint.y - this._previousWorldPoint.y;
			this._stickyNoteView.getModel().setSize(this._startSize.width + offsetX, this._startSize.height + offsetY);


		};

		/**
		 * @override
		 */
		ResizeStickyNoteTool.prototype.getDefaultCursor = function () {
			return "nwse-resize-cursor";
		};

		/**
		 * @override
		 */
		ResizeStickyNoteTool.prototype.allowDeactivate = function () {
			return !this._isResizing;
		};

		return ResizeStickyNoteTool;

	});
