define(
	[
		"WFEditor/tools/AbstractTool",
		"utils/Observer",
		"WFEditor/workflow/ModelConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/tools/WebCenterToolConstants",
		"WFEditor/tools/ToolConstants",
		"utils/webi18n!WFEditorWCR",
		"utils/Command",
		"underscore"
	],
	function (
		AbstractTool,
		Observer,
		ModelConstants,
		WebCenterNodeConstants,
		WebCenterToolConstants,
		ToolConstants,
		i18n,
		Command,
		_
	) {

		"use strict";

		/**
		 * @exports StickyNoteTextEditdorTool
		 * @class
		 * @augments AbstractTool
		 * The tool for editing node labels      
		 * @param {Observable} mouseInteractionTool 
		 * @param {WFEditorContext} WFEditorContext
		 */
		var StickyNoteTextEditdorTool = function (mouseInteractionTool, WFEditorContext) {
			AbstractTool.call(this, WFEditorContext.get("scene").getContext(), WebCenterToolConstants.Tools.StickyNoteTextEditdorTool, WFEditorContext);
			Observer.mixin(this);

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = WFEditorContext.get("scene");

			/**
			 * @member {ToolManager}
			 * @private
			 */
			this._toolManager = WFEditorContext.get("toolManager");

			/**
			 * @member {UndoRedoController}
			 * @private
			 */
			this._undoRedo = WFEditorContext.get("undoRedo");

			/**
			 * @member {I18n}
			 * @private
			 */
			this._i18n = WFEditorContext.get("i18n");

			/**
			 * @member {StickyNoteView}
			 * @private
			 */
			this._stickyNoteView = null;

			// If not active, listen to double clicks from the mouse interaction tool
			this._observe(mouseInteractionTool, "mousedown", this._checkClickOnEditIcon);
			this._observe(WFEditorContext.get("scene"), "StickyNotePopup.*.saved", this._doneEditing);

		};

		StickyNoteTextEditdorTool.prototype = Object.create(AbstractTool.prototype);
		StickyNoteTextEditdorTool.prototype.constructor = StickyNoteTextEditdorTool;


		/**
		 * check if the tool should start editing or delete the sticky note
		 * @param {Object} options 
		 */
		StickyNoteTextEditdorTool.prototype._checkClickOnEditIcon = function (options) {
			if (options.e.button !== 0) {
				return;
			}
			var hitArea = options.hitArea;

			var isLabelEditingHitArea = hitArea && (
				hitArea.hasClass(WebCenterNodeConstants.HitAreas.STICKYNOTE.EDITICON) ||
				hitArea.hasClass(WebCenterNodeConstants.HitAreas.STICKYNOTE.TEXTAREA));
			var isDeleteHitArea = hitArea && hitArea.hasClass(WebCenterNodeConstants.HitAreas.STICKYNOTE.DELETE);

			this._doneEditing();

			if (isLabelEditingHitArea) {
				this._stickyNoteView = hitArea.getView();
				this._stickyNoteView.getModel().setEditingText(true);

			} else if (isDeleteHitArea) {
				this._deleteStickyNote(hitArea.getView().getModel());
			}
		};


		/**
		 * indicates that the editing has been finished
		 */
		StickyNoteTextEditdorTool.prototype._doneEditing = function () {
			if (this._stickyNoteView && this._stickyNoteView.getModel().isEditingText()) {
				this._setNewStickyNoteText();
				this._stickyNoteView.getModel().setEditingText(false);
			}

		};

		/**
		 * set text of the sticky note
		 */
		StickyNoteTextEditdorTool.prototype._setNewStickyNoteText = function () {
			var currentText = this._stickyNoteView.getModel().getText();
			var newText = this._stickyNoteView.getStickyNotePopup().getEditedText().trim();
			if (currentText !== newText) {
				var command = new Command(
					_.bind(function () {
						this._stickyNoteView.getModel().setText(newText);
					}, this),
					_.bind(function () {
						this._stickyNoteView.getModel().setText(currentText);
					}, this),
					this._i18n.t("editStickyNoteTextDescription")
				);
				this._undoRedo.execute(command);
			}
		};

		/**
		 * Stops the editing without saving it to the node's label
		 */
		StickyNoteTextEditdorTool.prototype._stopWithoutSaving = function () {
			if (this._stickyNoteView && this._stickyNoteView.getModel().isEditingText()) {
				this._stickyNoteView.getModel().setEditingText(false);

			}
		};

		/**
		 * deletes a sticky note
		 * @param {StickyNoteHelper} stickyNote 
		 */
		StickyNoteTextEditdorTool.prototype._deleteStickyNote = function (stickyNote) {
			var command = new Command(_.bind(function () {
					this._scene.getWorkflowModel().removeHelper(stickyNote);
				}, this), _.bind(function () {
					this._scene.getWorkflowModel().addHelper(stickyNote);
				}, this),
				this._i18n.t("removeStickyNoteDescription"));
			this._undoRedo.execute(command);
		};

		/**
		 * @override
		 * @returns {String} 
		 */
		StickyNoteTextEditdorTool.prototype.getDefaultCursor = function () {
			return "default-cursor";
		};

		return StickyNoteTextEditdorTool;

	}
);
