define([
		"WFEditor/components/DraggablePopup",
		"WFEditorWCR/documentContextMenu/WebCenterDocumentContextMenu",
		"Template",
		"tpl!WFEditorWCR/webcenterInformationPopup/WebCenterInformationPopup",
		"utils/HTTPUtils",
		"utils/ShapeUtils",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery",
		"jqueryUI"
	],
	function (
		DraggablePopup,
		WebCenterDocumentContextMenu,
		Template,
		tpl,
		HTTPUtils,
		ShapeUtils,
		i18n,
		_,
		$) {
		"use strict";

		/**
		 * Token information is shown in a movable popup. It shows the tokens and their documents
		 * as a list in the popup. The popup extends DraggablePopup in the WFEditor framework.
		 *
		 * @exports WebCenterInformationPopup
		 * @class
		 * @augments DraggablePopup
		 *
		 * @param {Workflow} workflow
		 * @param {Scene} scene
		 * @param {String} backgroundColor
		 * @param {AbstractView} parentView - the view that is responsible for showing and hiding the popup
		 */
		var WebCenterInformationPopup = function (workflow, scene, backgroundColor, parentView) {
			DraggablePopup.call(this, scene.getContext());

			/**
			 * @member {Scene} general view object, popup is added to its context
			 * @private
			 */
			this._scene = scene;

			/**
			 * @member {Workflow}
			 * @private
			 */
			this._workflow = workflow;
			/**
			 * @member {String}
			 * @private
			 */
			this._color = backgroundColor;

			/**
			 * @member {AbstractView}
			 * @private
			 */
			this._parentView = parentView;

			/**
			 * @member {WebCenterDocumentContextMenu} context menu for a document in a token
			 * @private
			 */
			this._contextMenu = null;

			/**
			 * @member {ShapeUtils.Size} the amount of pixels the popup was moved by dragging
			 * @private
			 */
			this._movedSize = new ShapeUtils.Size();

			// initialize template
			this.t = new Template(i18n, tpl);
			//set the context for the template, this is the content of the popup
			this.t.setContext(this._content);
			//register click event an the token items
			this.t.bind("click", "li", this._tokenItemClicked, this);
			this.t.bind("contextmenu", "li", this._handleTokenItemContextMenu, this);
			//register a function to the template to fetch the correct class for selected items
			this.t.d.getSelectedClass = _.bind(this._getSelectedClass, this);
			
			// Containment gives weird effects if the popup is bigger than the scene
			this._element.draggable({
				containment: undefined
			});

		};

		WebCenterInformationPopup.prototype = Object.create(DraggablePopup.prototype);
		WebCenterInformationPopup.prototype.constructor = WebCenterInformationPopup;

		/**
		 * Renders the popup on the right position
		 */
		WebCenterInformationPopup.prototype.show = function () {
			if (!this.isVisible()) { //only show the popup if it not yet visible
				DraggablePopup.prototype.show.call(this);
				//set the title of the popup to the label of the pin
				this.setTitle(this.getTitle());
				var color = this._color;
				//get the location of the pin
				var location = this.getPopupLocation();
				//transform the point from world coordinates to pixels from scene div top left corner
				var t = this._scene.getTransformation();
				var loc = t.transformedPoint(location);
				this._element.css({
					left: loc.x, //draw to the right of the pin
					top: loc.y, //move some pixels upwards
					'background-color': color //set the correct background color
				});
				this.render();
			}
		};

		/**
		 * @override
		 */
		WebCenterInformationPopup.prototype.hide = function (forceHide) {
			if ((!this._isHovered && !this._isMoved && !this._isSticky) || forceHide) {
				DraggablePopup.prototype.hide.call(this);
				this._movedSize = new ShapeUtils.Size();
			}
		};

		/**
		 * recalculates the location of the popup
		 */
		WebCenterInformationPopup.prototype.relocate = function () {
			//get the location of the pin
			var location = this.getPopupLocation();
			//transform the point from world coordinates to pixels from scene div top left corner
			var t = this._scene.getTransformation();
			var loc = t.transformedPoint(location);
			this._element.css({
				left: loc.x + this._movedSize.width, //draw to the right of the pin
				top: loc.y + this._movedSize.height //move some pixels upwards
			});

		};

		/**
		 * Render the template of the popup.
		 */
		WebCenterInformationPopup.prototype.render = function () {
			//get the tokens that need to be rendered in the popup
			this.t.d.tokens = this.getTokens();
			//render the template
			this.t.render();
		};

		/**
		 * Get the correct class for the token items when they are selected. This method is
		 * used by the template.
		 * @param   {WebCenterToken} token
		 * @param   {String} versionId
		 * @returns {String}   class that is added to the list item or empty string if no class needs to be added.
		 */
		WebCenterInformationPopup.prototype._getSelectedClass = function (token, versionId) {
			if ((versionId && token.getSelectedDocumentVersionId() === versionId) || (token.getFileCount() === 0 && token.isSelected())) {
				return "docSelected";
			} else if (token.isSelected()) {
				return "tokenSelected";
			} else {
				var highlightedToken = this._workflow.getHightLightedToken();
				if (highlightedToken && this._parentView.getModel().isHighlighted() && token.isHighlighted()) {
					return "docPathSelected";
				} else {
					return "";
				}
			}
		};

		/**
		 * The popup is being moved around make sure the parentView is set to being interactive.
		 */
		WebCenterInformationPopup.prototype._startedMoving = function () {
			this._scene.setInteractive(this._parentView.getId(), true);
		};

		/**
		 * The popup has stopped moving around make sure the parentView is no longer marked as interactive.
		 */
		WebCenterInformationPopup.prototype._stoppedMoving = function () {
			this._scene.setInteractive(this._parentView.getId(), false);
			var position = this._element.position();
			var startPosition = this.getPopupLocation();
			//transform the point from world coordinates to pixels from scene div top left corner
			var loc = this._scene.getTransformation().transformedPoint(startPosition);
			this._movedSize = loc.sizeTo(new ShapeUtils.Point(position.left, position.top));
		};

		/**
		 * The popup has hidden itself (close button)
		 */
		WebCenterInformationPopup.prototype._closed = function () {
			var highlightedToken = this._workflow.getHightLightedToken();
			if (highlightedToken && _.contains(this.getTokens(), highlightedToken)) {
				this._scene.tokenItemSelected(null);
			}
		};

		/**
		 * Get the location of the popup in world coordinates
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterInformationPopup.prototype.getLocation = function () {
			var location = DraggablePopup.prototype.getLocation.call(this);
			var t = this._scene.getTransformation().createInverse();
			return t.transformedPoint(location);
		};

		/**
		 * Executed when a item of the token is clicked and needs to be highlighted.
		 * @param {DOMObject} el
		 * @param {Event} event
		 */
		WebCenterInformationPopup.prototype._tokenItemClicked = function (el, event) {
			var tokenId = $(el).attr("data-bind-tokenId");
			var versionId = $(el).attr("data-bind-versionId");
			var token = this._workflow.getToken(tokenId);

			this._scene.tokenItemSelected(token, versionId);
		};

		/**
		 * Handle the context menu click for the token. Initializes the view for the context menu.
		 * @param {JQueryObj} el
		 * @param {Object}   e
		 */
		WebCenterInformationPopup.prototype._handleTokenItemContextMenu = function (el, e) {
			// Make it stay open
			this._draggableStart();
			this._draggableStop();

			// Select the document
			this._tokenItemClicked(el, e);

			var versionId = $(el).attr("data-bind-versionId");
			var tokenId = $(el).attr("data-bind-tokenId");
			var token = this._workflow.getToken(tokenId);
			if (!this._contextMenu) {
				this._contextMenu = new WebCenterDocumentContextMenu(this._scene);
			}
			var sceneOffset = this._scene.getContext().offset();
			this._contextMenu.show(token.getPin(), versionId, tokenId, new ShapeUtils.Point(e.clientX - sceneOffset.left + window.pageXOffset, e.clientY - sceneOffset.top + window.pageYOffset));
			e.preventDefault();
			e.stopPropagation();
		};

		/**
		 * gets the title of the popup, should be implemented by specific popup
		 * @returns {String}
		 */
		WebCenterInformationPopup.prototype.getTitle = function () {};

		/**
		 * Gets the location of the popup, should be implemented by specific popup
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterInformationPopup.prototype.getPopupLocation = function () {};

		/**
		 * Gets the tokens that should be added to the popup, should be implemented by specific popup
		 * @returns {Array.<Token>}
		 */
		WebCenterInformationPopup.prototype.getTokens = function () {};

		return WebCenterInformationPopup;

	});
