define(["jquery",
		"underscore",
		"utils/ShapeUtils",
		"component/tooltip/Tooltip",
		"utils/webi18n!WFEditorWCRPopup",
		"WFEditorWCR/wfPopup/MinifiedWFEditorWCR",
		"WFEditorWCR/wfPopup/WFEditorWCRPopup",
		"WFEditorWCR/core/WFEditorWCR",
		"utils/Observer",
		"utils/webcss!WFEditorWCR/core/wfeditorwcr-style.css"
	],
	function ($,
		_,
		ShapeUtils,
		Tooltip,
		i18n,
		MinifiedWFEditorWCR,
		WFEditorWCRPopup,
		WFEditorWCR,
		Observer) {

		"use strict";

		/**
		 * This class configures everything to open the WFEditorWCR in a popup
		 *
		 * @class
		 * @exports WFEditorWCRPopupConfigurator
		 * @param {String} taskId - id of the task that should be opened on click
		 * @param {String} taskTypeId - id of the task type of the task
		 * @param {String} name - name of the task
		 * @param {Object} privileges
		 */
		var WFEditorWCRPopupConfigurator = function (taskId, taskTypeId, name, privileges) {
			Observer.mixin(this);
			/**
			 * @member {WFEditor}
			 * @private
			 */
			this._wFEditor = null;

			/** 
			 * @member {String} 
			 * @private
			 */
			this._taskTypeId = taskTypeId;

			/** 
			 * @member {String} 
			 * @private
			 */
			this._taskId = taskId;

			/** 
			 * @member {String} 
			 * @private
			 */
			this._name = name;

			/**
			 * @member {Object}
			 * @private
			 */
			this._privileges = privileges || {};

			/**
			 * @member {WFEditorWCRPopup}
			 * @private
			 */
			this._popup = null;

			/**
			 * @member {ShapeUtils.Rect} the visible rect of the WFEdidtor upon closing, to store the context for the user
			 * @private
			 */
			this._visibleRect = null;

		};

		/**
		 * opens the workflow on the location of the click event
		 * @param {Event} the mouse click event that triggered this function
		 */
		WFEditorWCRPopupConfigurator.prototype.openWorkflow = function (e) {
			var body = $("body");

			if (!this._wFEditor) {
				this._popup = new WFEditorWCRPopup("popup" + this._taskTypeId, body, this._name, "minified-wfeditor-popup preview-wfeditor-popup");
				// Add the "go to full screen, full editor" button

				var maximizeWFIcon = $("<a/>").addClass("enter-full-icon").append($("<i/>")
						.addClass("icon icon-WCR_WF-LargerWindow12 icon-normal"))
					.on("click", _.bind(function () {
						tooltip.destroy();
						this._goToFullEditor();
					}, this)).on("mousedown", function (e) {
						e.stopPropagation();
					});

				this._popup.getHeader().append(maximizeWFIcon);

				var tooltip = new Tooltip(null, maximizeWFIcon, body, body, false, i18n.t("wfeditorPopup.tooltip.goToFullEditor"));

				this._wFEditor = new MinifiedWFEditorWCR(this._taskId, this._taskTypeId, this._popup.content, this._popup.getFooter(), this._privileges);
				// resize the popup if the wfeditor wants to be resized
				this._observe(this._wFEditor, "resize", this._popup.resize.bind(this._popup));
				this._registerListeners();
			}
			this.open(e);

		};

		/**
		 * switches between the minified version of the WFEditor to the fully fledged one
		 */
		WFEditorWCRPopupConfigurator.prototype._goToFullEditor = function () {
			var body = $("body");

			if (this._wFEditor && this._taskId) {
				this._visibleRect = this._wFEditor.getVisibleRect();
				this._removeEditor();
			}
			if (!this._wFEditor) {
				this._popup = new WFEditorWCRPopup("popup" + this._taskTypeId, body, this._name, "full-wfeditor-popup preview-wfeditor-popup");
				this._popup.content.attr("tasktypeid", this._taskTypeId);
				this._popup.content.attr("taskid", this._taskId);
				this._popup.content.attr("canedit", this._privileges.hasTaskEditRights ? "1" : "0");
				this._popup.content.attr("autorefresh", this._privileges.autorefresh ? "1" : "0");
				this._popup.content.attr("mastertaskid", this._privileges.isMasterTask ? undefined : "0");
				this._popup.element.css({
					width: $(window).width() * 2 / 3,
					height: $(window).height() * 2 / 3
				});

				this._wFEditor = WFEditorWCR.wcrinit(this._popup.content[0]);
				this._registerListeners();
			}
			this._popup.show(new ShapeUtils.Point($(window).width() / 6, $(window).height() / 6 + window.pageYOffset));

		};

		/**
		 * registeres the listeners on the WFEditor
		 */
		WFEditorWCRPopupConfigurator.prototype._registerListeners = function () {
			this._popup.setTitle(this._name);
			this._observe(this._wFEditor, "workflowLoaded", function () {
				if (this._visibleRect) {
					this._wFEditor.setVisibleRect(this._visibleRect);
				}
				this._wFEditor.focus();
			});
			this._popup.element.on("mousedown", _.bind(function (e) {
				// Mouse down on the element should result in the popup being on top
				// This is not the same as the WFEditor having focus as the user
				// may have eg. clicked on an input field inside a parameter panel 
				this._movePopupToFront();
			}, this));
			this._observe(this._wFEditor, "focus", function () {
				this._setFocusedPopup();
			});
			this._observe(this._wFEditor, "blur", function () {
				this._popup.setTitle(this._name);
				this._popup.element.removeClass("focused");
			});
			this._observe(this._popup, "*.stop", function () {
				this._wFEditor.resized();
			});

			this._observe(this._popup, "*.start", function () {
				this._wFEditor.focus();
			});

			this._popup.getFooter().on("mousedown", _.bind(function () {
				this._wFEditor.focus();
			}, this));
			this._popup.getHeader().on("mousedown", _.bind(function () {
				this._wFEditor.focus();
			}, this));
			this._observe(this._wFEditor, "fullscreen.enter", _.bind(function () {
				this._popup.element.addClass("fullscreen");
			}, this));
			this._observe(this._wFEditor, "fullscreen.exit", _.bind(function () {
				this._popup.element.removeClass("fullscreen");
			}, this));

			this._observe(this._popup, "closed", function (canCloseObj) {
				if (this._taskId || this._wFEditor.canClose()) {
					canCloseObj.canClose = true;
					this._visibleRect = this._wFEditor.getVisibleRect();
					this._removeEditor();
					this._stopObserveAll();
				} else {
					// let the popup know that he shouldn't close
					canCloseObj.canClose = false;
				}
			});

		};

		/**
		 * Moves the popup to the front by appending it to the body
		 * @param {Boolean} focus - should the Editor be focused as well?
		 */
		WFEditorWCRPopupConfigurator.prototype._movePopupToFront = function (focus) {
			if (!this._popup.element.is('.preview-wfeditor-popup:last')) {
				if (this._wFEditor) {
					this._wFEditor.notifyRemoveFromDom();
				}
				this._popup.element.appendTo($("body"));
				if (this._wFEditor) {
					this._wFEditor.notifyAddToDom();
				}
				if (focus && this._wFEditor) {
					this._wFEditor.focus();
				}
			}
		};

		/**
		 * adds styles to let the user know the popup is focused
		 */
		WFEditorWCRPopupConfigurator.prototype._setFocusedPopup = function () {
			// Append the popup at the end of the dom to make sure it is on top even if it is out of focus due to opening of parameter popup
			this._popup.setTitle(this._name + " (" + i18n.t("wfeditorPopup.focused") + ")");
			this._popup.element.addClass("focused");
			this._movePopupToFront(true);
		};

		/**
		 * removes the editor from the dom
		 */
		WFEditorWCRPopupConfigurator.prototype._removeEditor = function () {
			this._wFEditor.destroy();
			this._wFEditor = null;
			this._popup.element.remove();
		};

		/**
		 * opens the editor on the location of a click event
		 * @param {Event}
		 */
		WFEditorWCRPopupConfigurator.prototype.open = function (event) {
			var position = new ShapeUtils.Point();
			if ((event.pageX || (event.pageX === 0)) && (event.pageY || event.pageY === 0)) {
				position = new ShapeUtils.Point(event.pageX, event.pageY);
			} else if ((event.clientX || event.clientX === 0) && (event.clientY || event.clientY === 0)) {
				position = new ShapeUtils.Point(event.clientX, event.clientY);
			}

			// make sure the popup is visible in the screen
			var height = this._popup.element.height();
			var footerHeight = $(".wcr-page-footer").height();
			if (($(window).height() + window.pageYOffset - height) < position.y - footerHeight || ($("body").height() - height) < position.y - footerHeight) {
				position.y = Math.min($(window).height() - height + window.pageYOffset, ($("body").height() - height)) - footerHeight;
			}

			this._popup.show(position);
		};


		return WFEditorWCRPopupConfigurator;

	});
