define([
		"WFEditor/workflow/Workflow",
		"utils/webi18n!WFEditorWCR",
		"WFEditorWCR/workflow/node/endState/EndStateNode",
		"WFEditorWCR/workflow/node/startNode/StartNode",
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"utils/ShapeUtils",
		"tpl!WFEditorWCR/workflow/WebCenterWorkflow",
		"WFEditorWCR/workflow/helper/stickyNote/StickyNoteHelper",
		"WFEditorWCR/workflow/WebCenterWorkflowState",

		"underscore"

	],
	function (
		Workflow,
		i18n,
		EndStateNode,
		StartNode,
		WebCenterNode,
		WebCenterNodeConstants,
		ShapeUtils,
		WebCenterWorkflowXML,
		StickyNoteHelper,
		WebCenterWorkflowState,
		_
	) {
		"use strict";

		/**
		 * WebCenterWorkflow contains specific model data for WebCenter
		 *
		 * @exports WebCenterWorkflow
		 * @class
		 * @augments Workflow
		 *
		 * @param {String} ID
		 * @param {String} taskTypeID - ID of the tasktype
		 * @param {String} taskID - WebCenter workflow task ID
		 * @param {String initialTaskStatusID - WebCenter Task Status ID of the workflow when the object was initialized
		 */
		var WebCenterWorkflow = function (ID, taskTypeID, taskID, initialTaskStatusID) {
			Workflow.call(this, ID);

			/**
			 * @member {String} config object with information of webcenter, loaction of the images, jsppages,...
			 * @private
			 */
			this._wcrConfig = null;

			/**
			 * @member {WebCenterWorkflowState} State of the workflow
			 * @private
			 */
			this._workflowState = null;

			/**
			 * @member {String} the current status of the workflow task
			 * @private
			 */
			this._taskStatus = null;

			/**
			 * @member {String} initial Task Status ID of the workflow task
			 * @private
			 */
			this._initialTaskStatusID = initialTaskStatusID;

			/**
			 * @member {String} WebCenter workflow task ID
			 * @private
			 */
			this._taskID = taskID;

			/**
			 * @member {String} TaskType ID
			 * @private
			 */
			this._taskTypeID = taskTypeID;
		};

		WebCenterWorkflow.prototype = Object.create(Workflow.prototype);
		WebCenterWorkflow.prototype.constructor = WebCenterWorkflow;

		/**
		 * Config object with information of webcenter, loaction of the images, jsppages,...
		 * @returns {String}
		 */
		WebCenterWorkflow.prototype.getWebCenterConfig = function () {
			return this._wcrConfig;
		};

		/**
		 * Set config object with information of webcenter, loaction of the images, jsppages,...
		 * @returns {String}
		 */
		WebCenterWorkflow.prototype.setWebCenterConfig = function (wcrConfig) {
			this._wcrConfig = wcrConfig;
		};

		/**
		 * Set the workflow state
		 * @param {WebCenterWorkflowState} workflowState - New state of the workflow
		 */
		WebCenterWorkflow.prototype.setWorkflowState = function (workflowState) {
			if (this._workflowState !== workflowState) {
				this._workflowState = workflowState;
				this._emit("workflowStateChanged");
			}
		};

		/**
		 * toggles the debug mode on the wf
		 */
		WebCenterWorkflow.prototype.toggleDebugMode = function () {
			this._debugMode = !this._debugMode;
			this._emit("changed.debugMode");
		};

		/**
		 * @return {Boolean} Whether or not the workflow is in debug mode
		 */
		WebCenterWorkflow.prototype.isDebugMode = function () {
			return this._debugMode;
		};

		/**
		 * Get the workflow state
		 * @returns {WebCenterWorkflowState} The current workflow state or null if not defined (yet)
		 */
		WebCenterWorkflow.prototype.getWorkflowState = function () {
			return this._workflowState;
		};

		/**
		 * Get the WebCenter workflow task ID
		 * @returns {String} The WebCenter workflow task ID
		 */
		WebCenterWorkflow.prototype.getTaskID = function () {
			return this._taskID;
		};

		/**
		 * Get the TaskType ID of te workflow
		 * @returns {String} the WebCenter Task Type ID
		 */
		WebCenterWorkflow.prototype.getTaskTypeID = function () {
			return this._taskTypeID;
		};


		/**
		 * Set the data for the current task status
		 * @param {object} taskStatus
		 * @example
		 * 	{
		 * 		icon: "url to the icon",
		 *   	status: "name of the status"
		 *	}
		 */
		WebCenterWorkflow.prototype.setTaskStatus = function (taskStatus) {
			this._taskStatus = taskStatus;
		};

		/**
		 * Get the current task status data
		 * @returns {Object}
		 * @example
		 * 	{
		 * 		icon: "url to the icon",
		 *   	status: "name of the status"
		 *	}
		 */
		WebCenterWorkflow.prototype.getTaskStatus = function () {
			return this._taskStatus;
		};

		/**
		 * Get the initial Task Status ID
		 * @returns {String}
		 */
		WebCenterWorkflow.prototype.getInitialTaskStatusID = function () {
			return this._initialTaskStatusID;
		};

		/**
		 * Generate XML for the Workflow
		 * @returns {String} XMLString
		 */
		WebCenterWorkflow.prototype.toXML = function () {
			return WebCenterWorkflowXML(this);
		};

		/**
		 * Returns the saveable nodes, ie. All webcenter nodes
		 * @returns {Array.<Node>}
		 */
		WebCenterWorkflow.prototype.getSaveableNodes = function () {
			var nodes = _.filter(this.getNodes(), function (node) {
				return node instanceof WebCenterNode;
			});
			return nodes;
		};

		/**
		 * Returns all sticky notes
		 * @return {Array.<StickyNoteHelper>}
		 */
		WebCenterWorkflow.prototype.getStickyNotes = function () {
			return _.filter(this.getHelpers(), function (helper) {
				return helper instanceof StickyNoteHelper;
			});
		};


		/**
		 * @override
		 * Adds the nodes with errors to the active nodes
		 * @returns {Array.<Node>}
		 */
		WebCenterWorkflow.prototype.getActiveNodes = function () {
			if (this.getWorkflowState() === WebCenterWorkflowState.FINISHED) {
				return [];
			}
			var activeNodes = Workflow.prototype.getActiveNodes.call(this);
			var errorNodes = _.uniq(
				_.map(
					_.filter(this._tokens, function (token) { // Map all processed tokens
						return token.getPin() && (token.hasErrors() && token.getPin().getNode().isVisible());
					}),
					function (token) { // To their nodes
						return token.getPin().getNode();
					}
				)
			);
			return _.union(activeNodes, errorNodes);
		};

		return WebCenterWorkflow;
	});
