define([
		"WFEditor/workflow/AbstractWorkflowFactory",
		"utils/webi18n!WFEditorWCR",
		"utils/Cache",
		"utils/ShapeUtils",
		"utils/ExecutionQueue",
		"utils/HTTPUtils",
		"WFEditorWCR/workflow/WebCenterWorkflow",
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/node/MultiAttributeWebCenterNode",
		"WFEditorWCR/workflow/node/dataCollector/DataCollectorNode",
		"WFEditorWCR/workflow/node/dataSplitter/DataSplitterNode",
		"WFEditorWCR/workflow/node/errorHandler/ErrorHandlerNode",
		"WFEditorWCR/workflow/node/fork/ForkNode",
		"WFEditorWCR/workflow/node/join/JoinNode",
		"WFEditorWCR/workflow/node/tokenMarker/TokenMarkerNode",
		"WFEditorWCR/workflow/node/tokenSelector/TokenSelectorNode",
		"WFEditorWCR/workflow/node/endState/EndStateNode",
		"WFEditorWCR/workflow/node/startNode/StartNode",
		"WFEditorWCR/workflow/node/task/TaskNode",
		"WFEditorWCR/workflow/node/router/RouterNode",
		"WFEditorWCR/workflow/node/submitWorkflow/SubmitWorkflowNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/edge/WebCenterEdge",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/token/WebCenterToken",
		"WFEditorWCR/workflow/token/WebCenterOperationToken",
		"WFEditorWCR/workflow/pin/OutputPinTerminus",
		"WFEditorWCR/workflow/pin/OutputPinError",
		"WFEditorWCR/workflow/WebCenterWorkflowState",
		"WFEditorWCR/workflow/node/errorHandling/ErrorHandlingNode",
		"WFEditorWCR/core/WebCenterWFEditorConstants",
		"WFEditor/nodeGallery/model/GalleryNode",
		"WFEditorWCR/nodeGallery/WebCenterNodeGalleryConstants",
		"webcenterapi/data/WebCenterTaskType",
		"underscore",
		"jquery"
	],
	function (
		AbstractWorkflowFactory,
		i18n,
		Cache,
		ShapeUtils,
		ExecutionQueue,
		HTTPUtils,
		WebCenterWorkflow,
		WebCenterNode,
		MultiAttributeWebCenterNode,
		DataCollectorNode,
		DataSplitterNode,
		ErrorHandlerNode,
		ForkNode,
		JoinNode,
		TokenMarkerNode,
		TokenSelectorNode,
		EndStateNode,
		StartNode,
		TaskNode,
		RouterNode,
		SubmitWorkflowNode,
		WebCenterNodeConstants,
		WebCenterEdge,
		WebCenterPinConstants,
		OutputPinOK,
		WebCenterToken,
		WebCenterOperationToken,
		OutputPinTerminus,
		OutputPinError,
		WebCenterWorkflowState,
		ErrorHandlingNode,
		WebCenterWFEditorConstants,
		GalleryNode,
		WebCenterNodeGalleryConstants,
		WebCenterTaskType,
		_,
		$
	) {
		"use strict";

		/**
		 * Factory to construct the entire workflow model for the workflow editor. This factory
		 * is specific for WebCenter and fetches the correct information with calls to WebCenter.
		 *
		 * @exports WebCenterWorkflowFactory
		 * @class
		 * @augments AbstractWorkflowFactor
		 * @param {WebCenterAPI} webCenterAPI - all communication with WebCenter
		 */
		var WebCenterWorkflowFactory = function (webCenterAPI, errorHandler) {
			AbstractWorkflowFactory.call(this);

			/**
			 * @member {WebCenterAPI}
			 * @private
			 */
			this._webCenterAPI = webCenterAPI;


			/**
			 * @member {String} config object with information of webcenter, loaction of the images, jsppages,...
			 * @private
			 */
			this._wcrConfig = webCenterAPI.getWebCenterConfig();


			/**
			 * @member {Object} a map containing information on TaskExecutors
			 * @private
			 */
			this._taskExecutorData = null;

			this._errorHandler = errorHandler;

			/**
			 * @member {Map.<String, Function>} maps the node type id on the constructor of specific node model
			 * @private
			 */
			this._constructNodesMap = {};
			this._constructNodesMap[WebCenterNodeConstants.Nodes.DATACOLLECTOR] = DataCollectorNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.DATASPLITTER] = DataSplitterNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.ERRORHANDLER] = ErrorHandlerNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.TOKENSELECTOR] = TokenSelectorNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.TOKENMARKER] = TokenMarkerNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.WCTASK] = TaskNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.ROUTER] = RouterNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.SUBMITWORKFLOW] = SubmitWorkflowNode;

			/**
			 * @member {Cache} - the cache for taskType information, currently only used for outputs
			 * @private
			 */
			this._taskTypeDefinitionsCache = new Cache();
			this._taskTypeDefinitionsCache._loadItem = _.partial(function (wcrWFFactory, taskTypeId) {

				var cacheItem = {};
				cacheItem.outputs = [];

				wcrWFFactory._webCenterAPI.getTaskTypeDefinition(taskTypeId)
					.done(_.bind(function (xml) {
						var taskType = xml.querySelector("wctask_type_view[id='" + taskTypeId + "'] wctask_type");
						if (taskType) {
							cacheItem.taskType = WebCenterTaskType.fromXML(taskType);

							var statusViews = xml.querySelectorAll("wctask_type_view[id='" + taskTypeId + "'] status_list task_type_status_view");
							var completedStatusNumber = wcrWFFactory.getCompletedStatusNumber(statusViews);
							_.each(statusViews, function (statusView) {
								var status = statusView.querySelector("wc_status");
								var displayOrder = parseInt(status.getElementsByTagName("display_order")[0].textContent);
								if (displayOrder >= completedStatusNumber) {
									var outputId = status.getAttribute("id");
									var outputStatusName = status.getElementsByTagName("status_name")[0].textContent;
									cacheItem.outputs.push({
										id: outputId,
										name: outputStatusName
									});
								}
							});
							this._registerItem(taskTypeId, cacheItem);
						} else {
							this._registerItem(taskTypeId);
						}
					}, this)).fail(_.bind(function () {
						this._registerItem(taskTypeId);
					}, this));

			}, this);

			// synchronous cache for quickly finding icons for tasktypes (only search them once)
			this._taskIconFinder = new Cache(50, false);
			this._taskIconFinder._loadItem = _.partial(function (wcrWfFactory, name) {
				var icon = wcrWfFactory._tasktypeDefinitionXML.querySelector("wctask_type_view[id='" + name + "'] wctask_type icon");

				if (icon) {
					this._registerItem(name, icon.textContent);
				}
			}, this);

			// synchronous cache for quickly finding labels for tasktypes (only search them once)
			this._taskNameFinder = new Cache(50, false);
			this._taskNameFinder._loadItem = _.partial(function (wcrWfFactory, id) {
				var name = wcrWfFactory._tasktypeDefinitionXML.querySelector("wctask_type_view[id='" + id + "'] wctask_type task_type_name");

				if (name) {
					this._registerItem(id, name.textContent);
				}
			}, this);
		};

		WebCenterWorkflowFactory.prototype = Object.create(AbstractWorkflowFactory.prototype);
		WebCenterWorkflowFactory.prototype.constructor = WebCenterWorkflowFactory;

		/**
		 * Creates the workflow from two xml files.
		 * @param {String} taskId - id of the task or task to be loaded.
		 * @param {String} taskTypeId - id of the task or task type to be loaded.
		 * @param {Map<String, Any>} privileges - The privileges for the workflows
		 * @param   {String} workflow - the description of the workflow in xml-format
		 * @param   {String} workflowDefinition - the definition of all the nodes the webcenter workflow can have.
		 * @param 	{String} taskTypeDefinitions - the definition of all the task types in the system
		 * @param 	{String} currentTaskTypeDefinition - the definition of current WF task type with more details (verbose)                                     
		 * @ param {String} [initialTaskStatusID - the initial TaskStatus ID of the workflow
		 * @returns {Workflow} Returns the model of the enitre workflow.
		 */
		WebCenterWorkflowFactory.prototype.createWorkflow = function (taskTypeId, taskId, privileges, workflowXml, workflowDefinition, taskTypeDefinitions, currentTaskTypeDefinition, initialTaskStatusID) {
			// reset task executor data
			this._taskExecutorData = null;

			this._taskTypeId = taskTypeId;
			var workflowID = taskTypeId;
			if (taskId) {
				workflowID += "_" + taskId;
			}

			this._workflowDefinitionXML = workflowDefinition;
			this._tasktypeDefinitionXML = taskTypeDefinitions;
			this._currentTasktypeDefinitionXML = currentTaskTypeDefinition;

			var workflow = new WebCenterWorkflow(workflowID, taskTypeId, taskId, initialTaskStatusID);
			workflow.setPrivileges(privileges);
			workflow.setWebCenterConfig(this._wcrConfig);

			//the location to find most icons (not for specific task types)
			var iconLocation = this._wcrConfig.requireBaseUrl + "WFEditorWCR/icons/";

			var resources;
			var layout;
			var stickyNotes;
			var icon;
			var newNode;
			var nodeID;
			var transitionsMap = {};

			// split up work in smalle chunks
			var queue = new ExecutionQueue();

			try {
				var rootNode = workflowXml.documentElement;

				// run over all child nodes
				queue.forEach(rootNode.childNodes, function (workflowEl, i) {
					newNode = null;
					// run through all children of the XML document, ignore text elements

					if (workflowEl.nodeType !== 1 /*Node.ELEMENT_NODE*/ ) {
						return;
					}

					nodeID = workflowEl.getAttribute("name");
					switch (workflowEl.tagName.toLowerCase()) {
					case 'node':
						// find the type of the node
						var actionEls = workflowEl.getElementsByTagName("action");
						if (actionEls.length >= 1) {
							var actionEl = actionEls[0];
							var nodeType = actionEl.getAttribute("class");
							var enabled = true;

							var taskExecutorData = this._getTaskExecutorData(nodeType, workflowDefinition);
							if (this._constructNodesMap[nodeType]) {
								// this Node has a specific implementation in WFEditorWCR

								icon = iconLocation;
								if (nodeType == WebCenterNodeConstants.Nodes.WCTASK) {
									// for WebCenter tasks we need to find the correct icon (using the taskIconFinder)
									var taskTypeEl = actionEl.getElementsByTagName("wctask_type");
									if (taskTypeEl[0]) {
										var taskTypeId = taskTypeEl[0].getAttribute("id");
										var taskTypeDef = taskTypeDefinitions.querySelector("wctask_type_view[id='" + taskTypeId + "'] wctask_type[id='" + taskTypeId + "']");

										var taskType = null;

										if (taskTypeDef) {
											taskType = WebCenterTaskType.fromXML(taskTypeDef);
										} else {
											taskType = new WebCenterTaskType(taskTypeId);
											enabled = false;
										}

										icon = this._wcrConfig.jspBaseUrl + taskType.getIcon();
										newNode = new this._constructNodesMap[nodeType](workflow, nodeID, icon, "", taskType, this._webCenterAPI);
									}
								} else {
									// Check if there is an icon for the node
									if (taskExecutorData && taskExecutorData.icon) {
										icon = iconLocation + taskExecutorData.icon;
									}
									// construct the node
									newNode = new this._constructNodesMap[nodeType](workflow, nodeID, icon, "", this._webCenterAPI);
								}
							} else {
								// no specific implementation exists for the Node (it is an ""AutoTask"")

								if (taskExecutorData) {
									if (taskExecutorData.icon) {
										icon = iconLocation + taskExecutorData.icon;
									} else {
										icon = undefined;
									}
									enabled = taskExecutorData.enabled;
								} else {
									icon = undefined;
									enabled = false;
								}

								// construct a WebCenter node with the correct type and icon
								newNode = this._getNodeOfCorrectType(workflow, nodeID, nodeType, icon, "", this._webCenterAPI);

							}
							// set it enabled or not
							newNode.setEnabled(enabled);
							//get all edges leaving the pins of this node
							this._getTransitions(workflowEl, nodeID, transitionsMap);

						}
						break;

					case WebCenterNodeConstants.Nodes.FORK:
						newNode = new ForkNode(workflow, nodeID, iconLocation, "", this._webCenterAPI);
						//get all edges leaving the pins of this node
						this._getTransitions(workflowEl, nodeID, transitionsMap);
						break;
					case WebCenterNodeConstants.Nodes.JOIN:
						newNode = new JoinNode(workflow, nodeID, iconLocation, "", this._webCenterAPI);
						//get all edges leaving the pins of this node
						this._getTransitions(workflowEl, nodeID, transitionsMap);
						break;
					case 'end-state':

						var type = workflowEl.getAttribute("type");
						if (!type) { //type is not defined in the XML, this means it is a completed end-state
							type = WebCenterNodeConstants.Nodes.COMPLETED;
						}

						newNode = new EndStateNode(workflow, nodeID, iconLocation, i18n.t(type), type, this._webCenterAPI);

						break;
					case 'start-state':
						newNode = new StartNode(workflow, nodeID, iconLocation, "", this._webCenterAPI);
						//get all edges leaving the pins of this node
						this._getTransitions(workflowEl, nodeID, transitionsMap);
						break;
					case 'resources':
						// store the resources XML for later usage
						resources = workflowEl;
						break;
					case 'layout':
						// store the layout XML for later usage
						layout = workflowEl;
						break;
					case 'extra':
						// check for sticky notes
						stickyNotes = workflowEl.getElementsByTagName("stickynotes")[0];
						break;

					}

					if (newNode) {
						// let the node process its XML and add it to the workflow
						newNode.setNodeXML(workflowEl);
						workflow.addNode(newNode);
					}



				}, this, "WebCenterWorkflowFactoryAnalyzeNodes", i18n.t("initializingNodes"));

				queue.run(function () {
					// process resource information
					if (resources) {
						queue.forEach(resources.childNodes, function (xmlElement) {
							this._handleResourceElement(xmlElement, workflow);
						}, this, "WebCenterWorkflowFactoryAnalyzeResources", i18n.t("initializingNodes"));
					}


					// process layout information (node locations and labels)
					if (layout) {
						queue.forEach(layout.childNodes, function (xmlElement) {
							this._handleLayoutElement(xmlElement, workflow);
						}, this, "WebCenterWorkflowFactoryAnalyzeLayout", i18n.t("initializingLayout"));
					}

					// Create sticky notes
					if (stickyNotes) {
						queue.forEach(stickyNotes.childNodes, function (xmlElement) {
							if (xmlElement.tagName === "stickynote") {
								this._handleStickyNoteElement(xmlElement, workflow);
							}
						}, this, "WebCenterWorkflowFactoryCreateStickyNotes");
					}


					queue.forEach(transitionsMap, function (transitionObj, from) {
						var fromNode = workflow.getNode(transitionObj.nodeID);
						var toNode = workflow.getNode(transitionObj.to);
						if (fromNode && toNode) {
							var fromPin = fromNode.getPin(_.first(from.split("/")));
							var toPin = toNode.getPin(toNode.getId() + WebCenterPinConstants.Type.In);
							if (fromPin && toPin) {
								var edge = this.createEdge(workflow, from, fromPin, toPin);
								fromPin.addEdge(edge);
								toPin.addEdge(edge);
								if (!fromPin.getEdgeTo(toPin)) {
									// the edge hasn't been added because the to or from pin doesn't accept the edge
									// This is a legacy issue, add it anyway. 
									fromPin.getEdges().push(edge);
									toPin.getEdges().push(edge);
								}
								workflow.addEdge(edge);
								if (toNode.isImplicit && toNode.isImplicit()) {
									// make the edge implicit
									edge.setImplicit(true);
								}

							}
						}
					}, this, "WebCenterWorkflowFactoryInitializeEdges", i18n.t("initializingConnections"));


					queue.run(function () {
						// If the workflow model doesn't have any nodes, create the default workflow items
						if (_.size(workflow.getNodes()) === 0) {

							var startNode = new StartNode(workflow, "InputCondition_1", iconLocation, null, this._webCenterAPI);
							var endNode = new EndStateNode(workflow, "00001_0000000003_3", iconLocation, i18n.t("Completed"), WebCenterNodeConstants.Nodes.COMPLETED, this._webCenterAPI);
							var errorImplicit = new EndStateNode(workflow, "errorcontinue_implicit", iconLocation, "", WebCenterNodeConstants.Nodes.ERRORCONTINUE, this._webCenterAPI);
							// Set locations of the nodes
							startNode.setLocation(new ShapeUtils.Point(WebCenterNodeConstants.DefaultStartNode.DefaultX, WebCenterNodeConstants.DefaultStartNode.DefaultY));
							endNode.setLocation(new ShapeUtils.Point(WebCenterNodeConstants.DefaultCompletedNode.DefaultX, WebCenterNodeConstants.DefaultCompletedNode.DefaultY));
							errorImplicit.setImplicit(true);

							workflow.addNode(errorImplicit);
							workflow.addNode(startNode);
							workflow.addNode(endNode);
						}
					}, this);
				}, this);

				var deferred = $.Deferred();

				// wait until operations are finished and call callback
				queue.whenFinished(function () {
					deferred.resolve(workflow);
				});

				return deferred.promise();
			} catch (e) {
				this._errorHandler.severe(i18n.t("CreateWorkflowFailed"), e, workflowXml, workflowDefinition, taskTypeDefinitions);

				var deferred = $.Deferred();
				deferred.reject(e);

				return deferred.promise();
			}

		};


		/**
		 * Get the data from the WorkflowDefinition XML for a specific executorType
		 * @param {String} executorType - type of the executer
		 * @param {Document} workflowDefinition - XML document with the workflowDefinitions
		 * @returns {Object} object containing task executor information (limited to icon and enabled for now) (or undefined if not available)
		 */
		WebCenterWorkflowFactory.prototype._getTaskExecutorData = function (executorType, workflowDefinition) {
			if (!this._taskExecutorData) {
				// if the taskExecuterData is not yet loaded, pars it for all TaskExecuters
				this._taskExecutorData = {};

				var taskExecuters = workflowDefinition.getElementsByTagName("TaskExecuterFactory");
				for (var j = 0; j < taskExecuters.length; j++) {
					var nameEl = taskExecuters[j].getElementsByTagName("type")[0];
					var icon = taskExecuters[j].querySelector("config-inline icon");

					var enabled = true;
					var enabledEl = taskExecuters[j].querySelector("config-inline enabled");
					if (enabledEl) {
						enabled = enabledEl.textContent == "1";
					}

					if (icon) {
						icon = _.last(icon.textContent.split("/"));
					}
					this._taskExecutorData[nameEl.textContent] = {
						icon: icon,
						enabled: enabled
					};
				}

			}

			return this._taskExecutorData[executorType];
		};

		/**
		 * Handle the Layout XML (set node locations and labels)
		 * @param {Element} wfElement - XML element with the layout information for a single workflowitem
		 * @param {Workflow} workflow - the workflow for which the layout is being handled
		 */
		WebCenterWorkflowFactory.prototype._handleLayoutElement = function (wfElement, workflow) {
			// ignore text nodes and only process wfelement tags
			if (wfElement.nodeType === 1 && wfElement.tagName.toLowerCase() == "wfelement") {
				var ID = wfElement.getAttribute("name");
				var node = workflow.getNode(ID);

				if (node) {
					// the node exists, it should handle its layoutXML
					node.setLayoutXML(wfElement);
				}
			}
		};


		/**
		 * Handle the Resources XML
		 * @param {Element} resources - XML element with the resource information for a single workflowitem
		 * @param {Workflow} workflow - the workflow for which the resources are being handled
		 */
		WebCenterWorkflowFactory.prototype._handleResourceElement = function (resource, workflow) {

			// ignore text node and only process resource elements
			if (resource.nodeType === 1 && resource.tagName.toLowerCase() == "resource") {
				var ID = resource.getAttribute("id");
				var node = workflow.getNode(ID);

				if (node) {
					// the node exists, it should handle its resource
					node.setResourceXML(resource);
				}
			}

		};

		/**
		 * Handle the Resources XML
		 * @param {Element} resources - XML element with the information for a single stickynote
		 * @param {Workflow} workflow
		 */
		WebCenterWorkflowFactory.prototype._handleStickyNoteElement = function (stickyNote, workflow) {
			requirejs(["WFEditorWCR/workflow/helper/stickyNote/StickyNoteHelper"], function (StickyNoteHelper) {
				workflow.addHelper(new StickyNoteHelper(workflow,
					new ShapeUtils.Point(parseInt(stickyNote.getAttribute("preflocx")), parseInt(stickyNote.getAttribute("preflocy"))),
					new ShapeUtils.Size(parseInt(stickyNote.getAttribute("width")), parseInt(stickyNote.getAttribute("height"))),
					stickyNote.textContent));
			});
		};

		/**
		 * Get all the transition out of the xml-object, workflowEl is the xml-object which represents a node.
		 * It looks for all the 'transition'-child nodes.
		 * @param {DOMObject} workflowEl
		 * @param {String} nodeID
		 * @param {Map.<String, Object>} transitionsMap - Map with all the transition, the method adds newly found transition to this collection.
		 */
		WebCenterWorkflowFactory.prototype._getTransitions = function (workflowEl, nodeID, transitionsMap) {
			var transitions = workflowEl.getElementsByTagName("transition");
			_.each(transitions, function (transition) {
				var from = transition.getAttribute("name");
				var to = transition.getAttribute("to");
				if (to !== WebCenterNodeConstants.Nodes.ERRORCONTINUEIMPLICIT) {
					// Don't add edges to errorcontinue_implicit, these will be added on save
					transitionsMap[from] = {
						'nodeID': nodeID,
						'to': to
					};
				}
			}, this);
		};

		/**
		 * Create workflow feedback, creates tokens and adds those tokens to the correct pins.
		 * @param {XMLObject} workflowReportEl
		 * @param {XMLObject} currentTaskTypeDefinitionEl
		 * @param {Workflow} workflow
		 */
		WebCenterWorkflowFactory.prototype.createWorkflowFeedback = function (workflowReportEl, currentTaskTypeDefinitionEl, workflow) {
			// Determine the workflow state
			var audit = workflowReportEl.getElementsByTagName("audit");
			var taskStatus = null;
			if (audit && audit.length > 0) {
				var wfFinished = audit.item(0).getAttribute("finished");

				var wf = audit.item(0).getElementsByTagName("wf");
				if (wf && wf.length > 0) {
					taskStatus = wf.item(0).getAttribute("taskstatus");
				}

				if (wfFinished == "1") { // == intentional
					workflow.setWorkflowState(WebCenterWorkflowState.FINISHED);
				} else if (wf && wf.length > 0 && workflow.getWorkflowState() !== WebCenterWorkflowState.STOPPING) {
					var paused = wf.item(0).getAttribute("paused");

					if (paused == "1") { // == intentional
						workflow.setWorkflowState(WebCenterWorkflowState.PAUSED);
					} else {
						workflow.setWorkflowState(WebCenterWorkflowState.RUNNING);
					}
				}

			} else {
				// there was no audit trail.. check if there was an sdk error
				var errorTag = _.first(workflowReportEl.getElementsByTagName("error"));
				if (errorTag) {
					var errorCode = _.first(errorTag.getElementsByTagName("code"));
					if (errorCode && errorCode.textContent == "154") {
						// no workflow report available, fall back on the initial task status id of the workflow
						taskStatus = workflow.getInitialTaskStatusID();
					}
				}
			}

			// reset all parent tokens
			_.invoke(workflow.getTokens(), "removeParentTokens");

			var tokens = workflowReportEl.getElementsByTagName("tokens"); //get the tokens-element
			var tokenMap = {}; //map to keep track of all created tokens, token ID mapped on Token object
			//maps parent token ID on the token that still needs a parent token
			var waitingForParent = {}; //map to keep track of all tokens that still need one of the parent tokens to be added

			if (tokens && tokens[0] && tokens[0].childNodes.length !== 0) {
				// create the init Token
				var initToken = new WebCenterToken(workflow, undefined, "init", null, null, true);
				tokenMap["init"] = initToken;
			}


			// a map to keep track of all processed states of the tokens, used to call setProcessed for each token after parents were added
			var processedMap = {};

			var errorHandlingWaitingForParent = {};
			try {
				//run over all the children of the tokens element, all place elements = each pin element with tokens
				for (var i = 0; tokens && tokens[0] && i < tokens[0].childNodes.length; i++) {
					// run through all children of the XML document, ignore text elements
					var placeEl = tokens[0].childNodes[i];
					if (placeEl.nodeType !== 1 /*Node.ELEMENT_NODE*/ ) {
						continue; //make sure to only handle element nodes
					}

					var nodeId = placeEl.getAttribute("node"); //ID of the node the pin belongs to
					var pinId = placeEl.getAttribute("channel"); //ID of the pin the token belongs to
					//Lets check if the token belongs to an error handling node
					var parentNodeId = nodeId.split("_");
					if (parentNodeId.pop() === "errorhandling") { //last part of the node id is equal to errorhandling
						var errorHandlingNode = workflow.getNode(nodeId); //get the error handling node
						var parentNode = workflow.getNode(parentNodeId.join("_"));
						if (!errorHandlingNode && parentNode) { //if the node is not yet present in the workflow, construct it, this way it only gets constructed when used
							errorHandlingNode = new ErrorHandlingNode(workflow, nodeId, this._wcrConfig.requireBaseUrl + "WFEditorWCR/icons/", null, this._webCenterAPI);
							workflow.addNode(errorHandlingNode);
							//now construct the transitions between the node and the error handling node
							//first from the node to the error handling node
							var fromPin = parentNode.getErrorPin();
							var toPin = errorHandlingNode.getInputPin();
							var edge = this.createEdge(workflow, fromPin.getId() + "_" + toPin.getId(), fromPin, toPin);
							fromPin.addEdge(edge);
							toPin.addEdge(edge);
							//now from the error handling node back to the main node
							fromPin = errorHandlingNode.getPin(errorHandlingNode.getId() + WebCenterPinConstants.Type.OutOK);
							toPin = parentNode.getInputPin();
							edge = this.createEdge(workflow, fromPin.getId() + "_" + toPin.getId(), fromPin, toPin);
							fromPin.addEdge(edge);
							toPin.addEdge(edge);
						}
					}
					var node = workflow.getNode(nodeId); //get the node from the workflow
					if (node) { // if the node can be found back
						var endPin = null;
						var pin = node.getPin(nodeId + pinId); //get the correct pin from the node
						//if it's an input pin or tokens that are generated internally add them to the input pin
						if (pinId === WebCenterPinConstants.FeedbackTypes.Input || pinId === WebCenterPinConstants.FeedbackTypes.Internal) {
							pin = node.getInputPin();
						} else if (pinId === WebCenterPinConstants.FeedbackTypes.Terminus) {
							//only when the end-state node has tokens in its 'output' pin add this pin to the node
							pin = node.getPin(nodeId + WebCenterPinConstants.Type.Terminus);
							if (!pin) {
								if (node.getNodeType() === WebCenterNodeConstants.Nodes.CANCEL) {
									endPin = new OutputPinError(workflow, node, WebCenterPinConstants.Type.Terminus, node.getLabel());
								} else {
									endPin = new OutputPinTerminus(workflow, node, WebCenterPinConstants.Type.Terminus, node.getLabel());
								}
								node.addPin(endPin);
								pin = node.getInputPin();
							}
						}

						// If the pin is actually a warning pin that isn't displayed, the token should be shown at the Error or OK pin
						if (!pin && node.canEditWarningPolicy() && _.contains(node.getWarningLabels(), pinId)) {

							if (node.getWarningPolicyString() === WebCenterNodeConstants.WarningPolicy.OK) {
								// Warning Policy === OK so show the token at the OK pin
								pin = _.find(node.getPins(), function (pin) {
									return pin instanceof OutputPinOK;
								});
							} else if (node.getWarningPolicyString() === WebCenterNodeConstants.WarningPolicy.ERROR) {
								// Warning Policy === ERROR so show the token at the ERROR pin
								pin = node.getErrorPin();
							}

						}

						if (pin) { //there is a pin found to add the token to
							_.each(placeEl.childNodes, function (tokenEL) { //run over all the tokens that are part of the pin
								if (!tokenEL || tokenEL.nodeType !== 1 /*Node.ELEMENT_NODE*/ ) {
									return;
								}
								var tokenId = tokenEL.getAttribute("id");
								var imported = tokenEL.getAttribute("import");

								// Has the token been processed yet?
								var processed = tokenEL.getAttribute("processed") !== "0" || wfFinished == "1"; // == intentional
								if (imported && imported === "1") {
									return;
								}
								//there can be multiple parents for one token (e.g. fork/join nodes)
								var parentIds = [tokenEL.getAttribute("parentid")];
								if (pinId === WebCenterPinConstants.FeedbackTypes.Internal) {
									//there are multiple parent possible for internally generated tokens
									parentIds = [];
									var parentsEL = tokenEL.getElementsByTagName("parents");
									_.each(parentsEL[0].childNodes, function (parentEL) {
										if (!parentEL || parentEL.nodeType !== 1 /*Node.ELEMENT_NODE*/ ) {
											return;
										}
										parentIds.push(parentEL.getAttribute("id"));
									}, this);
								}
								//a clone ID is present when there are multiple edges leaving the pin, tokens get cloned for each edge
								var cloneId = null;
								if (tokenEL.hasAttribute("cloneid")) {
									cloneId = tokenEL.getAttribute("cloneid");
								}


								var tokenData = tokenEL.getElementsByTagName("tokendata"); //docuemnts and extra information of the token


								var token = workflow.getToken(tokenId) || _.find(pin.getTokens(), function (token) {
									return token.getId() === tokenId;
								});
								if (!token) {
									// token doesn't exist yet, create it... (parent nodes will be set later on)
									token = new WebCenterToken(workflow, pin, tokenId, tokenData[0], cloneId, processed);
								} else {
									// update the token with the new token data
									token.updateFromXMLData(tokenData[0]);
								}

								// If there is an operation ID, a dedicated "OperationToken" is created. 
								// This OperationToken will aggregate all tokens in the operation
								var opId;
								if (tokenEL.hasAttribute("opid")) {
									opId = tokenEL.getAttribute("opid"); // Get the id of the operation
									// check if the operation is in the workflow or in the token map
									var opToken = workflow.getToken(opId) ? workflow.getToken(opId) : tokenMap[opId];


									// An 'R' opstate attribute indicates that the operation is in process
									if (tokenEL.getAttribute("opstate") !== 'R') {
										processed = true;
									} else {
										processed = false;
									}

									if (!opToken) {
										// Not yet created, create a new one
										opToken = new WebCenterOperationToken(workflow, pin, opId, tokenData[0], cloneId, processed);
										node.addOperationToken(opToken);
									}


									// If this operation doesn't contain the token yet, add it
									if (!opToken.hasSubToken(token)) {
										opToken.addSubToken(token);
									}

									processedMap[opToken.getId()] = processed;
									tokenMap[opId] = opToken; // add it the operation to the token map
									pin.addToken(opToken);
									// Tokens in OperationTokens are aggregated by the operationtoken and should be considered processed
									processed = true;

								} else if ((node.getNodeType() === WebCenterNodeConstants.Nodes.DATACOLLECTOR ||
										node.getNodeType() === WebCenterNodeConstants.Nodes.JOIN) &&
									pin.getPinType() === WebCenterPinConstants.Type.In) {
									// The node is an operation node but there is no opid, this happens when all operations in the node have been finished
									// -> set all operation tokens in the nodes to processed and remove them from the operation node
									_.each(node.getOperationTokens(), function (opToken) {
										processedMap[opToken.getId()] = true;
										node.removeOperationToken(opToken);
									});
								}

								// If the processed status hasn't been defined by now, it can be seen as processed
								if (processed === undefined || processed === null) {
									processed = true;
								}
								pin.addToken(token);
								processedMap[token.getId()] = processed;
								if (pinId === WebCenterPinConstants.FeedbackTypes.Terminus && endPin) {
									endPin.addToken(token);
								}
								//keep track of the created tokens by adding them to the tokenMap
								tokenMap[tokenId] = token;
								//run over all the parent id and check if they are already created
								_.each(parentIds, function (parentId) {
									if (parentId) {
										var parentToken = tokenMap[parentId];
										if (parentToken) {
											token.addParent(parentToken);
											if (nodeId && nodeId.split("_").pop() === "errorhandling") { //last part of the node id is equal to errorhandling
												parentToken.setErrorHandlingId(tokenId);
											}
										} else { //parent is not yet created add it to the map to be added later on
											if (!waitingForParent[parentId]) {
												waitingForParent[parentId] = [];
											}
											waitingForParent[parentId].push(token);
											if (nodeId && nodeId.split("_").pop() === "errorhandling") {
												errorHandlingWaitingForParent[parentId] = tokenId;
											}
										}
									}
								}, this);
								//check if there are tokens that have the newly created token as parent
								var tokenArray = waitingForParent[tokenId];
								_.each(tokenArray, function (childToken) {
									childToken.addParent(token);
								}, this);
								//each token can only go to one error handling node, so no need for foreach loop
								var errorHandlingId = errorHandlingWaitingForParent[tokenId];
								if (errorHandlingId) { //add error handling id if present
									token.setErrorHandlingId(errorHandlingId);
								}
							}, this);
						}
					}

				}

				_.each(processedMap, function (isProcessed, tokenId) {
					var token = tokenMap[tokenId] || workflow.getToken(tokenId);
					token.setProcessed(isProcessed);
				});
				workflow.setTokens(tokenMap);

				//start parsing the currentTaskTypeDefinitionEl to get the workflow status
				var statusListEl = currentTaskTypeDefinitionEl.getElementsByTagName("status_list");
				if (statusListEl && statusListEl[0] && taskStatus !== null && workflow.getWorkflowState() !== WebCenterWorkflowState.STOPPING) {
					var statusInfo = $(statusListEl).find("wc_status[id='" + taskStatus + "']");

					if (workflow.getWorkflowState() === WebCenterWorkflowState.PAUSED) {
						workflow.setTaskStatus({
							status: i18n.t("workflowStatusOnHold"),
							icon: null
						});
					} else if (statusInfo && statusInfo[0]) {
						var statusName = statusInfo[0].getElementsByTagName("status_name");
						var iconName = statusInfo[0].getElementsByTagName("icon_name");

						if (statusName && iconName && statusName[0] && iconName[0]) {
							workflow.setTaskStatus({
								status: statusName[0].textContent,
								icon: iconName[0].textContent && this._wcrConfig.jspBaseUrl + "images/statuses/" + iconName[0].textContent
							});
						}
					} else if (taskStatus === WebCenterWFEditorConstants.WorkflowErrorStatus) {
						workflow.setTaskStatus({
							status: i18n.t("workflowStatusError"),
							icon: this._wcrConfig.jspBaseUrl + "images/statuses/" + WebCenterWFEditorConstants.WorkflowErrorImage
						});
					} else if (taskStatus === WebCenterWFEditorConstants.WorkflowCanceledStatus) {
						workflow.setTaskStatus({
							status: i18n.t("workflowStatusCanceled"),
							icon: this._wcrConfig.jspBaseUrl + "images/statuses/" + WebCenterWFEditorConstants.WorkflowCanceledImage
						});
					}

				}

			} catch (e) {
				this._errorHandler.severe(i18n.t("CreateWorkflowFeedbackFailed"), e, workflowReportEl);
			}
		};

		/**
		 * Returns the definitions of the workflow construct nodes that should be shown in the NodeGallery
		 * @param {String} pathToIcons - the url to the folder where the icons can be found
		 * @return {Array.<GalleryNode>}
		 */
		WebCenterWorkflowFactory.prototype._getWFConstructNodeDefintions = function (pathToIcons) {
			var statusNodes = [];
			var newNode;

			_.each(WebCenterNodeGalleryConstants.WorkflowConstructNodeTypes, function (type) {
				var icon = pathToIcons + WebCenterNodeConstants.Icons[type];
				var options = {
					nodeType: WebCenterNodeConstants.Nodes[type]
				};
				if (type === "ROUTER") {
					options.taskTypeId = "genPropertyFilter";
					var taskExecutorData = this._getTaskExecutorData("bsroute", this._workflowDefinitionXML);
					if (taskExecutorData && taskExecutorData.icon) {
						icon = pathToIcons + taskExecutorData.icon;
					}
				}
				newNode = new GalleryNode(WebCenterNodeConstants.Nodes[type], icon, WebCenterNodeGalleryConstants.ListSelections.WFCONSTRUCT);
				newNode.setOptions(options);

				statusNodes.push(newNode);
			}, this);
			return statusNodes;
		};

		/**
		 * Returns the display order of the completed status
		 * @param   {NodeList} statuses - XML NodeList of status elements
		 * @returns {Number}
		 */
		WebCenterWorkflowFactory.prototype.getCompletedStatusNumber = function (statuses) {
			var completedOrderNr = 0;
			for (var i = 0; i < statuses.length; i++) {
				var wcstatus = statuses[i].querySelector("wc_status");
				var name = wcstatus.querySelector("status_name").textContent;
				if (name === "Completed") {
					completedOrderNr = parseInt(wcstatus.querySelector("display_order").textContent);
					break;
				}
			}
			return completedOrderNr;
		};

		/**
		 * Returns the definitions of the status nodes that should be shown in the NodeGallery.
		 * @param {String} pathToIcons - the url to the folder where the icons can be found
		 * @param {NodeList} statuses - XML NodeList of status elements
		 * @return {Array.<GalleryNode>}
		 */
		WebCenterWorkflowFactory.prototype._getStatusNodeDefintions = function (pathToIcons, statuses) {
			var statusNodes = [];
			var newNode;
			var wcstatus;
			var name;

			//only show statuses that have a display order >= 'Completed', so first need to find that display order number
			var completedOrderNr = this.getCompletedStatusNumber(statuses);
			newNode = new GalleryNode(WebCenterNodeConstants.Nodes.COMPLETED, pathToIcons + WebCenterNodeConstants.Icons.COMPLETED, WebCenterNodeGalleryConstants.ListSelections.STATUS);
			newNode.setOptions({
				id: WebCenterNodeConstants.Status.COMPLETED,
				nodeType: WebCenterNodeConstants.Nodes.COMPLETED
			});
			statusNodes.push(newNode);

			//make GalleryNodes if the display order is > completed display order
			for (var j = 0; j < statuses.length; j++) {
				wcstatus = statuses[j].querySelector("wc_status");
				var orderNr = parseInt(wcstatus.querySelector("display_order").textContent);
				if (orderNr <= completedOrderNr) {
					continue;
				}

				name = wcstatus.querySelector("status_name").textContent;
				var id = wcstatus.getAttribute("id");
				newNode = new GalleryNode(name, pathToIcons + WebCenterNodeConstants.Icons.COMPLETED, WebCenterNodeGalleryConstants.ListSelections.STATUS);
				newNode.setOptions({
					nodeType: WebCenterNodeConstants.Nodes.COMPLETED,
					id: id,
					name: name
				});
				statusNodes.push(newNode);
			}

			return statusNodes;
		};

		/**
		 * Returns the definitions of all the nodes that should be shown in the NodeGallery.
		 * @Override
		 * @return {Array.<GalleryNode>}
		 */
		WebCenterWorkflowFactory.prototype.getNodeDefinitions = function () {
			var nodes = [];
			var iconLocation = this._wcrConfig.requireBaseUrl + "WFEditorWCR/icons/";
			nodes = nodes.concat(this._getWFConstructNodeDefintions(iconLocation));

			if (this._workflowDefinitionXML && this._tasktypeDefinitionXML) {
				var newNode;

				var taskExecuters = this._workflowDefinitionXML.getElementsByTagName("TaskExecuterFactory");
				for (var j = 0; j < taskExecuters.length; j++) {
					var nodeType = (taskExecuters[j].getElementsByTagName("type")[0]).textContent;
					var visibility = taskExecuters[j].querySelector("config-inline visible");
					var enabled = taskExecuters[j].querySelector("config-inline enabled");
					if (visibility.textContent !== "1" || enabled.textContent !== "1") {
						continue;
					}
					var icon = taskExecuters[j].querySelector("config-inline icon");
					var path = null;
					if (icon) {
						path = this._wcrConfig.jspBaseUrl + icon.textContent;
					}
					newNode = new GalleryNode(nodeType, path, WebCenterNodeGalleryConstants.ListSelections.AUTOTASK);
					newNode.setOptions({
						nodeType: nodeType
					});
					nodes.push(newNode);
				}

				var taskTypes = this._tasktypeDefinitionXML.getElementsByTagName("wctask_type_view");
				_.each(taskTypes, function (task) {
					var id = task.getAttribute("id");
					//if taskId is the same as the current loaded workflow-id, add the statuses in the gallery, but don't add it
					if (id !== this._taskTypeId) {
						var icon = task.querySelector("wctask_type icon");
						var path = null;
						var name = task.querySelector("wctask_type task_type_name").textContent;
						if (icon) {
							path = this._wcrConfig.jspBaseUrl + icon.textContent;
						}
						newNode = new GalleryNode(name, path, WebCenterNodeGalleryConstants.ListSelections.TASKTYPE);
						newNode.setOptions({
							nodeType: WebCenterNodeConstants.Nodes.WCTASK,
							taskTypeId: id
						});
						nodes.push(newNode);
					}
				}, this);

				var currentTaskType = this._currentTasktypeDefinitionXML.getElementsByTagName("wctask_type_view");
				//if taskId is the same as the current loaded workflow-id, add the statuses in the gallery, but don't add it
				if (currentTaskType && currentTaskType[0]) {
					var statuses = currentTaskType[0].querySelectorAll("status_list task_type_status_view");
					nodes = nodes.concat(this._getStatusNodeDefintions(iconLocation, statuses));
				}
			}
			return nodes;
		};

		/**
		 * @override
		 */
		WebCenterWorkflowFactory.prototype.createEdge = function (workflow, id, fromPin, toPin) {
			return new WebCenterEdge(workflow, id, fromPin, toPin);
		};

		/**
		 * @override
		 * Possible options are:
		 * nodeType: the type of the node to be created
		 * [id]: sub id of the nodeType. eg. if nodeType is "Completed" the id is the correct type of completed status
		 * [taskTypeId]: id of the task type.
		 * [name]: used to generate a new label, a suffix will be added if a node with the same label exists
		 * [label]: if label is present, this will be the label of the new node, even if another node with the same label exists
		 * [configurationXML]: used to set the parameters of the newly created node
		 */
		WebCenterWorkflowFactory.prototype.createNode = function (workflow, options) {
			var deferred = $.Deferred();
			var promise = deferred.promise();
			var nodeTypeId = options.taskTypeId || options.id || "";
			var nodeType = options.nodeType;

			if (options.configurationXML) {
				promise.done(function (node) {
					var xmlDoc = $.parseXML(options.configurationXML);
					var rootNode = xmlDoc.firstChild;
					node.setNodeXML(rootNode);
				});
			}

			if (options.resourceXML) {
				promise.done(function (node) {
					var resourceDoc = $.parseXML(options.resourceXML);
					var rootNode = resourceDoc.firstChild;
					node.setResourceXML(rootNode);

				});
			}

			var i18nLabel = options.label || this._generateNewLabel(workflow, i18n.t(options.name || nodeType));

			var newNode;
			var nodeID = this._generateId(nodeType, nodeTypeId, workflow);
			var iconLocation = this._wcrConfig.requireBaseUrl + "WFEditorWCR/icons/";
			var taskExecutorData = this._getTaskExecutorData(nodeType, this._workflowDefinitionXML);
			switch (nodeType) {
			case WebCenterNodeConstants.Nodes.FORK:
				newNode = new ForkNode(workflow, nodeID, iconLocation, null, this._webCenterAPI);
				break;
			case WebCenterNodeConstants.Nodes.JOIN:
				newNode = new JoinNode(workflow, nodeID, iconLocation, null, this._webCenterAPI);
				break;
			case WebCenterNodeConstants.Nodes.COMPLETED:
				newNode = new EndStateNode(workflow, nodeID, iconLocation, i18n.t(nodeType), nodeType, this._webCenterAPI);
				break;
			case "errorstop":
				newNode = new EndStateNode(workflow, nodeID, iconLocation, i18n.t(nodeType), nodeType, this._webCenterAPI);
				break;
			case "end-state":
				newNode = new EndStateNode(workflow, nodeID, iconLocation, i18n.t(nodeType), nodeType, this._webCenterAPI);
				break;
			case WebCenterNodeConstants.Nodes.WCTASK:

				var resolveCacheFunction = _.bind(function (cacheItem) {
					if (cacheItem) {
						newNode = new TaskNode(workflow, nodeID, iconLocation, "", cacheItem.taskType, this._webCenterAPI);
						if (!options.resourceXML) {
							_.each(cacheItem.outputs, function (output) {
								newNode.addPin(new OutputPinOK(workflow, newNode, output.id, output.name), 1);
							});
						}
						newNode.setLabel(i18nLabel);
					} else {
						var taskType = new WebCenterTaskType(nodeTypeId);
						var icon = this._wcrConfig.jspBaseUrl + taskType.getIcon();
						newNode = new TaskNode(workflow, nodeID, icon, "", taskType, this._webCenterAPI);
						newNode.setEnabled(false);
						newNode.setLabel(i18nLabel);
					}
					deferred.resolve(newNode);
				}, this);

				iconLocation = this._taskIconFinder.get(nodeTypeId);
				var taskTypeName = this._taskNameFinder.get(nodeTypeId);
				i18nLabel = options.label || this._generateNewLabel(workflow, taskTypeName);
				// construct the node
				this._taskTypeDefinitionsCache.get(nodeTypeId, resolveCacheFunction);

				break;
			}

			if (newNode) {
				newNode.setLabel(i18nLabel);
				deferred.resolve(newNode);

			} else {
				// Check if a icon exists for the node
				var iconUrl = iconLocation;
				if (taskExecutorData && taskExecutorData.icon) {
					iconUrl = iconLocation + taskExecutorData.icon;
				}

				if (this._constructNodesMap[nodeType] && nodeType !== WebCenterNodeConstants.Nodes.WCTASK && nodeType !== WebCenterNodeConstants.Nodes.ERRORHANDLER) {
					newNode = new this._constructNodesMap[nodeType](workflow, nodeID, iconUrl, null, this._webCenterAPI);
					newNode.setLabel(i18nLabel);
					deferred.resolve(newNode);
				} else if (nodeType === WebCenterNodeConstants.Nodes.ERRORHANDLER) {
					// There can only be one error handler
					if (_.filter(workflow.getNodes(), function (node) {
							return node.getNodeType() === WebCenterNodeConstants.Nodes.ERRORHANDLER;
						}).length === 0) {
						newNode = new this._constructNodesMap[nodeType](workflow, nodeID, iconUrl, null, this._webCenterAPI);
						newNode.setLabel(i18nLabel);
						deferred.resolve(newNode);
					} else {
						deferred.reject(i18n.t("wferror.errorhandleralreadyexists"));
					}
				} else if (nodeType !== WebCenterNodeConstants.Nodes.WCTASK) {
					// construct a WebCenter node with the correct type and icon
					newNode = this._getNodeOfCorrectType(workflow, nodeID, nodeType, iconUrl, null, this._webCenterAPI);

					newNode.setLabel(i18nLabel);
					deferred.resolve(newNode);

				}
			}

			return promise;
		};


		// static number used for unique id generation
		var highestId = 0;

		/**
		 * generates the ID for a new node
		 * @param   {nodeType} nodeType 
		 * @param 	{nodeTypeId}                             
		 * @returns {String}
		 */
		WebCenterWorkflowFactory.prototype._generateId = function (nodeType, nodeTypeId, workflow) {
			if (highestId === 0) {
				_.each(workflow.getNodes(), function (node) {
					var idPostfix = parseInt(node.getId().substring(node.getId().lastIndexOf("_") + 1));
					if (idPostfix > highestId) {
						highestId = idPostfix;
					}
				});
			}
			highestId++;
			if (nodeTypeId) {
				return nodeTypeId + "_" + highestId;
			} else {
				return nodeType + "_" + highestId;
			}
		};

		/**
		 * Generates a label <prefix> - <counter> where counter is a unique number
		 * @param   {Workflow} workflow 
		 * @param   {String} prefix
		 * @returns {String}
		 */
		WebCenterWorkflowFactory.prototype._generateNewLabel = function (workflow, prefix) {
			var nodes = _.filter(workflow.getNodes(), function (node) {
				return node.getLabel().indexOf(prefix) === 0;
			});
			var counter = this._getNextPostfixCounter(nodes, prefix);
			return prefix + (counter === 1 ? "" : " - " + counter);

		};

		/**
		 * returns a unqiue postfix number
		 * @param   {Array.<Node>} nodes - the postfix should not be present in the labels of these nodes
		 * @param   {String} prefix 
		 * @returns {Number} 
		 */
		WebCenterWorkflowFactory.prototype._getNextPostfixCounter = function (nodes, prefix) {
			var postFixes = [];
			_.each(nodes, function (node) {
				var label = node.getLabel();
				var postFix = label.substring(prefix.length);
				if (postFix !== "" && (postFix.lastIndexOf(" - ") !== 0 || isNaN(parseInt(postFix.substring(3))))) {
					// Not a corrct node after all
					return;
				}
				var labelPostfix = label.substring(label.lastIndexOf(" - ") + 3);
				if (isNaN(parseInt(labelPostfix))) {
					labelPostfix = '1';
				}
				postFixes[labelPostfix] = true;
			});
			for (var i = 1; i < postFixes.length; i++) {
				if (!postFixes[i]) {
					return i;
				}
			}
			return i;
		};


		/**
		 * get correct node type
		 * @param {String} nodeType
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {ShapeUtils.Point} location - location of the node
		 * @param {String} nodeType - specifies the type of the node
		 * @param {String} [icon] - gives the url to the icon for this node
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data  
		 * @returns {Node} taskTypeId                             
		 */
		WebCenterWorkflowFactory.prototype._getNodeOfCorrectType = function (workflow, id, nodeType, icon, label, webCenterAPI) {
			switch (nodeType) {
			case WebCenterNodeConstants.Nodes.SETTASKSPECIFICATION:
			case WebCenterNodeConstants.Nodes.SETPROJECTATTRIBUTE:
			case WebCenterNodeConstants.Nodes.SETDOCUMENTATTRIBUTE:
				return new MultiAttributeWebCenterNode(workflow, id, nodeType, icon, label, webCenterAPI);
			default:
				return new WebCenterNode(workflow, id, nodeType, icon, label, webCenterAPI);
			}
		};

		return WebCenterWorkflowFactory;
	});