define([
		"utils/ShapeUtils",
		"utils/Observer",
		"WFEditor/workflow/edge/EdgeView",
		"utils/Animatable",
		"WFEditor/tools/drawEdge/MouseNode",
		"WFEditorWCR/scene/WebCenterViewConstants",
		"WFEditor/scene/hitarea/HitArea",
		"lib/bezier",
		"underscore"
	],
	function (
		ShapeUtils,
		Observer,
		EdgeView,
		Animatable,
		MouseNode,
		WebCenterViewConstants,
		HitArea,
		Bezier,
		_
	) {
		"use strict";

		/**
		 * The view for a webcenter edge in the workflow. An edge connects two pins and has
		 * an arrow to indicate direction.
		 *
		 * @exports WebCenterEdgeView
		 * @class
		 * @augments EdgeView
		 *
		 * @parma {WebCenterEdge} edgeModel - model for the edge
		 * @param {WebCenterPinView} fromPinView - view of the pin where the edge starts
		 * @param {WebCenterPinView} toPinView - view of the pin where the edge stops
		 */
		var WebCenterEdgeView = function (edgeModel, fromPinView, toPinView) {
			EdgeView.call(this, edgeModel, fromPinView, toPinView);


			this.addAnimatableProperties({
				edgeLocationX: _.bind(function () {
					return this._toPinView.getEdgeLocation(this, this._toPinView.getModel().isSplitEdges()).x;
				}, this),
				edgeLocationY: _.bind(function () {
					return this._toPinView.getEdgeLocation(this, this._toPinView.getModel().isSplitEdges()).y;
				}, this)
			});

			/**
			 * @member {ShapeUtils.Point} startpoint for which _bezier is calculated
			 * @private
			 */
			this._startPoint = null;

			/**
			 * @member {ShapeUtils.Point} endpoint for which _bezier is calculated
			 * @private
			 */
			this._endPoint = null;

			/**
			 * @member {Bezier} the bezier curve for this edge
			 * @private
			 */
			this._bezier = null;

			this._updateBezier();




			// Listen to changes on the to pin
			this._observe(toPinView.getModel(), "changed.splitEdges", this._splitEdgesChanged);
			this._observe(this._model, "changed.*", this.makeDirty);
			this._observe(this._model.getFromPin().getNode(), "Pin.*", this.makeDirty);
			this._observe(this._model, "Pin.*.Token.*changed.select", this.makeDirty);

			// Listen to node location changes of the pins of the edge
			this._observe(this._model.getToPin().getNode(), "changed.location", this._endPointsChanged);
			this._observe(this._model.getToPin(), "Edge.*", function () {
				this._updateBoundingRect();
				this._updateHitAreas();
				this.makeDirty();
			});
			this._observe(this._model.getFromPin().getNode(), "changed.location", this._endPointsChanged);
		};

		WebCenterEdgeView.prototype = Object.create(EdgeView.prototype);
		WebCenterEdgeView.prototype.constructor = WebCenterEdgeView;

		// mixin Animatable in the prototype of WebCenterEdgeView
		Animatable.mixin(WebCenterEdgeView.prototype);

		/**
		 * @override
		 * Renders the edge on the provided canvas context.
		 *
		 * @param   {CanvasRenderingContext2D} context
		 * @param   {Layer} layer
		 * @param 	{Boolean} quickRender
		 */
		WebCenterEdgeView.prototype.render = function (context, layer, quickRender) {
			if (quickRender) {
				// Edges shouldn't be rendered in quickRender mode
				return;
			}
			var isHighlighted = this._model.isHighlighted();


			context.save();
			context.translate(-0.5, -0.5);


			this._updateBezier();


			//render main thick dark bezier curve
			context.beginPath();
			if (isHighlighted) {
				context.strokeStyle = WebCenterViewConstants.HighlightedEdgeDarkColor;
			} else {
				context.strokeStyle = WebCenterViewConstants.EdgeDarkColor;
			}
			context.lineWidth = 4;
			this._renderBezierCurve(context, this._bezier);
			context.stroke();
			context.closePath();

			//render main thin lighter bezier curve
			context.beginPath();
			if (isHighlighted) {
				context.strokeStyle = WebCenterViewConstants.HighLightedEdgeLightColor;
			} else {
				context.strokeStyle = WebCenterViewConstants.EdgeLightColor;
			}
			context.lineWidth = 2;
			this._renderBezierCurve(context, this._bezier);
			context.stroke();
			context.closePath();

			//draw arrow head
			context.beginPath();
			if (isHighlighted) {
				context.strokeStyle = WebCenterViewConstants.HighlightedEdgeDarkColor;
				context.fillStyle = WebCenterViewConstants.HighlightedEdgeLightColor;
			} else {
				context.strokeStyle = WebCenterViewConstants.EdgeDarkColor;
				context.fillStyle = WebCenterViewConstants.EdgeLightColor;
			}

			if (this.getModel().isHovered()) {
				context.lineWidth = 3;
			} else {
				context.lineWidth = 2;
			}

			var endLocation = new ShapeUtils.Point(this.getValue("edgeLocationX"), this.getValue("edgeLocationY"));
			context.moveTo(endLocation.x + 5, endLocation.y);
			context.lineTo(endLocation.x - 5, endLocation.y - 4);
			context.quadraticCurveTo(endLocation.x - 2, endLocation.y, endLocation.x - 5, endLocation.y + 4);
			context.closePath();

			context.stroke();
			this._renderOpaqueBackground(context);
			context.fill();


			context.restore();
		};

		/**
		 * The end points have changed
		 * update the hit areas and bounding rects
		 */
		WebCenterEdgeView.prototype._endPointsChanged = function () {
			// we want some new beziers
			this._updateBezier();
			this._updateBoundingRect();
			if (!this.isAnimating()) {
				this._updateHitAreas();
			}
			this.makeDirty();

		};

		/**
		 * @override
		 */
		WebCenterEdgeView.prototype._updateHitAreas = function () {
			var endLocation;
			if (!this._toPinView.getModel().isSplitEdges()) {
				// from the begin point
				endLocation = this._toPinView.getLocation();
			} else {
				// from the end point
				endLocation = this._toPinView.getEdgeLocation(this, true);
			}

			var pinRect = new ShapeUtils.Rect(new ShapeUtils.Point(endLocation.x - 10, endLocation.y - 9), new ShapeUtils.Size(20, 18));
			this._setHitAreas([new HitArea(pinRect, this, [], undefined, {cursor: "pointer-cursor"})]);
		};

		/**
		 * Generates a new bezier curve with the locations of the start and end pins.
		 * @private
		 * @returns {Bezier}
		 */
		WebCenterEdgeView.prototype._updateBezier = function () {
			var startLocation = this._fromPinView.getLocation();

			var endLocation = new ShapeUtils.Point(this.getValue("edgeLocationX"), this.getValue("edgeLocationY"));

			if (!this._bezier || !this._startPoint || !this._endPoint || !this._endPoint.equals(endLocation) || !this._startPoint.equals(startLocation)) {
				// bezier was never calculated, or start or end point has changed
				this._startPoint = startLocation;
				this._endPoint = endLocation;

				var bezierOffset = Math.max(50, Math.abs(endLocation.x - startLocation.x) / 2);

				this._bezier = new Bezier(startLocation.x, startLocation.y,
					startLocation.x + bezierOffset, startLocation.y,
					endLocation.x - bezierOffset, endLocation.y,
					endLocation.x, endLocation.y);
			}

		};

		/**
		 * Render the bezier curve on the canvas context.
		 * @param {CanvasRenderingContext2D} context
		 * @param {Bezier}   bezierCurve
		 */
		WebCenterEdgeView.prototype._renderBezierCurve = function (context, bezierCurve) {
			var p = bezierCurve.points;
			context.moveTo(p[0].x, p[0].y);
			if (p.length === 4) {
				context.bezierCurveTo(
					p[1].x, p[1].y,
					p[2].x, p[2].y,
					p[3].x, p[3].y
				);
			}
		};

		/**
		 * @override
		 */
		WebCenterEdgeView.prototype._updateBoundingRect = function () {
			var boundingBox = this._bezier.bbox();
			var rect = new ShapeUtils.Rect(new ShapeUtils.Point(boundingBox.x.min - 4, boundingBox.y.min - 4),
				new ShapeUtils.Size(boundingBox.x.size + 8, boundingBox.y.size + 8));

			// add arrow rect
			rect = rect.calculateArea(this.getArrowRect());

			this._setBoundingRect(rect.getNormalizedRectArray());
		};

		/**
		 * Get the rect of the arrow
		 */
		WebCenterEdgeView.prototype.getArrowRect = function () {
			if (!this._arrowRect) {
				this._arrowRect = new ShapeUtils.Rect();
			}

			this._arrowRect.origin.x = this.getValue("edgeLocationX", true) - 8;
			this._arrowRect.origin.y = this.getValue("edgeLocationY", true) - 7;
			this._arrowRect.size.width = 16;
			this._arrowRect.size.height = 14;

			return this._arrowRect;
		};

		/**
		 * Get the pin view the edge originates from.
		 * @returns {WebCenterPinView}
		 */
		WebCenterEdgeView.prototype.getFromPin = function () {
			return this._fromPinView;
		};

		/**
		 * Split edges changed -> animate
		 */
		WebCenterEdgeView.prototype._splitEdgesChanged = function () {
			var isSplitEdges = this._toPinView.getModel().isSplitEdges();
			if (this.isAnimating()) {
				// from the current point
				this.animate("edgeLocationY", this.getValue("edgeLocationY"), 500);
				// update the hit areas
				this._updateHitAreas();
			} else {
				if (isSplitEdges) {
					// from the begin point
					this.animate("edgeLocationY", this._toPinView.getLocation().y, 500);
					// update the hit areas
					this._updateHitAreas();
				} else {
					// from the end point
					this.animate("edgeLocationY", this._toPinView.getEdgeLocation(this, true).y, 500);
					this.once("animation.edgeLocationY.finished", _.bind(function () {
						this._updateHitAreas();
					}, this));
				}
			}



			this._updateBoundingRect();

		};

		return WebCenterEdgeView;

	});