define(["underscore",
		"WFEditor/workflow/helper/Helper",
		"utils/ShapeUtils",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"tpl!./StickyNote"
	],
	function (_,
		Helper,
		ShapeUtils,
		WebCenterNodeConstants,
		StickyNoteXML) {

		"use strict";

		// integer for generating a unique stickynote id - only used client side
		var stickyNoteIdGenerator = 0;

		/**
		 * @class
		 * @exports StickyNoteHelper
		 * @augments Helper
		 * @param {Workflow}            workflow 
		 * @param {ShapeUtils.Location} location 
		 * @param {ShapeUtils.Size}     size     
		 * @param {String}              text     
		 */
		var StickyNoteHelper = function (workflow, location, size, text) {
			Helper.call(this, workflow, "sticky_" + stickyNoteIdGenerator++);

			/**
			 * @param {ShapeUtils.Point}
			 * @private
			 */
			this._location = location;

			/**
			 * @param {ShapeUtils.Size}
			 * @private
			 */
			this._size = size || new ShapeUtils.Size(WebCenterNodeConstants.StickyNote.DEFAULT_WIDTH, WebCenterNodeConstants.StickyNote.DEFAULT_HEIGHT);

			/**
			 * @param {String}
			 * @private
			 */
			this._text = text;

			/**
			 * @member {Boolean}
			 * @private
			 */
			this._isBeingEdited = false;

		};

		StickyNoteHelper.prototype = Object.create(Helper.prototype);
		StickyNoteHelper.prototype.constructor = StickyNoteHelper;

		/**
		 * sets the location of the sticky note
		 * @param {ShapeUtils.Point} location 
		 */
		StickyNoteHelper.prototype.setLocation = function (location) {
			if (location.x !== this._location.x && location.y !== this._location.y) {
				this._location = location;
				this._emit("changed.location");
			}
		};

		/**
		 * moves the location of the sticky note horizontally by dx and vertically by dy
		 * @param {Number} dx 
		 * @param {Number} dy 
		 */
		StickyNoteHelper.prototype.offsetXY = function (dx, dy) {
			if (dx !== 0 || dy !== 0) {
				this._location.offsetXY(dx, dy);
				this._emit("changed.location");
			}
		};

		/**
		 * returns the location of the sticky note
		 * @returns {ShapeUtils.Point} 
		 */
		StickyNoteHelper.prototype.getLocation = function () {
			return this._location;
		};

		/**
		 * sets the size of the sticky note
		 * @param {Number} width
		 * @param {Number} height
		 */
		StickyNoteHelper.prototype.setSize = function (width, height) {
			if (this._size.width !== width ||
				this._size.height !== height
			) {
				this._size.width = Math.max(Math.round(width), WebCenterNodeConstants.StickyNote.MIN_WIDTH);
				this._size.height = Math.max(Math.round(height), WebCenterNodeConstants.StickyNote.MIN_HEIGHT);
				this._emit("changed.size");
			}
		};

		/**
		 * gets the size of the sticky note
		 * @returns {ShapeUtils.Size} 
		 */
		StickyNoteHelper.prototype.getSize = function () {
			return this._size;
		};

		/**
		 * sets the text of the sticky notez
		 * @param {String} text 	
		 */
		StickyNoteHelper.prototype.setText = function (text) {
			if (text !== this._text) {
				this._text = text;
				this._emit("changed.text");
			}
		};

		/**
		 * @returns {String} the text of the sticky note
		 */
		StickyNoteHelper.prototype.getText = function () {
			return this._text;
		};

		/**
		 * returns the xml for the sticky note
		 * @returns {String} 
		 */
		StickyNoteHelper.prototype.toXML = function () {
			return StickyNoteXML(this);
		};

		/**
		 * sets whether or not this sticky note is being edited
		 * @param {Boolean} isEditing 
		 */
		StickyNoteHelper.prototype.setEditingText = function (isEditing) {
			if (this._isBeingEdited !== isEditing) {
				this._isBeingEdited = isEditing;
				this._emit("changed.editing");
			}
		};

		/**
		 * @returns {Boolean} if the text is being edited
		 */
		StickyNoteHelper.prototype.isEditingText = function () {
			return this._isBeingEdited;
		};

		return StickyNoteHelper;

	});
