define(["jquery",
	"underscore",
	"WFEditorWCR/workflow/node/WebCenterNodeConstants",
	"WFEditorWCR/scene/WebCenterViewConstants",
	"utils/ShapeUtils",
	"utils/Observable",
	"component/tooltip/Tooltip",
	"utils/webi18n!WFEditorWCR"
], function (
	$,
	_,
	WebCenterNodeConstants,
	WebCenterViewConstants,
	ShapeUtils,
	Observable,
	Tooltip,
	i18n) {

	"use strict";

	/**
	 * This popup is used for generating the html of a sticky note in the WFEditor
	 * @class
	 * @exports StickyNotePopup
	 * @mixes Observable
	 * @param {StickyNoteHelper} stickyNoteHelper 
	 * @param {Scene}            scene            
	 */
	var StickyNotePopup = function (stickyNoteHelper, scene) {
		Observable.mixin(this, ["Scene", "StickyNotePopup", stickyNoteHelper.getId()], "WFEditor." + scene.getWorkflowModel().getID());

		/**
		 * @member {StickyNoteHelper}
		 * @private
		 */
		this._stickyNote = stickyNoteHelper;

		/**
		 * @member {Scene}
		 * @private
		 */
		this._scene = scene;

		/**
		 * @member {JQueryObject} the encapsulating div
		 * @private
		 */
		this._stickyNoteDiv = $("<div/>").addClass("sticky-note");

		if (!stickyNoteHelper.getWorkflow().getPrivilege(WebCenterNodeConstants.Privileges.editStickyNotes)) {
			this._stickyNoteDiv.addClass("feedback-mode");
		}

		/**
		 * @member {JQueryObject} div for the header
		 * @private
		 */
		this._headerDiv = $("<div/>").addClass("sticky-note-header");


		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._editIcon = $("<span class='sticky-note-edit-icon'/>").append($("<i class='icon icon-WCR_Annotate'/>").css({
			"font-size": WebCenterViewConstants.StickyNoteHeaderHeight - 2 * WebCenterViewConstants.StickyNoteHeaderPadding
		})).css({
			padding: WebCenterViewConstants.StickyNoteHeaderPadding,
			"padding-left": WebCenterViewConstants.StickyNoteHeaderPadding
		});

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._deleteIcon = $("<span class='sticky-note-delete-icon'/>").append($("<i class='icon icon-WCR_Actions-Delete12'/>").css({
			"font-size": WebCenterViewConstants.StickyNoteHeaderHeight - 2 * WebCenterViewConstants.StickyNoteHeaderPadding
		})).css({
			padding: WebCenterViewConstants.StickyNoteHeaderPadding,
			"padding-right": WebCenterViewConstants.StickyNoteHeaderPadding
		});

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._confirmIcon = $("<span class='sticky-note-confirm-icon'/>").append(
			$("<i class='icon icon-WCR_Actions-Appr12'/>").css({
				"font-size": WebCenterViewConstants.StickyNoteHeaderHeight - 2 * WebCenterViewConstants.StickyNoteHeaderPadding
			})).css({
			padding: WebCenterViewConstants.StickyNoteHeaderPadding,
			"padding-left": WebCenterViewConstants.StickyNoteHeaderPadding
		});

		// init tooltip for confirm icon
		this._confirmationIconTooltip = new Tooltip(i18n, this._confirmIcon, this._scene.getContext(), this._scene.getContext(), false, i18n.t("confirmStickyNoteEditTooltip"));

		this._confirmIcon.on("click", _.bind(function () {
			this._emit("saved");
			this._scene.focus();
		}, this));

		this._headerDiv.append(this._editIcon).append(this._confirmIcon).append(this._deleteIcon);

		/**
		 * @member {JQueryObject} div for the  content
		 * @private
		 */
		this._contentDiv = $("<div/>").addClass("sticky-note-content");

		/**
		 * @member {JQueryObject} div for the text of the sticky note while not editing
		 * @private
		 */
		this._textDiv = $("<div/>").addClass("sticky-note-text-div");

		/**
		 * @member {JQueryObject} text area for the sticky note while editing
		 * @private
		 */
		this._textArea = $("<textarea/>").addClass("sticky-note-text-area");

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._resizeIcon = $("<span class='sticky-note-resize'/>").append($("<i class='icon icon-WCR_WF-Sticky-Resize'/>").css({
				"font-size": WebCenterViewConstants.StickyNoteResizeHitAreaSize
			}))
			.height(WebCenterViewConstants.StickyNoteResizeHitAreaSize)
			.width(WebCenterViewConstants.StickyNoteResizeHitAreaSize);

		this._stickyNoteDiv.append(this._headerDiv).append(this._contentDiv).append(this._resizeIcon);
	};

	/**
	 * shows the popup
	 */
	StickyNotePopup.prototype.show = function () {

		if (this._stickyNote.isEditingText()) {
			// Show text area while editing
			this._textArea.val(this._stickyNote.getText());
			this._contentDiv.empty().append(this._textArea);
			this._textArea.focus();
			this._stickyNoteDiv.addClass("is-editing");
		} else {
			// Show regular text while not editing
			this._confirmationIconTooltip.hide();
			this._textDiv.text(this._stickyNote.getText());
			this._contentDiv.empty().append(this._textDiv);
			this._stickyNoteDiv.removeClass("is-editing");
		}

		var location = this._stickyNote.getLocation().copy();
		var size = this._stickyNote.getSize();

		//transform the point from world coordinates to pixels from scene div top left corner
		var t = this._scene.getTransformation();
		var loc = t.transformedPoint(location);

		// the transformed size of the popup
		var transformedSize = t.transformedSize(new ShapeUtils.Size(size.width, size.height));

		// When editing show the popup with it's real width and height
		var height = transformedSize.height;
		var width = transformedSize.width;

		this._stickyNoteDiv.css({
			"height": height,
			"width": width,
			top: loc.y,
			left: loc.x,
		});

		var transformedHeaderConstantSizes = t.transformedSize(new ShapeUtils.Size(WebCenterViewConstants.StickyNoteHeaderPadding, WebCenterViewConstants.StickyNoteHeaderHeight));

		var headerHeight = transformedHeaderConstantSizes.height;

		// transform all elements in the popup to be the correct height and width  

		this._contentDiv.css({
			height: height - headerHeight
		});
		this._headerDiv.css({
			"height": headerHeight
		});

		this._editIcon.css({
			padding: transformedHeaderConstantSizes.width,
			"padding-left": transformedHeaderConstantSizes.width
		});
		this._editIcon.find("i").css({
			"font-size": (headerHeight - (2 * transformedHeaderConstantSizes.width))
		});

		this._deleteIcon.css({
			padding: transformedHeaderConstantSizes.width,
			"padding-right": transformedHeaderConstantSizes.width,
			width: headerHeight
		});
		this._deleteIcon.find("i").css({
			"font-size": (headerHeight - (2 * transformedHeaderConstantSizes.width))
		});

		this._confirmIcon.css({
			padding: transformedHeaderConstantSizes.width,
			"padding-right": transformedHeaderConstantSizes.width,
			width: headerHeight,
			height: headerHeight
		});
		this._confirmIcon.find("i").css({
			"font-size": (headerHeight - (2 * transformedHeaderConstantSizes.width))
		});


		var transformedResizeConstant = t.transformedSize(new ShapeUtils.Size(WebCenterViewConstants.StickyNoteResizeHitAreaSize, WebCenterViewConstants.StickyNoteResizeHitAreaSize));

		this._resizeIcon.find(".icon").css({
			"font-size": transformedResizeConstant.width
		});

		this._resizeIcon.css({
			"height": transformedResizeConstant.height,
			"width": transformedResizeConstant.width
		});
	};

	/**
	 * @returns {JQueryObject} the div of the popup
	 */
	StickyNotePopup.prototype.getElement = function () {
		return this._stickyNoteDiv;
	};

	/**
	 * returns the text of the text area
	 * @returns {String} 
	 */
	StickyNotePopup.prototype.getEditedText = function () {
		return this._textArea.val();
	};

	/**
	 * destroy the popup
	 */
	StickyNotePopup.prototype.destroy = function () {
		this._destroyObservable();
		this._stickyNoteDiv.remove();
	};

	return StickyNotePopup;

});
