define(["underscore",
		"jquery",
		"WFEditor/scene/AbstractView",
		"utils/ShapeUtils",
		"WFEditor/scene/hitarea/HitArea",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/scene/WebCenterViewConstants",
		"./StickyNotePopup",
		"utils/webi18n!WFEditorWCR",
		"utils/webcss!WFEditorWCR/core/wfeditorwcr-style.css"
	],
	function (_, $,
		AbstractView,
		ShapeUtils,
		HitArea,
		WebCenterNodeConstants,
		WebCenterViewConstants,
		StickyNotePopup,
		i18n) {


		"use strict";

		/**
		 * View for a sticky note - handles all events froom the sticky note and the scene. The actual rendering is done by StickyNotePopup
		 * 
		 * @class
		 * @exports StickyNoteView
		 * @augments AbstractView
		 * 
		 * @param {StickyNoteHelper} stickyNote 
		 * @param {Scene}            scene      Scene
		 */
		var StickyNoteView = function (stickyNote, scene) {
			AbstractView.call(this, stickyNote);

			/**
			 * @member {StickyNoteHelper}
			 * @private
			 */
			this._stickyNote = stickyNote;

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = scene;

			/**
			 * @member {StickyNotePopup}
			 * @private
			 */
			this._stickyNotePopup = new StickyNotePopup(stickyNote, scene);

			this._observe(stickyNote, "changed.location", this._updateRenderingInfo);
			this._observe(stickyNote, "changed.size", this._updateRenderingInfo);
			this._observe(stickyNote, "changed.text", this.makeDirty);
			this._observe(stickyNote, "changed.editing", this.makeDirty);

			this._setSceneModelListeners();

		};

		StickyNoteView.prototype = Object.create(AbstractView.prototype);
		StickyNoteView.prototype.constructor = StickyNoteView;

		/**
		 * update the rendering info
		 */
		StickyNoteView.prototype._updateRenderingInfo = function () {
			this._updateBoundingRect();
			this._updateHitAreas();
			this.makeDirty();
		};

		/**
		 * @override
		 */
		StickyNoteView.prototype.render = function (ctx, layer, quickRender) {
			var canvas = ctx.canvas;
			if (this._stickyNote.isEditingText()) {
				this._scene.getContext().append(this._stickyNotePopup.getElement());
			} else {
				$(canvas).nextAll("canvas:first").before(this._stickyNotePopup.getElement());

			}
			this._stickyNotePopup.show();

		};

		/**
		 * @override
		 */
		StickyNoteView.prototype._updateBoundingRect = function () {
			this._setBoundingRect(new ShapeUtils.Rect(new ShapeUtils.Point(this._stickyNote.getLocation().x, this._stickyNote.getLocation().y),
				new ShapeUtils.Size(this._stickyNote.getSize().width, this._stickyNote.getSize().height)).getNormalizedRectArray());
		};

		/**
		 * @override
		 */
		StickyNoteView.prototype._updateHitAreas = function (context) {
			var hasPrivilegeToEdit = this.getModel().getWorkflow().getPrivilege(WebCenterNodeConstants.Privileges.editStickyNotes);

			// The move hitArea - ie. the header minus the location for the edit icon and the delete icon
			var moveHitArea = new ShapeUtils.Rect(
				new ShapeUtils.Point(
					this._stickyNote.getLocation().x + (hasPrivilegeToEdit ? WebCenterViewConstants.StickyNoteHeaderPadding + WebCenterViewConstants.StickyNoteHeaderHeight : 0),
					this._stickyNote.getLocation().y
				), new ShapeUtils.Size(
					// If we have the privilege to edit leave some space for the edit and delete icons
					this._stickyNote.getSize().width - (hasPrivilegeToEdit ? (2 * (WebCenterViewConstants.StickyNoteHeaderPadding + WebCenterViewConstants.StickyNoteHeaderHeight)) : 0),
					WebCenterViewConstants.StickyNoteHeaderHeight)
			);

			// resizeHitArea - right bottom corner
			var resizeHitArea = new ShapeUtils.Rect(
				new ShapeUtils.Point(
					this._stickyNote.getLocation().x + this._stickyNote.getSize().width - WebCenterViewConstants.StickyNoteResizeHitAreaSize,
					this._stickyNote.getLocation().y + this._stickyNote.getSize().height - WebCenterViewConstants.StickyNoteResizeHitAreaSize),
				new ShapeUtils.Size(WebCenterViewConstants.StickyNoteResizeHitAreaSize, WebCenterViewConstants.StickyNoteResizeHitAreaSize));

			var hitAreas = [
				new HitArea(moveHitArea, this, WebCenterNodeConstants.HitAreas.STICKYNOTE.MOVE, this.getZIndex(), {
					cursor: "hand-cursor"
				}),
				new HitArea(resizeHitArea, this, WebCenterNodeConstants.HitAreas.STICKYNOTE.RESIZE, this.getZIndex(), {
					cursor: "nwse-resize-cursor"
				})
			];

			// add hit areas for privileged tools -> editing and deleting
			if (hasPrivilegeToEdit) {
				// Text area gets split in to as to not interfere with the resize hit area
				var textHitArea1 = new ShapeUtils.Rect(
					new ShapeUtils.Point(
						this._stickyNote.getLocation().x,
						this._stickyNote.getLocation().y + WebCenterViewConstants.StickyNoteHeaderHeight
					),
					new ShapeUtils.Size(
						this._stickyNote.getSize().width,
						this._stickyNote.getSize().height - WebCenterViewConstants.StickyNoteHeaderHeight - WebCenterViewConstants.StickyNoteResizeHitAreaSize
					)
				);

				hitAreas.push(new HitArea(textHitArea1, this, WebCenterNodeConstants.HitAreas.STICKYNOTE.TEXTAREA, this.getZIndex(), {
					cursor: "text-cursor"
				}));
				var textHitArea2 = new ShapeUtils.Rect(
					new ShapeUtils.Point(
						this._stickyNote.getLocation().x,
						this._stickyNote.getLocation().y + (this._stickyNote.getSize().height - WebCenterViewConstants.StickyNoteHeaderHeight - WebCenterViewConstants.StickyNoteResizeHitAreaSize)
					),
					new ShapeUtils.Size(
						this._stickyNote.getSize().width - WebCenterViewConstants.StickyNoteResizeHitAreaSize,
						WebCenterViewConstants.StickyNoteResizeHitAreaSize
					)
				);

				hitAreas.push(new HitArea(textHitArea2, this, WebCenterNodeConstants.HitAreas.STICKYNOTE.TEXTAREA, this.getZIndex(), {
					cursor: "text-cursor"
				}));
				// Edit icon floating left in the header
				var editTextIconArea = new ShapeUtils.Rect(
					new ShapeUtils.Point(
						this._stickyNote.getLocation().x + WebCenterViewConstants.StickyNoteHeaderPadding,
						this._stickyNote.getLocation().y + WebCenterViewConstants.StickyNoteHeaderPadding
					),
					new ShapeUtils.Size(
						WebCenterViewConstants.StickyNoteHeaderHeight,
						WebCenterViewConstants.StickyNoteHeaderHeight
					)
				);
				hitAreas.push(new HitArea(editTextIconArea, this, [WebCenterNodeConstants.HitAreas.STICKYNOTE.EDITICON], this.getZIndex(), {
					cursor: "pointer-cursor",
					tooltip: i18n.t("editStickyNoteIconTooltip")
				}));
				// delete icon floating right in the header
				var deleteIconArea = new ShapeUtils.Rect(
					new ShapeUtils.Point(
						this._stickyNote.getLocation().x + this._stickyNote.getSize().width - WebCenterViewConstants.StickyNoteHeaderPadding - WebCenterViewConstants.StickyNoteHeaderHeight,
						this._stickyNote.getLocation().y + WebCenterViewConstants.StickyNoteHeaderPadding
					),
					new ShapeUtils.Size(
						WebCenterViewConstants.StickyNoteHeaderHeight,
						WebCenterViewConstants.StickyNoteHeaderHeight
					)
				);
				hitAreas.push(new HitArea(deleteIconArea, this, [WebCenterNodeConstants.HitAreas.STICKYNOTE.DELETE], this.getZIndex(), {
					cursor: "pointer-cursor",
					tooltip: i18n.t("removeStickyNoteTooltip")
				}));

			}

			this._setHitAreas(hitAreas);
		};

		/**
		 * Listen to sceneModel for panning and zoom changes
		 */
		StickyNoteView.prototype._setSceneModelListeners = function () {
			this._observe(this._scene.getSceneModel(), "panningOffsetChanged", function () {

				// defer to ensure transformation has been done. The relocation of the note doesn't happen in the render
				// method because if the pin isn't in the visible rect anymore, the render method won't be called
				_.defer(_.bind(function () {
					this._stickyNotePopup.show();

				}, this));

			});

			this._observe(this._scene.getSceneModel(), "zoomChanged", _.bind(function () {
				// defer to ensure transformation has been done.
				_.defer(_.bind(function () {
					this._stickyNotePopup.show();
				}, this));

			}, this));
		};

		/**
		 * returns the sticky note popup for this view
		 * @returns {StickyNotePopup} 
		 */
		StickyNoteView.prototype.getStickyNotePopup = function () {
			return this._stickyNotePopup;
		};

		/**
		 * @override
		 */
		StickyNoteView.prototype.destroyView = function () {
			this._stickyNotePopup.destroy();
		};

		return StickyNoteView;
	});
