define([
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/panel/WebCenterPanelConstants",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"WFEditorWCR/workflow/node/WebCenterNodeConfigurationPanel",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"utils/webi18n!WFEditorWCR",
		"underscore"
	],
	function (
		WebCenterNode,
		WebCenterPanelConstants,
		WebCenterParameter,
		WebCenterNodeConfigurationPanel,
		WebCenterNodeConstants,
		i18n,
		_
	) {
		"use strict";

		/**
		 * A type of node which handles parameters in a special way, to allow multiple attributes/specifications
		 *
		 * @exports MultiAttributeWebCenterNode
		 * @class
		 * @augments WebCenterNode
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {String} nodeType - specifies the type of the node
		 * @param {String} [icon] - gives the url to the icon for this node
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var MultiAttributeWebCenterNode = function (workflow, id, nodeType, icon, label, webCenterAPI) {
			WebCenterNode.call(this, workflow, id, nodeType, icon, label, webCenterAPI);
		};

		MultiAttributeWebCenterNode.prototype = Object.create(WebCenterNode.prototype);
		MultiAttributeWebCenterNode.prototype.constructor = MultiAttributeWebCenterNode;

		/**
		 * Return a WebCenterNodeConfiguration panel, containing only a MultiAttribute panel
		 * @override
		 */
		MultiAttributeWebCenterNode.prototype.getConfigurationPanel = function () {
			var panelPromise = $.Deferred();
			var nodeConfigPanel = new WebCenterNodeConfigurationPanel(this);
			requirejs(["WFEditorWCR/workflow/panel/MultiAttributePanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService", "WFEditorWCR/workflow/panel/DateSmartNameDictionaryService"], _.bind(function (MultiAttributePanel, WorkflowSmartNameDictionaryService, DateSmartNameDictionaryService) {
				var nameLabel, valueLabel, restriction;
				switch (this._nodeType) {
				case WebCenterNodeConstants.Nodes.SETTASKSPECIFICATION:
					nameLabel = WebCenterNodeConstants.SetTaskSpecification.SPECNAMELABEL;
					valueLabel = WebCenterNodeConstants.SetTaskSpecification.SPECVALUE;
					restriction = this._webCenterAPI.getTaskSpecifications(this._workflow.getTaskTypeID());
					break;
				case WebCenterNodeConstants.Nodes.SETPROJECTATTRIBUTE:
					nameLabel = WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTENAMELABEL;
					valueLabel = WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTEVALUE;
					break;
				case WebCenterNodeConstants.Nodes.SETDOCUMENTATTRIBUTE:
					nameLabel = WebCenterNodeConstants.SetDocumentAttributes.ATTRIBUTENAMELABEL;
					valueLabel = WebCenterNodeConstants.SetDocumentAttributes.ATTRIBUTEVALUE;
					break;
				}

				var panel = new MultiAttributePanel("", i18n.t(nameLabel), i18n.t(valueLabel), new WorkflowSmartNameDictionaryService(this._webCenterAPI, this._workflow.getTaskTypeID()), new DateSmartNameDictionaryService(this._webCenterAPI, this._workflow.getTaskTypeID()), this._webCenterAPI, false, restriction);
				panel.getContext().addClass('parameter-panel form-horizontal form-compact');

				nodeConfigPanel.add(panel);
				panelPromise.resolve(nodeConfigPanel);
			}, this));

			return panelPromise.promise();
		};

		/**
		 * sets the configuration of this node
		 * @param {List.<Parameter>} config
		 */
		MultiAttributeWebCenterNode.prototype.setConfiguration = function (config) {
			// when no values are defined yet, the empty default attributes are added
			// instead of that, we need the special value map parameters (name = paramvalue 1, value = paramvalue 2)
			// we'll ignore the default parameters
			var realConfig = config;
			var defaultNameFound = false;
			var defaultValueFound = false;
			if (config.length === 2) {
				switch (this._nodeType) {
				case WebCenterNodeConstants.Nodes.SETTASKSPECIFICATION:
					_.each(config, function (param) {
						if (param.getName() === WebCenterNodeConstants.SetTaskSpecification.SPECNAME) {
							defaultNameFound = true;
						} else if (param.getName() === WebCenterNodeConstants.SetTaskSpecification.SPECVALUE) {
							defaultValueFound = true;
						}
					});
					break;
				case WebCenterNodeConstants.Nodes.SETPROJECTATTRIBUTE:
				case WebCenterNodeConstants.Nodes.SETDOCUMENTATTRIBUTE: // this is not a mistake; documents aso have project.attX
					_.each(config, function (param) {
						if (param.getName() === WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTENAME) {
							defaultNameFound = true;
						} else if (param.getName() === WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTEVALUE) {
							defaultValueFound = true;
						}
					});
					break;
				}
			}

			if (defaultNameFound && defaultValueFound) {
				realConfig = [];
			}

			// call super
			WebCenterNode.prototype.setConfiguration.call(this, realConfig);
		};

		/**
		 * Parse the node XML information
		 * @param {Element} nodeEl- XML element containing definition of the WorkflowNode
		 */
		MultiAttributeWebCenterNode.prototype.setNodeXML = function (nodeEl) {
			var params = nodeEl.querySelectorAll("action > parameters > parameter > Name");

			var oldName, oldValue;
			_.each(params, function (nameNode) {
				var name = nameNode.textContent;

				// Migrate from single attribute to multi project attribute
				var param;
				// Old behavior
				if (name === WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTENAME || name === WebCenterNodeConstants.SetTaskSpecification.SPECNAME) {
					param = new WebCenterParameter(WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTENAME);
					param.updateFromXML(nameNode.parentNode);
					oldName = param.getValue();
				} else if (name === WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTEVALUE || name === WebCenterNodeConstants.SetTaskSpecification.SPECVALUE) {
					param = new WebCenterParameter(WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTEVALUE);
					param.updateFromXML(nameNode.parentNode);
					oldValue = param.getValue();
				} else {
					// Correct current behavior
					param = new WebCenterParameter(name);
					param.updateFromXML(nameNode.parentNode);
					this._configuration.push(param); // we still have to push the parameters because setConfiguration makes config empty because backend still returns "old layout"
				}
			}, this);

			if (oldName) {
				var migratedParam = new WebCenterParameter(oldName, oldValue);
				this._configuration.push(migratedParam);
			}
		};

		/**
		 * @Override 
		 */
		MultiAttributeWebCenterNode.prototype.isConfigurable = function () {
			return this.isEnabled();
		};

		return MultiAttributeWebCenterNode;
	});