define([
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConfigurationPanel",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"utils/Observer",
		"utils/Map",
		"utils/webi18n!WFEditorWCR"
	],
	function (
		WebCenterNode,
		OutputPinOK,
		WebCenterPinConstants,
		WebCenterNodeConfigurationPanel,
		WebCenterParameter,
		Observer,
		Map,
		i18n
	) {
		"use strict";

		/**
		 * TokenMarkerSelectorNode is a common super class for the TokenMarkerNode and TokenSelectorNode.
		 *
		 * @exports TokenMarkerSelectorNode
		 * @class
		 * @augments WebCenterNode
		 *
		 * @param {Wofkflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {WebCenterNodeConstants.Nodes} nodeType - The type of the node e.g. WebCenterNodeConstants.Nodes.TOKENSELECTOR
		 * @param {String} pathToIcons - gives the url to the map where the icons can be found e.g. WebCenterNodeConstants.Icons.TOKENSELECTOR
		 * @param {WebCenterNodeConstants.Icons} - icon - The icon of this node 
		 * @param {String} label - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data   
		 */
		var TokenMarkerSelectorNode = function (workflow, id, nodeType, pathToIcons, icon, label, webCenterAPI) {
			Observer.mixin(this);

			/*
			 * @member {WebCenterParameter} Mark
			 */
			this._markParameter = new WebCenterParameter("wfconstruct-tokenmarker.tag"); // needs to be constructed first

			/*
			 * @member {TagSelectPanel}
			 */
			this._tagSelectPanel = null;

			WebCenterNode.call(this, workflow, id, nodeType, pathToIcons + icon, label, webCenterAPI);


			// Now workflow is defined
			var allMarkers = this._getAllMarkers(this._getMarkersSourceNodeType());
			var allOwnMarkers = this._getAllMarkers(this.getNodeType());
			allMarkers = _.difference(allMarkers, allOwnMarkers);
			if (_.size(allMarkers) > 0) {
				this._markParameter.setValue(allMarkers[0]);
			} else {
				this._markParameter.setValue(this._getNextMark());
			}

		};

		TokenMarkerSelectorNode.prototype = Object.create(WebCenterNode.prototype);
		TokenMarkerSelectorNode.prototype.constructor = TokenMarkerSelectorNode;

		/**
		 * @override
		 */
		TokenMarkerSelectorNode.prototype.setConfiguration = function (config) {
			if (!config || _.size(config) === 0) {
				// configuration is defined client side
				config = [];
				config.push(this._markParameter);
			}

			this._markParameter = config[0];
			if (this._tagSelectPanel) {
				// update the options
				this._tagSelectPanel.setOptions(this._getAllMarkers(this._getMarkersSourceNodeType(), true));
			}

			WebCenterNode.prototype.setConfiguration.call(this, config);
			// if the configuration changed, a redraw is already triggered so the visible parameters will be updated
		};

		/**
		 * @Override
		 */
		TokenMarkerSelectorNode.prototype._initializePins = function () {
			this._addInputPin();
			this.addPin(new OutputPinOK(this._workflow, this, WebCenterPinConstants.Type.WorkflowConstructOK));
		};

		/**
		 * @override
		 */
		TokenMarkerSelectorNode.prototype.setNodeXML = function (nodeEl) {
			var mark = nodeEl.querySelector("action mark").textContent;
			this._markParameter.setValue(mark);
		};

		/**
		 * Get the mark value
		 * @returns {String}
		 */
		TokenMarkerSelectorNode.prototype.getMark = function () {
			return this._markParameter.getValue();
		};

		/**
		 * @override
		 */
		TokenMarkerSelectorNode.prototype.getConfigurationPanel = function () {
			var nodeConfigPanel = new WebCenterNodeConfigurationPanel(this);
			this._observe(nodeConfigPanel, "save", function () {
				this._emit("changed.visibleParameters");
			});

			var panelPromise = $.Deferred();

			requirejs(["WFEditorWCR/workflow/panel/TagSelectPanel"], _.bind(function (TagSelectPanel) {
				this._tagSelectPanel = new TagSelectPanel(this._markParameter.getName(), i18n.t(this.getNodeType() + ".markLabel"));

				// set the tag list
				this._tagSelectPanel.setOptions(this._getAllMarkers(this._getMarkersSourceNodeType(), true));

				nodeConfigPanel.add(this._tagSelectPanel);
				panelPromise.resolve(nodeConfigPanel);
			}, this));

			return panelPromise.promise();
		};


		/**
		 * @override
		 */
		TokenMarkerSelectorNode.prototype.getVisibleParameters = function () {
			var params = [];
			params.push(this._markParameter);
			return params;
		};

		/**
		 * Get the markers source node type.
		 * This will be used to find markers of nodes of this type for the tag dropdown selector.
		 * Currently this is the opposite type of this.getNodeType();
		 * @returns {WebCenterNodeConstants.Nodes} Currently either WebCenterNodeConstants.Nodes.TOKENSELECTOR or WebCenterNodeConstants.Nodes.TOKENMARKER
		 */
		TokenMarkerSelectorNode.prototype._getMarkersSourceNodeType = function () {
			// implement yourself
		};


		/**
		 * Get all markers of the nodes of the given type
		 * @param {WebCenterNodeConstants.Nodes} nodeType - This is either WebCenterNodeConstants.Nodes.TOKENSELECTOR or WebCenterNodeConstants.Nodes.TOKENMARKER
		 * @param {Boolean} includeSelf - True to include this node mark                                                
		 * @returns {Array.<String>} All tags of the nodes of the given type, sorted alphabetically. Empty array if none found.
		 */
		TokenMarkerSelectorNode.prototype._getAllMarkers = function (nodeType, includeSelf) {
			var allNodes = this.getWorkflow().getNodes();
			// filter + map
			var marks = _.reduce(allNodes, function (memo, node, key) {
				if (node.getNodeType() === nodeType) {
					memo.push(node.getMark());
				}
				return memo;
			}, []);
			if (includeSelf) {
				marks.push(this.getMark());
			}
			var sortedMarks = _.sortBy(marks, function (mark) {
				return mark.toLowerCase();
			});
			return _.uniq(sortedMarks, true);
		};

		/**
		 * Get the next mark for this type node
		 * @returns {String}
		 */
		TokenMarkerSelectorNode.prototype._getNextMark = function () {
			var allMarkersOfMyType = this._getAllMarkers(this.getNodeType());

			var i = 1;
			var next;
			do {
				next = i18n.t("wfconstruct-tokenmarker.tag") + " - " + i;
				i++;
			} while (_.contains(allMarkersOfMyType, next));
			return next;
		};

		return TokenMarkerSelectorNode;
	});