define([
		"utils/ShapeUtils",
		"utils/XMLUtils",
		"utils/Map",
		"WFEditor/workflow/node/Node",
		"WFEditorWCR/workflow/pin/InputPin",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/OutputPinError",
		"WFEditorWCR/workflow/pin/OutputPinWarning",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/edge/WebCenterEdge",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"WFEditorWCR/workflow/panel/PinParameter",
		"webcenterapi/data/WebCenterAttribute",
		"tpl!WFEditorWCR/workflow/node/WebCenterNode",
		"tpl!WFEditorWCR/workflow/node/WebCenterNodeAction",
		"tpl!WFEditorWCR/workflow/node/WebCenterNodeLayout",
		"utils/webi18n!WFEditorWCR",
		"WFEditorWCR/workflow/panel/WebCenterPanelConstants",
		"WFEditorWCR/workflow/panel/WorkflowSmartNameConstants",
		"WFEditorWCR/data/CustomSmartName",
		"WFEditorWCR/workflow/node/WebCenterNodeConfigurationPanel",
		"packaging-content-management/ContentConstants",
		"WFEditorWCR/data/WebCenterTextContentProperty",
		"underscore"
	],
	function (
		ShapeUtils,
		XMLUtils,
		Map,
		Node,
		InputPin,
		OutputPinOK,
		OutputPinError,
		OutputPinWarning,
		WebCenterPinConstants,
		WebCenterEdge,
		WebCenterNodeConstants,
		WebCenterParameter,
		PinParameter,
		WebCenterAttribute,
		WebCenterNodeXML,
		WebCenterNodeActionXML,
		WebCenterNodeLayoutXML,
		i18n,
		WebCenterPanelConstants,
		WorkflowSmartNameConstants,
		CustomSmartName,
		WebCenterNodeConfigurationPanel,
		ContentConstants,
		WebCenterTextContentProperty,
		_
	) {
		"use strict";

		/**
		 * WebCenterNode represents the model for a node in the WebCenter Workflow Editor.
		 * The node has an icon and a specific type which are used in the rendering of the node.
		 *
		 * @exports WebCenterNode
		 * @class
		 * @augments Node
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {ShapeUtils.Point} location - location of the node
		 * @param {String} nodeType - specifies the type of the node
		 * @param {String} [icon] - gives the url to the icon for this node
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var WebCenterNode = function (workflow, id, nodeType, icon, label, webCenterAPI) {
			Node.call(this, workflow, id);

			/**
			 *  @member {String}
			 *  @private
			 */
			this._nodeType = nodeType;

			/**
			 *  @member {String} url relative to the webcenter instance
			 *  @private
			 */
			this._iconUrl = icon;

			/**
			 *  @member {String}
			 *  @private
			 */
			this._label = label;

			/**
			 * @member {InputPin} input pin if available
			 * @private
			 */
			this._inputPin = null;

			/**
			 * @member {OutputPinError} error pin for the node
			 * @private
			 */
			this._errorPin = null;

			/**
			 * @member {String} temporary storage of action xml
			 */
			this._actionXML = "";

			/**
			 * @member {String} temporary storage of resource xml
			 */
			this._resourceXML = "";

			/**
			 * @member {Boolean} is someone editing the label of this node?
			 */
			this._isEditingLabel = false;

			/**
			 * @member {WebCenterAPI}
			 * @private
			 */
			this._webCenterAPI = webCenterAPI;

			/**
			 * @member {String} topic of the help pages
			 * @private
			 */
			this._helpTopic = "";

			/**
			 * @member {$.Deferred} deferred used to check if the help topic has been fetched yet
			 * @private
			 */
			this._helpTopicDeferred = null;

			/** 
			 * @member {Parameter}
			 * @private
			 */
			this._warningPolicyParam = null;
			this._warningPolicyOk = null;
			this._warningPolicyError = null;
			this._warningPolicyDedicated = null;

			/**
			 * @member {Boolean}
			 * @private
			 */
			this._canEditWarningPolicy = false;

			/**
			 * @member {Array.<String>}
			 * @private 
			 */
			this._warningOutputNames = [];


			// Initialize the default parameters
			this._webCenterAPI.getWorkflowDefinitions().done(_.bind(function (workflowDefinitionXML) {
				var details = this._getConfigDetailsForNodeType(this.getNodeType(), workflowDefinitionXML);
				var parametersCopy = [];
				if (details.parameters) {
					_.each(details.parameters, function (param) {
						parametersCopy.push(param.copy());
					}, this);
				}
				this.setConfiguration(parametersCopy);

				this._helpTopic = details.helpTopic;
			}, this));

			this._initializePins();

			if (this._canEditWarningPolicy) {
				this._initWarningPolicyParams();
			}

		};

		WebCenterNode.prototype = Object.create(Node.prototype);
		WebCenterNode.prototype.constructor = WebCenterNode;


		/**
		 * Add the input pin
		 */
		WebCenterNode.prototype._addInputPin = function () {
			this._inputPin = new InputPin(this._workflow, this);
			this.addPin(this._inputPin);
		};

		/**
		 * Add an error pin to the node
		 * @param {WebCenterPinConstants.Type} [errorType] - type of the error pin, uses default one if undefined
		 */
		WebCenterNode.prototype._addErrorPin = function (errorType) {
			this._errorPin = new OutputPinError(this._workflow, this, errorType);
			this.addPin(this._errorPin);
		};

		/**
		 * Initialize the pins for the node
		 */
		WebCenterNode.prototype._initializePins = function () {
			// Add an input pin
			this._addInputPin();

			// Add custom output pins
			this._webCenterAPI.getWorkflowDefinitions().done(_.bind(function (workflowDefinitionXML) {
				var details = this._getConfigDetailsForNodeType(this.getNodeType(), workflowDefinitionXML);
				var outputPins = details.outputs;
				if (outputPins && _.size(outputPins) > 0) {
					_.each(outputPins, function (output) {
						if (output.type === "0") {
							this.addPin(new OutputPinOK(this._workflow, this, output.name, i18n.t(output.name)));
						} else if (output.type === "1") {
							this._warningOutputNames.push(output.name);
							this._canEditWarningPolicy = true;
						} else if (output.type === "2") {
							var errorPin = new OutputPinError(this._workflow, this, output.name, i18n.t(output.name));
							// this way the last one is the real error pin
							this._errorPin = errorPin;
							this.addPin(errorPin);
						}
					}, this);
				} else {
					// Add by default an error pin
					this._addErrorPin();
				}
			}, this));
		};

		/**
		 * Returns the type of this node
		 * @returns {String}
		 */
		WebCenterNode.prototype.getNodeType = function () {
			return this._nodeType;
		};

		/**
		 * Returns the path relative to the webcenter instance for the icon of this node
		 * @returns {String}
		 */
		WebCenterNode.prototype.getIconUrl = function () {
			return this._iconUrl;
		};

		/**
		 * Get visible parameters to render under the node
		 * @returns {Array.<Parameter>}
		 */
		WebCenterNode.prototype.getVisibleParameters = function () {
			return [];
		};

		/**
		 * Returns the label of the node
		 * @param {Boolean} [returnIdIfDebugMode = false] whether or not the id should be returned instead
		 *												  of the label when in debug mode
		 * @returns {String}
		 */
		WebCenterNode.prototype.getLabel = function (returnIdIfDebugMode) {
			return this._workflow.isDebugMode() && returnIdIfDebugMode ? this._id : this._label;
		};

		/**
		 * Set the label for the node.
		 * @param {String} label
		 */
		WebCenterNode.prototype.setLabel = function (label) {
			if (label !== this._label) {
				this._label = label;
				this._emit("changed.label");
			}
		};

		/**
		 * set the editing status of the node's label
		 * @param {Boolean} edit
		 */
		WebCenterNode.prototype.setEditingLabel = function (edit) {
			if (this._isEditingLabel !== edit) {
				this._isEditingLabel = edit;
				this._emit("changed.editingLabel");
			}
		};

		/**
		 * returns whether or not this label is being edited
		 * @returns {Boolean} 
		 */
		WebCenterNode.prototype.isEditingLabel = function () {
			return this._isEditingLabel;
		};

		/**
		 * Parse the node XML information
		 * @param {Element} nodeEl- XML element containing definition of the WorkflowNode
		 */
		WebCenterNode.prototype.setNodeXML = function (nodeEl) {
			// this is a temporary to save XML for using it when writing out node xml
			var actionEls = nodeEl.getElementsByTagName("action");
			if (actionEls[0]) {
				this._actionXML = new XMLSerializer().serializeToString(actionEls[0]);
			} else {
				this._actionXML = "";
			}
			var params = nodeEl.querySelectorAll("action > parameters > parameter > Name");
			_.each(params, function (nameNode) {
				var name = nameNode.textContent;
				var param = this.getConfigurationParameterByName(name);
				if (param) {
					param.updateFromXML(nameNode.parentNode);
				}
			}, this);
			if (this.canEditWarningPolicy()) {
				var warningPolicy = nodeEl.querySelector("action > warningpolicy");
				if (warningPolicy) {
					this._warningPolicyParam.setValue(warningPolicy.textContent);
				} else {
					this._warningPolicyParam.setValue(WebCenterNodeConstants.WarningPolicy.DEDICATED);
				}

				this._updateWarningPins();
			}
		};

		/**
		 * Handle the Layout XML
		 * --> read location and label
		 * @param {Element} layoutXML - xml element with layoutinformation (wfelement)
		 */
		WebCenterNode.prototype.setLayoutXML = function (layoutXML) {
			var labelEl = layoutXML.getElementsByTagName("label")[0];
			if (labelEl) {
				this.setLabel(labelEl.textContent);
			}

			var locXEl = layoutXML.getElementsByTagName("preflocx")[0];
			var locYEl = layoutXML.getElementsByTagName("preflocy")[0];

			if (locXEl && locYEl) {
				this.setLocation(new ShapeUtils.Point(parseFloat(locXEl.textContent), parseFloat(locYEl.textContent)));
			}
		};

		/** 
		 * Handle Resource XML
		 * --> handle extra resources for the node
		 * @parma {Element} resourceXML - xml element with resource information (resource)
		 */
		WebCenterNode.prototype.setResourceXML = function (resourceXML) {
			var outputs = resourceXML.getElementsByTagName("output");
			var i = 1;
			_.each(outputs, function (output) {
				var name = output.getElementsByTagName("name")[0].textContent;
				var id = output.getElementsByTagName("id")[0].textContent;
				this.addPin(new OutputPinOK(this._workflow, this, id, name), i);
				i++;
			}, this);
			// temporary hack to write out correct resource XML when generating XML for node
			this._resourceXML = new XMLSerializer().serializeToString(resourceXML);
		};

		/**
		 * Get the Sequence number of a pin
		 * For the input pin this is always 0, for other pins it depends on their position
		 * @param {WebCenterPin} pin
		 * @returns {Number}sequence number (-1 if pin is not found)
		 */
		WebCenterNode.prototype.getPinSequenceNumber = function (pin) {
			var sequenceNumber = this.getPinIndex(pin.getId());

			if (this.hasInputPin() && this._inputPin !== pin) {
				sequenceNumber--;
			}

			return sequenceNumber;
		};

		/**
		 * Get the number of output pins
		 * @returns {Number}
		 */
		WebCenterNode.prototype.getOutputPinCount = function () {
			var pinCount = this.getPinCount();
			if (this.hasInputPin()) {
				pinCount--;
			}
			return pinCount;
		};

		/**
		 * Get the input pin
		 * @returns {InputPin}
		 */
		WebCenterNode.prototype.getInputPin = function () {
			return this._inputPin;
		};

		/**
		 * Returns the set error pin, if no error pin is present it returns null
		 * @returns {OutputPinError|null}
		 */
		WebCenterNode.prototype.getErrorPin = function () {
			return this._errorPin;
		};

		/**
		 * Does this node has an input pin?
		 * @returns {Boolean}
		 */
		WebCenterNode.prototype.hasInputPin = function () {
			return this._inputPin != null;
		};

		/**
		 * Generate XML definition of the node
		 * @returns {String} XMLString
		 */
		WebCenterNode.prototype.toXML = function () {
			return WebCenterNodeXML(this);
		};

		/**
		 * Generate ActionXML
		 * @returns {String} XMLString
		 */
		WebCenterNode.prototype.getActionXML = function () {
			if (this._actionXML && !this._configuration) {
				return this._actionXML;
			} else {
				return WebCenterNodeActionXML(this);
			}
		};

		/**
		 * Generate the ResourceXML for the mode
		 * @returns {String} XMLString
		 */
		WebCenterNode.prototype.getResourceXML = function () {
			// when we start changing settings of the node, we need to make a decent implementation for this
			return this._resourceXML;
		};

		/**
		 * Generate the XML for defining the layout of the node
		 * @returns {String} XMLString
		 */
		WebCenterNode.prototype.getLayoutXML = function () {
			return WebCenterNodeLayoutXML(this);
		};

		/**
		 * Generate the XML that defines the connections leaving this node
		 * @returns {String} XMLString
		 */
		WebCenterNode.prototype.getTransitionsXML = function () {
			var inputPin = this.getInputPin();
			var errorPin = this._errorPin;
			var transitionsXML = "";
			var errorImplicitNode = this.getWorkflow().getNode(WebCenterNodeConstants.Nodes.ERRORCONTINUEIMPLICIT);
			var runOverEdges = function (edge) {
				transitionsXML += edge.toXML();
			};


			var edgeCounterForWarningOutput = 1;

			// This function is used to create the transitions out of the warning pin if a warning policy is set.
			var createWarningEdge = function (edge, warningOutputName) {
				var tempWarningPin = new OutputPinWarning(this._workflow, this, warningOutputName, i18n.t(warningOutputName));
				var tempEdge = new WebCenterEdge(this.getWorkflow(),
					tempWarningPin.getId() + "/" + edgeCounterForWarningOutput,
					tempWarningPin,
					edge.getToPin());
				tempWarningPin.addEdge(tempEdge);
				runOverEdges(tempEdge);
				edgeCounterForWarningOutput++;

			};

			_.each(this.getPins(), function (pin) {
				if (pin != inputPin && pin.getPinType() !== WebCenterPinConstants.Type.OutError) {
					_.each(pin.getEdges(), function (edge) {

						// If the warning policy is "OK", all edges going out of an OK pin should be copied for the warning pin. 
						if (this._canEditWarningPolicy && pin instanceof OutputPinOK && this._warningPolicyParam.getValue() === WebCenterNodeConstants.WarningPolicy.OK) {
							_.each(this._warningOutputNames, _.partial(_.bind(createWarningEdge, this), edge));
						}

						runOverEdges(edge);
					}, this);
				} else if (pin.getPinType() === WebCenterPinConstants.Type.OutError) {
					if (pin.getEdges().length === 0) {
						// There are no edges leaving the error pin -> set implicit error edge

						if (errorImplicitNode) {
							runOverEdges(new WebCenterEdge(this.getWorkflow(), pin.getId() + "/1", pin, errorImplicitNode.getInputPin()));
						}


					} else {
						// Run over the edges and add to xml
						_.each(pin.getEdges(), function (edge) {
							runOverEdges(edge);
						});

					}
					if (pin === errorPin && this._canEditWarningPolicy && this._warningPolicyParam.getValue() === WebCenterNodeConstants.WarningPolicy.ERROR) {
						// If the warning policy is "Error", all edges going out of the error pin should be copied for the warning pin. 
						// If the error pin has no outgoing edges, route the warning node to the implicit error node
						var errorEdges = [];
						if (pin.getEdges().length === 0) {
							if (errorImplicitNode) {
								errorEdges = [new WebCenterEdge(this.getWorkflow(), pin.getId() + "/1", pin, errorImplicitNode.getInputPin())];
							}
						} else {
							pin.getEdges();
						}
						_.each(errorEdges, function (edge) {
							_.each(this._warningOutputNames, _.partial(_.bind(createWarningEdge, this), edge));
						}, this);
					}
				}
			}, this);



			return transitionsXML;
		};

		/**
		 * All WebCenter nodes are configurable by default
		 * @override
		 * @returns {Boolean} 
		 */
		WebCenterNode.prototype.isConfigurable = function () {
			return this.getConfiguration().length > 0 && this.isEnabled();
		};

		/**
		 * @override
		 */
		WebCenterNode.prototype.getConfigurationPanel = function (wfEditorContext) {
			// probably custom node configuration panels need to be made based on the node type
			var panelPromise = $.Deferred();
			this._webCenterAPI.getWorkflowDefinitions().done(_.bind(function (workflowDefinitionXML) {
				var xmls = workflowDefinitionXML.querySelectorAll(
					"TaskExecuterFactories TaskExecuterFactory");
				var promise;
				var nodeConfigPanel;
				_.each(xmls, function (xml) {
					if (xml.getElementsByTagName("type")[0].textContent === this.getNodeType()) {
						if (!this._helpTopic) {
							var helpTopicNode = xml.querySelector("helptopic");
							if (helpTopicNode) {
								this._helpTopic = helpTopicNode.textContent;
							}
						}
						nodeConfigPanel = new WebCenterNodeConfigurationPanel(this);
						var config = xml.querySelector("config-inline");
						var smartNamesList = XMLUtils.getFirstChildElementByTagName(config, WebCenterPanelConstants.SMARTNAMES);
						var nodeCustomSmartNames = [];
						if (smartNamesList) {
							var smartNames = smartNamesList.getElementsByTagName(WebCenterPanelConstants.SMARTNAME);
							_.each(smartNames, function (smartName) {
								nodeCustomSmartNames.push(CustomSmartName.fromXML(smartName));
							});
						}
						promise = this._getSubPanels(xml.querySelector("config-inline"), nodeCustomSmartNames, wfEditorContext);
					}
				}, this);
				promise.done(_.bind(function (panels) {
					_.each(panels, function (panel) {
						nodeConfigPanel.add(panel);
					});

					panelPromise.resolve(nodeConfigPanel);
				}, this));
			}, this));

			return panelPromise.promise();
		};

		/**
		 * returns all panels that are specified in xml
		 * @param {Object}   xml - xml that contains the information for creating the panel
		 * @param {Array.<CustomSmartNames>} - Custom smartnames applicable for all node smartname enabled parameters
		 * @return {JQueryPromise.<Array.<Panel>>}                   
		 */
		WebCenterNode.prototype._getSubPanels = function (xml, nodeCustomSmartNames, wfEditorContext) {
			var deferreds = []; // list of deferreds the function should wait for before resolving its own deferred
			_.each(xml.childNodes, function (nodeXML) {
				if (!nodeXML.tagName) {
					return;
				}
				var taskTypeID = this.getWorkflow().getTaskTypeID();
				var subPromise;
				var myPromise = $.Deferred();
				var tagName = nodeXML.tagName.toLowerCase();
				switch (tagName) {
				case WebCenterPanelConstants.MENUS:
					// Get the promise of the sub panels
					subPromise = this._getSubPanels(nodeXML, nodeCustomSmartNames, wfEditorContext);
					// Add a promise for theses subpanels
					deferreds.push(myPromise);

					// Require tabbed panel
					requirejs(["WFEditorWCR/workflow/panel/TabbedPanel"], _.bind(function (TabbedPanel) {
						// if the tabbedpanel has been fetched and the subpanels have been created we can add those panels
						// to the tabbed panel and resolve our own promise.
						subPromise.done(function (panels) {
							var tabbedPanel = new TabbedPanel();
							_.each(panels, function (panel) {
								tabbedPanel.add(panel);
							});
							myPromise.resolve(tabbedPanel);
						});
					}, this));
					break;
				case WebCenterPanelConstants.PARAMETERS:
					// Add a promise for theses subpanels
					deferreds.push(myPromise);
					if (nodeXML.getAttribute(WebCenterPanelConstants.PANELURL)) {
						panelUrl = nodeXML.getAttribute(WebCenterPanelConstants.PANELURL);
						requirejs([panelUrl], _.bind(function (Panel) {
							myPromise.resolve(new Panel(this._webCenterAPI, taskTypeID, nodeXML, nodeCustomSmartNames, wfEditorContext));
						}, this));

					} else {

						// Get the promise of the sub panels
						subPromise = this._getSubPanels(nodeXML, nodeCustomSmartNames, wfEditorContext);


						requirejs(["WFEditorWCR/workflow/panel/WebCenterCompositePanelComponent"], _.bind(function (WebCenterCompositePanelComponent) {
							subPromise.done(function (panels) {
								var compositePanel = new WebCenterCompositePanelComponent([]);
								compositePanel.getContext().addClass('parameter-panel form-horizontal form-compact');
								_.each(panels, function (panel) {
									compositePanel.add(panel);
								});
								myPromise.resolve(compositePanel);
							});
						}, this));
					}
					break;
				case WebCenterPanelConstants.PARAMETER:
					var name, type, defaultValue, subparametersstate, paramId, options, panelUrl, data;
					var customSmartNames = nodeCustomSmartNames.slice(); // by default the node ones are added
					if (nodeXML.hasAttribute(WebCenterPanelConstants.PANELURL)) {
						panelUrl = nodeXML.getAttribute(WebCenterPanelConstants.PANELURL);
					}
					_.each(nodeXML.childNodes, function (child) {
						if (child.tagName === WebCenterPanelConstants.NAME) {
							paramId = child.textContent;
							name = i18n.tFallback(paramId, "");
						} else if (child.tagName === WebCenterPanelConstants.TYPE) {
							type = child.textContent;
						} else if (child.tagName === WebCenterPanelConstants.DEFAULT) {
							defaultValue = child.textContent;
						} else if (child.tagName === WebCenterPanelConstants.SUBPARAMETERSTATE.TAGNAME) {
							subparametersstate = child.textContent;
						} else if (child.tagName === WebCenterPanelConstants.SMARTNAMES) {
							var smartNames = child.getElementsByTagName(WebCenterPanelConstants.SMARTNAME);
							_.each(smartNames, function (smartName) {
								customSmartNames.push(CustomSmartName.fromXML(smartName));
							});
						} else if (child.tagName === WebCenterPanelConstants.DATA) {
							data = child;
						}
					}, this);
					if (paramId) {
						if (panelUrl) {
							deferreds.push(myPromise);
							requirejs([panelUrl], _.bind(function (Panel) {
								myPromise.resolve(new Panel(paramId, name, this._webCenterAPI, data, customSmartNames, taskTypeID));
							}, this));
						} else if (type) {
							deferreds.push(myPromise);
							// Check if subclasses have panels for this type, if not, try default implementation
							if (!this._getNodeSpecificSubPanel(myPromise, type, paramId, nodeXML, customSmartNames)) {
								switch (type) {
								case WebCenterPanelConstants.STRING:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameInputField, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new SmartNameInputField(paramId, name, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));

									}, this));
									break;
								case WebCenterPanelConstants.DATE:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/DateSmartNameDictionaryService"], _.bind(function (SmartNameInputField, DateSmartNameDictionaryService) {
										myPromise.resolve(new SmartNameInputField(paramId, name, new DateSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.URL:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/URLSmartNameDictionaryService"], _.bind(function (SmartNameInputField, URLSmartNameDictionaryService) {
										myPromise.resolve(new SmartNameInputField(paramId, name, new URLSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.EMAIL:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/EmailSmartNameDictionaryService"], _.bind(function (SmartNameInputField, EmailSmartNameDictionaryService) {
										myPromise.resolve(new SmartNameInputField(paramId, name, new EmailSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.NUMBER:
									// currently not used
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameInputField, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new SmartNameInputField(paramId, name, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.PM:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/PMSmartNameDictionaryService"], _.bind(function (SmartNameInputField, PMSmartNameDictionaryService) {
										myPromise.resolve(new SmartNameInputField(paramId, name, new PMSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.ATTRIBUTE:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/AttributeSmartNameDictionaryService"], _.bind(function (SmartNameInputField, AttributeSmartNameDictionaryService) {
										// ignore custom smartnames for attribute (iterate)
										myPromise.resolve(new SmartNameInputField(paramId, name, new AttributeSmartNameDictionaryService(this._webCenterAPI, taskTypeID)));
									}, this));
									break;
								case WebCenterPanelConstants.TASKSPECVALUE:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/AttributeValueSmartNameDictionaryService"], _.bind(function (SmartNameInputField, AttributeValueSmartNameDictionaryService) {
										// TODO: improve for restricted sets and dates.
										myPromise.resolve(new SmartNameInputField(paramId, name, new AttributeValueSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.PROJECTATTRVALUE:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/AttributeValueSmartNameDictionaryService"], _.bind(function (SmartNameInputField, AttributeValueSmartNameDictionaryService) {
										// TODO: improve for restricted sets and dates.
										myPromise.resolve(new SmartNameInputField(paramId, name, new AttributeValueSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.RADIO:
									subPromise = this._getSubPanels(nodeXML, nodeCustomSmartNames, wfEditorContext);
									requirejs(["WFEditorWCR/workflow/panel/RadioGroupPanel"], _.bind(function (RadioGroupPanel) {
										// find the name
										var legend = XMLUtils.getFirstChildElementByTagName(nodeXML, "name");
										if (!legend) {
											legend = "";
										} else {
											legend = legend.textContent;
										}

										var radioGroupPanel = new RadioGroupPanel(paramId, subparametersstate, legend);
										subPromise.done(function (panels) {
											_.each(panels, function (panel) {
												radioGroupPanel.add(panel);
											});
											myPromise.resolve(radioGroupPanel);
										});

									}, this));
									break;
								case WebCenterPanelConstants.BOOLEAN:
									subPromise = this._getSubPanels(nodeXML, nodeCustomSmartNames, wfEditorContext);
									requirejs(["WFEditorWCR/workflow/panel/BooleanPanel"], _.bind(function (BooleanPanel) {
										var booleanPanel = new BooleanPanel(paramId, name, subparametersstate);
										subPromise.done(function (panels) {
											_.each(panels, function (panel) {
												booleanPanel.add(panel);
											});
											myPromise.resolve(booleanPanel);
										});
									}, this));
									break;
								case WebCenterPanelConstants.SECRET:
									requirejs(["WFEditorWCR/workflow/panel/SecretPanel"], _.bind(function (SecretPanel) {
										myPromise.resolve(new SecretPanel(paramId, name));
									}, this));
									break;
								case WebCenterPanelConstants.HTML:
									requirejs(["WFEditorWCR/workflow/panel/HTMLPanel", "WFEditorWCR/workflow/panel/HTMLSmartNameDictionaryService"], _.bind(function (HTMLPanel, HTMLSmartNameDictionaryService) {
										myPromise.resolve(new HTMLPanel(paramId, name, new HTMLSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI));
									}, this));
									break;
								case WebCenterPanelConstants.MULTIPARAMETERS:
									requirejs(["WFEditorWCR/workflow/panel/MultiParametersPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiParametersPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiParametersPanel(paramId, i18n.t("wfui.multiparam.parameterName"), i18n.t("wfui.multiparam.parameterValue"), new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTIATTRIBUTES:
									requirejs(["WFEditorWCR/workflow/panel/MultiAttributePanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService", "WFEditorWCR/workflow/panel/DateSmartNameDictionaryService"], _.bind(function (MultiAttributePanel, WorkflowSmartNameDictionaryService, DateSmartNameDictionaryService) {
										myPromise.resolve(new MultiAttributePanel(paramId, i18n.t("wfui.multiprojattr.parameterName"), i18n.t("project.attvalue"), new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), new DateSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTIPROJATTRIBUTES:
									requirejs(["WFEditorWCR/workflow/panel/MultiProjectAttributePanel"], _.bind(function (MultiProjectAttributePanel) {
										myPromise.resolve(new MultiProjectAttributePanel(paramId, i18n.t("projattr.all"), i18n.t("projattr.selected"), this._webCenterAPI));
									}, this));
									break;
								case WebCenterPanelConstants.MULTIDYNAMICCONTENTIMPORTMAP:
									requirejs(["WFEditorWCR/workflow/panel/MultiDynamicContentImportPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiDynamicContentImportPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiDynamicContentImportPanel(paramId, i18n.t("wfui.multidynamiccontentmap.elementtype"), i18n.t("wfui.multidynamiccontentmap.docrefattr"), i18n.t("wfui.multidynamiccontentmap.doctempl"),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), true, this._webCenterAPI));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITEXTCONTENTLANGUAGEMAP:
									requirejs(["WFEditorWCR/workflow/panel/MultiTextContentLanguagePanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTextContentLanguagePanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTextContentLanguagePanel(paramId, i18n.t("wfui.importdc.langindex"), i18n.t("wfui.importdc.langdesc"),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITEXTCONTENTFIELD:
									requirejs(["WFEditorWCR/workflow/panel/MultiTextContentFieldPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTextContentFieldPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTextContentFieldPanel(paramId, i18n.t("wfui.settcfield.field"), i18n.t("wfui.settcfield.value"), this._getTextContentSettingsMap(false),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITEXTCONTENTPERMISSION:
									requirejs(["WFEditorWCR/workflow/panel/MultiTextContentPermissionPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTextContentPermissionPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTextContentPermissionPanel(paramId, i18n.t("wfui.settcfield.contentpermission"), i18n.t("wfui.settcfield.language"), i18n.t("wfui.settcfield.role"),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITEXTCONTENTVALUES:
									requirejs(["WFEditorWCR/workflow/panel/MultiTableParametersPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTableParametersPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTableParametersPanel(paramId, this._getTextContentFieldsMap(),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITEXTCONTENTLANGUAGESTATUS:
									requirejs(["WFEditorWCR/workflow/panel/MultiTextContentElementPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTextContentElementPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTextContentElementPanel(paramId, i18n.t("wfui.settcfield.language"), i18n.t("wfui.settcfield.langstatus"),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITABLESETTINGS:
									requirejs(["WFEditorWCR/workflow/panel/MultiTextContentFieldPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTextContentFieldPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTextContentFieldPanel(paramId, i18n.t("wfui.settcfield.field"), i18n.t("wfui.settcfield.value"), this._getTextContentSettingsMap(true),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITABLEELEMENTCONTENT:
									requirejs(["WFEditorWCR/workflow/panel/MultiTableParametersPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTableParametersPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTableParametersPanel(paramId, this._getTableContentFieldsMap(true),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITABLEELEMENTAPPROVAL:
									requirejs(["WFEditorWCR/workflow/panel/MultiTableParametersPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTableParametersPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTableParametersPanel(paramId, this._getTableContentFieldsMap(false),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.PROJECTSTATUS:
									requirejs(["WFEditorWCR/workflow/panel/SmartNameDropdownPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameDropdownPanel, WorkflowSmartNameDictionaryService) {
										var smartNameDropdownPanel = new SmartNameDropdownPanel(paramId, name, undefined, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames));
										this._addProjectStatusesToDropdown(smartNameDropdownPanel);
										myPromise.resolve(smartNameDropdownPanel);
									}, this));
									break;
								case WebCenterPanelConstants.TASKSTATUS:
									requirejs(["WFEditorWCR/workflow/panel/SmartNameDropdownPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameDropdownPanel, WorkflowSmartNameDictionaryService) {
										var smartNameDropdownPanel = new SmartNameDropdownPanel(paramId, name, undefined, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames));
										this._addTaskStatusesToDropdown(smartNameDropdownPanel);
										myPromise.resolve(smartNameDropdownPanel);
									}, this));
									break;
								case WebCenterPanelConstants.ATTRIBUTECATEGORY:
									requirejs(["WFEditorWCR/workflow/panel/SmartNameDropdownPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameDropdownPanel, WorkflowSmartNameDictionaryService) {
										var smartNameDropdownPanel = new SmartNameDropdownPanel(paramId, name, undefined, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames));
										this._addAttributeCategoriesToDropdown(smartNameDropdownPanel);
										myPromise.resolve(smartNameDropdownPanel);
									}, this));
									break;
								case WebCenterPanelConstants.USERS:
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getUsersGroupsRoles(), null, WorkflowSmartNameConstants.TASKUSERS.PREFIX);
									break;
								case WebCenterPanelConstants.WORKFLOWLIST:
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getWorkflowConnections(), null);
									break;
								case WebCenterPanelConstants.WORKFLOWLISTJMF:
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getWorkflowConnectionsJMF(), null);
									break;
								case WebCenterPanelConstants.TASKSPECNAME:
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getTaskSpecifications(taskTypeID), null);
									break;
								case WebCenterPanelConstants.PROJECTATTRIBUTENAME:
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getAttributes(), null);
									break;
								case WebCenterPanelConstants.DOCREFATTRIBUTEDEFAULT:
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getDocRefAttributes(), null);
									break;
								case WebCenterPanelConstants.DOCREFATTRIBUTE:
									// Adding new option 'Any Document Reference' in Attribute List
									var addCustomKeyValueToSDrop = {
										key: WebCenterPanelConstants.ANYDOCUMNETREFERENCE,
										value: new WebCenterAttribute(WebCenterPanelConstants.ANYDOCUMNETREFERENCE, i18n.t("wfui.anydocrefattr"), null, null)
									};
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getDocRefAttributes(), null, null, addCustomKeyValueToSDrop);
									break;

								case WebCenterPanelConstants.GROUPASSIGNMENT:
								case WebCenterPanelConstants.ROLEASSIGNMENT:
									requirejs(["WFEditorWCR/workflow/panel/DropdownPanel"], _.bind(function (DropdownPanel) {
										var dropdown = new DropdownPanel(paramId, i18n.t(paramId));
										options = [];
										var optionValues = (type === WebCenterPanelConstants.GROUPASSIGNMENT) ? WebCenterPanelConstants.GROUPASSIGNMENT_OPTIONS : WebCenterPanelConstants.ROLEASSIGNMENT_OPTIONS;
										_.each(optionValues, function (option) {
											options.push({
												value: option,
												label: i18n.t(option)
											});
										}, this);
										dropdown.setOptions(options);
										myPromise.resolve(dropdown);
									}, this));
									break;
								case WebCenterPanelConstants.ROLES:
								case WebCenterPanelConstants.GROUPS:
									requirejs(["WFEditorWCR/workflow/panel/SmartNameDropdownPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameDropdownPanel, WorkflowSmartNameDictionaryService) {

										var smartNameDropdownPanel = new SmartNameDropdownPanel(paramId, name, undefined, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames));
										this._webCenterAPI.getUsersGroupsRoles().done(_.bind(function (u, g, r) {
											if (type === WebCenterPanelConstants.GROUPS) {
												this._addGroupsToDropdown(smartNameDropdownPanel, g);
											} else if (type === WebCenterPanelConstants.ROLES) {
												this._addRolesToDropdown(smartNameDropdownPanel, r);
											}
										}, this));
										myPromise.resolve(smartNameDropdownPanel);

									}, this));
									break;
								case WebCenterPanelConstants.ENUM:
									requirejs(["WFEditorWCR/workflow/panel/SmartNameDropdownPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameDropdownPanel, WorkflowSmartNameDictionaryService) {
										var enums = nodeXML.querySelectorAll("Enum");
										options = {};
										_.each(enums, function (enumNode) {
											var option = {};
											option.value = enumNode.textContent;
											option.text = i18n.t(option.value);
											options[option.value] = option;
										}, this);
										myPromise.resolve(new SmartNameDropdownPanel(paramId, name, options, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.JAVASCRIPT:
									requirejs(["WFEditorWCR/workflow/panel/JavaScriptPanel"], _.bind(function (JavaScriptPanel) {
										var options = {};
										var maxScriptLength = data.querySelector("maxScriptLength");
										if (maxScriptLength.textContent !== undefined) {
											maxScriptLength = parseInt(maxScriptLength.textContent, 10);
											options.maxScriptLength = maxScriptLength;
										}
										myPromise.resolve(new JavaScriptPanel(paramId, name, options));
									}, this));
									break;
								default:
									requirejs(["WFEditorWCR/workflow/panel/ReadOnlyStringPanel"], _.bind(function (ReadOnlyStringPanel) {
										myPromise.resolve(new ReadOnlyStringPanel(paramId, name));
									}, this));
									break;

								}
							}
						}
					}
					break;
				case WebCenterPanelConstants.MENU:
					// a menu indicates a tab in the tabbel panel
					deferreds.push(myPromise);
					subPromise = this._getSubPanels(nodeXML, nodeCustomSmartNames, wfEditorContext);
					requirejs(["WFEditorWCR/workflow/panel/TabPanel"], _.bind(function (TabPanel) {
						var tab = new TabPanel([], i18n.t(nodeXML.getAttribute(WebCenterPanelConstants.NAME)));
						subPromise.done(function (panels) {
							_.each(panels, function (panel) {
								tab.add(panel);
							});
							myPromise.resolve(tab);
						});
					}, this));
					break;
				case WebCenterPanelConstants.RADIO:
					deferreds.push(myPromise);
					subPromise = this._getSubPanels(nodeXML, nodeCustomSmartNames, wfEditorContext);
					requirejs(["WFEditorWCR/workflow/panel/RadioPanel"], _.bind(function (RadioPanel) {
						var name = nodeXML.getElementsByTagName(WebCenterPanelConstants.NAME)[0].textContent;
						var radioComponent = new RadioPanel(name, i18n.t(name));
						subPromise.done(function (panels) {
							_.each(panels, function (panel) {
								radioComponent.add(panel);
							});
							myPromise.resolve(radioComponent);
						});

					}, this));
					break;
				}

			}, this);

			var returnPromise = $.Deferred();

			if (deferreds.length > 0) {

				$.when.apply($, deferreds).done(_.bind(function () {
					var panels = [];
					_.each(arguments, function (arg) {
						this._panelAdded(arg);
						panels.push(arg);
					}, this);
					returnPromise.resolve(panels);
				}, this));
			} else {
				returnPromise.resolve([]);
			}

			return returnPromise.promise();
		};

		/**
		 * Add a panel with a static SDrop provider
		 * @param {$.Deferred}                  deferred     - resolved when the sdrop panel is initialized
		 * @param {String}                      paramId      - ID of the parameter
		 * @param {String}                      name         - name of the parameter
		 * @param {String}                      defaultValue - default value of the parameter
		 * @param {Promise.<Map.<String, Any>>} mapPromise   - promise for the data of the SDrop
		 * @param {Function}                    [orderMethod]  - optional order method 
		 * @param {String}                      [idPrefix]    - Prefix for the keys in the map  
		 * @param {Object}                      [addCustomKeyValueToSDrop] - Object contains Key and Value. Structure of this object is like {key:"key", value: Any}. The idPrefix if supplied is NOT applied to this object.                                            
		 */
		WebCenterNode.prototype._addStaticSDropPanel = function (deferred, paramId, name, defaultValue, mapPromise, orderMethod, idPrefix, addCustomKeyValueToSDrop) {
			requirejs(["WFEditorWCR/workflow/panel/StaticSDropPanel"], _.bind(function (StaticSDropPanel) {
				var promise = mapPromise;
				var newDeferredMap = $.Deferred();

				mapPromise.done(function (map) {
					var newMap = map.transformKeys(function (key) {
						if (idPrefix) {
							return idPrefix + key;
						} else {
							return key;
						}
					});

					if (addCustomKeyValueToSDrop) {
						newMap.add(addCustomKeyValueToSDrop.key, addCustomKeyValueToSDrop.value);
					}
					newDeferredMap.resolve(newMap);
				});
				promise = newDeferredMap.promise();

				var sdropPanel = new StaticSDropPanel(paramId, name, defaultValue, promise, orderMethod);
				deferred.resolve(sdropPanel);
			}, this));
		};


		/**
		 * returns the parameters and outputs for the given nodeType, the result is cached based on the nodeType
		 * @param {String} nodeType - the type of the node
		 * @param {NodeList} workflowDefinitionXML - the xml that should be searched in if it hasn't been cached yet                         
		 * @returns Object {
		 *				parameters: Array.<Parameter> // see _getParameters
		 *				outputs: Array.<Object> // see _getOutputPins
		 *			}
		 */
		WebCenterNode.prototype._getConfigDetailsForNodeType = _.memoize(function (nodeType, workflowDefinitionXML) {
			var parameters;
			var outputs;
			var helpTopic;
			var xmls = workflowDefinitionXML.querySelectorAll(
				"TaskExecuterFactories TaskExecuterFactory");
			_.each(xmls, function (xml) {
				if (xml.getElementsByTagName("type")[0].textContent === nodeType) {
					var config = xml.querySelector("config-inline");
					parameters = _getParameters(config); // _getParameters static function because non bound memoized function
					outputs = _getOutputPins(config); // _getOutputPins static function because non bound memoized function
					helpTopic = _getHelpTopic(config);
				}
			}, this);
			return {
				parameters: parameters,
				outputs: outputs,
				helpTopic: helpTopic
			};
		});

		/**
		 * Get the help topic of this node from the given XML
		 * @param   {NodeList} xml
		 * @returns {String}
		 */
		var _getHelpTopic = function (xml) {
			var helpTopicNode = xml.getElementsByTagName("helptopic")[0];
			if (helpTopicNode) {
				return helpTopicNode.textContent;
			}
			return "";
		};

		/**
		 * returns the default parameters as specified in xml
		 * @param {NodeList} xml
		 * @returns {Array.<Parameter>}
		 */
		var _getParameters = function (xml) {
			var parameters = [];
			_.each(xml.childNodes, function (node) {
				var tagName = node.tagName;
				switch (tagName) {
				case WebCenterPanelConstants.MENUS:
					parameters = parameters.concat(_getParameters(node));
					break;
				case WebCenterPanelConstants.PARAMETERS:
					parameters = parameters.concat(_getParameters(node));
					break;
				case WebCenterPanelConstants.PARAMETER:
					var paramId, defaultValue;
					_.each(node.childNodes, function (child) {
						if (child.tagName === WebCenterPanelConstants.NAME) {
							paramId = child.textContent;
						} else if (child.tagName === WebCenterPanelConstants.DEFAULT) {
							var enums = child.getElementsByTagName(WebCenterPanelConstants.ENUM);
							if (enums.length > 0) {
								defaultValue = enums[0].textContent;
							} else {
								defaultValue = child.textContent;
							}
						}
					}, this);
					if (paramId) {
						parameters = parameters.concat([new WebCenterParameter(paramId, defaultValue)]);
						parameters = parameters.concat(_getParameters(node));
					}
					break;
				case WebCenterPanelConstants.MENU:
					parameters = parameters.concat(_getParameters(node));
					break;
				case WebCenterPanelConstants.RADIO:
					parameters = parameters.concat(_getParameters(node));
					break;
				}
			}, this);
			return parameters;
		};

		/**
		 * returns the outputs as specified in the xml
		 * @param {NodeList} xml
		 * @returns {Array.<{
		 *		type: type of the pin (0, 1 or 2)
		 *		name: name of the pin
		 *	}>}
		 */
		var _getOutputPins = function (xml) {
			var outputPins = [];

			var outputs = xml.getElementsByTagName(WebCenterPanelConstants.OUTPUTS);

			if (outputs[0]) {
				var outputList = outputs[0].getElementsByTagName(WebCenterPanelConstants.OUTPUT);

				_.each(outputList, function (output) {
					var name = output.getElementsByTagName(WebCenterPanelConstants.NAME)[0];
					var type = output.getElementsByTagName(WebCenterPanelConstants.TYPE)[0];

					if (name && type) {
						type = type.textContent;
						name = name.textContent;
						outputPins.push({
							type: type,
							name: name
						});
					}
				});
			}

			return outputPins;
		};

		/**
		 * Add project statuses to task status dropdown
		 * @param {SmartNameDropdownPanel} smartNameDropdownPanel - panel to add options to
		 */
		WebCenterNode.prototype._addProjectStatusesToDropdown = function (smartNameDropdownPanel) {
			this._webCenterAPI.getProjectStatusList().done(_.bind(function (statusesMap) {
				this._addStatusesToDropdown(statusesMap, smartNameDropdownPanel);
			}, this));
		};

		/**
		 * Add task statuses to task status dropdown
		 * @param {SmartNameDropdownPanel} smartNameDropdownPanel - panel to add options to
		 */
		WebCenterNode.prototype._addTaskStatusesToDropdown = function (smartNameDropdownPanel) {
			this._webCenterAPI.getWorkflowTaskStatusList(this.getWorkflow().getTaskTypeID()).done(_.bind(function (statusesMap) {
				this._addStatusesToDropdown(statusesMap, smartNameDropdownPanel);
			}, this));
		};

		/**
		 * Add statuses to a status dropdown
		 * @param {SmartNameDropdownPanel} smartNameDropdownPanel - panel to add options to
		 */
		WebCenterNode.prototype._addStatusesToDropdown = function (statusesMap, smartNameDropdownPanel) {
			var statuses = statusesMap.values().sort(function (a, b) {
				return a.getDisplayOrder() - b.getDisplayOrder();
			});

			var options = [];
			_.each(statuses, function (status) {
				options.push({
					text: status.getName(),
					value: status.getId()
				});
			});
			smartNameDropdownPanel.setOptions(options);
		};

		/**
		 * Add Attribute Categories to smart name dropdown
		 * @param {SmartNameDropdownPanel} smartNameDropdownPanel - panel to add options to
		 */
		WebCenterNode.prototype._addAttributeCategoriesToDropdown = function (smartNameDropdownPanel) {
			this._webCenterAPI.getAttributeCategoryList().done(_.bind(function (attributeCategoriesMap) {
				var options = [];
				_.each(attributeCategoriesMap.values(), function (attCat) {
					options.push({
						text: attCat.getName(),
						value: attCat.getId()
					});
				});
				smartNameDropdownPanel.setOptions(options);
			}, this));
		};

		/**
		 * add correct groups to a groups dropdown
		 * @param   {SmartNameDropdownPanel}       smartNameDropdownPanel 
		 * @param   {Map.<String,WebCenterGroup>}  g
		 */
		WebCenterNode.prototype._addGroupsToDropdown = function (smartNameDropdownPanel, g) {
			var mappedGroups = _.mapObject(g.asObject(), function (group) {
				return {
					text: group.getName(),
					value: WorkflowSmartNameConstants.TASKGROUPS.PREFIX + group.getId()
				};
			});
			smartNameDropdownPanel.setOptions(mappedGroups);

		};


		/**
		 * add correct roles to a roles dropdown
		 * @param   {SmartNameDropdownPanel}       smartNameDropdownPanel 
		 * @param   {Map.<String,WebCenterRole>}   r
		 */
		WebCenterNode.prototype._addRolesToDropdown = function (smartNameDropdownPanel, r) {
			// Add default roles 
			var invprojectmgrName = i18n.t(WebCenterPanelConstants.ROLE.PROJECTMANAGER.NAME);
			var invsalespersonName = i18n.t(WebCenterPanelConstants.ROLE.SALESPERSON.NAME);
			var invworkflowmaster = i18n.t(WebCenterPanelConstants.ROLE.WORKFLOWMASTER.NAME);
			var options = {};
			options[WebCenterPanelConstants.ROLE.PROJECTMANAGER.VALUE] = {
				text: "[" + invprojectmgrName + "]",
				value: WebCenterPanelConstants.ROLE.PROJECTMANAGER.VALUE
			};
			options[WebCenterPanelConstants.ROLE.SALESPERSON.VALUE] = {
				text: "[" + invsalespersonName + "]",
				value: WebCenterPanelConstants.ROLE.SALESPERSON.VALUE
			};
			options[WebCenterPanelConstants.ROLE.WORKFLOWMASTER.VALUE] = {
				text: "[" + invworkflowmaster + "]",
				value: WebCenterPanelConstants.ROLE.WORKFLOWMASTER.VALUE
			};

			// Map roles to the right format and add prefix to the id
			var mappedRoles = _.mapObject(r.asObject(), function (role) {
				return {
					text: role.getName(),
					// Roles in this context also need the group prefix due to historic reasons
					value: WorkflowSmartNameConstants.TASKGROUPS.PREFIX + role.getId()
				};
			});

			smartNameDropdownPanel.setOptions(_.extend(options, mappedRoles));
		};

		/**
		 * Constructs a map for the fields required for Content Settings
		 * @param isTableContent
		 * @returns {Map.<String,WebCenterTextContentProperty>}
		 */
		WebCenterNode.prototype._getTextContentSettingsMap = function (isTableContent) {
			var settingsMap = new Map();

			if (!isTableContent) {
				settingsMap.add(ContentConstants.UpdateField.ELEMENT_TYPE, new WebCenterTextContentProperty(ContentConstants.UpdateField.ELEMENT_TYPE, i18n.t('settcfield.elementtype')));
			}
			settingsMap.add(ContentConstants.UpdateField.APPROVAL_TYPE, new WebCenterTextContentProperty(ContentConstants.UpdateField.APPROVAL_TYPE, i18n.t('settcfield.approvaltype')));
			settingsMap.add(ContentConstants.UpdateField.TEXTTYPE, new WebCenterTextContentProperty(ContentConstants.UpdateField.TEXTTYPE, i18n.t('settcfield.texttype')));
			settingsMap.add(ContentConstants.UpdateField.INVARIANT, new WebCenterTextContentProperty(ContentConstants.UpdateField.INVARIANT, i18n.t('settcfield.invariant')));

			return settingsMap;
		};

		/**
		 * Constructs a map for the fields required for Table Content tabs
		 * @param isValuesTab
		 * @returns {Map.<String,String>}
		 */
		WebCenterNode.prototype._getTableContentFieldsMap = function (isValuesTab) {
			var fieldMap = new Map();
			fieldMap.add("wfui.settc.element", "elementtype_smartdropdown");
			fieldMap.add("wfui.settc.instseq", "instanceseq_smartinput");
			fieldMap.add("wfui.settc.language", "language_smartdropdown");

			if (isValuesTab) {
				fieldMap.add("wfui.settc.action", "languageaction_smartdropdown");
				fieldMap.add("wfui.settc.value", "content_smartinput");
			} else {
				fieldMap.add("wfui.settc.status", "approval_smartdropdown");
			}

			return fieldMap;
		};

		/**
		 * Constructs a map for the fields required for text Content tabs
		 * @returns {Map.<String,String>}
		 */
		WebCenterNode.prototype._getTextContentFieldsMap = function () {
			var fieldMap = new Map();
			fieldMap.add("wfui.settc.language", "language_smartdropdown");
			fieldMap.add("wfui.settc.action", "languageaction_smartdropdown");
			fieldMap.add("wfui.settc.value", "content_smartinput");

			return fieldMap;
		};


		/**
		 * Check if there is a panel of type that is specific for a node. Should be overwritten by the specific implementation of the webcenternode
		 * @param   {$.Deferred}               deferred         - deferred object to be resolved with the subpanel
		 * @param   {String}                   type             - the parameter type
		 * @param   {String}                   paramId          - id of the parameter
		 * @param   {NodeList}                 nodeXML          - XML of the node
		 * @param   {Array.<CustomSmartNames>} customSmartNames - list of custom smartnames
		 * @returns {Boolean}                  is there a node specific panel with this type? 
		 *                                     return true if so and the deferred will be resolved, false if deferred won't be resolved 
		 */
		WebCenterNode.prototype._getNodeSpecificSubPanel = function (deferred, type, paramId, nodeXML, customSmartNames) {
			return false;
		};

		/**
		 * @override
		 * @param isHighlighted
		 */
		WebCenterNode.prototype.setHighlighted = function (isHighlighted) {
			_.each(this.getPins(), function (pin) {
				pin.setHighlighted(isHighlighted);
			});
			Node.prototype.setHighlighted.call(this, isHighlighted);
		};

		/**
		 * called when a panel is added to the configuration panel
		 * @param {AbstractPanelComponent} panel 
		 */
		WebCenterNode.prototype._panelAdded = function (panel) {
			// Override me	
		};

		/**
		 * returns the location of the help page for this node
		 * @return {String}
		 */
		WebCenterNode.prototype.getHelpTopic = function () {
			return this._helpTopic;
		};

		/** 
		 * This is used in WebCenterWorkflowFactory.createNode to create a copy of this node.
		 * @override
		 */
		WebCenterNode.prototype.getStateObjectForCopy = function () {
			return {
				nodeType: this.getNodeType(),
				configurationXML: this.toXML(),
				resourceXML: this.getResourceXML(),
				label: this.getLabel()
			};
		};

		/** 
		 * initializes the warning policy params
		 */
		WebCenterNode.prototype._initWarningPolicyParams = function () {
			this._warningPolicyParam = new WebCenterParameter(WebCenterNodeConstants.WarningPolicy.WARNINGPOLICY, "", WebCenterNodeConstants.WarningPolicy.DEDICATED);
			this._warningPolicyOk = new WebCenterParameter(WebCenterNodeConstants.WarningPolicy.OK, "");
			this._warningPolicyError = new WebCenterParameter(WebCenterNodeConstants.WarningPolicy.ERROR, "");
			this._warningPolicyDedicated = new PinParameter(WebCenterNodeConstants.WarningPolicy.DEDICATED, "", "", [], _.bind(function () {
				var pins = [];
				_.each(this._warningOutputNames, function (warningOutputName) {
					pins.push(new OutputPinWarning(this._workflow, this, warningOutputName, i18n.t(warningOutputName)));
				}, this);
				return pins;
			}, this));
			this._warningPolicyDedicated.setNewPin();

			this._updateWarningPins();

		};

		/**
		 * sets the warning policy parameters
		 * @param {Array.<WebCenterParameter>}
		 */
		WebCenterNode.prototype.setWarningPolicy = function (config) {
			_.each(config, _.bind(function (param) {
				if (param.getName() === WebCenterNodeConstants.WarningPolicy.WARNINGPOLICY) {
					this._warningPolicyParam = param;
				} else if (param.getName() === WebCenterNodeConstants.WarningPolicy.OK) {
					this._warningPolicyOk = param;
				} else if (param.getName() === WebCenterNodeConstants.WarningPolicy.DEDICATED) {
					this._warningPolicyDedicated = param;
				} else {
					this._warningPolicyError = param;
				}
			}, this));
			this._updateWarningPins();
		};

		/**
		 * updates the warning pin according to the warning policy params. If the dedicated policy is set, the warning output pin is added
		 */
		WebCenterNode.prototype._updateWarningPins = function () {
			var i;
			var pin;
			var pins;
			if (this._warningPolicyParam.getValue() === WebCenterNodeConstants.WarningPolicy.DEDICATED) {

				pins = this._warningPolicyDedicated.pins;

				// Check if the warning pins exist already
				for (var p in pins) {
					if (pins.hasOwnProperty(p)) {
						if (_.contains(this._pins.getItemsAsObject(), pins[p])) {
							// The warning pins exist already
							return;
						}
					}
				}

				// look for index of first error pin
				for (i = 0; i < this._pins.getLength(); i++) {
					if (this._pins.getItemByIndex(i).getPinType() === WebCenterPinConstants.Type.OutError) {
						break;
					}
				}
				_.each(pins, function (pin) {
					// Add all warning pins and the edges
					this.addPin(pin, i);
					_.each(pin.getEdges(), _.bind(function (edge) {
						edge.getToPin().addEdge(edge);
						this._workflow.addEdge(edge);
					}, this));
					i++;
				}, this);

			} else {
				pins = [];
				// Get all warning pins
				for (i = 0; i < this._pins.getLength(); i++) {
					pin = this._pins.getItemByIndex(i);
					if (pin.getPinType() === WebCenterPinConstants.Type.OutWarning) {
						pins.push(pin);
					}
				}

				// Remove them
				_.each(pins, function (pin) {
					this.removePin(pin);
					_.each(pin.getEdges(), _.bind(function (edge) {
						edge.getToPin().removeEdge(edge);
						this._workflow.removeEdge(edge);
					}, this));
				}, this);

			}
		};

		/**
		 * returns the warning policy params
		 * @return {Array.<WebCenterParameter>}
		 */
		WebCenterNode.prototype.getWarningPolicy = function () {
			var config = [];
			config.push(this._warningPolicyParam.copy());
			config.push(this._warningPolicyOk.copy());
			config.push(this._warningPolicyError.copy());
			config.push(this._warningPolicyDedicated.copy());
			return config;
		};

		/**
		 * returns the label key for the warning output
		 * @return {Array.<String>}
		 */
		WebCenterNode.prototype.getWarningLabels = function () {
			return this._warningOutputNames;
		};

		/**
		 * returns the selected warning policy as string
		 * @return {String}
		 */
		WebCenterNode.prototype.getWarningPolicyString = function () {
			return this._warningPolicyParam.getValue();
		};

		/**
		 * can the warning policy be edited?
		 * @return {Boolean}
		 */
		WebCenterNode.prototype.canEditWarningPolicy = function () {
			return this._canEditWarningPolicy;
		};


		return WebCenterNode;
	});