define([
		"utils/ShapeUtils",
		"WFEditor/workflow/node/NodeView",
		"WFEditor/utils/IconDataStorage",
		"WFEditor/workflow/ModelConstants",
		"WFEditorWCR/scene/WebCenterViewConstants",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditor/scene/hitarea/HitArea",
		"utils/webi18n!WFEditorWCR",
		"underscore"
	],
	function (
		ShapeUtils,
		NodeView,
		IconDataStorage,
		ModelConstants,
		WebCenterViewConstants,
		WebCenterPinConstants,
		WebCenterNodeConstants,
		HitArea,
		i18n,
		_
	) {
		"use strict";

		/**
		 * The view for a webcenter node item. It renders the icon and the label of the node.
		 * It uses the IconDataStorage to fetch the icon.
		 *
		 * @exports WebCenterNodeView
		 * @class
		 * @augments NodeView
		 *
		 * @param {WebCenterNode}
		 * @param {Scene}
		 */
		var WebCenterNodeView = function (nodeModel, scene) {
			NodeView.call(this, nodeModel);

			/**
			 * @member {Number} the max text width of the labels
			 * @private
			 */
			this._maxTextWidth = 0;

			/**
			 * @member {String} the text align for the labels
			 * @private
			 */
			this._textAlign = "";

			/**
			 * @member {Object} cache the label lines so we don't have to recalculate all the time
			 * @private
			 */
			this._labelLines = null;

			/**
			 * @member {Boolean}
			 * @private
			 */
			this._labelLocationDirty = true;

			// Observer location changes
			this._observe(nodeModel, "changed.location", this._recalculateRenderInformation);
			this._observe(nodeModel, "changed.*", this.makeDirty);
			this._observe(nodeModel, "changed.label", this._recalculateRenderInformation);
			this._observe(nodeModel, "Pin.added", this._recalculateRenderInformation);
			this._observe(nodeModel.getWorkflow(), "changed.debugMode", this._recalculateRenderInformation);


		};

		WebCenterNodeView.prototype = Object.create(NodeView.prototype);
		WebCenterNodeView.prototype.constructor = WebCenterNodeView;

		/**
		 * Updates the bounding box, hit areas and label
		 */
		WebCenterNodeView.prototype._recalculateRenderInformation = function () {
			this._labelLocationDirty = true;
			this._updateBoundingRect();
			this._updateHitAreas();
			this.makeDirty();
		};

		/**
		 * @override
		 * Renders the node on the provided canvas context. Only renders if the icon is available.
		 * If the icon is not yet loaded, the iconstorage fetches the icon and the layer is marked dirty
		 * when the icon is correctly fetched.
		 *
		 * @param   {CanvasRenderingContext2D} context
		 * @param   {Layer} layer
		 * @param 	{Boolean} quickRender
		 */
		WebCenterNodeView.prototype.render = function (context, layer, quickRender) {
			var location = this._model.getLocation();
			var iconStorage = IconDataStorage.getInstance(); //get the singleton instance of the datastorage
			//request the icon from the datastorage, returns the icon if available, if not it starts loading the icon.
			if (this._model.getIconUrl()) {
				var icon = iconStorage.get(this._model.getIconUrl(), _.bind(this.makeDirty, this));
				if (icon) { //only render if the icon was already fetched

					var width = icon.naturalWidth;
					var height = icon.naturalHeight;


					// if the icon is too big, resize it proportionally
					if (width > WebCenterViewConstants.NodeIconSize || height > WebCenterViewConstants.NodeIconSize) {
						var ratio = width / height;
						if (ratio > 1) {
							width = WebCenterViewConstants.NodeIconSize;
							height = width / ratio;
						} else {
							height = WebCenterViewConstants.NodeIconSize;
							width = height * ratio;
						}
					}
					context.drawImage(icon, location.x, location.y, width, height);

					if (this._model.isSelected() || !this._model.isEnabled()) {
						// draw overlay
						context.save();

						if (!this._model.isEnabled()) {
							context.fillStyle = WebCenterViewConstants.NodeOverlayColorDisabled;
						} else {
							context.fillStyle = WebCenterViewConstants.NodeOverlayColorSelected;
						}
						var x = location.x + WebCenterViewConstants.NodeIconPadding;
						var y = location.y + WebCenterViewConstants.NodeIconPadding;
						var radius = 3;
						width = width - 2 * WebCenterViewConstants.NodeIconPadding;
						height = height - 2 * WebCenterViewConstants.NodeIconPadding;
						context.beginPath();
						context.moveTo(x + radius, y);
						context.lineTo(x + width - radius, y);
						context.quadraticCurveTo(x + width, y, x + width, y + radius);
						context.lineTo(x + width, y + height - radius);
						context.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
						context.lineTo(x + radius, y + height);
						context.quadraticCurveTo(x, y + height, x, y + height - radius);
						context.lineTo(x, y + radius);
						context.quadraticCurveTo(x, y, x + radius, y);
						context.closePath();
						context.fill();
						context.restore();

					}
				}
			}

			// only render labels in normal render mode and when the label isn't being edited
			if (!quickRender && !this._model.isEditingLabel()) {
				// Label rendering
				var labelLines = this._getLabelLines(context, this._model.getLabel(true), location, new ShapeUtils.Size(WebCenterViewConstants.NodeIconSize, WebCenterViewConstants.NodeIconSize));

				context.save();

				this._renderLabelBackground(context, labelLines);

				if (this._model.isSelected()) {
					context.fillStyle = WebCenterViewConstants.NodeLabelSelectedFontColor;
				} else {
					context.fillStyle = WebCenterViewConstants.NodeLabelColor;
				}
				context.font = WebCenterViewConstants.NodeLabelFont;

				_.each(labelLines, function (labelLine) {
					context.textAlign = this._textAlign;
					context.fillText(labelLine.text, labelLine.location.x, labelLine.location.y);
				}, this);

				context.restore();
			}

			// Render visible parameters under label
			if (!quickRender) {
				var visibleParams = this._model.getVisibleParameters();
				if (_.size(visibleParams) > 0) {
					var labelRect = this.getLabelRect();

					context.save();
					context.fillStyle = WebCenterViewConstants.NodeVisibleParameterColor;
					context.font = WebCenterViewConstants.NodeVisibleParameterFont;
					context.textAlign = this._textAlign;

					var xLoc = labelRect.center().x;
					var yLoc = labelRect.bottom() + WebCenterViewConstants.NodeLabelLineHeight - 3; // a bit less spacing

					_.each(visibleParams, function (param) {
						context.fillText(i18n.t(param.getName()) + ": " + param.getValue(), xLoc, yLoc);
						yLoc += WebCenterViewConstants.NodeLabelLineHeight;
					});
				}
			}
		};

		/**
		 * @Override
		 */
		WebCenterNodeView.prototype._updateHitAreas = function (context) {
			var nodeLocation = this._model.getLocation();
			var nodeRect = new ShapeUtils.Rect(nodeLocation, new ShapeUtils.Size(WebCenterViewConstants.NodeIconSize, WebCenterViewConstants.NodeIconSize));
			var labelRect = this.getLabelRect();
			if (labelRect.size.width === 0) {
				// There should be at least a little hit area for labels
				labelRect.size.width = WebCenterViewConstants.NodeIconSize;
				labelRect.size.height = WebCenterViewConstants.NodeLabelLineHeight;
			}
			var nodeSelectHitAreaCursor = this._model.getWorkflow().getPrivilege(ModelConstants.Privileges.viewNodeConfiguration) ? "pointer-cursor" : undefined;
			var editLabelHitAreaCursor = this._model.getWorkflow().getPrivilege(WebCenterNodeConstants.Privileges.editLabels) ? "text-cursor" : undefined;
			this._setHitAreas(
				[new HitArea(nodeRect, this, ModelConstants.HitAreaClass.NodeSelect, undefined, {
					cursor: nodeSelectHitAreaCursor
				}), new HitArea(labelRect, this, WebCenterNodeConstants.HitAreas.LABEL, 15, {
					cursor: editLabelHitAreaCursor
				})]);
		};

		/**
		 * Get the location for a certain pin
		 * @param {Pin} pin for which the location needs to be returned
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterNodeView.prototype.getLocationForPin = function (pin) {
			var pinLocation = this._model.getLocation().copy();
			pinLocation.y += WebCenterViewConstants.PinToNodeTop;

			var pinSeqNumber = this._model.getPinSequenceNumber(pin);

			if (pin === this._model.getInputPin()) {
				// input pin (we only have 1 location for this
				pinLocation.x -= WebCenterViewConstants.PinInToNodeLeft;
			} else {
				// output pin				
				pinLocation.x += WebCenterViewConstants.NodeIconSize + WebCenterViewConstants.PinOutToNodeRight;
			}

			pinLocation.y += pinSeqNumber * (WebCenterViewConstants.PinRadius * 2 + WebCenterViewConstants.PinToPinSpacing);

			return pinLocation;
		};

		/**
		 * Do the output pins come lower than the node icon?
		 * if so, the label should be right aligned
		 * @param {ShapeUtils.Size} nodeSize
		 * @returns {Boolean}
		 */
		WebCenterNodeView.prototype._shouldAdjustLabelPositionForPins = function (nodeSize) {
			var totalPinSize = (WebCenterViewConstants.PinRadius * 2) + WebCenterViewConstants.PinToPinSpacing;

			return (this._model.getOutputPinCount() * totalPinSize) >= nodeSize.height;
		};

		/**
		 * Get the given labelText wrapped in lines according to NodeLabelMaxWidth
		 * @param   {CanvasRenderingContext2D} context      - The canvas context
		 * @param   {String}                   labelText    - The label text
		 * @param   {ShapeUtils.Point}         nodeLocation - The top left location of the node (the icon)
		 * @param   {ShapeUtils.Size}          nodeSize     - The size of the node (the icon)
		 * @returns {Array.<Object>}           An array of lines with objects build like
		 *                                                  {	{String} text
		 *                                                      {ShapeUtils.Point} location
		 *                                                  	{Number} textWidth
		 *                                                  }
		 */
		WebCenterNodeView.prototype._getLabelLines = function (context, labelText, nodeLocation, nodeSize) {
			if (this._labelLocationDirty) {
				this._maxTextWidth = 0;
				var lines = [];

				context.fillStyle = WebCenterViewConstants.NodeLabelColor;
				context.font = WebCenterViewConstants.NodeLabelFont;

				var labelLocation = this.getLabelStartPoint(nodeLocation, nodeSize);

				var remainder = labelText;
				while (remainder.length > 0) {
					var line = this._getNextLineToken(context, remainder);
					var textWidth = context.measureText(line).width;
					if (textWidth > this._maxTextWidth) {
						this._maxTextWidth = textWidth;
					}
					lines.push({
						text: line.trim(),
						location: labelLocation.copy(),
						textWidth: textWidth
					});
					remainder = remainder.substring(line.length).trim();
					labelLocation.y += WebCenterViewConstants.NodeLabelLineHeight;
				}
				this._labelLines = lines;
				this._labelLocationDirty = false;
			}
			return this._labelLines;
		};

		/**
		 * returns the start point of the label
		 * @param   {ShapeUtils.Point} nodeLocation - the location of the node
		 * @param   {ShapeUtils.Size}   nodeSize - the size of the node
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterNodeView.prototype.getLabelStartPoint = function (nodeLocation, nodeSize) {
			if (typeof nodeLocation === "undefined") {
				nodeLocation = this._model.getLocation();
			}
			if (typeof nodeSize === "undefined") {
				nodeSize = new ShapeUtils.Size(WebCenterViewConstants.NodeIconSize, WebCenterViewConstants.NodeIconSize);
			}
			var labelLocation = nodeLocation.copy();

			if (this._shouldAdjustLabelPositionForPins(nodeSize)) {
				// label is aligned right because of pins
				this._textAlign = "right";
				labelLocation.y += nodeSize.height + WebCenterViewConstants.NodeLabelGap;
				labelLocation.x += nodeSize.width - WebCenterViewConstants.NodeIconPadding;
			} else {
				// label location is bottom-center of icon due to textAlign center
				this._textAlign = "center";
				labelLocation.y += nodeSize.height + WebCenterViewConstants.NodeLabelGap;
				labelLocation.x += nodeSize.width / 2;
			}

			return labelLocation;
		};

		/**
		 * Get next line token based on the maximum label width for the given text.
		 * It will try to split on a space if too big and if not present, to the point where it no longer fits the line.
		 * @param   {CanvasRenderingContext2D} context - The canvas context
		 * @param   {String} text - the text to get the first token from
		 * @returns {String} the next line token
		 */
		WebCenterNodeView.prototype._getNextLineToken = function (context, text) {
			var words = text.split(" ");
			var line = '';
			for (var i = 0; i < words.length; i++) {
				var word = words[i];
				line += i > 0 ? " " + word : word;
				var textMetrics = context.measureText(line);
				if (textMetrics.width > WebCenterViewConstants.NodeLabelMaxWidth) {
					if (i > 0) {
						// space found
						line = line.substring(0, line.length - word.length - 1);
						break;
					} else {
						// no space found -> find another split
						var newLine = "";
						for (var j = 0; j < line.length; j++) {
							newLine += line.charAt(j);
							textMetrics = context.measureText(newLine);
							if (textMetrics.width > WebCenterViewConstants.NodeLabelMaxWidth) {
								line = line.substring(0, newLine.length - 1);
								break;
							}
						}
					}
				}
			}
			return line;
		};

		/**
		 * @override
		 */
		WebCenterNodeView.prototype._updateBoundingRect = function () {
			var location = this._model.getLocation().copy();

			this._context.save();
			var labelLines = this._getLabelLines(this._context, this._model.getLabel(true), location, new ShapeUtils.Size(WebCenterViewConstants.NodeIconSize, WebCenterViewConstants.NodeIconSize));
			this._context.restore();

			// we assume WebCenterViewConstants.NodeLabelMaxWidth as label width
			if (this._textAlign === "center") {
				location.x += WebCenterViewConstants.NodeIconSize / 2 - WebCenterViewConstants.NodeLabelMaxWidth / 2 - WebCenterViewConstants.NodeLabelBackgroundPaddingWidth;
			} else {
				location.x += WebCenterViewConstants.NodeIconSize - WebCenterViewConstants.NodeLabelMaxWidth - WebCenterViewConstants.NodeLabelBackgroundPaddingWidth;

			}
			var width = Math.max(WebCenterViewConstants.NodeLabelMaxWidth + 2 * WebCenterViewConstants.NodeLabelBackgroundPaddingWidth, WebCenterViewConstants.NodeIconSize);
			// icon + gap + x lines: note that the first line is drawn on icon + gap y so, that's why the length - 1 is here
			var height = WebCenterViewConstants.NodeIconSize + WebCenterViewConstants.NodeLabelGap + (labelLines.length - 1) * WebCenterViewConstants.NodeLabelLineHeight + 2 * WebCenterViewConstants.NodeLabelBackgroundPaddingHeight + 1;


			this._setBoundingRect(new ShapeUtils.Rect(location, new ShapeUtils.Size(width, height)).getNormalizedRectArray());
		};


		/**
		 * render the label background for a selected node
		 * @param {CanvasRenderingContext2D}   context
		 * @param {Array.<Object>} labelLines info about the lines of the label
		 *
		 */
		WebCenterNodeView.prototype._renderLabelBackground = function (context, labelLines) {

			var labelRect = this.getLabelRect();
			var x = labelRect.origin.x;
			var y = labelRect.origin.y;
			var width = labelRect.size.width;
			var height = labelRect.size.height;
			if (width === 0 || height === 0) {
				return;
			}
			if (this._model.isSelected()) {
				context.fillStyle = WebCenterViewConstants.NodeLabelSelectedBackgroundColor;
			} else {
				context.fillStyle = WebCenterViewConstants.NodeLabelBackgroundColor;

			}

			var radius = 5;
			context.beginPath();
			context.moveTo(x + radius, y);
			context.lineTo(x + width - radius, y);
			context.quadraticCurveTo(x + width, y, x + width, y + radius);
			context.lineTo(x + width, y + height - radius);
			context.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
			context.lineTo(x + radius, y + height);
			context.quadraticCurveTo(x, y + height, x, y + height - radius);
			context.lineTo(x, y + radius);
			context.quadraticCurveTo(x, y, x + radius, y);
			context.closePath();
			context.fill();


		};

		/**
		 * returns the rect in which the label should be drawn
		 * @returns {ShapeUtils.Rect} 
		 */
		WebCenterNodeView.prototype.getLabelRect = function () {
			var labelLines = this._getLabelLines(this._context, this.getModel().getLabel(true), this.getModel().getLocation(), new ShapeUtils.Size(WebCenterViewConstants.NodeIconSize, WebCenterViewConstants.NodeIconSize));
			var maxTextWidth = this._maxTextWidth;
			var y = Infinity;
			var bottomY = -Infinity;
			var x = +Infinity;

			// get left most x and top most y value to start the rendering
			_.each(labelLines, function (labelLine) {
				if (labelLine.location.y < y) {
					y = labelLine.location.y;
				}
				if (labelLine.location.y > bottomY) {
					bottomY = labelLine.location.y;
				}
				if (labelLine.location.x < x) {
					x = labelLine.location.x;
				}
			});



			// the height of all text 
			var textHeight = bottomY - y + 12;

			// add horizontal padding
			var paddingHorizontal = WebCenterViewConstants.NodeLabelBackgroundPaddingWidth;
			var width = maxTextWidth + 2 * paddingHorizontal;

			// add vertical padding
			var paddingVertical = WebCenterViewConstants.NodeLabelBackgroundPaddingHeight;
			var height = textHeight + 2 * paddingVertical;

			// the starting x point depends on the alignment of the text
			if (this._textAlign === "center") {
				x = x - (maxTextWidth / 2) - paddingHorizontal;
			} else if (this._textAlign === "right") {
				x = x - (maxTextWidth) - paddingHorizontal;
			}
			y = y - 10 - paddingVertical;
			if (x === Infinity || y == Infinity) {

				var labelLocation = this.getLabelStartPoint(this._model.getLocation(), new ShapeUtils.Size(WebCenterViewConstants.NodeIconSize, WebCenterViewConstants.NodeIconSize));
				x = labelLocation.x;
				y = labelLocation.y -= WebCenterViewConstants.NodeLabelLineHeight;
				width = 0;
				height = 0;
			}
			return new ShapeUtils.Rect(new ShapeUtils.Point(x, y), new ShapeUtils.Size(width, height));

		};

		/**
		 * returns the text alignment of the node's label
		 * @returns {String} 
		 */
		WebCenterNodeView.prototype.getTextAlignment = function () {
			return this._textAlign;
		};

		/**
		 * @override
		 */
		WebCenterNodeView.prototype.getCenter = function () {
			var center = this._model.getLocation().copy();
			center.x += WebCenterViewConstants.NodeIconSize / 2;
			center.y += WebCenterViewConstants.NodeIconSize / 2;

			return center;
		};


		return WebCenterNodeView;

	});
