define([
		"WFEditorWCR/workflow/node/WebCenterNodeConfigurationPanel",
		"WFEditorWCR/workflow/node/operation/OperationNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"WFEditorWCR/workflow/panel/PinParameter",
		"tpl!WFEditorWCR/workflow/node/dataCollector/DataCollectorNode",
		"utils/XMLUtils",
		"utils/webi18n!WFEditorWCR",
		"WFEditorWCR/workflow/pin/OutputPinError",
		"WFEditorWCR/workflow/pin/InputPin"

	],
	function (
		WebCenterNodeConfigurationPanel,
		OperationNode,
		WebCenterNodeConstants,
		OutputPinOK,
		WebCenterPinConstants,
		WebCenterParameter,
		PinParameter,
		DataCollectorNodeXML,
		XMLUtils,
		i18n,
		OutputPinError,
		InputPin
	) {
		"use strict";

		/**
		 * DataCollectorNode represents the model for a  datacollector node in the WebCenter Workflow Editor.
		 *
		 * @exports DataCollectorNode
		 * @class
		 * @augments OperationNode
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {String} pathToIcons - gives the url to the map where the icons can be found
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var DataCollectorNode = function (workflow, id, pathToIcons, label, webCenterAPI) {
			this._icon = this._icon || WebCenterNodeConstants.Icons.DATACOLLECTOR;
			this._type = this._type || WebCenterNodeConstants.Nodes.DATACOLLECTOR;

			/**
			 * @member {String} error policy of the Data Collector Node
			 */
			this._errorPolicy = "";

			/**
			 * @member {RadioGroupPanel} - panel for error policy configuration
			 */
			this._radioGroupPanel = null;

			/**
			 * @member {Parameter} - parameters for the radio panel
			 */
			this._ErrorPolicyParam = null;
			this._FailParam = null;
			this._IgnoreParam = null;

			OperationNode.call(this, workflow, id, this._type, pathToIcons + this._icon, label, webCenterAPI);
		};

		DataCollectorNode.prototype = Object.create(OperationNode.prototype);
		DataCollectorNode.prototype.constructor = DataCollectorNode;

		/**
		 * @Override
		 */
		DataCollectorNode.prototype.setNodeXML = function (nodeXML) {
			// get error policy field
			this.readErrorPolicyFromXML(nodeXML);
			this._ErrorPolicyParam.setValue(this._errorPolicy);
		};

		/**
		 * @Override
		 */
		DataCollectorNode.prototype.setConfiguration = function (config) {
			if (config.length === 0) {
				// create fake parameters for error policy to be shown in a radio panel
				this._ErrorPolicyParam = new WebCenterParameter(WebCenterNodeConstants.DataCollector.ERRORPOLICY, WebCenterNodeConstants.DataCollector.FAILONERRORS);
				this._FailParam = new PinParameter(WebCenterNodeConstants.DataCollector.FAILONERRORS, "", WebCenterNodeConstants.DataCollector.FAILONERRORS, [], _.bind(function () {
					return [new OutputPinError(this._workflow, this, WebCenterPinConstants.Type.WorkflowConstructError)];
				}, this));

				this._IgnoreParam = new WebCenterParameter(WebCenterNodeConstants.DataCollector.IGNOREERRORS, "", WebCenterNodeConstants.DataCollector.IGNOREERRORS);
				config = [this._ErrorPolicyParam, this._FailParam, this._IgnoreParam];
			} else {
				_.each(config, _.bind(function (param) {
					if (param.getName() === WebCenterNodeConstants.DataCollector.ERRORPOLICY) {
						this._ErrorPolicyParam = param;
					} else if (param.getName() === WebCenterNodeConstants.DataCollector.FAILONERRORS) {
						this._FailParam = param;
					} else {
						this._IgnoreParam = param;
					}
				}, this));
			}
			this._updatePins();

			OperationNode.prototype.setConfiguration.call(this, config);
		};

		/**
		 * Get the error policy of the Join Node
		 * @returns {String}
		 * @override
		 */
		DataCollectorNode.prototype.getErrorPolicy = function () {
			this.updateErrorPolicy();
			return this._errorPolicy;
		};

		/**
		 * read the error policy from the given XML
		 * @param nodeXML
		 */
		DataCollectorNode.prototype.readErrorPolicyFromXML = function (nodeXML) {
			var nodeData = XMLUtils.xml2obj(nodeXML);
			this._errorPolicy = nodeData.action.errorpolicy;
			this._ErrorPolicyParam.setValue(this._errorPolicy);
			this._updatePins();
		};


		/**
		 * Update error policy value, based on the parameters that are passed to the radio panel
		 * @param nodeXML
		 */
		DataCollectorNode.prototype.updateErrorPolicy = function () {
			if (this._radioGroupPanel) {
				var parameters = this._radioGroupPanel.getParameters();
				this._errorPolicy = parameters[0].getValue();
			}
		};

		/**
		 * @Override
		 */
		DataCollectorNode.prototype._initializePins = function () {
			this._inputPin = new InputPin(this._workflow, this);
			this.addPin(this._inputPin, 0);
			this.addPin(new OutputPinOK(this._workflow, this, WebCenterPinConstants.Type.WorkflowConstructOK), 1);
		};

		/**
		 * @Override
		 */
		DataCollectorNode.prototype.getActionXML = function () {
			return DataCollectorNodeXML(this);
		};

		/**
		 * @override
		 */
		DataCollectorNode.prototype.getConfigurationPanel = function () {
			var panelPromise = $.Deferred();
			var nodeConfigPanel = new WebCenterNodeConfigurationPanel(this);
			nodeConfigPanel.getContext().addClass("parameter-panel form-horizontal form-compact");

			requirejs(["WFEditorWCR/workflow/panel/RadioPanel", "WFEditorWCR/workflow/panel/RadioGroupPanel"], _.bind(function (RadioPanel, RadioGroupPanel) {
				this._radioGroupPanel = new RadioGroupPanel(this._ErrorPolicyParam.getName(), undefined, WebCenterNodeConstants.DataCollector.ERRORPOLICY);
				nodeConfigPanel.add(this._radioGroupPanel);

				var ignorePanel = new RadioPanel(this._IgnoreParam.getName(), i18n.t(WebCenterNodeConstants.DataCollector.IGNOREERRORS));
				var failPanel = new RadioPanel(this._FailParam.getName(), i18n.t(WebCenterNodeConstants.DataCollector.FAILONERRORS));
				var self = this;

				// If the failPanel gets checked we wish to create a new error pin (there might be edges in the other one referring to nodes that have been deleted )
				failPanel.check = function (check, visibleWhenNotChecked) {
					RadioPanel.prototype.check.call(this, check, visibleWhenNotChecked);
					if (this._parameters[0]) {
						this._parameters[0].setNewPin();
					}
				};
				this._radioGroupPanel.add(failPanel);
				this._radioGroupPanel.add(ignorePanel);

				panelPromise.resolve(nodeConfigPanel);
			}, this));

			return panelPromise.promise();
		};

		/**
		 * updates the pins with respect to the error policy
		 */
		DataCollectorNode.prototype._updatePins = function () {
			if (this._ErrorPolicyParam.getValue() === WebCenterNodeConstants.DataCollector.FAILONERRORS) {
				if (!this._errorPin) {
					this._errorPin = this._FailParam.pins[0];
					this.addPin(this._errorPin);
					_.each(this._errorPin.getEdges(), _.bind(function (edge) {
						edge.getToPin().addEdge(edge);
						this._workflow.addEdge(edge);
					}, this));
				}
			} else {
				if (this._errorPin) {
					this.removePin(this._errorPin);
					_.each(this._errorPin.getEdges(), _.bind(function (edge) {
						edge.getToPin().removeEdge(edge);
						this._workflow.removeEdge(edge);
					}, this));
					this._errorPin = null;
				}
			}
		};

		/**
		 * @override
		 */
		DataCollectorNode.prototype.getHelpTopic = function () {
			return WebCenterNodeConstants.HelpTopics.DATACOLLECTOR;
		};
		return DataCollectorNode;
	});
