define([
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"tpl!WFEditorWCR/workflow/node/endState/EndStateNode",
		"utils/webi18n!WFEditorWCR"
	],
	function (
		WebCenterNode,
		WebCenterNodeConstants,
		EndStateNodeXML,
		i18n
	) {
		"use strict";

		/**
		 * EndStateNode represents the model for an end node in the WebCenter Workflow Editor.
		 *
		 * @exports EndStateNode
		 * @class
		 * @augments WebCenterNode
		 *
		 * @param {Wofkflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node
		 * @param {String} pathToIcons - gives the url to the map where the icons can be found
		 * @param {String} [label] - the label of the node
		 * @param {String} nodeType - specifies the type of the node
		 */
		var EndStateNode = function (workflow, id, pathToIcons, label, nodeType, webCenterAPI) {
			var icon = null;
			if (!label) {
				label = i18n.t("Completed");
			}
			if (nodeType === WebCenterNodeConstants.Nodes.COMPLETED) {
				icon = WebCenterNodeConstants.Icons.COMPLETED;
			} else if (nodeType === WebCenterNodeConstants.Nodes.CANCEL) {
				icon = WebCenterNodeConstants.Icons.CANCEL;
			} else {
				icon = WebCenterNodeConstants.Icons.ERROR;
			}

			WebCenterNode.call(this, workflow, id, nodeType, pathToIcons + icon, label, webCenterAPI);

			/**
			 * @member {Boolean} for implicit/invisible endstate node this is true
			 * @private
			 */
			this._isImplicit = false;
			
			/**
			 * @member {String} the id of the end state. Not to be confused with the id of the node
			 * @private
			 */
			this._endStateId = id.substr(0, id.lastIndexOf("_"));
		};

		EndStateNode.prototype = Object.create(WebCenterNode.prototype);
		EndStateNode.prototype.constructor = EndStateNode;

		/**
		 * @Override
		 */
		EndStateNode.prototype.setNodeXML = function (nodeEl) {
			WebCenterNode.prototype.setNodeXML.apply(this, arguments);

			this._isImplicit = nodeEl.getAttribute("implicit");

		};

		/** 
		 * Is the node implicit?
		 * @Override
		 */
		EndStateNode.prototype.isImplicit = function () {
			return this._isImplicit;
		};

		/**
		 * sets whether or not the node is implicit
		 * @param {Boolean} isImplicit
		 */
		EndStateNode.prototype.setImplicit = function (isImplicit) {
			this._isImplicit = isImplicit;
		};

		/**
		 * @Override
		 */
		EndStateNode.prototype._initializePins = function () {
			this._addInputPin();
		};

		/**
		 * @Override
		 */
		EndStateNode.prototype.getLayoutXML = function () {
			if (this.isImplicit()) {
				return "";
			} else {
				return WebCenterNode.prototype.getLayoutXML.call(this);
			}
		};
		/**
		 * Make sure that an implicit end state node is not visible
		 * @Override
		 */
		EndStateNode.prototype.isVisible = function () {
			return WebCenterNode.prototype.isVisible.call(this) && !this.isImplicit();
		};

		/**
		 * Generate Node XML
		 * @Override
		 */
		EndStateNode.prototype.toXML = function () {
			return EndStateNodeXML(this);
		};

		/**
		 * @override
		 */
		EndStateNode.prototype.getHelpTopic = function () {
			if (this.getNodeType() === WebCenterNodeConstants.Nodes.CANCEL) {
				return WebCenterNodeConstants.HelpTopics.CANCEL;
			}
			return "";
		};
	
		/**
		 * @override
		 * Adds the endStateId to the state object.
		 */
		EndStateNode.prototype.getStateObjectForCopy = function () {
			var stateObject = WebCenterNode.prototype.getStateObjectForCopy.call(this);
			/* In this context the id is of the type of state, not the id of the instance of the end state,
			   as the id of the instance shouldn't be copied along */
			stateObject.id = this._endStateId;
			return stateObject;
		}
	
		return EndStateNode;
	});