define([
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/node/router/RouterParameter",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"tpl!WFEditorWCR/workflow/node/router/RouterNodeResource",
		"tpl!WFEditorWCR/workflow/node/router/RouterNodeAction",
		"utils/XMLUtils",
		"underscore"
	],
	function (
		WebCenterNode,
		WebCenterNodeConstants,
		OutputPinOK,
		WebCenterPinConstants,
		RouterParameter,
		WebCenterParameter,
		RouterNodeResourceXML,
		RouterNodeActionXML,
		XMLUtils,
		_
	) {
		"use strict";

		/**
		 * RouterNode represents the model for a  router node in the WebCenter Workflow Editor.
		 *
		 * @exports RouterNode
		 * @class
		 * @augments WebCenterNode
		 *
		 * @param {Workflow}     workflow     - the workflow to which the node belongs
		 * @param {String}       id           - node id
		 * @param {String}       icon		  - icon url
		 * @param {String}       [label]      - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var RouterNode = function (workflow, id, icon, label, webCenterAPI) {
			WebCenterNode.call(this, workflow, id, WebCenterNodeConstants.Nodes.ROUTER, icon, label, webCenterAPI);

			// Set the default routernode configuration
			var routerParameter = new RouterParameter(null, "");
			routerParameter.setRouterType(WebCenterNodeConstants.RouterNode.Routers.ProjectField);
			this._configuration = [routerParameter];
			routerParameter.getRouterOutputs().push(this.getErrorPin());
		};

		RouterNode.prototype = Object.create(WebCenterNode.prototype);
		RouterNode.prototype.constructor = RouterNode;

		/**
		 * @Override
		 */
		RouterNode.prototype._initializePins = function () {
			this._addInputPin();
			this._addErrorPin(WebCenterPinConstants.Type.WorkflowConstructError);
		};

		/**
		 * Read the Resource XML
		 * @Override
		 */
		RouterNode.prototype.setResourceXML = function (resourceXML) {
			var resourceObj = XMLUtils.xml2obj(resourceXML);

			// save the resourceXML for generating correct XML when asked... 
			this._resourceXML = new XMLSerializer().serializeToString(resourceXML);



			if (resourceObj && resourceObj.filter) {
				var routerType = resourceObj.filter.attributes.class; // the type of the router
				var routerEntries = resourceObj.filter.numericrule || resourceObj.filter.stringrule || resourceObj.filter.enumrule;
				if (routerEntries && !_.isArray(routerEntries)) {
					routerEntries = [routerEntries];
				}
				this._configuration = [new RouterParameter(null, "")];
				this._configuration[0].setRouterType(routerType);
				this._configuration[0].setRouterRuleType(resourceObj.filter.numericrule ? WebCenterNodeConstants.RouterNode.RuleTypes.NumericRule :
					resourceObj.filter.enumrule ? WebCenterNodeConstants.RouterNode.RuleTypes.EnumRule : WebCenterNodeConstants.RouterNode.RuleTypes.StringRule);

				if (resourceObj.filter.outputs) {
					// At this point only the error node is added to the pins, we wish to have the error node at the back of all pins
					// Instead of inserting all the new pins before the errorPin, we first remove the error pin and then add it back.

					var errorPin = this.getErrorPin();
					this.removePin(errorPin);
					_.each(resourceObj.filter.outputs.output, function (output) {
						if (output && output.type == "OK") {
							this.addPin(new OutputPinOK(this._workflow, this, output.id, output.name));
						}

					}, this);
					this.addPin(errorPin);
					this._configuration[0].setRouterOutputs(_.filter(this.getPins(), _.bind(function (pin) {
						return pin !== this.getInputPin();
					}, this)));
				}

				var isCaseSensitive = false;
				_.each(routerEntries, function (entry) {
					var values = [];
					var i = 1;

					if (entry.casesensitive === "TRUE") {
						isCaseSensitive = true;
					}

					if (typeof entry.value !== "undefined") {
						values.push(entry.value);
					}

					if (entry.value_0) {
						values.push(entry.value_0);
					}
					while (entry["value_" + i]) {
						values.push(entry["value_" + i]);
						i++;
					}
					this._configuration[0].addRouterEntry(entry.op, values, entry.output);
				}, this);
				this._configuration[0].isCaseSensitive(isCaseSensitive);
				this._configuration[0].shouldIncludeExtension(resourceObj.filter.includeextension === "TRUE");
				if (resourceObj.filter.type) {
					this._configuration[0].setName(resourceObj.filter.type);
				}
				if (resourceObj.filter.parameter) {
					this._configuration[0].setValue(resourceObj.filter.parameter);
				}
				if (resourceObj.filter.subparameters) {
					var subParams = [];
					_.each(resourceObj.filter.subparameters, function (parameter) {
						if (parameter.Name) {
							subParams.push(new WebCenterParameter(parameter.Name, "", parameter.Value));
						}
					}, this);
					this._configuration[0].setSubParameters(subParams);
				}
				if (resourceObj.filter.unmatched) {
					if (resourceObj.filter.unmatched.output === "Error") {
						this._configuration[0].setDefaultOutputId(this.getErrorPin().getOutputId());
					} else {
						this._configuration[0].setDefaultOutputId(resourceObj.filter.unmatched.output);
					}
				}
				if (resourceObj.filter.rss) {
					var rss = [];
					_.each(resourceObj.filter.rss.rs, function (rs) {
						rss.push(rs);
					});
					this._configuration[0].setRestrictedSet(rss);
				}
			}
		};

		/**
		 * @override
		 */
		RouterNode.prototype.getResourceXML = function () {
			return RouterNodeResourceXML({
				node: this,
				param: this._configuration[0],
				OutError: WebCenterPinConstants.Type.OutError
			});
		};

		/**
		 * @override
		 */
		RouterNode.prototype.getActionXML = function () {
			return RouterNodeActionXML(this);
		};

		/**
		 * @override
		 */
		RouterNode.prototype.getStateObjectForCopy = function () {
			var stateObj = WebCenterNode.prototype.getStateObjectForCopy.call(this);
			stateObj.id = "genPropertyFilter";
			return stateObj;
		};

		/**
		 * @override
		 * @param   {Array.<RouterParameter>} configuration 
		 */
		RouterNode.prototype.setConfiguration = function (configuration) {
			if (configuration[0] instanceof RouterParameter) {
				WebCenterNode.prototype.setConfiguration.call(this, configuration);
				_.each(this.getPins(), function (pin) {
					// first remove all pins in the router
					if (pin !== this.getInputPin()) {
						this.removePin(pin);
						_.each(pin.getEdges(), function (edge) {
							edge.getToPin().removeEdge(edge);
							this.getWorkflow().removeEdge(edge);
						}, this);
					}
				}, this);
				_.each(configuration[0].getRouterOutputs(), function (output) {
					// then add all pins in the configuration and their edges
					this.addPin(output);
					_.each(output.getEdges(), function (edge) {
						edge.getToPin().addEdge(edge);
						this.getWorkflow().addEdge(edge);
					}, this);
				}, this);

			}
		};

		/**
		 * @override
		 * @returns {WebCenterNodeConfigurationPanel} 
		 */
		RouterNode.prototype.getConfigurationPanel = function () {
			var deferred = $.Deferred();
			requirejs([
				"WFEditorWCR/workflow/node/router/routerPanel/RouterPanel",
				"WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService",
				"WFEditorWCR/workflow/panel/DateSmartNameDictionaryService"
			], _.bind(function (
				RouterPanel,
				WorkflowSmartNameDictionaryService,
				DateSmartNameDictionaryService
			) {
				if (!this._smartNameDictionary) {
					this._smartNameDictionary = new WorkflowSmartNameDictionaryService(this._webCenterAPI, this.getWorkflow().getTaskTypeID());
					this._dateSmartNameDictionary = new DateSmartNameDictionaryService(this._webCenterAPI, this.getWorkflow().getTaskTypeID());
				}
				deferred.resolve(new RouterPanel(this, this._webCenterAPI, this._smartNameDictionary, this._dateSmartNameDictionary));
			}, this));
			return deferred.promise();

		};



		return RouterNode;
	});
