define(["underscore",
		"jquery",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants"
	],
	function (
		_,
		$,
		WebCenterParameter,
		WebCenterPinConstants,
		WebCenterNodeConstants) {

		"use strict";

		/**
		 * Parameter used for router nodes
		 * @class
		 * @exports RouterParameter
		 * @augments WebCenterParameter
		 * @param {String} valueType  - When routing on a specific attribute this should be the type of the attribute - fe. TEXT or DATETIME
		 * @param {String} value      - When routing on an attribute this is the id of the attribute
		 * @param {String} routerType - the type of the router used - see WebCenterNodeConstants.RouterNode.Routers for the different types available
		 */
		var RouterParameter = function (valueType, value, routerType) {
			WebCenterParameter.call(this, valueType, null, value, []);

			/**
			 * @member {String}
			 * @private 
			 */
			this._routerType = routerType || WebCenterNodeConstants.RouterNode.Routers.ProjectField;

			/**
			 * @member {Array.<Object>} the different router entries, has a ruleType, ruleValues and an outputId
			 * @private 
			 */
			this._routerEntries = [];

			/**
			 * @member {Array.<WebCenterOutputPin>} list of outputs
			 * @private
			 */
			this._routerOutputs = [];

			/**
			 * @member {String} the id of the output used for the default route
			 * @private
			 */
			this._defaultOutputId = WebCenterPinConstants.Type.WorkflowConstructError;

			/**
			 * @member {Boolean} 
			 * @private 
			 */
			this._isCaseSensitive = false;

			/**
			 * @member {String}
			 * @private 
			 */
			this._shouldIncludeExtension = false;

			/**
			 * @member {WebCenterNodeConstants.RouterNode.RuleTypes} used for xml generation of the rules
			 * @private
			 */
			this._rulesType = WebCenterNodeConstants.RouterNode.RuleTypes.StringRule;

			/**
			 * @member {Array.<String>} the values of the restricteds et
			 * @private 
			 */
			this._rss = [];

		};

		RouterParameter.prototype = Object.create(WebCenterParameter.prototype);
		RouterParameter.prototype.constructor = RouterParameter;

		/**
		 * Reset the parameter
		 */
		RouterParameter.prototype.reset = function () {
			this._routerEntries = [];
			this._routerOutputs = [];
			this._routerType = WebCenterNodeConstants.RouterNode.Routers.ProjectField;
			this._name = "";
			this._value = "";
			this._defaultOutputId = WebCenterPinConstants.Type.WorkflowConstructError;
			this._isCaseSensitive = false;
			this._shouldIncludeExtension = false;
			this._rulesType = WebCenterNodeConstants.RouterNode.RuleTypes.StringRule;
			this._rss = [];
			this._subparameters = [];
		};

		/**
		 * Add a router entry
		 * @param {String}         ruleType   
		 * @param {Array.<String>} ruleValues 
		 * @param {String}         outputId   
		 */
		RouterParameter.prototype.addRouterEntry = function (ruleType, ruleValues, outputId) {
			this._routerEntries.push({
				"ruleType": ruleType,
				"ruleValues": ruleValues,
				"outputId": outputId
			});
		};

		/**
		 * Sets the outputs 
		 * @param {Array.<WebCenterOutputPin>} outputs
		 */
		RouterParameter.prototype.setRouterOutputs = function (outputs) {
			this._routerOutputs = outputs;
		};

		/**
		 * used for xml generation of the router parameter so that the backend know which type it is
		 * 
		 * @param {WebCenterNodeConstants.RouterNodes.RuleTypes} ruleType - can be any of "numericrule", "stringrule", "enumrule"
		 */
		RouterParameter.prototype.setRouterRuleType = function (rulesType) {
			if(rulesType) {
				this._rulesType = rulesType;
			}
		};

		/**
		 * gets the router rule type - used for xml generation
		 * @returns {WebCenterNodeConstants.RouterNode.RuleTypes} 
		 */
		RouterParameter.prototype.getRouterRuleType = function () {
			return this._rulesType;
		};

		/**
		 * gets the restricted set options for the parameter
		 * @returns {Array.<String>} 
		 */
		RouterParameter.prototype.getRestrictedSet = function () {
			return this._rss;
		};

		/**
		 * sets the restricted set options for the parameter
		 * @param {Array.<String>} rss 
		 */
		RouterParameter.prototype.setRestrictedSet = function (rss) {
			this._rss = rss;
		};

		/**
		 * @param {String}
		 */
		RouterParameter.prototype.setDefaultOutputId = function (id) {
			this._defaultOutputId = id;
		};

		/**
		 * @returns {String}
		 */
		RouterParameter.prototype.getDefaultOutputId = function () {
			return this._defaultOutputId;
		};

		/**
		 * @returns {Array.<WebCenterOutputPin>}
		 */
		RouterParameter.prototype.getRouterOutputs = function () {
			return this._routerOutputs;
		};

		/**
		 * @returns {Array.<Object>}
		 */
		RouterParameter.prototype.getRouterEntries = function () {
			return this._routerEntries;
		};

		/**
		 * returns a copy of this routerparameter
		 * @returns {RouterParameter} 
		 */
		RouterParameter.prototype.copy = function () {
			var routerParameter = new RouterParameter(this._name, this._value, this._routerType);

			_.each(this.getRouterEntries(), function (entry) {
				var valueCopy = [];
				_.each(entry.ruleValues, function (value) {
					valueCopy.push(value);
				});
				routerParameter.addRouterEntry(entry.ruleType, valueCopy, entry.outputId);
			}, this);

			var routerOutputsCopy = [];

			_.each(this._routerOutputs, function (output) {
				routerOutputsCopy.push(output);
			});

			routerParameter.isCaseSensitive(this._isCaseSensitive);
			routerParameter.shouldIncludeExtension(this._shouldIncludeExtension);
			routerParameter.setRouterOutputs(routerOutputsCopy);
			routerParameter.setDefaultOutputId(this._defaultOutputId);
			routerParameter.setRouterRuleType(this._rulesType);
			var rss = [];
			_.each(this._rss, function (rs) {
				rss.push(rs);
			});
			routerParameter.setRestrictedSet(rss);

			var subParams = [];
			_.each(this.getSubparameters(), function (subParam) {
				subParams.push(subParam.copy());
			});
			routerParameter.setSubParameters(subParams);

			return routerParameter;
		};

		/**
		 * @param {String}
		 */
		RouterParameter.prototype.setRouterType = function (routerType) {
			this._routerType = routerType;
		};

		/**
		 * sets the case sensitivity of the parameter, is no argument is defined, returns the case sensitivity.
		 * @param {Boolean} [isCaseSensitive] - if defined, sets the caseSensitivity to 
		 * @returns {Boolean | undefined}                                   
		 */
		RouterParameter.prototype.isCaseSensitive = function (isCaseSensitive) {
			if (typeof isCaseSensitive === "undefined") {
				return this._isCaseSensitive;
			} else {
				this._isCaseSensitive = isCaseSensitive;
			}
		};

		/**
		 * sets the whether the file extension should be included. If no argument is defined, returns whether the file extension should be included.
		 * @param {Boolean} [shouldIncludeExtension]
		 * @returns {Boolean | undefined}                                   
		 */
		RouterParameter.prototype.shouldIncludeExtension = function (shouldIncludeExtension) {
			if (typeof shouldIncludeExtension === "undefined") {
				return this._shouldIncludeExtension;
			} else {
				this._shouldIncludeExtension = shouldIncludeExtension;
			}
		};

		/**
		 * @returns {String}
		 */
		RouterParameter.prototype.getRouterType = function () {
			return this._routerType;
		};

		/**
		 * Is this routerParameter equal to the given parameter
		 * @param   {Parameter} param
		 * @returns {Boolean}   	
		 */
		RouterParameter.prototype.equals = function (param) {

			if (!WebCenterParameter.prototype.equals.call(this, param)) {
				return false;
			}

			// If param isn't a routerparameter or if it doens't have the same type, entries length or outputs length they're mos def not the same
			if (!(param instanceof RouterParameter) ||
				this.getRouterType() !== param.getRouterType() ||
				this.getRouterEntries().length !== param.getRouterEntries().length ||
				this.getRouterOutputs().length !== param.getRouterOutputs().length ||
				this._isCaseSensitive !== param.isCaseSensitive() ||
				this._shouldIncludeExtension !== param.shouldIncludeExtension() ||
				this._rss.length !== param.getRestrictedSet().length ||
				this._rulesType !== param.getRouterRuleType() ||
				this._defaultOutputId !== param.getDefaultOutputId()) {
				return false;
			}

			var i;
			var otherRSS = param.getRestrictedSet();
			for (i = 0; i < this._rss.length; i++) {
				if (this._rss[i] !== otherRSS[i]) {
					return false;
				}
			}

			// check if the router entries are different
			for (i = 0; i < this._routerEntries.length; i++) {
				var entry1 = this._routerEntries[i];
				var entry2 = param.getRouterEntries()[i];
				if (entry1.ruleType !== entry2.ruleType ||
					entry1.outputId !== entry2.outputId) {
					return false;
				}
				for (var j = 0; j < entry1.ruleValues.length; j++) {
					if (entry1.ruleValues[j] !== entry2.ruleValues[j]) {
						return false;
					}
				}
			}

			// check if the router outputs are different
			for (i = 0; i < this._routerOutputs.length; i++) {
				var output1 = this._routerOutputs[i];
				var output2 = param.getRouterOutputs()[i];
				if (output1 !== output2) {
					return false;
				}
			}

			// They are the same
			return true;

		};


		return RouterParameter;

	});