define([
		"./CaseSensitiveDecorator",
		"../routerPanel/RouterRule",
		"utils/webi18n!WFEditorWCR",
		"component/sdrop/dataproviders/MapPromiseDataProvider",
		"component/sdrop/SDrop",
		"../routerRules/SmartNameRouterRule",
		"../routerRules/BetweenSmartNameRouterRule",
		"../routerRules/SelectSmartNameRouterRule",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"underscore",
		"jquery"],
	function (
		CaseSensitiveDecorator,
		RouterRule,
		i18n,
		MapPromiseDataProvider,
		SDrop,
		SmartNameRouterRule,
		BetweenSmartNameRouterRule,
		SelectSmartNameRouterRule,
		WebCenterNodeConstants,
		_,
		$
	) {

		"use strict";

		/**
		 * An AttributeRouter is a router decorator that adds controls to route on a specific attribute
		 * 
		 * @class
		 * @exports AttributeRouter
		 * @augments CaseSensitiveRouter
		 * 
		 * @param {String}                             id                      - Id of the router decorator
		 * @param {String}                             name                    - name for displaying purposes
		 * @param {WorkflowSmartNameDictionaryService} smartNameDictionary     - the smartname dictionary that can be used for smartnamerouterrule
		 * @param {DateSmartNameDictionaryService}     dateSmartNameDictionary - the smartname dicitionary that can be used for date smartnamerouterrules
		 * @param {String}	                           label                   - label used for the attribute dropdown selector
		 * @param {$.Deferred}                         promise                 - promise used to fetch the attributes that should be displayed
		 */
		var AttributeRouter = function (id, name, smartNameDictionary, dateSmartNameDictionary, label, promise) {
			CaseSensitiveDecorator.call(this, id, name, smartNameDictionary);

			/**
			 * @member {DateSmartNameDictionaryService} smartname dictionary service that can be used for date smartname router rules
			 * @private
			 */
			this._dateSmartNameDictionaryService = dateSmartNameDictionary;

			/**
			 * @member {JQueryObject} select used for the sDrop
			 * @private
			 */
			this._nameField = $('<select />');

			/**
			 * @member {Object} options for the sDrop
			 * @private
			 */
			this._options = {
				autoload: false,
				autoloadOnFirstUsage: true,
				manageStyleAndWidth: false,
				onChangedByUser: _.bind(this._attributeSelected, this)
			};

			/**
			 * @member {$.Deferred} promise of attributs for the sDrop
			 * @private
			 */
			this._attributesPromise = promise;

			/**
			 * @member {MapPromiseDataProvider} DataProvider for the sDrop
			 * @private
			 */
			this._dataProvider = new MapPromiseDataProvider({
				'promise': promise,
				'customFilter': function (attr) {
					return !attr.isDocRef();
				}
			});


			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._select = $("<select/>").attr("data-allowcustom", "1").attr("data-delimiter", " | ");

			// Router Rules for a Numeric attribute
			this._numberRules = [
				new SmartNameRouterRule("RN_EQUAL", i18n.t("wfui.route.ruleop.equal"), "=", this._smartNameDictionaryService, null, "="),
				new SmartNameRouterRule("RN_LESS_OR_EQUAL", i18n.t("wfui.route.ruleop.lessequal"), "<=", this._smartNameDictionaryService, null, "<="),
				new SmartNameRouterRule("RN_LESS", i18n.t("wfui.route.ruleop.less"), "<", this._smartNameDictionaryService, null, "<"),
				new SmartNameRouterRule("RN_GREATER_OR_EQUAL", i18n.t("wfui.route.ruleop.greaterequal"), ">=", this._smartNameDictionaryService, null, ">="),
				new SmartNameRouterRule("RN_GREATER", i18n.t("wfui.route.ruleop.greater"), ">", this._smartNameDictionaryService, null, ">"),
				new BetweenSmartNameRouterRule("RN_BETWEEN", i18n.t("wfui.route.ruleop.between"), "><", this._smartNameDictionaryService),
				new SmartNameRouterRule("RN_NOT_EQUAL", i18n.t("wfui.route.ruleop.noequal"), "<>", this._smartNameDictionaryService, null, "<>"),

			];

			// Router Rules for a text attribute
			this._stringRules = [
				new RouterRule("RS_IS_EMPTY", i18n.t("wfui.route.ruleop.empty"), i18n.t("wfui.route.ruleop.empty")),
				new SmartNameRouterRule("RS_IS_EQUAL", i18n.t("wfui.route.ruleop.equal"), i18n.t("wfui.route.ruleop.match"), this._smartNameDictionaryService),
				new SmartNameRouterRule("RS_IS_NOT_EQUAL", i18n.t("wfui.route.ruleop.noequal"), i18n.t("wfui.route.ruleop.nomatch"), this._smartNameDictionaryService, null, i18n.t("wfui.route.ruleop.prefix.not")),
				new SmartNameRouterRule("RS_CONTAINS", i18n.t("wfui.route.ruleop.contain"), i18n.t("wfui.route.ruleop.match"), this._smartNameDictionaryService),
				new SmartNameRouterRule("RS_DOES_NOT_CONTAIN", i18n.t("wfui.route.ruleop.nocontain"), i18n.t("wfui.route.ruleop.nomatch"), this._smartNameDictionaryService, null, i18n.t("wfui.route.ruleop.prefix.not")),
				new SmartNameRouterRule("RS_MATCHES", i18n.t("wfui.route.ruleop.match"), i18n.t("wfui.route.ruleop.match"), this._smartNameDictionaryService),
				new SmartNameRouterRule("RS_DOES_NOT_MATCH", i18n.t("wfui.route.ruleop.nomatch"), i18n.t("wfui.route.ruleop.nomatch"), this._smartNameDictionaryService, null, i18n.t("wfui.route.ruleop.prefix.not")),
				new SmartNameRouterRule("RS_BEGINS_WITH", i18n.t("wfui.route.ruleop.begins"), i18n.t("wfui.route.ruleop.match"), this._smartNameDictionaryService),
				new SmartNameRouterRule("RS_ENDS_WITH", i18n.t("wfui.route.ruleop.ends"), i18n.t("wfui.route.ruleop.match"), this._smartNameDictionaryService)
			];

			// Router rules for a date attribute
			this._dateRules = [
				new SmartNameRouterRule("RD_EARLIER", i18n.t("wfui.route.ruleop.earlierthan"), i18n.t("<"), this._dateSmartNameDictionaryService),
				new SmartNameRouterRule("RD_LATER", i18n.t("wfui.route.ruleop.laterthan"), i18n.t(">="), this._dateSmartNameDictionaryService)
			];

			this._rssRules = [
				new SelectSmartNameRouterRule("RE_IS", i18n.t("wfui.route.ruleop.is"), i18n.t("wfui.route.ruleop.is"), this._smartNameDictionaryService, this._select),
				new SelectSmartNameRouterRule("RE_IS_NOT", i18n.t("wfui.route.ruleop.isnot"), i18n.t("wfui.route.ruleop.isnot"), this._smartNameDictionaryService, this._select, null, i18n.t("wfui.route.ruleop.prefix.not"))
			];

			/**
			 * @member {Object.<Array.<RouterRule>>}
			 * @private
			 */
			this._rules = {
				"TEXT": this._stringRules,
				"DATETIME": this._dateRules,
				"UNITS": this._numberRules,
				"NUMBER": this._numberRules,
				"FLOAT": this._numberRules,
				"DOCREF": this._stringRules,
				"RICHTEXT": this._stringRules,
			};

			/**
			 * @member {String} - the type of the currently used attribute - used to initialize the correct router rules on init
			 * @private
			 */
			this._currentAttributeType = null;

			/**
			 * @member {String} the currently active attribute - used to initialize the sDrop
			 * @private
			 */
			this._currentAttribute = null;

			/**
			 * @member {String} is the current attribute a restricted set?
			 * @private
			 */
			this._currentAttributeIsRSS = false;


			this._element.prepend($("<div/>").addClass("form-group")
				.append($("<label/>").text(label))
				.append($("<div/>")
					.append(this._nameField)));
			this._sDrop = null;

		};

		AttributeRouter.prototype = Object.create(CaseSensitiveDecorator.prototype);
		AttributeRouter.prototype.constructor = AttributeRouter;

		/**
		 * When an attribute has been selected from the sDrop set the correct rules
		 */
		AttributeRouter.prototype._attributeSelected = function () {
			var val = this._nameField.val();
			var selected = this._sDrop.dataProvider.getById(val);
			if (selected && selected.item) {
				var attr = selected.item;
				this._currentAttribute = attr.getId();
				var rss = attr.getRestrictedSet();
				// Change the router rules only if the attribute type is different, the new attribute is a rs or the old one was a rs and the new one isn't
				if (this._currentAttributeType !== attr.getType().getType() || rss || (!rss && this._currentAttributeIsRSS)) {
					this._currentAttributeType = attr.getType().getType();
					if (rss) {
						var rssValues = rss.getValues();
						this._select.empty();
						_.each(rssValues, function (value) {
							this._select.append($("<option/>").val(value.getValue()).text(value.getValue()));
						}, this);
						this._currentAttributeIsRSS = true;
						this._setRules(this._rssRules);
					} else {
						this._currentAttributeIsRSS = false;
						this._setRules(this._rules[this._currentAttributeType]);

					}

					// Change the visibility of the case sensitivity checkbox to only show it if string
					if (this._rules[this._currentAttributeType] === this._stringRules && !this._currentAttributeIsRSS) {
						this._caseSensitiveCheckboxDiv.show();
					} else {
						this._caseSensitiveCheckboxDiv.hide();
					}
				}
			}
		};

		/**
		 * @override
		 */
		AttributeRouter.prototype.handleParameter = function (routerParameter) {
			CaseSensitiveDecorator.prototype.handleParameter.call(this, routerParameter);
			if (routerParameter.getRouterType() === this.getId()) {
				this._currentAttribute = routerParameter.getValue();
				this._currentAttributeType = routerParameter.getName();
				if (routerParameter.getRestrictedSet().length > 0) {
					this._currentAttributeIsRSS = true;
					this._select.empty();
					_.each(routerParameter.getRestrictedSet(), function (value) {
						this._select.append($("<option/>").val(value).text(value));
					}, this);
				}
			}
		};

		/**
		 * @override
		 * @param {RouterParameter}
		 */
		AttributeRouter.prototype.fillInParameter = function (routerParameter) {
			CaseSensitiveDecorator.prototype.fillInParameter.call(this, routerParameter);
			routerParameter.setValue(this._currentAttribute);
			routerParameter.setName(this._currentAttributeType);
			routerParameter.setRouterRuleType(
				this._currentAttributeIsRSS ? WebCenterNodeConstants.RouterNode.RuleTypes.EnumRule :
				(this._rules[this._currentAttributeType] === this._stringRules) ? WebCenterNodeConstants.RouterNode.RuleTypes.StringRule :
				(this._rules[this._currentAttributeType] === this._numberRules || this._rules[this._currentAttributeType] === this._dateRules) ? WebCenterNodeConstants.RouterNode.RuleTypes.NumericRule :
				WebCenterNodeConstants.RouterNode.RuleTypes.StringRule);
			if (this._currentAttributeIsRSS) {
				routerParameter.setRestrictedSet(this._rssRules[0].getOptions());
			}

		};

		/**
		 * @override
		 */
		AttributeRouter.prototype.init = function () {
			if (this._currentAttributeIsRSS) {
				this._setRules(this._rssRules);
			} else {
				this._setRules(this._rules[this._currentAttributeType]);
			}
			this._attributesPromise.done(_.bind(function (attributes) {
				if (this._currentAttribute && attributes.has(this._currentAttribute)) {
					// only set the initial value if it is still in the attributes map, it could have been deleted 
					this._options.initialValue = this._currentAttribute;
				} else if (this._currentAttribute) {
					// The currentAttribute doesn't exist in the attributes map -> it has been deleted, don't save it
					this._currentAttribute = null;
				}
				this._sDrop = new SDrop(this._nameField, "", this._dataProvider, this._options);
				if (this._disabled) {
					this._sDrop.setDisabled(true);
				}
				if (this._currentAttribute) {
					this._attributeSelected();
				}
			}, this));

		};

		/**
		 * @override
		 */
		AttributeRouter.prototype.destroy = function () {
			if (this._sDrop) {
				this._sDrop.destroy();
			}
		};

		/**
		 * sets the router rules
		 * @param {Array.<RouterRule>} rules 
		 */
		AttributeRouter.prototype._setRules = function (rules) {
			if (rules) {
				this._routerView.getRouter().setRouterRules(rules);
			} else {
				// If undefined rules are being set, just add a single empty rule
				this._routerView.getRouter().setRouterRules([new RouterRule("RS_IS_EMPTY", i18n.t("wfui.route.ruleop.empty"), i18n.t("wfui.route.ruleop.empty"))]);
			}
		};

		/**
		 * @override
		 */
		AttributeRouter.prototype.disable = function () {
			CaseSensitiveDecorator.prototype.disable.call(this);
			if (this._sDrop) {
				this._sDrop.setDisabled(true);
			}
		};


		return AttributeRouter;

	});
