define([
	"jquery",
	"underscore",
	"./RouterDecorator",
	"../routerRules/SelectSmartNameRouterRule",
	"WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService",
	"WFEditorWCR/workflow/node/WebCenterNodeConstants",
	"utils/webi18n!WFEditorWCR"
], function (
	$,
	_,
	RouterDecorator,
	SelectSmartNameRouterRule,
	WorkflowSmartNameDictionaryService,
	WebCenterNodeConstants,
	i18n) {

	"use strict";

	/**
	 * A router decorator to route on project status
	 * 
	 * @class
	 * @exports AttributeRouter
	 * @augments CaseSensitiveRouter
	 * 
	 * @param {WebCenterAPI}                       webCenterAPI            - used to get the status list
	 * @param {WorkflowSmartNameDictionaryService} smartNameDictionary     - the smartname dictionary that can be used for smartnamerouterrule
	 * @param {DateSmartNameDictionaryService}     dateSmartNameDictionary - the smartname dicitionary that can be used for date smartnamerouterrules
	 */
	var ProjectStatusRouter = function (webCenterAPI, smartNameDictionary, dateSmartNameDictionary) {
		RouterDecorator.call(this, WebCenterNodeConstants.RouterNode.Routers.ProjectStatus, i18n.t("router.projectstatus"), smartNameDictionary, dateSmartNameDictionary);

		/**
		 * @member {WebCenterAPI}
		 * @private
		 */
		this._webCenterAPI = webCenterAPI;

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._select = $("<select/>");

		this._webCenterAPI.getProjectStatusList().done(_.bind(function (map) {
			_.each(map.values(), function (attr) {
				var option = $("<option/>").val(attr.getId()).text(attr.getName());
				this._select.append(option);

			}, this);
			this._rules = [
				new SelectSmartNameRouterRule("RE_IS", i18n.t("wfui.route.ruleop.is"), i18n.t("wfui.route.ruleop.is"), this._smartNameDictionaryService, this._select),
				new SelectSmartNameRouterRule("RE_IS_NOT", i18n.t("wfui.route.ruleop.isnot"), i18n.t("wfui.route.ruleop.isnot"), this._smartNameDictionaryService, this._select)
			];
		}, this));

		/**
		 * @member {Boolean} - at init we get the status list so it might be possible 
		 * that the decorator has stopped being used by the time the list is back
		 * @private
		 */
		this._initDirty = false;

	};

	ProjectStatusRouter.prototype = Object.create(RouterDecorator.prototype);
	ProjectStatusRouter.prototype.constructor = ProjectStatusRouter;

	/**
	 * @override
	 */
	ProjectStatusRouter.prototype.init = function () {
		this._initDirty = true;
		this._webCenterAPI.getProjectStatusList().done(_.bind(function (map) {
			if (this._initDirty) {
				this._routerView.getRouter().setRouterRules(this._rules);
			}
		}, this));
	};

	ProjectStatusRouter.prototype.fillInParameter = function (routerParameter) {
		RouterDecorator.prototype.fillInParameter.call(this, routerParameter);
		routerParameter.setRouterRuleType(WebCenterNodeConstants.RouterNode.RuleTypes.EnumRule);
	};

	/**
	 * @override
	 */
	ProjectStatusRouter.prototype.destroy = function () {
		this._initDirty = false;
	};


	/**
	 * @override
	 */
	ProjectStatusRouter.prototype.disable = function () {
		RouterDecorator.prototype.disable.call(this);
		this._select.attr("disabled", true);
	};




	return ProjectStatusRouter;
});
