define([
		"underscore",
		"jquery",
		"utils/webi18n!WFEditorWCR"
	],
	function (
		_,
		$,
		i18n
	) {

		"use strict";

		/**
		 * A RouterDecorator is a class that enhances the view of a routerpanel by including extra options.
		 * An example of an implementation can be found in AttributeRouter.
		 * A RouterDecorator is also a controller of the router. When initialized it can set the available router rules.
		 * 
		 * @class
		 * @exports RouterDecorator
		 * 
		 * 
		 * @param {String}                             id                      - Id of the router decorator
		 * @param {String}                             name                    - name for displaying purposes
		 * @param {WorkflowSmartNameDictionaryService} smartNameDictionary     - the smartname dictionary that can be used for smartnamerouterrule
		 */
		var RouterDecorator = function (id, name, smartNameDictionary) {

			/**
			 * @member {JQueryObject} the element used to include extra options for the router eg. checkboxes, selects, ...
			 * @private
			 */
			this._element = $("<div class='form-group router-decorator'/>");

			/**
			 * @member {RouterView} the router view this decorator will decorate - will be set if the router decorator is enabled through the router panel
			 * @private
			 */
			this._routerView = null;

			/**
			 * @member {String}
			 * @private
			 */
			this._name = name;

			/**
			 * @member {String}
			 * @private
			 */
			this._id = id;

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._routerPanel = $("<div class='panel panel-default'/>");

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._routerPanelHeading = $("<div class='panel-heading router-panel-header'/>")
				.append($("<div/>").addClass("row")
					.append($("<div/>").addClass("router-header-view col-xs-12")
						.append($("<div/>").addClass("row")
							.append($("<div/>").addClass("col-md-3 col-xs-12").text(this._name))
							.append($("<div/>").addClass("col-md-3 col-md-offset-6 col-xs-12").text(i18n.t("wfui.route.panel.output"))))));

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._routerPanelBody = $("<div class='panel-body'/>");

			this._element.append(this._routerPanel.append(this._routerPanelHeading).append(this._routerPanelBody));

			/**
			 * @member {WorkflowSmartNameDictionaryService}
			 * @private
			 */
			this._smartNameDictionaryService = smartNameDictionary;

			/**
			 * @member {Boolean}
			 * @private
			 */
			this._disabled = false;
		};

		/**
		 * Sets the router view this routerdecorator should decorate
		 * @param {RouterView} routerView 
		 */
		RouterDecorator.prototype.setRouterView = function (routerView) {
			this._routerPanelBody.empty();
			this._routerView = routerView;
		};

		/**
		 * Sets the context of this decorator - first append it's own decoration and then the routerViews context
		 * @param {JQueryObject} context 
		 */
		RouterDecorator.prototype.setContext = function (context) {
			context.append(this._element);
			this._routerPanelBody.append(this._routerView.getContext());
		};

		/**
		 * In this method the router decorator can change the values of its decorations based on the router parameter
		 * @param {RouterParameter} routerParameter 
		 */
		RouterDecorator.prototype.handleParameter = function (routerParameter) {
			// override me
		};

		/**
		 * In this method the router decorator should fill in the router parameter based on its decorations 
		 * @param {RouterParameter} routerParameter 
		 */
		RouterDecorator.prototype.fillInParameter = function (routerParameter) {

			// Set the right outputs
			routerParameter.setRouterOutputs(this._routerView.getRouter().getUniqueRouterOutputsByAppearance());

			// add entries
			_.each(this._routerView.getRouter().getRouterEntries(), function (entry) {
				routerParameter.addRouterEntry(entry.getRule().getType(), entry.getRule().getSaveableValues(), entry.getOutput().getOutputId());
			});

			// default id
			routerParameter.setDefaultOutputId(this._routerView.getRouter().getDefaultRouteEntry().getOutput().getOutputId());

			// the type of the router decorator used
			routerParameter.setRouterType(this.getId());
		};

		/**
		 * the router decorator is being enabled, it should init itself and if applicable set the correct rules the router can use
		 */
		RouterDecorator.prototype.init = function () {
			// override me	
		};

		/**
		 * the router decorator isn't being used anymore for now
		 */
		RouterDecorator.prototype.destroy = function () {
			// override me	
		};

		/**
		 * The router panel is resized
		 */
		RouterDecorator.prototype.resize = function () {
			// override me if needed
		};

		/**
		 * gets the name of the decorator for display purposes
		 * @returns {String}
		 */
		RouterDecorator.prototype.getName = function () {
			return this._name;
		};

		/**
		 * gets the id of the decorator
		 * @returns {String}
		 */
		RouterDecorator.prototype.getId = function () {
			return this._id;
		};

		/**
		 * disable the decorator
		 */
		RouterDecorator.prototype.disable = function () {
			this._disabled = true;
		};

		return RouterDecorator;

	});