define([
	"jquery",
	"underscore",
	"./RouterDecorator",
	"../routerRules/SelectSmartNameRouterRule",
	"../routerPanel/RouterRule",
	"WFEditorWCR/workflow/panel/WebCenterParameter",
	"WFEditorWCR/workflow/panel/SmartNameDropdownPanel",
	"utils/webi18n!WFEditorWCR",
	"WFEditorWCR/workflow/node/WebCenterNodeConstants",
	"WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService",
	"packaging-content-management/ContentConstants"
], function (
	$,
	_,
	RouterDecorator,
	SelectSmartNameRouterRule,
	RouterRule,
	WebCenterParameter,
	SmartNameDropdownPanel,
	i18n,
	WebCenterNodeConstants,
	WorkflowSmartNameDictionaryService,
	ContentConstants) {

	"use strict";

	/**
	 * A router decorator to route on text content property
	 * 
	 * @class
	 * @exports TextContentPropertyRouter
	 * @augments RouterDecorator
	 * 
	 * @param {WorkflowSmartNameDictionaryService} smartNameDictionary     - the smartname dictionary that can be used for smartnamerouterrule
	 * @param {WebCenterAPI}                       webCenterAPI            - used to get the text content properties
	 */
	var TextContentPropertyRouter = function (webCenterAPI, smartNameDictionary) {
		RouterDecorator.call(this, WebCenterNodeConstants.RouterNode.Routers.TextContentProperty, i18n.t("router.textcontentproperty"),
			smartNameDictionary);

		/**
		 * @member {WebCenterAPI}
		 * @private
		 */
		this._webCenterAPI = webCenterAPI;

		/**
		 * @member {Object} used to create the select for the text content property
		 * @private
		 */
		this._options = [{
			id: ContentConstants.TextContent.STATE,
			label: i18n.t("router.tcstatus")
		}, {
			id: ContentConstants.TextContent.ELEMENTSTATUS,
			label: i18n.t("router.tcelemstatus")
		}, {
			id: ContentConstants.TextContent.ELEMENTTYPE,
			label: i18n.t("router.tcelementtype")
		}, {
			id: ContentConstants.TextContent.APPROVALTYPE,
			label: i18n.t("router.tcapprovaltype")
		}, {
			id: ContentConstants.TextContent.INVARIANT,
			label: i18n.t("router.tcinvariant")
		}];

		/**
		 * @member {Object} used to create the select for the text content status
		 * @private
		 */
		this._textContentStatusOptions = [{
			id: ContentConstants.State.DRAFT,
			label: i18n.t("tcstatus.draft")
		}, {
			id: ContentConstants.State.APPROVED,
			label: i18n.t("tcstatus.signedoff")
		}];

		/**
		 * @member {Object} used to create the select for the text content element status
		 * @private
		 */
		this._elementStatusOptions = [{
			id: ContentConstants.State.DRAFT,
			label: i18n.t("tcstatus.draft")
		}, {
			id: ContentConstants.State.READY_FOR_APPROVAL,
			label: i18n.t("tcstatus.pendingsignoff")
		}, {
			id: ContentConstants.State.APPROVED,
			label: i18n.t("tcstatus.signedoff")
		}, {
			id: ContentConstants.State.REJECTED,
			label: i18n.t("tcstatus.signoffrejected")
		}];

		/**
		 * @member {Object} used to create the select for the approval type
		 * @private
		 */
		this._approvalTypeOptions = [{
			id: ContentConstants.ApprovalSetup.DEFAULT,
			label: i18n.t("tcapprovaltype.default")
		}, {
			id: ContentConstants.ApprovalSetup.ONE_OF_GROUP,
			label: i18n.t("tcapprovaltype.allofroleonepergroup")
		}, {
			id: ContentConstants.ApprovalSetup.ONE_OF_ROLE,
			label: i18n.t("tcapprovaltype.oneofrole")
		}];

		/**
		 * @member {Object} used to create the select for the invariant flag
		 * @private
		 */
		this._invariantOptions = [{
			id: "true",
			label: i18n.t("tcinvariant.true")
		}, {
			id: "false",
			label: i18n.t("tcinvariant.false")
		}];

		/**
		 * @member {Object} used to create the select for the element types
		 * @private
		 */
		this._elementTypeOptions = [];

		/**
		 * @member {Object} used to create the select for the language elements
		 * @private
		 */
		this._langOptions = [];

		/**
		 * @member {JQueryObject} - Routing field dropdown
		 * @private
		 */
		this._select = $("<select/>").addClass("form-control");

		// Add options to select router field
		_.each(this._options, function (option) {
			this._select.append($("<option/>").val(option.id).text(option.label));
		}, this);

		//Add listener for changing router field
		this._select.change(_.bind(function (e) {
			this._updateRuleOptions();
		}, this));


		/**
		 * @member {JQueryObject} - Routing rule dropdown
		 * @private
		 */
		this._selectRule = $("<select/>");

		/**
		 * @member {Array.<RouterRule>}
		 * @private
		 */
		this._rules = [
		               new SelectSmartNameRouterRule("RS_IS_EQUAL", i18n.t("wfui.route.ruleop.is"), i18n.t("wfui.route.ruleop.is"), this._smartNameDictionaryService, this._selectRule),
		               new SelectSmartNameRouterRule("RS_IS_NOT_EQUAL", i18n.t("wfui.route.ruleop.isnot"), i18n.t("wfui.route.ruleop.isnot"), this._smartNameDictionaryService, this._selectRule)
		               ];

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._langSubParam = new WebCenterParameter("language", "", "");
		this._langSmartNameDropdownPanel = new SmartNameDropdownPanel("language", i18n.t("wfui.importdc.langdesc"), this._langOptions, this._smartNameDictionaryService);
		this._langSmartNameDropdownPanel.hide();
		//Default styling for smartname dropdown not required for this field
		this._langSmartNameDropdownPanel.removeStlying();

		this._element.prepend(this._langSmartNameDropdownPanel.getContext().addClass("form-group TexContentLanguagePanel"));

		this._element.prepend($("<div/>").addClass("form-group")
			.append($("<label/>").text(i18n.t("wfui.route.field.label")))
			.append(this._select));

		/**
		 * @member {Boolean} - in the at init method 
		 * we get the text content element type and language lists so it might be possible 
		 * that the decorator has stopped being used by the time the list is back
		 * @private
		 */
		this._initDirty = false;
	};

	TextContentPropertyRouter.prototype = Object.create(RouterDecorator.prototype);
	TextContentPropertyRouter.prototype.constructor = TextContentPropertyRouter;

	/**
	 * @override
	 */
	TextContentPropertyRouter.prototype.init = function () {
		this._initDirty = true;
		this._webCenterAPI.getTextContentLanguageList().done(_.bind(function (langMap) {
			if(this._initDirty){
				//Update the dropdown with PCM language list
				this._langOptions = [];
				this._langOptions.push({
					text: i18n.t('wfui.textcontent.languagedefaultlabel'),
					value: ""
				});
				_.each(langMap.get(ContentConstants.LanguageType.ALL).values(), function (lang) {
					this._langOptions.push({
						text: lang.getName(),
						value: lang.getId()
					});
				}, this);

				this._langSmartNameDropdownPanel.setParameters([this._langSubParam]);
				this._langSmartNameDropdownPanel.setOptions(this._langOptions);
			}
		}, this));
		
		this._webCenterAPI.getTextContentElementTypeList().done(_.bind(function (elementTypeMap) {
			if (this._initDirty) {
				//Populate element type options 
				this._elementTypeOptions = [];
				_.each(elementTypeMap.values(), function (elementType) {
					if(elementType.getId() !== "ALL_ELEMENT_TYPES"){
						this._elementTypeOptions.push({
							id: elementType.getId(),
							label: elementType.getName()
						});
					}
				}, this);
			}
		}, this));
		
		this._updateRuleOptions();
	};

	/**
	 * @override
	 */
	TextContentPropertyRouter.prototype.fillInParameter = function (routerParameter) {
		RouterDecorator.prototype.fillInParameter.call(this, routerParameter);
		routerParameter.setRouterRuleType(WebCenterNodeConstants.RouterNode.RuleTypes.EnumRule);
		var routingField = this._select.val();
		routerParameter.setValue(routingField);

		//Add language in sub paramater for element status routing
		if (routingField === ContentConstants.TextContent.ELEMENTSTATUS) {
			routerParameter.setSubParameters([this._langSubParam]);
		}
	};

	/**
	 * @override
	 * @param {RouterParameter} routerParameter 
	 */
	TextContentPropertyRouter.prototype.handleParameter = function (routerParameter) {
		RouterDecorator.prototype.handleParameter.call(this, routerParameter);
		if (routerParameter.getRouterType() === this.getId() && routerParameter.getValue()) {
			var routingField = routerParameter.getValue();
			this._select.val(routingField);

			//For text content element status routing, get the language from sub parameter
			if (routingField === ContentConstants.TextContent.ELEMENTSTATUS) {
				var subParams = routerParameter.getSubparameters();
				if (subParams.length > 0) {
					_.each(subParams, function (param) {
						if (param.getName() === "language") {
							this._langSubParam.setValue(param.getValue());
						}
					}, this);
				}
			}
		}
	};

	/**
	 * @override
	 */
	TextContentPropertyRouter.prototype.destroy = function () {
		this._initDirty = false;
	};

	/**
	 * @override
	 */
	TextContentPropertyRouter.prototype.disable = function () {
		RouterDecorator.prototype.disable.call(this);
		this._select.attr("disabled", true);
		this._langSmartNameDropdownPanel.disable();
	};

	/**
	 * When text content property has been selected set the correct rule options
	 */
	TextContentPropertyRouter.prototype._updateRuleOptions = function () {
		var val = this._select.val();
		this._selectRule.empty();
		//Lang options dropdown should be only shown for element status property
		this._langSmartNameDropdownPanel.hide();

		if( val === ContentConstants.TextContent.ELEMENTTYPE && this._elementTypeOptions.length === 0){
			this._fetchElementTypeList();
		} else {

			var optionToSet = [];
			switch (val) {
			case ContentConstants.TextContent.STATE:
				optionToSet = this._textContentStatusOptions;
				break;
			case ContentConstants.TextContent.ELEMENTTYPE:
				optionToSet = this._elementTypeOptions;
				break;
			case ContentConstants.TextContent.APPROVALTYPE:
				optionToSet = this._approvalTypeOptions;
				break;
			case ContentConstants.TextContent.INVARIANT:
				optionToSet = this._invariantOptions;
				break;
			case ContentConstants.TextContent.ELEMENTSTATUS:
				optionToSet = this._elementStatusOptions;
				// Show dropdown to select language element and populate the selected value
				this._langSmartNameDropdownPanel.show();
				break;
			default:
				//Keep text content status as default
				optionToSet = this._textContentStatusOptions;
			}

			// Add options to select
			this._selectRule.empty();
			_.each(optionToSet, function (option) {
				this._selectRule.append($("<option/>").val(option.id).text(option.label));
			}, this);

			this._routerView.getRouter().setRouterRules(this._rules);
		}
	};
	
	/**
	 * Retrieves element type list and populates the rule dropdown
	 */
	TextContentPropertyRouter.prototype._fetchElementTypeList = function(){
		this._initDirty = true;
		this._webCenterAPI.getTextContentElementTypeList().done(_.bind(function (elementTypeMap) {
			if (this._initDirty) {
				//Populate element type options 
				this._selectRule.empty();
				_.each(elementTypeMap.values(), function (elementType) {
					if(elementType.getId() !== "ALL_ELEMENT_TYPES"){
						this._selectRule.append($("<option/>").val(elementType.getId()).text(elementType.getName()));
					}
				}, this);
				
				//Make the new rule options enable
				this._routerView.getRouter().setRouterRules(this._rules);
			}
		}, this));
	};
	
	return TextContentPropertyRouter;
});