define(["underscore",
		"jquery",
		"./NoMatchRouterEntry",
		"./RouterEntry",
		"utils/Observable",
		"./RouterRule",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"utils/webi18n!WFEditorWCR"
	],
	function (_,
		$,
		NoMatchRouterEntry,
		RouterEntry,
		Observable,
		RouterRule,
		WebCenterPinConstants,
		OutputPinOK,
		i18n) {

		"use strict";

		/**
		 * Model used for the router panel
		 * 
		 * @class
		 * @exports Router
		 * @mixes Observable
		 * 
		 * @param {RouterNode} node
		 * @param {Boolean}	   isEnabled
		 */
		var Router = function (node, isEnabled) {
			Observable.mixin(this, "Router");

			/**
			 * @member {Array.<RouterRule>} list of all possible router rules. 
			 * These rules should be used as model and shouldn't be used directly. Instead we should copy them
			 * @private
			 */
			this._routerRules = [];

			/**
			 * @member {Array.<OutputPin>} list of the current router outputs
			 * @private
			 */
			this._routerOutputs = [];


			/**
			 * @member {Array.<RouterEntry>} list of router entries
			 * @private
			 */
			this._routerEntries = [];

			/**
			 * @member {NoMatchRouterEntry} the default 'no match' entry
			 * @private
			 */
			this._defaultRoute = new NoMatchRouterEntry(this);

			/**
			 * @member {OutputPinError}
			 * @private
			 */
			this._errorOutput = null;

			/**
			 * @member {RouterNode}
			 * @private
			 */
			this._node = node;

			/**
			 * @member {Boolean}
			 * @private
			 */
			this._isEnabled = true;

		};

		/**
		 * sets the router rules
		 * @param {Array.<RouterRule>} routerRules
		 */
		Router.prototype.setRouterRules = function (routerRules) {
			this._routerRules = routerRules;
			this._emit("changed.rules");
		};

		/**
		 * @returns {Array.<RouterRule>}
		 */
		Router.prototype.getRouterRules = function () {
			return this._routerRules;
		};

		/**
		 * creates a new router rule of a given type, if no type is specified, the first one will be chosen
		 * @param   {String}   [type] 
		 * @returns {RouterRule} 
		 */
		Router.prototype.createNewRouterRule = function (type) {
			if (!type) {
				type = this._routerRules[0].getType();
			}
			var rule = _.find(this._routerRules, function (rule) {
				return rule.getType() === type;
			});
			if (rule) {
				return rule.copy();
			}
			return new RouterRule(type, type, type);
		};

		/**
		 * changes the order of a router entry
		 * @param {Number} initialIndex 
		 * @param {Number} newIndex     
		 */
		Router.prototype.moveRouterEntryAtIndexTo = function (initialIndex, newIndex) {
			var entry = this._routerEntries[initialIndex];
			if (entry && newIndex < this._routerEntries.length) {
				this._routerEntries.splice(initialIndex, 1);
				this._routerEntries.splice(newIndex, 0, entry);
			}
		};

		/**
		 * Adds a router output
		 * @param {WebCenterPin} routerOutput 
		 */
		Router.prototype.addRouterOutput = function (routerOutput) {
			if (this._routerOutputs.indexOf(routerOutput) !== -1) {
				// The error output should always be at the end
				this._routerOutputs.splice(this._routerOutputs.length - 1, 0, routerOutput);
				this._emit("changed.outputs");
			}
		};

		/**
		 * Sets the router outputs
		 * @param   {Array.<OutputPin>} outputs 
		 */
		Router.prototype.setRouterOutputs = function (outputs) {
			this._routerOutputs = outputs;
			this._errorOutput = _.find(outputs, function (output) {
				return output.getOutputId() === WebCenterPinConstants.Type.WorkflowConstructError;
			});
			this._emit("changed.outputs");
		};

		/**
		 * returns the outputs
		 * @returns {Array.<OutputPin>} 
		 */
		Router.prototype.getRouterOutputs = function () {
			return this._routerOutputs;
		};

		/**
		 * returns a copy of all router outputs sorted by their appearance in the router entries. The error output is always at the end
		 * @returns {Array.<OutputPin>} 
		 */
		Router.prototype. getUniqueRouterOutputsByAppearance = function () {
			var outputs = [];
			_.each(this.getRouterEntries(), function (entry) {
				if (entry.getOutput() && entry.getOutput() !== this._errorOutput) {
					outputs.push(entry.getOutput());
				}
			}, this);
			if (this._defaultRoute && this._defaultRoute.getOutput()) {
				outputs.push(this._defaultRoute.getOutput());
			}
			if (this._errorOutput) {
				outputs.push(this._errorOutput);
			}
			return _.uniq(outputs);
		};

		/**
		 * set the output for the no match entry by id
		 * @param {String} id 
		 */
		Router.prototype.setDefaultOutput = function (id) {
			this._defaultRoute.setOutput(this.getRouterOutputById(id));
		};


		/**
		 * removes an output if it is not used by any router entries and it is not the error output
		 * @param   {OutputPin} output 
		 */
		Router.prototype.removeRouterOutputIfNotInUse = function (output) {
			if (!output || output === this._errorOutput) {
				return;
			}
			var isUsed = _.find(this._routerEntries, function (entry) {
				return entry.getOutput() === output;
			});
			if (!isUsed && this._defaultRoute.getOutput() !== output) {
				var index = this._routerOutputs.indexOf(output);
				if (index !== -1) {
					this._routerOutputs.splice(index, 1);
					this._emit("changed.outputs");
				}
			}
		};

		/**
		 * returns a router output by id
		 * @param   {String}    id 
		 * @returns {OutputPin} 
		 */
		Router.prototype.getRouterOutputById = function (id) {
			return _.find(this._routerOutputs, function (output) {
				return output.getOutputId() === id;
			});
		};

		/**
		 * returns the output with a given name, if it doesn't exists yet it creates a new one
		 * @param   {String}       name 
		 * @returns {WebCenterPin} 
		 */
		Router.prototype.getRouterOutputWithName = function (name) {
			var output = _.find(this._routerOutputs, function (routerOutput) {
				return routerOutput.getLabel() === name;
			}, this);
			if (!output) {
				output = new OutputPinOK(this._node.getWorkflow(), this._node, this._generateId(), name);
				this._routerOutputs.splice(this._routerOutputs.length - 1, 0, output);
				this._emit("changed.outputs");
			}
			return output;
		};

		/**
		 * Add a router entry
		 * @param {RouterEntry} routerEntry
		 */
		Router.prototype.addRouterEntry = function (routerEntry) {
			if (!routerEntry) {
				routerEntry = new RouterEntry(this, this._routerRules[0].copy());
			}
			this._routerEntries.push(routerEntry);
			this._emit("added.routerEntry", routerEntry);
		};

		/**
		 * removes a router entry if it isn't the last router entry.
		 * @param {RouterEntry} routerEntry
		 * @param {Boolean} forceRemoveLastEntry - true if the last entry can be removed
		 */
		Router.prototype.removeRouterEntry = function (routerEntry, forceRemoveLastEntry) {
			var indexOfRouterEntry = this._routerEntries.indexOf(routerEntry);
			if (indexOfRouterEntry > -1 && (forceRemoveLastEntry || this._routerEntries.length !== 1)) {
				routerEntry.destroy();
				this._routerEntries.splice(indexOfRouterEntry, 1);
				this.removeRouterOutputIfNotInUse(routerEntry.getOutput());
				this._emit("removed.routerEntry", routerEntry);
			}
		};

		/**
		 * returns all router entries
		 * @returns {Array.<RouterEntry>} 
		 */
		Router.prototype.getRouterEntries = function () {
			return this._routerEntries;
		};

		/**
		 * returns the no match router entry
		 * @returns {RouterEntry} 
		 */
		Router.prototype.getDefaultRouteEntry = function () {
			return this._defaultRoute;
		};

		/**
		 * @returns {RouterNode}
		 */
		Router.prototype.getNode = function () {
			return this._node;
		};


		/**
		 * if @param {isEnabled} is undefined, returns if the router is enabled. Otherwise sets the isEnabled property 
		 * @param {Boolean} [isEnabled]
		 * @returns {Boolean} 
		 */
		Router.prototype.isEnabled = function (isEnabled) {
			if (typeof isEnabled === "undefined") {
				return this._isEnabled;
			} else {
				if (this._isEnabled !== isEnabled) {
					this._isEnabled = isEnabled;
					this._emit("changed.enabled", isEnabled);
				}
			}
		};

		/**
		 * generates an id for a Output
		 * @returns {String} 
		 */
		Router.prototype._generateId = function () {
			var id;
			do {
				id = "Output_" + this._generateRandomString(6);
			} while (this.getRouterOutputById(id));
			return id;

		};

		/**
		 * generates a random string with a given length
		 * @param   {Number} length 
		 * @returns {String} 
		 */
		Router.prototype._generateRandomString = function (length) {
			var result = "";
			var mask = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
			for (var i = length; i > 0; --i) {
				result += mask[Math.floor(Math.random() * mask.length)];
			}
			return result;
		};

		return Router;
	});
