define(["underscore",
	"jquery",
	"utils/Observer",
	"utils/Observable",
	"./RuleSelectorView",
	"./RouterOutputSelectorView"
], function (_,
	$,
	Observer,
	Observable,
	RuleSelectorView,
	RouterOutputSelectorView
) {

	"use strict";

	/**
	 *
	 * a router entry component in the router. It has a rule and an output and views and controllers for those
	 *
	 * @class
	 * @exports RouterEntry
	 * @mixes Observer
	 * @mixes Observable
	 * @param {Router} router - the router this entry is added to
	 * @param {RouterRule} routerRule - the rule used for this entry
	 * @param {WebCenterPin} routerOutput - the output for this entry
	 */
	var RouterEntry = function (router, routerRule, routerOutput) {
		Observer.mixin(this);
		Observable.mixin(this, this);
		this._observe(router, "changed.rules", this._rulesChanged);

		/**
		 * @member {Router}
		 * @private
		 */
		this._router = router;

		/**
		 * @member {RouterRule}
		 * @private
		 */
		this._routerRule = routerRule;

		// was an output specified?
		var hasRouterOutput = typeof routerOutput !== "undefined";

		// if there is no output and there is a router rule, set an output with the router rule's name
		if (!routerOutput && routerRule) {
			routerOutput = router.getRouterOutputWithName(this._routerRule.getDisplayName());
		}

		/**
		 * @member {WebCenterPin}
		 * @private
		 */
		this._routerOutput = routerOutput;

		/**
		 * @member {RuleSelectorView} controller for the RouterRule
		 * @private
		 */
		this._ruleSelectorView = new RuleSelectorView(router, this);

		/**
		 * @member {RouterOutputSelectorView} view and controller for the output
		 * @private
		 */
		this._outputSelectorView = new RouterOutputSelectorView(router, this, hasRouterOutput);

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._ruleContext = $("<div/>").addClass("col-lg-6 col-sm-12");

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._context = $("<div/>").addClass("row router-panel-entry")
			.append($("<div/>").addClass("col-xs-12 router-entry-view")
				.append($("<div/>").addClass("row")
					.append($("<div/>").addClass("col-lg-3 col-sm-12").append(this._ruleSelectorView.getContext()))
					.append(this._ruleContext)
					.append($("<div/>").addClass("col-lg-3 col-sm-12").append(this._outputSelectorView.getContext()))
				));

		// Add a delete button if the router is enabled
		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._deleteBtn = $("<a href='#'/>").addClass("btn btn-xs btn-default")
			.append($('<i />').addClass("icon icon-WCR_Actions-Delete12 icon-small"))
			.on("click", _.bind(function () {
				this._router.removeRouterEntry(this);
			}, this));

		if (this._router.isEnabled()) {
			this._context.append($("<div/>").addClass("router-entry-delete").append(this._deleteBtn));
		}

		this._context.addClass("router-entry-sortable");

		this._observe(this._router, "changed.enabled", this._enabledChanged);

		if (routerRule) {
			this._routerRule.setContext(this._ruleContext);
		}

	};

	/**
	 * rules have changed, check if the entry should get another rule
	 */
	RouterEntry.prototype._rulesChanged = function () {
		// Check if our rule is interchangeable with a rule definition in the router
		var rule = _.find(this._router.getRouterRules(), function (rule) {
			return this._routerRule.canBeReplacedBy(rule) && rule.canBeReplacedBy(this._routerRule);
		}, this);

		if (rule) {
			// There was a rule, we still hold a valid rule, so nothing has to happen
			return;
		}
		// Our rule cannot be interchanged with any other rule currently available. set a new one
		// Create a new rule
		rule = this._router.createNewRouterRule();

		// copy values of previous rule
		rule.setValues(this._routerRule.getValues());

		this.setRule(rule);
	};

	/**
	 * returns the output of this entry
	 * @returns {WebCenterPin} 
	 */
	RouterEntry.prototype.getOutput = function () {
		return this._routerOutput;
	};

	/**
	 * set the output for this entry
	 * @param {WebCenterPin} output 
	 */
	RouterEntry.prototype.setOutput = function (output) {
		if (output !== this._routerOutput) {
			var previousOutput = this._routerOutput;
			this._routerOutput = output;
			this._emit("changed.output");
			this._router.removeRouterOutputIfNotInUse(previousOutput);
		}

	};

	/**
	 * @returns {Array.<RouterRule>}
	 */
	RouterEntry.prototype.getRule = function () {
		return this._routerRule;
	};

	/**
	 * @param {RouterRule}
	 */
	RouterEntry.prototype.setRule = function (rule) {
		// destroy previous rule
		this._routerRule.destroy();
		this._ruleContext.empty();
		this._routerRule = rule;
		this._routerRule.setContext(this._ruleContext);
		this._emit("changed.rule");
	};

	/**
	 * @returns {JqueryObject}
	 */
	RouterEntry.prototype.getContext = function () {
		return this._context;
	};

	/**
	 * destroy the router entry
	 */
	RouterEntry.prototype.destroy = function () {
		this._stopObserveAll();
		this._destroyObservable();
		this._ruleSelectorView.destroy();
		this._outputSelectorView.destroy();
		this._deleteBtn.remove();
		this._routerRule.destroy();
	};

	/**
	 * Resize the router entry
	 */
	RouterEntry.prototype.resize = function () {
		this._routerRule.resize();
	};

	/**
	 * dis/enable router entry
	 * @param {Boolean} isEnabled 
	 */
	RouterEntry.prototype._enabledChanged = function (isEnabled) {
		if (isEnabled) {
			this._context.append($("<div/>").addClass("router-entry-delete").append(this._deleteBtn));
			if (this._routerRule) {
				this._routerRule.enable();
			}
			this._outputSelectorView.enable();
			if (this._ruleSelectorView) {
				this._ruleSelectorView.enable();
			}
		} else {
			this._deleteBtn.detach();
			this._context.find(".router-entry-delete").remove();
			if (this._routerRule) {
				this._routerRule.disable();
			}
			this._outputSelectorView.disable();
			if (this._ruleSelectorView) {
				this._ruleSelectorView.disable();
			}
		}
	};

	return RouterEntry;

});