define(["underscore",
	"jquery",
	"utils/Observer",
	"component/inputDropdown/InputDropdown"

], function (
	_,
	$,
	Observer,
	InputDropdown) {

	"use strict";

	/**
	 *
	 * A controller and a view for the output of a router entry
	 *
	 * @class
	 * @exports RouterOutputSelectorView
	 * @mixes Observer
	 * @param {Router}      router             
	 * @param {RouterEntry} routerEntry        
	 * @param {Boolean}     isManuallySelected - was this router output added to the router manually or was it a just a default one?
	 */
	var RouterOutputSelectorView = function (router, routerEntry, isManuallySelected) {
		Observer.mixin(this);

		/**
		 * @member {Router}
		 * @private
		 */
		this._router = router;

		/**
		 * @member {RouterEntry}
		 * @private
		 */
		this._routerEntry = routerEntry;

		/**
		 * @member {InputDropdown}
		 * @private
		 */
		this._inputDropdown = new InputDropdown();

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._selectDiv = $("<div/>").append(this._inputDropdown.getElement());

		/**
		 * @member {Boolean}
		 * @private
		 */
		this._isManuallySelected = isManuallySelected;

		this._observe(router, "changed.outputs", this._fillSelect);

		// Rule is saved here to stop observing it if the rule was changed in the router entry
		var rule = this._routerEntry.getRule();

		this._observe(routerEntry, "changed.rule", function () {
			if (!this._isManuallySelected) {
				// Rule has changed and it wasn't manually selected -> rename the output
				this._stopObserveObject(rule);
				rule = this._routerEntry.getRule();
				this._routerEntry.setOutput(this._router.getRouterOutputWithName(this._routerEntry.getRule().getDisplayName()));
				this._observe(rule, "changed.value", function () {
					this._routerEntry.setOutput(this._router.getRouterOutputWithName(this._routerEntry.getRule().getDisplayName()));
				});
			}

		});

		// If there is a non-manually selected rule, listen for value changes to updat the output
		if (rule && !isManuallySelected) {
			this._observe(rule, "changed.value", function () {
				this._routerEntry.setOutput(this._router.getRouterOutputWithName(this._routerEntry.getRule().getDisplayName()));
			});
		}

		// Observe for output changes to set the value of the dropdown
		this._observe(routerEntry, "changed.output", function () {
			this._inputDropdown.val(this._routerEntry.getOutput().getOutputId());
		});

		this._observe(this._inputDropdown, "changed.value", function (value) {
			var output = this._router.getRouterOutputById(value);
			this._routerEntry.setOutput(output);
			this._isManuallySelected = true;

		});
		this._observe(this._inputDropdown, "new.input", function (value) {
			var output = this._router.getRouterOutputWithName(value);
			this._routerEntry.setOutput(output);
			this._isManuallySelected = true;

		});

		this._fillSelect();

		if (routerEntry.getOutput()) {
			this._inputDropdown.val(this._routerEntry.getOutput().getOutputId());
		}

	};

	/**
	 * fill the input dropdown with all possible outputs
	 */
	RouterOutputSelectorView.prototype._fillSelect = function () {
		this._inputDropdown.empty();
		_.each(this._router.getRouterOutputs(), function (output) {
			this._inputDropdown.addOption(output.getOutputId(), output.getLabel());
		}, this);
	};

	/**
	 * @returns {JQueryObject}
	 */
	RouterOutputSelectorView.prototype.getContext = function () {
		return this._selectDiv;
	};

	/**
	 * stop all observing
	 */
	RouterOutputSelectorView.prototype.destroy = function () {
		this._stopObserveAll();
	};


	/**
	 * enable ruleselector
	 */
	RouterOutputSelectorView.prototype.enable = function () {
		this._inputDropdown.enable();
	};

	/**
	 * disable ruleselector
	 */
	RouterOutputSelectorView.prototype.disable = function () {
		this._inputDropdown.disable();
	};

	return RouterOutputSelectorView;
});
