define(["underscore",
	"jquery",
	"utils/webi18n!WFEditorWCR",
	"./Router",
	"./RouterView",
	"../routerDecorators/ProjectStatusRouter",
	"../routerDecorators/ProjectFieldRouter",
	"../routerDecorators/ProjectAttributeRouter",
	"../routerDecorators/DocumentAttributeRouter",
	"../routerDecorators/DocumentNameRouter",
	"../routerDecorators/NumberOfDocumentsRouter",
	"../routerDecorators/WorkflowSpecificationRouter",
	"../routerDecorators/TextContentPropertyRouter",
	"./RouterEntry",
	"utils/Observable",
	"WFEditorWCR/workflow/node/WebCenterNodeConfigurationPanel"
], function (_,
	$,
	i18n,
	Router,
	RouterView,
	ProjectStatusRouter,
	ProjectFieldRouter,
	ProjectAttributeRouter,
	DocumentAttributeRouter,
	DocumentNameRouter,
	NumberOfDocumentsRouter,
	WorkflowSpecificationRouter,
	TextContentPropertyRouter,
	RouterEntry,
	Observable,
	WebCenterNodeConfigurationPanel
) {

	"use strict";

	/**
	 * NodeConfigurationPanel for the RouterNode
	 * 
	 * @class
	 * @augments WebCenterNodeConfigurationPanel
	 * @exports RouterPanel
	 * @mixes Observable
	 * 
	 * @param {RouterNode} node                               
	 * @param {WebCenterAPI} webCenterAPI                       
	 * @param {WorkflowSmartNameDictionaryService} workflowSmartNameDictionaryService 
	 * @param {DateSmartNameDictionaryService} dateSmartNameDictionaryService     
	 */
	var RouterPanel = function (node, webCenterAPI, workflowSmartNameDictionaryService, dateSmartNameDictionaryService) {
		WebCenterNodeConfigurationPanel.call(this, node);
		Observable.mixin(this, "routerPanel");

		/**
		 * @member {Router}
		 * @private
		 */
		this._router = new Router(node);

		/**
		 * @member {RouterView}
		 * @private
		 */
		this._routerView = new RouterView(this._router);

		/**
		 * @member {RouterParameter}
		 * @private
		 */
		this._routerParameter = null;

		/**
		 * @member {WebCenterAPI}
		 * @private
		 */
		this._webCenterAPI = webCenterAPI;

		/**
		 * @member {RouterNode}
		 * @private
		 */
		this._node = node;

		/**
		 * @member {WorkflowSmartNameDictionaryService}
		 * @private
		 */
		this._smartNameDictionary = workflowSmartNameDictionaryService;

		/**
		 * @member {DateSmartNameDictionaryService}
		 * @private
		 */
		this._dateSmartNameDictionary = dateSmartNameDictionaryService;

		/**
		 * @member {Object.<RouterDecorator>}
		 * @private
		 */
		this._routerDecorators = {};

		/**
		 * @member {RouterDecorator}
		 * @private
		 */
		this._currentRouterDecorator = null;

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._routerPanel = $("<div/>");

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._routerSelectorLabel = $("<label/>").text(i18n.t("wfui.route.panel.Attribute"));

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._routerSelector = $("<select/>").addClass("form-control");

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._routerSelectorDiv = $("<div/>").addClass("form-group").append(this._routerSelectorLabel).append(this._routerSelector);

		this._initRouterDecorators();

		this._element.addClass("container-fluid parameter-panel");

		this._element.append(this._routerSelectorDiv).append(this._routerPanel);
		this._routerView.getContext().append(this._buttonDiv);

	};

	RouterPanel.prototype = Object.create(WebCenterNodeConfigurationPanel.prototype);
	RouterPanel.prototype.constructor = RouterPanel;

	/**
	 * register a router decorator with the router panel
	 * @param {RouterDecorator} routerDecorator
	 */
	RouterPanel.prototype.registerRouterDecorator = function (routerDecorator) {
		this._routerDecorators[routerDecorator.getId()] = routerDecorator;
		routerDecorator.setRouterView(this._routerView);
		this._emit("added.routerDecorator", routerDecorator);
	};

	/**
	 * enable a router decorator
	 * @param {String} id
	 */
	RouterPanel.prototype.enableRouterDecorator = function (id) {
		this._routerView.getContext().detach();
		if (this._currentRouterDecorator) {
			this._currentRouterDecorator.destroy();
		}
		this._currentRouterDecorator = this._routerDecorators[id];
		this._routerPanel.empty();
		this._currentRouterDecorator.setContext(this._routerPanel);
		this._currentRouterDecorator.init();
		this._routerSelector.val(id);
	};

	/**
	 * @override
	 */
	RouterPanel.prototype.getPreferredWidth = function () {
		return 756;
	};

	/**
	 * @override
	 */
	RouterPanel.prototype.getParameters = function () {
		this._routerParameter.reset();
		this._currentRouterDecorator.fillInParameter(this._routerParameter);
		return [this._routerParameter];
	};

	/**
	 * @override
	 */
	RouterPanel.prototype.setParameters = function (parameters) {

		// First remove all entries currently in the router
		_.each(this._router.getRouterEntries().concat([]), function (entry) {
			this._router.removeRouterEntry(entry, true);
		}, this);

		this._routerParameter = parameters[0];

		// set the outputs
		this._router.setRouterOutputs(this._routerParameter.getRouterOutputs());
		this._router.setDefaultOutput(this._routerParameter.getDefaultOutputId());

		// Let the routerDecorators set their own state
		_.each(this._routerDecorators, function (routerDecorator) {
			routerDecorator.handleParameter(this._routerParameter);
		}, this);


		// Now enable the router decorator
		this.enableRouterDecorator(this._routerParameter.getRouterType());
		if (this._routerParameter.getRouterEntries().length === 0) {
			this._routerParameter.addRouterEntry("RS_CONTAINS", [""]);
		}
		// Create the router entries for display
		_.each(this._routerParameter.getRouterEntries(), function (entry) {
			var routerRule = this._router.createNewRouterRule(entry.ruleType);
			routerRule.setValues(entry.ruleValues);
			var routerOutput = this._router.getRouterOutputById(entry.outputId);
			var routerEntry = new RouterEntry(this._router, routerRule, routerOutput);
			this._router.addRouterEntry(routerEntry);
		}, this);

	};

	/**
	 * initiates the router decorators
	 */
	RouterPanel.prototype._initRouterDecorators = function () {
		this._buttonDiv = $("<div class='router-add-button'/>").append($('<a />').addClass("AddBtn")
			.attr("disabled", !this._isEnabled).addClass("btn btn-sm btn-primary")
			.append($('<i />').addClass("icon icon-WCR_Actions-Add icon-small"))
			.on("click", _.bind(function () {
				this._router.addRouterEntry();
				this.resize();
				this._fitContent(true);
			}, this)));

		this.registerRouterDecorator(new ProjectStatusRouter(this._webCenterAPI, this._smartNameDictionary));
		this.registerRouterDecorator(new ProjectFieldRouter(this._smartNameDictionary));
		this.registerRouterDecorator(new ProjectAttributeRouter(this._webCenterAPI, this._smartNameDictionary, this._dateSmartNameDictionary));
		this.registerRouterDecorator(new DocumentAttributeRouter(this._webCenterAPI, this._smartNameDictionary, this._dateSmartNameDictionary));
		this.registerRouterDecorator(new DocumentNameRouter(this._smartNameDictionary));
		this.registerRouterDecorator(new NumberOfDocumentsRouter(this._smartNameDictionary));
		this.registerRouterDecorator(new WorkflowSpecificationRouter(this._webCenterAPI, this._smartNameDictionary, this._dateSmartNameDictionary, this._node));

		//Show text content property fields in router only if PCM License enabled
		if (window.webcenterConfig.isPCMEnabled) {
			this.registerRouterDecorator(new TextContentPropertyRouter(this._webCenterAPI, this._smartNameDictionary));
		}

		_.each(this._routerDecorators, function (decorator, id) {
			this._routerSelector.append($("<option/>").val(id).text(decorator.getName()));
		}, this);

		this._routerSelector.on("change", _.bind(function () {
			this.enableRouterDecorator(this._routerSelector.val());
		}, this));


	};

	/**
	 * @override
	 */
	RouterPanel.prototype.disable = function () {
		WebCenterNodeConfigurationPanel.prototype.disable.call(this);
		this._router.isEnabled(false);
		this._buttonDiv.hide();
		this._currentRouterDecorator.disable();
		this._routerSelector.attr("disabled", true);
	};

	/**
	 * @override
	 */
	RouterPanel.prototype.enable = function () {
		WebCenterNodeConfigurationPanel.prototype.enable.call(this);
		this._router.isEnabled(true);
		this._buttonDiv.show();

	};

	/**
	 * @override
	 */
	RouterPanel.prototype.destroy = function () {
		this._currentRouterDecorator.destroy();
		this._routerView.destroy();
	};


	/**
	 * @override
	 */
	RouterPanel.prototype.resize = function () {
		this._currentRouterDecorator.resize();
		this._routerView.resize();
	};


	return RouterPanel;

});