define(["underscore", "jquery", "utils/Observable"], function (_, $, Observable) {

	"use strict";

	/**
	 *
	 * A RouterRule defines a certain rule the router can route on. A RouterRule can for example be 'matches abc'
	 *
	 * @class
	 * @exports RouterRule
	 * @mixes Observable
	 * @param {String} type  - the type of router rule
	 * @param {String} label - label of the router rule
	 */
	var RouterRule = function (type, label, shortName) {
		Observable.mixin(this, "routerrule" + type + label);

		/**
		 * @member {String} - the type of the rule. eg. "RN_EQUAL" for a numeric equals
		 * @private
		 */
		this._type = type;

		/**
		 * @member {String} - the label for the type
		 * @private
		 */
		this._label = label;

		/**
		 * @member {String} a shorter name for this rule
		 * @private
		 */
		this._shortName = shortName;

		/**
		 * @member {Array.<String>} the values for the router rule. In the example 'matches abc'. 'abc' is the value
		 * @private
		 */
		this._values = [];

		/**
		 * @member {JQueryObject} the element the router rule should render itself in
		 * @private
		 */
		this._element = $("<div/>");
	};

	/**
	 * @returns {String}
	 */
	RouterRule.prototype.getType = function () {
		return this._type;
	};

	/**
	 * @returns {String}
	 */
	RouterRule.prototype.getLabel = function () {
		return this._label;
	};

	/**
	 * checks whether this rule can be replaced by otherRule
	 * @returns {Boolean}
	 */
	RouterRule.prototype.canBeReplacedBy = function (otherRule) {
		return this._type === otherRule.getType();
	};

	/**
	 * makes a copy of this router rule
	 * @returns {RouterRule} 
	 */
	RouterRule.prototype.copy = function () {
		var rule = new RouterRule(this._type, this._label, this._shortName);
		var values = [];
		_.each(this._values, function (value) {
			values.push(value);
		});
		rule.setValues(values);
		return rule;
	};

	/**
	 * sets the values of this router rule
	 * @param {Array.<String>} values 
	 */
	RouterRule.prototype.setValues = function (values) {
		this._values = values;
		this._emit("changed.value");
	};

	/**
	 * sets the value at index @param {i} to @param {val}
	 * @param {Number} i   
	 * @param {String} val 
	 */
	RouterRule.prototype._setValue = function (i, val) {
		this._values[i] = val;
		this._emit("changed.value");
	};

	/**
	 * returns the values
	 * @returns {Array.<String>} 
	 */
	RouterRule.prototype.getValues = function () {
		return this._values;
	};

	/**
	 * returns the values that make sense in this router rule. fe. in a rule 'matches abc' there should only be one value. but in a rule 'between 1 and 5' there are two values
	 * @returns {Array.<String>} 
	 */
	RouterRule.prototype.getSaveableValues = function () {
		return [];
	};

	/**
	 * returns the display name of this rule. defaults to the shorter name
	 * @returns {String} 
	 */
	RouterRule.prototype.getDisplayName = function () {
		return this._shortName;
	};

	/**
	 * set the context of the rule
	 * @param {JQueryObject} context 
	 */
	RouterRule.prototype.setContext = function (context) {
		context.append(this._element);
	};

	/**
	 * destroy the rule
	 */
	RouterRule.prototype.destroy = function () {
		this._emit("destroyed.rule");
		this._destroyObservable();
	};

	/**
	 * Resize router rule
	 */
	RouterRule.prototype.resize = function () {
		// override me	
	};

	/**
	 * enable routerrule
	 */
	RouterRule.prototype.enable = function () {
		// override me
	};

	/**
	 * disable routerrule
	 */
	RouterRule.prototype.disable = function () {
		// override me
	};

	return RouterRule;
});