define(["underscore",
		"jquery",
		"utils/Observer",
		"jqueryUI"
	],
	function (
		_,
		$,
		Observer,
		jqueryUI
	) {

		"use strict";

		/**
		 * view for the router
		 * @class
		 * @mixes Observer
		 * @exports RouterView
		 * @param {Router} router
		 */
		var RouterView = function (router) {
			Observer.mixin(this);

			this._observe(router, "added.routerEntry", this._routerEntryAdded);
			this._observe(router, "removed.routerEntry", this._routerEntryRemoved);
			this._observe(router, "changed.enabled", this._enabledChanged);
			/**
			 * @member {Router}
			 * @private
			 */
			this._router = router;

			// indexes used when sorting
			var sortableStartIndex = null;
			var sortableStopIndex = null;

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._element = $("<div/>").addClass("router-panel form-compact");

			// router entries in the router panel should be sortable
			this._element.sortable({
				axis: "y",
				containment: this._element,
				items: ".router-entry-sortable",
				cancel: "input,select,.SmartNameInput,.SmartNameDropdown,.one-router-entry-left .router-entry-sortable",
				scroll: "false",
				cursor: "grabbing",
				placeholder: "router-entry-placeholder",
				tolerance: "pointer",
				start: function (event, ui) {
					sortableStartIndex = ui.item.index();
				},
				stop: _.bind(function (event, ui) {
					sortableStopIndex = ui.item.index();
					this._router.moveRouterEntryAtIndexTo(sortableStartIndex, sortableStopIndex);
				}, this)
			});

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._entriesList = $("<div/>").appendTo(this._element);

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._noMatchEntryDiv = $("<div/>").appendTo(this._element);

			this._router.getDefaultRouteEntry().getContext().appendTo(this._noMatchEntryDiv);


			_.each(router.getRouterEntries(), function (entry) {
				this._routerEntryAdded(entry);
			}, this);
		};

		/**
		 * @returns {Router}
		 */
		RouterView.prototype.getRouter = function () {
			return this._router;
		};

		/**
		 * append the router entry to the router panel
		 * @param {RouterEntry} routerEntry 
		 */
		RouterView.prototype._routerEntryAdded = function (routerEntry) {
			var routerEntryDiv = routerEntry.getContext();
			this._entriesList.append(routerEntryDiv);

			if (this._router.getRouterEntries().length !== 1) {
				this._element.removeClass("one-router-entry-left");
			} else {
				// Add a class if there is only one router entry left
				this._element.addClass("one-router-entry-left");
			}
		};

		/**
		 * remove the router entry from the router panel
		 * @param {RouterEntry} routerEntry 
		 */
		RouterView.prototype._routerEntryRemoved = function (routerEntry) {
			routerEntry.getContext().remove();
			routerEntry.destroy();
			if (this._router.getRouterEntries().length === 1) {
				// Add a class if there is only one router entry left
				this._element.addClass("one-router-entry-left");
			}
		};

		/**
		 * destroys the router view
		 */
		RouterView.prototype.destroy = function () {
			_.each(this._router.getRouterEntries(), function (entry) {
				entry.destroy();
			});
			this._stopObserveAll();

		};

		/**
		 * The router is resized
		 */
		RouterView.prototype.resize = function () {
			_.each(this._router.getRouterEntries(), function (entry) {
				entry.resize();
			});
		};

		/**
		 * @returns {JQueryObject}
		 */
		RouterView.prototype.getContext = function () {
			return this._element;
		};

		/**
		 * disable/enable sorting functionality
		 * @param {Boolean} isEnabled 
		 */
		RouterView.prototype._enabledChanged = function (isEnabled) {
			if (isEnabled) {
				this._element.sortable("enable");
			} else {
				this._element.sortable("disable");
			}
		};

		return RouterView;

	});