define([
	"component/smartnames/smartfield/smartfield",
	"component/smartnames/encoding/smartnameencoding",
	"../routerPanel/RouterRule",
	"utils/webi18n!WFEditorWCR",
	"jquery",
	"underscore"
], function (
	SmartField,
	SmartNameEncoding,
	RouterRule,
	i18n,
	$,
	_) {

	"use strict";

	/**
	 * A RouterRule that supports two values. for example "between 0 and 5". 
	 *
	 * @class
	 * @exports BetweenSmartNameRouterRule
	 * @augments RouterRule
	 * 
	 * @param {String} type                
	 * @param {String} label               
	 * @param {String} shortName           
	 * @param {SmartnameDictionaryService} smartNameDictionary
	 */
	var BetweenSmartNameRouterRule = function (type, label, shortName, smartNameDictionary) {
		RouterRule.call(this, type, label, shortName);

		/**
		 * @member {JQueryObject} input field
		 * @private
		 */
		this._inputFrom = $('<input type="text">');
		this._inputTo = $('<input type="text">');

		/**
		 * @member {JQueryObject} input field div
		 * @private
		 */
		this._element.empty();
		this._element.addClass("input-group");
		this._element.append($("<div/>").append(this._inputFrom));
		this._element.append($("<span/>").text(i18n.t("wfui.route.ruleop.and")).addClass("between-smartname-label"));
		this._element.append($("<div/>").append(this._inputTo));

		/**
		 * @member {SmartnameDictionaryService}
		 * @private
		 */
		this._smartNameDictionary = smartNameDictionary;

		/**
		 * @member {SmartField} the smartfield for the first value
		 * @private
		 */
		this._smartFieldFrom = new SmartField(this._inputFrom,
			false, // lazy init
			this._smartNameDictionary, // our dictionary
			_.bind(this._smartNameDictionary.displaySmartName, this._smartNameDictionary), // display in input field
			_.bind(this._smartNameDictionary.displaySmartNameSelectorSmartName, this._smartNameDictionary), // display in selector
			SmartNameEncoding.WORKFLOWEDITOR // encoding
		);

		/**
		 * @member {SmartField} the smartfield for the second value
		 * @private
		 */
		this._smartFieldTo = new SmartField(this._inputTo,
			false, // lazy init
			this._smartNameDictionary, // our dictionary
			_.bind(this._smartNameDictionary.displaySmartName, this._smartNameDictionary), // display in input field
			_.bind(this._smartNameDictionary.displaySmartNameSelectorSmartName, this._smartNameDictionary), // display in selector
			SmartNameEncoding.WORKFLOWEDITOR // encoding
		);

		this._smartFieldFrom.onValueChange(_.bind(function (value) {
			this._setValue(0, value);
		}, this));
		this._smartFieldTo.onValueChange(_.bind(function (value) {
			this._setValue(1, value);
		}, this));
		this._smartFieldFrom.setWidth("100%");
		this._smartFieldTo.setWidth("100%");

	};

	BetweenSmartNameRouterRule.prototype = Object.create(RouterRule.prototype);
	BetweenSmartNameRouterRule.prototype.constructor = BetweenSmartNameRouterRule;

	/**
	 * @override
	 */
	BetweenSmartNameRouterRule.prototype.copy = function () {
		return new BetweenSmartNameRouterRule(this._type, this._label, this._shortName, this._smartNameDictionary);
	};

	/**
	 * @override
	 */
	BetweenSmartNameRouterRule.prototype.getSaveableValues = function () {
		return [this._values[0], this._values[1]];
	};

	/**
	 * @override
	 */
	BetweenSmartNameRouterRule.prototype.setValues = function (values) {
		RouterRule.prototype.setValues.call(this, values);
		this._smartFieldFrom.setValue(this._values[0] || "");
		this._smartFieldTo.setValue(this._values[1] || "");

	};

	/**
	 * @override
	 */
	BetweenSmartNameRouterRule.prototype.getDisplayName = function () {
		if (!this._values[0]) {
			return this._shortName;
		}
		try {
			var fromText = this._getDisplayStringFromSmartField(this._smartFieldFrom);
			var toText = this._getDisplayStringFromSmartField(this._smartFieldTo);
			return fromText + " " + this._shortName + " " + toText;

		} catch (e) {
			return this._values[0];
		}
	};

	/**
	 * returns a display string from s smartname field
	 * @return {String}
	 */
	BetweenSmartNameRouterRule.prototype._getDisplayStringFromSmartField = function (smartfield) {
		var s = "";
		var smartNameDecoded = SmartNameEncoding.WORKFLOWEDITOR.decodeSmartNameString(smartfield.getValue());
		_.each(smartNameDecoded, function (smartNamePart) {
			if (smartNamePart.isTextPart()) {
				s += smartNamePart.getAsText();
			} else {
				s += this._smartNameDictionary.displaySmartNameSelectorSmartName(smartNamePart.getAsSmartName());
			}
		}, this);
		return s;

	};

	/**
	 * enable routerrule
	 */
	BetweenSmartNameRouterRule.prototype.enable = function () {
		this._smartFieldFrom.enable();
		this._smartFieldTo.enable();
	};

	/**
	 * disable routerrule
	 */
	BetweenSmartNameRouterRule.prototype.disable = function () {
		this._smartFieldFrom.disable();
		this._smartFieldTo.disable();
	};

	/**
	 * @override
	 */
	BetweenSmartNameRouterRule.prototype.destroy = function () {
		this._smartFieldFrom.destroy();
		this._smartFieldTo.destroy();
	};

	/**
	 * @override
	 */
	BetweenSmartNameRouterRule.prototype.resize = function () {
		this._smartFieldFrom.resize();
		this._smartFieldTo.resize();
	};

	return BetweenSmartNameRouterRule;

});