define(["underscore",
		"jquery",
		"component/smartnames/encoding/smartnameencoding",
		"./SmartNameRouterRule"
	],
	function (_,
		$,
		SmartNameEncoding,
		SmartNameRouterRule) {

		"use strict";

		/**
		 * A router rule in which you can select values from a select
		 *
		 * @class
		 * @exports SelectSmartNameRouterRule
		 * @augments SmartNameRouterRule
		 * @param {STring}                     type                
		 * @param {String}                     label               
		 * @param {String}                     shortName           
		 * @param {SmartNameDictionaryService} smartNameDictionary 
		 * @param {JQueryObject}               select              	- select used for the router rule
		 */
		var SelectSmartNameRouterRule = function (type, label, shortName, smartNameDictionary, select) {
			SmartNameRouterRule.call(this, type, label, shortName, smartNameDictionary, select);

			this._setValue(0, this._input.val());
		};

		SelectSmartNameRouterRule.prototype = Object.create(SmartNameRouterRule.prototype);
		SelectSmartNameRouterRule.prototype.constructor = SelectSmartNameRouterRule;

		/**
		 * @override
		 */
		SelectSmartNameRouterRule.prototype.copy = function () {
			return new SelectSmartNameRouterRule(this._type, this._label, this._shortName, this._smartNameDictionary, this._input.clone());
		};

		/**
		 * @override
		 */
		SelectSmartNameRouterRule.prototype.getDisplayName = function () {
			if (!this._values[0]) {
				return this._shortName;
			}
			var s = "";
			var smartNameDecoded = null;
			if (this._smartField) {
				// If smart field has been initialized, get the value to check for smart names
				smartNameDecoded = SmartNameEncoding.WORKFLOWEDITOR.decodeSmartNameString(this._smartField.getValue());
			}
			if ((smartNameDecoded && smartNameDecoded.length === 1 && smartNameDecoded[0].isTextPart()) || !smartNameDecoded) {
				// if there is only text, check if it's the value of one of the options of the select
				var value = (smartNameDecoded && smartNameDecoded[0].getAsText()) || this._values[0];
				var selectedLi = $(this._input.find("option").filter(function(i, elem){
					return $(elem).attr("value") === value;
				})[0]);
				if (selectedLi[0]) {
					return this._shortName + " " + selectedLi.text();
				} else {
					return this._shortName + " " + value;
				}
			}
			_.each(smartNameDecoded, function (smartNamePart) {
				if (smartNamePart.isTextPart()) {
					s += smartNamePart.getAsText();
				} else {
					s += this._smartNameDictionary.displaySmartNameSelectorSmartName(smartNamePart.getAsSmartName());
				}
			}, this);

			return this._shortName ? this._shortName + " " + s : s;

		};

		/**
		 * @override
		 * @param   {RouterRule} routerRule 
		 * @returns {Boolean}    
		 */
		SelectSmartNameRouterRule.prototype.canBeReplacedBy = function (routerRule) {
			if (!(routerRule instanceof SelectSmartNameRouterRule)) {
				return false;
			}
			if (!SmartNameRouterRule.prototype.canBeReplacedBy.call(this, routerRule)) {
				return false;
			}
			var myOptions = this.getOptions();
			var otherOptions = routerRule.getOptions();
			if (myOptions.length !== otherOptions.length) {
				return false;
			}
			for (var i = 0; i < myOptions.length; i++) {
				if (myOptions[i] !== otherOptions[i]) {
					return false;
				}
			}
			return true;
		};


		/**
		 * get the values of the options of this select
		 * @returns {Array.<String>} 
		 */
		SelectSmartNameRouterRule.prototype.getOptions = function () {
			var options = [];
			this._input.find("option").each(function () {
				options.push($(this).val());
			});
			return options;
		};

		/**
		 * @override
		 * Setting a value for a select can either be a smartname or an option of the select. If the select has the attribute data-allowcustom, the select can also 
		 * include custom data such as '|' etc to pipe different values or smartnames
		 */
		SelectSmartNameRouterRule.prototype.setValues = function (values) {
			if (!SmartNameEncoding.WORKFLOWEDITOR.containsSmartName(values[0]) && !this._input.attr("data-allowcustom")) {
				// Check if custom non smart names are allowed for this select
				var selectedLi = this._input.find("option[value='" + values[0] + "']");
				if (!selectedLi[0]) {
					values[0] = this._input.val();
				}
			}
			SmartNameRouterRule.prototype.setValues.call(this, values);
		};

		return SelectSmartNameRouterRule;
	});
