define([
	"component/smartnames/smartfield/smartfield",
	"component/smartnames/encoding/smartnameencoding",
	"component/smartnames/encoding/smartname",
	"../routerPanel/RouterRule",
	"jquery",
	"underscore"
], function (
	SmartField,
	SmartNameEncoding,
	SmartName,
	RouterRule,
	$,
	_) {

	"use strict";

	/**
	 * A RouterRule that enables choosing a value through a smartfield
	 * @param {String}                     type                
	 * @param {String}                     label               
	 * @param {String}                     emptyName            - the display label that will be used if the values are empty
	 * @param {SmartNameDictionaryService} smartNameDictionary 
	 * @param {JQueryObject}               input                - input used for the smartname
	 * @param {String}                     prefixName           - the display label that will be used to prefix the value if it is not empty
	 */
	var SmartNameRouterRule = function (type, label, emptyName, smartNameDictionary, input, prefixName) {
		RouterRule.call(this, type, label, emptyName);

		/**
		 * @member {JQueryObject} input field
		 * @private
		 */
		this._input = input || $('<input type="text">');

		this._prefixName = prefixName || "";

		/**
		 * @member {JQueryObject} input field div
		 * @private
		 */
		this._inputDiv = $("<div>").append(this._input);
		this._element.addClass("form-group").append(this._inputDiv);


		this._smartNameDictionary = smartNameDictionary;

		/**
		 * @member {SmartField} See init
		 * @private
		 */
		this._smartField = null;


	};

	SmartNameRouterRule.prototype = Object.create(RouterRule.prototype);
	SmartNameRouterRule.prototype.constructor = SmartNameRouterRule;

	/**
	 * @override
	 */
	SmartNameRouterRule.prototype.copy = function () {
		return new SmartNameRouterRule(this._type, this._label, this._shortName, this._smartNameDictionary, this._input.clone(), this._prefixName);
	};


	/**
	 * @override
	 */
	SmartNameRouterRule.prototype.setContext = function (context) {
		if (!this._smartField) {
			this._smartField = new SmartField(this._input,
				false, // lazy init
				this._smartNameDictionary, // our dictionary
				_.bind(this._smartNameDictionary.displaySmartName, this._smartNameDictionary), // display in input field
				_.bind(this._smartNameDictionary.displaySmartNameSelectorSmartName, this._smartNameDictionary), // display in selector
				SmartNameEncoding.WORKFLOWEDITOR // encoding
			);

			this._smartField.onValueChange(_.bind(function (value) {
				this._setValue(0, value);
			}, this));
			this._smartField.setWidth("100%");
		}
		this._smartField.setValue(this._values[0] || "");
		context.append(this._inputDiv);

	};

	/**
	 * @override
	 */
	SmartNameRouterRule.prototype.getSaveableValues = function () {
		return [this._values[0] || this._input.val()];
	};

	/**
	 * @override
	 */
	SmartNameRouterRule.prototype.getDisplayName = function () {
		if (!this._values[0]) {
			return this._shortName;
		}
		try {
			var s = "";
			var smartNameDecoded = SmartNameEncoding.WORKFLOWEDITOR.decodeSmartNameString(this._smartField.getValue());
			_.each(smartNameDecoded, function (smartNamePart) {
				if (smartNamePart.isTextPart()) {
					s += smartNamePart.getAsText();
				} else {
					s += this._smartNameDictionary.displaySmartNameSelectorSmartName(smartNamePart.getAsSmartName());
				}
			}, this);
			return this._prefixName ? this._prefixName + " " + s : s;
		} catch (e) {
			return this._values[0];
		}

	};

	/**
	 * enable routerrule
	 */
	SmartNameRouterRule.prototype.enable = function () {
		this._smartField.enable();
	};

	/**
	 * disable routerrule
	 */
	SmartNameRouterRule.prototype.disable = function () {
		this._smartField.disable();
	};

	SmartNameRouterRule.prototype.setValues = function (values) {
		RouterRule.prototype.setValues.call(this, values);
		if (this._smartField) {
			this._smartField.setValue(values[0]);
		}
	};

	/**
	 * @override
	 */
	SmartNameRouterRule.prototype.destroy = function () {
		if (this._smartField) {
			this._smartField.destroy();
		}
	};

	/**
	 * @override
	 */
	SmartNameRouterRule.prototype.resize = function () {
		if (this._smartField) {
			this._smartField.resize();
		}
	};

	return SmartNameRouterRule;

});