 define([
 		"WFEditorWCR/workflow/node/WebCenterNode",
 		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
 		"WFEditorWCR/workflow/pin/OutputPinOK",
 		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
 		"WFEditorWCR/workflow/panel/WebCenterPanelConstants",
 		"WFEditorWCR/workflow/panel/TaskSpecificationsParameter",
 		"tpl!WFEditorWCR/workflow/node/task/TaskNode",
 		"tpl!WFEditorWCR/workflow/node/task/TaskNodeResource",
 		"utils/webi18n!WFEditorWCR",
 		"WFEditorWCR/workflow/panel/WorkflowSmartNameConstants",
 		"underscore"
 	],
 	function (
 		WebCenterNode,
 		WebCenterNodeConstants,
 		OutputPinOK,
 		WebCenterPinConstants,
 		WebCenterPanelConstants,
 		TaskSpecificationsParameter,
 		TaskNodeXML,
 		TaskNodeResourceXML,
 		i18n,
 		WorkflowSmartNameConstants,
 		_
 	) {
 		"use strict";

 		/**
 		 * Tasknode represents the model for a task node in the WebCenter Workflow Editor.
 		 *
 		 * @exports TaskNode
 		 * @class
 		 * @augments WebCenterNode
 		 *
 		 * @param {Workflow} workflow - the workflow to which the node belongs
 		 * @param {String} id - node id
 		 * @param {String} icon - gives the url for fetching the icon
 		 * @param {String} [label] - the label of the node
 		 * @param {WebCenterTaskType} taskType - The task type of the node                                                                                
 		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
 		 */
 		var TaskNode = function (workflow, id, icon, label, taskType, webCenterAPI) {
 			/**
 			 * @member {WebCenterTaskType} the task type of the node
 			 * @private
 			 */
 			this._taskType = taskType;

 			/**
 			 * @member {$.Deferred} the promise for the task specifications
 			 * @private
 			 */
 			this._taskSpecificationsDeferred = webCenterAPI.getTaskSpecifications(this._taskType.getId());

 			WebCenterNode.call(this, workflow, id, WebCenterNodeConstants.Nodes.WCTASK, icon, label, webCenterAPI);
 		};

 		TaskNode.prototype = Object.create(WebCenterNode.prototype);
 		TaskNode.prototype.constructor = TaskNode;

 		/**
 		 * @Override
 		 */
 		TaskNode.prototype._initializePins = function () {
 			this._addInputPin();
 			this._addErrorPin(WebCenterPinConstants.Type.WorkflowConstructError);
 			// the custom output pins are set via WebCenterWorkflowFactory
 		};

 		/*
 		 * @Override
 		 */
 		TaskNode.prototype.getActionXML = function () {
 			return TaskNodeXML(this);
 		};

 		/**
 		 * gets the task type of this node
 		 * @returns {WebCenterTaskType} 
 		 */
 		TaskNode.prototype.getTaskType = function () {
 			return this._taskType;
 		};

 		/*
 		 * @Override
 		 */
 		TaskNode.prototype.setConfiguration = function (config) {
 			_.each(config, function (param) {
 				var assigneeRestriction;
 				switch (param.getName()) {
 				case WebCenterNodeConstants.WCTask.TYPE:
 					param.setValue(this._taskType.getName());
 					param.setDefaultValue(this._taskType.getName());
 					break;
 				case WebCenterNodeConstants.WCTask.NAME:
 					param.setDefaultValue(this._taskType.getName());
 					break;
 				case WebCenterNodeConstants.WCTask.DESCRIPTION:
 					param.setDefaultValue(this._taskType.getDescription());
 					break;
 				case WebCenterNodeConstants.WCTask.LEADDAYS:
 					param.setDefaultValue(this._taskType.getLeadDays());
 					break;
 				case WebCenterNodeConstants.WCTask.LEADTIME:
 					param.setDefaultValue(this._taskType.getLeadTime());
 					break;
 				case WebCenterNodeConstants.WCTask.ASSIGNEE:
 					assigneeRestriction = this._taskType.getRestrictedAssignee();
 					if (assigneeRestriction && assigneeRestriction.restriction === "roleOrGroup") {
 						this._webCenterAPI.getUsersGroupsRoles().done(function (u, g, r) {
 							if (g.has(assigneeRestriction.id)) {
 								param.setDefaultValue(WebCenterNodeConstants.WCTask.GROUPASSIGNMENT);
 							} else if (r.has(assigneeRestriction.id)) {
 								param.setDefaultValue(WebCenterNodeConstants.WCTask.ROLEASSIGNMENT);
 							}
 						});
 					}
 					break;
 				case WebCenterNodeConstants.WCTask.GROUPASSIGNEE:
 					assigneeRestriction = this._taskType.getRestrictedAssignee();
 					if (assigneeRestriction && assigneeRestriction.restriction === "roleOrGroup") {
 						this._webCenterAPI.getUsersGroupsRoles().done(function (u, g, r) {
 							if (g.has(assigneeRestriction.id)) {
 								param.setDefaultValue("grp." + assigneeRestriction.id);
 							}
 						});
 					}
 					break;
 				case WebCenterNodeConstants.WCTask.ROLEASSIGNEE:
 					assigneeRestriction = this._taskType.getRestrictedAssignee();
 					if (assigneeRestriction && assigneeRestriction.restriction === "roleOrGroup") {
 						this._webCenterAPI.getUsersGroupsRoles().done(function (u, g, r) {
 							if (r.has(assigneeRestriction.id)) {
 								param.setDefaultValue("grp." + assigneeRestriction.id);
 							}
 						});
 					}
 					break;
 				default:
 					break;
 				}
 			}, this);
 			this._taskSpecificationsDeferred.done(_.bind(function (specs) {
 				var taskSpecParam = _.find(config, function (param) {
 					return param.getName() === WebCenterPanelConstants.TASKSPECIFICATIONS_PARAMETERID;
 				}, this);

 				// intercept the taskSpecPAram webcenter parameter and turn it into a TaskSpecificationParameter
 				if (!(taskSpecParam instanceof TaskSpecificationsParameter)) {
 					// If the param is not of type taskspecificationsparameter, convert it
 					config.splice(config.indexOf(taskSpecParam), 1);
 					config.push(new TaskSpecificationsParameter(WebCenterPanelConstants.TASKSPECIFICATIONS_PARAMETERID, specs));
 				}
 				WebCenterNode.prototype.setConfiguration.call(this, config);
 			}, this));
 		};

 		/**
 		 * Should the given parameter be saved
 		 * @param   {WebCenterParameter} parameter
 		 * @returns {Boolean} True to save it, false for not to
 		 */
 		TaskNode.prototype.shouldSaveParameter = function (parameter) {
 			return parameter.getName() !== WebCenterNodeConstants.WCTask.TYPE;
 		};

 		/**
 		 * @Override
 		 */
 		TaskNode.prototype.setNodeXML = function (nodeEl) {
 			this._taskSpecificationsDeferred.done(_.bind(function () {
 				WebCenterNode.prototype.setNodeXML.call(this, nodeEl);
 			}, this));
 			this._taskTypeID = nodeEl.querySelector("action wctask_type").getAttribute("id");
 		};

 		/**
 		 * @override
 		 */
 		TaskNode.prototype.getResourceXML = function () {
 			return TaskNodeResourceXML(this);
 		};

 		/**
 		 *	returns all output pins with type OutOk. These are used to save in the resources xml file
 		 * @returns {Map.<String, WebCenterPin>}
 		 */
 		TaskNode.prototype.getOutputOkPins = function () {
 			return _.filter(this.getPins(), function (pin) {
 				return pin.getPinType() === WebCenterPinConstants.Type.OutOK;
 			});
 		};

 		/**
 		 * @override
 		 */
 		TaskNode.prototype._addTaskStatusesToDropdown = function (smartNameDropdownPanel) {
 			this._webCenterAPI.getWorkflowTaskStatusList(this._taskType.getId()).done(_.bind(function (statusesMap) {
 				this._addStatusesToDropdown(statusesMap, smartNameDropdownPanel);
 			}, this));
 		};

 		/**
 		 * @override
 		 */
 		TaskNode.prototype._getNodeSpecificSubPanel = function (deferred, type, paramId, nodeXML, customSmartNames) {
 			if (type === WebCenterPanelConstants.TASKSPECIFICATIONS) {
 				requirejs(["WFEditorWCR/workflow/panel/TaskSpecificationsPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService", "WFEditorWCR/workflow/panel/DateSmartNameDictionaryService"], _.bind(function (TaskSpecificationsPanel, WorkflowSmartNameDictionaryService, DateSmartNameDictionaryService) {
 					var nameLabel = WebCenterNodeConstants.SetTaskSpecification.SPECNAME;
 					var valueLabel = WebCenterNodeConstants.SetTaskSpecification.SPECVALUE;

 					var taskTypeID = this.getWorkflow().getTaskTypeID();
 					var panel = new TaskSpecificationsPanel(paramId, i18n.t(nameLabel), i18n.t(valueLabel), new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), new DateSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, this._taskSpecificationsDeferred);
 					panel.getContext().addClass('parameter-panel form-horizontal form-compact');
 					deferred.resolve(panel);
 				}, this));
 				return true;

 			}
 			return false;
 		};

 		/**
 		 * @override
 		 * intercept task specification parameters
 		 */
 		TaskNode.prototype.getConfigurationParameterByName = function (name) {
 			var taskSpecParam = WebCenterNode.prototype.getConfigurationParameterByName.call(this, WebCenterPanelConstants.TASKSPECIFICATIONS_PARAMETERID);
 			if (taskSpecParam && taskSpecParam.isSpecification(name)) {
 				// if it is a specification parameter return the wrapper parameter
 				return taskSpecParam;
 			} else {
 				return WebCenterNode.prototype.getConfigurationParameterByName.call(this, name);
 			}
 		};

 		/**
 		 * @override
 		 */
 		TaskNode.prototype._addRolesToDropdown = function (dropdown, r) {
 			// If there is a role restriction, only add the specified role to the dropdown and disable it
 			var restrictedAssignee = this._taskType.getRestrictedAssignee();
 			if (restrictedAssignee && restrictedAssignee.restriction === "roleOrGroup" && r.has(restrictedAssignee.id)) {
 				var role = r.get(restrictedAssignee.id);
 				dropdown.setOptions([{
 					text: role.getName(),
 					// Roles in this context also need the group prefix due to historic reasons
 					value: WorkflowSmartNameConstants.TASKROLES.PREFIX + role.getId()
 				}]);
 				dropdown.disable();
 				dropdown.getContext().append($("<label/>").addClass("col-sm-12 col-lg-3")).append($("<span/>").addClass("col-sm-12 col-lg-9").text(i18n.t("task.assignee.restriction.role")));
 			} else {
 				WebCenterNode.prototype._addRolesToDropdown.call(this, dropdown, r);

 			}
 		};

 		/**
 		 * @override
 		 */
 		TaskNode.prototype._addGroupsToDropdown = function (dropdown, g) {
 			// If there is a group restriction, only add the specified group to the dropdown and disable it
 			var restrictedAssignee = this._taskType.getRestrictedAssignee();
 			if (restrictedAssignee && restrictedAssignee.restriction === "roleOrGroup" && g.has(restrictedAssignee.id)) {
 				var group = g.get(restrictedAssignee.id);
 				dropdown.setOptions([{
 					text: group.getName(),
 					// Roles in this context also need the group prefix due to historic reasons
 					value: WorkflowSmartNameConstants.TASKGROUPS.PREFIX + group.getId()
 				}]);
 				dropdown.disable();
 				dropdown.getContext().append($("<label/>").addClass("col-sm-12 col-lg-3")).append($("<span/>").addClass("col-sm-12 col-lg-9").text(i18n.t("task.assignee.restriction.group")));
 			} else {
 				WebCenterNode.prototype._addGroupsToDropdown.call(this, dropdown, g);
 			}

 		};

 		/**
 		 * @override
 		 */
 		TaskNode.prototype._panelAdded = function (panel) {
 			// If the user assignee option is added and there is a noUser restriction on the assignee, disable the option
 			var restrictedAssignee = this._taskType.getRestrictedAssignee();
 			if (restrictedAssignee &&
 				restrictedAssignee.restriction === "noUser" &&
 				_.find(panel.getOwnParameterWishList(), function (param) {
 					return param === WebCenterNodeConstants.WCTask.USERASSIGNEE;
 				})) {
 				panel.disable();
 				panel.getContext().append($("<span/>").text(i18n.t("task.assignee.restriction.noUser")).addClass("panel-information-text"));
 			}
 		};

 		/** 
 		 * @override
 		 */
 		TaskNode.prototype.getStateObjectForCopy = function () {
 			var stateObj = WebCenterNode.prototype.getStateObjectForCopy.call(this);
 			stateObj.taskTypeId = this._taskType.getId();
 			return stateObj;
 		};



 		return TaskNode;
 	});