define(["WFEditorWCR/workflow/panel/WebCenterCompositePanelComponent",
		"WFEditorWCR/workflow/panel/WebCenterPanelConstants",
		"jquery",
		"underscore"
	],
	function (WebCenterCompositePanelComponent, WebCenterPanelConstants, $, _) {

		"use strict";

		/**
		 * @class
		 * @exports {BooleanPanel}
		 * @augments WebCenterCompositePanelComponent
		 * 
		 * A Panel Component that will generate a check box
		 * 
		 * @param {String} parameterId
		 * @param {String} name - label for the panel
		 * @param {String} subParameterState - one of "visible" , "invisible" or "enabled". 
		 *                                   Visible shows the subPanels on check
		 *                                   Invisible hides the subPanels on check
		 *                                   enabled enables the the subPanels on check
		 */
		var BooleanPanel = function (parameterId, name, subParameterState) {
			WebCenterCompositePanelComponent.call(this, [parameterId]);

			/**
			 * @member {String} the id of the parameter used for this boolean panel
			 * @private
			 */
			this._parameterId = parameterId;

			/**
			 * @member {JQueryObject} checkbox
			 * @private
			 */
			this._checkbox = $('<input type="checkbox"/>');

			/**
			 * @member {JqueryObject} label for the checkbox
			 * @private
			 */
			this._label = $('<label/>');
			this._label.append(this._checkbox).append(" " + name);

			/**
			 * @member {JQueryObject} div where all child elements will be added to
			 * @private
			 */
			this._childDiv = $('<div/>');


			/**
			 * @member {Boolean} if true the child panels will be hidden if the checkbox is checked.
			 * if false the children will be hidden if the checkbox isn't checked
			 * @private
			 */
			this._subParameterState = subParameterState;

			if (this._subParameterState === WebCenterPanelConstants.SUBPARAMETERSTATE.ENABLED ||
				this._subParameterState === WebCenterPanelConstants.SUBPARAMETERSTATE.DISABLED) {
				this._childDiv.addClass('sibling-panel');
			} else if (this._subParameterState === WebCenterPanelConstants.SUBPARAMETERSTATE.INVISIBLE) {
				this._childDiv.addClass('sibling-panel sibling-extra-padding-top');
			} else {
				this._childDiv.addClass('child-panel col-sm-12 col-lg-12');
			}
			this._childDiv.addClass("clearfix");

			this._checkbox.on('change', _.bind(function () {
				// Set the value of the parameter on checkbox change
				var param = this.getParameterByName(this._parameterId);

				if (this._checkbox.prop('checked')) {
					param.setValue("true");
				} else {
					param.setValue("false");

				}
				this._showOrHideSubPanel();

			}, this));


			this._element.append($("<label>").addClass("col-lg-3 offsetLabel")).append(
				$("<div class='clearfix form-group col-lg-9 col-sm-12'/>")
				.append($('<div class="checkbox"/>').append(this._label))
				.append(this._childDiv)
			);
			this.hasOffsetLabel = true;
			this._element.addClass("form-group boolean-panel");
		};

		BooleanPanel.prototype = Object.create(WebCenterCompositePanelComponent.prototype);
		BooleanPanel.prototype.constructor = BooleanPanel;

		/**
		 * @override
		 */
		BooleanPanel.prototype.add = function (abstractPanelComponent) {
			this._subPanels.push(abstractPanelComponent);
			this._childDiv.append(abstractPanelComponent.getContext());
			this._fixLabels(this._childDiv, this._subPanels);
		};

		/**
		 * @override
		 */
		BooleanPanel.prototype.setParameters = function (parameters) {
			WebCenterCompositePanelComponent.prototype.setParameters.call(this, parameters);
			var param = this.getParameterByName(this._parameterId);
			if (param.getValue() === "true") {
				this._checkbox.prop('checked', true);
			} else {
				this._checkbox.prop('checked', false);
			}
			this._showOrHideSubPanel();

		};

		/**
		 * @override - if the boolean isn't applicable it should be set to its default value
		 */
		BooleanPanel.prototype.setNotApplicable = function () {
			var param = this.getParameterByName(this._parameterId);
			if (param) {
				param.setValue(param.getDefaultValue() || "false");
				this._checkbox.prop('checked', param.getValue() === "true");
			}
			_.each(this._subPanels, function (panel) {
				panel.setNotApplicable();
			});
		};

		/**
		 * @override
		 */
		BooleanPanel.prototype.show = function () {
			WebCenterCompositePanelComponent.prototype.show.call(this);
			if (this.getParameterByName(this._parameterId)) {
				this._showOrHideSubPanel();
			}
		};

		/**
		 * shows or hides the sub panel depending on the value of the parameter and the value of this._subParameterState
		 */
		BooleanPanel.prototype._showOrHideSubPanel = function () {
			var param = this.getParameterByName(this._parameterId);
			if ((param.getValue() === "true" && this._subParameterState === WebCenterPanelConstants.SUBPARAMETERSTATE.INVISIBLE) || (param.getValue() === "false" && this._subParameterState === WebCenterPanelConstants.SUBPARAMETERSTATE.VISIBLE)) {
				_.each(this._subPanels, function (panel) {
					// let all subpanels know they are being hidden and that they are not applicable anymore
					panel.hide();
					panel.setNotApplicable();
				});
				this._childDiv.hide();
			} else {

				this._childDiv.show();
				_.each(this._subPanels, function (panel) {
					if ((this._subParameterState === WebCenterPanelConstants.SUBPARAMETERSTATE.ENABLED && param.getValue() === "false") ||
						(this._subParameterState === WebCenterPanelConstants.SUBPARAMETERSTATE.DISABLED && param.getValue() === "true") || !this._isEnabled) {
						panel.disable();
					} else {
						panel.enable();
					}

					// let all subpanels know they are being shown and that they are applicable
					panel.show();
					panel.setApplicable();
				}, this);
			}

			this._fitContent();
		};

		/**
		 * @override
		 */
		BooleanPanel.prototype.disable = function () {
			WebCenterCompositePanelComponent.prototype.disable.call(this);
			this._checkbox.attr("disabled", true);
		};



		return BooleanPanel;

	}
);
