define(["WFEditor/workflow/panel/AbstractPanelComponent",
		"WFEditor/components/InputField",
		"WFEditor/workflow/panel/Parameter",
		"tinyMCE",
		"underscore",
		"jquery",
		"utils/webi18n!WFEditorWCR"
	],
	function (
		AbstractPanelComponent,
		InputField,
		Parameter,
		tinyMCE,
		_,
		$,
		i18n) {

		"use strict";

		/**
		 * Languages supported by TinyMCE
		 */
		var SUPPORTED_LANGUAGES = ["br", "cn", "cz", "de", "en", "es", "fr", "hu", "hr", "it", "jp", "nl", "pl", "ru", "th", "tr", "tw"];
		var DEFAULT_LANGUAGE = "en";

		/**
		 * @class
		 * @exports HTMLPanel
		 * @augments AbstractPanelComponent
		 * 
		 * A panel component used for HTML editing
		 * @param {String} parameterId - id of the parameter for this panel
		 * @param {String} label       - label to be displayed
		 * @param {WorkflowSmartNameDictionaryService} smartNameDictionaryService - the smartname dictionary for smartnames in HTML
		 */
		var HTMLPanel = function (parameterId, label, smartNameDictionaryService, webCenterAPI) {
			AbstractPanelComponent.call(this, [parameterId]);

			/**
			 * @member {WebCenterAPI} webcenterAPI
			 * @private
			 */
			this._webCenterAPI = webCenterAPI;

			/**
			 * @member {TinyMCEObject} rich text editor
			 * @private
			 */
			this._editor = null;

			/**
			 * @member {SmartNameDictionaryService} dictionary service for smartname popup
			 * @private
			 */
			this._dictionaryService = smartNameDictionaryService;

			// Add elements to the DOM
			var textArea = $('<textarea />').addClass('form-control WFHtmlEditor');
			this._element.addClass("WFRichTextContainer col-lg-12 col-sm-12").append(textArea);

		};

		HTMLPanel.prototype = Object.create(AbstractPanelComponent.prototype);
		HTMLPanel.prototype.constructor = HTMLPanel;

		/**
		 * this method is called after the panel is added to the dom
		 * @override
		 */
		HTMLPanel.prototype.init = function () {
			var editorLang = DEFAULT_LANGUAGE;
			var userLang = this._webCenterAPI.getWebCenterConfig().userLang;
			if (SUPPORTED_LANGUAGES.indexOf(userLang) > -1) {
				editorLang = userLang;
			}

			// init editor
			tinyMCE.init({
				selector: "textarea.WFHtmlEditor",

				skin: 'lightgrayflat',

				language: editorLang,
				plugins: "smartname emoticons textcolor code pagebreak fullscreen print searchreplace charmap insertdatetime table hr link image fullpage",
				toolbar1: "formatselect fontselect fontsizeselect | charmap insertdatetime  | hr removeformat | " + "bold italic underline strikethrough | subscript superscript | forecolor backcolor | table | link unlink image | searchreplace | " + "undo redo | bullist numlist | alignleft aligncenter alignright alignjustify | outdent indent blockquote | code pagebreak smartname",
				fullpage_default_doctype: "",
				menubar: false,
				statusbar: false,
				convert_urls: false,
				paste_retain_style_properties: "margin, padding, width, height, font-size, font-weight, font-family, color, text-align, ul, ol, li, text-decoration, border, background, float, display",

				// pass the necessary information and callbacks to th editor
				setup: _.bind(function (ed) {
					ed.dictionaryService = this._dictionaryService;
					ed.on('init', _.bind(function () {
						this._editor = ed;
						if (!this._isEnabled) {
							this._editor.getBody().setAttribute('contenteditable', false);
						}
						if (this._parameters && this._parameters[0]) {
							this._editor.setContent(this._parameters[0].getValue());
						}
						_.defer(_.bind(this.resize, this));
					}, this));
					ed.on('change', _.bind(function () {
						var content = this._editor.getContent();
						if (content.toLowerCase().indexOf("<html>") === -1) {
							content = '<html>' + content + '</html>';
						}
						if (this._parameters[0]) {
							this._parameters[0].setValue(content);
						}
					}, this));
				}, this),
				readonly: this._isEnabled ? 0 : 1
			});
		};

		/**
		 * When the parameter is set, input field should be changed
		 * @param {Array.<Parameter>} parameters
		 * @override
		 */
		HTMLPanel.prototype.setParameters = function (parameters) {
			AbstractPanelComponent.prototype.setParameters.call(this, parameters);
			if (this._editor) {
				this._editor.setContent(this._parameters[0].getValue());
			}
		};

		/**
		 * Destroys the panel component
		 * @override
		 */
		HTMLPanel.prototype.destroy = function () {
			if (this._editor) {
				this._editor.destroy();
			}
		};

		/**
		 * handle resize of the panel
		 * @override
		 */
		HTMLPanel.prototype.resize = function () {
			if (this._editor) {
				// take up a maximum amount of space
				var popupContent = this._element.closest(".wfeditor-popup-content");
				var popupOffset = popupContent.offset();
				var popupHeight = popupContent.innerHeight();
				// subtract all other fields from the popup
				var children = this._element.closest(".parameter-panel").children().not(".WFRichTextContainer");
				var panelOffset = this._element.closest(".parameter-panel").offset();

				// If there are tabs, add them to the outheight
				var startHeight = panelOffset.top - popupOffset.top;
				children.each(function () {
					startHeight += $(this).outerHeight(true);
				});

				// first set width, so height of toolbar is defined
				// then resize again and subtract toolbar
				var height = popupHeight - startHeight - 15;
				this._editor.theme.resizeTo(this._element.width(), height);
				this._element.height(height);

				height -= $(".mce-toolbar-grp", this._element).outerHeight(true);
				this._editor.theme.resizeTo(this._element.width(), height);
			}
		};

		/**
		 * @override
		 */
		HTMLPanel.prototype.show = function () {
			AbstractPanelComponent.prototype.show.call(this);
			this.resize();
		};


		/**
		 * @override
		 */
		HTMLPanel.prototype.disable = function () {
			AbstractPanelComponent.prototype.disable.call(this);
			if (this._editor) {
				// This disables the textarea of the editor, however the buttons are still active.
				// This isn't a problem for now as the panel gets disabled before it is initialized, therefor we can set the readonly property
				// when the editor is initialized, if for some reason the editor should be switched to read only at run time this should be changed
				this._editor.getBody().setAttribute('contenteditable', false);
			}
		};

		/**
		 * @override
		 */
		HTMLPanel.prototype.getPreferredWidth = function () {
			return 540;
		};

		/**
		 * @override
		 */
		HTMLPanel.prototype.beforeRemoveFromDom = function () {
			this._editorId = this._editor.id;
			tinyMCE.EditorManager.execCommand("mceRemoveEditor", false, this._editorId);
		};

		/**
		 * @override
		 */
		HTMLPanel.prototype.afterAddToDom = function () {
			tinyMCE.EditorManager.execCommand("mceAddEditor", false, this._editorId);
		};


		return HTMLPanel;
	});