define(["WFEditorWCR/workflow/panel/MultiParametersPanel",
		"WFEditorWCR/workflow/panel/MapPromiseUniqueDataProvider",
		"component/sdrop/SDrop",
		"underscore",
		"jquery"
	],
	function (
		MultiParametersPanel,
		MapPromiseUniqueDataProvider,
		SDrop,
		_,
		$) {

		"use strict";

		/**
		 * @class
		 * @exports MultiAttributePanel
		 * @augments MultiParametersPanel
		 * 
		 * a panel component used for display of multiple attributes with their values
		 * @param {String} parameterId 
		 * 							- id of the parameter for this panel
		 * @param {String} label       
		 * 							- label to be displayed
		 * @param {WorkflowSmartNameDictionaryService} smartNameDictionaryService 
		 * 							- the smartname dictionary for smartnames in value fields
		 * @param {WorkflowSmartNameDictionaryService} smartNameDictionaryService 
		 * 							- the smartname dictionary for date attributes
		 * @param {WebCenterAPI} webCenterAPI	
		 * 							- webcenter inferface
		 * @param {Boolean} isWrapped 
		 * 							- if false, multiple parameters are passed, if true, one parameter with subparameters is passed
		 * @param {Promise} restriction	              
		 *                                            - promise object that will resolve to entries to be shown in a static sDrop
		 */
		var MultiAttributePanel = function (parameterId, nameLabel, valueLabel, smartNameDictionaryService, dateSmartNameDictionaryService, webCenterAPI, isWrapped, restriction) {
			MultiParametersPanel.call(this, parameterId, nameLabel, valueLabel, smartNameDictionaryService, isWrapped);

			/**
			 * @member {Array.<sDrop>} listObj
			 * @private
			 * @override
			 */
			this._nameFields = [];

			/**
			 * @member {Promise} restriction	
			 * 						- promise object that will resolve to entries to be shown in a static sDrop
			 * @private
			 */
			this._restriction = restriction;

			/**
			 * @member {WebCenterAPI}
			 * @private
			 */
			this._webCenterAPI = webCenterAPI;

			/**
			 * @member {WorkflowSmartNameDictionaryService} - separate smartNameDictionaryService for date attributes
			 * @private
			 */
			this._dateSmartNameDictionaryService = dateSmartNameDictionaryService;

			/**
			 * @member {MapPromiseUniqueDataProvider} - dataprovider to be shared over all entries in the panel
			 * @private
			 */
			this._sharedDateProvider = null;
		};

		MultiAttributePanel.prototype = Object.create(MultiParametersPanel.prototype);
		MultiAttributePanel.prototype.constructor = MultiAttributePanel;

		/**
		 * initialize name field
		 * @override
		 */
		MultiAttributePanel.prototype.initNameField = function (param, ctx) {
			// sDrop if multiattribute field
			var nameField = $('<select />');
			ctx.append($('<div />').append(nameField));

			// keep track of whether we need a select for the current input or not
			var restriction = null;
			var usedCustomDictionary = null;

			var onLoaded = _.bind(function () {
				if (!sDrop) {
					return; // an event is triggered when sDrop is destroyed
				}

				var val = nameField.val();
				param.setName(val);
				var selected = sDrop.dataProvider.getById(val);
				if (selected && selected.item) {
					var attr = selected.item;
					var customInput, customDictionary;
					var rebuildField = false;

					// check custom dictionary
					if (attr.getType().isDateTime()) {
						//change smartname library
						customDictionary = this._dateSmartNameDictionaryService;
					}
					if (customDictionary !== usedCustomDictionary) {
						rebuildField = true;
					}
					usedCustomDictionary = customDictionary;

					if (attr.getRestrictedSet()) {
						var rsName = attr.getRestrictedSet().getName();
						if (rsName !== restriction) {
							// change input field
							customInput = $('<select data-allowcustom="1" data-delimiter=" | " />');
							customInput.append($("<option />").val(""));
							_.each(attr.getRestrictedSet().getValues(), function (rsValue) {
								customInput.append($("<option />").val(rsValue.getValue()).html(rsValue.getValue()));
							});
							rebuildField = true;
							restriction = rsName;
						}
					} else if (restriction) {
						// we already have a select and want to recover the normal fields 
						customInput = $('<input type="text" />');
						rebuildField = true;
						restriction = undefined;
					}

					// rebuild smartname fields if needed
					if (rebuildField) {
						// rebuild fields
						var index = this._nameFields.indexOf(sDrop);
						this._valueFields[index].destroy();
						var normalInput = this._valueFields[index].getOriginalElement();
						if (!customInput) {
							customInput = normalInput;
						} else {
							normalInput.replaceWith(customInput);
						}
						var newSmartName = this.initValueField(param, ctx, customInput, customDictionary);
						this._valueFields[index] = newSmartName;
						// if dropdown doesn't contain current non-smart value, nothing will be selected
						// make sure that smartname and parameter are in sync
						param.setValue(newSmartName.getValue());
					}
				}
			}, this);

			// need to defer for initialization, because when data already exists, sDrop seems to load synchronously and 
			// load could be triggered before sDrop object is returned
			var deferredLoaded = function () {
				_.defer(onLoaded);
			};

			// make attribute SDrop
			var options = {
				onInitialized: deferredLoaded,
				onChangedByUser: onLoaded,
				autoload: false,
				autoloadOnFirstUsage: true,
				manageStyleAndWidth: false,
				initializeDataProvider: false
			};
			if (param.getName()) {
				options.initialValue = param.getName();
			}

			if (!this._sharedDateProvider) {
				this._sharedDateProvider = new MapPromiseUniqueDataProvider({
					'promise': this._restriction || this._webCenterAPI.getAttributes()
				});
			}

			var sDrop = new SDrop(nameField, "", this._sharedDateProvider, options);
			if (!this._isEnabled) {
				sDrop.setDisabled(true);
			}

			return sDrop;
		};

		/**
		 * @Override
		 */
		MultiAttributePanel.prototype.destroy = function () {
			MultiParametersPanel.prototype.destroy.call(this);

			_.each(this._nameFields, function (field) {
				field.destroy();
			});
		};

		/**
		 * @override
		 */
		MultiAttributePanel.prototype.disable = function () {
			MultiParametersPanel.prototype.disable.call(this);
			this._element.find(".sdrop").addClass("disabled");
			_.each(this._nameFields, function (sdrop) {
				sdrop.setDisabled(true);
			});
		};


		/**
		 * close any opened SDrop when moving/resizing
		 * @Override
		 */
		MultiAttributePanel.prototype.startInteractive = function () {
			_.each(this._nameFields, function (sdrop) {
				sdrop.disable(null, true);
			});
		};

		/**
		 * Entry is removed, now clean up components
		 * @override
		 * param {Number} - index of row where items are removed
		 */
		MultiAttributePanel.prototype.destroyComponentsOnEntry = function (index) {
			MultiParametersPanel.prototype.destroyComponentsOnEntry.call(this, index);
			this._nameFields[index].destroy();
		};

		return MultiAttributePanel;
	});