define(["WFEditor/workflow/panel/AbstractPanelComponent",
		"WFEditorWCR/workflow/panel/MultiParametersPanel",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"WFEditor/workflow/panel/PanelConstants",
		"underscore",
		"jquery",
		"component/smartnames/smartfield/smartfield",
		"WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService",
		"WFEditorWCR/workflow/panel/SmartnameInputField",
		"component/smartnames/encoding/smartnameencoding",
		"component/sdrop/dataproviders/MapPromiseDataProvider",
		"component/sdrop/SDrop",
		"WFEditorWCR/workflow/panel/MapPromiseUniqueDataProvider",
		"utils/Map"
	],
	function (
		AbstractPanelComponent,
		MultiParametersPanel,
		WebCenterParameter,
		PanelConstants,
		_,
		$,
		SmartField,
		WorkflowSmartNameDictionaryService,
		SmartnameInputField,
		SmartNameEncoding,
		MapPromiseDataProvider,
		SDrop,
		MapPromiseUniqueDataProvider,
		Map) {

		"use strict";

		/**
		 * @class
		 * @exports MultiDynamicContentImportPanel
		 * @augments MultiParametersPanel
		 * 
		 * a panel component used for display of dynamic content import mapping
		 * @param {String} parameterId - id of the parameter for this panel
		 * @param {String} elementTypeLabel
		 * @param {String} docRefAttrLabelLabel
		 * @param {String} docTempLabel
		 * @param {WorkflowSmartNameDictionaryService} smartNameDictionaryService 
		 * 		  - the smartname dictionary for smartnames in document template fields
		 * @param {Boolean} isWrapped - if false, multiple parameters are passed, if true, one parameter with subparameters is passed
		 * @param {WebCenterAPI} webCenterAPI
		 */
		var MultiDynamicContentImportPanel = function (parameterId, elementTypeLabel, docRefAttrLabelLabel, docTempLabel, smartNameDictionaryService, isWrapped, webCenterAPI) {
			MultiParametersPanel.call(this, parameterId, elementTypeLabel, docRefAttrLabelLabel, smartNameDictionaryService, isWrapped);
			/**
			 * @member {JQueryObject} the div this panel will render in
			 * @private
			 */
			this._element = $('<div/>');

			this._element.addClass("MultiParametersPanel MultiColumnPanel");

			/**
			 * @member {WebCenterAPI}
			 * @private
			 */
			this._webCenterAPI = webCenterAPI;

			/**
			 * @member {MapPromiseUniqueDataProvider} - dataprovider to be shared over all entries in the panel
			 * @private
			 */
			this._sharedElementTypeDataProvider = null;

			/**
			 * @member {JQueryObject} the context this panel will render in
			 * @private
			 */
			this._context = $('<div/>');
			this._context.append($("<div />").addClass("clearfix header").append($("<div />").addClass("has-3-cols offsetRightDeleteButton")
				.append($("<div />").addClass("elementTypeCol").append($("<label/>").append(elementTypeLabel)))
				.append($("<div />").addClass("docRefAttrCol").append($("<label/>").append(docRefAttrLabelLabel)))
				.append($("<div />").addClass("docTempCol").append($("<label/>").append(docTempLabel)))));

			/**
			 * @member {WorkflowSmartNameDictionaryService} - smartNameDictionaryService for document template field
			 * @private
			 */
			this._smartNameDictionaryService = smartNameDictionaryService;

			/**
			 * @member {Array.<jQueryObj>} listObj for holding import configuration objects
			 * @private
			 */
			this._importConfigArr = [];

			// add list and additional button for new entries to the element
			this._element.append(this._context);

			this._btn = $('<a />').addClass("AddBtn")
				.attr("disabled", !this._isEnabled).addClass("btn btn-sm btn-primary")
				.append($('<i />').addClass("icon icon-WCR_Actions-Add icon-small"))
				.click(_.bind(function () {
					if (this._isEnabled) {
						this.addEntry();
					}
				}, this));

			this._element.append(this._btn);
		};

		MultiDynamicContentImportPanel.prototype = Object.create(MultiParametersPanel.prototype);
		MultiDynamicContentImportPanel.prototype.constructor = MultiDynamicContentImportPanel;

		/**
		 * Add a new entry of input/smartname fields to the list
		 * @param {Parameter} [param] - parameter to put in the entry; if undefined, an empty entry is created
		 * @param {Boolean} [noFitContent] - true means do not request the container element to fit the panel
		 * @override
		 */
		MultiDynamicContentImportPanel.prototype.addEntry = function (param, noFitContent) {
			if (!(param instanceof WebCenterParameter)) {
				// if parameter does not exist(empty entry), create a new one
				//Parameter to hold value for element type
				//Sub parameter holds values for Document Reference Attribute and Doc Template 
				var subParamArr = [];
				subParamArr.push(new WebCenterParameter("docrefattr", "", ""));
				subParamArr.push(new WebCenterParameter("doctemp", "", ""));
				param = new WebCenterParameter("", "", "", subParamArr);

				this._subParameters.push(param);
			}

			var subParam = param.getSubparameters();
			// create entry
			var entryBlock = $("<div/>").addClass("block clearfix");
			var entry = $("<div/>").addClass("has-3-cols offsetRightDeleteButton");
			entryBlock.append(entry);

			// initialize fields
			var elementTypeField = this.initElementTypeField(param, entry);
			var docRefAttrField = this.initDocRefAttrField(subParam[0], entry);
			var docTemplField = this.initDocTemplField(subParam[1], entry);

			var configObj = {
				"elementType": elementTypeField,
				"docRefAttr": docRefAttrField,
				"docTempl": docTemplField
			};

			this._importConfigArr.push(configObj);

			var deleteBtn = $('<a />').attr("disabled", !this._isEnabled).addClass("btn btn-default btn-sm")
				.append($('<i />').addClass("icon icon-WCR_Actions-Delete12 icon-normal"))
				.click(_.bind(function (event, elem) {
					if (this._isEnabled) {
						entryBlock.remove();

						// handle remove fields in a row
						var index = this._importConfigArr.indexOf(configObj);
						if (index > -1) {
							this.destroyComponentsOnEntry(index);
							this._importConfigArr.splice(index, 1);
						}

						// remove subparameter
						index = this._subParameters.indexOf(param);
						if (index > -1) {
							this._subParameters.splice(index, 1);
						}
					}

					this._fitContent();

				}, this));

			this._context.append(entryBlock.append($('<div class="deleteBtn" />').append(deleteBtn)));

			// show and hide button on hover (only show if enabled)
			deleteBtn.hide();
			entryBlock.hover(_.bind(function () {
				if (this._isEnabled) {
					deleteBtn.show();
				}
			}, this), function () {
				deleteBtn.hide();
			});

			if (!noFitContent) {
				this.resize();
				this._fitContent();
			}
		};

		/**
		 * initialize Element Type field
		 * @param {Parameter} param - parameter for which field is created
		 * @param {JQueryObj} ctx - context to which field should be appended
		 * @returns {JQueryObject} - sDrop
		 */
		MultiDynamicContentImportPanel.prototype.initElementTypeField = function (param, ctx) {

			// sDrop of element type list
			var elementTypeField = $('<select />');
			ctx.append($('<div />').append(elementTypeField));

			// make attribute SDrop
			var options = {
				onChangedByUser: function () {
					param.setName(elementTypeField.val());
				},
				autoload: false,
				autoloadOnFirstUsage: true,
				manageStyleAndWidth: false,
				initializeDataProvider: false
			};
			if (param && param.getName()) {
				options.initialValue = param.getName();
			}

			if (!this._sharedElementTypeDataProvider) {
				this._sharedElementTypeDataProvider = new MapPromiseUniqueDataProvider({
					'promise': this._webCenterAPI.getTextContentElementTypeList()
				});
			}

			var sDrop = new SDrop(elementTypeField, "", this._sharedElementTypeDataProvider, options);
			if (!this._isEnabled) {
				sDrop.setDisabled(true);
			}

			return sDrop;
		};

		/**
		 * initialize document reference attribute field
		 * @param {Parameter} param - parameter for which field is created
		 * @param {JQueryObj} ctx - context to which field should be appended
		 * @returns {JQueryObject} - sDrop
		 */
		MultiDynamicContentImportPanel.prototype.initDocRefAttrField = function (param, ctx) {
			// sDrop of document reference attributes
			var docRefAttrField = $('<select />');
			ctx.append($('<div />').append(docRefAttrField));

			// make attribute SDrop
			var options = {
				onChangedByUser: function () {
					if (param) {
						param.setValue(docRefAttrField.val());
					}
				},
				autoload: false,
				autoloadOnFirstUsage: true,
				manageStyleAndWidth: false
			};
			if (param && param.getValue()) {
				options.initialValue = param.getValue();
			}

			var dataProvider = new MapPromiseDataProvider({
				'promise': this._webCenterAPI.getDocRefAttributes()
			});


			var sDrop = new SDrop(docRefAttrField, "", dataProvider, options);
			if (!this._isEnabled) {
				sDrop.setDisabled(true);
			}

			return sDrop;
		};

		/**
		 * initialize document template field
		 * @param {Parameter} param - parameter for which field is created
		 * @param {JQueryObj} ctx - context to which field should be appended
		 * @returns {JQueryObject} - smart name input
		 */
		MultiDynamicContentImportPanel.prototype.initDocTemplField = function (param, ctx) {
			var smartInput = new SmartnameInputField(param, undefined, this._smartNameDictionaryService);
			smartInput.setParameters([param]);
			smartInput.init();
			//Default styling for smart name not required in multi parameter panel
			smartInput.removeStlying();
			ctx.append($('<div />').append(smartInput.getContext()));
			return smartInput;
		};

		/**
		 * Handle resize
		 * @override
		 */
		MultiDynamicContentImportPanel.prototype.resize = function () {
			if (this._isVisible) {
				_.each(this._importConfigArr, _.bind(function (configObj) {
					var docTemplField = configObj.docTempl;
					docTemplField.resize();
				}, this));
			}
		};

		/**
		 * this method is called right before the panel is removed from the dom
		 * @override
		 */
		MultiDynamicContentImportPanel.prototype.destroy = function () {
			_.each(this._importConfigArr, _.bind(function (configObj) {
				var docTemplField = configObj.docTempl;
				docTemplField.destroy();
			}, this));
		};

		/**
		 * adds the entries to the multi parameter
		 * @override
		 */
		MultiDynamicContentImportPanel.prototype._addEntriesFromParameters = function () {
			_.each(this._subParameters, _.bind(function (subparam) {
				//UTGO:: Dynamic content map row should have 2nd level of sub parameters
				//Element type -> Sub parameter Document reference attribute has sub parameter having name and value
				//Element type -> Sub parameter Document Template has sub parameter having name and value
				if (subparam.getSubparameters().length > 0) {
					this.addEntry(subparam, true);
				}
			}, this));
			// if no subparameters: add empty entry
			if (this._subParameters.length === 0) {
				this.addEntry();
			}
			this._isDirtyParameters = false;

			this.resize();
			this._fitContent();
		};

		/**
		 * @override
		 */
		MultiDynamicContentImportPanel.prototype.disable = function () {
			// disable button
			this._btn.hide();

			_.each(this._importConfigArr, _.bind(function (configObj) {
				configObj.elementType.setDisabled(true);
				configObj.docRefAttr.setDisabled(true);
				configObj.docTempl.disable();
			}, this));
		};

		/**
		 * reset the panel
		 * @override
		 */
		MultiDynamicContentImportPanel.prototype.reset = function () {
			this._subParameters = [];
			this._importConfigArr = [];
			this._element.find("tr").not(":has(th)").remove();
		};


		/**
		 * close any opened SDrop when moving/resizing
		 * @Override
		 */
		MultiDynamicContentImportPanel.prototype.startInteractive = function () {
			_.each(this._importConfigArr, _.bind(function (configObj) {
				configObj.elementType.disable(null, true);
				configObj.docRefAttr.disable(null, true);
			}, this));
		};

		/**
		 * Entry is removed, now clean up components
		 * param {Number} - index of row where items are removed
		 * @override
		 */
		MultiDynamicContentImportPanel.prototype.destroyComponentsOnEntry = function (index) {
			var configObj = this._importConfigArr[index];
			configObj.docTempl.destroy();
			configObj.elementType.destroy();
			configObj.docRefAttr.destroy();
		};

		return MultiDynamicContentImportPanel;
	});