define(["WFEditor/workflow/panel/AbstractPanelComponent",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"WFEditor/workflow/panel/PanelConstants",
		"underscore",
		"jquery",
		"component/smartnames/smartfield/smartfield",
		"component/smartnames/encoding/smartnameencoding"
	],
	function (
		AbstractPanelComponent,
		WebCenterParameter,
		PanelConstants,
		_,
		$,
		SmartField,
		SmartNameEncoding) {

		"use strict";

		/**
		 * @class
		 * @exports MultiParametersPanel
		 * @augments AbstractPanelComponent
		 * 
		 * a panel component used for display of multiple parameters (names) with their values
		 * @param {String} parameterId - id of the parameter for this panel
		 * @param {String} label       - label to be displayed
		 * @param {WorkflowSmartNameDictionaryService} smartNameDictionaryService - the smartname dictionary for smartnames in HTML
		 * @param {Boolean} isWrapped 
		 * 								- if false, multiple parameters are passed, if true, one parameter with subparameters is passed
		 * @param {Number} maxRows 
		 * 								- Maximum number of rows allowed for this panel
		 */
		var MultiParametersPanel = function (parameterId, nameLabel, valueLabel, smartNameDictionaryService, isWrapped, maxRows) {
			AbstractPanelComponent.call(this, [parameterId]);
			this._element.addClass("MultiParametersPanel");

			/**
			 * @member {Array.<Parameter>} subparameters
			 * @private
			 */
			this._subParameters = [];

			/**
			 * @member {Boolean} isWrapped 
			 * 					- if false, multiple parameters are passed, if true, one parameter with subparameters is passed
			 * @private
			 */
			this._isWrapped = isWrapped;
			
			/**
			 * @member {Number} maxRows
			 * @private
			 */
			this._maxRows = maxRows;

			// if the content isn't wrapped, we want to accept all parameters
			if (!this._isWrapped) {
				this._parameterWishList = [PanelConstants.WILDCARD];
			}

			/**
			 * @member {jQueryObj} listObj
			 * @private
			 */
			this._context = $('<div/>');
			this._context.append($("<div />").addClass("header clearfix").append($("<div class='nameAndValue offsetRightDeleteButton'/>")
				.append($("<div />").addClass("nameCol").append($("<label/>").append(nameLabel)))
				.append($("<div />").addClass("valueCol").append($("<label/>").append(valueLabel)))));

			/**
			 * @member {jQueryObj} listObj
			 * @private
			 */
			this._smartNameDictionaryService = smartNameDictionaryService;

			/**
			 * @member {Array.<jQueryObj>} listObj
			 * @private
			 */
			this._nameFields = [];

			/**
			 * @member {Array.<SmartField>} listObj
			 * @private
			 */
			this._valueFields = [];

			// add list and additional button for new entries to the element
			this._element.append(this._context);

			var btn = $('<a />').addClass("AddBtn")
				.attr("disabled", !this._isEnabled).addClass("btn btn-sm btn-primary")
				.append($('<i />').addClass("icon icon-WCR_Actions-Add icon-small"))
				.click(_.bind(function () {
					if (this._isEnabled) {
						this.addEntry();
					}
				}, this));

			this._element.append(btn);

			/**
			 * @member {Boolean} Are the entries in the multiparameter not the same as the parameters?
			 * @private
			 */
			this._isDirtyParameters = false;
		};

		MultiParametersPanel.prototype = Object.create(AbstractPanelComponent.prototype);
		MultiParametersPanel.prototype.constructor = MultiParametersPanel;

		/**
		 * When the parameter is set, fields should be changed
		 * @param {Array.<Parameter>} parameters
		 */
		MultiParametersPanel.prototype.setParameters = function (parameters) {
			AbstractPanelComponent.prototype.setParameters.call(this, parameters);

			this.reset();

			if (this._isWrapped) {
				this._subParameters = parameters[0].getSubparameters();
			} else {
				this._subParameters = parameters;
			}

			if (this._isVisible) {
				this._addEntriesFromParameters();
			} else {
				this._isDirtyParameters = true;
			}

		};

		/**
		 * @override
		 */
		MultiParametersPanel.prototype.getParameters = function () {
			// When the parameters are asked, filter out the empty ones first
			for (var i = this._subParameters.length - 1; i >= 0; i--) {
				var subParameter = this._subParameters[i];
				if (!subParameter.getName() || subParameter.getName().length === 0) {
					this._subParameters.splice(i, 1);
				}
			}
			return AbstractPanelComponent.prototype.getParameters.call(this);
		};

		/**
		 * Add e new entry of input/smartname fields to the list
		 * @param {Parameter} [param] - parameter to put in the entry; if undefined, an empty entry is created
		 */
		MultiParametersPanel.prototype.addEntry = function (param, noFitContent) {
			if (!(param instanceof WebCenterParameter)) {
				// if parameter does not exist(empty entry), create a new one
				param = new WebCenterParameter("", "", "");
				this._subParameters.push(param);
			}

			// create entry
			var entry = $("<div/>").addClass("block clearfix");
			var nameAndValueDiv = $("<div/>").addClass("nameAndValue offsetRightDeleteButton");
			entry.append(nameAndValueDiv);

			// init fields
			var nameField = this.initNameField(param, nameAndValueDiv);
			var valueField = this.initValueField(param, nameAndValueDiv);

			this._nameFields.push(nameField);
			this._valueFields[this._nameFields.length - 1] = valueField;
			
			//Restrict no. of rows
			if(this._maxRows && this._nameFields.length >= this._maxRows ){
				// disable buttons
				this._element.find(".AddBtn").hide();
			}


			var deleteBtn = $('<a />').attr("disabled", !this._isEnabled).addClass("btn btn-default btn-sm")
				.append($('<i />').addClass("icon icon-WCR_Actions-Delete12 icon-normal"))
				.click(_.bind(function () {
					if (this._isEnabled) {
						entry.remove();

						// handle remove name and value fields
						var index = this._nameFields.indexOf(nameField);
						if (index > -1) {
							this.destroyComponentsOnEntry(index);
							this._nameFields.splice(index, 1);
							this._valueFields.splice(index, 1);
							
							//Show add button if new row is allowed
							if(this._maxRows && this._nameFields.length < this._maxRows ){
								// disable buttons
								this._element.find(".AddBtn").show();
							}
						}

						// remove subparameter
						index = this._subParameters.indexOf(param);
						if (index > -1) {
							this._subParameters.splice(index, 1);
						}
					}

					this._fitContent();

				}, this));

			this._context.append(entry.append($('<div class="deleteBtn" />').append(deleteBtn)));

			// show and hide button on hover (only show if enabled)
			deleteBtn.hide();
			entry.hover(_.bind(function () {
				if (this._isEnabled) {
					deleteBtn.show();
				}
			}, this), function () {
				deleteBtn.hide();
			});

			if (!noFitContent) {
				this.resize();
				this._fitContent(true);
			}
		};

		/**
		 * initialize name field
		 * 
		 * @param {Parameter} param - parameter for which field is created
		 * @param {JQueryObj} ctx - context to which field should be appended
		 * 
		 * @returns {Object}
		 */
		MultiParametersPanel.prototype.initNameField = function (param, ctx) {
			// normal input when not multiattribute
			var nameField = $('<input type="text" />');
			ctx.append($('<div />').append(nameField));

			// just use input field and register changes
			if (param.getName()) {
				nameField.val(param.getName());
			}
			nameField.on('change', function (e) {
				param.setName(nameField.val());
			});
			nameField.addClass("block form-control");
			nameField.attr("disabled", !this._isEnabled);

			return nameField;
		};

		/**
		 * initialize value field
		 * 
		 * @param {Parameter} param - parameter for which field is created
		 * @param {JQueryObj} ctx - context to which field should be appended
		 * @param {JQueryObj} [customInput] - if passed along, this element will be turned into a smartname, instead of appending a new field
		 * @param {WorkflowSmartNameDictionaryService} [customDictionary] - if passed along, this dictionary will be used for smartfield
		 * 
		 * @returns {Object}
		 */
		MultiParametersPanel.prototype.initValueField = function (param, ctx, customInput, customDictionary) {
			var valueField = customInput || $('<input type="text" />');
			if (!customInput) {
				// if there's a custom input, it should be in the DOM already
				ctx.append($('<div />').append(valueField));
			}
			var smartfield = new SmartField(valueField,
				false, // lazy init
				customDictionary || this._smartNameDictionaryService, // our dictionary
				_.bind(this._smartNameDictionaryService.displaySmartName, this._smartNameDictionaryService), // display in input field
				_.bind(this._smartNameDictionaryService.displaySmartNameSelectorSmartName, this._smartNameDictionaryService), // display in selector
				SmartNameEncoding.WORKFLOWEDITOR // encoding
			);
			smartfield.setValue(param.getValue());
			smartfield.onValueChange(function (val) {
				param.setValue(val);
			});
			if (!this._isEnabled) {
				smartfield.disable();
			}
			return smartfield;
		};

		/**
		 * Handle resize
		 * @override
		 */
		MultiParametersPanel.prototype.resize = function () {
			var valWidth = this._element.find(".valueCol").outerWidth(true);
			if (this._isVisible) {
				_.each(this._valueFields, _.bind(function (valuefield) {
					valuefield.setWidth(valWidth + 'px');
				}, this));
			}
		};

		/**
		 * this method is called right before the panel is removed from the dom
		 * @override
		 */
		MultiParametersPanel.prototype.destroy = function () {
			_.each(this._valueFields, _.bind(function (smartname) {
				smartname.destroy();
			}, this));
		};

		/**
		 * @override
		 */
		MultiParametersPanel.prototype.show = function () {
			AbstractPanelComponent.prototype.show.call(this);
			if (this._isDirtyParameters) {
				this._addEntriesFromParameters();
			}

			this.resize();

		};

		/**
		 * adds the entries to the multi parameter
		 */
		MultiParametersPanel.prototype._addEntriesFromParameters = function () {
			_.each(this._subParameters, _.bind(function (subparam) {
				this.addEntry(subparam, true);
			}, this));
			// if no subparameters: add empty entry
			if (this._subParameters.length === 0) {
				this.addEntry();
			}
			this._isDirtyParameters = false;

			this.resize();
			this._fitContent();
		};

		/**
		 * @override
		 */
		MultiParametersPanel.prototype.disable = function () {
			AbstractPanelComponent.prototype.disable.call(this);

			// disable buttons
			this._element.find(".AddBtn").hide();

			// disable smartfields
			_.each(this._valueFields, function (smartfield) {
				smartfield.disable();
			});
		};

		/**
		 * reset the panel
		 */
		MultiParametersPanel.prototype.reset = function () {
			this._subParameters = [];
			this._nameFields = [];
			this._valueFields = [];
			this._element.find( "div.block.clearfix" ).remove();
		};

		/**
		 * We want at least 470px 
		 * @override
		 * @returns {Number}
		 */
		MultiParametersPanel.prototype.getPreferredWidth = function () {
			return 470;
		};

		/**
		 * Entry is removed, now clean up components
		 * param {Number} - index of row where items are removed
		 */
		MultiParametersPanel.prototype.destroyComponentsOnEntry = function (index) {
			this._valueFields[index].destroy();
		};

		return MultiParametersPanel;
	});