define(["WFEditor/workflow/panel/AbstractPanelComponent",
		"component/multiSelect/view/MultiSelectList",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"WFEditorWCR/workflow/panel/WebCenterPanelConstants",
		"utils/webi18n!WFEditorWCR",
		"utils/Observer",
		"utils/FilterUtils",
		"jquery",
		"underscore"
	],
	function (AbstractPanelComponent,
		MultiSelectList,
		WebCenterParameter,
		WebCenterPanelConstants,
		i18n,
		Observer,
		FilterUtils,
		$,
		_) {

		"use strict";

		/**
		 * @class
		 * @exports MultiProjectAttributePanel
		 * @augments PanelComonent
		 * 
		 * A panel that allows to select multiple project attributes
		 * 
		 * @param {String} parameterId   - id of the parameter for this panel
		 * @param {String} libLabel      - label to be displayed above the multiselect's not selected entries
		 * @param {String} selectedLabel - label to be displayed above the multiselect's selected entries                           
		 * @param {WebCenterAPI} webcenterAPI                         
		 */
		var MultiProjectAttributePanel = function (parameterId, libLabel, selectedLabel, webcenterAPI) {
			AbstractPanelComponent.call(this, [parameterId]);
			Observer.mixin(this);
			/**
			 * @member {Function} function used to get filtered results when the user filters the multiSelect's library list
			 * @private
			 */

			/**
			 * @member {Object} all attributes 
			 * @private
			 */
			this._attributes = {};

			/**
			 * @member <$.Deferred.<Map.<WebCenterAttribute>>
			 * @private
			 */
			this._attributesPromise = webcenterAPI.getAttributes();

			// get the project attribute data
			this._attributesPromise.done(_.bind(function (attributes) {
				_.each(_.sortBy(attributes.values(), "_name"), function (entry) {

					var attribute = {
						"id": entry.getId(),
						"label": entry.getName(),
						"selected": false
					};

					this._attributes[entry.getId()] = attribute;

				}, this);
			}, this));

			/**
			 * @member {Object.<Object>} the selected attributes
			 * @privates
			 */
			this._selectedAttributes = {};

			/**
			 * @member {Number} the number of attributes that will be given to the filterlists of the multiselect
			 * with the intention of not creating too many list items at once
			 * @private
			 */
			this._maxRows = WebCenterPanelConstants.MULTIPROJATTRIBUTESMAXROWS;

			/**
			 * @member {Function} - filter used to get the attributes
			 * @private
			 */
			this._filter = _.bind(function (pattern, page) {
				var dfr = $.Deferred();
				this._attributesPromise.done(_.bind(function () {

					// variable for paging
					var selected = 0;

					var regex = FilterUtils.generateRegex(pattern, true);

					dfr.resolve(_.filter(this._attributes, _.bind(function (object) {
						if (regex.test(object.label) && !object.selected) {
							selected++;
							// Only get the not selected objects that are on the right page
							return selected >= (this._maxRows * (page - 1)) && selected <= this._maxRows * page;
						}
					}, this)));
				}, this));
				return dfr.promise();

			}, this);

			/**
			 * @member {Function} - filter used to get the selected attributes
			 * @private
			 */
			this._selectedFilter = _.bind(function (pattern, page) {
					var dfr = $.Deferred();
					var selected = 0;
					var regex = FilterUtils.generateRegex(pattern, true);

					dfr.resolve(_.filter(this._attributes, _.bind(function (object) {
						if (regex.test(object.label) && object.selected) {
							selected++;
							// Only get the not selected objects that are on the right page
							return selected >= (this._maxRows * (page - 1)) && selected <= this._maxRows * page;
						}
					}, this)));

					return dfr.promise();

				},
				this);


			/**
			 * @member {String}
			 * @private
			 */
			this._paramId = parameterId;


			/**
			 * @member {MultiSelect} - the multiselect used to get the selected project attributes
			 * @private
			 */
			this._multiSelect = new MultiSelectList(this._filter, this._selectedFilter, this._element, {
				"notSelectedLabel": libLabel,
				"selectedLabel": selectedLabel,
				"selectAllTooltip": i18n.t("sync.projattr.c.custom.addAll"),
				"deselectAllTooltip": i18n.t("sync.projattr.c.custom.removeAll")
			});
			this._multiSelect.setContext(this._element);

			// When an entry is selected set the selected property of each attribute to true
			this._observe(this._multiSelect, "entrySelected", function (ids) {
				_.each(ids, function (id) {
					var attr = this._attributes[id];
					attr.selected = true;
					this._selectedAttributes[id] = attr;
				}, this);
			});

			// When an entry is deselected set the selected property of each attribute to false
			this._observe(this._multiSelect, "entryDeselected", function (ids) {
				_.each(ids, function (id) {
					var attr = this._attributes[id];
					attr.selected = false;
					delete this._selectedAttributes[id];

				}, this);
			});

			// select all entries that have a label that starts with 'pattern'
			this._observe(this._multiSelect, "selectAll", function (pattern) {
				var regex = FilterUtils.generateRegex(pattern, true);
				var attrs = _.filter(this._attributes, function (attr) {
					return regex.test(attr.label) && !attr.selected;
				});
				var attrsObj = {};
				_.each(attrs, function (attr) {
					attr.selected = true;
					attrsObj[attr.id] = attr;
				});
				this._selectedAttributes = _.extend(this._selectedAttributes, attrsObj);
			});

			// deselect all entries that have a label that starts with 'pattern'
			this._observe(this._multiSelect, "deselectAll", function (pattern) {
				var regex = FilterUtils.generateRegex(pattern, true);
				var attrs = _.filter(this._attributes, function (attr) {
					return regex.test(attr.label) && attr.selected;
				});
				_.each(attrs, function (attr) {
					attr.selected = false;
					delete this._selectedAttributes[attr.id];
				}, this);
			});

			this._element.addClass("multi-project-attribute-panel");

		};

		MultiProjectAttributePanel.prototype = Object.create(AbstractPanelComponent.prototype);
		MultiProjectAttributePanel.prototype.constructor = MultiProjectAttributePanel;

		/**
		 * @override
		 */
		MultiProjectAttributePanel.prototype.getPreferredWidth = function () {
			return 500;
		};

		/**
		 * @override
		 */
		MultiProjectAttributePanel.prototype.getParameters = function () {
			var subParams = [];
			if (this._isApplicable) {
				_.each(this._selectedAttributes, function (entry, id) {
					var p = new WebCenterParameter(entry.id, entry.id, entry.id);
					subParams.push(p);
				});
			}
			var param = this.getParameterByName(this._paramId);
			this.getParameterByName(this._paramId).setSubParameters(subParams);
			return [param];


		};

		/**
		 * @override
		 */
		MultiProjectAttributePanel.prototype.setParameters = function (parameters) {
			AbstractPanelComponent.prototype.setParameters.call(this, parameters);
			// the selected project attributes will be placed in this object
			this._selectedAttributes = {};

			this._attributesPromise.done(_.bind(function () {
				_.each(parameters[0].getSubparameters(), function (parameter) {
					var attr = this._attributes[parameter.getName()];
					attr.selected = true;
					this._selectedAttributes[attr.id] = attr;
				}, this);
				this._multiSelect.render();
			}, this));


		};

		/**
		 * @override
		 */
		MultiProjectAttributePanel.prototype.disable = function () {
			AbstractPanelComponent.prototype.disable.call(this);
			this._multiSelect.disable();
		};

		/**
		 * @override
		 */
		MultiProjectAttributePanel.prototype.destroy = function () {
			this._multiSelect.destroy();
		}



		return MultiProjectAttributePanel;

	});
