define(["WFEditorWCR/workflow/panel/MultiParametersPanel",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"WFEditorWCR/workflow/panel/SmartNameDropdownPanel",
		"packaging-content-management/ContentConstants",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery",
	],
	function (
		MultiParametersPanel,
		WebCenterParameter,
		SmartNameDropdownPanel,
		ContentConstants,
		i18n,
		_,
		$) {

		"use strict";

		/**
		 * @class
		 * @exports MultiTableParametersPanel
		 * @augments MultiParametersPanel
		 * 
		 * An abstract panel component used for display of table content parameters
		 * @param {String} parameterId 
		 * 							- id of the parameter for this panel
		 * @param {Map.<String,String>} fieldMap     
		 * 							- Map of fields to be added as columns
		 * @param {WorkflowSmartNameDictionaryService} smartNameDictionaryService 
		 * 							- the smartname dictionary for smartnames in value fields
		 * @param {WebCenterAPI} webCenterAPI	
		 * 							- webcenter inferface
		 * @param {Boolean} isWrapped 
		 * 							- if false, multiple parameters are passed, if true, one parameter with subparameters is passed
		 */
		var MultiTableParametersPanel = function (parameterId, fieldMap, smartNameDictionaryService, webCenterAPI, isWrapped) {
			MultiParametersPanel.call(this, parameterId, "", "", smartNameDictionaryService, isWrapped);
			
			/**
			 * @member {WebCenterAPI}
			 * @private
			 */
			this._webCenterAPI = webCenterAPI;
			
			/**
			 * @member {Array.<jQueryObj>} listObj
			 * @private
			 */
			this._tableRows = [];
			
			/**
			 * @member {Array.<jQueryObj>} listObj
			 * @private
			 */
			this._actionFields = [];
			
			/**
			 * @member {Array.<jQueryObj>} listObj
			 * @private
			 */
			this._valueFields = [];
			
			/**
			 * @member {Array.<jQueryObj>} listObj
			 * @private
			 */
			this._tableRows = [];
			
			/**
			 * @member {JQueryObject} the div this panel will render in
			 * @private
			 */
			this._element = $('<div/>');

			this._element.addClass("MultiParametersPanel MultiColumnPanel");
			
			/**
			 * @member {JQueryObject} the context this panel will render in
			 * @private
			 */
			this._context = $('<div/>');
			
			this._fieldMap = fieldMap;
			
			this._labelArr = this._fieldMap.entries();
			
			this._colClass = "has-"+this._labelArr.length+"-cols";
			this._fieldDiv = $("<div />").addClass(this._colClass+" offsetRightDeleteButton");
			for(var i=0; i< this._labelArr.length ; i++){
				this._fieldDiv.append($("<div />").addClass("fieldCol").append($("<label/>").append(i18n.t(this._labelArr[i]))));
			}
			
			this._context.append($("<div />").addClass("clearfix header").append(this._fieldDiv));
			
			this._element.append(this._context);

			this._btn = $('<a />').addClass("AddBtn")
				.attr("disabled", !this._isEnabled).addClass("btn btn-sm btn-primary")
				.append($('<i />').addClass("icon icon-WCR_Actions-Add icon-small"))
				.click(_.bind(function () {
					if (this._isEnabled) {
						this.addEntry();
					}
				}, this));

			this._element.append(this._btn);
			
		};

		MultiTableParametersPanel.prototype = Object.create(MultiParametersPanel.prototype);
		MultiTableParametersPanel.prototype.constructor = MultiTableParametersPanel;
		
		/**
		 * @override
		 */
		MultiTableParametersPanel.prototype.addEntry = function (param, noFitContent) {
			if (!(param instanceof WebCenterParameter)) {
				// if parameter does not exist(empty entry), create a new one
				var subParamArr = [];
				
				for(var i=0; i< this._labelArr.length ; i++){
					subParamArr.push(new WebCenterParameter(this._labelArr[i], "", ""));
				}
				
				param = new WebCenterParameter("", "", "", subParamArr);
				
				//Generate unique param name (a sequence number)
				var paramName = 1;
				var exisitingParamNames = [];
				for (var i = 0; i < this._subParameters.length; i++) {
					exisitingParamNames.push(this._subParameters[i].getName());
				}
				while($.inArray(''+paramName, exisitingParamNames) != -1){
					paramName++;
				}
				param.setName(''+paramName);
				
				this._subParameters.push(param);
			}
			
			// create entry
			var entry = $("<div/>").addClass("block clearfix");
			var multiFieldDiv = $("<div/>").addClass(this._colClass+" offsetRightDeleteButton");
			entry.append(multiFieldDiv);

			// Initialize fields
			var rowFields = [];
			for(var i=0; i< this._labelArr.length ; i++){
				var field = this.initField(this._fieldMap.get(this._labelArr[i]), param.getSubparameters()[i], multiFieldDiv);
				rowFields.push(field);
			}

			this._tableRows.push(rowFields);

			var deleteBtn = $('<a />').attr("disabled", !this._isEnabled).addClass("btn btn-default btn-sm")
				.append($('<i />').addClass("icon icon-WCR_Actions-Delete12 icon-normal"))
				.click(_.bind(function () {
					if (this._isEnabled) {
						entry.remove();

						// handle remove row
						var index = this._tableRows.indexOf(rowFields);
						if (index > -1) {
							this.destroyComponentsOnEntry(index);
							this._tableRows.splice(index, 1);
						}

						// remove subparameter
						index = this._subParameters.indexOf(param);
						if (index > -1) {
							this._subParameters.splice(index, 1);
						}
					}

					this._fitContent();

				}, this));

			this._context.append(entry.append($('<div class="deleteBtn" />').append(deleteBtn)));

			// show and hide button on hover (only show if enabled)
			deleteBtn.hide();
			entry.hover(_.bind(function () {
				if (this._isEnabled) {
					deleteBtn.show();
				}
			}, this), function () {
				deleteBtn.hide();
			});

			if (!noFitContent) {
				this.resize();
				this._fitContent(true);
			}
		};
		
		/**
		 * initialize a field
		 * 
		 * @param {Parameter} param - parameter for which field is created
		 * @param {JQueryObj} ctx - context to which field should be appended
		 * 
		 * @returns {Object}
		 */
		MultiTableParametersPanel.prototype.initField = function (fieldType, param, ctx) {
			var field = null;
			
			switch(fieldType){

			case 'elementtype_smartdropdown':
				field = this._createElementField(param, ctx);
				break;
			case 'language_smartdropdown':
				field = this._createLanguageField(param, ctx);
				break;
			case 'instanceseq_smartinput':
				field = this._createInstSeqField(param, ctx);
				break;
			case 'languageaction_smartdropdown':
				field = this._createElementActionField(param, ctx);
				break;
			case 'approval_smartdropdown':
				field = this._createApprovalField(param, ctx);
				break;
			case 'content_smartinput':
				field = this._createElementContentField(param, ctx);
				break;
			default:
				field = this.initValueField(param, ctx);
				break;
			}
			
			return field;
		};
		
		/**
		 * @override
		 */
		MultiTableParametersPanel.prototype._addEntriesFromParameters = function () {
			_.each(this._subParameters, _.bind(function (subparam) {
				//Table parameters should have 2nd level of sub parameters
				if (subparam.getSubparameters().length > 0) {
					this.addEntry(subparam, true);
				} 
			}, this));

			// if no subparameters: add empty entry
			if (this._subParameters.length === 0) {
				this.addEntry();
			}
			this._isDirtyParameters = false;

			this.resize();
			this._fitContent();
		};
		

		/**
		 * initialize PCM language field
		 * 
		 * @param {Parameter} param - parameter for which field is created
		 * @param {JQueryObj} ctx - context to which field should be appended
		 * 
		 * @returns {Object}
		 */
		MultiTableParametersPanel.prototype._createLanguageField = function (param, ctx) {
			//Create default language dropdown option		
			var options = [];
			var elementTypeField = new SmartNameDropdownPanel(param.getName(), undefined, options, this._smartNameDictionaryService);
			elementTypeField.setParameters([param]);
			//Default styling for smartname dropdown not required in multi parameter panel
			elementTypeField.removeStlying();
			ctx.append($('<div />').append(elementTypeField.getContext()));

			if (!this._isEnabled) {
				elementTypeField.disable();
			}
			this._webCenterAPI.getTextContentLanguageList().done(_.bind(function (langMap) {
				//Update the dropdown with PCM language list
				options.push({
					text: i18n.t('wfui.textcontent.languagedefaultlabel'),
					value: ""
				});
				_.each(langMap.get(ContentConstants.LanguageType.ALL).values(), function (lang) {
					options.push({
						text: lang.getName(),
						value: lang.getId()
					});
				});
				
				elementTypeField.setOptions(options);
			}, this));

			return elementTypeField;
		};
		

		/**
		 * initialize Table element type field
		 * 
		 * @param {Parameter} param - parameter for which field is created
		 * @param {JQueryObj} ctx - context to which field should be appended
		 * 
		 * @returns {Object}
		 */
		MultiTableParametersPanel.prototype._createElementField = function (param, ctx) {
			var options = [];
			var elementTypeField = new SmartNameDropdownPanel(param.getName(), undefined, options, this._smartNameDictionaryService);
			elementTypeField.setParameters([param]);
			//Default styling for smartname dropdown not required in multi parameter panel
			elementTypeField.removeStlying();
			ctx.append($('<div />').append(elementTypeField.getContext()));

			if (!this._isEnabled) {
				elementTypeField.disable();
			}
			this._webCenterAPI.getTableContentElementTypeList().done(_.bind(function (elementTypeMap) {
				//Update the dropdown with table element list
				options.push({
					text: i18n.t('wfui.settc.selectelement'),
					value: ""
				});
				
				var elementList = elementTypeMap.values().sort(function(element1, element2) {
				    return (element1.getName().localeCompare(element2.getName()));
				});
				
				_.each(elementList, function (element) {
					options.push({
						text: element.getName(),
						value: element.getId()
					});
				});
				
				elementTypeField.setOptions(options);
			}, this));
			
			return elementTypeField;
			
		};
		
		/**
		 * initialize field for add /remove table element
		 * 
		 * @param {Parameter} param - parameter for which field is created
		 * @param {JQueryObj} ctx - context to which field should be appended
		 * 
		 * @returns {Object}
		 */
		MultiTableParametersPanel.prototype._createElementActionField = function (param, ctx) {
			//Create default dropdown option
			var options = [{
				value: ContentConstants.LanguageAction.UPDATE,
				text: i18n.t("settcfield.updateelement")
			},{
				value: ContentConstants.LanguageAction.ADD,
				text: i18n.t("settcfield.addlanguage")
			}, {
				value: ContentConstants.LanguageAction.REMOVE,
				text: i18n.t("settcfield.removelanguage")
			}];
			
			//Set the default value for field
			if(param.getValue().length === 0 ){
				param.setValue(ContentConstants.LanguageAction.UPDATE);
			}
			
			var statusDropDownPanel = new SmartNameDropdownPanel(param.getName(), undefined, options, this._smartNameDictionaryService);
			statusDropDownPanel.setParameters([param]);
			//Default styling for smartname dropdown not required in multi parameter panel
			statusDropDownPanel.removeStlying();
			ctx.append($('<div />').append(statusDropDownPanel.getContext()));
			
			var smartField = statusDropDownPanel.getSmartField();
			this._actionFields.push(smartField);
			
			//Disable "Value" field when action is "Remove"
			smartField.onValueChange(_.bind(function (action) {
				var index = this._actionFields.indexOf(smartField);
				var valueField = this._valueFields[index];
				if(action === ContentConstants.LanguageAction.REMOVE){
					valueField.setValue("");
					valueField.disable();
				} else {
					if(valueField.isDisabled()){
						valueField.enable();
					}
				}
			}, this));
			
			if (!this._isEnabled) {
				statusDropDownPanel.disable();
			}

			return statusDropDownPanel;
		};
		

		/**
		 * initialize field for setting approval status for a table element
		 * 
		 * @param {Parameter} param - parameter for which field is created
		 * @param {JQueryObj} ctx - context to which field should be appended
		 * 
		 * @returns {Object}
		 */
		MultiTableParametersPanel.prototype._createApprovalField = function (param, ctx) {
			//Create default dropdown option
			var options = [{
				value: ContentConstants.State.DRAFT,
				text: i18n.t("tcstatus.draft")
			}, {
				value: ContentConstants.State.READY_FOR_APPROVAL,
				text: i18n.t("tcstatus.pendingsignoff")
			}, {
				value: ContentConstants.State.APPROVED,
				text: i18n.t("tcstatus.signedoff")
			}, {
				value: ContentConstants.State.REJECTED,
				text: i18n.t("tcstatus.signoffrejected")
			}];
			
			//Set the default value for field
			if(param.getValue().length === 0 ){
				param.setValue(ContentConstants.State.DRAFT);
			}
			
			var elementStatusField = new SmartNameDropdownPanel(param.getName(), undefined, options, this._smartNameDictionaryService);
			elementStatusField.setParameters([param]);
			//Default styling for smartname dropdown not required in multi parameter panel
			elementStatusField.removeStlying();
			ctx.append($('<div />').append(elementStatusField.getContext()));
			
			if (!this._isEnabled) {
				elementStatusField.disable();
			}

			return elementStatusField;
		};
		
		/**
		 * initialize field for setting value for a table element
		 * 
		 * @param {Parameter} param - parameter for which field is created
		 * @param {JQueryObj} ctx - context to which field should be appended
		 * 
		 * @returns {Object}
		 */
		MultiTableParametersPanel.prototype._createElementContentField = function (param, ctx) {
			var valueField = this.initValueField(param, ctx);
			
			if(!valueField.isDisabled()){
				//Disable the value field if action is "Remove"
				var actionField = this._actionFields[this._valueFields.length];
				if(actionField){
					var action = actionField.getValue();
					if(action === ContentConstants.LanguageAction.REMOVE){
						valueField.setValue("");
						valueField.disable();
					}
				}
			}
			
			this._valueFields.push(valueField);
			
			return valueField;
		};
		
		/**
		 * initialize field for setting instance sequence of a table element
		 * 
		 * @param {Parameter} param - parameter for which field is created
		 * @param {JQueryObj} ctx - context to which field should be appended
		 * 
		 * @returns {Object}
		 */
		MultiTableParametersPanel.prototype._createInstSeqField = function (param, ctx) {
			var instSeqField = this.initValueField(param, ctx);
			//Set the default value for instance sequence as 1
			if(!param.getValue() || param.getValue().length === 0){
				param.setValue('1');
				instSeqField.setValue("1");
			}
			
			return instSeqField;
		};
		
		/**
		 * @override
		 */
		MultiTableParametersPanel.prototype.destroy = function () {
			for(var i=0; i< this._tableRows.length; i++){
				_.each(this._tableRows[i], function (field) {
					field.destroy();
				});
			}
		};
		
		/**
		 * @override
		 */
		MultiTableParametersPanel.prototype.destroyComponentsOnEntry = function (index) {
			var tableRow = this._tableRows[index];
			for(var i=0; i< tableRow.length; i++){
				tableRow[i].destroy();
			}
		};
		
		/**
		 * @override
		 */
		MultiTableParametersPanel.prototype.disable = function () {
			MultiParametersPanel.prototype.disable.call(this);
			
			for(var i=0; i< this._tableRows.length; i++){
				_.each(this._tableRows[i], function (field) {
					field.disable();
				});
			}
		};
		
		/**
		 * @override
		 */
		MultiTableParametersPanel.prototype.reset = function () {
			MultiParametersPanel.prototype.reset.call(this);
			
			this._actionFields = [];
			this._valueFields = [];
			this._tableRows = [];
		};
		
		/**
		 * We want at least 750px 
		 * @override
		 * @returns {Number}
		 */
		MultiTableParametersPanel.prototype.getPreferredWidth = function () {
			return 750;
		};
		
		return MultiTableParametersPanel;
	});