define(["WFEditorWCR/workflow/panel/MultiParametersPanel",
		"WFEditorWCR/workflow/panel/SmartNameDropdownPanel",
		"WFEditorWCR/workflow/panel/SmartNameDropdownParamNamePanel",
		"packaging-content-management/ContentConstants",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery",
		"wcr/sdrop"
	],
	function (
		MultiParametersPanel,
		SmartNameDropdownPanel,
		SmartNameDropdownParamNamePanel,
		ContentConstants,
		i18n,
		_,
		$) {

		"use strict";

		/**
		 * @class
		 * @exports MultiTextContentElementPanel
		 * @augments MultiParametersPanel
		 * 
		 * a panel component used for display of text content language and status
		 * @param {String} parameterId 
		 * 							- id of the parameter for this panel
		 * @param {String} nameLabel       
		 * 							- label for the language to be displayed
		 * @param {String} valueLabel       
		 * 							- label for the language element status to be displayed
		 * @param {WorkflowSmartNameDictionaryService} smartNameDictionaryService 
		 * 							- the smartname dictionary for smartnames in value fields
		 * @param {WebCenterAPI} webCenterAPI	
		 * 							- webcenter inferface
		 * @param {Boolean} isWrapped 
		 * 							- if false, multiple parameters are passed, if true, one parameter with subparameters is passed
		 */
		var MultiTextContentElementPanel = function (parameterId, nameLabel, valueLabel, smartNameDictionaryService, webCenterAPI, isWrapped) {
			MultiParametersPanel.call(this, parameterId, nameLabel, valueLabel, smartNameDictionaryService, isWrapped);
			
			/**
			 * @member {WebCenterAPI}
			 * @private
			 */
			this._webCenterAPI = webCenterAPI;
		};

		MultiTextContentElementPanel.prototype = Object.create(MultiParametersPanel.prototype);
		MultiTextContentElementPanel.prototype.constructor = MultiTextContentElementPanel;
		
		/**
		 * Initialize language element field
		 * 
		 * @override
		 */
		MultiTextContentElementPanel.prototype.initNameField = function (param, ctx) {
			//Create default language dropdown option		
			var options = [];
			var langDropdownField = new SmartNameDropdownParamNamePanel(param.getName(), undefined, options, this._smartNameDictionaryService);
			langDropdownField.setParameters([param]);
			//Default styling for smartname dropdown not required in multi parameter panel
			langDropdownField.removeStlying();
			ctx.append($('<div />').append(langDropdownField.getContext()));

			if (!this._isEnabled) {
				langDropdownField.disable();
			}
			this._webCenterAPI.getTextContentLanguageList().done(_.bind(function (langMap) {
				//Update the dropdown with PCM language list
				options.push({
					text: i18n.t('wfui.textcontent.languagedefaultlabel'),
					value: ""
				});
				_.each(langMap.get(ContentConstants.LanguageType.ALL).values(), function (lang) {
					options.push({
						text: lang.getName(),
						value: lang.getId()
					});
				});
				
				langDropdownField.setOptions(options);
			}, this));

			return langDropdownField;
		};
		
		
		/**
		 * Initialize element status field for a language
		 * 
		 * @override
		 */
		MultiTextContentElementPanel.prototype.initValueField = function (param, ctx) {
			//Create default dropdown option
			var options = [{
				value: ContentConstants.State.DRAFT,
				text: i18n.t("tcstatus.draft")
			}, {
				value: ContentConstants.State.READY_FOR_APPROVAL,
				text: i18n.t("tcstatus.pendingsignoff")
			}, {
				value: ContentConstants.State.APPROVED,
				text: i18n.t("tcstatus.signedoff")
			}, {
				value: ContentConstants.State.REJECTED,
				text: i18n.t("tcstatus.signoffrejected")
			}];
			
			//Set the default value for field
			if(param.getValue().length === 0 ){
				param.setValue(ContentConstants.State.DRAFT);
			}
			
			var elementStatusField = new SmartNameDropdownPanel(param.getName(), undefined, options, this._smartNameDictionaryService);
			elementStatusField.setParameters([param]);
			//Default styling for smartname dropdown not required in multi parameter panel
			elementStatusField.removeStlying();
			ctx.append($('<div />').append(elementStatusField.getContext()));
			
			if (!this._isEnabled) {
				elementStatusField.disable();
			}

			return elementStatusField;
		};
		
		/**
		 * @override
		 */
		MultiTextContentElementPanel.prototype.resize = function () {
			if (this._isVisible) {
				_.each(this._nameFields, _.bind(function (smartNameDropdown) {
					smartNameDropdown.resize();
				}, this));
				
				_.each(this._valueFields, _.bind(function (smartNameDropdown) {
					smartNameDropdown.resize();
				}, this));
			}
		};
		
		/**
		 * @Override
		 */
		MultiTextContentElementPanel.prototype.startInteractive = function () {
			_.each(this._nameFields, function (smartNameDropdown) {
				smartNameDropdown.startInteractive();
			});
			_.each(this._valueFields, function (smartNameDropdown) {
				smartNameDropdown.startInteractive();
			});
		};
		
		/**
		 * @override
		 */
		MultiTextContentElementPanel.prototype.disable = function () {
			MultiParametersPanel.prototype.disable.call(this);
			// disable smartfields
			_.each(this._nameFields, function (smartfield) {
				smartfield.disable();
			});
		};
		
		return MultiTextContentElementPanel;
	});