define(["WFEditorWCR/workflow/panel/MultiParametersPanel",
		"WFEditorWCR/workflow/panel/MapPromiseUniqueDataProvider",
		"WFEditorWCR/workflow/panel/SmartNameDropdownPanel",
		"component/sdrop/SDrop",
		"packaging-content-management/ContentConstants",
		"WFEditorWCR/data/WebCenterTextContentProperty",
		"utils/Map",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery"
	],
	function (
		MultiParametersPanel,
		MapPromiseUniqueDataProvider,
		SmartNameDropdownPanel,
		SDrop,
		ContentConstants,
		WebCenterTextContentProperty,
		Map,
		i18n,
		_,
		$) {

		"use strict";

		/**
		 * @class
		 * @exports MultiTextContentFieldPanel
		 * @augments MultiParametersPanel
		 * 
		 * a panel component used for display of text content language mapping
		 * @param {String} parameterId 
		 * 							- id of the parameter for this panel
		 * @param {String} nameLabel       
		 * 							- label for the text content field to be displayed
		 * @param {String} valueLabel       
		 * 							- label for the field value to be displayed
		 * @param {Map.<String,WebCenterTextContentProperty>}
		 * 							-  a map for the fields required for Content Settings
		 * @param {WorkflowSmartNameDictionaryService} smartNameDictionaryService 
		 * 							- the smartname dictionary for smartnames in value fields
		 * @param {WebCenterAPI} webCenterAPI	
		 * 							- webcenter inferface
		 * @param {Boolean} isWrapped 
		 * 							- if false, multiple parameters are passed, if true, one parameter with subparameters is passed
		 */
		var MultiTextContentFieldPanel = function (parameterId, nameLabel, valueLabel, settingsMap, smartNameDictionaryService, webCenterAPI, isWrapped) {
			MultiParametersPanel.call(this, parameterId, nameLabel, valueLabel, smartNameDictionaryService, isWrapped, settingsMap.entries().length);

			/**
			 * @member {WebCenterAPI}
			 * @private
			 */
			this._webCenterAPI = webCenterAPI;
			
			/**
			 * @member {Map.<String,WebCenterTextContentProperty>}
			 * @private
			 */
			this._settingsMap = settingsMap;
		};

		MultiTextContentFieldPanel.prototype = Object.create(MultiParametersPanel.prototype);
		MultiTextContentFieldPanel.prototype.constructor = MultiTextContentFieldPanel;
		
		/**
		 * initialize text content field
		 * 
		 * @override
		 */
		MultiTextContentFieldPanel.prototype.initNameField = function (param, ctx) {
			
			var nameField = $('<select />');
			ctx.append($('<div />').append(nameField));
			
			var options = {
					onChangedByUser: _.bind(function(event ) {
						param.setName(nameField.val());
						var rowIndex = this._nameFields.indexOf(sDrop);
						var valueField = this._valueFields[rowIndex];
						this._updateTextContentValueDropDown(nameField.val(), valueField, param, true);
					}, this),
					autoload: false,
					autoloadOnFirstUsage: true,
					manageStyleAndWidth: false,
					initializeDataProvider: false
				};
				if (param && param.getName()) {
					options.initialValue = param.getName();
				}
				
				var deferredTCPropRequest = $.Deferred();
				var tcPropPromise = deferredTCPropRequest.promise();
				
				deferredTCPropRequest.resolve(this._settingsMap);

				if (!this._sharedDataProvider) {
					this._sharedDataProvider = new MapPromiseUniqueDataProvider({
						'promise': tcPropPromise
					});
				}

				var sDrop = new SDrop(nameField, "", this._sharedDataProvider, options);
				if (!this._isEnabled) {
					sDrop.setDisabled(true);
				}

				return sDrop;
		};

		/**
		 * initialize value field for a text content field
		 * 
		 * @override
		 */
		MultiTextContentFieldPanel.prototype.initValueField = function (param, ctx){
			//Create default dropdown option
			var options = [];
			var valueField = new SmartNameDropdownPanel(param.getName(), undefined, options, this._smartNameDictionaryService);
			valueField.setParameters([param]);
			if( param.getName().length > 0 ){
				this._updateTextContentValueDropDown(param.getName(), valueField, param, false);
			}

			//Default styling for smartname dropdown not required in multi parameter panel
			valueField.removeStlying();
			ctx.append($('<div />').append(valueField.getContext()));
			
			if (!this._isEnabled) {
				valueField.disable();
			}

			return valueField;
		};
		
		/**
		 * Updates the value dropdown options according to field selection
		 * 
		 * @param selectedField
		 * @param valueDropdown
		 * @param param
		 * @param setDefault
		 */
		MultiTextContentFieldPanel.prototype._updateTextContentValueDropDown = function(selectedField, valueDropdown, param, setDefault){
			var options = [];

			if(valueDropdown){
				valueDropdown.clearOptions();
				switch(selectedField){
				case ContentConstants.UpdateField.INVARIANT:
					options = [{
						value: "true",
						text: i18n.t('tcresetstatus.true')
					},  {
						value: "false",
						text:  i18n.t('tcresetstatus.false')
					}];
					break;
				case ContentConstants.UpdateField.ELEMENT_TYPE:
					this._webCenterAPI.getTextContentElementTypeList().done(_.bind(function (elementTypeMap) {
						//Populate element type options 
						_.each(elementTypeMap.values(), function (elementType) {
							if(elementType.getId() !== "ALL_ELEMENT_TYPES"){
								options.push({
									text: elementType.getName(),
									value: elementType.getId()
								});
							}
						}, this);
						//Set the default value for param
						if( setDefault && options.length > 0){
							param.setValue(options[0].value);
						}
						valueDropdown.setOptions(options);
					}, this));
					break;
				case ContentConstants.UpdateField.APPROVAL_TYPE:
					options = [{
						value: ContentConstants.ApprovalSetup.DEFAULT,
						text:  i18n.t('tcapprovaltype.default')
					}, {
						value: ContentConstants.ApprovalSetup.ONE_OF_ROLE,
						text:  i18n.t('tcapprovaltype.oneofrole')
					}, {
						value: ContentConstants.ApprovalSetup.ONE_OF_GROUP,
						text:  i18n.t('tcapprovaltype.allofroleonepergroup')
					}];
					break;
				case ContentConstants.UpdateField.TEXTTYPE:
					options = [{
						value: ContentConstants.TextType.RICHTEXT,
						text:  i18n.t('tctexttype.richtext')
					},  {
						value: ContentConstants.TextType.SIMPLE,
						text:  i18n.t('tctexttype.simple')
					}];
					break;
				default:
					break;
				}

				//Set the default value for param
				if(setDefault && options.length > 0){
					param.setValue(options[0].value);
				}
				valueDropdown.setOptions(options);
			}
		};
		
		/**
		 * @override
		 */
		MultiTextContentFieldPanel.prototype.resize = function () {
			if (this._isVisible) {
				_.each(this._valueFields, _.bind(function (smartNameDropdown) {
					smartNameDropdown.resize();
				}, this));
			}
		};
		
		/**
		 * close any opened SDrop when moving/resizing
		 * @Override
		 */
		MultiTextContentFieldPanel.prototype.startInteractive = function () {
			_.each(this._nameFields, function (sdrop) {
				sdrop.disable(null, true);
			});
			_.each(this._valueFields, function (smartNameDropdown) {
				smartNameDropdown.startInteractive();
			});
		};
		
		/**
		 * this method is called right before the panel is removed from the dom
		 * @override
		 */
		MultiTextContentFieldPanel.prototype.destroy = function () {

			_.each(this._nameFields, function (nameField) {
				nameField.destroy();
			});
			
			_.each(this._valueFields, _.bind(function (valueField) {
				valueField.destroy();
			}, this));
		};
		
		/**
		 * Entry is removed, now clean up components
		 * param {Number} - index of row where items are removed
		 */
		MultiTextContentFieldPanel.prototype.destroyComponentsOnEntry = function (index) {
			this._nameFields[index].destroy();
			this._valueFields[index].destroy();
		};
		
		/**
		 * @override
		 */
		MultiTextContentFieldPanel.prototype.disable = function () {
			MultiParametersPanel.prototype.disable.call(this);
			this._element.find(".sdrop").addClass("disabled");
			_.each(this._nameFields, function (sdrop) {
				sdrop.setDisabled(true);
			});
		};
		
		return MultiTextContentFieldPanel;
	});