define(["WFEditorWCR/workflow/panel/MultiParametersPanel",
		"WFEditorWCR/workflow/panel/MapPromiseUniqueDataProvider",
		"WFEditorWCR/workflow/panel/SmartNameDropdownPanel",
		"packaging-content-management/ContentConstants",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery",
		"wcr/sdrop"
	],
	function (
		MultiParametersPanel,
		MapPromiseUniqueDataProvider,
		SmartNameDropdownPanel,
		ContentConstants,
		i18n,
		_,
		$) {

		"use strict";

		/**
		 * @class
		 * @exports MultiTextContentLanguagePanel
		 * @augments MultiParametersPanel
		 * 
		 * a panel component used for display of text content language mapping
		 * @param {String} parameterId 
		 * 							- id of the parameter for this panel
		 * @param {String} nameLabel       
		 * 							- label for the language index to be displayed
		 * @param {String} valueLabel       
		 * 							- label for the language string to be displayed
		 * @param {WorkflowSmartNameDictionaryService} smartNameDictionaryService 
		 * 							- the smartname dictionary for smartnames in value fields
		 * @param {WebCenterAPI} webCenterAPI	
		 * 							- webcenter inferface
		 * @param {Boolean} isWrapped 
		 * 							- if false, multiple parameters are passed, if true, one parameter with subparameters is passed
		 */
		var MultiTextContentLanguagePanel = function (parameterId, nameLabel, valueLabel, smartNameDictionaryService, webCenterAPI, isWrapped) {
			MultiParametersPanel.call(this, parameterId, nameLabel, valueLabel, smartNameDictionaryService, isWrapped);

			/**
			 * @member {WebCenterAPI}
			 * @private
			 */
			this._webCenterAPI = webCenterAPI;
		};

		MultiTextContentLanguagePanel.prototype = Object.create(MultiParametersPanel.prototype);
		MultiTextContentLanguagePanel.prototype.constructor = MultiTextContentLanguagePanel;
		
		
		/**
		 * initialize name field i.e language index
		 * @override
		 */
		MultiTextContentLanguagePanel.prototype.initNameField = function (param, ctx) {
			var nameField = MultiParametersPanel.prototype.initNameField.call(this, param, ctx);
			
			//Set the default value for language index
			//it is sequential number that is no already entered in any fields
			if(!param.getName() || param.getName().length === 0){
				var langIndex = 1;
				var langIndexArray = [];
				var parameters = this._subParameters;
				for(var i = 0; i < parameters.length; i++){
					langIndexArray.push(parameters[i].getName());
				}
				while($.inArray(''+langIndex, langIndexArray) != -1){
					langIndex++;
				}
				
				param.setName(''+langIndex);
				nameField.val(langIndex);
			}
			
			return nameField;
		};

		/**
		 * initialize value field for language i.e language description
		 * @override
		 */
		MultiTextContentLanguagePanel.prototype.initValueField = function (param, ctx){
			//Create default dropdown option
			var options = [];
				options.push({
					text: i18n.t('wfui.textcontent.languagedefaultlabel'),
					value: ""
				});
			var smartNameDropdownPanel = new SmartNameDropdownPanel(param, undefined, options,this._smartNameDictionaryService);
			//Default styling for smartname dropdown not required in multi parameter panel
			smartNameDropdownPanel.removeStlying();
			this._webCenterAPI.getTextContentLanguageList().done(_.bind(function (langMap) {
				//Update the dropdown with PCM language list
				options = [];
				_.each(langMap.get(ContentConstants.LanguageType.ALL).values(), function (lang) {
					options.push({
						text: lang.getName(),
						value: lang.getId()
					});
				});
				smartNameDropdownPanel.setOptions(options);
				smartNameDropdownPanel.setParameters([param]);
			}, this));
			ctx.append($('<div />').append(smartNameDropdownPanel.getContext()));
			
			if (!this._isEnabled) {
				smartNameDropdownPanel.disable();
			}
			
			return smartNameDropdownPanel;
		};
		
		/**
		 * @override
		 */
		MultiTextContentLanguagePanel.prototype.resize = function () {
			if (this._isVisible) {
				_.each(this._valueFields, _.bind(function (smartNameDropdown) {
					smartNameDropdown.resize();
				}, this));
			}
		};
		
		/**
		 * close any opened SDrop when moving/resizing
		 * @Override
		 */
		MultiTextContentLanguagePanel.prototype.startInteractive = function () {
			_.each(this._valueFields, function (smartNameDropdown) {
				smartNameDropdown.startInteractive();
			});
		};
		
		/**
		 * this method is called right before the panel is removed from the dom
		 * @override
		 */
		MultiTextContentLanguagePanel.prototype.destroy = function () {
			_.each(this._valueFields, _.bind(function (smartNameDropdown) {
				smartNameDropdown.destroy();
			}, this));
		};
		
		/**
		 * Entry is removed, now clean up components
		 * param {Number} - index of row where items are removed
		 */
		MultiTextContentLanguagePanel.prototype.destroyComponentsOnEntry = function (index) {
			this._valueFields[index].destroy();
		};
		
		return MultiTextContentLanguagePanel;
	});