define(["WFEditorWCR/workflow/panel/MultiParametersPanel",
		"WFEditorWCR/data/WebCenterTextContentProperty",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"component/sdrop/dataproviders/MapPromiseDataProvider",
		"component/sdrop/SDrop",
		"WFEditorWCR/workflow/panel/SmartNameDropdownPanel",
		"packaging-content-management/ContentConstants",
		"utils/Map",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery"
	],
	function (
		MultiParametersPanel,
		WebCenterTextContentProperty,
		WebCenterParameter,
		MapPromiseDataProvider,
		SDrop,
		SmartNameDropdownPanel,
		ContentConstants,
		Map,
		i18n,
		_,
		$) {

		"use strict";

		/**
		 * @class
		 * @exports MultiTextContentPermissionPanel
		 * @augments MultiParametersPanel
		 * 
		 * a panel component used for display of text content permission mapping
		 * @param {String} parameterId 
		 * 							- id of the parameter for this panel
		 * @param {String} permLabel       
		 * 							- label for the TCP permission to be displayed
		 * @param {String} langLabel       
		 * 							- label for the language string to be displayed
		 * @param {String} roleLabel       
		 * 							- label for the role to be displayed
		 * @param {WorkflowSmartNameDictionaryService} smartNameDictionaryService 
		 * 							- the smartname dictionary for smartnames in value fields
		 * @param {WebCenterAPI} webCenterAPI	
		 * 							- webcenter inferface
		 * @param {Boolean} isWrapped 
		 * 							- if false, multiple parameters are passed, if true, one parameter with subparameters is passed
		 */
		var MultiTextContentPermissionPanel = function (parameterId, permLabel, langLabel, roleLabel, smartNameDictionaryService, webCenterAPI, isWrapped) {
			MultiParametersPanel.call(this, parameterId, permLabel, langLabel, smartNameDictionaryService, isWrapped);

			/**
			 * @member {WebCenterAPI}
			 * @private
			 */
			this._webCenterAPI = webCenterAPI;

			/**
			 * @member {JQueryObject} the div this panel will render in
			 * @private
			 */
			this._element = $('<div/>');

			this._element.addClass("MultiParametersPanel MultiColumnPanel");

			/**
			 * @member {JQueryObject} the context this panel will render in
			 * @private
			 */
			this._context = $('<div/>');
			this._context.append($("<div />").addClass("clearfix header").append($("<div />").addClass("has-3-cols offsetRightDeleteButton")
				.append($("<div />").addClass("nameCol").append($("<label/>").append(permLabel)))
				.append($("<div />").addClass("valueCol").append($("<label/>").append(langLabel)))
				.append($("<div />").addClass("roleCol").append($("<label/>").append(roleLabel)))));

			this._element.append(this._context);

			this._btn = $('<a />').addClass("AddBtn")
				.attr("disabled", !this._isEnabled).addClass("btn btn-sm btn-primary")
				.append($('<i />').addClass("icon icon-WCR_Actions-Add icon-small"))
				.click(_.bind(function () {
					if (this._isEnabled) {
						this.addEntry();
					}
				}, this));

			this._element.append(this._btn);

			/**
			 * @member {Array.<SmartField>} listObj
			 * @private
			 */
			this._roleFields = [];
		};

		MultiTextContentPermissionPanel.prototype = Object.create(MultiParametersPanel.prototype);
		MultiTextContentPermissionPanel.prototype.constructor = MultiTextContentPermissionPanel;

		/**
		 * initialize PCM permission field
		 * @override
		 */
		MultiTextContentPermissionPanel.prototype.initNameField = function (param, ctx) {
			ctx.removeClass();
			ctx.addClass("has-3-cols offsetRightDeleteButton");

			var subParam = param.getSubparameters();
			var permParam = subParam[0];
			var langParam = subParam[1];

			var nameField = $('<select />');
			ctx.append($('<div />').append(nameField));
			var options = {
				onChangedByUser: _.bind(function () {
					permParam.setValue(nameField.val());
					//Disable/enable language dropdown according to Permission selection
					var rowIndex = this._nameFields.indexOf(sDrop);
					var valueField = this._valueFields[rowIndex];
					if (nameField.val() === ContentConstants.Permission.CONTENTMANAGER) {
						valueField.clearOptions();
						valueField.disable();
					} else {
						if (!valueField.isEnabled()) {
							valueField.enable();
							this._populateLangOptions(valueField, langParam);
						}
					}
				}, this),
				autoload: false,
				autoloadOnFirstUsage: true,
				manageStyleAndWidth: false,
				initializeDataProvider: false
			};
			if (permParam && permParam.getValue()) {
				options.initialValue = permParam.getValue();
			}

			var deferredTCPropRequest = $.Deferred();
			var tcPropPromise = deferredTCPropRequest.promise();

			var tcPropMap = new Map();
			tcPropMap.add(ContentConstants.Permission.EDITOR, new WebCenterTextContentProperty(ContentConstants.Permission.EDITOR, i18n.t('wfui.settcfield.tcp.EDITOR')));
			tcPropMap.add(ContentConstants.Permission.APPROVER, new WebCenterTextContentProperty(ContentConstants.Permission.APPROVER, i18n.t('wfui.settcfield.tcp.APPROVER')));
			tcPropMap.add(ContentConstants.Permission.CONTENTMANAGER, new WebCenterTextContentProperty(ContentConstants.Permission.CONTENTMANAGER, i18n.t('wfui.settcfield.tcp.CONTENTMANAGER')));

			deferredTCPropRequest.resolve(tcPropMap);

			if (!this._dataProvider) {
				this._dataProvider = new MapPromiseDataProvider({
					'promise': tcPropPromise
				});
			}

			var sDrop = new SDrop(nameField, "", this._dataProvider, options);
			if (!this._isEnabled) {
				sDrop.setDisabled(true);
			}

			return sDrop;
		};

		/**
		 * @override
		 */
		MultiTextContentPermissionPanel.prototype.addEntry = function (param, noFitContent) {
			if (!(param instanceof WebCenterParameter)) {
				// if parameter does not exist(empty entry), create a new one
				var subParamArr = [];
				subParamArr.push(new WebCenterParameter(ContentConstants.Permission.TCP, "", ""));
				subParamArr.push(new WebCenterParameter(ContentConstants.Permission.LANGUAGE, "", ""));
				subParamArr.push(new WebCenterParameter(ContentConstants.Permission.ROLE, "", ""));
				param = new WebCenterParameter("", "", "", subParamArr);

				//Generate unique param name (a sequence number)
				var paramName = 1;
				var exisitingParamNames = [];
				for (var i = 0; i < this._subParameters.length; i++) {
					exisitingParamNames.push(this._subParameters[i].getName());
				}
				while ($.inArray('' + paramName, exisitingParamNames) != -1) {
					paramName++;
				}
				param.setName('' + paramName);

				this._subParameters.push(param);
			}

			MultiParametersPanel.prototype.addEntry.call(this, param, noFitContent);
		};

		/**
		 * @override
		 */
		MultiTextContentPermissionPanel.prototype._addEntriesFromParameters = function () {
			_.each(this._subParameters, _.bind(function (subparam) {
				//UTGO:: Permission panel row should have 2nd level of sub parameters
				if (subparam.getSubparameters().length > 0) {
					this.addEntry(subparam, true);
				}
			}, this));

			// if no subparameters: add empty entry
			if (this._subParameters.length === 0) {
				this.addEntry();
			}
			this._isDirtyParameters = false;

			this.resize();
			this._fitContent();
		};
		
		/**
		 * @override
		 */
		MultiTextContentPermissionPanel.prototype.getParameters = function () {
			// When the parameters are asked, filter out the empty ones first
			for (var i = this._subParameters.length - 1; i >= 0; i--) {
				var subParameter = this._subParameters[i];
				var rowParam = subParameter.getSubparameters();
				var tcpParam = rowParam[0];
				
				if (!tcpParam.getValue() || tcpParam.getValue().length === 0) {
					this._subParameters.splice(i, 1);
				}
			}
			return MultiParametersPanel.prototype.getParameters.call(this);
		};

		/**
		 * initialize field for a language and role
		 * @override
		 */
		MultiTextContentPermissionPanel.prototype.initValueField = function (param, ctx) {
			//Create language dropdown option
			var subParam = param.getSubparameters();
			var fieldParam = subParam[1];
			var options = [];
			var langDropdownField = new SmartNameDropdownPanel(fieldParam, undefined, options, this._smartNameDictionaryService);
			langDropdownField.setParameters([fieldParam]);
			//Default styling for smartname dropdown not required in multi parameter panel
			langDropdownField.removeStlying();
			ctx.append($('<div />').append(langDropdownField.getContext()));

			if (subParam[0].getValue() === ContentConstants.Permission.CONTENTMANAGER) {
				langDropdownField.clearOptions();
				langDropdownField.disable();
			} else {
				this._populateLangOptions(langDropdownField, fieldParam);
			}

			//Add role field
			var roleField = this.initRoleField(param, ctx);
			this._roleFields.push(roleField);

			if (!this._isEnabled) {
				langDropdownField.disable();
			}

			return langDropdownField;
		};

		/**
		 * initialize field for a role
		 */
		MultiTextContentPermissionPanel.prototype.initRoleField = function (param, ctx) {
			//Create role dropdown option
			var subParam = param.getSubparameters();
			var fieldParam = subParam[2];
			var options = [];
			var roleDropdownPanel = new SmartNameDropdownPanel(fieldParam, undefined, options, this._smartNameDictionaryService);
			roleDropdownPanel.setParameters([fieldParam]);
			//Default styling for smartname dropdown not required in multi parameter panel
			roleDropdownPanel.removeStlying();
			ctx.append($('<div />').append(roleDropdownPanel.getContext()));

			this._webCenterAPI.getUsersGroupsRoles().done(_.bind(function (users, groups, roles) {
				_.each(roles.values(), function (role) {
					options.push({
						text: role.getName().trim(),
						value: role.getName().trim()
					});
				});

				//Set the default value for field
				if (fieldParam.getValue().length === 0 && options.length > 0) {
					fieldParam.setValue(options[0].value);
				}

				roleDropdownPanel.setOptions(options);
			}, this));

			if (!this._isEnabled) {
				roleDropdownPanel.disable();
			}
			return roleDropdownPanel;
		};

		/**
		 * Fetches the text content languages from system and add them in the dropdown field
		 * 
		 * @param langDropdownField
		 * @param param
		 */
		MultiTextContentPermissionPanel.prototype._populateLangOptions = function (langDropdownField, param) {

			this._webCenterAPI.getTextContentLanguageList().done(_.bind(function (langMap) {
				var options = [{
					text: i18n.t("wfui.textcontent.alllang"),
					value: "[LANGUAGE]"
				}];

				//Update the dropdown with PCM language list
				_.each(langMap.get(ContentConstants.LanguageType.ALL).values(), function (lang) {
					options.push({
						text: lang.getName(),
						value: lang.getId()
					});
				});

				langDropdownField.setOptions(options);
				
				//Set the default value for field
				if (param && param.getValue().length === 0 && options.length > 0) {
					param.setValue(options[0].value);
				}
				
				langDropdownField.setParameters([param]);
				
			}, this));
		};

		/**
		 * @override
		 */
		MultiTextContentPermissionPanel.prototype.resize = function () {
			if (this._isVisible) {
				_.each(this._roleFields, _.bind(function (smartNameDropdown) {
					smartNameDropdown.resize();
				}, this));

				_.each(this._valueFields, _.bind(function (smartNameDropdown) {
					smartNameDropdown.resize();
				}, this));
			}
		};

		/**
		 * @Override
		 */
		MultiTextContentPermissionPanel.prototype.startInteractive = function () {

			_.each(this._nameFields, function (sdrop) {
				sdrop.disable(null, true);
			});

			_.each(this._roleFields, function (smartNameDropdown) {
				smartNameDropdown.startInteractive();
			});
			_.each(this._valueFields, function (smartNameDropdown) {
				smartNameDropdown.startInteractive();
			});
		};

		/**
		 * @override
		 */
		MultiTextContentPermissionPanel.prototype.destroy = function () {

			_.each(this._nameFields, function (nameField) {
				nameField.destroy();
			});

			_.each(this._valueFields, _.bind(function (valueField) {
				valueField.destroy();
			}, this));

			_.each(this._roleFields, _.bind(function (roleField) {
				roleField.destroy();
			}, this));
		};

		/**
		 * @override
		 */
		MultiTextContentPermissionPanel.prototype.destroyComponentsOnEntry = function (index) {
			this._nameFields[index].destroy();
			this._roleFields[index].destroy();
			this._valueFields[index].destroy();
		};

		/**
		 * @override
		 */
		MultiTextContentPermissionPanel.prototype.disable = function () {
			MultiParametersPanel.prototype.disable.call(this);

			this._element.find(".sdrop").addClass("disabled");
			_.each(this._nameFields, function (sdrop) {
				sdrop.setDisabled(true);
			});

			// disable smartfields
			_.each(this._roleFields, function (smartfield) {
				smartfield.disable();
			});
		};

		/**
		 * We want at least 570px 
		 * @override
		 * @returns {Number}
		 */
		MultiTextContentPermissionPanel.prototype.getPreferredWidth = function () {
			return 570;
		};

		return MultiTextContentPermissionPanel;
	});