define(["WFEditorWCR/workflow/panel/WebCenterCompositePanelComponent",
		"WFEditorWCR/workflow/panel/WebCenterPanelConstants",
		"utils/Observer",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery"
	],
	function (WebCenterCompositePanelComponent,
		WebCenterPanelConstants,
		Observer,
		i18n,
		_,
		$) {

		"use strict";

		/**
		 * @class
		 * @exports {RadioGroupPanel}
		 * @augments CompositePanelComponent
		 * A Panel Component that manager different RadioPanel. Only one of the sub panel's radio button will be checked
		 * The value of this panel's parameter will be equal to the name of the parameter of the checked sub panel.
		 * 
		 * @param {<String>}  paramId           - id of the parameter for this panel
		 * @param {Boolean}   subParameterState - should the sub parameters remain visible if the radio button is not checked
		 * @param {String}	  [legend]	        - the language key for the title of the field set
		 */
		var RadioGroupPanel = function (paramId, subParametersState, legend) {
			WebCenterCompositePanelComponent.call(this, [paramId]);
			Observer.mixin(this);

			/**
			 * @member {Boolean}
			 * @private
			 */
			this._subParametersStateVisible = WebCenterPanelConstants.SUBPARAMETERSTATE.VISIBLE === subParametersState;

			/**
			 * @member {RadioPanel} the radio panel that is currently marked as selected
			 * @private
			 */
			this._checkedSubPanel = null;

			/**
			 * @member {String} id of the parameter for this panel
			 * @private
			 */
			this._paramId = paramId;

			/**
			 * @member {Function} function called when a radio button is checked
			 * @private
			 */
			this._checkedFn = _.bind(function (component) {
				this._fitContent();
				this._checkedSubPanel.check(false, this._subParametersStateVisible);
				this._checkedSubPanel = component;
				this._checkedSubPanel.check(true, this._subParametersStateVisible);
			}, this);

			this._fieldSet = $("<fieldset>");

			this._legend = $("<legend>").text(i18n.t(legend)).appendTo(this._fieldSet);
			this._element.append(this._fieldSet);


		};

		RadioGroupPanel.prototype = Object.create(WebCenterCompositePanelComponent.prototype);
		RadioGroupPanel.prototype.constructor = RadioGroupPanel;

		/**
		 * @override
		 * @param {RadioPanel} abstractPanelComponent
		 */
		RadioGroupPanel.prototype.add = function (abstractPanelComponent) {
			this._subPanels.push(abstractPanelComponent);

			this._observe(abstractPanelComponent, "checked", this._checkedFn);

			this._fieldSet.append(abstractPanelComponent.getContext());
			if (this._subPanels.length === 1) {
				abstractPanelComponent.check(true, this._subParametersStateVisible);
				this._checkedSubPanel = abstractPanelComponent;
			} else {
				abstractPanelComponent.check(false, this._subParametersStateVisible);
			}
		};

		/**
		 * @override
		 */
		RadioGroupPanel.prototype.setParameters = function (parameters) {
			WebCenterCompositePanelComponent.prototype.setParameters.call(this, parameters);
			// check the right checkbox and display the corresponding sub panel
			var param = this.getParameterByName(this._paramId);
			_.each(this._subPanels, function (panel) {
				if (panel.getParamId() === param.getValue() && panel !== this._checkedSubPanel) {
					this._checkedSubPanel.check(false, this._subParametersStateVisible);
					this._checkedSubPanel = panel;
					this._checkedSubPanel.check(true, this._subParametersStateVisible);
					return;
				}
			}, this);
		};


		/**
		 * @override
		 */
		RadioGroupPanel.prototype.getParameters = function () {
			if (this._paramId) {
				this.getParameterByName(this._paramId).setValue(this._checkedSubPanel.getParamId());
			}
			return WebCenterCompositePanelComponent.prototype.getParameters.call(this);
		};


		return RadioGroupPanel;

	}
);