define(["component/smartnames/smartfield/smartfield",
		"component/smartnames/encoding/smartnameencoding",
		"WFEditor/workflow/panel/AbstractPanelComponent",
		"WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService",
		"jquery",
		"underscore"
	],
	function (
		SmartField,
		SmartNameEncoding,
		AbstractPanelComponent,
		WorkflowSmartNameDictionaryService,
		$,
		_
	) {

		"use strict";

		/**
		 * @class SmartNameDropDownPanel
		 * @augments AbstractPanelComponent
		 * 
		 * A panel that consists of a smart name enabled dropdown
		 * 
		 * @param {String} paramId               - id of the paramter
		 * @param {String} label                 - label
		 * @param {Map.<String, Object>} options - The select options - the objects should have text and a value property
		 * @param {WorkflowSmartNameDictionaryService} [smartNameDictionaryService] - the smartname dictionary for the field if you want to override the default one
		 */
		var SmartNameDropdownPanel = function (paramId, label, options, smartNameDictionaryService) {
			AbstractPanelComponent.call(this, [paramId]);

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._inputDiv = $("<div/>").addClass(" col-sm-12 col-lg-9");

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._smartFieldInput = $("<select/>");

			/**
			 * @member {WorkflowSmartNameDictionaryService} available smartnames
			 * @private
			 */
			this._smartNameDictionaryService = smartNameDictionaryService || new WorkflowSmartNameDictionaryService();

			/**
			 * @member {SmartField}
			 * @private
			 */
			this._smartField = new SmartField(this._smartFieldInput,
				false, // lazy init
				this._smartNameDictionaryService, // our dictionary
				_.bind(this._smartNameDictionaryService.displaySmartName, this._smartNameDictionaryService), // display in input field
				_.bind(this._smartNameDictionaryService.displaySmartNameSelectorSmartName, this._smartNameDictionaryService), // display in selector
				SmartNameEncoding.WORKFLOWEDITOR // encoding
			);

			this._smartField.setWidth("100%");


			this._smartField.onValueChange(_.bind(function (val) {
				this._updateParameter(val);
			}, this));

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			if (label) {
				this._label = $("<label/>").text(label).addClass("col-sm-12 col-lg-3");

				this._element.append(this._label).addClass("form-group");
			}

			if (options) {
				this.setOptions(options);
			}

		};

		SmartNameDropdownPanel.prototype = Object.create(AbstractPanelComponent.prototype);
		SmartNameDropdownPanel.prototype.constructor = AbstractPanelComponent;

		/**
		 * @override
		 */
		SmartNameDropdownPanel.prototype.setParameters = function (parameters) {
			AbstractPanelComponent.prototype.setParameters.call(this, parameters);
			this._updateSmartField();
		};

		/**
		 * Sets the options for the select
		 * @param {Map.<String, Object>} options - a map of id to option object. The option object should have a text and value property
		 */
		SmartNameDropdownPanel.prototype.setOptions = function (options) {
			_.each(options, function (option) {
				this._smartFieldInput.append($("<option/>").val(option.value).text(option.text));
			}, this);

			this._element.append(this._inputDiv.append(this._smartField.getJQElement()));
			this._updateSmartField();
		};
		
		/**
		 * Clears the options from the select
		 */
		SmartNameDropdownPanel.prototype.clearOptions = function () {
			this._smartFieldInput.empty();
			this._smartField.clear();
		};
		
		/**
		 * Updates the parameter - To be called when the smart field value  is updated
		 * 
		 * @param {String} val - Value to be used to update the parameter
		 */
		SmartNameDropdownPanel.prototype._updateParameter = function(val){
			if (this._parameters[0]) {
				this._parameters[0].setValue(val);
			}
		};
		
		/**
		 * Updates the smartname field with parameter -To be called when the parameter is updated
		 * 
		 */
		SmartNameDropdownPanel.prototype._updateSmartField = function(){
			if (this._parameters[0]) {
				this._smartField.setValue(this._parameters[0].getValue());
			}
		};
		
		/**
		 * Gets the smartname field
		 * @returns {SmartField}
		 */
		SmartNameDropdownPanel.prototype.getSmartField = function(){
			return this._smartField;
		};

		/**
		 * @Override
		 */
		SmartNameDropdownPanel.prototype.resize = function () {
			if (this._smartField) {
				this._smartField.setWidth("100%");
			}
		};

		/**
		 * @override
		 */
		SmartNameDropdownPanel.prototype.startInteractive = function () {
			if (this._smartField) {
				this._smartField.stopEditing();
			}
		};

		/**
		 * @override
		 */
		SmartNameDropdownPanel.prototype.disable = function () {
			AbstractPanelComponent.prototype.disable.call(this);
			this._smartField.disable();
		};
		
		/**
		 * @override
		 */
		SmartNameDropdownPanel.prototype.enable = function () {
			AbstractPanelComponent.prototype.enable.call(this);
			this._smartField.enable();
		};

		/**
		 * Removes css styling applied on the panel elements
		 */
		SmartNameDropdownPanel.prototype.removeStlying = function () {
			this._element.removeClass();
			this._inputDiv.removeClass();
			if (this._label) {
				this._label.removeClass();
			}
		};

		/**
		 * @override
		 */
		SmartNameDropdownPanel.prototype.destroy = function () {
			this._smartField.destroy();
		};

		return SmartNameDropdownPanel;
	});