define(["WFEditor/workflow/panel/AbstractPanelComponent",
		"WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService",
		"component/smartnames/smartfield/smartfield",
		"component/smartnames/encoding/smartnameencoding",
		"jquery",
		"underscore"
	],
	function (AbstractPanelComponent,
		WorkflowSmartNameDictionaryService,
		SmartField,
		SmartNameEncoding,
		$,
		_) {

		"use strict";

		/**
		 * @class
		 * @exports SmartNameInputField
		 * @augments PanelComonent
		 * 
		 * A smartname input field
		 * 
		 * @param {String} parameterId - id of the parameter for this panel
		 * @param {String} label       - label to be displayed
		 * @param {WorkflowSmartNameDictionaryService} [smartNameDictionaryService] - the smartname dictionary for the field if you want to override the default one
		 */
		var SmartNameInputField = function (parameterId, label, smartNameDictionaryService) {
			AbstractPanelComponent.call(this, [parameterId]);


			/**
			 * @member {JQueryObject} input field
			 * @private
			 */
			this._input = $('<input type="text">');


			/**
			 * @member {JQueryObject} input field div
			 * @private
			 */
			this._inputDiv = $("<div>").append(this._input);
			this._element.addClass("form-group").append(this._inputDiv);

			/**
			 * @member {JQueryObject} label
			 * @private
			 */
			this._label = $('<label/>').addClass("col-sm-12 col-lg-3").text(label);

			if (label) {
				this._element.prepend(this._label);
				this._inputDiv.addClass(" col-sm-12 col-lg-9");
			} else {
				this._inputDiv.addClass(" col-sm-12 col-lg-12");

			}


			/**
			 * @member {WorkflowSmartNameDictionaryService} available smartnames
			 * @private
			 */
			this._smartNameDictionaryService = smartNameDictionaryService || new WorkflowSmartNameDictionaryService();

			/**
			 * @member {SmartField} See init
			 * @private
			 */
			this._smartField = null;

			/**
			 * @member {Parameter} See setParameters and init
			 * @private
			 */
			this._parameter = null;
		};

		SmartNameInputField.prototype = Object.create(AbstractPanelComponent.prototype);
		SmartNameInputField.prototype.constructor = SmartNameInputField;

		/**
		 * @override
		 */
		SmartNameInputField.prototype.init = function () {
			AbstractPanelComponent.prototype.init.call(this);


			// smartfields need already added to DOM input fields
			this._smartField = new SmartField(this._input,
				false, // lazy init
				this._smartNameDictionaryService, // our dictionary
				_.bind(this._smartNameDictionaryService.displaySmartName, this._smartNameDictionaryService), // display in input field
				_.bind(this._smartNameDictionaryService.displaySmartNameSelectorSmartName, this._smartNameDictionaryService), // display in selector
				SmartNameEncoding.WORKFLOWEDITOR // encoding
			);

			this._smartField.onValueChange(_.bind(function (value) {
				if (this._parameter) {
					this._parameter.setValue(value);
				}
			}, this));

			// if parameter already filled in
			if (this._parameter && this._parameter.getValue()) {
				this._smartField.setValue(this._parameter.getValue());
			}

			if (!this._isEnabled) {
				this._smartField.disable();
			}

			this.resize();
		};

		/**
		 * @override
		 */
		SmartNameInputField.prototype.destroy = function () {
			if (this._smartField) {
				this._smartField.destroy();
			}
		};

		/**
		 * @override
		 */
		SmartNameInputField.prototype.setParameters = function (parameters) {
			AbstractPanelComponent.prototype.setParameters.call(this, parameters);
			var param = parameters[0];
			if (param) {
				this._parameter = param;
				// if smartfield already made update it
				if (this._smartField) {
					this._smartField.setValue(param.getValue());
				}
			}
		};

		/**
		 * @Override
		 */
		SmartNameInputField.prototype.resize = function () {
			if (this._smartField) {
				this._smartField.setWidth("100%");
			}
		};

		/**
		 * @override
		 */
		SmartNameInputField.prototype.disable = function () {
			AbstractPanelComponent.prototype.disable.call(this);
			if (this._smartField) {
				this._smartField.disable();
			}
		};


		/**
		 * @override
		 */
		SmartNameInputField.prototype.startInteractive = function () {
			if (this._smartField) {
				this._smartField.stopEditing();
			}
		};

		/**
		 * @override
		 */
		SmartNameInputField.prototype.enable = function () {
			AbstractPanelComponent.prototype.enable.call(this);
			if (this._smartField) {
				this._smartField.enable();
			}
		};

		/**
		 * Removes css styling applied on the panel elements
		 */
		SmartNameInputField.prototype.removeStlying = function () {
			this._element.removeClass();
			this._inputDiv.removeClass();
		};

		return SmartNameInputField;

	});