define(["underscore", "jquery", "WFEditorWCR/workflow/panel/WebCenterParameter", "tpl!WFEditor/workflow/panel/Parameter"], function (_, $, WebCenterParameter, parameter) {

	"use strict";

	/**
	 * @class
	 * @augments WebCenterParameter
	 * @exports TaskSpecificationsParameter
	 * 
	 * a wrapper parameter for the task specifications
	 * 
	 * @param {String}                     name           - id of the parameter
	 * @param {Array.<WebCenterAttribute>} specifications - specifications
	 * @param {Array.<Parameter>}          subparameters  - subParameters, there is one subparameter for each specification that has a value
	 */
	var TaskSpecificationsParameter = function (name, specifications, subparameters) {
		WebCenterParameter.call(this, name, "", "", subparameters || []);

		/**
		 * @param {Array.<WebCenterAttribute>}
		 * @private
		 */
		this._specifications = specifications;

		/**
		 * @param {Boolean} should the specifications that are not selected be added to the toString method?
		 * @private
		 */
		this._shouldAddNotSelectedSpecifications = false;

	};

	TaskSpecificationsParameter.prototype = Object.create(WebCenterParameter.prototype);
	TaskSpecificationsParameter.prototype.constructor = TaskSpecificationsParameter;

	/**
	 * is @param {id} a specification?
	 * @param   {String}  id 
	 * @returns {Boolean} 
	 */
	TaskSpecificationsParameter.prototype.isSpecification = function (id) {
		return this._specifications.has(id);
	};

	/**
	 * @override
	 */
	TaskSpecificationsParameter.prototype.updateFromXML = function (xml) {
		var name, value;
		_.each(xml.childNodes, function (child) {
			if (child.tagName === "Name") {
				name = child.textContent.trim();
			} else if (child.tagName === "Value") {
				value = child.textContent;
				if(!value) {
					// If the value is empty we know that the not selected specifications are also added.
					// In a perfect world there could be a property for this, but to be compatible with the XMLs created by the late java applet this is the quicker solution
					this._shouldAddNotSelectedSpecifications = true;
				}
			}
		}, this);
		if (name && value) {
			this._subparameters.push(new WebCenterParameter(name, value, value));
		}
	};

	/**
	 * @override
	 */
	TaskSpecificationsParameter.prototype.toString = function () {
		var xml = "";
		_.each(this._specifications.values(), function (spec) {
			var subParam = _.find(this._subparameters, function (param) {
				return param.getName() === spec.getId();
			}, this);
			if (subParam) {
				xml += parameter(subParam);
			} else if(this._shouldAddNotSelectedSpecifications) {
				xml += parameter(new WebCenterParameter(spec.getId()));
			}
		}, this);
		return xml;
	};

	/**
	 * @override
	 */
	TaskSpecificationsParameter.prototype.copy = function () {
		var copiedSubparams = [];
		_.each(this._subparameters, function (subParam) {
			copiedSubparams.push(subParam.copy());
		});
		var copyParam =  new TaskSpecificationsParameter(this._name, this._specifications, copiedSubparams);
		copyParam.shouldAddNotSelectedSpecifications(this._shouldAddNotSelectedSpecifications);
		return copyParam;
	};

	/**
	 * gets or sets whether or not the not selected specifications should be added to the xml
	 * @param {Boolean}
	 * @return {Boolean | undefined}
	 */
	TaskSpecificationsParameter.prototype.shouldAddNotSelectedSpecifications = function (bool) {
		if (typeof bool !== "undefined") {
			this._shouldAddNotSelectedSpecifications = bool;
		} else {
			return this._shouldAddNotSelectedSpecifications;
		}
	};
	
	/**
	 * @override
	 * @param   {Parameter} param 
	 * @returns {Boolean}   
	 */
	TaskSpecificationsParameter.prototype.equals = function (param) {
		if(!(param instanceof TaskSpecificationsParameter)) {
			return false;
		}
		return WebCenterParameter.prototype.equals.call(this, param) && (this._shouldAddNotSelectedSpecifications === param.shouldAddNotSelectedSpecifications());
	};

	return TaskSpecificationsParameter;
});
