define(["component/smartnames/smartfield/smartnamedictionaryservice",
		"component/smartnames/smartfield/smartnamedictionarysection",
		"component/smartnames/encoding/smartname",
		"WFEditorWCR/workflow/panel/WorkflowSmartNameConstants",
		"utils/webi18n!WFEditorWCR",
		"utils/Map",
		"underscore",
		"jquery"
	],
	function (SmartnameDictionaryService,
		SmartNameDictionarySection,
		SmartName,
		WorkflowSmartNameConstants,
		i18n,
		Map,
		_,
		$) {
		"use strict";

		/**
		 * @member {Array.<SmartName>} arrays of sorted smartnames
		 */
		var projectSmartNames;
		var projectAttributeSmartNames;
		var documentSmartNames;
		var textContentSmartNames;
		var documentAttributeSmartNames;
		var taskSpecificationsTaskTypeID;
		var taskSpecificationSmartNames;
		var linkSmartNames;
		var emailSmartNames;
		var dateSmartNames;
		var urlSmartNames;
		var userSmartNames;
		var rolesSmartNames;
		var groupsSmartNames;
		var usersSmartNames;
		var calculatedSmartNames;

		/**
		 * @member {Map<WebCenterAttribute>} the specifications of task type defined by taskTypeID
		 */
		var taskTypeSpecificationsMap;

		/**
		 * @member {Map<WebCenterAttribute>} the attributes of WebCenter
		 */
		var attributesMap;

		/**
		 * @member {Map<WebCenterAttribute>} the specifications of WebCenter
		 */
		var specificationsMap;

		/**
		 * @member {Map<WebCenterUser>} the users of WebCenter
		 */
		var usersMap;

		/**
		 * @member {Map<WebCenterGroup>} the groups of WebCenter
		 */
		var groupsMap;
		/**
		 * @member {Map<WebCenterRole>} the roles of WebCenter
		 */
		var rolesMap;

		/**
		 * (Default) Dictionary service for workflow smartnames.
		 * @param {WebCenterAPI} webCenterAPI    - The WebCenter API to fetch information
		 * @param {String} currentTaskTypeID - Id of the currently opened workflow (to fetch task specifications)
		 * @param {Array.<CustomSmartName>} [customSmartNames] - Custom smartnames if any that need to be added
		 */
		var WorkflowSmartNameDicationaryService = function (webCenterAPI, currentTaskTypeID, customSmartNames) {
			SmartnameDictionaryService.call(this);

			/**
			 * @member {WebCenterAPI}
			 */
			this._webCenterAPI = webCenterAPI;

			/**
			 * @member {String}
			 */
			this._taskTypeID = currentTaskTypeID;

			/**
			 * @member {Map.<String (category) -> Array.<SmartName>>}
			 */
			this._extraSmartNames = new Map();
			if (customSmartNames) {
				_.each(customSmartNames, function (customSmartName) {
					var list = this._extraSmartNames.get(customSmartName.getCategory());
					if (!list) {
						list = [];
						this._extraSmartNames.add(customSmartName.getCategory(), list);
					}
					list.push(new SmartName(customSmartName.getSmartName()));
				}, this);
			}
		};

		WorkflowSmartNameDicationaryService.prototype = Object.create(SmartnameDictionaryService.prototype);
		WorkflowSmartNameDicationaryService.prototype.constructor = WorkflowSmartNameDicationaryService;

		/**
		 * @override
		 */
		WorkflowSmartNameDicationaryService.prototype.getAllSections = function (onSectionsRetrievedCallbackSuccess, onSectionsRetrievedCallbackFail) {
			var sections = new Map();

			var category = null;

			if (this.addProjectProperties()) {
				category = WorkflowSmartNameConstants.PROJECT.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addProjectAttributes()) {
				category = WorkflowSmartNameConstants.PROJECTATTRIBUTES.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addDocumentProperties()) {
				category = WorkflowSmartNameConstants.DOCUMENT.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addTextContentProperties()) {
				category = WorkflowSmartNameConstants.TEXTCONTENT.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addDocumentAttributes()) {
				category = WorkflowSmartNameConstants.DOCUMENTATTRIBUTES.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addTaskSpecifications()) {
				category = WorkflowSmartNameConstants.SPECIFICATIONS.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addIteration()) {
				category = WorkflowSmartNameConstants.ITERATION.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addLinks()) {
				category = WorkflowSmartNameConstants.LINKS.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addEmails()) {
				category = WorkflowSmartNameConstants.EMAIL.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addDates()) {
				category = WorkflowSmartNameConstants.DATE.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addURLs()) {
				category = WorkflowSmartNameConstants.URL.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addUser()) {
				category = WorkflowSmartNameConstants.USER.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addRoles()) {
				category = WorkflowSmartNameConstants.TASKROLES.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addGroups()) {
				category = WorkflowSmartNameConstants.TASKGROUPS.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addUsers()) {
				category = WorkflowSmartNameConstants.TASKUSERS.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			if (this.addCalculatedSmartNames()) {
				category = WorkflowSmartNameConstants.CALCULATED.CATEGORY;
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			}

			// Extra smartnames
			_.each(this._extraSmartNames.entries(), function (category) {
				sections.add(category, new SmartNameDictionarySection(category, i18n.t(category)));
			});

			// Sort alfabetically
			var sortedSections = _.sortBy(sections.values(), function (section) {
				return section.getSectionLangTag();
			});
			onSectionsRetrievedCallbackSuccess(sortedSections);
		};

		/**
		 * @override
		 */
		WorkflowSmartNameDicationaryService.prototype.getSmartNamesForSection = function (sectionName, onSmartNamesRetrievedCallbackSuccess, onSmartNamesRetrievedCallbackFail) {
			switch (sectionName) {
			case WorkflowSmartNameConstants.PROJECT.CATEGORY:
				if (!projectSmartNames) {
					projectSmartNames = this._sortSmartNames(this._createSmartNames(WorkflowSmartNameConstants.PROJECT.SMARTNAMES));
				}
				this._extendSmartNamesWithExtraSmartNames(sectionName, projectSmartNames, onSmartNamesRetrievedCallbackSuccess);
				break;
			case WorkflowSmartNameConstants.PROJECTATTRIBUTES.CATEGORY:
				if (!projectAttributeSmartNames) {
					$.when(this._webCenterAPI.getAttributes()).done(_.bind(function (attributes) {
						attributesMap = attributes;
						projectAttributeSmartNames = this._createSmartNamesWithId(attributes, WorkflowSmartNameConstants.PROJECTATTRIBUTES.PREFIX);
						this._extendSmartNamesWithExtraSmartNames(sectionName, projectAttributeSmartNames, onSmartNamesRetrievedCallbackSuccess);
					}, this));
				} else {
					this._extendSmartNamesWithExtraSmartNames(sectionName, projectAttributeSmartNames, onSmartNamesRetrievedCallbackSuccess);
				}
				break;
			case WorkflowSmartNameConstants.DOCUMENT.CATEGORY:
				if (!documentSmartNames) {
					documentSmartNames = this._sortSmartNames(this._createSmartNames(WorkflowSmartNameConstants.DOCUMENT.SMARTNAMES));
				}
				this._extendSmartNamesWithExtraSmartNames(sectionName, documentSmartNames, onSmartNamesRetrievedCallbackSuccess);
				break;
			case WorkflowSmartNameConstants.TEXTCONTENT.CATEGORY:
				if (!textContentSmartNames) {
					textContentSmartNames = this._sortSmartNames(this._createSmartNames(WorkflowSmartNameConstants.TEXTCONTENT.SMARTNAMES));
				}
				this._extendSmartNamesWithExtraSmartNames(sectionName, textContentSmartNames, onSmartNamesRetrievedCallbackSuccess);
				break;
			case WorkflowSmartNameConstants.DOCUMENTATTRIBUTES.CATEGORY:
				if (!documentAttributeSmartNames) {
					$.when(this._webCenterAPI.getAttributes()).done(_.bind(function (attributes) {
						attributesMap = attributes;
						documentAttributeSmartNames = this._createSmartNamesWithId(attributes, WorkflowSmartNameConstants.DOCUMENTATTRIBUTES.PREFIX);
						this._extendSmartNamesWithExtraSmartNames(sectionName, documentAttributeSmartNames, onSmartNamesRetrievedCallbackSuccess);
					}, this));
				} else {
					this._extendSmartNamesWithExtraSmartNames(sectionName, documentAttributeSmartNames, onSmartNamesRetrievedCallbackSuccess);
				}
				break;
			case WorkflowSmartNameConstants.SPECIFICATIONS.CATEGORY:
				if (this._taskTypeID && !taskSpecificationSmartNames || taskSpecificationsTaskTypeID !== this._taskTypeID) { // this will usually be the same tasktypeid
					taskSpecificationsTaskTypeID = this._taskTypeID;

					$.when(this._webCenterAPI.getTaskSpecifications(this._taskTypeID), this._webCenterAPI.getSpecifications()).done(_.bind(function (specifications, attributes) {
						taskTypeSpecificationsMap = specifications;
						specificationsMap = attributes;
						taskSpecificationSmartNames = this._createSmartNamesWithId(specifications, WorkflowSmartNameConstants.SPECIFICATIONS.PREFIX);
						this._extendSmartNamesWithExtraSmartNames(sectionName, taskSpecificationSmartNames, onSmartNamesRetrievedCallbackSuccess);
					}, this));
				} else {
					this._extendSmartNamesWithExtraSmartNames(sectionName, taskSpecificationSmartNames, onSmartNamesRetrievedCallbackSuccess);
				}
				break;
			case WorkflowSmartNameConstants.ITERATION.CATEGORY:
				var iteration = [];
				this._extendSmartNamesWithExtraSmartNames(sectionName, iteration, onSmartNamesRetrievedCallbackSuccess);
				break;
			case WorkflowSmartNameConstants.LINKS.CATEGORY:
				if (!linkSmartNames) {
					linkSmartNames = this._sortSmartNames(this._createSmartNames(WorkflowSmartNameConstants.LINKS.SMARTNAMES));
				}
				this._extendSmartNamesWithExtraSmartNames(sectionName, linkSmartNames, onSmartNamesRetrievedCallbackSuccess);
				break;
			case WorkflowSmartNameConstants.EMAIL.CATEGORY:
				if (!emailSmartNames) {
					emailSmartNames = this._sortSmartNames(this._createSmartNames(WorkflowSmartNameConstants.EMAIL.SMARTNAMES));
				}
				this._extendSmartNamesWithExtraSmartNames(sectionName, emailSmartNames, onSmartNamesRetrievedCallbackSuccess);
				break;
			case WorkflowSmartNameConstants.DATE.CATEGORY:
				if (!dateSmartNames) {
					dateSmartNames = this._sortSmartNames(this._createSmartNames(WorkflowSmartNameConstants.DATE.SMARTNAMES));
				}
				this._extendSmartNamesWithExtraSmartNames(sectionName, dateSmartNames, onSmartNamesRetrievedCallbackSuccess);
				break;
			case WorkflowSmartNameConstants.URL.CATEGORY:
				if (!urlSmartNames) {
					urlSmartNames = this._sortSmartNames(this._createSmartNames(WorkflowSmartNameConstants.URL.SMARTNAMES));
				}
				this._extendSmartNamesWithExtraSmartNames(sectionName, urlSmartNames, onSmartNamesRetrievedCallbackSuccess);
				break;
			case WorkflowSmartNameConstants.USER.CATEGORY:
				if (!userSmartNames) {
					userSmartNames = this._sortSmartNames(this._createSmartNames(WorkflowSmartNameConstants.USER.SMARTNAMES));
				}
				this._extendSmartNamesWithExtraSmartNames(sectionName, userSmartNames, onSmartNamesRetrievedCallbackSuccess);
				break;
			case WorkflowSmartNameConstants.TASKROLES.CATEGORY:
				if (!rolesSmartNames) {
					$.when(this._webCenterAPI.getUsersGroupsRoles()).done(_.bind(function (users, groups, roles) {
						rolesMap = roles;
						rolesSmartNames = this._createSmartNamesWithId(roles, WorkflowSmartNameConstants.TASKROLES.PREFIX);
						this._extendSmartNamesWithExtraSmartNames(sectionName, rolesSmartNames, onSmartNamesRetrievedCallbackSuccess);
					}, this));
				} else {
					this._extendSmartNamesWithExtraSmartNames(sectionName, rolesSmartNames, onSmartNamesRetrievedCallbackSuccess);
				}
				break;
			case WorkflowSmartNameConstants.TASKGROUPS.CATEGORY:
				if (!groupsSmartNames) {
					$.when(this._webCenterAPI.getUsersGroupsRoles()).done(_.bind(function (users, groups, roles) {
						groupsMap = groups;
						groupsSmartNames = this._createSmartNamesWithId(groups, WorkflowSmartNameConstants.TASKGROUPS.PREFIX);
						this._extendSmartNamesWithExtraSmartNames(sectionName, groupsSmartNames, onSmartNamesRetrievedCallbackSuccess);
					}, this));
				} else {
					this._extendSmartNamesWithExtraSmartNames(sectionName, groupsSmartNames, onSmartNamesRetrievedCallbackSuccess);
				}
				break;
			case WorkflowSmartNameConstants.TASKUSERS.CATEGORY:
				if (!usersSmartNames) {
					$.when(this._webCenterAPI.getUsersGroupsRoles()).done(_.bind(function (users, groups, roles) {
						usersMap = users;
						usersSmartNames = this._createSmartNamesWithId(users, WorkflowSmartNameConstants.TASKUSERS.PREFIX);
						this._extendSmartNamesWithExtraSmartNames(sectionName, usersSmartNames, onSmartNamesRetrievedCallbackSuccess);
					}, this));
				} else {
					this._extendSmartNamesWithExtraSmartNames(sectionName, usersSmartNames, onSmartNamesRetrievedCallbackSuccess);
				}
				break;
			case WorkflowSmartNameConstants.CALCULATED.CATEGORY:
				if (!calculatedSmartNames) {
					calculatedSmartNames = this._sortSmartNames(this._createSmartNames(WorkflowSmartNameConstants.CALCULATED.SMARTNAMES));
				}
				this._extendSmartNamesWithExtraSmartNames(sectionName, calculatedSmartNames, onSmartNamesRetrievedCallbackSuccess);
				break;
			default:
				break;
			}
		};

		/**
		 * Extend smartnames with extra smartnames for given section.
		 * The callback will be executed with the extended set.
		 * @param {String} sectionName
		 * @param {Array.<SmartName>} smartNames
		 * @param {Function} onSmartNamesRetrievedCallbackSuccess
		 */
		WorkflowSmartNameDicationaryService.prototype._extendSmartNamesWithExtraSmartNames = function (sectionName, smartNames, onSmartNamesRetrievedCallbackSuccess) {
			var extendedSmartNames;
			var extraSmartNames = this._extraSmartNames.get(sectionName);
			if (extraSmartNames) {
				extendedSmartNames = this._sortSmartNames(_.union(smartNames, extraSmartNames));
			} else {
				extendedSmartNames = smartNames;
			}
			onSmartNamesRetrievedCallbackSuccess(sectionName, extendedSmartNames);
		};

		/**
		 * Create smartnames for objects in map. They should have a method getId().
		 * @param {Map.<Any>} map
		 * @param {String} prefix
		 * @returns {Array.<SmartName>} The created smartnames ordered by translation key
		 */
		WorkflowSmartNameDicationaryService.prototype._createSmartNamesWithId = function (map, prefix) {
			var smartNames = [];
			_.each(map.values(), function (entry) {
				smartNames.push(new SmartName(prefix + entry.getId()));
			});
			return this._sortSmartNames(smartNames);
		};

		/**
		 * Create smartnames from a list or object of strings
		 * @param   {Array<String>|Object} smartNames - The smartName keys
		 * @returns {Array<SmartName>} The created smartnames as array
		 */
		WorkflowSmartNameDicationaryService.prototype._createSmartNames = function (smartNames) {
			var result = [];
			if (smartNames) {
				_.each(smartNames, function (smartname) {
					result.push(new SmartName(smartname));
				});
			}
			return result;
		};

		/**
		 * Sort smartnames array
		 * @param   {Array<SmartName>} smartNames - The SmartNames
		 * @returns {Array<SmartName>} The created and sorted smartnames as array
		 */
		WorkflowSmartNameDicationaryService.prototype._sortSmartNames = function (smartNames) {
			// Sort alphabetically
			var result = _.sortBy(smartNames, function (smartName) {
				return this.displaySmartNameSelectorSmartName(smartName);
			}, this);

			return result;
		};

		/**
		 * Function giving back how to display smartnames in input fields
		 * @param   {SmartName} smartName - The smartname to display
		 * @returns {String}        The display value
		 */
		WorkflowSmartNameDicationaryService.prototype.displaySmartName = function (smartName) {
			var key = smartName.getName();
			return "[" + this._translateSmartNameKey(key) + "]";
		};

		/**
		 * Function giving back how to display smartnames in the smartname selector
		 * @param   {SmartName]} smartName - The smartname to display
		 * @returns {String} The display value
		 */
		WorkflowSmartNameDicationaryService.prototype.displaySmartNameSelectorSmartName = function (smartName) {
			var key = smartName.getName();
			return this._translateSmartNameKey(key);
		};

		/**
		 * Translate a SmartName key
		 * @param   {String}  key             - The key to translate
		 * @returns {String}  The translated key.
		 */
		WorkflowSmartNameDicationaryService.prototype._translateSmartNameKey = function (key) {
			if (key.indexOf(WorkflowSmartNameConstants.PROJECTATTRIBUTES.PREFIX) === 0) {
				return this._translateMapSmartName(attributesMap, key, WorkflowSmartNameConstants.PROJECTATTRIBUTES.PREFIX, WorkflowSmartNameConstants.PROJECT.CATEGORY);
			} else if (key.indexOf(WorkflowSmartNameConstants.DOCUMENTATTRIBUTES.PREFIX) === 0) {
				return this._translateMapSmartName(attributesMap, key, WorkflowSmartNameConstants.DOCUMENTATTRIBUTES.PREFIX, WorkflowSmartNameConstants.DOCUMENT.CATEGORY);
			} else if (key.indexOf(WorkflowSmartNameConstants.SPECIFICATIONS.PREFIX) === 0) {
				return this._translateMapSmartName(specificationsMap, key, WorkflowSmartNameConstants.SPECIFICATIONS.PREFIX, WorkflowSmartNameConstants.SPECIFICATIONS.CATEGORY);
			} else if (key.indexOf(WorkflowSmartNameConstants.TASKUSERS.PREFIX) === 0) {
				return this._translateMapSmartName(usersMap, key, WorkflowSmartNameConstants.TASKUSERS.PREFIX, WorkflowSmartNameConstants.TASKUSERS.CATEGORY);
			} else if (key.indexOf(WorkflowSmartNameConstants.TASKGROUPS.PREFIX) === 0) {
				return this._translateMapSmartName(groupsMap, key, WorkflowSmartNameConstants.TASKGROUPS.PREFIX, WorkflowSmartNameConstants.TASKGROUPS.CATEGORY);
			} else if (key.indexOf(WorkflowSmartNameConstants.TASKROLES.PREFIX) === 0) {
				return this._translateMapSmartName(rolesMap, key, WorkflowSmartNameConstants.TASKROLES.PREFIX, WorkflowSmartNameConstants.TASKROLES.CATEGORY);
			} else if (key === WorkflowSmartNameConstants.EMAIL.SMARTNAMES.PROJECTMANAGER || key === WorkflowSmartNameConstants.EMAIL.SMARTNAMES.SALESPERSON || key === WorkflowSmartNameConstants.EMAIL.SMARTNAMES.WORKFLOWTASKASSIGNEE) {
				return i18n.t(WorkflowSmartNameConstants.EMAIL.CATEGORY + "." + key);
			} else {
				return i18n.t(key);
			}
		};

		/**
		 * Translate smartname of attributes, users, roles, groups
		 * (should have getName() method)
		 * @param   {String}   key           
		 * @param   {String} prefix
		 * @param   {String} suffixKey
		 * @returns {String}
		 */
		WorkflowSmartNameDicationaryService.prototype._translateMapSmartName = function (map, key, prefix, suffixKey) {
			if (map) {
				var id = key.substring(prefix.length);
				var entry = map.get(id);
				if (entry) {
					return entry.getName() + " (" + i18n.t(suffixKey) + ")";
				} else {
					if(prefix === WorkflowSmartNameConstants.TASKUSERS.PREFIX){
						return i18n.t("cannotLoadUserSmartName");
					}else{
						return i18n.t("cannotLoadSmartName");
					}
				}
			}
			// if map is undefined, we are still loading
			return i18n.t("loading");
		};

		/**
		 * Add project properties (e.g. project name) to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addProjectProperties = function () {
			return true;
		};

		/**
		 * Add project attributes to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addProjectAttributes = function () {
			return true;
		};

		/**
		 * Add document properties (e.g. document name) to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addDocumentProperties = function () {
			return true;
		};

		/**
		 * Add Text Content properties (e.g. element type, approval type etc.) to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addTextContentProperties = function () {
			//Show text content property fields in smart name only if PCM License enabled
			if (window.webcenterConfig.isPCMEnabled) {
				return true;
			}
		};

		/**
		 * Add document attributes to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addDocumentAttributes = function () {
			return true;
		};

		/**
		 * Add task specifications to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addTaskSpecifications = function () {
			return true;
		};


		/**
		 * Add iterator smartname to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addIteration = function () {
			return false;
		};

		/**
		 * Add links to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addLinks = function () {
			return false;
		};

		/**
		 * Add email to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addEmails = function () {
			return false;
		};

		/**
		 * Add dates to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addDates = function () {
			return false;
		};

		/**
		 * Add URLs to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addURLs = function () {
			return false;
		};


		/**
		 * Add user smartnames (e.g. first name) to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addUser = function () {
			return false;
		};

		/**
		 * Add roles to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addRoles = function () {
			return false;
		};

		/**
		 * Add groups to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addGroups = function () {
			return false;
		};

		/**
		 * Add users to smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addUsers = function () {
			return false;
		};

		/**
		 * Add calculated smartnames
		 * @returns {Boolean} True if they should be added, false otherwise.
		 */
		WorkflowSmartNameDicationaryService.prototype.addCalculatedSmartNames = function () {
			return true;
		};

		return WorkflowSmartNameDicationaryService;

	});