define([
		"utils/ShapeUtils",
		"WFEditor/workflow/pin/Pin",
		"underscore"
	],
	function (
		ShapeUtils,
		Pin,
		_
	) {
		"use strict";

		/**
		 * WebCenterPin represents the model for a pin in the WebCenter Workflow Editor.
		 * The pin has a label
		 *
		 * @exports WebCenterPin
		 * @class
		 * @augments Pin
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {Node} node - node to which the pin belongs
		 * @param {String} id - pin
		 * @param {WebCenterPinConstants.Type} type
		 * @param {String} [label=""] - label for the pin
		 */
		var WebCenterPin = function (workflow, node, id, type, label) {
			Pin.call(this, workflow, node, id);

			/**
			 * @Member {WebCenterPinConstants.Type}
			 * @private
			 */
			this._pinType = type;

			/**
			 *  @member {String}
			 *  @private
			 */
			this._label = label || "";
		};

		WebCenterPin.prototype = Object.create(Pin.prototype);
		WebCenterPin.prototype.constructor = WebCenterPin;

		/**
		 * Returns the label of the pin
		 * @returns {String}
		 */
		WebCenterPin.prototype.getLabel = function () {
			return this._label;
		};

		/**
		 * Get the type of the pin
		 * @returns {WebCenterPinConstants.Type}
		 */
		WebCenterPin.prototype.getPinType = function () {
			return this._pinType;
		};


		/**
		 * Get the number that needs to be displayed in the pin to indicate the tokens.
		 * @returns {Number}
		 */
		WebCenterPin.prototype.getNumberForPin = function () {
			var number = 0;
			_.each(this._tokens, function (token) {
				if (!token.isCloned()) {
					number += token.getWeight();
				}
			}, this);
			return number;
		};

		/**
		 * Returns all the tokens in this webcenter pin. Removes the cloned tokens.
		 * @returns {Array.<Token>}
		 */
		WebCenterPin.prototype.getTokens = function () {
			return _.filter(this._tokens, function (token) {
				return !token.isCloned();
			}, this);
		};

		/**
		 * Returns all the active tokens in this webcenter pin. Active tokens are tokens that aren't processed yet.
		 * @returns {Array.<Token>}
		 */
		WebCenterPin.prototype.getActiveTokens = function () {
			return _.filter(this._tokens, function (token) {
				return !token.isCloned() && !token.isProcessed();
			});
		};

		return WebCenterPin;
	});