define([
		"utils/ShapeUtils",
		"WFEditor/workflow/pin/PinView",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/scene/WebCenterViewConstants",
		"WFEditorWCR/webcenterInformationPopup/WebCenterPinInformationPopup",
		"utils/Animatable",
		"WFEditor/scene/hitarea/HitArea",
		"utils/webi18n!WFEditorWCR",
		"underscore"
	],
	function (
		ShapeUtils,
		PinView,
		WebCenterPinConstants,
		WebCenterViewConstants,
		WebCenterPinInformationPopup,
		Animatable,
		HitArea,
		webi18n,
		_
	) {
		"use strict";

		/**
		 * The view for a webcenter pin item.
		 *
		 * @exports WebCenterPinView
		 * @class
		 * @augments PinView
		 *
		 * @param {WebCenterPin} pinModel - model for the pin
		 * @param {WebCenterNodeView} nodeView - view for the node too which the pin belongs
		 * @param {Scene} scene
		 */
		var WebCenterPinView = function (pinModel, nodeView, scene) {
			PinView.call(this, pinModel, nodeView);

			this._scene = scene;

			/**
			 * @member {SceneModel}
			 * @private
			 */
			this._sceneModel = scene.getSceneModel();

			/**
			 * @member{Boolean} Is the popup opened
			 * @private
			 */
			this._isPopupOpen = false;


			/**
			 * @member {ShapeUtils.Rect} the rectangle of the lifeline between pin and popup (if it is being drawn)
			 * @private
			 */
			this._popupLifelineRect = null;

			/**
			 * @member {Boolean} has the location of the popup changed?
			 * @private
			 */
			this._isPopupLocationDirty = false;

			this.addAnimatableProperties({
				pinRadius: function () {
					if (pinModel.isHovered()) {
						return WebCenterViewConstants.PinRadiusHovered;
					} else {
						return WebCenterViewConstants.PinRadius;
					}
				}
			});

			this._observe(this._model, "changed.*", this.makeDirty);
			this._observe(this._model, "changed.hover", function () {
				if (this._model.getPinType() !== WebCenterPinConstants.Type.In) { // only animate output pins
					var previousPinRadius = !this._model.isHovered() ? WebCenterViewConstants.PinRadiusHovered : WebCenterViewConstants.PinRadius;
					this.animate("pinRadius", previousPinRadius, 350, "easeOutQuart");
				}
				this._hoverChanged();
			});
			this._observe(this._model, "Token.*.changed.select", this.makeDirty);
			this._observe(this._model, "Token.added", function () {
				this._updateBoundingRect();
				this._updateHitAreas();
				this.makeDirty();
			});
			this._observe(this._model.getNode(), "Pin.*", this._updateHitAreas);

			// observe node location changes
			this._observe(this._model.getNode(), "changed.location", this._locationChanged);
			// observe edges added
			_.each(this._model.getEdges(), function (edge) {
				this._edgeAdded(edge);
			}, this);
			this._observe(this._model, "Edge.added", this._edgeAdded);
			this._observe(this._model, "Edge.removed", this._edgeRemoved);

		};

		WebCenterPinView.prototype = Object.create(PinView.prototype);
		WebCenterPinView.prototype.constructor = WebCenterPinView;
		Animatable.mixin(WebCenterPinView.prototype);


		/**
		 * @override
		 * Renders the node on the provided canvas context. Only renders if the icon is available.
		 * If the icon is not yet loaded, the iconstorage fetches the icon and the layer is marked dirty
		 * when the icon is correctly fetched.
		 *
		 * @param   {CanvasRenderingContext2D} context
		 * @param   {DefaultLayer} layer
		 * @param	{Boolean} quickRender
		 */
		WebCenterPinView.prototype.render = function (context, layer, quickRender) {
			if (quickRender) {
				// Edges shouldn't be rendered in quickRender mode
				return;
			}

			// pin rendering
			context.save();
			context.translate(-0.5, -0.5);

			var location = this._nodeView.getLocationForPin(this._model);

			if (this._popup) {
				if (this._isPopupLocationDirty && this._popup.isVisible()) {
					this._popup.relocate();
					this._isPopupLocationDirty = false;
				}
				if (this._shouldShowPopup()) {

					if (this._popup.isMoved() && !this._isPopupLocationDirty) {
						context.save();
						context.strokeStyle = this.getPinColor();
						context.beginPath();
						context.moveTo(location.x, location.y);
						var popupLocation = this._popup.getLocation();
						context.lineTo(popupLocation.x, popupLocation.y);
						context.stroke();
						context.restore();

						this._popupLifelineRect = new ShapeUtils.Rect(location.copy());
						this._popupLifelineRect.size = location.sizeTo(popupLocation);
					} else {
						this._popupLifelineRect = null;
					}

					if (this._popup.isVisible()) {
						this._popup.render();
					}

					// make sure to update the bounding rect if there is a lifeline :)
					this._updateBoundingRect();
				} else if (this._popup) {
					this._popup.hide();
				}
			}


			// draw everything from location as origin
			context.translate(location.x, location.y);


			var pinType = this._model.getPinType();
			if (pinType === WebCenterPinConstants.Type.In) {

				//don't draw input pin

			} else {
				// output pin
				context.beginPath();
				context.lineWidth = 1;

				switch (pinType) {
				case WebCenterPinConstants.Type.OutError:
					context.fillStyle = WebCenterViewConstants.PinErrorBackground;
					context.strokeStyle = WebCenterViewConstants.PinErrorBorder;
					break;
				case WebCenterPinConstants.Type.OutWarning:
					context.fillStyle = WebCenterViewConstants.PinWarningBackground;
					context.strokeStyle = WebCenterViewConstants.PinWarningBorder;
					break;
				default:
					context.fillStyle = WebCenterViewConstants.PinOKBackground;
					context.strokeStyle = WebCenterViewConstants.PinOKBorder;
					break;
				}
				var radius = this.getValue("pinRadius");
				var pinTextWidth = 0;
				if (this._model.hasTokens()) {
					context.save();

					if (this._model.isHovered()) {
						context.font = WebCenterViewConstants.PinCharacterFeedbackFontHovered;
					} else {
						context.font = WebCenterViewConstants.PinCharacterFont;
					}



					context.textBaseline = "middle";
					context.textAlign = "left";
					pinTextWidth = context.measureText(this._model.getNumberForPin()).width;

					context.beginPath();
					context.moveTo(0, radius);
					context.arc(0, 0, radius, Math.PI / 2, -Math.PI / 2);
					context.lineTo(pinTextWidth, -radius);
					context.arc(pinTextWidth, 0, radius, -Math.PI / 2, Math.PI / 2);
					context.closePath();

					// first fill path with white opaque background
					this._renderOpaqueBackground(context);

					context.fill();
					context.stroke();
					if (pinType === WebCenterPinConstants.Type.OutError) {
						context.fillStyle = WebCenterViewConstants.PinErrorCharacterColor;
					} else {
						context.fillStyle = WebCenterViewConstants.PinOKCharacterColor;
					}

					context.fillText(this._model.getNumberForPin(), 0, 0);

					context.restore();
				} else {
					context.arc(0, 0, radius, 0, 2 * Math.PI, false);

					// first fill path with white opaque background
					this._renderOpaqueBackground(context);

					context.fill();
					context.stroke();

					if (pinType === WebCenterPinConstants.Type.OutError || pinType === WebCenterPinConstants.Type.OutWarning) {
						// render x or ! in the pin icon
						context.save();

						if (pinType === WebCenterPinConstants.Type.OutWarning) {
							// draw an exclamation mark
							var markPoint = Math.max(1, Math.floor(0.25 * radius));

							context.beginPath();
							context.moveTo(0, markPoint);
							context.lineTo(-0.5 * markPoint, -3.5 * markPoint);
							context.lineTo(0.5 * markPoint, -3.5 * markPoint);
							context.lineTo(0, markPoint);
							context.strokeStyle = WebCenterViewConstants.PinWarningCharacterColor;
							context.lineWidth = 1;
							context.fillStyle = WebCenterViewConstants.PinWarningCharacterColor;
							context.fill();
							context.stroke();

							context.beginPath();
							context.arc(0, 3.5 * markPoint, 0.75 * markPoint, 0, 2 * Math.PI, false);
							context.fill();


						} else {
							// draw a cross
							var crossSize = Math.floor(0.8 * radius);
							context.rotate(0.25 * Math.PI);
							context.beginPath();
							context.moveTo(-crossSize, 0);
							context.lineTo(crossSize, 0);
							context.moveTo(0, -crossSize);
							context.lineTo(0, crossSize);
							context.strokeStyle = WebCenterViewConstants.PinErrorCharacterColor;
							context.lineWidth = 1;
							context.stroke();
						}
						context.restore();
					}

				}

				if ((this._model.isHovered() || this._model.isConnectedEdgeHovered()) && this.getLabel() && !(this._popup && this._popup.isVisible())) {
					// draw label	
					context.fillStyle = WebCenterViewConstants.PinLabelColor;
					context.font = WebCenterViewConstants.PinLabelFont;
					context.textAlign = "left";

					context.fillText(this.getLabel(),
						radius + pinTextWidth + WebCenterViewConstants.PinLabelOffsetX, -radius + WebCenterViewConstants.PinLabelOffsetY);
				}
			}
			context.restore();
		};

		/**
		 * The location was changed of the pin view
		 * update hit areas, bounding rect and make the view dirty
		 */
		WebCenterPinView.prototype._locationChanged = function () {
			this._updateBoundingRect();
			this._updateHitAreas();
			this.makeDirty();
		};

		/**
		 * @Override
		 */
		WebCenterPinView.prototype._updateHitAreas = function () {
			var pinLocation = this._nodeView.getLocationForPin(this._model);
			var radius = WebCenterViewConstants.PinRadiusHovered + 1;
			var pinRect;

			var textWidth = 0;

			if (this._model.hasTokens() && this._model.getPinType() !== WebCenterPinConstants.Type.In) {

				// add size of token number text
				this._context.save();
				this._context.font = WebCenterViewConstants.PinCharacterFontHovered;
				this._context.textAlign = "left";
				textWidth = this._context.measureText(this._model.getNumberForPin()).width;

				this._context.restore();
			}

			pinRect = new ShapeUtils.Rect(pinLocation.copy(), new ShapeUtils.Size(2 * radius + textWidth, 2 * radius));
			pinRect.origin.offsetXY(-radius, -radius);

			this._setHitAreas([new HitArea(pinRect, this, [], undefined, {
				cursor: "pointer-cursor"
			})]);

		};

		/**
		 * Get the location of the pin.
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterPinView.prototype.getLocation = function () {
			return this._nodeView.getLocationForPin(this._model);
		};

		/**
		 * Get the location of the edge if all edges arriving in this pin are seperated.
		 * Returns the location of the endpoint of the edge, the location of the input pin does
		 * not change.
		 *
		 * @param {WebCenterEdgeView} the view of the edge the correct locatin needs to be returned for
		 * @param {Boolean} [isSplit=false] - The 'previous' value of isSplit in the model
		 * @returns {ShapeUtils.Point} the location of the end point of the provided edge
		 */
		WebCenterPinView.prototype.getEdgeLocation = function (edge, isSplit) {
			if ((isSplit !== undefined && !isSplit) || this._model.getNumberOfEdges() <= 1) {
				return this.getLocation();
			}

			var edgeViews = this._calculateEdgeOrder();

			var index = _.indexOf(edgeViews, edge);
			var location = this.getLocation();
			return new ShapeUtils.Point(location.x, location.y + (WebCenterViewConstants.PinRadius * 3.5 * index));
		};

		/**
		 * Get the view of the first edge in the pin.
		 *
		 * @returns {EdgeView}
		 */
		WebCenterPinView.prototype.getFirstEdge = function () {
			return this._calculateEdgeOrder()[0];
		};

		/**
		 * Calculate the order of the edges arriving in the pin.
		 * @returns {Array.<EdgeView>}
		 */
		WebCenterPinView.prototype._calculateEdgeOrder = function () {
			var edges = this._model.getEdges();
			var edgeViews = [];
			_.each(edges, function (edge) {
				var edgeView = this._scene.getView(edge.getId());
				if (edgeView) {
					edgeViews.push(edgeView);
				}
			}, this);
			var toLocation = this.getLocation();
			return _.sortBy(edgeViews, function (viewObj) {
				var fromPin = viewObj.getFromPin();
				var fromLocation = fromPin.getLocation();
				// the angle between the zero line (= through the toPoint parallel with the x-absces) and the toPoint
				var vector = new ShapeUtils.Vector(fromLocation.x - toLocation.x, -(fromLocation.y - toLocation.y));
				var angle = (vector.getAngleInDegrees() + 360) % 360;
				return angle;
			});
		};

		/**
		 * Returns the label if it should be shown (empty string if not)
		 * @returns {String}
		 */
		WebCenterPinView.prototype.getLabel = function () {
			return this._model.getLabel();
		};

		/**
		 * Returns the color used for this pin (stirng in #001122 format)
		 * @returns {String}
		 */
		WebCenterPinView.prototype.getPinColor = function () {
			var color = null;
			if (this._model.getPinType() === WebCenterPinConstants.Type.OutError ||
				this._model.getPinType() === WebCenterPinConstants.Type.WorkflowConstructError) {
				color = WebCenterViewConstants.PinErrorBackground;
			} else if (this._model.getPinType() === WebCenterPinConstants.Type.OutWarning) {
				color = WebCenterViewConstants.PinWarningBackground;
			} else {
				color = WebCenterViewConstants.PinOKBackground;
			}
			return color;
		};

		/**
		 * @override
		 */
		WebCenterPinView.prototype._updateBoundingRect = function () {
			var location = this._nodeView.getLocationForPin(this._model);

			var radius = WebCenterViewConstants.PinRadiusHovered + 1; // take in to account border (+1)
			// default bounding rect for Pin
			var boundingRect = new ShapeUtils.Rect(location.copy(), new ShapeUtils.Size(radius * 2, radius * 2));
			boundingRect.origin.x -= radius;
			boundingRect.origin.y -= radius;

			if (this._model.hasTokens()) {
				// add size of token number text
				this._context.save();
				this._context.font = WebCenterViewConstants.PinCharacterFontHovered;
				this._context.textAlign = "left";
				var width = this._context.measureText(this._model.getNumberForPin()).width;
				boundingRect.size.width += width;
				this._context.restore();
			}

			if (this.getLabel()) {
				//calculate label size and adapt boundingrect
				this._context.save();
				this._context.fillStyle = WebCenterViewConstants.PinLabelColor;
				this._context.font = WebCenterViewConstants.PinLabelFont;
				this._context.textAlign = "left";

				var textMetrics = this._context.measureText(this.getLabel());
				var labelYOffset = WebCenterViewConstants.PinLabelOffsetY + WebCenterViewConstants.PinLabelHeight;

				boundingRect.origin.y -= labelYOffset;
				boundingRect.size.height = boundingRect.size.height + labelYOffset;

				boundingRect.size.width = Math.max(boundingRect.size.width, textMetrics.width + WebCenterViewConstants.PinLabelOffsetX + 2 * WebCenterViewConstants.PinRadiusHovered);
				boundingRect.size.height = Math.max(boundingRect.size.height, WebCenterViewConstants.PinLabelHeight);

				this._context.restore();
			}

			if (this._popupLifelineRect && this._popup.isVisible()) {
				boundingRect = boundingRect.calculateArea(this._popupLifelineRect);
			}

			this._setBoundingRect(boundingRect.getNormalizedRectArray());
		};


		/**
		 * Get the location for a certain active token
		 * @param {Token} token for which the location needs to be returned
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterPinView.prototype.getLocationForToken = function (token) {
			var tokenLocation = this._nodeView.getLocationForPin(this._model).copy();
			var activeTokens = this._model.getActiveTokens();
			var i = 0;
			var found = false;
			// Find the sequence number of the token
			for (var j = 0; j < activeTokens.length && !found; j++) {
				if (token.getId() === activeTokens[j].getId()) {
					found = true;
				} else {
					i++;
				}
			}
			if (found) {
				tokenLocation.y -= WebCenterViewConstants.NodeStatusGap;
				tokenLocation.y -= i * WebCenterViewConstants.StatusToStatusGap;
				tokenLocation.x += WebCenterViewConstants.PinInToNodeLeft;
				return tokenLocation;
			}

		};

		/**
		 * An output pin has a higher z-index (default 30) than an input-pin (under the edge, so 5)
		 */
		WebCenterPinView.prototype.getZIndex = function () {
			if (this._model.getPinType() === WebCenterPinConstants.Type.In) {
				// input pin
				return 5;
			} else {
				// output pin
				return 30;
			}
		};

		/**
		 * Mark the popup open
		 */
		WebCenterPinView.prototype.setPopupOpen = function (isPopupOpen) {

			if (this._isPopupOpen !== isPopupOpen) {
				if (isPopupOpen) {
					this._setSceneModelListeners();
				} else {
					this._stopObserveObject(this._sceneModel);
				}
				this._isPopupOpen = isPopupOpen;
				this.makeDirty();
			}
		};

		/**
		 * React when an edge has been added to the model
		 * @param {Edge} edge - The added edge
		 */
		WebCenterPinView.prototype._edgeAdded = function (edge) {
			this._observe(edge, "changed.hover", this.makeDirty);
		};

		/**
		 * React when an edge has been removed from the model
		 * @param {Edge} edge - The removed edge
		 */
		WebCenterPinView.prototype._edgeRemoved = function (edge) {
			this._stopObserveObject(edge);
		};

		/**
		 * Hover statechanged
		 */
		WebCenterPinView.prototype._hoverChanged = function () {
			if (!this._popup && this._model.isHovered() && this._shouldShowPopup()) {
				this._popup = new WebCenterPinInformationPopup(this, this._model, this._scene);
				this._observe(this._popup, "changed.hover", this._hoverChanged);
			}

			if (this._popup) {
				this._popup.cancelDeferedClose();

				if (this._model.isHovered() && this._model.hasTokens() && this._model.getPinType() !== WebCenterPinConstants.Type.In) {
					this._popup.show();
				}

				if (!this._popup.isHovered() && !this._model.isHovered() && !this._popup.isSticky()) {
					this._popup.deferredClose();
				}
			}

		};

		/**
		 * Should the pin show a popup on hover?
		 * @returns {Boolean} 
		 */
		WebCenterPinView.prototype._shouldShowPopup = function () {
			return this._model.hasTokens() && this._model.getPinType() !== WebCenterPinConstants.Type.In;
		};

		/**
		 * Listen to sceneModel for panning and zoom changes
		 */
		WebCenterPinView.prototype._setSceneModelListeners = function () {
			this._observe(this._sceneModel, "panningOffsetChanged", _.bind(function () {
				if (this._popup && this._popup.isVisible()) {

					// defer to ensure transformation has been done. The relocation of the popup doesn't happen in the render
					// method because if the pin isn't in the visible rect anymore, the render method won't be called
					_.defer(_.bind(function () {
						this._popup.relocate();
						this.makeDirty();
					}, this));
				}
			}, this));

			this._observe(this._sceneModel, "zoomChanged", _.bind(function () {
				if (this._popup && this._popup.isVisible()) {
					// for zooming the change in location will be very big so we temporarily don't want to render the 
					// lifeline for popups as it will cause it to visibly jump from one position to another
					this._isPopupLocationDirty = true;

					// defer to ensure transformation has been done.
					_.defer(_.bind(function () {
						this._popup.relocate();
						this.makeDirty();
						this._isPopupLocationDirty = false;
					}, this));
				}
			}, this));
		};


		return WebCenterPinView;

	});
