define([
		"WFEditorWCR/workflow/token/WebCenterToken",
		"underscore"
	],
	function (
		WebCenterToken,
		_
	) {
		"use strict";

		/**
		 * Represents an operation, such as Join or DataCollector used in the WebCenter workflow editor. 
		 * The data of the this token consists of different Tokens
		 * 
		 * @exports WebCenterOperationToken
		 * @class
		 * @augments WebCenterToken
		 *
		 * @param {Workflow} workflow - the workflow to which the token belongs
		 * @param {Pin} pin - pin to which the token belongs
		 * @param {String} id
		 * @param {XMLObject} data - xml object with the data for this token
		 * @param {String|null} cloneId - id of the token this token has been cloned from, null if not cloned
		 * @param {Boolean} isProcessed - has this token been processed yet?                             
		 */
		var WebCenterOperationToken = function (workflow, pin, id, data, cloneId, isProcessed) {
			WebCenterToken.call(this, workflow, pin, id, data, cloneId, isProcessed);

			/**
			 * @member {Map.<String,WebCenterToken} 
			 * @private
			 */
			this._subTokens = {};

			if (data) { //parse provided XML token data
				this._processXMLData(data);
			}
		};

		WebCenterOperationToken.prototype = Object.create(WebCenterToken.prototype);
		WebCenterOperationToken.prototype.constructor = WebCenterOperationToken;


		/**
		 * The files of this token, are the files of its subtokens
		 * @returns {Map.<String, String>}
		 */
		WebCenterOperationToken.prototype.getFiles = function () {
			var files = {};
			var i = 0;
			_.each(this._subTokens, function (token) {
				var tokenFiles = token.getFiles();
				_.each(tokenFiles, function(file, key){
					files[key + "_" + i] = file;
				});
				i++;
			});
			return files;
		};

		/**
		 * Get the number of files in this token.
		 * @returns {Number}
		 */
		WebCenterOperationToken.prototype.getFileCount = function () {
			return _.size(this.getFiles());
		};

		/**
		 * Get the number of subtokens in this operation token
		 * @returns {WebCenterToken}
		 */
		WebCenterOperationToken.prototype.getSubTokens = function () {
			return this._subTokens;
		};

		/**
		 * Adds a subtoken to this operation token
		 * @param {Token} token
		 */
		WebCenterOperationToken.prototype.addSubToken = function (token) {
			this._subTokens[token.getId()] = token;
		};

		/**
		 * returns true if token is included in this operation token
		 * @param   {Token} token
		 * @returns {Boolean}
		 */
		WebCenterOperationToken.prototype.hasSubToken = function (token) {
			return this._subTokens[token.getId()] !== undefined;
		};





		return WebCenterOperationToken;
	});
