define([
		"WFEditor/workflow/token/Token",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"underscore"
	],
	function (
		Token,
		WebCenterPinConstants,
		_
	) {
		"use strict";

		/**
		 * Represents the token used in the WebCenter workflow editor. The data of the token
		 * consists of several files.
		 *
		 * @exports WebCenterToken
		 * @class
		 * @augments Token
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {Pin} [pin] - node to which the token belongs. Undefined for the init token.
		 * @param {String} id
		 * @param {XMLObject} data - xml object with the data for this token
		 * @param {String|null} cloneId - id of the token this token has been cloned from, null if not cloned
		 * @param {Boolean} isProcessed - has this token been processed yet?
		 */
		var WebCenterToken = function (workflow, pin, id, data, cloneId, isProcessed) {
			Token.call(this, workflow, pin, id, isProcessed);

			/**
			 * @member {Map.<String, String>} maps file id to file name
			 * @private
			 */
			this._files = {};

			/**
			 * @member {Map.<String, String>} maps error id to the error message
			 * @private
			 */
			this._errors = {};

			/**
			 * @member {String|null} the id of the token this token is cloned from
			 * @private
			 */
			this._cloneId = cloneId;

			/**
			 * @member {String} Id of the document version (of this token) that is currently selected
			 * @private
			 */
			this._selectedDocumentVersionId = null;

			/**
			 * @member {String} id for the error handling part if the token sits in the error pin
			 * @private
			 */
			this._errorHandlingId = null;

			/**
			 * If this is a wait token, it has a timer.
			 * @member {Number} milliseconds until wait is over
			 * @private
			 */
			this._timer = 0;

			/**
			 * Object holding the subtasks of this token. Each subtask has a name,an owner and a status
			 */
			this._subTasks = {};

			if (data) { //parse provided XML token data
				this._processXMLData(data);
			}
		};

		WebCenterToken.prototype = Object.create(Token.prototype);
		WebCenterToken.prototype.constructor = WebCenterToken;

		/**
		 * Add a file to the token
		 * @param {String} fileName
		 * @param {String} fileId
		 */
		WebCenterToken.prototype.addFile = function (fileName, fileId) {
			this._files[fileId] = fileName;
		};

		/**
		 * Get all the files that are part of this token.
		 * @returns {Map.<String, String>}
		 */
		WebCenterToken.prototype.getFiles = function () {
			if (this.getPin().getPinType() === WebCenterPinConstants.Type.In && _.size(this._files) === 0) {
				this._forwardFilesFromParent();
			}
			return this._files;
		};

		/**
		 * Get the number of files in this token.
		 * @returns {Number}
		 */
		WebCenterToken.prototype.getFileCount = function () {
			if (this.getPin().getPinType() === WebCenterPinConstants.Type.In && _.size(this._files) === 0) {
				this._forwardFilesFromParent();
			}
			return _.size(this._files);
		};

		/**
		 * Creates the file for this token form its parents
		 */
		WebCenterToken.prototype._forwardFilesFromParent = function () {
			this._files = {};
			_.each(this._parentTokens, function (parent) {
				_.each(parent.getFiles(), function (file, fileId) {
					this._files[fileId] = file;
				}, this);
			}, this);
		};

		/**
		 * Adds an error message to this token.
		 * @param {String} errorId
		 * @param {String} errorMessage
		 */
		WebCenterToken.prototype.addErrorMessage = function (errorId, errorMessage) {
			this._errors[errorId] = errorMessage;
		};

		/**
		 * Get all the errors of this token.
		 * @returns {Array.<object>}
		 */
		WebCenterToken.prototype.getErrors = function () {
			return this._errors;
		};

		/**
		 * Does this token contain any errors.
		 * @returns {Boolean}
		 */
		WebCenterToken.prototype.hasErrors = function () {
			return _.size(this._errors) > 0;
		};

		/**
		 * Update the Token with new XML Data
		 * @param {XMLObject} XML data
		 */
		WebCenterToken.prototype.updateFromXMLData = function (data) {
			// clear out existing errors and files
			this._errors = {};
			this._files = {};

			if (data) {
				// process the xml
				this._processXMLData(data);
			}
		};

		/**
		 * Processes the XML data and adds the parsed data to the token
		 * @param {XMLObject} data
		 */
		WebCenterToken.prototype._processXMLData = function (data) {
			var filesEl = data.getElementsByTagName("files");
			if (filesEl && filesEl[0]) {
				_.each(filesEl[0].childNodes, function (fileEl) {
					if (!fileEl || fileEl.nodeType !== 1 /*Node.ELEMENT_NODE*/ ) {
						return;
					}
					var fileName = fileEl.getElementsByTagName("name");
					var fileId = fileEl.getElementsByTagName("id");
					var fileIsProcessedEl = fileEl.getElementsByTagName("processed");
					var fileIsProcessed = false;
					if (fileIsProcessedEl && fileIsProcessedEl[0]) {
						fileIsProcessed = fileIsProcessedEl[0].textContent === "1";
					}

					if (fileName && fileId && fileName[0] && fileId[0] && !fileIsProcessed) {
						this.addFile(fileName[0].textContent, fileId[0].textContent);
					}
				}, this);
			}
			var errorsEl = data.getElementsByTagName("errors");
			if (errorsEl && errorsEl[0]) {
				_.each(errorsEl[0].childNodes, function (errorEl) {
					if (!errorEl || errorEl.nodeType !== 1) {
						return;
					}
					var messageEl = errorEl.getElementsByTagName("message");
					var errorIdEl = errorEl.getElementsByTagName("id");
					if (messageEl && messageEl[0] && errorIdEl && errorIdEl[0]) {
						this.addErrorMessage(errorIdEl[0].textContent, messageEl[0].textContent);
					}
				}, this);
			}
			var timerEl = data.getElementsByTagName("timer");
			if (timerEl && timerEl[0]) {
				this._timer = parseInt(timerEl[0].textContent);
			}
			// Get token owner and token name
			var subtasksEl = data.getElementsByTagName("subtasks");
			if (subtasksEl && subtasksEl[0]) {
				_.each(subtasksEl[0].childNodes, function (subtaskEl) {
					if (!subtaskEl || subtaskEl.nodeType !== 1) {
						return;
					}
					var taskId = subtaskEl.getElementsByTagName("taskid");
					var owner = subtaskEl.getElementsByTagName("owner");
					var name = subtaskEl.getElementsByTagName("name");
					var status = subtaskEl.getElementsByTagName("status");
					if (taskId && taskId[0] && owner && owner[0] && name && name[0] && status && status[0]) {
						this._addSubtask(taskId[0].textContent, owner[0].textContent, name[0].textContent, status[0].textContent);
					}
				}, this);
			}
		};

		/**
		 * Get the weight of the token, this is the number shown on the pin
		 * @returns {Number}
		 */
		WebCenterToken.prototype.getWeight = function () {
			return _.size(this._files) || 1; //return 1 if the token contains no files
		};

		/**
		 * Is the token been cloned from another token
		 * @returns {Boolean}
		 */
		WebCenterToken.prototype.isCloned = function () {
			return this._cloneId !== null;
		};

		/**
		 * Returns the ID of the token this token was cloned from
		 * @returns {String|null}
		 */
		WebCenterToken.prototype.getClonedId = function () {
			return this._cloneId;
		};

		/**
		 * Set the selected state of the document, possible to set the selected document version id
		 * @param {Boolean} isSelected
		 * @param {String} versionId
		 */
		WebCenterToken.prototype.setSelected = function (isSelected, versionId) {
			if (this._isSelected !== isSelected ||
				versionId && this._selectedDocumentVersionId !== versionId ||
				!versionId && this._selectedDocumentVersionId !== null) {
				// selected state changed

				this._isSelected = isSelected;
				if (versionId) {
					this._selectedDocumentVersionId = versionId;
				} else {
					this._selectedDocumentVersionId = null;
				}

				this._emit("changed.select");
			}

		};

		/**
		 * Get the selected document version id.
		 * @returns {String|null}
		 */
		WebCenterToken.prototype.getSelectedDocumentVersionId = function () {
			return this._selectedDocumentVersionId;
		};

		/**
		 * Set the highlighted state of the token
		 * @param {Boolean} highlight
		 */
		WebCenterToken.prototype.setHighlighted = function (highlight) {
			var clonedToken = this._workflow.getToken(this._cloneId);
			//get the token this token was cloned from and if found set the highlighted state
			if (clonedToken) { //this is the token that is shown in the workflow feedback
				clonedToken.setHighlighted(highlight);
			}
			Token.prototype.setHighlighted.call(this, highlight);
		};

		/**
		 * Set the id for the error handling this is used for the retry action of the token
		 * @param {String} errorHandlingId
		 */
		WebCenterToken.prototype.setErrorHandlingId = function (errorHandlingId) {
			this._errorHandlingId = errorHandlingId;
		};

		/**
		 * Returns the id for error handling.
		 * @returns {String}
		 */
		WebCenterToken.prototype.getErrorHandlingId = function () {
			return this._errorHandlingId;
		};

		/**
		 * returns the timer of the token
		 * @returns {Number} timer
		 */
		WebCenterToken.prototype.getTimer = function () {
			return this._timer;
		};

		/**
		 * returns the subtasks of this token
		 * @returns {Object} - subtasks have a name and an owner
		 */
		WebCenterToken.prototype.getSubtasks = function () {
			return this._subTasks;
		};

		/**
		 * adds a subtask to this token
		 * @param {String} id
		 * @param {String} owner
		 * @param {String} name
		 * @param {String} status
		 */
		WebCenterToken.prototype._addSubtask = function (id, owner, name, status) {
			var subTaskObject = {};
			this._subTasks[id] = subTaskObject;
			subTaskObject.owner = owner;
			subTaskObject.name = name;
			subTaskObject.status = status;
		};

		/**
		 * Returns the amount of subtasks in this token
		 * @returns {Number}
		 */
		WebCenterToken.prototype.getSubtaskCount = function () {
			return _.size(this._subTasks);
		};




		return WebCenterToken;
	});