define([
		"WFEditor/workflow/token/TokenView",
		"WFEditor/utils/IconDataStorage",
		"utils/webi18n!WFEditorWCR",
		"utils/ShapeUtils",
		"utils/dateformatter",
		"WFEditorWCR/scene/WebCenterViewConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/token/WebCenterTokenConstants",
		"WFEditorWCR/webcenterInformationPopup/WebCenterTokenInformationPopup",
		"WFEditor/scene/hitarea/HitArea",
		"underscore"
	],
	function (
		TokenView,
		IconDataStorage,
		webi18n,
		ShapeUtils,
		DateFormatter,
		WebCenterViewConstants,
		WebCenterNodeConstants,
		WebCenterTokenConstants,
		WebCenterTokenInformationPopup,
		HitArea,
		_

	) {
		"use strict";

		/**
		 *
		 * @exports WebCenterTokenView
		 * @class
		 * @augments TokenView
		 *
		 * @param {WebCenterTokenModel} tokenModel
		 * @param {WebCenterPinView} pinView
		 * @param {Scene} scene
		 */
		var WebCenterTokenView = function (tokenModel, pinView, scene) {
			TokenView.call(this, tokenModel);

			/**
			 * @member {PinView}
			 * @private
			 */
			this._pinView = pinView;

			/**
			 * @member {WebCenterTokenInformationPopup}
			 * @private
			 */
			this._popup = null;

			/**
			 * @member{Boolean} Is the popup opened
			 * @private
			 */
			this._isPopupOpen = false;

			var wcrConfig = this._model.getWorkflow().getWebCenterConfig();

			/**
			 * @member {String}
			 * @private
			 */
			this._fileIconUrl = wcrConfig.requireBaseUrl + "WFEditorWCR/icons/";

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = scene;

			/**
			 * @member {SceneModel}
			 * @private
			 */
			this._sceneModel = scene.getSceneModel();

			/**
			 * @member {ShapeUtils.Rect} the rectangle of the lifeline between pin and popup (if it is being drawn)
			 * @private
			 */
			this._popupLifelineRect = null;

			/**
			 * @member {Boolean} has the location of the popup changed?
			 * @private
			 */
			this._isPopupLocationDirty = false;

			// location change of node
			this._observe(tokenModel.getPin().getNode(), "changed.location", this._locationChanged);
			this._observe(tokenModel, "changed.*", this.makeDirty);
			this._observe(tokenModel, "changed.hover", this._hoverChanged);

		};

		WebCenterTokenView.prototype = Object.create(TokenView.prototype);
		WebCenterTokenView.prototype.constructor = WebCenterTokenView;

		/**
		 * React on location changes of the node
		 */
		WebCenterTokenView.prototype._locationChanged = function () {
			this._updateBoundingRect();
			this._updateHitAreas();
			this.makeDirty();
		};

		/**
		 * @override
		 * Renders the a in progress token above its node, if hovered, also renders the file name
		 * @param {CanvasRenderingContext2D} context
		 * @param {Layer} layer
		 * @param {Boolean} quickRender - if true, only high priority render actions should be executed (i.e. during panning)
		 */
		WebCenterTokenView.prototype.render = function (context, layer, quickRender) {
			if (quickRender) {
				// Don't render tokens when quick rendering
				return;
			}
			var location = this._pinView.getLocationForToken(this._model);
			var iconStorage = IconDataStorage.getInstance(); //get the singleton instance of the datastorage
			//request the icon from the datastorage, returns the icon if available, if not it starts loading the icon.
			var icon = iconStorage.get(this._getTokenStatusFileIconUrl(), _.bind(function () {
				layer.makeDirty();
			}, this));



			// only create popup when needed
			if (this._popup) {

				if (this._popup.isMoved() && !this._isPopupLocationDirty) {
					context.save();
					context.strokeStyle = WebCenterViewConstants.TokenStatusTextColor;
					context.beginPath();
					context.moveTo(location.x + 9, location.y + 9);
					var popupLocation = this._popup.getLocation();
					context.lineTo(popupLocation.x, popupLocation.y);
					context.stroke();
					context.restore();

					this._popupLifelineRect = new ShapeUtils.Rect(location.copy());
					this._popupLifelineRect.size = location.sizeTo(popupLocation);
				} else {
					this._popupLifelineRect = null;
				}

				if (this._popup.isVisible()) {
					this._updateBoundingRect();
					this._popup.render();
				}
			}

			if (icon) { //only render if the icon was already fetched
				context.drawImage(icon, location.x, location.y);
			}

			context.save();

			var status = this.getStatusText(); // Which text should be displayed next to the file icon?
			context.font = WebCenterViewConstants.TokenSatusFont;
			location.offsetXY(WebCenterViewConstants.TokenStatusFileIconWidth, 12);
			this._renderTextBackground(context, status, location);
			context.fillStyle = WebCenterViewConstants.TokenStatusTextColor;
			context.globalAlpha = 1;
			context.fillText(status, location.x, location.y);
			location.offsetXY(-WebCenterViewConstants.TokenStatusFileIconWidth, 0);

			context.restore();
		};

		/**
		 * @override
		 */
		WebCenterTokenView.prototype._updateHitAreas = function () {
			// The hit area for a token is the location of the file icon
			var tokenLocation = this._pinView.getLocationForToken(this._model);
			var iconRect = new ShapeUtils.Rect(tokenLocation, new ShapeUtils.Size(WebCenterViewConstants.TokenStatusFileIconWidth, WebCenterViewConstants.TokenStatusFileIconWidth));
			this._setHitAreas([new HitArea(iconRect, this)]);
		};

		/**
		 * @override
		 */
		WebCenterTokenView.prototype._updateBoundingRect = function () {
			var origin = this._pinView.getLocationForToken(this._model);
			var size = new ShapeUtils.Size(WebCenterViewConstants.TokenStatusFileIconWidth, WebCenterViewConstants.TokenStatusFileIconWidth);
			var text = this.getStatusText();
			if (text) {
				this._context.save();
				this._context.font = WebCenterViewConstants.TokenSatusFont;
				var textWidth = this._context.measureText(text).width;

				size.width += textWidth;
				this._context.restore();
			}
			var boundingRect = new ShapeUtils.Rect(origin, size);

			if (this._popupLifelineRect && this._popup.isVisible()) {
				boundingRect = boundingRect.calculateArea(this._popupLifelineRect);
			}

			this._setBoundingRect(boundingRect.getNormalizedRectArray());
		};

		/**
		 * Renders the opaque white background for text
		 * @param {CanvasRenderingContext2D} context
		 * @param {String} text
		 * @param {ShapeUtils.Point} textLocation
		 */
		WebCenterTokenView.prototype._renderTextBackground = function (context, text, textLocation) {
			context.save();
			var width = context.measureText(text).width;
			context.fillStyle = "rgb(255,255,255)";
			context.globalAlpha = 0.5;
			context.fillRect(textLocation.x - 2, textLocation.y + 4, width + 4, -16);
			context.restore();
		};

		/**
		 * Returns the text that should be displayed for this token in progress
		 * @returns {String}
		 */
		WebCenterTokenView.prototype.getStatusText = function () {
			var nodeType = this._model.getPin().getNode().getNodeType();

			if (this._model.getSubtaskCount() > 1) {
				// If there are multiple subtasks, check how much are completed
				var subtasksCompleted = 0;
				_.each(this._model.getSubtasks(), function (subtask) {
					if (subtask.status === WebCenterNodeConstants.Status.COMPLETED) {
						subtasksCompleted++;
					}
				});
				return webi18n.t("TasksCompleted", subtasksCompleted, this._model.getSubtaskCount());
			}

			// One or Zero subtasks, the text is in function of the node
			switch (nodeType) {
			case WebCenterNodeConstants.Nodes.STARTAPPROVAL:
				return webi18n.t("WaitingApproval");
			case WebCenterNodeConstants.Nodes.DATACOLLECTOR:
				return webi18n.t("WaitingForTasks");
			case WebCenterNodeConstants.Nodes.JOIN:
				return webi18n.t("WaitingForTasks");
			case WebCenterNodeConstants.Nodes.WAIT:
				return webi18n.t("WaitingUntil", DateFormatter.formatDate(new Date(this._model.getTimer()), "NNN dd, yyyy HH:mm:ss"));
			case WebCenterNodeConstants.Nodes.SUBMITWORKFLOW:
				return webi18n.t("RunningOnAE");
			}
			return webi18n.t("InProgress"); // Defaults to "In Progress"   
		};

		/**
		 * Returns the url of the icon for this token
		 * @returns {String}
		 */
		WebCenterTokenView.prototype._getTokenStatusFileIconUrl = function () {
			var nodeType = this._model.getPin().getNode().getNodeType();
			if (nodeType === WebCenterNodeConstants.Nodes.WAIT) {
				return this._fileIconUrl + WebCenterTokenConstants.Icons.WAIT;
			}
			if (this._model.getSubtaskCount() > 1) {
				return this._fileIconUrl + WebCenterTokenConstants.Icons.MULTIPLETASKS;
			}
			return this._fileIconUrl + WebCenterTokenConstants.Icons.DEFAULT;

		};

		/**
		 * Mark the popup open
		 */
		WebCenterTokenView.prototype.setPopupOpen = function (isPopupOpen) {

			if (this._isPopupOpen !== isPopupOpen) {
				if (isPopupOpen) {
					this._setSceneModelListeners();
				} else {
					this._stopObserveObject(this._sceneModel);
				}
				this._isPopupOpen = isPopupOpen;
				this.makeDirty();
			}
		};

		/**
		 * @override
		 * clears the resources of the popup
		 */
		WebCenterTokenView.prototype.destroyView = function () {
			if (this._popup) {
				this._popup.destroyPopup();
				delete this._popup;
			}
		};


		/**
		 * Hover state of popup or token has changed
		 */
		WebCenterTokenView.prototype._hoverChanged = function () {
			if (this._popup) {
				this._popup.cancelDeferedClose();
			} else if (this._model.isHovered()) {
				this._popup = new WebCenterTokenInformationPopup(this._pinView, this._model, this._scene, this);
				this._popup.show();
				this._observe(this._popup, "changed.hover", this._hoverChanged);
				this._observe(this._popup, "changed.visible", function () {
					this.makeDirty();
					this._updateBoundingRect();
				});
				this.makeDirty();
			}

			if (this._popup) {
				if (!this._popup.isHovered() && !this._model.isHovered() && !this._popup.isSticky()) {
					this._popup.deferredClose();
				}

				if (this._popup.isHovered() || this._model.isHovered()) {
					this._popup.show();
				}
			}
		};


		/**
		 * Listen to sceneModel for panning and zoom changes
		 */
		WebCenterTokenView.prototype._setSceneModelListeners = function () {
			this._observe(this._sceneModel, "panningOffsetChanged", _.bind(function () {
				if (this._popup && this._popup.isVisible()) {

					// defer to ensure transformation has been done. The relocation of the popup doesn't happen in the render
					// method because if the pin isn't in the visible rect anymore, the render method won't be called
					_.defer(_.bind(function () {
						this._popup.relocate();
						this.makeDirty();
					}, this));
				}
			}, this));

			this._observe(this._sceneModel, "zoomChanged", _.bind(function () {
				if (this._popup && this._popup.isVisible()) {
					// for zooming the change in location will be very big so we temporarily don't want to render the 
					// lifeline for popups as it will cause it to visibly jump from one position to another
					this._isPopupLocationDirty = true;

					// defer to ensure transformation has been done.
					_.defer(_.bind(function () {
						this._popup.relocate();
						this.makeDirty();
						this._isPopupLocationDirty = false;
					}, this));
				}
			}, this));
		};



		return WebCenterTokenView;


	});