define(["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/compare/CompareConstants",
		"web-viewer/compare/CompareDifferenceSubToolbarComponent",
		"web-viewer/compare/CompareSideBySideSubToolbarComponent",
		"underscore"
	],
	function (ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		LayoutConstants,
		CompareConstants,
		CompareDifferenceSubToolbarComponent,
		CompareSideBySideSubToolbarComponent,
		_) {
		"use strict";

		/**
		 * create a new compare toolbar component
		 * @memberof CompareModule
		 * @exports CompareToolbarComponent
		 * @class The toolbar component for the compare module
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {CompareModule.CompareModel} compareModel
		 */
		var CompareToolbarComponent = function (f, compareModule) {
			ToolbarComponent.call(this, LayoutConstants.ToolbarComponent.COMPARE, f);
			this.f = f;
			this.compareModule = compareModule;
			this.compareModel = compareModule.getCompareModel();
			this.buttons = {};
			this.modesGroup = new ButtonGroup("Compare", []);
			this.modesGroup.setTitle(this.f.i18n, "viewer", "CompareToolsTitle");
			this.addButtonGroup(this.modesGroup);

			this.compareModel.on("changed.mode", _.bind(function (oldMode, newMode) {
				if (oldMode && this.buttons[oldMode.getName()] !== undefined) {
					this.buttons[oldMode.getName()].setActive(false);
				}

				if (newMode && this.buttons[newMode.getName()] !== undefined) {
					this.buttons[newMode.getName()].setActive(true);
				}
			}, this));

			this.addButtonForMode(CompareConstants.mode.SHOW_CURRENT, "icon-WCR_Compare-Current", "DiffCurrentTT");
			this.addButtonForMode(CompareConstants.mode.SHOW_DIFFERENCE, "icon-WCR_Compare-Reference", "DiffPrevTT");
			this.addButtonForMode(CompareConstants.mode.BLINKING, "icon-WCR_Compare-Blinking", "DiffBothBlinkingTT");
			this.addButtonForMode(CompareConstants.mode.SHOWBOTHONTOP, "icon-WCR_Compare", "DiffShowBothOnTopTT");

			var differenceCompareMode = this.compareModel.getModeByName(CompareConstants.mode.DIFFERENCE);
			if (differenceCompareMode !== undefined) {
				this.addDifferenceModeButton(differenceCompareMode);

				var updateBadges = _.debounce(_.bind(this._updateBadges, this), 0);

				differenceCompareMode.on("changed.threshold", updateBadges);
				this.compareModel.on("changed.mode", updateBadges);
				this.f.getDocumentController().whenActivePageChanged(updateBadges);

				var sideBySideCompareMode = this.compareModel.getModeByName(CompareConstants.mode.SIDEBYSIDE);
				if (sideBySideCompareMode !== undefined) {
					this.addSideBySideModeButton(differenceCompareMode, sideBySideCompareMode);
					sideBySideCompareMode.on("changed.showDifference", updateBadges);
				}

				this._updateBadges();
			}



		};

		CompareToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		CompareToolbarComponent.prototype.constructor = CompareToolbarComponent;
		CompareToolbarComponent.toolbarName = LayoutConstants.ToolbarComponent.COMPARE;


		/**
		 * Add a button to active/deactivate a mode
		 * @param {String|CompareModule.CompareConstants.mode} name - name of the compare mode
		 * @param {String} icon - name of the icon
		 * @param {String} tooltip - tooltip
		 */
		CompareToolbarComponent.prototype.addButtonForMode = function (name, icon, tooltip) {
			this.buttons[name] = new ViewerToolbarButton(this.f, "compare_" + name, icon, true, _.partial(function (that) {
				if (this.isActive()) {
					that.compareModule.activateModeByName();
				} else {
					that.compareModule.activateModeByName(name);
				}
			}, this));
			this.buttons[name].setTooltip("viewer", tooltip);
			this.modesGroup.addButton(this.buttons[name]);
		};


		/**
		 * Difference compare mode button
		 * @param differenceCompareMode
		 */
		CompareToolbarComponent.prototype.addDifferenceModeButton = function (differenceCompareMode) {
			this.addButtonForMode(CompareConstants.mode.DIFFERENCE, "icon-WCR_Compare-BothDiff", "compareDifferenceMode");

			var button = this.buttons[CompareConstants.mode.DIFFERENCE];
			button.addSubtoolbarComponent(new CompareDifferenceSubToolbarComponent(this.f, differenceCompareMode, button));

			this._differenceCompareMode = differenceCompareMode;
		};


		/**
		 * Side by side compare mode button
		 * @param {CompareModule.CompareDifferenceMode} differenceCompareMode
		 * @param {CompareModule.CompareSideBySideMode} sideBySideCompareMode
		 */
		CompareToolbarComponent.prototype.addSideBySideModeButton = function (differenceCompareMode, sideBySideCompareMode) {
			this.addButtonForMode(CompareConstants.mode.SIDEBYSIDE, "icon-WCR_Compare2-SideBySide", "DiffBothSplitTT");

			var button = this.buttons[CompareConstants.mode.SIDEBYSIDE];
			button.addSubtoolbarComponent(new CompareSideBySideSubToolbarComponent(this.f, differenceCompareMode, sideBySideCompareMode, button));
			this._sideBySideCompareMode = sideBySideCompareMode;
		};


		/**
		 * Update the Warning Badges for the Threshold for difference and side by side mode
		 */

		CompareToolbarComponent.prototype._updateBadges = function () {
			var button;

			if (this.buttons[CompareConstants.mode.DIFFERENCE] && this._differenceCompareMode) {
				// difference compare mode is available
				button = this.buttons[CompareConstants.mode.DIFFERENCE];
				if (this._differenceCompareMode.isEnabled() && this._differenceCompareMode.getThreshold() > 0 && !this.f.getActivePage().hasCad()) {
					// show warning
					button.setBadge("!", "warning");
					button.setTooltip("viewer", "compareDifferenceThresholdWarning");
				} else {
					button.setBadge();
					button.setTooltip("viewer", "compareDifferenceMode");
				}

			}

			if (this.buttons[CompareConstants.mode.SIDEBYSIDE]) {
				// side by side compare mode is available
				button = this.buttons[CompareConstants.mode.SIDEBYSIDE];

				if (this._sideBySideCompareMode.isEnabled() && this._sideBySideCompareMode.isShowingDifference() && this._differenceCompareMode.getThreshold() > 0 && !this.f.getActivePage().hasCad()) {
					button.setBadge("!", "warning");
					button.setTooltip("viewer", "compareDifferenceThresholdWarning");
				} else {
					button.setBadge();
					button.setTooltip("viewer", "DiffBothSplitTT");
				}
			}
		};


		return CompareToolbarComponent;
	});