define(["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/compare/CompareConstants",
		"underscore"
	],
	function (ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		LayoutConstants,
		CompareConstants,
		_) {
		"use strict";

		/**
		 * create a new compare toolbar component
		 * @memberof CompareModule
		 * @exports CompareTouchToolbarComponent
		 * @class The toolbar component for the compare module
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {CompareModule.CompareModel} compareModel
		 */
		var CompareTouchToolbarComponent = function (f, compareModule) {
			ToolbarComponent.call(this, LayoutConstants.ToolbarComponent.COMPARE, f);
			this.f = f;
			this._compareModule = compareModule;
			this._compareModel = compareModule.getCompareModel();
			this._buttons = {};
			this._modesGroup = new ButtonGroup("Compare", []);
			this._modesGroup.element.removeClass("btn-group");
			this._modesGroup.element.addClass("compareModes");
			this._modesGroup.setTitle(this.f.i18n, "viewer", "CompareToolsTitle");
			this.addButtonGroup(this._modesGroup);



			var differenceCompareMode = this._compareModel.getModeByName(CompareConstants.mode.DIFFERENCE);
			this.addDifferenceModeButton(differenceCompareMode);

			this.addButtonForMode(CompareConstants.mode.BLINKING, "icon-WCR_Compare-Blinking", "DiffBothBlinkingTT");


			var sideBySideCompareMode = this._compareModel.getModeByName(CompareConstants.mode.SIDEBYSIDE);
			this.addSideBySideModeButton(differenceCompareMode, sideBySideCompareMode);



			this.addButtonForMode(CompareConstants.mode.SHOWBOTHONTOP, "icon-WCR_Compare-Both", "DiffShowBothOnTopTT");

			this.addButtonForMode(CompareConstants.mode.SHOW_CURRENT, "icon-WCR_Compare-Current", "DiffCurrentTT");
			this.addButtonForMode(CompareConstants.mode.SHOW_DIFFERENCE, "icon-WCR_Compare-Reference", "DiffPrevTT");


			this._compareModel.on("changed.mode", _.bind(this._updateActiveStatus, this));
			this._updateActiveStatus(undefined, this._compareModule.getActiveMode());



		};

		CompareTouchToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		CompareTouchToolbarComponent.prototype.constructor = CompareTouchToolbarComponent;
		CompareTouchToolbarComponent.toolbarName = LayoutConstants.ToolbarComponent.COMPARE;


		/**
		 * Update the active flag for the compare mode buttons
		 * @param {String} oldMode 
		 * @param {String} newMode 
		 */
		CompareTouchToolbarComponent.prototype._updateActiveStatus = function (oldMode, newMode) {
			if (oldMode && this._buttons[oldMode.getName()] !== undefined) {
				this._buttons[oldMode.getName()].setActive(false);
			}

			if (newMode && this._buttons[newMode.getName()] !== undefined) {
				this._buttons[newMode.getName()].setActive(true);
			}
		};

		/**
		 * Add a button to active/deactivate a mode
		 * @param {String|CompareModule.CompareConstants.mode} name - name of the compare mode
		 * @param {String} icon - name of the icon
		 * @param {String} tooltip - tooltip
		 */
		CompareTouchToolbarComponent.prototype.addButtonForMode = function (name, icon, tooltip) {
			this._buttons[name] = new ViewerToolbarButton(this.f, "compare_" + name, icon, true, _.partial(function (that) {
				if (this.isActive()) {
					that._compareModule.activateModeByName();
				} else {
					that._compareModule.activateModeByName(name);
				}
			}, this));
			this._buttons[name].setTooltip("viewer", tooltip);
			this._modesGroup.addButton(this._buttons[name]);
		};


		/**
		 * Difference compare mode button
		 * @param differenceCompareMode
		 */
		CompareTouchToolbarComponent.prototype.addDifferenceModeButton = function (differenceCompareMode) {
			this.addButtonForMode(CompareConstants.mode.DIFFERENCE, "icon-WCR_Compare-BothDiff", "compareDifferenceMode");


		};


		/**
		 * Side by side compare mode button
		 * @param {CompareModule.CompareDifferenceMode} differenceCompareMode
		 * @param {CompareModule.CompareSideBySideMode} sideBySideCompareMode
		 */
		CompareTouchToolbarComponent.prototype.addSideBySideModeButton = function (differenceCompareMode, sideBySideCompareMode) {
			this.addButtonForMode(CompareConstants.mode.SIDEBYSIDE, "icon-WCR_Compare-SideBySide", "DiffBothSplitTT");

		};
		return CompareTouchToolbarComponent;
	});