define([],
	function () {
		/**
		 * Utility tool to enable easy overriding of methods (with possibility to call the overriden method in the new implementation)
		 * 
		 * Usage: install Extendable to an object that you want to be easily extenable
		 * 
		 * Extendable.makeExtendable(obj)
		 * 
		 * To override a method 
		 * @exports Extendable
		 * @memberof Core
		 * @mixin 
		 * @deprecated proper prototyping should be used. Extendable is no longer needed
		 * @example
		 * this._override("methodName", function(x,y,z){
		 * 		//new implementation
		 * 		this._super(x,y,z); //call to original implementation of methodName
		 * 		// further new implementation
		 * });
		 * 
		 * var Abstract = function(){
		 * 		Extendable.makeExtendable(this);
		 * 		this.someAbstractMethod = function(){
		 * 			this.doSomething();
		 * 			// other stuff
		 * 		}
		 * };
		 * 
		 * var SpecificAbstract = function(){
		 * 		Abstract.call(this);
		 * 		
		 * 		this._override("someAbstractMethod", function(){
		 * 			//first call old implementation
		 * 			this._super();
		 * 			// new implementation
		 * 			this.doSomethingElse();
		 * 		});
		 * };
		 * SpecificAbstract.prototype = new Abstract();
		 * SpecificAbstract.prototype.constructor = SpecificAbstract;
		 */
		var Extendable = {
			/**
			 * Install the Extendable mixin to obj
			 * @static
			 * @param {Object} obj - The object that should become extendable
			 */
			installTo: function (obj) {
				/**
				 * Override an existing method
				 * 
				 * @alias Extendable~_override
				 * @param {String} method - name of the method to override
				 * @param {Function} override - the new implementation for method. Previous implementation is still available through this._super 
				 */
				obj._override = function (method, override) {
					var superMethod = this[method];
					if (superMethod === undefined) {
						superMethod = function () {};
					}
					this[method] = function () {
						var old_super = this._super;
						this._super = superMethod;
						var result = override.apply(this, arguments);
						this._super = old_super;
						return result;
					};
				};
			}
		};

		return Extendable;
	});