/**
 * @namespace Data
 */
define(["web-viewer/core/PrivateChannel"],
	function (PrivateChannel) {
		"use strict";
		/**
		 * Construct the data adapter
		 * 
		 * @exports AbstractDataAdapter
		 * @memberof Data
		 * @class DataAdapter is responsible for loading new documents and (their) data.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var AbstractDataAdapter = function (f, instance) {
			/**
			 * @protected
			 */
			this.f = f;

			/**
			 * @private
			 * instance URL for the application
			 */
			this._instance = instance;

			PrivateChannel.installTo(this, f, ["doc"]);
		};

		/**
		 * Open document(s)
		 * @param {any} documentsIdentifier - String or JSON object containing information on which document to load
		 * @param {Function} callback - Callback fired with as first argument the document (if loading succeeded, otherwise null) and as second argument whether the document should be made active. Can be fired multiple times when there are multiple documents to be loaded.
		 */
		AbstractDataAdapter.prototype.openDocuments = function (documentsIdentifier, callback) {
			// open documents based on documentsIdentifier
		};

		/**
		 * Returns the instance for the web application
		 * @returns {String}
		 */
		AbstractDataAdapter.prototype.getInstance = function () {
			return this._instance;
		};
	
		/**
		 * Returns an instance for the I18n provider
		 * @returns {I18n}
		 */
		AbstractDataAdapter.prototype.getI18nProvider = function(){

			// to be implemetened by specific dataadapter
			return null;
		};

		/**
		 * Load basic information on the document
		 * 
		 * @param {Any} fileIdentifier - identifier used by the adapter to get the needed document information
		 * @param {String} IID - internal ID of the document that is begin loaded
		 * @param {Function} callback - when the document is loaded, the callback function is called with the Document object as first argument, if loading failed the first argument is null
		 */
		AbstractDataAdapter.prototype.loadDocument = function (fileIdentifier, IID, callback) {
			// load the document
			// call the callback with the new Document object as first argument
			// if failed, have "null" as first element and send an error event
		};

		/**
		 * Load all documents of the passed object. Sets the link between graphics and CAD files.
		 * 
		 * for linked CAD files, no callback should be called
		 * @param {Object} documentsIdentifier - object with all document information (specific for webcenter)
		 * @param {Function} callback - for every generated document, the callback is executed
		 */
		AbstractDataAdapter.prototype.loadDocuments = function (documentsIdentifier, callback) {
			//information passed is an object containing all the data for initializing documents, documentversions and documentpages		
		};


		/**
		 * Makes a deep copy of a page, this means it reinitializes the document and document version. All parameters and data of the document, 
		 * document version and document page are copied to the new document, document version and document page 
		 * 
		 * @param {AbstractDocumentPage} page
		 * @returns {AbstractDocumentPage}
		 */
		AbstractDataAdapter.prototype.getDeepCopy = function (page) {
			//to be implemented by a specific dataAdapter
		};

		/**
		 * Load previous versions of the document for a given document page. If the version does not have the page with given pagenumber the version
		 * will not be loaded (only happens for pagelist documents). 
		 * 
		 * @param {Object} data
		 * @param {AbstractDocumentPage} document
		 * @param {Function} callback
		 * @param {WebCenterDocumentPage} [documentPage]
		 */
		AbstractDataAdapter.prototype.loadVersionsOfDocumentPage = function (data, document, callback, documentPage) {
			//to be implemented by a specific dataAdapter
		};


		/**
		 * Update the given pages with correct data.
		 * 
		 * !! Note !! if two consecutive calls are done, the first (ongoing) call should be cancelled, only the last call result can update the parameters
		 * @param {AbstractDocumentPage} currentPage - the currenct active page
		 * @param {AbstractDocumentPage} differencePage - the current d
		 * @param {AbstractDocumentPage} cadOverlay - the current cad overlay
		 * @param {function} callback - callback to be called when parameter call has finished
		 */
		AbstractDataAdapter.prototype.updateDocumentParameters = function (currentPage, differencePage, cadOverlay, callback) {
			//to be implemented by a specific dataAdapter
		};

		/**
		 * Update the data of the document, also updates the document reference info
		 * @param {AbstractDocumentPage}   page  
		 */
		AbstractDataAdapter.prototype.updateDocumentData = function (page) {
			//to be implemented by a specific dataAdapter
		};

		/**
		 * If a parameter update is ongoing, it should be cancelled, and it it shouldn't update any documents or trigger events
		 */
		AbstractDataAdapter.prototype.cancelDocumentParametersUpdate = function () {
			// to be implemented  by a specific dataAdapter
		};

		/**
		 * Save the CAD overlay settings
		 * @param {AbstractDocumentPage} graphicsPage - the graphics page in which the CAD reference should be stored
		 * @param {AbstractDocumentPage} cadOverlay   - the CAD document that should be set as reference
		 * @param {Number}               rotation     - rotation of the CAD overlay (0,90,180,270 degrees)
		 * @param {Boolean}              isFlipped    - is the CAD overlay flipped
		 * @param {ShapeUtils.Point}     offset       - the offset for the CAD overlay
		 * @param {Function}             [callback]   - callback function to be called when CAD overlay is saved (first parameter of the callback is a boolean indicating success)
		 */
		AbstractDataAdapter.prototype.saveCadOverlay = function (graphicsPage, CADPage, rotation, isFlipped, offset, callback) {
			// to be implemented by a specific dataAdapter
		};

		/**
		 * Open a 3D Viewer for a given page in a new browser tab
		 * @param {AbstractDocumentPage} page
		 */
		AbstractDataAdapter.prototype.show3DView = function (page) {
			//to be implemented
		};
		return AbstractDataAdapter;
	});