define(["web-viewer/core/Extendable",
		"web-viewer/core/PrivateChannel",
		"underscore"
	],
	function (Extendable,
		PrivateChannel
	) {
		"use strict";
		/**
		 * @class Abstract representation of a Document that can be loaded in the viewer. No
		 * data is fetched in the AbstractClass. Extend this class to have a working
		 * version fetching the correct information
		 * 
		 * each document has an internal id: documentID
		 * 
		 * if a change occurs events should be sent on the channel
		 * doc.[documentID]
		 * 
		 * @exports AbstractDocument
		 * @memberof Data
		 * @constructor
		 * @param {Core.ViewerFacade} f 
		 * @param {Any} documentID - Internal ID of the document
		 * @param {Data.DataConstants.documentPageType} type - type of the document
		 */
		var AbstractDocument = function (f, documentID, type) {
			PrivateChannel.installTo(this, f, ["doc", documentID]);

			/**
			 * @private
			 */
			this.f = f;
			/**
			 * @private
			 * @constant
			 */
			this.documentID = documentID;

			/**
			 * @protected
			 */
			this._documentType = type;

			/**
			 * collection of document versions
			 * !note index 0 won't have version, version numbers start at 1
			 * @private
			 */
			this._documentVersions = [];

			/**
			 * Object containing all data of the document.
			 * 
			 * This object should not be accessed externally!
			 * @private
			 */
			this._data = {};

			/**
			 * A document linked to this document
			 * @private
			 */
			this._linkedDocumentPage = null;
		};

		/**
		 * Get the documentID of the document
		 * @returns {Number} documentID of the document
		 */
		AbstractDocument.prototype.getIID = function () {
			return this.documentID;
		};

		/**
		 * Get a property of the document
		 * @param {String} key - name of the property
		 * @returns {Any} Property value for the given key, or undefined if not found
		 */
		AbstractDocument.prototype.get = function (key) {
			if (this._data[key] !== undefined) {
				return this._data[key];
			} else {
				return undefined;
			}
		};

		/**
		 * Set a linked document Page (link from Graphics to CAD)
		 * @param {AbstractDocumentPage} doc
		 */
		AbstractDocument.prototype.setLinkedDocumentPage = function (doc) {
			this._linkedDocumentPage = doc;
		};

		/**
		 * Get a reference to the linked document
		 * @returns {AbstractDocument} the linked document
		 */
		AbstractDocument.prototype.getLinkedDocument = function () {
			return this._linkedDocumentPage ? this._linkedDocumentPage.getDocument() : undefined;
		};

		/**
		 * returns the linked document page, if no versionNumber is given it selects the latest version. It selects the first
		 * page of the document version. This should be a CAD document which only has one page.
		 * @returns {Data.AbstractDocumentPage}
		 */
		AbstractDocument.prototype.getLinkedDocumentPage = function () {
			return this._linkedDocumentPage;
		};

		/**
		 * has the document a linked document
		 * @returns {Boolean}
		 */
		AbstractDocument.prototype.hasLinkedDocument = function () {
			return this._linkedDocumentPage !== null && this._linkedDocumentPage !== undefined;
		};

		/**
		 * Returns the type of the document
		 * @returns {Data.DataConstants.documentPageType}
		 */
		AbstractDocument.prototype.getDocumentType = function () {
			return this._documentType;
		};

		/**
		 * Change a property of the document
		 * 
		 * @param {String} key - name of the property
		 * @param {Any} newValue - new value
		 * @returns {Boolean} True if success
		 */
		AbstractDocument.prototype.set = function (key, newValue) {
			var oldValue = this._data[key];
			if (oldValue === undefined || (oldValue != undefined && oldValue != newValue)) {
				this._data[key] = newValue;
			}
			return true;
		};

		/**
		 * Adds a documentVersion to the document
		 * @param {Number} versionId - the id of the documentVersion
		 * @param {AbstractDocumentVersion} documentVersion
		 */
		AbstractDocument.prototype.addDocumentVersion = function (documentVersion) {
			this._documentVersions[documentVersion.getVersionId()] = documentVersion;
		};

		/**
		 * Add a document page to a document version
		 * @param {Number} documentVersionId
		 * @param {AbstractDocumentPage} documentPage
		 */
		AbstractDocument.prototype.addDocumentPage = function (documentVersionId, documentPage) {
			this._documentVersions[documentVersionId].addPage(documentPage.getPagenum(), documentPage);
		};

		/**
		 * Get the documentVersion, if versionId is not specified returns the last documentVersion
		 * @param {Number} [versionId]
		 * @returns {Data.AbstractDocumentVersion} undefined if the version is not found 
		 */
		AbstractDocument.prototype.getDocumentVersion = function (versionId) {
			if (versionId === undefined && this._documentVersions.length > 0) {
				return _.last(this._documentVersions);
			} else if (this._documentVersions[versionId]) {
				return this._documentVersions[versionId];
			} else {
				return undefined;
			}
		};

		/**
		 * Does the document have the documentVersion
		 * @param {Number} versionId
		 * @returns {Boolean}
		 */
		AbstractDocument.prototype.hasDocumentVersion = function (versionId) {
			return this._documentVersions[versionId] !== undefined;
		};

		/**
		 * Checks if there are multiple versions
		 * @returns {Boolean}
		 */
		AbstractDocument.prototype.hasMultipleVersions = function () {
			return _.size(this._documentVersions) > 2;
		};

		/**
		 * Get all the versions of the document
		 * @returns
		 */
		AbstractDocument.prototype.getAllDocumentVersions = function () {
			return this._documentVersions.slice(0);
		};

		/**
		 * Loads document pages of previous versions with he same document page number
		 * @param {AbstractDocumentPage} documentPage 
		 * @param {Function} callback
		 */
		AbstractDocument.prototype.loadVersionsForDocumentPage = function (documentPage, callback) {
			//to be implemented
		};

		return AbstractDocument;
	});