define(["web-viewer/layout/AbstractWidget",
        "Template",
        "tpl!web-viewer/documentInfo/DocumentInfoWidget",
        "web-viewer/documentInfo/DocumentInfoIcons",
        "component/toolbar/ToolbarComponent",
        "component/toolbar/ButtonGroup",
        "web-viewer/layout/component/MiniButton",
        "web-viewer/layout/component/ModalPopup",
        "jquery",
        "underscore",
        "web-viewer/compare/CompareConstants",
        "web-viewer/utils/ListenerUtil",
        "web-viewer/utils/ViewerUtils",
        "component/tooltip/Tooltip",
        "web-viewer/layout/LayoutConstants",
        "web-viewer/data/DataConstants"
    ],
    function (AbstractWidget,
        Template,
        tpl,
        DocumentInfoIcons,
        ToolbarComponent,
        ButtonGroup,
        MiniButton,
        ModalPopup,
        $,
        _,
        CompareConstants,
        ListenerUtil,
        ViewerUtils,
        Tooltip,
        LayoutConstants,
        DataConstants) {
        "use strict";

        /**
         * @constant {Number} the speed to render the widget
         */
        var RENDER_SPEED = 40;

        /**
         * @exports DocumentInfoWidget
         * @augments Layout.AbstractWidget
         * @class The DocumentInfoWidget displays the currently loaded document and logged in user
         * @constructor
         * @mixes {Utils.ListenerUtilMixin}
         * @param {Core.ViewerFacade} f
         */
        var DocumentInfoWidget = function DocumentInfoWidget(f) {
            AbstractWidget.call(this, f, LayoutConstants.Widget.DOCUMENTINFO);

            ListenerUtil.mixin(this, f);

            this.templateContext = $("<div>", {
                'class': "infoSection"
            });
            this.buttonsComponent = new ToolbarComponent("buttonsComponent", f);
            this.isListeningToCompare = false;
            this.diffMode = -1;
            this.compareMode = null;

            this._activePageInfoIcons = new DocumentInfoIcons(f);
            this._referencePageInfoIcons = new DocumentInfoIcons(f);

            var returnURL = f.getProperty("returnurl");
            var javaViewerURL = f.getProperty("javaViewerURL");

            var buttonGroup;
            if (returnURL || javaViewerURL) {
                buttonGroup = new ButtonGroup("buttonsGroup");
                buttonGroup.element.addClass("pull-right");
                this.buttonsComponent.addButtonGroup(buttonGroup);
            }

            // If return URL set, add close viewer button
            if (returnURL) {
                var returnButton = new MiniButton(f, "exit", "icon-WCR_Close12", true, function () {
                    f.emit("viewer.request.close", returnURL);
                });
                returnButton.element.addClass("pull-right");
                returnButton.setTooltip("viewer", "exitViewerBtnTT");
                buttonGroup.addButton(returnButton);
            }

            //show a help button
            var helpButton = new MiniButton(f, "help", "icon-WCR_Menu-Help", true, function (e) {
                var helpModule = this.f.getModule("help");
                if (helpModule) {
                    helpModule.openHelpPages();
                }
            });
            helpButton.element.addClass("pull-right");
            helpButton.setTooltip("viewer", "HelpTT");
            helpButton.hide();

            //show the shortcuts help button
            var shortcutsButton = new MiniButton(f, "shortcuts", "icon-WCR_KeyboardShortcuts-keyboard", true, function (e) {
                var helpModule = this.f.getModule("help");
                if (helpModule) {
                    helpModule.openShortcutsPopup();
                }
            });
            shortcutsButton.element.addClass("pull-right");
            shortcutsButton.setTooltip("viewer", "ShortcutsTT");
            shortcutsButton.hide();

            buttonGroup.addButton(helpButton);
            buttonGroup.addButton(shortcutsButton);
            this.f.whenModuleAvailable("help", function () {
                helpButton.show(); // show if help module available
                shortcutsButton.show();
            }, this);

            // If the javaViewerURL is set and the browser supports java, add the Go To Java Viewer button
            if (javaViewerURL && navigator.javaEnabled()) {
                var goToJavaButton = new MiniButton(f, "javaviewer", "icon-WCR_JavaViewer", false, _.bind(function (e) {
                    // Check if java viewer can open this type of document
                    var docType = this.f.getActiveDocument().getDocumentType();
                    if (docType === DataConstants.documentPageType.CAD ||
                        docType === DataConstants.documentPageType.GRAPHICS ||
                        docType === DataConstants.documentPageType.PAGELIST) {
                        if (e.ctrlKey || e.metaKey || e.which === 2) {
                            window.open(javaViewerURL, "_blank");
                        } else {
                            window.location = javaViewerURL;
                        }
                    } else {
                        this._showJavaViewerIncompatiblePopup();
                    }
                }, this));
                goToJavaButton.element.addClass("pull-right");
                goToJavaButton.setTooltip("viewer", "goToJavaViewerBtnTT");
                buttonGroup.addButton(goToJavaButton);
            }

            // Viewer options button
            var resetLayoutButton = new MiniButton(f, "resetLayout", "icon-WCR_ResetLayout", true, _.bind(function () {
                this._resetLayout();
            }, this));
            resetLayoutButton.element.addClass("pull-right");
            resetLayoutButton.setTooltip("viewer", "ResetLayout");
            buttonGroup.addButton(resetLayoutButton);
            this._resetLayoutPopup = null;

            // Show button to switch to mobile layout (if the device has touch)
            this.f.requestModule("featureDetector", _.bind(function (featureDetector) {
                if (featureDetector.isMobile() || featureDetector.hasTouch()) {
                    var switchToMobileButton = new MiniButton(this.f, "switchToMobile", "icon-WCR_Mobile", true, _.bind(function (e) {
                        this.f.switchLayout(LayoutConstants.LayoutType.MOBILE);
                    }, this));
                    switchToMobileButton.setTooltip("viewer", "SwitchToMobile");
                    buttonGroup.prependButton(switchToMobileButton);
                }
            }, this));

            // initialize template
            this.t = new Template(f.i18n, tpl);

            this.render = _.throttle(_.bind(this._render, this), RENDER_SPEED);
            // load language if needed
            this.f.loadBundle("viewer", this.render, this);
            this.f.getDocumentController().whenActiveOrDifferencePageChangedAndParametersAvailableOrChange(this.render, this);

            this.f.whenModuleAvailable("print", function (printModule) {
                var printButton = new MiniButton(f, "print", "icon-WCR_Print", true, _.bind(function () {
                    if (this.compareMode === CompareConstants.mode.BLINKING) {
                        this.f.emit("error.WARNING", "Cannot print when blinking compare mode is active.");
                    } else {
                        printModule.openPrintPopup();
                    }
                }, this));
                printButton.element.addClass("pull-right");
                printButton.setTooltip("viewer", "PrintTT");
                buttonGroup.addButton(printButton);
            }, this);

            this.f.whenModuleAvailable("compare", function (compareModule) {
                this._registerListener("changed.diffMode", function (oldValue, newValue) {
                    this.diffMode = newValue;
                    this.render();
                }, this, compareModule);
                this._registerListener("changed.mode", function (oldMode, newMode) {
                    if (newMode === null) {
                        this.compareMode = null;
                    } else {
                        this.compareMode = newMode.getName();
                    }
                }, this, compareModule);
                this.diffMode = compareModule.getDiffMode();
            }, this);

            // Page controls
            this.t.bind("click", "a[name='firstPage']", this._firstPage, this);
            this.t.bind("click", "a[name='previousPage']", this._previousPage, this);
            this.t.bind("click", "a[name='nextPage']", this._nextPage, this);
            this.t.bind("click", "a[name='lastPage']", this._lastPage, this);
            this.t.bind("mousedown", "input[name='currentPage']", this._currentPageClick, this);
            this.t.bind("keydown", "input[name='currentPage']", this._currentPageEnter, this);

            // add tooltips
            this.whenContextAvailable(function () {
                this.tooltip = new Tooltip(this.f.i18n, this.context, this.f.getContext(), this.f.getContext(), this.f.isMobile(), "");
                this.tooltip.disableTooltip();
                // filter on all elements with attribute tooltip that are not disabled
                this.t.bind("mouseover", "[tooltip]:not('.disabled')", function (source, event) {
                    this.tooltip.setTooltipContent($(source).attr("tooltip"));
                    this.tooltip.context = $(source);
                    this.tooltip.show();
                }, this);
                // filter on all elements with attribute tooltip that are not disabled
                this.t.bind("mouseout", "[tooltip]:not('.disabled')", function (source, event) {
                    this.tooltip.hide();
                }, this);
            }, this);
        };

        DocumentInfoWidget.prototype = Object.create(AbstractWidget.prototype);
        DocumentInfoWidget.prototype.constructor = DocumentInfoWidget;
        DocumentInfoWidget.widgetName = LayoutConstants.Widget.DOCUMENTINFO;

        /**
         * @override
         */
        DocumentInfoWidget.prototype.setContext = function (context) {
            AbstractWidget.prototype.setContext.call(this, context);
            this.context.append($("<div>").append(this.templateContext, this.buttonsComponent.element));
            this.context.addClass("enableTextSelection");
            this.t.setContext(this.templateContext);
            this.context.addClass("documentInfoWidget");
            this.render();
        };

        /**
         * Set the needed data for the widget
         */
        DocumentInfoWidget.prototype._render = function () {
            var docController = this.f.getDocumentController();
            var activePage = docController.getActivePage();
            var diffPage = docController.getDifferencePage();
            var overlay = docController.getCadOverlay();

            // reset template vars
            this.t.d.diffDocument = "";
            this.t.d.diffVersion = "";
            this.t.d.diffClass = "";
            this.t.d.documentClass = "";

            if (activePage) {
                // fill in active page details
                var doc = activePage.getDocument();
                this.t.d.project = doc.get("projectname") || "";
                this.t.d.document = doc.get("docname") || "";
                this.t.d.version = activePage.getDocumentVersion().get("docversion") || "";
                if (activePage.getDocumentVersion().get("docRevision")) {
                    this.t.d.version += "(" + activePage.getDocumentVersion().get("docRevision") + ")";
                }
                this.t.d.showPageControls = activePage.getDocumentVersion().isMultiPage();
                this.t.d.pageNumber = activePage.getPagenum();
                this.t.d.pageCount = activePage.getDocumentVersion().getPageCount();
                this.t.d.firstPageClass = "mini-btn" + (activePage.getPagenum() <= activePage.getDocumentVersion().getFirstPage().getPagenum() ? " disabled" : "");
                this.t.d.previousPageClass = "mini-btn" + (activePage.getPagenum() <= activePage.getDocumentVersion().getFirstPage().getPagenum() ? " disabled" : "");
                this.t.d.nextPageClass = "mini-btn" + (activePage.getPagenum() >= activePage.getDocumentVersion().getLastPage().getPagenum() ? " disabled" : "");
                this.t.d.lastPageClass = "mini-btn" + (activePage.getPagenum() >= activePage.getDocumentVersion().getLastPage().getPagenum() ? " disabled" : "");

                // if there is a diff page, show this information
                if (diffPage !== undefined && this.compareMode !== CompareConstants.mode.READERSPREAD) {
                    doc = diffPage.getDocument();
                    this.t.d.diffDocument = doc.get("docname") || "";
                    this.t.d.diffVersion = diffPage.getDocumentVersion().get("docversion") || "";
                    if (diffPage.getDocumentVersion().get("docRevision")) {
                        this.t.d.diffVersion += "(" + diffPage.getDocumentVersion().get("docRevision") + ")";
                    }
                    if (this.diffMode === CompareConstants.diffMode.DIFFVERSION) {
                        this.t.d.diffClass = "active";
                    } else if (this.diffMode === CompareConstants.diffMode.CURRENTVERSION || this.diffMode === CompareConstants.diffMode.UNDEFINED) {
                        this.t.d.documentClass = "active";
                    }
                }
            }
            // render user
            this.t.d.user = this.f.getCurrentUser().getFullName();

            // render the template
            this.t.render();

            if (activePage.hasCad() || overlay) {
                var activeCad = overlay ? overlay : activePage;
                var activePageInfo = this.templateContext.find(".activeDocumentInfo");
                if (activePageInfo.length > 0) {
                    this._activePageInfoIcons.setDocumentPage(activeCad);
                    activePageInfo.append(this._activePageInfoIcons.getPageIcons());
                }
            }
            if (diffPage && diffPage.hasCad()) {
                var referencePageInfo = this.templateContext.find(".referenceDocumentInfo");
                if (referencePageInfo.length > 0) {
                    this._referencePageInfoIcons.setDocumentPage(diffPage);
                    referencePageInfo.append(this._referencePageInfoIcons.getPageIcons());
                }
            }
        };


        /**
         * @override
         * @returns {String}
         */
        DocumentInfoWidget.prototype.getWidgetType = function () {
            return "topbar";
        };

        /**
         * Change to first page
         */
        DocumentInfoWidget.prototype._firstPage = function () {
            var pageListModule = this.f.getModule("pagelist");
            if (pageListModule) {
                pageListModule.firstPage();
            } else {
                this.f.emit("error.SEVERE", "Pagelist module not loaded.");
            }
        };

        /**
         * Change to previous page
         */
        DocumentInfoWidget.prototype._previousPage = function () {
            var pageListModule = this.f.getModule("pagelist");
            if (pageListModule) {
                pageListModule.previousPage();
            } else {
                this.f.emit("error.SEVERE", "Pagelist module not loaded.");
            }
        };

        /**
         * Change to next page
         */
        DocumentInfoWidget.prototype._nextPage = function () {
            var pageListModule = this.f.getModule("pagelist");
            if (pageListModule) {
                pageListModule.nextPage();
            } else {
                this.f.emit("error.SEVERE", "Pagelist module not loaded.");
            }
        };

        /**
         * Change to last page
         */
        DocumentInfoWidget.prototype._lastPage = function () {
            var pageListModule = this.f.getModule("pagelist");
            if (pageListModule) {
                pageListModule.lastPage();
            } else {
                this.f.emit("error.SEVERE", "Pagelist module not loaded.");
            }
        };


        /**
         * Executed on click of the input page control
         * @param el
         * @param event
         */
        DocumentInfoWidget.prototype._currentPageClick = function (el, event) {
            if (!$(el).is(":focus")) {
                $(el).select();
                event.preventDefault();
            }
        };

        /**
         * Executed on enter of the input page control
         * @param el
         * @param event
         */
        DocumentInfoWidget.prototype._currentPageEnter = function (el, event) {
            if (event.which == 13 || event.keyCode == 13) {
                var value = $(el).val();
                var parsedValue = parseInt(value, 10);
                if (!isNaN(parsedValue)) {
                    // valid number
                    var activeDocumentVersion = this.f.getDocumentController().getActiveDocumentVersion();
                    // check boundaries
                    if (parsedValue < 1) {
                        parsedValue = 1;
                    } else if (parsedValue > activeDocumentVersion.getPageCount()) {
                        parsedValue = activeDocumentVersion.getPageCount();
                    }
                    // change page
                    var newPage = activeDocumentVersion.getPage(parsedValue);
                    if (newPage && newPage !== this.f.getDocumentController().getActivePage()) {
                        this.f.getDocumentController().changeActivePage(newPage);
                    } else {
                        if (!newPage) {
                            this.f.emit("userMessage", "warn", this.f.i18n.t("WarningGetNonexistentPage"), true);
                        }
                        // re-render to reset
                        this.render();
                    }
                } else {
                    // not valid number, re-render to reset
                    this.render();
                }
                event.preventDefault();
            }
        };

        /**
         * This method performs the reset of the layout (sends event to layout manager)
         * It first shows a popup to confirm.
         */
        DocumentInfoWidget.prototype._resetLayout = function () {
            if (!this._resetLayoutPopup) {
                this._resetLayoutPopup = new ModalPopup(this.f, "resetLayoutConfirm", this.f.getContext());
                this._resetLayoutPopup.setOkAction(_.bind(function () {
                    this.f.emit("layout.resetToDefault");
                    this._resetLayoutPopup.hide();
                }, this));
                // Header + Content
                this._resetLayoutPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.i18n.t("ResetLayout")));
                this._resetLayoutPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.i18n.t("ResetLayoutConfirm")));
                this._resetLayoutPopup.setOkBtnText(this.f.i18n.t("YesResponse"));
                this._resetLayoutPopup.setCancelBtnText(this.f.i18n.t("NoResponse"));
            }
            this._resetLayoutPopup.show();
            this._resetLayoutPopup.recalculateSize(450);
        };

        DocumentInfoWidget.prototype._showJavaViewerIncompatiblePopup = function () {
            var modalPopup = new ModalPopup(this.f, "incompatibleJavaViewerDocument", this.f.getContext(), true);
            modalPopup.setRemoveFromDomAfterUsage(true);
            modalPopup.setOkAction(_.bind(function () {
                modalPopup.hide();
            }, this));
            // Header + Content
            modalPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.i18n.t("IncompatibleJavaViewerDocumentTitle")));
            modalPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.i18n.t("IncompatibleJavaViewerDocumentMessage")));
            modalPopup.setOkBtnText(this.f.i18n.t("OkButton"));

            modalPopup.show();
        }

        return DocumentInfoWidget;
    });
