define(["web-viewer/data/AbstractDocumentPage",
		"web-viewer/graphics/Channel",
		"underscore"
	],
	function (AbstractDocumentPage,
		Channel,
		_) {
		"use strict";

		/**
		 * @exports GraphicsDataExtensions
		 * @memberof Graphics
		 * @class Inject graphics functionality in the data model
		 * The AbstractDocumentPage prototype gets extended with methods for fetching graphics
		 * The graphics module only supports the WebCenter data adapter (in the future, this implementation here can be changed to work together with other data adapters too)
		 * @constructor
		 * 
		 * @param {Core.ViewerFacade} f - Viewer facade
		 */
		var GraphicsDataExtensions = function (f) {
			var dataAdapter = f.getDataAdapter();

			// only the WebCenter data-adapter is supported
			if (!dataAdapter.WebCenter) {
				f.emit("error.SEVERE", "Can't initialize Graphics on a non-webcenter data-adapter");
				return;
			}

			/**
			 * Initialize the page with needed data object, so the graphics module can work correctly
			 * @private
			 */
			AbstractDocumentPage.prototype._initGraphicsModule = function () {
				if (this.hasGraphics() && this.isParametersAvailable()) {
					/**
					 * Channels
					 * @private
					 */
					this._channels = [];


					if (!_.isArray(this._parameters.channel)) {
						this._parseChannelData(this._parameters.channel);
					} else {
						for (var i = 0; i < this._parameters.channel.length; i++) {
							this._parseChannelData(this._parameters.channel[i]);
						}
					}
				}
			};

			/**
			 * Parse to a channel from raw channel data and add it to the channels array.
			 * @param {Object} channelData - Raw channel data of one channel
			 */
			AbstractDocumentPage.prototype._parseChannelData = function (channelData) {
				var id = this.getUniqueID() + "_" + this._channels.length;

				var channelName = channelData.name;
				// if an empty channel name is defined, channelName can be undefined.
				if (channelName === undefined) {
					channelName = "";
				}
				// The viewserver uses a special form of encoding for channel names: & --> &amp; and / --> &sl;
				// decode the channelname

				channelName = channelName.replace(/&amp;/g, '&').replace(/&sl;/g, '/');
				var channel = new Channel(id, parseInt(channelData.serverID, 10), channelName, channelData.rgb, channelData.opaque, channelData.file, channelData.dmode, this, channelData.matchedWith);
				this._channels.push(channel);
			};

			/**
			 * Get channels
			 * @param {Boolean} forceUpdate - if true channel data is parsed
			 * @returns {Array.<Channel>} Array of all the channels.
			 */
			AbstractDocumentPage.prototype.getChannels = function (forceUpdate) {
				if (!this._channels || forceUpdate) {
					this._initGraphicsModule();
				}
				return _.clone(this._channels);
			};

			/**
			 * Get the image url for the view server
			 * @param {Object} params - Key,Value pairs as parameters
			 * @returns {String} URL for image data for the given parameters
			 */
			AbstractDocumentPage.prototype.getImageURL = function (params) {
				return dataAdapter.ViewServer("thumbdata.jsp", params);
			};

			/**
			 * Intercep fragment url generation and add the correct url parameters 
			 * @returns {Object} c - context
			 */
			AbstractDocumentPage.prototype.interceptFragmentURL = function (c) {
				// the file
				c.params.file = this._data.file;

				c.params.vdist = this._data.vdist;
				c.params.vsize = this._data.vsize;
				c.params.vscale = this._data.vscale;

				c.params.hscale = this._data.hscale;
				c.params.hsize = this._data.hsize;
				c.params.hdist = this._data.hdist;

				c.params.margtop = this._data.margtop;
				c.params.margright = this._data.margright;
				c.params.margbot = this._data.margbot;
				c.params.margleft = this._data.margleft;

			};

			/**
			 * Get the Channel which has a certain file defined. (used for finding matched channels
			 * @param {String} file - the file the channel should have
			 * @returns {Graphics.Channel} the matched channel or null
			 */
			AbstractDocumentPage.prototype.getChannelForFile = function (file) {
				if (this._channels) {
					for (var i = 0; i < this._channels.length; i++) {
						if (this._channels[i].getFile() == file) {
							return this._channels[i];
						}
					}
				}

				return null;
			};
		};

		return GraphicsDataExtensions;
	});