define('web-viewer/graphics/GraphicsModel',["web-viewer/core/PrivateChannel",
		"web-viewer/core/AbstractModel",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/graphics/GraphicsConstants",
		"utils/OrderedObject",
		"web-viewer/compare/CompareConstants",
		"web-viewer/viewControls/ViewControlsConstants",
		"underscore"
	],
	function (PrivateChannel,
		AbstractModel,
		ShapeUtils,
		ViewerUtils,
		GraphicsConstants,
		OrderedObject,
		CompareConstants,
		ViewControlsConstants,
		_) {
		"use strict";

		/**
		 * Simple datastructure for holding a page and a difference
		 */
		function PageGroup(page, difference) {
			this.page = page;
			this.difference = difference;
		}

		PageGroup.prototype.equals = function (ref) {
			if (this.page && this.difference) {
				return this.page.equals(ref.page) && this.difference.equals(ref.difference);
			}
			return this.page === ref.page && this.difference === ref.difference;
		};


		/**
		 * @exports GraphicsModel
		 * @memberof Graphics
		 * @class The Graphics model is used for representing all graphical information of a page
		 * @constructor
		 * @augments {Core.AbstractModel}
		 *
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {String} privateChannelName
		 * @param {Data.AbstractDocumentPage} page - document page
		 * @param {Data.AbstractDocumentPage} difference - the page that is set as a difference (for comparison)
		 */
		var GraphicsModel = function (f, privateChannelName, page, difference) {
			AbstractModel.call(this, f, privateChannelName);

			this.f = f;
			this.page = null;
			this.difference = null;

			/**
			 * compare mode
			 * @default 0 = no comparison
			 */
			this.dmode = 0;

			// compare module is loaded before graphics, it could be that the diffmode has changed already
			var compareModule = this.f.getModule("compare");
			if (compareModule) {
				this.dmode = compareModule.getDiffMode();
			}
			/**
			 * Keep track of the channels, the channels are copied from the document to enable restore of original values
			 * @private
			 */
			this._channels = new OrderedObject();

			/**
			 *  The default ordering of the channels
			 *  @private
			 */
			this._defaultOrder = null;

			/**
			 * @member {ViewControlsConstants.trimboxMode|null}
			 * @private
			 */
			this._trimboxMode = this.f.getOption("PREFERENCES_DISPLAY_MARGINS") === 1 ? this.f.getOption("PREFERENCES_MARGIN_DISPLAY_MODE") : null;

			/**
			 * True to reverse the channel order in the fragment url, false otherwise
			 * @member {Boolean}
			 */
			this._reverseChannelOrder = false;

			/**
			 * Also fetch image data from server when resolution is higher than pixel level
			 */
			// TODO: in the future when we have more drawing capabilities, we shouldn't fetch the high resolution images, but draw them ourselves 
			this.allowServerDeepZoom = true;

			/**
			 * The current channel view mode
			 * @default Composite View
			 */
			this.channelViewMode = GraphicsConstants.ChannelViewMode.COMPOSITE_VIEW;

			/**
			 * The current single channel mode
			 */
			this.singleChannelMode = this.f.getOption("PREFERENCES_CHANNELS_SINGESEPBLACK");


			// set the page
			this.setPage(page, difference, true);

		};

		GraphicsModel.prototype = Object.create(AbstractModel.prototype);
		GraphicsModel.prototype.constructor = GraphicsModel;

		/**
		 * Get the trimbox of the page
		 * @returns {ShapeUtils.Rect} the page trimbox
		 */
		GraphicsModel.prototype.getTrimBox = function () {
			var trimboxRect = this.page.getTrimboxRect();
			if (this.page.isReaderSpread() && this.page.isBlankPage() && this.page.get("isRightPage") && this.difference) {
				var offset = this.difference.getTrimboxRect().size.width + this.difference.getMargins().left - this.page.getMargins().left;
				trimboxRect.origin.offset(new ShapeUtils.Size(offset, 0));
			} else if (this.page.isReaderSpread() && this.page.isBlankPage() && this.page.get("isRightPage")) {
				trimboxRect.origin.offset(new ShapeUtils.Size(this.page.getPageSize().width, 0));
			}
			return trimboxRect;
		};

		/**
		 * Get the trimbox of the reference document page, returns null if there is no reference page
		 * @returns {ShapeUtils.Rect|null} the reference page trimbox
		 */
		GraphicsModel.prototype.getReferenceTrimBox = function () {
			//only return a trimbox if there is a reference page, if in reader spread only return the trimbox if the correct mode is set
			if (this.difference && !(this.page.isReaderSpread() && this._trimboxMode === ViewControlsConstants.trimboxMode.ONCURRENTPAGE)) {
				var trimboxRect = this.difference.getTrimboxRect();
				if (this.difference.isBlankPage() && !this.page.get("isRightPage") && this.page) {
					var offset = this.page.getTrimboxRect().size.width + this.page.getMargins().left - this.difference.getMargins().left;
					trimboxRect.origin.offset(new ShapeUtils.Size(offset, 0));
				}
				return trimboxRect;
			} else {
				return null;
			}
		};

		/**
		 * Set the page (and difference page) for the graphics
		 * @param {Data.AbstractDocumentPage} page - the current active page
		 * @param {Data.AbstractDocumentPage} difference - the currently configured difference page
		 * @param {Boolean} [silenced] - if true no event will be send (for internal use only)
		 */
		GraphicsModel.prototype.setPage = function (page, difference, silenced) {
			var oldGroup = new PageGroup(this.page, this.difference);
			this.page = page;
			this.difference = difference;
			var newGroup = new PageGroup(this.page, this.difference);
			if (page !== undefined) {
				this.initChannels(oldGroup, newGroup, silenced);
			}

			this._fireChange(silenced, ["page"], oldGroup, newGroup);
		};

		/**
		 * Get the active page that is set on the graphicsmodel
		 * @returns {Data.AbstractDocumentPage}
		 */
		GraphicsModel.prototype.getPage = function () {
			return this.page;
		};

		/**
		 * Get the reference page that is set on the graphicsmodel
		 * @returns {Data.AbstractDocumentPage}
		 */
		GraphicsModel.prototype.getReferencePage = function () {
			return this.difference;
		};

		//------------ CHANNELS ------------------------------------------------------------------------------	

		/**
		 * Initialize the channel data, set all channels as selected
		 * @param {PageGroup} oldPageGroup - the page/difference group before the init
		 * @param {PageGroup} newPageGroup - the current page/difference group
		 * @param {Boolean} silenced - if true, do not send out events
		 */
		GraphicsModel.prototype.initChannels = function (oldPageGroup, newPageGroup, silenced) {
			// old Channels 
			var oldChannels = this.getChannels();

			// here all the keys of the current existing channels are stored
			var existingChannels = [];



			// this is a help function to add the channels
			function addChannel(channel) {
				existingChannels.push(channel.getID());

				var newChannel = channel.copy();
				newChannel.toggle(true);

				var oldDefinition = this._channels.getItemByKey(channel.getID());
				if (oldDefinition) {
					// copy over previous settings
					newChannel.toggle(oldDefinition.isEnabled());
					newChannel.setOpaque(oldDefinition.isChannelOpaque());
					newChannel.setRgb(oldDefinition.getRGB());
					newChannel.setName(oldDefinition.getNameWithoutSuffix());
				}

				this._channels.addOrReplaceItem(newChannel.getID(), newChannel);
			}

			_.each(this.page.getChannels(true), addChannel, this);
			if (this.difference) {
				_.each(this.difference.getChannels(true), addChannel, this);
			}

			this._channels.omitOthers(existingChannels);

			if (this.difference) {
				// synchronize "enabled" flag for matched channels
				for (var i = 0; i < this._channels.getLength(); i++) {
					var channel = this._channels.getItemByIndex(i);
					if (channel.isInCurrentAndMatchedToPrevious()) {
						var matchedChannel = this.getMatchedChannel(channel);
						if (matchedChannel) {
							this._toggleChannel(matchedChannel, channel.isEnabled());
						}
					}
				}
			}


			this._defaultOrder = this._channels.getKeys().slice(0);
			this._fireChange(silenced, ["channels"], oldChannels, this.getChannels());
		};

		/**
		 * Returns a channel
		 * @param {Number} id
		 * @returns {Channel} - Channel
		 */
		GraphicsModel.prototype.getChannel = function (id) {
			return this._channels.getItemByKey(id);
		};

		/**
		 * Checks if there are changed channel definitions or the channel order has changed
		 * @returns {Boolean} True if the channel definition is changed or the order is changed, false if there are no changes
		 */
		GraphicsModel.prototype.hasChanges = function () {
			//test if channels have changed color def
			var hasChangedChannels = _.some(this.getChannels(), function (channel) {
				return channel.isChanged();
			});

			if (hasChangedChannels || !this.hasDefaultOrder()) {
				return true;
			} else {
				return false;
			}
		};

		/**
		 * Checks if the channels have the default order
		 * @returns {Boolean} True if the channels have the default order, false if they have another order
		 */
		GraphicsModel.prototype.hasDefaultOrder = function () {
			var orderedKeys = this._channels.getKeys().slice(0);
			var i = this._defaultOrder.length;
			while (i--) {
				if (orderedKeys[i] !== this._defaultOrder[i]) {
					return false;
				}
			}
			return true;
		};

		/**
		 * Toggle a channel (enable or disable it) (if another is channel is matched with this channel, the linked channel is also toggled
		 * @param {Number} channelID - the unique identifier of the channel
		 * @param {Boolean} toggle - enable/disable
		 * @param {Boolean} [silent] - if true no event will be fired
		 */
		GraphicsModel.prototype.toggleChannel = function (channelID, toggle, silent) {
			var channel = this.getChannel(channelID);
			var matchedChannel = this.getMatchedChannel(channel);

			this._toggleChannel(channel, toggle, silent);
			if (matchedChannel !== null) {
				this._toggleChannel(matchedChannel, toggle, silent);
			}
		};

		/**
		 * Toggle the channel without toggling any matched channels
		 * @private
		 * @param {Channel} channel
		 * @param {Boolean} toggle - enable/disable
		 * @param {Boolean} [silent] - if true no event will be fired
		 */
		GraphicsModel.prototype._toggleChannel = function (channel, toggle, silent) {
			var oldValue = channel.isEnabled();
			channel.toggle(toggle);

			// fire change if needed
			this._fireChange(silent, ["channel", channel.getID(), "enabled"], oldValue, toggle, channel.getID());
		};


		/**
		 * Get the matched channel
		 * @param {Graphics.Channel} channel - the channel for which you want the matched channel
		 * @returns {Graphics.Channel} the matched channel (or null if none is defined)
		 */
		GraphicsModel.prototype.getMatchedChannel = function (channel) {
			if (channel.isMatched()) {
				for (var i = 0; i < this._channels.getLength(); i++) {
					var match = this._channels.getItemByIndex(i);
					if (match !== channel &&
						match.getViewserverID() === channel.getMatchedChannelServerId()) {
						return match;
					}
				}
			}
			return null;
		};

		/**
		 * Check if a certain channel is enabled or not
		 * @param {Number} channelID - the unique identifier of the channel
		 * @returns {Boolean}
		 */
		GraphicsModel.prototype.isChannelEnabled = function (channelID) {
			return this._channels.getItemByKey(channelID).isEnabled();
		};


		/**
		 * Get the channel selection mask (as used by the viewserver)
		 *
		 * @returns {Number} the channel selectionmask
		 */
		GraphicsModel.prototype.getChannelSelectionMask = function () {
			var selectionMask = 0;
			_.each(this.getChannels(), function (channel) {
				if (channel.isEnabled()) {
					selectionMask += Math.pow(2, channel.getViewserverID());
				}
			});

			return selectionMask;
		};

		/**
		 * Get the definition of the changed channels
		 * @returns {String} - definition of the changed channels
		 */
		GraphicsModel.prototype.getColorDef = function () {
			var colorDef = "";
			_.each(this.getChannels(), function (channel) {
				if (channel.displayedRgb !== channel.rgb || channel.isOpaque !== channel.wasOpaque) {
					if (colorDef !== "") {
						colorDef += ",";
					}
					var isOpaque = channel.isChannelOpaque() ? 1 : 0;
					colorDef += channel.getViewserverID() + " [" + channel.displayedName + "] " + ViewerUtils.rgbIntToBgrInt(channel.displayedRgb) + " " + isOpaque;
				}
			});
			return colorDef;
		};

		/**
		 * Get the selection mask of the given channel only
		 * @param {Channel} Channel where you want the selection mask of
		 * @returns {Number} The selection mask of the given channel only
		 */
		GraphicsModel.prototype.getSpecificChannelSelectionMask = function (channel) {
			return Math.pow(2, channel.getViewserverID());
		};

		/**
		 * Get the selection mask for a certain diffMode
		 * @param diffMode
		 * @returns {Number}
		 */
		GraphicsModel.prototype.getChannelSelectionMaskForDiffMode = function (diffMode) {
			var selectionMask = 0;
			_.each(this.getChannels(false, diffMode), function (channel) {
				if (channel.isEnabled()) {
					selectionMask += Math.pow(2, channel.getViewserverID());
				}
			});
			return selectionMask;
		};

		/**
		 * Get the single channel mode
		 * @returns {GraphicsConstants.SingleChannelMode}
		 */
		GraphicsModel.prototype.getSingleChannelMode = function () {
			return this.singleChannelMode;
		};

		/**
		 * Set the single channel mode
		 * @param {GraphicConstants.SingleChannelMode} mode
		 */
		GraphicsModel.prototype.setSingleChannelMode = function (mode) {
			var oldValue = this.getSingleChannelMode();

			if (oldValue != mode) {
				this.singleChannelMode = mode;
				this.f.getCurrentUser().setOption("PREFERENCES_CHANNELS_SINGESEPBLACK", mode);
				this._fireChange(false, ["singleChannelMode"], oldValue, mode);
			}
		};

		/**
		 * Returns the channels that are not returned by getChannels(). (these are the channels of the difference document / active document, dependent on the diffMode)
		 * @returns {Array.<GraphicsModule.Channel>}
		 */
		GraphicsModel.prototype.getDisabledChannels = function () {
			var channels = this._channels.getOrderedObjectList();
			return _.difference(channels, this.getChannels());
		};

		/**
		 * Get all channels that are in the current viewserver setup (so all channels from both current and difference document)
		 * @returns {Array.<GraphicsModule.Channel>} all channels of the loaded document (both active and difference file)
		 */
		GraphicsModel.prototype.getAllChannels = function () {
			return this._channels.getOrderedObjectList();
		};

		/**
		 * Get all channels for the current visible document
		 *
		 * This depends on the diffMode:
		 * 	0: only channels that are in the current document (and the channels that are in the current document and matched with the difference doc)
		 *  1: only channels that are in the difference document (and the channels that are in the difference document and are matched with the current document)
		 * @param {Boolean} filterMatchedChannels - should the matched channels be filter out
		 * @param {CompareConstants.diffMode} [dmode] - this parameter is only for internal usage (it can force the model to give only channels for a certain dmode
		 * @returns channel information
		 */
		GraphicsModel.prototype.getChannels = function (filterMatchedChannels, dmode) {
			var channels = this._channels.getOrderedObjectList();
			if (dmode === undefined) {
				dmode = this.dmode;
			}
			return _.filter(channels, function (channel) {
				if (dmode === CompareConstants.diffMode.SHOWBOTHCHANGES ||
					dmode === CompareConstants.diffMode.SHOWBOTHCHANGESHIGHLIGHT ||
					dmode === CompareConstants.diffMode.SHOWBOTHCHANGESONLY ||
					dmode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDE ||
					dmode === CompareConstants.diffMode.SHOWBOTHONTOP ||
					dmode === CompareConstants.diffMode.READERSPREAD ||
					dmode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDEHIGHLIGHT ||
					dmode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDECHANGESONLY) {
					// difference compare, show all channels
					if (filterMatchedChannels) {
						return !channel.isInPreviousAndMatchedWithCurrent();
					} else {
						return true;
					}
				} else if (dmode === CompareConstants.diffMode.DIFFVERSION) {
					return channel.getDiffMode() === GraphicsConstants.ChannelDiffMode.PREVIOUS || channel.getDiffMode() === GraphicsConstants.ChannelDiffMode.IN_PREVIOUS_AND_MATCHED_WITH_CURRENT;
				} else {
					// default mode (show current version, or unrecognised/unsupported mode)
					return channel.getDiffMode() === null || channel.getDiffMode() === GraphicsConstants.ChannelDiffMode.CURRENT || channel.getDiffMode() === GraphicsConstants.ChannelDiffMode.IN_CURRENT_AND_MATCHED_WITH_PREVIOUS;
				}
			}, this);
		};

		/**
		 * Set the channel view mode
		 * @param {GraphicsConstants.ChannelViewMode} channelViewMode - The new channel view mode
		 */
		GraphicsModel.prototype.setChannelViewMode = function (channelViewMode) {
			if (this.channelViewMode != channelViewMode) {
				var oldValue = this.channelViewMode;
				this.channelViewMode = channelViewMode;
				this._fireChange(false, ["channelViewMode"], oldValue, this.channelViewMode);
			}
		};

		/**
		 * Get the channel view mode
		 * @returns {GraphicsConstants.ChannelViewMode} The current channel view mode
		 */
		GraphicsModel.prototype.getChannelViewMode = function () {
			return this.channelViewMode;
		};

		/**
		 * Returns an Object of all the selected channels, keys are the channel ID's and values are the selected channels
		 * @returns {Object}
		 */
		GraphicsModel.prototype.getSelectedChannels = function () {
			return _.filter(this.getChannels(), function (channel) {
				return channel.isEnabled();
			});
		};

		/**
		 * Returns the first channel that is enabled
		 * @returns {Channel}
		 */
		GraphicsModel.prototype.getFirstSelectedChannel = function () {
			return _.find(this.getChannels(), function (channel) {
				return channel.isEnabled();
			});
		};

		/**
		 * Checks if the list of selected channels has holes in it
		 * @returns {Boolean}
		 */
		GraphicsModel.prototype.hasEnabledChannelGaps = function () {
			var gapStarted = false;
			var previousIsEnabled = false;
			var channels = this.getChannels();
			for (var i = 0; i < channels.length; i++) {
				var channel = channels[i];
				if (i > 0) {
					if (!gapStarted && previousIsEnabled && !channel.isEnabled()) {
						gapStarted = true;
					} else if (gapStarted && channel.isEnabled()) {
						// GAP DETECTED
						return true;
					}
				}
				previousIsEnabled = channel.isEnabled();
			}
			return false;
		};

		/**
		 * How many channels are enabled?
		 * @param {Boolean} filterMatchedChannels - should the matched channels be filtered out
		 * @returns {Number}
		 */
		GraphicsModel.prototype.getEnabledChannelCount = function (filterMatchedChannels) {
			return _.reduce(this.getChannels(filterMatchedChannels), function (memo, channel) {
				if (channel.isEnabled()) {
					memo++;
				}
				return memo;
			}, 0);
		};

		/**
		 * Get the amount of channels
		 * @returns {Number}
		 */
		GraphicsModel.prototype.getChannelCount = function () {
			return this._channels.getLength();
		};

		/**
		 * Move the channel with key in the list to a specified index
		 * @param {Number} key - Id of the channel
		 * @param {Number} index - place to move the channel to
		 */
		GraphicsModel.prototype.moveChannelTo = function (channelID, index) {
			// copy the old order
			var oldOrder = this._channels.getKeys().slice(0);
			this._channels.moveItemByKeyToIndex(channelID, index);
			// emit event if needed
			this._fireChange(false, ["channelOrder"], oldOrder, this._channels.getKeys());
		};

		/**
		 * Reset the order of channels to the default order
		 */
		GraphicsModel.prototype.resetDefaultChannelOrder = function () {
			var oldOrder = this._channels.getKeys().slice(0);
			this._channels.setOrder(this._defaultOrder);
			this._fireChange(false, ["channelOrder"], oldOrder, this._channels.getKeys());
		};

		/**
		 * Reset all channels to default color definition
		 */
		GraphicsModel.prototype.resetAllChannelsColorDef = function () {
			_.each(this.getChannels(), _.bind(function (channel) {
				this.resetChannel(channel.getID());
			}, this));
		};

		/**
		 * Checks if any of the visible channels is opaque
		 * @returns {Boolean}
		 */
		GraphicsModel.prototype.hasVisibleOpaqueChannels = function () {
			return _.some(this.getChannels(), function (channel) {
				return (channel.isEnabled() && channel.isChannelOpaque());
			});
		};

		/**
		 * Change the channel order
		 * @param {Array<Number>} orderArray - list of the ChannelId's in the correct order
		 */
		GraphicsModel.prototype.setChannelOrder = function (orderArray) {
			var oldOrder = this._channels.getKeys().slice(0);
			var newOrder = oldOrder.slice(0);

			// when the order is changed, this is done for a limited subset of channels, we need to fit this in the whole set of channels
			var currentChannels = this.getChannels();
			if (currentChannels.length === orderArray.length) {
				_.each(this.getChannels(), function (channel, index) {
					if (_.indexOf(orderArray, channel.getID() >= 0)) {
						// we first find the position of the channel at index [index] in the oldOrder, on this place we put the channel from orderArray[index]
						var newIndex = _.indexOf(oldOrder, channel.getID());
						newOrder[newIndex] = orderArray[index];
					} else {
						// a channel was provide which was not in the previous order...
						newOrder = null;
					}

				});
			}

			if (newOrder !== null) {
				// the new order is ok, change the order and fire event
				this._channels.setOrder(newOrder);
				this._fireChange(false, ["channelOrder"], oldOrder, this._channels.getKeys());
			}
		};

		/**
		 * Reverse the channel order or not (only internally for url fragment, not in the view widget).
		 * @param reverseChannelOrder - True to reverse the channel order, false otherwise.
		 */
		GraphicsModel.prototype.setReverseChannelOrder = function (reverseChannelOrder) {
			this._reverseChannelOrder = reverseChannelOrder;
		};

		/**
		 * Get the order of the channels
		 * @returns {Array<Number>} - Array of the channelIds in the correct order
		 */
		GraphicsModel.prototype.getChannelKeysOrder = function () {
			return this._channels.getKeys();
		};

		/**
		 * Get a channel at a certain index
		 * @param {Number} index
		 * @returns {Channel}
		 */
		GraphicsModel.prototype.getChannelAtIndex = function (index) {
			return this._channels.getItemByIndex(index);
		};

		/**
		 * resets the definition of a channel to the default
		 * @param {String} channelId
		 */
		GraphicsModel.prototype.resetChannel = function (channelId) {
			var channel = this.getChannel(channelId);
			var oldChannel = channel.copy();
			channel.resetToDefault();
			this._fireChange(false, ["channel", channel.getID(), "channelRGB"], oldChannel, channel, channel.isEnabled(), channel.getID());
		};

		/**
		 * Returns the channel after the channel with channelID
		 * @param {String} channelID
		 * @returns {Channel}
		 */
		GraphicsModel.prototype.getNextChannel = function (channelID) {
			return this._channels.getNext(channelID);
		};

		/**
		 * Returns true if there is a next channel
		 * @param {String} channelID
		 * @returns {Boolean}
		 */
		GraphicsModel.prototype.hasNextChannel = function (channelID) {
			return this._channels.hasNext(channelID);
		};

		/**
		 * Returns the channel before the channel with channelID
		 * @param {String} channelID
		 * @returns {Channel}
		 */
		GraphicsModel.prototype.getPreviousChannel = function (channelID) {
			return this._channels.getPrevious(channelID);
		};

		/**
		 * Returns true if there is a previous channel
		 * @param {String} channelID
		 * @returns {Boolean}
		 */
		GraphicsModel.prototype.hasPreviousChannel = function (channelID) {
			return this._channels.hasPrevious(channelID);
		};

		/**
		 * Changes the definition of channel (first argument) with the definition from the channel with channelId (second argument)
		 * @param {String} channelIdToChange - the channel whose definition needs to be changed
		 * @param {String} channelIdToCopyFrom - the channelId of the channel which definition is copied to channel
		 * @param {Boolean} changeWithOriginalDef - use original definition or changed definition of the channel to copy
		 */
		GraphicsModel.prototype.changeChannelDef = function (channelIdToChange, channelIdToCopyFrom, changeWithOriginalDef) {
			var channel = this.getChannel(channelIdToChange);
			var channelToCopyFrom = this.getChannel(channelIdToCopyFrom);
			var oldChannel = channel.copy();
			if (changeWithOriginalDef) {
				channel.setName(channelToCopyFrom.name);
				channel.setRgb(channelToCopyFrom.rgb);
			} else {
				channel.setName(channelToCopyFrom.getNameWithoutSuffix());
				channel.setRgb(channelToCopyFrom.displayedRgb);
			}
			this._fireChange(false, ["channel", channelIdToChange, "channelRGB"], oldChannel, channel, channel.isEnabled(), channel.getID());
		};

		/**
		 * Set the matching between two channels.
		 * @param {Number} currentChannelId
		 * @param {Number} referenceChannelId
		 */
		GraphicsModel.prototype.setChannelMatching = function (currentChannelId, referenceChannelId) {
			var currentChannel = this.getChannel(currentChannelId);
			var referenceChannel = this.getChannel(referenceChannelId);
			referenceChannel.setMatchedChannelServerId(currentChannel.getViewserverID());
			currentChannel.setMatchedChannelServerId(referenceChannel.getViewserverID());
		};

		/**
		 * reset the matching between all channels.
		 */
		GraphicsModel.prototype.resetChannelMatching = function () {
			_.each(this.getChannels(), function (channel, index) {
				channel.setMatchedChannelServerId();
			});
		};

		/**
		 * Changes the rgb color of a channel
		 * @param {String} channelId - channel to change the rgb
		 * @param {Number} rgbColor
		 */
		GraphicsModel.prototype.changeChannelRgb = function (channelId, rgbColor) {
			var channel = this.getChannel(channelId);
			var oldChannel = channel.copy();
			channel.setRgb(rgbColor);
			this._fireChange(false, ["channel", channelId, "channelRGB"], oldChannel, channel, channel.isEnabled, channel.getID());
		};

		/**
		 * Changes the opaque of a channel
		 * @param {String} channelId
		 * @param {Boolean} isOpaqueChecked
		 */
		GraphicsModel.prototype.setChannelOpaque = function (channelId, isOpaqueChecked) {
			var channel = this.getChannel(channelId);
			var wasOpaqueChecked = channel.isChannelOpaque();
			channel.setOpaque(isOpaqueChecked);
			this._fireChange(false, ["channel", channelId, "channelRGB"], wasOpaqueChecked, isOpaqueChecked, channel.isEnabled(), channel.getID());
		};


		/**
		 * Parsed and formats the given channels info. Used in ChannelsWidget and MeasurePopoverModel.
		 * result.channels contains all the channels with as value the formatted info of the given object, result.total contains the total
		 * @param {Object} channelsInfo - channels information coming from the measure tools.
		 * 					- channelsinfo.makeTotalCount can be set and will result in making the total and setting it in the result, otherwise result.total is the empty string.
		 * 					- channelsinfo.unit can optionally be set and will be added as unit (default '')
		 * 					- channelsinfo.unitFix can optionally be set and will be be used for rounding the values (default 2).
		 * @return {Object} result.channels contains all the channels in order of this model with result.channels[i].value the formatted value and result.total contains the total if needed (see above)
		 */
		GraphicsModel.prototype.getParsedChannelsInfo = function (channelsInfo) {
			var resultInfo = {};
			resultInfo.channels = [];

			var unit = channelsInfo.unit || "";
			var unitFix = (channelsInfo.unitFix === undefined ? 2 : channelsInfo.unitFix);

			var total = 0;
			var diffTotal;

			_.each(this.getChannels(), function (c) {
				var channel = c.copy();

				var value = channelsInfo[channel.getViewserverID()];
				var parsedValue = null;
				if (value === undefined || value.length === 0) {
					// no readout available for this channel, default to ---
					value = "---";
				} else {
					// a readout is available, try to parse the number
					parsedValue = ViewerUtils.parseFirstFloat(value);
					if (isNaN(parsedValue)) {
						parsedValue = null;
					}
				}

				var extra = "";


				if (this.isInDifferenceCompareMode()) {
					if (channel.isInCurrentAndMatchedToPrevious()) {
						// we are in difference compare and the channel is in the current and matched with a channel from the difference file
						// find the matched channel
						var matchedChannel = this.getMatchedChannel(channel);
						if (matchedChannel) {
							// get the matched value, and try to parse it
							var matchedValue = channelsInfo[matchedChannel.getViewserverID()];
							if (matchedValue !== undefined && matchedValue.length > 0) {
								var parsedMatchedValue = ViewerUtils.parseFirstFloat(matchedValue);
								if (!isNaN(parsedMatchedValue) && channelsInfo.makeTotalCount) {
									// value was successfully parsed, and we are dealing with numbers
									extra = " (" + parsedMatchedValue.toFixed(unitFix) + ")";
								} else {
									// a value is found, no rounding of decimals is needed
									extra = " (" + matchedValue + ")";
								}
							}
						}
					}
				}

				if (parsedValue !== null && channelsInfo.makeTotalCount) {
					// we need to calculate a sum of all values

					if (!this.isInDifferenceCompareMode() || channel.isInCurrentAndMatchedToPrevious() || channel.isAdded()) {
						// channel is from current document
						total += parsedValue;
					}
					if (this.isInDifferenceCompareMode() && (channel.isInPreviousAndMatchedWithCurrent() || channel.isSubtracted())) {
						// channel is from difference document
						if (diffTotal === undefined) {
							// first time
							diffTotal = 0;
						}
						diffTotal += parsedValue;
					}

					// we are dealing with numbers, so round to desired fix
					value = parsedValue.toFixed(unitFix);
					// add unit
					extra += " " + unit;
				}

				channel.value = value + extra;
				resultInfo.channels[channel.getViewserverID()] = channel;
			}, this);

			// Add total to result
			if (channelsInfo.makeTotalCount) {
				if (diffTotal !== undefined) {
					resultInfo.total = total.toFixed(unitFix) + " (" + diffTotal.toFixed(unitFix) + ") " + unit;
				} else {
					resultInfo.total = total.toFixed(unitFix) + " " + unit;
				}
			} else {
				resultInfo.total = "";
			}

			return resultInfo;
		};

		//------------ IMAGE FETCHING ------------------------------------------------------------------------------

		/**
		 * Add correct parameters for fragment URL
		 * @param {Object} c - context for interceptor
		 * @returns {Object}
		 */
		GraphicsModel.prototype.interceptFragmentURL = function (c) {
			// channels
			c.params.color = this.getChannelSelectionMask();
			c.params.singlesepblack = this.getSingleChannelMode();

			c.params.coldef = this.getColorDef();
			if (this.channelOrderChanged || this._reverseChannelOrder) {

				var channels = this._channels.getOrderedObjectList();
				if (this._reverseChannelOrder) {
					channels = channels.reverse();
				}
				var channelList = []; //array with key=channelId and value=location in the ordered list
				_.each(channels, function (channel, key) { //run through the ordered channel list, 
					channelList[channel.getViewserverID()] = key; //key is the location of the channel in the ordered list
				});
				c.params.seq = channelList.toString(); //array to comma seperated string
			}
		};


		/*
		 * TODO: Figure out which resolution to use.
		 *
		 * For now, the implementation will not go over the maximum resolution of the document (to avoid too many server calls)
		 *
		 * Today, the java viewer does some magic tricks to simulate a higher resolution. The HTML Viewer could also use this, but it's at a cost (extra server calls)
		 * The HTML Viewer could offer an option to allow extra server calls, or should make its own implementation of "nearest neighbor" calculation
		 */
		/**
		 * Returns the maximum resolution
		 * @returns {Number}
		 */
		GraphicsModel.prototype.getMaximumResolution = function () {
			if (this.allowServerDeepZoom) {
				return -1;
			} else {
				return ViewerUtils.parseFloat(this.page.getParam("resolutionx"));
			}
		};

		//------------ OTHER ------------------------------------------------------------------------------ 

		/**
		 * Get the current compare mode
		 * @returns {Number}
		 */
		GraphicsModel.prototype.getCompareMode = function () {
			return this.dmode;
		};

		/**
		 * Is the graphicsmodule in difference compare mode?
		 * @returns {Boolean}
		 */
		GraphicsModel.prototype.isInDifferenceCompareMode = function () {
			return (this.dmode === CompareConstants.diffMode.SHOWBOTHCHANGES ||
				this.dmode === CompareConstants.diffMode.SHOWBOTHCHANGESHIGHLIGHT ||
				this.dmode === CompareConstants.diffMode.SHOWBOTHCHANGESONLY ||
				this.dmode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDE ||
				this.dmode === CompareConstants.diffMode.SHOWBOTHONTOP ||
				this.dmode === CompareConstants.diffMode.READERSPREAD ||
				this.dmode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDEHIGHLIGHT ||
				this.dmode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDECHANGESONLY);
		};
		/**
		 * Set the current compare mode
		 * @param {CompareModule.CompareConstants.diffMode} newMode
		 */
		GraphicsModel.prototype.setCompareMode = function (newMode) {
			var oldMode = this.dmode;
			var oldChannels = this.getChannels();
			this.dmode = newMode;

			this._fireChange(false, ["compareMode"], oldMode, newMode);
			this._fireChange(false, ["channels"], oldChannels, this.getChannels());
		};

		/**
		 * Set if the trimbox needs to be shown or not and which trimbox mode needs to be used
		 * @param {Boolean} showTrimbox
		 * @param {ViewControlsConstants.trimboxMode} trimboxMode
		 */
		GraphicsModel.prototype.setShowTrimbox = function (showTrimbox, trimboxMode) {
			var oldMode = this._trimboxMode; //oldmode
			this._trimboxMode = showTrimbox ? trimboxMode : null; //set the mode to null if no trimbox needs to be shown
			this._fireChange(false, ["trimbox"], oldMode, this._trimboxMode);
		};

		/**
		 * Is the trimbox shown
		 * @returns {Boolean}
		 */
		GraphicsModel.prototype.isShowTrimbox = function () {
			return this._trimboxMode !== null;
		};

		return GraphicsModel;
	});

define('web-viewer/graphics/Channel',["web-viewer/utils/ViewerUtils",
		"web-viewer/graphics/GraphicsConstants",
		"underscore"
	],
	function (ViewerUtils,
		GraphicsConstants,
		_) {

		/**
		 * @exports Channel
		 * @memberof Graphics
		 * @class Representation of a channel
		 * The channel is enabled by default.
		 * @constructor
		 * 
		 * @param {String} id - Id of the channel. This is the channel number.
		 * @param {Number} viewserverID - the number on the viewserver
		 * @param {String} name - Name of the channel
		 * @param {Number} rgb - Integer representation of the rgb color
		 * @param {Boolean} isOpaque - is the channel opaque
		 * @param {String} file - File of the channel
		 * @param {GraphicsConstants.ChannelDiffMode} dmode 
		 * @param {Data.AbstractDocumentPage} page - the page to which the channel belongs
		 * @param {Number} [matchedTo] - the channel-file too which this channel is matched
		 */
		var Channel = function (id, viewserverID, name, rgb, isOpaque, file, dmode, page, matchedWith) {
			this.id = id;
			this.viewserverID = viewserverID;
			this.name = name;
			this.displayedName = name;
			this.rgb = rgb;
			this.displayedRgb = rgb;
			this.wasOpaque = isOpaque;
			this.isOpaque = isOpaque;
			this.file = file;
			// set dmode (if undefined, make it null)
			this.dmode = (dmode === undefined) ? null : dmode;
			this.page = page;
			// this is the file-identifier for the channel to which this channel is matched
			this.matchedWith = matchedWith;
			if (this.matchedWith === undefined) {
				this.matchedWith = null;
			}
			// There is also valid but it should always be 3: this means Valid (bit 0) and RGB (bit 1)
			// we can ignore it because the view server always assures that rgb contains rgb values
			this.enabled = true;
		};

		/**
		 * Copy this channel
		 * @returns {Channel} New object with the same id, name, rgb, file, etc.
		 */
		Channel.prototype.copy = function () {
			var copiedChannel = new Channel(this.id, this.viewserverID, this.name, this.rgb, this.wasOpaque, this.file, this.dmode, this.page, this.matchedWith);
			copiedChannel.displayedRgb = this.displayedRgb;
			copiedChannel.isOpaque = this.isOpaque;
			copiedChannel.displayedName = this.displayedName;
			return copiedChannel;
		};

		/**
		 * Toggle whether this channel is enabled or not
		 * @param {Boolean} [toggle] - If defined, the channel will be enabled/disabled according to this parameter, 
		 * otherwise the value of 'enabled' is switched 
		 */
		Channel.prototype.toggle = function (toggle) {
			if (toggle === undefined) {
				this.enabled = !this.enabled;
			} else {
				this.enabled = toggle;
			}
		};

		/**
		 * Is this channel enabled?
		 * @returns {Boolean} True if the channel is enabled, false otherwise.
		 */
		Channel.prototype.isEnabled = function () {
			return this.enabled;
		};

		/**
		 * Get the id of the channel as used by the viewserver
		 * @returns {Number}
		 */
		Channel.prototype.getViewserverID = function () {
			return this.viewserverID;
		};

		/**
		 * Get id of the channel
		 * @returns {String} Id of the channel
		 */
		Channel.prototype.getID = function () {
			return this.id;
		};

		/**
		 * Get the name of the channel, if the name is changed a "*" character will be added
		 * @returns {String} Name of the channel
		 */
		Channel.prototype.getName = function () {
			if (this.isChanged()) {
				return this.displayedName + " *";
			} else {
				return this.displayedName;
			}
		};

		/**
		 * Get the original channel name
		 * @returns {String}
		 */
		Channel.prototype.getOriginalName = function () {
			return this.name;
		};

		/**
		 * Returns the displayed name of the channel without the optional *-char
		 * @returns {String}
		 */
		Channel.prototype.getNameWithoutSuffix = function () {
			return this.displayedName;
		};

		/**
		 * Get the RGB color of the channel in int format
		 * @returns {Number} RGB value of the channel in int format
		 */
		Channel.prototype.getRGB = function () {
			return this.displayedRgb;
		};

		/**
		 * Get the RGB color of the channel in hex format
		 * @returns {String} RGB value of the channel in hex format
		 */
		Channel.prototype.getHexRGB = function () {
			return ViewerUtils.hexFromInt(this.displayedRgb);
		};

		/**
		 * Get the file of the channel
		 * @returns {String} File of the channel
		 */
		Channel.prototype.getFile = function () {
			return this.file;
		};

		/**
		 * Get the diff mode for the channel, if no dmode is defined, it returns undefined
		 * @returns {GraphicsConstants.ChannelDiffMode} diffMode or undefined
		 */
		Channel.prototype.getDiffMode = function () {
			return this.dmode;
		};

		/**
		 * Resets the displayed name and rgb to the previous values
		 */
		Channel.prototype.resetToDefault = function () {
			this.displayedName = this.name;
			this.displayedRgb = this.rgb;
			this.isOpaque = this.wasOpaque;
		};

		/**
		 * is the channel opaque
		 * @returns {Boolean}
		 */
		Channel.prototype.isChannelOpaque = function () {
			return this.isOpaque;
		};

		/**
		 * was the channel opaque by default
		 * @returns {Boolean}
		 */
		Channel.prototype.wasChannelOpaque = function () {
			return this.wasOpaque;
		};

		/**
		 * Returns true is the channel is changed
		 * @returns {Boolean}
		 */
		Channel.prototype.isChanged = function () {
			return this.rgb != this.displayedRgb || this.isOpaque != this.wasOpaque || this.displayedName != this.name;
		};

		/**
		 * Change the Rgb of the channel
		 * @param {Number} Rgb
		 */
		Channel.prototype.setRgb = function (Rgb) {
			if (Rgb !== this.displayedRgb) {
				this.displayedRgb = Rgb;
			}
		};

		/**
		 * Change the opaque of the channel
		 * @param {Boolean} opaque
		 */
		Channel.prototype.setOpaque = function (opaque) {
			if (opaque !== this.isOpaque) {
				this.isOpaque = opaque;
			}
		};

		/**
		 * sets the displayed name of the channel
		 * @param {String} name
		 */
		Channel.prototype.setName = function (name) {
			this.displayedName = name;
		};

		/**
		 * Checks if both channels can be considered equal, does not take default values into account
		 * @param {Channel} channel - Channel to compare with
		 * @returns {Boolean}
		 */
		Channel.prototype.equals = function (channel) {
			if (channel.getRGB() === this.displayedRgb && channel.isChannelOpaque() === this.isOpaque) {
				return true;
			} else {
				return false;
			}
		};

		/**
		 * Checks if both channel can be considered equal, only takes the default values into account
		 * @param {Channel} channel
		 * @returns {Boolean}
		 */
		Channel.prototype.equalsOriginalDef = function (channel) {
			if (channel.rgb === this.rgb && channel.wasChannelOpaque() === this.isOpaque) {
				return true;
			} else {
				return false;
			}
		};

		/**
		 * Is the channel matched with another channel of another file
		 * @returns {Boolean}
		 */
		Channel.prototype.isMatched = function () {
			return this.matchedWith !== null;
		};

		/**
		 * Get the channel-file of the channel which is matched with this channel 
		 * @returns {Number} - viewerserverId
		 */
		Channel.prototype.getMatchedChannelServerId = function () {
			return this.matchedWith;
		};

		/**
		 * Set the channel-file of the channel which is matched with this channel
		 * @param {Channel} matchedChannel - if undefined the channel will no longer be matched 
		 */
		Channel.prototype.setMatchedChannelServerId = function (matchedChannelServerId) {
			if (matchedChannelServerId === undefined) {
				this.matchedWith = null;
			} else {
				this.matchedWith = matchedChannelServerId;
			}
		};

		/**
		 * Get the page to which this page belongs 
		 * @returns {Data.AbstractDocumentPage}
		 */
		Channel.prototype.getPage = function () {
			return this.page;
		};

		/**
		 * Is the channel in the current file and matched with the difference file
		 * @returns {Boolean}
		 */
		Channel.prototype.isInCurrentAndMatchedToPrevious = function () {
			return this.dmode === GraphicsConstants.ChannelDiffMode.IN_CURRENT_AND_MATCHED_WITH_PREVIOUS || (this.isMatched() && this.dmode === GraphicsConstants.ChannelDiffMode.CURRENT);
		};

		/**
		 * Is the channel in the difference file and matched with the current?
		 * @returns {Boolean}
		 */
		Channel.prototype.isInPreviousAndMatchedWithCurrent = function () {
			return this.dmode === GraphicsConstants.ChannelDiffMode.IN_PREVIOUS_AND_MATCHED_WITH_CURRENT || (this.isMatched() && this.dmode === GraphicsConstants.ChannelDiffMode.PREVIOUS);
		};

		/**
		 * Is this channel subtracted in comparison to the document too which the viewer is comparing?
		 * @returns {Boolean}
		 */
		Channel.prototype.isSubtracted = function () {
			return this.dmode === GraphicsConstants.ChannelDiffMode.PREVIOUS && !this.isMatched();
		};

		/**
		 * Is this channel added in comparison to the document too which the viewer is comparing?
		 * @returns {Boolean}
		 */
		Channel.prototype.isAdded = function () {
			return this.dmode === GraphicsConstants.ChannelDiffMode.CURRENT && !this.isMatched();
		};

		return Channel;
	});

define('web-viewer/graphics/GraphicsDataExtensions',["web-viewer/data/AbstractDocumentPage",
		"web-viewer/graphics/Channel",
		"underscore"
	],
	function (AbstractDocumentPage,
		Channel,
		_) {
		"use strict";

		/**
		 * @exports GraphicsDataExtensions
		 * @memberof Graphics
		 * @class Inject graphics functionality in the data model
		 * The AbstractDocumentPage prototype gets extended with methods for fetching graphics
		 * The graphics module only supports the WebCenter data adapter (in the future, this implementation here can be changed to work together with other data adapters too)
		 * @constructor
		 * 
		 * @param {Core.ViewerFacade} f - Viewer facade
		 */
		var GraphicsDataExtensions = function (f) {
			var dataAdapter = f.getDataAdapter();

			// only the WebCenter data-adapter is supported
			if (!dataAdapter.WebCenter) {
				f.emit("error.SEVERE", "Can't initialize Graphics on a non-webcenter data-adapter");
				return;
			}

			/**
			 * Initialize the page with needed data object, so the graphics module can work correctly
			 * @private
			 */
			AbstractDocumentPage.prototype._initGraphicsModule = function () {
				if (this.hasGraphics() && this.isParametersAvailable()) {
					/**
					 * Channels
					 * @private
					 */
					this._channels = [];


					if (!_.isArray(this._parameters.channel)) {
						this._parseChannelData(this._parameters.channel);
					} else {
						for (var i = 0; i < this._parameters.channel.length; i++) {
							this._parseChannelData(this._parameters.channel[i]);
						}
					}
				}
			};

			/**
			 * Parse to a channel from raw channel data and add it to the channels array.
			 * @param {Object} channelData - Raw channel data of one channel
			 */
			AbstractDocumentPage.prototype._parseChannelData = function (channelData) {
				var id = this.getUniqueID() + "_" + this._channels.length;

				var channelName = channelData.name;
				// if an empty channel name is defined, channelName can be undefined.
				if (channelName === undefined) {
					channelName = "";
				}
				// The viewserver uses a special form of encoding for channel names: & --> &amp; and / --> &sl;
				// decode the channelname

				channelName = channelName.replace(/&amp;/g, '&').replace(/&sl;/g, '/');
				var channel = new Channel(id, parseInt(channelData.serverID, 10), channelName, channelData.rgb, channelData.opaque, channelData.file, channelData.dmode, this, channelData.matchedWith);
				this._channels.push(channel);
			};

			/**
			 * Get channels
			 * @param {Boolean} forceUpdate - if true channel data is parsed
			 * @returns {Array.<Channel>} Array of all the channels.
			 */
			AbstractDocumentPage.prototype.getChannels = function (forceUpdate) {
				if (!this._channels || forceUpdate) {
					this._initGraphicsModule();
				}
				return _.clone(this._channels);
			};

			/**
			 * Get the image url for the view server
			 * @param {Object} params - Key,Value pairs as parameters
			 * @returns {String} URL for image data for the given parameters
			 */
			AbstractDocumentPage.prototype.getImageURL = function (params) {
				return dataAdapter.ViewServer("thumbdata.jsp", params);
			};

			/**
			 * Intercep fragment url generation and add the correct url parameters 
			 * @returns {Object} c - context
			 */
			AbstractDocumentPage.prototype.interceptFragmentURL = function (c) {
				// the file
				c.params.file = this._data.file;

				c.params.vdist = this._data.vdist;
				c.params.vsize = this._data.vsize;
				c.params.vscale = this._data.vscale;

				c.params.hscale = this._data.hscale;
				c.params.hsize = this._data.hsize;
				c.params.hdist = this._data.hdist;

				c.params.margtop = this._data.margtop;
				c.params.margright = this._data.margright;
				c.params.margbot = this._data.margbot;
				c.params.margleft = this._data.margleft;

			};

			/**
			 * Get the Channel which has a certain file defined. (used for finding matched channels
			 * @param {String} file - the file the channel should have
			 * @returns {Graphics.Channel} the matched channel or null
			 */
			AbstractDocumentPage.prototype.getChannelForFile = function (file) {
				if (this._channels) {
					for (var i = 0; i < this._channels.length; i++) {
						if (this._channels[i].getFile() == file) {
							return this._channels[i];
						}
					}
				}

				return null;
			};
		};

		return GraphicsDataExtensions;
	});


define('tpl!web-viewer/graphics/ChannelsWidget', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<table class="channelList sortableChannelsList">\r\n	<tr class="all">\r\n		<td><i class="eyeBox '+
((__t=((d.isAllEnabled()?'selected icon icon-WCR_Eye':'')))==null?'':_.escape(__t))+
'" data-bind="all"></i></td> \r\n		<td><i class="colorBox" style="background: #fff;" data-bind="all"></i></td> \r\n		<td>'+
((__t=(d.i18n("ChanAll")))==null?'':_.escape(__t))+
'</td>	\r\n		<td> '+
((__t=(d.totalCount))==null?'':_.escape(__t))+
'</td>\r\n	</tr>\r\n';
 _.each(d.getChannels(), function(channel){ 
__p+='\r\n	<tr class="dragAndDropChannel" data-bind="'+
((__t=(channel.getID()))==null?'':_.escape(__t))+
'">\r\n		<td><i class="eyeBox '+
((__t=((channel.isEnabled()?'selected icon icon-WCR_Eye':'')))==null?'':_.escape(__t))+
'" data-bind="'+
((__t=(channel.getID()))==null?'':_.escape(__t))+
'"></i></td>  \r\n		<td><i class="colorBox" data-bind="'+
((__t=(channel.getID()))==null?'':_.escape(__t))+
'" style="background: #'+
((__t=(channel.getHexRGB() ))==null?'':_.escape(__t))+
';"></i></td> \r\n		<td  data-bind="'+
((__t=(channel.getID()))==null?'':_.escape(__t))+
'" class="channelName">'+
((__t=(channel.getName() ))==null?'':_.escape(__t))+
' '+
((__t=(d.getChannelSuffix(channel)))==null?'':_.escape(__t))+
'</td>\r\n		<td class="pull-right"> '+
((__t=(d.getChannelsInfo(channel)))==null?'':_.escape(__t))+
'</td>\r\n	</tr>\r\n';
 }); 
__p+='\r\n</table>';
return __p;
}});

define('tpl!web-viewer/graphics/ChannelInformationPopup', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<table class="channelInformation">\r\n	<tr><th colspan="4">'+
((__t=(d.i18n("ChanOrigTitle")))==null?'':_.escape(__t))+
'</th></tr>\r\n	<tr class="colorBlockCell">\r\n		<td></td><td></td><td></td>\r\n		<td width="25%" rowspan="3"><span class="colorBlockText">'+
((__t=(d.i18n("ChannelInformationCurrent")))==null?'':_.escape(__t))+
'</span><div style="background-color:#'+
((__t=(d.originalChannelColor))==null?'':_.escape(__t))+
'" class="colorblock"></div></td>\r\n	</tr>\r\n	<tr>\r\n		<td width="75%" colspan="3"><input class="channelName" value="'+
((__t=(d.originalName))==null?'':_.escape(__t))+
'" disabled></td>	 	\r\n	</tr>	\r\n	<tr>\r\n		<td width="25%">R: <input class="rgbField" size="3" value="'+
((__t=(d.originalRgb.r))==null?'':_.escape(__t))+
'" disabled></td>\r\n		<td width="25%">G: <input class="rgbField" size="3" value="'+
((__t=(d.originalRgb.g))==null?'':_.escape(__t))+
'" disabled></td>\r\n		<td width="25%">B: <input class="rgbField" size="3" value="'+
((__t=(d.originalRgb.b))==null?'':_.escape(__t))+
'" disabled></td>	\r\n	</tr>\r\n	<tr>\r\n		<td colspan="4"><input type="checkbox" ';
 if(d.wasOpaque){ 
__p+=' checked ';
 } 
__p+=' disabled> '+
((__t=(d.i18n("Opaque")))==null?'':_.escape(__t))+
'</td>\r\n	</tr>\r\n</table>\r\n<hr/>\r\n<table class="channelInformation">\r\n	<tr><th colspan="4">'+
((__t=(d.i18n("ChanDispTitle")))==null?'':_.escape(__t))+
'</th></tr>\r\n	<tr class="colorBlockCell">\r\n		<td></td><td></td><td></td>\r\n		<td width="25%" rowspan="3"><span class="colorBlockText">'+
((__t=(d.i18n("ChannelInformationNew")))==null?'':_.escape(__t))+
'</span><div style="background-color:#'+
((__t=(d.channelColor))==null?'':_.escape(__t))+
'" class="colorblock" id="colorChangeBlock"></div></td>\r\n	</tr>\r\n	<tr>\r\n		<td width="75%" colspan="3">\r\n		<select class="channelSelectName">\r\n		';
 _.each(d.channels, function(channel){ 
__p+='\r\n			';
 if(channel.isChanged()){ 
__p+='\r\n			<option value="'+
((__t=(channel.getID() ))==null?'':_.escape(__t))+
'" class="changedChannelDef" ';
 if(channel.getName()===d.channelName){
__p+='selected';
}
__p+='>'+
((__t=(channel.getName()))==null?'':_.escape(__t))+
'</option>\r\n			';
 } 
__p+='\r\n			<option value="'+
((__t=(channel.getID() ))==null?'':_.escape(__t))+
'" class="originalChannelDef" ';
 if(channel.getOriginalName()===d.channelName){
__p+='selected';
}
__p+='>'+
((__t=(channel.getOriginalName()))==null?'':_.escape(__t))+
'</option>\r\n		';
 }); 
__p+='\r\n		</select></td>	 	\r\n	</tr>	\r\n	<tr>\r\n		<td width="25%">R: <input class="rgbField" name="r" size="3" value="'+
((__t=(d.rgb.r))==null?'':_.escape(__t))+
'" ></td>\r\n		<td width="25%">G: <input class="rgbField" name="g" size="3" value="'+
((__t=(d.rgb.g))==null?'':_.escape(__t))+
'" ></td>\r\n		<td width="25%">B: <input class="rgbField" name="b" size="3" value="'+
((__t=(d.rgb.b))==null?'':_.escape(__t))+
'" ></td>	\r\n	</tr>\r\n	<tr>\r\n		<td colspan="4"><input name="opaque" type="checkbox" ';
 if(d.isOpaque){ 
__p+=' checked ';
 } 
__p+='> '+
((__t=(d.i18n("Opaque")))==null?'':_.escape(__t))+
'</td>\r\n	</tr>\r\n</table>';
return __p;
}});
define('web-viewer/graphics/ChannelPopup',["web-viewer/layout/component/Popup",
		"Template",
		"tpl!web-viewer/graphics/ChannelInformationPopup",
		"utils/ShapeUtils",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ColorPicker",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/utils/ViewerUtils",
		"jquery",
		"underscore"
	],
	function (Popup,
		Template,
		tpl,
		ShapeUtils,
		ButtonGroup,
		ColorPicker,
		ViewerToolbarButton,
		ViewerUtils,
		$) {
		"use strict";
		/**
		 * Create a popup for the channel information.
		 * @exports ChannelPopup
		 * @memberof Graphics
		 * @constructor
		 * @augments Layout.Popup
		 *
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} popupId - Id of the popup used for its private channel
		 * @param {GraphicsModel} graphicsModel - the model that represents holds the information on all the channels
		 */
		var ChannelPopup = function (f, popupId, graphicsModel) {
			Popup.call(this, f, popupId, $("#viewerAPP"));

			this.f = f;

			this.element.addClass("channelsInfoPopup");

			/**
			 * @private
			 */
			this._graphicsModel = graphicsModel;

			/**
			 * This colorpicker is used to pick another channel RGB value
			 * @private
			 */
			this._colorPicker = null;

			/**
			 * Channel that is selected and for which the information is shown in the popup
			 */
			this.channel = null;

			/**
			 * @private
			 */
			this._nextButton = new ViewerToolbarButton(this.f, "nextChannel", "ForwardIcon", false, _.bind(function () {
				var nextChannel = this._graphicsModel.getNextChannel(this.channel.getID());
				if (nextChannel) {
					this.setSelectedChannel(nextChannel);
				}
			}, this));

			/**
			 * @private
			 */
			this._previousButton = new ViewerToolbarButton(this.f, "previousChannel", "PreviousIcon", false, _.bind(function () {
				var previousChannel = this._graphicsModel.getPreviousChannel(this.channel.getID());
				if (previousChannel) {
					this.setSelectedChannel(previousChannel);
				}
			}, this));

			this._stepButtonGroup = new ButtonGroup("stepThroughChannels", [this._previousButton, this._nextButton]);

			this._tplContent = $("<div>");
			this._popupBtns = $("<div>").addClass("channelPopupBtns");

			f.i18n.loadBundle("viewer", _.bind(function () {
				/**
				 * @private
				 */
				this._revertButton = new ViewerToolbarButton(this.f, "undoChannelRgb", "Undo16", false, _.bind(function () {
					this._graphicsModel.resetChannel(this.channel.getID());
				}, this));
				this._revertButton.element.append(f.i18n.t("AppDialogRevert"));
				this._revertButton.element.addClass("undoChannelRgbBtn");

				this.content.append(this._tplContent);
				this._popupBtns.append(this._revertButton.element);
				this._popupBtns.append(this._stepButtonGroup.element);
				this._outerContent.append(this._popupBtns);

				this.setHeader($("<span />").addClass("popupTitle").html(f.i18n.t("ChanEditTitle")));
			}, this));

			/**
			 * template for the popup
			 */
			this.t = new Template(this.f.i18n, tpl);
			this.t.setContext(this._tplContent);

			this.t.bind("change", "[name=r],[name=g],[name=b]", this._rgbValueChanged, this);
			this.t.bind("click", "[name=opaque]", this._opaqueValueChanged, this);
			this.t.bind("change", ".channelSelectName", this._channelDefinitionChanged, this);
			this.t.bind("click", "#colorChangeBlock", function () {
				if (this._colorPicker) {
					this._colorPicker.destroy();
				}
				this._colorPicker = new ColorPicker(_.bind(function (newColor) {
					this._changeChannelColor(ViewerUtils.intFromHex(newColor));
				}, this), $("#colorChangeBlock"));
				this._colorPicker.setColor(this.channel.getHexRGB());
				this._colorPicker.show();
			}, this);

			this.on("closed", _.bind(function () {
				this.hide();
			}, this));

			this._debouncedRender = _.debounce(this.render, 50);

			//Listen to model for change in channel definition
			this._graphicsModel.on("changed.channel.*.channelRGB", _.bind(function () {
				if (this.visible) {
					this._debouncedRender();
				}
			}, this));
		};

		ChannelPopup.prototype = Object.create(Popup.prototype);
		ChannelPopup.prototype.constructor = ChannelPopup;

		/**
		 * Called when the checkbox for the opaque value changes
		 * @param {HTMLObject} el - the element that was clicked
		 * @param {Event} e - the event object
		 */
		ChannelPopup.prototype._opaqueValueChanged = function (el, e) {
			this._graphicsModel.setChannelOpaque(this.channel.getID(), el.checked);
		};

		/**
		 * Called when one of the values (R,G,B) is changed, changes the values for RGB, emits that this has changed and rerenders the popup
		 * @param {HTMLObject} el - the element that was clicked
		 * @param {Event} e - the event object
		 */
		ChannelPopup.prototype._rgbValueChanged = function (el, e) {
			var colorRGB = ViewerUtils.rgbColor(this.channel.getHexRGB());
			if (el.name === "r") {
				colorRGB.r = parseInt(el.value, 10);
			} else if (el.name === "g") {
				colorRGB.g = parseInt(el.value, 10);
			} else if (el.name === "b") {
				colorRGB.b = parseInt(el.value, 10);
			}
			this._changeChannelColor(ViewerUtils.intFromHex(ViewerUtils.rgbToHex(colorRGB.r, colorRGB.g, colorRGB.b)));
		};

		/**
		 * Called when the color of the selected channel is changed
		 * @param {Number} color - Integer representation of the rgb color
		 */
		ChannelPopup.prototype._changeChannelColor = function (color) {
			this._graphicsModel.changeChannelRgb(this.channel.getID(), color);
		};

		/**
		 * Called when the RGB of another channel will be used for the current channel, also changes the displayname of the channel
		 * @param {HTMLObject} el - the element that was clicked
		 * @param {Event} e - the event object
		 */
		ChannelPopup.prototype._channelDefinitionChanged = function (el, e) {
			var channelId = el.value;
			var isOriginalChannelDef = $(el).find(":selected").hasClass("originalChannelDef");
			this._graphicsModel.changeChannelDef(this.channel.getID(), channelId, isOriginalChannelDef);
		};

		/**
		 * Set the current selected channel, should be called before showing the popup
		 * @param {graphics.Channel} channel
		 */
		ChannelPopup.prototype.setSelectedChannel = function (channel) {
			this.channel = channel;
			this.updateStepButtons();
			this.updateRevertButton();
			this.render();
		};

		/**
		 * Show the popup, be sure the correct channel has been set
		 * @param {ShapeUtils.Point} location - location where the popup should be drawn
		 */
		ChannelPopup.prototype.show = function (location) {
			Popup.prototype.show.call(this, location, 250);

			this.render();
		};

		/**
		 * check if the previous and nextbutton need to be enabled or disabled
		 */
		ChannelPopup.prototype.updateStepButtons = function () {
			this._previousButton.setEnabled(this._graphicsModel.hasPreviousChannel(this.channel.getID()));
			this._nextButton.setEnabled(this._graphicsModel.hasNextChannel(this.channel.getID()));
		};

		/**
		 * Updates the revert button when needed
		 */
		ChannelPopup.prototype.updateRevertButton = function () {
			if (this.channel.isChanged()) {
				this._revertButton.setEnabled(true);
			} else {
				this._revertButton.setEnabled(false);
			}
		};

		/**
		 * Sets the correct values for the template and renders the template
		 */
		ChannelPopup.prototype.render = function () {
			this.t.d.channelName = this.channel.getName();
			this.t.d.originalName = this.channel.name;
			this.t.d.channelColor = this.channel.getHexRGB();
			this.t.d.originalChannelColor = ViewerUtils.hexFromInt(this.channel.rgb);
			this.t.d.channels = this._graphicsModel.getChannels();
			this.t.d.rgb = ViewerUtils.rgbColor(this.channel.getHexRGB());
			this.t.d.originalRgb = ViewerUtils.rgbColor(ViewerUtils.hexFromInt(this.channel.rgb));
			this.t.d.isOpaque = this.channel.isChannelOpaque();
			this.t.d.wasOpaque = this.channel.wasChannelOpaque();
			this.updateStepButtons();
			this.updateRevertButton();
			this.t.render();
		};

		return ChannelPopup;
	});

define('web-viewer/measure/EyeDropperUnitSelector',["web-viewer/layout/component/DropdownMiniButton",
		"web-viewer/core/CoreConstants",
		"underscore"
	],
	function (DropdownMiniButton,
		CoreConstants,
		_) {
		"use strict";

		/**
		 * @exports EyeDropperUnitSelector
		 * @class The EyeDropperUnitSelector has a button that allows setting the unit of the eye dropper
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var EyeDropperUnitSelector = function (f) {
			this.f = f;

			/**
			 * Make the options dropdown
			 */
			this._optionButton = new DropdownMiniButton(this.f, "eyeDropperUnitSettings", "icon-WCR_Settings-submenu");
			this._optionButton.setTooltip("viewer", "UnitSettings");

			this.f.i18n.loadBundle("viewer", function () {
				this._optionButton.addOption({
					name: 'eyeDropperUnitPixels',
					content: this.f.i18n.t(CoreConstants.EyeDropperUnitsLocNames[CoreConstants.EyeDropperUnits.PIXELS]),
					onchange: _.bind(this._toggleEyeDropperUnit, this, CoreConstants.EyeDropperUnits.PIXELS),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'eyeDropperUnitSQMM',
					content: this.f.i18n.t(CoreConstants.EyeDropperUnitsLocNames[CoreConstants.EyeDropperUnits.SQMM]),
					onchange: _.bind(this._toggleEyeDropperUnit, this, CoreConstants.EyeDropperUnits.SQMM),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'eyeDropperUnitSQCM',
					content: this.f.i18n.t(CoreConstants.EyeDropperUnitsLocNames[CoreConstants.EyeDropperUnits.SQCM]),
					onchange: _.bind(this._toggleEyeDropperUnit, this, CoreConstants.EyeDropperUnits.SQCM),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'eyeDropperUnitSQINCH',
					content: this.f.i18n.t(CoreConstants.EyeDropperUnitsLocNames[CoreConstants.EyeDropperUnits.SQINCH]),
					onchange: _.bind(this._toggleEyeDropperUnit, this, CoreConstants.EyeDropperUnits.SQINCH),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'eyeDropperUnitPercent',
					content: this.f.i18n.t(CoreConstants.EyeDropperUnitsLocNames[CoreConstants.EyeDropperUnits.PERCENT]),
					onchange: _.bind(this._toggleEyeDropperUnit, this, CoreConstants.EyeDropperUnits.PERCENT),
					userCannotUncheck: true
				});

				// intial rendering
				this._updateOptions();

				// listen to changes
				this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_EYEDROPPER", _.bind(this._updateOptions, this));
			}, this);
		};

		/**
		 * Get the button element
		 * @returns {JQueryObj} element of the button
		 */
		EyeDropperUnitSelector.prototype.getButtonElement = function () {
			return this._optionButton.element;
		};

		/**
		 * Toggle the eye dropper unit
		 * @param {CoreConstants.EyeDropperUnits} mode - the new mode
		 * @param {Boolean} toggle - true or false
		 */
		EyeDropperUnitSelector.prototype._toggleEyeDropperUnit = function (mode, toggle) {
			if (toggle) {
				this.f.setOption("PREFERENCES_UNIT_EYEDROPPER", mode, true);

				// link with unit size
				if (mode !== CoreConstants.EyeDropperUnits.PERCENT) {
					this.f.setOption("PREFERENCES_UNIT_SIZE", mode, true);
				}
			}
		};

		/**
		 * Update the options in the menu (re-render)
		 */
		EyeDropperUnitSelector.prototype._updateOptions = function () {
			var currentEyeDropperUnit = this.f.getOption("PREFERENCES_UNIT_EYEDROPPER");
			this._optionButton.toggleOption("eyeDropperUnitPixels", currentEyeDropperUnit === CoreConstants.EyeDropperUnits.PIXELS);
			this._optionButton.toggleOption("eyeDropperUnitSQMM", currentEyeDropperUnit === CoreConstants.EyeDropperUnits.SQMM);
			this._optionButton.toggleOption("eyeDropperUnitSQCM", currentEyeDropperUnit === CoreConstants.EyeDropperUnits.SQCM);
			this._optionButton.toggleOption("eyeDropperUnitSQINCH", currentEyeDropperUnit === CoreConstants.EyeDropperUnits.SQINCH);
			this._optionButton.toggleOption("eyeDropperUnitPercent", currentEyeDropperUnit === CoreConstants.EyeDropperUnits.PERCENT);
		};

		return EyeDropperUnitSelector;
	});

define('web-viewer/measure/ScreenRulingUnitSelector',["web-viewer/layout/component/DropdownMiniButton",
		"web-viewer/core/CoreConstants",
		"web-viewer/measure/MeasureConstants",
		"underscore"
	],
	function (DropdownMiniButton,
		CoreConstants,
		MeasureConstants,
		_) {
		"use strict";

		/**
		 * @exports ScreenRulingUnitSelector
		 * @class The ScreenRulingUnitSelector has a button that allows setting the unit for screen ruling
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var ScreenRulingUnitSelector = function (f) {
			this.f = f;

			/**
			 * Make the options dropdown
			 */
			this._optionButton = new DropdownMiniButton(f, "screenRulingUnitSettings", "icon-WCR_Settings-submenu");

			f.i18n.loadBundle("viewer", function () {

				// Units for size
				function toggleScreenRulingUnits(unit, mode, toggle) {
					if (toggle) {
						this.setUnitAndMode(unit, mode);
					}
				};

				this._optionButton.addOption({
					name: 'unitLpmm',
					content: this.f.i18n.t("UnitScreenmm"),
					onchange: _.bind(toggleScreenRulingUnits, this, CoreConstants.ResolutionUnits.PPMM, MeasureConstants.ScreenRulingMode.LINES),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'unitLpcm',
					content: this.f.i18n.t("UnitScreencm"),
					onchange: _.bind(toggleScreenRulingUnits, this, CoreConstants.ResolutionUnits.PPCM, MeasureConstants.ScreenRulingMode.LINES),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'unitLpi',
					content: this.f.i18n.t("UnitScreenInch"),
					onchange: _.bind(toggleScreenRulingUnits, this, CoreConstants.ResolutionUnits.PPI, MeasureConstants.ScreenRulingMode.LINES),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'unitDeg',
					content: this.f.i18n.t("UnitDegree"),
					onchange: _.bind(toggleScreenRulingUnits, this, 0, MeasureConstants.ScreenRulingMode.ANGLE),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'unitLpmmDeg',
					content: this.f.i18n.t("UnitDegreemm"),
					onchange: _.bind(toggleScreenRulingUnits, this, CoreConstants.ResolutionUnits.PPMM, MeasureConstants.ScreenRulingMode.BOTH),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'unitLpcmDeg',
					content: this.f.i18n.t("UnitDegreecm"),
					onchange: _.bind(toggleScreenRulingUnits, this, CoreConstants.ResolutionUnits.PPCM, MeasureConstants.ScreenRulingMode.BOTH),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'unitLpiDeg',
					content: this.f.i18n.t("UnitDegreeInch"),
					onchange: _.bind(toggleScreenRulingUnits, this, CoreConstants.ResolutionUnits.PPI, MeasureConstants.ScreenRulingMode.BOTH),
					userCannotUncheck: true
				});

				this._optionButton.addDivider();

				// angle CW or CCW
				function toggleScreenOrientation(mode, toggle) {
					if (toggle) {
						this.setOrientation(mode);
					}
				};

				this._optionButton.addOption({
					name: 'scrOrientationCW',
					content: this.f.i18n.t("ScreenCW"),
					onchange: _.bind(toggleScreenOrientation, this, MeasureConstants.ScreenRulingOrientation.CW),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'scrOrientationCCW',
					content: this.f.i18n.t("ScreenCCW"),
					onchange: _.bind(toggleScreenOrientation, this, MeasureConstants.ScreenRulingOrientation.CCW),
					userCannotUncheck: true
				});

				this._updateUnitOptions();
				this._updateOrientationOptions();
				this._optionButton.setTooltip("viewer", "UnitSettings");

				this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_SCREENRULING", _.bind(this._updateUnitOptions, this));
				this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_SCREEN", _.bind(this._updateUnitOptions, this));
				this.f.getCurrentUser().on("options.changed.PREFERENCES_SCREEN_ORIENTATION", _.bind(this._updateOrientationOptions, this));
			}, this);

		};

		/**
		 * Get the button element
		 * @returns {JQueryObj} element of the button
		 */
		ScreenRulingUnitSelector.prototype.getButtonElement = function () {
			return this._optionButton.element;
		};

		/**
		 * Set the unit and mode. The unit itself is not persisted, but the mode is.
		 * @param {CoreConstants.ResolutionUnits} unit
		 * @param {MeasureConstants.ScreenRulingMode} mode
		 */
		ScreenRulingUnitSelector.prototype.setUnitAndMode = function (unit, mode) {
			// set options and send out events
			if (this.f.getOption("PREFERENCES_UNIT_SCREENRULING") !== unit) {
				this.f.setOption("PREFERENCES_UNIT_SCREENRULING", unit, true);
			}
			if (this.f.getOption("PREFERENCES_UNIT_SCREEN") !== mode) {
				this.f.setOption("PREFERENCES_UNIT_SCREEN", mode, false);
			}
		};

		/**
		 * Update the toggle options of the button for units
		 */
		ScreenRulingUnitSelector.prototype._updateUnitOptions = function () {
			var unit = this.f.getOption("PREFERENCES_UNIT_SCREENRULING");
			var mode = this.f.getOption("PREFERENCES_UNIT_SCREEN");

			this._optionButton.toggleOption("unitLpmm", unit === CoreConstants.ResolutionUnits.PPMM && mode === MeasureConstants.ScreenRulingMode.LINES);
			this._optionButton.toggleOption("unitLpcm", unit === CoreConstants.ResolutionUnits.PPCM && mode === MeasureConstants.ScreenRulingMode.LINES);
			this._optionButton.toggleOption("unitLpi", unit === CoreConstants.ResolutionUnits.PPI && mode === MeasureConstants.ScreenRulingMode.LINES);
			this._optionButton.toggleOption("unitDeg", mode === MeasureConstants.ScreenRulingMode.ANGLE);
			this._optionButton.toggleOption("unitLpmmDeg", unit === CoreConstants.ResolutionUnits.PPMM && mode === MeasureConstants.ScreenRulingMode.BOTH);
			this._optionButton.toggleOption("unitLpcmDeg", unit === CoreConstants.ResolutionUnits.PPCM && mode === MeasureConstants.ScreenRulingMode.BOTH);
			this._optionButton.toggleOption("unitLpiDeg", unit === CoreConstants.ResolutionUnits.PPI && mode === MeasureConstants.ScreenRulingMode.BOTH);
		};

		/**
		 * Set the screen orientation.
		 * @param {MeasureConstants.ScreenRulingOrientation} mode
		 */
		ScreenRulingUnitSelector.prototype.setOrientation = function (mode) {
			// set option and send out events
			this.f.setOption("PREFERENCES_SCREEN_ORIENTATION", mode, false);
		};

		/**
		 * Update the toggle options of the button for orientation
		 */
		ScreenRulingUnitSelector.prototype._updateOrientationOptions = function () {
			var orientation = this.f.getOption("PREFERENCES_SCREEN_ORIENTATION");

			this._optionButton.toggleOption("scrOrientationCW", orientation === MeasureConstants.ScreenRulingOrientation.CW);
			this._optionButton.toggleOption("scrOrientationCCW", orientation === MeasureConstants.ScreenRulingOrientation.CCW);
		};

		return ScreenRulingUnitSelector;
	});

define('web-viewer/graphics/ChannelsWidget',["web-viewer/layout/AbstractWidget",
		"Template",
		"tpl!web-viewer/graphics/ChannelsWidget",
		"web-viewer/graphics/GraphicsConstants",
		"web-viewer/layout/component/ViewerToolbarButton",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/DropdownToolbarButton",
		"web-viewer/layout/component/ColorPicker",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/graphics/ChannelPopup",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/compare/CompareConstants",
		"web-viewer/measure/EyeDropperUnitSelector",
		"web-viewer/measure/ScreenRulingUnitSelector",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/core/CoreConstants",
		"jquery",
		"underscore",
		"jqueryUI",
		"jqueryUITouchPunch"
	],
	function (AbstractWidget,
		Template,
		tpl,
		GraphicsConstants,
		ViewerToolbarButton,
		ButtonGroup,
		DropdownToolbarButton,
		ColorPicker,
		LayoutConstants,
		ChannelPopup,
		ShapeUtils,
		ViewerUtils,
		CompareConstants,
		EyeDropperUnitSelector,
		ScreenRulingUnitSelector,
		ListenerUtil,
		CoreConstants,
		$,
		_) {
		"use strict";

		/**
		 * @exports ChannelsWidget
		 * @memberof Graphics
		 * @augments Layout.AbstractWidget
		 * @class The ChannelsWidget
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {GraphicsModel} graphicsModel
		 */
		var ChannelsWidget = function ChannelsWidget(f, graphicsModel) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.CHANNELS);
			this.f = f;
			ListenerUtil.mixin(this, f);
			// set the model
			this._graphicsModel = null;
			this.setGraphicsModel(graphicsModel);

			this.isOrderingEnabled = true;
			this._contentDiv = $("<div/>");

			this._colorPicker = null;

			// Settings
			this._eyeDropperUnitSelector = null;
			this._screenRulingUnitSelector = null;
			
			this.setIcon("icon icon-WCR_Channel-Color");

			this.t = new Template(this.f.i18n, tpl);
			this.t.setContext(this._contentDiv);

			//Get the list of channels from the model
			this.t.d.getChannels = _.bind(function () {
				if (this._graphicsModel) {
					var channels = this._graphicsModel.getChannels(true);

					return channels;
				} else {
					return [];
				}
			}, this);

			this.t.d.getChannelsInfo = function (channel) {
				if (this.channelsInfo) {
					return this.channelsInfo[channel.getViewserverID()] || "";
				} else {
					return "";
				}
			};

			//check if all channels are enabled
			this.t.d.isAllEnabled = _.bind(function () {
				return this.isAllChannelsEnabled();
			}, this);

			// get the suffix for a channel (if it should be shown)
			this.t.d.getChannelSuffix = _.bind(function (channel) {
				if (this._graphicsModel.isInDifferenceCompareMode()) {
					if (channel.isAdded()) {
						return "(+)";
					}
					if (channel.isSubtracted()) {
						return "(-)";
					}
				} else {
					return "";
				}
			}, this);

			//listen to measure tools if information is added
			this.f.on("channels.informationAdded", _.bind(function (channelsInfo) {
				this.t.d.channelsInfo = [];

				var parsedInfo = this._graphicsModel.getParsedChannelsInfo(channelsInfo);

				_.each(parsedInfo.channels, function (channel) {
					if (channel) {
						this.t.d.channelsInfo[channel.getViewserverID()] = channel.value;
					}
				}, this);

				this.t.d.totalCount = parsedInfo.total;

				this.render();
			}, this));

			this.f.on("channels.informationNotVisible", _.bind(function () {
				this.t.d.channelsInfo = null;
				this.t.d.totalCount = "";
				this.render();
			}, this));


			/**
			 * The popup that shows the channel information if the color box of that channel is clicked
			 */
			this.channelPopup = new ChannelPopup(f, "channelInformation", graphicsModel);

			// Split channel button
			this._splitChannels = new DropdownToolbarButton(f, "splitChannels", " icon-WCR_Tiling-Auto", true, _.bind(function (activated) {
				if (activated) {
					this.setChannelViewMode(GraphicsConstants.ChannelViewMode.AUTOMATIC_TILING);
				} else {
					this.setChannelViewMode(GraphicsConstants.ChannelViewMode.COMPOSITE_VIEW);
				}
			}, this));
			this._splitChannels.element.addClass("toggleBtn");

			// single channel button
			this._singleChannelMode = new DropdownToolbarButton(f, "singleChannelMode", "icon-WCR_Channel-Black", true, _.bind(function (activated) {
				if (activated) {
					this.setSingleChannelMode(GraphicsConstants.SingleChannelMode.IN_BLACK);
				} else {
					this.setSingleChannelMode(GraphicsConstants.SingleChannelMode.IN_CHANNEL_COLOR);
				}
			}, this));
			this._singleChannelMode.element.addClass("toggleBtn");

			this._resetChannelsButton = new DropdownToolbarButton(f, "resetChannels", "icon-WCR_Revert", true, _.bind(function () {
				this._graphicsModel.resetDefaultChannelOrder();
				this._graphicsModel.resetAllChannelsColorDef();
			}, this));

			// Move channel(s) up button
			this._moveChannelsUp = new ViewerToolbarButton(f, "moveChannelUp", "icon-WCR_Up", false, _.bind(function () {
				var i;
				var channel;
				if (this._graphicsModel.hasEnabledChannelGaps()) {
					// there are gaps in the list, move all elements together				
					var moveTo = null;
					for (i = 0; i < this._graphicsModel.getChannelCount(); i++) {
						channel = this._graphicsModel.getChannelAtIndex(i);
						if (channel.isEnabled()) {
							if (moveTo === null || moveTo === i) {
								// there was no enabled channel found yet, from now on, move all elements just after this element
								moveTo = i + 1;
							} else {
								// there was an enabled channel found, move it to the enabled channel block
								this._graphicsModel.moveChannelTo(channel.getID(), moveTo);
								moveTo++;
							}
						}
					}
				} else {
					// we can only move up if the first channel is not enabled
					if (this._graphicsModel.getChannelCount() >= 2 && !this._graphicsModel.getChannelAtIndex(0).isEnabled()) {
						// move every enabled channel 1 up
						i = 1;
						while (i < this._graphicsModel.getChannelCount()) {
							channel = this._graphicsModel.getChannelAtIndex(i);
							if (channel.isEnabled()) {
								this._graphicsModel.moveChannelTo(channel.getID(), i - 1);
							} else {
								i++;
							}
						}
					}
				}

			}, this));

			// Move channel(s) down button
			this._moveChannelsDown = new ViewerToolbarButton(f, "moveChannelDown", "icon-WCR_Down", false, _.bind(function () {
				var i;
				var channel;
				if (this._graphicsModel.hasEnabledChannelGaps()) {
					// there are gaps in the list, move all elements together				
					var moveTo = null;
					for (i = this._graphicsModel.getChannelCount() - 1; i >= 0; i--) {
						channel = this._graphicsModel.getChannelAtIndex(i);
						if (channel.isEnabled()) {
							if (moveTo === null || moveTo === i) {
								// there was no enabled channel found yet, from now on, move all elements just before this element
								moveTo = i - 1;
							} else {
								// there was an enabled channel found, move it to the enabled channel block
								this._graphicsModel.moveChannelTo(channel.getID(), moveTo);
								moveTo--;
							}
						}
					}
				} else {
					// we can only move down if the last channel is not enabled
					if (this._graphicsModel.getChannelCount() > 1 && !this._graphicsModel.getChannelAtIndex(this._graphicsModel.getChannelCount() - 1).isEnabled()) {
						// move every enabled channel 1 up
						i = graphicsModel.getChannelCount() - 2;
						while (i >= 0) {
							channel = this._graphicsModel.getChannelAtIndex(i);
							if (channel.isEnabled()) {
								this._graphicsModel.moveChannelTo(channel.getID(), i + 1);
							} else {
								i--;
							}
						}
					}
				}
			}, this));

			this._channelMatching = new ViewerToolbarButton(f, "channelMatching", "icon-WCR_Channel-Matching", false, function () {});
			this._channelMatching.element.hide();

			this._moveChannelsButtonGroup = new ButtonGroup("moveChannels", [this._moveChannelsUp, this._moveChannelsDown]);

			f.i18n.loadBundle("viewer", _.bind(function () {
				// set the widget title
				this.setTitle(this.f.t("ChannelsTitle"));

				//set tooltips for the buttons
				this._splitChannels.setTooltip(f.i18n.t("ChanSplitChannelsTT") + " - " + f.i18n.t("RightClickOptionTT"));
				this._singleChannelMode.setTooltip(f.i18n.t("ChanSingleTT") + " - " + f.i18n.t("RightClickOptionTT"));
				this._moveChannelsDown.setTooltip(f.i18n.t("ChanDownTT"));
				this._moveChannelsUp.setTooltip(f.i18n.t("ChanUpTT"));
				this._resetChannelsButton.setTooltip(f.i18n.t("ChanResetDefsTT") + " - " + f.i18n.t("RightClickOptionTT"));
				this._channelMatching.setTooltip(f.i18n.t("MapChanTitle"));

				// set the options
				this._splitChannels.addOption({
					name: 'composite',
					content: f.i18n.t("SplitChannelsOff"),
					onchange: _.bind(function () {
						this.setChannelViewMode(GraphicsConstants.ChannelViewMode.COMPOSITE_VIEW);
					}, this),
					userCannotUncheck: true
				});

				this._splitChannels.addOption({
					name: 'automatic',
					content: f.i18n.t("SplitChannelsTileAuto"),
					onchange: _.bind(function () {
						this.setChannelViewMode(GraphicsConstants.ChannelViewMode.AUTOMATIC_TILING);
					}, this),
					userCannotUncheck: true
				});

				this._splitChannels.addOption({
					name: 'horizontal',
					content: f.i18n.t("SplitChannelsTileHor"),
					onchange: _.bind(function () {
						this.setChannelViewMode(GraphicsConstants.ChannelViewMode.HORIZONTAL_TILING);
					}, this),
					userCannotUncheck: true
				});

				this._splitChannels.addOption({
					name: 'vertical',
					content: f.i18n.t("SplitChannelsTileVer"),
					onchange: _.bind(function () {
						this.setChannelViewMode(GraphicsConstants.ChannelViewMode.VERTICAL_TILING);
					}, this),
					userCannotUncheck: true
				});

				// initial option
				this.setChannelViewMode(this._graphicsModel.getChannelViewMode());



				// single channel as black

				this._singleChannelMode.addOption({
					name: 'inChannelColor',
					content: f.i18n.t("ChanSingleOff"),
					onchange: _.bind(function () {
						this.setSingleChannelMode(GraphicsConstants.SingleChannelMode.IN_CHANNEL_COLOR);
					}, this),
					userCannotUncheck: true
				});

				this._singleChannelMode.addOption({
					name: 'inBlack',
					content: f.i18n.t("ChanSingleOn"),
					onchange: _.bind(function () {
						this.setSingleChannelMode(GraphicsConstants.SingleChannelMode.IN_BLACK);
					}, this),
					userCannotUncheck: true
				});

				this._singleChannelMode.addOption({
					name: 'inBlackWithShowAll',
					content: f.i18n.t("ChanSingleShowAll"),
					onchange: _.bind(function () {
						this.setSingleChannelMode(GraphicsConstants.SingleChannelMode.IN_BLACK_WITH_SHOW_ALL_EFFECT);
					}, this),
					userCannotUncheck: true
				});

				// initial option
				this.setSingleChannelMode(this._graphicsModel.getSingleChannelMode());

				this._resetChannelsButton.addAction({
					name: 'resetOnlyChannelOrder',
					content: f.i18n.t("ChanResetOrder"),
					action: _.bind(function () {
						this._graphicsModel.resetDefaultChannelOrder();
					}, this)
				});

				this._resetChannelsButton.addAction({
					name: 'resetOnlyChannelsColorDef',
					content: f.i18n.t("ChanResetColor"),
					action: _.bind(function () {
						this._graphicsModel.resetAllChannelsColorDef();
					}, this)
				});

				this._resetChannelsButton.addAction({
					name: 'resetBothChannelsColorDefAndOrder',
					content: f.i18n.t("ChanResetBoth"),
					action: _.bind(function () {
						this._graphicsModel.resetDefaultChannelOrder();
						this._graphicsModel.resetAllChannelsColorDef();
					}, this)
				});
			}, this));

			//Initialize keyevents for the channels widget
			this.keyEvents = [];
			this.keyEvents.push([
				["alt+1", "alt+2", "alt+3", "alt+4", "alt+5", "alt+6", "alt+7", "alt+8", "alt+9"], _.bind(function (e) {
					e.preventDefault();
					//get the number of the key pressed, 96 and up are the keys on the numpad, 48 and up are the number keys
					var number = e.keyCode - 96 < 0 ? e.keyCode - 48 : e.keyCode - 96;
					var channels = this._graphicsModel.getChannels(true);

					if (number <= channels.length) { //only if the channel exists
						_.each(channels, _.bind(function (channel, index) {
							this._graphicsModel.toggleChannel(channel.getID(), index == number - 1);
						}, this));
					}
				}, this)
			]);
			//select all channels
			this.keyEvents.push([
				["alt+a"], _.bind(function (e) {
					e.preventDefault();
					_.each(this._graphicsModel.getChannels(), _.bind(function (channel, index) {
						this._graphicsModel.toggleChannel(channel.getID(), true);
					}, this));
				}, this)
			]);
			//enable first selected channel if more than 1 is enabled, if not select the next channel
			this.keyEvents.push([
				["alt+n"], _.bind(function (e) {
					e.preventDefault();
					var channels = this._graphicsModel.getChannels(true);

					var channelID = _.find(channels, function (channel) {
						return channel.isEnabled();
					}).getID();

					if (this._graphicsModel.getEnabledChannelCount(true) > 1) {
						_.each(channels, _.bind(function (channel, index) {
							this._graphicsModel.toggleChannel(channel.getID(), channel.getID() == channelID); //only enable first selected channel
						}, this));
					} else {
						this._graphicsModel.toggleChannel(channelID, false);

						var i = 0;
						while (i < channels.length && channels[i].getID() !== channelID) {
							i++;
						}
						var nextChannel = channels[(i + 1) % channels.length];
						this._graphicsModel.toggleChannel(nextChannel.getID(), true);
					}
				}, this)
			]);
			this.keyEvents.push([
				["ctrl+alt+0", "ctrl+alt+1", "ctrl+alt+2", "ctrl+alt+3", "ctrl+alt+4", "ctrl+alt+5", "ctrl+alt+6", "ctrl+alt+7", "ctrl+alt+8", "ctrl+alt+9"], _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					//get the number of the key pressed, 96 and up are the keys on the numpad, 48 and up are the number keys
					var number = e.keyCode - 96 < 0 ? e.keyCode - 48 : e.keyCode - 96;
					number = number + 10; //because these are channels 11 to 19
					var channels = this._graphicsModel.getChannels(true);

					if (number - 1 < channels.length) { //only if the channel exists
						_.each(channels, _.bind(function (channel, index) {
							this._graphicsModel.toggleChannel(channel.getID(), index == number - 1); //-1 because array starts at index 0

						}, this));
					}
				}, this), "keydown"
			]);
			this.bindKeys(this.keyEvents);

			// react on eyebox clicks
			if (!this.f.isMobile()) {
				this.t.bind("mouseup", ".eyeBox", this.eyeBoxClicked, this);
			} else {
				// double tap detection
				this._clickTimer = null;
				this._lastClickedElement = null;
				this.t.bind("touchstart pointerdown", ".eyeBox", function (el, event) {
					if (this._clickTimer == null || this._lastClickedElement !== el) {
						this._lastClickedElement = el;
						this._clickTimer = setTimeout(_.bind(function () {
							this._clickTimer = null;
							// single tap 
							this.eyeBoxClicked(el, event);
						}, this), 350);
					} else {
						clearTimeout(this._clickTimer);
						this._clickTimer = null;
						// double tap
						event.which = 3;
						this.eyeBoxClicked(el, event);
					}
				}, this);
			}

			// react on colorbox clicks
			this.t.bind("click", ".colorBox,.channelName", this.colorBoxClicked, this);
			this.t.bind("contextmenu", ".colorBox", this.colorBoxRightClicked, this);

			// do not show context menus
			this.t.bind("contextmenu", ".eyeBox", function (el, e) {
				e.preventDefault();
				return false;
			}, this);

			this.f.on("module.compare.changed.mode", _.bind(function (oldMode, newMode) {
				var enableOrdering = true;
				if (newMode && (newMode.getName() === CompareConstants.mode.BLINKING ||
						newMode.getName() === CompareConstants.mode.DIFFERENCE ||
						newMode.getName() === CompareConstants.mode.SHOWBOTHONTOP ||
						newMode.getName() === CompareConstants.mode.SIDEBYSIDE)) {
					enableOrdering = false;
				}
				this.toggleOrdering(enableOrdering);
			}, this));


			// React on active page changes
			this.f.getDocumentController().whenActivePageChangedAndParametersAvailable(this._updateEyeDropperUnitSelectorButton, this);
			// React on tool activate changes
			this.f.on("tool.measureEyeDropperTool.activated", _.bind(this._updateEyeDropperUnitSelectorButton, this));
			this.f.on("tool.measureEyeDropperTool.deactivated", _.bind(this._updateEyeDropperUnitSelectorButton, this));
			this.f.on("tool.measureScreenRulingTool.activated", _.bind(this._updateScreenRulingUnitSelectorButton, this));
			this.f.on("tool.measureScreenRulingTool.deactivated", _.bind(this._updateScreenRulingUnitSelectorButton, this));
		};


		ChannelsWidget.prototype = Object.create(AbstractWidget.prototype);
		ChannelsWidget.prototype.constructor = ChannelsWidget;
		ChannelsWidget.widgetName = LayoutConstants.Widget.CHANNELS;

		/**
		 * Set the context of the widget
		 * @override
		 * @param context
		 * @param header
		 */
		ChannelsWidget.prototype.setContext = function (context, header) {
			AbstractWidget.prototype.setContext.call(this, context, header);
			this.context.addClass("channelsWidget");

			// eye dropper unit selector for digital film if needed
			this._updateEyeDropperUnitSelectorButton();

			var buttons = $("<div/>").addClass("buttons");
			buttons.append(this._splitChannels.element);
			buttons.append(this._singleChannelMode.element);

			// only add reset, move and channelmatching for non mobile devices
			if (!this.f.isMobile()) {
				buttons.append(this._resetChannelsButton.element);
				buttons.append(this._moveChannelsButtonGroup.element);
				buttons.append(this._channelMatching.element);
			} else {
				// mobile 
				this.headerExtra.addClass("flexEnd").append(this._resetChannelsButton.element);
			}
			this.context.append(buttons);
			this.context.append(this._contentDiv);
			this.render();
		};

		/**
		 * Shows the button for channel matching
		 * @param {Function} action
		 */
		ChannelsWidget.prototype.showChannelMatchingBtn = function (action) {
			this._channelMatching.setAction(action);
			this._channelMatching.element.show();
		};

		/**
		 * hides the button for channel matching
		 */
		ChannelsWidget.prototype.hideChannelMatchingBtn = function () {
			this._channelMatching.element.hide();
		};

		/**
		 * Set the graphicsModel for the widget
		 * @param {GraphicsModel} graphicsModel
		 */
		ChannelsWidget.prototype.setGraphicsModel = function (graphicsModel) {
			this._unregisterAllListeners();

			this._graphicsModel = graphicsModel;

			var throttledRender = _.throttle(_.bind(function () {
				this.render();
			}, this), 40);

			//render the widget when the graphicsmodel changes
			throttledRender();

			this._registerListener("changed.channel.**", throttledRender, this, this._graphicsModel);
			this._registerListener("changed.channels", throttledRender, this, this._graphicsModel);
			this._registerListener("changed.channelViewMode", _.bind(function (oldValue, newValue) {
				this._updateChannelViewModeBtn(newValue);
			}, this), this, this._graphicsModel);
			this._registerListener("changed.channelOrder", throttledRender, this, this._graphicsModel);
		};

		/**
		 * A channel eyebox was clicked, execute needed actions
		 * @param {HTMLObject} el - the element that was clicked
		 * @param {Event} event - the event object
		 */
		ChannelsWidget.prototype.eyeBoxClicked = function (el, event) {
			var channels;
			//get the channel id
			var id = $(el).attr("data-bind");
			if (id == "all") {
				// the "ALL" box was clicked
				if (this.isAllChannelsEnabled()) {
					channels = this._graphicsModel.getChannels();
					// if all channels are enabled, disable all channels, except the first
					_.each(channels, function (channel, index) {
						if (index > 0) {
							this._graphicsModel.toggleChannel(channel.getID(), false);
						}
					}, this);
					this._graphicsModel.toggleChannel(channels[0].getID(), true);
				} else {
					// some channels were deselected, enable all channels again
					_.each(this._graphicsModel.getChannels(), function (channel, index) {
						this._graphicsModel.toggleChannel(channel.getID(), true);
					}, this);
				}
			} else {
				if (event.which === 3) {
					// right click, only select the clicked channel
					_.each(this._graphicsModel.getChannels(), function (channel, index) {
						this._graphicsModel.toggleChannel(channel.getID(), false);
					}, this);
					this._graphicsModel.toggleChannel(id, true);
				} else {
					var newToggleValue = !$(el).hasClass("selected");
					this._graphicsModel.toggleChannel(id, newToggleValue);

					if (this.isNoChannelEnabled()) {
						// no channels are selected anymore, find the next channel and select it
						channels = this._graphicsModel.getChannels();
						var i = 0;
						while (i < channels.length && channels[i].getID() !== id) {
							i++;
						}
						var nextChannel = channels[(i + 1) % channels.length];
						this._graphicsModel.toggleChannel(nextChannel.getID(), true);
					}
				}
			}
		};

		/**
		 * A channel colorblock was clicked, set correct data for the popup and show popup
		 * @param {HTMLObject} el - the element that was clicked
		 * @param {Event} event - the event object
		 */
		ChannelsWidget.prototype.colorBoxClicked = function (el, event) {
			if (!this.f.isMobile()) {
				if (event.which < 3) {
					//get the channel id
					var id = $(el).attr("data-bind");
					var channel = null;
					if (id === "all") {
						// if the colorbox of "all" is clicked, show the channelpopup for the first channel
						channel = _.first(this._graphicsModel.getChannels(true));
					} else {
						channel = this._graphicsModel.getChannel(id);
					}

					this.channelPopup.setSelectedChannel(channel);

					//checks if there is room to the left or the right of the widget
					var left = this.context.offset().left < 250 ? this.context.width() : this.context.offset().left - 250;
					var location = new ShapeUtils.Point(left, this.context.offset().top);
					this.channelPopup.show(location);
				} else if (event.which === 3) {
					// right click... show the colorpicker
					this.colorBoxRightClicked(el, event);
				}
			}
		};

		/**
		 * A channel colorblock was right clicked, show a color picker for the channel RGB definition
		 * @param {HTMLObject} el - the element that was clicked
		 * @param {Event} event - the event object
		 */
		ChannelsWidget.prototype.colorBoxRightClicked = function (el, event) {
			event.preventDefault();

			var id = $(el).attr("data-bind");

			if (id !== "all") {
				var channel = this._graphicsModel.getChannel(id);

				if (this._colorPicker) {
					this._colorPicker.destroy();
					this._colorPicker = null;
				}
				this._colorPicker = new ColorPicker(_.bind(function (newColor) {
					this._graphicsModel.changeChannelRgb(id, ViewerUtils.intFromHex(newColor));
				}, this), $("<div/>").insertAfter($(el)));

				this._colorPicker.setColor(channel.getHexRGB());
				this._colorPicker.show();
			}
		};



		/**
		 * Render the widget (note, this is a debounced function to avoid unnecessary redraws
		 */
		ChannelsWidget.prototype.render = _.debounce(function () {
			//make drag and drop for channel list
			if (this.isContextAvailable()) {
				this.t.render();
				var sortableHandle;
				if (this.f.isMobile()) {
					sortableHandle = ".colorBox";
				}
				this.context.find(".sortableChannelsList").sortable({
					items: "tr.dragAndDropChannel",
					update: _.bind(function () { //when the channel is dropped on new location
						this._graphicsModel.channelOrderChanged = true;
						var list = this.context.find(".sortableChannelsList").sortable("toArray", {
							attribute: "data-bind"
						});
						this._graphicsModel.setChannelOrder(list);
						if (this.channelPopup.visible) { //hide channelPopup if the order is changed
							this.channelPopup.hide();
						}
						this.updateRevertButton();
					}, this),
					handle: sortableHandle
				});
				this.context.find(".sortableChannelsList").disableSelection();
				this.updateMoveChannelsButtons();
				this.updateRevertButton();
				this._fireResized();
			}
		}, 40);

		/**
		 * @returns {Boolean} true if all channels are enabled
		 */
		ChannelsWidget.prototype.isAllChannelsEnabled = function () {
			return _.every(this._graphicsModel.getChannels(), function (channel) {
				return channel.isEnabled();
			});
		};

		/**
		 * @returns {Boolean} true if no channel is enabled
		 */
		ChannelsWidget.prototype.isNoChannelEnabled = function () {
			return _.every(this._graphicsModel.getChannels(), function (channel) {
				return !channel.isEnabled();
			});
		};

		/**
		 * Set the channel view mode
		 * @param {GraphicsConstants.ChannelViewMode} mode
		 */
		ChannelsWidget.prototype.setChannelViewMode = function (mode) {
			this._updateChannelViewModeBtn(mode);
			this._graphicsModel.setChannelViewMode(mode);
		};

		/**
		 * Update the channel view mode button
		 * @param {GraphicsConstants.ChannelViewMode} mode
		 */
		ChannelsWidget.prototype._updateChannelViewModeBtn = function (mode) {
			// update toggles
			this._splitChannels.toggleOption("composite", mode === GraphicsConstants.ChannelViewMode.COMPOSITE_VIEW);
			this._splitChannels.toggleOption("automatic", mode === GraphicsConstants.ChannelViewMode.AUTOMATIC_TILING);
			this._splitChannels.toggleOption("horizontal", mode === GraphicsConstants.ChannelViewMode.HORIZONTAL_TILING);
			this._splitChannels.toggleOption("vertical", mode === GraphicsConstants.ChannelViewMode.VERTICAL_TILING);

			this._splitChannels.setActive(mode !== GraphicsConstants.ChannelViewMode.COMPOSITE_VIEW);
		};

		/**
		 * Set the single channel mode
		 * @param {GraphicsConstants.SingleChannelMode} mode
		 */
		ChannelsWidget.prototype.setSingleChannelMode = function (mode) {
			this._singleChannelMode.setActive(mode !== GraphicsConstants.SingleChannelMode.IN_CHANNEL_COLOR);

			this._singleChannelMode.toggleOption("inChannelColor", mode === GraphicsConstants.SingleChannelMode.IN_CHANNEL_COLOR);
			this._singleChannelMode.toggleOption("inBlack", mode === GraphicsConstants.SingleChannelMode.IN_BLACK);
			this._singleChannelMode.toggleOption("inBlackWithShowAll", mode === GraphicsConstants.SingleChannelMode.IN_BLACK_WITH_SHOW_ALL_EFFECT);

			this._singleChannelMode.setTooltip("viewer", mode !== GraphicsConstants.SingleChannelMode.IN_CHANNEL_COLOR ? "ChanSingleTT2" : "ChanSingleTT");

			this._graphicsModel.setSingleChannelMode(mode);
		};

		/**
		 * Checks if the buttons for moving channels need to be disabled or enabled.
		 */
		ChannelsWidget.prototype.updateMoveChannelsButtons = function () {
			if (this.isContextAvailable()) {
				if (!this.isOrderingEnabled) {
					this._moveChannelsUp.setEnabled(false);
					this._moveChannelsDown.setEnabled(false);
					this.context.find(".sortableChannelsList").sortable("disable");
				} else {
					if (this._graphicsModel.hasEnabledChannelGaps()) {
						this._moveChannelsUp.setEnabled(true);
						this._moveChannelsDown.setEnabled(true);
					} else {
						var channels = this._graphicsModel.getChannels();
						if (channels.length > 0) {
							this._moveChannelsDown.setEnabled(!_.last(channels).isEnabled());
							this._moveChannelsUp.setEnabled(!_.first(channels).isEnabled());
						} else {
							this._moveChannelsDown.setEnabled(false);
							this._moveChannelsUp.setEnabled(false);
						}
					}
					this.context.find(".sortableChannelsList").sortable("enable");
				}
			}
		};

		/**
		 * Enable or disable ordering
		 * @param {Boolean} toggle
		 */
		ChannelsWidget.prototype.toggleOrdering = function (toggle) {
			if (this.isOrderingEnabled != toggle) {
				this.isOrderingEnabled = toggle;
				this.updateMoveChannelsButtons();
			}
		};

		/**
		 * Updates the revert button
		 */
		ChannelsWidget.prototype.updateRevertButton = function () {
			if (this._graphicsModel.hasChanges()) {
				this._resetChannelsButton.setEnabled(true);
			} else {
				this._resetChannelsButton.setEnabled(false);
			}
		};

		/**
		 * Update the eye dropper unit selector button.
		 * It will create it when needed.
		 */
		ChannelsWidget.prototype._updateEyeDropperUnitSelectorButton = function () {
			if (this.isContextAvailable()) {
				this.f.emit("toolManager.activeTool", _.bind(function (activeTool) {
					if ((activeTool !== undefined && activeTool.getIID() === CoreConstants.TOOLS.MEASURE_EYEDROPPER && activeTool.isActive()) && this.f.getActivePage().isDigitalFilmPage()) {
						if (this._eyeDropperUnitSelector === null) {
							this._eyeDropperUnitSelector = new EyeDropperUnitSelector(this.f);
							this.headerExtra.append(this._eyeDropperUnitSelector.getButtonElement());
						} else {
							this._eyeDropperUnitSelector.getButtonElement().show();
						}
					} else {
						if (this._eyeDropperUnitSelector !== null) {
							this._eyeDropperUnitSelector.getButtonElement().hide();
						}
					}
				}, this));
			}
		};

		/**
		 * Update the screen ruling unit selector button.
		 * It will create it when needed.
		 */
		ChannelsWidget.prototype._updateScreenRulingUnitSelectorButton = function () {
			if (this.isContextAvailable()) {
				this.f.emit("toolManager.activeTool", _.bind(function (activeTool) {
					if ((activeTool !== undefined && activeTool.getIID() === CoreConstants.TOOLS.MEASURE_SCREEN_RULING && activeTool.isActive()) && this.f.getActivePage().isDigitalFilmPage()) {
						if (this._screenRulingUnitSelector === null) {
							this._screenRulingUnitSelector = new ScreenRulingUnitSelector(this.f);
							this.headerExtra.append(this._screenRulingUnitSelector.getButtonElement());
						} else {
							this._screenRulingUnitSelector.getButtonElement().show();
						}
					} else {
						if (this._screenRulingUnitSelector !== null) {
							this._screenRulingUnitSelector.getButtonElement().hide();
						}
					}
				}, this));
			}
		};

		/**
		 * Fill on mobile
		 * @returns {Boolean}
		 * @override
		 */
		ChannelsWidget.prototype.willFill = function () {
			return this.f.isMobile();
		};


		return ChannelsWidget;
	});

define('web-viewer/graphics/GraphicsLayerModel',["underscore",
		"web-viewer/viewport/model/ViewportLayerModel",
		"web-viewer/utils/LineworkWrapper",
		"web-viewer/utils/DrawUtils",
		"utils/ShapeUtils",
		"web-viewer/utils/LineworkPoint",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/compare/CompareConstants"
	],
	function (_,
		ViewportLayerModel,
		LineworkWrapper,
		DrawUtils,
		ShapeUtils,
		LineworkPoint,
		ListenerUtil,
		ViewerUtils,
		CompareConstants
	) {
		"use strict";
		/**
		 * @exports GraphicsLayerModel
		 * @memberof Graphics
		 * @augments ViewportLayerModel
		 * @class This layer model is responsible to draw the bitmaps on the canvas.
		 * It also contains an option to display the trimbox or not (by default off).
		 * The main functions is "setOverwriteOption" which allows to call the exact same viewer calls, but with one or more options overwritten, like the compare mode
		 * @constructor
		 * @mixes Utils.ListenerUtilMixin
		 *
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {String} ID - the ID of the layer model
		 * @param {GraphicsModel} graphicsModel
		 */

		var GraphicsLayerModel = function (f, ID, graphicsModel) {
			ViewportLayerModel.call(this, f, ID);
			ListenerUtil.mixin(this, f);

			this._graphicsModel = graphicsModel;

			this._showMargin = false; //shows the margin of the graphics or not

			this.f.whenModuleAvailable("viewport", function (viewportModule) {
				this._viewport = viewportModule;
			}, this);

			this._hasGrid = false;

			this._buffer = null;

			/**
			 * Mark all fragments as dirty (debounced version)
			 */
			this.markAllAsDirty = _.throttle(_.bind(this._markAllAsDirty, this), 20);
			this._registerListener("changed.**", this.modelChanged, this, this._graphicsModel);


			this._registerListener("changed.channel.**", this.markAllAsDirty, undefined, this._graphicsModel);
			this._registerListener("changed.channelOrder", this.markAllAsDirty, undefined, this._graphicsModel);
			this._registerListener("changed.singleChannelMode", this.markAllAsDirty, undefined, this._graphicsModel);
			this._registerListener("viewport.viewserverParametersChanged", this.markAllAsDirty);
			this._registerListener("doc.parameters.changed", this.markAllAsDirty);
			this._registerListener("changed.page", this._cleanupFragments, this, this._graphicsModel);
			this._registerListener("needsFragmentCleanup", this._cleanupFragments, this, this._graphicsModel);
		};

		// inherit
		GraphicsLayerModel.prototype = Object.create(ViewportLayerModel.prototype);
		// correct the constructor pointer 
		GraphicsLayerModel.prototype.constructor = GraphicsLayerModel;


		/**
		 * Remove all listeners
		 */
		GraphicsLayerModel.prototype.destroy = function () {
			ViewportLayerModel.prototype.destroy.call(this);

			this._unregisterAllListeners();
		};


		/**
		 * This model has graphics!
		 * @returns {Boolean} true
		 */
		GraphicsLayerModel.prototype.hasGraphics = function () {
			return (this._graphicsModel.page && this._graphicsModel.page.hasGraphics()) || (this._graphicsModel.difference && this._graphicsModel.difference.hasGraphics());
		};

		/**
		 * Does the layermodel have a buffer, depends on the panel
		 * @param {String} panelName
		 * @returns {Boolean}
		 */
		GraphicsLayerModel.prototype.hasBuffer = function (panelName) {
			if (panelName === "default") {
				return this._buffer !== null;
			} else {
				return false;
			}
		};

		/**
		 * Returns the layers's buffer
		 * @returns {null|Fragment}
		 */
		GraphicsLayerModel.prototype.getBuffer = function () {
			return this._buffer;
		};

		/**
		 * Set the buffer for the layer
		 * @param {null|Fragment} buffer
		 */
		GraphicsLayerModel.prototype.setBuffer = function (buffer) {
			this._buffer = buffer;
		};

		/**
		 * Check if the layer model has CAD, returns true, do the same as for CAD layer if the graphics shows a grid (grid is a linework)
		 * @returns {Boolean}
		 */
		GraphicsLayerModel.prototype.hasLinework = function (panelName) {
			return this._hasGrid && panelName === "default";
		};


		/**
		 * For a graphics layer, the layer should autofetch fragments (this is not a task for the graphicsLayerModel)
		 * @override
		 * @returns {Boolean} true
		 */
		GraphicsLayerModel.prototype.shouldAutoFetchFragments = function () {
			return true;
		};

		/**
		 * @override
		 * @param {Array.<LineworkWrapper>} lineworks
		 * @param {Boolean} isAnimating
		 * @param {Boolean} isAnimatingRotation
		 * @param {ShapeUtils.Rect} drawrect
		 * @param {String} viewName - name of the view to get the grid from
		 * @param {String} panelName - the name of the panel this layer is part of
		 * @param {Boolean} isZooming - is there zooming active at this moment
		 */
		GraphicsLayerModel.prototype.getLinework = function (isAnimating, isAnimatingRotation, drawrect, viewName, panelName, isZooming) {
			var lineworks = [];

			if (this._graphicsModel.page !== undefined && !isZooming && drawrect !== undefined) {
				this._getTrimbox(lineworks, drawrect, panelName);

				this._getGridLinework(lineworks, isAnimating, isAnimatingRotation, drawrect, viewName, panelName);

				this._getReaderSpreadLines(lineworks);
			}

			return lineworks;


		};

		/**
		 * Construct the linework to show a grid on the graphics document if zoomed in on the document
		 * @param {Array.<LineworkWrapper>} lineworks
		 * @param {Boolean} isAnimating
		 * @param {Boolean} isAnimatingRotation
		 * @param {ShapeUtils.Rect} drawrect
		 * @param {String} viewName - name of the view to get the grid from
		 */
		GraphicsLayerModel.prototype._getGridLinework = function (lineworks, isAnimating, isAnimatingRotation, drawrect, viewName, panelName) {
			//only return linework for the grid if the zoom lever is bigger than 1000
			if (this._viewport && this._viewport.getWorld().getZoomPercentage() >= 1000) { //make sure the viewport is available (should be as the call for linework comes from the viewport)
				this._hasGrid = true;

				var view = this._viewport.getFirstViewOccurrence(viewName); //get the view for which grid needs te be drawn
				if (panelName === "default" && view) { //first check if this view is available, only draw grid on default panel
					var visibleRect = this._viewport.getVisibleRect(); //get the currently visible rectangle (scene-rect)
					//only the grid for the currently active page is shown in the viewport
					var activePage = this.f.getActivePage(); //get the currently active page from the viewerfacade for the correct resolution of the active page
					var resX = ViewerUtils.parseFloat(activePage.getParam("resolutionx")); //get the resolution for the x and the y axis
					var resY = ViewerUtils.parseFloat(activePage.getParam("resolutiony")); //resolution can be different for the y-axis, interval between grid lines will differ if this is the case	
					var zoom = this._viewport.getWorld().getZoom(); //ppmm, get the current zoom of the world, this are screen pixels per mm 
					var gridOffsetHeight = 0; //calculate the extra offset to shift the grid as the offset causes the active page to shift part of a document pixel
					var gridOffsetWidth = 0;

					//Calculate all the x-coordinates to draw lines
					var xCoord = Math.ceil(visibleRect.origin.x * resX) / resX + gridOffsetWidth; //mm, first value for the x-coordinate, will be changed to the next coordinate
					var initXDistancePx = Math.round((xCoord - visibleRect.origin.x) * zoom); //px, distance between the visibleRect border and the first grid line in pixels
					var intervalX = 1 / resX; //mmpp, mm per docucument pixel, is the interval between in mm between the grid lines
					var intervalPx = intervalX * zoom; //ppp, screen pixels per document pixel, amount of screen pixels between grid lines
					var endXCoord = visibleRect.origin.x + visibleRect.size.width; //mm, end of the visible rectangle
					var xPoints = []; //make an array for all the points of the grid on the x-axis
					while (xCoord < endXCoord) { //run over the width of the visible rect and add a grid line for every interval still inside the visible rect
						xPoints.push(xCoord);
						xCoord += intervalX;
					}
					xPoints.push(visibleRect.origin.x + visibleRect.size.width);

					//Calculate all the y-coordinates to draw grid lines
					var yCoord = Math.ceil(visibleRect.origin.y * resY) / resY + gridOffsetHeight; //mm, first value for the y-coordinate
					var initYDistancePx = Math.round((yCoord - visibleRect.origin.y) * zoom); //px, distance between the visibleRect border and the first grid line in pixels
					var endYCoord = visibleRect.origin.y + visibleRect.size.height; //mm, end of the visible rectangle
					var intervalY = 1 / resY; //mmpp, mm per docucument pixel, is the interval in mm between the grid lines
					var intervalPy = intervalY * zoom; //ppp, screen pixels per document pixel, amount of screen pixels between grid lines
					var yPoints = []; //make an array for all the points of the grid on the y axis
					while (yCoord < endYCoord) { // run over all the gridlines that fit in the visible rectangle
						yPoints.push(yCoord);
						yCoord += intervalY;
					}
					yPoints.push(visibleRect.origin.y + visibleRect.size.height);

					//The Graphics canvas is needed to reques the pixel data to calculate the correct RGB values for the grid
					this._graphicsLayer = view.getLayer("Graphics").getLayerElement();

					var rotation = this._viewport.getWorld().getRotation() % 360; //get the current rotation 
					var flip = this._viewport.getWorld().isFlip(); //get the flipped values of the world
					var isViewFromBack = this._viewport.getWorld().isFlipViewFromBack(); // is flipped view from back
					if (isViewFromBack) {
						if (rotation === 90 || rotation === 270) {
							rotation = (rotation + 180) % 360;
						}
						flip = !flip;
					}


					//to be sure always the correct value for the color is selected, move to half-pixels
					var xhalfPx = Math.ceil(intervalPx / 2);
					var yhalfPx = Math.ceil(intervalPy / 2);

					var width = $(this._graphicsLayer).innerWidth(); //we'll use the width and height of the canvas, gives the amount of screen pixels for the width
					var height = $(this._graphicsLayer).innerHeight();

					var worldBorderWidth = 0;
					if (visibleRect.origin.x === 0) {
						worldBorderWidth = width - Math.ceil(visibleRect.size.width * zoom);
					}
					var worldBorderHeight = 0;
					if (visibleRect.origin.y === 0) {
						worldBorderHeight = height - Math.ceil(visibleRect.size.height * zoom);
					}

					//only if the canvas has a width and height go further
					if (width !== 0 && height !== 0) {
						//request the RGB values for all the visible screen pixels, for each pixel 4 values are returned (red, green, blue and alpha)
						var imageColor = $(this._graphicsLayer)[0].getContext("2d").getImageData(0, 0, width, height); //return a 1D array with all the values row by row
						var color = imageColor.data; //make a copy of the array of pixels
						_.each(xPoints, function (xPoint, indexX) { //run over all the points for the grid to construct the grid, every pass construct two lines to the side of the pixel
							_.each(yPoints, function (yPoint, indexY) {
								//depending on rotation/flip the pixel to use for lines at the border of the viewport differs (try to keep it in the viewport)
								//the normal offset, yhalfPx/xhalfPx (to get to the center of the document pixel) is reverted or added twice, depending on the sign
								var ySign = 1;
								var xSign = 1;

								var x1 = xPoint; //current x-point
								var x0 = 0;
								if (indexX == 0) { //if it's the first point for the x-coordinate, take the origin of the visible rect as second x-coordinate
									x0 = visibleRect.origin.x;
								} else { //if not, take the previous point as second x-coordinate
									x0 = xPoints[indexX - 1];
								}

								//same as for the x-coordinate, construct the needed y-coordinates
								var y1 = yPoint;
								var y0 = 0;
								if (indexY == 0) {
									y0 = visibleRect.origin.y;
								} else {
									y0 = yPoints[indexY - 1];
								}

								//First correct values need to be calculated due to rotation and flipped state of the viewport. Visible rect rotates, the origin 
								//changes when the world is rotated. The width and height stay the same and the image pixel data stays with its origin at the left-top corner.
								var indexCoordX = indexX; //make sure to not change the index, this will be changed when the world is rotated/flipped
								var indexCoordY = indexY; //number of the row
								var xOffset = initXDistancePx; //need to add an offset to get the correct rgb value from the pixeldata-object, use the width betweeen the first grid line and the visible rectangle border. (changed when rotated/flipped)
								var yOffset = initYDistancePx;
								if (rotation === 90 || rotation === 180) { //if the rotation is 90 or 180 degrees, the origin of the visible rectangle will be on the right not on the left
									indexCoordY = yPoints.length - indexY - 1; //the index to calculate the inverse RGB-value needs to be moved to the other side of the viewport
									if (rotation === 90) { //calculate the correct offset, as with the index, we need the offset from the left side, not the right side
										yOffset = (width - initYDistancePx) % Math.round(intervalPy); //width is the yOffset as the coordinate system is rotated (the pixel data is not), %interval to get the pixel between the grid line and the visible rect border

									} else { //180 degrees
										yOffset = (height - initYDistancePx) % Math.round(intervalPy); //for 180 also the y-offset needs to be from the top and not the bottom of the viewport
									}
									if (indexY === 0) { //a line at the border of the viewport
										ySign = -1; //add yhalfPx twice to get to a pixel that's visible
									}
								} else if (_.last(yPoints) == yPoint) { //a line at the border of the viewport
									ySign = -1; //add yhalfPx twice to get to a pixel that's visible
								}

								//do the same for the x-coordinates, it's needed to take the flipped-state into account
								if (((rotation === 180 || rotation === 270) && !flip) || ((rotation === 90 || rotation === 0) && flip)) {
									indexCoordX = xPoints.length - indexX - 1; //x-coordinate lays on the wrong side of the visible rectangle, move it to the correct

									if (rotation === 90 || rotation === 270) { //offset needs to be calculated as 90+flipped or 270 are wrong (and since it's rotated height is used)
										xOffset = (height - initXDistancePx) % Math.round(intervalPx);
									} else { //0°+flipped or 180° not flipped
										xOffset = (width - initXDistancePx) % Math.round(intervalPx);
									}
									if (indexX === 0) { //a line at the border of the viewport
										xSign = -1;
									}
								} else if (_.last(xPoints) == xPoint) { //a line at the border of the viewport
									xSign = -1;
								}

								//calculate the index in the 1D array for the color of the grid lines, rows are placed behind eachother in the array
								//For all Y-coordinates multiply with the width of the viewport to get to the correct 'row'. Everything is multiplied by 4
								//because every pixel has 4 values (RGB+Alpha).
								var colorIndex = 0;
								if (rotation == 90 || rotation == 270) { //for 90° and 270°, all the x- and y-coordinates are switched
									colorIndex = width * Math.round(indexCoordX * intervalPx) + xOffset * width + Math.round(intervalPy * indexCoordY) + yOffset - xhalfPx * width - yhalfPx;
									//if the color is for a line at the border of the viewport, compensate colorIndex to get a value of the correct (visible) pixel
									if (indexX === 0 || indexY === 0 || _.last(xPoints) == xPoint || _.last(yPoints) == yPoint) {
										colorIndex += xSign * (xhalfPx - 2) * width + ySign * (yhalfPx - 2); //either add the xhalfPx/yhalfPx twice or subtract it with a difference of 2px to avoid borders of document pixels
									}
								} else {
									colorIndex = width * Math.round(indexCoordY * intervalPy) //multiply the number of the row with the number of pixels per document pixel and the width => y-coordinate
										+ yOffset * width //multiply the initial distance between the first grid-line and visible rect with the width of the viewport => y-coordinate
										+ worldBorderHeight * width + worldBorderWidth + Math.round(intervalPx * indexCoordX) //multiply the number of document pixel width the number of pixels per document pixel
										+ xOffset //add the distance between the most left grid-line and the left of the visible rectangel
										- yhalfPx * width - xhalfPx;
									//if the color is for a line at the border of the viewport, compensate colorIndex to get a value of the correct (visible) pixel
									if (indexX === 0 || indexY === 0 || _.last(xPoints) == xPoint || _.last(yPoints) == yPoint) {
										colorIndex += ySign * (yhalfPx - 2) * width + xSign * (xhalfPx - 2);
									}
								}

								colorIndex *= 4; //multiply with 4 as every pixel has 4 values
								//get the RGB values for the pixel from the pixel data array and invert them to get the values for the grid-lines
								var r = (255 - color[colorIndex + 0]).toString(16);
								var g = (255 - color[colorIndex + 1]).toString(16);
								var b = (255 - color[colorIndex + 2]).toString(16);

								if (r.length === 1) {
									r = "0" + r;
								}
								if (g.length === 1) {
									g = "0" + g;
								}
								if (b.length === 1) {
									b = "0" + b;
								}

								//Construct the two lines to draw arround the pixel
								var linework = new LineworkWrapper(); //use a lineworkwrapper to push in the linework array
								linework.type = DrawUtils.LWTYPE_LINE;
								linework.linewidth = 1;
								linework.color = '#' + r + g + b; //use the iverted RGB-values
								linework.pointArray[0] = new LineworkPoint();
								linework.pointArray[0].makeMoveTo(x0, y1); //make an horizontal line
								linework.pointArray[1] = new LineworkPoint();
								linework.pointArray[1].makeLineTo(x1, y1);
								lineworks.push(linework); //push the linework in the linework array
								linework = new LineworkWrapper();
								linework.type = DrawUtils.LWTYPE_LINE;
								linework.linewidth = 1;
								linework.color = '#' + r + g + b;
								linework.pointArray[0] = new LineworkPoint();
								linework.pointArray[0].makeMoveTo(x1, y0); //make a vertical line
								linework.pointArray[1] = new LineworkPoint();
								linework.pointArray[1].makeLineTo(x1, y1);
								lineworks.push(linework);

							}, this);
						}, this);
					}
				}
			} else {
				this._hasGrid = false;
			}
		};

		/**
		 * Get the linework for the small lines for multipage/page list documents when in reader spread mode
		 * @param {Array.<LineworkWrapper>} lineworks
		 */
		GraphicsLayerModel.prototype._getReaderSpreadLines = function (lineworks) {
			//if the document is a multi page document, and reader spread is active show 2 small lines indicating border of the active document
			if (this._graphicsModel.page.isReaderSpread()) {
				//create two lineworkwrappers for the top and bottom
				var lineworkTop = new LineworkWrapper();
				lineworkTop.type = DrawUtils.LWTYPE_LINE;
				lineworkTop.linewidth = 1;
				lineworkTop.color = '#ababab';
				var lineworkBottom = new LineworkWrapper();
				lineworkBottom.type = DrawUtils.LWTYPE_LINE;
				lineworkBottom.linewidth = 1;
				lineworkBottom.color = '#ababab';

				var page = this._graphicsModel.getPage();
				var pageSize = page.getPageSize();
				var referencePage = this._graphicsModel.getReferencePage();
				var lineLocationX = 0;
				if (page && page.get("isRightPage")) {
					if (referencePage) {
						if (referencePage.isBlankPage()) {
							lineLocationX = referencePage.getTrimboxSize().width;
							if (page.isBlankPage()) {
								lineLocationX += referencePage.getMargins().left;
							}
						} else {
							lineLocationX = referencePage.getPageSize().width - referencePage.getMargins().right;
						}
					} else {
						if (page.isBlankPage()) {
							lineLocationX = pageSize.width + page.getMargins().left;
						} else {
							lineLocationX = page.getTrimboxRect().origin.x - page.getMargins().left;
						}
					}
				} else {
					if (page.isBlankPage() && referencePage) { //page is blank and reference is not empty
						lineLocationX = page.getTrimboxSize().width;
						if (referencePage.isBlankPage()) {
							lineLocationX += page.getMargins().left;
						}
					} else {
						lineLocationX = pageSize.width - page.getMargins().right;
					}
				}

				lineworkTop.pointArray[0] = new LineworkPoint();
				lineworkTop.pointArray[0].makeMoveTo(lineLocationX, -(pageSize.height * 0.1));
				lineworkTop.pointArray[1] = new LineworkPoint();
				//don't draw against the document, leave some space (use 0.01 of the document height)
				lineworkTop.pointArray[1].makeLineTo(lineLocationX, -(pageSize.height * 0.01));

				lineworkBottom.pointArray[0] = new LineworkPoint();
				lineworkBottom.pointArray[0].makeMoveTo(lineLocationX, pageSize.height + (pageSize.height * 0.1));
				lineworkBottom.pointArray[1] = new LineworkPoint();
				lineworkBottom.pointArray[1].makeLineTo(lineLocationX, pageSize.height + (pageSize.height * 0.01));

				lineworks.push(lineworkTop);
				lineworks.push(lineworkBottom);
			}
		};

		/**
		 * Get the linework for the trimbox
		 * @param {Array.<LineworkWrapper>} lineworks
		 */
		GraphicsLayerModel.prototype._getTrimbox = function (lineworks, drawRect, panelName) {
			if (this._graphicsModel.isShowTrimbox()) {
				var trimbox = new LineworkWrapper();
				var activePageTrimbox = this._graphicsModel.getTrimBox();

				var isInSideBySide = this._graphicsModel.dmode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDE ||
					this._graphicsModel.dmode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDEHIGHLIGHT ||
					this._graphicsModel.dmode === CompareConstants.diffMode.SHOWBOTHSIDEBYSIDECHANGESONLY;

				if (activePageTrimbox !== null && !activePageTrimbox.containsRect(drawRect) && this._graphicsModel.dmode !== CompareConstants.diffMode.DIFFVERSION && ((isInSideBySide && (panelName === "default" || panelName === "compare_align_result" || panelName === "compare_differenceMode")) || !isInSideBySide)) {
					trimbox.type = DrawUtils.LWTYPE_DASHEDBOX;
					trimbox.dash = [10, 3, 2, 3, 2, 3];
					trimbox.linewidth = 1;
					trimbox.color = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_MARGINS");
					trimbox.fillcolor = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_MARGINS");
					trimbox.rect = activePageTrimbox;
					lineworks.push(trimbox);
				}

				var referenceTrimBox = this._graphicsModel.getReferenceTrimBox();
				if (referenceTrimBox !== null && !referenceTrimBox.containsRect(drawRect) && this._graphicsModel.dmode !== CompareConstants.diffMode.CURRENTVERSION && ((isInSideBySide && (panelName === "compare_difference" || panelName === "compare_align_result" || panelName === "compare_differenceMode")) || !isInSideBySide)) {
					trimbox = new LineworkWrapper();
					trimbox.type = DrawUtils.LWTYPE_DASHEDBOX;
					trimbox.dash = [10, 3, 2, 3, 2, 3];
					trimbox.linewidth = 1;
					trimbox.color = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_MARGINS_DIFF");
					trimbox.fillcolor = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_MARGINS_DIFF");
					trimbox.rect = referenceTrimBox;
					lineworks.push(trimbox);
				}
			}
		};

		/**
		 * @override
		 */
		GraphicsLayerModel.prototype.getMaximumResolution = function () {
			return this._graphicsModel.getMaximumResolution();
		};

		/**
		 * @override
		 */
		GraphicsLayerModel.prototype.getBackgroundOptions = function () {
			var options = {};

			if (this._graphicsModel.page && this._graphicsModel.page.isBlankPage()) {
				options.fillStyle = "rgba(255,255,255,1)";
			}

			return options;
		};

		return GraphicsLayerModel;
	});

define('web-viewer/graphics/GraphicsViewportWidgetController',["web-viewer/graphics/GraphicsConstants",
		"web-viewer/graphics/GraphicsLayerModel",
		"web-viewer/viewport/ViewportConstants",
		"web-viewer/utils/ListenerUtil",
		"underscore"
	],
	function (GraphicsConstants,
		GraphicsLayerModel,
		ViewportConstants,
		ListenerUtil,
		_) {

		/**
		 * @exports GraphicsViewportWidgetController
		 * @memberof Graphics
		 * @class This class controls the viewport widget model for graphics.
		 * It creates for example the views for channels, etc.
		 * @constructor
		 * @mixes Util.ListenerUtilMixin
		 * @param {Core.ViewerFacade} f
		 * @param {GraphicsModel} graphicsModel - Model for graphics
		 */
		var GraphicsViewportWidgetController = function (f, graphicsModel) {
			this.f = f;
			ListenerUtil.mixin(this, f);
			this._graphicsModel = null;
			this.viewport = this.f.getModule("viewport");
			this._createdViewLayerModels = {};
			this._shownViews = [];

			this.setGraphicsModel(graphicsModel);

			//register listeners for the change of the trimbox colors in the settings, needs to update the viewport (trigger a rerender)
			this._registerListener("options.changed.PREFERENCES_HIGHLIGHTCOLORS_MARGINS_DIFF", function (oldValue, newValue) {
				if (oldValue !== newValue) { //only if the color changed repaint the viewport
					this._repaintViewport();
				}
			}, this, this.f.getCurrentUser());

			this._registerListener("options.changed.PREFERENCES_HIGHLIGHTCOLORS_MARGINS", function (oldValue, newValue) {
				if (oldValue !== newValue) {
					this._repaintViewport();
				}
			}, this, this.f.getCurrentUser());
		};

		/**
		 * Change the graphics model
		 * @param {GraphicsModule.GraphicsModel} graphicsModel
		 */
		GraphicsViewportWidgetController.prototype.setGraphicsModel = function (graphicsModel) {
			this._unregisterAllListeners();

			this._graphicsModel = graphicsModel;

			this._registerListener("changed.channels", this.updateTiles, this, this._graphicsModel);
			this._registerListener("changed.channelViewMode", this.updateTiles, this, this._graphicsModel);
			this._registerListener("changed.channel.*.enabled", this.updateTiles, this, this._graphicsModel);
			this._registerListener("changed.channelOrder", this.updateTiles, this, this._graphicsModel);
			this._registerListener("changed.trimbox", this._repaintViewport, this, this._graphicsModel);
			this._registerListener("changed.page", function () {
				this.destructViews();
				this.updateTiles();
			}, this, this._graphicsModel);
		};

		/**
		 * get/generate the layermodel for a certain channel
		 * If the layer does not exist yet, create a new one
		 * @param {Graphics.Channel} channel - the channel for which a layermodel/layer needs to be created
		 * @returns {Graphics.GraphicsLayerModel} 
		 */
		GraphicsViewportWidgetController.prototype.getLayerModelForChannel = function (channel) {
			var layerModel = this._createdViewLayerModels[channel.getID()];

			if (!layerModel) {
				// NOTE: in the future we might need to limit the amount of existing layermodels...
				// create the layermodel and view
				this.viewport.createView(channel.getID(), channel.getName(), false);
				layerModel = new GraphicsLayerModel(this.f, "Graphics", this._graphicsModel);
				var selectionMask = this._graphicsModel.getSpecificChannelSelectionMask(channel);
				layerModel.setOverwriteOption("color", selectionMask);
				this.viewport.addLayer("Graphics", layerModel, channel.getID());
				this._createdViewLayerModels[channel.getID()] = layerModel;
			}

			return layerModel;
		};


		/**
		 * Called when the channel view mode has changed
		 */
		GraphicsViewportWidgetController.prototype.updateTiles = function () {
			var isTiledViewEnabled = this.isTiledViewEnabled();

			// disable tiled mode is there is only one extra view available
			if (isTiledViewEnabled) {
				if (this._graphicsModel.getChannelViewMode() !== GraphicsConstants.ChannelViewMode.COMPOSITE_VIEW) {
					if (this._graphicsModel.getEnabledChannelCount() <= 1) {
						isTiledViewEnabled = false;
					}
				}
			}

			// in here all the views that will be hidden are stored
			// by default mark all shown views to be hidden
			var viewsToHide = this._shownViews;

			// in here all the channels that need to be shown are stored
			var channelsToShow = [];


			if (isTiledViewEnabled) {
				// only run over all visible channels if the tiled mode is enabled
				_.each(this._graphicsModel.getChannels(), function (channel) {
					if (channel.isEnabled()) {
						if (this._graphicsModel.isInDifferenceCompareMode()) {
							if (channel.isInPreviousAndMatchedWithCurrent()) {
								// if viewer is in difference compare mode and the channel is from the difference file and matched with a channel from the current file, just hide id
								return;
							}
						}

						// the channel is visible. and it should be shown, remove it from the viewsToHide array and push in on the "show" array
						viewsToHide = _.without(viewsToHide, channel.getID());
						channelsToShow.push(channel);
					}
				}, this);
			}

			// hide all views that need to be hidden
			_.each(viewsToHide, function (view) {
				this.viewport.toggleView(view, false);
			}, this);

			// rebuild the _shownViews variable
			this._shownViews = [];
			_.each(channelsToShow, function (channel) {
				// get/generate the layer
				var layerModel = this.getLayerModelForChannel(channel);

				// get the color selection mode
				var selectionMask = this._graphicsModel.getSpecificChannelSelectionMask(channel);

				if (this._graphicsModel.isInDifferenceCompareMode() && channel.isInCurrentAndMatchedToPrevious()) {
					// this channel was matched onto a channel of the difference file, add matched channel in the mask
					var matchedChannel = this._graphicsModel.getMatchedChannel(channel);
					if (matchedChannel) {
						selectionMask += this._graphicsModel.getSpecificChannelSelectionMask(matchedChannel);
					}
				}
				// overwrite the channel selection mask for this view
				layerModel.setOverwriteOption("color", selectionMask);

				this._shownViews.push(channel.getID());
				this.viewport.toggleView(channel.getID(), true);

			}, this);


			if (isTiledViewEnabled) {
				// generate the needed order and push it to the viewport widget
				var order = [];
				_.each(channelsToShow, function (channel) {
					order.push(channel.getID());
				});
				// set the order
				this.viewport.setViewsOrder(order);
			}


			// disable animation if in tiled mode
			this.f.emit("viewport.toggleAnimation", "tiledMode", !isTiledViewEnabled);
			// to make it more readable in the GraphicsConstants the Channel View Mode 'names' are mapped to the Layout Mode 'names' of ViewportConstants
			this.viewport.setViewsLayoutMode(ViewportConstants.ViewsLayoutMode[GraphicsConstants.ChannelViewModeLayoutMode[_.invert(GraphicsConstants.ChannelViewMode)[this._graphicsModel.getChannelViewMode()]]]);
		};


		/**
		 * is the tiled mode active
		 * returns {Boolean}
		 */
		GraphicsViewportWidgetController.prototype.isTiledViewEnabled = function () {
			return this._graphicsModel.getChannelViewMode() !== GraphicsConstants.ChannelViewMode.COMPOSITE_VIEW;
		};

		/**
		 * Trigger a repaint of the viewport 
		 */
		GraphicsViewportWidgetController.prototype._repaintViewport = function () {
			this.f.emit("viewport.repaint");
		};


		/**
		 * Destruct all views that are created
		 */
		GraphicsViewportWidgetController.prototype.destructViews = function () {
			_.each(this._createdViewLayerModels, function (value, key) {
				this.viewport.removeView(key);
				value.destroy();
			}, this);
			this._createdViewLayerModels = {};
		};

		return GraphicsViewportWidgetController;
	});

define('web-viewer/graphics/ChannelMatchingModel',["web-viewer/core/PrivateChannel",
		"web-viewer/graphics/GraphicsConstants",
		"jquery",
		"underscore"
	],
	function (
		PrivateChannel,
		GraphicsConstants,
		$) {
		"use strict";
		/**
		 * 
		 * @exports ChannelMatchingPopup
		 * @memberof GraphicsModule
		 * @class ChannelMatchingModel holds all the information for the channel matching. It saves all current channels, reference channels and all the matched channels.
		 * @constructor
		 * 
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {Array.<Channel>} channels - all the channels of the current and reference document page
		 */
		var ChannelMatchingModel = function (f, channels) {
			PrivateChannel.installTo(this, f, 'channelMatchingModel');

			/**
			 * @private
			 * @member {Array.<Channel>} the channels of the active document page
			 */
			this._currentChannels = {};

			/**
			 * @private
			 * @member {Array.<Channel>} the channels of the reference document page
			 */
			this._referenceChannels = {};

			/**
			 * @private
			 * @member {GraphicsConstants.ChannelMatching} how are the channels matched, saved in user preferences
			 */
			this._currentChannelMatching = f.getCurrentUser().getOption("PREFERENCES_COLORMATCHING");

			/**
			 * @private
			 * @member {Array.<Number>} ViewServerId's of all the matched channels of the current document page, index determines to which channel of the reference document page it's matched 
			 */
			this._currentMatchedChannels = [];

			/**
			 * @private
			 * @member {Array.<Number>} ViewServerId's of all the matched channel of the reference document page
			 */
			this._referenceMatchedChannels = [];

			/**
			 * @private
			 * @member {Array.<Boolean>} is the channel of the current document page active (copies its definition to the matched channel) 
			 */
			this._isCurrentChannelActive = [];

			/**
			 * @private
			 * @member {Array.<Boolean>} collection of all the channel id's of the current document page that have to stay matched, cannot be unmatched by the user
			 */
			this._stayMatchedCurrentChannels = [];

			/**
			 * @private
			 * @member {Array.<Boolean>} collection of all the channel id's of the reference document page that have to stay matched, cannot be unmatched by the user
			 */
			this._stayMatchedReferenceChannels = [];

			/**
			 * @private
			 * @member {Boolean} is the setting of the channel matching persisted (saved in the user preferences) or uses it the default
			 */
			this._isChannelMatchingPersisted = this._currentChannelMatching !== GraphicsConstants.SMART_NAME_MATCHING;

			this.setChannels(channels); // set the channels
		};

		/**
		 * Get a channel of the current document page
		 * @param {Number} viewserverId - channel id 
		 * @returns {Channel}
		 */
		ChannelMatchingModel.prototype.getCurrentChannel = function (viewserverId) {
			return this._currentChannels[viewserverId];
		};

		/**
		 * Get a channel of the reference document page
		 * @param {Number} viewserverId
		 * @returns {Channel}
		 */
		ChannelMatchingModel.prototype.getReferenceChannel = function (viewserverId) {
			return this._referenceChannels[viewserverId];
		};

		/**
		 * get channels of the active document page that are not matched
		 * @returns {Array.<Channel>}
		 */
		ChannelMatchingModel.prototype.getCurrentChannels = function () {
			return _.reject(this._currentChannels, function (channel, key) {
				return _.contains(this._currentMatchedChannels, parseInt(key, 10));
			}, this);
		};

		/**
		 * get channels of the reference document page that are not matched
		 * @returns {Array.<Channel>}
		 */
		ChannelMatchingModel.prototype.getReferenceChannels = function () {
			return _.reject(this._referenceChannels, function (channel, key) {
				return _.contains(this._referenceMatchedChannels, parseInt(key, 10));
			}, this);
		};

		/**
		 * Has the current AND the reference document page unmatched channels
		 * @returns {Boolean}
		 */
		ChannelMatchingModel.prototype.hasUnmatchedChannels = function () {
			return this.getCurrentChannels().length !== 0 && this.getReferenceChannels().length !== 0;
		};

		/**
		 * get the matched channels of the active document page
		 * @returns {Array.<Channel>}
		 */
		ChannelMatchingModel.prototype.getMatchedCurrentChannels = function () {
			return _.map(this._currentMatchedChannels, _.bind(function (channelId) {
				return this._currentChannels[channelId];
			}, this));
		};

		/**
		 * get the matched channels of the reference document page
		 * @returns {Array.<Channel>}
		 */
		ChannelMatchingModel.prototype.getMatchedReferenceChannels = function () {
			return _.map(this._referenceMatchedChannels, _.bind(function (channelId) {
				return this._referenceChannels[channelId];
			}, this));
		};

		/**
		 * Returns the array of the active channels in the channel matching
		 * @returns {Array.<Boolean>}
		 */
		ChannelMatchingModel.prototype.getIsCurrentChannelActive = function () {
			return this._isCurrentChannelActive;
		};

		/**
		 * Set if the channel matching is persisted or not
		 * @param {Boolean} isPersisted
		 */
		ChannelMatchingModel.prototype.setIsChannelMatchingPersisted = function (isPersisted) {
			this._isChannelMatchingPersisted = isPersisted;
		};

		/**
		 * Returns true if the used channel matching is saved in the user preferences
		 * @returns {Boolean}
		 */
		ChannelMatchingModel.prototype.isChannelMatchingPersisted = function () {
			return this._isChannelMatchingPersisted;
		};

		/**
		 * Sets a channel active, it deactivates the channel it is matched to. Can set current/reference channels active.
		 * @param {Number} channelId
		 */
		ChannelMatchingModel.prototype.setChannelActive = function (channelId) {
			if (_.indexOf(this._currentMatchedChannels, channelId) !== -1) {
				this._isCurrentChannelActive[_.indexOf(this._currentMatchedChannels, channelId)] = true;
			} else if (_.indexOf(this._referenceMatchedChannels, channelId) !== -1) {
				this._isCurrentChannelActive[_.indexOf(this._referenceMatchedChannels, channelId)] = false;
			}
		};

		/**
		 * Returns the currently used channel matching.
		 * @returns {GraphicsConstants.ChannelMatching}
		 */
		ChannelMatchingModel.prototype.getChannelMatching = function () {
			return this._currentChannelMatching;
		};

		/**
		 * Sets the currently used channel matching.
		 * @param {GraphicsConstants.ChannelMatching} channelMatching
		 */
		ChannelMatchingModel.prototype.setChannelMatching = function (channelMatching) {
			this._currentChannelMatching = channelMatching;
		};

		/**
		 * Resets the channel matching.
		 */
		ChannelMatchingModel.prototype.resetChannelMatching = function () {
			this._currentMatchedChannels = [];
			this._referenceMatchedChannels = [];
			this._isCurrentChannelActive = [];
			this._stayMatchedCurrentChannels = [];
			this._stayMatchedReferenceChannels = [];
		};

		/**
		 * Set all the matched channels of the current document page. It first copies the stay matched channels to the current matched channels
		 * @param {Array.<String>} currentMatchedChannels
		 */
		ChannelMatchingModel.prototype.setMatchedCurrentChannels = function (currentMatchedChannels) {
			//make sure the channels that need to stay matched stay in the matched channels array, they are not in currentMatchedChannels due to JQuery UI
			this._currentMatchedChannels = this._stayMatchedCurrentChannels.slice(0);
			this._currentMatchedChannels = _.union(this._currentMatchedChannels, _.map(currentMatchedChannels, function (channel) {
				return parseInt(channel, 10);
			}));
		};

		/**
		 * Set all the matched channels of the reference document page. It first copies the stay matched channels to the reference matched channels
		 * @param {Array.<String>} referenceMatchedChannels
		 */
		ChannelMatchingModel.prototype.setMatchedReferenceChannels = function (referenceMatchedChannels) {
			this._referenceMatchedChannels = this._stayMatchedReferenceChannels.slice(0);
			this._referenceMatchedChannels = _.union(this._referenceMatchedChannels, _.map(referenceMatchedChannels, function (channel) {
				return parseInt(channel, 10);
			}));
		};

		/**
		 * Match a current and a reference channel.
		 * @param {Number} currentChannelId
		 * @param {Number} referenceChannelId
		 * @param {Boolean} isCurrentCopied - does the reference channel take the definition of the current channel
		 * @param {Boolean} stayMatched - does the channel need to stay matched
		 */
		ChannelMatchingModel.prototype.setChannelMatch = function (currentChannelId, referenceChannelId, isCurrentCopied, stayMatched) {
			if (isCurrentCopied === undefined) {
				isCurrentCopied = false;
			}
			if (_.contains(this._currentMatchedChannels, currentChannelId)) {
				var channelIndex = _.indexOf(this._currentMatchedChannels, currentChannelId);
				this._referenceMatchedChannels[channelIndex] = referenceChannelId;
				this._isCurrentChannelActive[channelIndex] = isCurrentCopied;
				if (stayMatched && !_.contains(this._stayMatchedCurrentChannels, currentChannelId)) {
					this._stayMatchedCurrentChannels.push(currentChannelId);
				}
			} else if (_.contains(this._referenceMatchedChannels, referenceChannelId)) {
				var channelIndex = _.indexOf(this._referenceMatchedChannels, referenceChannelId);
				this._currentMatchedChannels[channelIndex] = currentChannelId;
				this._isCurrentChannelActive[channelIndex] = isCurrentCopied;
				if (stayMatched && !_.contains(this._stayMatchedReferenceChannels, referenceChannelId)) {
					this._stayMatchedReferenceChannels.push(referenceChannelId);
				}
			} else {
				this._currentMatchedChannels.push(currentChannelId);
				this._referenceMatchedChannels.push(referenceChannelId);
				this._isCurrentChannelActive.push(isCurrentCopied);
				if (stayMatched) {
					if (!_.contains(this._stayMatchedCurrentChannels, currentChannelId)) {
						this._stayMatchedCurrentChannels.push(currentChannelId);
					}
					if (!_.contains(this._stayMatchedReferenceChannels, referenceChannelId)) {
						this._stayMatchedReferenceChannels.push(referenceChannelId);
					}
				}
			}

		};

		/**
		 * Make sure that channels that stay matched are on top of the list.
		 */
		ChannelMatchingModel.prototype.orderMatchedChannels = function () {
			var currentMatchedChannels = this._stayMatchedCurrentChannels.slice(0);
			var referenceMatchedChannels = this._stayMatchedReferenceChannels.slice(0);

			_.each(this._currentMatchedChannels, _.bind(function (currentChannel) {
				if (!_.contains(currentMatchedChannels, currentChannel)) {
					currentMatchedChannels.push(currentChannel);
				}
			}, this));
			_.each(this._referenceMatchedChannels, _.bind(function (referenceChannel) {
				if (!_.contains(referenceMatchedChannels, referenceChannel)) {
					referenceMatchedChannels.push(referenceChannel);
				}
			}, this));

			this._currentMatchedChannels = currentMatchedChannels;
			this._referenceMatchedChannels = referenceMatchedChannels;
		};

		/**
		 * Returns the channels of the current document page that have to stay matched
		 * @returns {Array.<Number>}
		 */
		ChannelMatchingModel.prototype.getStayMatchedCurrentChannels = function () {
			return this._stayMatchedCurrentChannels;
		};

		/**
		 * Returns the channels of the reference document page that have to stay matched
		 * @returns {Array.<Number>}
		 */
		ChannelMatchingModel.prototype.getStayMatchedReferenceChannels = function () {
			return this._stayMatchedReferenceChannels;
		};

		/**
		 * Sets the channels of the model. Removes all the previous channels.  
		 * @param {Array.<Channel>} channels
		 */
		ChannelMatchingModel.prototype.setChannels = function (channels) {
			this._referenceChannels = {};
			this._currentChannels = {};
			this._currentMatchedChannels = [];
			this._referenceMatchedChannels = [];
			this._isCurrentChannelActive = [];
			this._stayMatchedCurrentChannels = [];
			this._stayMatchedReferenceChannels = [];

			if (channels === undefined) {
				channels = [];
			}
			_.each(channels, _.bind(function (channel) {
				channel = channel.copy(); //make sure copied channel is used
				if (channel.getDiffMode() === GraphicsConstants.ChannelDiffMode.CURRENT || channel.getDiffMode() === GraphicsConstants.ChannelDiffMode.IN_CURRENT_AND_MATCHED_WITH_PREVIOUS) {
					this._currentChannels[channel.getViewserverID()] = channel;
				} else {
					this._referenceChannels[channel.getViewserverID()] = channel;
				}
			}, this));
		};

		return ChannelMatchingModel;
	});


define('tpl!web-viewer/graphics/ChannelMatchingPopup', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div class="channelMatchingSettings">\r\n	<h2>'+
((__t=(d.i18n("channelMatchingSettings")))==null?'':_.escape(__t))+
'</h2>\r\n	<div class="channelMatchingSetting"><input type="radio" name="channelMatching" value="0" ';
 if(d.channelMatching===0){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("MapChanAuto0")))==null?'':_.escape(__t))+
' </div>\r\n	<div class="channelMatchingSetting"><input type="radio" name="channelMatching" value="1" ';
 if(d.channelMatching===1){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("MapChanAuto1")))==null?'':_.escape(__t))+
' </div>\r\n	<div class="channelMatchingSetting"><input type="radio" name="channelMatching" value="2" ';
 if(d.channelMatching===2){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("MapChanAuto2")))==null?'':_.escape(__t))+
' </div>\r\n	\r\n	<div class="channelMatchingSetDefault"><input type="checkbox" name="saveChannelMatchingAsDefault" ';
if(d.isChannelMatchingPersisted){
__p+='checked';
}
__p+=' />'+
((__t=(d.i18n("MapChanAutoDefault")))==null?'':_.escape(__t))+
'</div>\r\n</div>\r\n\r\n<div class="channelMatchingTitles">\r\n	<div class="channelMatchingTitle">\r\n		'+
((__t=(d.i18n("matchedChannelsTitle")))==null?'':_.escape(__t))+
'\r\n	</div>\r\n</div>\r\n\r\n<div class="channelMatchingTables">\r\n\r\n<ul class="channelMatchingList sortableCurrent">\r\n	<li class="channelMatchingTitle">'+
((__t=(d.i18n("currentChannelsTitle")))==null?'':_.escape(__t))+
'</li>\r\n';
 _.each(d.currentChannels, function(channel){ 
__p+='\r\n	<li class="dragAndDropChannel" data-bind="'+
((__t=(channel.getViewserverID()))==null?'':_.escape(__t))+
'">\r\n		<span><i class="colorBox" style="background: #'+
((__t=(channel.getHexRGB() ))==null?'':_.escape(__t))+
';"></i></span> \r\n		<span class="channelName">'+
((__t=(channel.getName() ))==null?'':_.escape(__t))+
' </span>\r\n	</li>\r\n';
 }); 
__p+='\r\n</ul>\r\n\r\n<ul class="channelMatchingList sortableCurrentMatched">\r\n	<li class="channelMatchingTitle">'+
((__t=(d.i18n("CaptionCurrent")))==null?'':_.escape(__t))+
'</li>\r\n';
 _.each(d.matchedCurrentChannels, function(channel,key){ 
__p+='\r\n	<li class="dragAndDropChannel ';
if(d.isMatchedChannelClickable[key]){
__p+='clickable ';
if(d.currentMatchedChannelActive[key]){
__p+='matchedChannelActive';
}}
__p+=' ';
if(_.contains(d.stayMatchedCurrentChannels,channel.getViewserverID())){
__p+='stayMatched';
}else{
__p+='';
}
__p+='" data-bind="'+
((__t=(channel.getViewserverID()))==null?'':_.escape(__t))+
'">\r\n		<span><i class="colorBox" style="background: #'+
((__t=(channel.getHexRGB() ))==null?'':_.escape(__t))+
';"></i></span> \r\n		<span class="channelName">'+
((__t=(channel.getName() ))==null?'':_.escape(__t))+
' </span>\r\n	</li>\r\n';
 }); 
__p+='\r\n</ul>\r\n\r\n<ul class="channelMatchingList sortableReferenceMatched">\r\n	<li class="channelMatchingTitle">'+
((__t=(d.i18n("CaptionPrevious")))==null?'':_.escape(__t))+
'</li>\r\n';
 _.each(d.matchedReferenceChannels, function(channel,key){ 
__p+='\r\n	<li class="dragAndDropChannel ';
if(d.isMatchedChannelClickable[key]){
__p+='clickable ';
if(!d.currentMatchedChannelActive[key]){
__p+='matchedChannelActive';
}}
__p+=' ';
if(_.contains(d.stayMatchedReferenceChannels,channel.getViewserverID())){
__p+='stayMatched';
}else{
__p+='';
}
__p+='" data-bind="'+
((__t=(channel.getViewserverID()))==null?'':_.escape(__t))+
'">\r\n		<span><i class="colorBox" style="background: #'+
((__t=(channel.getHexRGB() ))==null?'':_.escape(__t))+
';"></i></span> \r\n		<span class="channelName">'+
((__t=(channel.getName() ))==null?'':_.escape(__t))+
' </span>\r\n	</li>\r\n';
 }); 
__p+='\r\n</ul>\r\n\r\n<ul class="channelMatchingList sortableReference">\r\n	<li class="channelMatchingTitle">'+
((__t=(d.i18n("referenceChannelsTitle")))==null?'':_.escape(__t))+
'</li>\r\n';
 _.each(d.referenceChannels, function(channel){ 
__p+='\r\n	<li class="dragAndDropChannel" data-bind="'+
((__t=(channel.getViewserverID()))==null?'':_.escape(__t))+
'">\r\n		<span><i class="colorBox" style="background: #'+
((__t=(channel.getHexRGB() ))==null?'':_.escape(__t))+
';"></i></span> \r\n		<span class="channelName">'+
((__t=(channel.getName() ))==null?'':_.escape(__t))+
'</span>\r\n	</li>\r\n';
 }); 
__p+='\r\n</ul>\r\n</div>\r\n<div class="channelMatchingDescription">'+
((__t=(d.i18n("explenationHightlightChannels")))==null?'':_.escape(__t))+
'</div>';
return __p;
}});
define('web-viewer/graphics/ChannelMatchingPopup',["web-viewer/core/PrivateChannel",
		"web-viewer/utils/ListenerUtil",
		"utils/ShapeUtils",
		"web-viewer/layout/component/ModalPopup",
		"Template",
		"tpl!web-viewer/graphics/ChannelMatchingPopup",
		"jquery",
		"underscore"
	],
	function (
		PrivateChannel,
		ListenerUtil,
		ShapeUtils,
		ModalPopup,
		Template,
		tpl,
		$,
		_) {
		"use strict";
		/**
		 *
		 * @exports ChannelMatchingPopup
		 * @memberof GraphicsModule
		 * @augments layout.component.ModelPopup
		 * @class View for the channel matching. A modal popup that allows channel matching with drag-and-drop operations.
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {String} id - Id of the popup. It is used in its private channel.
		 * @param {ChannelMatchingModel} channelMatchingModel
		 */
		var ChannelMatchingPopup = function (f, id, channelMatchingModel) {
			ModalPopup.call(this, f, id, f.getContext());
			PrivateChannel.installTo(this, f, ['channelMatchingPopup']);
			ListenerUtil.mixin(this, f);

			f.i18n.loadBundle("viewer", _.bind(function () {
				this.setHeader($("<span/>").addClass("popupTitle").html(f.i18n.t("MapChanTitle")));
				this.setOkBtnText(f.i18n.t("saveChannelMatching"));
			}, this));

			this.element.addClass("channelsMatchingPopup");

			/**
			 * @private
			 * @member {ChannelMatchingModel} - model used to construct the popup
			 */
			this._channelMatchingModel = channelMatchingModel;

			this.setOkAction(_.bind(function () { //sets the action when the okAction button is pressed
				this._emit("channelMatchingComplete");
				this.hide();
			}, this));

			/**
			 * @member {Template} template for the popup
			 */
			this.t = new Template(this.f.i18n, tpl);
			this.t.setContext(this._popupContent);

			//action for the radio buttons to change the channel matching, emits channelMatchingPopup.channelMatching
			this.t.bind("change", "[name=channelMatching]", function (source, event) {
				this._emit("channelMatching", source.value);
			}, this);

			//action for matched channels that are clickable, where the defintion of the reference channel doesn't match the defintion of the current channel
			this.t.bind("click", ".clickable", _.bind(function (source, event) {
				this._emit("matchedChannelClicked", parseInt(source.getAttribute("data-bind"), 10));
			}, this));

			//action for checkbox to persist the selected channel matching option to the user preferences
			this.t.bind("change", "[name=saveChannelMatchingAsDefault]", _.bind(function (source, event) {
				this._emit("persistDefaultChannelMatching", source.checked);
			}, this));

			this._registerListener("modelChanged", this.render, this, this._channelMatchingModel);

		};

		ChannelMatchingPopup.prototype = Object.create(ModalPopup.prototype);
		ChannelMatchingPopup.prototype.constructor = ChannelMatchingPopup;

		/**
		 * Renders the popup for channel matching, it's a throttled function
		 */
		ChannelMatchingPopup.prototype.render = _.throttle(function () {
			this.t.d.currentChannels = this._channelMatchingModel.getCurrentChannels();
			this.t.d.referenceChannels = this._channelMatchingModel.getReferenceChannels();
			this.t.d.matchedCurrentChannels = this._channelMatchingModel.getMatchedCurrentChannels();
			this.t.d.matchedReferenceChannels = this._channelMatchingModel.getMatchedReferenceChannels();
			this.t.d.channelMatching = this._channelMatchingModel.getChannelMatching();
			this.t.d.currentMatchedChannelActive = this._channelMatchingModel.getIsCurrentChannelActive();
			this.t.d.stayMatchedCurrentChannels = this._channelMatchingModel.getStayMatchedCurrentChannels();
			this.t.d.stayMatchedReferenceChannels = this._channelMatchingModel.getStayMatchedReferenceChannels();
			this.t.d.isChannelMatchingPersisted = this._channelMatchingModel.isChannelMatchingPersisted();

			this.t.d.isMatchedChannelClickable = _.map(this.t.d.matchedCurrentChannels, _.bind(function (channel, key) {
				return (this.t.d.matchedReferenceChannels[key] !== undefined && !channel.equals(this.t.d.matchedReferenceChannels[key]));
			}, this));

			this.t.render();

			//Make the channel lists sortable
			var current = this._popupContent.find(".sortableCurrentMatched").sortable({
				items: "li.dragAndDropChannel:not(.stayMatched)", //list-items with the class stayMatched are not possible to be drag-and-dropped
				connectWith: ".sortableCurrent", //connect the list with the current channels list, possible to drag-and-drop to each other
				receive: function (event, ui) {
					ui.item.addClass("clickable"); //make item clickable when entering the list of matched channels
				},
				remove: function (event, ui) {
					ui.item.removeClass("clickable"); //remove clickable when item leaves the list of matched channels
				},
				update: _.bind(function (event, ui) { //when drag-and-dropping ended
					var currentMatchedChannels = this._popupContent.find(".sortableCurrentMatched").sortable("toArray", {
						attribute: "data-bind"
					});
					var toMatchedList = false;
					if (ui.sender !== null) { // if sender is not null, the item was dropped in the matched list
						toMatchedList = true;
					}
					this.recalculateSize();
					this._emit("currentChannelMatch", ui.item.context.getAttribute("data-bind"), currentMatchedChannels, toMatchedList);
				}, this)
			});

			//same as the list for the matched current channels
			var reference = this._popupContent.find(".sortableReferenceMatched").sortable({
				items: "li.dragAndDropChannel:not(.stayMatched)",
				connectWith: ".sortableReference",
				receive: function (event, ui) {
					ui.item.addClass("clickable");
				},
				remove: function (event, ui) {
					ui.item.removeClass("clickable");
				},
				update: _.bind(function (event, ui) {
					var referenceMatchedChannels = this._popupContent.find(".sortableReferenceMatched").sortable("toArray", {
						attribute: "data-bind"
					});
					var toMatchedList = false;
					if (ui.sender !== null) {
						toMatchedList = true;
					}
					this.recalculateSize();
					this._emit("referenceChannelMatch", ui.item.context.getAttribute("data-bind"), referenceMatchedChannels, toMatchedList);
				}, this)
			});

			//list for the channels of the current document page
			this._popupContent.find(".sortableCurrent").sortable({
				items: "li.dragAndDropChannel",
				connectWith: current, //connect the list with the matched channels of the current document page
				stop: _.bind(this.render, this) //make sure the list itself is not sortable, only possible to move items to match list
			});

			//list for the channels of the reference document page
			this._popupContent.find(".sortableReference").sortable({
				items: "li.dragAndDropChannel",
				connectWith: reference,
				stop: _.bind(this.render, this)
			});


		}, 40);

		/**
		 * Shows the popup.
		 */
		ChannelMatchingPopup.prototype.show = function () {
			this.render();

			ModalPopup.prototype.show.call(this);
		};

		return ChannelMatchingPopup;
	});

define('web-viewer/graphics/ChannelMatchingController',["web-viewer/utils/ListenerUtil",
		"web-viewer/graphics/ChannelMatchingModel",
		"web-viewer/graphics/ChannelMatchingPopup",
		"web-viewer/graphics/GraphicsConstants",
		"jquery",
		"underscore"
	],
	function (
		ListenerUtil,
		ChannelMatchingModel,
		ChannelMatchingPopup,
		GraphicsConstants,
		$) {
		"use strict";
		/**
		 * 
		 * @exports ChannelMatchingController
		 * @memberof GraphicsModule
		 * @class Controller for the channel matching popup. Listens to events of the channelMatchingPopup and saves changed data to the channelMatchingModel.
		 * Implements the matching algorithm, channels can be matched by name and RGB. There are three levels of matching: same name and RGB definition, same name or
		 * smart name matching. When the matching is complete, the controller will persist the changes to the graphicsmodel.
		 * @constructor
		 * 
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {ChannelMatchingPopup} channelMatchingPopup - the view for the channel matching
		 * @param {ChannelMatchingModel} channelMatchingModel - the model for the channel matching
		 * @param {GraphicsModel} graphicsModel - used for persisting changes made in the channel matching model
		 */
		var ChannelMatchingController = function (f, channelMatchingPopup, channelMatchingModel, graphicsModel) {
			ListenerUtil.mixin(this, f);

			this.f = f;

			/**
			 * @member {ChannelMatchingPopup}
			 */
			this.channelMatchingPopup = channelMatchingPopup;

			/**
			 * @member {ChannelMatchingModel}
			 */
			this.channelMatchingModel = channelMatchingModel;

			/**
			 * @member {GraphicsModel}
			 */
			this._graphicsModel = graphicsModel;

			//Do initial matching (default or in user preferences saved matching)
			this._matchChannels(this.channelMatchingModel.getChannelMatching());

			//Register all the listeners to the channelMatchingPopup
			this._registerListener("currentChannelMatch", this._currentChannelMatchingChanged, this, this.channelMatchingPopup);
			this._registerListener("referenceChannelMatch", this._referenceChannelMatchingChanged, this, this.channelMatchingPopup);
			this._registerListener("channelMatching", this._matchChannels, this, this.channelMatchingPopup);
			this._registerListener("channelMatchingComplete", _.bind(function () {
				this._completeChannelMatching(true);
			}, this), this, this.channelMatchingPopup);
			this._registerListener("matchedChannelClicked", this._activateMatchedChannel, this, this.channelMatchingPopup);
			this._registerListener("persistDefaultChannelMatching", this._persistChannelMatching, this, this.channelMatchingPopup);
		};

		/**
		 * @private
		 * Called when the list of matched channels of the current document page is changed. Will reset the definition of the moved channel if it moves from
		 * the matched channels list to the unmatched channels list.
		 * 
		 * @param {String} movedChannelId - Id of the moved channel
		 * @param {Array.<String>} currentMatchedChannels - id list of the matched channels, id are strings 
		 * @param {Boolean} toMatchedList - is the channel moved to the list of matched channels
		 */
		ChannelMatchingController.prototype._currentChannelMatchingChanged = function (movedChannelId, currentMatchedChannels, toMatchedList) {
			this.channelMatchingModel.setMatchedCurrentChannels(currentMatchedChannels);
			if (!toMatchedList) {
				this.channelMatchingModel.getCurrentChannel(parseInt(movedChannelId, 10)).resetToDefault();
			}
			this.channelMatchingPopup.render();
		};

		/**
		 * @private
		 * Called when the list of matched channels of the reference document page is changed. Will reset the definition of the moved channel if it moves from
		 * the matched channels list to the unmatched channels list.
		 * 
		 * @param {String} movedChannelId - Id of the moved channel
		 * @param {Array<String>} referenceMatchedChannels - id list of the matched channels, id are strings 
		 * @param {Boolean} toMatchedList - is the channel moved to the list of matched channels
		 */
		ChannelMatchingController.prototype._referenceChannelMatchingChanged = function (movedChannelId, referenceMatchedChannels, toMatchedList) {
			this.channelMatchingModel.setMatchedReferenceChannels(referenceMatchedChannels);
			if (!toMatchedList) {
				this.channelMatchingModel.getReferenceChannel(parseInt(movedChannelId, 10)).resetToDefault();
			}
			this.channelMatchingPopup.render();
		};

		/**
		 * @private
		 * Saves the channel matching to the user preferences. Reset these preferences to the default if no parameter or parameters is false.
		 * 
		 * @param {Boolean} persistChannelMatching - does the channel matching need to be persisted
		 */
		ChannelMatchingController.prototype._persistChannelMatching = function (persistChannelMatching) {
			if (persistChannelMatching) {
				this.f.getCurrentUser().setOption("PREFERENCES_COLORMATCHING", this.channelMatchingModel.getChannelMatching());
			} else {
				this.f.resetOption("PREFERENCES_COLORMATCHING");
			}
			this.channelMatchingModel.setIsChannelMatchingPersisted(persistChannelMatching);
		};

		/**
		 * Set the channels for the channelModel, it is possible to do initial matching for the new channels for the channelMatchingModel
		 * @param {Array.<Channel>} channels
		 */
		ChannelMatchingController.prototype.setChannels = function (channels) {
			this.channelMatchingModel.setChannels(channels);
			this._matchChannels(this.channelMatchingModel.getChannelMatching());
			this._completeChannelMatching(false);
		};

		/**
		 * @private
		 * Matches the channels, the parameter determines which matching is used.
		 * 
		 * @param {String} channelMatchKey
		 */
		ChannelMatchingController.prototype._matchChannels = function (channelMatchKey) {
			this.channelMatchingModel.resetChannelMatching(); //reset all the channel matching before trying to match again
			channelMatchKey = parseInt(channelMatchKey, 10); //convert the string to GraphicsConstants.ChannelMatching
			this.channelMatchingModel.setChannelMatching(channelMatchKey);
			var currentChannels = this.channelMatchingModel.getCurrentChannels();
			var referenceChannels = this.channelMatchingModel.getReferenceChannels();


			_.each(currentChannels, function (currentChannel) { //all the current channels
				var match = undefined;
				if (channelMatchKey === GraphicsConstants.ChannelMatching.SMART_NAME_MATCHING) {
					// we are matching with smart names, first try without smart names (this is to prevent following situation)
					/**
					 * @example
					 * 
					 * File 1:  C, M, Y, K, PROCESS BLACK
					 * File 2:  C, M, Y, PROCESS BLACK, K
					 * 
					 * if we just search using smart name matching, process black will be mapped on K and the other way arround... but we want to map k on k and process black on process black
					 */
					match = _.find(referenceChannels, function (referenceChannel) {
						return this._canChannelsBeMatched(currentChannel, referenceChannel, GraphicsConstants.ChannelMatching.NAME_MATCHING);
					}, this);
				}

				if (match === undefined) {
					// either we are not in SMART_NAME_MATCHING, or nothing was found using the NAME_MATCHING (in this case we try again with SMAR_NAME_MATCHING)
					match = _.find(referenceChannels, function (referenceChannel) {
						return this._canChannelsBeMatched(currentChannel, referenceChannel, channelMatchKey);
					}, this);
				}

				if (match !== undefined) {
					// there was a match
					var isStayMatched = this._canChannelsBeMatched(currentChannel, match, channelMatchKey, true);
					this.channelMatchingModel.setChannelMatch(currentChannel.getViewserverID(), match.getViewserverID(), true, isStayMatched);
				}
			}, this);


			this.channelMatchingModel.orderMatchedChannels(); //to make sure the stay matched channels are on top
			this.channelMatchingPopup.render();
			this._persistChannelMatching(this.channelMatchingModel.isChannelMatchingPersisted());
		};


		/**
		 * @private
		 * Checks if the two channels can be matched to each other depending on the channelMatchKey
		 * 
		 * @param {Channel} currentChannel - channel of the current document page
		 * @param {Channel} referenceChannel - channel of the reference document page
		 * @param {GraphicsConstants.ChannelMatching} channelMatchKey - key for the channel matching
		 * @param {Boolean} useOriginalDef - determines whether the original definition of the channels is used or the changed definition
		 * 
		 * @returns {Boolean}
		 */
		ChannelMatchingController.prototype._canChannelsBeMatched = function (currentChannel, referenceChannel, channelMatchKey, useOriginalDef) {
			var currentChannelName = currentChannel.getNameWithoutSuffix();
			var referenceChannelName = referenceChannel.getNameWithoutSuffix();
			if (useOriginalDef) {
				currentChannelName = currentChannel.getOriginalName();
				referenceChannelName = referenceChannel.getOriginalName();
			}
			if (channelMatchKey === GraphicsConstants.ChannelMatching.SMART_NAME_MATCHING) { //smart name matching
				//first remove the process, pantone key words from the name
				currentChannelName = currentChannelName.replace(/process\s*/i, "").replace(/pantone\s*/i, "");
				referenceChannelName = referenceChannelName.replace(/process\s*/i, "").replace(/pantone\s*/i, "");
				//see if the strings match:
				if (currentChannelName.toLowerCase() === referenceChannelName.toLowerCase()) {
					return true;
				}
				//match if names are similar: cyan and C, magenta and M, yellow and Y, black and K
				currentChannelName = currentChannelName.replace(/cyan/i, "c").replace(/magenta/i, "m").replace(/yellow/i, "y").replace(/black/i, "k");
				referenceChannelName = referenceChannelName.replace(/cyan/i, "c").replace(/magenta/i, "m").replace(/yellow/i, "y").replace(/black/i, "k");
				if (currentChannelName.toLowerCase() === referenceChannelName.toLowerCase()) {
					return true;
				} else {
					return false;
				}
			} else if (channelMatchKey === GraphicsConstants.ChannelMatching.NAME_MATCHING) { //match channels (ignore RGB differences)
				if (currentChannelName.toLowerCase() === referenceChannelName.toLowerCase()) {
					return true;
				} else {
					return false;
				}
			} else if (channelMatchKey === GraphicsConstants.ChannelMatching.IDENTICAL_MATCHING) { //only match if identical (no RGB differences)
				if (currentChannelName.toLowerCase() === referenceChannelName.toLowerCase() && useOriginalDef && currentChannel.equalsOriginalDef(referenceChannel)) {
					return true;
				} else if (currentChannelName.toLowerCase() === referenceChannelName.toLowerCase() && !useOriginalDef && currentChannel.equals(referenceChannel)) {
					return true;
				} else {
					return false;
				}
			}
		};

		/**
		 * @private
		 * Persist the channel matching to the graphicsmodel. Changes the definition of matched channels.
		 * 
		 * @param {Boolean} resetChannelDefinition
		 * 
		 */
		ChannelMatchingController.prototype._completeChannelMatching = function (resetChannelDefinition) {
			this._graphicsModel.resetChannelMatching();
			var currentMatchedChannels = this.channelMatchingModel.getMatchedCurrentChannels();
			var referenceMatchedChannels = this.channelMatchingModel.getMatchedReferenceChannels();
			var copyCurrentMatchedChannels = this.channelMatchingModel.getIsCurrentChannelActive();

			_.each(currentMatchedChannels, _.bind(function (currentMatchedChannel, index) { //run over all matched channels
				var referenceMatchedChannel = referenceMatchedChannels[index];
				var copyCurrentMatchedChannel = copyCurrentMatchedChannels[index];
				if (referenceMatchedChannel !== undefined) { //check if current channel is matched with a reference channel, if not the case dont match the channel
					this._graphicsModel.setChannelMatching(currentMatchedChannel.getID(), referenceMatchedChannel.getID());
					if (copyCurrentMatchedChannel) { //which of the two channels is active and copies its definition to the other channel
						if (!currentMatchedChannel.isChanged()) { //if the channel is not changed (definition was reset) make sure to use the original definition of the channel
							this._graphicsModel.getChannel(currentMatchedChannel.getID()).resetToDefault();
						}
						this._graphicsModel.changeChannelDef(referenceMatchedChannel.getID(), currentMatchedChannel.getID(), false);
					} else {
						if (!referenceMatchedChannel.isChanged()) { //if the channel is not changed (definition was reset) make sure to use the original definition of the channel
							this._graphicsModel.getChannel(referenceMatchedChannel.getID()).resetToDefault();
						}
						this._graphicsModel.changeChannelDef(currentMatchedChannel.getID(), referenceMatchedChannel.getID(), false);
					}
				} else {
					this._graphicsModel.changeChannelDef(currentMatchedChannel.getID(), currentMatchedChannel.getID(), true);
				}
			}, this));

			if (resetChannelDefinition) {
				//make sure reseted channels are saved to the graphicsModel
				var currentChannels = this.channelMatchingModel.getCurrentChannels();
				var referenceChannels = this.channelMatchingModel.getReferenceChannels();
				_.each(currentChannels, _.bind(function (channel) {
					this._graphicsModel.resetChannel(channel.getID());
				}, this));
				_.each(referenceChannels, _.bind(function (channel) {
					this._graphicsModel.resetChannel(channel.getID());
				}, this));
			}
		};

		/**
		 * @private
		 * Changes the active channel, the active channel is the channel which defintion will be copied to the matched channels.
		 * @param {Number} channelId - identifier for the channel that becomes active
		 */
		ChannelMatchingController.prototype._activateMatchedChannel = function (channelId) {
			this.channelMatchingModel.setChannelActive(channelId);
			this.channelMatchingPopup.render();
		};

		return ChannelMatchingController;
	});

/**
 *  @namespace Graphics
 */
define('web-viewer/graphics/GraphicsModule',["web-viewer/core/AbstractModule",
		"web-viewer/graphics/GraphicsModel",
		"web-viewer/graphics/GraphicsDataExtensions",
		"web-viewer/graphics/ChannelsWidget",
		"web-viewer/graphics/GraphicsLayerModel",
		"web-viewer/graphics/GraphicsViewportWidgetController",
		"web-viewer/graphics/ChannelMatchingModel",
		"web-viewer/graphics/ChannelMatchingPopup",
		"web-viewer/graphics/ChannelMatchingController",
		"web-viewer/layout/component/ModalPopup",
		"web-viewer/graphics/GraphicsConstants",
		"underscore",
		"utils/ShapeUtils"
	],
	function (AbstractModule,
		GraphicsModel,
		GraphicsDataExtensions,
		ChannelsWidget,
		GraphicsLayerModel,
		GraphicsViewportWidgetController,
		ChannelMatchingModel,
		ChannelMatchingPopup,
		ChannelMatchingController,
		ModalPopup,
		GraphicsConstants,
		_,
		ShapeUtils) {
		"use strict";

		/**
		 * @exports GraphicsModule
		 * @memberof GraphicsModule
		 * @class the graphics module adds the graphics layer if applicable
		 * @constructor
		 * @augments Core.AbstractModule
		 *
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {Boolean} silenced - if true, the module shouldn't publish tools/widgets/toolbarcomponents
		 */
		var GraphicsModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "graphics");

			// Inject graphics functionality in data objects
			GraphicsDataExtensions(f);

			this.graphicsViewportWidgetController = null;
			this.graphicsModel = null;
			this.graphicsLayerModel = null;
			this.channelsWidget = null;

			//channelmatching
			this.channelMatchingPopup = null;
			this.channelMatchingModel = null;
			this.channelMatchingController = null;
			this.isChannelMatchingActive = false;
			this.nonMatchedChannelsPopup = null;

			this.isGraphicsActive = false;

			this._waitingForParameterUpdate = false;

			this.f.on("intercept.fragmentURL", _.bind(function (context) {
				if (this.graphicsModel) {
					this.graphicsModel.interceptFragmentURL(context);
				}
				if (this.isChannelMatchingActive && this.channelMatchingModel !== null) {
					context.params.colormatching = this.channelMatchingModel.getChannelMatching();
				}
				if (this.f.getProperty("viewquality")) {
					context.params.vq = this.f.getProperty("viewquality");
				}

				// send trimbox information	
				this._setTrimboxInformation(context);
			}, this));

			this.f.on("module.compare.changed.diffMode", _.bind(function (oldMode, newMode) {
				if (this.graphicsModel) {
					this.graphicsModel.setCompareMode(newMode);
				}
			}, this));
			// listen to page changes or parameter changes
			this.f.getDocumentController().whenParametersAvailable(function () {
				this._whenParametersAvailable();
				this.f.getDocumentController().whenActiveOrDifferencePageChanged(this._whenActiveOrDifferencePageChanged, this);
			}, this);
		};

		// inherit
		GraphicsModule.prototype = Object.create(AbstractModule.prototype);
		GraphicsModule.prototype.constructor = GraphicsModule;


		/**
		 * When the active or difference page has changed, wait for the parameters to become available
		 * @private
		 */
		GraphicsModule.prototype._whenActiveOrDifferencePageChanged = function () {
			var docController = this.f.getDocumentController();
			if (!this._waitingForParameterUpdate) {
				if (docController.isParameterUpdateOngoing()) {
					this.f.getDocumentController().once("parameters.afterEndUpdate", _.bind(this._whenParametersAvailable, this));
					this._waitingForParameterUpdate = true;
				} else {
					// parameters are available already
					this._whenParametersAvailable();
				}

			}
		};

		/**
		 * Execute when active document page has changed or new document page is loaded. Initializes all components of the Graphics module
		 * @private
		 */
		GraphicsModule.prototype._whenParametersAvailable = function () {
			this._waitingForParameterUpdate = false;
			var channels;
			var activePage = this.f.getActivePage();
			var differencePage = this.f.getDocumentController().getDifferencePage();

			if (!activePage) {
				return;
			}

			//the active document page has changed, create the graphics model and update the widget		
			if (activePage.hasGraphics() || activePage.isBlankPage()) {
				//create models if not existing
				if (this.graphicsModel == null) {
					this.graphicsModel = new GraphicsModel(this.f, [this.getPrivateChannelName(), "model"], activePage, differencePage);
					this.graphicsLayerModel = new GraphicsLayerModel(this.f, "Graphics", this.graphicsModel);
				} else {
					// if the active page has changed, remove all views
					if ((this.graphicsModel.getPage() != activePage || !this.isGraphicsActive) && this.graphicsViewportWidgetController) {
						this.graphicsViewportWidgetController.destructViews();
					}
					// update the graphics model
					this.graphicsModel.setPage(activePage, differencePage);
					if (this.isChannelMatchingActive) {
						channels = this.graphicsModel.getAllChannels();
						this.channelMatchingController.setChannels(channels);
						if (activePage !== undefined && differencePage !== undefined && this.nonMatchedChannelsPopup === null) {
							//only try opening the popup when the popup is null or is was set to null 
							this.showChannelMatchingPopupIfNeeded();
						}
					}
				}

				var compareModule = this.f.getModule("compare");
				if (compareModule) {
					this.graphicsModel.setCompareMode(compareModule.getDiffMode());
				}

				if (!this.isGraphicsActive) { //add the layer if there weren't graphics active before
					this.f.getModule("viewport").addLayer("Graphics", this.graphicsLayerModel, "Main");
					this.isGraphicsActive = true;
				}

				if (this.graphicsViewportWidgetController == null) {
					this.graphicsViewportWidgetController = new GraphicsViewportWidgetController(this.f, this.graphicsModel);
					this.graphicsViewportWidgetController.updateTiles();
				}

				// it is an image file, it makes sense to display the channels widget
				if (!this.channelsWidget) {
					// the channels widget was never intialized, create it
					this._publishWidget(ChannelsWidget.widgetName, _.bind(function (layoutType) {
						this.channelsWidget = new ChannelsWidget(this.f, this.graphicsModel);
						return this.channelsWidget;
					}, this));
				} else {
					//widget already existed, make sure it is visible
					this.channelsWidget.setEnabled(true);
				}
			} else {
				this.f.getModule("viewport").removeLayer("Graphics", "Main");
				//remove all the views when there is no graphics on the page
				if (this.graphicsModel.getPage() != activePage && this.graphicsViewportWidgetController) {
					this.graphicsViewportWidgetController.destructViews();
				}
				// update the graphics model, no graphics files are active so both pages are undefined
				this.graphicsModel.setPage(undefined, undefined);
				this.isGraphicsActive = false;
				// not an image file, hide the channels widget if already available			
				if (this.channelsWidget) {
					this.channelsWidget.setEnabled(false);
				}
				this.graphicsModel.setChannelViewMode(GraphicsConstants.ChannelViewMode.COMPOSITE_VIEW);
			}

			// channel matching
			if (!activePage.isReaderSpread()) {
				if (activePage !== undefined && differencePage !== undefined && activePage.hasGraphics() && differencePage.hasGraphics()) {
					// we are comparing two files... initialize channel matching correctly
					this.isChannelMatchingActive = true;
					channels = this.graphicsModel.getAllChannels();

					if (this.channelMatchingModel === null) {
						this.channelMatchingModel = new ChannelMatchingModel(this.f, channels);
					}
					if (this.channelMatchingPopup === null) {
						this.channelMatchingPopup = new ChannelMatchingPopup(this.f, "channelMatchingPopup", this.channelMatchingModel);
					}
					if (this.channelMatchingController === null) {
						this.channelMatchingController = new ChannelMatchingController(this.f, this.channelMatchingPopup, this.channelMatchingModel, this.graphicsModel);
					}
					this.channelMatchingController.setChannels(channels);

					this.showChannelMatchingPopupIfNeeded();

					if (this.channelsWidget) {
						this.channelsWidget.showChannelMatchingBtn(_.bind(this.showChannelMatchingPopup, this));
					}
				} else if (this.isChannelMatchingActive) {
					// channelmatching was still active.. clean it up
					this.isChannelMatchingActive = false;
					if (this.nonMatchedChannelsPopup) {
						// if the nonMatchedChannelsPopup exist, we want to clean it up completely
						this.nonMatchedChannelsPopup.remove();
					}
					this.nonMatchedChannelsPopup = null;
					if (this.channelsWidget) {
						//hide the channel matching button when not both current and reference document pages are selected
						this.channelsWidget.hideChannelMatchingBtn();
					}
				}
			} else {
				if (this.channelsWidget) {
					// in reader spread, hide the channel matching btn
					this.channelsWidget.hideChannelMatchingBtn();
				}
			}

		};



		/**
		 * Show the channel matching popup if it is needed (there are unmatched channels)
		 */
		GraphicsModule.prototype.showChannelMatchingPopupIfNeeded = function () {
			//when not all the channels are matched, show popup asking to do channel matching if not on mobile
			if (this.channelMatchingModel !== undefined && this.channelMatchingModel.hasUnmatchedChannels() && !this.f.isMobile()) {
				if (this.nonMatchedChannelsPopup === null) {
					this.nonMatchedChannelsPopup = new ModalPopup(this.f, "nonMatchingChannelsPopup", this.f.getContext());
				}
				this.f.i18n.loadBundle("viewer", _.bind(function () {
					this.nonMatchedChannelsPopup.setContent(($("<div>").addClass("nonMatchingChannelsDialog").html(this.f.i18n.t("DocumentListWarning"))));
					this.nonMatchedChannelsPopup.setOkBtnText(this.f.i18n.t("startChannelMatching"));
					this.nonMatchedChannelsPopup.setHeader($("<span/>").addClass("popupTitle").html(this.f.i18n.t("MapChanTitle")));
				}, this));

				this.nonMatchedChannelsPopup.setOkAction(_.bind(function () {
					this.channelMatchingPopup.show();
					this.nonMatchedChannelsPopup.hide();
				}, this));

				this.nonMatchedChannelsPopup.show();
				this.nonMatchedChannelsPopup.recalculateSize(820);
			}
		};

		/**
		 * Get the graphics module for the currently active page
		 * @returns {Graphics.GraphicsModel}
		 *
		 */
		GraphicsModule.prototype.getGraphicsModel = function () {
			return this.graphicsModel;
		};

		/**
		 * Get the selection mask for a certain diffMode
		 * @param {CompareConstants.difMode} diffMode - the mode for which the selection mask should be calculated
		 * @returns {Number}
		 */
		GraphicsModule.prototype.getChannelSelectionMaskForDiffMode = function (diffMode) {
			if (this.graphicsModel !== null) {
				return this.graphicsModel.getChannelSelectionMaskForDiffMode(diffMode);
			}
		};

		/**
		 * Shows the popup for channel matching
		 */
		GraphicsModule.prototype.showChannelMatchingPopup = function () {
			var channels = this.graphicsModel.getAllChannels();
			this.channelMatchingController.setChannels(channels);
			this.channelMatchingPopup.show();
		};

		/**
		 * @override
		 */
		GraphicsModule.prototype.getState = function () {
			if (this.graphicsModel !== null) {
				var channelsState = {};
				_.each(this.graphicsModel.getAllChannels(), function (channel) {
					var channelState = {
						enabled: channel.isEnabled(),
						displayedRgb: channel.getRGB(),
						isOpaque: channel.isChannelOpaque()
					};
					channelsState[channel.getID()] = channelState;
				}, this);

				var state = {
					// Graphics model
					graphicsModel: {
						channelViewMode: this.graphicsModel.getChannelViewMode(),
						channelKeysOrder: this.graphicsModel.getChannelKeysOrder(),
						singleChannelMode: this.graphicsModel.getSingleChannelMode(),
						channels: channelsState
					}
				};

				if (this.channelMatchingModel !== null) {
					// Channel Matching model
					state.channelMatchingModel = {
						channelMatching: this.channelMatchingModel.getChannelMatching()
					};
				}
				return state;
			}
		};

		/**
		 * @override
		 * @param state
		 */
		GraphicsModule.prototype.setState = function (state) {
			this.f.whenModuleAvailable("graphics", function () {
				if (this.graphicsModel === null) {
					this.graphicsModel = new GraphicsModel(this.f, [this.getPrivateChannelName(), "model"], this.f.getDocumentController().getActivePage(), this.f.getDocumentController().getDifferencePage());
					this.graphicsLayerModel = new GraphicsLayerModel(this.f, "Graphics", this.graphicsModel);
				}
				// This module is ready
				if (state !== undefined) {
					// Graphics model
					var graphicsModelState = state.graphicsModel;
					if (graphicsModelState !== undefined) {
						this.graphicsModel.setChannelViewMode(graphicsModelState.channelViewMode);
						this.graphicsModel.setChannelOrder(graphicsModelState.channelKeysOrder);
						this.graphicsModel.setSingleChannelMode(graphicsModelState.singleChannelMode);

						// Channels
						var channelsState = graphicsModelState.channels;
						_.each(this.graphicsModel.getAllChannels(), function (channel) {
							var channelID = channel.getID();
							var channelState = channelsState[channelID];
							if (channelState !== undefined) {
								this.graphicsModel.toggleChannel(channelID, channelState.enabled);
								this.graphicsModel.setChannelOpaque(channelID, channelState.isOpaque);
								this.graphicsModel.changeChannelRgb(channelID, channelState.displayedRgb);
							}
						}, this);
					}
					// Channel matching model
					var channelMatchingModelState = state.channelMatchingModel;
					if (channelMatchingModelState !== null && channelMatchingModelState !== undefined) {
						if (this.channelMatchingModel === null) {
							this.channelMatchingModel = new ChannelMatchingModel(this.f, undefined);
						}
						this.channelMatchingModel.setChannelMatching(channelMatchingModelState.channelMatching);
					}
				}
			}, this);
		};

		/**
		 * Get the enabled status of all channels
		 * @returns {Object} with key-value pairs (key=channelID, value=enabled)
		 */
		GraphicsModule.prototype.getChannelSelection = function () {
			var channelSelection = {};
			_.each(this.graphicsModel.getAllChannels(), function (channel) {
				channelSelection[channel.getID()] = channel.isEnabled();
			});

			return channelSelection;
		};

		/**
		 * Set the channel selection (restore a state retrieved from getChannelSelection())
		 * @param {Object} channelSelection -  with key-value pairs with the enabled state of the channels (key=channelID, value=enabled)
		 */
		GraphicsModule.prototype.setChannelSelection = function (channelSelection) {
			_.each(channelSelection, function (enabled, ID) {
				this.graphicsModel.toggleChannel(ID, enabled);
			}, this);
		};

		/**
		 * Set the channel view mode
		 * @param {GraphicsConstants.ChannelViewMode} channelViewMode - The new channel view mode
		 */
		GraphicsModule.prototype.setChannelViewMode = function (mode) {
			this.graphicsModel.setChannelViewMode(mode);
		};

		/**
		 * Get the channel view mode
		 * @returns {GraphicsConstants.ChannelViewMode} The current channel view mode
		 */
		GraphicsModule.prototype.getChannelViewMode = function () {
			return this.graphicsModel.getChannelViewMode();
		};

		/**
		 * Set if the trimbox needs to be shown and which mode is used
		 * @param {Boolean} showTrimbox
		 * @param {ViewControlsConstants.trimboxMode} trimboxMode
		 */
		GraphicsModule.prototype.setShowTrimbox = function (showTrimbox, trimboxMode) {
			if (this.graphicsModel) {
				this.graphicsModel.setShowTrimbox(showTrimbox, trimboxMode);
			}
		};

		/**
		 * Set the correct trimbox information in the context
		 * @param {Object} c
		 */
		GraphicsModule.prototype._setTrimboxInformation = function (c) {
			var diffPage = this.f.getDocumentController().getDifferencePage();
			var currentPage = this.f.getDocumentController().getActivePage();

			var currentPageMargin = currentPage.getMargins();
			var currentPageTrimboxSize = currentPage.getTrimboxSize(); // NOTE: we use getTrimboxSize() --> the ViewServer expects the original dimensions of trimbox, not the rotated version

			c.params.ftbtlx = currentPageMargin.left;
			c.params.ftbtly = currentPageMargin.top;
			c.params.ftbbrx = currentPageTrimboxSize.width + currentPageMargin.left;
			c.params.ftbbry = currentPageTrimboxSize.height + currentPageMargin.top;

			if (diffPage && !diffPage.isBlankPage()) {
				var diffPageMargin = diffPage.getMargins();
				var diffPageTrimboxSize = diffPage.getTrimboxSize(); // NOTE: we use getTrimboxSize() --> the ViewServer expects the original dimensions of trimbox, not the rotated version

				c.params.diff = diffPage.get("file");
				c.params.dtbtlx = diffPageMargin.left;
				c.params.dtbtly = diffPageMargin.top;
				c.params.dtbbrx = diffPageTrimboxSize.width + diffPageMargin.left;
				c.params.dtbbry = diffPageTrimboxSize.height + diffPageMargin.top;
			}
		};

		return GraphicsModule;
	});

