define(["domReady!",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/layout/LayoutModel",
		"web-viewer/layout/LayoutConstants",
		"utils/DomUtils",
		"underscore",
		"jquery"
	],
	function (
		doc,
		ListenerUtil,
		LayoutModel,
		LayoutConstants,
		DomUtils,
		_,
		$
	) {
		"use strict";

		/**
		 * @class
		 * The LayoutManager is responsible for the layout of the application.
		 *
		 * The LayoutManager is the controller for the layout, it adds widgets, toolbarcomponents to the LayoutModel and lets the layout re-render.
		 *
		 * @exports LayoutManager
		 * @memberof Layout
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {JQueryObj} context - the container in which the viewer application should be drawn
		 * @param {LoadinginIndicator} loadingIndicator - the loading indicator, which was created before all requests started
		 * @param {JQueryObj} initialLoader - the JQuery reference to the element indicating the initial loader (needed to safely remove it without emptying the context)
		 * @param {Function} whenInitialized - callback called once the layout manager is ready to work
		 */

		var LayoutManager = function (f, context, loadingIndicator, initialLoader, whenInitialized) {
			ListenerUtil.mixin(this, f);

			this.f = f;

			/**
			 * Layout model saves the added layoutItems with a reference to the those items (widgets, toolbarcomponents,...)
			 */
			this._layoutModel = new LayoutModel(this.f);

			this._registerListener("layout.loadCSS", this.loadCSS, this);

			/**
			 * Active layout, is set during initialization of the viewer
			 */
			this._layout = null;

			/**
			 * The type of the layout
			 */
			this._layoutType = this.f.getLayoutType();


			var state = this._getLayoutStatePreference();

			// which layout should we load?
			var layoutImplementation = "";
			// which state should be injected on initialization?
			var layoutState = null;


			if (this._layoutType === LayoutConstants.LayoutType.MOBILE) {
				layoutImplementation = "web-viewer/layout/mobile/MobileLayout";
				layoutState = state.layout[LayoutConstants.LayoutType.MOBILE];

			} else if (this._layoutType === LayoutConstants.LayoutType.FLEX) {
				layoutImplementation = "web-viewer/layout/flex/FlexLayout";
				layoutState = state.layout[LayoutConstants.LayoutType.FLEX];
			} else if(this._layoutType === LayoutConstants.LayoutType.MINIMAL){
                layoutImplementation = "web-viewer/layout/minimal/MinimalLayout";
				layoutState = state.layout[LayoutConstants.LayoutType.MINIMAL];
            } else {
				// default to DESKTOP layout
				layoutImplementation = "web-viewer/layout/desktop/DesktopLayout";
				layoutState = state.layout[LayoutConstants.LayoutType.DESKTOP];
			}

			// Temp array used to save widgets that get added before the layout has been loaded
			var widgetsAdded = [],
				toolbarComponentsAdded = [];

			// Add listeners for widgets and toolbarComponents and add them to the temporary array to load them when the layout has been loaded
			var widgetListenerObject = this._registerListener("layout.addWidget", function (name, factory) {
				widgetsAdded.push({
					name: name,
					factory: factory
				});
			});
			var toolbarComponentListenerObject = this._registerListener("layout.addToolbarComponent", function (name, factory) {
				toolbarComponentsAdded.push({
					name: name,
					factory: factory
				});
			});


			// dynamically load the correct layout implementation
			requirejs([layoutImplementation], _.bind(function (Layout) {

				this._layout = new Layout(this.f, this._layoutModel, context, layoutState, loadingIndicator, initialLoader);

				// Add all the widgets and toolbarComponents to the layoutModel that were added prior to layout load
				_.each(widgetsAdded, function (widget) {
					if (!this._layoutModel.hasItem(widget.name)) {
						this._layoutModel.addWidget(widget.name, widget.factory);
					}
				}, this);
				_.each(toolbarComponentsAdded, function (widget) {
					if (!this._layoutModel.hasItem(widget.name)) {
						this._layoutModel.addToolbarComponent(widget.name, widget.factory);
					}
				}, this);

				// Remove the listeners
				this._unregisterListener(widgetListenerObject);
				this._unregisterListener(toolbarComponentListenerObject);


				//Register a listener to when widget needs to be added to the layout
				this._registerListener("layout.addWidget", function (name, factory) {
					//check if the widget wasn't added before, it needs to be unique
					if (!this._layoutModel.hasItem(name)) {
						//add the new widget to the layoutModel
						this._layoutModel.addWidget(name, factory);
					}
				}, this, this.f);

				//register a listener for when a toolbarcomponent needs to be added to the layout
				this._registerListener("layout.addToolbarComponent", function (name, factory) {
					//check if the widget wasn't added before, it needs to be unique
					if (!this._layoutModel.hasItem(name)) {
						//add the new widget to the layoutModel
						this._layoutModel.addToolbarComponent(name, factory);
					}
				}, this, this.f);

				this._registerListener("layout.changed", this._persistLayout, this, this._layout);
				this._registerListener("layout.resetToDefault", _.partial(this._persistLayout, true), this, this.f);

				// notify that the layout is initialized
				if (whenInitialized) {
					whenInitialized(this);
				}
			}, this));

		};

		/**
		 * Dynamically load CSS file
		 *
		 * @param {String} filename - path to css from requireJS base
		 * @param {Function} onload - callback function when css is loaded
		 */
		LayoutManager.prototype.loadCSS = function (filename, onload) {
			DomUtils.loadCss(filename, onload);
		};

		/**
		 * Persist the state of the layout. It will check if the layout needs to be persisted.
		 */
		LayoutManager.prototype._persistLayout = _.debounce(function (reset) {
			if (this._layout.isLayoutStatePersisted()) {
				// we don't wanna overwrite the other states
				var state = this._getLayoutStatePreference();

				if (reset) {
					state.layout[this._layoutType] = undefined;
				} else {
					state.layout[this._layoutType] = this._layout.getLayoutState();
				}
				this.f.setOption("LAYOUT_STATE", JSON.stringify(state));
			}
		}, 100);


		/**
		 * Get the currently saved layout state
		 * @returns {Object} layout state object
		 */
		LayoutManager.prototype._getLayoutStatePreference = function () {
			//get the layoutstate from the user options
			var state = this.f.getOption("LAYOUT_STATE");
			if (state !== undefined && state !== "") {
				try {
					state = $.parseJSON(state); //parse the JSON string to an Object

					// BACKWARDS COMPATIBILITY CODE
					if (!state.layout) {
						// before the LAYOUT_STATE just contained the state information for the DESKTOP layout
						var desktopState = state;
						state = {
							layout: {}
						};
						state.layout[LayoutConstants.LayoutType.DESKTOP] = desktopState;
					}
				} catch (err) {
					// could not parse layout state
					this.f.emit("error.SEVERE", "Could not restore layout configuration", err);
					state = {
						layout: {}
					};
				}
			} else {
				state = {
					layout: {}
				};
			}

			return state;
		};

		/**
		 * Switch the the other layout
		 * @param {LayoutConstants.Layout} layoutType
		 */
		LayoutManager.prototype.switchLayout = function (layoutType) {
			if (this._layoutType != layoutType) {
				var modalBackground = $("<div/>").show().addClass("modalOverlay").text(this.f.t("Loading"));
				this.f.getContext().append(modalBackground);
				this.f.setOptionCallback("LAYOUT_PREFERENCE", layoutType, _.bind(function (success) {
					if (success) {
						this.f.emit("viewer.request.close", window.location, "SaveAndSwitchLayout", "SwitchLayoutWithoutSave");
					}
					modalBackground.remove();
				}, this));
			}
		};

		return LayoutManager;
	});
