define(["web-viewer/utils/ViewerUtils",
		"component/tooltip/Tooltip",
		"underscore",
		"jquery",
		"colorpicker"
	],
	function (ViewerUtils,
		Tooltip,
		_,
		$) {
		"use strict";

		/**
		 * @exports ColorPicker
		 * @constructor
		 * @param {Function} onColorPicked - callback called when a color is picked, parameter: {String} (hex)color without #
		 * @param {HTMLObject} [element] - the element where the colorpicker needs to be added to
		 */
		var ColorPicker = function (onColorPicked, element) {

			// Callback on color picked
			this._onColorPicked = onColorPicked;

			// Current color (without #)
			this._color = undefined;

			// The color block
			this._colorBlock = undefined;
			if (element === undefined) {
				this._colorBlock = $("<div />").addClass("colorblock");
			} else {
				this._colorBlock = element;
			}

			// Public element
			this.element = this._colorBlock;


			this._lazyOnChange = _.debounce(_.bind(this._onChange, this), 50);
			// Init colorpicker
			this.element.ColorPicker({
				'onChange': this._lazyOnChange,
				'onBeforeShow': _.bind(function () {
					$(this).ColorPickerSetColor(this._color);
				}, this),
				'onSubmit': function (hsb, hex, rgb, el) {
					$(el).ColorPickerHide();
				}
			});
		};

		/**
		 * On Change of the color picker
		 * @param {Object} hsb - HSB values
		 * @param {String}   hex - Hex value
		 * @param {Object} rgb - RGB values
		 */
		ColorPicker.prototype._onChange = function (hsb, hex, rgb) {
			this._color = hex.toUpperCase();
			this._updateColor();
			this._onColorPicked(this._color);
		};

		/**
		 * Update the color of the color block if it was added by this class
		 */
		ColorPicker.prototype._updateColor = function () {
			if (this._colorBlock.hasClass("colorblock")) {
				this._colorBlock.css('background-color', "#" + this._color);
			}
		};

		/**
		 * Set the color of the colorpicker.
		 * @param {String} hexColor New color in hex format (with or without #)
		 */
		ColorPicker.prototype.setColor = function (hexColor) {
			if (ViewerUtils.startsWith(hexColor, "#")) {
				hexColor = hexColor.substring(1);
			}
			if (this._color != hexColor) {
				this._color = hexColor;
				this._updateColor();
			}
		};

		/**
		 * Get the color of the colorpicker
		 * @return {String} current color in hex format (without #)
		 */
		ColorPicker.prototype.getColor = function () {
			return this._color;
		};

		/**
		 * Show the color picker
		 */
		ColorPicker.prototype.show = function () {
			this.element.ColorPickerShow();
		};

		/**
		 * Hide the color picker
		 */
		ColorPicker.prototype.hide = function () {
			this.element.ColorPickerHide();
		};

		/**
		 * Destroy the color picker
		 */
		ColorPicker.prototype.destroy = function () {
			this.element.ColorPickerHide();
			$("#" + this.element.data('colorpickerId')).detach();
		};

		/**
		 * Set Tooltip for the color picker element.
		 * @param {String}         tooltipOrBundle - tooltip to show if langKey is undefined or language bundle
		 * @param {String}         [langKey]       - not required
		 * @param {Array.<String>} [stringArgs]    - not required, parameters that need to be filled in the language string
		 */
		ColorPicker.prototype.setTooltip = function (f, tooltipOrBundle, langKey, stringArgs) {
			if (this.tooltip === undefined) {
				this.tooltip = new Tooltip(f.i18n, this._colorBlock, f.getContext(), f.getContext(), f.isMobile(), tooltipOrBundle, langKey, stringArgs);
			} else {
				this.tooltip.setTooltipContent(tooltipOrBundle, langKey, stringArgs);
			}
		};

		return ColorPicker;
	});