define(["web-viewer/layout/component/RichTextEditor",
		"web-viewer/utils/ViewerUtils",
		"tinyMCE",
		"jquery",
		"underscore"
	],
	function (RichTextEditor,
		ViewerUtils,
		tinyMCE,
		$) {
		"use strict";

		/**
		 * This is a list of supported languages.
		 * 
		 * Whenever TinyMCE is updated, make sure these languages are available in the tinymce/langs folder.
		 * extra note: The language file for Czech should be cz.js and not cs.js (as it is provided by TinyMCE)
		 */
		var SUPPORTED_LANGUAGES = ["br", "cn", "cz", "de", "en", "es", "fr", "hu", "hr", "it", "jp", "nl", "pl", "ru", "th", "tr", "tw"];
		var UNSUPPORTED_LANGUAGES = [];
		var DEFAULT_LANGUAGE = "en";

		/**
		 * Create a new TinyMCE rich text editor. The richtext editor is not shown by default, it will be initialized by calling .init()
		 * @class
		 * A rich text editor has a title, a rich text area and an action bar with some buttons.
		 * This class hangs on the DOM element in the property "richTextObj"
		 * @exports RichTextEditorTinyMCE
		 */
		var RichTextEditorTinyMCE = function (f) {
			RichTextEditor.call(this, f);


			this.editor = undefined;
			
			// Check if there is a custom file for this layout type
			this._cssContentFile = "../wcr-static/web-viewer/layout/component/less/tinymce/tinymce_content_" + f.getLayoutType()+".css";
		};

		RichTextEditorTinyMCE.prototype = Object.create(RichTextEditor.prototype);
		RichTextEditorTinyMCE.prototype.constructor = Object.create(RichTextEditorTinyMCE);
		/**
		 * Set content
		 * @param {String} content - Content to set in the text area
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.setContent = function (content) {
			this.textArea.val(content);
			if (this.initialized) {
				this.editor.setContent(content);
			}
		};

		/**
		 * Get content as HTML
		 * @returns {String} Content as HTML if the editor was initialized, otherwise ""
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.getContent = function () {
			if (this.initialized) {
				return this.editor.getContent();
			} else {
				return "";
			}
		};


		/**
		 * Init the rich text editor
		 * Should be called after the element is added to the DOM
		 * @param {Function} [callback] - Callback when the editor is initialized (also fired when it was already initialized). From then on, you can safely use methods like getContent().
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.init = function (callback) {
			if (!this.initialized) {
				var userLocale = this.f.getCurrentUser().getLocale();

				if (_.contains(SUPPORTED_LANGUAGES, userLocale)) {
					// we support the user locale for TinyMCE, initialize TinyMCE
					this._initMCE(callback, userLocale);
				} else {
					// we don't support this locale by default... check if the locale is available on the server
					if (_.contains(UNSUPPORTED_LANGUAGES, userLocale)) {
						// we already checked, and this language doesn't exist... load default language
						this._initMCE(callback, DEFAULT_LANGUAGE);
					} else {
						// check if the language file exist					
						this.f.ajax(requirejs.toUrl("lib/tinymce/langs/" + userLocale + ".js"), {
							dataType: "text",
							timeout: 2000,
							success: _.bind(function () {
								// this language is also supported, register it, and initialize TinyMCE
								SUPPORTED_LANGUAGES.push(userLocale);
								this._initMCE(callback, userLocale);
							}, this),
							error: _.bind(function () {
								// this language doesn't exist, register it as unsupported and initialize TinyMCE
								UNSUPPORTED_LANGUAGES.push(userLocale);
								this._initMCE(callback, DEFAULT_LANGUAGE);
							}, this)
						});
					}
				}
			} else {
				if (callback) {
					callback();
				}
				this.editor.focus();
			}
		};

		/**
		 * Start up TinyMCE
		 * @param {Function} callback - callback function to be called when TinyMCE is initialized 
		 * @param {String} locale - the locale in which TinyMCE should be started
		 */
		RichTextEditorTinyMCE.prototype._initMCE = function (callback, locale) {
			tinyMCE.init({
				selector: "#" + this.textAreaId,
				menubar: false,
				statusbar: false,
				plugins: "paste fullpage",
				toolbar: "bold italic underline strikethrough | subscript superscript | undo redo",
				toolbar_items_size: "small",
				skin: 'lightgrayflat',
				content_css: this._cssContentFile,
				element_format: "xhtml",
				doctype: "<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Strict//EN' " + "'http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd'>",
				fullpage_default_doctype: "<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Strict//EN' " + "'http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd'>",
				inline_styles: true,
				language: locale,
				valid_elements: "p[style],p[style]/li,div,span[style],span[style],br,nobr,sub,sup,strike/s,strong/b,i,u,s,a,h?,big,small,font,tt,code,var,kbd,em,samp,center,a[href|title|style]",
				auto_focus: this.textAreaId,
				setup: _.bind(function (ed) {
					this.editor = ed;
					ed.on("keyup", _.bind(function (e) {
						this._fireTextChanged();
					}, this));
					ed.on("redo", _.bind(function (e) {
						this._fireTextChanged();
					}, this));
					ed.on("undo", _.bind(function (e) {
						this._fireTextChanged();
					}, this));
					ed.on("change", _.bind(function (e) {
						this._fireTextChanged();
					}, this));
					ed.on('init', _.bind(function (e) {
						this.initialized = true;
						if (callback) {
							callback();
						}
					}, this));
				}, this),
				// this method is called before text is pasted in the richtext editor
				paste_preprocess: function (pl, o) {
					// convert the tables to div/p/span elements, as the java viewer doesn's support tables
					o.content = o.content.replace(/<(\/)?table[^>]*>/gi, "<$1div>");
					o.content = o.content.replace(/<(\/)?tr[^>]*>/gi, "<$1p>");
					o.content = o.content.replace(/<th[^>]*>/gi, "<span><strong>");
					o.content = o.content.replace(/<\/th[^>]*>/gi, "</strong></span>&nbsp;");
					o.content = o.content.replace(/<td[^>]*>/gi, "<span>");
					o.content = o.content.replace(/<\/td[^>]*>/gi, "</span>&nbsp;");
				},
				height: "100%"
			});
		};

		/**
		 * should a text change be fired?
		 * @returns {Boolean}
		 * @override
		 */
		RichTextEditorTinyMCE.prototype._shouldFireTextChanged = function () {
			return this.editor && this.editor.isDirty() && this._fireTextChanges;
		};


		/**
		 * Destroy the rich text editor.
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.destroy = function () {
			if (this.initialized) {
				this.editor.destroy();
			}
			this.initialized = false;
		};

		/**
		 * Should be called when an editor is detached from the DOM
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.beforeDetach = function () {
			if (this.isInitialized()) {
				this._fireTextChanges = false;
				this.destroy();
				this._fireTextChanges = true;
			}
		};

		/**
		 * Repair the TinyMCE instance by destroying it and reinializing it
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.repair = function () {
			this.init();
		};

		/**
		 * @override
		 */
		RichTextEditorTinyMCE.prototype.setHeight = function (newHeight) {
			if (this.isInitialized()) {
				// just calculating it here doesn't work as tinymce loads its css asynchronously
				var toolbarHeight = this.textAreaDiv.find(".mce-toolbar-grp").outerHeight(true);
				// THIS IS A VERY UGLY HACK but there is no usable alternative known at the moment!
				// one toolbar row is 26px, 2px padding at top and bottom
				if (toolbarHeight % 26 !== 4) {
					toolbarHeight = 30;
				}
				var height = newHeight - this.actionBar.outerHeight(true) - toolbarHeight;
				height = Math.max(height, 80); // min 80 px
				var iframe = "#" + this.textAreaId + "_ifr";
				$(iframe).css("height", height);
			}
		};

		return RichTextEditorTinyMCE;
	});