define('web-viewer/layout/Layout',["web-viewer/utils/ListenerUtil", "underscore"],
	function (
		ListenerUtil, _
	) {
		"use strict";

		/**
		 * @exports Layout
		 * @class
		 * Defines a general implementation for different layouts. Every layout has different modes it can change to.
		 * @constructor
		 * @param {core.ViewerFacade} f
		 * @param {LayoutModel} layoutModel
		 * @param {JQueryObj} context
		 * @param {Object} layoutState
         * @param {LoadingIndicator} loadingIndicator - the loading indicator, which was created before all requests started
         * @param {JQueryObj} initialLoader - the JQuery reference to the element indicating the initial loader (needed to safely remove it without emptying the context)            
		 */
		var Layout = function (f, layoutModel, context, layoutState, loadingIndicator, initialLoader) {
			ListenerUtil.mixin(this, f);

			this.f = f;

			/**
			 * Context of this layout, this is the DOM-object the layout will use
			 * @member {JQueryObj}
			 */
			this._context = context;

			/**
			 * The layout model, holds the layoutItems
			 * @member {LayoutModel}
			 */
			this._layoutModel = layoutModel;

			/**
			 * All items that are not allowed to be visible in the current mode but can become visible in another mode.
			 * @member {Object}
			 */
			this._notVisibleItems = [];
			
			/**
             * The loaded state of the layout.
             */
            this._layoutState = layoutState;
			
			 /**
             * The loadingIndicator
             */
            this._loadingIndicator = loadingIndicator;
			
			/**
			 * The DOM element used to show initial loader in the context
			 */
			this._initialLoader = initialLoader;

			/**
			 * Currently active mode for this layout
			 * @member {LayoutMode}
			 */
			this._activeMode = null; //to be set during initialization of the specific layout

			//register listener to the layoutModel for when the layout model is updated (item is added)
			this._registerListener("itemAdded", function (layoutItem) {
				if (this._activeMode.isItemAllowed(layoutItem.name)) {
					this._addItem(layoutItem);
				} else {
					this._notVisibleItems.push(layoutItem);
				}
			}, this, this._layoutModel);
			this._registerListener("layout.switchLayoutMode", _.bind(this._switchLayoutMode, this), this, this.f);
		};

		/**
		 * Adds a layoutItem to the layout
		 * @param {Object} layoutItem
		 */
		Layout.prototype._addItem = function (layoutItem) {
			//to be implemented by specific layout
		};

		/**
		 * Decide which of the added items should be shown and which ones should be hidden.
		 */
		Layout.prototype._updateItemVisibility = function () {
			//to be implemented by specific layout
		};

		/**
		 * The layout has changed, emit this.
		 */
		Layout.prototype._emitLayoutChanged = function () {
			this._emit("layout.changed");
		};

		/**
		 * Check whether the layout should be persisted or not, depends on the active mode.
		 * @returns {Boolean}
		 */
		Layout.prototype.isLayoutStatePersisted = function () {
			return this._activeMode.isPersisted();
		};

		/**
		 * Get the state of the layout.
		 * @returns {Object}
		 */
		Layout.prototype.getLayoutState = function () {
			//to be implemented by specific layout
		};

		/**
		 * Switch the active mode.
		 * @param {LayoutConstants.LayoutMode} [mode] - if not provided it will use the default mode for the layout
		 */
		Layout.prototype._switchLayoutMode = function (mode) {
			if (this._activeMode === null || mode !== this._activeMode.getName()) {
				var newMode = null;
				if (mode !== undefined) {
					newMode = this._layoutModel.getLayoutMode(mode);
				} else {
					newMode = this._getDefaultLayoutMode();
				}
				if (newMode !== undefined) {
					if (this._activeMode !== null) {
						this._activeMode.toggle(false);
					}
					this._activeMode = newMode;
					this._activeMode.toggle(true);

					var addedItems = [];
					_.each(this._notVisibleItems, function (item, key) {
						if (this._activeMode.isItemAllowed(item.name)) {
							this._addItem(item);
							addedItems.push(item);
						}
					}, this);
					if (addedItems.length) {
						this._notVisibleItems = _.difference(this._notVisibleItems, addedItems);
					}
					this._updateItemVisibility();

					this._emit("changed.layoutMode");
				}
			}
		};

		/**
		 * Returns the default layout mode for the layout.
		 * @return {LayoutMode}
		 */
		Layout.prototype._getDefaultLayoutMode = function () {
			//to be implemented by specific layout
		};

		/**
		 * Get a reference to the active layout mode
		 * @returns {LayoutMode}
		 */
		Layout.prototype.getActiveLayoutMode = function () {
			return this._activeMode;
		};


		/**
		 * Returns the type of the layout.
		 * @return {Number}
		 */
		Layout.prototype.getLayoutType = function () {
			//to be implemented by specific layout
			return -1;
		};

		return Layout;
	});

define('web-viewer/layout/DraggableWidgets',["jquery",
		"jqueryUI",
		"underscore"
	],
	function ($) {
		"use strict";

		/**
		 * HELPER interface for draggable widgetwrappers
		 * Provides structured storage for extra information needed for dragging
		 */
		function DraggableWidgetWrapper(widgetWrapper) {
			this.wrapper = widgetWrapper;
			this.ghost = $("<div/>").addClass("ghostWrapper");

			this.sizeBeforeDrag = {
				width: undefined,
				height: undefined
			};
		}

		/**
		 * This method updates the dragging ghost element dimensions.
		 * The ghost element is transparent, but has the same size as the original element being dragged
		 */
		DraggableWidgetWrapper.prototype.updateGhost = function () {
			this.ghost.css({
				width: this.wrapper.element.outerWidth(true),
				height: this.wrapper.element.outerHeight(true)
			});
		};



		/**
		 * Enable draggable/droppable/sortable widgets on a container
		 * 
		 * When initialized widgets should be added (dragging enabled by enableDragging(...)) and dropzones should be defined
		 */
		var DraggableWidgets = function (container) {
			this.container = container;
			this._acceptSelector = ".widgetWrapper";
			this._siblingSelector = ".widgetWrapper";
			this.hoveredSortableDropzone = undefined;
		};

		/**
		 * Enable a WidgetWrapper to be dragged
		 * @param widgetWrapper
		 */
		DraggableWidgets.prototype.enableDragging = function (widgetWrapper) {
			//make the widgetwrapper draggable
			widgetWrapper.drag = new DraggableWidgetWrapper(widgetWrapper);

			$(widgetWrapper.element).draggable({
				containment: this.container,
				// only when the item is dragged more than 20px, we start dragging
				distance: 20,
				// dragging widgets doesn't introdicue scrolling of the page
				scroll: false,
				// when a dropzone is invalid, the widget is reverted to its orignal lcation
				revert: "invalid",
				// when dragging starts, create a semi transparent clone of the widget
				helper: function () {
					var clone = widgetWrapper.element.clone();
					clone.css({
						width: widgetWrapper.element.innerWidth(),
						opacity: 0.9
					});
					return clone;
				},
				// elements should be added to the container
				appendTo: this.container,
				// start dragging
				start: _.bind(function () {

					//update the ghost size and store the size before dragging or later usage
					this.drag.updateGhost();
					this.drag.sizeBeforeDrag.width = this.element.innerWidth();
					this.drag.sizeBeforeDrag.height = this.element.innerHeight();

					// hide the original element
					this.element.hide();
				}, widgetWrapper),

				// the element is being dragged
				drag: _.bind(function (event, ui) {
					// if a sortable zone is being hovered, insert the ghost at the correct location (this make the dropzone sortable!)
					if (this.hoveredSortableDropzone) {
						this.insertAt(this.hoveredSortableDropzone, widgetWrapper.drag.ghost, ui.offset.top - this.hoveredSortableDropzone.offset().top);
					}
				}, this),
				// dragging has stopped, remove the helper element, and detac the ghost, display the original element again
				stop: _.bind(function (event, ui) {
					this.drag.ghost.detach();
					this.element.show();

					this.widget.dragged(ui.offset.top, ui.offset.left);

					ui.helper.detach();
				}, widgetWrapper),

				// dragging is being done using the header element as handle
				handle: ".header"
			});
		};

		/**]
		 * Define a sortable widget dropzone
		 * @param panel
		 */
		DraggableWidgets.prototype.addSortableWidgetDropzone = function (panel, section, workspace) {
			$(panel).droppable({
				// only accept widgetwrappers to be dropped
				accept: this._acceptSelector,
				drop: _.bind(function (e, ui) {
					// drop in sortable widget dropzone, find the correct sibbling
					var sibling = this.findSibling(panel, ui.draggable, ui.offset.top - panel.offset().top);
					var widgetWrapper = ui.draggable[0].wrapper;
					if (sibling === undefined) {
						// no sibling found, just append to the section
						workspace.moveItem(widgetWrapper.getID(), section, undefined, true);
					} else {
						if (sibling.wrapper) {
							sibling = sibling.wrapper;
							workspace.moveItem(widgetWrapper.getID(), section, sibling.getID(), true);
						}
					}
				}, this),

				// a widgetwrapper is being dragged over this section, register this section as the current dropzone
				over: _.bind(function (e, ui) {
					this.hoveredSortableDropzone = panel;
				}, this),

				// the widgetwrapper is no long being dragged over this section
				out: _.bind(function (e, ui) {
					this.hoveredSortableDropzone = undefined;
				}, this),
				tolerance: "touch"
			});


		};

		/**
		 * Add a DropZone in which widgets can be dropped on top of all content (widget becomes modal!)
		 * @param panel
		 */
		DraggableWidgets.prototype.addModalWidgetDropzone = function (panel, section, workspace) {
			$(panel).droppable({
				accept: this._acceptSelector,
				drop: _.bind(function (e, ui) {
					var widgetWrapper = ui.draggable[0].wrapper;
					if (widgetWrapper) {
						widgetWrapper.setModal(true, ui.offset.top, ui.offset.left, widgetWrapper.drag.sizeBeforeDrag.width);

						workspace.moveItem(widgetWrapper.getID(), section);

						widgetWrapper.widget.fireModalResize();
					}
				}, this)
			});

		};


		/**
		 * Remove a panel as dropzone
		 * @param panel
		 */
		DraggableWidgets.prototype.removeDropzone = function (panel) {
			$(panel).droppable('disable');
		};


		/**
		 * Insert a element in a panel at a certain position (used for enabling sorting)
		 * @param list
		 * @param el
		 * @param position
		 */
		DraggableWidgets.prototype.insertAt = function (list, el, position) {
			var insertBefore = this.findSibling(list, el, position);
			if (insertBefore === undefined) {
				list.append(el);
			} else {
				$(insertBefore).before(el);
			}
		};

		/**
		 * Find the correct position to insert element
		 * @param list
		 * @param el
		 * @param position
		 * @returns element before which the element should be inserted
		 */
		DraggableWidgets.prototype.findSibling = function (list, el, position) {
			var currentHeight = 0;
			var insertBefore;
			var canContinue = true;

			list.children(this._siblingSelector).each(function (key, el) {
				var outerHeight = $(el).outerHeight(true);
				if (canContinue && position <= currentHeight + (outerHeight / 2)) {
					insertBefore = el;
					canContinue = false;
				}
				currentHeight += outerHeight;
			});

			return insertBefore;
		};

		return DraggableWidgets;
	});

define('web-viewer/layout/WidgetWrapper',["jquery",
		"utils/ShapeUtils"
	],
	function ($, ShapeUtils) {
		"use strict";

		/**
		 * A widgetwrapper is a wrapper for a widget, which makes enables the widget to be displayed with some extra controls
		 * @param {AbstractWidget} widget          - the widget to wrap
		 * @param {String}         widgetName      - the name of the widget
		 * @param {Boolean} allowCollapse - is collapsing the widget allowed?
		 */
		var WidgetWrapper = function (widget, widgetName, allowCollapse) {
			this._headerEnabled = true;
			// main wrapper
			this.element = $("<div/>").addClass("widgetWrapper");
			this.element[0].wrapper = this;

			// webkit has a bug with iFrames, which appearantly they won't be solving soon
			// if an input in an iframe is focused, the page scrolls to the iframe
			// this can occur when TinyMCE is used... we can fix it by scrolling to top whenever the context is scrolled
			this.element.scroll(function (event) {
				$(this).scrollTop(0);
			});

			this.name = widgetName;

			// header + title
			this.header = $("<div/>").addClass("header");
			this.headerText = $("<h3>", {
				text: widget.getTitle(),
				title: widget.getTitle()
			});
			this.headerHandle = $("<div/>").addClass("headerHandle").append(this.headerText);
			if (allowCollapse) {
				this.headerHandle.click(_.bind(function () {
					this.toggle();
				}, this));
			}
			this.header.append(this.headerHandle);

			this.headerExtra = $("<div/>").addClass("headerExtra");
			this.header.append(this.headerExtra);

			// listen to title changed of widget
			widget.on("titlechanged", _.bind(this._onTitleChanged, this));

			// content holder
			this.content = $("<div/>").addClass("content");

			// create dom structure
			this.element.append(this.header, this.content);

			// activate the widget
			widget.setContext(this.content, this.headerExtra);

			//is the widget resizable
			this.resizable = false;

			// the widget
			this.widget = widget;

			this.isModal = false;

			this.isCollapsed = false;
			this._isVisible = true;

			//location; only use if the widgetwrapper is hidden
			this._locationWhenHidden = new ShapeUtils.Point();
		};

		/**
		 * Convert the widget in to a modal widget or normal widget
		 * @param modal {boolean} should the widget be modal or not?
		 * @param top {int} position top
		 * @param left {int} position left
		 * @param width {int} width of modal window
		 * @param height {int} height of modal window
		 */
		WidgetWrapper.prototype.setModal = function (modal, top, left, width, height) {
			this.widget.setModal(modal);
			if (modal) {

				this.element.css({
					position: "absolute",
					top: top,
					left: left,
					width: width || this.element.width() || undefined,
					height: height || this.element.height() || undefined
				});
				var newHeight = height || this.element.height() || 0;
				this.content.css("height", newHeight - this.getHeaderHeight());

				this.setResizable(!this.isCollapsed);

				this.isModal = true;
			} else {
				this.isModal = false;
				// restore normal css
				this.element.css({
					position: "relative",
					top: "",
					left: "",
					width: "auto"
				});

				this.setResizable(false);
			}
		};

		/**
		 * Function called when the title of the widget changed
		 * @param {String} newTitle - The new title
		 */
		WidgetWrapper.prototype._onTitleChanged = function (newTitle) {
			this.headerText.text(newTitle).attr("title", newTitle);
		};

		/**
		 * Toggle collapse state of the widget
		 */
		WidgetWrapper.prototype.toggle = function () {
			this.setCollapsed(!this.isCollapsed);
		};

		/**
		 * Hides/shows a widget
		 * @param {Boolean} isVisible
		 */
		WidgetWrapper.prototype.setVisible = function (isVisible) {
			this._isVisible = isVisible;
			if (isVisible) {
				this.element.removeClass("hiddenElement");
			} else {
				var offset = this.element.offset();
				//location is only used when the widgetwrapper is hidden, if visible the element is used
				this._locationWhenHidden = new ShapeUtils.Point(offset.left, offset.top);
				this.element.addClass("hiddenElement");
			}
		};

		/**
		 * Is the widgetwrapper visible.
		 * @returns {Boolean}
		 */
		WidgetWrapper.prototype.isVisible = function () {
			return this._isVisible;
		};

		/**
		 * (un)Collapse the widget
		 * @param {Boolean} collapsed
		 */
		WidgetWrapper.prototype.setCollapsed = function (collapse) {
			var wasCollapsed = this.isCollapsed;
			this.element.toggleClass("collapsed", collapse);
			this.isCollapsed = collapse;
			this.widget.setCollapsed(collapse);

			if (this.isModal && wasCollapsed !== collapse) {
				var newHeight = Math.min($(window).height() - (this.element.offset().top - $(window).scrollTop()), this.getPreferredSize().height);
				this.setHeight(newHeight);
				if (!collapse) {
					this.setResizable(true);
				}
				this.setResizable(!this.isCollapsed);
				this.widget.fireModalResize();
			}

		};

		WidgetWrapper.prototype.setResizable = function (resizable) {
			if (this.resizable && !resizable) {
				this.element.resizable('destroy');
				this.resizable = false;
			} else if (!this.resizable && resizable) {
				//make the element resizable
				var minimumSize = this.getMinimumSize();
				this.element.resizable({
					minWidth: Math.max(100, minimumSize.width),
					minHeight: Math.max(40, minimumSize.height),
					stop: _.bind(function (event, ui) {
						this.setHeight(ui.size.height);
						this.widget.fireModalResize();
					}, this),
					resize: _.throttle(_.bind(function (event, ui) {
						this.setHeight(ui.size.height);
						this.widget.resizing();
					}, this), 40)
				});
				this.resizable = true;
			}
		};

		/**
		 * Get the ID of the enclosed widget
		 * @returns
		 */
		WidgetWrapper.prototype.getID = function () {
			return this.widget.getID();
		};

		/**
		 * Get the minimum size of the widget content
		 *
		 * note it is possible that this size is not respected by the layoutmanager
		 *
		 * @returns {ShapeUtils.Size}
		 */
		WidgetWrapper.prototype.getMinimumSize = function () {
			if (this.isCollapsed) {
				return new ShapeUtils.Size(0, this.getHeaderHeight());
			} else {
				var minSize = this.widget.getMinimumSize().copy();
				minSize.height += this.getHeaderHeight() + 3;

				return minSize;
			}
		};

		/**
		 * Get the preferred size of the widget content
		 *
		 * note it is possible that this size is not respected by the layoutmanager
		 *
		 * @returns {ShapeUtils.Size} if undefined, no preferred size is taken in to account by layoutmanager, the layout managers tries to guess the needed size
		 */
		WidgetWrapper.prototype.getPreferredSize = function () {
			if (this.isCollapsed) {
				return new ShapeUtils.Size(0, this.getHeaderHeight());
			} else {
				var prefSize = this.widget.getPreferredSize();
				if (prefSize) {
					prefSize = prefSize.copy();
					prefSize.height += this.getHeaderHeight() + 3;
					return prefSize;
				} else {
					return undefined;
				}
			}
		};

		/**
		 * Get the maximum dimension of the widget content
		 *
		 * @returns {ShapeUtils.Size} if undefined the widget can be made as big as the user wants
		 */
		WidgetWrapper.prototype.getMaximumSize = function () {
			if (this.isCollapsed) {
				return new ShapeUtils.Size(1000, this.getHeaderHeight());
			} else {
				var maxSize = this.widget.getMaximumSize();
				if (maxSize) {
					maxSize = maxSize.copy();
					maxSize.height += this.getHeaderHeight() + 3;
					return maxSize;
				} else {
					return undefined;
				}
			}
		};


		/**
		 * Get the height of the header
		 * @returns
		 */
		WidgetWrapper.prototype.getHeaderHeight = function () {
			if (this._headerEnabled) {
				return this.header.outerHeight(true);
			} else {
				return 0;
			}
		};

		/**
		 * Get the current content height of the widget
		 * @returns
		 */
		WidgetWrapper.prototype.getContentHeight = function () {
			return this.content.getOuterHeight(true);
		};

		/**
		 * Will the widgetwrapper extend if needed?
		 * @returns
		 */
		WidgetWrapper.prototype.willFill = function () {
			return this.widget.willFill();
		};

		/**
		 * Set the height of the widgetwrapper
		 * @param height
		 */
		WidgetWrapper.prototype.setHeight = function (height) {
			var prevContentHeight = this.content.height();
			var newContentHeight = height - this.getHeaderHeight();
			this.element.css("height", height);
			this.content.css("height", newContentHeight);



			if (newContentHeight != prevContentHeight) {
				this.widget.resized();

				this.content.css("overflow", "hidden");
				_.defer(function (el) {
					el.css("overflow", "auto");
				}, this.content);
			}
		};

		/**
		 * Get the size of the WidgetWrapper
		 *
		 * @returns {ShapeUtils.Size}
		 */
		WidgetWrapper.prototype.getSize = function () {
			return new ShapeUtils.Size(this.element.width(), this.element.height());
		};

		/**
		 * Get the outer size of the widget wrapper, including borders and padding
		 *
		 * @returns {ShapeUtils.Size}
		 */
		WidgetWrapper.prototype.getOuterSize = function () {
			return new ShapeUtils.Size(this.element.outerWidth(true), this.element.outerHeight(true));
		};

		/**
		 * Get the location of the WidgetWrapper
		 *
		 * @returns {ShapeUtils.Point}
		 */
		WidgetWrapper.prototype.getLocation = function () {
			if (this._isVisible) {
				var offset = this.element.offset();
				return new ShapeUtils.Point(offset.left, offset.top);
			} else {
				return this._locationWhenHidden;
			}
		};

		/**
		 * Enable/Disable the header display of the widget wrapper
		 * @param {Boolean} enable
		 */
		WidgetWrapper.prototype.setHeaderEnabled = function (enable) {
			this._headerEnabled = enable;
			this.header.toggleClass("displayNone", !enable);
		};

		return WidgetWrapper;
	});

define('web-viewer/layout/desktop/Panel',["web-viewer/core/PrivateChannel"],
	function (PrivateChannel) {
		"use strict";

		/**
		 * Panel interface element (for sidepanels/topbar/centerpanel)
		 * 
		 * @constructor
		 * @memberof Layout
		 * @param {ViewerFacade} f
		 * @param {String} name - name of the panel
		 * @param {JQueryObj} el - main object in which the panel is drawn 
		 */
		var Panel = function (f, name, el) {
			this.f = f;
			PrivateChannel.installTo(this, f, ["panel", name]);

			this.name = name;
			this.el = el;

			this.width = 0;
			this.height = 0;

			this._visible = true;
			this._disabled = false;
		};


		/**
		 * Set visibility of panel to visible
		 */
		Panel.prototype.show = function () {
			var oldVisibility = this.isVisible();

			this._visible = true;
			this._updateVisibility();

			if (oldVisibility != this.isVisible()) {
				this._emit("changed.visibility", oldVisibility, this.isVisible());
			}
		};

		/**
		 * Set visiblity of panel to hidden
		 */
		Panel.prototype.hide = function () {
			var oldVisibility = this.isVisible();
			this._visible = false;
			this._updateVisibility();

			if (oldVisibility != this.isVisible()) {
				this._emit("changed.visibility", oldVisibility, this.isVisible());
			}
		};

		/**
		 * Toggle the panel
		 * @param {Boolean} [toggle] 
		 */
		Panel.prototype.toggle = function (toggle) {
			if (toggle === undefined) {
				toggle = !this.getVisibility();
			}

			if (toggle) {
				this.show();
			} else {
				this.hide();
			}
		};

		/**
		 * Check if the panel is shown or not (note, this doesn't take into account autocollapse or disabled)
		 * @returns {Boolean}
		 */
		Panel.prototype.getVisibility = function () {
			return this._visible;
		};







		/**
		 * Enable/disable the panel
		 * @param {Boolean} disabled
		 */
		Panel.prototype.setDisabled = function (disabled) {
			var oldVisibility = this.isVisible();

			this._disabled = disabled;
			this._updateVisibility();

			if (oldVisibility != this.isVisible()) {
				this._emit("changed.visibility", oldVisibility, this.isVisible());
			}
		};

		/**
		 * Check if the panel is disabled
		 * @returns {Boolean}
		 */
		Panel.prototype.isDisabled = function () {
			return this._disabled;
		};




		/**
		 * Set the width of the panel
		 * @param {Number} width - width of the panel 
		 */
		Panel.prototype.setWidth = function (width) {
			if (this.width !== width) {
				this.width = width;
				this.el.css('width', width);
			}
		};

		/**
		 * Set the height of the panel
		 * @param {Number} height - height of the panel
		 */
		Panel.prototype.setHeight = function (height) {
			if (this.height !== height) {
				this.height = height;
				this.el.css('height', height);
			}
		};

		/**
		 * Get the height of the panel
		 * @returns {Number}
		 */
		Panel.prototype.getHeight = function () {
			return this.isVisible() ? this.height : 0;
		};

		/**
		 * Get the width of the panel
		 * @returns {Number}
		 */
		Panel.prototype.getWidth = function () {
			return this.isVisible() ? this.width : 0;
		};

		/**
		 * et the width of the panel (don't take into account visibility)
		 * @returns {Number}
		 */
		Panel.prototype.getWidthWhenVisible = function () {
			return this.width;
		};

		/**
		 * Get the height of the panel (don't take into account visibility)
		 * @returns {Number}
		 */
		Panel.prototype.getHeightWhenVisible = function () {
			return this.height;
		};


		/**
		 * Get the outerwidth of the panel
		 * @returns {Number}
		 */
		Panel.prototype.outerWidth = function () {
			if (this.isVisible()) {
				return this.el.outerWidth(true);
			} else {
				return 0;
			}
		};

		/**
		 * Get the innerWidth of the panel
		 * @returns {Number}
		 */
		Panel.prototype.innerWidth = function () {
			if (this.isVisible()) {
				return this.el.innerWidth();
			} else {
				return 0;
			}
		};

		/**
		 * Get the outerHeight of the pane;
		 * @returns {Number}
		 */
		Panel.prototype.outerHeight = function () {
			if (this.isVisible()) {
				return this.el.outerHeight(true);
			} else {
				return 0;
			}
		};

		/**
		 * Get the position of the panel
		 * @returns {Object} - position 
		 */
		Panel.prototype.position = function () {
			return this.el.position();
		};


		/**
		 * Set both width and height of the panel
		 * @private
		 * @param {Number} [width]
		 * @param {Number} [height]
		 */
		Panel.prototype.setDim = function (width, height) {
			if (width !== undefined) {
				this.setWidth(width);
			}

			if (height !== undefined) {
				this.setHeight(height);
			}
		};

		/**
		 * Check if a panel is visible (takes in to account all parameters (visible, autocollapsed, disabled)
		 * @returns {Boolean}
		 */
		Panel.prototype.isVisible = function () {
			return this._visible && !this._disabled;
		};

		/**
		 * Update the visibility of the panel
		 */
		Panel.prototype._updateVisibility = function () {
			this.el.toggle(this.isVisible());
		};

		/**
		 * Get the state of the panel (note only the visible property is saved in the state)
		 * @returns {Object}
		 */
		Panel.prototype.getState = function () {
			return {
				visible: this._visible
			};
		};

		/**
		 * Restore the state of the panel
		 * @param {Object} state - as returned by getState
		 */
		Panel.prototype.restoreState = function (state) {
			this.toggle(state.visible);
		};


		return Panel;
	});

define('web-viewer/layout/LayoutUtils',["underscore"],
	function (_) {
		"use strict";
		/**
		 * LayoutUtils contains some helper functions used by different components\
		 * @exports LayoutUtils
		 * @memberof Layout
		 */
		return {
			/**
			 * Layout al WidgetWrappers in a certain element.
			 * It respects the "willFill" and minSize, maxSize and preferredSize of the widgets
			 * When there is not enough space to fit all widgets, it adds a scrollbar to containerElement
			 * @param {JQueryObj} containerElement - the element in which the widgets reside. The element should have a height for the algorithm to work
			 * @param {Number} widgetBorder - this is some extra spacing that is not taken into account by outerheight
			 */
			layoutWidgets: function (containerElement, widgetBorders) {
				var widgets = containerElement.find(".widgetWrapper");
				var innerHeight = containerElement.innerHeight() - widgetBorders;
				var data = [];
				var w;
				var currentHeight = 0;
				var max;
				var canShrink = [];
				var canGrow = [];

				// first build up the needed data structure
				for (var i = 0; i < widgets.length; i++) {
					var widget = widgets[i].wrapper;
					if (widget.isVisible()) {
						max = undefined;
						if (widget.getMaximumSize()) {
							max = widget.getMaximumSize().height;
						}
						w = {
							widget: widget,
							min: widget.getMinimumSize().height,
							pref: widget.getPreferredSize().height,
							max: max,
							height: 0
						};
						data.push(w);

						//calculate preferred height (taking into account min and max)

						w.height = Math.max(w.min, w.pref);
						if (w.max) {
							w.height = Math.min(w.max, w.height);
						}

						w.height = Math.min(w.height, innerHeight);

						// first we only want to shrink willFill elements 
						// keep track if the element can be shrinked
						if (w.widget.willFill() && w.height > w.min) {
							canShrink.push(w);
						}
						currentHeight += w.height + widgetBorders; // 4: 2px padding and 2px border
					}
				}
				var pixelsNeeded = 0;
				// this function is used to shrink all shrinkable elements
				// it is defined as a inner function because it is used two times
				var shrinkShrinkables = function () {
					pixelsNeeded = currentHeight - innerHeight;
					//as long as we need to shrink, and we have shrinkable elements, try to shrink
					while (pixelsNeeded > 0 && canShrink.length > 0) {
						//we want to shrink proportionally
						var w = _.first(canShrink);
						var shrinkablesPrefHeight = _.reduce(canShrink, function (memo, w) {
							return memo + w.pref;
						}, 0);
						var diff = Math.ceil((w.pref / shrinkablesPrefHeight) * pixelsNeeded);

						canShrink = _.rest(canShrink, 1);

						var prevHeight = w.height;
						w.height = Math.max(w.min, w.height - diff);

						var pixelsRemoved = prevHeight - w.height;
						pixelsNeeded -= pixelsRemoved;
						currentHeight -= pixelsRemoved;
						if (w.height > w.min) {
							canShrink.push(w);
						}
					}
				};

				// first try to shrink all "willFill" elements... these are usually very flexible elements, so they care less if they are shrunk
				shrinkShrinkables();

				if (pixelsNeeded > 0) {
					// we need another round of shrinking...
					// now we take every element that could be shrunk (so its current height is still higher than its minimum size
					canShrink = _.filter(data, function (w) {
						return w.height > w.min;
					});

					shrinkShrinkables();
				}



				pixelsNeeded = currentHeight - innerHeight;
				if (pixelsNeeded > 0) {
					// oh ooh... we can't get small enough... show a scrollbar
					containerElement.css("overflow-y", "auto");
				} else {
					containerElement.css("overflow-y", "hidden");
				}

				// at this point, we should no longer exceed the maximum size (we could still be too big if an element has a big minimum size 

				// check if we can extend some elements
				var pixelsLeft = innerHeight - currentHeight;
				if (pixelsLeft > 0) {

					_.each(data, function (w) {
						if ((!w.max || w.height < w.max) && (w.height < w.pref || w.widget.willFill())) {
							canGrow.push(w);
						}
					});

					while (pixelsLeft > 0 && canGrow.length > 0) {
						var diff = Math.ceil(pixelsLeft / canGrow.length);
						w = _.first(canGrow);
						canGrow = _.rest(canGrow, 1);

						var prevHeight = w.height;
						if (w.max) {
							w.height = Math.min(w.height + diff, w.max);
						} else {
							w.height = w.height + diff;
						}
						var pixelsAdded = w.height - prevHeight;

						pixelsLeft -= pixelsAdded;
						if ((!w.max || w.height < w.max) && (w.height < w.pref || w.widget.willFill())) {
							canGrow.push(w);
						}
					}
				}

				// set all heights
				_.each(data, function (w) {
					w.widget.setHeight(w.height);
				});
			}

		};
	});

define('web-viewer/layout/desktop/SidePanel',["web-viewer/layout/desktop/Panel",
		"web-viewer/layout/component/MiniButton",
		"web-viewer/layout/LayoutUtils",
		"underscore"
	],
	function (Panel,
		MiniButton,
		LayoutUtils,
		_) {
		"use strict";

		/**
		 * Panel interface element (for sidepanels/topbar/centerpanel) used by DesktopLayout
		 * 
		 * @class SidePanel anel interface element (for sidepanels/topbar/centerpanel) used by DesktopLayout
		 * @constructor
		 * @augments Panel
		 * @memberof Layout
		 * @param {ViewerFacade} f
		 * @param {String} name - name of the panel
		 * @param {JQueryObj} el - main object in which the panel is drawn 
		 * @param {JQueryObj} [inner] - inner element of the panel
		 */
		var SidePanel = function (f, name, el, inner) {
			Panel.call(this, f, name, el);

			this.toggleButton = new MiniButton(this.f, "toggleHide" + name, (name == "west") ? "icon-WCR_Panel-Left" : "icon-WCR_Panel-Right", false, _.bind(function () {
				this.toggle();
			}, this));

			this.toggleButton.setTooltip("viewer", (name == "west") ? "LayoutToggleLeftSidebar" : "LayoutToggleRightSidebar");
			this.toggleButton.element.addClass("panelToggleButton");
			this.inner = inner;
			this._autoCollapsed = false;
		};

		SidePanel.prototype = Object.create(Panel.prototype);
		SidePanel.prototype.constructor = SidePanel;


		/**
		 * Autocollapse the panel (or undo it)
		 * @param {Boolean} autoCollapsed
		 */
		SidePanel.prototype.setAutoCollapsed = function (autoCollapsed) {
			this._autoCollapsed = autoCollapsed;
			this._updateVisibility();
		};

		/**
		 * Check if the panel is auto collapsed
		 * @returns {Boolean} 
		 */
		SidePanel.prototype.isAutoCollapsed = function () {
			return this._autoCollapsed;
		};

		/**
		 * @override
		 */
		SidePanel.prototype.isVisible = function () {
			return Panel.prototype.isVisible.call(this) && !this._autoCollapsed;
		};

		/**
		 * Show/hide the panel
		 * @override
		 */
		SidePanel.prototype._updateVisibility = function () {
			Panel.prototype._updateVisibility.call(this);

			// Set the widgets on state invisible
			var widgets = this.inner.find(".widgetWrapper");
			_.each(widgets, function (widgetWrapperElement) {
				var wrapper = widgetWrapperElement.wrapper;
				// NOTE: this doesn't check whether a widget is allowed or not in the layout
				// SidePanel is only used in the DesktopLayout, so this is not a problem (for now)
				if (this.isVisible()) {
					if (wrapper.widget.isEnabled()) {
						wrapper.widget.setVisible(true);
					}
				} else {
					wrapper.widget.setVisible(false);
				}
			}, this);

			// if the panel is disabled don't show the toggle buttons
			if (!this.isDisabled()) {
				this.toggleButton.element.show();
			} else {
				this.toggleButton.element.hide();
			}
		};

		/**
		 * recalculate the position of all widgetwrappers inside the panel
		 */
		SidePanel.prototype.invalidate = function () {
			if (this.isVisible()) {
				LayoutUtils.layoutWidgets(this.inner, 2);
			}
		};

		/**
		 * Also store the width
		 * @override
		 */
		SidePanel.prototype.getState = function () {
			var state = Panel.prototype.getState.call(this);
			state.width = this.width;
			return state;
		};

		/**
		 * Also restore the width of the panel
		 * @param {Object} state
		 */
		SidePanel.prototype.restoreState = function (state) {
			Panel.prototype.restoreState.call(this, state);

			this.setWidth(state.width);
		};


		/**
		 * Get the width of the panel when it would not be autocolapsed
		 * @returns {Number}
		 */
		SidePanel.prototype.getWidthWhenNotAutoCollapsed = function () {
			if (!this.isDisabled() && this.getVisibility()) {
				return this.getWidthWhenVisible();
			} else {
				return 0;
			}
		};

		/**
		 * @override
		 * @param {Number} height
		 */
		SidePanel.prototype.setHeight = function (height) {
			Panel.prototype.setHeight.call(this, height);

			this.inner.css('height', height);
		};

		return SidePanel;
	});

define('web-viewer/layout/LayoutMode',["web-viewer/layout/LayoutConstants"],
	function (
		LayoutConstants

	) {
		"use strict";

		/**
		 * @exports LayoutMode
		 * @memberof Layout
		 * @class
		 * Defines a general implementation of layout mode, the mode decides if items should be added to the layout
		 * and if the state should be persisted.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 * @param {LayoutConstants.LayoutMode} name
		 */
		var LayoutMode = function (f, layout, name) {
			this.f = f;

			/**
			 * Name of the layoutMode
			 */
			this._name = name;

			/**
			 * Is the layoutMode active
			 */
			this._isActive = false;

			/**
			 * The layout for which this mode is active
			 */
			this._layout = layout;

			/**
			 * is the modepanel expanded
			 */
			this._isModePanelExpanded = true;
		};

		/**
		 * Get the name of the mode.
		 * @returns {LayoutConstants.LayoutMode}
		 */
		LayoutMode.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		LayoutMode.prototype.toggle = function (toggle) {
			//to be implemented by specific layout mode
		};

		/**
		 * Checks if the item is allowed for this layout mode. Prevents channels and CAD widget from loading.
		 *
		 * @param {String} itemname
		 * @returns {Boolean}
		 */
		LayoutMode.prototype.isItemAllowed = function (itemname) {
			//to be implemented by specific layout mode
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		LayoutMode.prototype.isPersisted = function () {
			//to be implemented by specific layout mode
		};

		/**
		 * Checks if the mode is active at the moment.
		 */
		LayoutMode.prototype.isActive = function () {
			return this._isActive;
		};

		/**
		 * Decide whether or not to enable the mode panel
		 */
		LayoutMode.prototype.isModePanelEnabled = function () {
			// should be overwritten for mobile layouts in which the mode panel should be shown
			return false;
		};

		/**
		 * If the mode panel is enabled, should it be expanded by default?
		 * @returns {Boolean} is de mode panel expanded by default?
		 */
		LayoutMode.prototype.isModePanelExpanded = function () {
			return this._isModePanelExpanded;
		};

		/**
		 * Should the widget titles be shown?
		 * @returns {Boolean}
		 */
		LayoutMode.prototype.showModePanelWidgetTitles = function () {
			return true;
		};

		/**
		 * Is the modepnale expanded or not?
		 * This method is called by the layout if a modepanel was expanded or collapsed
		 * @param {Boolean} expanded
		 */
		LayoutMode.prototype.setIsmodePanelExpanded = function (expanded) {
			this._isModePanelExpanded = expanded;
		};

		/** 
		 * This method should return a list of widgets that should appear in the modepanel (or an empty array if none should be shown)
		 * @returns {Array.<String>} widget names of the widgets shown in the mode panel
		 */
		LayoutMode.prototype.getModePanelContent = function () {
			return [];
		};

		/**
		 * Get the name of the widget which should be shown in the Main Panel
		 * By default this is the ViewPortWidget (note this is only supported by the mobile layout at the moment)
		 * @returns {String} widget name of the widget shown in the main panel
		 */
		LayoutMode.prototype.getMainPanelContent = function () {
			return LayoutConstants.Widget.VIEWPORT;
		};


		return LayoutMode;
	});

define('web-viewer/layout/desktop/BasicMode',["web-viewer/layout/LayoutConstants",
		"web-viewer/layout/LayoutMode"
	],
	function (
		LayoutConstants,
		LayoutMode
	) {
		"use strict";

		/**
		 * @exports BasicMode
		 * @memberof Layout
		 * @class
		 * Basic mode of the desktoplayout. Will allow all the layout items to be added and the state is persisted.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 */
		var BasicMode = function (f, layout) {
			LayoutMode.call(this, f, layout, LayoutConstants.LayoutMode.BASIC);
		};

		BasicMode.prototype = new LayoutMode();
		BasicMode.prototype.constructor = BasicMode;

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		BasicMode.prototype.toggle = function (toggle) {
			if (toggle === undefined) {
				toggle = !this._isActive;
			}
			this._isActive = toggle;
		};

		/**
		 * Checks if the item is allowed for this layout mode.
		 *
		 * @param {String} itemname
		 * @returns {Boolean}
		 */
		BasicMode.prototype.isItemAllowed = function (itemname) {
			// prevents documentselector toolbarcomponent
			if (itemname === LayoutConstants.ToolbarComponent.DOCUMENTSELECTOR) {
				return false;
			} else {
				return true; //for basic mode all items are allowed to be visible
			}
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		BasicMode.prototype.isPersisted = function () {
			return true;
		};

		return BasicMode;
	});

define('web-viewer/layout/desktop/RestrictedMode',["web-viewer/layout/LayoutConstants",
		"web-viewer/layout/LayoutMode"
	],
	function (
		LayoutConstants,
		LayoutMode
	) {
		"use strict";

		/**
		 * @exports RestrictedMode
		 * @memberof Layout
		 * @class
		 * Restricted mode of the desktoplayout. Will not allow Channels or CAD widgets to be added and the state is persisted.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 */
		var RestrictedMode = function (f, layout) {
			LayoutMode.call(this, f, layout, LayoutConstants.LayoutMode.RESTRICTED);
		};

		RestrictedMode.prototype = new LayoutMode();
		RestrictedMode.prototype.constructor = RestrictedMode;

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		RestrictedMode.prototype.toggle = function (toggle) {
			if (toggle === undefined) {
				toggle = !this._isActive;
			}
			this._isActive = toggle;
		};

		/**
		 * Checks if the item is allowed for this layout mode.
		 *
		 * @param {String} itemname
		 * @returns {Boolean}
		 */
		RestrictedMode.prototype.isItemAllowed = function (itemname) {
			// prevents channels widget, cad widget, viewcontrols widget, documentselector toolbarcomponent
			if (itemname === LayoutConstants.Widget.CHANNELS || itemname === LayoutConstants.Widget.CAD || itemname === LayoutConstants.Widget.VIEWCONTROLS || itemname === LayoutConstants.ToolbarComponent.DOCUMENTSELECTOR) {
				return false;
			} else {
				return true;
			}
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		RestrictedMode.prototype.isPersisted = function () {
			return true;
		};

		return RestrictedMode;
	});

define('web-viewer/layout/desktop/WizardMode',["web-viewer/layout/LayoutConstants",
		"web-viewer/layout/LayoutMode",
		"underscore"
	],
	function (
		LayoutConstants,
		LayoutMode,
		_
	) {
		"use strict";

		/**
		 * @exports WizardMode
		 * @memberof Layout
		 * @class
		 * Wizard mode of the desktoplayout. Will allow all the layout items to be added and the state is persisted.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 */
		var WizardMode = function (f, layout) {
			LayoutMode.call(this, f, layout, LayoutConstants.LayoutMode.WIZARD);

			/**
			 * Saves the names of all modal widgets that are hidden
			 * @member {Array.<String>}
			 */
			this._hiddenModalWidgets = [];

			/**
			 * Saves the visible state of toolbarComponent, maps ItemName to Boolean. Initializes to all the toolbarComponents.
			 * @member {Object.<String, Boolean>}
			 */
			this._wasToolbarComponentVisible = _.invert(_.values(LayoutConstants.ToolbarComponent));

			/**
			 * Save all the open popup when the wizard is visible to be able to hide them
			 */
			this._openPopups = [];
		};

		WizardMode.prototype = Object.create(LayoutMode.prototype);
		WizardMode.prototype.constructor = WizardMode;

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		WizardMode.prototype.toggle = function (toggle) {
			toggle = toggle === undefined ? !this._isActive : toggle;
			this._isActive = toggle;
			if (toggle) {
				this._hideSidePanels();

				this._layout.showItem(LayoutConstants.Widget.ALIGNWIZARD); //show alignment widget

				this._hideModalWidgets();

				this._hidePopups();

				this._hideToolbarComponents();
			} else {
				this._showSidePanels();

				this._layout.hideItem(LayoutConstants.Widget.ALIGNWIZARD); //hide alignment widget

				this._showModalWidgets();

				this._showPopups();

				this._showToolbarComponents();
			}
		};


		/**
		 * Hide the side panels
		 */
		WizardMode.prototype._hideSidePanels = function () {
			this._layout.disablePanelInteraction(); //disable panel interaction, not possible to open side panels
			this._layout.disablePanel("east");
			this._layout.disablePanel("west");

		};

		/**
		 * Show the side panels
		 */
		WizardMode.prototype._showSidePanels = function () {
			this._layout.initializePanelInteraction(); //make sure to enable panel interaction again
			this._layout.enablePanel("east");
			this._layout.enablePanel("west");
		};


		/**
		 * Hide modal widgets
		 */
		WizardMode.prototype._hideModalWidgets = function () {
			//hides all modal widgets and returns the names of the hidden elements
			this._hiddenModalWidgets = this._layout.hideAllModalWidgets();
		};
		/**
		 * Show the modal widget  that were previously hidden
		 */
		WizardMode.prototype._showModalWidgets = function () {
			_.each(this._hiddenModalWidgets, function (widgetName) { //run over the hidden modal widgets
				this._layout.showItem(widgetName); //show them again
			}, this);
		};



		/**
		 * Hide all popups in the viewport
		 */
		WizardMode.prototype._hidePopups = function () {
			//gets all popups to hide and returns those popups to show them when wizardMode ends
			this._openPopups = this._layout.getAllVisiblePopups();
			this._openPopups.hide();
		};

		/**
		 * Show all the popups that were previously hidden
		 */
		WizardMode.prototype._showPopups = function () {
			this._openPopups.show();
			this._openPopups = [];
		};


		/**
		 * Hide all toolbar components except for navigation controls
		 */
		WizardMode.prototype._hideToolbarComponents = function () {
			//run over all the toolbarComponents and check if they are visible
			_.each(this._wasToolbarComponentVisible, function (wasVisible, itemName) {
				//save the visible state of each toolbarcomponent
				this._wasToolbarComponentVisible[itemName] = this._layout.isItemVisible(itemName);
				if (this._wasToolbarComponentVisible[itemName] //only hide items that are visible
					&& itemName !== LayoutConstants.ToolbarComponent.NAVIGATE) { //and are not the navigate controls
					this._layout.hideItem(itemName);
				}
			}, this);
		};

		/** 
		 * Show all toolbar ocmponetns that were previously hidden
		 */
		WizardMode.prototype._showToolbarComponents = function () {
			_.each(this._wasToolbarComponentVisible, function (wasVisible, itemName) { //run over all toolbarcomponents
				if (wasVisible) { //only if it was visible, show it again
					this._layout.showItem(itemName);
				}
			}, this);
		};


		/**
		 * Checks if the item is allowed for this layout mode. Prevents channels and CAD widget from loading.
		 * 
		 * @param {String} itemname
		 * @returns {Boolean}
		 */
		WizardMode.prototype.isItemAllowed = function (itemname) {
			return itemname === LayoutConstants.Widget.ALIGNWIZARD; //don't add anything when in wizard mode
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		WizardMode.prototype.isPersisted = function () {
			return false;
		};

		return WizardMode;
	});

define('web-viewer/layout/desktop/DesktopLayout',["web-viewer/layout/Layout",
		"web-viewer/layout/DraggableWidgets",
		"web-viewer/layout/WidgetWrapper",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/layout/desktop/Panel",
		"web-viewer/layout/desktop/SidePanel",
		"web-viewer/layout/desktop/BasicMode",
		"web-viewer/layout/desktop/RestrictedMode",
		"web-viewer/layout/desktop/WizardMode",
		"web-viewer/core/PrivateChannel",
		"utils/ShapeUtils",
		"web-viewer/utils/ListenerUtil",
		"underscore",
		"jquery"
	],
    function (
        Layout,
        DraggableWidgets,
        WidgetWrapper,
        LayoutConstants,
        Panel,
        SidePanel,
        BasicMode,
        RestrictedMode,
        WizardMode,
        PrivateChannel,
        ShapeUtils,
        ListenerUtil,
        _,
        $
    ) {
        "use strict";

        /**
         * @const minimum width of viewport before autocollapse occurs
         */
        var AUTOCOLLAPSE_THRESHOLD = 45;
        /**
         * @const default width of panels
         */
        var DEFAULT_PANELWIDTH = 300;

        /**
         * @class
         * Layout for desktop screens. This is the view initialized by the LayoutManager that will add all the layoutcomponents
         * to the viewer. It can generate a String representing the current state of the viewer and can be initialized by
         * a state-String.
         *
         * The desktop layout defines 4 main content areas: nord, west, east and center. These hold different kinds of
         * components. The layout is not responsible for rendering in the viewport.
         *
         * @exports DesktopLayout
         * @constructor
         * @param {Core.ViewerFacade} f
         * @param {LayoutModel} layoutModel
         * @param {JQueryObj} context - context where the layout will add its components to
         *                            @param {Object} layoutState
         * @param {LoadingIndicator} loadingIndicator - the loading indicator, which was created before all requests started
         * @param {JQueryObj} initialLoader - the JQuery reference to the element indicating the initial loader (needed to safely remove it without emptying the context)
         */
        var DesktopLayout = function (f, layoutModel, context, layoutState, loadingIndicator, initialLoader) {
            Layout.call(this, f, layoutModel, context, layoutState, loadingIndicator, initialLoader);
            ListenerUtil.mixin(this, f);
            PrivateChannel.installTo(this, f, "layout.desktop");

			
			
            /**
             * Default state of the layout, used to reset the layout
             */
            this._defaultLayout = {};

            /**
             * Mapping for all the items, maps items to their section
             */
            this._sectionMapping = {};


            /**
             * All the layout items mapped as Name -> WidgetWrapper
             */
            this._items = {};

            /**
             * The component that is used to display the content in the viewport: graphics, CAD, annotations,...
             */
            this._mainContent = null;

            //register all possible modes to the model
            this._layoutModel.registerLayoutMode(new RestrictedMode(this.f, this));
            this._layoutModel.registerLayoutMode(new BasicMode(this.f, this));
            this._layoutModel.registerLayoutMode(new WizardMode(this.f, this));

            /**
             * Inidicates whether the CSS is available or not
             */
            this._cssLoaded = false;


            //initialize the correct mode for the desktoplayout
            this._switchLayoutMode();

            //DOM-elements to build the layout 
            this._wrapper = $("<div/>").addClass("wrapper disableTextSelection");
            this._north = $("<div/>").addClass("northPane");
            this._mainWrapper = $("<div/>");
            this._topbar = $("<div/>").addClass("topbar");
            this._messagebar = $("<div />").addClass("messagebar");
            this._west = $("<div/>").addClass("westPane");
            this._innerWest = $("<div/>").addClass("accordeon").addClass("inner");
            this._center = $("<div/>").addClass("centerPane");
            this._east = $("<div/>").addClass("eastPane");
            this._innerEast = $("<div/>").addClass("accordeon").addClass("inner");
            this._aboveCenter = $("<div/>").addClass("aboveCenter");

            //construct the layout with the previously defined DOM-elements
            this._west.append(this._innerWest);
            this._east.append(this._innerEast);
            this._mainWrapper.append(this._west).append(this._aboveCenter).append(this._center).append(this._east);
            this._wrapper.append(this._topbar).append(this._north).append(this._messagebar).append(this._mainWrapper).append(this._loadingIndicator.element);


            /**
             * Information on the current layout (state of the panels,...)
             */
            this._layout = {
                width: 0,
                height: 0,
                top: new Panel(f, "topbar", this._topbar),
                north: new Panel(f, "north", this._north),
                east: new SidePanel(f, "east", this._east, this._innerEast),
                west: new SidePanel(f, "west", this._west, this._innerWest),
                center: new Panel(f, "center", this._center)
            };

            this._layout.east.setWidth(DEFAULT_PANELWIDTH);
            this._layout.west.setWidth(DEFAULT_PANELWIDTH);

            this._enforceRestrictedMode();

            /**
             * Maps all sections to their JQuery-objects
             */
            this._sections = {
                'default': this._innerWest,
                'west': this._innerWest,
                'north': this._north,
                'east': this._innerEast,
                'center': this._mainWrapper
            };

            //a debounced invalidate, to make sure not to many invalidates are triggered
            this._debouncedInvalidate = _.debounce(_.bind(this._invalidateLayout, this), 200);
            this._deferredInvalidate = function () {
                _.defer(_.bind(this._invalidateLayout, this));
            };

            /**
             * Invalidate of the panels in the layout is debounced (with 100ms)
             */
            this._debouncedPanelInvalidate = _.debounce(_.bind(function () {
                this._layout.east.invalidate();
                this._layout.west.invalidate();
                this._emitLayoutChanged();
            }, this), 100);


            var onCSSLoaded = _.after(2, _.bind(function () {
                this._cssLoaded = true;
				this._initialLoader.remove();
				delete this._initialLoader;
                this._context.append(this._wrapper);
                this._deferredInvalidate();
            }, this));


            this.f.emit("layout.loadCSS", "style/css/style.css", onCSSLoaded);
            this.f.emit("layout.loadCSS", "web-viewer/layout/desktop/css/desktop.css", onCSSLoaded);

            //listen to resize events of the window
            $(window).resize(_.bind(function (e) {
                // http://bugs.jqueryui.com/ticket/7514: resize of popups bubbles to window
                if (e.target === window) {
                    this._debouncedInvalidate();
                }
            }, this));

            // constrain all widgets when window is resized
            $(window).resize(_.debounce(_.bind(function (e) {
                // http://bugs.jqueryui.com/ticket/7514: resize of popups bubbles to window
                if (e.target === window) {
                    this._constrainAllModalWidgets();
                }
            }, this), 500));

            //make sure the default layout is initialized
            this._initializeDefaultLayout();
            //initialize the section mapping of the layoutItems
            this._initializeSectionMapping(this._layoutState);
            //initialize dragging for widgets
            this._initializeDragging();
            //initialize panels, size and visibility 
            this._initializePanels();
            //initialize all behaviour for resizing/hiding panels
            this.initializePanelInteraction();
            //listen to widget events, to act on resized/collapsed/...
            this._registerOnLayoutEvents();

            // webkit has a bug with iFrames, which appearantly they won't be solving soon
            // if an input in an iframe is focused, the page scrolls to the iframe
            // this can occur when TinyMCE is used... we can fix it by scrolling to top whenever the context is scrolled
            context.scroll(function (event) {
                $(this).scrollTop(0);
            });



            this._invalidateLayout(); //this one is needed to provide the correct world coordinates
        };

        DesktopLayout.prototype = Object.create(Layout.prototype);
        DesktopLayout.prototype.constructor = DesktopLayout;

        /**
         * Initializes the default layout. This is the layout that's used when the viewer is opened for the first time.
         * The layout can be reseted to this layout. It maps all the components to the correct section.
         */
        DesktopLayout.prototype._initializeDefaultLayout = function () {
            this._defaultLayout = {
                "north": [],
                "east": [],
                "center": [],
                "west": []
            };
            this._defaultLayout.north = [LayoutConstants.ToolbarComponent.PAGELIST,
				LayoutConstants.ToolbarComponent.NAVIGATE,
				LayoutConstants.ToolbarComponent.MEASURE,
                LayoutConstants.ToolbarComponent.TEXTSELECTION,
				LayoutConstants.ToolbarComponent.COMPARE,
				LayoutConstants.ToolbarComponent.ANNOTATION,
				LayoutConstants.ToolbarComponent.DYNCONTENT,
				LayoutConstants.ToolbarComponent.VIEW3D,
				LayoutConstants.ToolbarComponent.APPROVAL
			];
            this._defaultLayout.east = [LayoutConstants.Widget.NAVIGATOR,
				LayoutConstants.Widget.VIEW3D,
				LayoutConstants.Widget.VIEWCONTROLS,
				LayoutConstants.Widget.CHANNELS,
				LayoutConstants.Widget.CAD,
				LayoutConstants.Widget.DEBUG,
				LayoutConstants.Widget.APPROVAL,
				LayoutConstants.Widget.INFO,
				LayoutConstants.Widget.HELP
			];
            this._defaultLayout.west = [LayoutConstants.Widget.COMPARE,
				LayoutConstants.Widget.ATTRIBUTE_CATEGORY,
				LayoutConstants.Widget.ANNOTATION,
				LayoutConstants.Widget.PAGELIST,
				LayoutConstants.Widget.TEXTCONTENT
			];

        };

        /**
         * Initializes the section mapping. All items will be mapped to their section so it's easier to find the item's section.
         * Items that aren't in the layoutstate but are in the default state will also be added to the mapping.
         *
         * @param {Object} [layoutState] - if not defined only the default layout is used for the mapping
         */
        DesktopLayout.prototype._initializeSectionMapping = function (layoutState) {
            //first run over all the sections and items in the layoutState
            if (layoutState !== undefined && layoutState !== "" && layoutState != {}) {
                _.each(layoutState, function (section, sectionName) { //runs over all sections
                    _.each(section.items, function (item) { //runs over all items in those sections
                        this._sectionMapping[item.name] = sectionName; //map each itemName->sectionName
                    }, this);
                }, this);
            }
            //run over the defaultLayout, if items of the default layout aren't added yet, add them to the mapping
            _.each(this._defaultLayout, function (items, sectionName) {
                _.each(items, function (item) {
                    if (this._sectionMapping[item] === undefined) { //check if item isn't added by layoutState yet
                        this._sectionMapping[item] = sectionName;
                    }
                }, this);
            }, this);
        };

        /**
         * Register listeners for general layout event.
         */
        DesktopLayout.prototype._registerOnLayoutEvents = function () {
            this.f.on("layout.disableTextSelection", _.bind(function () {
                this._disableTextSelection();
            }, this));
            this.f.on("layout.enableTextSelection", _.bind(function () {
                this._enableTextSelection();
            }, this));
            this.f.on("layout.resetToDefault", _.bind(this.resetToDefault, this));
        };

        /**
         * Initialize draggable widgets, makes it possible to move widgets.
         */
        DesktopLayout.prototype._initializeDragging = function () {
            this._draggableWidgets = new DraggableWidgets(this._wrapper);
            //add dropzones for widgets, these are the parts widgets can be moved to
            this._draggableWidgets.addModalWidgetDropzone(this._center, "center", this);
            this._draggableWidgets.addSortableWidgetDropzone(this._innerWest, "west", this);
            this._draggableWidgets.addSortableWidgetDropzone(this._innerEast, "east", this);
        };

        /**
         * Initialize panel interaction. Adds listeners to the global wrapper div-element. It checks if
         * the users wants to resize/collapse the side panels.
         */
        DesktopLayout.prototype.initializePanelInteraction = function () {
            var THRESHOLD = 7;
            var draggingHandle;
            var previousDragX;
            var maxWidth;
            var minWidth = 100;
            var hideTreshold = 30;
            var dragDirection = 1;

            var inWestHandle = _.bind(function (x, y) {
                return this._layout.west.isVisible() && Math.abs(x - this._layout.west.getWidth()) < THRESHOLD && y >= this._layout.west.position().top;
            }, this);

            var inEastHandle = _.bind(function (x, y) {
                return this._layout.east.isVisible() && Math.abs(x - this._layout.west.getWidth() - this._layout.center.getWidth()) < THRESHOLD && y >= this._layout.east.position().top;
            }, this);

            this._registerListener("mousedown", function (e) {
                if (inWestHandle(e.clientX, e.clientY)) {
                    draggingHandle = this._layout.west;
                    dragDirection = 1;
                } else if (inEastHandle(e.clientX, e.clientY)) {
                    draggingHandle = this._layout.east;
                    dragDirection = -1;
                }
                if (draggingHandle) {
                    previousDragX = e.clientX;
                    maxWidth = draggingHandle.getWidth() + this._layout.center.getWidth() - (AUTOCOLLAPSE_THRESHOLD + 5);
                    this.f.emit("layout.disableTextSelection");
                }
            }, this, this._wrapper);


            this._registerListener("mousemove", function (e) {
                if (draggingHandle !== undefined || inWestHandle(e.clientX, e.clientY) || inEastHandle(e.clientX, e.clientY)) {
                    this._wrapper.toggleClass("panelResize", true);
                } else {
                    this._wrapper.toggleClass("panelResize", false);
                }
                if (draggingHandle !== undefined) {
                    var diff = dragDirection * (e.clientX - previousDragX);
                    var newWidth = draggingHandle.getWidth() + diff;

                    if (newWidth > minWidth && newWidth < maxWidth) {
                        previousDragX = e.clientX;
                        draggingHandle.setWidth(newWidth);
                        this._layout.center.setWidth(this._layout.width - this._layout.east.outerWidth(true) - this._layout.west.outerWidth(true));
                    }

                    if (newWidth < hideTreshold) {
                        draggingHandle.hide();
                        draggingHandle.setWidth(DEFAULT_PANELWIDTH);
                        draggingHandle = undefined;
                        this.f.emit("layout.enableTextSelection");
                        this._deferredInvalidate();
                    }
                }

            }, this, this._wrapper);

            this._registerListener("mouseout", function (e) {
                this._wrapper.toggleClass("panelResize", false);
                if (draggingHandle && e.target == this._wrapper[0]) {
                    draggingHandle = undefined;
                    this.f.emit("layout.enableTextSelection");
                    this._deferredInvalidate();
                }
            }, this, this._wrapper);

            this._registerListener("mouseup", function (e) {
                if (draggingHandle) {
                    this.f.emit("layout.enableTextSelection");
                    draggingHandle = undefined;
                    this._deferredInvalidate();
                }
            }, this, this._wrapper);

        };

        /**
         * Disable all possible interaction with the side panels.
         */
        DesktopLayout.prototype.disablePanelInteraction = function () {
            this._unregisterAllListenersForObject(this._wrapper);
        };

        /**
         * Initializes the panels, gets the collapsed/width state from the layoutState.
         */
        DesktopLayout.prototype._initializePanels = function () {
            if (this._layoutState) {
                _.each(this._layoutState, function (section, sectionName) { //run over all sections
                    if (this._layout[sectionName] !== undefined) {
                        this._layout[sectionName].restoreState(section);
                    }
                }, this);
            }

            // listen to panels to invalidate layout when toggled
            this._registerListener("changed.visibility", this._deferredInvalidate, this, this._layout.west);
            this._registerListener("changed.visibility", this._deferredInvalidate, this, this._layout.east);


            this._toggleButtonWest = this._layout.west.toggleButton.element;
            this._toggleButtonEast = this._layout.east.toggleButton.element;
            this._wrapper.append(this._toggleButtonWest);
            this._wrapper.append(this._toggleButtonEast);
        };

        /**
         * Returns the default layout mode for the layout.
         * @return {LayoutMode}
         */
        Layout.prototype._getDefaultLayoutMode = function () {
            if (this.f.getProperty("viewerStyle") == LayoutConstants.ViewerStyle.RESTRICTED) { //is the viewer loaded in restricted style (hide channels and CAD widgets)
                return this._layoutModel.getLayoutMode(LayoutConstants.LayoutMode.RESTRICTED);
            } else {
                return this._layoutModel.getLayoutMode(LayoutConstants.LayoutMode.BASIC);
            }
        };

        /**
         * Add an item to the layout. This is called by Layout to add an item.
         * @param {Object} layoutItem
         * @example
         * 	{ 	type: {LayoutConstants.LayoutItemType},
         * 		name: 'itemName',
         * 		item: {AbstractWidget/ToolbarComponent}
         * 	}
         */
        DesktopLayout.prototype._addItem = function (layoutItem) {
            if (layoutItem.type === LayoutConstants.LayoutItemType.WIDGET) {
                var widget = layoutItem.getItem(this.getLayoutType());
                switch (widget.getWidgetType()) { //different layoutitems are of different widgetType
                    case "topbar": //widget needs to be added to the topbar
                        widget.setContext(this._topbar);
                        break;
                    case "maincontent": //widget is part of the maincontent
                        widget.setContext(this._center);
                        this._mainContent = widget;
                        break;
                    case "messagebar": //message bar is visible in the centerpanel 
                        widget.setContext(this._messagebar);
                        break;
                    case "wizard":
                        widget.setContext(this._aboveCenter);
                        this._items[widget.getID()] = widget;
                        break;
                    default: //default widgets are part of either the sidepanels or the center panel
                        var widgetWrapper = new WidgetWrapper(widget, layoutItem.name, true);
                        this._draggableWidgets.enableDragging(widgetWrapper);
                        this._items[layoutItem.name] = widgetWrapper;
                        if (!this._layout[this._sectionMapping[layoutItem.name]].isVisible()) {
                            widgetWrapper.widget.setVisible(false);
                        }
                        this._addLayoutItem(layoutItem.name); //make sure the item is added on the correct location in one of the panels
                        //register listeners to the widget to make sure the layout gets invalidated (if needed) or the state is saved
                        this._registerListener("collapsed", function () {
                            if (widgetWrapper.isModal) { //if model no invalidate is needed as the widget is not part of a side panel
                                this._emitLayoutChanged();
                            } else {
                                this._debouncedPanelInvalidate();
                            }
                        }, this, widget);
                        this._registerListener("modalResize", function () {
                            if (widgetWrapper.isModal) { //make sure the state is saved when modal widget is moved
                                this._emitLayoutChanged();
                            }
                        }, this, widget);
                        this._registerListener("dragged", function () { //the item is dragged, change its location 
                            if (widgetWrapper.isModal) {
                                this._emitLayoutChanged();
                            }
                        }, this, widget);

                        this._registerListener("enabled", function (isEnabled) {
                            this._toggleWidget(layoutItem.name, widgetWrapper, isEnabled);
                            this._debouncedPanelInvalidate();
                        }, this, widget);

                        this._registerListener("resized", this._debouncedPanelInvalidate, this, widget);
                        this._registerListener("makeVisible", _.bind(this._makeWidgetVisible, this, layoutItem.name, widgetWrapper), undefined, widget);

                        if (widget.didRequestMakeVisible()) {
                            this._makeWidgetVisible(layoutItem.name, widgetWrapper);
                        }
                        this._debouncedInvalidate();

                        break;
                }
            } else if (layoutItem.type === LayoutConstants.LayoutItemType.TOOLBARCOMPONENT) {
                //add a toolbarcomponent
                var toolbarItem = layoutItem.getItem(this.getLayoutType());
                toolbarItem.element[0].name = layoutItem.name;
                this._items[layoutItem.name] = toolbarItem;
                this._addLayoutItem(layoutItem.name);
                this._registerListener("changed.visibility", function () {
                    this._invalidateNorthPane();
                    this._debouncedInvalidate();
                }, this, toolbarItem);
                this._debouncedInvalidate();
            } else {
                //not supported in this layout
            }
        };

        /**
         * Add an item to the layout, the items needs to already added to the items-collection.
         * It adds the item to the corect element on the correct location (depends on the previous/default state and already added items)
         * @param {String} itemName
         */
        DesktopLayout.prototype._addLayoutItem = function (itemName) {
            var section = this._sectionMapping[itemName]; //get the section the items belongs to
            if (section !== undefined) { //make sure there is a section to add the item to
                var sectionItems = null;
                var itemState = null;
                if (this._layoutState) {
                    sectionItems = _.map(this._layoutState[section].items, function (item) {
                        return item.name;
                    }); //just get all the names of the items in this section, not other information
                    itemState = _.find(this._layoutState[section].items, function (item) {
                        return item.name === itemName;
                    }); //get the state of the to be added item
                } else {
                    sectionItems = this._defaultLayout[section]; //default state only holds names of the items
                }
                if (sectionItems !== null && sectionItems !== undefined) { //the section should at least hold the item itself
                    var index = _.indexOf(sectionItems, itemName);
                    var nextItem = null;
                    if (index === -1) {
                        index = _.indexOf(this._defaultLayout[section], itemName);
                        nextItem = this._items[this._defaultLayout[section][++index]];
                        while (index < this._defaultLayout[section].length && nextItem === undefined) { //as long nextItem is not yet added to the layout, search for a next one
                            nextItem = this._items[this._defaultLayout[section][++index]];
                        }
                    } else {
                        nextItem = this._items[sectionItems[++index]]; //get the next item, so we know infront of which item the item needs to be added to
                        while (index < sectionItems.length && nextItem === undefined) { //as long nextItem is not yet added to the layout, search for a next one
                            nextItem = this._items[sectionItems[++index]];
                        }
                    }

                    if (nextItem === undefined) { //no nextItem is found, the item is (for now) the last item in the section
                        this._appendItemToSection(itemName, section); //append the item to the section
                    } else { //add the item in front of the found nextItem
                        this._prependItemBeforeItem(nextItem.name, itemName, section);
                    }
                    //now be sure to set the correct state of the item (collapsed/size/location)
                    if (itemState !== null && itemState !== undefined) {
                        this._setCollapsed(itemName, itemState.isCollapsed);
                        if (itemState.width !== undefined || itemState.height !== undefined) {
                            this._setSize(itemName, new ShapeUtils.Size(itemState.width, itemState.height));
                        }
                        if (itemState.left !== undefined || itemState.top !== undefined) {
                            this._setLocation(itemName, new ShapeUtils.Point(itemState.left, itemState.top));
                        }
                    }

                }
            }
        };

        /**
         * Appends an already added item (to the items collection) to a section.
         * @param {String} itemName
         * @param {String} section
         */
        DesktopLayout.prototype._appendItemToSection = function (itemName, section) {
            var layoutItem = this._items[itemName];
            if (layoutItem.widget) {
                layoutItem.widget.movingStarted();
            }

            this._sections[section].append(layoutItem.element);

            if (layoutItem.widget) {
                layoutItem.widget.movingStopped();
            }
        };

        /**
         * Prepends an already added item (to the items collection) before the nextItem in a given section.
         * @param {String} nextItemName
         * @param {String} itemName
         * @param {String} section
         */
        DesktopLayout.prototype._prependItemBeforeItem = function (nextItemName, itemName, section) {
            var layoutItem = this._items[itemName];
            if (layoutItem.widget) {
                layoutItem.widget.movingStarted();
            }
            //add item before other item in the layout
            this._items[nextItemName].element.before(layoutItem.element);

            if (layoutItem.widget) {
                layoutItem.widget.movingStopped();
            }
        };

        /**
         * Moves an item to another section. Is used by draggable widgets to let the layout now it is moved.
         * @param {String} itemName
         * @param {String} sectionName
         * @param {String} nextItemName - item will be moved before this item
         * @param {Boolean} resetSizeAndLocation
         * @param {Boolean} resetCollapsed
         */
        DesktopLayout.prototype.moveItem = function (itemName, sectionName, nextItemName, resetSizeAndLocation, resetCollapsed) {
            if (itemName !== nextItemName) {
                if (resetSizeAndLocation) { //resets the size and location of the item (moves from center to a side-panel)
                    this._setLocation(itemName, null);
                    this._setSize(itemName, null);
                }
                if (resetCollapsed) { //resets the collapsed state of the item
                    this._setCollapsed(itemName, false);
                }
                var currentSection = this._sectionMapping[itemName];
                this._detachItem(itemName, currentSection); //removes the item from the previous section
                this._sectionMapping[itemName] = sectionName;
                if (nextItemName === undefined || this._sectionMapping[nextItemName] !== sectionName) {
                    this._appendItemToSection(itemName, sectionName);
                } else {
                    //check if no problems with after parameter
                    this._prependItemBeforeItem(nextItemName, itemName, sectionName);
                }
            }
        };

        /**
         * Set the location of an item. It will make sure the item stays within the window bounds.
         * If no location is provide the item will no longer be modal (is moved to one of the side-panels)
         * @param {String} itemName
         * @param {ShapeUtils.Point} location
         */
        DesktopLayout.prototype._setLocation = function (itemName, location) {
            var widgetWrapper = this._items[itemName];

            if (widgetWrapper !== undefined && widgetWrapper.widget) {
                if (location === null || location === undefined) {
                    widgetWrapper.setModal(false);
                } else {
                    this._constrainModalWidgetWrapper(widgetWrapper, location);
                }
            }
        };

        /**
         * Constrain a widgetwrapper to the windowsize/viewerapp size
         * @param {WidgetWrapper} widgetWrapper - the widgetWrapper that should be constrained to the window
         * @param {ShapeUtils.Size}   location - the wanted location of the the widgetwrapper
         */
        DesktopLayout.prototype._constrainModalWidgetWrapper = function (widgetWrapper, location) {
            var itemSize = widgetWrapper.getOuterSize();

            //get the location so the widget stays inside the window, will change the location if the widget lays outside the window
            if (location.x > this._context.innerWidth() - itemSize.width) {
                location.x = this._context.innerWidth() - itemSize.width;
            }
            if (location.y > this._context.innerHeight() - itemSize.height) {
                location.y = this._context.innerHeight() - itemSize.height;
            }

            widgetWrapper.setModal(true, location.y, location.x);
        };

        /**
         * Make sure all modal widgets are visible
         */
        DesktopLayout.prototype._constrainAllModalWidgets = function () {
            _.each(this._items, function (item) {
                if (item.isModal) {
                    var widgetWrapper = this._items[item.name];
                    if (widgetWrapper !== undefined && widgetWrapper.widget) {
                        this._constrainModalWidgetWrapper(widgetWrapper, item.getLocation());
                    }
                }
            }, this);
        };

        /**
         * Set a widget to collapsed (only possible if the layout item is a widget)
         * @param {String} itemName
         * @param {Boolean} collapsed
         */
        DesktopLayout.prototype._setCollapsed = function (itemName, collapsed) {
            var widgetWrapper = this._items[itemName];
            if (widgetWrapper !== undefined && widgetWrapper.widget) {
                widgetWrapper.setCollapsed(collapsed);
            }
        };

        /**
         * Hide side panel.
         * @param {String} panelName
         */
        DesktopLayout.prototype.disablePanel = function (panelName) {
            this._layout[panelName].setDisabled(true);
        };

        /**
         * Show side panel.
         * @param {String} panelName
         */
        DesktopLayout.prototype.enablePanel = function (panelName) {
            this._layout[panelName].setDisabled(false);
        };

        /**
         * Is the panel visible.
         * @param {String} panelName
         * @returns {Boolean}
         */
        DesktopLayout.prototype.isPanelVisible = function (panelName) {
            return this._layout[panelName].isVisible();
        };

        /**
         * Sets the size of the item, if no size is provided the item will no longer be modal (it is added to one of the side panels)
         * @param {String} itemName
         * @param {ShapeUtils.Size} size
         */
        DesktopLayout.prototype._setSize = function (itemName, size) {
            var widgetWrapper = this._items[itemName];
            if (widgetWrapper !== undefined && widgetWrapper.widget) {
                if (size === null || size === undefined) {
                    widgetWrapper.setModal(false);
                } else {
                    var widgetWrapperLocation = widgetWrapper.getLocation();
                    widgetWrapper.setModal(true, widgetWrapperLocation.y, widgetWrapperLocation.x, size.width, size.height);
                }
            }

        };

        /**
         * Remove an item from a section, it removes the item from the DOM, but doesn't delete the DOM-element.
         * @param {String} itemName
         * @param {String} sectionName
         */
        DesktopLayout.prototype._detachItem = function (itemName, sectionName) {
            var widgetWrapper = this._items[itemName];
            if (widgetWrapper) {
                if (widgetWrapper.widget) {
                    widgetWrapper.widget.movingStarted();
                }

                //remove element from the DOM (doesn't delete DOM-element)
                this._items[itemName].element.detach();

                if (widgetWrapper.widget) {
                    widgetWrapper.widget.movingStopped();
                }
                //the invalidate is only done after all other changes (e.g. re-add DOM-element) are done
                _.defer(_.bind(function () {
                    this._invalidateLayout();
                }, this));
            }
        };

        /**
         * Hide a layout item.
         * @param {LayoutConstants.Widget|LayoutConstants.ToolbarComponent} itemName
         */
        DesktopLayout.prototype.hideItem = function (itemName) {
            var layoutItem = this._items[itemName];
            if (layoutItem !== undefined && layoutItem.context !== undefined) {
                layoutItem.context.hide();
            } else if (layoutItem !== undefined && layoutItem.element !== undefined) {
                layoutItem.element.hide();
            }
        };

        /**
         * Makes a hidden layout item visible.
         * @param {LayoutConstants.Widget|LayoutConstants.ToolbarComponent} itemName
         */
        DesktopLayout.prototype.showItem = function (itemName) {
            var layoutItem = this._items[itemName];
            if (layoutItem !== undefined && layoutItem.context !== undefined) {
                layoutItem.context.show();
            } else if (layoutItem !== undefined && layoutItem.element !== undefined) {
                layoutItem.element.show();
            }
        };

        /**
         * Is the layout item visible.
         * @param {String} itemName
         * @returns {Boolean}
         */
        DesktopLayout.prototype.isItemVisible = function (itemName) {
            var layoutItem = this._items[itemName];
            if (layoutItem !== undefined) {
                return layoutItem.isVisible();
            }
        };

        /**
         * Hide all modal widgets.
         * @returns {Array.<String>} - returns all the names of the hidden widgets
         */
        DesktopLayout.prototype.hideAllModalWidgets = function () {
            var modalWidgets = [];
            _.each(this._items, function (item) {
                if (item.isModal) {
                    modalWidgets.push(item.name);
                    this.hideItem(item.name);
                }
            }, this);
            return modalWidgets;
        };

        /**
         * Get all the currently visible popups
         * @returns {Array.<JQueryObj>}
         */
        DesktopLayout.prototype.getAllVisiblePopups = function () {
            return this._context.find(".popup:visible");
        };

        /**
         * Disable text selection in the entire viewer.
         */
        DesktopLayout.prototype._disableTextSelection = function () {
            this._context.find(".enableTextSelection").toggleClass("enableTextSelection", false).toggleClass("enableTextSelectionTempDisabled", true);
        };

        /**
         * Enable text selection in the entire viewer.
         */
        DesktopLayout.prototype._enableTextSelection = function () {
            this._context.find(".enableTextSelectionTempDisabled").toggleClass("enableTextSelection", true).toggleClass("enableTextSelectionTempDisabled", false);
        };

        /**
         * Restore a certain state of the layout to the state provided. (not used at this moment)
         * @param {Object} state
         */
        DesktopLayout.prototype.restore = function (state) {
            _.each(state, _.bind(function (section, sectionName) {
                this._layout[sectionName].restoreState(section);

                _.each(section.items, _.bind(function (item) {
                    if (this._items[item.name] !== undefined && this._activeMode.isItemAllowed(item.name)) {
                        if (item.isCollapsed) {
                            this._items[item.name].setCollapsed(item.isCollapsed);
                        }
                        if (item.width && item.height) {
                            this._items[item.name].setSize(new ShapeUtils.Size(item.width, item.height));
                        }
                        if (item.left && item.top) {
                            this._items[item.name].setLocation(new ShapeUtils.Point(item.left, item.top));
                        }
                        this.moveItem(item.name, sectionName);
                    }
                }, this));
            }, this));
        };

        /**
         * Reset the sate of the layout to the default state.
         */
        DesktopLayout.prototype.resetToDefault = function () {
            _.each(this._defaultLayout, function (section, sectionName) {
                this._layout[sectionName].setDisabled(false); //make sure all panels are enabled
                this._layout[sectionName].toggle(true); //make all the panels visible (this is the default at this moment)
                if (sectionName === "west" || sectionName === "east") { //reset the width of the side panels to the default
                    this._layout[sectionName].setWidth(DEFAULT_PANELWIDTH);
                    this._layout[sectionName].show();
                }
                _.each(section, function (itemName) { //run over each item in the section
                    var item = this._items[itemName];
                    if (item !== undefined && this._activeMode.isItemAllowed(itemName)) { //move the item to its default location.
                        this.moveItem(itemName, sectionName, undefined, true, true);
                    }
                }, this);
            }, this);
            this._layoutState = undefined; //reset the state of the layout, it now is the default\

            this._enforceRestrictedMode();

            this._invalidateLayout();
        };

        /**
         * Generates the state of the layout as a string. It takes the not-visible elements into account so state of those
         * elements is not lost.
         * @returns {Object}
         */
        DesktopLayout.prototype.getLayoutState = function () {
            var stateObject = {};
            _.each(this._sections, _.bind(function (section, sectionName) {
                if (this._layout[sectionName] !== undefined) {

                    stateObject[sectionName] = this._layout[sectionName].getState();
                    stateObject[sectionName].items = [];

                    _.each(this._sections[sectionName].children(), _.bind(function (item) {
                        if (item !== null && item !== undefined) {
                            var itemWrapper = item.wrapper;
                            var itemObject = {};
                            if (itemWrapper === undefined) {
                                itemObject.name = item.name;
                                itemObject.isCollapsed = !$(item).is(':visible');
                            } else {
                                itemObject.name = itemWrapper.name;
                                itemObject.isCollapsed = itemWrapper.isCollapsed;

                                var itemLocation = itemWrapper.getLocation();
                                if (itemLocation !== null && itemWrapper.isModal) {
                                    itemObject.left = itemLocation.x;
                                    itemObject.top = itemLocation.y;
                                }
                                var itemSize = itemWrapper.getSize();
                                if (itemSize !== null && itemWrapper.isModal) {
                                    itemObject.width = itemSize.width;
                                    itemObject.height = itemSize.height;
                                }
                            }

                            if (itemObject.name !== undefined) {
                                stateObject[sectionName].items.push(itemObject);
                            }
                        }
                    }, this));
                }
            }, this));

            //construct a mapping of all the items per section
            var layoutItemsPerSection = {};
            _.each(this._layoutState, function (section, sectionName) {
                layoutItemsPerSection[sectionName] = [];
                _.each(section.items, function (item) {
                    layoutItemsPerSection[sectionName].push(item.name);
                });
            });

            //if there is a layout state defined get the state of non-visible element (elements that have not yet been added) from here.
            if (this._layoutState) {
                //runs over all the elements of the state, if the item wasn't added to the state yet, add it
                _.each(this._layoutState, function (section, sectionName) {
                    _.each(section.items, function (item) {
                        //if the item is not part of the items-collection it has not yet been added.
                        if (this._items[item.name] === undefined && this._layout[sectionName] !== undefined && this._sectionMapping[item.name]) {
                            var section = layoutItemsPerSection[this._sectionMapping[item.name]]; //get the section for the item
                            var itemIndex = _.indexOf(section, item.name); //index of the item in the section
                            itemIndex++; //start running over all the items after the current item and check if they are in the items collection (present in the stateObject)
                            while (itemIndex < section.length && this._items[section[itemIndex]] === undefined) {
                                itemIndex++; //run over the default section untill an item is found that is present in the stateObject

                            }
                            if (itemIndex < section.length) { //there is an item found in items (sits in stateObject)
                                //get the index of this item in the stateObject, place the new item before nextItem
                                var nextItem = _.findWhere(stateObject[sectionName].items, {
                                    name: section[itemIndex]
                                });
                                var index = _.indexOf(stateObject[sectionName].items, nextItem);
                                stateObject[sectionName].items.splice(index, 0, item);
                            } else {
                                stateObject[sectionName].items.push(item); //insert at the end, it's the last item
                            }
                        }
                    }, this);
                }, this);
            }
            this._layoutState = stateObject;
            return stateObject;
        };

        /**
         * Invalidates the layout. Makes sure all panels have the correct size, resize is triggered on the correct elements,...
         */
        DesktopLayout.prototype._invalidateLayout = function () {
            this._layout.height = $(this._context).innerHeight();
            this._layout.width = $(this._context).innerWidth();

            if (this._layout.width === 0 || this._layout.height === 0 || !this._cssLoaded) {
                // it makes no sense to invalidate the layout if the viewer is size(0,0) or no css is loaded
                return;
            }


            this._wrapper.css({
                height: this._layout.height,
                width: this._layout.width
            });


            var contentHeight = this._layout.height;

            // when the browser is zoomed in or out, getting width of elemens becomes very unreliable
            // widths can become floats, we can't rely on default jquery methods to get the correct width
            // if we make an element 1px to wide, the east panel can disappear
            // we can rely on the widths returned by getBoundClientRect() and the border width of the getComputedStyle
            // they return the float "width" values of the elements
            // make sure we ceil and floor (width of side panels should be ceiled, total width floored)
            // worst case scenario: 3 pixels are not used in the horizontal axis... (but users should zoom/in out anyway)
            var realWestWidth = this._layout.west.el.get(0).getBoundingClientRect().width;
            realWestWidth += parseFloat(getComputedStyle(this._layout.west.el.get(0)).borderRightWidth);
            realWestWidth = Math.ceil(realWestWidth);

            var realEastWidth = this._layout.east.el.get(0).getBoundingClientRect().width;
            realEastWidth += parseFloat(getComputedStyle(this._layout.east.el.get(0)).borderLeftWidth);
            realEastWidth = Math.ceil(realEastWidth);

            var realWidth = Math.floor($(this._context).get(0).getBoundingClientRect().width);

            var centerWidth = realWidth - realWestWidth - realEastWidth;

            // if there is not enough space, auto collapse the side panels
            if (centerWidth < AUTOCOLLAPSE_THRESHOLD) {
                if (!this._layout.west.isAutoCollapsed() || !this._layout.east.isAutoCollapsed()) {
                    this._layout.west.setAutoCollapsed(true);
                    this._layout.east.setAutoCollapsed(true);

                    // we did an autocollapse, invalidate the layout, and notify the user
                    this._invalidateLayout();

                    this.f.loadBundle("viewer", function () {
                        this.f.emit("userMessage", "info", this.f.t("infoSidepanelsAutoCollapsed"), undefined, false, "AUTOCOLLAPSE", true);
                    }, this);
                    return;
                }
            } else if (this._layout.west.isAutoCollapsed() || this._layout.east.isAutoCollapsed()) {
                // if can have enough space with both panels open, open them again
                var possibleCenterWidth = this._layout.width - this._layout.west.getWidthWhenNotAutoCollapsed() - this._layout.east.getWidthWhenNotAutoCollapsed();
                if (possibleCenterWidth > AUTOCOLLAPSE_THRESHOLD) {
                    this._layout.west.setAutoCollapsed(false);
                    this._layout.east.setAutoCollapsed(false);

                    // no need to show the autocollapse messages anymore
                    this.f.emit("userMessage.markObsolete", "AUTOCOLLAPSE");
                    this._invalidateLayout();
                    return;
                }
            }

            this._invalidateNorthPane();


            contentHeight -= this._layout.top.outerHeight();
            contentHeight -= this._layout.north.outerHeight();

            if (this._aboveCenter.is(":visible")) {
                contentHeight -= this._aboveCenter.outerHeight();
            }

            this._layout.west.setHeight(contentHeight);
            this._layout.east.setHeight(contentHeight);

            this._layout.center.setDim(centerWidth, contentHeight - 1); // minus 1: take into account the top border of the element

            var centerPosition = this._layout.center.position();

            // Center message bar inside center
            this._messagebar.css({
                width: this._layout.center.outerWidth() - 36,
                top: $(this._context).position().top + this._layout.top.outerHeight() + this._layout.north.outerHeight() + 18,
                left: centerPosition.left + 18
            });

            // Layout loading indicator

            this._loadingIndicator.element.css({
                top: centerPosition.top + 5,
                left: centerPosition.left + this._layout.center.width - 29
            });


            // position toggle buttons
            this._toggleButtonWest.css({
                left: 3,
                top: centerPosition.top - 16 // 16: height of the icon
            });
            this._toggleButtonEast.css({
                left: this._layout.width - 21, // 22 width of the icon + padding
                top: centerPosition.top - 16
            });

            this._layout.east.invalidate();
            this._layout.west.invalidate();

            if (this._mainContent !== null) {
                this._mainContent.resized();
            }

            _.each(this._items, function (layoutItem, itemName) {
                if (this._layoutModel.getLayoutItemType(itemName) === LayoutConstants.LayoutItemType.WIDGET && layoutItem.widget !== undefined) {
                    layoutItem.widget.resized();
                } else if (this._layoutModel.getLayoutItemType(itemName) === LayoutConstants.LayoutItemType.TOOLBARCOMPONENT) {
                    layoutItem.moved();
                }
            }, this);

            this._emitLayoutChanged();
        };


        /**
         * Position the toolbarcomponents in the northpane
         */
        DesktopLayout.prototype._invalidateNorthPane = function () {
            // align the toolbar items to the left of the viewport 
            // if not enough space, move the more to the left
            var northContentWidth = 0; // calculate content width (sum of all children)
            this._layout.north.el.children().each(function () {
                if ($(this).is(':visible')) {
                    northContentWidth += $(this).outerWidth(true) + 2; // add to for borders that are not taken into account in some way...
                }
            });
            if (northContentWidth === 0) {
                // no children yet, so take the whole width
                northContentWidth = this._layout.north.outerWidth();
            }

            var paddingLeft = this._layout.west.outerWidth();
            var availableWidth = this._layout.width - (2 * 30); //-2 * 25 : make space for the collapse icons
            if (paddingLeft + northContentWidth >= availableWidth) {
                paddingLeft -= paddingLeft + northContentWidth - availableWidth;
            }
            paddingLeft = Math.max(paddingLeft, 30);

            this._layout.north.el.css("padding-left", paddingLeft);
        };

        /**
         * Returns the type of the layout.
         * @override
         * @return {Number}
         */
        DesktopLayout.prototype.getLayoutType = function () {
            return LayoutConstants.LayoutType.DESKTOP;
        };


        /**
         * Update visibility of a Widget
         * @param {String}        widgetName    - name of the widget
         * @param {WidgetWrapper} widgetWrapper - widgetWrapper in which the widget is displayed
         * @param {Boolean}       visible       - should the widget be visible?
         */
        DesktopLayout.prototype._toggleWidget = function (widgetName, widgetWrapper, isEnabled) {
            if (isEnabled) {
                if (this._activeMode.isItemAllowed(widgetName)) {
                    widgetWrapper.setVisible(true);
                }
            } else {
                widgetWrapper.setVisible(false);
            }
        };


        /**
         * Enforce restricted mode (collapse side panels)
         */
        DesktopLayout.prototype._enforceRestrictedMode = function () {
            if (this.f.getProperty("viewerStyle") == LayoutConstants.ViewerStyle.RESTRICTED) {
                this._layout.east.toggle(false);
                this._layout.west.toggle(false);
            }
        };

        /**
         * Make sure a widget is visible
         * Uncollapse it if it is collapsed
         *
         * If the widget is modal, show it
         * If the widget is not modal, make the widget modal if it is in a hidden/collapsed sidebar
         * @param {String} layoutItemName - the name of the Layout Item
         * @param {WidgetWrapper} widgetWrapper - the widgetWrapper for the widget
         */
        DesktopLayout.prototype._makeWidgetVisible = function (layoutItemName, widgetWrapper) {
            if (!widgetWrapper.isModal) {
                // the widget is not modal
                if (!this._layout[this._sectionMapping[layoutItemName]].isVisible()) {
                    // the widget is in a collapsed sidepanel, make it modal
                    this.moveItem(widgetWrapper.getID(), "center");


                    var width = 450;
                    var height = 350;
                    width = Math.min(this._layout.center.width - 20, width);
                    height = Math.min(this._layout.center.height - 20, height);

                    var position = this._layout.center.position();
                    var top = position.top + 10;
                    var left = position.left + this._layout.center.width - 10 - width;
                    widgetWrapper.setCollapsed(false);
                    widgetWrapper.setModal(true, top, left, width, height);
                }

            }
            widgetWrapper.widget.setMakeVisibleRequestAccepted();
            widgetWrapper.setCollapsed(false);
        };

        return DesktopLayout;
    });

