define('web-viewer/layout/Layout',["web-viewer/utils/ListenerUtil", "underscore"],
	function (
		ListenerUtil, _
	) {
		"use strict";

		/**
		 * @exports Layout
		 * @class
		 * Defines a general implementation for different layouts. Every layout has different modes it can change to.
		 * @constructor
		 * @param {core.ViewerFacade} f
		 * @param {LayoutModel} layoutModel
		 * @param {JQueryObj} context
		 * @param {Object} layoutState
         * @param {LoadingIndicator} loadingIndicator - the loading indicator, which was created before all requests started
         * @param {JQueryObj} initialLoader - the JQuery reference to the element indicating the initial loader (needed to safely remove it without emptying the context)            
		 */
		var Layout = function (f, layoutModel, context, layoutState, loadingIndicator, initialLoader) {
			ListenerUtil.mixin(this, f);

			this.f = f;

			/**
			 * Context of this layout, this is the DOM-object the layout will use
			 * @member {JQueryObj}
			 */
			this._context = context;

			/**
			 * The layout model, holds the layoutItems
			 * @member {LayoutModel}
			 */
			this._layoutModel = layoutModel;

			/**
			 * All items that are not allowed to be visible in the current mode but can become visible in another mode.
			 * @member {Object}
			 */
			this._notVisibleItems = [];
			
			/**
             * The loaded state of the layout.
             */
            this._layoutState = layoutState;
			
			 /**
             * The loadingIndicator
             */
            this._loadingIndicator = loadingIndicator;
			
			/**
			 * The DOM element used to show initial loader in the context
			 */
			this._initialLoader = initialLoader;

			/**
			 * Currently active mode for this layout
			 * @member {LayoutMode}
			 */
			this._activeMode = null; //to be set during initialization of the specific layout

			//register listener to the layoutModel for when the layout model is updated (item is added)
			this._registerListener("itemAdded", function (layoutItem) {
				if (this._activeMode.isItemAllowed(layoutItem.name)) {
					this._addItem(layoutItem);
				} else {
					this._notVisibleItems.push(layoutItem);
				}
			}, this, this._layoutModel);
			this._registerListener("layout.switchLayoutMode", _.bind(this._switchLayoutMode, this), this, this.f);
		};

		/**
		 * Adds a layoutItem to the layout
		 * @param {Object} layoutItem
		 */
		Layout.prototype._addItem = function (layoutItem) {
			//to be implemented by specific layout
		};

		/**
		 * Decide which of the added items should be shown and which ones should be hidden.
		 */
		Layout.prototype._updateItemVisibility = function () {
			//to be implemented by specific layout
		};

		/**
		 * The layout has changed, emit this.
		 */
		Layout.prototype._emitLayoutChanged = function () {
			this._emit("layout.changed");
		};

		/**
		 * Check whether the layout should be persisted or not, depends on the active mode.
		 * @returns {Boolean}
		 */
		Layout.prototype.isLayoutStatePersisted = function () {
			return this._activeMode.isPersisted();
		};

		/**
		 * Get the state of the layout.
		 * @returns {Object}
		 */
		Layout.prototype.getLayoutState = function () {
			//to be implemented by specific layout
		};

		/**
		 * Switch the active mode.
		 * @param {LayoutConstants.LayoutMode} [mode] - if not provided it will use the default mode for the layout
		 */
		Layout.prototype._switchLayoutMode = function (mode) {
			if (this._activeMode === null || mode !== this._activeMode.getName()) {
				var newMode = null;
				if (mode !== undefined) {
					newMode = this._layoutModel.getLayoutMode(mode);
				} else {
					newMode = this._getDefaultLayoutMode();
				}
				if (newMode !== undefined) {
					if (this._activeMode !== null) {
						this._activeMode.toggle(false);
					}
					this._activeMode = newMode;
					this._activeMode.toggle(true);

					var addedItems = [];
					_.each(this._notVisibleItems, function (item, key) {
						if (this._activeMode.isItemAllowed(item.name)) {
							this._addItem(item);
							addedItems.push(item);
						}
					}, this);
					if (addedItems.length) {
						this._notVisibleItems = _.difference(this._notVisibleItems, addedItems);
					}
					this._updateItemVisibility();

					this._emit("changed.layoutMode");
				}
			}
		};

		/**
		 * Returns the default layout mode for the layout.
		 * @return {LayoutMode}
		 */
		Layout.prototype._getDefaultLayoutMode = function () {
			//to be implemented by specific layout
		};

		/**
		 * Get a reference to the active layout mode
		 * @returns {LayoutMode}
		 */
		Layout.prototype.getActiveLayoutMode = function () {
			return this._activeMode;
		};


		/**
		 * Returns the type of the layout.
		 * @return {Number}
		 */
		Layout.prototype.getLayoutType = function () {
			//to be implemented by specific layout
			return -1;
		};

		return Layout;
	});

define('web-viewer/layout/LayoutMode',["web-viewer/layout/LayoutConstants"],
	function (
		LayoutConstants

	) {
		"use strict";

		/**
		 * @exports LayoutMode
		 * @memberof Layout
		 * @class
		 * Defines a general implementation of layout mode, the mode decides if items should be added to the layout
		 * and if the state should be persisted.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 * @param {LayoutConstants.LayoutMode} name
		 */
		var LayoutMode = function (f, layout, name) {
			this.f = f;

			/**
			 * Name of the layoutMode
			 */
			this._name = name;

			/**
			 * Is the layoutMode active
			 */
			this._isActive = false;

			/**
			 * The layout for which this mode is active
			 */
			this._layout = layout;

			/**
			 * is the modepanel expanded
			 */
			this._isModePanelExpanded = true;
		};

		/**
		 * Get the name of the mode.
		 * @returns {LayoutConstants.LayoutMode}
		 */
		LayoutMode.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		LayoutMode.prototype.toggle = function (toggle) {
			//to be implemented by specific layout mode
		};

		/**
		 * Checks if the item is allowed for this layout mode. Prevents channels and CAD widget from loading.
		 *
		 * @param {String} itemname
		 * @returns {Boolean}
		 */
		LayoutMode.prototype.isItemAllowed = function (itemname) {
			//to be implemented by specific layout mode
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		LayoutMode.prototype.isPersisted = function () {
			//to be implemented by specific layout mode
		};

		/**
		 * Checks if the mode is active at the moment.
		 */
		LayoutMode.prototype.isActive = function () {
			return this._isActive;
		};

		/**
		 * Decide whether or not to enable the mode panel
		 */
		LayoutMode.prototype.isModePanelEnabled = function () {
			// should be overwritten for mobile layouts in which the mode panel should be shown
			return false;
		};

		/**
		 * If the mode panel is enabled, should it be expanded by default?
		 * @returns {Boolean} is de mode panel expanded by default?
		 */
		LayoutMode.prototype.isModePanelExpanded = function () {
			return this._isModePanelExpanded;
		};

		/**
		 * Should the widget titles be shown?
		 * @returns {Boolean}
		 */
		LayoutMode.prototype.showModePanelWidgetTitles = function () {
			return true;
		};

		/**
		 * Is the modepnale expanded or not?
		 * This method is called by the layout if a modepanel was expanded or collapsed
		 * @param {Boolean} expanded
		 */
		LayoutMode.prototype.setIsmodePanelExpanded = function (expanded) {
			this._isModePanelExpanded = expanded;
		};

		/** 
		 * This method should return a list of widgets that should appear in the modepanel (or an empty array if none should be shown)
		 * @returns {Array.<String>} widget names of the widgets shown in the mode panel
		 */
		LayoutMode.prototype.getModePanelContent = function () {
			return [];
		};

		/**
		 * Get the name of the widget which should be shown in the Main Panel
		 * By default this is the ViewPortWidget (note this is only supported by the mobile layout at the moment)
		 * @returns {String} widget name of the widget shown in the main panel
		 */
		LayoutMode.prototype.getMainPanelContent = function () {
			return LayoutConstants.Widget.VIEWPORT;
		};


		return LayoutMode;
	});

define('web-viewer/layout/flex/modes/FlexBasicMode',["web-viewer/layout/LayoutConstants",
		"web-viewer/layout/LayoutMode"
	],
	function (
		LayoutConstants,
		LayoutMode
	) {
		"use strict";

		/**
		 * @exports FlexBasicMode
		 * @memberof Layout
		 * @class
		 * Basic mode of the desktoplayout. Will allow all the layout items to be added and the state is persisted.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 */
		var FlexBasicMode = function (f, layout) {
			LayoutMode.call(this, f, layout, LayoutConstants.LayoutMode.BASIC);
		};

		FlexBasicMode.prototype = new LayoutMode();
		FlexBasicMode.prototype.constructor = FlexBasicMode;

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		FlexBasicMode.prototype.toggle = function (toggle) {
			if (toggle === undefined) {
				toggle = !this._isActive;
			}
			this._isActive = toggle;
		};

		/**
		 * Checks if the item is allowed for this layout mode.
		 *
		 * @param {String} itemname
		 * @returns {Boolean}
		 */
		FlexBasicMode.prototype.isItemAllowed = function (itemname) {
			// prevents documentselector toolbarcomponent
			if (itemname === LayoutConstants.ToolbarComponent.DOCUMENTSELECTOR) {
				return false;
			} else {
				return this._layout.isItemAllowed(itemname);
			}
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		FlexBasicMode.prototype.isPersisted = function () {
			return true;
		};
	
	

		return FlexBasicMode;
	});

define('web-viewer/layout/desktop/WizardMode',["web-viewer/layout/LayoutConstants",
		"web-viewer/layout/LayoutMode",
		"underscore"
	],
	function (
		LayoutConstants,
		LayoutMode,
		_
	) {
		"use strict";

		/**
		 * @exports WizardMode
		 * @memberof Layout
		 * @class
		 * Wizard mode of the desktoplayout. Will allow all the layout items to be added and the state is persisted.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 */
		var WizardMode = function (f, layout) {
			LayoutMode.call(this, f, layout, LayoutConstants.LayoutMode.WIZARD);

			/**
			 * Saves the names of all modal widgets that are hidden
			 * @member {Array.<String>}
			 */
			this._hiddenModalWidgets = [];

			/**
			 * Saves the visible state of toolbarComponent, maps ItemName to Boolean. Initializes to all the toolbarComponents.
			 * @member {Object.<String, Boolean>}
			 */
			this._wasToolbarComponentVisible = _.invert(_.values(LayoutConstants.ToolbarComponent));

			/**
			 * Save all the open popup when the wizard is visible to be able to hide them
			 */
			this._openPopups = [];
		};

		WizardMode.prototype = Object.create(LayoutMode.prototype);
		WizardMode.prototype.constructor = WizardMode;

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		WizardMode.prototype.toggle = function (toggle) {
			toggle = toggle === undefined ? !this._isActive : toggle;
			this._isActive = toggle;
			if (toggle) {
				this._hideSidePanels();

				this._layout.showItem(LayoutConstants.Widget.ALIGNWIZARD); //show alignment widget

				this._hideModalWidgets();

				this._hidePopups();

				this._hideToolbarComponents();
			} else {
				this._showSidePanels();

				this._layout.hideItem(LayoutConstants.Widget.ALIGNWIZARD); //hide alignment widget

				this._showModalWidgets();

				this._showPopups();

				this._showToolbarComponents();
			}
		};


		/**
		 * Hide the side panels
		 */
		WizardMode.prototype._hideSidePanels = function () {
			this._layout.disablePanelInteraction(); //disable panel interaction, not possible to open side panels
			this._layout.disablePanel("east");
			this._layout.disablePanel("west");

		};

		/**
		 * Show the side panels
		 */
		WizardMode.prototype._showSidePanels = function () {
			this._layout.initializePanelInteraction(); //make sure to enable panel interaction again
			this._layout.enablePanel("east");
			this._layout.enablePanel("west");
		};


		/**
		 * Hide modal widgets
		 */
		WizardMode.prototype._hideModalWidgets = function () {
			//hides all modal widgets and returns the names of the hidden elements
			this._hiddenModalWidgets = this._layout.hideAllModalWidgets();
		};
		/**
		 * Show the modal widget  that were previously hidden
		 */
		WizardMode.prototype._showModalWidgets = function () {
			_.each(this._hiddenModalWidgets, function (widgetName) { //run over the hidden modal widgets
				this._layout.showItem(widgetName); //show them again
			}, this);
		};



		/**
		 * Hide all popups in the viewport
		 */
		WizardMode.prototype._hidePopups = function () {
			//gets all popups to hide and returns those popups to show them when wizardMode ends
			this._openPopups = this._layout.getAllVisiblePopups();
			this._openPopups.hide();
		};

		/**
		 * Show all the popups that were previously hidden
		 */
		WizardMode.prototype._showPopups = function () {
			this._openPopups.show();
			this._openPopups = [];
		};


		/**
		 * Hide all toolbar components except for navigation controls
		 */
		WizardMode.prototype._hideToolbarComponents = function () {
			//run over all the toolbarComponents and check if they are visible
			_.each(this._wasToolbarComponentVisible, function (wasVisible, itemName) {
				//save the visible state of each toolbarcomponent
				this._wasToolbarComponentVisible[itemName] = this._layout.isItemVisible(itemName);
				if (this._wasToolbarComponentVisible[itemName] //only hide items that are visible
					&& itemName !== LayoutConstants.ToolbarComponent.NAVIGATE) { //and are not the navigate controls
					this._layout.hideItem(itemName);
				}
			}, this);
		};

		/** 
		 * Show all toolbar ocmponetns that were previously hidden
		 */
		WizardMode.prototype._showToolbarComponents = function () {
			_.each(this._wasToolbarComponentVisible, function (wasVisible, itemName) { //run over all toolbarcomponents
				if (wasVisible) { //only if it was visible, show it again
					this._layout.showItem(itemName);
				}
			}, this);
		};


		/**
		 * Checks if the item is allowed for this layout mode. Prevents channels and CAD widget from loading.
		 * 
		 * @param {String} itemname
		 * @returns {Boolean}
		 */
		WizardMode.prototype.isItemAllowed = function (itemname) {
			return itemname === LayoutConstants.Widget.ALIGNWIZARD; //don't add anything when in wizard mode
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		WizardMode.prototype.isPersisted = function () {
			return false;
		};

		return WizardMode;
	});

define('web-viewer/layout/flex/modes/FlexWizardMode',["web-viewer/layout/LayoutConstants",
		"web-viewer/layout/desktop/WizardMode",
		"underscore"
	],
	function (
		LayoutConstants,
		WizardMode,
		_
	) {
		"use strict";

		/**
		 * @exports FlexWizardMode
		 * @memberof Layout
		 * @class
		 * Wizard mode of the desktoplayout. Will allow all the layout items to be added and the state is persisted.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 */
		var FlexWizardMode = function (f, layout) {
			WizardMode.call(this, f, layout);


		};

		FlexWizardMode.prototype = Object.create(WizardMode.prototype);
		FlexWizardMode.prototype.constructor = FlexWizardMode;

		/**
		 * @Override
		 */
		FlexWizardMode.prototype._hideSidePanels = function () {
			this._layout.setSidebarsEnabled(false);
		};

		/**
		 * @Override
		 */
		FlexWizardMode.prototype._showSidePanels = function () {
			this._layout.setSidebarsEnabled(true);
		};


		/**
		 * @Override
		 */
		FlexWizardMode.prototype._hideModalWidgets = function () {
			// nothing to do
		};

		/**
		 * @Override
		 */
		FlexWizardMode.prototype._showModalWidgets = function () {
			// nothing to do
		};

		return FlexWizardMode;
	});

define('web-viewer/layout/WidgetWrapper',["jquery",
		"utils/ShapeUtils"
	],
	function ($, ShapeUtils) {
		"use strict";

		/**
		 * A widgetwrapper is a wrapper for a widget, which makes enables the widget to be displayed with some extra controls
		 * @param {AbstractWidget} widget          - the widget to wrap
		 * @param {String}         widgetName      - the name of the widget
		 * @param {Boolean} allowCollapse - is collapsing the widget allowed?
		 */
		var WidgetWrapper = function (widget, widgetName, allowCollapse) {
			this._headerEnabled = true;
			// main wrapper
			this.element = $("<div/>").addClass("widgetWrapper");
			this.element[0].wrapper = this;

			// webkit has a bug with iFrames, which appearantly they won't be solving soon
			// if an input in an iframe is focused, the page scrolls to the iframe
			// this can occur when TinyMCE is used... we can fix it by scrolling to top whenever the context is scrolled
			this.element.scroll(function (event) {
				$(this).scrollTop(0);
			});

			this.name = widgetName;

			// header + title
			this.header = $("<div/>").addClass("header");
			this.headerText = $("<h3>", {
				text: widget.getTitle(),
				title: widget.getTitle()
			});
			this.headerHandle = $("<div/>").addClass("headerHandle").append(this.headerText);
			if (allowCollapse) {
				this.headerHandle.click(_.bind(function () {
					this.toggle();
				}, this));
			}
			this.header.append(this.headerHandle);

			this.headerExtra = $("<div/>").addClass("headerExtra");
			this.header.append(this.headerExtra);

			// listen to title changed of widget
			widget.on("titlechanged", _.bind(this._onTitleChanged, this));

			// content holder
			this.content = $("<div/>").addClass("content");

			// create dom structure
			this.element.append(this.header, this.content);

			// activate the widget
			widget.setContext(this.content, this.headerExtra);

			//is the widget resizable
			this.resizable = false;

			// the widget
			this.widget = widget;

			this.isModal = false;

			this.isCollapsed = false;
			this._isVisible = true;

			//location; only use if the widgetwrapper is hidden
			this._locationWhenHidden = new ShapeUtils.Point();
		};

		/**
		 * Convert the widget in to a modal widget or normal widget
		 * @param modal {boolean} should the widget be modal or not?
		 * @param top {int} position top
		 * @param left {int} position left
		 * @param width {int} width of modal window
		 * @param height {int} height of modal window
		 */
		WidgetWrapper.prototype.setModal = function (modal, top, left, width, height) {
			this.widget.setModal(modal);
			if (modal) {

				this.element.css({
					position: "absolute",
					top: top,
					left: left,
					width: width || this.element.width() || undefined,
					height: height || this.element.height() || undefined
				});
				var newHeight = height || this.element.height() || 0;
				this.content.css("height", newHeight - this.getHeaderHeight());

				this.setResizable(!this.isCollapsed);

				this.isModal = true;
			} else {
				this.isModal = false;
				// restore normal css
				this.element.css({
					position: "relative",
					top: "",
					left: "",
					width: "auto"
				});

				this.setResizable(false);
			}
		};

		/**
		 * Function called when the title of the widget changed
		 * @param {String} newTitle - The new title
		 */
		WidgetWrapper.prototype._onTitleChanged = function (newTitle) {
			this.headerText.text(newTitle).attr("title", newTitle);
		};

		/**
		 * Toggle collapse state of the widget
		 */
		WidgetWrapper.prototype.toggle = function () {
			this.setCollapsed(!this.isCollapsed);
		};

		/**
		 * Hides/shows a widget
		 * @param {Boolean} isVisible
		 */
		WidgetWrapper.prototype.setVisible = function (isVisible) {
			this._isVisible = isVisible;
			if (isVisible) {
				this.element.removeClass("hiddenElement");
			} else {
				var offset = this.element.offset();
				//location is only used when the widgetwrapper is hidden, if visible the element is used
				this._locationWhenHidden = new ShapeUtils.Point(offset.left, offset.top);
				this.element.addClass("hiddenElement");
			}
		};

		/**
		 * Is the widgetwrapper visible.
		 * @returns {Boolean}
		 */
		WidgetWrapper.prototype.isVisible = function () {
			return this._isVisible;
		};

		/**
		 * (un)Collapse the widget
		 * @param {Boolean} collapsed
		 */
		WidgetWrapper.prototype.setCollapsed = function (collapse) {
			var wasCollapsed = this.isCollapsed;
			this.element.toggleClass("collapsed", collapse);
			this.isCollapsed = collapse;
			this.widget.setCollapsed(collapse);

			if (this.isModal && wasCollapsed !== collapse) {
				var newHeight = Math.min($(window).height() - (this.element.offset().top - $(window).scrollTop()), this.getPreferredSize().height);
				this.setHeight(newHeight);
				if (!collapse) {
					this.setResizable(true);
				}
				this.setResizable(!this.isCollapsed);
				this.widget.fireModalResize();
			}

		};

		WidgetWrapper.prototype.setResizable = function (resizable) {
			if (this.resizable && !resizable) {
				this.element.resizable('destroy');
				this.resizable = false;
			} else if (!this.resizable && resizable) {
				//make the element resizable
				var minimumSize = this.getMinimumSize();
				this.element.resizable({
					minWidth: Math.max(100, minimumSize.width),
					minHeight: Math.max(40, minimumSize.height),
					stop: _.bind(function (event, ui) {
						this.setHeight(ui.size.height);
						this.widget.fireModalResize();
					}, this),
					resize: _.throttle(_.bind(function (event, ui) {
						this.setHeight(ui.size.height);
						this.widget.resizing();
					}, this), 40)
				});
				this.resizable = true;
			}
		};

		/**
		 * Get the ID of the enclosed widget
		 * @returns
		 */
		WidgetWrapper.prototype.getID = function () {
			return this.widget.getID();
		};

		/**
		 * Get the minimum size of the widget content
		 *
		 * note it is possible that this size is not respected by the layoutmanager
		 *
		 * @returns {ShapeUtils.Size}
		 */
		WidgetWrapper.prototype.getMinimumSize = function () {
			if (this.isCollapsed) {
				return new ShapeUtils.Size(0, this.getHeaderHeight());
			} else {
				var minSize = this.widget.getMinimumSize().copy();
				minSize.height += this.getHeaderHeight() + 3;

				return minSize;
			}
		};

		/**
		 * Get the preferred size of the widget content
		 *
		 * note it is possible that this size is not respected by the layoutmanager
		 *
		 * @returns {ShapeUtils.Size} if undefined, no preferred size is taken in to account by layoutmanager, the layout managers tries to guess the needed size
		 */
		WidgetWrapper.prototype.getPreferredSize = function () {
			if (this.isCollapsed) {
				return new ShapeUtils.Size(0, this.getHeaderHeight());
			} else {
				var prefSize = this.widget.getPreferredSize();
				if (prefSize) {
					prefSize = prefSize.copy();
					prefSize.height += this.getHeaderHeight() + 3;
					return prefSize;
				} else {
					return undefined;
				}
			}
		};

		/**
		 * Get the maximum dimension of the widget content
		 *
		 * @returns {ShapeUtils.Size} if undefined the widget can be made as big as the user wants
		 */
		WidgetWrapper.prototype.getMaximumSize = function () {
			if (this.isCollapsed) {
				return new ShapeUtils.Size(1000, this.getHeaderHeight());
			} else {
				var maxSize = this.widget.getMaximumSize();
				if (maxSize) {
					maxSize = maxSize.copy();
					maxSize.height += this.getHeaderHeight() + 3;
					return maxSize;
				} else {
					return undefined;
				}
			}
		};


		/**
		 * Get the height of the header
		 * @returns
		 */
		WidgetWrapper.prototype.getHeaderHeight = function () {
			if (this._headerEnabled) {
				return this.header.outerHeight(true);
			} else {
				return 0;
			}
		};

		/**
		 * Get the current content height of the widget
		 * @returns
		 */
		WidgetWrapper.prototype.getContentHeight = function () {
			return this.content.getOuterHeight(true);
		};

		/**
		 * Will the widgetwrapper extend if needed?
		 * @returns
		 */
		WidgetWrapper.prototype.willFill = function () {
			return this.widget.willFill();
		};

		/**
		 * Set the height of the widgetwrapper
		 * @param height
		 */
		WidgetWrapper.prototype.setHeight = function (height) {
			var prevContentHeight = this.content.height();
			var newContentHeight = height - this.getHeaderHeight();
			this.element.css("height", height);
			this.content.css("height", newContentHeight);



			if (newContentHeight != prevContentHeight) {
				this.widget.resized();

				this.content.css("overflow", "hidden");
				_.defer(function (el) {
					el.css("overflow", "auto");
				}, this.content);
			}
		};

		/**
		 * Get the size of the WidgetWrapper
		 *
		 * @returns {ShapeUtils.Size}
		 */
		WidgetWrapper.prototype.getSize = function () {
			return new ShapeUtils.Size(this.element.width(), this.element.height());
		};

		/**
		 * Get the outer size of the widget wrapper, including borders and padding
		 *
		 * @returns {ShapeUtils.Size}
		 */
		WidgetWrapper.prototype.getOuterSize = function () {
			return new ShapeUtils.Size(this.element.outerWidth(true), this.element.outerHeight(true));
		};

		/**
		 * Get the location of the WidgetWrapper
		 *
		 * @returns {ShapeUtils.Point}
		 */
		WidgetWrapper.prototype.getLocation = function () {
			if (this._isVisible) {
				var offset = this.element.offset();
				return new ShapeUtils.Point(offset.left, offset.top);
			} else {
				return this._locationWhenHidden;
			}
		};

		/**
		 * Enable/Disable the header display of the widget wrapper
		 * @param {Boolean} enable
		 */
		WidgetWrapper.prototype.setHeaderEnabled = function (enable) {
			this._headerEnabled = enable;
			this.header.toggleClass("displayNone", !enable);
		};

		return WidgetWrapper;
	});

define('web-viewer/layout/flex/FlexWidgetWrapper',["web-viewer/layout/WidgetWrapper",
		"utils/ShapeUtils",
		"underscore",
		"jquery",
		"jqueryUI"
	],
	function (WidgetWrapper,
		ShapeUtils,
		_,
		$) {

		"use strict";

		/**
		 * A widgetwrapper is a wrapper for a widget, which makes enables the widget to be displayed with some extra controls
		 * @exports FlexWidgetWrapper
		 * @augments WidgetWrapper
		 * @param {AbstractWidget} widget          - the widget to wrap
		 * @param {String}         widgetName      - the name of the widget
		 * @param {Boolean} allowCollapse - is collapsing the widget allowed?
		 */
		var FlexWidgetWrapper = function (widget, widgetName, allowCollapse) {
			WidgetWrapper.call(this, widget, widgetName, allowCollapse);

			/**
			 * @member {Boolean} is the widgetwrapper currently resizable
			 */
			this._isResizable = false;
			/**
			 * @member {Object} contains the dimensions for the widget, if null, the widgetwrapper should try to fit around the containt of the widget
			 */
			this._stickyDimensions = null;

			/**
			 * @member {Boolean} tru if the widgtet is sticky
			 */
			this._isSticky = false;


			/**
			 * @member {JQueryObj} Tile to represent the widget
			 */
			this._tile = $("<a>", {
				'class': "tile",
				title: widget.getTitle()
			});

			this._tile[0].wrapper = this;

			/**
			 * @member {JQueryObj} icon used for the tile
			 */
			this._tileIcon = $("<i>", {
				'class': widget.getIcon()
			});
			/**
			 * @member {JQueryObj} Label for the tile
			 */
			this._tileSpan = $("<span>", {
				text: widget.getTitle()
			});

			this._tile.click(_.bind(function () {
				if (this._sidebar) {
					this._sidebar.toggleWidgetWrapperActive(this);
				}
			}, this));

			// reset dimensions on double click
			this._tile.dblclick(_.bind(function () {
				this.setStickyDimensions(null);
			}, this));

			if (widget.willFill()) {
				this.element.addClass("flex");
			}

			widget.on("iconchanged", function (newIcon) {
				this._tileIcon.removeClass().addClass(newIcon);
			}, this);

			this._tile.append(this._tileIcon, this._tileSpan);


			/**
			 * @member {FlexSidebar} sidebar in which the widget is placed
			 */
			this._sidebar = null;

			/**
			 * @member {Boolean}
			 */
			this._isActive = true;
		};

		FlexWidgetWrapper.prototype = Object.create(WidgetWrapper.prototype);
		FlexWidgetWrapper.prototype.constructor = FlexWidgetWrapper;

		/**
		 * Get a reference to the tile element of the widget wrapper
		 * @returns {JQueryObj}
		 */
		FlexWidgetWrapper.prototype.getTileElement = function () {
			return this._tile;
		};

		/**
		 * @Override
		 */
		FlexWidgetWrapper.prototype._onTitleChanged = function (newTitle) {
			WidgetWrapper.prototype._onTitleChanged.call(this, newTitle);
			this._tile.attr("title", newTitle);
			this._tileSpan.text(newTitle);
		};

		/**
		 * Stick the widget wrapper to its tile
		 * This makes the widgetwrapper absolute positioned
		 */
		FlexWidgetWrapper.prototype.stickToTile = function () {	
			var css = this._calculateStickyCSS();
			// for a flexbox bug in Safare we quickly set the widgetwrapper to display block, and then remove it later on...
			// if we don't do this hack, the sidepanel container is visible...
			this.element.addClass("absolute-position").css("display", "block").css(css);
			this._setHeightFixForIE();
			this._isSticky = true;
			this.setResizable(true, css);
			// remove the temporarily set display css property
			this.element.css("display", "");
		};

		/**
		 * Calculate CSS for a sticky widget
		 * This calculates the correct min/max dimensions and position
		 * @returns {Object} key-value pairs of CSS properties to be set
		 */
		FlexWidgetWrapper.prototype._calculateStickyCSS = function () {
			var offset = this._tile.position();
			var tileWidth = this._tile.outerWidth();
			var isRight = this._tile.parents(".sidebar").is(".right");
			var topPos = offset.top;
			var leftPos = "";
			var rightPos = "";
			if (isRight) {
				rightPos = tileWidth;
			} else {
				leftPos = tileWidth;
			}
			var container = this._sidebar.getContainer();

			var maxHeight = container.innerHeight() - topPos;
			// only allow widgets to come to the middle of the container
			var maxWidth = Math.floor(0.495 * (container.innerWidth() - 2 * tileWidth));

			if (this.getMaximumSize()) {
				var maxSize = this.getMaximumSize();
				maxHeight = maxSize.height;
				maxWidth = maxSize.width;
			}

			var minSize = this.widget.getMinimumSize() || {};

			var css = {
				top: topPos,
				left: leftPos,
				right: rightPos,
				'max-height': maxHeight,
				'max-width': maxWidth,
				'min-width': minSize.width,
				'min-height': minSize.height
			};

			if (this._stickyDimensions) {
				css.width = this._stickyDimensions.width ? Math.min(this._stickyDimensions.width, maxWidth) : "";
				css.height = this._stickyDimensions.height ? Math.min(this._stickyDimensions.height, maxHeight) : "";
			} else {
				css.width = "";
				css.height = "";
			}

			return css;

		};

		/**
		 * This sets height of the flex widget wrapper as IE's flexbox doesn't work well with max-heights.
		 * The content div will overflow the FlexWidgetWrapper without a scrollbar. Setting the height of the 
		 * FlexWidgetWrapper makes sure that the content does overflow with a scrollbar.
		 */
		FlexWidgetWrapper.prototype._setHeightFixForIE = function () {
			if(this._sidebar && this._tile && this.content && this.header) {
				var container = this._sidebar.getContainer();
				var topPos = this._tile.position().top;

				var maxHeight = container.innerHeight() - topPos;
				var contentHeight = this.content.outerHeight();
				var headerHeight = this.header.outerHeight();
				
				// If the height of the content + header is bigger than the max size of the widgetwrapper we should 
				// set the height of the widget wrapper.
				if(maxHeight < contentHeight + headerHeight) {
					this.element.height(maxHeight);
				}
			}
			
		};

		/**
		 * Reset the sticky dimensions of the widget wrapper
		 */
		FlexWidgetWrapper.prototype.resetStickyDimensions = function () {
			this._stickyDimensions = null;
			if (this._isSticky) {
				this.element.css({
					width: "",
					height: ""
				});
			}
		};

		/**
		 * Unstick the widgetwrapper
		 */
		FlexWidgetWrapper.prototype.unstick = function () {
			this._isSticky = false;

			this.setResizable(false);

			this.element.removeClass("absolute-position").css({
				"max-height": "",
				top: "",
				left: "",
				right: "",
				width: "",
				height: "",
				'max-width': "",
				position: "",
			});
		};

		/**
		 * Set the sidebar of the widgetwrapper
		 * @param {FlexSidebar} sidebar
		 */
		FlexWidgetWrapper.prototype.setSidebar = function (sidebar) {
			if (this._sidebar && this._sidebar != sidebar) {
				this._sidebar.unregisterWidgetWrapper(this);
			}

			this._sidebar = sidebar;
			if (this._sidebar) {
				this._sidebar.registerWidgetWrapper(this);
			}
		};

		/**
		 * Set the widget as active
		 * @param {Boolean} active
		 * @param {Boolean} [uncollapse] - if active and uncollapse true, the widget is uncollapsed
		 */
		FlexWidgetWrapper.prototype.setActive = function (active, uncollapse) {
			if (active & uncollapse) {
				this.setCollapsed(false);
			}

			this._isActive = active;
			this._tile.toggleClass("active", active);
			this._updateWidgetVisibility();

		};
		/**
		 * Is the widgetwrapper active?
		 * @returns {Boolean}
		 */
		FlexWidgetWrapper.prototype.isActive = function () {
			return this._isActive;
		};

		/**
		 * @override
		 */
		FlexWidgetWrapper.prototype.setVisible = function (isVisible) {
			this._isVisible = isVisible;
			if (!this._isVisible) {
				var offset = this.element.offset();
				//location is only used when the widgetwrapper is hidden, if visible the element is used
				this._locationWhenHidden = new ShapeUtils.Point(offset.left, offset.top);
				this._tile.addClass("hiddenElement");
			} else {
				this._tile.removeClass("hiddenElement");
			}


			this._updateWidgetVisibility();
		};

		/**
		 * Update the visibility of the widget (based on active and visible
		 */
		FlexWidgetWrapper.prototype._updateWidgetVisibility = function () {
			var isVisible = this.isVisible() && this.isActive();
			this.element.toggleClass("hiddenElement", !isVisible);
			
			this.widget.setVisible(isVisible);
			if(isVisible)  {
				this._setHeightFixForIE();
			}
		};


		/**
		 * Toggle resizable functionality of the widget wrapper
		 * @param {Boolean} resizable
		 * @param {Object} [stickyCSS] - if not provided, sticky CSS is calculated automatically
		 */
		FlexWidgetWrapper.prototype.setResizable = function (resizable, stickyCSS) {
			if (this.resizable && !resizable) {
				this.element.resizable('destroy');
				this.resizable = false;
			} else if (resizable) {
				if (!stickyCSS) {
					stickyCSS = this._calculateStickyCSS();
				}

				//make the element resizable
				var minimumSize = this.getMinimumSize();
				_.defer(_.bind(function () {
					if (this._isSticky) {
						var resizeHandler = _.throttle(_.bind(function (event, ui) {
							this.setHeight(ui.size.height);
							this.widget.resizing();
						}, this), 40);

						var resizableOptions = {
							maxWidth: stickyCSS["max-width"],
							maxHeight: stickyCSS["max-height"],
							minWidth: Math.max(100, minimumSize.width),
							minHeight: Math.max(40, minimumSize.height),
							stop: _.bind(function (event, ui) {
								if (!this._stickyDimensions) {
									this._stickyDimensions = {};
								}
								
								this._stickyDimensions.width = ui.size.width;
								this._stickyDimensions.height = ui.size.height;

								this.setHeight(ui.size.height);
								this.widget.fireModalResize();
							}, this),
							resize: resizeHandler
						};

						if (this._sidebar.isRight()) {
							resizableOptions.handles = "w, sw, s";
							resizableOptions.resize = function (event, ui) {
								$(this).css("left", "");
								resizeHandler(event, ui);
							};
						} else {
							resizableOptions.handles = "e, se, s";
						}

						this.element.resizable(resizableOptions);
						this.resizable = true;
					}
				}, this));

			}

		};

		/**
		 * Only notify the widget that is was resized
		 * @override 
		 */
		FlexWidgetWrapper.prototype.setHeight = function () {
			this.widget.resized();
		};

		/**
		 * Get the sticky dimensions
		 * @returns {Object}
		 */
		FlexWidgetWrapper.prototype.getStickyDimensions = function () {
			return this._stickyDimensions;
		};

		/**
		 * Restore the sticky dimensions
		 */
		FlexWidgetWrapper.prototype.setStickyDimensions = function (stickyDimensions) {
			this._stickyDimensions = stickyDimensions;
			if (this._isSticky) {
				this.stickToTile();
			}
		};

		/**
		 * Set the widget wrapper collapsed. If collapsed is false, the widget becomes resizable, else it's sticked to tile if sticky.
		 * @param {Boolean} collapsed - True to set collapsed, false otherwise
		 */
		FlexWidgetWrapper.prototype.setCollapsed = function (collapsed) {
			WidgetWrapper.prototype.setCollapsed.call(this, collapsed);

			this.setResizable(!collapsed);
			if (collapsed) {
				this.element.css("height", "");
			} else {
				if (this._isSticky) {
					this.stickToTile();
				}
			}
		};


		return FlexWidgetWrapper;
	});

define('web-viewer/layout/flex/FlexSidebar',["web-viewer/core/PrivateChannel",
		"underscore",
		"jquery",
	   	"jqueryUI"],
	function (PrivateChannel,
		_,
		$) {
		"use strict";

		/**
		 * Sidebar that can be used in a flex layout
		 * it has a Tile bar and a widgetbar and can toggle between the modes
		 * @parma {ViewerFacade} f
		 * @param {String} name - unique name for the sidebar
		 * @param {String} positionClass - position of the sidebar (left|right)
		 * @param {Boolean} isIconified - isIconified
		 */
		var FlexSidebar = function (f, name, positionClass, container, isIconified) {
			PrivateChannel.installTo(this, f, ["layout", "flex", "sidebar", name]);

			this._isResizable = false;
			this._isIconified = true;
			this._widgetBarWidth = null;
			this._container = container;

			this._tilebar = $("<div>", {
				'class': "sidebar iconified " + positionClass
			});
			this._widgetbar = $("<div>", {
				'class': "widgetbar iconified " + positionClass
			});

			// To enable resizable with a scrolling div we unfortunately need an extra container div for the widgets...
			this._widgetbarContainer = $("<div>", {
				'class': "widgetbarcontainer"
			}).appendTo(this._widgetbar);

			this._toggleIconified = $("<a>", {
				title: f.t(this.isRight() ? "LayoutToggleRightSidebar" : "LayoutToggleLeftSidebar"),
				'class': "iconifierToggle",
				click: _.bind(function () {
					this.setIconified(!this._isIconified);
				}, this),
				dblclick: _.bind(function () {
					this._widgetbar.css("width", "");
					this._widgetBarWidth = null;
				}, this)
			}).append($("<i>", {
				'class': "icon icon-WCR_flexsidebarToggleIcon"
			}));

			this._tilebar.append(this._toggleIconified);
			this._widgetWrappers = [];
			this._activeWrapper = null;

			this.setIconified(isIconified);
		};

		/**
		 * Get the context element for the tile bar
		 * @returns {JQueryObj}
		 */
		FlexSidebar.prototype.getTilebarContext = function () {
			return this._tilebar;
		};

		/**
		 * Get the context element for the widget bar
		 * @returns {JQueryObj}
		 */
		FlexSidebar.prototype.getWidgetbarContext = function () {
			return this._widgetbar;
		};

		/**
		 * Get the container in which widgets reside
		 * @returns {JQueryObject}
		 */
		FlexSidebar.prototype.getWidgetbarContainer = function () {
			return this._widgetbarContainer;
		};

		/**
		 * Append a WidgetWrapper to the sidebar
		 * @param {FlexWidgetWrapper} item
		 */
		FlexSidebar.prototype.appendItem = function (item) {
			this._widgetbarContainer.append(item.element);
			this._tilebar.append(item.getTileElement());

			this._handleNewItem(item);
		};

		/**
		 * Prepend a WidgetWrapper begore another existing WidgetWrapper
		 * @param {FlexWidgetWrapper} itemToPrepend - the new widgetwrapper to add to the bar
		 * @param {FlexWidgetWrapper} existingItem - the existing WidgetWrapper before which the new item should be prepended
		 */
		FlexSidebar.prototype.prependItemBeforeItem = function (itemToPrepend, existingItem) {
			existingItem.element.before(itemToPrepend.element);
			existingItem.getTileElement().before(itemToPrepend.getTileElement());

			this._handleNewItem(itemToPrepend);
		};

		/**
		 * Internal method to handle a new item
		 * @param {FlexWidgetWrapper} item
		 */
		FlexSidebar.prototype._handleNewItem = function (item) {
			item.setSidebar(this);
			if (this._isIconified) {
				if (item.isActive()) {
					item.unstick();
					item.setActive(false);
					_.defer(_.bind(function () {
						this.toggleWidgetWrapperActive(item);
					}, this));
				} else {

					this._fixActiveWrapperPosition();
				}
			} else {
				item.setActive(true);
				item.unstick();
			}
		};

		/**
		 * Unregister a widgetwrapper from the sidebar
		 * @param {FlexWidgetWrapper} widgetWrapper
		 */
		FlexSidebar.prototype.unregisterWidgetWrapper = function (widgetWrapper) {
			this._widgetWrappers = _.without(this._widgetWrappers, widgetWrapper);
			if (this._activeWrapper === widgetWrapper) {
				this._activeWrapper = null;
			}

			this._fixActiveWrapperPosition();
		};

		/**
		 * Register a widgetwrapper at the sidebar (used for active behavior in iconified mode)
		 * @param {FlexWidgetWrapper} widgetWrapper
		 */
		FlexSidebar.prototype.registerWidgetWrapper = function (widgetWrapper) {
			this._widgetWrappers.push(widgetWrapper);

		};

		/**
		 * Toggle a widget wrapper to be active
		 * @param {FlexWidgetWrapper} the widgetwrapper to change
		 * @param {Boolean} [active] - should the wrapper become active of not (if undefined, the active state is toggled)
		 */
		FlexSidebar.prototype.toggleWidgetWrapperActive = function (widgetWrapper, active) {
			if (active === undefined) {
				active = !widgetWrapper.isActive();
			}

			if (this._isIconified) {
				if (active) {
					_.each(this._widgetWrappers, function (wrapper) {
						wrapper.setActive(wrapper === widgetWrapper, true);
					});
					this._activeWrapper = widgetWrapper;
					this._activeWrapper.stickToTile();
				} else {
					widgetWrapper.setActive(false);
					if (this._activeWrapper === widgetWrapper) {
						this._activeWrapper = null;
					}
				}
			}
		};

		/**
		 * Make the widgetbar resizable
		 * @param {Boolean} resizable
		 */
		FlexSidebar.prototype.setResizable = function (resizable) {
			if (!resizable && this._isResizable) {
				this._widgetbar.resizable("destroy");
				this._isResizable = false;
			}

			if (resizable) {
				var sizeBeforeStart;
				this._widgetbar.resizable({
					handles: this.isRight() ? "w" : "e",
					start: function (event, ui) {
						sizeBeforeStart = ui.size.width;
						$(this).css("position", "");
					},
					resize: function (event, ui) {
						ui.position.left = "0";
						$(this).css("left", "");
					},
					stop: _.bind(function (event, ui) {
						this._widgetbar.css({
							"position": "",
							"left": "",
							"top": ""
						});
						if (sizeBeforeStart != ui.size.width) {
							this._widgetBarWidth = ui.size.width;
							_.each(this._widgetWrappers, function (wrapper) {
								wrapper.widget.resizing();
							});
							this._emit("changed.resize");
						}
					}, this),
					maxWidth: "50%",
					minWidth: 150
				});
				this._isResizable = true;
			}
		};

		/**
		 * Switch between iconified and expanded mode
		 * @param {Boolean} iconify - true if sidebar should become iconified
		 */
		FlexSidebar.prototype.setIconified = function (iconify) {
			if (this._isIconified != iconify) {

				_.each(this._widgetWrappers, function (wrapper) {
					wrapper.setActive(!iconify);
					if (!iconify) {
						wrapper.unstick();
					}
				});

				this._isIconified = iconify;

				this._tilebar.toggleClass("hiddenElement", !iconify);

				this._emit("changed.iconified", !iconify, iconify);

				this._widgetbar.toggleClass("iconified", iconify);
				if (!iconify) {
					this._activeWrapper = null;
					this._widgetbarContainer.prepend(this._toggleIconified);

					if (this._widgetBarWidth !== null) {
						this._widgetbar.css("width", this._widgetBarWidth);
					}
				} else {
					this._tilebar.prepend(this._toggleIconified);
					this._widgetbar.css({
						width: "",
						height: ""
					});
				}

				this.setResizable(!iconify);
			}


		};
		/**
		 * Check if the sidebar is iconified
		 * @returns {Boolean}
		 */
		FlexSidebar.prototype.isIconified = function () {
			return this._isIconified;
		};

		/**
		 * Fix the position of the active wrapper
		 */
		FlexSidebar.prototype._fixActiveWrapperPosition = function () {
			if (this._activeWrapper) {
				this._activeWrapper.stickToTile();
			}
		};

		/**
		 * Get the state of the sidebar
		 * @returns {Object} serializable state object
		 */
		FlexSidebar.prototype.getState = function () {
			return {
				iconified: this.isIconified(),
				widgetbarWidth: this._widgetBarWidth
			};
		};

		/**
		 * Restore the state of the sidebar
		 * @param {Object} state - serialized state as returned from getState()
		 */
		FlexSidebar.prototype.restoreState = function (state) {
			this.setIconified(state.iconified);
			this.setWidgetBarWidth(state.widgetbarWidth);
		};

		/**
		 * Set the sidebar enabled
		 * @param {Boolean} enabled - if false the sidebar is hidden
		 */
		FlexSidebar.prototype.setEnabled = function (enabled) {
			if (!enabled) {
				this._tilebar.toggleClass("hiddenElement", true);
				this._widgetbar.toggleClass("hiddenElement", true);
			} else {
				this._tilebar.toggleClass("hiddenElement", !this.isIconified());
				this._widgetbar.toggleClass("hiddenElement", false); // widgetbar should also be visible in iconified mode (the sticky widget are still in the widgetbar container when they are stuck to their tile)
				this._fixActiveWrapperPosition();
			}

			this._emit("changed.enabled", enabled);
		};

		/**
		 * Set the width of the widgetbar
		 * @param {Number} widgetbarWidth - set the width of the widgetbar, if null, the width is based on its contents
		 */
		FlexSidebar.prototype.setWidgetBarWidth = function (widgetbarWidth) {
			if (!this.isIconified()) {
				if (widgetbarWidth !== null) {
					this._widgetbar.css("width", widgetbarWidth);
				} else {
					this._widgetbar.css("width", "");
				}
			}

			this._widgetBarWidth = widgetbarWidth;
		};

		/**
		 * Is this a right of left aligned sidebar?
		 * @returns {Boolean} true if is right
		 */
		FlexSidebar.prototype.isRight = function () {
			return this._tilebar.is(".right");
		};

		/**
		 * Get the container of the FlexSideBar
		 * @returns {JQueryObj}
		 */
		FlexSidebar.prototype.getContainer = function () {
			return this._container;
		};


		/**
		 * Make sure the layout looks OK when resized:
		 * Fix the resizable properties when in iconified mode
		 */
		FlexSidebar.prototype.invalidateLayout = function () {
			this._fixActiveWrapperPosition();
		};

		return FlexSidebar;
	});

define('web-viewer/layout/flex/FlexSection',[], function () {

	/**
	 * Simple container for showing elements
	 * It has methods for appding items (WidgetWrapper or ToolbarComponent) to the container
	 * @class
	 * @constructor
	 * @exports FlexSection
	 * @param {JQueryObj} the context of the FlexSection
	 */
	var FlexSection = function (context) {
		this._context = context;
	};

	/**
	 * Append an item
	 * @param {WidgetWrapper,ToolbarComponent} item - item to add (should have ".element" property)
	 */
	FlexSection.prototype.appendItem = function (item) {
		this._context.append(item.element);
	};

	/**
	 * Prepend an item before another item
	 * @param {WidgetWrapper,ToolbarComponent} itemToPrepend- item to add (should have ".element" property)
	 * @param {WidgetWrapper,ToolbarComponent} existingitem - sibbling that hsould be after itemToPrepend  (should have ".element" property)
	 */
	FlexSection.prototype.prependItemBeforeItem = function (itemToPrepend, existingItem) {
		existingItem.element.before(itemToPrepend.element);
	};

	return FlexSection;

});

define('web-viewer/layout/DraggableWidgets',["jquery",
		"jqueryUI",
		"underscore"
	],
	function ($) {
		"use strict";

		/**
		 * HELPER interface for draggable widgetwrappers
		 * Provides structured storage for extra information needed for dragging
		 */
		function DraggableWidgetWrapper(widgetWrapper) {
			this.wrapper = widgetWrapper;
			this.ghost = $("<div/>").addClass("ghostWrapper");

			this.sizeBeforeDrag = {
				width: undefined,
				height: undefined
			};
		}

		/**
		 * This method updates the dragging ghost element dimensions.
		 * The ghost element is transparent, but has the same size as the original element being dragged
		 */
		DraggableWidgetWrapper.prototype.updateGhost = function () {
			this.ghost.css({
				width: this.wrapper.element.outerWidth(true),
				height: this.wrapper.element.outerHeight(true)
			});
		};



		/**
		 * Enable draggable/droppable/sortable widgets on a container
		 * 
		 * When initialized widgets should be added (dragging enabled by enableDragging(...)) and dropzones should be defined
		 */
		var DraggableWidgets = function (container) {
			this.container = container;
			this._acceptSelector = ".widgetWrapper";
			this._siblingSelector = ".widgetWrapper";
			this.hoveredSortableDropzone = undefined;
		};

		/**
		 * Enable a WidgetWrapper to be dragged
		 * @param widgetWrapper
		 */
		DraggableWidgets.prototype.enableDragging = function (widgetWrapper) {
			//make the widgetwrapper draggable
			widgetWrapper.drag = new DraggableWidgetWrapper(widgetWrapper);

			$(widgetWrapper.element).draggable({
				containment: this.container,
				// only when the item is dragged more than 20px, we start dragging
				distance: 20,
				// dragging widgets doesn't introdicue scrolling of the page
				scroll: false,
				// when a dropzone is invalid, the widget is reverted to its orignal lcation
				revert: "invalid",
				// when dragging starts, create a semi transparent clone of the widget
				helper: function () {
					var clone = widgetWrapper.element.clone();
					clone.css({
						width: widgetWrapper.element.innerWidth(),
						opacity: 0.9
					});
					return clone;
				},
				// elements should be added to the container
				appendTo: this.container,
				// start dragging
				start: _.bind(function () {

					//update the ghost size and store the size before dragging or later usage
					this.drag.updateGhost();
					this.drag.sizeBeforeDrag.width = this.element.innerWidth();
					this.drag.sizeBeforeDrag.height = this.element.innerHeight();

					// hide the original element
					this.element.hide();
				}, widgetWrapper),

				// the element is being dragged
				drag: _.bind(function (event, ui) {
					// if a sortable zone is being hovered, insert the ghost at the correct location (this make the dropzone sortable!)
					if (this.hoveredSortableDropzone) {
						this.insertAt(this.hoveredSortableDropzone, widgetWrapper.drag.ghost, ui.offset.top - this.hoveredSortableDropzone.offset().top);
					}
				}, this),
				// dragging has stopped, remove the helper element, and detac the ghost, display the original element again
				stop: _.bind(function (event, ui) {
					this.drag.ghost.detach();
					this.element.show();

					this.widget.dragged(ui.offset.top, ui.offset.left);

					ui.helper.detach();
				}, widgetWrapper),

				// dragging is being done using the header element as handle
				handle: ".header"
			});
		};

		/**]
		 * Define a sortable widget dropzone
		 * @param panel
		 */
		DraggableWidgets.prototype.addSortableWidgetDropzone = function (panel, section, workspace) {
			$(panel).droppable({
				// only accept widgetwrappers to be dropped
				accept: this._acceptSelector,
				drop: _.bind(function (e, ui) {
					// drop in sortable widget dropzone, find the correct sibbling
					var sibling = this.findSibling(panel, ui.draggable, ui.offset.top - panel.offset().top);
					var widgetWrapper = ui.draggable[0].wrapper;
					if (sibling === undefined) {
						// no sibling found, just append to the section
						workspace.moveItem(widgetWrapper.getID(), section, undefined, true);
					} else {
						if (sibling.wrapper) {
							sibling = sibling.wrapper;
							workspace.moveItem(widgetWrapper.getID(), section, sibling.getID(), true);
						}
					}
				}, this),

				// a widgetwrapper is being dragged over this section, register this section as the current dropzone
				over: _.bind(function (e, ui) {
					this.hoveredSortableDropzone = panel;
				}, this),

				// the widgetwrapper is no long being dragged over this section
				out: _.bind(function (e, ui) {
					this.hoveredSortableDropzone = undefined;
				}, this),
				tolerance: "touch"
			});


		};

		/**
		 * Add a DropZone in which widgets can be dropped on top of all content (widget becomes modal!)
		 * @param panel
		 */
		DraggableWidgets.prototype.addModalWidgetDropzone = function (panel, section, workspace) {
			$(panel).droppable({
				accept: this._acceptSelector,
				drop: _.bind(function (e, ui) {
					var widgetWrapper = ui.draggable[0].wrapper;
					if (widgetWrapper) {
						widgetWrapper.setModal(true, ui.offset.top, ui.offset.left, widgetWrapper.drag.sizeBeforeDrag.width);

						workspace.moveItem(widgetWrapper.getID(), section);

						widgetWrapper.widget.fireModalResize();
					}
				}, this)
			});

		};


		/**
		 * Remove a panel as dropzone
		 * @param panel
		 */
		DraggableWidgets.prototype.removeDropzone = function (panel) {
			$(panel).droppable('disable');
		};


		/**
		 * Insert a element in a panel at a certain position (used for enabling sorting)
		 * @param list
		 * @param el
		 * @param position
		 */
		DraggableWidgets.prototype.insertAt = function (list, el, position) {
			var insertBefore = this.findSibling(list, el, position);
			if (insertBefore === undefined) {
				list.append(el);
			} else {
				$(insertBefore).before(el);
			}
		};

		/**
		 * Find the correct position to insert element
		 * @param list
		 * @param el
		 * @param position
		 * @returns element before which the element should be inserted
		 */
		DraggableWidgets.prototype.findSibling = function (list, el, position) {
			var currentHeight = 0;
			var insertBefore;
			var canContinue = true;

			list.children(this._siblingSelector).each(function (key, el) {
				var outerHeight = $(el).outerHeight(true);
				if (canContinue && position <= currentHeight + (outerHeight / 2)) {
					insertBefore = el;
					canContinue = false;
				}
				currentHeight += outerHeight;
			});

			return insertBefore;
		};

		return DraggableWidgets;
	});

define('web-viewer/layout/flex/FlexDrag',["web-viewer/layout/DraggableWidgets",
		"underscore",
		"jquery",
		"jqueryUI"],
	function (DraggableWidgets,
		_,
		$) {
		"use strict";


		/**
		 * Dragging for FlexLayout
		 * @constructor
		 * @class
		 * @augments DraggableWidgets
		 * @param {JQueryObj} container - container for all dragging/dropping
		 */
		var FlexDrag = function (container) {
			DraggableWidgets.call(this, container);

			this._acceptSelector = ".widgetWrapper, .tile";
			this._siblingSelector = ".widgetWrapper, .tile:not(.tileGhost)";

		};

		FlexDrag.prototype = Object.create(DraggableWidgets.prototype);
		FlexDrag.prototype.constructor = FlexDrag;


		/**
		 * This method is similar to DraggableWidgets.enableDragging, but uses a tile as a dragging ghost 
		 * instead of the widgetwrapper clone and doesn't need to to extra sizing "paperwork"
		 * @Override
		 */
		FlexDrag.prototype.enableDragging = function (wrapper) {
			var flexGhost = null;
			wrapper.element.add(wrapper.getTileElement()).draggable({
				distance: 20,
				scroll: false,
				revert: "invalid",
				helper: function () {
					flexGhost = wrapper.getTileElement().clone().addClass("tileGhost active");
					return flexGhost.clone();
				},
				appendTo: this.container,
				containment: this.contaner,
				start: _.bind(function (e, ui) {
					$(e.target).addClass("hiddenWhileDragging");
				}, this),
				drag: _.bind(function (e, ui) {
					if (this.hoveredSortableDropzone) {
						this.insertAt(this.hoveredSortableDropzone, flexGhost, ui.offset.top - this.hoveredSortableDropzone.offset().top);
					} else {
						flexGhost.detach();
					}
				}, this),
				cursorAt: { top:15, left: 30},
				stop: _.bind(function (e, ui) {
					flexGhost.remove();
					$(e.target).removeClass("hiddenWhileDragging");

					if (wrapper.widget) {
						wrapper.widget.dragged(ui.offset.top, ui.offset.left);
					}
				}, this)
			});

			wrapper.element.draggable("option", "handle", ".header");
		};


		/**
		 * Define a FlexSidebar dropzone
		 * @param panel
		 */
		FlexDrag.prototype.addFlexSidebarDropzone = function (sidebar, section, workspace) {
			var onIconfiedChanged = _.bind(function (oldValue, isIconified) {
				if (isIconified) {
					this.addSortableWidgetDropzone(sidebar.getTilebarContext(), section, workspace);
					sidebar.getWidgetbarContainer().droppable("destroy");
				} else {
					this.addSortableWidgetDropzone(sidebar.getWidgetbarContainer(), section, workspace);
					sidebar.getTilebarContext().droppable("destroy");
				}
			}, this);

			if (sidebar.isIconified()) {
				this.addSortableWidgetDropzone(sidebar.getTilebarContext(), section, workspace);
			} else {
				this.addSortableWidgetDropzone(sidebar.getWidgetbarContainer(), section, workspace);
			}
			sidebar.on("changed.iconified", onIconfiedChanged);
		};

		return FlexDrag;

	});

define('web-viewer/layout/flex/FlexConfiguration',["web-viewer/layout/LayoutConstants", "jquery"], function (LayoutConstants, $) {
	return {
		load: function (name, req, onload, config) {
			if (config.isBuild) {
				//Indicate that the optimizer should not wait
				//for this resource any more and complete optimization.
				//This resource will be resolved dynamically during
				//run time in the web browser.
				onload();
				return;
			}

			// this fallback implementation is the default CustomFormExtension.js implementation
			var defaultConfiguration = {
				layout: {
					north: [LayoutConstants.ToolbarComponent.PAGELIST,
						LayoutConstants.ToolbarComponent.NAVIGATE,
						LayoutConstants.ToolbarComponent.MEASURE,
						LayoutConstants.ToolbarComponent.TEXTSELECTION,
						LayoutConstants.ToolbarComponent.COMPARE,
						LayoutConstants.ToolbarComponent.ANNOTATION,
						LayoutConstants.ToolbarComponent.VIEW3D,
						LayoutConstants.ToolbarComponent.APPROVAL,
						LayoutConstants.ToolbarComponent.DYNCONTENT
					],
					east: [LayoutConstants.Widget.NAVIGATOR,
						LayoutConstants.Widget.VIEW3D,
						LayoutConstants.Widget.VIEWCONTROLS,
						LayoutConstants.Widget.CHANNELS,
						LayoutConstants.Widget.CAD,
						LayoutConstants.Widget.DEBUG,
						LayoutConstants.Widget.APPROVAL,
						LayoutConstants.Widget.INFO,
						LayoutConstants.Widget.HELP
					],
					west: [LayoutConstants.Widget.COMPARE,
						LayoutConstants.Widget.ATTRIBUTE_CATEGORY,
						LayoutConstants.Widget.ANNOTATION,
						LayoutConstants.Widget.PAGELIST,
						LayoutConstants.Widget.TEXTCONTENT
					]
				},
				west: {
					iconified: true
				},
				east: {
					iconified: true
				},
				disabledItems: []
			};

			req(["wcrBase/custom/viewer/" + name], function (CustomLayout) {
				if (CustomLayout) {
					onload(CustomLayout);
				} else {
					onload(defaultConfiguration);
				}
			}, function () {
				// something went wrong
				onload(defaultConfiguration);
			});

		}
	};
});


define('web-viewer/layout/flex/FlexLayout',["web-viewer/layout/Layout",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/core/PrivateChannel",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/layout/flex/modes/FlexBasicMode",
		"web-viewer/layout/flex/modes/FlexWizardMode",
		"web-viewer/layout/flex/FlexWidgetWrapper",
		"web-viewer/layout/flex/FlexSidebar",
		"web-viewer/layout/flex/FlexSection",
		"web-viewer/layout/flex/FlexDrag",
		"web-viewer/layout/flex/FlexConfiguration!restricted",
		"utils/ShapeUtils",
		"underscore",
		"jquery"
	],
	function (
		Layout,
		LayoutConstants,
		PrivateChannel,
		ListenerUtil,
		FlexBasicMode,
		FlexWizardMode,
		FlexWidgetWrapper,
		FlexSidebar,
		FlexSection,
		FlexDrag,
		FlexConfiguration,
		ShapeUtils,
		_,
		$
	) {
		"use strict";


		/**
		 * @class
		 * Icon Side Panel layout This is the view initialized by the LayoutManager that will add all the layoutcomponents
		 * to the viewer. It can generate a an object representing the current state of the viewer and can be initialized by
		 * a state-String.
		 *
		 * NOTE widget that are both MainPanelContent and ModelPanelContent are not supported at the moment!
		 *
		 * @exports FlexLayout
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {LayoutModel} layoutModel
		 * @param {JQueryObj} context - context where the layout will add its components to
		 * @param {Object} layoutState
		 * @param {LoadingIndicator} loadingIndicator - the loading indicator, which was created before all requests started
         * @param {JQueryObj} initialLoader - the JQuery reference to the element indicating the initial loader (needed to safely remove it without emptying the context)
		 */
		var FlexLayout = function (f, layoutModel, context, layoutState, loadingIndicator, initialLoader) {
			Layout.call(this, f, layoutModel, context, layoutState, loadingIndicator, initialLoader);
			PrivateChannel.installTo(this, f, "layout.flex");
			ListenerUtil.mixin(this, f);

			/**
			 * Default state of the layout, used to reset the layout
			 */

			this._configuration = FlexConfiguration;

			this._defaultLayout = this._getDefaultLayout();

			/**
			 * Mapping for all the items, maps items to their section
			 */
			this._sectionMapping = {};

			/**
			 * All the layout items mapped as Name -> FlexWidgetWrapper
			 */
			this._items = {};

			/**
			 * The component that is used to display the content in the viewport: graphics, CAD, annotations,...
			 */
			this._mainContent = null;

			//a debounced invalidate, to make sure not to many invalidates are triggered
			this._debouncedInvalidate = _.debounce(_.bind(this._invalidateLayout, this), 200);

			/**
			 * In this array all items that are for the main content are kept
			 */
			this._mainContentWidgetWrappers = [];

			this._topbar = $("<div>", {
				'class': "topbar"
			});
			this._toolbar = $("<div>", {
				'class': "toolbar"
			});
			this._mainContentWrapper = $("<div>", {
				'class': "maincontentwrapper"
			});

			var isEastUndefined = typeof this._configuration.east === "undefined" || typeof this._configuration.east.iconified === "undefined";
			var isWestUndefined = typeof this._configuration.west === "undefined" || typeof this._configuration.west.iconified === "undefined";

			this._sections = {
				north: new FlexSection(this._toolbar),
				east: new FlexSidebar(this.f, "right", "right", this._mainContentWrapper, isEastUndefined ? true : this._configuration.east.iconified),
				west: new FlexSidebar(this.f, "left", "left", this._mainContentWrapper, isWestUndefined ? true : this._configuration.west.iconified)
			};

			this._centerContent = $("<div>", {
				'class': "centercontent"
			});


			this._messagebar = $("<div />").addClass("messagebar");
			this._aboveCenter = $("<div />").addClass("aboveCenter");

			this._centerContent.append(this._aboveCenter, this._loadingIndicator.element, this._messagebar);




			this._mainContentWrapper.append(this._sections.west.getTilebarContext(),
				this._sections.west.getWidgetbarContext(),
				this._centerContent,
				this._rightWidgetbar,
				this._sections.east.getWidgetbarContext(),
				this._sections.east.getTilebarContext()
			);


			// initialize the section mapping of the layoutItems
			this._initializeSectionMapping(this._layoutState);
			// initialize the sections
			this._initializeSections(this._layoutState);

			// by default load the view mode
			this._activeMode = new FlexBasicMode(this.f, this);
			this._activeMode.toggle(true);

			this._wizardMode = new FlexWizardMode(this.f, this);


			// register all modes
			this._layoutModel.registerLayoutMode(this._activeMode);
			this._layoutModel.registerLayoutMode(this._wizardMode);




			var onCSSLoaded = _.after(2, _.bind(function () {
				this._cssLoaded = true;
				this._initializeDOM();
			}, this));

			this.f.emit("layout.loadCSS", "style/css/style.css", onCSSLoaded);
			this.f.emit("layout.loadCSS", "web-viewer/layout/flex/css/flexLayout.css", onCSSLoaded);

			//listen to resize events of the window
			$(window).resize(_.bind(function (e) {
				// http://bugs.jqueryui.com/ticket/7514: resize of popups bubbles to window
				if (e.target === window) {
					this._debouncedInvalidate();
				}
			}, this));

			//listen to widget events, to act on resized/collapsed/...
			this._registerOnLayoutEvents();

			this._invalidateLayout(); //this one is needed to provide the correct world coordinates		

			this._updateModePanel();

			this._flexDrag = new FlexDrag(this._mainContentWrapper);
			this._flexDrag.addFlexSidebarDropzone(this._sections.west, "west", this);
			this._flexDrag.addFlexSidebarDropzone(this._sections.east, "east", this);

		};

		FlexLayout.prototype = Object.create(Layout.prototype);
		FlexLayout.prototype.constructor = FlexLayout;

		FlexLayout.prototype._initializeDOM = function () {

			this._initialLoader.remove();
			delete this._initialLoader;
			this._context.append(this._topbar, this._toolbar, this._mainContentWrapper);
			this._invalidateLayout();
		};


		/**
		 * Add an item to the layout, the items needs to already added to the items-collection.
		 * It adds the item to the corect element on the correct location (depends on the previous/default state and already added items)
		 * @param {String} itemName
		 */
		FlexLayout.prototype._addLayoutItem = function (itemName) {
			var section = this._sectionMapping[itemName]; //get the section the items belongs to
			if (section !== undefined) { //make sure there is a section to add the item to
				var sectionItems = null;
				var itemState = null;
				if (this._layoutState) {
					sectionItems = _.map(this._layoutState[section].items, function (item) {
						return item.name;
					}); //just get all the names of the items in this section, not other information
					itemState = _.find(this._layoutState[section].items, function (item) {
						return item.name === itemName;
					}); //get the state of the to be added item
				} else {
					sectionItems = this._defaultLayout[section]; //default state only holds names of the items
				}
				if (sectionItems !== null && sectionItems !== undefined) { //the section should at least hold the item itself
					var index = _.indexOf(sectionItems, itemName);
					var nextItem = null;
					if (index === -1) {
						index = _.indexOf(this._defaultLayout[section], itemName);
						nextItem = this._items[this._defaultLayout[section][++index]];
						while (index < this._defaultLayout[section].length && nextItem === undefined) { //as long nextItem is not yet added to the layout, search for a next one
							nextItem = this._items[this._defaultLayout[section][++index]];
						}
					} else {
						nextItem = this._items[sectionItems[++index]]; //get the next item, so we know infront of which item the item needs to be added to
						while (index < sectionItems.length && nextItem === undefined) { //as long nextItem is not yet added to the layout, search for a next one
							nextItem = this._items[sectionItems[++index]];
						}
					}

					if (nextItem === undefined) { //no nextItem is found, the item is (for now) the last item in the section
						this._appendItemToSection(itemName, section); //append the item to the section
					} else { //add the item in front of the found nextItem
						this._prependItemBeforeItem(nextItem.name, itemName, section);
					}
					//now be sure to set the correct state of the item (active and collapsed or not)
					if (itemState !== null && itemState !== undefined) {
						this._setActive(itemName, itemState.isActive, itemState.isCollapsed);
						var widgetWrapper = this._items[itemName];
						if (widgetWrapper !== undefined && widgetWrapper.widget) {
							if (itemState.stickyDimensions) {
								widgetWrapper.setStickyDimensions(itemState.stickyDimensions);
							} else {
								widgetWrapper.setStickyDimensions(null);
							}
						}

					}

				}
			} else {
				this.f.debug.warn("FlexLayout :: No section found for item " + itemName);
			}
		};

		/**
		 * Set a widget active
		 * @param {String} itemName
		 * @param {Boolean} collapsed
		 */
		FlexLayout.prototype._setActive = function (itemName, active, collapsed) {
			var widgetWrapper = this._items[itemName];
			if (widgetWrapper !== undefined && widgetWrapper.widget) {
				var sectionName = this._sectionMapping[itemName];
				if (sectionName) {
					var section = this._sections[sectionName];
					if (section) {
						section.toggleWidgetWrapperActive(widgetWrapper, active);
						widgetWrapper.setCollapsed(collapsed);

					}
				}
			}
		};

		/**
		 * Appends an already added item (to the items collection) to a section.
		 * @param {String} itemName
		 * @param {String} section
		 */
		FlexLayout.prototype._appendItemToSection = function (itemName, section) {
			var layoutItem = this._items[itemName];
			if (layoutItem.widget) {
				layoutItem.widget.movingStarted();
			}

			this._sections[section].appendItem(layoutItem);

			if (layoutItem.widget) {
				layoutItem.widget.movingStopped();
			}
		};

		/**
		 * Prepends an already added item (to the items collection) before the nextItem in a given section.
		 * @param {String} nextItemName
		 * @param {String} itemName
		 * @param {String} section
		 */
		FlexLayout.prototype._prependItemBeforeItem = function (nextItemName, itemName, section) {
			var layoutItem = this._items[itemName];
			if (layoutItem.widget) {
				layoutItem.widget.movingStarted();
			}


			//add item before other item in the layout
			this._sections[section].prependItemBeforeItem(layoutItem, this._items[nextItemName]);

			if (layoutItem.widget) {
				layoutItem.widget.movingStopped();
			}
		};


		/**
		 * Moves an item to another section. Is used by draggable widgets to let the layout now it is moved.
		 * @param {String} itemName
		 * @param {String} sectionName
		 * @param {String} nextItemName - item will be moved before this item
		 * @param {Boolean} resetSizeAndLocation
		 * @param {Boolean} resetCollapsed
		 */
		FlexLayout.prototype.moveItem = function (itemName, sectionName, nextItemName, resetSizeAndLocation, resetCollapsed) {
			if (itemName !== nextItemName) {
				/*	if (resetSizeAndLocation) { //resets the size and location of the item (moves from center to a side-panel)
						this._setLocation(itemName, null);
						this._setSize(itemName, null);
					}
					if (resetCollapsed) { //resets the collapsed state of the item
						this._setCollapsed(itemName, false);
					}*/
				var currentSection = this._sectionMapping[itemName];
				this._detachItem(itemName, currentSection); //removes the item from the previous section
				this._sectionMapping[itemName] = sectionName;
				if (nextItemName === undefined || this._sectionMapping[nextItemName] !== sectionName) {
					this._appendItemToSection(itemName, sectionName);
				} else {
					//check if no problems with after parameter
					this._prependItemBeforeItem(nextItemName, itemName, sectionName);
				}
			}
		};

		/**
		 * Remove an item from a section, it removes the item from the DOM, but doesn't delete the DOM-element.
		 * @param {String} itemName
		 * @param {String} sectionName
		 */
		FlexLayout.prototype._detachItem = function (itemName, sectionName) {
			var widgetWrapper = this._items[itemName];
			if (widgetWrapper) {
				if (widgetWrapper.widget) {
					widgetWrapper.widget.movingStarted();
				}

				//remove element from the DOM (doesn't delete DOM-element)
				this._items[itemName].element.detach();

				if (widgetWrapper.widget) {
					widgetWrapper.widget.movingStopped();
				}
				//the invalidate is only done after all other changes (e.g. re-add DOM-element) are done
				_.defer(_.bind(function () {
					this._invalidateLayout();
				}, this));
			}
		};

		/**
		 * Initializes the section mapping. All items will be mapped to their section so it's easier to find the item's section.
		 * Items that aren't in the layoutstate but are in the default state will also be added to the mapping.
		 *
		 * @param {Object} [layoutState] - if not defined only the default layout is used for the mapping
		 */
		FlexLayout.prototype._initializeSectionMapping = function (layoutState) {
			//first run over all the sections and items in the layoutState
			if (layoutState !== undefined && layoutState !== "" && _.size(layoutState) !== 0) {
				_.each(layoutState, function (section, sectionName) { //runs over all sections
					_.each(section.items, function (item) { //runs over all items in those sections
						this._sectionMapping[item.name] = sectionName; //map each itemName->sectionName
					}, this);
				}, this);
			}
			//run over the defaultLayout, if items of the default layout aren't added yet, add them to the mapping
			_.each(this._getDefaultLayout(), function (items, sectionName) {
				_.each(items, function (item) {
					if (this._sectionMapping[item] === undefined) { //check if item isn't added by layoutState yet
						this._sectionMapping[item] = sectionName;
					}
				}, this);
			}, this);
		};


		/**
		 * Initializes the sections, get state from the layoutState.
		 */
		FlexLayout.prototype._initializeSections = function (layoutState) {
			if (layoutState !== undefined && layoutState !== "" && _.size(layoutState) !== 0) {
				_.each(layoutState, function (section, sectionName) { //run over all sections
					if (this._sections[sectionName] !== undefined && this._sections[sectionName].restoreState) {
						this._sections[sectionName].restoreState(section);
					}
				}, this);
			}

			var onSidebarChange = _.bind(function () {
				this._emitLayoutChanged();
				this._debouncedInvalidate();
			}, this);

			this._sections.east.on("changed.*", onSidebarChange);
			this._sections.west.on("changed.*", onSidebarChange);
		};

		/**
		 * Register listeners for general layout event.
		 */
		FlexLayout.prototype._registerOnLayoutEvents = function () {
			this.f.on("layout.disableTextSelection", _.bind(function () {
				this._disableTextSelection();
			}, this));

			this.f.on("layout.enableTextSelection", _.bind(function () {
				this._enableTextSelection();
			}, this));

			this.f.on("layout.resetToDefault", _.bind(this.resetToDefault, this));

			this.f.on("layout.switchLayoutMode", _.bind(function () {
				//this._updateModePanel();
				//this._updateMainContent();
			}, this));
		};

		/**
		 * Add an item to the layout. This is called by Layout to add an item.
		 * @param {Object} layoutItem
		 * @example
		 * 	{ 	type: {LayoutConstants.LayoutType},
		 * 		name: 'itemName',
		 * 		item: {AbstractWidget/ToolbarComponent}
		 * 	}
		 */
		FlexLayout.prototype._addItem = function (layoutItem) {
			if (layoutItem.type === LayoutConstants.LayoutItemType.WIDGET) {
				var widget = layoutItem.getItem(this.getLayoutType());
				var widgetType = widget.getWidgetType();

				if (layoutItem.name === this._activeMode.getMainPanelContent()) {
					widgetType = "maincontent";
				}

				switch (widgetType) {
				case "messagebar":
					widget.setContext(this._messagebar);

					break;
				case "topbar": //widget needs to be added to the topbar
					widget.setContext(this._topbar);
					break;
				case "wizard":
					widget.setContext(this._aboveCenter);
					this._items[widget.getID()] = widget;
					break;

				case "maincontent":
				default:
					var widgetWrapper = new FlexWidgetWrapper(widget, layoutItem.name, true);
					this._items[layoutItem.name] = widgetWrapper;


					if (widgetType === "maincontent") {
						this._centerContent.append(widgetWrapper.element);
						this._mainContentWidgetWrappers.push(widgetWrapper);
						this._updateMainContent();
					} else {
						widgetWrapper.setActive(false);
						this._flexDrag.enableDragging(widgetWrapper);
						this._addLayoutItem(layoutItem.name); //make sure the item is added on the correct location in one of the panels
					}

					this._registerListener("enabled", function (isEnabled) {
						this._toggleWidget(layoutItem.name, widgetWrapper, isEnabled);
						this._emitLayoutChanged();
					}, this, widget);

					this._registerListener("visible", function () {
						this._emitLayoutChanged();
					}, this, widget);

					this._registerListener("collapsed", function () {
						this._emitLayoutChanged();
					}, this, widget);

					this._registerListener("dragged", function () {
						this._emitLayoutChanged();
					}, this, widget);

					this._registerListener("modalResize", function () {
						this._emitLayoutChanged();
					}, this, widget);

					this._registerListener("makeVisible", _.bind(this._makeWidgetVisible, this, layoutItem.name, widgetWrapper), undefined, widget);

					if (widget.didRequestMakeVisible()) {
						this._makeWidgetVisible(layoutItem.name, widgetWrapper);
					}


					break;
				}
			} else if (layoutItem.type === LayoutConstants.LayoutItemType.TOOLBARCOMPONENT) {
				//add a toolbarcomponent
				var toolbarItem = layoutItem.getItem(this.getLayoutType());
				toolbarItem.element[0].name = layoutItem.name;
				this._items[layoutItem.name] = toolbarItem;
				this._addLayoutItem(layoutItem.name);
				this._registerListener("changed.visibility", function () {
					this._debouncedInvalidate();
				}, this, toolbarItem);
				this._debouncedInvalidate();
			} else {
				//not supported in this layout
			}
		};




		/**
		 * Update visibility of the added items based on whether or not they are allowed in the current mode.
		 * E.g. item was added in previous mode, but should not be shown in current mode
		 */
		FlexLayout.prototype._updateItemVisibility = function () {

			_.each(this._items, _.bind(function (layoutItem) {
				if (layoutItem.type === LayoutConstants.LayoutItemType.TOOLBARCOMPONENT) {
					var toolbar = layoutItem.getItem(this.getLayoutType());
					this._navigationBar.toggleComponent(toolbar.name, this._activeMode.isItemAllowed(layoutItem.name));
				} else if (layoutItem.type === LayoutConstants.LayoutItemType.WIDGET) {
					var widgetWrapper = layoutItem.getItem(this.getLayoutType());

					widgetWrapper.setVisible(this._activeMode.isItemAllowed(layoutItem.name) && widgetWrapper.widget.isVisible());
				}
			}, this));
		};

		/**
		 * Disable text selection in the entire viewer.
		 */
		FlexLayout.prototype._disableTextSelection = function () {
			this._context.find(".enableTextSelection").toggleClass("enableTextSelection", false).toggleClass("enableTextSelectionTempDisabled", true);
		};

		/**
		 * Enable text selection in the entire viewer.
		 */
		FlexLayout.prototype._enableTextSelection = function () {
			this._context.find(".enableTextSelectionTempDisabled").toggleClass("enableTextSelection", true).toggleClass("enableTextSelectionTempDisabled", false);
		};

		/**
		 * Reset the sate of the layout to the default state.
		 */
		FlexLayout.prototype.resetToDefault = function () {

			var iconifiedWest = this._configuration.west && this._configuration.west.iconified;
			var iconifiedEast = this._configuration.east && this._configuration.east.iconified;

			this._sections.east.setIconified(iconifiedEast);
			this._sections.east.setWidgetBarWidth(null);
			this._sections.west.setIconified(iconifiedWest);
			this._sections.west.setWidgetBarWidth(null);

			_.each(this._getDefaultLayout(), function (section, sectionName) {

				if (sectionName === "west" || sectionName === "east") { //reset the width of the side panels to the default

					_.each(section, function (itemName) { //run over each item in the section
						var item = this._items[itemName];
						if (item !== undefined && this._activeMode.isItemAllowed(itemName)) { //move the item to its default location.
							this.moveItem(itemName, sectionName, undefined, true, true);
						}

						if (item && item.wrapper) {
							item.wrapper.setStickyDimensions(null);
						}
					}, this);
				}
			}, this);

			this._layoutState = this._getDefaultLayout(); //reset the state of the layout, it now is the default\

			this._updateItemVisibility();
			this._invalidateLayout();
		};

		/**
		 * Get default layout state
		 * @returns {Object}
		 */
		FlexLayout.prototype._getDefaultLayout = function () {
			return this._configuration.layout;
		};

		/**
		 * Generates the state of the layout as a string. It takes the not-visible elements into account so state of those
		 * elements is not lost.
		 * @returns {Object}
		 */
		FlexLayout.prototype.getLayoutState = function () {
			var stateObject = {};
			_.each(this._sections, _.bind(function (section, sectionName) {

				var context;
				if (sectionName === "north") {
					context = this._toolbar;
					stateObject[sectionName] = {};
				} else {
					context = this._sections[sectionName].getWidgetbarContainer();
					stateObject[sectionName] = this._sections[sectionName].getState();
				}

				stateObject[sectionName].items = [];

				_.each(context.children(), _.bind(function (item) {
					if (item !== null && item !== undefined) {
						var itemWrapper = item.wrapper;
						var itemObject = {};
						if (itemWrapper) {
							itemObject.name = itemWrapper.name;
							itemObject.isCollapsed = itemWrapper.isCollapsed;
							itemObject.isActive = itemWrapper.isActive();
							itemObject.stickyDimensions = itemWrapper.getStickyDimensions();
							stateObject[sectionName].items.push(itemObject);
						}
					}
				}, this));
			}, this));

			return stateObject;
		};

		/**
		 * Invalidates the layout. Makes sure all panels have the correct size, resize is triggered on the correct elements,...
		 */
		FlexLayout.prototype._invalidateLayout = function () {
			if (!this._cssLoaded) {
				return;
			}


			if (this._activeMainPanelContent) {
				this._activeMainPanelContent.content.css({
					"height": this._centerContent.innerHeight(),
					"width": this._centerContent.innerWidth()
				});

				if (this._activeMainPanelContent.widget !== null) {
					this._activeMainPanelContent.widget.resized();
				}

			}
			// make sure subtoolbar are moved when new items are added
			_.each(this._items, function (layoutItem, itemName) {
				if (this._layoutModel.getLayoutItemType(itemName) === LayoutConstants.LayoutItemType.TOOLBARCOMPONENT) {
					layoutItem.moved();
				}
			}, this);


			this._sections.east.invalidateLayout();
			this._sections.west.invalidateLayout();
		};

		/**
		 * Returns the type of the layout.
		 * @override
		 * @return {Number}
		 */
		FlexLayout.prototype.getLayoutType = function () {
			return LayoutConstants.LayoutType.FLEX;
		};

		/**
		 * Get all the currently visible popups
		 * @returns {Array.<JQueryObj>}
		 */
		FlexLayout.prototype.getAllVisiblePopups = function () {
			return this._context.find(".popup:visible");
		};

		/**
		 * Enable/disable fullscreen
		 * Note: this method should be called from a user generated event... (or it won't work...)
		 * @param enabled
		 */
		FlexLayout.prototype.toggleFullscreen = function (enabled) {
			if (enabled) {
				this._requestFullscreen();
			} else {
				this._exitFullscreen();
			}
		};

		/**
		 * Check if the app is fullscreen
		 * @returns {Boolean}
		 */
		FlexLayout.prototype.isFullscreen = function () {
			return document.fullscreenElement || // alternative standard method
				document.mozFullScreenElement || document.webkitFullscreenElement || document.msFullscreenElement;
		};

		/**
		 * Check if the browser supports the fullescren API
		 * @returns {Boolean}
		 */
		FlexLayout.prototype.supportsFullscreen = function () {
			var viewerDiv = this.f.getContext().get(0);
			return viewerDiv.requestFullscreen || viewerDiv.mozRequestFullScreen || viewerDiv.webkitRequestFullscreen || viewerDiv.msRequestFullscreen;
		};

		/**
		 * Request fullscreen for the whole document.body
		 */
		FlexLayout.prototype._requestFullscreen = function () {
			var viewerDiv = this.f.getContext().get(0);
			if (viewerDiv.requestFullscreen) {
				viewerDiv.requestFullscreen();
			} else if (viewerDiv.mozRequestFullScreen) {
				viewerDiv.mozRequestFullScreen();
			} else if (viewerDiv.webkitRequestFullscreen) {
				viewerDiv.webkitRequestFullscreen();
			} else if (viewerDiv.msRequestFullscreen) {
				viewerDiv.msRequestFullscreen();
			}
		};

		/**
		 * Exit fullscreen mode
		 */
		FlexLayout.prototype._exitFullscreen = function () {
			if (document.exitFullscreen) {
				document.exitFullscreen();
			} else if (document.msExitFullscreen) {
				document.msExitFullscreen();
			} else if (document.mozCancelFullScreen) {
				document.mozCancelFullScreen();
			} else if (document.webkitExitFullscreen) {
				document.webkitExitFullscreen();
			}
		};

		/**
		 * Show or hide mode panel, depending on the active mode
		 */
		FlexLayout.prototype._updateModePanel = function () {
			//
		};

		/**
		 * Update the contents of the main panel
		 * A mode can decide to show something else than the viewport widget
		 */
		FlexLayout.prototype._updateMainContent = function () {

			var mainPanelContent = this._activeMode.getMainPanelContent();
			this._activeMainPanelContent = null;

			// run through all possible main content widget wrappers
			_.each(this._mainContentWidgetWrappers, function (wrapper) {
				if (wrapper.widget.getID() === mainPanelContent) {
					// this wrapper should be visible
					wrapper.setVisible(true);
					wrapper.setHeaderEnabled(false);

					this._activeMainPanelContent = wrapper;
				} else {
					// hide, the wrapper should not be active
					wrapper.setVisible(false);
					wrapper.setHeaderEnabled(true);
				}
			}, this);

			if (!this._activeMainPanelContent && this._items[mainPanelContent]) {
				// we didn't find the widget in the _mainContentWidgetWrappers, but it might be initialized anyway
				// append it to the center content

				this._mainContentWidgetWrappers.push(this._items[mainPanelContent]);
				this._centerContent.append(this._items[mainPanelContent].element);
				this._updateMainContent();
				return;
			}

			this._debouncedInvalidate();
		};



		/**
		 * Enable/disable a widget
		 * @param {String}        widgetName    - name of the widget
		 * @param {WidgetWrapper} widgetWrapper - widgetWrapper in which the widget is displayed
		 * @param {Boolean}       isEnabled     - should the widget be enabled
		 */
		FlexLayout.prototype._toggleWidget = function (widgetName, widgetWrapper, isEnabled) {
			if (isEnabled) {
				if (this._activeMode.isItemAllowed(widgetName)) {
					widgetWrapper.setVisible(true);
				}
			} else {
				widgetWrapper.setVisible(false);
			}
		};

		/**
		 * Returns the default layout mode for the layout.
		 * @return {LayoutMode}
		 */
		FlexLayout.prototype._getDefaultLayoutMode = function () {
			return this._layoutModel.getLayoutMode(LayoutConstants.LayoutMode.BASIC);
		};

		FlexLayout.prototype.isItemAllowed = function (itemname) {
			return !_.contains(this._configuration.disabledItems, itemname);
		};

		/**
		 * Enable/disable the sidebars
		 * @param {Boolean} enabled
		 */
		FlexLayout.prototype.setSidebarsEnabled = function (enabled) {
			this._sections.east.setEnabled(enabled);
			this._sections.west.setEnabled(enabled);
		};

		/**
		 * Hide a layout item.
		 * @param {LayoutConstants.Widget|LayoutConstants.ToolbarComponent} itemName
		 */
		FlexLayout.prototype.hideItem = function (itemName) {
			var layoutItem = this._items[itemName];
			if (layoutItem !== undefined && layoutItem.context !== undefined) {
				layoutItem.context.hide();
			} else if (layoutItem !== undefined && layoutItem.element !== undefined) {
				layoutItem.element.hide();
			}
		};

		/**
		 * Makes a hidden layout item visible.
		 * @param {LayoutConstants.Widget|LayoutConstants.ToolbarComponent} itemName
		 */
		FlexLayout.prototype.showItem = function (itemName) {
			var layoutItem = this._items[itemName];
			if (layoutItem !== undefined && layoutItem.context !== undefined) {
				layoutItem.context.show();
			} else if (layoutItem !== undefined && layoutItem.element !== undefined) {
				layoutItem.element.show();
			}
		};

		/**
		 * Is the layout item visible.
		 * @param {String} itemName
		 * @returns {Boolean}
		 */
		FlexLayout.prototype.isItemVisible = function (itemName) {
			var layoutItem = this._items[itemName];
			if (layoutItem !== undefined) {
				return layoutItem.isVisible();
			}
		};
		/**
		 * Make sure a widget is visible
		 * Uncollapse it if it is collapsed
		 *
		 * If the widget is modal, show it
		 * If the widget is not modal, make the widget modal if it is in a hidden/collapsed sidebar
		 * @param {String} layoutItemName - the name of the Layout Item
		 * @param {WidgetWrapper} widgetWrapper - the widgetWrapper for the widget
		 */
		FlexLayout.prototype._makeWidgetVisible = function (layoutItemName, widgetWrapper) {
			this._sections[this._sectionMapping[layoutItemName]].toggleWidgetWrapperActive(widgetWrapper, true);
			widgetWrapper.widget.setMakeVisibleRequestAccepted();
			widgetWrapper.setCollapsed(false);
		};
		return FlexLayout;
	});

