define(["web-viewer/layout/LayoutConstants",
		"web-viewer/layout/LayoutMode"
	],
	function (
		LayoutConstants,
		LayoutMode
	) {
		"use strict";

		/**
		 * @exports MobileCompareMode
		 * @memberof Layout
		 * @class
		 * Mode for mobile layout, shows the compare view
		 * @constructor
		 * @augments LayoutMode
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 */
		var MobileCompareMode = function (f, layout) {
			LayoutMode.call(this, f, layout, LayoutConstants.LayoutMode.MOBILE_COMPARE);


			this._emitModeAvailability(false);
			this.f.whenModuleAvailable("compare", function (compareModule) {
				var isCompareAvailable = compareModule.isCompareAvailable();
				this._emitModeAvailability(isCompareAvailable);

				compareModule.on("changed.mode", _.bind(function (oldMode, newMode) {
					if (newMode && !compareModule.isReaderSpreadActive()) {
						this.f.emit("layout.switchLayoutMode", LayoutConstants.LayoutMode.MOBILE_COMPARE);
					}
				}, this));

				if (!isCompareAvailable) {
					compareModule.on("compareAvailable", _.bind(function () {
						this._emitModeAvailability(true);
					}, this));
				}

				// only execute this code after initialization
				_.defer(_.bind(function () {
					if (compareModule.getActiveMode() && !compareModule.isReaderSpreadActive()) {
						this.f.emit("layout.switchLayoutMode", LayoutConstants.LayoutMode.MOBILE_COMPARE);
					}
				}, this));

			}, this);

		};

		MobileCompareMode.prototype = new LayoutMode();
		MobileCompareMode.prototype.constructor = MobileCompareMode;


		/**
		 * Notify the change in availability of the mode
		 * @param {Boolean} available [[Description]]
		 */
		MobileCompareMode.prototype._emitModeAvailability = function (available) {
			this.f.emit("layout.modeAvailabilityChanged", this, available, {
				icon: "icon-WCR_Compare",
				title: "CompareToolsTitle"
			});
		};

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		MobileCompareMode.prototype.toggle = function (toggle) {
			if (toggle === undefined) {
				toggle = !this._isActive;
			}

			this._isActive = toggle;

			var compareModule = this.f.getModule("compare");
			// turn off compare when leaving the mode
			if (!toggle) {
				compareModule.activateModeByName();
			} else {
				if (!compareModule.getActiveMode()) {
					compareModule.activateLastUsedCompareMode();
				}
			}
		};

		/**
		 * Checks if the item is allowed for this layout mode. Prevents channels and CAD widget from loading.
		 * 
		 * @param {String} itemname
		 * @returns {Boolean}
		 */
		MobileCompareMode.prototype.isItemAllowed = function (itemname) {
			// only allow viewport widget
			return itemname === LayoutConstants.Widget.VIEWPORT || itemname === LayoutConstants.Widget.COMPARE || itemname === LayoutConstants.ToolbarComponent.COMPARE || itemname === LayoutConstants.ToolbarComponent.APPROVAL || itemname === LayoutConstants.Widget.MESSAGE;
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		MobileCompareMode.prototype.isPersisted = function () {
			return false;
		};

		/**
		 * Decide whether or not to show the mode panel
		 * @override
		 */
		MobileCompareMode.prototype.isModePanelEnabled = function () {
			return true;
		};

		/**
		 * @override
		 */
		MobileCompareMode.prototype.showModePanelWidgetTitles = function () {
			return true;
		};

		/**
		 * list of needed widgets in modepanel for mobile compare mode
		 * @returns {Array}    
		 * @override
		 */
		MobileCompareMode.prototype.getModePanelContent = function () {
			return [LayoutConstants.Widget.COMPARE];
		};

		return MobileCompareMode;
	});