define('web-viewer/layout/Layout',["web-viewer/utils/ListenerUtil", "underscore"],
	function (
		ListenerUtil, _
	) {
		"use strict";

		/**
		 * @exports Layout
		 * @class
		 * Defines a general implementation for different layouts. Every layout has different modes it can change to.
		 * @constructor
		 * @param {core.ViewerFacade} f
		 * @param {LayoutModel} layoutModel
		 * @param {JQueryObj} context
		 * @param {Object} layoutState
         * @param {LoadingIndicator} loadingIndicator - the loading indicator, which was created before all requests started
         * @param {JQueryObj} initialLoader - the JQuery reference to the element indicating the initial loader (needed to safely remove it without emptying the context)            
		 */
		var Layout = function (f, layoutModel, context, layoutState, loadingIndicator, initialLoader) {
			ListenerUtil.mixin(this, f);

			this.f = f;

			/**
			 * Context of this layout, this is the DOM-object the layout will use
			 * @member {JQueryObj}
			 */
			this._context = context;

			/**
			 * The layout model, holds the layoutItems
			 * @member {LayoutModel}
			 */
			this._layoutModel = layoutModel;

			/**
			 * All items that are not allowed to be visible in the current mode but can become visible in another mode.
			 * @member {Object}
			 */
			this._notVisibleItems = [];
			
			/**
             * The loaded state of the layout.
             */
            this._layoutState = layoutState;
			
			 /**
             * The loadingIndicator
             */
            this._loadingIndicator = loadingIndicator;
			
			/**
			 * The DOM element used to show initial loader in the context
			 */
			this._initialLoader = initialLoader;

			/**
			 * Currently active mode for this layout
			 * @member {LayoutMode}
			 */
			this._activeMode = null; //to be set during initialization of the specific layout

			//register listener to the layoutModel for when the layout model is updated (item is added)
			this._registerListener("itemAdded", function (layoutItem) {
				if (this._activeMode.isItemAllowed(layoutItem.name)) {
					this._addItem(layoutItem);
				} else {
					this._notVisibleItems.push(layoutItem);
				}
			}, this, this._layoutModel);
			this._registerListener("layout.switchLayoutMode", _.bind(this._switchLayoutMode, this), this, this.f);
		};

		/**
		 * Adds a layoutItem to the layout
		 * @param {Object} layoutItem
		 */
		Layout.prototype._addItem = function (layoutItem) {
			//to be implemented by specific layout
		};

		/**
		 * Decide which of the added items should be shown and which ones should be hidden.
		 */
		Layout.prototype._updateItemVisibility = function () {
			//to be implemented by specific layout
		};

		/**
		 * The layout has changed, emit this.
		 */
		Layout.prototype._emitLayoutChanged = function () {
			this._emit("layout.changed");
		};

		/**
		 * Check whether the layout should be persisted or not, depends on the active mode.
		 * @returns {Boolean}
		 */
		Layout.prototype.isLayoutStatePersisted = function () {
			return this._activeMode.isPersisted();
		};

		/**
		 * Get the state of the layout.
		 * @returns {Object}
		 */
		Layout.prototype.getLayoutState = function () {
			//to be implemented by specific layout
		};

		/**
		 * Switch the active mode.
		 * @param {LayoutConstants.LayoutMode} [mode] - if not provided it will use the default mode for the layout
		 */
		Layout.prototype._switchLayoutMode = function (mode) {
			if (this._activeMode === null || mode !== this._activeMode.getName()) {
				var newMode = null;
				if (mode !== undefined) {
					newMode = this._layoutModel.getLayoutMode(mode);
				} else {
					newMode = this._getDefaultLayoutMode();
				}
				if (newMode !== undefined) {
					if (this._activeMode !== null) {
						this._activeMode.toggle(false);
					}
					this._activeMode = newMode;
					this._activeMode.toggle(true);

					var addedItems = [];
					_.each(this._notVisibleItems, function (item, key) {
						if (this._activeMode.isItemAllowed(item.name)) {
							this._addItem(item);
							addedItems.push(item);
						}
					}, this);
					if (addedItems.length) {
						this._notVisibleItems = _.difference(this._notVisibleItems, addedItems);
					}
					this._updateItemVisibility();

					this._emit("changed.layoutMode");
				}
			}
		};

		/**
		 * Returns the default layout mode for the layout.
		 * @return {LayoutMode}
		 */
		Layout.prototype._getDefaultLayoutMode = function () {
			//to be implemented by specific layout
		};

		/**
		 * Get a reference to the active layout mode
		 * @returns {LayoutMode}
		 */
		Layout.prototype.getActiveLayoutMode = function () {
			return this._activeMode;
		};


		/**
		 * Returns the type of the layout.
		 * @return {Number}
		 */
		Layout.prototype.getLayoutType = function () {
			//to be implemented by specific layout
			return -1;
		};

		return Layout;
	});

define('web-viewer/layout/mobile/ModeBar',["web-viewer/layout/LayoutConstants",
		"web-viewer/core/PrivateChannel",
		"web-viewer/layout/component/ViewerToolbarButton",
		"underscore",
		"jquery"
	],
	function (LayoutConstants,
		PrivateChannel,
		ViewerToolbarButton,
		_,
		$) {
		"use strict";
		/**
		 * @class ModeBar The ModeBar is a mobile specific component. It displays the available modes on the left side of the application, and enables the user to switch between them
		 * @exports ModeBar
		 * @memberof Layout
		 * @constructor
		 *  
		 * @param {Core.ViewerFacade} f
		 * @param {JQueryObj} element - the element in which the mode bar should be rendered
		 * @param {Layout.MobileLayout} mobileLayout - a reference to the mobile layout
		 */
		var ModeBar = function (f, element, mobileLayout) {
			this.f = f;
			PrivateChannel.installTo(this, f, "layout.mobile.modebar");

			this.element = element;
			this.element.addClass("modeBar");
			this._mobileLayout = mobileLayout;
			this._visible = true;

			/**
			 * Modes registered for the mode bar
			 * {mode: {layout.LayoutMode},
			 *  icon: {String},
			 *  name: {String}}
			 */
			this._availableModes = {};

			this.switchMode(LayoutConstants.LayoutMode.MOBILE_VIEW);
			this.renderModeBar();

			this.f.on("layout.modeAvailabilityChanged", _.bind(function (mode, available, options) {
				if (available) {
					this._registerMode(mode, options.icon, options.title);
				} else {
					this._unregisterMode(mode.getName());
				}
			}, this));

			mobileLayout.on("changed.layoutMode", _.bind(this.renderModeBar, this));
			this.f.loadBundle("viewer", this.renderModeBar, this);
		};

		/**
		 * Toggle the mode bars visibility
		 */
		ModeBar.prototype.toggle = function () {
			this.setVisible(!this.isVisible());
		};

		/**
		 * Change the visibility of the mode bar
		 * @param {Boolean} visible
		 */
		ModeBar.prototype.setVisible = function (visible) {
			var oldValue = this._visible;
			this._visible = visible;
			if (oldValue !== visible) {
				this._emit("visible", visible);
			}
		};

		/**
		 * Check if the mode bar is visible
		 * @returns {Boolean}
		 */
		ModeBar.prototype.isVisible = function () {
			return this._visible;
		};

		/**
		 * Get the width of the mode bar
		 * @returns {Number} the width of the bar
		 */
		ModeBar.prototype.getWidth = function () {
			return this.isVisible() ? this.element.outerWidth(true) : 0;
		};

		/**
		 * Get the state of the mode bar, used for restoring the layout state
		 * @returns {Object}
		 */
		ModeBar.prototype.getState = function () {
			return {
				visible: this.isVisible()
			};
		};

		/**
		 * Restore the state of the mode bar
		 * @param {Object} state - state as returned by getState()
		 */
		ModeBar.prototype.setState = function (state) {
			this.setVisible(state.visible);
		};

		/**
		 * Register a mode in the Mode bar
		 */
		ModeBar.prototype._registerMode = function (mode, icon, title) {
			this._availableModes[mode.getName()] = {
				mode: mode,
				icon: icon,
				title: title
			};
			this.renderModeBar();
		};

		/**
		 * Unregister a mode in the Mode bar
		 */
		ModeBar.prototype._unregisterMode = function (name) {
			if (this._availableModes[name]) {
				delete this._availableModes[name];
			}
			this.renderModeBar();
		};

		/**
		 * Render mode bar
		 */
		ModeBar.prototype.renderModeBar = function () {
			this.element.html('');
			_.each(LayoutConstants.MobileLayoutModeBarOrder, _.bind(function (modeName) {
				var mobileMode = this._availableModes[modeName];
				if (mobileMode) {
					var button = $("<div>").addClass("mode");
					if (mobileMode.mode.isActive()) {
						button.addClass("active");
					}
					button.append($("<i>").addClass(mobileMode.icon).addClass("icon-large"));
					button.append($("<span>").html(this.f.i18n.t(mobileMode.title)));
					button.on("touchstart pointerdown", _.bind(function (name) {
						this.switchMode(name);
					}, this, mobileMode.mode.getName()));
					this.element.append(button);
				}
			}, this));
		};

		/**
		 * Activate the mode by name
		 * @param {String} name
		 */
		ModeBar.prototype.switchMode = function (name) {
			this.f.emit("layout.switchLayoutMode", name);
			this.renderModeBar();
		};

		return ModeBar;
	});

define('web-viewer/layout/mobile/NavigationBar',["web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/utils/ViewerUtils",
		"underscore",
		"jquery"
	],
	function (
		ViewerToolbarButton,
		LayoutConstants,
		ViewerUtils,
		_,
		$) {
		"use strict";

		/**
		 * @class NavigationBar The NavigationBar is a mobile specific component. It displays the navigation bar with options to show widgets, toggle the mode bar and see the document title
		 * @exports NavigationBar
		 * @memberof Layout
		 * @constructor
		 *  
		 * @param {Core.ViewerFacade} f
		 * @param {JQueryObj} element - the element in which the navigation bar should be rendered
		 * @param {Layout.MobileLayout} mobileLayout - a reference to the mobile layout
		 */
		var NavigationBar = function (f, element, mobileLayout) {
			this.f = f;
			this._mobileLayout = mobileLayout;

			/**
			 * Maps names of components to the context in which they are shown
			 */
			this._components = {};

			this.element = element;
			this.element.addClass("navigationBar");
			this._mobileLayout = mobileLayout;
			this._left = $("<div>").addClass("navigationLeft").appendTo(this.element);
			this._right = $("<div>").addClass("navigationRight").appendTo(this.element);

			/**
			 * @member {ViewerToolbarButton}
			 */
			this._hamburger = new ViewerToolbarButton(this.f, "menu", "icon-WCR_Hamburger", true, _.bind(function () {
				this._mobileLayout.toggleModeBar();
			}, this));

			this._hamburger.element.addClass("hamburgerButton");
			this._left.append(this._hamburger.element);

			// show the exit icon if needed
			if (f.getProperty("returnurl")) {
				this._returnButton = new ViewerToolbarButton(this.f, "exit", "icon-WCR_Exit", false, _.bind(function () {
					this.f.emit("viewer.request.close", f.getProperty("returnurl"));
				}, this));
				this._right.append(this._returnButton.element);
			}

			if (mobileLayout.supportsFullscreen()) {
				this._fullscreenButton = new ViewerToolbarButton(this.f, "fullscreen", "icon-WCR_FullScreen", false, _.bind(function () {
					mobileLayout.toggleFullscreen(!mobileLayout.isFullscreen());
					this._updateFullscreenButton();
				}, this));

				$(window).on("resize", _.debounce(_.bind(this._updateFullscreenButton, this), 100));
				this._updateFullscreenButton();

				this._right.prepend(this._fullscreenButton.element);
			}

			// show the button to switch to desktop layout if the device is not mobile
			this.f.requestModule("featureDetector", _.bind(function (featureDetector) {
				if (!featureDetector.isMobile() || !featureDetector.hasTouch()) {
					// not a mobile device, but the mobile layout was loaded
					this._switchLayoutModeButton = new ViewerToolbarButton(this.f, "switchToDesktop", "icon-WCR_Desktop", false, _.bind(function () {
						this.f.switchLayout(LayoutConstants.LayoutType.DESKTOP);
					}, this));

					this._right.prepend(this._switchLayoutModeButton.element);
				}
			}, this));

			this._mobileLayout.on("changed.layoutMode", _.bind(this._updateButtonVisibility, this));


		};

		/**
		 * Update full screen button
		 */
		NavigationBar.prototype._updateFullscreenButton = function () {
			this._fullscreenButton.setIcon(this._mobileLayout.isFullscreen() ? "icon-WCR_NormalScreen" : "icon-WCR_FullScreen");
		};

		/**
		 * Update the visibility of the goto desktop and full screen buttons based on the active mode
		 */
		NavigationBar.prototype._updateButtonVisibility = function () {
			var activeLayoutMode = this._mobileLayout.getActiveLayoutMode();
			if (activeLayoutMode && activeLayoutMode.getName() === LayoutConstants.LayoutMode.MOBILE_VIEW) {
				// full screen button is available in view mode only
				if (this._fullscreenButton) {
					this._fullscreenButton.show();
				}
			} else {
				// hide full screen button in other modes
				if (this._fullscreenButton) {
					this._fullscreenButton.hide();
				}
			}
		};

		/**
		 * Add toolbar at left side. Add to extra div, so they can be shown and hidden ignoring the module behavior.
		 * e.g. toolbar can be hidden by module, so we don't want to show it when it becomes available
		 * @param {ToolbarComponent} toolbar
		 */
		NavigationBar.prototype.appendToolbarLeft = function (toolbar) {
			var context = $("<div />");
			this._left.append(context.append(toolbar.element));
			this._components[toolbar.name] = context;
		};

		/**
		 * Add toolbar at right side. Add to extra div, so they can be shown and hidden ignoring the module behavior.
		 * e.g. toolbar can be hidden by module, so we don't want to show it when it becomes available
		 * @param {ToolbarComponent} toolbar
		 */
		NavigationBar.prototype.prependToolbarRight = function (toolbar) {
			var context = $("<div />");
			this._right.prepend(context.append(toolbar.element));
			this._components[toolbar.name] = context;
		};

		/**
		 * Toggle visibility of a component on the navigation bar.
		 * @param {String} name - name of component
		 * @param {Boolean} show - will show if true and hide if false.
		 */
		NavigationBar.prototype.toggleComponent = function (name, show) {
			if (this._components[name]) {
				if (show) {
					this._components[name].show();
				} else {
					this._components[name].hide();
				}
			}
		};

		return NavigationBar;
	});

define('web-viewer/layout/LayoutMode',["web-viewer/layout/LayoutConstants"],
	function (
		LayoutConstants

	) {
		"use strict";

		/**
		 * @exports LayoutMode
		 * @memberof Layout
		 * @class
		 * Defines a general implementation of layout mode, the mode decides if items should be added to the layout
		 * and if the state should be persisted.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 * @param {LayoutConstants.LayoutMode} name
		 */
		var LayoutMode = function (f, layout, name) {
			this.f = f;

			/**
			 * Name of the layoutMode
			 */
			this._name = name;

			/**
			 * Is the layoutMode active
			 */
			this._isActive = false;

			/**
			 * The layout for which this mode is active
			 */
			this._layout = layout;

			/**
			 * is the modepanel expanded
			 */
			this._isModePanelExpanded = true;
		};

		/**
		 * Get the name of the mode.
		 * @returns {LayoutConstants.LayoutMode}
		 */
		LayoutMode.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		LayoutMode.prototype.toggle = function (toggle) {
			//to be implemented by specific layout mode
		};

		/**
		 * Checks if the item is allowed for this layout mode. Prevents channels and CAD widget from loading.
		 *
		 * @param {String} itemname
		 * @returns {Boolean}
		 */
		LayoutMode.prototype.isItemAllowed = function (itemname) {
			//to be implemented by specific layout mode
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		LayoutMode.prototype.isPersisted = function () {
			//to be implemented by specific layout mode
		};

		/**
		 * Checks if the mode is active at the moment.
		 */
		LayoutMode.prototype.isActive = function () {
			return this._isActive;
		};

		/**
		 * Decide whether or not to enable the mode panel
		 */
		LayoutMode.prototype.isModePanelEnabled = function () {
			// should be overwritten for mobile layouts in which the mode panel should be shown
			return false;
		};

		/**
		 * If the mode panel is enabled, should it be expanded by default?
		 * @returns {Boolean} is de mode panel expanded by default?
		 */
		LayoutMode.prototype.isModePanelExpanded = function () {
			return this._isModePanelExpanded;
		};

		/**
		 * Should the widget titles be shown?
		 * @returns {Boolean}
		 */
		LayoutMode.prototype.showModePanelWidgetTitles = function () {
			return true;
		};

		/**
		 * Is the modepnale expanded or not?
		 * This method is called by the layout if a modepanel was expanded or collapsed
		 * @param {Boolean} expanded
		 */
		LayoutMode.prototype.setIsmodePanelExpanded = function (expanded) {
			this._isModePanelExpanded = expanded;
		};

		/** 
		 * This method should return a list of widgets that should appear in the modepanel (or an empty array if none should be shown)
		 * @returns {Array.<String>} widget names of the widgets shown in the mode panel
		 */
		LayoutMode.prototype.getModePanelContent = function () {
			return [];
		};

		/**
		 * Get the name of the widget which should be shown in the Main Panel
		 * By default this is the ViewPortWidget (note this is only supported by the mobile layout at the moment)
		 * @returns {String} widget name of the widget shown in the main panel
		 */
		LayoutMode.prototype.getMainPanelContent = function () {
			return LayoutConstants.Widget.VIEWPORT;
		};


		return LayoutMode;
	});

define('web-viewer/layout/mobile/MobileViewMode',["web-viewer/layout/LayoutConstants",
		"web-viewer/layout/LayoutMode"
	],
	function (
		LayoutConstants,
		LayoutMode
	) {
		"use strict";

		/**
		 * @exports MobileViewMode
		 * @memberof Layout
		 * @class
		 * Default mode of the mobile layout
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 */
		var MobileViewMode = function (f, layout) {
			LayoutMode.call(this, f, layout, LayoutConstants.LayoutMode.MOBILE_VIEW);

			// by default the modepanel is collapsed for the mobile view mode
			this.setIsmodePanelExpanded(false);

			this.f.emit("layout.modeAvailabilityChanged", this, true, {
				icon: "icon-WCR_View",
				title: "ViewControlsTitle"
			});
		};

		MobileViewMode.prototype = new LayoutMode();
		MobileViewMode.prototype.constructor = MobileViewMode;

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		MobileViewMode.prototype.toggle = function (toggle) {
			if (toggle === undefined) {
				toggle = !this._isActive;
			}
			this._isActive = toggle;
		};

		/**
		 * Checks if the item is allowed for this layout mode.
		 * Allow viewport, documentselector, approval toolbar and message widget
		 * @param {String} itemname - the name of the item
		 * @returns {Boolean}
		 */
		MobileViewMode.prototype.isItemAllowed = function (itemname) {
			return (itemname === LayoutConstants.Widget.VIEWPORT ||
					itemname === LayoutConstants.ToolbarComponent.DOCUMENTSELECTOR ||
					itemname === LayoutConstants.ToolbarComponent.APPROVAL ||
					itemname === LayoutConstants.Widget.MESSAGE) ||
				/* Channels and CAD should not be shown restricted mode */
				((itemname === LayoutConstants.Widget.CHANNELS ||
						itemname === LayoutConstants.Widget.CAD) &&
					this.f.getProperty("viewerStyle") != LayoutConstants.ViewerStyle.RESTRICTED);
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		MobileViewMode.prototype.isPersisted = function () {
			return true;
		};

		/**
		 * The modepanel is enabled in the view mode
		 * @returns {Boolean}
		 * @override
		 */
		MobileViewMode.prototype.isModePanelEnabled = function () {
			return this.f.getProperty("viewerStyle") != LayoutConstants.ViewerStyle.RESTRICTED;
		};

		/**
		 * Display the cad and channels widget
		 * @override
		 */
		MobileViewMode.prototype.getModePanelContent = function () {
			return [LayoutConstants.Widget.CAD, LayoutConstants.Widget.CHANNELS];
		};


		return MobileViewMode;
	});

define('web-viewer/layout/mobile/AnnotationViewMode',["web-viewer/layout/LayoutConstants",
		"web-viewer/layout/LayoutMode"
	],
	function (
		LayoutConstants,
		LayoutMode
	) {
		"use strict";

		/**
		 * @exports AnnotationViewMode
		 * @memberof Layout
		 * @class
		 * Mode for mobile layout, shows the annotation widget
		 * @constructor
		 * @augments LayoutMode
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 */
		var AnnotationViewMode = function (f, layout) {
			LayoutMode.call(this, f, layout, LayoutConstants.LayoutMode.MOBILE_ANNOTATION);

			this.f.emit("layout.modeAvailabilityChanged", this, true, {
				icon: "icon-WCR_Annotate",
				title: "AnnToolsTitle"
			});
		};

		AnnotationViewMode.prototype = new LayoutMode();
		AnnotationViewMode.prototype.constructor = AnnotationViewMode;

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		AnnotationViewMode.prototype.toggle = function (toggle) {
			if (toggle === undefined) {
				toggle = !this._isActive;
			}
			this._isActive = toggle;
		};

		/**
		 * Checks if the item is allowed for this layout mode. Prevents channels and CAD widget from loading.
		 * 
		 * @param {String} itemname
		 * @returns {Boolean}
		 */
		AnnotationViewMode.prototype.isItemAllowed = function (itemname) {
			// only allow viewport widget
			return itemname === LayoutConstants.Widget.VIEWPORT || itemname === LayoutConstants.Widget.ANNOTATION || itemname === LayoutConstants.ToolbarComponent.DOCUMENTSELECTOR || itemname === LayoutConstants.ToolbarComponent.APPROVAL || itemname === LayoutConstants.ToolbarComponent.ANNOTATION || itemname === LayoutConstants.Widget.MESSAGE;
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		AnnotationViewMode.prototype.isPersisted = function () {
			return false;
		};

		/**
		 * Decide whether or not to show the mode panel
		 * @override
		 */
		AnnotationViewMode.prototype.isModePanelEnabled = function () {
			return true;
		};

		/**
		 * @override
		 */
		AnnotationViewMode.prototype.showModePanelWidgetTitles = function () {
			return false;
		};

		/**
		 * list of needed widgets in modepanel for annotation view mode
		 * @returns {Array}    
		 * @override
		 */
		AnnotationViewMode.prototype.getModePanelContent = function () {
			return [LayoutConstants.Widget.ANNOTATION];
		};

		return AnnotationViewMode;
	});

define('web-viewer/layout/mobile/MobileCompareMode',["web-viewer/layout/LayoutConstants",
		"web-viewer/layout/LayoutMode"
	],
	function (
		LayoutConstants,
		LayoutMode
	) {
		"use strict";

		/**
		 * @exports MobileCompareMode
		 * @memberof Layout
		 * @class
		 * Mode for mobile layout, shows the compare view
		 * @constructor
		 * @augments LayoutMode
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 */
		var MobileCompareMode = function (f, layout) {
			LayoutMode.call(this, f, layout, LayoutConstants.LayoutMode.MOBILE_COMPARE);


			this._emitModeAvailability(false);
			this.f.whenModuleAvailable("compare", function (compareModule) {
				var isCompareAvailable = compareModule.isCompareAvailable();
				this._emitModeAvailability(isCompareAvailable);

				compareModule.on("changed.mode", _.bind(function (oldMode, newMode) {
					if (newMode && !compareModule.isReaderSpreadActive()) {
						this.f.emit("layout.switchLayoutMode", LayoutConstants.LayoutMode.MOBILE_COMPARE);
					}
				}, this));

				if (!isCompareAvailable) {
					compareModule.on("compareAvailable", _.bind(function () {
						this._emitModeAvailability(true);
					}, this));
				}

				// only execute this code after initialization
				_.defer(_.bind(function () {
					if (compareModule.getActiveMode() && !compareModule.isReaderSpreadActive()) {
						this.f.emit("layout.switchLayoutMode", LayoutConstants.LayoutMode.MOBILE_COMPARE);
					}
				}, this));

			}, this);

		};

		MobileCompareMode.prototype = new LayoutMode();
		MobileCompareMode.prototype.constructor = MobileCompareMode;


		/**
		 * Notify the change in availability of the mode
		 * @param {Boolean} available [[Description]]
		 */
		MobileCompareMode.prototype._emitModeAvailability = function (available) {
			this.f.emit("layout.modeAvailabilityChanged", this, available, {
				icon: "icon-WCR_Compare",
				title: "CompareToolsTitle"
			});
		};

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		MobileCompareMode.prototype.toggle = function (toggle) {
			if (toggle === undefined) {
				toggle = !this._isActive;
			}

			this._isActive = toggle;

			var compareModule = this.f.getModule("compare");
			// turn off compare when leaving the mode
			if (!toggle) {
				compareModule.activateModeByName();
			} else {
				if (!compareModule.getActiveMode()) {
					compareModule.activateLastUsedCompareMode();
				}
			}
		};

		/**
		 * Checks if the item is allowed for this layout mode. Prevents channels and CAD widget from loading.
		 * 
		 * @param {String} itemname
		 * @returns {Boolean}
		 */
		MobileCompareMode.prototype.isItemAllowed = function (itemname) {
			// only allow viewport widget
			return itemname === LayoutConstants.Widget.VIEWPORT || itemname === LayoutConstants.Widget.COMPARE || itemname === LayoutConstants.ToolbarComponent.COMPARE || itemname === LayoutConstants.ToolbarComponent.APPROVAL || itemname === LayoutConstants.Widget.MESSAGE;
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		MobileCompareMode.prototype.isPersisted = function () {
			return false;
		};

		/**
		 * Decide whether or not to show the mode panel
		 * @override
		 */
		MobileCompareMode.prototype.isModePanelEnabled = function () {
			return true;
		};

		/**
		 * @override
		 */
		MobileCompareMode.prototype.showModePanelWidgetTitles = function () {
			return true;
		};

		/**
		 * list of needed widgets in modepanel for mobile compare mode
		 * @returns {Array}    
		 * @override
		 */
		MobileCompareMode.prototype.getModePanelContent = function () {
			return [LayoutConstants.Widget.COMPARE];
		};

		return MobileCompareMode;
	});

define('web-viewer/layout/mobile/Mobile3DViewMode',["web-viewer/layout/LayoutConstants",
		"web-viewer/layout/LayoutMode"
	],
	function (
		LayoutConstants,
		LayoutMode
	) {
		"use strict";

		/**
		 * @exports Mobile3DViewMode
		 * @memberof Layout
		 * @class
		 * Mode for mobile layout, shows the annotation widget
		 * @constructor
		 * @augments LayoutMode
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 */
		var Mobile3DViewMode = function (f, layout) {
			LayoutMode.call(this, f, layout, LayoutConstants.LayoutMode.MOBILE_3D);

			// by default this mode is disabled
			this._emitModeAvailability(false);

			var coldBootCheck3DPerspective = _.bind(function () {
				this._emitModeAvailability(this.f.getActivePage().get("has3DPerspective"));
			}, this);

			this.f.on("doc.activePageParametersChange", coldBootCheck3DPerspective);

			// check if a 3D perspective is availble for the current page (this check is needed because the 3D module is not loaded by default)
			if (this.f.getActivePage() && this.f.getActivePage().isParametersAvailable()) {
				coldBootCheck3DPerspective();
			}

			this.f.whenModuleAvailable("view3D", function (view3DModule) {
				// whenever 3D module becomes available, start listening to the module
				// stop listening to active page parameters changes
				this.f.off("doc.activePageParametersChange", coldBootCheck3DPerspective);

				// if 3D view is started (and this widget will be published), switch to 3D mode
				view3DModule.on("start3D", _.bind(function () {
					this._emitModeAvailability(true);
					this.f.emit("layout.switchLayoutMode", LayoutConstants.LayoutMode.MOBILE_3D);
				}, this));

				// 3D view is stopped (this will be because of switching documents)
				view3DModule.on("stop3D", _.bind(function () {
					// if the mobile 3D view mode is active, and there is a 3D perspective for the active file: make sure to iniate and show the 3D view of the current fikel
					if (this.isActive() && view3DModule.has3DPerspective()) {
						view3DModule.show3D();
					}
				}, this));

				// whenever the 3D perspective availability changes, hide/show the 3D view mode
				view3DModule.on("has3DPerspective", _.bind(function (is3DAvailable) {
					this._emitModeAvailability(is3DAvailable);
				}, this));
				this._emitModeAvailability(view3DModule.has3DPerspective());

			}, this);


		};

		Mobile3DViewMode.prototype = new LayoutMode();
		Mobile3DViewMode.prototype.constructor = Mobile3DViewMode;

		/**
		 * Notify the change in availability of the mode
		 * @param {Boolean} available [[Description]]
		 */
		Mobile3DViewMode.prototype._emitModeAvailability = function (available) {
			this.f.emit("layout.modeAvailabilityChanged", this, available, {
				icon: "icon-WCR_3D-Create",
				title: "View3DBtnGroupTitle"
			});

			if (this.isActive() && !available) {
				// if the 3D view mode is active, we switch to the default mode (since it doesn't make sense anymore to have it)
				this.f.emit("layout.switchLayoutMode");
			}
		};

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		Mobile3DViewMode.prototype.toggle = function (toggle) {
			if (toggle === undefined) {
				toggle = !this._isActive;
			}
			this._isActive = toggle;

			if (toggle) {
				// request the view3D module to show the 3D widget
				this.f.requestModule("view3D", _.bind(function (view3DModule) {
					view3DModule.show3D();
				}, this), false);
			}
		};

		/**
		 * Checks if the item is allowed for this layout mode. Prevents channels and CAD widget from loading.
		 *
		 * @param {String} itemname
		 * @returns {Boolean}
		 */
		Mobile3DViewMode.prototype.isItemAllowed = function (itemname) {
			// only allow viewport widget
			return itemname === LayoutConstants.Widget.VIEWPORT ||
				itemname === LayoutConstants.ToolbarComponent.DOCUMENTSELECTOR ||
				itemname === LayoutConstants.ToolbarComponent.APPROVAL ||
				itemname === LayoutConstants.Widget.MESSAGE ||
				itemname === LayoutConstants.Widget.VIEW3D;
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		Mobile3DViewMode.prototype.isPersisted = function () {
			return false;
		};

		/**
		 * Decide whether or not to show the mode panel
		 * @override
		 */
		Mobile3DViewMode.prototype.isModePanelEnabled = function () {
			return false;
		};

		/**
		 * @override
		 */
		Mobile3DViewMode.prototype.showModePanelWidgetTitles = function () {
			return false;
		};

		/**
		 * list of needed widgets in modepanel for annotation view mode
		 * @returns {Array}
		 * @override
		 */
		Mobile3DViewMode.prototype.getModePanelContent = function () {
			return [];
		};

		/**
		 * We want the 3D widget as Main Content
		 * @override
		 */
		Mobile3DViewMode.prototype.getMainPanelContent = function () {
			return LayoutConstants.Widget.VIEW3D;
		};

		return Mobile3DViewMode;
	});

define('web-viewer/layout/WidgetWrapper',["jquery",
		"utils/ShapeUtils"
	],
	function ($, ShapeUtils) {
		"use strict";

		/**
		 * A widgetwrapper is a wrapper for a widget, which makes enables the widget to be displayed with some extra controls
		 * @param {AbstractWidget} widget          - the widget to wrap
		 * @param {String}         widgetName      - the name of the widget
		 * @param {Boolean} allowCollapse - is collapsing the widget allowed?
		 */
		var WidgetWrapper = function (widget, widgetName, allowCollapse) {
			this._headerEnabled = true;
			// main wrapper
			this.element = $("<div/>").addClass("widgetWrapper");
			this.element[0].wrapper = this;

			// webkit has a bug with iFrames, which appearantly they won't be solving soon
			// if an input in an iframe is focused, the page scrolls to the iframe
			// this can occur when TinyMCE is used... we can fix it by scrolling to top whenever the context is scrolled
			this.element.scroll(function (event) {
				$(this).scrollTop(0);
			});

			this.name = widgetName;

			// header + title
			this.header = $("<div/>").addClass("header");
			this.headerText = $("<h3>", {
				text: widget.getTitle(),
				title: widget.getTitle()
			});
			this.headerHandle = $("<div/>").addClass("headerHandle").append(this.headerText);
			if (allowCollapse) {
				this.headerHandle.click(_.bind(function () {
					this.toggle();
				}, this));
			}
			this.header.append(this.headerHandle);

			this.headerExtra = $("<div/>").addClass("headerExtra");
			this.header.append(this.headerExtra);

			// listen to title changed of widget
			widget.on("titlechanged", _.bind(this._onTitleChanged, this));

			// content holder
			this.content = $("<div/>").addClass("content");

			// create dom structure
			this.element.append(this.header, this.content);

			// activate the widget
			widget.setContext(this.content, this.headerExtra);

			//is the widget resizable
			this.resizable = false;

			// the widget
			this.widget = widget;

			this.isModal = false;

			this.isCollapsed = false;
			this._isVisible = true;

			//location; only use if the widgetwrapper is hidden
			this._locationWhenHidden = new ShapeUtils.Point();
		};

		/**
		 * Convert the widget in to a modal widget or normal widget
		 * @param modal {boolean} should the widget be modal or not?
		 * @param top {int} position top
		 * @param left {int} position left
		 * @param width {int} width of modal window
		 * @param height {int} height of modal window
		 */
		WidgetWrapper.prototype.setModal = function (modal, top, left, width, height) {
			this.widget.setModal(modal);
			if (modal) {

				this.element.css({
					position: "absolute",
					top: top,
					left: left,
					width: width || this.element.width() || undefined,
					height: height || this.element.height() || undefined
				});
				var newHeight = height || this.element.height() || 0;
				this.content.css("height", newHeight - this.getHeaderHeight());

				this.setResizable(!this.isCollapsed);

				this.isModal = true;
			} else {
				this.isModal = false;
				// restore normal css
				this.element.css({
					position: "relative",
					top: "",
					left: "",
					width: "auto"
				});

				this.setResizable(false);
			}
		};

		/**
		 * Function called when the title of the widget changed
		 * @param {String} newTitle - The new title
		 */
		WidgetWrapper.prototype._onTitleChanged = function (newTitle) {
			this.headerText.text(newTitle).attr("title", newTitle);
		};

		/**
		 * Toggle collapse state of the widget
		 */
		WidgetWrapper.prototype.toggle = function () {
			this.setCollapsed(!this.isCollapsed);
		};

		/**
		 * Hides/shows a widget
		 * @param {Boolean} isVisible
		 */
		WidgetWrapper.prototype.setVisible = function (isVisible) {
			this._isVisible = isVisible;
			if (isVisible) {
				this.element.removeClass("hiddenElement");
			} else {
				var offset = this.element.offset();
				//location is only used when the widgetwrapper is hidden, if visible the element is used
				this._locationWhenHidden = new ShapeUtils.Point(offset.left, offset.top);
				this.element.addClass("hiddenElement");
			}
		};

		/**
		 * Is the widgetwrapper visible.
		 * @returns {Boolean}
		 */
		WidgetWrapper.prototype.isVisible = function () {
			return this._isVisible;
		};

		/**
		 * (un)Collapse the widget
		 * @param {Boolean} collapsed
		 */
		WidgetWrapper.prototype.setCollapsed = function (collapse) {
			var wasCollapsed = this.isCollapsed;
			this.element.toggleClass("collapsed", collapse);
			this.isCollapsed = collapse;
			this.widget.setCollapsed(collapse);

			if (this.isModal && wasCollapsed !== collapse) {
				var newHeight = Math.min($(window).height() - (this.element.offset().top - $(window).scrollTop()), this.getPreferredSize().height);
				this.setHeight(newHeight);
				if (!collapse) {
					this.setResizable(true);
				}
				this.setResizable(!this.isCollapsed);
				this.widget.fireModalResize();
			}

		};

		WidgetWrapper.prototype.setResizable = function (resizable) {
			if (this.resizable && !resizable) {
				this.element.resizable('destroy');
				this.resizable = false;
			} else if (!this.resizable && resizable) {
				//make the element resizable
				var minimumSize = this.getMinimumSize();
				this.element.resizable({
					minWidth: Math.max(100, minimumSize.width),
					minHeight: Math.max(40, minimumSize.height),
					stop: _.bind(function (event, ui) {
						this.setHeight(ui.size.height);
						this.widget.fireModalResize();
					}, this),
					resize: _.throttle(_.bind(function (event, ui) {
						this.setHeight(ui.size.height);
						this.widget.resizing();
					}, this), 40)
				});
				this.resizable = true;
			}
		};

		/**
		 * Get the ID of the enclosed widget
		 * @returns
		 */
		WidgetWrapper.prototype.getID = function () {
			return this.widget.getID();
		};

		/**
		 * Get the minimum size of the widget content
		 *
		 * note it is possible that this size is not respected by the layoutmanager
		 *
		 * @returns {ShapeUtils.Size}
		 */
		WidgetWrapper.prototype.getMinimumSize = function () {
			if (this.isCollapsed) {
				return new ShapeUtils.Size(0, this.getHeaderHeight());
			} else {
				var minSize = this.widget.getMinimumSize().copy();
				minSize.height += this.getHeaderHeight() + 3;

				return minSize;
			}
		};

		/**
		 * Get the preferred size of the widget content
		 *
		 * note it is possible that this size is not respected by the layoutmanager
		 *
		 * @returns {ShapeUtils.Size} if undefined, no preferred size is taken in to account by layoutmanager, the layout managers tries to guess the needed size
		 */
		WidgetWrapper.prototype.getPreferredSize = function () {
			if (this.isCollapsed) {
				return new ShapeUtils.Size(0, this.getHeaderHeight());
			} else {
				var prefSize = this.widget.getPreferredSize();
				if (prefSize) {
					prefSize = prefSize.copy();
					prefSize.height += this.getHeaderHeight() + 3;
					return prefSize;
				} else {
					return undefined;
				}
			}
		};

		/**
		 * Get the maximum dimension of the widget content
		 *
		 * @returns {ShapeUtils.Size} if undefined the widget can be made as big as the user wants
		 */
		WidgetWrapper.prototype.getMaximumSize = function () {
			if (this.isCollapsed) {
				return new ShapeUtils.Size(1000, this.getHeaderHeight());
			} else {
				var maxSize = this.widget.getMaximumSize();
				if (maxSize) {
					maxSize = maxSize.copy();
					maxSize.height += this.getHeaderHeight() + 3;
					return maxSize;
				} else {
					return undefined;
				}
			}
		};


		/**
		 * Get the height of the header
		 * @returns
		 */
		WidgetWrapper.prototype.getHeaderHeight = function () {
			if (this._headerEnabled) {
				return this.header.outerHeight(true);
			} else {
				return 0;
			}
		};

		/**
		 * Get the current content height of the widget
		 * @returns
		 */
		WidgetWrapper.prototype.getContentHeight = function () {
			return this.content.getOuterHeight(true);
		};

		/**
		 * Will the widgetwrapper extend if needed?
		 * @returns
		 */
		WidgetWrapper.prototype.willFill = function () {
			return this.widget.willFill();
		};

		/**
		 * Set the height of the widgetwrapper
		 * @param height
		 */
		WidgetWrapper.prototype.setHeight = function (height) {
			var prevContentHeight = this.content.height();
			var newContentHeight = height - this.getHeaderHeight();
			this.element.css("height", height);
			this.content.css("height", newContentHeight);



			if (newContentHeight != prevContentHeight) {
				this.widget.resized();

				this.content.css("overflow", "hidden");
				_.defer(function (el) {
					el.css("overflow", "auto");
				}, this.content);
			}
		};

		/**
		 * Get the size of the WidgetWrapper
		 *
		 * @returns {ShapeUtils.Size}
		 */
		WidgetWrapper.prototype.getSize = function () {
			return new ShapeUtils.Size(this.element.width(), this.element.height());
		};

		/**
		 * Get the outer size of the widget wrapper, including borders and padding
		 *
		 * @returns {ShapeUtils.Size}
		 */
		WidgetWrapper.prototype.getOuterSize = function () {
			return new ShapeUtils.Size(this.element.outerWidth(true), this.element.outerHeight(true));
		};

		/**
		 * Get the location of the WidgetWrapper
		 *
		 * @returns {ShapeUtils.Point}
		 */
		WidgetWrapper.prototype.getLocation = function () {
			if (this._isVisible) {
				var offset = this.element.offset();
				return new ShapeUtils.Point(offset.left, offset.top);
			} else {
				return this._locationWhenHidden;
			}
		};

		/**
		 * Enable/Disable the header display of the widget wrapper
		 * @param {Boolean} enable
		 */
		WidgetWrapper.prototype.setHeaderEnabled = function (enable) {
			this._headerEnabled = enable;
			this.header.toggleClass("displayNone", !enable);
		};

		return WidgetWrapper;
	});

define('web-viewer/layout/LayoutUtils',["underscore"],
	function (_) {
		"use strict";
		/**
		 * LayoutUtils contains some helper functions used by different components\
		 * @exports LayoutUtils
		 * @memberof Layout
		 */
		return {
			/**
			 * Layout al WidgetWrappers in a certain element.
			 * It respects the "willFill" and minSize, maxSize and preferredSize of the widgets
			 * When there is not enough space to fit all widgets, it adds a scrollbar to containerElement
			 * @param {JQueryObj} containerElement - the element in which the widgets reside. The element should have a height for the algorithm to work
			 * @param {Number} widgetBorder - this is some extra spacing that is not taken into account by outerheight
			 */
			layoutWidgets: function (containerElement, widgetBorders) {
				var widgets = containerElement.find(".widgetWrapper");
				var innerHeight = containerElement.innerHeight() - widgetBorders;
				var data = [];
				var w;
				var currentHeight = 0;
				var max;
				var canShrink = [];
				var canGrow = [];

				// first build up the needed data structure
				for (var i = 0; i < widgets.length; i++) {
					var widget = widgets[i].wrapper;
					if (widget.isVisible()) {
						max = undefined;
						if (widget.getMaximumSize()) {
							max = widget.getMaximumSize().height;
						}
						w = {
							widget: widget,
							min: widget.getMinimumSize().height,
							pref: widget.getPreferredSize().height,
							max: max,
							height: 0
						};
						data.push(w);

						//calculate preferred height (taking into account min and max)

						w.height = Math.max(w.min, w.pref);
						if (w.max) {
							w.height = Math.min(w.max, w.height);
						}

						w.height = Math.min(w.height, innerHeight);

						// first we only want to shrink willFill elements 
						// keep track if the element can be shrinked
						if (w.widget.willFill() && w.height > w.min) {
							canShrink.push(w);
						}
						currentHeight += w.height + widgetBorders; // 4: 2px padding and 2px border
					}
				}
				var pixelsNeeded = 0;
				// this function is used to shrink all shrinkable elements
				// it is defined as a inner function because it is used two times
				var shrinkShrinkables = function () {
					pixelsNeeded = currentHeight - innerHeight;
					//as long as we need to shrink, and we have shrinkable elements, try to shrink
					while (pixelsNeeded > 0 && canShrink.length > 0) {
						//we want to shrink proportionally
						var w = _.first(canShrink);
						var shrinkablesPrefHeight = _.reduce(canShrink, function (memo, w) {
							return memo + w.pref;
						}, 0);
						var diff = Math.ceil((w.pref / shrinkablesPrefHeight) * pixelsNeeded);

						canShrink = _.rest(canShrink, 1);

						var prevHeight = w.height;
						w.height = Math.max(w.min, w.height - diff);

						var pixelsRemoved = prevHeight - w.height;
						pixelsNeeded -= pixelsRemoved;
						currentHeight -= pixelsRemoved;
						if (w.height > w.min) {
							canShrink.push(w);
						}
					}
				};

				// first try to shrink all "willFill" elements... these are usually very flexible elements, so they care less if they are shrunk
				shrinkShrinkables();

				if (pixelsNeeded > 0) {
					// we need another round of shrinking...
					// now we take every element that could be shrunk (so its current height is still higher than its minimum size
					canShrink = _.filter(data, function (w) {
						return w.height > w.min;
					});

					shrinkShrinkables();
				}



				pixelsNeeded = currentHeight - innerHeight;
				if (pixelsNeeded > 0) {
					// oh ooh... we can't get small enough... show a scrollbar
					containerElement.css("overflow-y", "auto");
				} else {
					containerElement.css("overflow-y", "hidden");
				}

				// at this point, we should no longer exceed the maximum size (we could still be too big if an element has a big minimum size 

				// check if we can extend some elements
				var pixelsLeft = innerHeight - currentHeight;
				if (pixelsLeft > 0) {

					_.each(data, function (w) {
						if ((!w.max || w.height < w.max) && (w.height < w.pref || w.widget.willFill())) {
							canGrow.push(w);
						}
					});

					while (pixelsLeft > 0 && canGrow.length > 0) {
						var diff = Math.ceil(pixelsLeft / canGrow.length);
						w = _.first(canGrow);
						canGrow = _.rest(canGrow, 1);

						var prevHeight = w.height;
						if (w.max) {
							w.height = Math.min(w.height + diff, w.max);
						} else {
							w.height = w.height + diff;
						}
						var pixelsAdded = w.height - prevHeight;

						pixelsLeft -= pixelsAdded;
						if ((!w.max || w.height < w.max) && (w.height < w.pref || w.widget.willFill())) {
							canGrow.push(w);
						}
					}
				}

				// set all heights
				_.each(data, function (w) {
					w.widget.setHeight(w.height);
				});
			}

		};
	});

define('web-viewer/layout/mobile/ModePanel',["web-viewer/core/PrivateChannel",
		"web-viewer/layout/component/MiniButton",
		"web-viewer/layout/LayoutUtils",
		"utils/ShapeUtils",
		"jquery",
		"underscore"
	],
	function (PrivateChannel,
		MiniButton,
		LayoutUtils,
		ShapeUtils,
		$,
		_
	) {
		"use strict";


		/**
		 * ModePanel interface element (for mobile, information of the mode can be displayed here), it is possible to collapse the modal panel.
		 *
		 * @constructor
		 * @memberof Layout/mobile
		 * @param {ViewerFacade} f
		 * @param {String} name - name of the panel
		 */
		var ModePanel = function (f, name) {
			PrivateChannel.installTo(this, f, ["modePanel", name]);

			this.f = f;

			/**
			 * @member {String} name of the mode panel
			 */
			this.name = name;

			/**
			 * @member {JQueryObj} Element the mode panel uses to add the collapse button to
			 */
			this.element = $("<div/>").addClass("modePanel");

			/**
			 * @member {JQueryObj} div that contains the content of the mode panel
			 */
			this._content = $("<div/>");

			/** 
			 * @member {Map.<String,JQueryObj>} keeps all available content and maps the name of the content to the content
			 */
			this._contentMap = {};

			/**
			 * @member {JQueryObj} content that is currently active in the mode panel
			 */
			this._activeContent = null;

			//append the content to the mode panel element
			this.element.append(this._content);

			/**
			 * @member {MiniButton} button used to collapse the mode panel
			 */
			this._collapseButton = new MiniButton(f, "collapseButton", "icon-WCR_Widget-Collapse", true, _.bind(function () {
				this._togglePanel();
			}, this));

			/**
			 * @member {JQueryObj} div that has both the minibutton and the containing div-element
			 */
			this._collapseButtonElement = $("<div>").addClass("collapseButton").append(this._collapseButton.element);

			//append the collapse button
			this.element.append(this._collapseButtonElement);

			/**
			 * @member {Boolean} is the panel expanded or collapsed
			 */
			this._expanded = true;

			/**
			 * @member {Boolean} is the panel disabled or not
			 */
			this._disabled = false;

			/**
			 * Invalidate the panel, layout all element in the panel
			 */
			this.invalidate = _.debounce(_.bind(this._invalidate, this), 50);
		};

		/**
		 * Collapse or Expand the mode panel
		 * @param {Boolean} expanded
		 */
		ModePanel.prototype.setExpanded = function (expanded) {
			if (this._expanded !== expanded) {
				this._togglePanel();
			}
		};

		/**
		 * Collapse or uncollapse the panel
		 */
		ModePanel.prototype._togglePanel = function () {
			var wasVisible = this.isVisible();

			this.element.toggleClass("collapsed");
			this._expanded = !this._expanded;

			this._collapseButton.setIcon(this._expanded ? "icon-WCR_Widget-Collapse" : "icon-WCR_Widget-Expand");
			var isVisible = this.isVisible();
			if (wasVisible != isVisible) {
				this._emit("changed.expanded", wasVisible, isVisible);
			}
		};

		/**
		 * Is the mode panel disabled or enabled?
		 * @returns {Boolean}
		 */
		ModePanel.prototype.isEnabled = function () {
			return !this._disabled;
		};


		/**
		 * Is the mode panel visible or not
		 * @returns {Boolean}
		 */
		ModePanel.prototype.isVisible = function () {
			return this._expanded && !this._disabled;
		};

		/**
		 * Get the width of the mode panel
		 * @returns {Number}
		 */
		ModePanel.prototype.getWidth = function () {
			if (this.isVisible()) {
				return this.element.outerWidth(true);
			} else {
				return 0;
			}
		};

		/**
		 * Get the height of the mode panel
		 * @returns {Number}
		 */
		ModePanel.prototype.getHeight = function () {
			return this.element.outerHeight(true);
		};

		/**
		 * Enable/disable the mode panel
		 * @param {Boolean} enabled
		 */
		ModePanel.prototype.setEnabled = function (enabled) {
			if (enabled) {
				this.enable();
			} else {
				this.disable();
			}
		};

		/**
		 * Enable the mode panel (shows the panel)
		 */
		ModePanel.prototype.enable = function () {
			var wasVisible = this.isVisible();
			this.element.show();


			this._disabled = false;

			var isVisible = this.isVisible();
			if (wasVisible != isVisible) {
				this._emit("changed.expanded", wasVisible, isVisible);
			}
		};

		/**
		 * Disable the mode panel, hides it completely (including the hide/show button)
		 */
		ModePanel.prototype.disable = function () {
			var wasVisible = this.isVisible();
			this.element.hide();


			this._disabled = true;

			var isVisible = this.isVisible();
			if (wasVisible != isVisible) {
				this._emit("changed.expanded", wasVisible, isVisible);
			}
		};

		/**
		 * Invalidate the layout of the modepanel, resize all widgets
		 */
		ModePanel.prototype._invalidate = function () {
			if (this.isEnabled()) {
				LayoutUtils.layoutWidgets(this._content, 0);
			}
		};

		/**
		 * Append an element to the panel
		 * @param {JQueryObj} element - the object to add to the panel
		 */
		ModePanel.prototype.append = function (element) {
			this._content.append(element);
		};

		return ModePanel;
	});

define('web-viewer/layout/mobile/MobileReaderSpreadMode',["web-viewer/layout/LayoutConstants",
		"web-viewer/layout/LayoutMode"
	],
	function (
		LayoutConstants,
		LayoutMode
	) {
		"use strict";

		/**
		 * @exports MobileReaderSpreadMode
		 * @memberof Layout
		 * @class
		 * ReaderSpread mode of the mobile layout
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Layout} layout
		 */
		var MobileReaderSpreadMode = function (f, layout) {
			LayoutMode.call(this, f, layout, LayoutConstants.LayoutMode.MOBILE_READERSPREAD);

			// show/hide based on whether or not active page is multipage
			this.f.getDocumentController().whenActivePageChanged(function (oldPage, page) {
				this.f.emit("layout.modeAvailabilityChanged", this, page.getDocumentVersion().isMultiPage(), {
					icon: "icon-WCR_ReaderSpread",
					title: "ReaderSpreadModeTitle"
				});
			}, this);
		};

		MobileReaderSpreadMode.prototype = new LayoutMode();
		MobileReaderSpreadMode.prototype.constructor = MobileReaderSpreadMode;

		/**
		 * Turns On/Off the layoutMode
		 * @param {Boolean} toggle
		 */
		MobileReaderSpreadMode.prototype.toggle = function (toggle) {
			if (toggle === undefined) {
				toggle = !this._isActive;
			}
			this._isActive = toggle;
			this.f.requestModule("pagelist", _.bind(function (pageListModule) {
				pageListModule.toggleReaderspread(this._isActive);
			}, this));
		};

		/**
		 * Checks if the item is allowed for this layout mode. Prevents channels and CAD widget from loading.
		 *
		 * @param {String} itemname
		 * @returns {Boolean}
		 */
		MobileReaderSpreadMode.prototype.isItemAllowed = function (itemname) {
			// only allow viewport widget and page selection toolbar
			return itemname === LayoutConstants.Widget.VIEWPORT || itemname === LayoutConstants.Widget.PAGELIST || itemname === LayoutConstants.Widget.MESSAGE || itemname === LayoutConstants.ToolbarComponent.PAGESELECTION || itemname === LayoutConstants.ToolbarComponent.APPROVAL;
		};

		/**
		 * Checks if the state of the layout should be persisted.
		 * @return {Boolean}
		 */
		MobileReaderSpreadMode.prototype.isPersisted = function () {
			return true;
		};

		/**
		 * Decide whether or not to show the mode panel
		 */
		MobileReaderSpreadMode.prototype.isModePanelEnabled = function () {
			return true;
		};

		/**
		 * @override
		 */
		MobileReaderSpreadMode.prototype.getModePanelContent = function () {
			return [LayoutConstants.Widget.PAGELIST];
		};

		return MobileReaderSpreadMode;
	});

define('web-viewer/layout/mobile/MobileLayout',["web-viewer/layout/Layout",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/layout/mobile/ModeBar",
		"web-viewer/layout/mobile/NavigationBar",
		"web-viewer/core/PrivateChannel",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/layout/mobile/MobileViewMode",
		"web-viewer/layout/mobile/AnnotationViewMode",
		"web-viewer/layout/mobile/MobileCompareMode",
		"web-viewer/layout/mobile/Mobile3DViewMode",
		"web-viewer/layout/WidgetWrapper",
		"web-viewer/layout/mobile/ModePanel",
		"utils/ShapeUtils",
		"web-viewer/layout/mobile/MobileReaderSpreadMode",
		"underscore",
		"jquery"
	],
	function (
		Layout,
		LayoutConstants,
		ModeBar,
		NavigationBar,
		PrivateChannel,
		ListenerUtil,
		MobileViewMode,
		AnnotationViewMode,
		MobileCompareMode,
		Mobile3DViewMode,
		WidgetWrapper,
		ModePanel,
		ShapeUtils,
		MobileReaderSpreadMode,
		_,
		$
	) {
		"use strict";


		/**
		 * @class
		 * Layout for mobile screens. This is the view initialized by the LayoutManager that will add all the layoutcomponents
		 * to the viewer. It can generate a an object representing the current state of the viewer and can be initialized by
		 * a state-String.
		 *
		 * NOTE widget that are both MainPanelContent and ModelPanelContent are not supported at the moment!
		 *
		 * @exports MobileLayout
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {LayoutModel} layoutModel
		 * @param {JQueryObj} context - context where the layout will add its components to
		 * @param {Object} layoutState
		 * @param {LoadingIndicator} loadingIndicator - the loading indicator, which was created before all requests started
         * @param {JQueryObj} initialLoader - the JQuery reference to the element indicating the initial loader (needed to safely remove it without emptying the context)                                           
		 */
		var MobileLayout = function (f, layoutModel, context, layoutState, loadingIndicator, initialLoader) {
			Layout.call(this, f, layoutModel, context, layoutState, loadingIndicator, initialLoader);
			PrivateChannel.installTo(this, f, "layout.mobile");
			ListenerUtil.mixin(this, f);


			if (!this._layoutState){
				this._layoutState = this._getDefaultLayout();
			}

			/**
			 * All the layout items mapped as Name -> WidgetWrapper
			 */
			this._items = {};


			/**
			 * The component that is used to display the content in the viewport: graphics, CAD, annotations,...
			 */
			this._mainContent = null;

			/**
			 * In this array all items that are for the main content are kept
			 */
			this._mainContentWidgetWrappers = [];

			this._centerContent = $("<div>").addClass("centerContent");

			this._modeBar = new ModeBar(this.f, $("<div>"), this);
			this._modeBar.setState(this._layoutState.modeBar);

			this._appContentWrapper = $("<div>").addClass("appContentWrapper");

			this._messagebar = $("<div />").addClass("messagebar");

			this._navigationBar = new NavigationBar(this.f, $("<div>"), this);

			this._modePanel = new ModePanel(f, "modePanel");

			// stop the bouncing effect in iOS safari (we can only stop propagation if we are not in an interactive element (or button or mode)
			$(context).bind("touchmove", _.bind(function (e) {
				if (e.target === this._modeBar.element[0] ||
					(this._modeBar.element.find(e.target).length > 0 && $(e.target).parentsUntil(this._modeBar.element, ".btn,.interactive").length === 0) ||
					e.target === this._navigationBar.element[0] ||
					(this._navigationBar.element.find(e.target).length > 0 && $(e.target).parentsUntil(this._navigationBar.element, ".btn,.interactive,.mode").length === 0)) {

					e.preventDefault();
					e.stopPropagation();

				}
			}, this));

			this._appContentWrapper.append(this._navigationBar.element);
			this._centerContent.append(this._modePanel.element);

			this._appContentWrapper.append(this._centerContent);
			this._appContentWrapper.append(this._messagebar);

			// by default load the view mode
			this._activeMode = new MobileViewMode(this.f, this);
			this._activeMode.toggle(true);

			// register all modes
			this._layoutModel.registerLayoutMode(this._activeMode);
			this._layoutModel.registerLayoutMode(new AnnotationViewMode(this.f, this));
			this._layoutModel.registerLayoutMode(new MobileReaderSpreadMode(this.f, this));
			this._layoutModel.registerLayoutMode(new MobileCompareMode(this.f, this));
			this._layoutModel.registerLayoutMode(new Mobile3DViewMode(this.f, this));

			//a debounced invalidate, to make sure not to many invalidates are triggered
			this._debouncedInvalidate = _.debounce(_.bind(this._invalidateLayout, this), 200);
			this.f.emit("layout.loadCSS", "web-viewer/layout/mobile/css/mobile.css", _.bind(function () {
				this._cssLoaded = true;
				this._initialLoader.remove();
				delete this._initialLoader;
				this._context.append(this._modeBar.element).append(this._appContentWrapper).addClass("disableTextSelection").append(this._loadingIndicator.element);
				this._debouncedInvalidate();
			}, this));

			//listen to resize events of the window
			$(window).resize(_.bind(function (e) {
				// http://bugs.jqueryui.com/ticket/7514: resize of popups bubbles to window
				if (e.target === window) {
					this._debouncedInvalidate();
				}
			}, this));


			//listen to widget events, to act on resized/collapsed/...
			this._registerOnLayoutEvents();

			// listen to the modebar
			this._registerListener("visible", this._debouncedInvalidate, this, this._modeBar);

			//listen to changes in the visibility of the mode panel
			this._registerListener("changed.visibility", this._invalidateLayout, this, this._modePanel);

			this._invalidateLayout(); //this one is needed to provide the correct world coordinates		

			this._updateModePanel();

			// invalidate layout when modebar or modepanel transition is finished
			this._modeBar.element.on("webkitAnimationEnd oanimationend msAnimationEnd animationend transitionend ", this._debouncedInvalidate);
			this._modePanel.element.on("webkitAnimationEnd oanimationend msAnimationEnd animationend transitionend ", this._debouncedInvalidate);

			this._registerListener("changed.expanded", this._modePanelChanged, this, this._modePanel);
		};

		MobileLayout.prototype = Object.create(Layout.prototype);
		MobileLayout.prototype.constructor = MobileLayout;

		/**
		 * Register listeners for general layout event.
		 */
		MobileLayout.prototype._registerOnLayoutEvents = function () {
			this.f.on("layout.disableTextSelection", _.bind(function () {
				this._disableTextSelection();
			}, this));

			this.f.on("layout.enableTextSelection", _.bind(function () {
				this._enableTextSelection();
			}, this));

			this.f.on("layout.resetToDefault", _.bind(this.resetToDefault, this));

			this.f.on("layout.switchLayoutMode", _.bind(function () {
				this._updateModePanel();
				this._updateMainContent();
			}, this));
		};


		/**
		 * Add an item to the layout. This is called by Layout to add an item.
		 * @param {Object} layoutItem
		 * @example
		 * 	{ 	type: {LayoutConstants.LayoutType},
		 * 		name: 'itemName',
		 * 		item: {AbstractWidget/ToolbarComponent}
		 * 	}
		 */
		MobileLayout.prototype._addItem = function (layoutItem) {
			// the mobile layout only supports the ViewportWidget for now...
			if (layoutItem.type === LayoutConstants.LayoutItemType.WIDGET) {
				var widget = layoutItem.getItem(this.getLayoutType());
				var widgetType = widget.getWidgetType();

				if (widgetType == "messagebar") {
					widget.setContext(this._messagebar);
				} else {
					var widgetWrapper = new WidgetWrapper(widget, layoutItem.name, false);
					this._items[layoutItem.name] = widgetWrapper;

					this._modePanel.element.find(".widgetWrapper:visible").removeClass("first").first().addClass("first");
					this._modePanel.invalidate();

					this._registerListener("enabled", function (isEnabled) {
						this._toggleWidget(layoutItem.name, widgetWrapper, isEnabled);
					}, this, widget);

					this._registerListener("resized", this._invalidateModePanel, this, widget);

					if (widgetType == "maincontent" || layoutItem.name === this._activeMode.getMainPanelContent()) {
						// show as main content
						widgetWrapper.setHeaderEnabled(false);
						this._centerContent.append(widgetWrapper.element);
						this._mainContentWidgetWrappers.push(widgetWrapper);
						this._updateMainContent();
					} else {
						var modePanelContent = this._activeMode.getModePanelContent();
						var nextItem;
						var index = _.indexOf(modePanelContent, layoutItem.name);
						while (index < modePanelContent.length && !nextItem) {
							nextItem = this._items[modePanelContent[++index]];
						}

						if (nextItem) {
							nextItem.element.before(widgetWrapper.element);
						} else {
							this._modePanel.append(widgetWrapper.element);
						}
					}

				}
			} else if (layoutItem.type === LayoutConstants.LayoutItemType.TOOLBARCOMPONENT) {
				var toolbar = layoutItem.getItem(this.getLayoutType());
				this._items[layoutItem.name] = layoutItem;
				switch (toolbar.name) {
				case LayoutConstants.ToolbarComponent.APPROVAL:
				case LayoutConstants.ToolbarComponent.ANNOTATION:
					this._navigationBar.prependToolbarRight(toolbar);
					break;
				default:
					this._navigationBar.appendToolbarLeft(toolbar);
				}
			}
		};


		/**
		 * Invalidate the ModePanel (update heights of the widgets)
		 */
		MobileLayout.prototype._invalidateModePanel = function () {
			this._modePanel.invalidate();
		};

		/**
		 * Update visibility of the added items based on whether or not they are allowed in the current mode.
		 * E.g. item was added in previous mode, but should not be shown in current mode
		 */
		MobileLayout.prototype._updateItemVisibility = function () {
			_.each(this._items, _.bind(function (layoutItem) {
				if (layoutItem.type === LayoutConstants.LayoutItemType.TOOLBARCOMPONENT) {
					var toolbar = layoutItem.getItem(this.getLayoutType());
					this._navigationBar.toggleComponent(toolbar.name, this._activeMode.isItemAllowed(layoutItem.name));
				} else if (layoutItem.type === LayoutConstants.LayoutItemType.WIDGET) {
					var widgetWrapper = layoutItem.getItem(this.getLayoutType());

					widgetWrapper.setVisible(this._activeMode.isItemAllowed(layoutItem.name) && widgetWrapper.widget.isVisible());
				}
			}, this));
		};

		/**
		 * Disable text selection in the entire viewer.
		 */
		MobileLayout.prototype._disableTextSelection = function () {
			this._context.find(".enableTextSelection").toggleClass("enableTextSelection", false).toggleClass("enableTextSelectionTempDisabled", true);
		};

		/**
		 * Enable text selection in the entire viewer.
		 */
		MobileLayout.prototype._enableTextSelection = function () {
			this._context.find(".enableTextSelectionTempDisabled").toggleClass("enableTextSelection", true).toggleClass("enableTextSelectionTempDisabled", false);
		};


		/**
		 * Restore a certain state of the layout to the state provided. (not used at this moment)
		 * @param {Object} state
		 */
		MobileLayout.prototype.restore = function (state) {
			this._modeBar.setState(state);
		};

		/**
		 * Reset the sate of the layout to the default state.
		 */
		MobileLayout.prototype.resetToDefault = function () {
			this._layoutState = this._getDefaultLayout(); //reset the state of the layout, it now is the default
			this._modeBar.setState(this._layoutState.modeBar);
			this._invalidateLayout();
		};

		/**
		 * Get default layout state
		 * @returns {Object}
		 */
		MobileLayout.prototype._getDefaultLayout = function () {
			return {
				modeBar: {
					visible: true
				}
			};
		};

		/**
		 * Generates the state of the layout as a string. It takes the not-visible elements into account so state of those
		 * elements is not lost.
		 * @returns {Object}
		 */
		MobileLayout.prototype.getLayoutState = function () {
			return {
				modeBar: this._modeBar.getState()
			};
		};

		/**
		 * Toggle the mode bar
		 */
		MobileLayout.prototype.toggleModeBar = function (toggle) {
			this._modeBar.toggle();
		};

		/**
		 * Invalidates the layout. Makes sure all panels have the correct size, resize is triggered on the correct elements,...
		 */
		MobileLayout.prototype._invalidateLayout = function () {
			if (!this._cssLoaded) {
				return;
			}

			this._context.toggleClass("hideModeBar", !this._modeBar.isVisible());



			this._modePanel.element.css("height", this._context.innerHeight() - this._navigationBar.element.outerHeight(true));

			var modelPanelWidth = this._modePanel.getWidth();

			if (this._activeMainPanelContent) {
				this._activeMainPanelContent.content.css({
					"height": this._context.innerHeight() - this._navigationBar.element.outerHeight(true),
					"width": this._context.innerWidth() - this._modeBar.getWidth() - modelPanelWidth
				});

				if (this._activeMainPanelContent.widget !== null) {
					this._activeMainPanelContent.widget.resized();
				}
			}

			// Layout loading indicator
			var navigationBarHeight = this._navigationBar.element.outerHeight(true);
			this._loadingIndicator.element.css({
				top: navigationBarHeight + 5,
				left: this._context.innerWidth() - 29
			});

			// Center message bar inside content wrapper
			var contentPosition = this._appContentWrapper.position();
			this._messagebar.css({
				width: this._appContentWrapper.outerWidth() - 20,
				top: contentPosition.top + navigationBarHeight + 10,
				left: contentPosition.left + 10
			});

			this._modePanel.invalidate();

			this._emitLayoutChanged();
		};

		/**
		 * Returns the type of the layout.
		 * @override
		 * @return {Number}
		 */
		MobileLayout.prototype.getLayoutType = function () {
			return LayoutConstants.LayoutType.MOBILE;
		};

		/**
		 * Enable/disable fullscreen
		 * Note: this method should be called from a user generated event... (or it won't work...)
		 * @param enabled
		 */
		MobileLayout.prototype.toggleFullscreen = function (enabled) {
			if (enabled) {
				this._requestFullscreen();
			} else {
				this._exitFullscreen();
			}
		};

		/**
		 * Check if the app is fullscreen
		 * @returns {Boolean}
		 */
		MobileLayout.prototype.isFullscreen = function () {
			return document.fullscreenElement || // alternative standard method
				document.mozFullScreenElement || document.webkitFullscreenElement || document.msFullscreenElement;
		};

		/**
		 * Check if the browser supports the fullescren API
		 * @returns {Boolean}
		 */
		MobileLayout.prototype.supportsFullscreen = function () {
			var viewerDiv = this.f.getContext().get(0);
			return viewerDiv.requestFullscreen || viewerDiv.mozRequestFullScreen || viewerDiv.webkitRequestFullscreen || viewerDiv.msRequestFullscreen;
		};

		/**
		 * Request fullscreen for the whole document.body
		 */
		MobileLayout.prototype._requestFullscreen = function () {
			var viewerDiv = this.f.getContext().get(0);
			if (viewerDiv.requestFullscreen) {
				viewerDiv.requestFullscreen();
			} else if (viewerDiv.mozRequestFullScreen) {
				viewerDiv.mozRequestFullScreen();
			} else if (viewerDiv.webkitRequestFullscreen) {
				viewerDiv.webkitRequestFullscreen();
			} else if (viewerDiv.msRequestFullscreen) {
				viewerDiv.msRequestFullscreen();
			}
		};

		/**
		 * Exit fullscreen mode
		 */
		MobileLayout.prototype._exitFullscreen = function () {
			if (document.exitFullscreen) {
				document.exitFullscreen();
			} else if (document.msExitFullscreen) {
				document.msExitFullscreen();
			} else if (document.mozCancelFullScreen) {
				document.mozCancelFullScreen();
			} else if (document.webkitExitFullscreen) {
				document.webkitExitFullscreen();
			}
		};

		/**
		 * Show or hide mode panel, depending on the active mode
		 */
		MobileLayout.prototype._updateModePanel = function () {
			this._modePanel.setEnabled(this._activeMode.isModePanelEnabled());

			// update the modepanel content
			// hide existing items
			this._modePanel.element.find(".widgetWrapper").each(function (index, element) {
				element.wrapper.setVisible(false);
			});

			var modePanelContent = this._activeMode.getModePanelContent();
			var first = true;
			_.each(modePanelContent, function (itemName) {
				if (this._items[itemName]) {
					var item = this._items[itemName];
					this._modePanel.append(item.element);
					var visible = item.widget.isVisible() && item.widget.isEnabled();
					item.setVisible(visible);
					if (visible) {
						item.element.toggleClass("first", first);
						first = false;
					}
				}
			}, this);
			this._modePanel.element.toggleClass("hideWidgetTitles", !this._activeMode.showModePanelWidgetTitles());
			this._modePanel.invalidate();
			this._modePanel.setExpanded(this._activeMode.isModePanelExpanded());
		};

		/**
		 * Update the contents of the main panel
		 * A mode can decide to show something else than the viewport widget
		 */
		MobileLayout.prototype._updateMainContent = function () {
			var mainPanelContent = this._activeMode.getMainPanelContent();
			this._activeMainPanelContent = null;

			// run through all possible main content widget wrappers
			_.each(this._mainContentWidgetWrappers, function (wrapper) {
				if (wrapper.widget.getID() === mainPanelContent) {
					// this wrapper should be visible
					wrapper.setVisible(true);

					this._activeMainPanelContent = wrapper;
				} else {
					// hide, the wrapper should not be active
					wrapper.setVisible(false);
				}
			}, this);

			if (!this._activeMainPanelContent && this._items[mainPanelContent]) {
				// we didn't find the widget in the _mainContentWidgetWrappers, but it might be initialized anyway
				// append it to the center content

				this._mainContentWidgetWrappers.push(this._items[mainPanelContent]);
				this._centerContent.append(this._items[mainPanelContent].element);
				this._updateMainContent();
				return;
			}

			this._debouncedInvalidate();
		};


		/**
		 * React on collapse/expand operation of the modepanel
		 * @param {Boolean} oldValue
		 * @param {Boolean} newValue
		 */
		MobileLayout.prototype._modePanelChanged = function (oldValue, newValue) {
			this._activeMode.setIsmodePanelExpanded(newValue);
			this._debouncedInvalidate();
		};


		/**
		 * Enable/disable a widget
		 * @param {String}        widgetName    - name of the widget
		 * @param {WidgetWrapper} widgetWrapper - widgetWrapper in which the widget is displayed
		 * @param {Boolean}       isEnabled     - should the widget be enabled
		 */
		MobileLayout.prototype._toggleWidget = function (widgetName, widgetWrapper, isEnabled) {
			if (isEnabled) {
				if (this._activeMode.isItemAllowed(widgetName)) {
					widgetWrapper.setVisible(true);
				}
			} else {
				widgetWrapper.setVisible(false);
			}
		};

		/**
		 * Returns the default layout mode for the layout.
		 * @return {LayoutMode}
		 */
		MobileLayout.prototype._getDefaultLayoutMode = function () {
			return this._layoutModel.getLayoutMode(LayoutConstants.LayoutMode.MOBILE_VIEW);
		};
		return MobileLayout;
	});

