define('web-viewer/measure/Ruler',["web-viewer/utils/LineworkWrapper",
		"web-viewer/utils/LineworkPoint",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/utils/DrawUtils",
		"web-viewer/measure/MeasureConstants",
		"underscore"
	],
	function (LineworkWrapper,
		LineworkPoint,
		ShapeUtils,
		ViewerUtils,
		DrawUtils,
		MeasureConstants,
		_) {
		/**
		 * @exports Ruler
		 * @memberof MeasureModule
		 * @constructor
		 */
		var Ruler = function (f) {
			this.f = f;

			this.rulerType = 1; // 0 - line, 1 - square

			this.startPoint = null;
			this.endPoint = null;
		};

		/**
		 * Reset ruler; remove both end points
		 */
		Ruler.prototype.reset = function () {
			this.startPoint = null;
			this.endPoint = null;
		};

		/**
		 * Snap the ruler to the closest angle
		 * @param snapStart - controls whether you want to snap the startpoint or the end point
		 */
		Ruler.prototype.snapToClosestAngle = function (snapStart) {
			if (this.rulerType === 0) {
				var radius = this.getLength();
				var angle = ViewerUtils.closestAngle(this.getAngle(), 8);
				if (snapStart) {
					this.startPoint.x = this.endPoint.x - radius * Math.cos(angle * Math.PI / 180);
					this.startPoint.y = this.endPoint.y - radius * Math.sin(angle * Math.PI / 180);
				} else {
					this.endPoint.x = radius * Math.cos(angle * Math.PI / 180) + this.startPoint.x;
					this.endPoint.y = radius * Math.sin(angle * Math.PI / 180) + this.startPoint.y;
				}

			}
		};

		/**
		 * Get length of ruler in mm
		 * @returns {Number}
		 */
		Ruler.prototype.getLength = function () {
			if (this.endPoint && this.startPoint) {
				var w = Math.abs(this.endPoint.x - this.startPoint.x);
				var h = Math.abs(this.endPoint.y - this.startPoint.y);
				return Math.sqrt(w * w + h * h);
			} else {
				return 0;
			}
		};

		/**
		 * Get the angle of the ruler in degrees
		 * @returns {Number}
		 */
		Ruler.prototype.getAngle = function () {
			return Math.atan2(this.endPoint.y - this.startPoint.y, this.endPoint.x - this.startPoint.x) * 180 / Math.PI;
		};

		/**
		 * Get the linework of the ruler tool
		 * @returns
		 */
		Ruler.prototype.getLinework = function () {
			if (this.endPoint && this.startPoint) {
				if (this.rulerType === 0) {
					return this.getRulerLinework();
				} else {
					return this.getSquareLinework();
				}
			} else {
				return [];
			}
		};

		/**
		 * Get square linework for popover
		 * @returns {Array.<LineworkWrapper>}
		 */
		Ruler.prototype.getSquareLinework = function () {
			var linework = [];

			if (this.startPoint && this.endPoint) {
				var ref = new LineworkWrapper();
				ref.fillcolor = 'rgba(0, 0, 0, 0.30)';
				ref.color = 'rgba(0, 0, 0, 1.00)';
				ref.linewidth = 1;
				ref.type = DrawUtils.LWTYPE_RECT;

				var startpoint = new LineworkPoint();
				startpoint.makeMoveTo(this.startPoint.x, this.startPoint.y);
				ref.pointArray[0] = startpoint;

				var endpoint = new LineworkPoint();
				endpoint.makeLineTo(this.endPoint.x, this.endPoint.y);
				ref.pointArray[1] = endpoint;

				ref.rect = new ShapeUtils.Rect(this.startPoint, this.startPoint.sizeTo(this.endPoint));
				linework.push(ref);

				// dash rectangle
				ref = new LineworkWrapper();
				ref.fillcolor = 'rgba(0, 0, 0, 0.0)';
				ref.color = 'rgba(220, 220, 220, 0.9)';
				ref.linewidth = 1;
				ref.type = DrawUtils.LWTYPE_DASHEDBOX;
				ref.dash = [4, 3];

				var startpoint = new LineworkPoint();
				startpoint.makeMoveTo(this.startPoint.x, this.startPoint.y);
				ref.pointArray[0] = startpoint;

				var endpoint = new LineworkPoint();
				endpoint.makeLineTo(this.endPoint.x, this.endPoint.y);
				ref.pointArray[1] = endpoint;

				ref.rect = new ShapeUtils.Rect(this.startPoint, this.startPoint.sizeTo(this.endPoint));

				linework.push(ref);
			}

			return linework;
		};

		/**
		 * Get the linework for drawing the ruler
		 *  (use cached version if possible)
		 * @returns {Array.<LineworkWrapper>}
		 */
		Ruler.prototype.getRulerLinework = function () {
			var color = 'black';
			if (this.f) {
				color = this.f.getCurrentUser().getColor("PREFERENCES_RULER_DISPLAY_COLOR");
			}


			if (this.startPoint && this.endPoint) {
				if (this._cachedRulerLinework &&
					this._cachedRulerLinework.startPoint.equals(this.startPoint) &&
					this._cachedRulerLinework.endPoint.equals(this.endPoint) &&
					this._cachedRulerLinework.color == color) {
					// return cached version
					return this._cachedRulerLinework.linework;
				} else {
					return this.generateRulerLinework(this.startPoint, this.endPoint, color);
				}
			}
		};

		/**
		 * Generate the linework for drawing the ruler
		 * @param origStartPoint {ShapeUtils.Point} - start point
		 * @param origEndPoint {ShapeUtils.Point} - end point
		 * @param color {String} - color of dashes
		 * @returns {Array.<LineworkWrapper>}
		 */
		Ruler.prototype.generateRulerLinework = function (origStartPoint, origEndPoint, color) {
			var linework = [];

			var startpoint = new LineworkPoint();
			startpoint.makeMoveTo(origStartPoint.x, origStartPoint.y);

			var endpoint = new LineworkPoint();
			endpoint.makeLineTo(origEndPoint.x, origEndPoint.y);

			// to get the double colored dashed line, we draw a colored dashed line on top of a white full line
			var whiteLine = new LineworkWrapper();
			whiteLine.pointArray.push(startpoint);
			whiteLine.pointArray.push(endpoint);
			whiteLine.color = "white";
			whiteLine.type = DrawUtils.LWTYPE_LINE;
			linework.push(whiteLine);

			var colorLine = new LineworkWrapper();
			colorLine.pointArray.push(startpoint);
			colorLine.pointArray.push(endpoint);
			colorLine.color = color;
			colorLine.dash = [10, 10];
			colorLine.type = DrawUtils.LWTYPE_LINE;
			linework.push(colorLine);

			// generate anchors
			var startPoint = this.getCursor(origStartPoint);
			var endPoint = this.getCursor(origEndPoint);
			linework.push(startPoint);
			linework.push(endPoint);

			// store in cache
			this._cachedRulerLinework = {
				startPoint: origStartPoint.copy(),
				endPoint: origEndPoint.copy(),
				color: color,
				linework: linework
			};

			return linework;
		};

		/**
		 * Generate the crosshair to put at the end points of the ruler
		 * @param point {ShapeUtils.Point} - point on which cursor should be applied
		 * @returns {LineworkWrapper}
		 */
		Ruler.prototype.getCursor = function (point) {
			var cursor = new LineworkWrapper();
			cursor.type = DrawUtils.LWTYPE_IMAGE;
			cursor.linewidth = 1;
			cursor.imageRef = MeasureConstants.RulerPoint['DEFAULT'];
			cursor.staticsize = true;
			cursor.setRect(new ShapeUtils.Rect(new ShapeUtils.Point(point.x, point.y)));
			cursor.imageOffset = new ShapeUtils.Size(MeasureConstants.RulerPoint.OFFSET.width, MeasureConstants.RulerPoint.OFFSET.height);
			return cursor;
		};

		return Ruler;
	});

define('web-viewer/utils/SnapIndicator',["web-viewer/utils/LineworkWrapper",
		"web-viewer/utils/DrawUtils",
		"utils/ShapeUtils"
	],
	function (LineworkWrapper, DrawUtils, ShapeUtils) {
		"use strict";

		var IMG_SRC = "/wcr-static/web-viewer/layout/img/crosshair.png";
		var IMG_OFFSET = new ShapeUtils.Size(7, 7);

		/**
		 * @exports SnapIndicator
		 * @memberof MeasureModule
		 * @class SnapIndicator is a LineworkWrapper object that is used to display a hint to snap to
		 * @augments Utils.LineworkWrapper
		 * @constructor
		 * @param {String} name - Name of the oneup
		 * @param {ShapeUtils.Point} [center] - center for where the indication should be
		 */


		var SnapIndicator = function (center) {
			LineworkWrapper.call(this);

			this.type = DrawUtils.LWTYPE_IMAGE;
			this.linewidth = 0;
			this.imageRef = IMG_SRC;
			this.staticsize = true;
			this.imageOffset = IMG_OFFSET;

			if (center) {
				this.setCenter(center);
			}



		};

		// inherit
		SnapIndicator.prototype = Object.create(LineworkWrapper.prototype);
		// correct the constructor pointer 
		SnapIndicator.prototype.constructor = SnapIndicator;

		/**
		 * Set the center point of the snap indicator
		 * @param {ShapeUtils.Point} center - center of the indicator
		 */
		SnapIndicator.prototype.setCenter = function (center) {
			this.setRect(new ShapeUtils.Rect(new ShapeUtils.Point(center.x, center.y)));
		};


		return SnapIndicator;

	});


define('tpl!web-viewer/measure/RulerPopoverTemplate', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<table>\r\n';
 if (d.showXY) { 
__p+='\r\n	<tr>\r\n		<td><strong>'+
((__t=(d.i18n('InfoX')))==null?'':_.escape(__t))+
'</strong></td>\r\n		<td align="right">'+
((__t=(d.x))==null?'':_.escape(__t))+
' '+
((__t=(d.i18n(d.unitsloc)))==null?'':_.escape(__t))+
'</td>\r\n	</tr>\r\n	<tr>\r\n		<td><strong>'+
((__t=(d.i18n('InfoY')))==null?'':_.escape(__t))+
'</strong></td>\r\n		<td align="right">'+
((__t=(d.y))==null?'':_.escape(__t))+
' '+
((__t=(d.i18n(d.unitsloc)))==null?'':_.escape(__t))+
'</td>\r\n	</tr>\r\n';
 } 
__p+='\r\n';
 if (d.showDA) { 
__p+='\r\n	<tr>\r\n		<td><strong>'+
((__t=(d.i18n('InfoDist')))==null?'':_.escape(__t))+
'</strong></td>\r\n		<td align="right">'+
((__t=(d.distance))==null?'':_.escape(__t))+
' '+
((__t=(d.i18n(d.unitsloc)))==null?'':_.escape(__t))+
'</td>\r\n	</tr>\r\n	<tr>\r\n		<td><strong>'+
((__t=(d.i18n("InfoAngle")))==null?'':_.escape(__t))+
'</strong></td>\r\n		<td align="right">'+
((__t=(d.angle))==null?'':_.escape(__t))+
'&#176;</td>\r\n	</tr>\r\n';
 } 
__p+='\r\n';
 if (d.showWH) { 
__p+='\r\n	<tr>\r\n		<td><strong>'+
((__t=(d.i18n('InfoWidth')))==null?'':_.escape(__t))+
'</strong></td>\r\n		<td align="right">'+
((__t=(d.width))==null?'':_.escape(__t))+
' '+
((__t=(d.i18n(d.unitsloc)))==null?'':_.escape(__t))+
'</td>\r\n	</tr>\r\n	<tr>\r\n		<td><strong>'+
((__t=(d.i18n('InfoHeight')))==null?'':_.escape(__t))+
'</strong></td>\r\n		<td align="right">'+
((__t=(d.height))==null?'':_.escape(__t))+
' '+
((__t=(d.i18n(d.unitsloc)))==null?'':_.escape(__t))+
'</td>\r\n	</tr>\r\n';
 } 
__p+='\r\n\r\n</table>\r\n';
 if(d.createAnnotation) { 
__p+='\r\n    <a class="addAnnotation">\r\n        <i class="icon-WCR_Annotation-Arrow12"></i>\r\n        <span>'+
((__t=(d.i18n("AnnFromReadout")))==null?'':_.escape(__t))+
'</span>\r\n    </a>\r\n';
 } 
__p+='';
return __p;
}});
define('web-viewer/measure/RulerSnapIndicatorPopoverModel',["underscore",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/viewport/model/ViewportPopoverModel",
		"utils/ShapeUtils",
		"Template",
		"tpl!web-viewer/measure/RulerPopoverTemplate",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		ViewerUtils,
		ViewportPopoverModel,
		ShapeUtils,
		Template,
		tpl,
		CoreConstants) {
		"use strict";
		/**
		 * @exports RulerSnapIndicatorPopoverModel
		 * @memberof MeasureModule
		 * @class Model for the popover rulersnapindicator
		 * @constructor
		 */
		var RulerSnapIndicatorPopoverModel = function (f, name, snapIndicator, cssClass) {
			ViewportPopoverModel.call(this, f, name);

			var div = $("<div/>");

			this.fix = 3;
			this.f = f;
			this.snapIndicator = snapIndicator;

			// initialize template
			this.t = new Template(f.i18n, tpl);

			// load language if needed
			f.i18n.loadBundle("viewer", _.bind(function () {
				this.t.render();
			}, this));

			this.t.setContext(div);
			this.t.render();

			this.getFrame = function () {
				if (this.snapIndicator) {
					return this.snapIndicator.rect;
				} else {
					return new ShapeUtils.Rect();
				}
			};

			/**
			 * @override
			 */
			this.getContent = function () {
				return div;
			};

			/**
			 * @override
			 */
			this.getRestrictBorder = function () {
				return 0;
			};

			/**
			 * Get the class for the popover
			 */
			this.getClass = function () {
				return cssClass;
			};

			/**
			 * Get the context for the popover
			 */
			this.getContext = _.bind(function () {
				return this.f.getContext().find(".viewport");
			}, this);
		};

		// inherit
		RulerSnapIndicatorPopoverModel.prototype = Object.create(ViewportPopoverModel.prototype);
		// correct the constructor pointer 
		RulerSnapIndicatorPopoverModel.prototype.constructor = RulerSnapIndicatorPopoverModel;

		RulerSnapIndicatorPopoverModel.prototype.setPoint = function (point) {
			this.point = point;
			this.render();
		};

		RulerSnapIndicatorPopoverModel.prototype.render = function () {
			if (!this.snapIndicator) {
				return;
			}

			var center = this.snapIndicator.rect.center();
			var resx = null,
				resy = null;
			if (this.f.getUnits() === CoreConstants.SizeUnits.PIXELS) {
				var page = this.f.getActivePage();
				resx = page.getParam("resolutionx");
				resy = page.getParam("resolutiony");
			}

			/**
			 * Show values
			 */
			this.t.d.showXY = true;
			this.t.d.unitsloc = this.f.getUnitsLocName();

			// convert to correct coordinate system
			var centerPoint = new ShapeUtils.Point(center.x, center.y);
			var transformedCenterPoint = this._coordinates.getTransformedPoint(centerPoint);

			this.t.d.x = ViewerUtils.convertFromMmValue(transformedCenterPoint.x, this.f.getUnits(), this.f.getUnitsFix(), ViewerUtils.parseFloat(resx));
			this.t.d.y = ViewerUtils.convertFromMmValue(transformedCenterPoint.y, this.f.getUnits(), this.f.getUnitsFix(), ViewerUtils.parseFloat(resy));

			this.t.render();
		};

		return RulerSnapIndicatorPopoverModel;
	});

define('web-viewer/measure/MeasureLayerModel',["underscore",
		"jquery",
		"web-viewer/viewport/model/ViewportLayerModel",
		"web-viewer/measure/Ruler",
		"utils/XMLUtils",
		"utils/ShapeUtils",
		"web-viewer/measure/MeasureConstants",
		"web-viewer/utils/SnapIndicator",
		"web-viewer/measure/RulerSnapIndicatorPopoverModel"
	],
	function (_,
		$,
		LayerModel,
		Ruler,
		XMLUtils,
		ShapeUtils,
		MeasureConstants,
		SnapIndicator,
		RulerSnapIndicatorPopoverModel) {
		"use strict";

		/**
		 * @exports MeasureLayerModel
		 * @memberof MeasureModule
		 * @constructor
		 */
		var MeasureLayerModel = function (f, ID) {
			LayerModel.call(this, f, ID);

			this.f = f;

			this.ruler = new Ruler(f);
			this._drawType = MeasureConstants.MeasureDrawType.RECTANGLE;

			this._allowStartPointSnapping = false;
			this.snapIndicator = null;
			this.snapIndicatorModel = new RulerSnapIndicatorPopoverModel(f, "RulerSnapIndicatorPopover", this.snapIndicator);
		};

		// inherit
		MeasureLayerModel.prototype = Object.create(LayerModel.prototype);
		// correct the constructor pointer 
		MeasureLayerModel.prototype.constructor = MeasureLayerModel;

		/**
		 * @override
		 */
		MeasureLayerModel.prototype.getLinework = function (animated, rotationanimation) {
			var rulerLines = _.clone(this.ruler.getLinework());
			if (this.snapIndicator) {
				rulerLines.push(this.snapIndicator);
			}
			return rulerLines;
		};

		/**
		 * Set the start point of the measure tool
		 * @param {ShapeUtils.Point} startPoint
		 * @param {Boolean} snap - if true, snap to the closest angle
		 */
		MeasureLayerModel.prototype.setStartPoint = function (startPoint, snap) {
			this.snapIndicator = null;

			this.ruler.startPoint = startPoint;
			if (snap && this._allowStartPointSnapping) {
				this.ruler.snapToClosestAngle(true);
			}
			this.repaint();
			this.modelChanged();
		};

		/**
		 * Set the endpoint of the measure tool
		 * @param {ShapeUtils.Point} endPoint - the endpoint of the ruler/measure tool
		 * @param {Boolean} snap - if true, snap to the closest angle
		 */
		MeasureLayerModel.prototype.setEndPoint = function (endPoint, snap) {
			var previousEndPoint = (this.ruler.endPoint ? this.ruler.endPoint.copy() : null);

			this.ruler.endPoint = endPoint;
			if (snap) {
				this.ruler.snapToClosestAngle();
			}
			if (!previousEndPoint || !previousEndPoint.equals(this.ruler.endPoint)) {
				this.repaint();

				if (this.ruler.rulerType == MeasureConstants.MeasureType.Ruler) {
					this.calculate();
				}
			}
			this.modelChanged();
		};

		/**
		 * Get the start point of the measure tool
		 * @returns {ShapeUtils.Point}
		 */
		MeasureLayerModel.prototype.getStartPoint = function () {
			return this.ruler.startPoint;
		};

		/**
		 * Get the end point of the measure tool
		 * @returns {ShapeUtils.Point}
		 */
		MeasureLayerModel.prototype.getEndPoint = function () {
			return this.ruler.endPoint;
		};

		MeasureLayerModel.prototype.repaint = function () {
			this.modelChanged();
		};

		MeasureLayerModel.prototype.reset = function () {
			this.ruler.reset();
			this.modelChanged();
		};

		MeasureLayerModel.prototype.hidePopover = function () {
			this.removePopover(this.name);
		};

		MeasureLayerModel.prototype.calculate = function (options) {
			if (this.ruler.endPoint && this.ruler.startPoint) {
				if (this.ruler.rulerType == MeasureConstants.MeasureType.Ruler) {
					var object = {};

					object.x1 = this.ruler.startPoint.x;
					object.y1 = this.ruler.startPoint.y;
					object.x2 = this.ruler.endPoint.x;
					object.y2 = this.ruler.endPoint.y;

					object.distance = this.ruler.getLength();
					object.angle = this.ruler.getAngle();

					var size = this.ruler.startPoint.sizeTo(this.ruler.endPoint);
					object.width = size.width;
					object.height = size.height;
				}
			}
		};

		MeasureLayerModel.prototype.setRulerType = function (type) {
			this.ruler.rulerType = type;
			this.reset();
		};

		/**
		 * Set the point that should indicate that the user can snap
		 * @param {ShapeUtils.Point} point
		 */
		MeasureLayerModel.prototype.setSnapIndicatorPoint = function (point) {
			var redraw = false; //only redraw if the situation changed
			if (point) {
				if (!this._allowStartPointSnapping && this.ruler.startPoint && point.equals(this.ruler.startPoint)) {
					if (this.snapIndicator) {
						this.snapIndicator = null;
						redraw = true;
					}
				} else {
					if (!this.snapIndicator) {
						this.snapIndicator = new SnapIndicator(point);
						redraw = true;
					} else {
						if (!point.equals(this.snapIndicator.rect.center())) {
							this.snapIndicator.setCenter(point);
							redraw = true;
						}
					}
				}
			} else {
				if (this.snapIndicator) {
					this.snapIndicator = null;
					redraw = true;
				}
			}

			if (redraw) {
				var indicatorIsSameAsEndpoint = false; //check if the indicator is the same as the endpoint
				var indicatorIsSameAsStartpoint = false;
				if (this.ruler && this.snapIndicator) {
					if (this.ruler.endPoint && this.snapIndicator.rect.center().equals(this.ruler.endPoint)) {
						indicatorIsSameAsEndpoint = true;
					}
					if (this.ruler.startPoint && this.snapIndicator.rect.center().equals(this.ruler.startPoint)) {
						indicatorIsSameAsStartpoint = true;
					}
				}

				var showSnapIndicator = (this.snapIndicator !== null && !indicatorIsSameAsEndpoint && !indicatorIsSameAsStartpoint);
				if (showSnapIndicator && !this.getPopoverModel(this.snapIndicatorModel.getID())) {
					this.snapIndicatorModel.snapIndicator = this.snapIndicator;
					this.addPopoverModel(this.snapIndicatorModel);
				} else if (!showSnapIndicator && this.getPopoverModel(this.snapIndicatorModel.getID())) {
					this.removePopoverModel(this.snapIndicatorModel.getID());
				}

				this.snapIndicatorModel.render();
				this.modelChanged();
			}
		};

		MeasureLayerModel.prototype.clearSnapIndicator = function () {
			this.setSnapIndicatorPoint(null);
		};

		/**
		 * Set whether or not start point can be snapped
		 * @param {Boolean} b
		 */
		MeasureLayerModel.prototype.setAllowStartPointSnapping = function (b) {
			this._allowStartPointSnapping = b;
		};

		/**
		 * Set the type of the selection that will be made (Rectangle, Line, Width, Height)
		 * @param {MeasureConstants.MeasureType} drawType
		 */
		MeasureLayerModel.prototype.setDrawType = function (drawType) {
			this._drawType = drawType;
		};

		/**
		 * Override the modelchanged, so a selection changed event is sent out
		 * @param {boolean} deactivate - true when tool using this model gets deactivated
		 * @override
		 */
		MeasureLayerModel.prototype.modelChanged = function (deactivate) {
			var selectionObject = null;

			if (!deactivate) {
				if (this.ruler && this.ruler.startPoint && this.ruler.endPoint) {
					var size = this.ruler.startPoint.sizeTo(this.ruler.endPoint);
					selectionObject = {
						mode: this._drawType,
						w: size.width,
						h: size.height
					};
					if (this._drawType === MeasureConstants.MeasureDrawType.RECTANGLE) {
						selectionObject.x = this.ruler.startPoint.x;
						selectionObject.y = this.ruler.startPoint.y;
					} else if (this._drawType === MeasureConstants.MeasureDrawType.LINE) {
						selectionObject.d = this.ruler.getLength();
						selectionObject.a = this.ruler.getAngle();
					}
				}
			}

			// structure of selectionObject: see {DocumentInfoModel.setSelection}
			this.f.emit("selection.changed", selectionObject);

			// make sure the normal model changed behavior is still executed
			LayerModel.prototype.modelChanged.call(this);
		};

		return MeasureLayerModel;
	});


define('tpl!web-viewer/measure/MeasurePopoverTemplate', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<ul>\r\n	';
 _.each(d.channels, function(channel){ if(channel){ 
__p+='\r\n		<li><span class="colorBlock" style="background-color: #'+
((__t=(channel.getHexRGB()))==null?'':_.escape(__t))+
';"></span><span>'+
((__t=(channel.value))==null?'':_.escape(__t))+
'</span></li>\r\n	';
 }}); 
__p+='\r\n</ul>\r\n';
 if(d.makeTotalCount){ 
__p+='\r\n<ul>\r\n	<li><div style="text-align:right">'+
((__t=(d.totalCount))==null?'':_.escape(__t))+
'</div></li>\r\n</ul>\r\n';
 } 
__p+='\r\n<ul>\r\n	<li class="addAnnotation"><a><i class="icon-WCR_Annotation-Rect12"></i><span>'+
((__t=(d.i18n("AnnFromReadout")))==null?'':_.escape(__t))+
'</span></a></li>\r\n</ul>';
return __p;
}});
define('web-viewer/measure/MeasurePopoverModel',["web-viewer/utils/ViewerUtils",
		"web-viewer/viewport/model/ViewportPopoverModel",
		"tpl!web-viewer/measure/MeasurePopoverTemplate",
		"Template",
		"underscore",
		"jquery"
	],
	function (ViewerUtils,
		ViewportPopoverModel,
		tpl,
		Template,
		_,
		$) {
		"use strict";

		/**
		 * PoveroverModel for the measure tools, defines the layout (uses the the MeasurePopoverTemplate) of the Popover for measure tools
		 * 
		 * @exports MeasurePopoverModel
		 * @memberof MeasureModule
		 * @class MeasurePopoverModel
		 * @augments Viewport.Model.ViewportPopoverModel
		 * @constructor
		 * 
		 * @param {Core.ViewerFacade} f
		 * @param {LineworkWrapper} area
		 * @param {String} name
		 * @param {String | JQuery.Element} title - Title to be shown in the header
		 * @param {Array<Number> | Array<String>} channels
		 * @param {Boolean} [makeTotalCount] Defines if a total of the different channel values needs to be calculated, only possible if the 
		 * 										values of the channels array are numbers
		 * @param {String | JQuery.Element} headerExtra - A jQuery component or string that is displayed in upper-right corner of this popver
		 */
		var MeasurePopoverModel = function (f, area, name, title, channels, makeTotalCount, headerExtra) {
			ViewportPopoverModel.call(this, f, name);

			this._content = $("<div/>").addClass("measureInfo");

			// Header
			this._header = $("<div />").addClass("measureInfoHeader");
			this._headerTitle = $("<div />").addClass("headerTitle").html(title);
			this._headerExtra = $("<div />").addClass("headerExtra").html(headerExtra);
			this._header.append(this._headerTitle).append(this._headerExtra);
			this._content.append(this._header);

			// Body
			this._body = $("<div />");
			this._content.append(this._body);

			// initialize template
			this.t = new Template(f.i18n, tpl);
			this.t.setContext(this._body);
			this.makeTotalCount = makeTotalCount;
			this._channelsInfo = channels;
			this.channels = [];

			this.t.bind("click", ".addAnnotation", _.bind(function (source, event) {
				//Create a settings XML document
				var content = "<p>";
				_.each(this.channels, function (channel, i) {
					content += (i > 0 ? "<br />" : "") + "<b>" + channel.name + "</b>: " + channel.value;
				}, this);
				content += "</p>";

				this.f.emit("module.annotation.createAnnotation", this.area.rect, content);
				this.f.emit("toolManager.deactivateTool");

			}, this));


			// load language if needed
			this.f.i18n.loadBundle("viewer", _.bind(function () {
				this.t.render();
			}, this));


			// Fixed Template vars
			this.t.d.channels = [];
			this.t.d.makeTotalCount = makeTotalCount;

			this.area = area;

			this.update();

			this.getFrame = function () {
				if (this.area) {
					return this.area.rect;
				} else {
					return null;
				}
			};

			this.getContent = function () {
				return this._content;
			};

			this.getRestrictBorder = function () {
				return 0;
			};

			/**
			 * Is this an interactive popover
			 * @returns {Boolean} True
			 */
			this.isInteractive = function () {
				return true;
			};
		};

		// inherit
		MeasurePopoverModel.prototype = Object.create(ViewportPopoverModel.prototype);
		// correct the constructor pointer 
		MeasurePopoverModel.prototype.constructor = MeasurePopoverModel;

		MeasurePopoverModel.prototype.getChannelsInformation = function () {
			return this._channelsInfo;
		};

		/**
		 * Update the popover
		 */
		MeasurePopoverModel.prototype.update = function () {
			this.f.requestModule("graphics", _.bind(function (graphicsModule) {
				var graphicsModel = graphicsModule.getGraphicsModel();
				this.channels = [];
				this.t.d.channels = [];

				var parsedInfo = graphicsModel.getParsedChannelsInfo(this._channelsInfo);

				_.each(parsedInfo.channels, function (channel) {
					if (!graphicsModel.isInDifferenceCompareMode() || !channel.isInPreviousAndMatchedWithCurrent()) {
						this.channels.push(channel);
						this.t.d.channels.push(channel);
					}
				}, this);

				if (this.makeTotalCount) {
					this.t.d.totalCount = parsedInfo.total;
				} else {
					this.t.d.totalCount = "";
				}

				this.t.render();
			}, this));
		};

		/**
		 * Update the channels info
		 * @param channelInfo
		 */
		MeasurePopoverModel.prototype.setChannelsInfo = function (channelInfo) {
			this._channelsInfo = channelInfo;
			this.update();
		};

		/**
		 * Set the area of the popover
		 * @param {LineworkWrapper} area
		 */
		MeasurePopoverModel.prototype.setArea = function (area) {
			this.area = area;
		};
    
        /**
         * Replaces/sets the extra header with this new element
         * @param {jQuery.Element} el The new extraHeader component
         */
        MeasurePopoverModel.prototype.setHeaderExtra = function(el){
            this._headerExtra.html(el);
        };
        
        /**
         * Replaces/sets the title header with this new element
         * @param {jQuery.Element} el The new header component
         */
        MeasurePopoverModel.prototype.setHeader = function(el){
            this._headerTitle.html(el);
        };

		return MeasurePopoverModel;
	});

define('web-viewer/measure/AbstractMeasureTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"utils/ShapeUtils",
		"web-viewer/utils/GEOUtils",
		"web-viewer/utils/DrawUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/measure/MeasureLayerModel",
		"web-viewer/measure/MeasureConstants",
		"web-viewer/measure/MeasurePopoverModel",
		"utils/XMLUtils",
		"web-viewer/utils/ConversionConstants",
		"utils/Cache"
	],
	function (_,
		AbstractTool,
		ShapeUtils,
		GEOUtils,
		DrawUtils,
		ViewerUtils,
		MeasureLayerModel,
		MeasureConstants,
		MeasurePopoverModel,
		XMLUtils,
		ConversionConstants,
		Cache) {
		"use strict";

		/**
		 * AbstractMeasureTool, defines the drawing functionality for the measure tools and the functionality to get the information for specific
		 * measure tools from the viewserver.
		 *
		 * @exports AbstractMeasureTool
		 * @memberof MeasureModule
		 * @class AbstractMeasureTool
		 * @augments Viewport.Tools.AbstractTool
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {String} IID
		 */
		var AbstractMeasureTool = function (f, IID) {
			AbstractTool.call(this, f, IID);

			this.f = f;

			this.popover = null;

			/**
			 * States
			 */
			this.defaultState = "default";
			this.drawingState = "drawing";
			this.clickState = "click";

			this.model = new MeasureLayerModel(f, IID);

			this.touchPoint = null;
			this.minimum = 16; //minimum that a user has to drag to start dragging (in pixels)

			this.snapDistance = 16; //distance of the mouse to snap to a linework

			this.infoFetchedFor = null;
			this.isPopoverShown = false;

			// initialize a cache for getting info from the server (this is to prevent flickering in blinking compare mode)
			this._infoCache = new Cache(4, true);
			this._infoCache._loadItem = _.bind(function (url) {
				this.f.ajax(url, {
					success: _.bind(function (data) {
						var info = XMLUtils.xml2obj(data);
						// show error to user when loading info fails
						if (info && info.status && info.status.value == "-1") {
							this.f.emit("userMessage", "warn", info.status.message);
						}

						// register item in cache (so callbacks will be executed)
						this._infoCache._registerItem(url, info);
					}, this),
					error: _.bind(function (jqXHR, textStatus, errorThrown) {
						// ajax request failed, remove from cache
						this._infoCache._registerItem(url);
						this.f.i18n.loadBundle("viewer", function () {
							// show error
							this.f.emit("error.SEVERE", this.f.i18n.t("MeasurementInfoError"), jqXHR, textStatus, errorThrown);
						}, this);
					}, this)
				});
			}, this);


			/**
			 * This is the mode when the tool is active,
			 * it will zoom on click and go the couldStartDraggingMode if left mousedown is clicked
			 */
			this.registerState(this.defaultState, function (that) {
				return {
					"enterState": function () {
						// do nothing
					},
					"mousedown": function (e) {
						that.removePopover();
						that.model.reset();
						that.didReset();


						that.touchPoint = that.keepPointWithinWorldBounds(that.getWorldPoint(e), true);

						var point = that.getPointToSnapTo(e, that.snapDistance);
						if (!point) {
							point = that.keepPointWithinWorldBounds(that.getWorldPoint(e), false);
						}
						that.model.setStartPoint(point);
						that.model.hidePopover();
						that.changeState(that.drawingState);
						e.originalEvent.preventDefault();
					},
					"mouseup": function (e) {
						if (that.model.ruler.startPoint && !that.model.ruler.endPoint) {
							that.changeState(that.clickState);
						} else {
							that.changeState(that.defaultState);
							if (that.model.ruler.startPoint && that.model.ruler.endPoint) {
								that.calculate();
							}
						}
					},
					"mousemove": function (e) {
						that.checkForSnapPoint(e);
					},
					"mouseout": function () {},
					"contextmenu": function (e) {
						// prevent the context menu
						e.preventDefault();
						e.stopPropagation();
						return false;
					},
					"mouseleave": function () {}
				};
			}(this));

			this.registerState(this.drawingState, _.defaults(function (that) {
				return {
					"enterState": function () {
						f.emit("layout.disableTextSelection");
						f.emit("viewport.startInteractive");
					},
					"leaveState": function () {
						setTimeout(function () {
							f.emit("layout.enableTextSelection");
						}, 500);
						f.emit("viewport.stopInteractive");
					},
					"mousemove": function (e) {
						var size = that.keepPointWithinWorldBounds(that.getWorldPoint(e), true).sizeTo(that.touchPoint);
						if (that.model.ruler.endPoint || Math.abs(size.width) >= that.minimum || Math.abs(size.height) > that.minimum) {
							that.registerEndPoint(e);
						}
						that.checkForSnapPoint(e);
					}
				};
			}(this), this.getState(this.defaultState))); //copies the not overridden attributes from defaultstate to this state

			this.registerState(this.clickState, _.defaults(function (that) {
				return {
					"enterState": function () {
						that.changeState(that.defaultState);
					}
				};
			}(this), this.getState(this.drawingState)));

			// always show the crosshair cursor when active
			this.getDefaultCursor = function () {
				return "rulerCursor";
			};

			this._override("initialize", function (widget, toolManager) {
				this._super(widget, toolManager);
				this.changeState(this.defaultState);
				this.registerActiveEvents(["mousedown", "mousemove", "mouseup", "mouseout", "mouseleave", "contextmenu"]);
			});

			this._override("activate", function (interruptRequest) {
				this._super(interruptRequest);
				if (!interruptRequest) {
					this.f.getModule("viewport").addLayer(this.getIID(), this.model);
					if (this.model.ruler.startPoint && this.model.ruler.endPoint) {
						if (this.popover) {
							this.model.addPopoverModel(this.popover);
							this.f.emit("channels.informationAdded", this.popover.getChannelsInformation());
						}
					}
				}
			});

			this._override("deactivate", function (interruptRequest) {
				this._super(interruptRequest);
				if (!interruptRequest) {
					this.model.modelChanged(true);
					this.removePopover();
					this.f.getModule("viewport").removeLayer(this.getIID());
					f.emit("channels.informationNotVisible");
				}
			});
		};

		// extend from the AbstractTool
		AbstractMeasureTool.prototype = new AbstractTool();
		AbstractMeasureTool.prototype.constructor = AbstractMeasureTool;

		/**
		 * Abstract method to set the specific options for the call to the viewserver, calls getInfo()
		 */
		AbstractMeasureTool.prototype.calculate = function () {};

		AbstractMeasureTool.prototype.endPointSet = function () {};

		AbstractMeasureTool.prototype.didReset = function () {};

		/**
		 * Register the end point based on a mouse event. This will check if the endpoint is not near a snap point etc
		 * @param {MouseEvent} e - mouse event
		 */
		AbstractMeasureTool.prototype.registerEndPoint = function (e) {
			var point = this.getPointToSnapTo(e, this.snapDistance);
			if (!point) {
				point = this.keepPointWithinWorldBounds(this.getWorldPoint(e), false);
			}

			this.model.setEndPoint(point, e.shiftKey);
			this.endPointSet();
		};

		/**
		 * Register the start point based on a mouse event. This will check if the startpoint is not near a snap point etc
		 * @param {MouseEvent} e - mouse event
		 */
		AbstractMeasureTool.prototype.registerStartPoint = function (e) {
			var point = this.getPointToSnapTo(e, this.snapDistance);
			if (!point) {
				point = this.keepPointWithinWorldBounds(this.getWorldPoint(e), false);
			}

			this.model.setStartPoint(point, e.shiftKey);
			this.endPointSet();
		};


		/**
		 * Get a point to snap to, if needed
		 * @param {Event} event - reference event
		 * @param {Number} distance - maximum distance to the mouse to snap
		 * @returns {ShapeUtils.Point}
		 */
		AbstractMeasureTool.prototype.getPointToSnapTo = function (e, distance) {
			return null;
		};

		/**
		 * Check if there is a point to snap to
		 * @param {ShapeUtils.Point} refpoint - reference point
		 */
		AbstractMeasureTool.prototype.checkForSnapPoint = function (e) {
			var point = this.getPointToSnapTo(e, this.snapDistance);
			this.model.setSnapIndicatorPoint(point);
		};


		/**
		 * Abstract method to get the info from the received object and show it in a popover
		 */
		AbstractMeasureTool.prototype.showInfo = function (object, requestParameters) {};



		/**
		 * Gets the required information from the viewserver and converts the xml to an object before passing it to showInfo
		 * 
		 * @param {Object} options
		 */
		AbstractMeasureTool.prototype.getInfo = function (options) {
			var parameters = this._generateParameters();
			var generateParams = _.clone(parameters);
			if (parameters != null) {

				_.each(options, function (value, key) {
					parameters[key] = value;
				});

				var page = this.f.getActivePage();
				var url = page.getInfoURL(parameters);

				this._infoCache.get(url, _.bind(function (info) {
					if (info) {
						// the info is available
						this.infoFetchedFor = generateParams;
						this.showInfo(info, parameters);
					}
				}, this));

			}
		};

		/**
		 * Generate the needed parameters for a data request
		 * @returns {Object} the needed parameters
		 */
		AbstractMeasureTool.prototype._generateParameters = function () {
			if (this.model.ruler.startPoint && this.model.ruler.endPoint) {
				var rect = new ShapeUtils.Rect(this.model.ruler.startPoint.copy(), this.model.ruler.startPoint.sizeTo(this.model.ruler.endPoint));
				rect.normalize();

				//lib on viewserver expects this constant value for calculating barcodes and brailles
				var RESOLUTION = ConversionConstants.FIXED_BARCODE_DPMM; //300 dpi converted to dpmm;		

				var context = {
					rectangle: rect,
					// the servlet that should be used to fetch the image information
					servlet: "",
					// the parameters used for requesting the image
					params: {
						x1: 0,
						y1: 0,
						x2: 0,
						y2: 0
					}
				};

				// ask all interceptors to update the context. All listeners have access to the context object and can adapt the information that is in the object
				// there is no predefined order in which these interceptors are called, their actions thus should not overlap!
				this.f.emit("intercept.fragmentURL", context);

				var data = context.params;
				data.res = RESOLUTION;

				return data;
			} else {
				return null;
			}
		};


		/**
		 * @override
		 */
		AbstractMeasureTool.prototype.canBeInterrupted = function () {
			return this.getCurrentStateName() != this.drawingState;
		};


		/**
		 * Can the tool be used on the currently active page?
		 *
		 * @returns {Boolean}
		 */
		AbstractMeasureTool.prototype.canBeUsedOnActivePage = function () {
			return true;
		};

		/**
		 * Update measure information when it is deemed to be outdated
		 * @param {Boolean} forceUpdate - forces the measure tool to update
		 */
		AbstractMeasureTool.prototype.updateIfNeeded = function (forceUpdate) {

			// on force or when we have already data and we have to update it
			if (forceUpdate || (this.infoFetchedFor && this.infoFetchedFor != this._generateParameters())) {
				this.calculate();
			}

			if (this.popover && this.isPopoverShown) {
				this.popover.update();
				this.model.modelChanged();
			}
		};

		/**
		 * Show/update information popover
		 * @param {Object} channelInfo
		 * @param {LineworkWrapper} (area)
		 * @param {JQuery.Element} headerExtra - element that is shown in the header of popover                           
		 */
		AbstractMeasureTool.prototype.showPopover = function (channelInfo, area, title, headerExtra) {
			if (!area) {
				area = this.model.ruler.getSquareLinework()[0];
			}

			if (!this.popover) {
				this.popover = this.createPopoverModel(channelInfo, area, title, headerExtra);
			} else {
				this.popover.setArea(area); // order matters!
				this.popover.setChannelsInfo(channelInfo);
                this.popover.setHeader(title);
                this.popover.setHeaderExtra(headerExtra);
			}

			// note as this popover keeps his ID, it should be added only once 
			this.model.addPopoverModel(this.popover);
			this.isPopoverShown = true;

			this.model.modelChanged();
		};

		/**
		 * Create a popover model for the tool
		 * @param {Object} channelInfo
		 * @param {LineworkWrapper} area
		 * @param {String OR JQuery.Element} An object that is shown in the title (defaults to "Channels" when null)
		 * @param {JQuery.Element} headerExtra - element that is shown in the header of popover
		 * @returns {MeasurePopoverModel}
		 */
		AbstractMeasureTool.prototype.createPopoverModel = function (channelInfo, area, title, headerExtra) {            
			return new MeasurePopoverModel(this.f, area, "Channels", title ? title : this.f.i18n.t("ChannelsTitle"), channelInfo, this._needsTotalCount(), headerExtra);
		};

		/**
		 * Remove the popover
		 */
		AbstractMeasureTool.prototype.removePopover = function () {
			if (this.popover) {
				this.model.removePopoverModel(this.popover.getID());
			}
			this.isPopoverShown = false;
		};

		/**
		 * Does this tool need a total count
		 * @returns {Boolean} False
		 */
		AbstractMeasureTool.prototype._needsTotalCount = function () {
			return false;
		};

		/**
		 * Get the state of the measure tool
		 * @returns {Object}
		 */
		AbstractMeasureTool.prototype.getToolState = function () {
			var state = {
				startPoint: this.model.getStartPoint(),
				endPoint: this.model.getEndPoint(),
				isActive: this.isActive()
			};
			return state;
		};

		/**
		 * Set the state of the measure tool
		 * @param {Object} state
		 */
		AbstractMeasureTool.prototype.setToolState = function (state) {
			if (state.startPoint !== null) {
				this.model.setStartPoint(state.startPoint);
			}
			if (state.endPoint !== null) {
				this.model.setEndPoint(state.endPoint);
				this.endPointSet();
			}
		};

		/**
		 * @override
		 */
		AbstractMeasureTool.prototype.doActivation = function () {
			this.model.modelChanged();
			AbstractTool.prototype.doActivation.call(this);
		};

		return AbstractMeasureTool;

	});

define('web-viewer/measure/RulerPopoverModel',["underscore",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/viewport/model/ViewportPopoverModel",
		"utils/ShapeUtils",
		"Template",
		"tpl!web-viewer/measure/RulerPopoverTemplate",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		ViewerUtils,
		ViewportPopoverModel,
		ShapeUtils,
		Template,
		tpl,
		CoreConstants) {
		"use strict";
		/**
		 * @exports RulerPopoverModel
		 * @memberof MeasureModule
		 * @constructor
		 */
		var RulerPopoverModel = function (f, name, ruler, end, cssClass) {
			ViewportPopoverModel.call(this, f, name);

			var div = $("<div/>").addClass("rulerMeasurement");

			this.ruler = ruler;
			this.end = end;
			this.fix = 3;
			this.f = f;
			if (ruler) {
				this.point = ruler.startPoint;
			}

			// initialize template
			this.t = new Template(f.i18n, tpl);

			// load language if needed
			f.i18n.loadBundle("viewer", _.bind(function () {
				this.t.render();
			}, this));

			// listen to units change
			this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_SIZE", _.bind(this.render, this));

			this.t.setContext(div);

			this.t.bind("mousedown", ".addAnnotation", function (source, event) {
				event.preventDefault();
				event.stopPropagation();
			}, this);

			this.t.bind("click", ".addAnnotation", function (source, event) {
				this._emit("createRulerAnnotation");
				event.preventDefault();
				event.stopPropagation();
			}, this);

			this.t.render();

			this.getFrame = function () {
				if (this.ruler) {
					if (this.end && ruler.endPoint) {
						return new ShapeUtils.Rect(ruler.endPoint.copy(), null);
					} else if (ruler.startPoint) {
						return new ShapeUtils.Rect(ruler.startPoint.copy(), null);
					}
				} else if (this.point) {
					return new ShapeUtils.Rect(this.point.copy(), null);
				}
				return null;
			};

			/**
			 * @override
			 */
			this.getContent = function () {
				return div;
			};

			/**
			 * @override
			 */
			this.getPreferredPosition = function () {
				if (this.ruler.startPoint && this.ruler.endPoint) {
					if (this.ruler.startPoint.x.toFixed(this.fix) == this.ruler.endPoint.x.toFixed(this.fix)) {
						if (this.ruler.startPoint.y <= this.ruler.endPoint.y) {
							return !this.end ? "top" : "bottom";
						} else {
							return !this.end ? "bottom" : "top";
						}
					} else {
						if (this.ruler.startPoint.x <= this.ruler.endPoint.x) {
							return this.end ? "right" : "left";
						} else {
							return this.end ? "left" : "right";
						}
					}
				} else {
					return "bottom";
				}
			};

			/**
			 * @override
			 */
			this.getRestrictBorder = function () {
				return 0;
			};

			/**
			 * Get the context for the popover
			 */
			this.getContext = _.bind(function () {
				return this.f.getContext().find(".viewport");
			}, this);

			/**
			 * Get the class for the popover
			 */
			this.getClass = function () {
				return cssClass;
			};

			/**
			 * Is this an interactive popover
			 * @returns {Boolean} True
			 */
			this.isInteractive = function () {
				return true;
			};
		};

		// inherit
		RulerPopoverModel.prototype = Object.create(ViewportPopoverModel.prototype);
		// correct the constructor pointer 
		RulerPopoverModel.prototype.constructor = RulerPopoverModel;

		/**
		 * is the popover model empty
		 * @returns {Boolean}
		 */
		RulerPopoverModel.prototype.isEmpty = function () {
			if (!this.end) {
				return !(this.f.getOption("PREFERENCES_RULER_DISPLAY_MODE_XY") || (this.point ? true : false));
			} else {
				return !(this.f.getOption("PREFERENCES_RULER_DISPLAY_MODE_DA") || this.f.getOption("PREFERENCES_RULER_DISPLAY_MODE_XY") || this.f.getOption("PREFERENCES_RULER_DISPLAY_MODE_WH"));
			}
		};

		/**
		 * Renders the contents of the ruler popover
		 */
		RulerPopoverModel.prototype.render = function () {
			if (this.ruler) {
				if (!this.ruler.startPoint || !this.ruler.endPoint) {
					return;
				}
			} else {
				if (!this.point) {
					return;
				}
			}

			//add the data for the template to the data object
			_.extend(this.t.d, this.getMeasureData());
			this.t.render();
		};

		/**
		 * Get the measurement information so it can be used by the template
		 * @returns {Object}
		 */
		RulerPopoverModel.prototype.getMeasureData = function () {
			var measureData = {};

			measureData.unitsloc = this.f.getUnitsLocName();
			this.fix = this.f.getUnitsFix();

			var units = this.f.getUnits();

			var rect;
			var resX = null,
				resY = null,
				distanceRes = null;
			if (this.ruler) {
				rect = new ShapeUtils.Rect(this.ruler.startPoint.copy(), this.ruler.startPoint.sizeTo(this.ruler.endPoint));
				if (units === CoreConstants.SizeUnits.PIXELS) {
					var page = this.f.getActivePage();
					resX = ViewerUtils.parseFloat(page.getParam("resolutionx"));
					resY = ViewerUtils.parseFloat(page.getParam("resolutiony"));
				}
			} else if (this.point) {
				rect = new ShapeUtils.Rect(this.point);
			} else {
				rect = new ShapeUtils.Rect();
			}

			measureData.end = this.end;

			if (!this.end) {
				if (this.ruler && !this.ruler.startPoint) {
					return;
				}
				measureData.showXY = this.f.getOption("PREFERENCES_RULER_DISPLAY_MODE_XY") || (this.point ? true : false);
				measureData.showDA = false;
				measureData.showWH = false;

				// convert to correct coordinate system
				var origin = new ShapeUtils.Point(rect.origin.x, rect.origin.y);
				origin = this._coordinates.getTransformedPoint(origin);

				measureData.x = ViewerUtils.convertFromMmValue(origin.x, units, this.fix, resX);
				measureData.y = ViewerUtils.convertFromMmValue(origin.y, units, this.fix, resY);
			} else {
				if (!this.ruler.endPoint) {
					return;
				}
				measureData.createAnnotation = true;
				measureData.showDA = this.f.getOption("PREFERENCES_RULER_DISPLAY_MODE_DA");
				measureData.showXY = this.f.getOption("PREFERENCES_RULER_DISPLAY_MODE_XY");
				measureData.showWH = this.f.getOption("PREFERENCES_RULER_DISPLAY_MODE_WH");

				// convert to correct coordinate system
				var posPoint = new ShapeUtils.Point(rect.right(), rect.bottom());
				posPoint = this._coordinates.getTransformedPoint(posPoint);

				measureData.x = ViewerUtils.convertFromMmValue(posPoint.x, units, this.fix, resX);
				measureData.y = ViewerUtils.convertFromMmValue(posPoint.y, units, this.fix, resY);

				if (units === CoreConstants.SizeUnits.PIXELS) {
					// calculate resolution for diagonal distance
					var angleRad = this.ruler.getAngle() * Math.PI / 180;
					distanceRes = Math.sqrt(Math.cos(angleRad) * Math.cos(angleRad) * resX * resX + Math.sin(angleRad) * Math.sin(angleRad) * resY * resY);
				}
				measureData.distance = ViewerUtils.convertFromMmValue(this.ruler.getLength(), units, this.fix, distanceRes);
				measureData.angle = this.f.getOption("PREFERENCES_RULER_ANGLE_MODE") ? -this.ruler.getAngle().toFixed(2) : this.ruler.getAngle().toFixed(2);

				measureData.width = ViewerUtils.convertFromMmValue(rect.size.width, units, this.fix, resX);
				measureData.height = ViewerUtils.convertFromMmValue(rect.size.height, units, this.fix, resY);
			}

			return measureData;
		};

		return RulerPopoverModel;
	});


define('tpl!web-viewer/measure/RulerToAnnotationText', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='';
 if (d.showXY) { 
__p+='\r\n    <b>'+
((__t=(d.i18n("RulerAnnotationFrom")))==null?'':_.escape(__t))+
'</b><br/>\r\n        &nbsp;'+
((__t=(d.i18n('InfoX')))==null?'':_.escape(__t))+
' '+
((__t=(d.fromX))==null?'':_.escape(__t))+
' '+
((__t=(d.i18n(d.unitsloc)))==null?'':_.escape(__t))+
'<br/>\r\n        &nbsp;'+
((__t=(d.i18n('InfoY')))==null?'':_.escape(__t))+
' '+
((__t=(d.fromY))==null?'':_.escape(__t))+
' '+
((__t=(d.i18n(d.unitsloc)))==null?'':_.escape(__t))+
'<br/>\r\n    <b>'+
((__t=(d.i18n("RulerAnnotationTo")))==null?'':_.escape(__t))+
'</b><br/>\r\n        &nbsp;'+
((__t=(d.i18n('InfoX')))==null?'':_.escape(__t))+
' '+
((__t=(d.x))==null?'':_.escape(__t))+
' '+
((__t=(d.i18n(d.unitsloc)))==null?'':_.escape(__t))+
'<br/>\r\n        &nbsp;'+
((__t=(d.i18n('InfoY')))==null?'':_.escape(__t))+
' '+
((__t=(d.y))==null?'':_.escape(__t))+
' '+
((__t=(d.i18n(d.unitsloc)))==null?'':_.escape(__t))+
'<br/><br/>\r\n';
 } 
__p+='\r\n';
 if (d.showDA) { 
__p+='\r\n    <b>'+
((__t=(d.i18n('InfoDist')))==null?'':_.escape(__t))+
'</b> '+
((__t=(d.distance))==null?'':_.escape(__t))+
' '+
((__t=(d.i18n(d.unitsloc)))==null?'':_.escape(__t))+
'<br/>\r\n    <b>'+
((__t=(d.i18n("InfoAngle")))==null?'':_.escape(__t))+
'</b> '+
((__t=(d.angle))==null?'':_.escape(__t))+
'&#176;<br/>\r\n';
 } 
__p+='\r\n';
 if (d.showWH) { 
__p+='\r\n    <b>'+
((__t=(d.i18n('InfoWidth')))==null?'':_.escape(__t))+
'</b> '+
((__t=(d.width))==null?'':_.escape(__t))+
' '+
((__t=(d.i18n(d.unitsloc)))==null?'':_.escape(__t))+
'<br/>\r\n    <b>'+
((__t=(d.i18n('InfoHeight')))==null?'':_.escape(__t))+
'</b> '+
((__t=(d.height))==null?'':_.escape(__t))+
' '+
((__t=(d.i18n(d.unitsloc)))==null?'':_.escape(__t))+
'<br/>\r\n';
 } 
__p+='';
return __p;
}});
define('web-viewer/measure/RulerTool',["underscore",
		"web-viewer/measure/AbstractMeasureTool",
		"web-viewer/measure/MeasureConstants",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/measure/RulerPopoverModel",
		"Template",
		"tpl!web-viewer/measure/RulerToAnnotationText",
		"web-viewer/annotation/AnnotationConstants",
		"utils/ShapeUtils",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractMeasureTool,
		MeasureConstants,
		ListenerUtil,
		RulerPopoverModel,
		Template,
		tpl,
		AnnotationConstants,
		ShapeUtils,
		CoreConstants) {
		"use strict";

		/**
		 * The RulerTool enables the user to measure a distance on the viewport
		 *
		 * @class RulerTool the ruler tool is an interactive ruler
		 * @memberof Measure
		 * @constructor
		 * @param {Core.ViewerFacade}
		 */
		var RulerTool = function RulerTool(f) {
			AbstractMeasureTool.call(this, f, CoreConstants.TOOLS.MEASURE_RULER);
			ListenerUtil.mixin(this, f);

			this.hoverState = "hover";
			this.draggingState = "dragging";
			this.drawnState = "drawn";

			// initialize template used to generate annotation with
			this.t = new Template(f.i18n, tpl);

			this.model.setRulerType(MeasureConstants.MeasureType.Ruler);
			this.model.setAllowStartPointSnapping(true);

			this.model.setDrawType(MeasureConstants.MeasureDrawType.LINE);

			this.startPopover = new RulerPopoverModel(f, "startRulerPopover", this.model.ruler, false, "transparent");
			this.endPopover = new RulerPopoverModel(f, "endRulerPopover", this.model.ruler, true, "transparent");

			this._registerListener("createRulerAnnotation", this._createAnnotation, this, this.endPopover);


			this.getState(this.defaultState).canBeInterrupted = true;

			/**
			 * Drawn State --> Default state after ruler is drawn
			 */
			this.registerState(this.drawnState, _.defaults({
				// check if we will go to hover mode
				"mousemove": _.bind(function (e) {
					if (this._isHovering(e)) {
						this.changeState(this.hoverState);
					}
					this.checkForSnapPoint(e);
				}, this),
				"mouseup": _.bind(function (e) {
					if (this.model.ruler.startPoint && !this.model.ruler.endPoint) {
						this.changeState(this.clickState);
					} else {
						this.changeState(this.drawnState);
						if (this.model.ruler.startPoint && this.model.ruler.endPoint) {
							this.calculate();
						}
					}
				}, this),
				"mouseleave": _.bind(function (e) {
					this.changeState(this.drawnState);
				}, this),
				canBeInterrupted: true
			}, this.getState(this.defaultState)));

			/**
			 * Hover State --> Mouse is over one of the anchors.
			 * When in this state, we can start dragging anchors around.
			 */
			this.registerState(this.hoverState, _.defaults({
				// start dragging
				"mousedown": _.bind(function (e) {
					var distanceToPoints = this._getDistanceToPoints(e);
					if (distanceToPoints) {
						this._isMovingStartPoint = (distanceToPoints.toStartPoint < distanceToPoints.toEndPoint);
						this.changeState("dragging");
					}
				}, this),
				"mousemove": _.bind(function (e) {
					if (!this._isHovering(e)) {
						this.changeState(this.drawnState);
					}
					this.checkForSnapPoint(e);
				}, this),

				canBeInterrupted: false,
				canBeInterruptedForNavigation: true
			}, this.getState(this.drawnState)));

			/**
			 * Dragging State --> We're moving one of the end points.
			 */
			this.registerState(this.draggingState, _.defaults({
				"enterState": _.bind(function () {
					this.setCursor("cursor-none");
				}, this),
				// move end points
				"mousemove": _.bind(function (e) {
					this.setCursor("cursor-none");
					if (this._isMovingStartPoint) {
						this.registerStartPoint(e);
					} else {
						this.registerEndPoint(e);
					}
					this.checkForSnapPoint(e);
				}, this),
				// when releasing, we are above an end point, so we will go to hover
				"mouseup": _.bind(function (e) {
					this.changeState(this.hoverState);
				}, this),

				canBeInterrupted: false,
				canBeInterruptedForNavigation: true
			}, this.getState(this.drawnState)));

			/**
			 * Drawing State --> We're drawing the ruler
			 * Overwrites Drawing state of AbstractMeasureTool
			 */
			this.registerState(this.drawingState, _.defaults({
				"enterState": function () {
					f.emit("layout.disableTextSelection");
					f.emit("viewport.startInteractive");
				},
				"leaveState": function () {
					setTimeout(function () {
						f.emit("layout.enableTextSelection");
					}, 500);
					f.emit("viewport.stopInteractive");
				},
				// update ruler
				"mousemove": _.bind(function (e) {
					if (this.model.ruler.endPoint) {
						this.setCursor("cursor-none");
					}
					var size = this.keepPointWithinWorldBounds(this.getWorldPoint(e), true).sizeTo(this.touchPoint);
					if (this.model.ruler.endPoint || Math.abs(size.width) >= this.minimum || Math.abs(size.height) > this.minimum) {
						this.registerEndPoint(e);
					}
					this.checkForSnapPoint(e);
				}, this),
				canBeInterrupted: false,
				canBeInterruptedForNavigation: true
			}, this.getState(this.drawnState)));

			/**
			 * Click State --> We clicked, and will draw/update the ruler until we click again
			 */
			this.registerState(this.clickState, _.defaults({
				"enterState": function () {},
				"mousedown": _.bind(function (e) {
					this.registerEndPoint(e);
					this.changeState(this.drawnState);
				}, this)
			}, this.getState(this.drawingState)));

			this._override("activate", function (interruptRequest) {
				this._super(interruptRequest);

				this.endPointSet();
			});

			f.getCurrentUser().on("options.changed.PREFERENCES_RULER_ANGLE_MODE", _.bind(function (oldValue, newValue) {
				this.startPopover.render();
				this.endPopover.render();
				this.model.repaint();
			}, this));
			f.getCurrentUser().on("options.changed.PREFERENCES_RULER_DISPLAY_MODE", _.bind(function (oldValue, newValue) {
				if (this.startPopover.isEmpty()) {
					this.model.removePopoverModel(this.startPopover.getID());
				} else {
					if (this.model.getPopoverModel(this.startPopover.getID()) === undefined) {
						this.model.addPopoverModel(this.startPopover);
					}
					this.startPopover.render();
				}
				if (this.endPopover.isEmpty()) {
					this.model.removePopoverModel(this.endPopover.getID());
				} else {
					if (this.model.getPopoverModel(this.endPopover.getID()) === undefined) {
						this.model.addPopoverModel(this.endPopover);
					}
					this.endPopover.render();
				}
				this.model.repaint();
			}, this));
			f.getCurrentUser().on("options.changed.PREFERENCES_RULER_DISPLAY_COLOR", _.bind(function (oldValue, newValue) {
				this.startPopover.render();
				this.endPopover.render();
				this.model.repaint();
			}, this));
		};

		// extend from the AbstractTool
		// inherit
		RulerTool.prototype = Object.create(AbstractMeasureTool.prototype);
		RulerTool.prototype.constructor = RulerTool;

		/**
		 * Actions that need to be done when an endpoint is set
		 * @override
		 */
		RulerTool.prototype.endPointSet = _.throttle(function () {
			if (this.model.ruler.startPoint && !this.model.getPopoverModel(this.startPopover.getID()) && !this.startPopover.isEmpty()) {
				this.model.addPopoverModel(this.startPopover);
			}
			if (this.model.ruler.endPoint && !this.model.getPopoverModel(this.endPopover.getID()) && !this.endPopover.isEmpty()) {
				this.model.addPopoverModel(this.endPopover);
			}

			this.startPopover.render();
			this.endPopover.render();
		}, 100);

		/**
		 * Remove popover for start and end point
		 * @override
		 */
		RulerTool.prototype.didReset = function () {
			this.model.removePopoverModel(this.startPopover.getID());
			this.model.removePopoverModel(this.endPopover.getID());
		};

		/**
		 * Remove popover for start and end point
		 * @override
		 */
		RulerTool.prototype.removePopover = function () {
			this.didReset();
		};

		/**
		 * Get a point to snap to, if needed
		 * @param {ShapeUtils.Point} refpoint - reference point
		 * @param {Number} distance - maximum distance to the mouse to snap
		 * @returns {ShapeUtils.Point}
		 * @override
		 */
		RulerTool.prototype.getPointToSnapTo = function (e, distance) {
			if (!this.f.getActivePage().hasCad() && !this.f.getActivePage().hasLinkedPage()) {
				return null;
			}
			var v = this._widget.requestView(e);
			if (v) {
				var refpoint = this.getScenePoint(e);
				var point = v.getClosestLineworkPointFromPoint(refpoint, distance);
				if (point) {
					point = this.sceneToWorld(point);
				}
				return point;
			} else {
				return null;
			}
		};

		/**
		 * Check whether or not we are hovering over the start or end point of the ruler by measuring the distance to the points.
		 * @param e
		 * @returns {Boolean}
		 */
		RulerTool.prototype._isHovering = function (e) {
			var distanceToPoints = this._getDistanceToPoints(e);
			if (distanceToPoints && (distanceToPoints.toStartPoint < MeasureConstants.RulerPoint.CLOSETODISTANCE || distanceToPoints.toEndPoint < MeasureConstants.RulerPoint.CLOSETODISTANCE)) {
				this.setCursor("cursor-moveObject");
				return true;
			} else {
				this.setCursor(this.getDefaultCursor());
				return false;
			}
		};

		/**
		 * Get the distance from the current mouse location to the ruler's end points.
		 * It only returns something if the anchors are already defined
		 * @param e
		 * @returns {Object} - contains distance to start and to end point
		 */
		RulerTool.prototype._getDistanceToPoints = function (e) {
			var scenePoint = this.getScenePoint(e);
			var worldStartPoint = this.model.ruler.startPoint;
			var worldEndPoint = this.model.ruler.endPoint;

			if (worldStartPoint && worldEndPoint) {
				var startPoint = this._widget.getWorld().getScenePointFromWorld(worldStartPoint);
				var endPoint = this._widget.getWorld().getScenePointFromWorld(worldEndPoint);
				var sizeToStart = scenePoint.sizeTo(startPoint);
				var distToStart = Math.sqrt(Math.abs(sizeToStart.width * sizeToStart.width + sizeToStart.height * sizeToStart.height));
				var sizeToEnd = scenePoint.sizeTo(endPoint);
				var distToEnd = Math.sqrt(Math.abs(sizeToEnd.width * sizeToEnd.width + sizeToEnd.height * sizeToEnd.height));

				return {
					toStartPoint: distToStart,
					toEndPoint: distToEnd
				};
			}

			return undefined;
		};

		/**
		 * Create an annotation out of the ruler. emits an event to request an annotation to be added.
		 */
		RulerTool.prototype._createAnnotation = function () {
			var startPoint = this.startPopover.getMeasureData();
			this.t.d.fromX = startPoint.x;
			this.t.d.fromY = startPoint.y;
			_.extend(this.t.d, this.endPopover.getMeasureData());

			var content = this.t.getRenderedHtml();
			var rect = new ShapeUtils.Rect(this.model.ruler.startPoint, this.model.ruler.startPoint.sizeTo(this.model.ruler.endPoint));
			this.f.emit("module.annotation.createAnnotation", rect, content, AnnotationConstants.AnnotationType.ANNOT_ARROW);
			this.f.emit("toolManager.deactivateTool");
		};

		/**
		 * @override
		 */
		RulerTool.prototype.canBeInterrupted = function (interruptor) {
			return this.getCurrentState().canBeInterrupted ||
				(this.getCurrentState().canBeInterruptedForNavigation && interruptor &&
					(interruptor.getIID() === CoreConstants.TOOLS.PAN ||
						interruptor.getIID() === CoreConstants.TOOLS.NAVIGATIONPAN ||
						interruptor.getIID() === CoreConstants.TOOLS.TOUCH ||
						interruptor.getIID() === CoreConstants.TOOLS.ZOOM));
		};

		return RulerTool;
	});

define('web-viewer/measure/BarcodeMeasureTool',["underscore",
		"web-viewer/measure/AbstractMeasureTool",
		"web-viewer/measure/MeasureConstants",
        "web-viewer/measure/MeasurePopoverModel",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractMeasureTool,
		MeasureConstants,
        MeasurePopoverModel,
		ViewerUtils,
		CoreConstants) {
		"use strict";

		/**
		 * Barcode measure tool,  You can see the barcode readout of your selected area
		 *
		 * @exports BarcodeMeasureTool
		 * @memberof MeasureModule
		 * @class BarcodeMeasureTool
		 * @augments Measure.AbstractMeasureTool
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {MeasureModule} measureModule
		 */
		var BarcodeMeasureTool = function (f, measureModule) {
			AbstractMeasureTool.call(this, f, CoreConstants.TOOLS.MEASURE_BARCODE);

			this.f = f;
			this.measureModule = measureModule;
			this.model.setRulerType(MeasureConstants.MeasureType.Barcode);

		};

		// extend from the AbstractTool
		BarcodeMeasureTool.prototype = Object.create(AbstractMeasureTool.prototype);
		BarcodeMeasureTool.prototype.constructor = BarcodeMeasureTool;


		/**
		 * @override
		 *
		 * Gets the information from the to object converted xml and creates a MeasurePopoverModel that shows the information and tells the AbstractModel to add the popover
		 *
		 * @param {Object} object
		 */
		BarcodeMeasureTool.prototype.showInfo = function (object) {
			var info = [];
			if (object) {
				if (object.status.info) {
					if (!_.isArray(object.status.info.barcodegv)) {
						object.status.info.barcodegv = [object.status.info.barcodegv];
					}
					this.f.i18n.loadBundle("viewer", _.bind(function () {
						_.each(object.status.info.barcodegv, _.bind(function (barcodesWrapper) {
							var barcodes = barcodesWrapper.xml.Barcodes;
							var barcodesArray = barcodes.Barcode;
							if (!_.isArray(barcodesArray)) {
								barcodesArray = [barcodesArray];
							}
							info[barcodesWrapper.color] = "";
							_.each(barcodesArray, _.bind(function (barcode, i) {
								// multiple barcodes
								if (i > 0) {
									info[barcodesWrapper.color] += " | ";
								}
								info[barcodesWrapper.color] += barcode.attributes.Type + ": " + barcode.Decoded;
								if (barcode["Decoded_reverse_dir"]) {
									info[barcodesWrapper.color] += " (-->) " + barcode["Decoded_reverse_dir"] + " (<--)";
								}
								if (barcode.Magnification !== undefined && parseInt(barcode.Magnification, 10) !== 100 && parseInt(barcode.Magnification, 10) !== 999) {
									info[barcodesWrapper.color] += " " + barcode.Magnification + "%";
								}
								var BWR = ViewerUtils.parseFloat(barcode.BWR);
								if (BWR < 999 && BWR !== 0) {
									info[barcodesWrapper.color] += " [" + BWR + " " + this.f.i18n.t("Unitmm") + "] ";
								}
							}, this));
						}, this));
					}, this));
				}
				this.f.emit("channels.informationAdded", info);
			}
            
			this.showPopover(info);
		};

		/**
		 * @override
		 *
		 * Removes the popover from the AbstractModel
		 */
		BarcodeMeasureTool.prototype.removePopover = function () {
			if (this.popover) {
				this.model.removePopoverModel(this.popover.getID());
			}
		};

		/**
		 * @override
		 *
		 * Sets the required options for the call to the viewserver
		 */
		BarcodeMeasureTool.prototype.calculate = function () {
			var options = {
				opmask: 6
			};

			this.getInfo(options);
		};

		/**
		 * Can the tool be used on the currently active page
		 * @Override
		 * @returns {Boolean}
		 */
		BarcodeMeasureTool.prototype.canBeUsedOnActivePage = function () {
			var page = this.f.getActivePage();
			return page && page.hasGraphics();
		};
    
        BarcodeMeasureTool.prototype.createPopoverModel = function (channelInfo, area) {            
			return new MeasurePopoverModel(this.f, area, "Channels", $("<i>").addClass("icon icon-normal icon-WCR_Measure-Barcode"), channelInfo, this._needsTotalCount());
		};

		return BarcodeMeasureTool;

	});

define('web-viewer/measure/BrailleMeasureTool',["underscore",
		"web-viewer/measure/AbstractMeasureTool",
		"web-viewer/measure/MeasureConstants",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/core/CoreConstants",
        "component/tooltip/Tooltip"
	],
	function (_,
		AbstractMeasureTool,
		MeasureConstants,
		ViewerUtils,
		CoreConstants,
        Tooltip) {

		/**
		 * Braille measure tool,  You can see the braille readout of your selected area
		 *
		 * @exports BrailleMeasureTool
		 * @memberof MeasureModule
		 * @class BrailleMeasureTool
		 * @augments Measure.AbstractMeasureTool
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {MeasureModule} measureModule
		 */
		var BrailleMeasureTool = function (f, measureModule) {
			AbstractMeasureTool.call(this, f, CoreConstants.TOOLS.MEASURE_BRAILLE);

			this.model.setRulerType(MeasureConstants.MeasureType.Braille);

			if (MeasureConstants.BrailleLanguage[f.getOption("PREFERENCES_BRAILLE_LANGUAGE")]) {
				this.language = MeasureConstants.BrailleLanguage[f.getOption("PREFERENCES_BRAILLE_LANGUAGE")].url;
			}
			else {
				this.language = MeasureConstants.BrailleLanguage[MeasureConstants.LanguageDefault].url;
			}
			this.f = f;
			this.measureModule = measureModule;

			//Change the braille language
			f.getCurrentUser().on("options.changed.PREFERENCES_BRAILLE_LANGUAGE", _.bind(function (oldValue, newValue) {
				this.language = MeasureConstants.BrailleLanguage[newValue].url;
				this.removePopover();
				this.calculate();
			}, this));
		};

		// extend from the AbstractMeasureTool
		BrailleMeasureTool.prototype = Object.create(AbstractMeasureTool.prototype);
		BrailleMeasureTool.prototype.constructor = BrailleMeasureTool;

		/**
		 * @override
		 *
		 *  Gets the information from the to object converted xml and creates a MeasurePopoverModel that shows the information and tells the AbstractModel to add the popover
		 *
		 *  @param {Object} object
		 *  @param {Object} requestParameters - parameters used to gather information
		 */
		BrailleMeasureTool.prototype.showInfo = function (object, requestParameters) {
            var passBrailleCheck = true;
            var containsBraille = false;
			var brailleChannels = [];
			if (object) {
				if (object.status.info) {
					var brailleInfo = object.status.info.braille;
					//possible that braille consists of multiple colors or is in both documents of side-by-side compare
					if (!(brailleInfo instanceof Array)) {
						brailleInfo = [brailleInfo];
					}
					//run over all braille information of different colors
					_.each(brailleInfo, function (braille) {
						//it is also possible that there are multiple regions of braille
						var regions = braille.xml.Regions.Braille;
						if (!(regions instanceof Array)) { //make sure that regions is an array
							regions = [regions];
						}
						//run over all braille regions
						_.each(regions, function (region, index) {
                            containsBraille = true;
                            if(region.attributes.Status !== "Pass") {
                                passBrailleCheck = false;
                            }
                            
							var regionIndex = "";
							//if there are multiple regions make sure the region index is places before each line number
							if (regions.length > 1) {
								regionIndex = (index + 1) + ",";
							}
							if (brailleChannels[braille.color] === undefined) {
								brailleChannels[braille.color] = "";
							}
							//when more lines of braille are detected display the index of the line
							if (region.Line instanceof Array) {
								_.each(region.Line, function (brailleLine, lineIndex) {
									if (lineIndex >= 1) {
										brailleChannels[braille.color] += " ";
									}
									brailleChannels[braille.color] += "[" + regionIndex + (lineIndex + 1) + "] " + brailleLine;
								});
							} else if (regions.length > 1) { //if there are multiple region always add region number and line number (even if only one line)
								brailleChannels[braille.color] += "[" + regionIndex + "1" + "] " + region.Line + " ";
							} else { //only one region and one line
								brailleChannels[braille.color] += region.Line;
							}
						}, this);

					}, this);
				}
				this.f.emit("channels.informationAdded", brailleChannels);
			}
            
            var icon = null;
            if (containsBraille) {
                icon = passBrailleCheck ? this._createPassIcon() : this._createFailIcon();
            }

            var title = $("<span>")
            .append(
                $("<i>").addClass("icon icon-normal icon-WCR_Measure-Braille")
            ).append($("<span>").addClass("titleText").text(requestParameters.brlang));
            this.showPopover(brailleChannels, null, title, icon);
		};

		/**
		 * @override
		 *
		 * Sets the required options for the call to the viewserver
		 */
		BrailleMeasureTool.prototype.calculate = function () {
			var viewServerLang = this.language;
			if (viewServerLang === undefined || viewServerLang === MeasureConstants.BrailleLanguage[MeasureConstants.LanguageAutomatic].url) {
				// select the language automatically, default to English
				viewServerLang = MeasureConstants.BrailleLanguage[MeasureConstants.LanguageLocales[this.f.getCurrentUser().getLocale()]].url;
				if (viewServerLang === undefined) {
					viewServerLang = MeasureConstants.BrailleLanguage[MeasureConstants.LanguageDefault].url;
				}
			}
			var options = {
				opmask: 5,
				brlang: viewServerLang
			};

			this.getInfo(options);

		};

		BrailleMeasureTool.prototype.canBeUsedOnActivePage = function () {
			var page = this.f.getActivePage();
			return page && page.hasGraphics();
		};
    
        /**
         * Creates a success icon with appropriate tooltip (used to display braille check results)
         * @private
         * @returns {jQuery.Element} the icon
         */
        BrailleMeasureTool.prototype._createPassIcon = function(){           
            var icon = $("<i>").addClass("icon icon-normal icon icon-WCR_Msg-Success-round");
            var wrapper = $("<div>").append(icon);
            new Tooltip(this.f.i18n, icon, wrapper, this.f.getContext(), false, "viewer", "BraillePass");
            return wrapper;
        }
        
        /**
         * Creates a fail icon with appropriate tooltip (used to display braille check results)
         * @private
         * @returns {jQuery.Element} the icon
         */
        BrailleMeasureTool.prototype._createFailIcon = function(){
            var icon = $("<i>").addClass("icon icon-normal icon icon-WCR_Msg-Warning");
            var wrapper = $("<div>").append(icon);
            new Tooltip(this.f.i18n, icon, wrapper, this.f.getContext(), false, "viewer", "BrailleFail");
            return wrapper;
        }

		return BrailleMeasureTool;

	});

define('web-viewer/measure/EyeDropperUnitSelector',["web-viewer/layout/component/DropdownMiniButton",
		"web-viewer/core/CoreConstants",
		"underscore"
	],
	function (DropdownMiniButton,
		CoreConstants,
		_) {
		"use strict";

		/**
		 * @exports EyeDropperUnitSelector
		 * @class The EyeDropperUnitSelector has a button that allows setting the unit of the eye dropper
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var EyeDropperUnitSelector = function (f) {
			this.f = f;

			/**
			 * Make the options dropdown
			 */
			this._optionButton = new DropdownMiniButton(this.f, "eyeDropperUnitSettings", "icon-WCR_Settings-submenu");
			this._optionButton.setTooltip("viewer", "UnitSettings");

			this.f.i18n.loadBundle("viewer", function () {
				this._optionButton.addOption({
					name: 'eyeDropperUnitPixels',
					content: this.f.i18n.t(CoreConstants.EyeDropperUnitsLocNames[CoreConstants.EyeDropperUnits.PIXELS]),
					onchange: _.bind(this._toggleEyeDropperUnit, this, CoreConstants.EyeDropperUnits.PIXELS),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'eyeDropperUnitSQMM',
					content: this.f.i18n.t(CoreConstants.EyeDropperUnitsLocNames[CoreConstants.EyeDropperUnits.SQMM]),
					onchange: _.bind(this._toggleEyeDropperUnit, this, CoreConstants.EyeDropperUnits.SQMM),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'eyeDropperUnitSQCM',
					content: this.f.i18n.t(CoreConstants.EyeDropperUnitsLocNames[CoreConstants.EyeDropperUnits.SQCM]),
					onchange: _.bind(this._toggleEyeDropperUnit, this, CoreConstants.EyeDropperUnits.SQCM),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'eyeDropperUnitSQINCH',
					content: this.f.i18n.t(CoreConstants.EyeDropperUnitsLocNames[CoreConstants.EyeDropperUnits.SQINCH]),
					onchange: _.bind(this._toggleEyeDropperUnit, this, CoreConstants.EyeDropperUnits.SQINCH),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'eyeDropperUnitPercent',
					content: this.f.i18n.t(CoreConstants.EyeDropperUnitsLocNames[CoreConstants.EyeDropperUnits.PERCENT]),
					onchange: _.bind(this._toggleEyeDropperUnit, this, CoreConstants.EyeDropperUnits.PERCENT),
					userCannotUncheck: true
				});

				// intial rendering
				this._updateOptions();

				// listen to changes
				this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_EYEDROPPER", _.bind(this._updateOptions, this));
			}, this);
		};

		/**
		 * Get the button element
		 * @returns {JQueryObj} element of the button
		 */
		EyeDropperUnitSelector.prototype.getButtonElement = function () {
			return this._optionButton.element;
		};

		/**
		 * Toggle the eye dropper unit
		 * @param {CoreConstants.EyeDropperUnits} mode - the new mode
		 * @param {Boolean} toggle - true or false
		 */
		EyeDropperUnitSelector.prototype._toggleEyeDropperUnit = function (mode, toggle) {
			if (toggle) {
				this.f.setOption("PREFERENCES_UNIT_EYEDROPPER", mode, true);

				// link with unit size
				if (mode !== CoreConstants.EyeDropperUnits.PERCENT) {
					this.f.setOption("PREFERENCES_UNIT_SIZE", mode, true);
				}
			}
		};

		/**
		 * Update the options in the menu (re-render)
		 */
		EyeDropperUnitSelector.prototype._updateOptions = function () {
			var currentEyeDropperUnit = this.f.getOption("PREFERENCES_UNIT_EYEDROPPER");
			this._optionButton.toggleOption("eyeDropperUnitPixels", currentEyeDropperUnit === CoreConstants.EyeDropperUnits.PIXELS);
			this._optionButton.toggleOption("eyeDropperUnitSQMM", currentEyeDropperUnit === CoreConstants.EyeDropperUnits.SQMM);
			this._optionButton.toggleOption("eyeDropperUnitSQCM", currentEyeDropperUnit === CoreConstants.EyeDropperUnits.SQCM);
			this._optionButton.toggleOption("eyeDropperUnitSQINCH", currentEyeDropperUnit === CoreConstants.EyeDropperUnits.SQINCH);
			this._optionButton.toggleOption("eyeDropperUnitPercent", currentEyeDropperUnit === CoreConstants.EyeDropperUnits.PERCENT);
		};

		return EyeDropperUnitSelector;
	});

define('web-viewer/measure/EyeDropperPopoverModel',["web-viewer/measure/MeasurePopoverModel",
		"web-viewer/measure/EyeDropperUnitSelector",
		"underscore",
		"jquery"
	],
	function (MeasurePopoverModel,
		EyeDropperUnitSelector,
		_,
		$) {
		"use strict";

		/**
		 * MeasurePopoverModel specific for the eye dropper.
		 * 
		 * @exports EyeDropperPopoverModel
		 * @memberof MeasureModule
		 * @class EyeDropperPopoverModel
		 * @augments MeasurePopoverModel
		 * @constructor
		 * 
         * @param {Core.ViewerFacade} f
		 * @param {LineworkWrapper} area
		 * @param {String} name
		 * @param {String | JQuery.Element} title - Title to be shown in the header
		 * @param {Array<Number> | Array<String>} channels
		 * @param {Boolean} [makeTotalCount] Defines if a total of the different channel values needs to be calculated, only possible if the 
		 * 										values of the channels array are numbers
		 * @param {String | JQuery.Element} headerExtra - A jQuery component or string that is displayed in upper-right corner of this popver
		 */
		var EyeDropperPopoverModel = function (f, area, name, title, channels, makeTotalCount, headerExtra) {
			MeasurePopoverModel.call(this, f, area, name, title, channels, makeTotalCount, headerExtra);

			this._eyeDropperUnitSelector = null;

			this.f.getDocumentController().whenActivePageChangedAndParametersAvailable(this._updateEyeDropperUnitSelectorButton, this);
		};

		// inherit
		EyeDropperPopoverModel.prototype = Object.create(MeasurePopoverModel.prototype);
		// correct the constructor pointer 
		EyeDropperPopoverModel.prototype.constructor = EyeDropperPopoverModel;

		/**
		 * Called when the active page is changed to either show it or not for digital film
		 */
		EyeDropperPopoverModel.prototype._updateEyeDropperUnitSelectorButton = function () {
			if (this.f.getActivePage().isDigitalFilmPage()) {
				if (this._eyeDropperUnitSelector === null) {
					this._eyeDropperUnitSelector = new EyeDropperUnitSelector(this.f);
					this._headerExtra.append(this._eyeDropperUnitSelector.getButtonElement());
				} else {
					this._eyeDropperUnitSelector.getButtonElement().show();
				}
			} else {
				if (this._eyeDropperUnitSelector !== null) {
					this._eyeDropperUnitSelector.getButtonElement().hide();
				}
			}
		};

		return EyeDropperPopoverModel;
	});

define('web-viewer/measure/EyeDropperMeasureTool',["underscore",
		"web-viewer/measure/AbstractMeasureTool",
		"web-viewer/measure/MeasureConstants",
		"web-viewer/measure/EyeDropperPopoverModel",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractMeasureTool,
		MeasureConstants,
		EyeDropperPopoverModel,
		ShapeUtils,
		ViewerUtils,
		CoreConstants) {
		"use strict";

		/**
		 * Eye dropper tool, You can see the color density of your selected area/point
		 *
		 * @exports EyeDropperMeasureTool
		 * @memberof MeasureModule
		 * @class EyeDropperMeasureTool
		 * @augments Measure.AbstractMeasureTool
		 *
		 * @param {Core.ViewerFacade} f
		 */
		var EyeDropperMeasureTool = function (f) {
			AbstractMeasureTool.call(this, f, CoreConstants.TOOLS.MEASURE_EYEDROPPER);

			this.model.setRulerType(MeasureConstants.MeasureType.EyeDropper);

			//override the enterState event for the clickState
			this.registerState(this.clickState, _.defaults({
				"enterState": _.bind(function () {
					var worldResolution = this.getWorldResolution();
					if (worldResolution !== null) {
						var pixelSize = MeasureConstants.ClickSize.NORMAL;
						if (this.f.getActivePage().isDigitalFilmPage()) {
							pixelSize = MeasureConstants.ClickSize.DIGITAL_FILM;
						}
						var sizeX = pixelSize / worldResolution.x; //get the amount of px per mm for the document, not for the screen
						var sizeY = pixelSize / worldResolution.y;
						this.model.setStartPoint(new ShapeUtils.Point(this.model.ruler.startPoint.x - sizeX / 2, this.model.ruler.startPoint.y - sizeY / 2));
						this.model.setEndPoint(new ShapeUtils.Point(this.model.ruler.startPoint.x + sizeX, this.model.ruler.startPoint.y + sizeY));

						this.calculate();
						this.changeState(this.defaultState);
					}
				}, this)
			}, this.getState(this.drawingState)));


			// parsed channel info in %
			this._channels = [];

			// listen to preference changes
			this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_EYEDROPPER", _.bind(this._onEyeDropperUnitChanged, this));

			// listen to unit size for adapting eye dropper units
			this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_SIZE", _.bind(function () {
				var currentSizeUnit = this.f.getOption("PREFERENCES_UNIT_SIZE");

				// sync with size units
				if (currentSizeUnit !== CoreConstants.SizeUnits.POINTS) { // safety measure
					this.f.setOption("PREFERENCES_UNIT_EYEDROPPER", currentSizeUnit, true);
				}
			}, this));
		};

		// extend from the AbstractTool
		EyeDropperMeasureTool.prototype = Object.create(AbstractMeasureTool.prototype);
		EyeDropperMeasureTool.prototype.constructor = EyeDropperMeasureTool;

		/**
		 * @override
		 *
		 * Gets the information from the to object converted xml and creates a MeasurePopoverModel that shows the information and tells the AbstractModel to add the popover
		 *
		 * @param {Object} object
		 */
		EyeDropperMeasureTool.prototype.showInfo = function (object) {
			this._channels = [];
			if (object) {
				if (object.status.info.dotpercentage) {
					if (!_.isArray(object.status.info.dotpercentage)) {
						object.status.info.dotpercentage = [object.status.info.dotpercentage];
					}

					_.each(object.status.info.dotpercentage, function (channel) {
						this._channels[channel.color] = ViewerUtils.parseFloat(channel.dotvalue);
					}, this);
				}
			}
			this._onEyeDropperUnitChanged(true);
		};

		/**
		 * Converts the values to the right unit
		 * @param {Boolean} forceShow
		 */
		EyeDropperMeasureTool.prototype._onEyeDropperUnitChanged = function (forceShow) {
			if ((this.popover && this.isPopoverShown) || forceShow) {
				var ruler = this.model.ruler;
				if (ruler && ruler.startPoint && ruler.endPoint) {
					var size = ruler.startPoint.sizeTo(ruler.endPoint);

					var currentUnit = CoreConstants.EyeDropperUnits.PERCENT; // default for non-digital films
					if (this.f.getActivePage().isDigitalFilmPage()) {
						currentUnit = this.f.getOption("PREFERENCES_UNIT_EYEDROPPER");
					}

					var width = size.width; // in mm
					var height = size.height; // in mm

					var area = 0;

					switch (currentUnit) {
					case CoreConstants.EyeDropperUnits.SQMM:
						area = width * height;
						break;
					case CoreConstants.EyeDropperUnits.SQCM:
						area = ViewerUtils.parseFloat(ViewerUtils.convertFromMmValue(width, CoreConstants.SizeUnits.CM)) *
							ViewerUtils.parseFloat(ViewerUtils.convertFromMmValue(height, CoreConstants.SizeUnits.CM));
						break;
					case CoreConstants.EyeDropperUnits.SQINCH:
						area = ViewerUtils.parseFloat(ViewerUtils.convertFromMmValue(width, CoreConstants.SizeUnits.INCH)) *
							ViewerUtils.parseFloat(ViewerUtils.convertFromMmValue(height, CoreConstants.SizeUnits.INCH));
						break;
					case CoreConstants.EyeDropperUnits.PIXELS:
						var worldResolution = this.getWorldResolution();
						if (worldResolution !== null) {
							area = (width * worldResolution.x) * (height * worldResolution.y);
						}
						break;
					case CoreConstants.EyeDropperUnits.PERCENT:
						area = 100;
						break;
					}

					// convert
					var convertedChannels = [];
					_.each(this._channels, function (value, key) {
						if (value) {
							convertedChannels[key] = "" + (value / 100 * area); // graphics model parsing expects strings 
						}
					}, this);

					// some extra properties
					convertedChannels.makeTotalCount = true;
					convertedChannels.unit = this.f.i18n.t(CoreConstants.EyeDropperUnitsShortLocNames[currentUnit]);
					convertedChannels.unitFix = CoreConstants.EyeDropperUnitsFixes[currentUnit];

					this.f.emit("channels.informationAdded", convertedChannels);
                    
                    this.showPopover(convertedChannels);
				}
			}
		};

		/**
		 * @override
		 *
		 * Sets the required options for the call to the viewserver
		 */
		EyeDropperMeasureTool.prototype.calculate = function () {
			var options = {
				opmask: 1
			};
			//calls getInfo in AbstactMeasureTool
			this.getInfo(options);
		};

		/**
		 * @override
		 * @returns {Boolean}
		 */
		EyeDropperMeasureTool.prototype.canBeUsedOnActivePage = function () {
			var page = this.f.getActivePage();
			return page && page.hasGraphics();
		};

		/**
		 * @override
		 * Does this tool need a total count
		 * @returns {Boolean} True
		 */
		EyeDropperMeasureTool.prototype._needsTotalCount = function () {
			return true;
		};

		/**
		 * @override
		 */
		EyeDropperMeasureTool.prototype.createPopoverModel = function (channelInfo, area) {
		    return new EyeDropperPopoverModel(this.f, area, "Channels", $("<i>").addClass("icon icon-normal icon-WCR_Measure-Colorpicker"), channelInfo, this._needsTotalCount());
		};

		return EyeDropperMeasureTool;

	});


define('tpl!web-viewer/measure/CaliperPopoverTemplate', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<table>\r\n	<tr>\r\n		<td><strong>'+
((__t=(d.endPointLabel))==null?'':_.escape(__t))+
': </strong></td>\r\n		<td>'+
((__t=(d.endPointValue))==null?'':_.escape(__t))+
'</td>\r\n	</tr>\r\n	<tr>\r\n		<td><strong>'+
((__t=(d.i18n('GapSize')))==null?'':_.escape(__t))+
': </strong></td>\r\n		<td>'+
((__t=(d.gap))==null?'':_.escape(__t))+
'</td>\r\n	</tr>\r\n</table>';
return __p;
}});
define('web-viewer/measure/CaliperPopoverModel',["web-viewer/viewport/model/ViewportPopoverModel",
		"tpl!web-viewer/measure/CaliperPopoverTemplate",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils",
		"Template",
		"web-viewer/core/CoreConstants",
		"underscore"
	],
	function (ViewportPopoverModel,
		tpl,
		ShapeUtils,
		ViewerUtils,
		Template,
		CoreConstants,
		_) {
		"use strict";

		/**
		 * PoveroverModel for the measure tools, defines the layout (uses the the MeasurePopoverTemplate) of the Popover for measure tools
		 *
		 * @exports CaliperPopoverModel
		 * @memberof MeasureModule
		 * @class CaliperPopoverModel
		 * @augments Viewport.Model.ViewportPopoverModel
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {LineworkWrapper} area
		 * @param {String} name
		 * @param {Array<Number> | Array<String>} channels
		 * @param {Boolean} [makeTotalCount] Defines if a total of the different channel values needs to be calculated, only possible if the
		 * 										values of the channels array are numbers
		 * @param {String} unit The String to be displayed behind the the values of the different channels
		 * @param {CaliperLayerModel} caliperLayerModel
		 */
		var CaliperPopoverModel = function (f, area, name) {
			ViewportPopoverModel.call(this, f, name);

			this.div = $("<div/>").addClass("measureInfo caliperPopover");

			// initialize template
			this.t = new Template(f.i18n, tpl);
			this.t.setContext(this.div);

			this.t.d.isMeasuringHeight = true;
			this._endPoint = new ShapeUtils.Point();
			this._endPointValueInMm = null;
			this._gapinMm = null;


			// load language if needed
			f.i18n.loadBundle("viewer", _.bind(function () {
				this.t.render();
			}, this));

			// listen to unit changes
			this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_SIZE", _.bind(this.recalculateGap, this));

			this.frame = null;

			this.getFrame = function () {
				if (this.frame) {
					return this.frame;
				} else {
					return new ShapeUtils.Rect(new ShapeUtils.Point(0, 0), new ShapeUtils.Size(1, 1));
				}
			};

			this.getContent = function () {
				return this.div;
			};

			this.getPreferredPosition = function () {
				// override
				return this.t.d.isMeasuringHeight ? "bottom" : "right";
			};

			this.getRestrictBorder = function () {
				return 0;
			};

			/**
			 * Is this an interactive popover
			 * @returns {Boolean} True
			 */
			this.isInteractive = function () {
				return true;
			};

			this.getContext = function () {
				return this.f.getContext().find(".viewport");
			};

			this.render = _.throttle(_.bind(function () {
				this.t.render();
			}, this), 100);

			/**
			 * Get the class for the popover
			 */
			this.getClass = function () {
				return "transparent";
			};
		};

		// inherit
		CaliperPopoverModel.prototype = Object.create(ViewportPopoverModel.prototype);
		// correct the constructor pointer 
		CaliperPopoverModel.prototype.constructor = CaliperPopoverModel;

		CaliperPopoverModel.prototype.getChannelsInformation = function () {
			return this._channelsInfo;
		};

		/**
		 * Update the popover
		 */
		CaliperPopoverModel.prototype.setGap = function (caliperGap) {
			this._gapinMm = caliperGap.getVector().getLength();
			this.t.d.gap = this._convertToCorrectUnits(this._gapinMm);
			this.render();
		};

		/**
		 * Recalculate the content of the popup (correct units) and render
		 */
		CaliperPopoverModel.prototype.recalculateGap = function () {
			if (this._gapinMm !== null && this._endPointValueInMm !== null) {
				this.t.d.endPointValue = this._convertToCorrectUnits(this._endPointValueInMm);
				this.t.d.gap = this._convertToCorrectUnits(this._gapinMm);
				this.render();
			}
		};

		CaliperPopoverModel.prototype.setEndPoint = function (endPoint) {
			this._endPoint = endPoint;
			if (this._endPoint) {
				this.t.d.endPointValue = this._convertToCorrectUnits(this.t.d.isMeasuringHeight ? this._endPoint.y : this._endPoint.x);
			}
		};

		CaliperPopoverModel.prototype._convertToCorrectUnits = function (value) {
			var resolution = null;
			if (this.f.getUnits() === CoreConstants.SizeUnits.PIXELS) {
				var page = this.f.getActivePage();
				if (this.t.d.isMeasuringHeight) {
					resolution = page.getParam("resolutiony");
				} else {
					resolution = page.getParam("resolutionx");
				}
			}
			var converted = ViewerUtils.convertFromMmValue(value, this.f.getUnits(), this.f.getUnitsFix(), ViewerUtils.parseFloat(resolution));
			var units = this.f.i18n.t(this.f.getUnitsLocName());

			return converted + " " + units;
		};

		/**
		 * Set the area of the popover
		 * @param {ShapeUtils.Rect} area for which the popover should be shown
		 */
		CaliperPopoverModel.prototype.setFrame = function (frame) {
			this.frame = frame;
		};


		/**
		 * Change the mode of the popover
		 * @param isMeasuringHeight
		 */
		CaliperPopoverModel.prototype.setMeasuringHeight = function (isMeasuringHeight) {
			this.t.d.isMeasuringHeight = isMeasuringHeight;
			if (this._endPoint) {
				if (this.t.d.isMeasuringHeight) {
					this.t.d.endPointLabel = "Y";
					// convert to correct coordinate system
					var endPointY = this._endPoint.copy();
					var transformedEndPointY = this._coordinates.getTransformedPoint(endPointY);
					this.t.d.endPointValue = this._convertToCorrectUnits(transformedEndPointY.y);
					this._endPointValueInMm = transformedEndPointY.y;
				} else {
					this.t.d.endPointLabel = "X";
					// convert to correct coordinate system
					var endPointX = this._endPoint.copy();
					var transformedEndPointX = this._coordinates.getTransformedPoint(endPointX);
					this.t.d.endPointValue = this._convertToCorrectUnits(transformedEndPointX.x);
					this._endPointValueInMm = transformedEndPointX.x;
				}
			}
			this.render();
		};
		return CaliperPopoverModel;
	});

define('web-viewer/measure/CaliperLayerModel',["underscore",
		"web-viewer/viewport/model/ViewportLayerModel",
		"web-viewer/measure/CaliperPopoverModel",
		"web-viewer/utils/DrawUtils",
		"utils/ShapeUtils",
		"web-viewer/utils/LineworkWrapper",
		"web-viewer/utils/LineworkPoint",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/measure/MeasureConstants"
	],
	function (_,
		ViewportLayerModel,
		CaliperPopoverModel,
		DrawUtils,
		ShapeUtils,
		LineworkWrapper,
		LineworkPoint,
		ListenerUtil,
		MeasureConstants
	) {
		"use strict";
		/**
		 * @exports CaliperLayerModel
		 * @memberof MeasureModule
		 * @augments ViewportLayerModel
		 * @mixes {Utils.ListenerUtil}
		 * @class This class gives the linework to render for the caliper tool.
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {String} ID - the ID of the layer model
		 * @param {CaliperModel} caliperModel - Model of the caliper tool
		 */
		var CaliperLayerModel = function (f, ID, caliperModel) {
			ViewportLayerModel.call(this, f, ID);
			ListenerUtil.mixin(this, f);

			// Variables
			this._caliperModel = caliperModel;
			this._caliperPopover = null;
			this._popoverFrame = new ShapeUtils.Rect();
			/**
			 * {ShapeUtils.Point}
			 * The start point in world coordinates (so in mm).
			 * The end point/line is calculated as start point/line + gap of the current gap in the caliper model.
			 */
			this._startPoint = null;

			// Listen to the model
			this._registerListener("changed.currentGap", this.modelChanged, this, this._caliperModel);
			this._registerListener("changed.currentGap.vector", this.modelChanged, this, this._caliperModel);
		};

		// inherit
		CaliperLayerModel.prototype = Object.create(ViewportLayerModel.prototype);
		// correct the constructor pointer
		CaliperLayerModel.prototype.constructor = CaliperLayerModel;

		/**
		 * Set a new start point
		 * @param {ShapeUtils.Point} startPoint - The new start point
		 */
		CaliperLayerModel.prototype.setStartPoint = function (startPoint) {
			this._startPoint = startPoint;
			this.modelChanged();
		};

		/**
		 * Get the current start point
		 * @returns {ShapeUtils.Point}
		 */
		CaliperLayerModel.prototype.getStartPoint = function () {
			return this._startPoint;
		};

		/**
		 * @override
		 */
		CaliperLayerModel.prototype.getLinework = function (animated, rotationanimation, drawrect) {
			if (this._startPoint) {
				var startLine = new LineworkWrapper(); // first line
				var endLine = new LineworkWrapper(); // second line
				var beginRect = new LineworkWrapper(); // rect before start line
				var endRect = new LineworkWrapper(); // rect after end line	

				var vector = this._caliperModel.getCurrentGap().getVector();
				var gap = vector.getLength();
				var angle = vector.getAngleInDegrees();

				if (this._caliperModel.isMeasuringHeight(vector)) {
					// measuring heights

					if (angle === 0 || angle === 90) {
						// vector pointed to the top but we are drawing from top,left
						gap = -gap;
					}

					// upper rect
					var upperRect = new ShapeUtils.Rect(new ShapeUtils.Point(drawrect.left(), drawrect.top()), new ShapeUtils.Size(drawrect.right() - drawrect.left(), Math.min(this._startPoint.y, this._startPoint.y + gap) - drawrect.top()));
					upperRect.normalize();
					beginRect.rect = upperRect;

					// lower rect
					var max = Math.max(this._startPoint.y, this._startPoint.y + gap);
					var lowerRect = new ShapeUtils.Rect(new ShapeUtils.Point(drawrect.left(), max), new ShapeUtils.Size(drawrect.right() - drawrect.left(), drawrect.bottom() - max));
					lowerRect.normalize();
					endRect.rect = lowerRect;

					// first line
					var startLineBegin = new LineworkPoint();
					startLineBegin.makeMoveTo(drawrect.left(), this._startPoint.y);
					startLine.pointArray.push(startLineBegin);

					var startLineEnd = new LineworkPoint();
					startLineEnd.makeLineTo(drawrect.right(), this._startPoint.y);
					startLine.pointArray.push(startLineEnd);

					// second line
					var endLineBegin = new LineworkPoint();
					endLineBegin.makeMoveTo(drawrect.left(), this._startPoint.y + gap);
					endLine.pointArray.push(endLineBegin);

					var endLineEnd = new LineworkPoint();
					endLineEnd.makeLineTo(drawrect.right(), this._startPoint.y + gap);
					endLine.pointArray.push(endLineEnd);

					this._popoverFrame.origin.x = drawrect.left();
					this._popoverFrame.origin.y = this._startPoint.y + gap;
					this._popoverFrame.size.width = drawrect.size.width;
					this._popoverFrame.size.height = 0.00000001;

				} else if (this._caliperModel.isMeasuringWidth(vector)) {
					// measuring widths

					if (angle === 180) {
						// pointed to the left
						gap = -gap;
					}

					// left rect
					var leftRect = new ShapeUtils.Rect(new ShapeUtils.Point(drawrect.left(), drawrect.top()), new ShapeUtils.Size(Math.min(this._startPoint.x, this._startPoint.x + gap) - drawrect.left(), drawrect.bottom() - drawrect.top()));
					leftRect.normalize();
					beginRect.rect = leftRect;

					// right rect
					var max = Math.max(this._startPoint.x, this._startPoint.x + gap);
					var rightRect = new ShapeUtils.Rect(new ShapeUtils.Point(max, drawrect.top()), new ShapeUtils.Size(drawrect.right() - max, drawrect.bottom() - drawrect.top()));
					rightRect.normalize();
					endRect.rect = rightRect;

					// first line
					var startLineBegin = new LineworkPoint();
					startLineBegin.makeMoveTo(this._startPoint.x, drawrect.top());
					startLine.pointArray.push(startLineBegin);

					var startLineEnd = new LineworkPoint();
					startLineEnd.makeLineTo(this._startPoint.x, drawrect.bottom());
					startLine.pointArray.push(startLineEnd);

					// second line
					var endLineBegin = new LineworkPoint();
					endLineBegin.makeMoveTo(this._startPoint.x + gap, drawrect.top());
					endLine.pointArray.push(endLineBegin);

					var endLineEnd = new LineworkPoint();
					endLineEnd.makeLineTo(this._startPoint.x + gap, drawrect.bottom());
					endLine.pointArray.push(endLineEnd);

					this._popoverFrame.origin.x = this._startPoint.x + gap;
					this._popoverFrame.origin.y = drawrect.top();
					this._popoverFrame.size.width = 0.00000001;
					this._popoverFrame.size.height = drawrect.size.height;

				}

				// Properties
				startLine.dash = [4, 3];
				startLine.color = "rgba(210,210,210,0.9)";
				startLine.type = DrawUtils.LWTYPE_LINE;

				endLine.dash = [4, 3];
				endLine.color = "rgba(210,210,210,0.9)";
				endLine.type = DrawUtils.LWTYPE_LINE;

				beginRect.type = DrawUtils.LWTYPE_RECT;
				beginRect.fillcolor = "rgba(0, 0, 0, 0.40)";
				beginRect.color = "rgba(0, 0, 0, 1.00)";
				beginRect.linewidth = 1;

				endRect.type = DrawUtils.LWTYPE_RECT;
				endRect.fillcolor = "rgba(0, 0, 0, 0.40)";
				endRect.color = 'rgba(0, 0, 0, 1.00)';
				endRect.linewidth = 1;

				return [beginRect, endRect, startLine, endLine];
			}
		};

		/**
		 * Get the end point.
		 * @returns {ShapeUtils.Point} - The point on the end line as start point + gap (vector)
		 */
		CaliperLayerModel.prototype.getEndPoint = function () {
			if (this._startPoint) {
				var vector = this._caliperModel.getCurrentGap().getVector();
				var gap = vector.getLength();
				var angle = vector.getAngleInDegrees();

				// calculate end point
				if (this._caliperModel.isMeasuringHeight(vector)) {
					// measuring heights
					if (angle === 0 || angle === 90) {
						// pointed to the top but we are drawing from top,left
						gap = -gap;
					}
					return new ShapeUtils.Point(this._startPoint.x, this._startPoint.y + gap);
				} else if (this._caliperModel.isMeasuringWidth(vector)) {
					// measuring widths

					if (angle === 180) {
						// pointed to the left
						gap = -gap;
					}
					return new ShapeUtils.Point(this._startPoint.x + gap, this._startPoint.y);
				}
			} else {
				return null;
			}
		};

		/**
		 * Set new start and end point.
		 * Either start point can be changed or end point (other one given by the getEndPoint and getStartPoint methods), but not both
		 * @param {ShapeUtils.Point} startPoint New start point in world coordinates
		 * @param {ShapeUtils.Point} endPoint - New end point in world coordinates
		 * @param {Boolean} keepVector - True to keep the current vector (e.g. for fixed gaps)
		 * @param {Boolean} [keepAngle] - if true, the angle won;t be changed
		 */
		CaliperLayerModel.prototype.setPoints = function (startPoint, endPoint, keepVector, keepAngle) {
			var oldStartPoint = this.getStartPoint();
			var oldEndPoint = this.getEndPoint();
			var currentVector = this._caliperModel.getCurrentGap().getVector();

			if (!keepVector) {
				// Vector can be changed (e.g. for freeform, depending on the largest x/y distance)
				var angle = 0;
				var gap = 0;

				var distX = endPoint.x - startPoint.x;
				var distY = endPoint.y - startPoint.y;
				var distXAbs = Math.abs(distX);
				var distYAbs = Math.abs(distY);
				var shouldMeasureWidth = distXAbs > distYAbs;


				if (keepAngle) {
					shouldMeasureWidth = this._caliperModel.isMeasuringWidth(currentVector);
				}

				if (shouldMeasureWidth) {
					// we are measuring widths

					// change orientation if needed
					if (distX < 0) {
						// to the left of the begin point
						angle = 180;
					} else {
						// to the right of the begin point
						angle = 0;
					}
					gap = distXAbs;
				} else {
					// we are measuring heights

					// change orientation if needed
					if (distY < 0) {
						// more to the top than begin point
						angle = 90;
					} else {
						// more to the bottom than begin point
						angle = -90;
					}
					gap = distYAbs;
				}
				currentVector.setAngleInDegrees(angle);
				currentVector.setLength(gap);

			} else {
				// Vector is kept
				// Check if the end point changed -> change start point accordingly
				var distEndY = endPoint.y - oldEndPoint.y;
				var distEndX = endPoint.x - oldEndPoint.x;

				if (distEndY !== 0) {
					startPoint = new ShapeUtils.Point(oldStartPoint.x, oldStartPoint.y + distEndY);
				} else if (distEndX !== 0) {
					startPoint = new ShapeUtils.Point(oldStartPoint.x + distEndX, oldStartPoint.y);
				}
			}

			// start point changed or not?
			if (!startPoint.equals(oldStartPoint)) {
				this._startPoint = startPoint;
			}

			this._caliperModel.setCurrentGapVector(currentVector);
			this.modelChanged();
		};


		/**
		 * Start showing the caliper popover
		 */
		CaliperLayerModel.prototype.showPopover = function () {
			if (!this._caliperPopover) {
				this._caliperPopover = new CaliperPopoverModel(this.f, null, "CaliperPopover", this);
				this.modelChanged();

				this.addPopoverModel(this._caliperPopover);
			}
		};

		/**
		 * Hide the caliper popover
		 */
		CaliperLayerModel.prototype.hidePopover = function () {
			if (this._caliperPopover) {
				this.removePopoverModel(this._caliperPopover.getID());
				this.removePopover(this._caliperPopover.getID());
				this._caliperPopover = null;
			}
		};

		/**
		 * Override the modelchanged, so the popovermodel is updated when needed. Also sends out current selection.
		 * @param {Boolean} deactivate - True when tool using this model gets deactivated (see CaliperTool)
		 * @override
		 */
		CaliperLayerModel.prototype.modelChanged = function (deactivate) {
			var selectionObject = null;

			// deactivate can also be other variables, so when it is other variable,or when it is a boolean and it is false (= activate) execute block 
			if (!_.isBoolean(deactivate) || (_.isBoolean(deactivate) && !deactivate)) {
				// first update the popover
				if (this._caliperPopover) {
					this._caliperPopover.setGap(this._caliperModel.getCurrentGap());
					this._caliperPopover.setEndPoint(this.getEndPoint());
					this._caliperPopover.setMeasuringHeight(this._caliperModel.isMeasuringHeight(this._caliperModel.getCurrentGap().getVector()));
					this._caliperPopover.setFrame(this._popoverFrame);
				}

				// created updated updated selection object
				if (this._startPoint && this._caliperModel.getCurrentGap() && this._caliperModel.getCurrentGap().getVector().getLength() !== 0) {
					if (this._caliperModel.isMeasuringHeight(this._caliperModel.getCurrentGap().getVector())) {
						selectionObject = {
							mode: MeasureConstants.MeasureDrawType.HEIGHT,
							y: this._startPoint.y,
							h: this._caliperModel.getCurrentGap().getVector().getLength()
						};
					} else {
						selectionObject = {
							mode: MeasureConstants.MeasureDrawType.WIDTH,
							x: this._startPoint.x,
							w: this._caliperModel.getCurrentGap().getVector().getLength()
						};
					}
				}
			}

			// structure of selectionObject: see {DocumentInfoModel.setSelection}
			this.f.emit("selection.changed", selectionObject);

			// make sure the normal model changed behavior is still executed
			ViewportLayerModel.prototype.modelChanged.call(this);
		};

		return CaliperLayerModel;
	});

define('web-viewer/measure/CaliperTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"utils/ShapeUtils",
		"web-viewer/measure/CaliperLayerModel",
		"web-viewer/measure/MeasureConstants",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractTool,
		ShapeUtils,
		CaliperLayerModel,
		MeasureConstants,
		CoreConstants) {
		"use strict";
		/**
		 * @exports CaliperTool
		 * @memberof MeasureModule
		 * @class The Caliper tool allows to measure widths/heights
		 * @constructor
		 * @augments {Core.AbstractTool}
		 *
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {CaliperModel} caliperModel - The model of the caliper tool
		 */
		var CaliperTool = function (f, caliperModel) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.MEASURE_CALIPER);

			// the distance (in pixels) of a line for going to the hover state
			this.CLOSETODISTANCE = 16;

			this._caliperModel = caliperModel;
			this._caliperLayerModel = new CaliperLayerModel(f, this.getIID(), caliperModel);

			/**
			 * DEFAULT STATE -> nothing set
			 */
			this.registerState("default", {
				enterState: _.bind(function () {
					this._caliperLayerModel.hidePopover();
					this._caliperLayerModel.setStartPoint(null);
					this._caliperModel.resetFreeformVector();
				}, this),

				mousedown: _.bind(function (e) {
					var startPoint = this.keepPointWithinWorldBounds(this.getWorldPoint(e), false);
					this._caliperLayerModel.setStartPoint(startPoint);
					if (!this._caliperLayerModel.getEndPoint()) {
						// if the end point is not defined yet, move the endline to the start point (in this way we prevent some flickering in the UI)
						this._moveEndLine(e);
					}
					this.changeState("dragging");
				}, this),

				mouseup: function () {},

				mousemove: function () {},

				mouseout: function () {},

				mouseleave: function () {},

				/**
				 * no context menu!
				 */
				contextmenu: _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
				}, this),

				canBeInterrupted: true
			});

			/**
			 * DRAGGING STATE -> after setting start point: end line can be set (freeform), or a new start point can be set (fixed)
			 */
			this.registerState("dragging", _.defaults({
				enterState: _.bind(function () {
					this._caliperLayerModel.showPopover();
					this.f.emit("viewport.startInteractive");
				}, this),
				leaveState: _.bind(function () {
					this.f.emit("viewport.stopInteractive");
				}, this),
				mousedown: function () {},
				mouseup: _.bind(function () {
					var currentGap = this._caliperModel.getCurrentGap();
					var currentStyle = currentGap.getStyle();
					if (currentStyle === MeasureConstants.CaliperStyle.Fixed || currentStyle === MeasureConstants.CaliperStyle.Preset || currentGap.getVector().getLength() > 0) {
						this.changeState("drawn");
					}
				}, this),

				mousemove: _.bind(function (e) {
					var currentGap = this._caliperModel.getCurrentGap();
					var currentStyle = currentGap.getStyle();
					if (currentStyle === MeasureConstants.CaliperStyle.Fixed || currentStyle === MeasureConstants.CaliperStyle.Preset) {
						this._moveStartLine(e);
					} else {
						this._moveEndLine(e);
					}
				}, this),

				canBeInterrupted: false
			}, this.getState("default")));

			/**
			 * DRAWN STATE -> end point also set
			 */
			this.registerState("drawn", _.defaults({
				enterState: function () {},
				mousemove: _.bind(function (e) {
					if (this._isHovering(e)) {
						this.changeState("hover");
					}
				}, this),

				canBeInterrupted: true
			}, this.getState("default")));

			/**
			 * HOVER STATE -> hover over a line
			 */
			this.registerState("hover", _.defaults({
				enterState: function () {},
				mousedown: _.bind(function (e) {
					var worldPoint = this.getWorldPoint(e);
					var startPoint = this._caliperLayerModel.getStartPoint();
					var endPoint = this._caliperLayerModel.getEndPoint();

					if (this._isCloseToLine(startPoint, worldPoint)) {
						// close to start line -> move start line
						this.changeState("movingStartLine");
					} else if (this._isCloseToLine(endPoint, worldPoint)) {
						// close to end line -> move end line
						this.changeState("movingEndLine");
					}
				}, this),

				mousemove: _.bind(function (e) {
					if (!this._isHovering(e)) {
						this.changeState("drawn");
					}
				}, this),

				mouseleave: _.bind(function (e) {
					this.changeState("drawn");
				}, this),

				canBeInterrupted: true
			}, this.getState("default")));

			/**
			 * MOVE END LINE STATE -> end line is being moved
			 */
			this.registerState("movingEndLine", _.defaults({
				enterState: _.bind(function () {
					this.f.emit("viewport.startInteractive");
				}, this),
				leaveState: _.bind(function () {
					this.f.emit("viewport.stopInteractive");
				}, this),
				mouseleave: _.bind(function (e) {
					this.changeState("drawn");
				}, this),

				mouseup: _.bind(function (e) {
					this.changeState("drawn");
				}, this),

				mousemove: _.bind(function (e) {
					this._moveEndLine(e, true);
				}, this),

				canBeInterrupted: false
			}, this.getState("default")));

			/**
			 * MOVE START LINE STATE -> start line is being moved
			 */
			this.registerState("movingStartLine", _.defaults({
				enterState: _.bind(function () {
					this.f.emit("viewport.startInteractive");
				}, this),
				leaveState: _.bind(function () {
					this.f.emit("viewport.stopInteractive");
				}, this),
				mouseleave: _.bind(function (e) {
					this.changeState("drawn");
				}, this),

				mouseup: _.bind(function (e) {
					this.changeState("drawn");
				}, this),

				mousemove: _.bind(function (e) {
					this._moveStartLine(e);
				}, this),

				canBeInterrupted: false
			}, this.getState("default")));

			this.registerActiveEvents(["mousedown", "mousemove", "mouseup", "mouseleave", "contextmenu"]);

			// Esc allows to reset selection
			this.activeKeyEvents.push(["esc", _.bind(function (e) {
				this.changeState("default");
				e.preventDefault();
				e.stopPropagation();
			}, this), "keyup"]);


			/**
			 * @override
			 */
			this._override("initialize", function (widget, toolManager) {
				this._super(widget, toolManager);
				this.changeState("default");
			});

			/**
			 * @override
			 */
			this._override("activate", function (interruptRequest) {
				this._super(interruptRequest);
				if (!interruptRequest) {
					this.f.getModule("viewport").addLayer(this.getIID(), this._caliperLayerModel);
				}
			});

			/**
			 * @override
			 */
			this._override("deactivate", function (interruptRequest) {
				this._super(interruptRequest);
				if (!interruptRequest) {
					this._caliperLayerModel.modelChanged(true);
					this.f.getModule("viewport").removeLayer(this.getIID());
				}

				if (!interruptRequest || _.contains(["movingStartLine", "movingEndLine", "dragging"], this.getCurrentStateName())) {
					this.changeState("default");
				}
			});
		};

		// extend from the AbstractTool
		// inherit
		CaliperTool.prototype = Object.create(AbstractTool.prototype);
		CaliperTool.prototype.constructor = CaliperTool;

		/**
		 * Move the end line to the new point given by e
		 * @param {EventObj} e
		 * @param {Boolean} [keepAngle] - if the true, the angle of te vector is not changed/altered by moving the endline
		 */
		CaliperTool.prototype._moveEndLine = function (e, keepAngle) {
			var currentStyle = this._caliperModel.getCurrentGap().getStyle();
			var newPoint = this.keepPointWithinWorldBounds(this.getWorldPoint(e), false);
			var startPoint = this._caliperLayerModel.getStartPoint();
			if (currentStyle === MeasureConstants.CaliperStyle.Freeform) {
				this._caliperLayerModel.setPoints(startPoint, newPoint, false, keepAngle);
			} else if (currentStyle === MeasureConstants.CaliperStyle.Fixed || currentStyle === MeasureConstants.CaliperStyle.Preset) {
				this._caliperLayerModel.setPoints(startPoint, newPoint, true, keepAngle);
			}
		};

		/**
		 * Move the start line to the new point given by e
		 * @param {Event} e - mouse event
		 */
		CaliperTool.prototype._moveStartLine = function (e) {
			var currentStyle = this._caliperModel.getCurrentGap().getStyle();
			var newPoint = this.keepPointWithinWorldBounds(this.getWorldPoint(e), false);
			if (currentStyle === MeasureConstants.CaliperStyle.Freeform) {
				var endPoint = this._caliperLayerModel.getEndPoint();
				this._caliperLayerModel.setPoints(newPoint, endPoint, false, true);
			} else if (currentStyle === MeasureConstants.CaliperStyle.Fixed || currentStyle === MeasureConstants.CaliperStyle.Preset) {
				this._caliperLayerModel.setStartPoint(newPoint);
			}
		};

		/**
		 * Check if hovered over begin or end line by point given by e
		 * Also sets the cursor correctly.
		 * @param {EventObj} e
		 * @returns {Boolean} True if hovering over begin or end line, false otherwise
		 */
		CaliperTool.prototype._isHovering = function (e) {
			var worldPoint = this.getWorldPoint(e);
			var startPoint = this._caliperLayerModel.getStartPoint();
			var endPoint = this._caliperLayerModel.getEndPoint();

			if (this._isCloseToLine(startPoint, worldPoint) || this._isCloseToLine(endPoint, worldPoint)) {
				var rotation = this._widget.getWorld().getRotation();
				if (this._caliperModel.isMeasuringHeight(this._caliperModel.getCurrentGap().getVector())) {
					if (rotation % 180 === 90) {
						this.setCursor("cursor-resizeE");
					} else {
						this.setCursor("cursor-resizeN");
					}
				} else if (this._caliperModel.isMeasuringWidth(this._caliperModel.getCurrentGap().getVector())) {
					if (rotation % 180 === 90) {
						this.setCursor("cursor-resizeN");
					} else {
						this.setCursor("cursor-resizeE");
					}
				}
				return true;
			} else {
				this.setCursor(this.getDefaultCursor());
				return false;
			}
		};


		/**
		 * Check if the given worldPoint is close to the line through worldStartorEndPoint according to measuring heights/widths
		 * @param {ShapeUtils.Point} startOrEndPoint - world start or end point
		 * @param {ShapeUtils.Point} worldPoint - world point to check
		 * @returns {Boolean} True if the worldPoint is close to the line through worldStartorEndPoint according to the current gap vector, false otherwise
		 */
		CaliperTool.prototype._isCloseToLine = function (worldStartorEndPoint, worldPoint) {
			var zoom = this._widget.getWorld().getZoom();
			var dist = Math.abs(this._getDistanceToLine(worldStartorEndPoint, worldPoint));
			return dist < this.CLOSETODISTANCE / zoom;
		};


		/**
		 * Get the distance of the worldPoint to the line through worldStartorEndPoint according to measuring heights/widths
		 * @param {ShapeUtils} worldStartorEndPoint - world start or end point
		 * @param {ShapeUtils} worldPoint - world point to check
		 * @returns {Number} distance of the worldPoint to the line through worldStartorEndPoint according to measuring heights/widths
		 */
		CaliperTool.prototype._getDistanceToLine = function (worldStartorEndPoint, worldPoint) {
			var vector = this._caliperModel.getCurrentGap().getVector();
			if (this._caliperModel.isMeasuringHeight(vector)) {
				return worldStartorEndPoint.y - worldPoint.y;
			} else if (this._caliperModel.isMeasuringWidth(vector)) {
				return worldStartorEndPoint.x - worldPoint.x;
			}
		};

		/**
		 * @override
		 */
		CaliperTool.prototype.canBeInterrupted = function () {
			return this.getCurrentState().canBeInterrupted;
		};

		/**
		 * Get the caliper model. Used to give it to the sub tool bar component
		 * @returns {CaliperModel} The caliper model of the tool.
		 */
		CaliperTool.prototype.getCaliperModel = function () {
			return this._caliperModel;
		};

		/**
		 * @see AbstractMeasureTool or MeasureModule
		 * Can the tool be used on the currently active page?
		 *
		 * @returns {Boolean}
		 */
		CaliperTool.prototype.canBeUsedOnActivePage = function () {
			// can be used on all active pages
			return true;
		};

		/**
		 * @see AbstractMeasureTool or MeasureModule
		 * Update the data in the tool if needed
		 *
		 * @returns {Boolean}
		 */
		CaliperTool.prototype.updateIfNeeded = function (forceUpdate) {
			// do nothing
		};

		/**
		 * @override
		 */
		CaliperTool.prototype.getDefaultCursor = function () {
			return "rulerCursor";
		};
		/**
		 * Override, so activating the tool triggers a model update
		 * @override
		 */
		CaliperTool.prototype.doActivation = function () {
			this._caliperLayerModel.modelChanged(false);
			AbstractTool.prototype.doActivation.call(this);
		};


		/**
		 * Get the state of the measure tool
		 * @see AbstractMeasureTool
		 * @returns {Object}
		 */
		CaliperTool.prototype.getToolState = function () {
			var currentGap = this._caliperModel.getCurrentGap();
			var state = {
				// We cannot rely on the ID being right because of random ordering, so we don't save the gap, but rather the style and vector separately and search it back in the set state method
				caliperStyle: currentGap.getStyle(),
				caliperGapVector: currentGap.getVector(),
				startPoint: this._caliperLayerModel.getStartPoint(),
				isActive: this.isActive()
			};
			return state;
		};

		/**
		 * Set the state of the measure tool
		 * @see AbstractMeasureTool
		 * @param {Object} state
		 */
		CaliperTool.prototype.setToolState = function (state) {
			if (state.isActive) {
				// first activate tool, and then disable interruption
				this.f.emit("toolManager.activateTool", this);
				this.f.emit("toolManager.interruptTools", true);
				var style = state.caliperStyle;
				var vector = state.caliperGapVector;
				if (style !== undefined && vector !== undefined && state.startPoint !== undefined) {
					// Find back the right caliper gap
					// We cannot rely on the ID being right because of random ordering, so we search it ourself in the model

					// Set the gap by id and vector
					var setGap = _.bind(function (id, vector) {
						this._caliperModel.setCurrentGapById(id);
						this._caliperModel.setCurrentGapVector(vector);
					}, this);

					if (style === MeasureConstants.CaliperStyle.Freeform) {
						// freeform
						var caliperGap = this._caliperModel.getFreeformGap();
						setGap(caliperGap.getId(), vector);
					} else {
						// fixed or preset
						var caliperGap = this._caliperModel.getFixedGap();
						setGap(caliperGap.getId(), vector);

						// first set fixed, then find right similar preset
						if (style === MeasureConstants.CaliperStyle.Preset) {
							var preset = this._caliperModel.getSimilarPreset(this._caliperModel.getCurrentGap());
							if (preset !== undefined) {
								// set to the right preset gap
								this._caliperModel.setCurrentGapById(preset.getId());
							}
						}
					}

					// Set start point
					this._caliperLayerModel.setStartPoint(state.startPoint);
					// Show popover
					this._caliperLayerModel.showPopover();
				}
			}
		};

		return CaliperTool;
	});

define('web-viewer/measure/CaliperGap',["utils/ShapeUtils",
		"underscore"
	],
	function (ShapeUtils,
		_) {
		"use strict";
		/**
		 * @exports CaliperGap
		 * @memberof MeasureModule
		 * @class A caliper gap is used to hold information about a gap of the caliper tool. e.g. the angle and gap size (represented by a vector)
		 * @constructor
		 * @param {String} id - Style
		 * @param {MeasureConstants.CaliperStyle} style - Style of the gap (e.g. freeform)
		 * @param {ShapeUtils.Vector} vector - The vector representing the gap (angle and gap size)
		 */
		var CaliperGap = function (id, style, vector) {
			/**
			 * {String} Id of the gap
			 */
			this._id = id;
			/**
			 * {MeasureConstants.CaliperStyle} Style of the gap
			 */
			this._style = style;

			/**
			 * {ShapeUtils.Vector} Vector representing the angle and size of the gap
			 */
			this._vector = vector;
		};

		/**
		 * Get the id of the gap
		 * @returns {String}
		 */
		CaliperGap.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Get the style of the gap
		 * @returns {MeasureConstants.CaliperStyle}
		 */
		CaliperGap.prototype.getStyle = function () {
			return this._style;
		};

		/**
		 * Get the vector of the gap
		 * It always returns a copy!
		 * @returns {ShapeUtils.Vector}
		 */
		CaliperGap.prototype.getVector = function () {
			return this._vector.copy();
		};

		/**
		 * Set a new vector for the gap
		 * @param {ShapeUtils.Vector} vector - The new vector for this gap
		 */
		CaliperGap.prototype.setVector = function (vector) {
			this._vector = vector;
		};

		/**
		 * Check whether the given gap is similar to this gap.
		 * Similar means the same gap and orientation, so the same vector.
		 * @param {CaliperGap} similarGap - The gap to check with this gap
		 * @returns {Boolean} Returns true if the given similarGap is similar to this gap, false otherwise
		 */
		CaliperGap.prototype.isSimilarTo = function (similarGap) {
			return this._vector.equals(similarGap.getVector());
		};

		return CaliperGap;
	});

define('web-viewer/measure/CaliperModel',["web-viewer/core/AbstractModel",
		"web-viewer/measure/CaliperGap",
		"web-viewer/measure/MeasureConstants",
		"utils/ShapeUtils",
		"underscore"
	],
	function (AbstractModel,
		CaliperGap,
		MeasureConstants,
		ShapeUtils,
		_) {
		"use strict";

		/**
		 * @exports CaliperModel
		 * @memberof MeasureModule
		 * @class The Caliper model is used for representing all caliper tool gap info
		 * @constructor
		 * @augments {Core.AbstractModel}
		 *
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {String} privateChannelName - The privateChannelName of the model
		 */
		var CaliperModel = function (f, privateChannelName) {
			AbstractModel.call(this, f, privateChannelName);

			// ids of the different gaps (for this session only)
			// These ids are not saved to the view server. They are just used to identify the different gaps.
			this._nextId = 0;

			// default freeform vector
			this._freeformVector = new ShapeUtils.Vector(0, 0, -Math.PI / 2); // a zero vector with an angle of 90 degrees 
			this._freeform = new CaliperGap(this._getNextId(), MeasureConstants.CaliperStyle.Freeform, this._freeformVector);

			// default fixed vector
			var fixedVector = new ShapeUtils.Vector(0, 0.8);
			this._fixed = new CaliperGap(this._getNextId(), MeasureConstants.CaliperStyle.Fixed, fixedVector);

			// array of CaliperGaps
			this._presetGaps = [];
			this._readPresets(); // read them from the view server

			// current active gap
			this._currentGap = null;
		};

		CaliperModel.prototype = Object.create(AbstractModel.prototype);
		CaliperModel.prototype.constructor = CaliperModel;

		/**
		 * Reset the vector of the current caliper gap if it's freeform to its default value
		 */
		CaliperModel.prototype.resetFreeformVector = function () {
			var currentGap = this.getCurrentGap();
			if (currentGap && currentGap.getStyle() === MeasureConstants.CaliperStyle.Freeform) {
				// a freeform selection is active, reset the vector
				this.setCurrentGapVector(this._freeformVector);
			}
		};

		/**
		 * Get the next id.
		 * @returns {String} The next id
		 */
		CaliperModel.prototype._getNextId = function () {
			var id = this._nextId;
			this._nextId++;
			return "" + id;
		};

		/**
		 * Get the freeform gap
		 * @returns {CaliperGap} The freeform gap.
		 */
		CaliperModel.prototype.getFreeformGap = function () {
			return this._freeform;
		};

		/**
		 * Get the fixed gap
		 * @returns {CaliperGap}
		 */
		CaliperModel.prototype.getFixedGap = function () {
			return this._fixed;
		};

		/**
		 * Get all gaps, including freeform, fixed and all presets.
		 * @returns {Array<CaliperGap>} Array of all gaps.
		 */
		CaliperModel.prototype.getAllGaps = function () {
			var result = [this._freeform, this._fixed];
			var presets = _.sortBy(this._presetGaps, function (presetGap) {
				// first sort by angle, then by length
				return Math.abs(presetGap.getVector().getAngleInDegrees()) * 10000 + presetGap.getVector().getLength();
			});

			return _.union(result, presets);
		};

		/**
		 * Get the current active gap.
		 * @returns {CaliperGap} Current active gap
		 */
		CaliperModel.prototype.getCurrentGap = function () {
			return this._currentGap;
		};

		/**
		 * Get a gap by id
		 * @param {String} id - Id of the gap
		 * @returns {CaliperGap} The caliper gap with the given id or undefined if not found.
		 */
		CaliperModel.prototype.getGapById = function (id) {
			return _.find(this.getAllGaps(), function (gap) {
				return gap.getId() === id;
			});
		};

		/**
		 * Set the current gap by id
		 * @param {String} id - Id of the gap to set
		 */
		CaliperModel.prototype.setCurrentGapById = function (id) {
			var gap = this.getGapById(id);
			if (gap) {
				var oldGap = this._currentGap;
				this._currentGap = gap;
				this._fireChange(false, ["currentGap"], oldGap, this._currentGap);
			} else {
				this.f.debug.error("Gap with id " + id + " not found");
			}
		};

		/**
		 * Set the vector of the current active gap.
		 * @param {ShapeUtils.Vector} newGapVector - The new gap vector
		 */
		CaliperModel.prototype.setCurrentGapVector = function (newGapVector) {
			var oldVector = this._currentGap.getVector();
			this._currentGap.setVector(newGapVector);
			this._fireChange(false, ["currentGap", "vector"], oldVector, newGapVector);
		};

		/**
		 * Save the current active gap as preset to the view server.
		 */
		CaliperModel.prototype.saveCurrentGapAsPreset = function () {
			var newFixedGap = new CaliperGap(this._getNextId(), MeasureConstants.CaliperStyle.Preset, this._currentGap.getVector());
			this._presetGaps.push(newFixedGap);

			this._persistPresets();

			this._fireChange(false, ["presetGaps", "new"], null, newFixedGap);
		};

		/**
		 * Read the presets from the view server and put them in presetGaps.
		 */
		CaliperModel.prototype._readPresets = function () {
			var vectorsString = this.f.getOption("CALIPER_PRESETS");
			if (vectorsString) {
				var vectors = JSON.parse(vectorsString);
				this._presetGaps = _.map(vectors, function (vector) {
					return new CaliperGap(this._getNextId(), MeasureConstants.CaliperStyle.Preset, new ShapeUtils.Vector(vector.x, vector.y));
				}, this);
			}
		};

		/**
		 * Persist the presets to the view server.
		 */
		CaliperModel.prototype._persistPresets = function () {
			// only vectors
			var vectors = _.map(this._presetGaps, function (gap) {
				return gap.getVector();
			});
			var jsonnedPresets = JSON.stringify(vectors);
			this.f.setOption("CALIPER_PRESETS", jsonnedPresets);
		};

		/**
		 * Delete the current active gap from the presets.
		 */
		CaliperModel.prototype.deleteCurrentGapFromPresets = function () {
			if (this._currentGap.getStyle() !== MeasureConstants.CaliperStyle.Preset) {
				this.f.debug.error("Cannot delete a non-preset style");
			}
			this._presetGaps = _.without(this._presetGaps, this._currentGap);

			this._persistPresets();

			this._fireChange(false, ["presetGaps", "deleted"], this._currentGap, null);
		};

		/**
		 * Check whether a preset with the same vector as similarPreset already exists.
		 * @param {CaliperGap} similarPreset - The preset to check with the existing presets
		 * @returns {CaliperGap} The preset that is similar to the given similarPreset or undefined if not found
		 */
		CaliperModel.prototype.getSimilarPreset = function (similarPreset) {
			return _.find(this._presetGaps, function (presetGap) {
				return presetGap.isSimilarTo(similarPreset);
			});
		};


		/*
		 * UTILITY FUNCTIONS
		 */

		/**
		 * Check if the given vector is representing a height measurement
		 * This depends on the angle of the vector
		 * @param {ShapeUtils.Vector} vector - The vector to check.
		 * @returns {Boolean} True if measuring heights, false otherwise.
		 */
		CaliperModel.prototype.isMeasuringHeight = function (vector) {
			var angle = vector.getAngleInDegrees();

			return (angle === 90 || angle === -90);
		};

		/**
		 * Check if the given vector is representing a width measurement
		 * This depends on the angle of the vector
		 * @param {ShapeUtils.Vector} vector - The vector to check.
		 * @returns {Boolean} True if measuring widths, false otherwise
		 */
		CaliperModel.prototype.isMeasuringWidth = function (vector) {
			var angle = vector.getAngleInDegrees();

			return (angle === 0 || angle === 180);
		};

		return CaliperModel;
	});

define('web-viewer/measure/ScreenRulingUnitSelector',["web-viewer/layout/component/DropdownMiniButton",
		"web-viewer/core/CoreConstants",
		"web-viewer/measure/MeasureConstants",
		"underscore"
	],
	function (DropdownMiniButton,
		CoreConstants,
		MeasureConstants,
		_) {
		"use strict";

		/**
		 * @exports ScreenRulingUnitSelector
		 * @class The ScreenRulingUnitSelector has a button that allows setting the unit for screen ruling
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var ScreenRulingUnitSelector = function (f) {
			this.f = f;

			/**
			 * Make the options dropdown
			 */
			this._optionButton = new DropdownMiniButton(f, "screenRulingUnitSettings", "icon-WCR_Settings-submenu");

			f.i18n.loadBundle("viewer", function () {

				// Units for size
				function toggleScreenRulingUnits(unit, mode, toggle) {
					if (toggle) {
						this.setUnitAndMode(unit, mode);
					}
				};

				this._optionButton.addOption({
					name: 'unitLpmm',
					content: this.f.i18n.t("UnitScreenmm"),
					onchange: _.bind(toggleScreenRulingUnits, this, CoreConstants.ResolutionUnits.PPMM, MeasureConstants.ScreenRulingMode.LINES),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'unitLpcm',
					content: this.f.i18n.t("UnitScreencm"),
					onchange: _.bind(toggleScreenRulingUnits, this, CoreConstants.ResolutionUnits.PPCM, MeasureConstants.ScreenRulingMode.LINES),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'unitLpi',
					content: this.f.i18n.t("UnitScreenInch"),
					onchange: _.bind(toggleScreenRulingUnits, this, CoreConstants.ResolutionUnits.PPI, MeasureConstants.ScreenRulingMode.LINES),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'unitDeg',
					content: this.f.i18n.t("UnitDegree"),
					onchange: _.bind(toggleScreenRulingUnits, this, 0, MeasureConstants.ScreenRulingMode.ANGLE),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'unitLpmmDeg',
					content: this.f.i18n.t("UnitDegreemm"),
					onchange: _.bind(toggleScreenRulingUnits, this, CoreConstants.ResolutionUnits.PPMM, MeasureConstants.ScreenRulingMode.BOTH),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'unitLpcmDeg',
					content: this.f.i18n.t("UnitDegreecm"),
					onchange: _.bind(toggleScreenRulingUnits, this, CoreConstants.ResolutionUnits.PPCM, MeasureConstants.ScreenRulingMode.BOTH),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'unitLpiDeg',
					content: this.f.i18n.t("UnitDegreeInch"),
					onchange: _.bind(toggleScreenRulingUnits, this, CoreConstants.ResolutionUnits.PPI, MeasureConstants.ScreenRulingMode.BOTH),
					userCannotUncheck: true
				});

				this._optionButton.addDivider();

				// angle CW or CCW
				function toggleScreenOrientation(mode, toggle) {
					if (toggle) {
						this.setOrientation(mode);
					}
				};

				this._optionButton.addOption({
					name: 'scrOrientationCW',
					content: this.f.i18n.t("ScreenCW"),
					onchange: _.bind(toggleScreenOrientation, this, MeasureConstants.ScreenRulingOrientation.CW),
					userCannotUncheck: true
				});

				this._optionButton.addOption({
					name: 'scrOrientationCCW',
					content: this.f.i18n.t("ScreenCCW"),
					onchange: _.bind(toggleScreenOrientation, this, MeasureConstants.ScreenRulingOrientation.CCW),
					userCannotUncheck: true
				});

				this._updateUnitOptions();
				this._updateOrientationOptions();
				this._optionButton.setTooltip("viewer", "UnitSettings");

				this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_SCREENRULING", _.bind(this._updateUnitOptions, this));
				this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_SCREEN", _.bind(this._updateUnitOptions, this));
				this.f.getCurrentUser().on("options.changed.PREFERENCES_SCREEN_ORIENTATION", _.bind(this._updateOrientationOptions, this));
			}, this);

		};

		/**
		 * Get the button element
		 * @returns {JQueryObj} element of the button
		 */
		ScreenRulingUnitSelector.prototype.getButtonElement = function () {
			return this._optionButton.element;
		};

		/**
		 * Set the unit and mode. The unit itself is not persisted, but the mode is.
		 * @param {CoreConstants.ResolutionUnits} unit
		 * @param {MeasureConstants.ScreenRulingMode} mode
		 */
		ScreenRulingUnitSelector.prototype.setUnitAndMode = function (unit, mode) {
			// set options and send out events
			if (this.f.getOption("PREFERENCES_UNIT_SCREENRULING") !== unit) {
				this.f.setOption("PREFERENCES_UNIT_SCREENRULING", unit, true);
			}
			if (this.f.getOption("PREFERENCES_UNIT_SCREEN") !== mode) {
				this.f.setOption("PREFERENCES_UNIT_SCREEN", mode, false);
			}
		};

		/**
		 * Update the toggle options of the button for units
		 */
		ScreenRulingUnitSelector.prototype._updateUnitOptions = function () {
			var unit = this.f.getOption("PREFERENCES_UNIT_SCREENRULING");
			var mode = this.f.getOption("PREFERENCES_UNIT_SCREEN");

			this._optionButton.toggleOption("unitLpmm", unit === CoreConstants.ResolutionUnits.PPMM && mode === MeasureConstants.ScreenRulingMode.LINES);
			this._optionButton.toggleOption("unitLpcm", unit === CoreConstants.ResolutionUnits.PPCM && mode === MeasureConstants.ScreenRulingMode.LINES);
			this._optionButton.toggleOption("unitLpi", unit === CoreConstants.ResolutionUnits.PPI && mode === MeasureConstants.ScreenRulingMode.LINES);
			this._optionButton.toggleOption("unitDeg", mode === MeasureConstants.ScreenRulingMode.ANGLE);
			this._optionButton.toggleOption("unitLpmmDeg", unit === CoreConstants.ResolutionUnits.PPMM && mode === MeasureConstants.ScreenRulingMode.BOTH);
			this._optionButton.toggleOption("unitLpcmDeg", unit === CoreConstants.ResolutionUnits.PPCM && mode === MeasureConstants.ScreenRulingMode.BOTH);
			this._optionButton.toggleOption("unitLpiDeg", unit === CoreConstants.ResolutionUnits.PPI && mode === MeasureConstants.ScreenRulingMode.BOTH);
		};

		/**
		 * Set the screen orientation.
		 * @param {MeasureConstants.ScreenRulingOrientation} mode
		 */
		ScreenRulingUnitSelector.prototype.setOrientation = function (mode) {
			// set option and send out events
			this.f.setOption("PREFERENCES_SCREEN_ORIENTATION", mode, false);
		};

		/**
		 * Update the toggle options of the button for orientation
		 */
		ScreenRulingUnitSelector.prototype._updateOrientationOptions = function () {
			var orientation = this.f.getOption("PREFERENCES_SCREEN_ORIENTATION");

			this._optionButton.toggleOption("scrOrientationCW", orientation === MeasureConstants.ScreenRulingOrientation.CW);
			this._optionButton.toggleOption("scrOrientationCCW", orientation === MeasureConstants.ScreenRulingOrientation.CCW);
		};

		return ScreenRulingUnitSelector;
	});

define('web-viewer/measure/ScreenRulingPopoverModel',["web-viewer/measure/MeasurePopoverModel",
		"web-viewer/measure/ScreenRulingUnitSelector",
		"underscore",
		"jquery"
	],
	function (MeasurePopoverModel,
		ScreenRulingUnitSelector,
		_,
		$) {
		"use strict";

		/**
		 * MeasurePopoverModel specific for the screen ruling tool.
		 * 
		 * @exports ScreenRulingPopoverModel
		 * @memberof MeasureModule
		 * @class ScreenRulingPopoverModel
		 * @augments MeasurePopoverModel
		 * @constructor
		 * 
		 * @param {Core.ViewerFacade} f
		 * @param {LineworkWrapper} area
		 * @param {String} name
		 * @param {String | JQuery.Element} title - Title to be shown in the header
		 * @param {Array<Number> | Array<String>} channels
		 * @param {Boolean} [makeTotalCount] Defines if a total of the different channel values needs to be calculated, only possible if the 
		 * 										values of the channels array are numbers
		 * @param {String | JQuery.Element} headerExtra - A jQuery component or string that is displayed in upper-right corner of this popver
		 */
		var ScreenRulingPopoverModel = function (f, area, name, title, channels, makeTotalCount, headerExtra) {
			MeasurePopoverModel.call(this, f, area, name, title, channels, makeTotalCount, headerExtra);

			this._screenRulingUnitSelector = null;

			this.f.getDocumentController().whenActivePageChangedAndParametersAvailable(this._updateScreenRulingUnitSelectorButton, this);
		};

		// inherit
		ScreenRulingPopoverModel.prototype = Object.create(MeasurePopoverModel.prototype);
		// correct the constructor pointer 
		ScreenRulingPopoverModel.prototype.constructor = ScreenRulingPopoverModel;

		/**
		 * Called when the active page is changed to either show it or not for digital film
		 */
		ScreenRulingPopoverModel.prototype._updateScreenRulingUnitSelectorButton = function () {
			if (this.f.getActivePage().isDigitalFilmPage()) {
				if (this._screenRulingUnitSelector === null) {
					this._screenRulingUnitSelector = new ScreenRulingUnitSelector(this.f);
					this._headerExtra.append(this._screenRulingUnitSelector.getButtonElement());
				} else {
					this._screenRulingUnitSelector.getButtonElement().show();
				}
			} else {
				if (this._screenRulingUnitSelector !== null) {
					this._screenRulingUnitSelector.getButtonElement().hide();
				}
			}
		};

		return ScreenRulingPopoverModel;
	});

define('web-viewer/measure/ScreenRulingMeasureTool',["underscore",
		"web-viewer/measure/AbstractMeasureTool",
		"web-viewer/measure/MeasureConstants",
		"web-viewer/measure/ScreenRulingPopoverModel",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractMeasureTool,
		MeasureConstants,
		ScreenRulingPopoverModel,
		ShapeUtils,
		ViewerUtils,
		CoreConstants) {
		"use strict";

		/**
		 * Screen ruling tool. With this, you can see the angle and line density of a digital film document.
		 *
		 * @exports ScreenRulingMeasureTool
		 * @memberof MeasureModule
		 * @class ScreenRulingMeasureTool
		 * @augments Measure.AbstractMeasureTool
		 *
		 * @param {Core.ViewerFacade} f
		 */
		var ScreenRulingMeasureTool = function (f) {
			AbstractMeasureTool.call(this, f, CoreConstants.TOOLS.MEASURE_SCREEN_RULING);

			this.model.setRulerType(MeasureConstants.MeasureType.ScreenRuling);

			// store results so it can be rerendered when units change
			this._storedResults = null;

			//override the enterState event for the clickState
			this.registerState(this.clickState, _.defaults({
				"enterState": _.bind(function () {
					var worldResolution = this.getWorldResolution();
					if (worldResolution !== null) {
						var sizeX = MeasureConstants.ClickSize.DIGITAL_FILM / worldResolution.x; // square of 64 world pixels
						var sizeY = MeasureConstants.ClickSize.DIGITAL_FILM / worldResolution.y;
						this.model.setStartPoint(new ShapeUtils.Point(this.model.ruler.startPoint.x - sizeX / 2, this.model.ruler.startPoint.y - sizeY / 2));
						this.model.setEndPoint(new ShapeUtils.Point(this.model.ruler.startPoint.x + sizeX, this.model.ruler.startPoint.y + sizeY));
						this.calculate();
						this.changeState(this.defaultState);
					}
				}, this)
			}, this.getState(this.drawingState)));

			// rerender when unit settings change
			this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_SCREENRULING", _.bind(this.showInfo, this, null));
			this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_SCREEN", _.bind(this.showInfo, this, null));
			this.f.getCurrentUser().on("options.changed.PREFERENCES_SCREEN_ORIENTATION", _.bind(this.showInfo, this, null));
		};

		// extend from the AbstractTool
		ScreenRulingMeasureTool.prototype = Object.create(AbstractMeasureTool.prototype);
		ScreenRulingMeasureTool.prototype.constructor = ScreenRulingMeasureTool;

		/**
		 * @override
		 * Can the tool be shown for this page?
		 * @returns {Boolean} false
		 */
		ScreenRulingMeasureTool.prototype.canBeUsedOnActivePage = function () {
			var page = this.f.getActivePage();
			return page && page.hasGraphics() && page.isDigitalFilmPage();
		};

		/**
		 * @override
		 * Does this tool need a total count
		 * @returns {Boolean} false
		 */
		ScreenRulingMeasureTool.prototype._needsTotalCount = function () {
			return false;
		};

		/**
		 * @override
		 * Sets the required options for the call to the viewserver
		 */
		ScreenRulingMeasureTool.prototype.calculate = function () {
			var options = {
				scrmode: 1
			};
			//calls getInfo in AbstactMeasureTool
			this.getInfo(options);
		};

		/**
		 * @override
		 * Gets the information from the to object converted xml and creates a MeasurePopoverModel that shows the information and tells the AbstractModel to add the popover
		 * @param {Object} object
		 */
		ScreenRulingMeasureTool.prototype.showInfo = function (object) {
			var channels = [];

			// store object for rerendering when units change
			if (object) {
				this._storedResults = object;
			} else {
				object = this._storedResults;
			}

			// read modes from preferences
			var mode = this.f.getOption("PREFERENCES_UNIT_SCREEN");
			var orientation = this.f.getOption("PREFERENCES_SCREEN_ORIENTATION");

			if (object) {

				if (object.status.info.screen) {
					if (!_.isArray(object.status.info.screen)) {
						object.status.info.screen = [object.status.info.screen];
					}

					_.each(object.status.info.screen, _.bind(function (channel) {
						var channelInfo = "";
						if (channel.found === "1") {
							if (mode === MeasureConstants.ScreenRulingMode.LINES || mode === MeasureConstants.ScreenRulingMode.BOTH) {
								channelInfo += ViewerUtils.convertResolutionFromPpmmValue(ViewerUtils.parseFloat(channel.ruling), this.f.getScreenUnits(), this.f.getScreenUnitsFix());
								channelInfo += " " + this.f.i18n.t(this.f.getScreenUnitsLocName());
							}
							if (mode === MeasureConstants.ScreenRulingMode.BOTH) {
								channelInfo += " / ";
							}
							if (mode === MeasureConstants.ScreenRulingMode.ANGLE || mode === MeasureConstants.ScreenRulingMode.BOTH) {
								var angle = ViewerUtils.parseFloat(channel.angle);
								if (orientation === MeasureConstants.ScreenRulingOrientation.CCW) {
									angle = 90 - angle;
								}
								channelInfo += angle.toFixed(2) + "°";
							}
						}
						channels[channel.color] = channelInfo;
					}, this));
				}
				this.f.emit("channels.informationAdded", channels);
			}
			this.showPopover(channels);
		};

		/**
		 * @override
		 */
		ScreenRulingMeasureTool.prototype.createPopoverModel = function (channelInfo, area) {
			return new ScreenRulingPopoverModel(this.f, area, "Channels", $("<i>").addClass("icon icon-normal icon-WCR_Measure-Screen"), channelInfo, this._needsTotalCount());
		};

		return ScreenRulingMeasureTool;

	});

define('web-viewer/layout/component/Spinner',["component/tooltip/Tooltip",
		"web-viewer/utils/ViewerUtils",
		"underscore",
		"jquery",
		"jqueryUI"
	],
	function (Tooltip,
		ViewerUtils,
		_,
		$) {
		"use strict";

		/**
		 * @class A spinner is a component that has an input field with arrows next to it to spin through the values.
		 * You can also use page keys (page) or arrow keys (step).
		 * The wrapping element to add to a component sits in Spinner.element
		 * @exports Spinner
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {Object} options - put undefined for default values
		 *    {Number} min: minimum value of the spinner (Default: 0)
		 *    {Number} max: maximum value of the spinner (Default: 100)
		 *    {Number} step: step value of the spinner (Default: 1);
		 *    {Number} page: page size of the spinner (Default: 10);
		 *    {Function} onSpinnerChanged: called when the spinner value is changed. Arguments: value (Required)
		 *    {String} inputUnit: Unit of the input field (put behind the input field) (Default: %)
		 *    {String} tooltip: Tooltip text that should be shown when you hover over the component. (Optional)
		 */
		var Spinner = function (f, options) {
			this.f = f;

			// Set Options
			this._min = options.min === undefined ? 0 : options.min;
			this._max = options.max === undefined ? 100 : options.max;
			this._step = options.step === undefined ? 1 : options.step;
			this._page = options.page === undefined ? 10 : options.page;
			this._onSpinnerChanged = options.onSpinnerChanged;
			this._inputUnit = options.inputUnit === undefined ? "%" : options.inputUnit;

			// Wrapper element
			this.element = $("<div />").addClass("spinnerWrapper");

			// Add the input field if needed
			this._input = $("<input />").attr("size", ("" + this._max).length).addClass("input"); // the input field
			this._input.on("keydown", _.bind(this._valueChanged, this));
			this._input.on("blur", _.bind(this._valueChanged, this));
			this._labelUnit = $("<span>" + this._inputUnit + "</span>");
			this._label = $("<label />").addClass("unit").append(this._input).append(this._labelUnit); // the unit
			this.element.append(this._label);

			// spinner behavior
			this._input.spinner({
				min: this._min,
				max: this._max,
				step: this._step,
				page: this._page,
				spin: _.bind(function (event, ui) {
					this._fireSpinnerChanged(ui.value);
				}, this)
			});
			this.setValue(this._min);

			// Tooltip
			if (options.tooltip !== undefined) {
				this._tooltip = new Tooltip(this.f.i18n, this.element, this.f.getContext(), this.f.getContext(), this.f.isMobile(), options.tooltip);
			}
		};

		/**
		 * Change the value of the spinner (by pressing enter in the input or on blur)
		 * @param {Object} event
		 */
		Spinner.prototype._valueChanged = function (event) {
			if ((event.which == 13 || event.keyCode == 13) && event.type === "keydown") {
				// on enter, lose focus, let the blur event handle it
				this._input.blur();
			} else if (event.type !== "keydown") {
				var value = ViewerUtils.parseFloat($(event.target).val());
				if (isNaN(value)) { // if nothing is filled in or only letter(s), behave like entered the minimum
					value = this._min;
				}
				// limit to boundaries
				value = Math.max(Math.min(value, this._max), this._min);
				$(event.target).val(value); // by doing this letters added to the end will be filtered out and bounderies are enforced
				this._fireSpinnerChanged(value);
			}
		};

		/**
		 * @private
		 * Fire the value to the listener
		 * @param {Number} value - New Spinner value
		 */
		Spinner.prototype._fireSpinnerChanged = function (value) {
			this._onSpinnerChanged(value);
		};

		/**
		 * Set the value of the Spinner
		 * @param {Number} value - New value of the Spinner
		 */
		Spinner.prototype.setValue = function (value) {
			if (this.getValue() !== value) {
				this._input.spinner("value", value);
			}
		};

		/**
		 * Get the value of the Spinner
		 * @returns {Number} The value of the Spinner
		 */
		Spinner.prototype.getValue = function () {
			return this._input.spinner("value");
		};

		/**
		 * Is the Spinner disabled?
		 * @returns {Boolean} True if the Spinner is disabled, false otherwise
		 */
		Spinner.prototype.isDisabled = function () {
			return this._input.spinner("option", "disabled");
		};

		/**
		 * Set the Spinner disabled
		 * @param {Boolean} disabled - True if the Spinner should be disabled, false if the Spinner should be enabled.
		 */
		Spinner.prototype.setDisabled = function (disabled) {
			this._input.spinner("option", "disabled", disabled);
		};

		/**
		 * Set the tool tip of the Spinner
		 * @param {String} tooltip - Tool tip text
		 */
		Spinner.prototype.setTooltip = function (tooltip) {
			if (this._tooltip === undefined) {
				this._tooltip = new Tooltip(this.f.i18n, this.element, this.f.getContext(), this.f.getContext(), this.f.isMobile(), tooltip);
			} else {
				this._tooltip.setTooltipContent(tooltip);
			}
		};

		/**
		 * Update one of the initial options.
		 * Currently supported: "min","max","step","page".
		 * @param {String} optionName - Name of the option, e.g. "max"
		 * @param {Object} value - New value for the option
		 */
		Spinner.prototype.setOption = function (optionName, newValue) {
			var oldValue = this["_" + optionName];
			if (oldValue !== undefined && oldValue !== newValue) {
				// update the current value
				this["_" + optionName] = newValue;
				// update the spinner
				this._input.spinner("option", optionName, newValue);
			}
		};

		/**
		 * Set the label of the spinner and update the DOM element
		 * @param {String} newValuet - new value of the label
		 */
		Spinner.prototype.setInputUnit = function (newValuet) {
			this._inputUnit = newValuet;
			this._labelUnit.empty();
			this._labelUnit.append(this._inputUnit);
		};



		return Spinner;
	});

define('web-viewer/measure/CaliperSubtoolbarComponent',["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/SubtoolbarButton",
		"web-viewer/layout/component/Spinner",
		"web-viewer/core/CoreConstants",
		"web-viewer/measure/MeasureConstants",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/utils/ListenerUtil",
		"jquery",
		"underscore"
	],
	function (ToolbarComponent,
		ButtonGroup,
		SubtoolbarButton,
		Spinner,
		CoreConstants,
		MeasureConstants,
		ViewerUtils,
		ListenerUtil,
		$,
		_) {

		/**
		 * @exports CaliperSubtoolbarComponent
		 * @memberof MeasureModule
		 * @class This is the subtoolbar of the caliper tool
		 * @augments ToolbarComponent
		 * @mixes {Utils.ListenerUtil}
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {CaliperModel} caliperModel - The model of the caliper tool
		 */
		var CaliperSubtoolbarComponent = function (f, caliperModel) {
			ToolbarComponent.call(this, "CaliperSubtoolbar", f);
			ListenerUtil.mixin(this, f);

			this.f = f;
			this._caliperModel = caliperModel;

			this.f.i18n.loadBundle("viewer", function () {
				// Style selector
				this._stylesLabel = $("<span />").addClass("styleLabel").text(this.f.i18n.t("GapStyle") + ":");
				this._styles = $("<select />").addClass("styleSelect mousetrap");
				this._refreshGaps();
				this._styles.on("change", _.bind(this._styleSelectorChanged, this));
				this.element.append($("<div class='caliperStyle'/>").append(this._stylesLabel).append(this._styles));

				// Gap
				this._gapLabel = $("<span />").addClass("gapLabel").text(this.f.i18n.t("GapSize") + ":");
				this._gapSpinner = new Spinner(this.f, {
					min: 0,
					max: 9999,
					step: 0.01,
					page: 10,
					onSpinnerChanged: _.bind(this._spinnerChanged, this),
					inputUnit: this.f.i18n.t(this.f.getUnitsLocName())
				});
				this.element.append($("<div class='caliperGap'/>").append(this._gapLabel).append(this._gapSpinner.element.addClass("mousetrap")));


				// Width/Height buttons
				this._heightWidthButtonGroup = new ButtonGroup("heightWidthButtonGroup");
				this._heightWidthButtonGroup.element.addClass("heightWidthButtonGroup");
				this._heightButton = new SubtoolbarButton(this.f, "checkHeight", "icon-WCR_Height", true, _.bind(this._heightButtonClicked, this));
				this._heightButton.element.append(this.f.i18n.t("GapHeight"));
				this._heightWidthButtonGroup.addButton(this._heightButton);
				this._widthButton = new SubtoolbarButton(this.f, "checkWidth", "icon-WCR_Width", true, _.bind(this._widthButtonClicked, this));
				this._widthButton.element.append(this.f.i18n.t("GapWidth"));
				this._heightWidthButtonGroup.addButton(this._widthButton);
				this.addButtonGroup(this._heightWidthButtonGroup);

				// Buttons Group
				this._saveButtonGroup = new ButtonGroup("saveButtonGroup");
				// Save button
				this._saveGapButton = new SubtoolbarButton(this.f, "SaveGap", "icon-WCR_Save", true, _.bind(this._saveClicked, this));
				this._saveGapButton.setTooltip("viewer", "SaveGapTT");
				this._saveButtonGroup.addButton(this._saveGapButton);

				// Delete button
				this._deleteGapButton = new SubtoolbarButton(this.f, "DeleteGap", "icon-WCR_Actions-Delete12", true, _.bind(this._deleteClicked, this));
				this._deleteGapButton.setTooltip("viewer", "DeleteGapTT");
				this._saveButtonGroup.addButton(this._deleteGapButton);
				this.addButtonGroup(this._saveButtonGroup);


				// Listen to model
				this._registerListener("changed.currentGap", this._gapChanged, this, this._caliperModel);
				this._registerListener("changed.currentGap.vector", this._gapVectorChanged, this, this._caliperModel);
				this._registerListener("changed.presetGaps.new", this._refreshGaps, this, this._caliperModel);
				this._registerListener("changed.presetGaps.deleted", this._deletedPresetGap, this, this._caliperModel);

				// listen to unit change
				this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_SIZE", _.bind(this._unitsUpdated, this));

				// Set initial gap to freeform
				this._caliperModel.setCurrentGapById(this._caliperModel.getFreeformGap().getId());
			}, this);
		};

		CaliperSubtoolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		CaliperSubtoolbarComponent.prototype.constructor = CaliperSubtoolbarComponent;

		/**
		 * On change of style selector.
		 * @param {EventObj} e
		 */
		CaliperSubtoolbarComponent.prototype._styleSelectorChanged = function (e) {

			// if switching between freeform and fixed, copy the vector
			var currentCaliperGap = this._caliperModel.getCurrentGap();
			var newCaliperGap = this._caliperModel.getGapById(e.target.value);
			if ((newCaliperGap.getStyle() === MeasureConstants.CaliperStyle.Fixed &&
					currentCaliperGap.getStyle() === MeasureConstants.CaliperStyle.Freeform) ||
				(newCaliperGap.getStyle() === MeasureConstants.CaliperStyle.Freeform &&
					currentCaliperGap.getStyle() === MeasureConstants.CaliperStyle.Fixed)) {
				newCaliperGap.setVector(currentCaliperGap.getVector());
			}
			this._caliperModel.setCurrentGapById(e.target.value);
		};

		/**
		 * On change of the gap spinner
		 * @param {Number} value - New value
		 */
		CaliperSubtoolbarComponent.prototype._spinnerChanged = function (value) {
			var resolution = null;
			if (this.f.getUnits() === CoreConstants.SizeUnits.PIXELS) {
				resolution = this._getResolution(this._caliperModel.getCurrentGap().getVector());
			}
			value = ViewerUtils.convertToMmValue(value, this.f.getUnits(), this.f.getUnitsFix(CoreConstants.SizeUnits.MM), ViewerUtils.parseFloat(resolution));

			if (this._caliperModel.getCurrentGap().getStyle() === MeasureConstants.CaliperStyle.Preset) {
				var oldGap = this._caliperModel.getCurrentGap();
				this._caliperModel.setCurrentGapById(this._caliperModel.getFixedGap().getId());
				this._caliperModel.setCurrentGapVector(oldGap.getVector()); // copy settings
			}
			var currentVector = this._caliperModel.getCurrentGap().getVector();
			currentVector.setLength(value);
			this._caliperModel.setCurrentGapVector(currentVector);
		};

		/**
		 * On click of the height button
		 * @param {EventObj} e
		 */
		CaliperSubtoolbarComponent.prototype._heightButtonClicked = function (e) {
			if (this._caliperModel.getCurrentGap().getStyle() === MeasureConstants.CaliperStyle.Preset) {
				var oldGap = this._caliperModel.getCurrentGap();
				this._caliperModel.setCurrentGapById(this._caliperModel.getFixedGap().getId());
				this._caliperModel.setCurrentGapVector(oldGap.getVector()); // copy settings
			}
			var currentVector = this._caliperModel.getCurrentGap().getVector();
			currentVector.setAngleInDegrees(-90);
			this._caliperModel.setCurrentGapVector(currentVector);
			e.stopPropagation();
		};

		/**
		 * On click of the width button
		 * @param {EventObj} e
		 */
		CaliperSubtoolbarComponent.prototype._widthButtonClicked = function (e) {
			if (this._caliperModel.getCurrentGap().getStyle() === MeasureConstants.CaliperStyle.Preset) {
				var oldGap = this._caliperModel.getCurrentGap();
				this._caliperModel.setCurrentGapById(this._caliperModel.getFixedGap().getId());
				this._caliperModel.setCurrentGapVector(oldGap.getVector()); // copy settings
			}
			var currentVector = this._caliperModel.getCurrentGap().getVector();
			currentVector.setAngleInDegrees(0);
			this._caliperModel.setCurrentGapVector(currentVector);
			e.stopPropagation();
		};

		/**
		 * Called when the gap of the model changed.
		 * @param {CaliperGap} oldValue - Old gap
		 * @param {CaliperGap} newValue - New gap
		 */
		CaliperSubtoolbarComponent.prototype._gapChanged = function (oldValue, newValue) {
			this._styles.find('option[value="' + newValue.getId() + '"]').prop("selected", true);

			this._gapVectorChanged(null, newValue.getVector());
			this._updateButtons();
		};

		/**
		 * Called when the gap vector of current caliper gap of the model changed
		 * @param {ShapeUtils.Vector} oldValue - Old vector
		 * @param {ShapeUtils.Vector} newValue - New vector
		 */
		CaliperSubtoolbarComponent.prototype._gapVectorChanged = function (oldValue, newValue) {
			// angle
			if (this._caliperModel.isMeasuringHeight(newValue)) {
				this._heightButton.setActive(true);
				this._widthButton.setActive(false);
			} else if (this._caliperModel.isMeasuringWidth(newValue)) {
				this._widthButton.setActive(true);
				this._heightButton.setActive(false);
			}

			// gap size
			var resolution = null;
			if (this.f.getUnits() === CoreConstants.SizeUnits.PIXELS) {
				resolution = this._getResolution(newValue);
			}
			var newGap = newValue.getLength();
			newGap = ViewerUtils.convertFromMmValue(newGap, this.f.getUnits(), this.f.getUnitsFix(), ViewerUtils.parseFloat(resolution));
			this._gapSpinner.setValue(newGap);
		};

		/**
		 * Refresh the gaps.
		 * The style selector will be replaced with all gaps from the caliper model.
		 * If newValue is passed, it will become the selected gap.
		 * @param {CaliperGap} [oldValue] - Old caliper gap
		 * @param {CaliperGap} [newValue] - New caliper gap
		 */
		CaliperSubtoolbarComponent.prototype._refreshGaps = function (oldValue, newValue) {
			this._styles.empty();

			_.each(this._caliperModel.getAllGaps(), function (caliperGap) {
				this._newGap(caliperGap);
			}, this);

			// a new gap -> select it as current gap
			if (newValue !== undefined) {
				this._caliperModel.setCurrentGapById(newValue.getId());
			}
		};

		/**
		 * Handle a change of units: recalculate and redraw toolbar
		 */
		CaliperSubtoolbarComponent.prototype._unitsUpdated = function () {
			this._gapSpinner.setInputUnit(this.f.i18n.t(this.f.getUnitsLocName()));
			this._gapChanged(this._caliperModel.getCurrentGap(), this._caliperModel.getCurrentGap());
			this._refreshGaps();
		};

		/**
		 * Called when a new gap is added to the presets of the model (or initially), see _refreshGaps.
		 * @param {CaliperGap} newValue - New caliper gap
		 */
		CaliperSubtoolbarComponent.prototype._newGap = function (newValue) {
			// new value is the new gap
			var style = $("<option />").html(this._toString(newValue)).val(newValue.getId());

			this._styles.append(style);
		};

		/**
		 * Called when a preset gap is deleted from the model
		 * @param {CaliperGap} oldValue - This is the removed preset
		 * @param {CaliperGap} newValue - Always null
		 */
		CaliperSubtoolbarComponent.prototype._deletedPresetGap = function (oldValue, newValue) {
			// old value is the removed fixed gap
			this._styles.find('option[value="' + oldValue.getId() + '"]').remove();

			// switch back to fixed
			this._caliperModel.setCurrentGapById(this._caliperModel.getFixedGap().getId());
			this._caliperModel.setCurrentGapVector(oldValue.getVector()); // copy settings
		};

		/**
		 * Called when the save button is clicked in the sub tool bar
		 * Saves the current gap as preset.
		 */
		CaliperSubtoolbarComponent.prototype._saveClicked = function () {
			var similarPreset = this._caliperModel.getSimilarPreset(this._caliperModel.getCurrentGap());
			if (similarPreset) {
				// there already exists a similar preset, so just switch to that one instead of saving a new one
				this._caliperModel.setCurrentGapById(similarPreset.getId());
			} else {
				this._caliperModel.saveCurrentGapAsPreset();
			}
		};

		/**
		 * Called when the delete button is clicked in the sub tool bar
		 * Deletes the current gap from the presets.
		 */
		CaliperSubtoolbarComponent.prototype._deleteClicked = function () {
			this._caliperModel.deleteCurrentGapFromPresets();
		};

		/**
		 * Update the buttons save/delete (decides which ones to show)
		 */
		CaliperSubtoolbarComponent.prototype._updateButtons = function () {
			var currentGap = this._caliperModel.getCurrentGap();
			switch (currentGap.getStyle()) {
			case MeasureConstants.CaliperStyle.Freeform:
				this._deleteGapButton.element.addClass("hiddenElement");
				this._saveGapButton.element.removeClass("hiddenElement");
				break;
			case MeasureConstants.CaliperStyle.Fixed:
				this._deleteGapButton.element.addClass("hiddenElement");
				this._saveGapButton.element.removeClass("hiddenElement");
				break;
			case MeasureConstants.CaliperStyle.Preset:
				this._saveGapButton.element.addClass("hiddenElement");
				this._deleteGapButton.element.removeClass("hiddenElement");
				break;
			}
		};

		/**
		 * Convert a gap to a String representation for the drop down.
		 * i18n general bundle should be available!
		 * @param {CaliperGap} caliperGap - Gap to represent as string
		 * @returns {String} String representation of the given caliper gap
		 */
		CaliperSubtoolbarComponent.prototype._toString = function (caliperGap) {
			switch (caliperGap.getStyle()) {
			case MeasureConstants.CaliperStyle.Freeform:
				return this.f.i18n.t("GapStyleFreeform");
				break;
			case MeasureConstants.CaliperStyle.Fixed:
				return this.f.i18n.t("GapStyleFixed");
				break;
			case MeasureConstants.CaliperStyle.Preset:
				var resolution = null;
				if (this.f.getUnits() === CoreConstants.SizeUnits.PIXELS) {
					resolution = this._getResolution(caliperGap.getVector());
				}
				var gap = ViewerUtils.convertFromMmValue(caliperGap.getVector().getLength(), this.f.getUnits(), this.f.getUnitsFix(), ViewerUtils.parseFloat(resolution));
				var units = this.f.i18n.t(this.f.getUnitsLocName());
				var orientation = "";
				if (this._caliperModel.isMeasuringHeight(caliperGap.getVector())) {
					// heights
					orientation = this.f.i18n.t("GapHeight");
				} else if (this._caliperModel.isMeasuringWidth(caliperGap.getVector())) {
					// widths
					orientation = this.f.i18n.t("GapWidth");
				}
				return gap + " " + units + " (" + orientation + ")";
				break;
			}
		};

		/**
		 * Get the resolution to use for unit conversion
		 */
		CaliperSubtoolbarComponent.prototype._getResolution = function (vector) {
			var page = this.f.getActivePage();
			var resolution = null;
			if (this._caliperModel.isMeasuringHeight(vector)) {
				resolution = page.getParam("resolutiony");
			} else if (this._caliperModel.isMeasuringWidth(vector)) {
				resolution = page.getParam("resolutionx");
			}
			return resolution;
		};

		return CaliperSubtoolbarComponent;
	});

define('web-viewer/measure/MeasureToolbarComponent',["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/component/SubtoolbarButton",
		"component/button/Button",
		"web-viewer/layout/component/ColorPicker",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/measure/MeasureConstants",
		"web-viewer/measure/CaliperSubtoolbarComponent",
		"utils/OrderedObject",
		"jquery"
	],
	function (ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		SubtoolbarButton,
		Button,
		ColorPicker,
		LayoutConstants,
		MeasureConstants,
		CaliperSubtoolbarComponent,
		OrderedObject,
		$
	) {
		"use strict";

		/**
		 * @exports MeasureToolbarComponent
		 * @memberof MeasureModule
		 * @class This is the toolbar for all measure tools
		 * @augments ToolbarComponent
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {Measure.RulerTool} [rulerTool]
		 * @param {Measure.EyeDropperMeasureTool} [eyedropperTool]
		 * @param {Measure.BarcodeMeasureTool} [barcodeTool]
		 * @param {Measure.BrailleMeasureTool} [brailleTool]
		 * @param {Measure.CaliperTool} [caliperTool]
		 * @param {Measure.ScreenRulingMeasureTool} [screenRulingTool]
		 */
		var MeasureToolbarComponent = function (f, rulerTool, eyedropperTool, barcodeTool, brailleTool, caliperTool, screenRulingTool) {
			ToolbarComponent.call(this, LayoutConstants.ToolbarComponent.MEASURE, f);
			this.f = f;

			this.rulerTool = rulerTool;
			this.eyedropperTool = eyedropperTool;
			this.barcodeTool = barcodeTool;
			this.brailleTool = brailleTool;
			this.caliperTool = caliperTool;
			this.screenRulingTool = screenRulingTool;

			this.rulerButton = null;
			this.eyeDropperButton = null;
			this.barcodeButton = null;
			this.brailleButton = null;
			this.caliperButton = null;
			this.screenRulingButton = null;

			/**
			 * Group: measure tools
			 */
			this.measureGroup = new ButtonGroup("MeasureGroup");
			this.measureGroup.setTitle(f.i18n, "viewer", "MeasurementToolsTitle");
			this.addButtonGroup(this.measureGroup);

			this.f.getDocumentController().whenActivePageChangedAndParametersAvailable(_.bind(function () {
				this.updateTools();
			}, this));
		};

		MeasureToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		MeasureToolbarComponent.prototype.constructor = MeasureToolbarComponent;
		MeasureToolbarComponent.toolbarName = LayoutConstants.ToolbarComponent.MEASURE;
		/**
		 * Initialize button for ruler tool
		 * @param {Measure.RulerTool} tool
		 */
		MeasureToolbarComponent.prototype.initializeRulerButton = function (tool) {
			if (!this.rulerButton) {
				// Ruler button
				this.rulerButton = new ViewerToolbarButton(this.f, "RulerButton", "icon icon-WCR_Measure-Ruler", true);
				this.rulerButton.listenToTool(tool, true);

				this.rulerButton.setTooltip("viewer", "RulerTT");

				/**
				 * Subtoolbar Component:
				 */
				var rulerSubtoolbar = new ToolbarComponent("RulerSubtoolbar", this.f);
				this.rulerButton.addSubtoolbarComponent(rulerSubtoolbar);
				/**
				 * Group:
				 */
				var rulerSubGroup = new ButtonGroup("RulerSubGroup");
				rulerSubtoolbar.addButtonGroup(rulerSubGroup);

				// Distance and angle
				var daButton = new SubtoolbarButton(this.f, "DAButton", "icon-WCR_Measure-Ruler-DA-cont", true, _.partial(function (f) {
					f.setOption("PREFERENCES_RULER_DISPLAY_MODE_DA", f.getOption("PREFERENCES_RULER_DISPLAY_MODE_DA") ? 0 : 1);
					f.setOption("PREFERENCES_RULER_DISPLAY_MODE", f.getOption("PREFERENCES_RULER_DISPLAY_MODE") >= 1 ? 0 : 1);
				}, this.f));
				daButton.setIcon("icon-WCR_Measure-Ruler-DA-cont", ["icon-WCR_Measure-Ruler-DA-sub icon-red"]);
				daButton.setTooltip("viewer", "RulerDATT");
				rulerSubGroup.addButton(daButton);
				//daButton.setActive(true);

				// X and Y
				var xyButton = new SubtoolbarButton(this.f, "XYButton", "icon-WCR_Measure-Ruler-XY-cont", true, _.partial(function (f) {
					f.setOption("PREFERENCES_RULER_DISPLAY_MODE_XY", f.getOption("PREFERENCES_RULER_DISPLAY_MODE_XY") ? 0 : 1);
					f.setOption("PREFERENCES_RULER_DISPLAY_MODE", f.getOption("PREFERENCES_RULER_DISPLAY_MODE") >= 2 ? 1 : 2);
				}, this.f));
				xyButton.setIcon("icon-WCR_Measure-Ruler-XY-cont", ["icon-WCR_Measure-Ruler-XY-sub icon-red"]);
				xyButton.setTooltip("viewer", "RulerXYTT");
				rulerSubGroup.addButton(xyButton);

				// W and H
				var whButton = new SubtoolbarButton(this.f, "WHButton", "icon-WCR_Measure-Ruler-WH-cont", true, _.partial(function (f) {
					f.setOption("PREFERENCES_RULER_DISPLAY_MODE_WH", f.getOption("PREFERENCES_RULER_DISPLAY_MODE_WH") ? 0 : 1);
					f.setOption("PREFERENCES_RULER_DISPLAY_MODE", f.getOption("PREFERENCES_RULER_DISPLAY_MODE") >= 3 ? 2 : 3);
				}, this.f));
				whButton.setIcon("icon-WCR_Measure-Ruler-WH-cont", ["icon-WCR_Measure-Ruler-WH-sub icon-red"]);
				whButton.setTooltip("viewer", "RulerWHTT");
				rulerSubGroup.addButton(whButton);


				var setRulerButtonsActive = _.partial(function (f) {
					daButton.setActive(f.getOption("PREFERENCES_RULER_DISPLAY_MODE_DA") === 1);
					xyButton.setActive(f.getOption("PREFERENCES_RULER_DISPLAY_MODE_XY") === 1);
					whButton.setActive(f.getOption("PREFERENCES_RULER_DISPLAY_MODE_WH") === 1);
				}, this.f);

				this.f.getCurrentUser().on("options.changed.PREFERENCES_RULER_DISPLAY_MODE", function (oldValue, newValue) {
					setRulerButtonsActive();
				});

				setRulerButtonsActive();

				var colorPicker = new ColorPicker(_.partial(function (f, newColor) {
					f.getCurrentUser().setColor("PREFERENCES_RULER_DISPLAY_COLOR", newColor);
				}, this.f));

				this.f.getCurrentUser().on("options.changed.PREFERENCES_RULER_DISPLAY_COLOR", _.partial(function (f, oldValue, newValue) {
					colorPicker.setColor(f.getCurrentUser().getColor("PREFERENCES_RULER_DISPLAY_COLOR"));
				}, this.f));
				colorPicker.setColor(this.f.getCurrentUser().getColor("PREFERENCES_RULER_DISPLAY_COLOR"));

				var colorSelectorButton = new SubtoolbarButton(this.f, "ColorSelector", colorPicker.element, true);
				colorSelectorButton.setTooltip("viewer", "RulerDisplayColorChange");

				var colorResetButton = new SubtoolbarButton(this.f, "ColorReset", "icon-WCR_Revert", true, _.partial(function (f) {
					f.resetOption("PREFERENCES_RULER_DISPLAY_COLOR");
				}, this.f));
				colorResetButton.setTooltip("viewer", "RulerDisplayColorReset");

				var angleButton = new SubtoolbarButton(this.f, "angleButton", this.f.getOption("PREFERENCES_RULER_ANGLE_MODE") ? "measureCCW" : "measureCW", true, _.partial(function (f) {
					f.setOption("PREFERENCES_RULER_ANGLE_MODE", f.getOption("PREFERENCES_RULER_ANGLE_MODE") ? 0 : 1);
					if (f.getOption("PREFERENCES_RULER_ANGLE_MODE")) {
						this.setTooltip("viewer", "RulerAngleToCW");
						this.setIcon("icon-WCR_Measure-Ruler-CCW-cont", ["icon-WCR_Measure-Ruler-CCW-sub icon-red"]);
					} else {
						this.setTooltip("viewer", "RulerAngleToCCW");
						this.setIcon("icon-WCR_Measure-Ruler-CW-cont", ["icon-WCR_Measure-Ruler-CW-sub icon-red"]);
					}
				}, this.f));
				if (this.f.getOption("PREFERENCES_RULER_ANGLE_MODE")) {
					angleButton.setTooltip("viewer", "RulerAngleToCW");
					angleButton.setIcon("icon-WCR_Measure-Ruler-CCW-cont", ["icon-WCR_Measure-Ruler-CCW-sub icon-red"]);
				} else {
					angleButton.setTooltip("viewer", "RulerAngleToCCW");
					angleButton.setIcon("icon-WCR_Measure-Ruler-CW-cont", ["icon-WCR_Measure-Ruler-CW-sub icon-red"]);
				}


				var rulerSubGroupOptions = new ButtonGroup("RulerSubGroupOptions");
				rulerSubtoolbar.addButtonGroup(rulerSubGroupOptions);

				rulerSubGroupOptions.addButton(angleButton);

				rulerSubGroupOptions.addButton(colorSelectorButton);
				rulerSubGroupOptions.addButton(colorResetButton);
			}

		};

		/**
		 * Initialize button for eyedropper tool
		 * @param {Measure.EyeDropperMeasureTool} tool
		 */
		MeasureToolbarComponent.prototype.initializeEyeDropperButton = function (tool) {
			if (!this.eyeDropperButton) {
				// Eye dropper button
				this.eyeDropperButton = new ViewerToolbarButton(this.f, "EyeDropperButton", "icon icon-WCR_Measure-Colorpicker", false);
				this.eyeDropperButton.listenToTool(tool, true);
				this.eyeDropperButton.setTooltip("viewer", "ToolPercentageTT");
			}
		};

		/**
		 * Initialize button for barcode tool
		 * @param {Measure.BarcodeMeasureTool} tool
		 */
		MeasureToolbarComponent.prototype.initializeBarCodeButton = function (tool) {
			if (!this.barcodeButton) {
				// Barcode button
				this.barcodeButton = new ViewerToolbarButton(this.f, "BarcodeButton", "icon icon-WCR_Measure-Barcode", false);
				this.barcodeButton.listenToTool(tool, true);
				this.barcodeButton.setTooltip("viewer", "ToolBarcodeTT");
			}
		};

		/**
		 * Initialize button for braille tool
		 * @param {Measure.BrailleMeasureTool} tool
		 */
		MeasureToolbarComponent.prototype.initializeBrailleButton = function (tool) {
			if (!this.brailleButton) {
				// Braille button
				this.brailleButton = new ViewerToolbarButton(this.f, "BrailleButton", "icon icon-WCR_Measure-Braille", false);
				this.brailleButton.listenToTool(tool, true);
				this.brailleButton.setTooltip("viewer", "ToolBrailleTT");

				// translate the strings and sort alphabetically
				var langListTranslated = new OrderedObject();
				_.each(MeasureConstants.BrailleLanguage, function (lang, key) {
					if (key !== MeasureConstants.LanguageAutomatic) {
						langListTranslated.addItem(this.f.i18n.t(MeasureConstants.LanguageTranslationPrefix + lang.lang), key);
					}
				}, this);
				langListTranslated.setOrder(_.sortBy(langListTranslated.getKeys()));
				// Add Option for Automatic as well, but only after sorting, as it should always be the first option
				langListTranslated.addItemAtIndex(this.f.i18n.t(MeasureConstants.LanguageTranslationPrefix + MeasureConstants.BrailleLanguage[MeasureConstants.LanguageAutomatic].lang), MeasureConstants.LanguageAutomatic);

				// Braille language selector
				var brailleSubtoolbar = new ToolbarComponent("BrailleSubToolbar", this.f);
				this.brailleButton.addSubtoolbarComponent(brailleSubtoolbar);
				var langPicker = $("<div/>").addClass("brailleLanguages");
				var currentLang = $("<span/>").addClass("currentlang").append(langListTranslated.getKeyByValue(this.f.getOption("PREFERENCES_BRAILLE_LANGUAGE").toString()));
				var languageList = $("<ul/>").addClass("languagelist").hide();

				var language = new Button(this.f.i18n, this.f.getContext(), this.f.isMobile(), "language", currentLang, false, function () {
					languageList.toggle();
				});

				// add language strings to dropdown
				_.each(langListTranslated.getKeys(), function (lang, key) {
					var langButton = new Button(this.f.i18n, this.f.getContext(), this.f.isMobile(), "brailleLang" + langListTranslated.getItemByKey(lang), lang, false, _.partial(function (f) {
						var langOptionID = parseInt(langListTranslated.getItemByKey(lang), 10);
						if (isNaN(langOptionID)){
							langOptionID = 0;
						}
						f.setOption("PREFERENCES_BRAILLE_LANGUAGE", langOptionID);
						languageList.hide();
					}, this.f));
					languageList.append($("<li/>").append(langButton.element));
				}, this);

				language.element.append($("<b/>").addClass("caret"));
				langPicker.append(language.element);
				langPicker.append(languageList);
				brailleSubtoolbar.element.append(langPicker);

				//Change the contents of the brailleSubtoolbar to the current language
				this.f.getCurrentUser().on("options.changed.PREFERENCES_BRAILLE_LANGUAGE", function (oldValue, newValue) {
					currentLang.html(langListTranslated.getKeyByValue(newValue+""));
				});
			}
		};

		/**
		 * Initialize button for caliper tool
		 * @param {Measure.CaliperTool} caliperTool
		 */
		MeasureToolbarComponent.prototype.initializeCaliperButton = function (caliperTool) {
			if (!this.caliperButton) {
				// Caliper button
				this.caliperButton = new ViewerToolbarButton(this.f, "CaliperButton", "icon-WCR_Measure-Caliper2-cont", false);
				this.caliperButton.listenToTool(caliperTool, true);
				this.caliperButton.setIcon("icon-WCR_Measure-Caliper2-cont", ["icon-WCR_Measure-Caliper2-sub icon-red"]);
				this.caliperButton.setTooltip("viewer", "CaliperTT");

				this.caliperButton.addSubtoolbarComponent(new CaliperSubtoolbarComponent(this.f, caliperTool.getCaliperModel()));
			}
		};

		/**
		 * Initialize button for screen ruling tool
		 * @param {Measure.ScreenRulingMeasureTool} screenRulingTool
		 */
		MeasureToolbarComponent.prototype.initializeScreenRulingButton = function (screenRulingTool) {
			if (!this.screenRulingButton) {
				// Screen Ruling button
				this.screenRulingButton = new ViewerToolbarButton(this.f, "ScreenRulingButton", "icon icon-WCR_Measure-Screen", false);
				this.screenRulingButton.listenToTool(screenRulingTool, true);
				this.screenRulingButton.setTooltip("viewer", "ToolScreenTT");
			}
		};

		/**
		 * Check which buttons need to be shown and update button group + deactivate tool if needed
		 */
		MeasureToolbarComponent.prototype.updateTools = function () {
			// Tools
			this.f.emit("toolManager.activeTool", _.bind(function (activeTool) {
				// one of our tools if canBeUsedOnActivePage is defined?
				if (activeTool && activeTool.canBeUsedOnActivePage && !activeTool.canBeUsedOnActivePage()) {
					this.f.emit("toolManager.deactivateTool", activeTool);
				}
			}, this));

			// Buttons
			this.measureGroup.clearButtons();

			if (this.rulerTool && this.rulerTool.canBeUsedOnActivePage()) {
				this.initializeRulerButton(this.rulerTool);
				this.measureGroup.addButton(this.rulerButton);
			}

			if (this.eyedropperTool && this.eyedropperTool.canBeUsedOnActivePage()) {
				this.initializeEyeDropperButton(this.eyedropperTool);
				this.measureGroup.addButton(this.eyeDropperButton);
			}

			if (this.screenRulingTool && this.screenRulingTool.canBeUsedOnActivePage()) {
				this.initializeScreenRulingButton(this.screenRulingTool);
				this.measureGroup.addButton(this.screenRulingButton);
			}

			if (this.barcodeTool && this.barcodeTool.canBeUsedOnActivePage()) {
				this.initializeBarCodeButton(this.barcodeTool);
				this.measureGroup.addButton(this.barcodeButton);
			}

			if (this.brailleTool && this.brailleTool.canBeUsedOnActivePage()) {
				this.initializeBrailleButton(this.brailleTool);
				this.measureGroup.addButton(this.brailleButton);
			}

			if (this.caliperTool && this.caliperTool.canBeUsedOnActivePage()) {
				this.initializeCaliperButton(this.caliperTool);
				this.measureGroup.addButton(this.caliperButton);
			}

			this._emit("changed.content");
		};

		return MeasureToolbarComponent;
	});

/**
 * @namespace MeasureModule
 */
define('web-viewer/measure/MeasureModule',["underscore",
		"web-viewer/core/AbstractModule",
		"web-viewer/measure/RulerTool",
		"web-viewer/measure/BarcodeMeasureTool",
		"web-viewer/measure/BrailleMeasureTool",
		"web-viewer/measure/EyeDropperMeasureTool",
		"web-viewer/measure/CaliperTool",
		"web-viewer/measure/CaliperModel",
		"web-viewer/measure/ScreenRulingMeasureTool",
		"web-viewer/measure/MeasureToolbarComponent",
		"web-viewer/measure/MeasureConstants",
		"web-viewer/utils/ListenerUtil",
		"utils/XMLUtils",
		"jquery"
	],
	function (_,
		AbstractModule,
		RulerTool,
		BarcodeMeasureTool,
		BrailleMeasureTool,
		EyeDropperMeasureTool,
		CaliperTool,
		CaliperModel,
		ScreenRulingMeasureTool,
		MeasureToolbarComponent,
		MeasureConstants,
		ListenerUtil,
		XMLUtils,
		$) {

		"use strict";

		/**
		 * Create the MeasureModule
		 *
		 * @exports MeasureModule
		 * @memberof MeasureModule
		 * @class
		 * Responsible for publishing the correct tools and toolbarcomponents enabling navigation
		 * @augments Core.AbstractModule
		 * @mixes Utils.ListenerUtilMixin
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var MeasureModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "measure");
			ListenerUtil.mixin(this, f);

			$.when(this._checkBarCodeAndBrailleLicenses()).done(_.bind(function (hasBarCodeLicense, hasBrailleLicense) {
				this.tools = [];

				this.rulerTool = new RulerTool(f, "ruler");
				this._publishTool(this.rulerTool);
				this.tools.push(this.rulerTool);

				if (hasBarCodeLicense) {
					this.barcodeTool = new BarcodeMeasureTool(f, this);
					this._publishTool(this.barcodeTool);
					this.tools.push(this.barcodeTool);
				}

				if (hasBrailleLicense) {
					this.brailleTool = new BrailleMeasureTool(f, this);
					this._publishTool(this.brailleTool);
					this.tools.push(this.brailleTool);
				}

				this.eyedropperTool = new EyeDropperMeasureTool(f, "eyedropper");
				this._publishTool(this.eyedropperTool);
				this.tools.push(this.eyedropperTool);

				this.screenRulingTool = new ScreenRulingMeasureTool(f, "screenruling");
				this._publishTool(this.screenRulingTool);
				this.tools.push(this.screenRulingTool);

				this.caliperModel = new CaliperModel(f, [this.getPrivateChannelName(), "caliper", "model"]);
				this.caliperTool = new CaliperTool(f, this.caliperModel);
				this._publishTool(this.caliperTool);
				this.tools.push(this.caliperTool);

				this._publishToolbarComponent(MeasureToolbarComponent.toolbarName, _.bind(function () {
					this._measureToolbarComponent = new MeasureToolbarComponent(f, this.rulerTool, this.eyedropperTool, this.barcodeTool, this.brailleTool, this.caliperTool, this.screenRulingTool);
					return this._measureToolbarComponent;
				}, this));

				this._registerListener("module.graphics.model.changed.**", _.debounce(_.partial(this.updateMeasureTools, false), 10), this);
				this._registerListener("module.compare.referenceOffset.changed", _.partial(this.updateMeasureTools, true), this);
			}, this));
		};


		// inherit
		MeasureModule.prototype = Object.create(AbstractModule.prototype);
		// correct the constructor pointer 
		MeasureModule.prototype.constructor = MeasureModule;

		/**
		 * Updates the data displayed in the measure tool
		 * @param {Boolean} forceUpdate - forces the update of the data in the measure tool
		 */
		MeasureModule.prototype.updateMeasureTools = function (forceUpdate) {
			_.each(this.tools, function (tool) {
				if (tool.isActive()) {
					tool.updateIfNeeded(forceUpdate);
				}
			});
		};

		/**
		 * Get the state of all the measure tools
		 * @returns {Object}
		 */
		MeasureModule.prototype.getState = function () {
			var measureState = {
				rulerState: this.rulerTool.getToolState(),
				eyedropperState: this.eyedropperTool.getToolState(),
				caliperState: this.caliperTool.getToolState(),
				sizeUnitState: this.f.getOption("PREFERENCES_UNIT_SIZE")
			};

			if (this.barcodeTool) {
				measureState.barcodeState = this.barcodeTool.getToolState();
			}
			if (this.brailleTool) {
				measureState.brailleState = this.brailleTool.getToolState();
			}

			return measureState;
		};

		/**
		 * Set the state of the measure tools
		 * @param {Object} state
		 */
		MeasureModule.prototype.setState = function (state) {
			if (state.rulerState !== undefined) {
				if (state.rulerState.isActive) {
					this.f.emit("toolManager.activateTool", this.rulerTool);
				}
				this.rulerTool.setToolState(state.rulerState);
				this.f.emit("toolManager.interruptTools", true);
			}
			if (state.barcodeState !== undefined && this.barcodeTool) {
				this.barcodeTool.setToolState(state.barcodeState);
			}
			if (state.brailleState !== undefined && this.brailleTool) {
				this.brailleTool.setToolState(state.brailleState);
			}
			if (state.eyedropperState !== undefined) {
				this.eyedropperTool.setToolState(state.eyedropperState);
			}
			if (state.caliperState !== undefined) {
				this.caliperTool.setToolState(state.caliperState);
			}
			if (state.sizeUnitState !== undefined) {
				this.f.setOption("PREFERENCES_UNIT_SIZE", state.sizeUnitState);
			}

		};

		/**
		 * Check licenses for barcode and braille
		 * @returns {$.Deferred.<Boolean, Boolean>} Two parameters where the first one if true indicates has barcode license 
		 *                                          and second one if true indicates has braille license
		 */
		MeasureModule.prototype._checkBarCodeAndBrailleLicenses = function () {
			var deferred = $.Deferred();

			$.when(this._getServerInfoFlags()).done(function (flags) {
				var hasBarCodeLicense = (flags & MeasureConstants.ServerInfoFlags.barCode) === MeasureConstants.ServerInfoFlags.barCode ||
					(flags & MeasureConstants.ServerInfoFlags.barCodeGV) === MeasureConstants.ServerInfoFlags.barCodeGV;
				var hasBrailleLicense = (flags & MeasureConstants.ServerInfoFlags.braille) === MeasureConstants.ServerInfoFlags.braille;

				deferred.resolve(hasBarCodeLicense, hasBrailleLicense);
			});

			return deferred.promise();
		};


		/**
		 * Get the ServerInfo flags (can be used for checking licenses)
		 * @return {$.Deferred.<Number>} Flags number. 0 if something went wrong.
		 */
		MeasureModule.prototype._getServerInfoFlags = function () {
			var deferred = $.Deferred();
			if (this._serverFlags !== undefined) {
				deferred.resolve(this._serverFlags);
			} else {
				this.f.ajax(this.f.getDataAdapter().ViewServer("getserverinfo.jsp"), {
					success: _.bind(function (result) {
						if (result) {
							result = XMLUtils.xml2obj(result);
							if (result && result.status && result.status.value == "0") {
								if (result.status.server) {
									this._serverFlags = parseInt(result.status.server.flags, 10);
									deferred.resolve(this._serverFlags);
									return;
								}
							}
						}

						this.f.emit("error.WARNING", "Problem while fetching server flags");
						deferred.resolve(0);
					}, this),
					error: _.bind(function (jqXHR, textStatus, errorThrown) {
						// Error while fetching licensing info
						this.f.emit("error.WARNING", "Problem while fetching server flags", textStatus, errorThrown);
						deferred.resolve(0);
					}, this)
				});
			}
			return deferred.promise();
		};

		return MeasureModule;
	});

