define('web-viewer/print/PrintModel',["web-viewer/core/PrivateChannel",
		"web-viewer/print/PrintConstants",
		"web-viewer/core/AbstractModel",
		"web-viewer/data/DataConstants",
		"web-viewer/utils/ViewerUtils",
		"jquery",
		"underscore"
	],
	function (
		PrivateChannel,
		PrintConstants,
		AbstractModel,
		DataConstants,
		ViewerUtils,
		$) {
		"use strict";
		/**
		 * @exports PrintModel
		 * @memberof PrintModule
		 * @class PrintModel holds all the information for printing. It saves all printing options and persists them to user preferences.
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f - Viewer facade
		 */
		var PrintModel = function (f) {
			AbstractModel.call(this, f, "printModel");
			this.f = f;
			/**
			 * @member {PrintConstants.PrintType} - print mode used
			 * @private
			 */
			this._printType = this.f.getCurrentUser().getOption("PREFERENCES_PRINT_TYPE");

			/**
			 * @member {PrintConstants.PrintArea} - what image area is used for print
			 * @private
			 */
			this._printArea = this.f.getCurrentUser().getOption("PREFERENCES_PRINT_AREA");

			/**
			 * @member {PrintConstants.PrintChannels} - print all channels or only selected channels
			 * @private
			 */
			this._printChannels = this.f.getCurrentUser().getOption("PREFERENCES_PRINT_CHANNELS");

			/**
			 * @member {Number} the resolution to print
			 * @private
			 */
			this._printResolution = ViewerUtils.parseFloat(this.f.getCurrentUser().getOption("PREFERENCES_PRINT_RESOLUTION"));

			/**
			 * @member {CoreConstants.ResolutionUnits}
			 */
			this._printResolutionUnit = this.f.getCurrentUser().getOption("PREFERENCES_UNIT_RESOLUTION");

			/**
			 * @member {PrintConstants.PrintAnnotationsSource}
			 * @private
			 */
			this._printAnnotationsSource = PrintConstants.PrintAnnotationsSource.PRINTANNOTATIONSCURRENT;

			/**
			 * @member {Boolean} - is the viewport zoomed in, for print area the option current view area is added
			 * @private
			 */
			this._isViewportZoomed = false;

			/**
			 * @member {Boolean} - is there a selection (measure tools), for print area the option current selection is added
			 * @private
			 */
			this._hasViewportActiveSelection = false;

			/**
			 * @member {Boolean} - are all the channels selected, options to print all channels or only selected channels is added
			 * @private
			 */
			this._areAllChannelsSelected = true;

			/**
			 * @member {DataConstants.documentPageType}
			 * @private
			 */
			this._fileType = null;

			/**
			 * @member {Boolean} - Is reader spread active
			 * @private
			 */
			this._readerSpreadActive = false;
		};

		/**
		 * Get the print type
		 * @returns {PrintConstants.PrintType}
		 */
		PrintModel.prototype.getPrintType = function () {
			return this._printType;
		};

		/**
		 * Get the print area option
		 * @returns {PrintConstants.PrintArea}
		 */
		PrintModel.prototype.getPrintArea = function () {
			return this._printArea;
		};

		/**
		 * Get the option for print channels
		 * @returns {PrintConstants.PrintChannels}
		 */
		PrintModel.prototype.getPrintChannels = function () {
			return this._printChannels;
		};

		/**
		 * Get the resolution to print
		 * @returns {Number} - in PPMM
		 */
		PrintModel.prototype.getResolution = function () {
			return this._printResolution;
		};

		/**
		 * Get the unit for the print resolution
		 * @returns {CoreConstants.ResolutionUnits}
		 */
		PrintModel.prototype.getResolutionUnit = function () {
			return this._printResolutionUnit;
		};

		/**
		 * Get the annotation source to print
		 * @returns {PrintConstants.PrintAnnotationsSource}
		 */
		PrintModel.prototype.getPrintAnnotationsSource = function () {
			return this._printAnnotationsSource;
		};

		/**
		 * Does the view have to show the options for print area
		 * @returns {Boolean}
		 */
		PrintModel.prototype.showPrintArea = function () {
			return this._printType === PrintConstants.PrintType.PRINTIMAGEAREA && (this._hasViewportActiveSelection || this._isViewportZoomed);
		};

		/**
		 * Does the view have to show the options to print all channels or only the selected channels
		 * @returns {Boolean}
		 */
		PrintModel.prototype.showPrintChannels = function () {
			return !this._areAllChannelsSelected && this._printType !== PrintConstants.PrintType.PRINTANNOTATIONOVERVIEW;
		};

		/**
		 * Does the view have to show the option to specify the resolution
		 * @returns {Boolean}
		 */
		PrintModel.prototype.showResolution = function () {
			return this._printType === PrintConstants.PrintType.PRINTIMAGEAREA && (this._fileType === DataConstants.documentPageType.GRAPHICS);
		};

		/**
		 * Set the print mode
		 * @param {PrintConstants.PrintType} printType
		 */
		PrintModel.prototype.setPrintType = function (printType) {
			this._printType = printType;
			this.f.getCurrentUser().setOption("PREFERENCES_PRINT_TYPE", this._printType);
		};

		/**
		 * Set the printing area
		 * @param {PrintConstants.PrintArea} printArea
		 */
		PrintModel.prototype.setPrintArea = function (printArea) {
			this._printArea = printArea;
			this.f.getCurrentUser().setOption("PREFERENCES_PRINT_AREA", this._printArea);
		};

		/**
		 * Set the print channels option
		 * @param {PrintConstants.PrintChannels} printChannels
		 */
		PrintModel.prototype.setPrintChannels = function (printChannels) {
			this._printChannels = printChannels;
			this.f.getCurrentUser().setOption("PREFERENCES_PRINT_CHANNELS", this._printChannels);
		};

		/**
		 * Set the resolution to print in
		 * @param {Number} resolution -  in PPMM
		 */
		PrintModel.prototype.setPrintResolution = function (resolution) {
			this._printResolution = resolution;
			this.f.getCurrentUser().setOption("PREFERENCES_PRINT_RESOLUTION", this._printResolution + "");
		};

		/**
		 * Set the resolution unit to print
		 * @param {CoreConstants.ResolutionUnits} resolutionUnit
		 */
		PrintModel.prototype.setPrintResolutionUnit = function (resolutionUnit) {
			this._printResolutionUnit = resolutionUnit;
		};

		/**
		 * Set the annotation source to print
		 * @param {PrintConstants.PrintAnnotationsSource} printAnnotationsSource
		 */
		PrintModel.prototype.setPrintAnnotationsSource = function (printAnnotationsSource) {
			this._printAnnotationsSource = printAnnotationsSource;
		};

		/**
		 * Set the if the viewport is zoomed in, to show options for image area printing mode
		 * @param {Boolean} isZoomed
		 */
		PrintModel.prototype.setViewportZoomed = function (isZoomed) {
			this._isViewportZoomed = isZoomed;
		};

		/**
		 * set hasViewportActiveSelection, for when a selection (measure tools) is made or not
		 * @deprecated
		 * @param {Boolean} hasViewportActiveSelection
		 */
		PrintModel.prototype.setViewportActiveSelection = function (hasViewportActiveSelection) {
			this._hasViewportActiveSelection = hasViewportActiveSelection;
		};

		/**
		 * set if all channels are selected
		 * @param {Boolean} areAllChannelsSelected
		 */
		PrintModel.prototype.setAreAllChannelsSelected = function (areAllChannelsSelected) {
			this._areAllChannelsSelected = areAllChannelsSelected;
		};

		/**
		 * Show the print annotation source options
		 * @returns {Boolean} True if it needs to be shown, false otherwise
		 */
		PrintModel.prototype.showPrintAnnotationsSource = function () {
			return this._printType === PrintConstants.PrintType.PRINTANNOTATIONOVERVIEW &&
				this.f.getDocumentController().getDifferencePage() && !this.isReaderSpreadActive();
			// give the choice if there is a diff page too and it's not in readerspread
		};

		/**
		 * Set the page type that's being printed
		 * @param {DataConstants.documentPageType} pageType
		 */
		PrintModel.prototype.setPageType = function (fileType) {
			this._fileType = fileType;
		};

		/**
		 * Get the type of the file that's being printed
		 * @returns {DataConstants.documentPageType}
		 */
		PrintModel.prototype.getPageType = function () {
			return this._fileType;
		};

		/**
		 * Set whether reader spread is active
		 * @param {Boolean} readerSpreadActive
		 */
		PrintModel.prototype.setReaderSpreadActive = function (readerSpreadActive) {
			this._readerSpreadActive = readerSpreadActive;
		};

		/**
		 * Get whether reader spread is active
		 * @returns {Boolean} True if active, false otherwise
		 */
		PrintModel.prototype.isReaderSpreadActive = function () {
			return this._readerSpreadActive;
		};

		return PrintModel;
	});


define('tpl!web-viewer/print/PrintPopup', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<h2>'+
((__t=(d.i18n("PrintType")))==null?'':_.escape(__t))+
'</h2>\r\n<div class="printTypeSettings">\r\n	<div class="printType">\r\n		<input type="radio" name="printType" value="0" ';
 if(d.printType===0){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("PrintTypeCurrentView")))==null?'':_.escape(__t))+
' \r\n	';
 if(d.showPrintImageArea){ 
__p+='\r\n		<br/><input type="radio" name="printType" value="1" ';
 if(d.printType===1){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("PrintTypeImageArea")))==null?'':_.escape(__t))+
'\r\n	';
 } 
__p+='\r\n		<br/><input type="radio" name="printType" value="2" ';
 if(d.printType===2){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("PrintTypeAnnotationOverview")))==null?'':_.escape(__t))+
'\r\n	</div>\r\n</div>\r\n';
 if(d.showResolution || d.showPrintArea || d.showPrintChannels || d.showPrintAnnotationsSource){ 
__p+='\r\n	<h2>'+
((__t=(d.i18n("PrintParameters")))==null?'':_.escape(__t))+
'</h2>\r\n	<div class="printOptions">	\r\n		';
 if(d.showResolution){ 
__p+='\r\n			'+
((__t=(d.i18n("InfoResolution")))==null?'':_.escape(__t))+
'<input value="'+
((__t=(d.printResolution))==null?'':_.escape(__t))+
'" type="text" name="printResolution" class="printResolution"></input>\r\n			<select name="printResolutionUnit">\r\n				<option value="0" ';
 if(d.printResolutionUnit===0 ){
__p+='selected';
}
__p+='>'+
((__t=(d.i18n("UnitResInch")))==null?'':_.escape(__t))+
'</option>\r\n				<option value="1" ';
 if(d.printResolutionUnit===1 ){
__p+='selected';
}
__p+='>'+
((__t=(d.i18n("UnitResmm")))==null?'':_.escape(__t))+
'</option>\r\n				<option value="2" ';
 if(d.printResolutionUnit===2 ){
__p+='selected';
}
__p+='>'+
((__t=(d.i18n("UnitRescm")))==null?'':_.escape(__t))+
'</option>\r\n			</select>\r\n		';
 } 
__p+='\r\n		';
 if(d.showPrintArea){ 
__p+=' \r\n			<div class="printArea">\r\n				<input type="checkbox" name="printArea" ';
 if(d.printArea===1){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("PrintCurrentViewArea")))==null?'':_.escape(__t))+
' \r\n			</div>\r\n		';
 } 
__p+='\r\n		';
 if(d.showPrintChannels){ 
__p+='\r\n			<div class="printChannels">\r\n				<input type="checkbox" name="printChannels" ';
 if(d.printChannels===1){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("PrintSelectedChannels")))==null?'':_.escape(__t))+
' \r\n			</div>\r\n		';
 } 
__p+='\r\n		';
 if (d.showPrintAnnotationsSource) { 
__p+='\r\n			<div class="printAnnotationsSource">\r\n				<input type="radio" name="printAnnotationsSource" value="0" ';
 if(d.printAnnotationsSource===0){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("PrintAnnotationsCurrent")))==null?'':_.escape(__t))+
' <br/>\r\n				<input type="radio" name="printAnnotationsSource" value="1" ';
 if(d.printAnnotationsSource===1){ 
__p+=' checked ';
}
__p+=' /> '+
((__t=(d.i18n("PrintAnnotationsReference")))==null?'':_.escape(__t))+
' \r\n			</div>\r\n		';
 } 
__p+='\r\n	</div>\r\n';
 } 
__p+='';
return __p;
}});
define('web-viewer/print/PrintPopup',["web-viewer/core/PrivateChannel",
		"web-viewer/layout/component/ModalPopup",
		"web-viewer/print/PrintConstants",
		"web-viewer/data/DataConstants",
		"Template",
		"tpl!web-viewer/print/PrintPopup",
		"web-viewer/utils/ViewerUtils",
		"jquery",
		"underscore"
	],
	function (PrivateChannel,
		ModalPopup,
		PrintConstants,
		DataConstants,
		Template,
		tpl,
		ViewerUtils,
		$,
		_) {
		"use strict";
		/**
		 * @exports PrintPopup
		 * @memberof PrintModule
		 * @augments layout.component.ModelPopup
		 * @class Popup for printing, shows all the options for printing and
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {String} id - Id of the popup. It is used in its private channel.
		 * @param {PrintModel} printModel
		 */
		var PrintPopup = function (f, id, printModel) {
			ModalPopup.call(this, f, id, f.getContext());
			PrivateChannel.installTo(this, f, ['printPopup']);

			f.i18n.loadBundle("viewer", _.bind(function () {
				this.setHeader($("<span/>").addClass("popupTitle").html(f.i18n.t("PrintSetupTitle")));

				this.setOkBtnText(f.i18n.t("PrintPrintButton"));
				this.setOkAction(_.bind(function () {
					this._emit("print");
				}, this));
			}, this));

			this.element.addClass("printPopup");

			/**
			 * @private
			 * @member {PrintModel} - model used to construct the popup
			 */
			this._printModel = printModel;

			/**
			 * @member {Template} template for the popup
			 */
			this.t = new Template(this.f.i18n, tpl);
			this.t.setContext(this._popupContent);

			//action for the radio buttons to change the print type, emits printPopup.printType
			this.t.bind("change", "[name=printType]", function (source, event) {
				this._emit("printType", parseInt(source.value, 10));
			}, this);

			//action for the checkbox to change the print area, emits printPopup.printArea
			this.t.bind("change", "[name=printArea]", function (source, event) {
				this._emit("printArea", source.checked);
			}, this);

			//action for the checkbox to change the channels to be printed, emits printPopup.printChannels
			this.t.bind("change", "[name=printChannels]", function (source, event) {
				this._emit("printChannels", source.checked);
			}, this);

			//action for the input field for the resolution, emits printPopup.resolutionChanged
			this.t.bind("input", "[name=printResolution]", _.debounce(_.partial(function (that, source, event) {
				var value = ViewerUtils.parseFloat(source.value);
				if (isNaN(value)) { // if nothing is filled in or only letter(s), behave like entered the minimum
					value = 0;
				}
				source.value = value; // by doing this letters added to the end will be filtered out
				that._emit("resolutionChanged", value);
			}, this), 400));

			this.t.bind("change", "[name=printResolutionUnit]", function (source, event) {
				this._emit("resolutionUnitChanged", parseInt(source.value, 10));
			}, this);

			//action for the radio buttons to select the annotations of the current/reference
			this.t.bind("change", "[name=printAnnotationsSource]", function (source, event) {
				this._emit("printAnnotationsSource", parseInt(source.value, 10));
			}, this);

		};

		PrintPopup.prototype = Object.create(ModalPopup.prototype);
		PrintPopup.prototype.constructor = PrintPopup;

		/**
		 * Renders the popup for channel matching, it's a throttled function
		 */
		PrintPopup.prototype.render = _.throttle(function () {
			this.t.d.printType = this._printModel.getPrintType();
			this.t.d.showResolution = this._printModel.showResolution();
			this.t.d.printResolutionUnit = this._printModel.getResolutionUnit();
			this.t.d.showPrintImageArea = this._printModel.getPageType() === DataConstants.documentPageType.GRAPHICS;
			this.t.d.printResolution = ViewerUtils.convertResolutionFromPpmmValue(this._printModel.getResolution(), this._printModel.getResolutionUnit(), this.f.getResolutionUnitsFix());
			this.t.d.showPrintArea = this._printModel.showPrintArea();
			this.t.d.printArea = this._printModel.getPrintArea();
			this.t.d.showPrintChannels = this._printModel.showPrintChannels();
			this.t.d.printChannels = this._printModel.getPrintChannels();
			this.t.d.showPrintAnnotationsSource = this._printModel.showPrintAnnotationsSource();
			this.t.d.printAnnotationsSource = this._printModel.getPrintAnnotationsSource();

			this.t.render();
			if (this._isInitialized) {
				this.recalculateSize(300);
			}
		}, 40);

		/**
		 * Show the modal popup
		 */
		PrintPopup.prototype.show = function () {
			this._printModel.setPrintResolutionUnit(this.f.getOption("PREFERENCES_UNIT_RESOLUTION"));
			this.render();
			ModalPopup.prototype.show.call(this);
			this.recalculateSize(300);
		};

		return PrintPopup;
	});

/**
 * @namespace PrintModule
 */
define('web-viewer/print/PrintModule',["web-viewer/core/AbstractModule",
		"web-viewer/core/CoreConstants",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/print/PrintConstants",
		"web-viewer/print/PrintModel",
		"web-viewer/print/PrintPopup",
		"utils/HTTPUtils",
		"web-viewer/utils/ViewerUtils",
		"utils/ShapeUtils",
		"web-viewer/data/DataConstants",
		"jquery",
		"underscore"
	],
	function (AbstractModule,
		CoreConstants,
		ListenerUtil,
		PrintConstants,
		PrintModel,
		PrintPopup,
		HTTPUtils,
		ViewerUtils,
		ShapeUtils,
		DataConstants,
		$,
		_
	) {
		"use strict";

		/**
		 * Initialize the Print Module
		 *
		 * @exports PrintModule
		 * @memberof PrintModule
		 * @class
		 * The print module enables printing in the viewer.
		 * @constructor
		 * @augments {Core.AbstractModule}
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced - if true, the module shouldn't publish tools/widgets/toolbarcomponents
		 */
		var PrintModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "print");
			ListenerUtil.mixin(this, f);
			this.f = f;
			/**
			 * @member {PrintModule.PrintModel}
			 */
			this._printModel = null;

			/**
			 * @member {PrintModule.PrintPopup}
			 */
			this._printPopup = null;
		};

		PrintModule.prototype = Object.create(AbstractModule.prototype);
		PrintModule.prototype.constructor = PrintModule;

		/**
		 * Opens the popup for print
		 */
		PrintModule.prototype.openPrintPopup = function () {
			this._initializePrintModel();
			this._initializePrintPopup();

			this._printPopup.show();

			this._checkForUnsavedAnnotations();
		};

		/**
		 * initializes the print model with correct options which it gets from the state of other modules
		 */
		PrintModule.prototype._initializePrintModel = function () {
			var isZoomed = false;
			var areAllChannelsSelected = true;

			if (this._printModel === null) {
				this._printModel = new PrintModel(this.f);
			}

			var state;
			//get state of the viewport
			var viewportModule = this.f.getModule("viewport");
			if (viewportModule !== undefined) {
				state = viewportModule.getState();
				isZoomed = !state.world.isFullView;
				this._printModel.setViewportZoomed(isZoomed);
			}

			//get state of the graphics 
			var graphicsModule = this.f.getModule("graphics");
			if (graphicsModule) {
				state = graphicsModule.getState();
				var channels = state.graphicsModel.channels;
				areAllChannelsSelected = !_.any(channels, function (channel) {
					return !channel.enabled;
				});
				this._printModel.setAreAllChannelsSelected(areAllChannelsSelected);
			}

			//for when viewed file is CAD, no channels!
			var activePage = this.f.getDocumentController().getActivePage();
			if (activePage.hasCad()) { //the active page has CAD
				this._printModel.setPageType(DataConstants.documentPageType.CAD);
				this._printModel.setAreAllChannelsSelected(true); //to make sure the option to print all/selected annotations is not shown
			} else if (activePage.hasSvg && activePage.hasSvg()) {
				this._printModel.setPageType(DataConstants.documentPageType.SVG);
			} else {
				this._printModel.setPageType(DataConstants.documentPageType.GRAPHICS);
			}

			// get if reader spread is active
			var compareModule = this.f.getModule("compare");
			if (compareModule) {
				this._printModel.setReaderSpreadActive(compareModule.isReaderSpreadActive());
			}
		};

		/**
		 * initializes the print popup and registers all the listeners if the popup was not yet initialized
		 */
		PrintModule.prototype._initializePrintPopup = function () {
			if (this._printPopup === null) {
				this._printPopup = new PrintPopup(this.f, "printPopup", this._printModel);

				//Register all the listeners to the channelMatchingPopup
				this._registerListener("print", this._showPrintPreview, this, this._printPopup);

				this._registerListener("printType", this._printTypeChanged, this, this._printPopup);
				this._registerListener("printArea", this._printAreaChanged, this, this._printPopup);
				this._registerListener("printChannels", this._printChannelsChanged, this, this._printPopup);
				this._registerListener("resolutionChanged", this._printResolutionChanged, this, this._printPopup);
				this._registerListener("printAnnotationsSource", this._printAnnotationsSourceChanged, this, this._printPopup);
				this._registerListener("resolutionUnitChanged", this._printResolutionUnitChanged, this, this._printPopup);

				this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_RESOLUTION", _.bind(function (oldValue, newValue) {
					this._printResolutionUnitChanged(newValue);
				}, this));
			}
		};

		/**
		 * Action for when the print type changes, sets the correct value in the print model
		 * @param {PrintConstants.PrintType} printType
		 */
		PrintModule.prototype._printTypeChanged = function (printType) {
			this._printModel.setPrintType(printType);
			this._printPopup.render();
		};

		/**
		 * Action for when the print area option changes, sets the correct value in the print model
		 * @param {Boolean} printCurrentView - print only the current view
		 */
		PrintModule.prototype._printAreaChanged = function (printCurrentView) {
			if (printCurrentView) {
				this._printModel.setPrintArea(PrintConstants.PrintArea.PRINTCURRENTVIEWAREA);
			} else {
				this._printModel.setPrintArea(PrintConstants.PrintArea.PRINTENTIREIMAGE);
			}
		};

		/**
		 * Action for when the print channel option changes, sets the correct value in the print model
		 * @param {Boolean} printOnlySelectedChannels - print only the selected channels
		 */
		PrintModule.prototype._printChannelsChanged = function (printOnlySelectedChannels) {
			if (printOnlySelectedChannels) {
				this._printModel.setPrintChannels(PrintConstants.PrintChannels.PRINTSELECTEDCHANNELS);
			} else {
				this._printModel.setPrintChannels(PrintConstants.PrintChannels.PRINTALLCHANNELS);
			}
		};

		/**
		 * Action for when the print annotations source option changes, sets the correct value in the print model
		 * @param {PrintConstants.PrintAnnotationsSource} printAnnotations
		 */
		PrintModule.prototype._printAnnotationsSourceChanged = function (printAnnotationsSource) {
			this._printModel.setPrintAnnotationsSource(printAnnotationsSource);
		};

		/**
		 * Action for when the print resolution changes, sets the correct value in the print model
		 * @param {Number} resolution - in either ppi, ppcm or ppmm
		 */
		PrintModule.prototype._printResolutionChanged = function (resolution) {
			var resInPpmm = ViewerUtils.convertResolutionToPpmmValue(resolution, this._printModel.getResolutionUnit());
			this._printModel.setPrintResolution(ViewerUtils.parseFloat(resInPpmm));
		};

		/**
		 * Action for when the print resolution unit changed
		 * @param {CoreConstants.ResolutionUnits} resolutionUnit
		 */
		PrintModule.prototype._printResolutionUnitChanged = function (resolutionUnit) {
			this._printModel.setPrintResolutionUnit(resolutionUnit);
			if (this._printPopup && this._printPopup.isVisible()) {
				this._printPopup.render();
			}
		};

		/**
		 * Check for unsaved annotations and asks the user to save them if needed.
		 */
		PrintModule.prototype._checkForUnsavedAnnotations = function () {
			// check whether there are unsaved annotations
			var annotationModule = this.f.getModule("annotation");
			if (annotationModule !== undefined) {
				var hasUnsavedItems = annotationModule.promptToSaveUnsavedAnnotations(_.bind(function () {
					this._printPopup.show(); //show the printpopup, the annotations are saved
				}, this), function () {
					//don't do anything, cancel will not show the print popup and won't save the annotations
				}, _.bind(function () {
					annotationModule.reload(); //Don't save the annotations, reload them
					this._printPopup.show(); //now show the print popup
				}, this));

				if (hasUnsavedItems) {
					// hide the printpopup while we are prompting for saving unsaved annotations
					this._printPopup.hide();
				}
			}
		};

		/**
		 * Show the print preview, it will check which print type is selected to open the correct print preview
		 */
		PrintModule.prototype._showPrintPreview = function () {
			//get the document information from the documentcontroller
			var activePage = this.f.getDocumentController().getActivePage();
			if (this._printModel.getPrintType() === PrintConstants.PrintType.PRINTANNOTATIONOVERVIEW) {
				this._printAnnotations();
			} else if (this._printModel.getPrintType() === PrintConstants.PrintType.PRINTIMAGEAREA && !activePage.hasCad()) {
				this._printImageArea();
			} else { //print what is currently visible, and image area for CAD
				this._printCurrentView();
			}
		};

		/**
		 * Print the annotations overview
		 * @private
		 */
		PrintModule.prototype._printAnnotations = function () {
			if (!this._printModel.showPrintAnnotationsSource()) {
				// no choice, so take current
				this._printModel.setPrintAnnotationsSource(PrintConstants.PrintAnnotationsSource.PRINTANNOTATIONSCURRENT);
			}
			var docVersionID = null;
			if (this._printModel.getPrintAnnotationsSource() === PrintConstants.PrintAnnotationsSource.PRINTANNOTATIONSCURRENT) {
				var activePage = this.f.getDocumentController().getActivePage();
				if (activePage !== undefined) {
					docVersionID = activePage.getDocumentVersion().get("docversionid");
				} else {
					this.f.emit("error.SEVERE", "No active page selected to print");
					return;
				}
			} else {
				var referencePage = this.f.getDocumentController().getDifferencePage();
				if (referencePage !== undefined) {
					docVersionID = referencePage.getDocumentVersion().get("docversionid");
				} else {
					this.f.emit("error.SEVERE", "No reference page selected to print");
					return;
				}
			}
			var url = this._getHostURL() + "docdetailsannotations.jsp?docVerID=" + docVersionID + "&reportStyle=2&thumbnailSize=200&userColors=1&reviewState=0&userName=0&print=1";

			if (this.f.getProperty("showuser")) {
				_.each(this.f.getProperty("showuser").split("|"), function (showuser) {
					if (showuser.length > 0) {
						url += "&showUser=" + showuser;
					}
				}, this);
			}

			window.open(url, "_blank");
		};

		/**
		 * Print the image area
		 * @private
		 */
		PrintModule.prototype._printImageArea = function () {
			//get the document information from the documentcontroller
			var activePage = this.f.getDocumentController().getActivePage();
			var viewerState = this.f.getViewerState(); //get the state of the viewer
			var worldSize = viewerState.viewport.world.worldBounds.size;
			var worldState = viewerState.viewport.world;
			//open a new window (or tab, browsers choice)
			var printPreview = window.open("", "_blank");
			//the rect that is requested from the viewserver
			var viewRect = new ShapeUtils.Rect(new ShapeUtils.Point(0, 0), worldSize);
			if (this._printModel.getPrintArea() === PrintConstants.PrintArea.PRINTCURRENTVIEWAREA) {
				//when only the current view area needs to be printed, change the view rectangle to the visible rect
				viewRect = worldState.visibleRect;
			}
			//set the basic context for the fragment url intercepter
			var context = {
				rectangle: viewRect,
				resolution: this._printModel.getResolution(),
				// the servlet that should be used to fetch the image information
				servlet: "",
				// the parameters used for requesting the image
				params: {
					x1: 0,
					x2: 0,
					y1: 0,
					y2: 0
				}
			};

			// ask all interceptors to update the context. All listeners have access to the context object and can adapt the information that is in the object
			// there is no predefined order in which these interceptors are called, their actions thus should not overlap!
			this.f.emit("intercept.fragmentURL", context);
			//make sure to remove the diff document page from the params
			delete context.params.diff;
			//if all channels need to be visible, but not all are selected remove the color from the context parameters
			if (this._printModel.getPrintChannels() === PrintConstants.PrintChannels.PRINTALLCHANNELS) {
				delete context.params.color;
			}

			// generate URL
			var url = HTTPUtils.constructURL(context.servlet, context.params);

			// ViewServer can't handle urlencoded commas, fix commas 
			url = url.replace(/%2C/g, ",");

			//create the img tag to display the requested image, it will resize to the worldSize of the document (or only the visible part) 
			// but it still prints in the requested resolution! The browser decides how to translate mm to pixels
			var image = null;
			if (this._printModel.showPrintArea() && this._printModel.getPrintArea() === PrintConstants.PrintArea.PRINTCURRENTVIEWAREA) {
				//if its a viewarea, only the size of the viewarea is needed (in world coordinates)
				image = "<img style='height:" + worldState.visibleRect.size.height + "mm; width:" + worldState.visibleRect.size.width + "mm;' src='" + this._getBaseURL() + url + "' />";
			} else {
				image = "<img style='height:" + worldState.worldBounds.size.height + "mm; width:" + worldState.worldBounds.size.width + "mm;' src='" + this._getBaseURL() + url + "' />";
			}

			this.f.i18n.loadBundle("viewer", _.bind(function () {
				var printButton = "<a class='noPrint printBtn' onclick='window.print()'>" + this.f.i18n.t("PrintTT") + "</a><br/>";
				var css = "<link href='" + this._getBaseURL() + "/wcr-static/web-viewer/layout/desktop/css/desktop.css' type='text/css' rel='stylesheet'></link>";

				//write the img-tag to the window
				printPreview.document.open();
				printPreview.document.write("<!doctype html><html><head>" + css + "<title>Image Area - " + activePage.getDocument().get("docname") + "</title></head>");
				printPreview.document.write("<body>" + printButton + image + "</body></html>");
				printPreview.document.close();
			}, this));
		};

		/**
		 * Print the current view
		 * @private
		 */
		PrintModule.prototype._printCurrentView = function () {


			//first check if not all channels are selected
			if (this._printModel.showPrintChannels()) {
				var printAllChannels = this._printModel.getPrintChannels(); //get the setting for printing all or only selected channels
				if (printAllChannels === PrintConstants.PrintChannels.PRINTALLCHANNELS && this.f.getModule("graphics")) { //print all channels
					// enable all channels before showing the print popup
					// set channels
					var graphicsModule = this.f.getModule("graphics");

					var selectionBefore = graphicsModule.getChannelSelection();
					var allSelected = {};
					_.each(graphicsModule.getChannelSelection(), function (selected, ID) {
						allSelected[ID] = true;
					});

					graphicsModule.setChannelSelection(allSelected);


					this._openWindowForPrintCurrentView(function () {
						graphicsModule.setChannelSelection(selectionBefore);
					});

					return;
				}
			}

			// no need to wait for stuff, just open the print popup
			this._openWindowForPrintCurrentView();

		};

		/**
		 * This opens the popup for printing the current view
		 * Once all image data has been loaded, the canvas data is copied to the print popup canvases
		 * @param {Function} afterInitialization - callback called when the printpopup is correctly initialized and the canvas data is transferred
		 */
		PrintModule.prototype._openWindowForPrintCurrentView = function (afterInitialization) {
			var activePage = this.f.getDocumentController().getActivePage();
			var referencePage = this.f.getDocumentController().getDifferencePage();
			var docNames = activePage.getDocument().get("docname");
			if (referencePage !== undefined) {
				docNames += "-" + referencePage.getDocument().get("docname");
			}

			var printPreview = window.open("", "_blank");

			// it is safe to get the viewport like this, otherwise we wouldn't be printing :)
			var viewportModule = this.f.getModule("viewport");
			var viewportElement = viewportModule.widget.getElement();

			// generate the HTML for the print popup
			var sceneSize = {
				width: viewportElement.width(),
				height: viewportElement.height()
			};
			var css = "<link href='" + this._getBaseURL() + "/wcr-static/web-viewer/layout/print/css/print.css' type='text/css' rel='stylesheet'></link>";
			var viewport = "<div class='viewport' id='viewport'></div>";
			var centerPane = "<div class='centerPane' style='display: block;height:" + sceneSize.height + "px;width:" + sceneSize.width + "px;'>" + viewport + "</div>";
			var viewerDiv = "<div class='viewerApp' id='viewerApp' style='height:" + sceneSize.height + "px;width:" + sceneSize.width + "px;'>" + centerPane + "</div>";

			//add the css-file to the header and the div to the body of the document of the new window
			printPreview.document.open();
			printPreview.document.write("<!doctype html><html><head>" + css + "<title>Current View - " + docNames + "</title></head>");
			var loadingIndicator = "<img src='" + this._getBaseURL() + "/wcr-static/web-viewer/layout/img/loaderTransparent.gif' alt='Loading...' class='noprint' id='printLoadingIndicator' style='margin-left: 10px' />";
			var header = "<div class='noPrint' style='line-height: 20px'><a class='printBtn' onclick='window.print()'> Print</a>" + loadingIndicator + "</div>";
			printPreview.document.write("<body>" + header + "<div class='printPreview'>" + viewerDiv + "</div></body></html>");
			printPreview.document.close();


			setTimeout(_.bind(function () { // wait for update of channel selection (this is debounced at 40ms)
				// wait until all fragments have finished loading
				viewportModule.whenFinishedLoadingImageData(function () {
					setTimeout(_.bind(function () { // wait for the rendering of fragments (rendering is throttled at 40ms)
						// copy over the HTML of the viewport element
						printPreview.document.getElementById("viewport").innerHTML = viewportElement.html();

						// if there are multiple views shown, we want to viewerapp to have border
						if (printPreview.document.getElementsByClassName("view").length > 1) {
							printPreview.document.getElementById("viewerApp").className = "viewerApp withBorder";
						}

						// now we want to transfer all canvas data to the popup
						try {
							// copy the data for every canvas
							viewportElement.find("canvas").each(function (number, canvas) {
								var destinationCanvas = printPreview.document.getElementById(canvas.id);
								if (destinationCanvas) {
									var ctx = destinationCanvas.getContext('2d');
									ctx.drawImage(canvas, 0, 0);
								}
							});
						} catch (err) {
							// IE doesn't support drawing canvasdata from another window... but we do have a fix :)
							// replace every canvas with an image with the imagedata from the original canvas
							viewportElement.find("canvas").each(function (number, canvas) {
								var image = printPreview.document.createElement("img");

								var oldCanvas = printPreview.document.getElementById(canvas.id);
								if (oldCanvas) {
									// initialize the image with correct settings (and copy styling)
									image.alt = "";
									image.id = oldCanvas.id;
									oldCanvas.id = "";
									image.style.cssText = oldCanvas.style.cssText;
									image.width = oldCanvas.width;
									image.height = oldCanvas.height;

									// replace the canvas with the image
									oldCanvas.parentNode.replaceChild(image, oldCanvas);

									// copy over the imageData from the original canvas
									image.src = canvas.toDataURL("image/png");
								}
							});

						} finally {
							// we finished copying the image data

							// hide loading indicator
							var loadingIndicator = printPreview.document.getElementById("printLoadingIndicator");
							loadingIndicator.parentNode.removeChild(loadingIndicator);

							// call callbacks if needed
							if (afterInitialization) {
								afterInitialization();
							}
						}
					}, this), 100);
				}, this);
			}, this), 100);
		};


		/**
		 * Get the host URL
		 * @returns {String} e.g. "http(s)://localhost:8081/wcr-web"
		 */
		PrintModule.prototype._getHostURL = function () {
			var href = window.location.href;
			var indexOfView = href.indexOf("webviewer.jsp");
			return href.substr(0, indexOfView);
		};

		/**
		 * Get the base url
		 * @returns {String} e.g. "http(s)://localhost:8081"
		 */
		PrintModule.prototype._getBaseURL = function () {
			var href = window.location.href.split("/");
			return href[0] + "//" + href[2];
		};

		return PrintModule;
	});

