define(["web-viewer/utils/LineworkWrapper",
		"web-viewer/utils/LineworkPoint",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/utils/DrawUtils",
		"packaging-content-management/ContentConstants",
		"underscore"
	],
	function (LineworkWrapper,
		LineworkPoint,
		ShapeUtils,
		ViewerUtils,
		DrawUtils,
		ContentConstants,
		_) {
		"use strict";


		/**
		 * @class Represents an dynamic content object. It includes the linework and the data of the dynamic content.
		 * @exports DynContent
		 * @memberof TextContentModule
		 * @augments Objects.LineworkWrapper
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var DynContentAnnotation = function (f, dynContent, dynContentLocation, pageSize) {
			LineworkWrapper.call(this);
			this.f = f;
			this._dynContent = dynContent;
			this._dynContentLocation = dynContentLocation;
			this._pageSize = pageSize;

			// Keep track wether this annotation has been selected or not.
			this.selected = false;

			// Boolean indicating whether this is a full page annotation. 
			// (check whether a location is provided)
			this._fillPage = dynContentLocation === undefined;

			this.calcRect();
			this.calcStyle();

		};

		// inherit
		DynContentAnnotation.prototype = Object.create(LineworkWrapper.prototype);
		// correct the constructor pointer 
		DynContentAnnotation.prototype.constructor = DynContentAnnotation;

		/**
		 * Calculate the position of the rectangle.
		 */
		DynContentAnnotation.prototype.calcRect = function () {

			if (this._dynContent.getDynContentType() == "Text") {
				// All text dyn contents annotations are rectangle.
				this.type = DrawUtils.LWTYPE_RECT;
				// Check whether this annotation fills the whole page.
				if (!this._fillPage) {
					// Create the rectangle at the correct position.
					var point = new ShapeUtils.Point(this._dynContentLocation.orgx, this._dynContentLocation.orgy - this._dynContentLocation.sizey);
					var size = new ShapeUtils.Size(this._dynContentLocation.sizex, this._dynContentLocation.sizey);
					this.rect = new ShapeUtils.Rect(point, size);
					// Find out wether a rotation was provided ... (not 360 or 0 degrees).
					var epsilon = 2.2204460492503130808472633361816E-16;
					if (Math.abs(this._dynContentLocation.rotation - 360) > 2 * epsilon && Math.abs(this._dynContentLocation.rotation) > 2 * epsilon) {
						// A rotation was provided, but we don't support rotated rectangles ... let's draw the bounding rectangle instead.
						// Calculate the points of the rectangle ...
						var point1 = point;
						var point2 = new ShapeUtils.Point(point.x + size.width, point.y);
						var point3 = new ShapeUtils.Point(point.x, point.y + size.height);
						var point4 = new ShapeUtils.Point(point.x + size.width, point.y + size.height);
						var center = new ShapeUtils.Point(point.x + size.width / 2, point.y + size.height / 2);

						// Rotate the points of the rectangle ...
						point1 = this._rotatePoint(point1, center, this._dynContentLocation.rotation);
						point2 = this._rotatePoint(point2, center, this._dynContentLocation.rotation);
						point3 = this._rotatePoint(point3, center, this._dynContentLocation.rotation);
						point4 = this._rotatePoint(point4, center, this._dynContentLocation.rotation);
						var rotPoints = [point1, point2, point3, point4];

						// Calculate the bounding rectangle of the rotated points ...
						var smallestX = Number.MAX_VALUE;
						var smallestY = Number.MAX_VALUE;
						var biggestX = Number.MIN_VALUE;
						var biggestY = Number.MIN_VALUE;
						for (var i = 0; i < rotPoints.length; i++) {
							if (rotPoints[i].x < smallestX) {
								smallestX = rotPoints[i].x;
							}
							if (rotPoints[i].x > biggestX) {
								biggestX = rotPoints[i].x;
							}
							if (rotPoints[i].y < smallestY) {
								smallestY = rotPoints[i].y;
							}
							if (rotPoints[i].y > biggestY) {
								biggestY = rotPoints[i].y;
							}
						}

						// Based on this, create the bounding rectangle ...
						var newPoint = new ShapeUtils.Point(smallestX, smallestY);
						var newSize = new ShapeUtils.Size(biggestX - smallestX, biggestY - smallestY);
						this.rect = new ShapeUtils.Rect(newPoint, newSize);
					}
				} else {
					// No location was provided for the DynContentAnnotation
					var origin = new ShapeUtils.Point(0, 0);
					this.rect = new ShapeUtils.Rect(origin, this._pageSize);
				}

				// Calc the location of the popup of this annotation.
				var popupOriginX = this.rect.origin.x;
				var popupOriginY = this.rect.origin.y - 38;
				var popupOrigin = new ShapeUtils.Point(popupOriginX, popupOriginY);
				this.popuprect = new ShapeUtils.Rect(popupOrigin, new ShapeUtils.Size(120, 38));
			}
		};

		/**
		 * Calculate the style of the annotation.
		 */
		DynContentAnnotation.prototype.calcStyle = function () {

			if (this._dynContent.getDynContentType() == "Text") {

				// Color according to the current state ...
				var state = this._dynContent.getTextElement().state;
				// However ... if somebody just reviewed ... color the soon to be state.
				var approvals = this._dynContent.getTextElement().approval;
				if (this._dynContent.getTextElement().approvalChanged && approvals && approvals.length > 0) {
					var lastApproval = approvals[approvals.length - 1];
					if (lastApproval.state) {
						state = lastApproval.state;
					}
				}

				// Determine the color based on the approval state of the element of the dynContent.
				if (state == ContentConstants.State.APPROVED) {
					this.color = ContentConstants.ApprovedColor.BASICCOLOR;
					if (!this._fillPage) {
						if (this.selected) {
							this.fillcolor = ContentConstants.ApprovedColor.FILLLIGHTCOLOR;
						} else {
							this.fillcolor = ContentConstants.ApprovedColor.FILLDARKCOLOR;
						}
					}
				} else if (state == ContentConstants.State.REJECTED) {
					this.color = ContentConstants.RejectColor.BASICCOLOR;
					if (!this._fillPage) {
						if (this.selected) {
							this.fillcolor = ContentConstants.RejectColor.FILLLIGHTCOLOR;
						} else {
							this.fillcolor = ContentConstants.RejectColor.FILLLIGHTCOLOR;
						}
					}
				} else if (state == ContentConstants.State.READY_FOR_APPROVAL) {
					this.color = ContentConstants.PendingSignedOffColor.BASICCOLOR;
					if (!this._fillPage) {
						if (this.selected) {
							this.fillcolor = ContentConstants.PendingSignedOffColor.FILLLIGHTCOLOR;
						} else {
							this.fillcolor = ContentConstants.PendingSignedOffColor.FILLLIGHTCOLOR;
						}
					}
				} else {
					this.color = ContentConstants.DraftColor.BASICCOLOR;
					if (!this._fillPage) {
						if (this.selected) {
							this.fillcolor = ContentConstants.DraftColor.FILLLIGHTCOLOR;
						} else {
							this.fillcolor = ContentConstants.DraftColor.FILLLIGHTCOLOR;
						}
					}
				}

				if (this.selected) {
					this.linewidth = 4;
				} else {
					this.linewidth = 2;
				}
			}
		};


		/**
		 * Recalculate the style and return wether there was a change.
		 */
		DynContentAnnotation.prototype.reCalcStyle = function () {

			// Remember the old style.
			var oldFillcolor = this.fillcolor;
			var oldColor = this.color;
			var oldLineWidth = this.linewidth;

			// Calculate the style again.
			this.calcStyle();

			// Return whether the style changed.
			if (oldFillcolor !== this.fillcolor || oldColor !== this.color || oldLineWidth !== this.linewidth) {
				return true;
			} else {
				return false;
			}
		};


		/**
		 * Return the dynContent.
		 */
		DynContentAnnotation.prototype.getDynContent = function () {
			return this._dynContent;
		};

		/**
		 * Indicate that the dynContent is selected.
		 */
		DynContentAnnotation.prototype.setSelected = function (isSelected) {
			this.selected = isSelected;
			this.calcStyle();
		};

		/**
		 * Utility function to calculate the rotation of the point
		 */
		DynContentAnnotation.prototype._rotatePoint = function (point, origin, angle) {
			angle = angle * Math.PI / 180.0;
			var x = Math.cos(angle) * (point.x - origin.x) - Math.sin(angle) * (point.y - origin.y) + origin.x;
			var y = Math.sin(angle) * (point.x - origin.x) + Math.cos(angle) * (point.y - origin.y) + origin.y;
			return new ShapeUtils.Point(x, y);
		};

		/**
		 * Set the scaling of the dynamic content annotation, all points of the annotation (linework) are scaled witht his value
		 * @param {ShapeUtils.Point|null} scaling
		 */
		DynContentAnnotation.prototype.setScale = function (scaling) {
			this.scale(scaling);
			this._scaling = scaling;
		};
		
		/**
		 * After setting the new content location, re-calculate the rectangle and style
		 * Reset the pageSize as well.
		 * @param dynContentLocation
		 */
		DynContentAnnotation.prototype.setDynContentLocation = function (dynContentLocation, pageSize) {
			this._dynContentLocation = dynContentLocation;
			this._pageSize = pageSize;
			this._fillPage = dynContentLocation === undefined;
			this.calcRect();
			this.calcStyle();
			this._dynContent.setLocations(undefined);
		};
		
		/**
		 * Returns _dynContentLocation
		 */
		DynContentAnnotation.prototype.getDynContentLocation = function () {
			return this._dynContentLocation;
		};

		return DynContentAnnotation;
	});