define(["jquery",
		"underscore",
		"packaging-content-management/ContentConstants",
		"./DynContentEntryView"],
	function (
		$,
		_,
		ContentConstants,
		DynContentEntryView) {
		"use strict";

		/**
		 * This is the model class for Dynamic Content Entry which supports all the operation that is on the view
		 */
		var DynContentEntry = function (contentStatement, masterLang, transLang, languagesList, elementTypeList) {

			this._contentStatement = contentStatement;
			this._masterElement = this._contentStatement.getElement(masterLang, true);
			this._transElement = this._contentStatement.getElement(transLang, false);
			this._languagesList = languagesList;

			// The callbacks that need to be executed when a statement is clicked.
			this._onSelected = [];
			this._onElementSelected = [];

			var masterPlainText = this._getPlainText(this._masterElement);
			var transPlainText = this._getPlainText(this._transElement);
			var masterStatus;
			var transStatus;
			var masterElementLang;
			var transElementLang;
			var elementTypeColor;
			var elementHumanReadable = this._contentStatement.elementType;

			if (this._masterElement) {
				masterStatus = this._masterElement.state;
				masterElementLang = this._masterElement.language;
			}
			if (this._transElement) {
				transStatus = this._transElement.state;
				transElementLang = this._transElement.language;
			}
			var elementType = elementTypeList[this._contentStatement.elementType];
			if (elementType) {
				elementTypeColor = elementType.color;
				elementHumanReadable = elementType.elementHumanReadable;
			}

			// Create the view.
			this._view = new DynContentEntryView(masterPlainText, transPlainText, masterStatus, transStatus, masterElementLang, transElementLang, languagesList, elementTypeColor, elementHumanReadable, contentStatement.isChanged());

			// When the whole view is clicked
			this._view.onClicked(function () {
				for (var i = 0; i < this._onSelected.length; i++) {
					this._onSelected[i]();
				}
			}.bind(this));

			// When an element in the view is clicked
			this._view.onElementClicked(function (langCode) {
				for (var i = 0; i < this._onElementSelected.length; i++) {
					this._onElementSelected[i](langCode);
				}
			}.bind(this));
		};

		DynContentEntry.prototype.getContentStatement = function () {
			return this._contentStatement;
		};

		DynContentEntry.prototype.update = function (textElement) {
			if (textElement == this._masterElement) {
				this.setMasterLang(this._masterElement);
			} else if (textElement == this._transElement) {
				this.setTransLang(this._transElement);
			}
			if (textElement.isChanged()) {
				this._view.setUnsaved(true);
			} else {
				this._view.setUnsaved(false);
			}
		};

		DynContentEntry.prototype.onSelected = function (callback) {
			this._onSelected.push(callback);
		};

		DynContentEntry.prototype.onElementSelected = function (callback) {
			this._onElementSelected.push(callback);
		};

		DynContentEntry.prototype.highlight = function () {
			this._view.highlight();
		};

		DynContentEntry.prototype.unHighlight = function () {
			this._view.unHighlight();
		};

		DynContentEntry.prototype.setMasterLang = function (textElem) {
			var masterLang = textElem.language;
			this._masterElement = this._contentStatement.getElement(masterLang, true);
			var masterPlainText = this._getPlainText(this._masterElement);
			//Set the new status if status changed else the original status(from revert)
			var masterStatus = textElem.newState || textElem.state;
			this._view.setMasterText(masterLang, masterPlainText, masterStatus);
		};

		DynContentEntry.prototype.setTransLang = function (textElem) {
			var transLang = textElem.language;
			this._transElement = this._contentStatement.getElement(transLang, false);
			var transPlainText = this._getPlainText(this._transElement);
			//Set the new status if status changed else the original status(from revert)
			var transStatus = textElem.newState || textElem.state;
			this._view.setTransText(transLang, transPlainText, transStatus);
		};

		DynContentEntry.prototype._getPlainText = function (element) {
			if (element) {
				var plainText = element.content.content;
				if (this._contentStatement.textType == ContentConstants.TextType.RICHTEXT) {
					plainText = $("<div>").html(plainText).text();
				}
				return plainText.replace(/(\r\n|\n|\r)/gm, "");
			} else {
				return undefined;
			}
		};

		DynContentEntry.prototype.getJQElement = function () {
			return this._view.getJQElement();
		};

		return DynContentEntry;
	});