define(["jquery",
		"underscore",
		"packaging-content-management/ContentConstants"],
	function (
		$,
		_,
		ContentConstants) {
		"use strict";

		/**
		 * This class enables the view for every text content in DynContentWidget
		 * Depending on the language selected, DynContentEntry View will be updated accordingly
		 */
		var DynContentEntryView = function (masterText, transText, masterStatus, transStatus, masterLangCode, transLangCode, languagesList, elementTypeColor, elementHumanReadable, isUnsaved) {
			this._masterText = masterText;
			this._transText = transText;
			this._masterStatus = masterStatus;
			this._transStatus = transStatus;
			this._languagesList = languagesList;
			this._masterLangCode = masterLangCode;
			this._transLangCode = transLangCode;
			this._elementTypeColor = elementTypeColor;
			this._elementHumanReadable = elementHumanReadable;
			this._isUnsaved = isUnsaved;

			// All the callbacks that need to be executed when the view is clicked.
			this._onClicked = [];
			this._onElementClicked = [];

			// Create the wrapping div.
			this._wrapper = $("<div>", {
				"class": "DynContentEntry"
			});

			// Create the wrapping div.
			this.init();

			// When the wrapping div is clicked ...
			this._wrapper.on("click", function () {
				for (var i = 0; i < this._onClicked.length; i++) {
					this._onClicked[i]();
				}
			}.bind(this));
		};

		/**
		 * Initialize the widget for single Text Content
		 * This includes - Color Code, Flag, Status, Actual Text for Master and translation
		 */
		DynContentEntryView.prototype.init = function () {

			// Empty the wrapper.
			this._wrapper.empty();

			// Create a lang table.
			this._langTable = $("<table>", {
				"class": "LangTable"
			});

			// Add the "Unsaved"  class if the entry is currently unsaved.
			if (this._isUnsaved) {
				this._langTable.addClass("Unsaved");
			}

			// Create the elements for the master text
			this._masterTextRow = $("<tr>", {
				"class": "MasterTextRow"
			});
			this._masterTextFlagCell = $("<td>", {
				"class": "MasterTextFlagCell"
			});
			this._masterTextFlagCell.append($("<img>", {
				"src": this._getLangFlag(true, this._masterLangCode)
			}));
			this._masterTextStateCell = $("<td>", {
				"class": "MasterTextStateCell"
			});
			this._masterTextStateCell.append($("<i>", {
				"class": this._getStateIconClass(this._masterStatus)
			}));
			this._masterTextContentCell = $("<td>", {
				"class": "MasterTextContentCell"
			});
			this._masterTextContentContainer = $("<div>", {
				"class": "MasterTextContentContainer"
			});

			// If master language is still not added, then we get master text as undefined
			if (undefined !== this._masterText && this._masterText.length > 0) {
				this._masterTextContentContainer.text(this._masterText);
			} else {
				this._masterTextContentContainer.html("&nbsp;");
			}
			this._masterTextContentCell.append(this._masterTextContentContainer);
			this._masterTextFlagCell.on("click", function (event) {
				event.stopPropagation();
				this._execOnElementClicked(this._masterLangCode);
			}.bind(this));
			this._masterTextStateCell.on("click", function (event) {
				event.stopPropagation();
				this._execOnElementClicked(this._masterLangCode);
			}.bind(this));
			this._masterTextContentContainer.on("click", function (event) {
				event.stopPropagation();
				this._execOnElementClicked(this._masterLangCode);
			}.bind(this));

			// Add them to the wrapper.
			this._langTable.append(this._masterTextRow);
			this._masterTextRow.append(this._masterTextFlagCell);
			this._masterTextRow.append(this._masterTextStateCell);
			this._masterTextRow.append(this._masterTextContentCell);

			if (this._transLangCode) {
				// Create the elements for the translation
				this._transTextRow = $("<tr>", {
					"class": "TransTextRow"
				});
				this._transTextFlagCell = $("<td>", {
					"class": "TransTextFlagCell"
				});
				this._transTextFlagCell.append($("<img>", {
					"src": this._getLangFlag(false, this._transLangCode)
				}));
				this._transTextStateCell = $("<td>", {
					"class": "TransTextStateCell"
				});
				this._transTextStateCell.append($("<i>", {
					"class": this._getStateIconClass(this._transStatus)
				}));
				this._transTextContentCell = $("<td>", {
					"class": "TransTextContentCell"
				});
				this._transTextContentContainer = $("<div>", {
					"class": "TransTextContentContainer"
				});
				if (this._transText.length > 0) {
					this._transTextContentContainer.text(this._transText);
				} else {
					this._transTextContentContainer.html("&nbsp;");
				}
				this._transTextContentCell.append(this._transTextContentContainer);
				this._transTextStateCell.on("click", function (event) {
					event.stopPropagation();
					this._execOnElementClicked(this._transLangCode);
				}.bind(this));
				this._transTextFlagCell.on("click", function (event) {
					event.stopPropagation();
					this._execOnElementClicked(this._transLangCode);
				}.bind(this));
				this._transTextContentContainer.on("click", function (event) {
					event.stopPropagation();
					this._execOnElementClicked(this._transLangCode);
				}.bind(this));

				// Add them to the wrapper.
				this._langTable.append(this._transTextRow);
				this._transTextRow.append(this._transTextFlagCell);
				this._transTextRow.append(this._transTextStateCell);
				this._transTextRow.append(this._transTextContentCell);
			}

			// If the element type of this contentstatement has a particular color ...
			if (this._elementTypeColor) {
				// Create a new wrapping table that will hold the color handle
				var mainTable = $("<table>", {
					"class": "MainTable"
				});
				var mainRow = $("<tr>", {
					"class": "MainRow"
				});
				var colorCell = $("<td>", {
					"class": "ColorCell",
					"style": "background-color: " + this._elementTypeColor + "; cursor: pointer"
				});
				var langCell = $("<td>", {
					"class": "LangCell"
				});


				// When hovering over the color cell, make it lighter ...
				colorCell.hover(function () {
					colorCell.css("opacity", 0.3);
				}.bind(this), function () {
					colorCell.css("opacity", 1);
				}.bind(this));

				// Add the element type description as a tooltip.
				if (this._elementHumanReadable) {
					colorCell.attr('title', this._elementHumanReadable);
				}

				// Put everything together.
				langCell.append(this._langTable);
				mainRow.append(colorCell);
				mainRow.append(langCell);
				mainTable.append(mainRow);
				this._wrapper.append(mainTable);
			} else {
				this._wrapper.append(this._langTable);
			}
		};

		DynContentEntryView.prototype.highlight = function () {
			this._wrapper.addClass("highlight");
			// Try to scroll the element into view.
			try {
				this._wrapper[0].scrollIntoView();
			} catch (e) {}
		};

		DynContentEntryView.prototype.unHighlight = function () {
			this._wrapper.removeClass("highlight");
		};

		DynContentEntryView.prototype.setMasterText = function (masterLang, masterPlainText, masterStatus) {
			this._masterText = masterPlainText;
			this._masterLangCode = masterLang;
			this._masterStatus = masterStatus;
			this.init();
		};

		DynContentEntryView.prototype.setTransText = function (transLang, transPlainText, transStatus) {
			this._transText = transPlainText;
			this._transLangCode = transLang;
			this._transStatus = transStatus;
			this.init();
		};

		DynContentEntryView.prototype.setUnsaved = function (isUnsaved) {
			this._isUnsaved = isUnsaved;
			// Add the "Unsaved"  class if the entry is currently unsaved.
			if (this._isUnsaved) {
				this._langTable.addClass("Unsaved");
			} else {
				this._langTable.removeClass("Unsaved");
			}
		};

		DynContentEntryView.prototype._getStateIconClass = function (state) {
			if (state == ContentConstants.State.DRAFT) {
				return "icon-WCR_Text-Status-Draft";
			} else if (state == ContentConstants.State.READY_FOR_APPROVAL) {
				return "icon-WCR_Text-Status-Pending";
			} else if (state == ContentConstants.State.APPROVED) {
				return "icon-WCR_Text-Status-Approved icon-WCR_Approve";
			} else if (state == ContentConstants.State.REJECTED) {
				return "icon-WCR_Text-Status-Rejected icon-WCR_Reject";
			}
		};

		DynContentEntryView.prototype._getLangFlag = function (isMaster, langCode) {
			if (isMaster) {
				for (var i = 0; i < this._languagesList.allMasterLangs.length; i++) {
					if (this._languagesList.allMasterLangs[i].langCode == langCode) {
						var mFlagName = this._languagesList.allMasterLangs[i].langFlag;
						return "./images/country_flags/" + mFlagName;
					}
				}
			} else {
				for (var j = 0; j < this._languagesList.allTranslationLangs.length; j++) {
					if (this._languagesList.allTranslationLangs[j].langCode == langCode) {
						var tFlagName = this._languagesList.allTranslationLangs[j].langFlag;
						return "./images/country_flags/" + tFlagName;
					}
				}
			}
		};

		DynContentEntryView.prototype._execOnElementClicked = function (langCode) {
			for (var i = 0; i < this._onElementClicked.length; i++) {
				this._onElementClicked[i](langCode);
			}
		};

		DynContentEntryView.prototype.onElementClicked = function (callback) {
			this._onElementClicked.push(callback);
		};

		DynContentEntryView.prototype.onClicked = function (callback) {
			this._onClicked.push(callback);
		};

		DynContentEntryView.prototype.getJQElement = function () {
			return this._wrapper;
		};

		return DynContentEntryView;
	});