define(["web-viewer/viewport/model/ViewportLayerModel",
        "web-viewer/textContent/DynContentAnnotation",
		"utils/ShapeUtils",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/viewControls/ViewControlsConstants",
		"underscore"
	],
	function (ViewportLayerModel,
		DynContentAnnotation,
		ShapeUtils,
		ListenerUtil,
		ViewControlsConstants,
		_
	) {
		"use strict";

		/**
		 * Create a new annotation layer model
		 * @exports DynContentLayerModel
		 * @memberof TextContentModule
		 *
		 * @class This class is responsible for drawing annotations	 *
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} ID - Id of the layer
		 * @param {DynContentModel} dynContentModel - Model of the dynamic content objects.
		 */
		var DynContentLayerModel = function (f, ID, dynContentModel) {
			ViewportLayerModel.call(this, f, ID);
			ListenerUtil.mixin(this, f);

			this.f = f;
			this.dynContentModel = dynContentModel;

			this._registerListener("modelChanged", this._calculateTransformation, this, this.dynContentModel);
			this._registerListener("modelContentChanged", this.modelChanged, this, this.dynContentModel);
			this._registerListener("parameters.changed", this._calculateTransformation, this, this.f.getDocumentController());
			this._registerListener("transform.viewportWidget.transformChanged", this._calculateTransformation, this);
		};
		
		DynContentLayerModel.prototype = Object.create(ViewportLayerModel.prototype);
		DynContentLayerModel.prototype.constructor = DynContentLayerModel;

		/**
		 * Calculates the transformation for the dynamic content layer, takes compensation for distortion and scaling into account
		 */
		DynContentLayerModel.prototype._calculateTransformation = function () {
			var page = this.dynContentModel.getDynContentPage();
			if (page !== null && page !== undefined) {
				this.resetTransformation();
				this.getTransform().scale(1, -1);

				var rotation = page.getRotation();

				var worldBounds = this.f.getModule("viewport").getWorldBounds();

				//calculate the rest of the tranformation, for rotation, flipped state
				this.calculateTransformation(worldBounds, page.getPageSize(), new ShapeUtils.Size(), rotation, page.isFlipped());
			}
			this.modelChanged();
		};
		
		/**
		 * @override
		 */
		DynContentLayerModel.prototype.getLinework = function (animated, rotationanimation) {
			
			// Get all visible dyncontent annotations.
			var dynContentAnnotations = this.dynContentModel.getVisibleDynContents();
			// Make a quick shallow copy of this array.
			dynContentAnnotations = dynContentAnnotations.slice(0,dynContentAnnotations.length);
			
			// Get the selected annotations.
			var selectedDynContents = this.dynContentModel.getSelectedDynamicContents();
			// Generate the selection boxes for the selected annotations.
			var selectionBoxes = [];
			for (var i=0; i < selectedDynContents.length; i++) {
				// Only show selection boxes if the dynContent is visible.
				if ($.inArray(selectedDynContents[i], dynContentAnnotations) > -1) {
					selectionBoxes.push(selectedDynContents[i].getSelectionBox());
				}
			}
			
			// Make sure we don't double fill annotations on top of each other.
			// Group the annotations based on their location.
			var annotationsGrouped = [];
			while (dynContentAnnotations.length > 0) {
				var annotation1 = dynContentAnnotations.shift();
				var annotationGroup = [annotation1];
				for (var i=0; i < dynContentAnnotations.length; i++) {
					var annotation2 = dynContentAnnotations[i];
					if (annotation1.rect.rectEquals(annotation2.rect)) {
						annotationGroup.push(annotation2);
						dynContentAnnotations.splice(i, 1);
						i--;
					}
				}
				annotationsGrouped.push(annotationGroup);
			}
			
			// Iterate over every annotation group (all annotations in a group or on the same location).
			for (var i=0; i < annotationsGrouped.length; i++) {
				var annotationGroup = annotationsGrouped[i];
				
				// Check if there are multiple annotations in this group.
				if (annotationGroup.length > 1) {
					
					var transLang = this.dynContentModel.getCurrentTransLangCode();
					annotationGroup.sort(function(annotation1, annotation2) {
						// First sort on selected.
						if (annotation1.selected && !annotation2.selected) {
							return 1;
						} else if (!annotation1.selected && annotation2.selected) {
							return -1;
						} else if (annotation1.selected && annotation2.selected) {
							return 0;
						} else {
							// Sort on language.
							var lang1 = annotation1.getDynContent().getTextElement().language;
							var lang2 = annotation2.getDynContent().getTextElement().language;
							if (lang1 == transLang) {
								return 1;
							} else if (lang2 == transLang) {
								return -1;
							} else {
								return 0;
							}
						}
					});
					
					// Now disable the fill color of every annotation but the last.
					for (var j=0; j < annotationGroup.length; j++) {
						if (j != annotationGroup.length-1) {
							annotationGroup[j].fillcolor = undefined;
						} else {
							annotationGroup[j].reCalcStyle();
						}
					}
				}
			}
			// After removing any potential anoying fills ... flatten our grouped annotations again.
			dynContentAnnotations = _.flatten(annotationsGrouped);
			
			var linework = _.union(dynContentAnnotations, _.without(_.flatten(selectionBoxes), undefined, null));
			return linework;
			
		};
		
		return DynContentLayerModel;
});