define(["web-viewer/core/PrivateChannel",
		"utils/ShapeUtils",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/layout/component/ModalPopup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/textContent/DynContentAnnotation",
		"packaging-content-management/dyn-content-services/dynContentService",
		"packaging-content-management/text-content-services/ContentService",
		"packaging-content-management/ContentConstants",
		"component/passwordpopup/PasswordValidationPopup",
		"underscore",
		"jquery"
	],
	function (PrivateChannel,
		ShapeUtils,
		ListenerUtil,
		ViewerUtils,
		ModalPopup,
		ViewerToolbarButton,
		LayoutConstants,
		DynContentAnnotation,
		dynContentService,
		ContentService,
		ContentConstants,
		PasswordValidationPopup,
		_,
		$) {
		"use strict";

		/**
		 * Constructs a new Dynamic Content model
		 * @exports DynContentModel
		 * @memberof TextContentModule
		 * @class
		 * The dynamic content model is the main model within the viewer that keeps track of the dynamic content objects and their selection. The model has its main function "setDynamicContents".
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {String} IID - Unique channel ID of the dynamic content model
		 */
		var DynContentModel = function (f, IID) {
			PrivateChannel.installTo(this, f, ['dynContentModel', IID]);
			ListenerUtil.mixin(this, f);

			/**
			 * Facade
			 */
			this.f = f;

			/**
			 * The page to which the current dynamic content annotations belong
			 * @private
			 * @type {Data.AbstractDocumentPage}
			 */
			this._page = null;

			/**
			 * Dynamic content annotations of the main document.
			 */
			this._dynContentAnnotations = [];

			/**
			 * Flag to specify loading dyn content 
			 */
			this._isDynContentsLoading = false;

			/**
			 * Keeps the list of dynamic content annotations that are selected.
			 */
			this._selectedDynContentAnnotations = [];

			/**
			 * An array containing all the annotations that are currently visible.
			 */
			this._visibleDynContentAnnotations = [];

			/**
			 * Keeps the list of Text Content objects that are unsaved.
			 */
			this._unsavedTextContents = [];

			/**
			 *  The currently active scaling, all dynamic content annotations are scaled with this scaling. This is used to initialize new annotations.
			 */
			this._currentScaling = null;

			/**
			 * Indicates whether we're currently in the process of saving the dynamic content.
			 */
			this._isSavingDynamicContents = false;

			/**
			 * The state on which we're filtering in the viewer.
			 */
			this._dynContentFilterState = undefined;

			/**
			 * By default the model is not disabled.
			 */
			this._disabled = false;
			
			/**
			 * This keep the list of Text Content that we got from initial page load
			 */
			this._dynContents = [];
		};

		/**
		 * Toggle the dynamic content annotation visibility
		 * @param {Boolean} [toggle] - force visibility
		 */
		DynContentModel.prototype.toggleVisibility = function (toggle) {
			if (toggle === undefined) {
				toggle = this.f.getOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE") == 1;
				toggle = !toggle;
			} else if (toggle && this.f.getOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE") == 1) { // yes! we compare with == and not with === (we are not sure what is in the preference (true, false)
				// nothing changed
				return;
			}

			if (!toggle) {
				this.f.setOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE", 0);
				this.unselectAllDynContents(false, true);
			} else {
				this.f.setOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE", 1);
			}
			this.modelChanged();
		};

		/**
		 * Get the page for which the dynamic content is visible
		 * @returns {Data.AbstractDocumentPage}
		 */
		DynContentModel.prototype.getDynContentPage = function () {
			return this._page;
		};

		/**
		 * Disable the model, e.g. clear all the dynamic content annotations and prevent them to be set.
		 */
		DynContentModel.prototype.disable = function () {
			this._disabled = true;
			this._dynContentAnnotations = [];
			this._visibleDynContentAnnotations = [];
			this.modelContentChanged();
		};

		/**
		 * Set the annotations objects for this model
		 * @param {Array<DynContentAnnotation>} dynContents - List of annotations
		 */
		DynContentModel.prototype.setDynContents = function (dynContents) {
			if (!this._disabled) {
				this.scaleAllDynContents(null);
				this._dynContentAnnotations = dynContents;
				this._visibleDynContentAnnotations = dynContents.slice(0, dynContents.length);
				//TODO allow the reference page the compensated, the Java Viewer does not allow this, as it returns wrong parameter values
				if (this._page && this._page !== this.f.getDocumentController().getDifferencePage()) {
					this.scaleAllDynContents(this._page.getCurrentScaling());
				}
				this.modelContentChanged();
			}
		};

		/**
		 * @returns {Array.<DynContentAnnotation>} dyn content of the model (unfiltered)
		 */
		DynContentModel.prototype.getDynContents = function () {
			return this._dynContentAnnotations;
		};

		/**
		 * @returns {Boolean} - True if loading dyn content is in progress 
		 */
		DynContentModel.prototype.getDynContentsLoading = function () {
			return this._isDynContentsLoading;
		};

		/**
		 * Save 1 or all dynamic content.
		 * 
		 * @param {ContentStatementElement} [textElement] if specified, only this statements will be saved. If not, all statements will be saved
		 * @param {Function} [callbackFunction] will be called when the annotations have been saved
		 */
		DynContentModel.prototype.save = function (textElement, callbackFunction) {
			if (!this._isSavingDynamicContents) {
				
				this._isSavingDynamicContents = true;
				if (this._page) {
					this.unselectAllDynContents(false, true);

					var unsavedTextContents = [];
					if (textElement) {
						// First make sure if the text content of this passed dynamic content is in the array of unsaved text contents.
						var textContent = textElement.parentStatement;
						if ($.inArray(textContent, this._unsavedTextContents) < 0) {
							// Attempting to save a text content that is not in the list of unsaved.
							// Stop here.
							this._isSavingDynamicContents = false;
							if (callbackFunction) {
								callbackFunction();
							}
							this._emit("save.finished");
							this.modelChanged();
							this.modelContentChanged();
							return;
						} else {
							unsavedTextContents.push(textContent);
						}
					} else {
						unsavedTextContents = this.getUnsavedTextContents();
					}

					// Find out if we need password validation.
					var isPasswordRequired = false;
					for (var i = 0; i < unsavedTextContents.length; i++) {
						isPasswordRequired = unsavedTextContents[i].isPasswordRequiredForApproval();
						if (isPasswordRequired) {
							break;
						}
					}

					// Create a function that will send the actual content statement save request.
					var doSaveContentStatements = function () {
						this._emit("save.started");
						// Save the content statements
						for(var j=0; j < unsavedTextContents.length; j++){
							unsavedTextContents[j].instanceId = 1;
						}
						
						ContentService.saveContents(unsavedTextContents, function (updatedTextContents, errorMsgs) {

							// Make sure no annotations are selected.
							this.unselectAllDynContents(false, true);

							// Show the errors ...
							var numErrors = Object.keys(errorMsgs).length;
							var errorString = this.f.i18n.t("PCMDynContentSaveFailed", numErrors)+"\n";
							for (var id in errorMsgs) {
								  if (errorMsgs.hasOwnProperty(id)) {
									  errorString += "\n" + errorMsgs[id];
								  }
							}
							if (numErrors > 0) {
								this.f.emit("userMessage", "warn", errorString, false);
							} else {
								this.f.emit("userMessage", "success", this.f.i18n.t("PCMDynContentSaveSucces"), true);
							}

							// Find out which text content documents were successfully saved and remove them from the array.
							$.each(updatedTextContents, function (id, updatedTextContent) {
								for (var i = 0; i < this._unsavedTextContents.length; i++) {
									if (this._unsavedTextContents[i].docVersionId == id) {
										this._unsavedTextContents.splice(i, 1);
										i--;
									}
								}
							}.bind(this));

							// Update the annotations of the successfully updated text content docuemnts.
							$.each(updatedTextContents, function (id, updatedTextContent) {
								for (var i = 0; i < this._dynContentAnnotations.length; i++) {
									var parentStatement = this._dynContentAnnotations[i].getDynContent().getTextElement().parentStatement;
									if (parentStatement.docVersionId == id) {
										var oldTextElement = this._dynContentAnnotations[i].getDynContent().getTextElement();
										var updatedTextElement = updatedTextContent.getElement(oldTextElement.language, oldTextElement.isMaster());
										// Update the Text Element.
										this._dynContentAnnotations[i].getDynContent().setTextElement(updatedTextElement);
										this._dynContentAnnotations[i].reCalcStyle();
									}
								}
							}.bind(this));

							// Finish the saving action ...
							this._isSavingDynamicContents = false;
							if (callbackFunction) {
								callbackFunction();
							}
							this._emit("save.finished");
							this.modelChanged();
							this.modelContentChanged();

						}.bind(this));

					}.bind(this);


					// Query the password ...
					if (isPasswordRequired) {
						var options = {};
						options.f = this.f;
						options.viewType = "viewer";

						this.passwordValidationPopup = new PasswordValidationPopup(options);
						// When the validation of the password was successfull ...
						this.passwordValidationPopup.onSubmit(function () {
							// Finally save the content statements.
							doSaveContentStatements();
						}.bind(this));

						// When the validation of the password failed ...
						this.passwordValidationPopup.onCancel(function () {
							// We're no longer saving.
							this._isSavingDynamicContents = false;
						}.bind(this));

						// Show the password validation popup
						this.passwordValidationPopup.open();

					} else {
						// No need for password validation first ... just save them.
						doSaveContentStatements();
					}

				} else {
					this._isSavingDynamicContents = false;
					// we didn't need to save, notify that the safe action is finsihed
					this._emit("save.finished");
				}
			} else {
				this.f.emit("error.WARNING", this.f.i18n.t("PCMDynContentAlreadyBeingSaved"), true);
			}
		};

		/**
		 * Get the currently selected translation language.
		 */
		DynContentModel.prototype.setDynContentFilterState = function (filterState) {

			// If there's no new filter state just return.
			if (filterState == this._dynContentFilterState) {
				return;
			}

			// Set the new state.
			this._dynContentFilterState = filterState;

			// Check if the "NO FILTER" state is set.
			if (!filterState || filterState == undefined) {
				this._visibleDynContentAnnotations = this._dynContentAnnotations;
			} else {
				// A new filter state was specified, calculate the newly visible annotations.
				this._visibleDynContentAnnotations = [];
				for (var i = 0; i < this._dynContentAnnotations.length; i++) {
					var approvals = this._dynContentAnnotations[i].getDynContent().getTextElement().pastApprovals;
					var state = this._dynContentAnnotations[i].getDynContent().getTextElement().state;
					if (state == filterState) {
						this._visibleDynContentAnnotations.push(this._dynContentAnnotations[i]);
					} else if (approvals.length > 0 && approvals[approvals.length - 1].state == filterState) {
						this._visibleDynContentAnnotations.push(this._dynContentAnnotations[i]);
					}
				}
				// Check if we have to deselect the currently selected annotation as they might be no longer visible.
				for (i = 0; i < this._selectedDynContentAnnotations.length; i++) {
					var selectedAnnotation = this._selectedDynContentAnnotations[i];
					if ($.inArray(selectedAnnotation, this._visibleDynContentAnnotations) < 0) {
						// One of the selected annotations is no longer visible, deselect the annotations.
						this.unselectAllDynContents(false, false);
						break;
					}
				}
			}

			this._emit("visibleDynContentChanged", this._visibleDynContentAnnotations);
			this.modelChanged();
		};

		/**
		 * Get the currently selected translation language.
		 */
		DynContentModel.prototype.getDynContentFilterState = function (state) {
			return this._dynContentFilterState;
		};

		/**
		 * Get the currently selected translation language.
		 */
		DynContentModel.prototype.setCurrentTransLangCode = function (langCode) {
			this._currentTransLangCode = langCode;
			this._emit("translationLangChanged", langCode);
		};

		/**
		 * Get the currently selected translation language.
		 */
		DynContentModel.prototype.getCurrentTransLangCode = function () {
			return this._currentTransLangCode;
		};

		/**
		 *
		 * @returns {Number} the count of all unsaved text content objects.
		 */
		DynContentModel.prototype.getUnsavedTextContentsCount = function () {
			return this.getUnsavedTextContents().length;
		};

		/**
		 * Get an array of unsaved text content objects.
		 * @returns {Array.<Annotation>} Array of unsaved annotations
		 */
		DynContentModel.prototype.getUnsavedTextContents = function () {
			return this._unsavedTextContents;
		};

		/**
		 * @returns {Array.<DynContentAnnotation>} the visible annotations to be displayed on the viewport
		 */
		DynContentModel.prototype.getVisibleDynContents = function () {
			// if annotation tools are disabled, or the module is temporarily disabled, do not show any annotations
			if (this.f.getOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE") === 0) {
				return [];
			} else {
				return this._visibleDynContentAnnotations;
			}
		};

		/**
		 * Get annotations for the given text element
		 * @param {ContentStatementElement} textElement - The text element
		 * @returns {Array.<<DynContentAnnotation>>} the dynamic content annotations linked to a specific text element
		 */
		DynContentModel.prototype.getDynContentAnnotationsForTextElement = function (textElement) {
			var forTextElement = [];
			for (var i = 0; i < this._dynContentAnnotations.length; i++) {
				if (this._dynContentAnnotations[i].getDynContent().getTextElement() == textElement) {
					forTextElement.push(this._dynContentAnnotations[i]);
				}
			}
			return forTextElement;
		};

		/**
		 * When a text element changed ...
		 * 
		 * @param {ContentStatementElement} textElement - The text element which got changed.
		 * @param {Object} fromWidget - The source of the changed.
		 */
		DynContentModel.prototype.textElementChanged = function (textElement, fromWidget) {
			this._emit("textElementChanged", textElement, fromWidget);

			// Put the parent text object in the list of text content objects that need to be saved.
			var existingTCIndex = $.inArray(textElement.parentStatement, this._unsavedTextContents);
			if ( existingTCIndex < 0) {
				this._unsavedTextContents.push(textElement.parentStatement);
				this._emit("unsavedTextContent", this._unsavedTextContents);
			} else {
				//Remove text content object from  array if it is not changed(status reverted)
				var existingTC = this._unsavedTextContents[existingTCIndex];
				if(!existingTC.isContentChanged()){
					this._unsavedTextContents.splice(existingTCIndex, 1);
					this._emit("unsavedTextContent", this._unsavedTextContents);
				}
			}

			// Check to see if any dynamic content annotations are affected (e.g. they need to change color or ...).
			var affectedDynContentAnnotations = this.getDynContentAnnotationsForTextElement(textElement);
			var changed = false;
			for (var i = 0; i < affectedDynContentAnnotations.length; i++) {
				var annotation = affectedDynContentAnnotations[i];
				// Recalculate the style for the annotation.
				var styleChanged = annotation.reCalcStyle();
				changed = changed || styleChanged;
			}

			// If the style for any annotation changed ...
			if (changed) {
				this.modelChanged();
			}
		};


		/**
		 * 
		 * @param contentStatement
		 */
		DynContentModel.prototype.selectDynContentBasedOn = function (contentStatement, langCode) {
			this.unselectAllDynContents(false, true);
			for (var i = 0; i < this._visibleDynContentAnnotations.length; i++) {
				var dynContentAnnotation = this._visibleDynContentAnnotations[i];
				var dynContent = dynContentAnnotation.getDynContent();
				if (dynContent.getTextElement().parentStatement == contentStatement) {
					// If a langCode was specified, filter on that ... but only when one was specified.
					if (langCode && langCode == dynContent.getTextElement().language) {
						this._selectedDynContentAnnotations.push(dynContentAnnotation);
						dynContentAnnotation.setSelected(true);
					} else if (!langCode) {
						this._selectedDynContentAnnotations.push(dynContentAnnotation);
						dynContentAnnotation.setSelected(true);
					}
				}
			}

			//If user has selected state filter and then selects the text in text content widget that is other than the filter, 
			//then we dont zoom or show the text highlight on the canvas, as the annotations are removed for other states than filter.
			//Eg: filter is set to draft, and in widget user select statement that is signed-off, then we dont do anything on canvas

			if (this._selectedDynContentAnnotations.length > 0) {
				// Indicate that we should zoom to these dynContents
				this.f.emit("module.dyncontent.zoomToDynContents", this._selectedDynContentAnnotations);

				// Indicate that these dynContents are selected 
				this._emit("selectedDynContents", this._selectedDynContentAnnotations, true);

				this.ensureVisibility();
			}

			this.modelChanged();
		};


		/**
		 * Select a dynamic content. All other selected dynamic contents will be unselected first
		 * 
		 * @param {DynContentAnnotation} dynContentAnnotation - Dynamic Content to be selected
		 * @param {Boolean} force - makes the unselectAllDynamicContents forced
		 */
		DynContentModel.prototype.selectDynContent = function (dynContentAnnotation, force) {

			if (!dynContentAnnotation) {
				return;
			}

			if (this._visibleDynContentAnnotations.length === 1) {
				var sDyn = this._visibleDynContentAnnotations[0];
				if (dynContentAnnotation == sDyn) {
					return;
				}
			}

			this.unselectAllDynContents(false, force);
			// Select the targeted one.
			this._selectedDynContentAnnotations.push(dynContentAnnotation);
			dynContentAnnotation.setSelected(true);
			// Select also all other dyncontent annotation with the same text element.
			var textElement = dynContentAnnotation.getDynContent().getTextElement();
			// Loop over all dyncontent annotations.
			for (var i = 0; i < this._visibleDynContentAnnotations.length; i++) {
				// If the dyncontent has the same text element as the selected one ...
				if (this._visibleDynContentAnnotations[i].getDynContent().getTextElement() == textElement) {
					// Select that annotation if it's not already.
					if ($.inArray(this._visibleDynContentAnnotations[i], this._selectedDynContentAnnotations) < 0) {
						this._selectedDynContentAnnotations.push(this._visibleDynContentAnnotations[i]);
						this._visibleDynContentAnnotations[i].setSelected(true);
					}
				}
			}
			this.ensureVisibility();

			// On mobile devices ... also zoom 
			if (this._layoutType == LayoutConstants.LayoutType.MOBILE) {
				this.f.emit("module.dyncontent.zoomToDynContents", this._selectedDynContentAnnotations);
			}

			// Indicate that we selected a dynContent annotation.
			this._emit("selectedDynContents", this._selectedDynContentAnnotations, true);
			this.modelChanged();
		};

		/**
		 * Make sure that the annotations are visible. If not, set the visible property back to 1
		 */
		DynContentModel.prototype.ensureVisibility = function () {
			if (!this.f.getOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE")) {
				this.f.setOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE", 1);
			}
		};

		/**
		 * Get an annotation based on the annotation name
		 * @param {Number} i - The index of the dynamic content.
		 */
		DynContentModel.prototype.getDynamicContent = function (i) {
			return this._dynContentAnnotations[i];
		};

		/**
		 * Unselect all the dynamic contents
		 * @param {Boolean} [donotbroadcast] - if set, the model will not broadcast that this happened (only used when the caller will call modelChanged() itself)
		 * @param {Boolean} [force] - if set, it will ignore the lockSelection flag
		 */
		DynContentModel.prototype.unselectAllDynContents = function (donotbroadcast, force) {
			if (this._selectedDynContentAnnotations.length <= 0) {
				return;
			}

			var selectedDynContents = this._selectedDynContentAnnotations;
			this._selectedDynContentAnnotations = [];

			_.each(selectedDynContents, _.bind(function (dynContent) {
				dynContent.setSelected(false);
			}, this));

			if (!donotbroadcast) {
				this.modelChanged();
				this._emit("unselectedDynContents", selectedDynContents, force);
			}
		};

		/**
		 * Get the selected dynamic contents
		 * 
		 * @returns {Array.<DynContentAnnotation>} list of selected annotations
		 */
		DynContentModel.prototype.getSelectedDynamicContents = function () {
			return this._selectedDynContentAnnotations;
		};

		/**
		 * Inform the listeners that the model has changed.
		 * 
		 * @param {DynContentAnnotation} dynContent - the dynamic content that has been updated
		 */
		DynContentModel.prototype.modelChanged = function () {
			this._emit("modelChanged");
		};

		/**
		 * Inform the listeners that the complete content has changed for the model (i.o.w, the _annotations are changed)
		 */
		DynContentModel.prototype.modelContentChanged = function () {
			this._emit("modelContentChanged");
		};

		/**
		 * Set the page for which dynamic content  should be displayed (this method will fetch the annotation for the page if necessary)
		 * @param {Data.AbstractDocumentPage} newPage - page for which dynamic content should be displayed
		 * @param {Function} [callback] - called when the page has changed
		 */
		DynContentModel.prototype.setPage = function (newPage, callback, forceUpdate) {
			if (newPage != this._page || forceUpdate) {

				// reset annotations
				this._isDynContentsLoading = true;
				this.setDynContents([]);

				this._emit("changed.page");
				this.unselectAllDynContents(false, true);
				this._page = newPage;

				if (newPage) {
					// load the annotations for the newPage
					var dynContentReq = dynContentService.getDynContentFor(newPage.getDocumentVersion().getDocumentVersionID(), forceUpdate);
					$.when(dynContentReq).done(function (dynContents) {
							this._dynContents = dynContents;
							var dynContentAnnotations = [];
							var pageSize = this._page.getPageSize();
							for (var i = 0; i < dynContents.length; i++) {
								var dynContLocations = dynContents[i].getLocations();
								if (dynContLocations.length > 0) {
									for (var j = 0; j < dynContLocations.length; j++) {
										var loc = dynContLocations[j];
										loc.orgy = loc.orgy + pageSize.height;
										dynContentAnnotations.push(new DynContentAnnotation(this.f, dynContents[i], loc));
									}
								} else {
									dynContentAnnotations.push(new DynContentAnnotation(this.f, dynContents[i], undefined, pageSize));
								}
							}
							this._isDynContentsLoading = false;
							this.setDynContents(dynContentAnnotations);
							if (callback) {
								callback(dynContents);
							}
						}.bind(this))
						.fail(_.bind(function (errorMsg) { // on error
							this.f.emit("userMessage", "warn", this.f.i18n.t("PCMDynContentErrorMessage"), false);
						}, this));

				}
			} else {
				if (callback !== undefined) {
					callback([]);
				}
			}
			this.modelChanged();
		};

		/**
		 * Get the page for which annotations are shown currently
		 * @returns {Data.AbstractDocumentPage}
		 */
		DynContentModel.prototype.getPage = function () {
			return this._page;
		};

		/**
		 * Prompt to save unsaved dynamic contents if there are any
		 * if there are unsaved dynamic contents, save them
		 *
		 * NOTE: this is not a pure model function, but to limit regressions, we keep it this way...
		 * @param {Function} callbackOnOK - function which is called when the popup is closed and annotations are saved
		 * @param {Function} [callbackOnCancel] - if provided, this callback is called when the user presses cancel
		 * @param {Function} [callbackOnDontSave] - adds the possibility to not save unsaved annotation but to continue with the action, on the contrary, cancel will cancel the action and don't save. Note that reloading the annotations should be done by the caller!
		 * @returns {Boolean} true if there are unsaved items and the popup is shown, false if not
		 */
		DynContentModel.prototype.promptToSaveUnsavedDynamicContent = function (callbackOnOK, callbackOnCancel, callbackOnDontSave) {
			
			if (this._page && this.getUnsavedTextContents().length > 0) {
				this.f.loadBundle("viewer", function () {
					// show popup if there are unsaved items
					var modalPopup = new ModalPopup(this.f, "unsavedDynContentsPopup", this.f.getContext());
					modalPopup.setRemoveFromDomAfterUsage(true);
					modalPopup.setOkAction(_.bind(function () {
						modalPopup.enableOkBtn(false);

						// save annotations
						this.save(null, _.bind(function () {
							// saving succeeded, so hide
							modalPopup.hide();
							if (callbackOnOK) {
								callbackOnOK();
							}
							this.toggleVisibility(true);
						}, this));
					}, this));

					modalPopup.setCancelAction(_.bind(function () {
						if (callbackOnCancel) {
							callbackOnCancel();
						}
						this.toggleVisibility(true);
					}, this));

					if (callbackOnDontSave) {
						var dontSaveButton = new ViewerToolbarButton(this.f, "dontSaveDynContents", $("<span>").append(this.f.t("RevertDynChanges")), false, _.bind(function () {
							modalPopup.hide();
							this.toggleVisibility(true);

							if (callbackOnDontSave) {
								callbackOnDontSave();
							}
						}, this));
						modalPopup.addButton(dontSaveButton);
					}

					// Header + Content
					modalPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.i18n.t("PCMUnsavedWarningTitle")));
					modalPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.i18n.t("PCMUnsavedWarningMsg")));
					modalPopup.setOkBtnText(this.f.i18n.t("Save"));
					this.toggleVisibility(false);
					modalPopup.show();
					modalPopup.recalculateSize(600);
				}, this);

				return true;
			} else {
				return false;
			}
		};

		/**
		 * Scale all the dynamic contents of the model with a given factor, both horizontal as vertical, changes all the points and rectangle of the annotation,
		 * first undos the scaling and than applies the new scaling, if no scaleFactor is provided the current scaling is undone
		 * @param {ShapeUtils.Point} scaleFactor
		 */
		DynContentModel.prototype.scaleAllDynContents = function (scaleFactor) {
			if (this._currentScaling) {
				_.each(this._dynContentAnnotations, function (dynContent) {
					if (dynContent.isScaled()) {
						dynContent.scale(new ShapeUtils.Point(1 / dynContent.getScale().x, 1 / dynContent.getScale().y));
					}
				}, this);
			}
			this._currentScaling = scaleFactor;
			_.each(this._dynContentAnnotations, function (dynContent) {
				dynContent.setScale(scaleFactor);
			});
		};
		
		/**
		 * Identify the location information from dynContentLocale for dynContent - Use element type, instance id to determine the location
		 * @param dynContent
		 * @param selectDropDown
		 * @param dynContLocale
		 * @param allLocales
		 * @returns
		 */
		DynContentModel.prototype._getLocations = function (dynContent, selectDropDown, dynContLocale, allLocales){
			var lang = dynContent.getTextElement().language;
			var localeId = undefined;
			
			// find localeid for this language, if user has specified one!!
			for(var i=0; i<selectDropDown.length; i++){
				if(selectDropDown[i].val() == lang){
					localeId = allLocales[i];
					break;
				}
			}
			
			if(localeId){
				// find the location list of this localeId
				var matchingDynContLocale = $.grep(dynContLocale, function(dynContentLocale, index){
					return dynContentLocale.id == dynContent.getUid()
							&& dynContentLocale.locale == localeId;
				});
				
				// Found the matching dynContent which has location information - This should be mostly 1 entry in the array
				if(matchingDynContLocale.length > 0){
					return matchingDynContLocale[0].localeLocation;
				}else{
					return null;
				}
			}
		};
		
		/**
		 * Action method what should happend when user updates locale mapping and clicks ok 
		 * @param selectDropDown - Options updated by user in drop down
		 * @param dynContLocale - Array of TextDynContentLocale object
		 * @param allLocales - All locales supported by artwork
		 */
		DynContentModel.prototype.localeMappingAction = function (selectDropDown, dynContLocale, allLocales) {
			this.setDynContents([]);
			this.unselectAllDynContents(true, true);
			
			var pagesize = this._page.getPageSize();
			var dynContentAnnotations = [];
			
			for (var i = 0; i < this._dynContents.length; i++) {
				var dynContLocations = this._getLocations(this._dynContents[i], selectDropDown, dynContLocale, allLocales);
				if (dynContLocations != null && dynContLocations.length > 0) {
					for (var j = 0; j < dynContLocations.length; j++) {
						var loc = {};
						// make a copy of location information, lets not update original dynContLocale
						$.extend(true, loc, dynContLocations[j]);
						loc.orgy = loc.orgy + pagesize.height;
						dynContentAnnotations.push(new DynContentAnnotation(this.f, this._dynContents[i], loc));
					}
				} else {
					dynContentAnnotations.push(new DynContentAnnotation(this.f, this._dynContents[i], undefined, pagesize));
				}
			}
			
			this.setDynContents(dynContentAnnotations);
		};
	
		/**
		 * Get the currently active scaling
		 * @returns {ShapeUtils.Point}
		 */
		DynContentModel.prototype.getCurrentScaling = function () {
			return this._currentScaling;
		};

		return DynContentModel;

	});