define(["web-viewer/viewport/model/ViewportPopupModel",
		"web-viewer/textContent/DynContentPopup",
		"utils/ShapeUtils",
		"underscore"
	],
	function (ViewportPopupModel,
		DynContentPopup,
		ShapeUtils,
		_) {
		"use strict";

		/**
		 * Create a new DynContent Popup Model.
		 * @exports DynContentPopupModel
		 * @memberof TextContentModule
		 * @class
		 * The dyncontent popup model is responsible for a dyncontent popup and its content.
		 * It binds the events of the popup and reacts on them.
		 * @augments Viewport.ViewportPopupModel
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} id - Id of the popup model.
		 * @param {DynContentAnnotation} dynContentAnnotation - The dycontent annotation this model belongs to.
		 * @param {DynContentModel} dynContentModel - Model of the dyncontents.
		 */
		var DynContentPopupModel = function (f, id, dynContentAnnotations, dynContentModel) {
			ViewportPopupModel.call(this, f, id);

			this.dynContentAnnotations = dynContentAnnotations;
			this.targetedDynContentAnnotation = dynContentAnnotations[0];
			this.dynContentModel = dynContentModel;
			this.popup = null;

			// When the content of the model changed.
			this.dynContentModel.on("modelContentChanged", function () {
				if (this.popup) {
					this.popup.update(this.targetedDynContentAnnotation.getDynContent().getTextElement());
				}
			}.bind(this));
		};

		DynContentPopupModel.prototype = Object.create(ViewportPopupModel.prototype);
		DynContentPopupModel.prototype.constructor = DynContentPopupModel;

		/**
		 * Repaint the popup
		 */
		DynContentPopupModel.prototype.repaint = function (refDynContentAnnotation) {
			if (this.popup != null) {
				this.popup.repaint(refDynContentAnnotation);
			}
		};

		/**
		 * Set the targeted annotation.
		 */
		DynContentPopupModel.prototype.setTargetAnnotation = function (annotation) {
			this.targetedDynContentAnnotation = annotation;
		};

		/**
		 * Set the targeted annotation.
		 */
		DynContentPopupModel.prototype.focus = function (focus) {
			this.popup.focus(focus);
		};

		/**
		 * @override
		 * This method saves the origin in the annotation.
		 * @param {ShapeUtils.Point} point - Point (origin) of the popup in world coordinates
		 */
		DynContentPopupModel.prototype.onDragged = function (worldPoint) {
			var point = worldPoint.copy();
			point.y -= this.targetedDynContentAnnotation.popuprect.size.height;

			if (!this.targetedDynContentAnnotation.popuprect.origin.equals(point)) {
				this.targetedDynContentAnnotation.popuprect.origin = point;
			}
		};

		/**
		 * @override
		 * This method saves the rectangle in the annotation.
		 * @param {ShapeUtils.Rect} rect - Rectangle (origin/size) of the popup in world coordinates
		 * @param {Boolean} autoAdapted - True if the event is fired because the content was auto adapted (e.g. from a makeMeVisible trigger from an element in the content), false otherwise
		 */
		DynContentPopupModel.prototype.onResized = function (worldRect, autoAdapted) {
			var copy = worldRect.copy();
			// compensate for origin in left bottom corner
			copy.origin.y -= copy.size.height;

			if (!this.targetedDynContentAnnotation.popuprect.rectEquals(copy)) {
				this.targetedDynContentAnnotation.popuprect = copy;
			}
		};

		/**
		 * Get the DynContent popup
		 * @param {ViewportLayer} viewportLayer - The layer where the popups should be shown.
		 * @returns {DynContentPopup} The dyncontent popup of this model.
		 */
		DynContentPopupModel.prototype.getPopup = function (viewportLayer) {
			var container = viewportLayer.getLayerElement();
			if (this.popup === null) {
				this._createPopup(container);
			} else {
				this.popup.setContainment(container);
			}
			return this.popup;
		};

		/**
		 * Create a dyncontent annotation popup {DynContentPopup} for the given layer
		 * and bind all events.
		 * @param {JQueryObj} container - Container for the popup
		 */
		DynContentPopupModel.prototype._createPopup = function (container) {

			// Get the text element.
			var textElement = this.targetedDynContentAnnotation.getDynContent().getTextElement();
			// Create the popup.
			this.popup = new DynContentPopup(this.f, this.id, this.dynContentModel, this, textElement, container);

			// we want the resized in world rect instead of scene rect, so don't bind them here, but in the layer, and let the layer call the model functions
			this.popup.on("closed", _.bind(this.onClosed, this));
			this.popup.on("clicked", _.bind(this.onClicked, this));
		};

		/**
		 * Method called when the model is destroyed.
		 * Useful for some cleanup in the popup/content
		 */
		DynContentPopupModel.prototype.onDestroy = function () {
			if (this.popup != null) {
				this.popup.remove();
			}
		};

		/**
		 * This function is bind to the "closed" popup event and called when the popup is closed.
		 */
		DynContentPopupModel.prototype.onClosed = function () {
			this.popup.focus(false);
			this.dynContentModel.unselectAllDynContents(false, true);
		};

		/**
		 * This function is bind to the "clicked" dyncontent annotation popup event and called when the popup is clicked.
		 */
		DynContentPopupModel.prototype.onClicked = function () {
			this.popup.focus(true);
		};

		/**
		 * Fire that a dyncontent annotation has changed
		 * @param {String} what - indicate what was changed
		 * @returns {Boolean} True when fired, false otherwise
		 */
		DynContentPopupModel.prototype._fireDynContentAnnotationChanged = function (what) {
			for (var i = 0; i < this.dynContentAnnotations.length; i++) {
				this.dynContentAnnotations[i].setModified(what);
			}
			this.dynContentModel.modelChanged();
		};

		/**
		 * Get the origin of the popup in world coordinates
		 * @returns {ShapeUtils.Point} - The origin of the popup in world coordinates
		 */
		DynContentPopupModel.prototype.getWorldOrigin = function () {
			// origin in left bottom corner
			return new ShapeUtils.Point(this.targetedDynContentAnnotation.popuprect.left(), this.targetedDynContentAnnotation.popuprect.bottom());
		};


		/**
		 * Get the size of the popup in world coordinates
		 * @returns {ShapeUtils.Size} - The size of the popup in world coordinates
		 */
		DynContentPopupModel.prototype.getWorldSize = function () {
			return this.targetedDynContentAnnotation.popuprect.size;
		};

		return DynContentPopupModel;

	});