define(["web-viewer/viewport/model/ViewportLayerModel",
		"web-viewer/textContent/DynContentPopupModel",
		"utils/ShapeUtils",
		"underscore"
	],
	function (ViewportLayerModel,
		DynContentPopupModel,
		ShapeUtils,
		_) {
		"use strict";

		/**
		 * @exports DynContentPopupsLayerModel
		 * @memberof TextContentModule
		 * @class
		 * Create a new dynamic content layer model for dynamic content popups
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} ID - Id of the layer
		 * @param {DynContentModel} dynContentModel - Model of the dynamic contents.
		 */
		var DynContentPopupsLayerModel = function (f, ID, dynContentModel) {
			ViewportLayerModel.call(this, f, ID);

			this.f = f;
			this.dynContentModel = dynContentModel;
			
			// New annotations received
			this._createModels();
			this.dynContentModel.on("modelContentChanged", _.bind(function () {
				this.removeAllPopupModels();
				this._createModels();
			}, this));
			
			// DynContent model changed events
			this.dynContentModel.on("modelChanged", _.bind(function () {
				var page = this.dynContentModel.getDynContentPage();
				if (page !== null && page !== undefined) {
					this.resetTransformation();
					this.getTransform().scale(1, -1);
					var rotation = page.getRotation();
					var worldBounds = this.f.getModule("viewport").getWorldBounds();
					this.calculateTransformation(worldBounds, page.getPageSize(), new ShapeUtils.Size(), rotation, page.isFlipped());
				}
				this.modelChanged();
			}, this));

			/*
			 * When dynamic content annotations are selected.
			 */
			this.dynContentModel.on("selectedDynContents", _.bind(function (dynamicContentAnnotations, isDirectSelect) {
				// Only show a popup when annotations are selected for a single dynContent object.
				var singleDynContentSelected = false;
				if (dynamicContentAnnotations && dynamicContentAnnotations.length > 0) {
					var selectedDynContent =  dynamicContentAnnotations[0].getDynContent();
					singleDynContentSelected = true;
					for (var i=1; i < dynamicContentAnnotations.length; i++) {
						if (selectedDynContent != dynamicContentAnnotations[i].getDynContent()) {
							singleDynContentSelected = false;
							break;
						}
					}
				}
				// If a single dynamic content element is selected.
				if (singleDynContentSelected) {
					// The selected annotation.
					var annotation = dynamicContentAnnotations[0];
					var popupModel = this.getPopupModel(annotation.getDynContent().getId());
					popupModel.setTargetAnnotation(annotation);
					popupModel.show();
					this.modelChanged();
				}
			}, this));

			/*
			 * When dynamic content annotations are deselected.
			 */ 	
			this.dynContentModel.on("unselectedDynContents", _.bind(function (dynamicContentAnnotations, force) {
				// Hide the popups for all these dynamic contents.
				for (var i=0; i < dynamicContentAnnotations.length; i++) {
					var annotation = dynamicContentAnnotations[i];
					var popup = this.getPopupModel(annotation.getDynContent().getId());
					popup.focus(false);
					popup.hide();
				}
				this.modelChanged();
			}, this));
		};
		
		DynContentPopupsLayerModel.prototype = Object.create(ViewportLayerModel.prototype);
		DynContentPopupsLayerModel.prototype.constructor = DynContentPopupsLayerModel;
		
		/**
		 * Create the annotation popup models
		 */
		DynContentPopupsLayerModel.prototype._createModels = function () {
			// Get all the textElements for all the dynamic contents.
			var textElements = [];
			_.each(this.dynContentModel.getDynContents(), function (dynContentAnnotation) {
				var textElement = dynContentAnnotation.getDynContent().getTextElement();
				if ($.inArray(textElement, textElements) < 0) {
					textElements.push(textElement);
				}
			});
			// Create a popup for each textElement and it's linked annotations.
			_.each(textElements, function (textElement) {
				var annotations = this.dynContentModel.getDynContentAnnotationsForTextElement(textElement);
				this._createModel(annotations);
			}, this);
			this.modelChanged();
		};
		
		/**
		 * Create the annotation popup model for the given annotation
		 * @param {Annotation} annotation - Annotation to create the model of
		 */
		DynContentPopupsLayerModel.prototype._createModel = function (dynContentAnnotations) {
			// This will create the appropriate popup (AnnotationPopupNew vs AnnotationPopupExisting)
			var popupModel = new DynContentPopupModel(this.f, dynContentAnnotations[0].getDynContent().getId(), dynContentAnnotations, this.dynContentModel);
			this.addPopupModel(popupModel);
			// Hide the popupModels by default ...
			popupModel.hide();
		};
		
		return DynContentPopupsLayerModel;
		
});
