define(["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"web-viewer/textContent/DynContentAnnotation",
		"utils/ShapeUtils",
		"web-viewer/utils/DrawUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractTool,
		DynContentAnnotation,
		ShapeUtils,
		DrawUtils,
		ViewerUtils,
		CoreConstants) {
		"use strict";
		
		/**
		 * Construct a new Dynamic Content Tool
		 * @exports DynContentTool
		 * @memberof TextContentModule
		 *
		 * @class Tool to select and edit annotations
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {AnnotationModel} annotationModel - Annotation model
		 */
		var DynContentTool = function (f, dynContentModel) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.DYNCONTENT);

			/**
			 * Different states constants
			 */
			this._defaultState = "defaultState";
			this._hoverState = "hoverState";
			
			this._targetDynContent = null;
			this._dynContentModel = dynContentModel;
			
			/**
			 * Variable to keep track of where interaction started
			 */
			this._touchDown = null;
			
			/**
			 * The default state
			 */
			this.registerState(this._defaultState, (function (that) {
				return {
					"mousemove": function (e) {
						var l = that._getDynContentLayer(e);
						if (!l || !l.isVisible()) {
							return;
						}

						that._verifyObjects(e, l);
						if (that._targetDynContent || that._anchor) {
							that.requestActivation();
						}
					},
					"mouseup": function (e) {

					},
					"mousedown": function (e) {
						var l = that._getDynContentLayer(e);
						if (!l) {
							return;
						}

						dynContentModel.unselectAllDynContents();
						e.originalEvent.preventDefault();
					},
					"mouseout": function (e) {
						if (that.isActive()) {
							f.emit("toolManager.request.deactivation", that);
						}
					},
					"mouseleave": function () {
						if (that.isActive()) {
							f.emit("toolManager.request.deactivation", that);
						}
					},
					"contextmenu": function (e) {
						e.preventDefault();
						e.stopPropagation();
						return false;
					}
				};
			}(this)));

			/**
			 * The hover state
			 */
			this.registerState(this._hoverState, _.defaults(function (that) {
				return {
					"mousedown": function (e) {
						if (e.which === 1) {
							that._verifyObjects(e);
							dynContentModel.selectDynContent(that._targetDynContent);
						}
						e.originalEvent.preventDefault();
					},
					"mousemove": function (e) {
						that._verifyObjects(e);
						that.cursorShouldChange();
					}
				};
			}(this), this.getState(this._defaultState)));

			this._override("initialize", _.bind(function (widget, toolManager) {
				this._super(widget, toolManager);
				this.changeState(this._defaultState);
				this.registerPassiveEvents(["mousemove", "mousedown"]);
				this.registerActiveEvents(["mousemove", "mousedown", "mouseup", "mouseout", "mouseleave", "contextmenu"]);
			}, this));

			this._override("activate", _.bind(function (interruptRequest) {
				this._super(interruptRequest);
				this.changeState(this._hoverState);
			}, this));

			this._override("deactivate", _.bind(function (interruptRequest) {
				this._super(interruptRequest);
				this.changeState(this._defaultState);
			}, this));

			this.f = f;
		};
		
		// inherit
		DynContentTool.prototype = Object.create(AbstractTool.prototype);
		// correct the constructor pointer 
		DynContentTool.prototype.constructor = DynContentTool;

		
		/**
		 * Returns default select cursor
		 */
		DynContentTool.prototype.getDefaultCursor = function () {
			return this._cursorForAnchor();			
		};

		/**
		 * Deactivate the current Dynamic Content tool
		 * @param that
		 * @param e
		 */
		DynContentTool.prototype._deActivate = function (e) {
			if (this.isActive()) {
				this.changeState(this._defaultState);
				this.f.emit("toolManager.request.deactivation", this);
			}
		};
		
		/**
		 * @returns {String} the cursor for the anchor state
		 */
		DynContentTool.prototype._cursorForAnchor = function () {
			return "cursor-select";
		};
		
		/**
		 * Get the dynamic content layer based on a point
		 * @param {String|Event} either the name of the view or an event that will calculate the view of the dynamic content layer
		 * @returns {ViewportLayer} Annotations layer of null if not found
		 */
		DynContentTool.prototype._getDynContentLayer = function (e) {
			var v = this._widget.requestView(e);
			if (v) {
				var l = v.getLayer("Dyncontent");
				if (l) {
					return l;
				}
			}
			return null;
		};
		
		/**
		 * Get the closest linework from a given point
		 * @param {Event} e - mouse event to calcuate the point to
		 * @param {ViewportLayer} [layer] - if the layer was already fetched
		 * @returns {LineworkWrapper} the closest lineworkwrapper to a certain point
		 */
		DynContentTool.prototype._getClosestLineworkFromPoint = function (e, layer) {
			var l = layer;
			if (!l) {
				l = this._getDynContentLayer(e);
			}

			if (l) {
				return l.getClosestLineworkFromPoint(this.getScenePoint(e));
			} else {
				return null;
			}
		};

		/**
		 * Converts a point from an event to the world, makes sure the point is containt in the document page
		 * @param {Event} e - mouse event
		 * @returns {ShapeUtils.Point} point in world coordinates
		 */
		DynContentTool.prototype._getWorldPoint = function (e) {
			var activePage = this._dynContentModel.getDynContentPage();
			var worldPoint = this.keepPointWithinWorldBounds(this.getWorldPoint(e), false);

			//convert the coordinate system from world coordinates to annotation coordinates
			worldPoint.y = activePage.getPageSize().height - worldPoint.y;

			var origin = activePage.getDocumentPageOrigin();

			worldPoint.offset(new ShapeUtils.Size(-origin.x, origin.y));

			var trimboxSize = activePage.getTrimboxRect().size;
			var missingHeight = 0; //pages with wrong XMP-data (page size) can have a missing part in the height, annotationlocation needs to be compensated for this
			if (trimboxSize.width !== 0 && trimboxSize.height !== 0) {
				missingHeight = activePage.getPageSize().height - (trimboxSize.height + activePage.getMargins().top + activePage.getMargins().bottom);
			}

			var margin = activePage.getMargins();
			if (activePage.isReaderSpread() && activePage.get("isRightPage")) { //check if the active page is on the right side of reader spread
				margin.left = -margin.left;
			}
			worldPoint.offset(new ShapeUtils.Size(-margin.left, -margin.bottom - missingHeight));

			return worldPoint;
		};

		/**
		 * Verify the current situation, so calcutate where the mouse is for the moment on the canvas
		 * @param {Event} e - mouse event to calcuate the point to
		 * @param {ViewportLayer} [layer] - if the layer was already fetched
		 */
		DynContentTool.prototype._verifyObjects = function (e, layer) {
			if (!layer) {
				layer = this._getDynContentLayer(e);
				if (!layer) {
					return;
				}
			}
			var object = this._getClosestLineworkFromPoint(e, layer);

			this._targetDynContent = null;
			this._anchor = null;
			if (object) {
				if (object instanceof DynContentAnnotation) {
					this._targetDynContent = object;
				} else if (object.anchor) {
					this._anchor = object;
					this._targetDynContent = object.linework;
				}
			}
			if (!object && !this._targetDynContent) {
				this._deActivate(e);
			}
		};
		
		
		return DynContentTool;
});