define(["web-viewer/layout/AbstractWidget",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/layout/component/MiniButton",
		"web-viewer/layout/component/DropdownMiniButton",
		"packaging-content-management/ContentConstants",
		"packaging-content-management/text-content-utils/LanguageUtils",
		"utils/ShapeUtils",
		"jquery",
		"underscore",
		"web-viewer/textContent/DynContentEntry",
		"utils/webi18n!viewer",
		"packaging-content-management/dyn-content-services/dynContentService"],
	function (AbstractWidget,
		LayoutConstants,
		MiniButton,
		DropdownMiniButton,
		ContentConstants,
		languageUtils,
		ShapeUtils,
		$,
		_,
		DynContentEntry,
		i18nViewerStrings,
		dynContentService) {
		"use strict";

		/**
		 * @exports DynContentWidget
		 * @class DynContentWidget
		 * @param {TextContentModule} textContentModule - 
		 * @param {Core.ViewerFacade} f - 
		 * @param {Array} contentStatementList
		 * @param {Array} elementTypeList
		 * @param {Object} languagesList
		 */
		var DynContentWidget = function DynContentWidget(textContentModule, f, contentStatementList, elementTypeList, languagesList) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.TEXTCONTENT);

			// The module.
			this._textContentModule = textContentModule;
			this._dynContentModel = textContentModule.dynContentModel;

			// Viewer facade
			this.f = f;
			
			//Attribute category module required for full view
			this._acatModule = null;

			// Set the icon
			this.setIcon("icon icon-WCR_Measure-Text");

			// Array of complete element types
			this._elementTypeList = elementTypeList;

			// Array of TextContent documents referenced to the artwork
			this._contentStatementList = contentStatementList;
			this._sortContentStatementList();

			// Array of master and translation languages
			this._languagesList = languagesList;
			this._projectMasterLangs = languagesList.projectMasterLangs;
			this._projectTranslationLangs = languagesList.projectTranslationLangs;
			this._allMasterLangs = languagesList.allMasterLangs;
			this._allTranslationLangs = languagesList.allTranslationLangs;

			// Array of langues
			this._allAvailableMasterLangs = [];
			this._allAvailableTranslationLangs = [];
			this._findAllAvailableLanguages();

			// The wrapping div
			this._wrapper = $('<div>').addClass("DynContentViewerWidget");

			// Create the widget buttons and dropdowns
			this._saveButton = null;
			this._localeMappingButton = null;
			this._languageSelectDropdown = null;
			this._statusFilterDropdown = null;
			this._fullWidthViewButton = null;

			// The selected languages/state to filter on.
			this._selectedMasterLanguage = '';
			this._selectedTranslationLanguage = '';
			this._selectedState = undefined;

			// The create dynamic content entries ...
			this._dynContentEntries = [];

			// Create empty arrays for handling locale mapping
			this._allLocales = []; // All locales that is supported by the artwork
			this._selectDropDown = []; // Array if language mapping that is shown to user - locale to languages available in project
			this._dynContentLocale = []; // Holds array of TextDynContentLocale
			
			// Options for the widget
			this._options = {};
			this._options.viewOptions = {
				widgetview: 'viewer'
			};
			this._options.f = this.f;

			// Set the widget title
			this.setTitle(i18nViewerStrings.t("PCMTextWidgetTitle"));

			// Construct the widget ...
			this._constructWidget();

			/*
			 * When a dynamic content annotation was selected ...
			 */
			this._dynContentModel.on("selectedDynContents", _.bind(function (dynamicContentAnnotations, isDirectSelect) {
				// Extract the dynamic content objects from the dynamic content annotations.
				var selectedDynContents = [];
				for (var i = 0; i < dynamicContentAnnotations.length; i++) {
					selectedDynContents.push(dynamicContentAnnotations[i].getDynContent());
				}
				// highlight the selected dynamic contents.
				if (selectedDynContents.length > 0) {
					this.highlight(selectedDynContents, isDirectSelect);
				}
			}, this));

			/*
			 * When a new unsaved Text Content object was added ...
			 */
			this._dynContentModel.on("unsavedTextContent", _.bind(function (dynamicContentAnnotations, isDirectSelect) {
				this.updateSaveButton();
			}, this));

			/*
			 * When a text element was changed ...
			 */
			this._dynContentModel.on("textElementChanged", _.bind(function (textElement) {

				// Find the affected dynContentEntry.
				var affected;
				for (var i = 0; i < this._dynContentEntries.length; i++) {
					var contentStatement = this._dynContentEntries[i].getContentStatement();
					if (textElement.parentStatement == contentStatement) {
						affected = this._dynContentEntries[i];
					}
				}

				// If an affected entry was found.
				if (affected) {
					// Update it.
					affected.update(textElement);
				}

			}, this));

			/*
			 * When the content of the model changed. 
			 */
			this._dynContentModel.on("modelContentChanged", _.bind(function () {
				// Extract the text content documents from the annotations in the model.
				var annotations = this._dynContentModel.getDynContents();
				this._contentStatementList = [];
				for (var i = 0; i < annotations.length; i++) {
					if (annotations[i].getDynContent().getDynContentType() == "Text") {
						var element = annotations[i].getDynContent().getTextElement();
						var contentStatement = element.parentStatement;
						if ($.inArray(contentStatement, this._contentStatementList) < 0) {
							this._contentStatementList.push(contentStatement);
						}
					}
				}
				// Update the widget for these new content statements.
				this._sortContentStatementList();
				this._createDynContentEntries();
				this.updateSaveButton();
			}, this));
			
			// while saving, disable save button
			this._dynContentModel.on("save.started", _.bind(function () {
				if(this._saveButton){
					this._saveButton.setEnabled(false);
				}
			}, this));

		};

		DynContentWidget.prototype = Object.create(AbstractWidget.prototype);
		DynContentWidget.prototype.constructor = DynContentWidget;
		DynContentWidget.widgetName = LayoutConstants.Widget.TEXTCONTENT;

		/**
		 * 
		 */
		DynContentWidget.prototype._constructWidget = function () {
			this._createSaveButton();
			this._createLanguageSelectionButton();
			this._createLocaleMappingButton();
			this._createStatusFilterButton();
			this._createFullWidthViewButton();
			this._createDynContentEntries();
		};

		DynContentWidget.prototype._createSaveButton = function () {
			this._saveButton = new MiniButton(this.f, "saveDynContents", "icon-WCR_Save", true, _.bind(function () {
				// start saving annotations
				this._dynContentModel.save();
			}, this), "fontIconDisabled icon-WCR_Save");
			this.updateSaveButton();
		};

		DynContentWidget.prototype._createLanguageSelectionButton = function () {
			this._languageSelectDropdown = new DropdownMiniButton(this.f, "pcmLangSelection", "icon-WCR_Text-Translation2");
			this._languageSelectDropdown.setTooltip("viewer", "PCMSelLangTT");
			this._languageSelectDropdown.addButtonClass("pcmSelectLang");

			var onMasterLangChange = _.bind(function (selectedLangCode, toggle) {
				for (var i = 0; i < this._allAvailableMasterLangs.length; i++) {
					this._languageSelectDropdown.toggleOption(this._allAvailableMasterLangs[i].langCode, selectedLangCode === this._allAvailableMasterLangs[i].langCode);
					if (selectedLangCode === this._allAvailableMasterLangs[i].langCode) {
						this._selectedMasterLanguage = selectedLangCode;
					}
				}
				this._updateStatusFilterCount();
				this._createDynContentEntries();
			}, this);

			if (this._allAvailableMasterLangs && this._allAvailableMasterLangs.length > 0) {
				this._languageSelectDropdown.addHeader(this.f.i18n.t("PCMMasterLangMenuHeader"));

				for (var i = 0; i < this._allAvailableMasterLangs.length; i++) {
					var language = this._allAvailableMasterLangs[i];

					var selected = false;
					if (i === 0) {
						this._selectedMasterLanguage = language.langCode;
						selected = true;
					}

					this._languageSelectDropdown.addOption({
						name: language.langCode,
						content: language.langDesc,
						onchange: _.bind(onMasterLangChange, this, language.langCode),
						selected: selected,
						userCannotUncheck: true
					});
				}
			}

			var onTranslationLangChange = _.bind(function (selectedLangCode, toggle) {
				for (var i = 0; i < this._allAvailableTranslationLangs.length; i++) {
					this._languageSelectDropdown.toggleOption(this._allAvailableTranslationLangs[i].langCode, selectedLangCode === this._allAvailableTranslationLangs[i].langCode);
					if (selectedLangCode === this._allAvailableTranslationLangs[i].langCode) {
						this._selectedTranslationLanguage = selectedLangCode;
					}
				}
				this._dynContentModel.setCurrentTransLangCode(this._selectedTranslationLanguage);
				this._updateStatusFilterCount();
				this._createDynContentEntries();
			}, this);

			if (this._allAvailableTranslationLangs && this._allAvailableTranslationLangs.length > 0) {
				this._languageSelectDropdown.addHeader(this.f.i18n.t("PCMTranslationLangMenuHeader"));

				for (var i = 0; i < this._allAvailableTranslationLangs.length; i++) {
					var language = this._allAvailableTranslationLangs[i];

					if (language) {
						var selected = false;
						if (i === 0) {
							this._selectedTranslationLanguage = language.langCode;
							this._dynContentModel.setCurrentTransLangCode(this._selectedTranslationLanguage);
							selected = true;
						}

						this._languageSelectDropdown.addOption({
							name: language.langCode,
							content: language.langDesc,
							onchange: _.bind(onTranslationLangChange, this, language.langCode),
							selected: selected,
							userCannotUncheck: true
						});
					}
				}
			}

			if ((!this._allAvailableMasterLangs || this._allAvailableMasterLangs.length === 0) && (!this._allAvailableTranslationLangs || this._allAvailableTranslationLangs.length === 0)) {
				this._languageSelectDropdown.addHeader(this.f.i18n.t("PCMLangNotAddedTT"));
			}
		};
				
		/**
		 * Create locale mapping button. Make ajax call and get all the locales and location lists
		 */
		DynContentWidget.prototype._createLocaleMappingButton = function () {
			
			this._localeMappingButton = new DropdownMiniButton(this.f, "pcmLocaleMapping", "icon-WCR_Actions-Convert12");
			this._localeMappingButton.setTooltip("viewer", "PCMLocaleMappingTT");
			this._localeMappingButton.addButtonClass("pcmSelectLang");
			
			// Get all the locales that are available on this artwork
			var dynContLocaleReq = dynContentService.getDynContentLocale( this.f.getActiveDocument().getDocumentVersion().get("docversionid"));
			$.when(dynContLocaleReq).done(function (dynContLocale) {
				this._dynContentLocale = dynContLocale;
				for (var i = 0; i < dynContLocale.length; i++) {
					var locale = dynContLocale[i].locale;
					if ($.inArray(locale, this._allLocales) == -1) 
						this._allLocales.push(locale);
					
				}	
				this._allLocales.sort();
				
				// Create the locale mapping table.
				var localeMapTable = $("<table>", {"class" : "LocaleMapping"});
				var localeMapThead = $("<thead>", {"class" : "Header"});
				var localeMapTBody = $("<tbody>", {"class" : "Entries"});
				var localeMapFooter = $("<tbody>", {"class" : "Footer"});
				var localeMapTheadRow = $("<tr>");
				var localeMapTheadCell1 = $("<th>").text(this.f.i18n.t("PCMLocale"));
				var localeMapTheadCell2 = $("<th>").text(this.f.i18n.t("PCMLanguage"));
				localeMapTheadRow.append(localeMapTheadCell1, localeMapTheadCell2);
				localeMapThead.append(localeMapTheadRow);
				localeMapTable.append(localeMapThead);
				localeMapTable.append(localeMapTBody);
				localeMapTable.append(localeMapFooter);
				
				this._localeMappingButton.addContent(localeMapTable);
				
				// For every locale create an entry with locale id and drop down of all languages in project	
				for(var i=0; i< this._allLocales.length; i++){
					
					// Create a row for this locale mapping.
					var localeMapRow = $("<tr>");
					var localeLabel = $("<td>", {"class" : "localeCell"}).text(this._allLocales[i]);
					var localeLangSelector = $("<td>", {"class" : "langCell"});
					localeMapRow.append(localeLabel, localeLangSelector);
					localeMapTBody.append(localeMapRow);
					
					// Create the language selector.
					var languageCombo = $("<select>");
					localeLangSelector.append(languageCombo);
					var blankOption = $("<option/>").text("").val("blank");
					languageCombo.append(blankOption);
					
					// Add Master languages to drop down
					if (this._allAvailableMasterLangs && this._allAvailableMasterLangs.length > 0) {						
						for (var j = 0; j < this._allAvailableMasterLangs.length; j++) {
							var language = this._allAvailableMasterLangs[j];
							if (language) {									
								var option = $("<option/>").text(language.langDesc).val(language.langCode);
								languageCombo.append(option);
							}
						}
					}
					
					// Add Translation languages to drop down
					if (this._allAvailableTranslationLangs && this._allAvailableTranslationLangs.length > 0) {						
						for (var k = 0; k < this._allAvailableTranslationLangs.length; k++) {
							var language = this._allAvailableTranslationLangs[k];
							if (language) {								
								var option = $("<option/>").text(language.langDesc).val(language.langCode);
								languageCombo.append(option);
							}
						}
					}
					this._selectDropDown.push(languageCombo);
				}

				//Create a OK button
				var localeMapFooterRow = $("<tr>");
				var localeMapFooterCell = $("<td>", {"colspan" : "2"});
				var langButton = $("<button>").addClass("btn btn-primary").attr({"type":"button"}).text("OK");
				localeMapFooter.append(localeMapFooterRow);
				localeMapFooterRow.append(localeMapFooterCell);
				localeMapFooterCell.append(langButton);

				// On click of OK Button, update all DynContentAnnotations with new location list for corresponding language and instance id
				langButton.on("click", _.bind(function(){
					this._localeMappingButton.dropdown.hide();
					this._dynContentModel.localeMappingAction(this._selectDropDown, this._dynContentLocale, this._allLocales);
				}, this));
				
			}.bind(this)).fail(_.bind(function (errorMsg) { 
				this._localeMappingButton.setTooltip("viewer", "PCMDynContentLocaleErrorMessage");
			}, this));
		};
		
		
		
		DynContentWidget.prototype._createStatusFilterButton = function () {
			this._statusFilterDropdown = new DropdownMiniButton(this.f, "pcmStatusFilter", "icon-WCR_Filter");
			this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterTT");
			this._statusFilterDropdown.addButtonClass("textStatusFilter");
			this._statusFilterDropdown.element.click(_.bind(function () {
				this._updateStatusFilterCount();
			}, this));

			this._statusFilterDropdown.addAction({
				name: "pcmDraftCountText",
				content: this.f.i18n.t("PCMDraftBtnTT") + " (0)",
				icon: "icon-WCR_Text-Status-Draft",
				enabled: true,
				action: _.bind(function (event) {
					if (!$(event.target).hasClass("selectedFilterState")) {
						$(this._statusFilterDropdown.dropdown.menu).find(".selectedFilterState").removeClass("selectedFilterState");
						$(event.target).addClass("selectedFilterState");
						this._selectedState = ContentConstants.State.DRAFT;
						this._statusFilterDropdown.setWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterActiveTT");
					} else {
						$(event.target).removeClass("selectedFilterState");
						this._selectedState = undefined;
						this._statusFilterDropdown.removeWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterTT");
					}
					this._dynContentModel.setDynContentFilterState(this._selectedState);
					this._createDynContentEntries();
					this._fireResized();
				}, this)
			});

			this._statusFilterDropdown.addAction({
				name: "pcmRFApprovalCountText",
				content: this.f.i18n.t("PCMPendingApprovalBtnTT") + " (0)",
				icon: "icon-WCR_Text-Status-Pending",
				enabled: true,
				action: _.bind(function (event) {
					if (!$(event.target).hasClass("selectedFilterState")) {
						$(this._statusFilterDropdown.dropdown.menu).find(".selectedFilterState").removeClass("selectedFilterState");
						$(event.target).addClass("selectedFilterState");
						this._selectedState = ContentConstants.State.READY_FOR_APPROVAL;
						this._statusFilterDropdown.setWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterActiveTT");
					} else {
						$(event.target).removeClass("selectedFilterState");
						this._selectedState = undefined;
						this._statusFilterDropdown.removeWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterTT");
					}
					this._dynContentModel.setDynContentFilterState(this._selectedState);
					this._createDynContentEntries();
					this._fireResized();
				}, this)
			});

			this._statusFilterDropdown.addAction({
				name: "pcmApprovedCountText",
				content: this.f.i18n.t("PCMTextSignedOff") + " (0)",
				icon: "icon-WCR_Text-Status-Approved icon-WCR_Approve",
				enabled: true,
				action: _.bind(function (event) {
					if (!$(event.target).hasClass("selectedFilterState")) {
						$(this._statusFilterDropdown.dropdown.menu).find(".selectedFilterState").removeClass("selectedFilterState");
						$(event.target).addClass("selectedFilterState");
						this._selectedState = ContentConstants.State.APPROVED;
						this._statusFilterDropdown.setWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterActiveTT");
					} else {
						$(event.target).removeClass("selectedFilterState");
						this._selectedState = undefined;
						this._statusFilterDropdown.removeWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterTT");
					}
					this._dynContentModel.setDynContentFilterState(this._selectedState);
					this._createDynContentEntries();
					this._fireResized();
				}, this)
			});

			this._statusFilterDropdown.addAction({
				name: "pcmRejectedCountText",
				content: this.f.i18n.t("PCMTextSignoffRejected") + " (0)",
				icon: "icon-WCR_Text-Status-Rejected icon-WCR_Reject",
				enabled: true,
				action: _.bind(function (event) {
					if (!$(event.target).hasClass("selectedFilterState")) {
						$(this._statusFilterDropdown.dropdown.menu).find(".selectedFilterState").removeClass("selectedFilterState");
						$(event.target).addClass("selectedFilterState");
						this._selectedState = ContentConstants.State.REJECTED;
						this._statusFilterDropdown.setWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterActiveTT");
					} else {
						$(event.target).removeClass("selectedFilterState");
						this._selectedState = undefined;
						this._statusFilterDropdown.removeWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterTT");
					}
					this._dynContentModel.setDynContentFilterState(this._selectedState);
					this._createDynContentEntries();
					this._fireResized();
				}, this)
			});
		};

		/**
		 * Update the count in the filter.
		 */
		DynContentWidget.prototype._updateStatusFilterCount = function () {
			var statusCount = {
				draft: 0,
				pending: 0,
				approved: 0,
				rejected: 0
			};
			for (var i = 0; i < this._contentStatementList.length; i++) {
				for (var j = 0; j < this._contentStatementList[i].element.length; j++) {
					var state = this._contentStatementList[i].element[j].state;
					if (state == ContentConstants.State.DRAFT && (this._selectedMasterLanguage == this._contentStatementList[i].element[j].language
							|| this._selectedTranslationLanguage == this._contentStatementList[i].element[j].language)) {
						statusCount.draft++;						
					}
					
				}
			}
			for (var i = 0; i < this._contentStatementList.length; i++) {
				for (var j = 0; j < this._contentStatementList[i].element.length; j++) {
					var state = this._contentStatementList[i].element[j].state;
					if (state == ContentConstants.State.READY_FOR_APPROVAL && (this._selectedMasterLanguage == this._contentStatementList[i].element[j].language
							|| this._selectedTranslationLanguage == this._contentStatementList[i].element[j].language)) {
						statusCount.pending++;
					}
					
				}
			}
			for (var i = 0; i < this._contentStatementList.length; i++) {
				for (var j = 0; j < this._contentStatementList[i].element.length; j++) {
					var state = this._contentStatementList[i].element[j].state;
					if (state == ContentConstants.State.APPROVED && (this._selectedMasterLanguage == this._contentStatementList[i].element[j].language
							|| this._selectedTranslationLanguage == this._contentStatementList[i].element[j].language)) {
						statusCount.approved++;
					}
					
				}
			}
			for (var i = 0; i < this._contentStatementList.length; i++) {
				for (var j = 0; j < this._contentStatementList[i].element.length; j++) {
					var state = this._contentStatementList[i].element[j].state;
					if (state == ContentConstants.State.REJECTED && (this._selectedMasterLanguage == this._contentStatementList[i].element[j].language
							|| this._selectedTranslationLanguage == this._contentStatementList[i].element[j].language)) {
						statusCount.rejected++;
					}
					
				}
			}

			var draftLabel = this.f.i18n.t("PCMDraftBtnTT") + " (" + statusCount.draft + ")";
			var pendingLabel = this.f.i18n.t("PCMPendingApprovalBtnTT") + " (" + statusCount.pending + ")";
			var approvedLabel = this.f.i18n.t("PCMTextSignedOff") + " (" + statusCount.approved + ")";
			var rejectedLabel = this.f.i18n.t("PCMTextSignoffRejected") + " (" + statusCount.rejected + ")";

			var updateDOMElement = function (htmlElement, statusLabel) {
				if (htmlElement) {
					htmlElement.textContent = statusLabel;
				}
			};

			updateDOMElement($(".pcmDraftCountText a[name='btn_pcmDraftCountText']").first().contents().last()[0], draftLabel);
			updateDOMElement($(".pcmRFApprovalCountText a[name='btn_pcmRFApprovalCountText']").first().contents().last()[0], pendingLabel);
			updateDOMElement($(".pcmApprovedCountText a[name='btn_pcmApprovedCountText']").first().contents().last()[0], approvedLabel);
			updateDOMElement($(".pcmRejectedCountText a[name='btn_pcmRejectedCountText']").first().contents().last()[0], rejectedLabel);
		};

		DynContentWidget.prototype._createFullWidthViewButton = function () {
			var fullWidthViewButtonHandler = _.bind(function () {
				var callback = _.bind(function () {
					//AttributeCategoryModule is already loaded while creating the button
					//Just directly open the "TEXTVIEW" in the modal popup
					if( this._acatModule ){
						this._acatModule.showViewInModalPopup("TEXTVIEW", this.f.i18n.t("pcmFullViewTitle"), this.f.i18n.t("PCMFullViewSaveBtnTT"));
					}
				}, this);

				//Before opening the Full view(Attributes View), prompt the user with unsaved text content changes popup
				//promptToSavedDynamicContent returns false if there were no pending save items in the list. Then we can 
				//directly call callback to show the full width mode
				var unsavedDynContent = this._dynContentModel.promptToSaveUnsavedDynamicContent(callback);
				//No unsaved text content elements found, so directly open the full width mode
				if(!unsavedDynContent){
					callback();
				}
			}, this);


			this._fullWidthViewButton = new MiniButton(this.f, "pcmFullView", "icon-WCR_FullScreen", true, fullWidthViewButtonHandler);
			this._fullWidthViewButton.setEnabled(false);

			//AttributeCategoryModule is required to open the full view
			this.f.requestModule("attributeCategory", _.bind(function (acatModule) {
				this._acatModule = acatModule;
				//Enable full view button only if view "TEXTVIEW" exists
				if($.inArray("TEXTVIEW", this.f.getActivePage().getAllAttributeCategoryViews()) !== -1){
					this._fullWidthViewButton.setEnabled(true);
					this._fullWidthViewButton.setTooltip("viewer", "PCMOpenFullViewTT");
				} else {
					this._fullWidthViewButton.setTooltip("viewer", "PCMNoFullViewTT");
				}
			}, this), false);
		};


		DynContentWidget.prototype.updateSaveButton = function () {
			if (this._saveButton) {
				var unsavedCount = this._dynContentModel.getUnsavedTextContentsCount();
				if (unsavedCount > 0) {
					this._saveButton.setEnabled(true);
					this._saveButton.setBadge(unsavedCount);
					this._saveButton.setTooltip("viewer", "PCMSaveMiniBtnTT", [unsavedCount]);
				} else {
					this._saveButton.setEnabled(false);
				}
			}
		};

		/**
		 * 
		 */
		DynContentWidget.prototype.highlight = function (selectedDynContents, isDirectSelect) {

			// If this was a direct single select, we need to make sure that the language and state which was selected
			// is not hidden by a filter ... if so ... reset the filter.
			if (isDirectSelect && selectedDynContents.length == 1) {
				this.ensureVisibility(selectedDynContents[0]);
			}

			// Find the content statements that we need to highlight.
			var contentStatementsToHighlight = [];
			for (var i = 0; i < selectedDynContents.length; i++) {
				var contentStatement = selectedDynContents[i].getTextElement().parentStatement;
				if ($.inArray(contentStatement, contentStatementsToHighlight) < 0) {
					contentStatementsToHighlight.push(contentStatement);
				}
			}

			// Unhighlight/highlight the right dynContentEntries
			for (var i = 0; i < this._dynContentEntries.length; i++) {
				var contentStatement = this._dynContentEntries[i].getContentStatement();
				if ($.inArray(contentStatement, contentStatementsToHighlight) < 0) {
					this._dynContentEntries[i].unHighlight();
				} else {
					this._dynContentEntries[i].highlight();
				}
			}
		};

		/**
		 * Make sure that the passed dynContent object is visible in the DynContent Widget.
		 * 
		 * @param dynContent
		 */
		DynContentWidget.prototype.ensureVisibility = function (dynContent) {
			var element = dynContent.getTextElement();
			var filterChanged = false;
			// Check that the correct master language is displayed.
			if (element.isMaster() && element.language != this._selectedMasterLanguage) {
				this._languageSelectDropdown.toggleOption(this._selectedMasterLanguage, false);
				this._selectedMasterLanguage = element.language;
				this._languageSelectDropdown.toggleOption(this._selectedMasterLanguage, true);
				filterChanged = true;
			}
			// Check that the correct translation language is displayed.
			if (!element.isMaster() && element.language != this._selectedTranslationLanguage) {
				this._languageSelectDropdown.toggleOption(this._selectedTranslationLanguage, false);
				this._selectedTranslationLanguage = element.language;
				this._languageSelectDropdown.toggleOption(this._selectedTranslationLanguage, true);
				this._dynContentModel.setCurrentTransLangCode(this._selectedTranslationLanguage);
				filterChanged = true;
			}
			// Check that the correct state is displayed, e.g. check that the parent contentstatement has an element witht the selected state.
			if (this._selectedState != undefined && !this._containsState(element.parentStatement, this._selectedState)) {
				this._selectedState = undefined;
				$(this._statusFilterDropdown.dropdown.menu).find(".selectedFilterState").removeClass("selectedFilterState");
				this._statusFilterDropdown.removeWarning();
				this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterTT");
				filterChanged = true;
			}

			// If the filter changed.
			if (filterChanged) {
				this._dynContentModel.setDynContentFilterState(this._selectedState);
				this._createDynContentEntries();
			}
		};

		/**
		 * Check if the contentStatement contains an element with the given state.
		 */
		DynContentWidget.prototype._containsState = function (contentStatement, state) {
			var foundElementWithState = false;
			for (var j = 0; j < contentStatement.element.length; j++) {
				var textElement = contentStatement.element[j];
				if (textElement.state == state && (textElement.language == this._selectedMasterLanguage || 
						textElement.language == this._selectedTranslationLanguage)) {
					foundElementWithState = true;
					break;
				} else if (textElement.pastApprovals.length > 0) {
					var lastApprovedState = textElement.pastApprovals[textElement.pastApprovals.length - 1].state;
					if (lastApprovedState == state  && (textElement.language == this._selectedMasterLanguage || 
							textElement.language == this._selectedTranslationLanguage)) {
						foundElementWithState = true;
						break;
					}
				}
			}
			return foundElementWithState;
		};

		/**
		 * Create the dynamic content entries.
		 */
		DynContentWidget.prototype._createDynContentEntries = function () {

			// Empty the dynamic content entries
			this._wrapper.empty();
			this._dynContentEntries = [];

			// Create a new wrapper for the dynamic content entries.
			this._entryWrapper = $('<div>').addClass("DynContentEntries");
			this._wrapper.append(this._entryWrapper);

			if (this._dynContentModel.getDynContentsLoading()) {
				this._entryWrapper.append($('<div>', {
					"class": "NoTextContentsAvailable"
				}).text(i18nViewerStrings.t("PCMDynContentLoading")));
				return;
			}

			// Make sure languages were selected.
			if (this._allAvailableMasterLangs.length === 0 && this._allAvailableTranslationLangs.length === 0) {
				this._entryWrapper.append($('<div>', {
					"class": "NoTextContentsAvailable"
				}).text(i18nViewerStrings.t("PCMNoLangSelected")));
				return;
			}

			if ((this._allAvailableMasterLangs && this._allAvailableMasterLangs.length > 0) || 
					(this._allAvailableTranslationLangs && this._allAvailableTranslationLangs.length > 0)) {

				var buildSelectedCallback = function (dynContentEntry) {
					return function (langCode) {
						// When the dynamic content entry is selected do ...
						var contentStatement = dynContentEntry.getContentStatement();
						// Ask the module to select on dyn content annotations based on this statement.
						this._textContentModule.selectDynContentBasedOn(contentStatement, langCode);
					}.bind(this);
				}.bind(this);

				for (var i = 0; i < this._contentStatementList.length; i++) {
					var contentStatement = this._contentStatementList[i];

					// Check if there's a filter value on state.
					if (this._selectedState != undefined && !this._containsState(contentStatement, this._selectedState)) {
						// If we did not find element with the selected state, skip this contentstatement.
						continue;
					}

					var dynContentEntry = new DynContentEntry(contentStatement, this._selectedMasterLanguage, 
							this._selectedTranslationLanguage, this._languagesList, this._elementTypeList);
					dynContentEntry.onSelected(buildSelectedCallback(dynContentEntry));
					dynContentEntry.onElementSelected(buildSelectedCallback(dynContentEntry));
					this._entryWrapper.append(dynContentEntry.getJQElement());
					this._dynContentEntries.push(dynContentEntry);
				}
			}

			// no cards are available display no text contents message
			if (this._entryWrapper.children().length === 0) {
				this._entryWrapper.append($('<div>', {
					"class": "NoTextContentsAvailable"
				}).text(i18nViewerStrings.t("PCMNoDynContentFound")));
			}
		};

		/**
		 * Sort the content statements on color and element type name.
		 */
		DynContentWidget.prototype._sortContentStatementList = function () {

			// Define a function which compare two contentstatements ...
			var compare = function (c1, c2) {

				if (!this._elementTypeList[c1.elementType]) {
					return -1;
				}

				var color1 = this._elementTypeList[c1.elementType].color;
				var name1 = this._elementTypeList[c1.elementType].elementHumanReadable;
				var color2 = this._elementTypeList[c2.elementType] !== undefined ? this._elementTypeList[c2.elementType].color : "";
				var name2 = this._elementTypeList[c2.elementType] !== undefined ? this._elementTypeList[c2.elementType].elementHumanReadable : c2.elementType;

				// First sort on color ...
				if (color1 != color2) {
					// If the colors differ ... return wether we have the biggest color.
					if (color1 < color2) {
						return -1;
					} else {
						return 1;
					}
				} else {
					// If it's the same color ... sort on name ...
					if (name1 < name2) {
						return -1;
					} else if (name1 > name2) {
						return 1;
					} else {
						return 0;
					}
				}
			}.bind(this);
			this._contentStatementList.sort(compare);
		};

		/**
		 * Find all the available languages in the current content statement list.
		 */
		DynContentWidget.prototype._findAllAvailableLanguages = function () {
			// Iterate over all the contentStatements.
			for (var i = 0; i < this._contentStatementList.length; i++) {
				var contentStatement = this._contentStatementList[i];
				// Iterate over all the text elements.
				for (var j = 0; j < contentStatement.element.length; j++) {
					var element = contentStatement.element[j];
					// Check whether the element is a master or translation language.
					if (element.isMaster()) {
						// Get the master language of the element.
						var masterLang = languageUtils.getLangForCode(this._projectMasterLangs, element.language);
						// Check whether this master language of the element is already added to the "_allAvailableMasterLangs" array.
						if (masterLang && $.inArray(masterLang, this._allAvailableMasterLangs) < 0) {
							this._allAvailableMasterLangs.push(masterLang);
						}
					} else {
						// Get the master language of the element.
						var transLang = languageUtils.getLangForCode(this._projectTranslationLangs, element.language);
						// Check whether this translation language of the element is already added to the "_allAvailbeTransLangs" array.
						if (transLang && $.inArray(transLang, this._allAvailableTranslationLangs) < 0) {
							this._allAvailableTranslationLangs.push(transLang);
						}
					}
				}
			}
		};

		/**
		 * Set the context of the widget
		 * @override
		 * @param context
		 * @param headerExtra
		 */
		DynContentWidget.prototype.setContext = function (context, headerExtra) {
			AbstractWidget.prototype.setContext.call(this, context, headerExtra);
			this.context.addClass("enableTextSelection");
			this.headerExtra.append(this._saveButton.element).append(this._localeMappingButton.element).append(this._languageSelectDropdown.element).append(this._statusFilterDropdown.element).append(this._fullWidthViewButton.element);
			this._wrapper.appendTo(this.context);
		};

		/**
		 * The text content  widget is greedy, and wants to have all available extra space
		 * @override
		 * @returns {Boolean} true
		 */
		DynContentWidget.prototype.willFill = function () {
			return true;
		};

		/**
		 * Get the preferred size
		 * @override
		 */
		DynContentWidget.prototype.getPreferredSize = function () {
			return new ShapeUtils.Size(this._entryWrapper.outerWidth(true), this._entryWrapper.outerHeight(true));
		};

		/**
		 * Get the preferred size
		 * @override
		 */
		DynContentWidget.prototype.getMinimumSize = function () {
			return new ShapeUtils.Size(200, 100);
		};


		return DynContentWidget;
	});