define('web-viewer/view3D/View3DWidget',[
		"web-viewer/layout/AbstractWidget",
		"web-viewer/layout/component/MiniButton",
		"web-viewer/layout/LayoutConstants",
		"utils/ShapeUtils",
		"component/3DViewerLauncher/3DViewerLauncher",
		"jquery",
		"underscore"
	],
	function (
		AbstractWidget,
		MiniButton,
		LayoutConstants,
		ShapeUtils,
		Viewer3DLauncher,
		$,
		_) {
		"use strict";

		/**
		 * @exports
		 * @memberof Compare
		 * @augments Layout.AbstractWidget
		 * @class The View3DWidget shows the 3D Viewer inside a widget of the HTML5 Viewer.
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 */
		var View3DWidget = function View3DWidget(f) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.VIEW3D);

			/**
			 * @member {Core.ViewerFacade} the viewer facade
			 */
			this.f = f;

			this.setIcon("icon icon-WCR_3D");

			this._3DContent = $("<div>").addClass("View3DContent");

			// intialize close button
			this._closeButton = new MiniButton(this.f, "close3D", "icon-WCR_Close12", true, _.bind(function () {
				this.setEnabled(false);
			}, this));
			this._closeButton.setTooltip("viewer", "PanelClose");

			// intialize the open in new window button
			this._openNewWindow = new MiniButton(this.f, "open3D", "icon-WCR_FullScreen", true, _.bind(function () {
				this.f.getDataAdapter().show3DView(this.f.getActivePage());
				this.setEnabled(false);
			}, this));
			this._openNewWindow.setTooltip("viewer", "open3DViewerInNewWindow");

			this.f.i18n.loadBundle("viewer", _.bind(function () {
				// set the widget title 
				this.setTitle(this.f.t("view3DTitle"));
			}, this));

		};

		View3DWidget.prototype = Object.create(AbstractWidget.prototype);
		View3DWidget.prototype.constructor = View3DWidget;
		View3DWidget.widgetName = LayoutConstants.Widget.VIEW3D;

		/**
		 * Shows the 3D widget and starts to fetch the 3D model
		 * @param {Function} [callback]
		 * @param {Object} [position] - the initial camera position. See setCameraPosition.
		 */
		View3DWidget.prototype.show = function (callback, position) {
			this.whenContextAvailable(function () {
				var activePage = this.f.getActivePage();

				this.makeVisible();

				var docVersionId = activePage.getDocumentVersion().getDocumentVersionID();
				var finalCallback = _.bind(function (success, error) {
					if (success) {
						activePage.set("has3DPerspective", true);
						if (callback) {
							callback();
						}
					}

					this.resized();

					if (!success) {
						this.f.loadBundle("viewer", function () {
							if (!error) {
								error = this.f.t("errorLoading3D");
							}
							this.f.emit("userMessage", "error", error, undefined, false, "3DERROR", false);
							this.setEnabled(false);
						}, this);
					}


				}, this);

				//make sure the 3D Viewer is not initialized with a height smaller than the minimum widget height
				var minimumSize = this.getMinimumSize();
				if (this.context.innerHeight() < minimumSize.height) {
					this.context.innerHeight(minimumSize.height);
				}

				// start loading the 3D view				
				if (!this._3DLauncher) {
					this._3DLauncher = new Viewer3DLauncher(this._3DContent, docVersionId, finalCallback, position);
				} else {
					this._3DLauncher.load3DForDocVersion(docVersionId, finalCallback, position);
				}
			}, this);
		};

		/**
		 * @override
		 */
		View3DWidget.prototype.setContext = function (context, headerExtra) {
			// call the old implementation of setContext() (the implementation of AbstractWidget.js)
			AbstractWidget.prototype.setContext.call(this, context, headerExtra);

			this.context.addClass("view3DWidget");
			this.context.append(this._3DContent);

			headerExtra.append(this._openNewWindow.element);
			headerExtra.append(this._closeButton.element);
		};


		/**
		 * Get the minimum size for the View3DWidget
		 * @returns {ShapeUtils.Size}
		 */
		View3DWidget.prototype.getMinimumSize = function () {
			return new ShapeUtils.Size(250, 300);
		};

		/**
		 * Get the prefered size for the View3DWidget
		 * @returns {ShapeUtils.Size}
		 */
		View3DWidget.prototype.getPreferredSize = function () {
			return new ShapeUtils.Size(250, 300);
		};

		/**
		 * Call when the widget is resizing and the contents needs to adapt
		 */
		View3DWidget.prototype.resizing = function () {
			this.resized();
		};

		/**
		 * This method notifies the widget that it has been resized
		 */
		View3DWidget.prototype.resized = function () {
			if (this._3DLauncher) {
				this._3DLauncher.resize();
			}
		};

		/**
		 * Get the current camera position if the 3D viewer is loaded
		 * @returns {Object} @example 
		 *                   {scale: 1.5,
		 *                   azimuth: 46.6,
		 *                   elevation: -52}
		 */
		View3DWidget.prototype.getCameraPosition = function () {
			if (this._3DLauncher) {
				return this._3DLauncher.getCameraPosition();
			}
			return null;
		};

		/**
		 * Set the current camera position of the 3D widget if the 3D viewer is loaded
		 * @param {Object} position - 
		 *                 @example 
		 *                    {scale: 1.5,
		 *                    azimuth: 46.6,
		 *                    elevation: -52}
		 */
		View3DWidget.prototype.setCameraPosition = function (position) {
			if (this._3DLauncher) {
				this._3DLauncher.setCameraPosition(position);
			}
		};

		return View3DWidget;
	});

define('web-viewer/view3D/View3DToolbarComponent',["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/LayoutConstants"
	],
	function (ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		LayoutConstants
	) {
		"use strict";

		/**
		 * @exports View3DToolbarComponent
		 * @memberof View3DModule
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {View3DModule} viewController
		 */
		var View3DToolbarComponent = function (f, viewController) {
			ToolbarComponent.call(this, LayoutConstants.ToolbarComponent.VIEW3D, f);

			this.f = f;

			this._viewController = viewController;

			var view3DBtnGroup = new ButtonGroup("View3DGroup");
			view3DBtnGroup.setTitle(this.f.i18n, "viewer", "View3DBtnGroupTitle");
			this.addButtonGroup(view3DBtnGroup);

			var view3DButton = new ViewerToolbarButton(this.f, "view3DButton", "icon-WCR_3D", false, _.bind(this._show3DView, this));
			view3DButton.setTooltip("viewer", "Open3DViewTT");
			view3DBtnGroup.addButton(view3DButton);

		};

		View3DToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		View3DToolbarComponent.prototype.constructor = View3DToolbarComponent;
		View3DToolbarComponent.toolbarName = LayoutConstants.ToolbarComponent.VIEW3D;

		/**
		 * Show the 3D view, take unsaved alignment into account.
		 * @private
		 */
		View3DToolbarComponent.prototype._show3DView = function () {
			var compareModule = this.f.getModule("compare");
			if (compareModule) { //pass opening of 3D through the compare module
				compareModule.checkAligmentAndShow3D();
			} else { //no compare module (only CAD is open) load the 3D View
				this._viewController.show3D();
			}
		};

		return View3DToolbarComponent;

	});

/**
 * @namespace View3DModule
 */
define('web-viewer/view3D/View3DModule',["underscore",
		"web-viewer/core/AbstractModule",
		"web-viewer/view3D/View3DWidget",
		"web-viewer/view3D/View3DToolbarComponent",
		"jquery"
	],
	function (_,
		AbstractModule,
		View3DWidget,
		View3DToolbarComponent,
		$) {
		"use strict";
		/**
		 * Creates the View3DModule
		 *
		 * @exports View3DModule
		 * @memberof View3DModule
		 * @class
		 * Responsible for publishing a 3D Widget which can show the 3D model of ARD or Graphics if it's linked to an ARD
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var View3DModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "view3D");

			this._3DWidget = null;
			this._toolbarComponent = null;

			var boundUpdateToolbarVisibility = _.bind(this._updateToolbarVisibility, this);

			this.f.getDocumentController().whenActivePageChangedAndParametersAvailable(function (oldPage, newPage) {

				if (this._3DWidget) { //hide the widget when the active document page changed
					this._3DWidget.setEnabled(false);
					this._emit("stop3D");
				}

				// remove old listeners from old page
				if (oldPage) {
					oldPage.off("property.has3DPerspective.changed", boundUpdateToolbarVisibility);
				}

				// listen to has3DPerspective property changes
				newPage.on("property.has3DPerspective.changed", boundUpdateToolbarVisibility);

				this._updateToolbarVisibility();
			}, this);
		};


		// inherit
		View3DModule.prototype = Object.create(AbstractModule.prototype);
		// correct the constructor pointer 
		View3DModule.prototype.constructor = View3DModule;

		/**
		 * Shows the 3D widget with the HTML5 3D Collada viewer
		 */
		View3DModule.prototype.show3D = function (position) {
			this._emit("start3D");
			if (!this._3DWidget) {
				this._publishWidget(View3DWidget.widgetName, _.bind(function (layoutType) {
					this._3DWidget = new View3DWidget(this.f);
					this._3DWidget.show(null, position);

					// React on enabled changes
					this._3DWidget.on("enabled", _.bind(function (enabled) {
						this._emit("3DWidget.enabled", enabled);
					}, this));
					this._emit("3DWidget.enabled", this._3DWidget.isEnabled());

					this._emit("3Dstarted");

					return this._3DWidget;
				}, this));
			} else {
				this._3DWidget.show(null, position);
				this._emit("3Dstarted");
			}
		};

		/**
		 * Is the 3D widget currently enabled
		 * @returns {Boolean} True if enabled, false if not enabled
		 */
		View3DModule.prototype.is3DWidgetEnabled = function () {
			if (this._3DWidget) {
				return this._3DWidget.isEnabled();
			}
			return false;
		};

		/**
		 * Initialize and shows the 3D toolbar component.
		 */
		View3DModule.prototype._show3DToolbarButton = function () {
			if (!this._toolbarComponent) {
				this._publishToolbarComponent(View3DToolbarComponent.toolbarName, _.bind(function (layoutType) {
					this._toolbarComponent = new View3DToolbarComponent(this.f, this);

					return this._toolbarComponent;
				}, this));
			} else {
				this._toolbarComponent.show();
			}
		};

		/**
		 * Update the visibility of the 3D toolbar component
		 */
		View3DModule.prototype._updateToolbarVisibility = function () {
			if (this.has3DPerspective()) {
				this._show3DToolbarButton();
				this._emit("has3DPerspective", true);
			} else {
				if (this._toolbarComponent) {
					this._toolbarComponent.hide();
				}
				this._emit("has3DPerspective", false);
			}
		};

		/**
		 * Is there a 3D perspective available? should the 3D view toolbarbutton or mode be available?
		 * @returns {Boolean}
		 */
		View3DModule.prototype.has3DPerspective = function () {
			return this.f.getActivePage().get("has3DPerspective");
		};


		/**
		 * Get the current camera position of the 3D widget if it is enabled and visible or null otherwise
		 * @returns {Object} @example 
		 *                   {scale: 1.5,
		 *                   azimuth: 46.6,
		 *                   elevation: -52}
		 */
		View3DModule.prototype.getCameraPosition = function () {
			if (this._3DWidget && this._3DWidget.isVisible() && this._3DWidget.isEnabled()) {
				return this._3DWidget.getCameraPosition();
			} else {
				return null;
			}
		};


		/**
		 * Set the current camera position of the 3D widget if it is enabled and visible
		 * Otherwise it will start loading the 3d widget
		 * @param {Object} position - 
		 *                 @example 
		 *                    {scale: 1.5,
		 *                    azimuth: 46.6,
		 *                    elevation: -52}
		 */
		View3DModule.prototype.setCameraPosition = function (position) {
			if (this._3DWidget && this._3DWidget.isVisible() && this._3DWidget.isEnabled()) {
				if (position !== undefined && position.scale !== undefined && position.azimuth !== undefined && position.elevation !== undefined) {
					this._3DWidget.setCameraPosition(position);
				}
			} else {
				this.show3D(position);
			}
		};


		return View3DModule;
	});

