define([
		"web-viewer/layout/AbstractWidget",
		"web-viewer/layout/component/MiniButton",
		"web-viewer/layout/LayoutConstants",
		"utils/ShapeUtils",
		"component/3DViewerLauncher/3DViewerLauncher",
		"jquery",
		"underscore"
	],
	function (
		AbstractWidget,
		MiniButton,
		LayoutConstants,
		ShapeUtils,
		Viewer3DLauncher,
		$,
		_) {
		"use strict";

		/**
		 * @exports
		 * @memberof Compare
		 * @augments Layout.AbstractWidget
		 * @class The View3DWidget shows the 3D Viewer inside a widget of the HTML5 Viewer.
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 */
		var View3DWidget = function View3DWidget(f) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.VIEW3D);

			/**
			 * @member {Core.ViewerFacade} the viewer facade
			 */
			this.f = f;

			this.setIcon("icon icon-WCR_3D");

			this._3DContent = $("<div>").addClass("View3DContent");

			// intialize close button
			this._closeButton = new MiniButton(this.f, "close3D", "icon-WCR_Close12", true, _.bind(function () {
				this.setEnabled(false);
			}, this));
			this._closeButton.setTooltip("viewer", "PanelClose");

			// intialize the open in new window button
			this._openNewWindow = new MiniButton(this.f, "open3D", "icon-WCR_FullScreen", true, _.bind(function () {
				this.f.getDataAdapter().show3DView(this.f.getActivePage());
				this.setEnabled(false);
			}, this));
			this._openNewWindow.setTooltip("viewer", "open3DViewerInNewWindow");

			this.f.i18n.loadBundle("viewer", _.bind(function () {
				// set the widget title 
				this.setTitle(this.f.t("view3DTitle"));
			}, this));

		};

		View3DWidget.prototype = Object.create(AbstractWidget.prototype);
		View3DWidget.prototype.constructor = View3DWidget;
		View3DWidget.widgetName = LayoutConstants.Widget.VIEW3D;

		/**
		 * Shows the 3D widget and starts to fetch the 3D model
		 * @param {Function} [callback]
		 * @param {Object} [position] - the initial camera position. See setCameraPosition.
		 */
		View3DWidget.prototype.show = function (callback, position) {
			this.whenContextAvailable(function () {
				var activePage = this.f.getActivePage();

				this.makeVisible();

				var docVersionId = activePage.getDocumentVersion().getDocumentVersionID();
				var finalCallback = _.bind(function (success, error) {
					if (success) {
						activePage.set("has3DPerspective", true);
						if (callback) {
							callback();
						}
					}

					this.resized();

					if (!success) {
						this.f.loadBundle("viewer", function () {
							if (!error) {
								error = this.f.t("errorLoading3D");
							}
							this.f.emit("userMessage", "error", error, undefined, false, "3DERROR", false);
							this.setEnabled(false);
						}, this);
					}


				}, this);

				//make sure the 3D Viewer is not initialized with a height smaller than the minimum widget height
				var minimumSize = this.getMinimumSize();
				if (this.context.innerHeight() < minimumSize.height) {
					this.context.innerHeight(minimumSize.height);
				}

				// start loading the 3D view				
				if (!this._3DLauncher) {
					this._3DLauncher = new Viewer3DLauncher(this._3DContent, docVersionId, finalCallback, position);
				} else {
					this._3DLauncher.load3DForDocVersion(docVersionId, finalCallback, position);
				}
			}, this);
		};

		/**
		 * @override
		 */
		View3DWidget.prototype.setContext = function (context, headerExtra) {
			// call the old implementation of setContext() (the implementation of AbstractWidget.js)
			AbstractWidget.prototype.setContext.call(this, context, headerExtra);

			this.context.addClass("view3DWidget");
			this.context.append(this._3DContent);

			headerExtra.append(this._openNewWindow.element);
			headerExtra.append(this._closeButton.element);
		};


		/**
		 * Get the minimum size for the View3DWidget
		 * @returns {ShapeUtils.Size}
		 */
		View3DWidget.prototype.getMinimumSize = function () {
			return new ShapeUtils.Size(250, 300);
		};

		/**
		 * Get the prefered size for the View3DWidget
		 * @returns {ShapeUtils.Size}
		 */
		View3DWidget.prototype.getPreferredSize = function () {
			return new ShapeUtils.Size(250, 300);
		};

		/**
		 * Call when the widget is resizing and the contents needs to adapt
		 */
		View3DWidget.prototype.resizing = function () {
			this.resized();
		};

		/**
		 * This method notifies the widget that it has been resized
		 */
		View3DWidget.prototype.resized = function () {
			if (this._3DLauncher) {
				this._3DLauncher.resize();
			}
		};

		/**
		 * Get the current camera position if the 3D viewer is loaded
		 * @returns {Object} @example 
		 *                   {scale: 1.5,
		 *                   azimuth: 46.6,
		 *                   elevation: -52}
		 */
		View3DWidget.prototype.getCameraPosition = function () {
			if (this._3DLauncher) {
				return this._3DLauncher.getCameraPosition();
			}
			return null;
		};

		/**
		 * Set the current camera position of the 3D widget if the 3D viewer is loaded
		 * @param {Object} position - 
		 *                 @example 
		 *                    {scale: 1.5,
		 *                    azimuth: 46.6,
		 *                    elevation: -52}
		 */
		View3DWidget.prototype.setCameraPosition = function (position) {
			if (this._3DLauncher) {
				this._3DLauncher.setCameraPosition(position);
			}
		};

		return View3DWidget;
	});