define('web-viewer/viewControls/BackgroundColorSelector',["web-viewer/core/PrivateChannel",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/component/ColorPicker",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/utils/ViewerUtils",
		"jquery",
		"underscore"
	],
	function (PrivateChannel,
		ViewerToolbarButton,
		ColorPicker,
		ListenerUtil,
		ViewerUtils,
		$,
		_) {
		"use strict";

		/**
		 * @exports BackgroundColorSelector
		 * @memberof ViewControlsModule 
		 * @class The BackgroundColorSelector allows to change the image and window background color via color pickers from a tool bar when clicking the change background color button.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Function} Callback called when change background button is clicked
		 */
		var BackgroundColorSelector = function (f, viewControlsModel) {
			ListenerUtil.mixin(this, f);
			PrivateChannel.installTo(this, f, "backgroundColorSelector");

			this.f = f;
			this._viewControlsModel = viewControlsModel;

			/**
			 * @member {ViewerToolbarButton}
			 */
			this._changeBackgroundButton = new ViewerToolbarButton(this.f, "changeBackgroundButton", "changeBackgroundColor", true, _.bind(this._changeBackgroundButtonClicked, this));
			this._changeBackgroundButton.setTooltip("viewer", "ToolBackgroundColorTT");

			/**
			 * @member {JQueryObj}
			 */
			this._toolbar = $("<div />").addClass("toolbar");
			/**
			 * @member {JQueryObj}
			 */
			this._toolbarTitle = null;

			/**
			 * @member {ColorPicker}
			 */
			this._changeImageBackgroundColorPicker = null;
			/**
			 * @member {ViewerToolbarButton}
			 */
			this._changeImageBackgroundColorButton = null;

			/**
			 * @member {ColorPicker}
			 */
			this._changeWindowBackgroundColorPicker = null;
			/**
			 * @member {ViewerToolbarButton}
			 */
			this._changeWindowBackgroundColorButton = null;

			/**
			 * @member {ColorPicker}
			 */
			this._changeBothBackgroundColorPicker = null;
			/**
			 * @member {ViewerToolbarButton}
			 */
			this._changeBothBackgroundColorButton = null;

			/**
			 * @member {ViewerToolbarButton}
			 */
			this._resetBackgroundColorButton = null;

			this._registerListener("changed.backgroundColorActive", this._onBackgroundColorActiveChanged, this, this._viewControlsModel);
			this._onBackgroundColorActiveChanged();
		};

		/*
		 * INIT
		 */

		/**
		 * Init the toolbar components
		 */
		BackgroundColorSelector.prototype._initToolbarComponents = function () {
			// Init title
			if (this._toolbarTitle === null) {
				this._toolbarTitle = $("<p />").addClass("toolbarTitle");

				this.f.i18n.loadBundle("viewer", function () {
					this._toolbarTitle.html(this.f.i18n.t("BackgroundTitle"));
				}, this);

				this._toolbar.append(this._toolbarTitle);
			}

			// Init color pickers
			if (this._changeImageBackgroundColorPicker === null) {
				this._changeImageBackgroundColorButton = new ViewerToolbarButton(this.f, "changeImageBackgroundColorButton", "icon-WCR_Color-Img", true, _.bind(function () {
					this._changeImageBackgroundColorPicker.show();
				}, this));
				this._changeImageBackgroundColorButton.setTooltip("viewer", "BackgroundImage");

				this._changeImageBackgroundColorPicker = new ColorPicker(_.bind(this._changeImageBackgroundColor, this), this._changeImageBackgroundColorButton.element);

				this._toolbar.append(this._changeImageBackgroundColorButton.element);

				// initial update and further updates
				this._registerListener("changed.imageBackgroundColor", this._onImageBackgroundColorChanged, this, this._viewControlsModel);
				this._updateImageBackgroundColorPicker();
			}
			if (this._changeWindowBackgroundColorPicker === null) {
				this._changeWindowBackgroundColorButton = new ViewerToolbarButton(this.f, "changeWindowBackgroundColorButton", "icon-WCR_Color-Bg", true, _.bind(function () {
					this._changeWindowBackgroundColorPicker.show();
				}, this));
				this._changeWindowBackgroundColorButton.setTooltip("viewer", "BackgroundWindow");

				this._changeWindowBackgroundColorPicker = new ColorPicker(_.bind(this._changeWindowBackgroundColor, this), this._changeWindowBackgroundColorButton.element);

				this._toolbar.append(this._changeWindowBackgroundColorButton.element);

				// initial update and further updates
				this._registerListener("changed.windowBackgroundColor", this._onWindowBackgroundColorChanged, this, this._viewControlsModel);
				this._updateWindowBackgroundColorPicker();
			}
			if (this._changeBothBackgroundColorPicker === null) {
				// icon
				this._changeBothBackgroundColorButtonImageIcon = $("<i/>").addClass("icon subIcon").addClass("icon-WCR_Color-Img");
				this._changeBothBackgroundColorButtonWindowIcon = $("<i/>").addClass("icon containerIcon").addClass("icon-WCR_Color-Bg");
				this._changeBothBackgroundColorButtonWindowIcon.append(this._changeBothBackgroundColorButtonImageIcon);

				this._changeBothBackgroundColorButton = new ViewerToolbarButton(this.f, "changeBothBackgroundColorButton", this._changeBothBackgroundColorButtonWindowIcon, true, _.bind(function () {
					this._changeBothBackgroundColorPicker.show();
				}, this));
				this._changeBothBackgroundColorButton.setTooltip("viewer", "BackgroundBoth");

				this._changeBothBackgroundColorPicker = new ColorPicker(_.bind(this._changeBothBackgroundColor, this), this._changeBothBackgroundColorButton.element);

				this._toolbar.append(this._changeBothBackgroundColorButton.element);

				// initial update, further updates: see image/background changed methods
				this._updateBothBackgroundColorPicker();
			}
			if (this._resetBackgroundColorButton === null) {
				this._resetBackgroundColorButton = new ViewerToolbarButton(this.f, "resetBackgroundColor", "icon icon-WCR_WF-Undo", true, _.bind(this._resetBackgroundColor, this));

				this._resetBackgroundColorButton.setTooltip("viewer", "BackgroundReset");

				this._toolbar.append(this._resetBackgroundColorButton.element);

				// initial update
				this._updateResetBackgroundButton();
			}
		};


		/*
		 * RENDERING
		 */

		/**
		 * Re-render the main change background button
		 */
		BackgroundColorSelector.prototype._updateChangeBackgroundButton = function () {
			// set state of button
			this._changeBackgroundButton.setActive(this._viewControlsModel.isBackgroundColorActive());
		};

		/**
		 * Re-render the color picker of image background
		 */
		BackgroundColorSelector.prototype._updateImageBackgroundColorPicker = function () {
			if (this._changeImageBackgroundColorPicker !== null) {
				// color picker
				var hexColor = ViewerUtils.hexFromInt(this._viewControlsModel.getImageBackgroundColor());
				this._changeImageBackgroundColorPicker.setColor(hexColor);

				// button
				this._changeImageBackgroundColorButton.element.css({
					'color': "#" + hexColor
				});
			}
		};

		/**
		 * Re-render the color picker of window background
		 */
		BackgroundColorSelector.prototype._updateWindowBackgroundColorPicker = function () {
			if (this._changeWindowBackgroundColorPicker !== null) {
				// color picker
				var hexColor = ViewerUtils.hexFromInt(this._viewControlsModel.getWindowBackgroundColor());
				this._changeWindowBackgroundColorPicker.setColor(hexColor);

				// button
				this._changeWindowBackgroundColorButton.element.css({
					'color': "#" + hexColor
				});
			}
		};

		/**
		 * Re-render the color picker of both background colors
		 */
		BackgroundColorSelector.prototype._updateBothBackgroundColorPicker = function () {
			if (this._changeBothBackgroundColorPicker !== null) {
				// color picker
				var hexImageColor = ViewerUtils.hexFromInt(this._viewControlsModel.getImageBackgroundColor());
				var hexWindowColor = ViewerUtils.hexFromInt(this._viewControlsModel.getWindowBackgroundColor());

				// by default, take color of window
				this._changeBothBackgroundColorPicker.setColor(hexWindowColor);

				// button
				this._changeBothBackgroundColorButtonImageIcon.css({
					'color': "#" + hexImageColor
				});
				this._changeBothBackgroundColorButtonWindowIcon.css({
					'color': "#" + hexWindowColor
				});
			}
		};

		/**
		 * Re-render the reset background button
		 */
		BackgroundColorSelector.prototype._updateResetBackgroundButton = function () {
			if (this._resetBackgroundColorButton !== null) {
				var isDirty = !this._viewControlsModel.isImageBackgroundColorDefault() ||
					!this._viewControlsModel.isWindowBackgroundColorDefault();
				this._resetBackgroundColorButton.setEnabled(isDirty);
			}
		};

		/*
		 * CONTROLLING
		 */

		/**
		 * Called when the changeBackgroundColorButton is clicked.
		 */
		BackgroundColorSelector.prototype._changeBackgroundButtonClicked = function () {
			// just toggle the button
			this._toggleActive();
		};

		/**
		 * Toggle the active state of this selector
		 * @param {Boolean} [toggle] - If defined, take that active state, otherwise toggle the current state.
		 */
		BackgroundColorSelector.prototype._toggleActive = function (toggle) {
			if (toggle === undefined) {
				// if undefined, toggle the active state
				toggle = !this._viewControlsModel.isBackgroundColorActive();
			}

			this._viewControlsModel.setBackgroundColorActive(toggle);
		};

		/**
		 * Called when the image background color changed in the model
		 */
		BackgroundColorSelector.prototype._onImageBackgroundColorChanged = function () {
			this._updateImageBackgroundColorPicker();
			this._updateBothBackgroundColorPicker();
			this._updateResetBackgroundButton();
		};

		/**
		 * Called when the window background color changed in the model
		 */
		BackgroundColorSelector.prototype._onWindowBackgroundColorChanged = function () {
			this._updateWindowBackgroundColorPicker();
			this._updateBothBackgroundColorPicker();
			this._updateResetBackgroundButton();
		};

		/**
		 * Called on the active state change of the background color
		 */
		BackgroundColorSelector.prototype._onBackgroundColorActiveChanged = function () {
			this._updateChangeBackgroundButton();

			var active = this._viewControlsModel.isBackgroundColorActive();
			if (active) {
				// we need to show toolbar

				// init if needed
				this._initToolbarComponents();
				this._toolbar.show();
			} else {
				// we need to hide toolbar

				this._toolbar.hide();
			}
			this._emit("resized");
		};

		/**
		 * Called when color picker changed color for image background
		 * @param {String} hexColor
		 */
		BackgroundColorSelector.prototype._changeImageBackgroundColor = function (hexColor) {
			var intColor = ViewerUtils.intFromHex(hexColor);
			this._viewControlsModel.setImageBackgroundColor(intColor);
		};

		/**
		 * Called when color picker changed color for window background
		 * @param {String} hexColor
		 */
		BackgroundColorSelector.prototype._changeWindowBackgroundColor = function (hexColor) {
			var intColor = ViewerUtils.intFromHex(hexColor);
			this._viewControlsModel.setWindowBackgroundColor(intColor);
		};

		/**
		 * Called when color picker changed color for both image and window background
		 * @param {String} hexColor
		 */
		BackgroundColorSelector.prototype._changeBothBackgroundColor = function (hexColor) {
			// update both settings
			this._changeImageBackgroundColor(hexColor);
			this._changeWindowBackgroundColor(hexColor);
		};

		/**
		 * Reset the background color of image and window to its original values
		 */
		BackgroundColorSelector.prototype._resetBackgroundColor = function () {
			this._viewControlsModel.resetBackgroundColors();
			// switch off after reset
			this._viewControlsModel.setBackgroundColorActive(false);
		};

		/*
		 * Public methods
		 */

		/**
		 * Get the button element to change the background color
		 * @returns {JQueryObj}
		 */
		BackgroundColorSelector.prototype.getButtonElement = function () {
			return this._changeBackgroundButton.element;
		};

		/**
		 * Get the toolbar
		 * @returns {JQueryObj}
		 */
		BackgroundColorSelector.prototype.getToolbar = function () {
			return this._toolbar;
		};

		return BackgroundColorSelector;
	});

define('web-viewer/viewControls/OverprintHighlightColorSelector',["web-viewer/core/PrivateChannel",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/component/ColorPicker",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/utils/ViewerUtils",
		"Mousetrap",
		"jquery",
		"underscore"
	],
	function (PrivateChannel,
		ViewerToolbarButton,
		ColorPicker,
		ListenerUtil,
		ViewerUtils,
		Mousetrap,
		$,
		_) {
		"use strict";

		/**
		 * @exports OverprintHighlightColorSelector
		 * @memberof ViewControlsModule 
		 * @class The OverprintHighlightColorSelector allows to change the overprint highlight color from a tool bar when clicking the set overprint highlight color button.
		 * It also listens to CTRL+H to switch between non-highlight and highlight color view.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {ViewControlsModel} viewControlsModel - The view controls model to interact with.
		 */
		var OverprintHighlightColorSelector = function (f, viewControlsModel) {
			ListenerUtil.mixin(this, f);
			PrivateChannel.installTo(this, f, "overprintHighlightColorSelector");

			this.f = f;
			this._viewControlsModel = viewControlsModel;

			/**
			 * @member {ViewerToolbarButton}
			 */
			this._setOverprintHighlightColorButton = new ViewerToolbarButton(this.f, "setOverprintHighlightColor", "setOverprintHighlightColorOff", true, _.bind(this._setOverprintHighlightColorButtonClicked, this));

			/**
			 * @member {JQueryObj}
			 */
			this._toolbar = $("<div />").addClass("toolbar");
			/**
			 * @member {JQueryObj}
			 */
			this._toolbarTitle = null;

			/**
			 * @member {ColorPicker}
			 */
			this._changeOverprintHighlightColorPicker = null;
			/**
			 * @member {ViewerToolbarButton}
			 */
			this._changeOverprintHighlightColorButton = null;

			// Keyboard short cuts
			this._listeningToShortcuts = false;
			this._onCtrlHBound = _.bind(function (e) {
				this._onCtrlH();
				e.preventDefault();
				e.stopPropagation();
			}, this);
			this._listenToShortcuts(true);

			// Update initial state and listen to future changes
			this._registerListener("changed.overprintHighlightColorActive", this._onOverprintHighlightColorActiveChanged, this, this._viewControlsModel);
			this._onOverprintHighlightColorActiveChanged();
		};

		/*
		 * INIT
		 */

		/**
		 * Init the toolbar components
		 */
		OverprintHighlightColorSelector.prototype._initToolbarComponents = function () {
			// Init title
			if (this._toolbarTitle === null) {
				this._toolbarTitle = $("<p />").addClass("toolbarTitle");

				this.f.i18n.loadBundle("viewer", function () {
					this._toolbarTitle.html(this.f.i18n.t("OverprintTitle"));
				}, this);

				this._toolbar.append(this._toolbarTitle);
			}

			// Init color picker
			if (this._changeOverprintHighlightColorPicker === null) {
				this._changeOverprintHighlightColorPicker = new ColorPicker(_.bind(this._changeOverprintHighlightColor, this));

				this._changeOverprintHighlightColorButton = new ViewerToolbarButton(this.f, "changeOverprintHighlightColorButton", this._changeOverprintHighlightColorPicker.element, true, _.bind(function () {
					this._changeOverprintHighlightColorPicker.show();
				}, this));
				this._changeOverprintHighlightColorButton.setTooltip("viewer", "OverprintColorTT");

				this._registerListener("changed.overprintHighlightColor", this._updateChangeOverprintHighlightColorPicker, this, this._viewControlsModel);

				this._toolbar.append(this._changeOverprintHighlightColorButton.element);

				// initial update
				this._updateChangeOverprintHighlightColorPicker();
			}
		};


		/*
		 * RENDERING
		 */
		/**
		 * Re-render the set overprint highlight color button
		 */
		OverprintHighlightColorSelector.prototype._updateSetOverprintHighlightColorButton = function () {
			var active = this._viewControlsModel.isOverprintHighlightColorActive();
			if (active) {
				this._setOverprintHighlightColorButton.setIcon("setOverprintHighlightColorOn");
				this._setOverprintHighlightColorButton.setTooltip("viewer", "OverprintTT2");
			} else {
				this._setOverprintHighlightColorButton.setIcon("setOverprintHighlightColorOff");
				this._setOverprintHighlightColorButton.setTooltip("viewer", "OverprintTT");
			}
			this._setOverprintHighlightColorButton.setActive(active);
		};


		/**
		 * Re-render the color picker of overprint highlight color
		 */
		OverprintHighlightColorSelector.prototype._updateChangeOverprintHighlightColorPicker = function () {
			if (this._changeOverprintHighlightColorPicker !== null) {
				// color picker
				var overprintHighlightColor = this._viewControlsModel.getOverprintHighlightColor();
				var hexColor = ViewerUtils.hexFromInt(overprintHighlightColor);
				this._changeOverprintHighlightColorPicker.setColor(hexColor);
			}
		};

		/*
		 * CONTROLLING
		 */
		/**
		 * Called when the setOverprintHighlightColorButton is clicked.
		 */
		OverprintHighlightColorSelector.prototype._setOverprintHighlightColorButtonClicked = function () {
			// toggle active mode
			this._toggleActive();
		};


		/**
		 * Called when color picker changed color for overprint highlight color
		 * @param {String} hexColor
		 */
		OverprintHighlightColorSelector.prototype._changeOverprintHighlightColor = function (hexColor) {
			var intColor = ViewerUtils.intFromHex(hexColor);
			this._viewControlsModel.setOverprintHighlightColor(intColor);
		};

		/**
		 * Toggle the active state of this selector
		 * @param {Boolean} [toggle] - If defined, take that active state, otherwise toggle the current state.
		 */
		OverprintHighlightColorSelector.prototype._toggleActive = function (toggle) {
			if (toggle === undefined) {
				// if undefined, toggle the active state
				toggle = !this._viewControlsModel.isOverprintHighlightColorActive();
			}

			this._viewControlsModel.setOverprintHighlightColorActive(toggle);
		};

		/**
		 * Called when the overprint highlight color active state changes in the model.
		 */
		OverprintHighlightColorSelector.prototype._onOverprintHighlightColorActiveChanged = function () {
			this._updateSetOverprintHighlightColorButton();

			var active = this._viewControlsModel.isOverprintHighlightColorActive();
			if (active) {
				// init if needed
				this._initToolbarComponents();
				this._toolbar.show();

				// show the color picker if default color (no color set in past)
				if (this._viewControlsModel.isOverprintHighlightColorDefault()) {
					this._changeOverprintHighlightColorPicker.show();
				}
			} else {
				this._toolbar.hide();
			}
			this._emit("resized");
		};

		/**
		 * Called on key combo CTRL+H
		 */
		OverprintHighlightColorSelector.prototype._onCtrlH = function () {
			this._toggleActive();
		};

		/**
		 * Start/stop listening to short cuts
		 * @param {Boolean} listen - True to listen to shortcuts, false to stop listening
		 */
		OverprintHighlightColorSelector.prototype._listenToShortcuts = function (listen) {
			if (listen) {
				if (!this._listeningToShortcuts) {
					// if we were not listening yet
					Mousetrap.bind("ctrl+h", this._onCtrlHBound, "keydown");
					this._listeningToShortcuts = true;
				}
			} else {
				if (this._listeningToShortcuts) {
					// if we were listening
					Mousetrap.unbind("ctrl+h", this._onCtrlHBound, "keydown");
					this._listeningToShortcuts = false;
				}
			}
		};

		/*
		 * Public methods
		 */

		/**
		 * Get the button element to set the overprint highlight color
		 * @returns {JQueryObj}
		 */
		OverprintHighlightColorSelector.prototype.getButtonElement = function () {
			return this._setOverprintHighlightColorButton.element;
		};

		/**
		 * Get the toolbar
		 * @returns {JQueryObj}
		 */
		OverprintHighlightColorSelector.prototype.getToolbar = function () {
			return this._toolbar;
		};

		/**
		 * Show/hide the overprint highlight color selector
		 * @param {Boolean} toggle - True to show the button, false to hide and disable it
		 */
		OverprintHighlightColorSelector.prototype.toggle = function (toggle) {
			if (toggle) {
				this._listenToShortcuts(true);
				this._setOverprintHighlightColorButton.show();
			} else {
				this._listenToShortcuts(false);
				this._toggleActive(false);
				this._setOverprintHighlightColorButton.hide();
			}
		};

		return OverprintHighlightColorSelector;
	});

define('web-viewer/layout/component/DoubleColorBlock',["jquery"],
	function (
		$
	) {

		/**
		 * @class This component is an element that consists of two color blocks that are glued together.
		 * @exports DoubleColorBlock
		 * @constructor
		 * @param {String} [initialLeftColor] - Initial color of the left block (in hex with # format)
		 * @param {String} [initialRightColor] - Initial color of the right block (in hex with # format)
		 */
		var DoubleColorBlock = function (initialLeftColor, initialRightColor) {
			this.element = $("<div />").addClass("doubleColorBlock");

			this._leftBlock = $("<div />").addClass("part");
			this._rightBlock = $("<div />").addClass("part");

			if (initialLeftColor !== undefined) {
				this.setLeftColor(initialLeftColor);
			}

			if (initialRightColor !== undefined) {
				this.setRightColor(initialRightColor);
			}

			this.element.append(this._leftBlock);
			this.element.append(this._rightBlock);
		};

		/**
		 * Set the color of the left color block.
		 * @param {String} leftColor - Color for the left block (in hex with # format)
		 */
		DoubleColorBlock.prototype.setLeftColor = function (leftColor) {
			this._leftBlock.css("background-color", leftColor);
		};

		/**
		 * Set the color of the right color block.
		 * @param {String} rightColor - Color for the right block (in hex with # format)
		 */
		DoubleColorBlock.prototype.setRightColor = function (rightColor) {
			this._rightBlock.css("background-color", rightColor);
		};

		return DoubleColorBlock;
	});

define('web-viewer/viewControls/TrimboxDropdownToolbarButton',["web-viewer/layout/component/DropdownToolbarButton",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/layout/component/ColorPicker",
		"web-viewer/layout/component/DoubleColorBlock",
		"web-viewer/viewControls/ViewControlsConstants",
		"underscore"
	],
	function (DropdownToolbarButton,
		ListenerUtil,
		ColorPicker,
		DoubleColorBlock,
		ViewControlsConstants,
		_) {
		"use strict";

		/**
		 * @exports TrimboxDropdownToolbarButton
		 * @memberof ViewControlsModule 
		 * @class Extends the dropdownToolbarButton, makes is possible to show/hide the trimbox of the document page and the dropdown lets the user change
		 * the color of the trimbox for the active document page and the reference document page 
		 * @augments {DropdownToolbarButton}
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {String} name - the name of the button
		 * @param {String | JQueryObj } icon - The class which is used for displaying the icon (css class for 16px * 16px icon), or your own jQuery element 
		 * @param {Boolean} isShowingTrimbox - does the button needs to be activated by default
		 * @param {ViewControlsConstants.trimboxMode} trimboxMode - the trimbox mode to initialize the dropdown with
		 */
		var TrimboxDropdownToolbarButton = function (f, name, icon, isShowingTrimbox, trimboxMode) {
			DropdownToolbarButton.call(this, f, name, icon, false, this._toggleTrimbox);
			ListenerUtil.mixin(this, f);

			/**
			 * @member {Core.ViewerFacade}
			 */
			this.f = f;

			//set the button to active if it needs to be active by default
			this.setActive(isShowingTrimbox);

			/**
			 * @member {ViewControlsConstants.trimboxMode}
			 * @private
			 */
			this._trimboxMode = trimboxMode;

			/**
			 * @member {ColorPicker} - the color picker for the color of the trimbox of the active document page
			 * @private
			 */
			this._trimboxColorPicker = new ColorPicker(_.bind(this._changeTrimboxColor, this));
			this._trimboxColorPicker.setColor(this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_MARGINS")); //get the color from the user settings (is persisted)

			/**
			 * @member {ColorPicker} -  the color picker to change the color of the trimbox of the reference document page
			 * @private
			 */
			this._secondaryTrimboxColorPicker = new ColorPicker(_.bind(this._changeSecondaryTrimboxColor, this));
			this._secondaryTrimboxColorPicker.setColor(this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_MARGINS_DIFF")); //get the color from the user settings

			/**
			 * @member {DoubleColorBlock} makes it possible to change both the color of the active document pages' trimbox as the trimbox of the reference pages' trimbox
			 * @private
			 */
			this._changeBothTrimboxColorBlock = new DoubleColorBlock(this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_MARGINS"), this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_MARGINS_DIFF"));

			/**
			 * @member {ColorPicker} the color picker to change both colors
			 */
			this._changeBothTrimboxColor = new ColorPicker(_.bind(this._changeBothTrimboxColor, this), this._changeBothTrimboxColorBlock.element);
			this._changeBothTrimboxColor.setColor(this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_MARGINS")); //set the color from the active document page as the default selected color on opening the color picker
			//make sure the language bundle is loaded before initializing the dropdown menu
			this.f.loadBundle("viewer", function () {
				this._initializeDropdown();
			}, this);
		};

		//inherit
		TrimboxDropdownToolbarButton.prototype = Object.create(DropdownToolbarButton.prototype);
		TrimboxDropdownToolbarButton.prototype.constructor = TrimboxDropdownToolbarButton;

		/**
		 * Toggle the show trimbox button, sets the trimbox mode to the graphics module
		 * @private
		 * @param {Boolean} toggle - show or hide the trimbox
		 */
		TrimboxDropdownToolbarButton.prototype._toggleTrimbox = function (toggle) {
			var graphicsModule = this.f.getModule("graphics"); //get the graphics module, the setting is set to this module
			this.setActive(toggle); //set the button active/not-active
			if (graphicsModule !== undefined) { //only if the graphicsmodule is available
				graphicsModule.setShowTrimbox(toggle, this._trimboxMode);
			} else { //wait for the module to be loaded if this was not yet the case
				this.f.whenModuleAvailable("graphics", _.bind(function (graphicsModule) {
					graphicsModule.setShowTrimbox(this.isActive(), this._trimboxMode);
				}, this));
			}
			//also set the settings of the users to persist the trimbox settings
			this.f.setOption("PREFERENCES_DISPLAY_MARGINS", toggle ? 1 : 0);
			this.f.setOption("PREFERENCES_MARGIN_DISPLAY_MODE", this._trimboxMode);
		};

		/**
		 * initializes the dropdown menu
		 * @private
		 */
		TrimboxDropdownToolbarButton.prototype._initializeDropdown = function () {
			this._initializeShowTrimboxOptions(); //first add the trimbox options
			this.addDivider(); //add a divider
			this._initializeColorOptions(); //add the color options for the trimbox
		};

		/**
		 * initializes the trimbox options in the dropdown menu
		 * @private
		 */
		TrimboxDropdownToolbarButton.prototype._initializeShowTrimboxOptions = function () {
			this.addOption({ //add as options the user cannot uncheck, either one of the two options needs to be active
				name: 'onCurrentPage', //the trimbox is only shown on the current page (only valid in reader spread mode)
				content: this.f.i18n.t("ToolMarginsShowCurrent"),
				onchange: _.bind(function () {
					this._trimboxMode = ViewControlsConstants.trimboxMode.ONCURRENTPAGE; //set to the correct trimbox mode
					this._toggleTrimbox(this.isActive()); //make sure to toggle the show trimbox to trigger the setting to be set in the graphics module
					this.toggleOption('bothPagesInReaderSpread', false); //toggle the other option
				}, this),
				selected: this._trimboxMode === ViewControlsConstants.trimboxMode.ONCURRENTPAGE, //should it be selected on initialization
				userCannotUncheck: true
			});

			this.addOption({
				name: 'bothPagesInReaderSpread',
				content: this.f.i18n.t("ToolMarginsShowBoth"),
				onchange: _.bind(function () {
					this._trimboxMode = ViewControlsConstants.trimboxMode.ONBOTHINREADERSPREAD;
					this._toggleTrimbox(this.isActive());
					this.toggleOption('onCurrentPage', false);
				}, this),
				selected: this._trimboxMode === ViewControlsConstants.trimboxMode.ONBOTHINREADERSPREAD,
				userCannotUncheck: true
			});
		};

		/**
		 * initializes the color options in the dropdown menu
		 * @private
		 */
		TrimboxDropdownToolbarButton.prototype._initializeColorOptions = function () {
			this.addAction({ //add as actions, and add the colorblock as icon in the dropdown menu
				name: 'trimboxColor',
				content: this.f.i18n.t("ToolMarginsColorPrompt"),
				action: _.bind(function () {
					this._trimboxColorPicker.show(); //show the colorpicker on click
				}, this),
				enabled: true,
				icon: this._trimboxColorPicker.element
			});

			this.addAction({
				name: 'secondaryTrimboxColor',
				content: this.f.i18n.t("ToolMarginsColorPromptDiff"),
				action: _.bind(function () {
					this._secondaryTrimboxColorPicker.show();
				}, this),
				enabled: true,
				icon: this._secondaryTrimboxColorPicker.element
			});

			this.addAction({
				name: 'changeBothTrimboxColor',
				content: this.f.i18n.t("ToolMarginsColorPromptBoth"),
				action: _.bind(function () {
					this._changeBothTrimboxColor.show();
				}, this),
				enabled: true,
				icon: this._changeBothTrimboxColor.element
			});

			this.addAction({
				name: 'revertTrimboxColors',
				content: this.f.i18n.t("ToolMarginsColorPromptReset"),
				action: _.bind(function () {
					this.f.getCurrentUser().resetOption("PREFERENCES_HIGHLIGHTCOLORS_MARGINS");
					this.f.getCurrentUser().resetOption("PREFERENCES_HIGHLIGHTCOLORS_MARGINS_DIFF");
					var activeColor = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_MARGINS");
					var referenceColor = this.f.getCurrentUser().getColor("PREFERENCES_HIGHLIGHTCOLORS_MARGINS_DIFF");
					this._changeBothTrimboxColorBlock.setLeftColor(activeColor);
					this._changeBothTrimboxColorBlock.setRightColor(referenceColor);
					this._trimboxColorPicker.setColor(activeColor); //set the colors in the color pickers of the active and reference document pages
					this._secondaryTrimboxColorPicker.setColor(referenceColor);
				}, this),
				enabled: true,
				icon: "resetColor"
			});
		};


		/**
		 * Change only the trimbox color of the active document page
		 * @param {String} color - as RGB
		 */
		TrimboxDropdownToolbarButton.prototype._changeTrimboxColor = function (color) {
			this.f.getCurrentUser().setColor("PREFERENCES_HIGHLIGHTCOLORS_MARGINS", color);
			this._changeBothTrimboxColorBlock.setLeftColor("#" + color); //set the color to the double color block
		};

		/**
		 * Change only the trimbox color of the reference document page
		 * @param {String} color - as RGB
		 */
		TrimboxDropdownToolbarButton.prototype._changeSecondaryTrimboxColor = function (color) {
			this.f.getCurrentUser().setColor("PREFERENCES_HIGHLIGHTCOLORS_MARGINS_DIFF", color);
			this._changeBothTrimboxColorBlock.setRightColor("#" + color);
		};

		/**
		 * Change only the trimbox color of both the active and the reference document page
		 * @param {String} color - as RGB
		 */
		TrimboxDropdownToolbarButton.prototype._changeBothTrimboxColor = function (color) {
			this._changeTrimboxColor(color);
			this._changeSecondaryTrimboxColor(color);
			this._trimboxColorPicker.setColor(color); //set the colors in the color pickers of the active and reference document pages
			this._secondaryTrimboxColorPicker.setColor(color);
		};

		return TrimboxDropdownToolbarButton;
	});

define('web-viewer/viewControls/TrimboxClippingDropdownToolbarButton',["web-viewer/layout/component/DropdownToolbarButton",
		"web-viewer/viewControls/ViewControlsConstants",
		"web-viewer/utils/ListenerUtil",
		"underscore"
	],
	function (
		DropdownToolbarButton,
		ViewControlsConstants,
		ListenerUtil,
		_) {

		var TrimboxClippingDropdownToolbarButton = function (f, name) {
			DropdownToolbarButton.call(this, f, name, "trimboxClipping", false, _.bind(this._toggleClipping, this));
			ListenerUtil.mixin(this, f);




			this.f.loadBundle("viewer", function () {

				// clip when in readerspread
				this.addOption({
					name: 'CLIP_IN_READERSPREAD',
					content: this.f.i18n.t("ToolMarginsClipSpread"),
					onchange: _.bind(function () { //same as the 'only distortion'-option but for compensate scaling
						this.f.setOption("PREFERENCES_MARGIN_CLIP_MODE", ViewControlsConstants.marginClipMode.CLIP_IN_READERSPREAD);
					}, this),
					userCannotUncheck: true
				});

				// clip always
				this.addOption({
					name: 'CLIP',
					content: this.f.i18n.t("ToolMarginsClipOn"),
					onchange: _.bind(function () { //same as the 'only distortion'-option but for compensate scaling
						this.f.setOption("PREFERENCES_MARGIN_CLIP_MODE", ViewControlsConstants.marginClipMode.CLIP);
					}, this),
					userCannotUncheck: true
				});

				this._registerListener("options.changed.PREFERENCES_MARGIN_CLIP_MODE", this.update, this, this.f.getCurrentUser());
				this.update();
			}, this);
		};

		// inherit
		TrimboxClippingDropdownToolbarButton.prototype = Object.create(DropdownToolbarButton.prototype);
		TrimboxClippingDropdownToolbarButton.prototype.constructor = TrimboxClippingDropdownToolbarButton;


		/**
		 * Toggle the button
		 */
		TrimboxClippingDropdownToolbarButton.prototype._toggleClipping = function () {
			if (!this._isClipping()) {
				// enable clipping
				if (this.f.getActivePage().isReaderSpread()) {
					this.f.setOption("PREFERENCES_MARGIN_CLIP_MODE", ViewControlsConstants.marginClipMode.CLIP_IN_READERSPREAD);
				} else {
					this.f.setOption("PREFERENCES_MARGIN_CLIP_MODE", ViewControlsConstants.marginClipMode.CLIP);
				}
			} else {
				this.f.setOption("PREFERENCES_MARGIN_CLIP_MODE", ViewControlsConstants.marginClipMode.CLIP_OFF);
			}
		};


		/**
		 * Update the display of the button
		 */
		TrimboxClippingDropdownToolbarButton.prototype.update = function () {
			// only update if language bundle is avaialble
			this.f.loadBundle("viewer", function () {
				var clippingMode = this.f.getOption("PREFERENCES_MARGIN_CLIP_MODE");
				if (this.f.getActivePage().hasGraphics()) {
					this.show();

					this.setActive(this._isClipping());
					this.toggleOption("CLIP_IN_READERSPREAD", clippingMode === ViewControlsConstants.marginClipMode.CLIP_IN_READERSPREAD);
					this.toggleOption("CLIP", clippingMode === ViewControlsConstants.marginClipMode.CLIP);
				} else {
					this.hide();
				}
			}, this);
		};

		/**
		 * Check if margin clipping is currently enabled
		 * @returns {Boolean}
		 */
		TrimboxClippingDropdownToolbarButton.prototype._isClipping = function () {
			var clippingMode = this.f.getOption("PREFERENCES_MARGIN_CLIP_MODE");
			return (clippingMode === ViewControlsConstants.marginClipMode.CLIP ||
				(clippingMode === ViewControlsConstants.marginClipMode.CLIP_IN_READERSPREAD && this.f.getActivePage().isReaderSpread()));
		};
		return TrimboxClippingDropdownToolbarButton;
	});

define('web-viewer/viewControls/ViewControlsWidget',["web-viewer/layout/AbstractWidget",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/layout/component/DropdownToolbarButton",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/viewControls/BackgroundColorSelector",
		"web-viewer/viewControls/OverprintHighlightColorSelector",
		"web-viewer/viewControls/TrimboxDropdownToolbarButton",
		"web-viewer/viewControls/TrimboxClippingDropdownToolbarButton",
		"web-viewer/utils/ListenerUtil",
		"underscore",
		"jquery"
	],
	function (AbstractWidget,
		LayoutConstants,
		DropdownToolbarButton,
		ViewerToolbarButton,
		BackgroundColorSelector,
		OverprintHighlightColorSelector,
		TrimboxDropdownToolbarButton,
		TrimboxClippingDropdownToolbarButton,
		ListenerUtil,
		_,
		$
	) {
		"use strict";

		/**
		 * @exports ViewControlsWidget
		 * @memberof ViewControlsModule
		 * @augments Layout.AbstractWidget
		 * @class The View widget shows several advanced options and tools.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var ViewControlsWidget = function ViewControlsWidget(f, viewControlsModel, viewControlsModule) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.VIEWCONTROLS);
			ListenerUtil.mixin(this, f);

			/**
			 * @member {ViewControlsModle} model for view controls
			 */
			this._viewControlsModel = viewControlsModel;

			/**
			 * @member {ViewControlsModule} module, acts as a controller for the view controls
			 */
			this._viewControlsModule = viewControlsModule;

			/**
			 * @member {Button} this button enables/disables the inverse view
			 */
			this._negateButton = new ViewerToolbarButton(this.f, "negate", "icon-WCR_Invert", false, _.bind(function () {
				this._viewControlsModel.setIsNegated(!this._viewControlsModel.isNegated());
			}, this));

			/**
			 * @member {ViewerToolbarButton} this button enables/disables the view from back view
			 */
			this._viewFromBackButton = new ViewerToolbarButton(this.f, "viewFromBackButton", "viewFromBackOff", false, _.bind(function () {
				this._viewControlsModel.setViewFromBack(!this._viewControlsModel.isViewFromBack());
			}, this));

			/**
			 * @member {DropdownToolbarButton} dropdown button to compensate distortion and scaling in digital film files,
			 * will only show the dropdown if both distortion and scaling are in the file
			 */
			this._distortionButton = new DropdownToolbarButton(f, "distortion", "distortionOffIcon", false, _.bind(function () {

			}, this));

			this.setIcon("icon icon-WCR_View");

			/**
			 * @member {DropdownToolbarButton} dropdown button to enable disabling trimbox clipping
			 */
			this._marginClippingButton = new TrimboxClippingDropdownToolbarButton(this.f, "marginClipping");
			this._marginClippingButton.setTooltip("viewer", "TrimboxClippingTT");
			this.toggleCompensateDistortionButton(false);

			this._trimboxDropdown = new TrimboxDropdownToolbarButton(this.f, "showTrimboxDropdown", "showTrimboxDropdown", this.f.getOption("PREFERENCES_DISPLAY_MARGINS") === 1, this.f.getOption("PREFERENCES_MARGIN_DISPLAY_MODE"));
			this._trimboxDropdown.setTooltip("viewer", "TrimboxEnableColorTT");

			this.f.loadBundle("viewer", function () {
				this.setTitle(this.f.t("ViewControlsTitle"));

				// add options to the 'compensate distortion'-button
				this._distortionButton.addOption({ //first option to only compensate distortion
					name: 'onlyDistortion',
					content: f.i18n.t("FileDistChoiceDistOnly"),
					onchange: _.bind(function () { //when the option is clicked, enable/disable compensate distortion
						var isOtherActive = this._distortionButton.getOptionValue('onlyScale') || this._distortionButton.getOptionValue('both');
						this._distortionButton.toggleOption('onlyScale', false); //toggle the other two option to false only the distortion is altered
						this._distortionButton.toggleOption('both', false);
						if (this._viewControlsModel.isScaleCompensated()) { //if the scale was compensated, undo this only distortion gets altered
							this._viewControlsModel.setIsScaleCompensated(false);
						}
						if (isOtherActive && this._viewControlsModel.isDistCompensated()) { //only if another option was active disable distortion to get the correct data for only distortion
							this._viewControlsModel.setIsDistCompensated(false);
						}
						this._viewControlsModule.compensateDistortion(); //compensate distortion, will turn distortion off if it was on before (option was selected)
					}, this),
					userCannotUncheck: false //it is possible to disable the compensate distortion again
				});

				this._distortionButton.addOption({ //only compensate scale option
					name: 'onlyScale',
					content: f.i18n.t("FileDistChoiceScaleOnly"),
					onchange: _.bind(function () { //same as the 'only distortion'-option but for compensate scaling
						var isOtherActive = this._distortionButton.getOptionValue('onlyDistortion') || this._distortionButton.getOptionValue('both');
						this._distortionButton.toggleOption('onlyDistortion', false); //disable other options
						this._distortionButton.toggleOption('both', false);
						if (this._viewControlsModel.isDistCompensated()) { //if distortion was compensated, undo this compensation
							this._viewControlsModel.setIsDistCompensated(false);
						}
						if (isOtherActive && this._viewControlsModel.isScaleCompensated()) { //only when the option wasn't active disable to get data for only scaling
							this._viewControlsModel.setIsScaleCompensated(false);
						}
						this._viewControlsModule.compensateScale(); //enable/disable compensate scaling
					}, this),
					userCannotUncheck: false
				});

				this._distortionButton.addOption({ //compensate both distortion and scaling
					name: 'both',
					content: f.i18n.t("FileDistChoiceBoth"),
					onchange: _.bind(function () { //when clicked set other option to false and trigger compensation for both distortion and scaling
						this._distortionButton.toggleOption('onlyDistortion', false);
						this._distortionButton.toggleOption('onlyScale', false);
						this._viewControlsModule.compensateScaleAndDistortion(true);
					}, this),
					userCannotUncheck: false
				});

				this._distortionButton.disableDropdown(); //disable the dropdown by default


				this._updateNegateButton();
				this._updateViewFromBackButton();
			}, this);


			// Change background color for image and window
			this._backgroundColorSelector = new BackgroundColorSelector(this.f, this._viewControlsModel);
			this._registerListener("resized", function () {
				this._fireResized();
			}, this, this._backgroundColorSelector);

			// Set overprint highlight color
			this._overprintHighlightColorSelector = new OverprintHighlightColorSelector(this.f, this._viewControlsModel);
			this._registerListener("resized", function () {
				this._fireResized();
			}, this, this._overprintHighlightColorSelector);


			//make the distortion button renderer debounced, might get trigger by both changing scaling and distortion (in this case it gets called twice)
			var debouncedDistortionButtonRenderer = _.debounce(this._updateDistortionButton, 50);
			this._registerListener("scaleOrDistortionChanged", debouncedDistortionButtonRenderer, this, this._viewControlsModel);
			this._registerListener("changed.negated", this._updateNegateButton, this, this._viewControlsModel);
			this._registerListener("changed.viewFromBack", this._updateViewFromBackButton, this, this._viewControlsModel);

			var updateClippingButton = _.bind(function () {
				this._marginClippingButton.update();
			}, this);

			this._registerListener("module.compare.changed.diffMode", function () {
				_.defer(updateClippingButton);
			}, this._marginClippingButton); // defer the update, to make sure the modules already did the needed changes after activepage changed (i.e. setting isReaderspread)


			this.f.getDocumentController().whenActivePageChanged(_.bind(this._updateButtons, this));

		};

		//inherit
		ViewControlsWidget.prototype = Object.create(AbstractWidget.prototype);
		ViewControlsWidget.prototype.constructor = ViewControlsWidget;
		ViewControlsWidget.widgetName = LayoutConstants.Widget.VIEWCONTROLS;

		/**
		 * Set the context of the widget
		 * @override
		 * @param {JQueryObj} context
		 * @param {JQueryObj} header
		 */
		ViewControlsWidget.prototype.setContext = function (context, header) {
			AbstractWidget.prototype.setContext.call(this, context, header);

			var buttons = $("<div/>").addClass("buttons"); //make a wrapper for all buttons (or group button in the future)
			buttons.append(this._distortionButton.element);
			buttons.append(this._backgroundColorSelector.getButtonElement());
			buttons.append(this._viewFromBackButton.element);
			buttons.append(this._overprintHighlightColorSelector.getButtonElement());
			buttons.append(this._negateButton.element);
			buttons.append(this._trimboxDropdown.element);
			buttons.append(this._marginClippingButton.element);

			this.context.append(buttons);
			this.context.append(this._backgroundColorSelector.getToolbar());
			this.context.append(this._overprintHighlightColorSelector.getToolbar());

			this.context.addClass("viewWidget");
		};

		/**
		 * Render the distortion button (and show/hide its dropdown), change the default action for the toolbarbutton
		 */
		ViewControlsWidget.prototype._updateDistortionButton = function () {
			//if there is no distortion and no scaling for both the active and reference document pages be sure to hide the distortion button
			if ((!this._viewControlsModel.hasPageDistortion() && !this._viewControlsModel.hasPageScale()) && (!this._viewControlsModel.hasDiffPageDistortion() && !this._viewControlsModel.hasDiffPageScale())) {
				this.toggleCompensateDistortionButton(false);
			} else { //render the distortion button correctly
				this.toggleCompensateDistortionButton(true);
				this.f.loadBundle("viewer", function () { //load language strings for the tooltip of the distortion button 
					this._distortionButton.toggleOption('both', false);
					this._distortionButton.toggleOption('onlyDistortion', false);
					this._distortionButton.toggleOption('onlyScale', false);
					if (this._viewControlsModel.isScaleCompensated() || this._viewControlsModel.isDistCompensated()) { //if there's a compensation active 
						if (this._viewControlsModel.isScaleCompensated() && this._viewControlsModel.isDistCompensated()) {
							this._distortionButton.toggleOption('both', true);
						} else if (this._viewControlsModel.isScaleCompensated()) {
							this._distortionButton.toggleOption('onlyScale', true);
						} else {
							this._distortionButton.toggleOption('onlyDistortion', true);
						}
						this._distortionButton.setActive(true); //render the button as being active
						this._distortionButton.setIcon("distortionOnIcon"); //and change the icon on the toolbarbutton
					} else { //compensation for distortion or scaling is not active at this moment
						this._distortionButton.setActive(false);
						this._distortionButton.setIcon("distortionOffIcon");
					}
					//if either the active or reference page has both distortion and scaling, active the dropdown of the toolbarbutton
					if ((this._viewControlsModel.hasPageDistortion() && this._viewControlsModel.hasPageScale()) || (this._viewControlsModel.hasDiffPageDistortion() && this._viewControlsModel.hasDiffPageScale())) {
						this._distortionButton.enableDropdown(); //enables the dropdown
					} else {
						this._distortionButton.disableDropdown();
					}
					//start rendering the tooltip for the 'compensate distortion'-button
					var tooltip = "";
					//if the active or reference page has distortion, show the distortion in the tooltip
					if (this._viewControlsModel.hasPageDistortion() || this._viewControlsModel.hasDiffPageDistortion()) {
						//set the default (left click) action for the 'compensate distortion'-button to only compensate distortion
						this._distortionButton.setAction(_.bind(this._viewControlsModule.compensateDistortion, this._viewControlsModule));
						var referenceHDist = "";
						var referenceVDist = "";
						if (this._viewControlsModel.hasDiffPageDistortion()) { //if the reference page has distortion show this in the tooltip
							referenceHDist = " / " + (this._viewControlsModel.getHorizontalDistortion(false) * 100).toFixed(1);
							referenceVDist = " / " + (this._viewControlsModel.getVerticalDistortion(false) * 100).toFixed(1);
						}
						if (this._viewControlsModel.isDistCompensated()) { //if compensation for distortion is active or not, render a different tooltip
							tooltip += this.f.t("FileCompensateDistOnTT");
						} else {
							tooltip += this.f.t("FileCompensateDistOffTT");
						}
						tooltip += "<br/>&nbsp;&nbsp;" + this.f.t("FileVertDistTT", (this._viewControlsModel.getVerticalDistortion(true) * 100).toFixed(1) + referenceVDist) + " %";
						tooltip += "<br/>&nbsp;&nbsp;" + this.f.t("FileHorDistTT", (this._viewControlsModel.getHorizontalDistortion(true) * 100).toFixed(1) + referenceHDist) + " %";
					}
					//if the active or reference page has scaling, show the scaling in the tooltip
					if (this._viewControlsModel.hasPageScale() || this._viewControlsModel.hasDiffPageScale()) {
						if (tooltip !== "") { //if there is already a tooltip constructed for the distortion, add a <br>
							tooltip += "<br/>";
						}
						//set the default (left click) action for the button to compensate only the scaling of the active and reference document page
						this._distortionButton.setAction(_.bind(this._viewControlsModule.compensateScale, this._viewControlsModule));
						var referenceHScale = "";
						var referenceVScale = "";
						if (this._viewControlsModel.hasDiffPageScale()) { //if there is reference page add the scaling to the tooltip
							referenceHScale = " / " + (this._viewControlsModel.getHorizontalScale(false) * 100).toFixed(1);
							referenceVScale = " / " + (this._viewControlsModel.getVerticalScale(false) * 100).toFixed(1);
						}
						if (this._viewControlsModel.isScaleCompensated()) {
							tooltip += this.f.t("FileCompensateScaleOnTT");
						} else {
							tooltip += this.f.t("FileCompensateScaleOffTT");
						}
						tooltip += "<br/>&nbsp;&nbsp;" + this.f.t("FileVertScaleTT", (this._viewControlsModel.getVerticalScale(true) * 100).toFixed(1) + referenceVScale) + " %";
						tooltip += "<br/>&nbsp;&nbsp;" + this.f.t("FileHorScaleTT", (this._viewControlsModel.getHorizontalScale(true) * 100).toFixed(1) + referenceHScale) + " %";
					}
					//if the reference or active page has both distortion and scaling, change the default action for the button to compensate both the distortion and the scaling
					if ((this._viewControlsModel.hasPageDistortion() && this._viewControlsModel.hasPageScale()) || (this._viewControlsModel.hasDiffPageDistortion() && this._viewControlsModel.hasDiffPageScale())) {
						this._distortionButton.setAction(_.bind(function () { //change the action
							if (this._distortionButton.isActive()) {
								this._viewControlsModule.compensateScaleAndDistortion(); //compensate both distortion and scaling
							} else {
								this._distortionButton.showDropdown();
							}

						}, this));
					}

					this._distortionButton.setTooltip(tooltip); //set the tooltip of the toolbarbutton
				}, this);
			}
		};

		/**
		 * Render the view from back button
		 */
		ViewControlsWidget.prototype._updateViewFromBackButton = function () {
			if (this.f.getActivePage().hasGraphics()) {
				this._viewFromBackButton.show();
				if (this._viewControlsModel.isViewFromBack()) {
					this._viewFromBackButton.setIcon("viewFromBackOn");
					this._viewFromBackButton.setTooltip("viewer", "ToolBackTT2");
					this._viewFromBackButton.setActive(true);
				} else {
					this._viewFromBackButton.setIcon("viewFromBackOff");
					this._viewFromBackButton.setTooltip("viewer", "ToolBackTT");
					this._viewFromBackButton.setActive(false);
				}
			} else {
				this._viewFromBackButton.hide();
			}
		};

		/**
		 * Toggle the 'compensate distortion'-button, shows or hides the button
		 * @param {Boolean} toggle
		 */
		ViewControlsWidget.prototype.toggleCompensateDistortionButton = function (toggle) {
			if (toggle) {
				this._distortionButton.element.show();
			} else {
				this._distortionButton.element.hide();
			}
		};

		/**
		 * Toggle the overprint highlight color button, shows or hides the button
		 * @param {Boolean} toggle
		 */
		ViewControlsWidget.prototype.toggleOverprintHighlightColorButton = function (toggle) {
			this._overprintHighlightColorSelector.toggle(toggle);
		};

		/**
		 * Update the display of the inverse view button
		 */
		ViewControlsWidget.prototype._updateNegateButton = function () {
			if (this.f.getActivePage().hasGraphics()) {
				this._negateButton.show();
				var isNegated = this._viewControlsModel.isNegated();
				this._negateButton.setActive(isNegated);
				this._negateButton.setIcon(isNegated ? "icon-WCR_InvertReverse" : "icon-WCR_Invert");
				this._negateButton.setTooltip("viewer", isNegated ? "ToolInvertTT2" : "ToolInvertTT");
			} else {
				this._negateButton.hide();
			}
		};

		/**
		 * Update the 'show trimbox dropdown'-button, shows or hides the button
		 */
		ViewControlsWidget.prototype._updateShowTrimboxDropdownToolbarButton = function () {
			if (this.f.getActivePage().hasGraphics() || this.f.getActivePage().isBlankPage()) {
				this._trimboxDropdown.show();
			} else {
				this._trimboxDropdown.hide();
			}
		};

		/**
		 * Update the overprint highlight color button
		 */
		ViewControlsWidget.prototype._updateOverprintHighlightColorButton = function () {
			this._overprintHighlightColorSelector.toggle(this.f.getActivePage().hasGraphics());
		};

		/**
		 * Update the buttons after a active page change
		 */
		ViewControlsWidget.prototype._updateButtons = function () {
			this._marginClippingButton.update();
			this._updateShowTrimboxDropdownToolbarButton();
			this._updateNegateButton();
			this._updateOverprintHighlightColorButton();
			this._updateViewFromBackButton();
		};

		return ViewControlsWidget;
	});

define('web-viewer/viewControls/ViewControlsModel',["web-viewer/core/PrivateChannel",
		"web-viewer/core/AbstractModel",
		"web-viewer/utils/ViewerUtils"
	],
	function (
		PrivateChannel,
		AbstractModel,
		ViewerUtils) {
		"use strict";
		/**
		 * @exports ViewControlsModel
		 * @memberof ViewControlsModule
		 * @class ViewControlsModule holds all the information for the view controls. 
		 * @constructor
		 * 
		 * @param {Core.ViewerFacade} f - Viewer facade
		 */
		var ViewControlsModel = function (f) {
			AbstractModel.call(this, f, "ViewControlsModel");
			PrivateChannel.installTo(this, f, "viewControlsModel");

			/**
			 * @member {Number} - horizontal scale, only for digital film (if no scaling this will be 1) 
			 */
			this._hscale = 1;

			/**
			 * @member {Number} - vertical scale, only for digital film (if no scaling this will be 1) 
			 */
			this._vscale = 1;

			/**
			 * @member {Number} - horizontal distortion, only for digital film (if no distortion this will be 1) 
			 */
			this._hdist = 1;

			/**
			 * @member {Number} - vertical distortion, only for digital film (if no distortion this will be 1) 
			 */
			this._vdist = 1;

			/**
			 * @member {Number} - horizontal scale, only for digital film (if no scaling this will be 1), for reference page
			 */
			this._diffhscale = 1;

			/**
			 * @member {Number} - vertical scale, only for digital film (if no scaling this will be 1) for reference page 
			 */
			this._diffvscale = 1;

			/**
			 * @member {Number} - horizontal distortion, only for digital film (if no distortion this will be 1) for reference page 
			 */
			this._diffhdist = 1;

			/**
			 * @member {Number} - vertical distortion, only for digital film (if no distortion this will be 1) for reference page 
			 */
			this._diffvdist = 1;

			/**
			 * @member {Boolean} is the distortion compensated or not
			 */
			this._isDistCompensated = false;

			/**
			 * @member {Boolean} is the scaling compensated or not
			 */
			this._isScaleCompensated = false;

			/**
			 * @member {Boolean} is the inverse view mode enabled
			 */
			this._isNegated = false;

			/**
			 * @member {Boolean} is view from back or not
			 */
			this._viewFromBack = false;


			/*
			 * Overprint
			 */
			/**
			 * @member {Boolean} is OverprintHighlightColor active or not
			 */
			this._overprintHighlightColorActive = false;
			/**
			 * @member {Number} Overprint highlight color (rgb int)
			 */
			this._overprintHighlightColor = this.f.getOption("PREFERENCES_HIGHLIGHTCOLORS");
			this.f.getCurrentUser().on("options.changed.PREFERENCES_HIGHLIGHTCOLORS", _.bind(function (oldValue, newValue) {
				this.setOverprintHighlightColor(newValue);
			}, this));


			/*
			 * Background color 
			 */
			/**
			 * @member {Boolean}
			 */
			this._backgroundColorActive = false;

			/**
			 * @member {Number}
			 */
			this._imageBackgroundColor = this.f.getCurrentUser().getOption("PREFERENCES_IMAGE_BACKGROUNDCOLOR");
			/**
			 * @member {Number}
			 */
			this._windowBackgroundColor = this.f.getCurrentUser().getOption("PREFERENCES_WINDOW_BACKGROUNDCOLOR");
		};

		ViewControlsModel.prototype = Object.create(AbstractModel.prototype);
		ViewControlsModel.prototype.constructor = ViewControlsModel;

		/**
		 * Set the scale for the currently active digital film file
		 * @param {String|Number} hscale - horizontal scale
		 * @param {String|Number} vscale - vertical scale
		 * @param {Boolean} forCurrentPage - for the current page
		 */
		ViewControlsModel.prototype.setScale = function (hscale, vscale, forCurrentPage) {
			if (hscale && vscale) { //check if both horizontal as vertical scale are defined
				if (forCurrentPage) {
					this._hscale = ViewerUtils.parseFloat(hscale); //parse the hscale and vscale
					this._vscale = ViewerUtils.parseFloat(vscale);
				} else {
					this._diffhscale = ViewerUtils.parseFloat(hscale); //parse the hscale and vscale
					this._diffvscale = ViewerUtils.parseFloat(vscale);
				}
			} else { //set to null if not both are defined
				if (forCurrentPage) {
					this._hscale = 1;
					this._vscale = 1;
				} else {
					this._diffhscale = 1;
					this._diffvscale = 1;
				}
			}
			this._emit("scaleOrDistortionChanged");
		};

		/**
		 * Reset the scale for the currently active page (set to null), also reset the is compensating
		 * @param {Boolean} forCurrentPage - for the current page
		 */
		ViewControlsModel.prototype.resetScale = function (forCurrentPage) {
			if (forCurrentPage) {
				this._hscale = 1;
				this._vscale = 1;
			} else {
				this._diffhscale = 1;
				this._diffvscale = 1;
			}
			this._isScaleCompensated = false;
			this._emit("scaleOrDistortionChanged");
		};

		/**
		 * Set the distortion for the currently active digital film file
		 * @param {String|Number} hdist - horizontal distortion
		 * @param {String|Number} vdist - vertical distortion
		 */
		ViewControlsModel.prototype.setDistortion = function (hdist, vdist, forCurrentPage) {
			if (hdist && vdist) {
				if (forCurrentPage) {
					this._hdist = ViewerUtils.parseFloat(hdist);
					this._vdist = ViewerUtils.parseFloat(vdist);
				} else {
					this._diffhdist = ViewerUtils.parseFloat(hdist);
					this._diffvdist = ViewerUtils.parseFloat(vdist);
				}
			} else {
				if (forCurrentPage) {
					this._hdist = 1;
					this._vdist = 1;
				} else {
					this._diffhdist = 1;
					this._diffvdist = 1;
				}
			}
			this._emit("scaleOrDistortionChanged");
		};

		/**
		 * Reset the distortion for the currently active page (set to null), also reset the is compensating
		 */
		ViewControlsModel.prototype.resetDistortion = function (forCurrentPage) {
			if (forCurrentPage) {
				this._hdist = 1;
				this._vdist = 1;
			} else {
				this._diffhdist = 1;
				this._diffvdist = 1;
			}
			this._isDistCompensated = false;
			this._emit("scaleOrDistortionChanged");
		};

		/**
		 * Has the currently active page a distortion (vdist and hdist are not 100%)
		 * @returns {Boolean}
		 */
		ViewControlsModel.prototype.hasPageDistortion = function () {
			return (this._hdist !== undefined && this._vdist !== undefined) && (this._hdist !== 1 || this._vdist !== 1);
		};

		/**
		 * Has the reference page a distortion (vdist and hdist are not 100%)
		 * @returns {Boolean}
		 */
		ViewControlsModel.prototype.hasDiffPageDistortion = function () {
			return (this._diffhdist !== undefined && this._diffvdist !== undefined) && (this._diffhdist !== 1 || this._diffvdist !== 1);
		};

		/**
		 * Has the currently active page a scaling (vscale and hscale are not 100%)
		 * @returns {Boolean}
		 */
		ViewControlsModel.prototype.hasPageScale = function () {
			return (this._hscale !== undefined && this._vscale !== undefined) && (this._hscale !== 1 || this._vscale !== 1);
		};

		/**
		 * Has the reference page a scaling (vscale and hscale are not 100%)
		 * @returns {Boolean}
		 */
		ViewControlsModel.prototype.hasDiffPageScale = function () {
			return (this._diffhscale !== undefined && this._diffvscale !== undefined) && (this._diffhscale !== 1 || this._diffvscale !== 1);
		};

		/**
		 * Get the horizontal scale of the currently active page
		 * @param {Boolean} forCurrentPage - for the current page
		 * @returns {Number} - 1 is not scaling (or 100%)
		 */
		ViewControlsModel.prototype.getHorizontalScale = function (forCurrentPage) {
			if (forCurrentPage) {
				return this._hscale;
			} else {
				return this._diffhscale;
			}
		};

		/**
		 * Get the vertical scale of the currently active page
		 * @param {Boolean} forCurrentPage - for the current page
		 * @returns {Number} - 1 is not scaling (or 100%)
		 */
		ViewControlsModel.prototype.getVerticalScale = function (forCurrentPage) {
			if (forCurrentPage) {
				return this._vscale;
			} else {
				return this._diffvscale;
			}
		};

		/**
		 * Get the horizontal distortion of the currently active page
		 * @param {Boolean} forCurrentPage - for the current page
		 * @returns {Number} - 1 is not scaling (or 100%)
		 */
		ViewControlsModel.prototype.getHorizontalDistortion = function (forCurrentPage) {
			if (forCurrentPage) {
				return this._hdist;
			} else {
				return this._diffhdist;
			}
		};

		/**
		 * Get the vertical distortion of the currently active page
		 * @param {Boolean} forCurrentPage - for the current page
		 * @returns {Number} - 1 is not scaling (or 100%)
		 */
		ViewControlsModel.prototype.getVerticalDistortion = function (forCurrentPage) {
			if (forCurrentPage) {
				return this._vdist;
			} else {
				return this._diffvdist;
			}
		};

		/**
		 * Set if the scale is currently compensated or not
		 * @param {Boolean} isCompensated
		 */
		ViewControlsModel.prototype.setIsScaleCompensated = function (isCompensated) {
			this._isScaleCompensated = isCompensated;
			this._emit("scaleOrDistortionChanged");
		};

		/**
		 * Set if the distortion is currently compensated or not
		 * @param {Boolean} isCompensated
		 */
		ViewControlsModel.prototype.setIsDistCompensated = function (isCompensated) {
			this._isDistCompensated = isCompensated;
			this._emit("scaleOrDistortionChanged");
		};

		/**
		 * Enable/disable the inverse view option
		 * @param {Boolean} isNegated
		 */
		ViewControlsModel.prototype.setIsNegated = function (isNegated) {
			var oldValue = this._isNegated;
			this._isNegated = isNegated;
			this._fireChange(false, ["negated"], oldValue, isNegated);
		};

		/**
		 * Is the distortion compensated or not
		 * @returns {Boolean}
		 */
		ViewControlsModel.prototype.isDistCompensated = function () {
			return this._isDistCompensated;
		};

		/**
		 * Is the scaling compensated or not
		 * @returns {Boolean}
		 */
		ViewControlsModel.prototype.isScaleCompensated = function () {
			return this._isScaleCompensated;
		};

		/**
		 * Is the inverse view option enabled or not
		 * @returns {Boolean}
		 */
		ViewControlsModel.prototype.isNegated = function () {
			return this._isNegated;
		};


		/**
		 * Get the state of the viewControls 
		 * @returns {Object} state object
		 */
		ViewControlsModel.prototype.getState = function () {
			return {
				negated: this.isNegated(),
				viewFromBack: this.isViewFromBack(),
				overprintHighlightColorActive: this.isOverprintHighlightColorActive(),
				overprintHighlightColor: this.getOverprintHighlightColor(),
				backgroundColorActive: this.isBackgroundColorActive(),
				imageBackgroundColor: this.getImageBackgroundColor(),
				windowBackgroundColor: this.getWindowBackgroundColor()
			};
		};

		/**
		 * Return the default state
		 * @returns {Object}
		 */
		ViewControlsModel.prototype.getDefaultState = function () {
			// defaults
			return {
				negated: false,
				viewFromBack: false,
				overprintHighlightColorActive: false,
				overprintHighlightColor: this.f.getOption("PREFERENCES_HIGHLIGHTCOLORS"),
				backgroundColorActive: false,
				imageBackgroundColor: this.f.getOption("PREFERENCES_IMAGE_BACKGROUNDCOLOR"),
				windowBackgroundColor: this.f.getOption("PREFERENCES_WINDOW_BACKGROUNDCOLOR")
			};
		};

		/**
		 * Set the state of the viewControls
		 * @param {Object} state - state object (if undefined, default settings are loaded)
		 */
		ViewControlsModel.prototype.setState = function (state) {
			if (!state) {
				state = this.getDefaultState();
			}

			this.setIsNegated(state.negated);
			this.setViewFromBack(state.viewFromBack);
			// order matters: in viewcontrolsmodule, there is first an active check before appending URL parameters
			this.setOverprintHighlightColorActive(state.overprintHighlightColorActive);
			this.setOverprintHighlightColor(state.overprintHighlightColor);
			// order matters: in viewcontrolsmodule, there is first an active check before appending URL parameters
			this.setBackgroundColorActive(state.backgroundColorActive);
			this.setImageBackgroundColor(state.imageBackgroundColor);
			this.setWindowBackgroundColor(state.windowBackgroundColor);
		};

		/** 
		 * Set view from back or not
		 * @param {Boolean} viewFromBack
		 */
		ViewControlsModel.prototype.setViewFromBack = function (viewFromBack) {
			var oldValue = this._viewFromBack;
			this._viewFromBack = viewFromBack;
			this._fireChange(false, ["viewFromBack"], oldValue, viewFromBack);
		};

		/**
		 * Is view from back on or not
		 * @returns {Boolean}
		 */
		ViewControlsModel.prototype.isViewFromBack = function () {
			return this._viewFromBack;
		};

		/**
		 * Set the overprint high light color active or not
		 * @param {Boolean} toggle
		 */
		ViewControlsModel.prototype.setOverprintHighlightColorActive = function (toggle) {
			var oldValue = this._overprintHighlightColorActive;
			this._overprintHighlightColorActive = toggle;
			this._fireChange(false, ["overprintHighlightColorActive"], oldValue, toggle);
		};

		/**
		 * Is overprint highlight color active or not
		 * @returns {Boolean} True if active, false if not active
		 */
		ViewControlsModel.prototype.isOverprintHighlightColorActive = function () {
			return this._overprintHighlightColorActive;
		};

		/**
		 * Set the overprint highlight color
		 * @param {Number} rgbInt - Color as RGB encoded number
		 */
		ViewControlsModel.prototype.setOverprintHighlightColor = function (rgbInt) {
			var oldValue = this._overprintHighlightColor;
			this._overprintHighlightColor = rgbInt;
			// save the setting
			this.f.setOption("PREFERENCES_HIGHLIGHTCOLORS", rgbInt);
			this._fireChange(false, ["overprintHighlightColor"], oldValue, rgbInt);
		};

		/**
		 * Get the overprint highlight color
		 * @returns {Number} The current overprint highlight color: rgb encoded number
		 */
		ViewControlsModel.prototype.getOverprintHighlightColor = function () {
			return this._overprintHighlightColor;
		};

		/**
		 * Check whether the current overprint highlight color is the same as the default value
		 * @returns {Boolean} True if default, false otherwise
		 */
		ViewControlsModel.prototype.isOverprintHighlightColorDefault = function () {
			return this._overprintHighlightColor === this.f.getCurrentUser().getOptionDefault("PREFERENCES_HIGHLIGHTCOLORS");
		};

		/**
		 * Is Background color active or not
		 * @returns {Boolean} True if active, false if not active
		 */
		ViewControlsModel.prototype.isBackgroundColorActive = function () {
			return this._backgroundColorActive;
		};

		/**
		 * Set the background colors active or not
		 * @param {Boolean} toggle - True for active, false for deactivation
		 */
		ViewControlsModel.prototype.setBackgroundColorActive = function (toggle) {
			var oldValue = this._backgroundColorActive;
			this._backgroundColorActive = toggle;
			this._fireChange(false, ["backgroundColorActive"], oldValue, toggle);
		};


		/**
		 * Set the window background color
		 * @param {Number} rgbInt - RGB number encoded color
		 */
		ViewControlsModel.prototype.setWindowBackgroundColor = function (rgbInt) {
			var oldValue = this._windowBackgroundColor;
			this._windowBackgroundColor = rgbInt;
			// save the setting
			this.f.setOption("PREFERENCES_WINDOW_BACKGROUNDCOLOR", rgbInt);
			this._fireChange(false, ["windowBackgroundColor"], oldValue, rgbInt);
		};

		/**
		 * Get the window background color
		 * @returns {Number} The current window background color: rgb encoded number
		 */
		ViewControlsModel.prototype.getWindowBackgroundColor = function () {
			return this._windowBackgroundColor;
		};

		/**
		 * Set the image background color
		 * @param {Number} rgbInt - RGB number encoded color
		 */
		ViewControlsModel.prototype.setImageBackgroundColor = function (rgbInt) {
			var oldValue = this._imageBackgroundColor;
			this._imageBackgroundColor = rgbInt;
			// save the setting
			this.f.setOption("PREFERENCES_IMAGE_BACKGROUNDCOLOR", rgbInt);
			this._fireChange(false, ["imageBackgroundColor"], oldValue, rgbInt);
		};

		/**
		 * Get the image background color
		 * @returns {Number} The current image background: rgb encoded number
		 */
		ViewControlsModel.prototype.getImageBackgroundColor = function () {
			return this._imageBackgroundColor;
		};

		/**
		 * Check whether the current image background color is the same as the default value
		 * @returns {Boolean} True if default, false otherwise
		 */
		ViewControlsModel.prototype.isImageBackgroundColorDefault = function () {
			return this._imageBackgroundColor === this.f.getCurrentUser().getOptionDefault("PREFERENCES_IMAGE_BACKGROUNDCOLOR");
		};

		/**
		 * Check whether the current window background color is the same as the default value
		 * @returns {Boolean} True if default, false otherwise
		 */
		ViewControlsModel.prototype.isWindowBackgroundColorDefault = function () {
			return this._windowBackgroundColor === this.f.getCurrentUser().getOptionDefault("PREFERENCES_WINDOW_BACKGROUNDCOLOR");
		};

		/**
		 * Reset the background colors to its default values
		 */
		ViewControlsModel.prototype.resetBackgroundColors = function () {
			this.setImageBackgroundColor(this.f.getCurrentUser().getOptionDefault("PREFERENCES_IMAGE_BACKGROUNDCOLOR"));
			this.setWindowBackgroundColor(this.f.getCurrentUser().getOptionDefault("PREFERENCES_WINDOW_BACKGROUNDCOLOR"));
		};

		return ViewControlsModel;
	});

/**
 * @namespace ViewControlsModule
 */
define('web-viewer/viewControls/ViewControlsModule',["web-viewer/core/AbstractModule",
		"web-viewer/viewControls/ViewControlsWidget",
		"web-viewer/viewControls/ViewControlsModel",
		"web-viewer/viewControls/ViewControlsConstants",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/utils/ViewerUtils",
		"underscore"
	],
	function (AbstractModule,
		ViewControlsWidget,
		ViewControlsModel,
		ViewControlsConstants,
		ListenerUtil,
		ViewerUtils,
		_) {
		"use strict";

		/**
		 * @class
		 * This module is responsible for displaying the view widget.
		 * Also acts as the controller for the compensate distortion button, and compensates the scale and distortion for digital film files.
		 *
		 * @exports ViewControlsModule
		 * @constructor
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var ViewControlsModule = function ViewControlsModule(f, silenced) {
			AbstractModule.call(this, f, silenced, "viewControls");
			ListenerUtil.mixin(this, f);
			/**
			 * @member {ViewControlsModel} - model for the view controls, saves data needed for some of the controls
			 */
			this._viewControlsModel = new ViewControlsModel(f);

			/**
			 * @member {ViewControlsWidget} - the view widget, all view controls are accesable from this widget
			 */
			this._viewControlsWidget = null;
			this._publishWidget(ViewControlsWidget.widgetName, _.bind(function (layoutType) {
				this._viewControlsWidget = new ViewControlsWidget(f, this._viewControlsModel, this);
				return this._viewControlsWidget;
			}, this));

			//listen to when the active or difference pages changes and be sure the parameters are available	
			this.f.getDocumentController().whenActiveOrDifferencePageChangedAndParametersAvailable(_.bind(function (activePage, diffPage, oldPage, isActivePage) {

				var isScaleCompensated = this._viewControlsModel.isScaleCompensated();
				var isDistCompensated = this._viewControlsModel.isDistCompensated();

				this._viewControlsModel.resetScale(true); //be sure to reset the scale and distortion saved in the model
				this._viewControlsModel.resetDistortion(true);
				this._viewControlsModel.resetScale(false); //be sure to reset the scale and distortion saved in the model
				this._viewControlsModel.resetDistortion(false);
				if (activePage.isDigitalFilmPage()) { //check if the file is a digital film 
					this._viewControlsModel.setScale(activePage.get("hscale"), activePage.get("vscale"), true); //set the scale parameters of the page in the model
					this._viewControlsModel.setDistortion(activePage.get("hdist"), activePage.get("vdist"), true); //set the distortion parameters of the page in the model
				}
				if (diffPage) { //for the diff page no imagetype-parameter is present, always add the distortion/scaling, view is smart enough to not show the button
					this._viewControlsModel.setScale(diffPage.get("hscale"), diffPage.get("vscale"), false); //set the scale parameters of the page in the model
					this._viewControlsModel.setDistortion(diffPage.get("hdist"), diffPage.get("vdist"), false); //set the distortion parameters of the page in the model
				}

				if (activePage.isDigitalFilmPage() || (diffPage && diffPage.isDigitalFilmPage())) {
					if (isScaleCompensated && isDistCompensated) {
						this.compensateScaleAndDistortion();
					} else if (isScaleCompensated) {
						this.compensateScale();
					} else if (isDistCompensated) {
						this.compensateDistortion();
					}
				}



				// Save/Restore state when not print
				if (isActivePage) {
					// save state in old page
					if (oldPage !== undefined) {
						oldPage.set("viewControls", this._viewControlsModel.getState());
					}

					var state = activePage.get("viewControls");
					if (state !== undefined) {
						var defaultState = this._viewControlsModel.getDefaultState();
						// fall back on defaults for highlight color to reset it on page change
						state.overprintHighlightColor = defaultState.overprintHighlightColor;
						state.overprintHighlightColorActive = defaultState.overprintHighlightColorActive;
					}

					// restore state
					this._viewControlsModel.setState(state);
				}
			}, this));

			// catch requests for fragments to add parameters to the request
			this.f.on("intercept.fragmentURL", _.bind(this._interceptFragmentURL, this));
			this.f.on("intercept.parameters", _.bind(this._fillCompensationParameters, this));


			this._registerListener("changed.backgroundColorActive", this._backgroundColorActiveChanged, this, this._viewControlsModel);
			this._registerListener("changed.imageBackgroundColor", this._imageBackgroundColorChanged, this, this._viewControlsModel);
			this._registerListener("changed.windowBackgroundColor", this._windowBackgroundColorChanged, this, this._viewControlsModel);
			this._registerListener("changed.overprintHighlightColorActive", this._viewControlsModelChanged, this, this._viewControlsModel);
			this._registerListener("changed.overprintHighlightColor", this._viewControlsModelChanged, this, this._viewControlsModel);
			this._registerListener("changed.negated", this._viewControlsModelChanged, this, this._viewControlsModel);
			this._registerListener("changed.viewFromBack", this._viewFromBackChanged, this, this._viewControlsModel);
			this._registerListener("options.changed.PREFERENCES_MARGIN_CLIP_MODE", this._viewControlsModelChanged, this, this.f.getCurrentUser());


			this.f.whenModuleAvailable("cad", this._cadBackgroundColorChanged, this);
		};

		// inherit
		ViewControlsModule.prototype = Object.create(AbstractModule.prototype);
		// correct the constructor pointer 
		ViewControlsModule.prototype.constructor = ViewControlsModule;

		/**
		 * Compensate distortion for digital film files, only possible when active page is digital film (and 'compensate distortion'-button is active).
		 */
		ViewControlsModule.prototype.compensateDistortion = function () {
			this.f.getDocumentController().willNeedParameterUpdate("COMPENSTATE_DISTORTION");
			if (this._viewControlsModel.isDistCompensated()) { //check if the distortion is currently compensated, if so undo the compensation
				this.f.getActivePage().resetStateParam("compensateDist"); //reset the parameter of the active page, we'll refetch them since we are changing dimension of the page again
				this._viewControlsModel.setIsDistCompensated(false); //the compensation is no longer active, set this correctly in the model
			} else { //we want to compensate distortion!
				this.f.getActivePage().setStateParam("compensateDist", ViewControlsConstants.compensateDist.DISTONLY); //set a parameter of the currently active document page to compensate distortion
				this._viewControlsModel.setIsDistCompensated(true); //the distortion is not compensated
			}
			this.f.getDocumentController().readyForParameterUpdate("COMPENSTATE_DISTORTION", function () {
				this.f.emit("viewControls.compensationChanged");
			}, this);
		};

		/**
		 * Compensate the scale of the active digital film page.
		 */
		ViewControlsModule.prototype.compensateScale = function () { //implementation is ~same as compensateDistortion
			this.f.getDocumentController().willNeedParameterUpdate("COMPENSTATE_SCALE");
			if (this._viewControlsModel.isScaleCompensated()) {
				this.f.getActivePage().resetStateParam("compensateDist");
				this._viewControlsModel.setIsScaleCompensated(false);
			} else {
				this.f.getActivePage().setStateParam("compensateDist", ViewControlsConstants.compensateDist.SCALEONLY);

				this._viewControlsModel.setIsScaleCompensated(true);
			}
			this.f.getDocumentController().readyForParameterUpdate("COMPENSTATE_SCALE", function () {
				this.f.emit("viewControls.compensationChanged");
			}, this);

		};

		/**
		 * Compensate the scale and distortion of the digital film page
		 * @param {Boolean} needsBothCompensated - do both the scaling and distortion need to be enabled before disabling them
		 */
		ViewControlsModule.prototype.compensateScaleAndDistortion = function (needsBothCompensated) {
			this.f.getDocumentController().willNeedParameterUpdate("COMPENSTATE_SCALE_AND_DISTORTION");
			if ((needsBothCompensated && this._viewControlsModel.isScaleCompensated() && this._viewControlsModel.isDistCompensated()) || (!needsBothCompensated && (this._viewControlsModel.isScaleCompensated() || this._viewControlsModel.isDistCompensated()))) {
				this.f.getActivePage().resetStateParam("compensateDist");
				this._viewControlsModel.setIsDistCompensated(false);
				this._viewControlsModel.setIsScaleCompensated(false);
			} else {
				this.f.getActivePage().setStateParam("compensateDist", ViewControlsConstants.compensateDist.DISTANDSCALE);
				this._viewControlsModel.setIsDistCompensated(true);
				this._viewControlsModel.setIsScaleCompensated(true);
			}
			this.f.getDocumentController().readyForParameterUpdate("COMPENSTATE_SCALE_AND_DISTORTION", function () {
				this.f.emit("viewControls.compensationChanged");
			}, this);

		};

		/**
		 * The viewControlsModel has changed, invalidate fragments and rerender
		 */
		ViewControlsModule.prototype._viewControlsModelChanged = function () {
			this.f.emit("viewport.viewserverParametersChanged");
		};

		/**
		 * Fill in the parameters for scale and distortion compensation
		 * @param c
		 */
		ViewControlsModule.prototype._fillCompensationParameters = function (c) {
			if (this._viewControlsModel.isDistCompensated()) { //if distortion needs to be compensated
				c.params.applydist = ViewControlsConstants.compensateDist.DISTONLY;
				if (this._viewControlsModel.hasPageDistortion()) {
					c.params.fdistx = this._viewControlsModel.getHorizontalDistortion(true);
					c.params.fdisty = this._viewControlsModel.getVerticalDistortion(true);
				}

				if (this._viewControlsModel.hasDiffPageDistortion()) {
					c.params.ddistx = this._viewControlsModel.getHorizontalDistortion(false);
					c.params.ddisty = this._viewControlsModel.getVerticalDistortion(false);
				}
			}
			if (this._viewControlsModel.isScaleCompensated()) { //if scale needs to be compensated
				c.params.applydist = ViewControlsConstants.compensateDist.SCALEONLY;
				if (this._viewControlsModel.hasPageScale()) {
					c.params.fscalex = this._viewControlsModel.getHorizontalScale(true);
					c.params.fscaley = this._viewControlsModel.getVerticalScale(true);
				}

				if (this._viewControlsModel.hasDiffPageScale()) {
					c.params.dscalex = this._viewControlsModel.getHorizontalScale(false);
					c.params.dscaley = this._viewControlsModel.getVerticalScale(false);
				}
			}
			//if both are being compensated, scaling and distortion parameters are set correctly, just set the applydist correct
			if (this._viewControlsModel.isDistCompensated() && this._viewControlsModel.isScaleCompensated()) {
				c.params.applydist = ViewControlsConstants.compensateDist.DISTANDSCALE;
			}
		};

		/**
		 * Method for interception the request for new fragment, will add if the distortion or scale needs to be compensated
		 * @private
		 * @param {Object} c
		 */
		ViewControlsModule.prototype._interceptFragmentURL = function (c) {
			this._fillCompensationParameters(c);

			// inverse color
			if (this._viewControlsModel.isNegated()) {
				c.params.negate = "1";
			}

			// overprint highlight color
			if (this._viewControlsModel.isOverprintHighlightColorActive()) {
				c.params.overprint = ViewerUtils.rgbIntToBgrInt(this._viewControlsModel.getOverprintHighlightColor());
			}

			// graphics image background color
			if (this._viewControlsModel.isBackgroundColorActive()) {
				c.params.background = ViewerUtils.rgbIntToBgrInt(this._viewControlsModel.getImageBackgroundColor());
			}

			var clipMode = this.f.getOption("PREFERENCES_MARGIN_CLIP_MODE");
			if (clipMode === ViewControlsConstants.marginClipMode.CLIP ||
				(clipMode === ViewControlsConstants.marginClipMode.CLIP_IN_READERSPREAD && this.f.getActivePage().isReaderSpread())) {

				c.params.trimmed = "1";
			}
		};

		/**
		 * Control other modules for set view from back
		 */
		ViewControlsModule.prototype._viewFromBackChanged = function () {
			var isViewFromBack = this._viewControlsModel.isViewFromBack();
			var graphicsModule = this.f.getModule("graphics");
			if (graphicsModule) {
				var graphicsModel = graphicsModule.getGraphicsModel();
				graphicsModel.setReverseChannelOrder(isViewFromBack);
			}
			this.f.emit("viewport.transform", {
				flipViewFromBack: isViewFromBack,
				animated: false
			});

			this._viewControlsModelChanged();
		};

		/**
		 * Called when the background color active state changed
		 * -> update other modules
		 */
		ViewControlsModule.prototype._backgroundColorActiveChanged = function () {
			this._imageBackgroundColorChanged();
			this._windowBackgroundColorChanged();
		};

		/**
		 * Control other modules for cad background color
		 */
		ViewControlsModule.prototype._cadBackgroundColorChanged = function () {
			// cad
			var cadModule = this.f.getModule("cad");
			if (cadModule) {
				if (this._viewControlsModel.isBackgroundColorActive()) {
					// the chosen one
					var hexColor = ViewerUtils.hexFromInt(this._viewControlsModel.getImageBackgroundColor());
					cadModule.setCadBackgroundColor(hexColor);
				} else {
					// none
					cadModule.setCadBackgroundColor();
				}
			}
		};

		/**
		 * Control other modules for image background color
		 */
		ViewControlsModule.prototype._imageBackgroundColorChanged = function () {
			// for cad
			this._cadBackgroundColorChanged();
			// for graphics
			this._viewControlsModelChanged();
		};

		/**
		 * Control other modules for window background color
		 */
		ViewControlsModule.prototype._windowBackgroundColorChanged = function () {
			var viewportModule = this.f.getModule("viewport");
			if (viewportModule) {
				if (this._viewControlsModel.isBackgroundColorActive()) {
					// the chosen one
					var hexColor = ViewerUtils.hexFromInt(this._viewControlsModel.getWindowBackgroundColor());
					viewportModule.setWidgetBackgroundColor(hexColor);
				} else {
					// none
					viewportModule.setWidgetBackgroundColor();
				}
			}
		};

		/**
		 * @override
		 */
		ViewControlsModule.prototype.getState = function () {
			return {
				modelState: this._viewControlsModel.getState(),
				isDistCompensated: this._viewControlsModel.isDistCompensated(),
				isScaleCompensated: this._viewControlsModel.isScaleCompensated(),
				hscale: this._viewControlsModel.getHorizontalScale(true),
				vscale: this._viewControlsModel.getVerticalScale(true),
				hdist: this._viewControlsModel.getHorizontalDistortion(true),
				vdist: this._viewControlsModel.getVerticalDistortion(true),
				diffhscale: this._viewControlsModel.getHorizontalScale(),
				diffvscale: this._viewControlsModel.getVerticalScale(),
				diffhdist: this._viewControlsModel.getHorizontalDistortion(),
				diffvdist: this._viewControlsModel.getVerticalDistortion()
			};
		};

		/**
		 * @Override
		 * @param {Object} state
		 */
		ViewControlsModule.prototype.setState = function (state) {
			if (state) {
				this._viewControlsModel.setState(state.modelState);
				this._viewControlsModel.setScale(state.hscale, state.vscale, true); //set the scale parameters of the page in the model
				this._viewControlsModel.setDistortion(state.hdist, state.vdist, true); //set the distortion parameters of the page in the model
				this._viewControlsModel.setScale(state.diffhscale, state.diffvscale, false); //set the scale parameters of the page in the model
				this._viewControlsModel.setDistortion(state.diffhdist, state.diffvdist, false); //set the distortion parameters of the page in the model
				if (state.isDistCompensated && state.isScaleCompensated) {
					this.compensateScaleAndDistortion();
				} else if (state.isDistCompensated) {
					this.compensateDistortion();
				} else if (state.isScaleCompensated) {
					this.compensateScale();
				}
			}
		};

		return ViewControlsModule;
	});

