define(["./ViewportLayer",
	"web-viewer/core/PrivateChannel",
	"web-viewer/utils/ConversionConstants",
	"jquery"
], function (ViewportLayer,
	PrivateChannel,
	ConversionConstants,
	$) {
	var SVGNS = "http://www.w3.org/2000/svg";

	/**
	 * Create a ViewportLayer that displays svg documents
	 *
	 * @exports ViewportLayer
	 * @memberof ViewportModule
	 * @class A layer is responsible to render the content of one specific part of a view
	 * @constructor
	 * @mixes Utils.ListenerUtilMixin
	 *
	 * @param {Core.ViewerFacade} f
	 * @param {ViewportModule.viewportView} viewportView - view of the layer
	 * @param {ViewportModule.ViewportWorld} world - the world of the Viewport
	 * @param {Transformation} viewTransform - transformation dictated by the view
	 * @param {String} id - the ID of the Layer
	 */
	var ViewportSvgLayer = function (f, viewportView, world, viewTransform, id) {
		ViewportLayer.call(this, f, viewportView, world, viewTransform, id);
		PrivateChannel.installTo(this, f, "viewport.svgLayer");

		this._container = $("<div>").css({
			position: 'absolute',
			top: 0,
			left: 0,
			bottom: 0,
			right: 0
		}).addClass("svgContainer");

		this._svg = $(document.createElementNS(SVGNS, "svg")).css({
			position: 'absolute',
			top: 0,
			left: 0,
			bottom: 0,
			right: 0,
			width: "100%",
			height: "100%"
		}).appendTo(this._container);

		/**
		 * @member {JQueryObject} SVG Element used for transforming the svg
		 * @private
		 */
		this._svgTransform = $(document.createElementNS(SVGNS, "g")).appendTo(this._svg);

		/**
		 * @member {JQueryObject} SVG Element used for offsetting if readerspread
		 * @private
		 */
		this._readerSpreadTransform = $(document.createElementNS(SVGNS, "g")).appendTo(this._svgTransform);


		/**
		 * @member {JQueryObject} SVG Element used to give the SVG a standard white background
		 * @private
		 */
		this._svgRect = $(document.createElementNS(SVGNS, "rect")).appendTo(this._readerSpreadTransform);

	};

	ViewportSvgLayer.prototype = Object.create(ViewportLayer.prototype);
	ViewportSvgLayer.prototype.constructor = ViewportSvgLayer;

	/** 
	 * @override
	 */
	ViewportSvgLayer.prototype.getLayerElement = function () {
		return this._container;
	};

	/** 
	 * @override
	 * render the svg in the viewport
	 */
	ViewportSvgLayer.prototype.render = function () {
		if (!this._layerModel ||
			!this._world ||
			!this._transform ||
			(this._world.getWorldSize().width === 0 && this._world.getWorldSize().height === 0) ||
			!this.isVisible()) {

			// we do not want to draw if there is no this._layerModel, this._world, this._transform
			// or if the worldsize is 0x0
			// or if we are hidden

			return;
		}
		this._setSvgElement();

		// hide all popovers/popups when an animation is ongoing
		if (this._world.isAnimating()) {
			this.hideAllPopovers();
			this.hideAllPopups();
		}

		this._updateReaderSpreadTransform();
		this._updateSVGRect();
		this._updateTransformation();

		// show popups if the world is not animating
		if (!this._world.isAnimating()) {
			this.drawPopovers();
			this.drawPopups();
		}
		this._emit("repainted");


	};

	/**
	 * @override
	 */
	ViewportSvgLayer.prototype.destroy = function () {
		if (this._svgElement) {
			$(this._svgElement).detach();
		}
		this._container.remove();
		ViewportLayer.prototype.destroy.call(this);
	};

	/**
	 * @override
	 */
	ViewportSvgLayer.prototype.setLayerModel = function (layerModel) {
		ViewportLayer.prototype.setLayerModel.call(this, layerModel);

		this._svgElement = this._layerModel.getSVGElement();
		this._readerSpreadTransform.append(this._svgElement);

		this._registerListener("changed.visible", function () {
			if (this.isVisible()) {
				this._svgTransform.appendTo(this._svg);
			} else {
				this._svgTransform.detach();
			}
			this.repaint();
		}, this, this._layerModel);

		this._registerListener("pageChanged", this._setSvgElement, this, this._layerModel);



	};

	/**
	 * update the g element to transform the svg
	 */
	ViewportSvgLayer.prototype._updateTransformation = function () {
		var matrix = this.getTransformMatrix();

		// svg transformation happens on a pixel basis while viewer transformations are in mm
		var scale = ConversionConstants.PX_TO_MM_RATIO;
		matrix.scale(scale, scale);

		var matrixString = "matrix(" + [matrix.a, matrix.b, matrix.c, matrix.d, matrix.e, matrix.f].join(",") + ")";

		this._svgTransform[0].setAttribute("transform", matrixString);

	};

	/**
	 * set the size of the svg
	 */
	ViewportSvgLayer.prototype.setSize = function (dim) {

		this._svg[0].setAttributeNS(SVGNS, "width", dim.width);
		this._svg[0].setAttributeNS(SVGNS, "height", dim.height);
		this._svg[0].setAttributeNS(SVGNS, "viewBox", "0 0 " + dim.width + " " + dim.height);

		this.render(); // we have to render immediately otherwise there could be flickering (when size of canvas is set, all pixels are reset)
	};


	/**
	 * Update the white background rect
	 */
	ViewportSvgLayer.prototype._updateSVGRect = function () {
		var dim = this._layerModel.getSVGSize();
		if (dim) {
			this._svgRect[0].setAttribute("x", 0);
			this._svgRect[0].setAttribute("y", 0);
			this._svgRect[0].setAttribute("width", dim.width / ConversionConstants.PX_TO_MM_RATIO);
			this._svgRect[0].setAttribute("height", dim.height / ConversionConstants.PX_TO_MM_RATIO);
			this._svgRect[0].setAttribute("fill", "#fff");
		}

	};

	/**
	 * Updates the transformation that will translate the page if this is viewed in readerspread mode
	 */
	ViewportSvgLayer.prototype._updateReaderSpreadTransform = function () {
		var offset = this._layerModel.getReaderSpreadOffset();
		var offsetString = "translate(" + offset.x / ConversionConstants.PX_TO_MM_RATIO + "," + offset.y / ConversionConstants.PX_TO_MM_RATIO + ")";
		this._readerSpreadTransform[0].setAttribute("transform", offsetString);
	};

	/**
	 * detaches / attaches the correct svg element 
	 */
	ViewportSvgLayer.prototype._setSvgElement = function () {
		if (this._svgElement !== this._layerModel.getSVGElement()) {
			if (this._svgElement) {
				$(this._svgElement).detach();
			}

			if (this._layerModel.getSVGElement()) {
				this._svgElement = this._layerModel.getSVGElement();
				this._readerSpreadTransform.append(this._svgElement);
			}
		}
	}

	return ViewportSvgLayer;
});
