/**
 * @namespace WebCenter
 */
define(["web-viewer/data/AbstractDataAdapter",
		"web-viewer/webcenter/WebCenterDocument",
		"web-viewer/webcenter/WebCenterDocumentVersion",
		"web-viewer/webcenter/WebCenterDocumentPage",
		"web-viewer/data/DataConstants",
		"utils/HTTPUtils",
		"utils/XMLUtils",
		"utils/WCRI18n",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/graphics/GraphicsConstants",
		"web-viewer/viewControls/ViewControlsConstants",
		"web-viewer/annotation/AnnotationConstants",
		"web-viewer/cad/CadFile",
		"web-viewer/utils/ConversionConstants",
		"utils/ShapeUtils",
		"jquery",
		"underscore"
	],

	function (AbstractDataAdapter,
		WebCenterDocument,
		WebCenterDocumentVersion,
		WebCenterDocumentPage,
		DataConstants,
		HTTPUtils,
		XMLUtils,
		WCRI18n,
		ViewerUtils,
		GraphicsConstants,
		ViewControlsConstants,
		AnnotationConstants,
		CadFile,
		ConversionConstants,
		ShapeUtils,
		$,
		_
	) {
		"use strict";

		/**
		 * @exports WebCenterDataAdapter
		 * @memberof WebCenter
		 * @class WebCenter specific DataAdapter <<extends AbstractDataAdapter>>
		 * @augments AbstractDataAdapter
		 * @param {Core.ViewerFacade} f
		 * @param {String} URL to the WebCenter instance
		 * @param {String} URL to the viewserver
		 */
		var WebCenterDataAdapter = function (f, instance, viewserver) {
			AbstractDataAdapter.call(this, f, instance);

			/**
			 * @private
			 */
			this.instance = instance;
			/**
			 * @private
			 */
			this.viewserver = viewserver;

			/**
			 * This counter is used to see when parameters return, they are still valid
			 */
			this._parameterUpdateToken = 0;

			this._cadStyles = {};

			/**
			 * Keep the session alive in WebCenter
			 */
			var keepSessionAlive = _.bind(function () {
				f.ajax(this.WebCenter("sessionalive.jsp"), {
					error: function (jqXHR, textStatus, errorThrown) {
						f.i18n.loadBundle("viewer", function () {
							f.emit("error.WARNING", f.i18n.t("WCSessionLostError"), jqXHR, textStatus, errorThrown);
						});
					},
					success: function (data) {
						f.debug.log("Keep WebCenter session alive");
					}
				});
			}, this);

			setInterval(keepSessionAlive, 300000); // every 5 minutes


			// inject the viewserver servlet when a fragmentURL is requested  
			this.f.on("intercept.fragmentURL", _.bind(this.interceptFragmentURL, this));

		};


		WebCenterDataAdapter.prototype = Object.create(AbstractDataAdapter.prototype);
		WebCenterDataAdapter.prototype.constructor = WebCenterDataAdapter;

		/**
		 * Returns the number of documents that are loaded with the documentsIdentifier
		 * @param {String|Object} documentsIdentifier
		 * @returns {Number}
		 */
		WebCenterDataAdapter.prototype.getNumberOfDocuments = function (documentsIdentifier) {
			if (documentsIdentifier === undefined) {
				return 0;
			} else if (_.isString(documentsIdentifier)) {
				return 1;
			} else if (_.isObject(documentsIdentifier)) {
				var data = documentsIdentifier.pagelist.page;
				if (_.isArray(data)) {
					return data.length;
				} else {
					return 1;
				}
			}
		};

		/**
		 * Open document(s)
		 * @param {any} documentsIdentifier - String or JSON object containing information on which document to load
		 * @param {Function} callback - Callback fired with as first argument the document (if loading succeeded, otherwise null) and as second argument whether the document should be made active. Can be fired multiple times when there are multiple documents to be loaded.
		 */
		WebCenterDataAdapter.prototype.openDocuments = function (documentsIdentifier, callback) {
			if (_.isString(documentsIdentifier)) {
				// just a doc version id
				this.loadDocument(documentsIdentifier, callback, true);
			} else if (_.isObject(documentsIdentifier)) {
				// an object
				this.loadDocuments(documentsIdentifier, callback);
			} else if (documentsIdentifier === undefined) {
				//no documents to load
			} else {
				this.f.i18n.loadBundle("viewer", function () {
					this.f.emit("error.SEVERE", this.f.t("UnknownDocumentError"), documentsIdentifier);
				}, this);
			}
		};

		/**
		 * Load basic information on the document, make all the needed documents
		 *
		 * SHOULD ONLY BE USED BY THE DATA ADAPTER
		 *
		 * @param {String} fileIdentifier - Identifier of the file
		 * @param {Function} callback - Executed when document is loaded
		 * @param {Boolean} makeActive - make the loaded document active
		 */
		WebCenterDataAdapter.prototype.loadDocument = function (fileIdentifier, callback, makeActive) {
			var url = this.WebCenter("getdocumentinfo.jsp", {
				docversionid: fileIdentifier
			});
			this.f.ajax(url, {
				success: _.bind(function (data) {
					data = XMLUtils.xml2obj(data);
					if (data.status.value == "0") {
						if (_.isArray(data.status.page)) {
							_.each(data.status.page, _.bind(function (page, i) {

								// Check if there is a requested pageIndex
								var requestedPageIndex = this.f.getProperty("WebCenterDataAdapter.pageIndex");
								var active = makeActive;
								if (page.pageno !== undefined && requestedPageIndex !== undefined) {
									active = makeActive && page.pageno == this.f.getProperty("WebCenterDataAdapter.pageIndex");
								}

								this._initDocument(page, callback, active);
							}, this));
						} else {
							this._initDocument(data.status.page, callback, makeActive);
						}
					} else {
						this.f.i18n.loadBundle("viewer", function () {
							this.f.emit("error.SEVERE", this.f.i18n.t("LoadingDocumentInfoError"), data);
						}, this);
						callback(null);
					}

				}, this),
				error: _.bind(function (jqXHR, textStatus, errorThrown) {
					// error while loading the document			
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("LoadingDocumentInfoError"), jqXHR, textStatus, errorThrown);
					}, this);
					callback(null);
				}, this)
			});
		};

		/**
		 * Load all documents of the passed object. Sets the link between graphics and CAD files.
		 * SHOULD ONLY BE USED BY THE DATA ADAPTER
		 * @param {Object} documentsIdentifier - object with all document information (specific for webcenter)
		 * @param {Function} callback - for every generated document, the callback is executed
		 */
		WebCenterDataAdapter.prototype.loadDocuments = function (documentsIdentifier, callback) {
			var data = documentsIdentifier.pagelist.page;
			//check if multiple documents are loaded
			if (_.isArray(data)) {
				//multiple documents need to be loaded
				_.each(data, _.bind(function (document, index) {
					// for page lists, if startpageindex is defined, activate it, otherwise, activate page 1
					var makeActive = index === 0;
					if (documentsIdentifier.pagelist.startpageindex !== undefined) {
						makeActive = index === documentsIdentifier.pagelist.startpageindex;
					}
					this._initDocument(document, callback, makeActive);
				}, this));
			} else {
				this._initDocument(data, callback, true);
			}
		};

		/**
		 * Initializes documents with single document information
		 * @param {Object} data
		 * @param {Function} callback
		 * @param {Boolean} makeActive - make a loaded document active
		 */
		WebCenterDataAdapter.prototype._initDocument = function (data, callback, makeActive) {
			var docController = this.f.getDocumentController();
			var graphicsDocument = null;
			var cadDocument = null;
			var activeVersion = data.docversion;
			var document;
			//check if file is graphics file
			if (data.file !== undefined || data.blank !== undefined) {
				// Check if the file is an SVG file or a graphics file
				var docType = null;
				if (data.file && data.file.match(/\.svg$/i)) {
					docType = DataConstants.documentPageType.SVG;
				} else {
					docType = DataConstants.documentPageType.GRAPHICS;
				}

				//first check if the document doesn't already exist (multiple pages belonging to the same document)
				document = this.getDocumentByDocId(data.docid);
				data.hasDefaultData = true;
				if (data.pagecount !== undefined && data.pagecount > 1) {
					data.isMultiPage = true;
				} else {
					data.isMultiPage = false;
				}
				if (document === undefined) {
					graphicsDocument = new WebCenterDocument(this.f, data, docController.getNextIID(), docType, this);
				} else { //this is only the case for multipage and pagelists (versions are in the definition of the document in versionlist
					document.addDocumentPageFromData(data);
					graphicsDocument = document;
				}

			}
			if (data.cad !== undefined) { //it's a CAD file
				if (graphicsDocument !== null) {
					if (data.cadid !== undefined) {
						data.docid = data.cadid;
						data.docversionid = data.cadversionid;
					}
				}
				if (graphicsDocument === null || !graphicsDocument.hasLinkedDocument()) {
					cadDocument = new WebCenterDocument(this.f, data, docController.getNextIID(), DataConstants.documentPageType.CAD, this);
				}
				if (callback && graphicsDocument === null) {
					callback(cadDocument, false); // do not yet make active, see further
				}

			}
			if (data.cad === undefined && data.file === undefined && (data.pageno === undefined || parseInt(data.pageno, 10) === 1) && data.blank != 1) {
				this.loadDocument(data.docversionid, callback, makeActive);
			}
			//check if there is a CAD and Graphics loaded, there is a link between CAD and Graphics
			if (graphicsDocument !== null && cadDocument !== null) {
				// find the correct document page for the link
				var documentPage;
				var allVersions = cadDocument.getAllDocumentVersions();
				var checkPage = function (page) {
					return page && page.get("docversionid") === data.cadversionid;
				};
				for (var i = 1; !documentPage && i <= allVersions.length; i++) {
					var documentVersion = allVersions[i];
					if (documentVersion) {
						documentPage = _.find(documentVersion.getAllPages(), checkPage);
					}
				}
				graphicsDocument.setLinkedDocumentPage(documentPage);
			}

			if (callback && graphicsDocument !== null) {
				callback(graphicsDocument, false); //do not yet make active, maybe there is a link to CAD
			} else if (document) {
				callback(document, false);
			}

			if (makeActive) {
				// If there are no graphics and only CAD, activate the right cad version
				if (graphicsDocument === null && cadDocument !== null && cadDocument.getDocumentVersion(activeVersion) !== undefined && cadDocument.getDocumentVersion(activeVersion).getFirstPage() !== undefined) {
					docController.changeActivePage(cadDocument.getDocumentVersion(activeVersion).getFirstPage());
					// If there are graphics, activate the graphics version
				} else if (document && document.getDocumentVersion() !== undefined && document.getDocumentVersion().getLastPage() !== undefined) {
					// if the makeActive flag is on, but we're dealing with a page instead of a new document, activate the page.
					// since the pages are added in order, we can take the last added page to activate
					// NOTE: pagelist will always have all pages in latest version.
					docController.changeActivePage(document.getDocumentVersion().getLastPage());
				} else if (graphicsDocument !== null && graphicsDocument.getDocumentVersion(activeVersion) !== undefined && graphicsDocument.getDocumentVersion(activeVersion).getFirstPage() !== undefined) {
					//if document wasn't defined yet use graphicsDocument with the first page.
					docController.changeActivePage(graphicsDocument.getDocumentVersion(activeVersion).getFirstPage());
				}
			}
		};

		/**
		 * Returns an instance for the I18n provider
		 * @returns {I18n}
		 */
		WebCenterDataAdapter.prototype.getI18nProvider = function () {
			return WCRI18n.getInstance();
		};

		/**
		 * Construct an URL to WebCenter
		 * @example WebCenter("DownloadLocKeys.jsp", { scope : "viewer" }) - will call the DownloadLocKeys.jsp?scope="viewer" page in WebCenter
		 * @param {String} servlet - Name of the page, relative to WebCenter instance
		 * @param {Object} args - Key,Value pairs as parameters
		 * @returns {String} URL to call
		 */
		WebCenterDataAdapter.prototype.WebCenter = function (servlet, args) {
			return HTTPUtils.constructURL(servlet, args, this.instance);
		};

		/**
		 * Construct an URL to the Viewer
		 * @see WebCenter but for the ViewServer
		 * @param {String} servlet - Name of the page, relative to viewserver root
		 * @param {Object} args - Key,Value pairs as parameters
		 * @returns {String} URL to call
		 */
		WebCenterDataAdapter.prototype.ViewServer = function (servlet, args) {
			return HTTPUtils.constructURL(servlet, args, this.viewserver);
		};

		/**
		 * Get path for images referenced to WebCenter
		 * @returns {String} URL for images referenced to WebCenter
		 */
		WebCenterDataAdapter.prototype.getPathToWCRImages = function () {
			return this.instance + "images/";
		};

		/**
		 * Get path to JS directory of WebCenter
		 * @returns {String} URL to javascript directory of WebCenter
		 */
		WebCenterDataAdapter.prototype.getPathToJSFiles = function () {
			return this.instance + "javascript/";
		};

		/**
		 * Set the base parameters when a FragmentURL is generated
		 * @param {Object} c - context for interceptor
		 */
		WebCenterDataAdapter.prototype.interceptFragmentURL = function (c) {
			// set rectangle parameters
			c.params.x1 += c.rectangle.left();
			c.params.y1 += c.rectangle.top();
			c.params.x2 += c.rectangle.right();
			c.params.y2 += c.rectangle.bottom();

			// resolution
			c.params.res = c.resolution;

			// other settings
			c.params.wc = 6;

			// set correct servlet for viewdata
			c.servlet = this.viewserver + "thumbdata.jsp";
		};


		/**
		 * Check if the given update token still valid
		 * @param   {Number}  updateToken - the token generated by gneerateUpdateToken when de update was requested
		 * @returns {Boolean}
		 */
		WebCenterDataAdapter.prototype.isUpdateTokenValid = function (updateToken) {
			return this._parameterUpdateToken === updateToken;
		};

		/**
		 * Generate a new update token (this makes the previous generated update token invalid)
		 * @returns {Number} updateToken, used for checking wether an asynchronous result is still valid
		 */
		WebCenterDataAdapter.prototype.generateUpdateToken = function () {
			return ++this._parameterUpdateToken;
		};

		/**
		 * We need to cancel the current parameter update
		 */
		WebCenterDataAdapter.prototype.cancelDocumentParametersUpdate = function () {
			this.generateUpdateToken();
		};

		/**
		 * Should the CAD data and style file be requested for a certain page
		 * @param   {AbstractDocumentPage} cadPage - the file for which CAD might need to be fetched
		 * @returns {Boolean} true if CAD data and style still need to be requested
		 */
		WebCenterDataAdapter.prototype.shouldRequestCadDataForPage = function (cadPage) {
			var cadData = cadPage.get("cadData");

			return (!cadData || !cadData.loaded);
		};

		/**
		 * Should the style be requested for this page?
		 * @param   {AbstractDocumentPage} cadPage - the page for we want to have the CAD style
		 * @returns {Boolean}
		 */
		WebCenterDataAdapter.prototype.shouldRequestCadStyleForPage = function (cadPage) {
			return !this._cadStyles[cadPage.get("style")];
		};


		/**
		 * Update the given pages with correct data.
		 *
		 * !! Note !! if two consecutive calls are done, the first (ongoing) call should be cancelled, only the last call result can update the parameters
		 * @param {AbstractDocumentPage} currentPage - the currenct active page
		 * @param {AbstractDocumentPage} differencePage - the current d
		 * @param {AbstractDocumentPage} cadOverlay - the current cad overlay
		 * @param {function} callback - callback to be called when parameter call has finished
		 * @override
		 */
		WebCenterDataAdapter.prototype.updateDocumentParameters = function (currentPage, differencePage, cadOverlay, callback) {
			var updateToken = this.generateUpdateToken();

			if (currentPage.getDocumentVersion().isPageList() || !currentPage.get("hasCorrectMargin")) {
				// we are dealing with a page list, make sure we have the correct page info
				this._fetchPageInfo(currentPage, _.bind(function () {
					if (differencePage && differencePage.getDocumentVersion().isPageList()) {
						// the difference page is also a difference page is also a page list, make sure we have the correct page info
						this._fetchPageInfo(differencePage, _.bind(function () {
							this._fetchParameters(updateToken, currentPage, differencePage, cadOverlay, callback);
						}, this));
					} else {
						this._fetchParameters(updateToken, currentPage, differencePage, cadOverlay, callback);
					}
				}, this));
			} else {
				if (differencePage && (differencePage.getDocumentVersion().isPageList() || !differencePage.get("hasCorrectMargin"))) {
					// the difference page is a pagelist or we don't have correct page info
					this._fetchPageInfo(differencePage, _.bind(function () {
						this._fetchParameters(updateToken, currentPage, differencePage, cadOverlay, callback);
					}, this));
				} else {
					this._fetchParameters(updateToken, currentPage, differencePage, cadOverlay, callback);
				}
			}
		};

		/**
		 * Fetch the parameters
		 * @override
		 * @param {Number}               updateToken      - the unique token for the update
		 * @param {AbstractDocumentPage} currentPage      - current page
		 * @param {AbstractDocumentPage} [differencePage] - difference page
		 * @param {AbstractDocumentPage} [cadOverlay]     - cad Overlay (if defined)
		 * @param {Function}             callback         - callback to be called when parameters are available
		 */
		WebCenterDataAdapter.prototype._fetchParameters = function (updateToken, currentPage, differencePage, cadOverlay, callback) {
			var urlParams;
			var newActivePage;

			if (currentPage.hasSvg()) {
				// SVG file can be received the same way as the cad data.
				urlParams = {
					wc: 6,
					cad: currentPage.get("file")
				};

				// get the difference page svg as well
				if (differencePage !== undefined && differencePage.hasSvg()) {
					urlParams.dcad = differencePage.get("file");
				}


			} else if (currentPage.hasGraphics() || (currentPage.isBlankPage() && differencePage && differencePage.hasGraphics())) {
				// either current or difference page has graphics
				if (currentPage.isBlankPage() && differencePage && differencePage.hasGraphics()) {
					// current page is a blank page, switch current en difference so it works correctly
					newActivePage = differencePage;
					differencePage = currentPage;
					currentPage = newActivePage;
				}

				// set requestparameters for current page
				var currentPageMargin = currentPage.getMargins();
				var currentPageTrimboxSize = currentPage.getTrimboxSize();

				urlParams = {
					wc: 6,
					file: currentPage.get("file"),
					ftbtlx: currentPageMargin.left,
					ftbtly: currentPageMargin.top,
					ftbbrx: currentPageTrimboxSize.width + currentPageMargin.left,
					ftbbry: currentPageTrimboxSize.height + currentPageMargin.top
				};

				if (currentPage.getDocumentVersion().isPageList()) {
					urlParams.pageno = currentPage.getPagenum();
				}

				if (currentPage.getDocumentVersion().isMultiPage()) {
					urlParams.fmp = 1;
				}

				// set parameters for difference pagfe

				if (differencePage) {
					var diffPageMargin = differencePage.getMargins();
					var diffPageTrimboxSize = differencePage.getTrimboxSize();

					if (differencePage.getDocumentVersion().isPageList()) {
						urlParams.diffpageno = differencePage.getPagenum();
					}
					urlParams.dtbtlx = diffPageMargin.left;
					urlParams.dtbtly = diffPageMargin.top;
					urlParams.dtbbrx = diffPageTrimboxSize.width + diffPageMargin.left;
					urlParams.dtbbry = diffPageTrimboxSize.height + diffPageMargin.top;
				}

				// do an interception for the parameters
				this.f.emit("intercept.parameters", {
					params: urlParams
				});

				//switch pages on the viewserver (the cuurentpage was a blank page)
				if (newActivePage && urlParams.rs) {
					urlParams.rs = urlParams.rs == 1 ? 2 : 1;
				}

				if (differencePage !== undefined && !differencePage.isBlankPage()) {
					urlParams.diff = differencePage.get("file");
					differencePage.resetParameters();
				}


				if (cadOverlay) {
					// there is a CAD overlay set, load the CAD information if not available
					if (this.shouldRequestCadDataForPage(cadOverlay)) {
						// we still need to request the data
						urlParams.cad = cadOverlay.get("cad");

						if (this.shouldRequestCadStyleForPage(cadOverlay)) {
							urlParams.style = cadOverlay.get("style");
						}
					}
				}
			} else if (currentPage.hasCad()) {
				// we are in CAD mode

				if (this.shouldRequestCadDataForPage(currentPage)) {
					// we still need to fetch CAD data for the current page

					urlParams = {
						wc: 6,
						cad: currentPage.get("cad")
					};

					if (this.shouldRequestCadStyleForPage(currentPage)) {
						urlParams.style = currentPage.get("style");
					}

					if (differencePage && differencePage.hasCad() && this.shouldRequestCadDataForPage(differencePage)) {
						// we also need to fetch cad for the difference
						urlParams.dcad = differencePage.get("cad");
					}
				} else if (differencePage && differencePage.hasCad() && this.shouldRequestCadDataForPage(differencePage)) {
					// we already have the cad data for the active page, but not yet for the difference page

					//switch current and difference page (note this is possible because we are not dealing with graphics!!)
					var realCurrentPage = currentPage;
					currentPage = differencePage;
					differencePage = realCurrentPage;

					urlParams = {
						wc: 6,
						cad: currentPage.get("cad")
					};

					if (this.shouldRequestCadStyleForPage(currentPage)) {
						urlParams.style = currentPage.get("style");
					}
				} else {
					// no data should be fetched, just call the callback
					callback();
				}

			}

			if (urlParams) {
				// we should fetch some data
				urlParams.afs = 1;
				var url = this.ViewServer("parameters.jsp", urlParams);
				this.f.ajax(url, {
					success: _.bind(this._handleParametersResult, this, updateToken, urlParams, currentPage, differencePage, cadOverlay, callback)
				});
			} else {
				// no new data needs to be fetched

				// it could be that CAD data is still being fetched, wait for the data to be fully fetched before calling the callback
				if (callback) {
					this._whenCadDataAvailableForAllActiveFiles(updateToken, currentPage, differencePage, cadOverlay, callback);
				}
			}
		};


		/**
		 * Wait for all ongoing CAD data requests to be finished
		 * !! note !! callback is only executed when the updateToken is still valid
		 * @param {Number}               updateToken      - the unique token for the update
		 * @param {AbstractDocumentPage} currentPage      - current page
		 * @param {AbstractDocumentPage} [differencePage] - difference page
		 * @param {AbstractDocumentPage} [cadOverlay]     - cad Overlay (if defined)
		 * @param {Function}             callback         - callback to be called when all CAD data is loaded
		 */
		WebCenterDataAdapter.prototype._whenCadDataAvailableForAllActiveFiles = function (updateToken, currentPage, differencePage, cadOverlay, callback) {
			this.f.requestModule("cad", _.bind(function () {
				if (this.isUpdateTokenValid(updateToken)) {
					// first find out if there are still ongoing CAD data calls
					var stillWaitingForCad = _.filter([currentPage, differencePage, cadOverlay], function (page) {
						if (page && (page.hasCad() || page.hasSvg())) {
							var cadData = page.get("cadData") || page.get("svgData");
							return cadData.requested && cadData.loaded === false;
						} else {
							return false;
						}
					}, this);

					// we only want to fire the callback when the update token is still valid
					var finalcallback = _.bind(function (updateToken, callback) {
						if (this.isUpdateTokenValid(updateToken)) {
							callback();
						}
					}, this, updateToken, callback);


					if (stillWaitingForCad.length > 0) {
						// some CAD data loading is still ongoing, listen on each pending page for the cadData.available event and execute the callback when all is loaded
						finalcallback = _.after(stillWaitingForCad.length, finalcallback);

						_.each(stillWaitingForCad, function (page) {
							page.on("cadData.available", finalcallback);
						});
					} else {
						// no need to wait, we can just call the callback
						finalcallback();
					}
				}
			}, this), true); // don't publish CAD widget!!
		};


		/**
		 * Call to parameters.jsp has finished, handle the CAD files first  (and after that normal graphical parameters)
		 *
		 * @param {Number}               updateToken      - the unique token for the update
		 * @param {Object}               urlParams        - parameters used for parameters.jsp call
		 * @param {AbstractDocumentPage} currentPage      - current page
		 * @param {AbstractDocumentPage} [differencePage] - difference page
		 * @param {AbstractDocumentPage} [cadOverlay]     - cad Overlay (if defined)
		 * @param {Function}             callback         - callback to be called whe
		 * @param {Object}               result           - result returned from parameters.jsp
		 */
		WebCenterDataAdapter.prototype._handleParametersResult = function (updateToken, urlParams, currentPage, differencePage, cadOverlay, callback, result) {
			var res = XMLUtils.xml2obj(result);

			if (!res.status || res.status.value == -1) {
				if (this.isUpdateTokenValid(updateToken)) {
					// there was an error	
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("ServerImageParmsError", this.f.i18n.t("file")), res.status.message, res);
					}, this);
				}
			} else {
				if (res.status.parameters.cad) {
					if (res.status.parameters.cad.data) {
						this._loadCadDataFor(cadOverlay ? cadOverlay : currentPage, res.status.parameters.cad.data);
					}

					if (res.status.parameters.cad.ddata) {
						this._loadCadDataFor(differencePage, res.status.parameters.cad.ddata);
					}

					if (res.status.parameters.cad.style) {
						this._loadCadStyle(urlParams.style, res.status.parameters.cad.style);
					}
				}


				this._whenCadDataAvailableForAllActiveFiles(updateToken, currentPage, differencePage, cadOverlay, _.bind(function () {
					this._handleGraphicsParametersResult(updateToken, currentPage, differencePage, cadOverlay, callback, res);
				}, this));
			}
		};
		/**
		 * Execute callback when a certain Cad Style is available
		 * @param {String}   styleName - name of the CAD style
		 * @param {Function} callback  - callback to be called when CAD Style is available
		 * @param {Object}   [binding]   - binding for the callback function
		 */
		WebCenterDataAdapter.prototype._whenCadStyleAvailable = function (styleName, callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			if (this._cadStyles[styleName] && this._cadStyles[styleName] !== "LOADING") {
				callback(this._cadStyles[styleName]);
			} else {
				// wait for the style to become available
				this.once(["cadStyle", styleName, "available"], callback);
			}
		};

		/**
		 * Load a CAD style
		 * @param {String} styleName     - name of the CAD style
		 * @param {String} styleLocation - URL indication the location of the TMP file on the server
		 */
		WebCenterDataAdapter.prototype._loadCadStyle = function (styleName, styleLocation) {
			this._cadStyles[styleName] = "LOADING";

			this._fetchPreviewData(styleLocation, _.bind(function (result) {
				this._cadStyles[styleName] = result.documentElement;
				this._emit(["cadStyle", styleName, "available"], result.documentElement);
			}, this));
		};

		/**
		 * Load CAD data for a page
		 * @param {AbstractDocumentPage} page           - the page for which CAD data should be retrieved
		 * @param {String}               cadPreviewFile - URL of the location of the TMP CAD file on the server
		 * @param {Function}             [callback]       - callback function, called when CAD file is available
		 */
		WebCenterDataAdapter.prototype._loadCadDataFor = function (page, cadPreviewFile, callback) {
			var fileCadData = {
				requested: true,
				loaded: false
			};
			if (page.hasSvg()) {
				page.set("svgData", fileCadData);
			} else {
				page.set("cadData", fileCadData);
			}
			this._fetchPreviewData(cadPreviewFile, _.bind(function (cadData) {
				if (page.hasSvg()) {
					// make sure svg module is loaded
					this.f.requestModule("svg", _.bind(function () {
						// no need to wait for the style
						fileCadData.loaded = true;

						var svgElement = cadData.querySelector("svg");
						svgElement.id = "SVG_" + page.getUniqueID();

						fileCadData.rawSVG = svgElement;

						page.set("SVGElementID", svgElement.id);
						
						
						/*
						 * The svg needs a width and height attribute to display correctly
						 * We first look at the attributes, if not present, use the viewbox to determine the width and height
						 */
						var width = $(svgElement).attr("width");
						if (width && width.indexOf("pt") > -1) {
							// Width of the SVG defined in pt, convert to mm
							width = parseFloat(width) / ConversionConstants.MM_TO_POINT_RATIO;
						} else if (width) {
							width = parseFloat(width) / ConversionConstants.MM_TO_PX_RATIO;
						} else if (svgElement.viewBox) {
							width = svgElement.viewBox.baseVal.width / ConversionConstants.MM_TO_PX_RATIO;
							svgElement.setAttribute("width", svgElement.viewBox.baseVal.width + "px");
						}
						var height = $(svgElement).attr("height");
						if (height && height.indexOf("pt") > -1) {
							// height of the SVG defined in pt, convert to mm
							height = parseFloat(height) / ConversionConstants.MM_TO_POINT_RATIO;
						} else if (height) {
							height = parseFloat(height) / ConversionConstants.MM_TO_PX_RATIO;
						} else if (svgElement.viewBox) {
							height = svgElement.viewBox.baseVal.height / ConversionConstants.MM_TO_PX_RATIO;
							svgElement.setAttribute("height", svgElement.viewBox.baseVal.height + "px");
						}

						page.set("SVGSize", new ShapeUtils.Size(width, height));

						page.setCadDataAvailable();


						if (callback) {
							callback();
						}
					}, this), true);

				} else {
					//for cad wait for styles
					this._whenCadStyleAvailable(page.get("style"), function (cadStyle) {
						fileCadData.loaded = true;
						fileCadData.rawCad = cadData.documentElement;
						fileCadData.rawStyle = cadStyle;
						page.set("cadData", fileCadData);
						page.setCadDataAvailable();

						if (callback) {
							callback();
						}
					}, this);
				}
			}, this));
		};

		/** 
		 * @param {AbstractDocumentPage} page
		 * @param {Object} res
		 */
		WebCenterDataAdapter.prototype._getSVGParameters = function (page, res) {

			if (page.get("SVGSize")) {
				var width = page.get("SVGSize").width;
				var height = page.get("SVGSize").height;
				return {
					resolutionx: ViewerUtils.parseFloat(res.status.parameters.resolutionx),
					resolutiony: ViewerUtils.parseFloat(res.status.parameters.resolutiony),
					"sizex": width,
					"sizey": height,
					"fileminx": 0,
					"fileminy": 0,
					"filemaxx": width,
					"filemaxy": height
				};
			} else {
				return {};
			}

		};

		/**
		 * Fetch PreviewData file from the ViewServer (fetch the data, remove the preview and execute the callback)
		 * @param {String}   name            - URL of the file on the ViewServer
		 * @param {Function} callback        - callback to be called when data is available (the first parameter of the callback is the needed data)
		 * @param {Function} [errorCallback] - callback to be called when an error occurs
		 */
		WebCenterDataAdapter.prototype._fetchPreviewData = function (name, callback, errorCallback) {

			this.f.ajax(this.ViewServer("getcaddata.jsp", {
				url: name
			}), {
				success: _.bind(function (result) {
					try {
						callback(result);
					} finally {
						// make sure to delete the tmp file
						this.f.ajax(this.ViewServer("previewdelete.jsp", {
							url: name
						}), {
							error: _.bind(function (jqXHR, textStatus, errorThrown) {
								this.f.i18n.loadBundle("viewer", function () {
									this.f.emit("error.SEVERE", this.f.i18n.t("PreviewDeleteError"), jqXHR, textStatus, errorThrown);
								}, this);
							}, this)
						});
					}
				}, this),
				error: _.bind(function (jqXHR, textStatus, errorThrown) {
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("ServerCadReadError", this.f.i18n.t("file")), jqXHR, textStatus, errorThrown);
					});

					if (errorCallback) {
						errorCallback(jqXHR, textStatus, errorThrown);
					}
				}, this)
			});
		};

		/**
		 * This function is called whe nthe CAD data is already handled, it sets the correct parameters in the pages (page dimensions, file info, channel info)
		 *
		 * @param {Number}               updateToken      - the unique token for the update
		 * @param {AbstractDocumentPage} currentPage      - current page
		 * @param {AbstractDocumentPage} [differencePage] - difference page
		 * @param {AbstractDocumentPage} [cadOverlay]     - cad Overlay (if defined)
		 * @param {Function}             callback         - callback to be called whe
		 * @param {Object}               result           - result returned from parameters.jsp
		 */
		WebCenterDataAdapter.prototype._handleGraphicsParametersResult = function (updateToken, currentPage, differencePage, cadOverlay, callback, res) {
			if (!this.isUpdateTokenValid(updateToken)) {
				this.f.debug.log("_fetchParameters returned dirty result");
			} else {
				// the update token is still valid, we can handle the parameters result
				var currentPageParams;

				if (currentPage.hasSvg()) {
					currentPageParams = this._getSVGParameters(currentPage, res);
				} else {
					currentPageParams = {
						resolutionx: ViewerUtils.parseFloat(res.status.parameters.resolutionx),
						resolutiony: ViewerUtils.parseFloat(res.status.parameters.resolutiony),
						sizex: ViewerUtils.parseFloat(res.status.parameters.sizex),
						sizey: ViewerUtils.parseFloat(res.status.parameters.sizey),
						fileminx: ViewerUtils.parseFloat(res.status.parameters.fileminx),
						fileminy: ViewerUtils.parseFloat(res.status.parameters.fileminy),
						filemaxx: ViewerUtils.parseFloat(res.status.parameters.filemaxx),
						filemaxy: ViewerUtils.parseFloat(res.status.parameters.filemaxy),
						filemedminx: ViewerUtils.parseFloat(res.status.parameters.filemedminx),
						filemedminy: ViewerUtils.parseFloat(res.status.parameters.filemedminy),
						filemedmaxx: ViewerUtils.parseFloat(res.status.parameters.filemedmaxx),
						filemedmaxy: ViewerUtils.parseFloat(res.status.parameters.filemedmaxy),
						imagetype: ViewerUtils.parseFloat(res.status.parameters.imagetype)
					};
				}

				if (cadOverlay) {
					cadOverlay.setParameters({
						resolutionx: ViewerUtils.parseFloat(res.status.parameters.resolutionx),
						resolutiony: ViewerUtils.parseFloat(res.status.parameters.resolutiony),
						sizex: ViewerUtils.parseFloat(res.status.parameters.sizex),
						sizey: ViewerUtils.parseFloat(res.status.parameters.sizey),
						imagetype: -1
					});
				}
				var diffPageParams = {
					resolutionx: ViewerUtils.parseFloat(res.status.parameters.diffresx),
					resolutiony: ViewerUtils.parseFloat(res.status.parameters.diffresy),
					sizex: ViewerUtils.parseFloat(res.status.parameters.sizex),
					sizey: ViewerUtils.parseFloat(res.status.parameters.sizey),
					fileminx: ViewerUtils.parseFloat(res.status.parameters.diffminx),
					fileminy: ViewerUtils.parseFloat(res.status.parameters.diffminy),
					filemaxx: ViewerUtils.parseFloat(res.status.parameters.diffmaxx),
					filemaxy: ViewerUtils.parseFloat(res.status.parameters.diffmaxy),
					filemedminx: ViewerUtils.parseFloat(res.status.parameters.diffmedminx),
					filemedminy: ViewerUtils.parseFloat(res.status.parameters.diffmedminy),
					filemedmaxx: ViewerUtils.parseFloat(res.status.parameters.diffmedmaxx),
					filemedmaxy: ViewerUtils.parseFloat(res.status.parameters.diffmedmaxy)
				};

				currentPageParams.channel = [];
				diffPageParams.channel = [];

				// now automatic channel matching will be done (this is based on the information we get from the viewserver)
				// build for both the current and difference page a list of (ordered) matched channels
				// currentMatched[i] is matched to diffMatched[i]
				var currentPageMatchedChannels = [];
				var diffPageMatchedChannels = [];

				if (res.status.parameters.channel !== undefined && !_.isArray(res.status.parameters.channel)) {
					res.status.parameters.channel = [res.status.parameters.channel];
				}

				_.each(res.status.parameters.channel, _.bind(function (channel, index) {
					channel.serverID = index;

					// parse the dmode
					if (channel.dmode !== undefined || channel.dmode !== null) {
						channel.dmode = parseInt(channel.dmode, 10);
						if (isNaN(channel.dmode)) {
							channel.dmode = null;
						}
					} else {
						// no dmode is set
						channel.dmode = null;
					}

					// convert 0/1 into boolean
					channel.opaque = channel.opaque ? true : false;

					if (channel.dmode === null ||
						channel.dmode === GraphicsConstants.ChannelDiffMode.CURRENT ||
						channel.dmode === GraphicsConstants.ChannelDiffMode.IN_CURRENT_AND_MATCHED_WITH_PREVIOUS) {

						currentPageParams.channel.push(channel);

						if (channel.dmode === GraphicsConstants.ChannelDiffMode.IN_CURRENT_AND_MATCHED_WITH_PREVIOUS) {
							// the channel is in the current document and matched with difference channel add to the matches list
							currentPageMatchedChannels.push(channel);
						}
					} else if (channel.dmode === GraphicsConstants.ChannelDiffMode.PREVIOUS ||
						channel.dmode === GraphicsConstants.ChannelDiffMode.IN_PREVIOUS_AND_MATCHED_WITH_CURRENT) {

						diffPageParams.channel.push(channel);

						if (channel.dmode === GraphicsConstants.ChannelDiffMode.IN_PREVIOUS_AND_MATCHED_WITH_CURRENT) {
							// the channel is in the difference document and matched with a channel from current document add to matches list
							diffPageMatchedChannels.push(channel);
						}
					}
				}, this));

				// link the current page channel to the matched difference page channel 
				// (this link is bi-directional: the matched difference page channel is also linked to the current page channel)
				for (var i = 0; i < currentPageMatchedChannels.length; i++) {
					currentPageMatchedChannels[i].matchedWith = diffPageMatchedChannels[i].serverID;
					diffPageMatchedChannels[i].matchedWith = currentPageMatchedChannels[i].serverID;
				}

				var hasUserAnnotations;
				var hasGVAnnotations;
				var hasAnnotationsObj;
				if (differencePage !== undefined) {
					hasUserAnnotations = res.status.parameters.annotations.diffexists == 1;
					hasGVAnnotations = res.status.parameters.annotations.diffexistsext == 1;
					hasAnnotationsObj = this._createHasAnnotationsObj(hasGVAnnotations, hasUserAnnotations);

					differencePage.setParameters(diffPageParams);
					differencePage.setHasAnnotations(hasUserAnnotations || hasGVAnnotations, hasAnnotationsObj);
				}
				if (currentPage !== undefined) {
					hasUserAnnotations = res.status.parameters.annotations.exists == 1;
					hasGVAnnotations = res.status.parameters.annotations.existsext == 1;
					hasAnnotationsObj = this._createHasAnnotationsObj(hasGVAnnotations, hasUserAnnotations);

					currentPage.setHasAnnotations(hasUserAnnotations || hasGVAnnotations, hasAnnotationsObj);
					currentPage.setParameters(currentPageParams);
				}


				if (callback) {
					callback();
				}
			}
		};

		/**
		 * Creates the object to pass to setHasAnnotations
		 * @return {Object} 
		 */
		WebCenterDataAdapter.prototype._createHasAnnotationsObj = function (hasGVAnnotations, hasUserAnnotations) {
			var hasAnnotationsObj = {};
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.user.id] = hasUserAnnotations;

			// These are all considered the same as they are in the same file
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.barcode.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.braille.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.artwork.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.spelling.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.text.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.editor.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.ext.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.acrobat.id] = hasGVAnnotations;
			return hasAnnotationsObj;
		};

		/**
		 * Fetch document parameters with the getdocumentinfo.jsp-call, used for fetching basic information of the doucment page
		 * for blank pages this is the only information that's available.
		 * @param {WebCenterDocumentPage} documentPage
		 * @param {Function} callback
		 */
		WebCenterDataAdapter.prototype._fetchPageInfo = function (documentPage, callback) {
			var url = this.WebCenter("getdocumentinfo.jsp", {
				docid: documentPage.getDocument().getDocumentID(),
				docversionid: documentPage.getDocumentVersion().getDocumentVersionID(),
				pageno: documentPage.getPagenum()
			});
			this.f.ajax(url, {
				success: _.bind(function (data) {
					data = XMLUtils.xml2obj(data);
					var pageData = data.status.page;

					if (pageData !== undefined) {
						documentPage.set("margbot", data.status.page.margbot);
						documentPage.set("margtop", data.status.page.margtop);
						documentPage.set("margleft", data.status.page.margleft);
						documentPage.set("margright", data.status.page.margright);
						documentPage.set("hsize", data.status.page.hsize);
						documentPage.set("vsize", data.status.page.vsize);
						documentPage.set("hasCorrectMargin", true);
						documentPage.set("pageno", data.status.page.pageno);
						documentPage.set("offsetx", data.status.page.offsetx);
						documentPage.set("offsety", data.status.page.offsety);
						if (data.status.page.cadregedit !== undefined) {
							documentPage.set("cadregedit", data.status.page.cadregedit);
							documentPage.set("isCadAlignmentSaved", data.status.page.saved == 1 ? true : false);
						}

						if (pageData.metadata && (pageData.metadata.design || pageData.metadata.layout)) {
							var cadData = pageData.metadata.design || pageData.metadata.layout;
							documentPage.set("grainDirection", cadData.grain_direction);
							documentPage.set("defaultSide", cadData.default_side);
							documentPage.set("dieSide", cadData.die_side);
						}
						documentPage.set("has3DPerspective", pageData.has3DPerspective == "1" ? true : false);
						documentPage.set("canGenerate3D", pageData.canGenerate3D == "1" ? true : false);

						documentPage.set("canEditAttributes", pageData.can_edit_attributes);
						documentPage.set("attributeCategoryID", pageData.attribute_category_id);

						documentPage.set("canChangeAnnotations", pageData.can_change_annotations);

					}

					if (documentPage.isBlankPage()) {
						var pageParams = {
							sizex: ViewerUtils.parseFloat(pageData.hsize),
							sizey: ViewerUtils.parseFloat(pageData.vsize),
							filemaxx: ViewerUtils.parseFloat(pageData.hsize),
							filemaxy: ViewerUtils.parseFloat(pageData.vsize)
						};
						documentPage.setHasAnnotations(false);
						documentPage.setParameters(pageParams);
					}
					if (callback) {
						callback(data);
					}
				}, this),
				error: _.bind(function (jqXHR, textStatus, errorThrown) {
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("LoadingDocumentInfoError"), jqXHR, textStatus, errorThrown);
					}, this);
					if (callback) {
						callback();
					}
				}, this)
			});
		};

		/**
		 * Makes a deep copy of a page, this means it reinitializes the document and document version. All parameters and data of the document,
		 * document version and document page are copied to the new document, document version and document page
		 *
		 * @param {WebCenterDocumentPage} page - to page thats needs to be copied
		 * @param {WebCenterDocument} [doc] - if no new document needs to be made, add the copied page to this document. If this parameter is undefinied a deep copy of the document will be made
		 * @param {WebCenterDocumentVersion} [docVersion] - if undefined the document version will be copied, if not undefined the copied document page will be added to this document version
		 * @param {Number} [documentID] - the new ID for the document, if not defined same id as document will be used
		 * @returns {WebCenterDocumentPage}
		 */
		WebCenterDataAdapter.prototype.getDeepCopy = function (page, doc, docVersion, documentID) {
			var oldDocumentVersion = page._docVersion; //get the old document 
			var oldDocument = oldDocumentVersion._doc; //get the old documentVersion
			var document = doc;
			if (documentID === undefined) {
				documentID = oldDocument.documentID;
			}
			if (document === undefined) {
				//make a new WebCenterDocument, use the parameters of the old document, dont specify documentData to prevent it making the version and page automatically
				document = new WebCenterDocument(this.f, undefined, documentID, page.documentType, this);
				document._data = $.extend(true, {}, oldDocument._data); //copy all the data of the old document to the new document
			}
			var documentVersion = docVersion;
			if (documentVersion === undefined) {
				//make a new document version, doesn't use any getters because it can be used in new window and not possible to access prototype-methods (internet explorer)
				documentVersion = new WebCenterDocumentVersion(this.f, document, oldDocumentVersion._versionData.docversion, {
					'docversionid': oldDocumentVersion._versionData.docversionid,
					'docRevision': oldDocumentVersion._versionData.docRevision
				}, undefined, this);
			}
			//make a new WebCenterDocumentPage, dont specify data string, this will be copied from the old document page		
			var currentPage = new WebCenterDocumentPage(this.f, documentVersion, page._pagenum, undefined, undefined, this);
			currentPage._data = $.extend(true, {}, page._data); //copy the data of the old document page 
			currentPage._parameters = $.extend(true, {}, page._parameters); //copy the parameters of the old document page
			currentPage._state = $.extend(true, {}, page._state); //copy the state parameters of the old document page

			//copy the CAD data, in case of CAD document page
			if (page._data.cadData) {
				currentPage._data.cadData = _.clone(page._data.cadData);
				if (page._data.cadData.cad) {
					// NOTE: selected layers and such are not copied
					currentPage._data.cadData.cad = new CadFile(page._data.cadData.cad.cstyles, page._data.cadData.cad._xmlData);
				}
			}

			if (currentPage._state.compareOffset) {
				currentPage._state.compareOffset = page._state.compareOffset.copy();
			}
			currentPage.setReaderSpread(page.isReaderSpread());
			//For Graphics+CAD:
			if (docVersion === undefined) {
				document.addDocumentVersion(documentVersion); //add the document version to the document
			}
			documentVersion.addPage(page.getPagenum(), currentPage); //add the page to the document version

			return currentPage;
		};

		/**
		 * Fetches all pages of a document version, callback is used after all pages are initailized and added to the document version.
		 * @param {WebCenterDocumentVersion} docVersionId
		 * @param {Function} callback
		 */
		WebCenterDataAdapter.prototype.getPagesOfDocumentVersion = function (docVersion, callback) {
			var document = docVersion.getDocument();
			var url = this.WebCenter("getdocumentinfo.jsp", {
				docid: document.getDocumentID(),
				docversionid: docVersion.getDocumentVersionID()
			});

			this.f.ajax(url, {
				success: _.bind(function (data) {
					data = XMLUtils.xml2obj(data);
					var pagelist = data.status.page;

					_.each(pagelist, _.bind(_.bind(function (page) {
						document.addDocumentPageFromData(page);
					}, this)));

					if (callback !== undefined) {
						callback();
					}
				}, this),
				error: _.bind(function (jqXHR, textStatus, errorThrown) {
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("LoadingDocumentInfoError"), jqXHR, textStatus, errorThrown);
					}, this);
				}, this)
			});
		};

		/**
		 * Returns the document with the WebCenter docId (not the number but UID of the document that's saved in the data of the document)
		 * @param {String} docId - document id of a WebCenter document (obtained by calling getDocumentID())
		 * @returns {WebCenterDocument}
		 */
		WebCenterDataAdapter.prototype.getDocumentByDocId = function (docId) {
			var doc = _.find(this.f.getDocumentController().getAllDocuments(), function (doc) {
				return doc.getDocumentID() === docId;
			});

			if (doc) {
				return doc;
			} else {
				return _.find(this.f.getDocumentController().getLinkedDocuments(), function (doc) {
					return doc.getDocumentID() === docId;
				});
			}
		};

		/**
		 * Load previous versions of the document for a given document page. If the version does not have the page with given pagenumber the version
		 * will not be loaded (only happens for pagelist documents).
		 * @param {Object} data
		 * 	@example
		 * 	{
		 * 		docid: "document id used by WebCenter",
		 * 		docversionid: "document version id used by WebCenter",
		 * 		diffversion: "difference document version id used by WebCenter",
		 * 		pageno: "number of the page to be used
		 * 	}
		 * @param {WebCenterDocument} document
		 * @param {WebCenterDocumentPage} [documentPage]
		 * @param {Function} callback
		 */
		WebCenterDataAdapter.prototype.loadVersionsOfDocumentPage = function (data, document, callback, documentPage) {
			var url = this.WebCenter("getdocumentinfo.jsp", {
				docid: data.docid,
				docversionid: data.docversionid,
				diffversion: data.diffversion,
				pageno: data.pageNumber
			});
			var pageNumber = data.pageNumber;
			var isPageList = data.isPageList;
			var isMultiPage = data.isMultiPage;

			this.f.ajax(url, {
				success: _.bind(function (data) {
					data = XMLUtils.xml2obj(data);

					if (data.status.page !== undefined) {
						var versionlist = data.status.page.versionlist;

						//make sure to update margins in the data as this is only provided in this documentinfo call
						if (documentPage !== undefined) {
							documentPage.set("margbot", data.status.page.margbot);
							documentPage.set("margtop", data.status.page.margtop);
							documentPage.set("margleft", data.status.page.margleft);
							documentPage.set("margright", data.status.page.margright);
							documentPage.set("hasCorrectMargin", true);
						}

						if (versionlist !== undefined) {
							//create the documentversions from the versionlist:
							if (!_.isArray(versionlist)) { //check if versionlist is an array (more than 2 versions) are object (2 versions)
								versionlist = [versionlist]; //make it an array if it's only one element
							}
							var pages = [];
							_.each(versionlist, _.bind(function (version) {
								//first check if the version already exists, if so set the docversion of this version
								var docVersion = document._documentVersions[parseInt(version.docversion, 10)];
								if (docVersion === undefined) {
									version.isPageList = isPageList;
									version.isMultiPage = isMultiPage;
									docVersion = new WebCenterDocumentVersion(this.f, document, parseInt(version.docversion, 10), version, undefined, this);
									document._documentVersions[parseInt(version.docversion, 10)] = docVersion;
								} else {
									docVersion.set('docversionid', version.docversionid);
									docVersion.set('isMultiPage', version.isMultiPage === 'true');
								}
								//add the page to this documentVersion
								var newPage = new WebCenterDocumentPage(this.f, docVersion, pageNumber, {
									file: version.file,
									versionNumber: version.docversion
								}, undefined, this);
								docVersion.addPage(pageNumber, newPage);
								pages[version.docversion] = newPage;
							}, this));
							callback(pages);
						}
					}
				}, this),
				error: _.bind(function (jqXHR, textStatus, errorThrown) {
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("LoadingDocumentInfoError"), jqXHR, textStatus, errorThrown);
					}, this);
				}, this)
			});
		};


		/**
		 * Save the CAD overlay settings
		 * @param {AbstractDocumentPage} graphicsPage - the graphics page in which the CAD reference should be stored
		 * @param {AbstractDocumentPage} cadOverlay   - the CAD document that should be set as reference
		 * @param {Number}               rotation     - rotation of the CAD overlay (0,90,180,270 degrees)
		 * @param {Boolean}              isFlipped    - is the CAD overlay flipped
		 * @param {ShapeUtils.Point}     offset       - the offset for the CAD overlay
		 * @param {Function}             [callback]   - callback function to be called when CAD overlay is saved (first parameter of the callback is a boolean indicating success)
		 */
		WebCenterDataAdapter.prototype.saveCadOverlay = function (graphicsPage, cadOverlay, rotation, isFlipped, offset, callback) {
			if (graphicsPage.canSaveCadOverlay()) {

				var cadOrientation = (4 - (rotation / 90)) % 4;
				if (isFlipped) {
					cadOrientation += 4;
				}

				var newVersionId;
				if (cadOverlay) {
					newVersionId = cadOverlay.get("docversionid");
				}

				var pageMargins = graphicsPage.getMargins();
				var url = this.WebCenter("setcadalignment.jsp", {
					docversionid: graphicsPage.get("docversionid"),
					cadversionid: newVersionId,
					cadorgx: offset.x,
					cadorgy: offset.y,
					cadorientation: cadOrientation,
					hs: graphicsPage.get("hsize"),
					vs: graphicsPage.get("vsize"),
					ml: pageMargins.left,
					mt: pageMargins.top,
					mr: pageMargins.right,
					mb: pageMargins.bottom
				});

				this.f.ajax(url, {
					dataType: "xml",
					success: _.bind(function (data) {
						var result = XMLUtils.xml2obj(data);
						var error = "";

						if (result) {
							if (result.status && result.status.value === "0") {
								this.f.loadBundle("viewer", function () {
									this.f.emit("userMessage", "success", this.f.t("SaveCadOverlaySuccess"), true);
								}, this);

								var oldCadOverlay = graphicsPage.getLinkedPage();
								if (oldCadOverlay !== this.f.getDocumentController().getCadOverlay()) {
									this.f.getDocumentController().createNewLinkedCadFromOverlay();
								}

								//persist data in the linked CAD file
								var newLinkedCad = this.f.getDocumentController().getCadOverlay();
								if (newLinkedCad) {
									newLinkedCad.set("offsetx", offset.x);
									newLinkedCad.set("offsety", offset.y);
									newLinkedCad.set("overlayRotation", rotation);
									newLinkedCad.set("overlayIsFlipped", isFlipped);
								} else {
									this.updateDocumentData(graphicsPage, undefined, true);
								}

								if (callback) {
									callback(true);
								}

								return;
							} else if (result.status === "-1") {
								error = result.status.message;
							}
						}

						// something went wrong...
						this.f.emit("userMessage", "warn", this.f.t("SaveCadOverlayFailed") + " " + error);
						if (callback) {
							callback(false);
						}

					}, this),
					error: _.bind(function () {
						this.f.loadBundle("viewer", function () {
							this.f.emit("userMessage", "warn", this.f.t("SaveCadOverlayFailed"));
						}, this);

						if (callback) {
							callback(false);
						}
					}, this)
				});
			} else {
				this.f.emit("userMessage", "warn", this.f.t("SaveCadOverlayFailed"));
				if (callback) {
					callback(false);
				}
			}
		};

		/**
		 * Show the 3D Collada viewer in a new browser tab
		 * @param {WebCenterDocumentPage} page - Graphics page to show the 3D Viewer for
		 */
		WebCenterDataAdapter.prototype.show3DView = function (page) {
			var url = this.WebCenter("doctypecolladaview.jsp", {
				'docVerID': page.get("docversionid"),
				'sourcePage': "javascript:window.close()"
			});
			window.open(url, '_blank');
		};

		/**
		 * Update the data of the document, also updates the document reference info
		 * @param {AbstractDocumentPage}   page
		 * @param {Function} callback - the callback that is executed when the document data is correctly fetched
		 * @param {Boolean} [doNotActivate] - do not activate any newly created CAD reference document pages
		 */
		WebCenterDataAdapter.prototype.updateDocumentData = function (page, callback, doNotActivate) {
			this._fetchPageInfo(page, _.bind(function (data) {
				if (data.status.page && data.status.page.file && data.status.page.cad && page === this.f.getActivePage()) {
					data = data.status.page; //get the data of the page into the data object
					var linkedCadId = data.cadid; //get the document id of the linked cad
					var oldCadLink = page.getLinkedPage(); //get the old linked CAD page
					data.docid = data.cadid;
					data.docversionid = data.cadversionid;
					if (!this.getDocumentByDocId(linkedCadId)) { //if the CAD document doesn't exist yet      
						//create a new document from the cad info
						var cadDocument = new WebCenterDocument(this.f, data, this.f.getDocumentController().getNextIID(), DataConstants.documentPageType.CAD, this);
						//add this new document as link the currently active document page
						this.f.getDocumentController().addNewLinkedDocument(cadDocument, doNotActivate);
					} else { //the CAD already exists in the system, create a new link to it
						var newCadLink = this.getDocumentByDocId(linkedCadId).getDocumentVersion().getPage();
						if (newCadLink !== oldCadLink) {
							this.f.getDocumentController().createNewLinkedCadFromOverlay(doNotActivate, newCadLink);
						}
						var cadOverlayPage = page.getLinkedPage();
						if (cadOverlayPage) {
							cadOverlayPage.setPageAlignmentData(data);
						}
					}
				} else if (page.getDocument().hasLinkedDocument()) {
					page.getDocument().setLinkedDocumentPage();
					this.f.getDocumentController().changeCadOverlay();
				}

				if (callback) {
					callback();
				}
			}, this));
		};

		return WebCenterDataAdapter;

	});
