define(["web-viewer/data/AbstractDocument",
		"web-viewer/data/DataConstants",
		"underscore",
		"web-viewer/webcenter/WebCenterDocumentVersion",
		"web-viewer/webcenter/WebCenterDocumentPage",
		"utils/XMLUtils"
	],
	function (
		AbstractDocument,
		DataConstants,
		_,
		WebCenterDocumentVersion,
		WebCenterDocumentPage,
		XMLUtils
	) {
		"use strict";

		/**
		 * @class Specific implementation of a document for WebCenter.
		 * @exports WebCenterDocument
		 * @augments Data.AbstractDocument
		 * @memberof WebCenter
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {Object} documentData - information of the document
		 * @param {Number} IID - id of the document
		 * @param {DataConstants.documentPageType} type
		 * @param {AbstractAdapter} adapter -
		 */
		var WebCenterDocument = function (f, documentData, IID, type, adapter) {
			AbstractDocument.call(this, f, IID, type);

			this.f = f;

			this._adapter = adapter;

			if (documentData !== undefined) {
				this._initializeWithDocInfo(documentData);
			}
		};

		WebCenterDocument.prototype = Object.create(AbstractDocument.prototype);
		WebCenterDocument.prototype.constructor = WebCenterDocument;

		/**
		 * Use a data string to initialize a document page and add it to the correct documentVersion
		 * @param {String} pageData
		 */
		WebCenterDocument.prototype.addDocumentPageFromData = function (pageData) {
			//first fetch the correct version from data where the page should be added to
			var documentVersion = this.getDocumentVersion(pageData.docversion);
			if (documentVersion !== undefined && pageData.docversionid === documentVersion._versionData.docversionid) { //make sure the document version exists
				documentVersion.addDocumentPageFromData(pageData);
			} else if (pageData.isMultiPage && pageData.pageindex !== undefined) { //for pagelist the docversion isn't always correct
				documentVersion = this.getDocumentVersion(); //get the latest document version for this document
				documentVersion.addDocumentPageFromData(pageData); //add the document page to the latest version
				//if the page data added has a higher version number change the version data in the document version
				if (pageData.docversion > documentVersion.getVersionId()) {
					var previousId = documentVersion.getVersionId();
					documentVersion._versionData.docversionid = pageData.docversionid;
					documentVersion._versionData.docversion = pageData.docversion;
					documentVersion._versionId = pageData.docversion;
					this._documentVersions[pageData.docversion] = documentVersion;
					delete this._documentVersions[previousId];
				} else if (pageData.docversion === documentVersion.getVersionId() && pageData.docversionid > documentVersion._versionData.docversionid) {
					documentVersion._versionData.docversionid = pageData.docversionid;
					this._documentVersions[pageData.docversion] = documentVersion;
				}
			}
			if (pageData.diff !== undefined) { //a default reference is supplied for this page, (the page with same pagenumber of a previous version)
				var diffDocumentVersion = this.getDocumentVersion(pageData.diffversion);
				if (diffDocumentVersion === undefined) { //if not defined make a temp document version and fill the docversionid in later
					diffDocumentVersion = new WebCenterDocumentVersion(this.f, this, pageData.diffversion, {}, undefined, this._adapter);
					this.addDocumentVersion(diffDocumentVersion);
				}
				var diffDocumentPage = new WebCenterDocumentPage(this.f, diffDocumentVersion, pageData.pageno, {
					file: pageData.diff
				}, undefined, this._adapter);
				diffDocumentVersion.addPage(pageData.pageno, diffDocumentPage);
			}
		};

		/**
		 * The viewer is initialized with document info provided
		 * @private
		 * @param {Object} data
		 */
		WebCenterDocument.prototype._initializeWithDocInfo = function (data) {
			this.set("docid", data.docid);

			this.set("projectid", data.projectid);
			this.set("projectname", data.projectname + "");
			this.set("binding", data.binding);

			if (this._documentType === DataConstants.documentPageType.GRAPHICS  || this._documentType === DataConstants.documentPageType.SVG) {
				//first add the current version
				var currentVersion = {};
				currentVersion.docversionid = data.docversionid;
				this.set("docname", data.docname + "");
				this.set("projectname", data.projectname + "");
				currentVersion.file = data.file;
				currentVersion.isPageList = data.pageindex !== undefined;
				currentVersion.docrevision = data.docrevision;
				var docVersion = new WebCenterDocumentVersion(this.f, this, data.docversion, currentVersion, data, this._adapter);
				this.addDocumentVersion(docVersion);

				if (data.cadversionid) {
					// this document has a linked cad, store this ID so we can decide on showing the compare controls or not...
					this.set("linkedCADVersionID", data.cadversionid);
				}

				if (data.isMultiPage && data.versionlist === undefined && data.pageindex === undefined) { //should only happen for multipage
					//first add a documentVersion for each version before calling the viewserver for info
					for (var i = 1; i < parseInt(data.docversion, 10); i++) { //only versions besides the current version
						var tempVersion = new WebCenterDocumentVersion(this.f, this, i, {
							isMultiPage: data.isMultiPage
						}, undefined, this._adapter);
						this.addDocumentVersion(tempVersion);
					}

					if (data.diff !== undefined) { //default reference page for this page (page with same page number of a previous version)
						var diffDocumentVersion = this.getDocumentVersion(data.diffversion);
						var diffDocumentPage = new WebCenterDocumentPage(this.f, diffDocumentVersion, data.pageno, {
							file: data.diff
						}, undefined, this._adapter);
						diffDocumentVersion.addPage(data.pageno, diffDocumentPage);
					}

					//get the versionlist with getdocumentinfo, this is not supplied in the data string
					data.pageNumber = 1;
					this._adapter.loadVersionsOfDocumentPage(data, this, _.bind(function () {
						this.f.getDocumentController().loadedDocumentChanged();
					}, this));
				}

				//multiple versions, make multiple AbstractDocumentVersions
				if (data.versionlist !== undefined) {
					if (!_.isArray(data.versionlist)) { //check if versionlist is an array (more than 2 versions) are object (2 versions)
						data.versionlist = [data.versionlist]; //make it an array if it's only one element
					}
					_.each(data.versionlist, _.bind(function (version) {
						var docVersion = new WebCenterDocumentVersion(this.f, this, version.docversion, version, undefined, this._adapter);
						this.addDocumentVersion(docVersion);
					}, this));
				}
			} else if (this._documentType === DataConstants.documentPageType.CAD) { //the file is CAD
				if (data.caddocname) {
					this.set("docname", data.caddocname + "");
				} else {
					this.set("docname", data.docname + "");
				}
				//multiple versions, make multiple AbstractDocumentVersions
				if (data.cadversionlist !== undefined) { //if CAD with graphics, the cad versionlist is defined in cadversionlist
					if (!_.isArray(data.cadversionlist)) { //check if cadversionlist is an array
						data.cadversionlist = [data.cadversionlist];
					}
					_.each(data.cadversionlist, _.bind(function (version) {
						var docVersion = new WebCenterDocumentVersion(this.f, this, version.docversion, version, data, this._adapter);
						this.addDocumentVersion(docVersion);
					}, this));
				} else if (data.versionlist !== undefined) {
					if (!_.isArray(data.versionlist)) { //check if versionlist is an array
						data.versionlist = [data.versionlist];
					}
					_.each(data.versionlist, _.bind(function (version) {
						var docVersion = new WebCenterDocumentVersion(this.f, this, version.docversion, version, data, this._adapter);
						this.addDocumentVersion(docVersion);
					}, this));
				}
			}
		};

		/**
		 * Loads document pages of previous versions with he same document page number
		 * @param {WebCenterDocumentPage} documentPage
		 * @param {Function} callback
		 */
		WebCenterDocument.prototype.loadVersionsForDocumentPage = function (documentPage, callback) {
			var data = {
				docid: documentPage.getDocument().getDocumentID(),
				docversionid: documentPage.getDocumentVersion().getDocumentVersionID(),
				diffversion: null,
				pageNumber: documentPage.getPagenum(),
				isPageList: documentPage.getDocumentVersion().isPageList()
			};
			this._adapter.loadVersionsOfDocumentPage(data, this, callback, documentPage);
		};


		/**
		 * Get the WebCenter docID of the document
		 * @returns {String}
		 */
		WebCenterDocument.prototype.getDocumentID = function () {
			return this.get("docid");
		};

		/**
		 * Get the WebCenter Project ID of the document
		 * @returns {String}
		 */
		WebCenterDocument.prototype.getProjectID = function () {
			return this.get("projectid");
		};

		return WebCenterDocument;
	});