define(["web-viewer/data/AbstractDocumentPage",
		"utils/XMLUtils",
		"web-viewer/utils/ViewerUtils",
		"utils/ShapeUtils",
		"web-viewer/data/DataConstants",
		"web-viewer/viewControls/ViewControlsConstants",
		"underscore"
	],
	function (AbstractDocumentPage,
		XMLUtils,
		ViewerUtils,
		ShapeUtils,
		DataConstants,
		ViewControlsConstants,
		_) {
		"use strict";

		/**
		 * @exports WebCenterDocumentPage
		 * @augments Data.AbstractDocumentPage
		 * @memberof WebCenter
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {AbstractDocumentVersion} docVersion
		 * @param {Number} pagenum
		 * @param {Object} info
		 * @param {AbstractDataAdapter} adapter
		 */
		var WebCenterDocumentPage = function (f, docVersion, pagenum, fileInfo, info, adapter) {
			AbstractDocumentPage.call(this, f, docVersion, pagenum);

			/**
			 * @private
			 */
			this._adapter = adapter;

			//check what type the document is before initializing.
			this.documentType = this.getDocument().getDocumentType();

			this._pageVersionId = null;

			if (info !== undefined && info.pageindex !== undefined) { //only for pagelist documents 
				this._pageVersionId = info.docversion;
			}

			if (info !== undefined && info.blank !== undefined) {
				this._isBlankPage = true;
			}


			if (info !== undefined) {
				this.set("canChangeAnnotations", info.can_change_annotations);
				this.set("canEditAttributes", info.can_edit_attributes);
				this.set("attributeCategoryID", info.attribute_category_id);
			}

			if ((this.documentType === DataConstants.documentPageType.GRAPHICS || this.documentType === DataConstants.documentPageType.SVG) && fileInfo !== undefined && info !== undefined) {
				//set the correct data coming from the server
				this.set("hasDefaultData", info.hasDefaultData);
				this.set("margbot", info.margbot);
				this.set("margtop", info.margtop);
				this.set("margleft", info.margleft);
				this.set("margright", info.margright);
				this.set("vsize", info.vsize);
				this.set("hsize", info.hsize);
				this.set("hasCorrectMargin", !docVersion.isPageList());
				this.set("docversionid", fileInfo.docversionid);
				this.set("pageDocversionid", info.docversionid);
				if (fileInfo.file === undefined) {
					this.set("file", info.file);
				} else {
					this.set("file", fileInfo.file);
				}
				this.set("offsetx", info.offsetx);
				this.set("offsety", info.offsety);
				this.set("pageindex", info.pageindex);
				this.set("pagename", info.pagename);
				this.set("hscale", info.hscale);
				this.set("vscale", info.vscale);
				this.set("hdist", info.hdist);
				this.set("vdist", info.vdist);

				this.set("has3DPerspective", fileInfo.has3DPerspective == "1" ? true : false);
				this.set("canGenerate3D", fileInfo.canGenerate3D == "1" ? true : false);

				this.set("document_approval_code", info.document_approval_code);
				this.set("user_approval_code", info.user_approval_code);
				if (info.cadregedit !== undefined) {
					this.set("cadregedit", info.cadregedit);
					this.set("isCadAlignmentSaved", info.saved == 1 ? true : false);
				}
			} else if (this.documentType === DataConstants.documentPageType.CAD && fileInfo !== undefined && info !== undefined) {
				this.set("cad", fileInfo.file);
				this.set("pageDocversionid", info.docversionid);
				if (_.isArray(info.style)) {
					this.set("style", info.style[0]);
				} else {
					this.set("style", info.style);
				}
				if (info.metadata && (info.metadata.design || info.metadata.layout)) {
					var cadData = info.metadata.design || info.metadata.layout;
					this.set("grainDirection", cadData.grain_direction);
					this.set("defaultSide", cadData.default_side);
					this.set("dieSide", cadData.die_side);
				}
				//for Graphics + CAD
				this.setPageAlignmentData(info);
				this.set("docversionid", fileInfo.docversionid);
				this.set("has3DPerspective", fileInfo.has3DPerspective == "1" ? true : false);
				this.set("canGenerate3D", fileInfo.canGenerate3D == "1" ? true : false);

			} else if (fileInfo !== undefined && fileInfo.file !== undefined) { //for pages with only the file as data (multipage or pagelist)
				this.set("file", fileInfo.file);
				this._pageVersionId = fileInfo.docversion;
				this.set("hasCorrectMargin", false);
			}

			if (info !== undefined) {
				this.set("diffversion", info.diffversion ? true : false);
			}
		};

		WebCenterDocumentPage.prototype = Object.create(AbstractDocumentPage.prototype);
		WebCenterDocumentPage.prototype.constructor = WebCenterDocumentPage;

		/**
		 * Set the alignment data for this page
		 * @param {object} alignData
		 * @example
		 *  {
		 *      offsetx: ,
		 *      offsety: ,
		 *      orientation:
		 *  }
		 */
		WebCenterDocumentPage.prototype.setPageAlignmentData = function (alignData) {
			this.set("offsetx", alignData.offsetx);
			this.set("offsety", alignData.offsety);
			if (ViewerUtils.isCloseTo(this._data.offsetx, 0, 7)) { //viewserver returns other values for close to 0
				this.set("offsetx", 0); //set to 0 if close to 0
			}
			if (ViewerUtils.isCloseTo(this._data.offsety, 0, 7)) {
				this.set("offsety", 0);
			}
			// parse orientation info
			// ViewServer uses a bitmask: 
			//      first two bits: rotation (0=0, 1=270, 2=180, 3=180)
			//      third bit: flipped
			if (alignData.orientation !== undefined) {
				this.set("overlayIsFlipped", (+alignData.orientation & 4) > 0);
				this.set("overlayRotation", ((4 - (alignData.orientation % 4)) * 90) % 360);
			}
		};

		/**
		 * Get the margin of the document
		 * @returns {Object} margins{bottom, top, left, right}
		 */
		WebCenterDocumentPage.prototype.getMargins = function () {
			var margins = {};

			margins.bottom = ViewerUtils.parseFloat(this._data['margbot'] ? this._data['margbot'] : 0);
			margins.top = ViewerUtils.parseFloat(this._data['margtop'] ? this._data['margtop'] : 0);
			margins.left = ViewerUtils.parseFloat(this._data['margleft'] ? this._data['margleft'] : 0);
			margins.right = ViewerUtils.parseFloat(this._data['margright'] ? this._data['margright'] : 0);

			return margins;
		};

		/**
		 * Get the origin of the document page, calculates this from the trimbox origin and the margins
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterDocumentPage.prototype.getDocumentPageOrigin = function () {
			var rotation = this.getRotation() % 360;
			var isFlipped = this.isFlipped();
			var margin = this.getMargins();

			var left = this.getParam("fileminx");
			var top = this.getParam("fileminy");
			if (rotation === 90) {
				left -= margin.bottom;
				if (isFlipped) {
					top -= margin.right;
				} else {
					top -= margin.left;
				}

			} else if (rotation === 180) {
				top -= margin.bottom;
				if (isFlipped) {
					left -= margin.left;
				} else {
					left -= margin.right;
				}
			} else if (rotation === 270) {
				left -= margin.top;
				if (isFlipped) {
					top -= margin.left;
				} else {
					top -= margin.right;
				}
			} else {
				top -= margin.top;
				if (isFlipped) {
					left -= margin.right;
				} else {
					left -= margin.left;
				}
			}
			return new ShapeUtils.Point(left, top);
		};

		/**
		 * get the trimbox rectangle for the page
		 * @returns {ShapeUtils.Rect}
		 */
		WebCenterDocumentPage.prototype.getTrimboxRect = function () {
			var origin, size;
			if (this.isBlankPage()) {
				var margins = this.getMargins();
				origin = new ShapeUtils.Point(margins.left, margins.top);
				size = new ShapeUtils.Size(ViewerUtils.parseFloat(this.get("hsize")), ViewerUtils.parseFloat(this.get("vsize")));
			} else {
				origin = new ShapeUtils.Point(ViewerUtils.parseFloat(this.getParam("fileminx")), ViewerUtils.parseFloat(this.getParam("fileminy")));
				size = new ShapeUtils.Size(ViewerUtils.parseFloat(this.getParam("filemaxx")) - origin.x, ViewerUtils.parseFloat(this.getParam("filemaxy")) - origin.y);
			}
			return new ShapeUtils.Rect(origin, size);
		};

		/**
		 * Return the size of the document page
		 * @returns {ShapeUtils.Size}
		 */
		WebCenterDocumentPage.prototype.getPageSize = function () {
			var margins = this.getMargins();

			if (this.hasGraphics()) {
				// graphics document
				if (Math.abs(ViewerUtils.parseFloat(this.get("hsize"))) > 0.000000001) { // float, can't compare safely with 0
					var graphicsSize = new ShapeUtils.Size(ViewerUtils.parseFloat(this.get("hsize")) + margins.left + margins.right, ViewerUtils.parseFloat(this.get("vsize")) + margins.bottom + margins.top);
					var scaling = this.getCurrentScaling();
					if (scaling) {
						graphicsSize.width *= scaling.x;
						graphicsSize.height *= scaling.y;
					}

					return graphicsSize;
				} else {
					// hsize and vsize are not defined in the metadata... return data we got from the parameters
					return new ShapeUtils.Size(ViewerUtils.parseFloat(this.getParam("sizex")), ViewerUtils.parseFloat(this.getParam("sizey")));
				}

			} else if (this.isBlankPage()) {
				// blank page
				if (this.isReaderSpread()) { //calculation of size is different for when in readerspread
					var margins = this.getMargins();
					var trimboxSize = this.getTrimboxSize();
					return new ShapeUtils.Size(trimboxSize.width + margins.left + margins.right, trimboxSize.height + margins.top + margins.bottom);
				} else {
					return new ShapeUtils.Size(this.getParam("filemaxx") + margins.left + margins.right, this.getParam("filemaxy") + margins.top + margins.bottom);
				}
			} else if (this.hasCad()) {
				// CAD
				if (this.isCadAvailable && this.isCadAvailable()) {
					return this.getCad().getSize();
				} else {
					// CAD is not yet available
					return new ShapeUtils.Size();
				}
			} else if (this.hasSvg()) {
				var size = this.get("SVGSize");
				if (size) {
					return size;
				} else {
					return new ShapeUtils.Size();
				}
			}
		};

		/**
		 * Get the mediabox of this page. For blank pages this becomes 0.
		 * @returns {ShapeUtils.Rect}
		 */
		WebCenterDocumentPage.prototype.getMediaBoxRect = function () {
			var origin = new ShapeUtils.Point(ViewerUtils.parseFloat(this.getParam("filemedminx")), ViewerUtils.parseFloat(this.getParam("filemedminy")));
			var size = new ShapeUtils.Size(ViewerUtils.parseFloat(this.getParam("filemedmaxx")) - origin.x, ViewerUtils.parseFloat(this.getParam("filemedmaxy")) - origin.y);
			return new ShapeUtils.Rect(origin, size);
		};

		/**
		 * Get the current scaling of the document page, takes the compensation state (dist or scale or both) into account, returns null if no compensation present
		 * @returns {ShapeUtils.Point|null}
		 */
		WebCenterDocumentPage.prototype.getCurrentScaling = function () {
			var yComp = 1;
			var xComp = 1;
			if (this.getStateParam("compensateDist") === ViewControlsConstants.compensateDist.DISTONLY || this.getStateParam("compensateDist") === ViewControlsConstants.compensateDist.DISTANDSCALE) {
				yComp = yComp * 1 / (ViewerUtils.parseFloat(this.get("vdist")));
				xComp = xComp * 1 / (ViewerUtils.parseFloat(this.get("hdist")));
			}
			if (this.getStateParam("compensateDist") === ViewControlsConstants.compensateDist.SCALEONLY || this.getStateParam("compensateDist") === ViewControlsConstants.compensateDist.DISTANDSCALE) {
				yComp = yComp * 1 / (ViewerUtils.parseFloat(this.get("vscale")));
				xComp = xComp * 1 / (ViewerUtils.parseFloat(this.get("hscale")));
			}

			if (yComp === 1 && xComp === 1) {
				return null;
			} else {
				return new ShapeUtils.Point(xComp, yComp);
			}
		};

		/**
		 * @override
		 */
		WebCenterDocumentPage.prototype.getInfoURL = function (params) {
			return this._adapter.ViewServer("info.jsp", params);
		};

		/**
		 * Is the rectangle on the page, only for part on the page returns true
		 * @param {ShapeUtils.Rect} rect
		 * @returns {Boolean}
		 */
		WebCenterDocumentPage.prototype.isRectOnPage = function (rect) {
			var margin = this.getMargins();
			var pageSize = this.getPageSize();
			var pageRect = new ShapeUtils.Rect(new ShapeUtils.Point(0, 0), pageSize);
			pageRect.origin.offset(new ShapeUtils.Size(-margin.left, -margin.bottom));
			return pageRect.rectIntersect(rect);
		};

		/**
		 * Return the version Id for this page (used for page list document)
		 * @returns {Number} - version id for this page
		 */
		WebCenterDocumentPage.prototype.getPageVersionId = function () {
			return this._pageVersionId;
		};

		/**
		 * Check if the page is a digital film
		 * @returns {Boolean}
		 * @override
		 */
		WebCenterDocumentPage.prototype.isDigitalFilmPage = function () {
			// imagetype parameter is 0 for digital film and CAD, 1 for graphics
			return this.hasGraphics() && this.getParam("imagetype") === DataConstants.imageType.OTHER;
		};

		/**
		 * get the size of the trimbox, return null if the trimbox size is not known by the page (need to use margins)
		 * @return {ShapeUtils.Size}
		 */
		WebCenterDocumentPage.prototype.getTrimboxSize = function () {
			var hsize = ViewerUtils.parseFloat(this.get("hsize"));
			var vsize = ViewerUtils.parseFloat(this.get("vsize"));
			if (!hsize || !vsize) { // undefined or 0
				return new ShapeUtils.Size();
			} else {
				return new ShapeUtils.Size(hsize, vsize);
			}
		};

		/**
		 * Get the url for fetching tumbnails (50x50)
		 * @returns {String} - url for fetching the thumbdata of the page
		 */
		WebCenterDocumentPage.prototype.getThumbnailUrl = function () {
			var options;
			if (this.hasGraphics()) {
				//http://localhost:8081/ViewServer/thumbdata.jsp?file=000000\00002_0000001004\82b0fc110a1f8506202dd44c45aa7a91.0&px=50&py=50
				options = {
					file: this._data.file,
					px: 50,
					py: 50
				};
				return this.f.getDataAdapter().ViewServer("thumbdata.jsp", options);
			} else if (this.hasCad() || this.hasSvg()) {
				options = {
					docverid: this._docVersion.get("docversionid"),
				};
				if (this.getDocumentVersion().isMultiPage()) {
					options.pageindex = this.getPagenum()
				}
				return this.f.getDataAdapter().WebCenter("docgetthumbdata.jsp", options);
			}
		};

		/**
		 * Returns the information needed for a DocumentDropdownmenu to add an document to the dropdown menu
		 * @returns {Object}
		 */
		WebCenterDocumentPage.prototype.getInformationForDocumentSelector = function () {
			var document = this.getDocument();
			var docVersion = this.getDocumentVersion();
			var pageInfo = {};
			pageInfo.documentName = document.get("docname");
			pageInfo.project = document.get("projectname");
			if (document.hasMultipleVersions()) {
				pageInfo.versionNumber = this.getPageVersionId();
				if (pageInfo.versionNumber === undefined || pageInfo.versionNumber === null) {
					pageInfo.versionNumber = docVersion.getVersionId();
				}
				if (pageInfo.versionNumber && docVersion.get("docRevision")) {
					pageInfo.versionNumber += "(" + docVersion.get("docRevision") + ")";
				}
				pageInfo.versionId = docVersion.getVersionId();
			}
			var pageCount = docVersion.getPageCount();
			if (pageCount > 1) {
				pageInfo.pageCount = pageCount;
				pageInfo.pageNumber = this.getPagenum();
			}
			pageInfo.thumbnailUrl = this.getThumbnailUrl();
			pageInfo.pageId = this.getUniqueID();
			pageInfo.docType = document.getDocumentType();

			return pageInfo;
		};

		/**
		 * Is it possible to change the CAD overlay settings for this page?
		 * @override
		 * @returns {Boolean}
		 */
		WebCenterDocumentPage.prototype.canSaveCadOverlay = function () {
			return this.get("cadregedit") == 1;
		};

		/**
		 * Is the user allowed the edit/save te attributes of this page?
		 * @returns {Boolean}
		 */
		WebCenterDocumentPage.prototype.canEditAttributes = function () {
			return this.get("canEditAttributes") == 1;
		};

		/**
		 * Get the Attribute Category ID for the current document
		 * @returns {String} Category ID of the current document
		 */
		WebCenterDocumentPage.prototype.getAttributeCategoryID = function () {
			return this.get("attributeCategoryID");
		};

		/**
		 * Does the document page need an update of its data
		 * @returns {Boolean}
		 */
		WebCenterDocumentPage.prototype.needsDataUpdate = function () {
			return this.get("hasDefaultData");
		};

		/**
		 * Can the annotations be changed for this document page (not latest version, pharma settings for non-active projects, ...)
		 * @returns {Boolean}
		 */
		WebCenterDocumentPage.prototype.canChangeAnnotations = function () {
			return this.get("canChangeAnnotations") == 1;
		};

		return WebCenterDocumentPage;
	});
