define(["web-viewer/data/AbstractDocumentVersion",
		"web-viewer/data/AbstractDocumentPage",
		"web-viewer/webcenter/WebCenterDocumentPage",
		"utils/XMLUtils",
		"web-viewer/utils/ViewerUtils",
		"underscore"
	],
	function (AbstractDocumentVersion,
		AbstractDocumentPage,
		WebCenterDocumentPage,
		XMLUtils,
		ViewerUtils,
		_) {
		/**
		 * @class The implementation of AbstractDocumentVersion for WebCenter. Initializes a version and constructs
		 * the page(s) in the documentversion.
		 * @exports WebCenterDocumentVersion
		 * @augments Data.AbstractDocumentVersion
		 * @memberof WebCenter
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {AbstractDocument} doc - reference to the document
		 * @param {Number} versionId - id of the version of the document
		 */
		var WebCenterDocumentVersion = function (f, doc, versionId, versionInfo, info, adapter) {
			AbstractDocumentVersion.call(this, f, doc, versionId);

			this._adapter = adapter;

			this._versionData.docversionid = versionInfo.docversionid;

			this._versionData.docversion = versionId;

			this._versionData.isMultiPage = versionInfo.isMultiPage !== undefined ? versionInfo.isMultiPage : false;
			if (_.isString(this._versionData.isMultiPage)) {
				this._versionData.isMultiPage = this._versionData.isMultiPage === "true" ? true : false;
			}

			this._versionData.isPageList = versionInfo.isPageList !== undefined ? versionInfo.isPageList : false;
			if (_.isString(this._versionData.isPageList)) {
				this._versionData.isPageList = this._versionData.isPageList === "true" ? true : false;
			}

			this._versionData.docRevision = versionInfo.docrevision;

			if (info !== undefined && info.pagecount !== undefined) {
				this._versionData.pagecount = info.pagecount;
			}

			if (!_.isArray(versionInfo) && versionInfo !== undefined) {
				var pageNumber = 1;
				if (info) {
					pageNumber = info.pageno || 1;
				}
				var page = new WebCenterDocumentPage(f, this, pageNumber, versionInfo, info, adapter);
				this.addPage(pageNumber, page);
			}
		};

		WebCenterDocumentVersion.prototype = Object.create(AbstractDocumentVersion.prototype);
		WebCenterDocumentVersion.prototype.constructor = WebCenterDocumentVersion;

		/**
		 * Add a document to the document version with a data string
		 * @param {String} data
		 */
		WebCenterDocumentVersion.prototype.addDocumentPageFromData = function (data) {
			var page = new WebCenterDocumentPage(this._f, this, data.pageno, this._versionData, data, this._adapter);
			this.addPage(page.getPagenum(), page);
			if (data.pagecount !== undefined) {
				this._versionData.pagecount = parseInt(data.pagecount);
			}
		};

		/**
		 * Check if the document version has multiple pages
		 * @returns {Boolean}
		 */
		WebCenterDocumentVersion.prototype.isMultiPage = function () {
			return this.getPageCount() > 1 || this._versionData.isMultiPage;
		};

		/**
		 * @returns {String} The WebCenter document version id
		 */
		WebCenterDocumentVersion.prototype.getDocumentVersionID = function () {
			return this.get("docversionid");
		};

		/**
		 * Page count of the document
		 * @returns {Number} Page count of the document
		 */
		WebCenterDocumentVersion.prototype.getPageCount = function () {
			if (this.get("pagecount")) {
				return this.get("pagecount");
			} else if (this._versionData.isMultiPage) {
				return this._pages.length - 1;
			} else {
				return 1;
			}
		};

		return WebCenterDocumentVersion;
	});