define(["web-viewer/data/User",
		"web-viewer/data/PreferencesConstants",
		"web-viewer/data/PreferencesDefaults",
		"web-viewer/data/DataConstants",
		"web-viewer/core/CoreConstants",
		"underscore"
	],
	function (User,
		PreferencesConstants,
		PreferencesDefaults,
		DataConstants,
		CoreConstants,
			 _) {
		"use strict";

		/**
		 * @exports WebCenterUser
		 * @memberof WebCenter
		 * 
		 * @class
		 * User implementation for WebCenter
		 * @augments Data.User 
		 * @constructor
		 */
		var WebCenterUser = function (id, username, first, last, locale, isAdmin) {
			User.call(this, id, username, first, last, locale);

			this.userIsAdmin = isAdmin;

			this.viewServerID = undefined;
		};


		WebCenterUser.prototype = Object.create(User.prototype);
		WebCenterUser.prototype.constructor = WebCenterUser;

		/**
		 * WebCenter way of displaying user
		 */
		WebCenterUser.prototype.getFullName = function () {
			var fullname = this.getLastName();

			if (this.first) {
				fullname += ", " + this.first;
			}

			fullname += " (" + this.getUsername().toUpperCase() + ")";

			return fullname;
		};

		WebCenterUser.prototype.isAdmin = function () {
			return this.userIsAdmin;
		};


		/**
		 * @override
		 */
		WebCenterUser.prototype.loadOptions = function (onLoadedCallback) {
			this.f.requestModule("featureDetector", _.bind(function (featureDetector) {
				var isMobile = "";
				if (featureDetector.isMobile()) {
					isMobile = "m";
				}

				var userinfo = {
					name: this.getFullName(),
					address: window.screen.width + "x" + window.screen.height + isMobile
				};

				//http://eaw11dd413:8081/ViewServer/userid.jsp?description=Java%20Version:%201.7.0_25&host=EAW11DD413&name=Cagnie%2c%20Ruben%20(RUCA <http://eaw11dd413:8081/ViewServer/userid.jsp?description=Java Version: 1.7.0_25&host=EAW11DD413&name=Cagnie, Ruben (RUCA> )
				var adapter = this.f.getDataAdapter();
				if (!adapter.ViewServer) {
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsReadError"), "Adapter cannot connect to Viewserver");
					}, this);
					this.loadDefaultOptions();
					onLoadedCallback();
					return;
				}
				var userid = adapter.ViewServer("userid.jsp", userinfo);
				this.f.ajax(userid, {
					success: _.bind(function (data) {
						var root = data.documentElement;
						var status = root.getElementsByTagName("value")[0].textContent || root.getElementsByTagName("value")[0].text; //ie9
						if (status == "0") {
							this.viewServerID = root.getElementsByTagName("userid")[0].textContent || root.getElementsByTagName("userid")[0].text; //ie9

							//http://eaw11dd413:8081/ViewServer/getsettings.jsp?userid=1					
							var getsettings = adapter.ViewServer("getsettings.jsp", {
								userid: this.viewServerID
							});
							this.f.ajax(getsettings, {
								success: _.bind(function (data) {
									var root = data.documentElement;
									var status = root.getElementsByTagName("value")[0].textContent || root.getElementsByTagName("value")[0].text; //ie9
									if (status == "0") {
										var settings = root.getElementsByTagName("settings")[0];
										this._handleSettings(settings);
										onLoadedCallback();
									} else {
										this.f.i18n.loadBundle("viewer", function () {
											this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsReadError"), root.getElementsByTagName("error")[0].textContent || root.getElementsByTagName("error")[0].text); //ie9
										}, this);
										this.loadDefaultOptions();
										onLoadedCallback();
									}
								}, this),
								error: _.bind(function (jqXHR, textStatus, errorThrown) {
									this.f.i18n.loadBundle("viewer", function () {
										this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsReadError"), jqXHR, textStatus, errorThrown);
									}, this);
									this.loadDefaultOptions();
									onLoadedCallback();
								}, this)
							});

						} else {
							this.f.i18n.loadBundle("viewer", function () {
								this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsReadError"), root.getElementsByTagName("error")[0] ? root.getElementsByTagName("error")[0].textContent : undefined);
							}, this);
							this.loadDefaultOptions();
							onLoadedCallback();
						}
					}, this),
					error: _.bind(function (jqXHR, textStatus, errorThrown) {
						this.f.i18n.loadBundle("viewer", function () {
							this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsReadError"), jqXHR, textStatus, errorThrown);
						}, this);
						this.loadDefaultOptions();
						onLoadedCallback();
					}, this)
				});
			}, this));
		};

		WebCenterUser.prototype.loadDefaultOptions = function () {
			this.setOptions(_.clone(PreferencesDefaults));
		};

		/**
		 * Read out result of getsettings call. Initialize the options for the current user
		 * @param {Element} settings - XML node with the settings
		 */
		WebCenterUser.prototype._handleSettings = function (settings) {
			var preferences = _.invert(PreferencesConstants);

			var options = _.clone(PreferencesDefaults);

			var child = settings.firstChild;
			while (child) {
				if (child.nodeType == 1) {
					var type = child.nodeName;

					var id = child.getElementsByTagName("id")[0].textContent || child.getElementsByTagName("id")[0].text; //ie9
					var value = child.getElementsByTagName("value")[0].textContent || child.getElementsByTagName("value")[0].text; //ie9
					if (type == "settingint") {
						value = parseInt(value, 10);
					}

					// this check will ignore settingsint that should by charsequences
					if (type == "settingchar" || !_.contains(DataConstants.charsequencePreferences, preferences[id])) {
						options[preferences[id]] = value;
					}
				}
				child = child.nextSibling;
			}

			// set unit options based on units property
			var units = this.f.getUnits();
			if (units === CoreConstants.SizeUnits.INCH) {
				options["PREFERENCES_UNIT_SIZE"] = CoreConstants.SizeUnits.INCH;
				options["PREFERENCES_UNIT_RESOLUTION"] = CoreConstants.ResolutionUnits.PPI;
				options["PREFERENCES_UNIT_SCREENRULING"] = CoreConstants.ResolutionUnits.PPI;
			} else {
				options["PREFERENCES_UNIT_SIZE"] = CoreConstants.SizeUnits.MM;
				options["PREFERENCES_UNIT_RESOLUTION"] = CoreConstants.ResolutionUnits.PPCM;
				options["PREFERENCES_UNIT_SCREENRULING"] = CoreConstants.ResolutionUnits.PPCM;
			}

			this.initializeOptions(options);
		};

		/**
		 * Persist the settings by POST-ing an XML to the setsettings.jsp servlet
		 */
		WebCenterUser.prototype.persistOptions = function (options, onPersistedCallback) {
            if (this.f.getProperty("WebCenterUser.disablePersistOptions")) { //disable saving settings in mini-viewers
                if (onPersistedCallback) {
                    onPersistedCallback(true);
                }
                return;
            }
            
			var adapter = this.f.getDataAdapter();
			if (!adapter.ViewServer) {
				this.f.i18n.loadBundle("viewer", function () {
					this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsWriteError"), "Adapter cannot connect to Viewserver");
				}, this);
				if (onPersistedCallback) {
					onPersistedCallback(false);
				}
				return;
			}

			//Create a settings XML document
			var settings = document.implementation.createDocument("", 'settings', null);
			_.each(options, function (value, key) {
				var setting = settings.createElementNS(null, _.isNumber(value) ? "settingint" : "settingchar");

				var idNode = settings.createElementNS(null, "id");
				idNode.textContent = PreferencesConstants[key];

				var valueNode = settings.createElementNS(null, "value");
				valueNode.textContent = value;

				setting.appendChild(idNode);
				setting.appendChild(valueNode);

				settings.documentElement.appendChild(setting);

			});
			
			// because of a bug in IE9, we need to serialize the XML Document to a string ourselves
			// if we don't do this step, IE9 might add newline characters at places where spaces are being used
			// this can break JSONs stored in the user preferences
			var serializedSettings = new XMLSerializer().serializeToString(settings);
	
			var url = adapter.ViewServer("setsettings.jsp", {
				userid: this.viewServerID
			});
			this.f.ajax(url, {
				error: _.bind(function (jqXHR, textStatus, errorThrown) {
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsWriteError"), jqXHR, textStatus, errorThrown);
					}, this);
					if (onPersistedCallback) {
						onPersistedCallback(false);
					}
				}, this),
				success: function (data, textStatus, jqXHR) {
					if (onPersistedCallback) {
						onPersistedCallback(true);
					}
				},
				processData: false,
				contentType: 'text/xml',
				data: serializedSettings,
				type: 'POST'
			});
		};


		return WebCenterUser;

	});