define(["utils/HTTPUtils",
		"underscore",
		"jquery"
	],
	function (
		HTTPUtils,
		_,
		$) {
		"use strict";

		/**
		 * Abstract utility class for all API's derived from it
		 * @exports AbstractAPI
		 * @class
		 * 
		 * @param {Object} wcrConfig - webcenter configuration object
		 * @param {ErrorHandler} errorHandler - DEPRECATED: please use the rejected promises
		 */
		var AbstractAPI = function (wcrConfig, errorHandler) {
			/**
			 * @member {Object}
			 * @private
			 */
			this._wcrConfig = wcrConfig;

			/**
			 * @member {ErrorHandler}
			 * @private
			 */
			this._errorHandler = errorHandler;
		};

		/**
		 * Get a reference to the WebCenterConfiguration object (with location of JSPs and Images)
		 * @returns {Object} WebCenter configuration object
		 */
		AbstractAPI.prototype.getWebCenterConfig = function () {
			return this._wcrConfig;
		};

		/**
		 * Constrcut an URL with a number of parameters
		 * @param   {String} servlet    - servlet to contact
		 * @param   {Object} parameters - set of parameters
		 * @returns {String} URL with querystring
		 */
		AbstractAPI.prototype.constructURL = function (servlet, parameters) {
			return HTTPUtils.constructURL(servlet, parameters, this._wcrConfig.jspBaseUrl);
		};

		/**
		 * Make an SDK call to the WebCenter SDK
		 * This will scan the result for errors and rejext the promise if there is any error
		 * @param   {String}  servlet         - WebCenter Servlet
		 * @param   {Object}  [urlParameters] - set of URL parameters to be appended to the servlet
		 * @param   {Object} [options]       - options for the Ajax Call. Currently supported:
		 *                                  defaultOptions = {
		 *										dataType: 'xml',
		 *										cache: false,
		 *										type: "GET",
		 *										data: "",
		 *									};
		 * @returns {Promise} resolved as the JQuery.ajax call
		 */
		AbstractAPI.prototype.callSDK = function (servlet, urlParameters, options) {
			var deferred = $.Deferred();

			var defaultOptions = {
				dataType: 'xml',
				cache: false,
				type: "GET",
				data: "",
			};
			options = _.extend(defaultOptions, options);

			$.ajax(this.constructURL(servlet, urlParameters), {
				dataType: options.dataType,
				cache: options.cache,
				type: options.type,
				data: options.data
			}).done(function (data, textStatus, jqXHR) {
				if (options.dataType === 'xml' && (!data || !data.documentElement || data.documentElement.tagName.toLowerCase() === "error")) {
					// something went wrong
					var errorString = "";
					var message = data.documentElement.getElementsByTagName("message")[0];
					if (message) {
						errorString = message.textContent;
					}

					var code = data.documentElement.getElementsByTagName("code")[0];
					var codeString;
					if (code) {
						codeString = code.textContent;
						errorString += " (" + codeString + ")";
					}
					deferred.reject(jqXHR, "error", errorString, codeString, data);
				} else if (data && data.error) {
					var errorString = "";
					var message = data.error.msg;
					if (message) {
						errorString = message;
					}
					var addonMsg = data.error.addonMsg;
					if (addonMsg) {
						errorString += " " + addonMsg;
					}
					deferred.reject(jqXHR, "error", errorString, data);
				} else {
					// all should be fine	
					deferred.resolve(data, textStatus, jqXHR);
				}
			}).fail(function (jqXHR, textStatus, errorThrown) {
				deferred.reject(jqXHR, textStatus, errorThrown);
			});

			return deferred.promise();
		};

		return AbstractAPI;
	});