define(["./AbstractAPI",
		"./data/WebCenterAttribute",
		"./data/WebCenterAttributeUnit",
		"./data/WebCenterRestrictedSet",
		"./data/WebCenterRestrictedSetValue",
		"utils/Map",
		"utils/webi18n!WebCenterAPI",
		"underscore",
		"jquery"
	],
	function (
		AbstractAPI,
		WebCenterAttribute,
		WebCenterAttributeUnit,
		WebCenterRestrictedSet,
		WebCenterRestrictedSetValue,
		Map,
		i18n,
		_,
		$) {
		"use strict";

		/**
		 * Utility class used for all communication with WebCenter around attributes
		 * @exports AttributesAPI
		 * @class
		 * 
		 * @param {Object} wcrConfig - webcenter configuration object
		 * @param {ErrorHandler} errorHandler
		 */
		var AttributesAPI = function (wcrConfig, errorHandler) {
			AbstractAPI.call(this, wcrConfig, errorHandler);

			/**
			 * @member {Promise.<Map.<WebCenterAttribute>>} Promise for the WebCenterattributes
			 * @private
			 */
			this._deferredAttributesPromise = null;
			/**
			 * @member {Promise.<XMLDocument>} Promise for the call for fetching attributes
			 * @private
			 */
			this._attributesPromise = null;

			/**
			 * @member {Promise.<Map.<WebCenterAttribute>>} Promise for the WebCenterSpecifications
			 * @private
			 */
			this._deferredSpecificationsPromise = null;
			/**
			 * @member {Promise.<XMLDocument>} Promise for the call for fetching specifications
			 * @private
			 */
			this._specificationPromise = null;
		};

		AttributesAPI.prototype = Object.create(AbstractAPI.prototype);
		AttributesAPI.prototype.constructor = AttributesAPI;

		/**
		 * Get all WebCenter attribute definitions
		 * @returns {$.Deferred.<Map.<WebCenterAttribute>} Promise for the webcenter attributes
		 */
		AttributesAPI.prototype.getAttributes = function () {
			if (!this._deferredAttributesPromise) {
				var deferred = $.Deferred();
				this._deferredAttributesPromise = deferred.promise();

				if (!this._attributesPromise) {

					this._attributesPromise = this.callSDK("GetAttributeListXML.jsp", {}, {
						cache: true
					}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
						this._errorHandler.severe(i18n.t("LoadAttributesFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
						deferred.reject(jqXHR, textStatus, i18n.t("LoadAttributesFailed") + " (" + errorThrown + ")");
					}, this));
				}

				$.when(this._attributesPromise).done(_.partial(this._createAttributesMap, deferred));
			}
			return this._deferredAttributesPromise;
		};


		/**
		 * Get all WebCenter specification definitions
		 * @returns {$.Deferred.<Map.<WebCenterAttribute>} Promise for the webcenter specifiations
		 */
		AttributesAPI.prototype.getSpecifications = function () {
			if (!this._deferredSpecificationsPromise) {
				var deferred = $.Deferred();
				this._deferredSpecificationsPromise = deferred.promise();

				if (!this._specificationPromise) {

					this._specificationPromise = this.callSDK("GetAttributeListXML.jsp", {
						type: "task"
					}, {
						cache: true
					}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
						this._errorHandler.severe(i18n.t("LoadAttributesFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
						deferred.reject();
					}, this));
				}

				$.when(this._specificationPromise).done(_.partial(this._createAttributesMap, deferred));
			}
			return this._deferredSpecificationsPromise;
		};

		/**
		 * creates a map with all attributes that are present in the result XML and resolved the given deferred with this map
		 * @param {$.Deferred} deferred - the deferred to resolve when the map is finished
		 * @param {NodeList} result - XML containing all 
		 */
		AttributesAPI.prototype._createAttributesMap = function (deferred, result) {

			var attributeUnitsMap = new Map();
			var restrictedSetsMap = new Map();
			var attributesMap = new Map();

			if (result && result.documentElement) {
				var rootNode = result.documentElement;

				var attributeUnits = rootNode.getElementsByTagName("attribute_unit");

				var id, name, attrUnitId, unit;
				// attribute units
				for (var i = 0; i < attributeUnits.length; i++) {
					var attributeUnit = attributeUnits.item(i);

					id = attributeUnit.getAttribute("id");
					name = attributeUnit.getElementsByTagName("name")[0];
					var type = attributeUnit.getElementsByTagName("type")[0];
					if (id && name && type) {
						attributeUnitsMap.add(id, new WebCenterAttributeUnit(id, name.textContent, type.textContent));
					}
				}

				// restricted sets
				var rs;
				var restrictedSets = rootNode.getElementsByTagName("restricted_set");
				for (i = 0; i < restrictedSets.length; i++) {
					rs = restrictedSets.item(i);
					id = rs.getAttribute("id");
					name = rs.getElementsByTagName("name")[0];
					attrUnitId = rs.getElementsByTagName("attribute_unit_id")[0];

					if (id && name && attrUnitId) {
						unit = attributeUnitsMap.get(attrUnitId.textContent);

						// restricted set values
						var rsValues = rs.getElementsByTagName("restricted_set_value");
						var values = [];
						for (var j = 0; j < rsValues.length; j++) {
							var rsValue = rsValues.item(j);

							var rsValId = rsValue.getAttribute("id");
							var rsValVal = rsValue.getElementsByTagName("value")[0];

							if (rsValId && rsValVal) {
								values.push(new WebCenterRestrictedSetValue(rsValId, rsValVal.textContent));
							}
						}
						restrictedSetsMap.add(id, new WebCenterRestrictedSet(id, name.textContent, unit, values));
					}
				}

				// attributes
				var attributes = rootNode.getElementsByTagName("attribute");
				for (i = 0; i < attributes.length; i++) {
					var attr = attributes.item(i);

					id = attr.getAttribute("id");
					name = attr.getElementsByTagName("name")[0];
					attrUnitId = attr.getElementsByTagName("attribute_unit_id")[0];
					var rsId = attr.getElementsByTagName("restricted_set_id")[0];

					if (id && name && attrUnitId) {
						unit = attributeUnitsMap.get(attrUnitId.textContent);

						if (rsId) {
							rs = restrictedSetsMap.get(rsId.textContent);
						} else {
							rs = undefined;
						}

						attributesMap.add(id, new WebCenterAttribute(id, name.textContent, unit, rs));
					}
				}
			}

			deferred.resolve(attributesMap);

		};

		/**
		 * Get all document reference attributes
		 * @returns {$.Deferred.<Map.<id, WebCenterAttribute>>}
		 */
		AttributesAPI.prototype.getDocRefAttributes = function () {
			var deferred = $.Deferred();
			this.getAttributes().done(function (attributes) {
				var result = new Map();

				_.each(attributes.asObject(), function (attribute, id) {
					if (attribute.isDocRef()) {
						result.add(id, attribute);
					}
				});

				deferred.resolve(result);
			}).fail(function () {
				deferred.reject();
			});
			return deferred.promise();
		};

		return AttributesAPI;
	});