define(["./AbstractAPI",
		"utils/webi18n!WebCenterAPI",
		"utils/ConvertUtils",
		"utils/Promisify",
		"pages/rules/Rule",
		"pages/rules/RuleFactory",
		"underscore",
		"jquery"
	],
	function (
		AbstractAPI,
		i18n,
		ConvertUtils,
		Promisify,
		Rule,
		RuleFactory,
		_,
		$) {
		"use strict";

		/**
		 * Utility class used for all communication with WebCenter around rules
		 * @exports RuleEngineAPI
		 * @class
		 * 
		 * @param {Object} wcrConfig - webcenter configuration object
		 * @param {ErrorHandler} errorHandler
		 */
		var RuleEngineAPI = function (wcrConfig, errorHandler) {
			AbstractAPI.call(this, wcrConfig, errorHandler);

			this._ruleFactory = new RuleFactory();
		};

		RuleEngineAPI.prototype = Object.create(AbstractAPI.prototype);
		RuleEngineAPI.prototype.constructor = RuleEngineAPI;

		/**
		 * Save a rule. If the id of the rule is unset, a new rule will be created; otherwise the existing rule will be changed.
		 * @param {Rule} rule - The rule to save
		 * @param {String} projectid - The project id
		 * @returns {Promise}
		 */
		RuleEngineAPI.prototype.saveRule = function (rule, projectid) {
			var parameters = {
				"action": "SAVE_RULE",
				"projectid": projectid,
				"name": rule.getName(),
				"enabled": rule.isEnabled(),
				"description": rule.getDescription(),
				"order": rule.getOrder(),
				"ruleJSON": JSON.stringify({
					triggers: rule.getTriggers(),
					actions: rule.getActions()
				}),
				"ruleUpdateDependent": true
			};

			if (rule.getID()) {
				// if the ID is filled in, send it along
				parameters.id = rule.getID();
			}

			return Promisify.deferred(this.callSDK("RuleEngineServlet.wcr", {}, {
				data: parameters,
				type: 'POST'
			}));
		};

		/**
		 * Delete rules
		 * @param {Array.<String>} ruleids - The rule ids of the rules to delete
		 * @param {String} projectid - The project id in which to delete the rule
		 * @returns {Promise}
		 */
		RuleEngineAPI.prototype.deleteRules = function (ruleids, projectid) {
			var parameters = {
				"action": "DELETE_RULES",
				"projectid": projectid,
				"ruleid": ruleids
			};

			return Promisify.deferred(this.callSDK("RuleEngineServlet.wcr", parameters));
		};

		/**
		 * Get all the rules of the given project
		 * @param   {String} projectid - The projectid of the project to get the rules from
		 * @returns {Promise.<Object>} A promise with a ruleset as result containing rules ({Array.<Rule>} "rules") and the owner ({Object} "owner")
		 */
		RuleEngineAPI.prototype.getRules = function (projectid) {
			var parameters = {
				"action": "LOAD_RULES",
				"projectid": projectid
			};

			return Promisify.deferred(this.callSDK("RuleEngineServlet.wcr", parameters)).then(_.bind(function (result) {
				var ruleSet = {};
				var owner = {};
				var rules = [];
				ruleSet.owner = owner;
				ruleSet.rules = rules;

				if (result && result.documentElement) {
					var rootNode = result.documentElement;

					var ownerID = rootNode.getElementsByTagName("owner")[0];
					var ownerName = rootNode.getElementsByTagName("owner_name")[0];
					if (ownerID && ownerName) {
						owner.id = ownerID.textContent;
						owner.name = ownerName.textContent;
					}

					var rulesElements = rootNode.getElementsByTagName("rule");
					for (var i = 0; i < rulesElements.length; i++) {
						var ruleElement = rulesElements[i];
						var id = ruleElement.getElementsByTagName("id")[0];
						var name = ruleElement.getElementsByTagName("name")[0];
						var description = ruleElement.getElementsByTagName("description")[0];
						var order = ruleElement.getElementsByTagName("order")[0];
						var enabled = ruleElement.getElementsByTagName("enabled")[0];
						var ruleJSON = ruleElement.getElementsByTagName("json")[0];

						if (id && name && description && order && enabled && ruleJSON) {
							var obj = JSON.parse(ruleJSON.textContent);
							var type;

							var triggersElements = obj.triggers;
							var triggersArray = triggersElements;
							if (!_.isArray(triggersElements)) {
								triggersArray = [triggersElements];
							}
							var triggers = [];
							for (var j = 0; j < triggersArray.length; j++) {
								var triggerElement = triggersArray[j];
								if (triggerElement) {
									type = triggerElement.type;
									var trigger = this._ruleFactory.createTrigger(type);
									trigger = _.extend(trigger, triggerElement);
									triggers.push(trigger);
								}
							}

							var actionElements = obj.actions;
							var actionsArray = actionElements;
							if (!_.isArray(actionElements)) {
								actionsArray = [actionElements];
							}
							var actions = [];
							for (j = 0; j < actionsArray.length; j++) {
								var actionElement = actionsArray[j];
								if (actionElement) {
									type = actionElement.type;
									var action = this._ruleFactory.createAction(type);
									action = _.extend(action, actionElement);
									actions.push(action);
								}
							}

							var parsedOrder = parseInt(order.textContent, 10);

							var enabledBoolean = enabled.textContent === "1";

							var rule = new Rule(id.textContent, name.textContent, description.textContent, parsedOrder, enabledBoolean, triggers, actions);
							rules.push(rule);
						}
					}
				}
				return ruleSet;
			}, this));
		};

		/**
		 * Export the rule set of a template
		 * @param   {string} projectId the Id of the template
		 */
		RuleEngineAPI.prototype.exportRules = function (projectId, ruleIds) {

			var params = {
				action: 'EXPORT_RULES',
				projectid: projectId,
				ruleid: ruleIds
			};

			var url = "RuleEngineServlet.wcr?" + $.param(params, true);
			document.location.href = url;
		};

		return RuleEngineAPI;
	});