define(["./AbstractAPI",
		"./data/WebCenterTaskType",
		"utils/Map",
		"utils/webi18n!WebCenterAPI",
		"underscore",
		"jquery"
	],
	function (
		AbstractAPI,
		WebCenterTaskType,
		Map,
		i18n,
		_,
		$) {
		"use strict";

		/**
		 * Utility class used for all communication with WebCenter around attributes
		 * @exports TaskTypeAPI
		 * @class
		 * 
		 * @param {Object} wcrConfig - webcenter configuration object
		 * @param {ErrorHandler} errorHandler
		 */
		var TaskTypeAPI = function (wcrConfig, errorHandler) {
			AbstractAPI.call(this, wcrConfig, errorHandler);

			/**
			 * @member {Promise.<XMLDocument>} Promise for the task type definition call
			 * @private
			 */
			this._tasktypeDefinitionPromise = null;

			/**
			 * @member {Promise.<Map.<WebCenterTaskType>>} Promise for the WebCenterTaskTypes
			 * @private
			 */
			this._deferredTaskTypesPromise = null;
		};

		TaskTypeAPI.prototype = Object.create(AbstractAPI.prototype);
		TaskTypeAPI.prototype.constructor = TaskTypeAPI;


		/**
		 * Get the TaskTypeDefinitions XML
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise for the TaskTypeDefinitions
		 */
		TaskTypeAPI.prototype.getTaskTypeDefinitions = function () {
			if (!this._tasktypeDefinitionPromise) {

				this._tasktypeDefinitionPromise = this.callSDK("GetTaskTypes.jsp", {
					verbose: "0",
					loadicons: "1",
					ajax: "1"
				}, {
					cache: true
				}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
					this._errorHandler.severe(i18n.t("LoadTaskTypesFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				}, this));
			}

			return this._tasktypeDefinitionPromise;
		};

		/**
		 * Get the task types as a Map
		 * @returns {$.Deferred.<Map.<id, WebCenterTaskType>>}} Promise for the TaskTypeDefinitions
		 */
		TaskTypeAPI.prototype.getTaskTypes = function () {
			if (!this._deferredTaskTypesPromise) {
				var deferred = $.Deferred();
				this._deferredTaskTypesPromise = deferred.promise();

				$.when(this.getTaskTypeDefinitions()).done(function (xml, textStatus, jqXHR) {
					var taskTypesMap = new Map();
					var taskTypeElements = xml.getElementsByTagName("wctask_type");

					for (var i = 0; i < taskTypeElements.length; i++) {
						var taskTypeElement = taskTypeElements.item(i);
						var taskType = WebCenterTaskType.fromXML(taskTypeElement);
						if (taskType) {
							taskTypesMap.add(taskType.getId(), taskType);
						}
					}
					deferred.resolve(taskTypesMap);
				}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
					deferred.reject(jqXHR, textStatus, i18n.t("LoadTaskTypesFailed") + " (" + errorThrown + ")");
				}, this));
			}
			return this._deferredTaskTypesPromise;
		};

		return TaskTypeAPI;
	});