define(["utils/FilterUtils", "underscore"],
	function (FilterUtils, _) {
		"use strict";

		/**
		 * Represents a WebCenter group
		 * @param {String} id - Id of the group
		 * @param {String} name - Name of the group
		 * @param {Array.<WebCenterUser>} members - members of the group
		 */
		var WebCenterGroup = function (id, name, members) {
			this._id = id;
			this._name = name;
			this._members = members;

		};

		/**
		 * Get the ID of the group
		 * @returns {String}
		 */
		WebCenterGroup.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Get the name of the group
		 * @returns {String}
		 */
		WebCenterGroup.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Get the members of a group
		 * @returns {Array.<WebCenterUser>} 
		 */
		WebCenterGroup.prototype.getMembers = function () {
			return this._members;
		};

		/**
		 * Does the group (or one of its members) match a query
		 * @param   {String}  query          
		 * @param   {Boolean} includeMembers - should members be checked as well?
		 * @returns {Boolean} 
		 */
		WebCenterGroup.prototype.match = function (query, includeMembers) {
			var regex = FilterUtils.generateRegex(query, true);

			return this._name.match(regex) || (includeMembers && _.some(this._members, function (member) {
				return member.match(query);
			})) || (this.getGroupAssignee() && this.getGroupAssignee().match(query));
		};

		/**
		 * Get the display icon classes for a group
		 * @returns {string} 
		 */
		WebCenterGroup.getDisplayIcon = function () {
			return "icon-green icon-WCR_Assignee-Group";
		};

		/**
		 * Get the display Element for a group
		 * @param {Boolean} includeIcon
		 * @returns {JQueryObj} 
		 */
		WebCenterGroup.prototype.getDisplayElement = function (includeIcon) {
			var displayElement = $("<span>", {
				text: this.getName()
			});
			if (includeIcon) {
				displayElement.prepend($("<i>", {
					'class': "icon icon-normal " + this.getDisplayIcon()
				}));
			}
			return displayElement;

		};

		/**
		 * Add extra markers for SAHI if needed (they are added after the default SHAI marker)
		 * @param {String} extraMarker - or null to reset extra markers
		 */
		WebCenterGroup.prototype.setExtraSAHIMarkers = function (extraMarker) {
			this._extraMarker = extraMarker;
		};

		/**
		 * Get the SAHI Marker that can be used for this group
		 * @returns {string} [[Description]]
		 */
		WebCenterGroup.prototype.getSAHIMarker = function () {
			var extra = (this._extraMarker ? (" " + this._extraMarker) : "");
			return "WCRGROUP " + this.getName() + extra;
		};

		/**
		 * Set the group assignee
		 * @param {WebCenterUser} groupAssignee 
		 */
		WebCenterGroup.prototype.setGroupAssignee = function (groupAssignee) {
			this._groupAssignee = groupAssignee;
			
			if (!_.find(this._members, function(user){
				return user.getId() === groupAssignee.getId();
			})){
				this.addMember(groupAssignee);	
			}
		};
	
		/**
		 * Get the group assignee
		 * @returns {WebCenterUser} undefined if not set	
		 */
		WebCenterGroup.prototype.getGroupAssignee = function () {
			return this._groupAssignee;
		};

		/**
		 * Add a member to the memberlist
		 * @param {WebCenterUser} member 
		 */
		WebCenterGroup.prototype.addMember = function (member) {
			this._members.push(member);
		};
		return WebCenterGroup;
	});