define(["wcr/webcenter_html",
		"utils/webi18n!WebCenterUser",
		"utils/FilterUtils", 
		"underscore"
	],
	function (WCHTML, 
			   i18n, 
			   FilterUtils, 
			   _) {
		"use strict";

		/**
		 * Represents a WebCenter user
		 * @param {String} id - Id of the user
		 * @param {String} name - Name of the user
		 * @param {String} firstname - First name of the user
		 * @param {String} lastname - Last name of the user
		 * @param {String} email - Email of the user
		 * @param {Boolean} isActive - Whether user is active or not
		 */
		var WebCenterUser = function (id, name, firstname, lastname, email, isActive) {
			this._id = id;
			this._name = name || "";
			this._firstname = firstname || "";
			this._lastname = lastname || "";
			this._email = email || "";
			this._isActive = isActive;
		};

		/**
		 * Get the ID of the user
		 * @returns {String}
		 */
		WebCenterUser.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Get the name of the user (username)
		 * @returns {String}
		 */
		WebCenterUser.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Get the first name of the user
		 * @returns {String}
		 */
		WebCenterUser.prototype.getFirstName = function () {
			return this._firstname;
		};

		/**
		 * Get the last name of the user
		 * @returns {String}
		 */
		WebCenterUser.prototype.getLastName = function () {
			return this._lastname;
		};

		/**
		 * Get the email of the user
		 * @returns {String}
		 */
		WebCenterUser.prototype.getEmail = function () {
			return this._email;
		};

		/**
		 * Returns true if user is active.
		 * @returns {Boolean}
		 */
		WebCenterUser.prototype.isActiveUser = function () {
			return this._isActive;
		};

		/**
		 * Set the user status flag
		 * @param activeFlag
		 * @returns
		 */
		WebCenterUser.prototype.setActiveUserFlag = function (activeFlag) {
			return this._isActive = activeFlag;
		};
		
		WebCenterUser.prototype.match = function (query) {
			var regex = FilterUtils.generateRegex(query, true);
			var tests = [this.getName(), this.getEmail(), this.getFirstName() + " " + this.getLastName(), this.getDisplayName()];
			return _.some(tests, function (name) {
				return name.match(regex);
			});
		};


		/**
		 * Get the Display Icon of the user
		 * @returns {string} CSS Classes for the icon
		 */
		WebCenterUser.prototype.getDisplayIcon = function () {
			if (this._isActive) {
				return "icon-grey icon-WCR_Assignee-User";
			} else {
				return "icon-warning icon-WCR_Actions-User-Deactivate12";
			}
		};

		/**
		 * Get the display name of the user
		 * if includeLabels is true, [deactivated] will be added for deactivated users
		 * @param   {Boolean}  [includeLabels=false]
		 * @returns {String}  
		 */
		WebCenterUser.prototype.getDisplayName = function(includeLabels){
			if (includeLabels){
				return WCHTML.getUserNameString(this.getFirstName(), this.getLastName(), this.getName());
			} else {
				return WCHTML.getUserNameStringActiveText(this.getFirstName(), this.getLastName(), this.getName(), this.isActiveUser(), i18n.t('Deactivated'));
			}
		};
	
		/**
		 * Get a HTML object which displays the user
		 * @param   {Boolean}   includeIcon - if true, an icon will be included
		 * @param   {Boolean}   compact     - if true, for deactivated user no <label> will be added but just a string [deactivated]
		 * @returns {JQueryObj} 
		 */
		WebCenterUser.prototype.getDisplayElement = function(includeIcon, compact){
			var displayElement = $("<span>", {
				text: this.getDisplayName(!compact),
				'class': !this._isActive?"text-warning":""
			});
									
			if (includeIcon){
				displayElement.prepend($("<i>", {'class': "icon icon-normal "+this.getDisplayIcon()}));
			}
			
			if (!compact){
				var labels = this.getLabels();
				if (labels.length > 0){
					displayElement.append("&nbsp;");
					displayElement.append(this.getLabels());			
				}
			}
						
			return displayElement;			
		};
	
		/**
		 * Get the labels that need to be displayed along the user
		 * @returns {JQueryObj}
		 */
		WebCenterUser.prototype.getLabels = function(){
			if (!this._isActive){
				return $("<span>", {'class': "label label-warning", text: i18n.t('Deactivated')});
			} else {
				return $();
			}			
		};
	
		/**
		 * Add extra markers for SAHI if needed (they are added after the default SHAI marker)
		 * @param {String} extraMarker - or null to reset extra markers
		 */
		WebCenterUser.prototype.setExtraSAHIMarkers = function(extraMarker){
			this._extraMarker = extraMarker;
		};
	
		/**
		 * Get the SAHI Marker that can be used for this user
		 * @returns {string} [[Description]]
		 */
		WebCenterUser.prototype.getSAHIMarker = function(){
			var extra = (this._extraMarker?(" " + this._extraMarker): "");
			return "WCRUSER "+this.getDisplayName(true) + extra;
		};
		return WebCenterUser;
	});