define('webcenterapi/AbstractAPI',["utils/HTTPUtils",
		"underscore",
		"jquery"
	],
	function (
		HTTPUtils,
		_,
		$) {
		"use strict";

		/**
		 * Abstract utility class for all API's derived from it
		 * @exports AbstractAPI
		 * @class
		 * 
		 * @param {Object} wcrConfig - webcenter configuration object
		 * @param {ErrorHandler} errorHandler - DEPRECATED: please use the rejected promises
		 */
		var AbstractAPI = function (wcrConfig, errorHandler) {
			/**
			 * @member {Object}
			 * @private
			 */
			this._wcrConfig = wcrConfig;

			/**
			 * @member {ErrorHandler}
			 * @private
			 */
			this._errorHandler = errorHandler;
		};

		/**
		 * Get a reference to the WebCenterConfiguration object (with location of JSPs and Images)
		 * @returns {Object} WebCenter configuration object
		 */
		AbstractAPI.prototype.getWebCenterConfig = function () {
			return this._wcrConfig;
		};

		/**
		 * Constrcut an URL with a number of parameters
		 * @param   {String} servlet    - servlet to contact
		 * @param   {Object} parameters - set of parameters
		 * @returns {String} URL with querystring
		 */
		AbstractAPI.prototype.constructURL = function (servlet, parameters) {
			return HTTPUtils.constructURL(servlet, parameters, this._wcrConfig.jspBaseUrl);
		};

		/**
		 * Make an SDK call to the WebCenter SDK
		 * This will scan the result for errors and rejext the promise if there is any error
		 * @param   {String}  servlet         - WebCenter Servlet
		 * @param   {Object}  [urlParameters] - set of URL parameters to be appended to the servlet
		 * @param   {Object} [options]       - options for the Ajax Call. Currently supported:
		 *                                  defaultOptions = {
		 *										dataType: 'xml',
		 *										cache: false,
		 *										type: "GET",
		 *										data: "",
		 *									};
		 * @returns {Promise} resolved as the JQuery.ajax call
		 */
		AbstractAPI.prototype.callSDK = function (servlet, urlParameters, options) {
			var deferred = $.Deferred();

			var defaultOptions = {
				dataType: 'xml',
				cache: false,
				type: "GET",
				data: "",
			};
			options = _.extend(defaultOptions, options);

			$.ajax(this.constructURL(servlet, urlParameters), {
				dataType: options.dataType,
				cache: options.cache,
				type: options.type,
				data: options.data
			}).done(function (data, textStatus, jqXHR) {
				if (options.dataType === 'xml' && (!data || !data.documentElement || data.documentElement.tagName.toLowerCase() === "error")) {
					// something went wrong
					var errorString = "";
					var message = data.documentElement.getElementsByTagName("message")[0];
					if (message) {
						errorString = message.textContent;
					}

					var code = data.documentElement.getElementsByTagName("code")[0];
					var codeString;
					if (code) {
						codeString = code.textContent;
						errorString += " (" + codeString + ")";
					}
					deferred.reject(jqXHR, "error", errorString, codeString, data);
				} else if (data && data.error) {
					var errorString = "";
					var message = data.error.msg;
					if (message) {
						errorString = message;
					}
					var addonMsg = data.error.addonMsg;
					if (addonMsg) {
						errorString += " " + addonMsg;
					}
					deferred.reject(jqXHR, "error", errorString, data);
				} else {
					// all should be fine	
					deferred.resolve(data, textStatus, jqXHR);
				}
			}).fail(function (jqXHR, textStatus, errorThrown) {
				deferred.reject(jqXHR, textStatus, errorThrown);
			});

			return deferred.promise();
		};

		return AbstractAPI;
	});

define('webcenterapi/data/WebCenterAttribute',[],
	function () {
		"use strict";

		/**
		 * Represents a WebCenter attribute
		 * @param {String}                 id            - Id of the attribute
		 * @param {String}                 name          - Name of the atttribute
		 * @param {WebCenterAttributeUnit} unit          - Unit of the attribute
		 * @param {WebCenterRestrictedSet} [restrictedSet] - Restricted set of the attribute (if any)
		 */
		var WebCenterAttribute = function (id, name, unit, restrictedSet) {
			this._id = id;
			this._name = name;
			this._unit = unit;
			this._restrictedSet = restrictedSet;
		};

		/**
		 * Get the ID of the attribute
		 * @returns {String}
		 */
		WebCenterAttribute.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Get the name of attribute
		 * @returns {String}
		 */
		WebCenterAttribute.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Get the unit attribute
		 * @returns {WebCenterAttributeUnit}
		 */
		WebCenterAttribute.prototype.getType = function () {
			return this._unit;
		};

		/**
		 * Get the restrictedSet or undefined if it isn't linked to one
		 * @returns {WebCenterRestrictedSet}
		 */
		WebCenterAttribute.prototype.getRestrictedSet = function () {
			return this._restrictedSet;
		};

		/**
		 * is this attribute of the Document Reference type?
		 * @returns {Boolean} 
		 */
		WebCenterAttribute.prototype.isDocRef = function () {
			if (this._unit) {
				return this._unit.isDocRef();
			} else {
				return false;
			}
		};


		return WebCenterAttribute;
	});

define('webcenterapi/data/WebCenterAttributeUnit',[],
	function () {
		"use strict";

		/**
		 * Represents a WebCenter attribute unit
		 * @param {String} id - Id of the attribute unit
		 * @param {String} name - Name of the attribute unit e.g. Distance
		 * @param {String} type - Type of the attribute unit e.g. UNITS
		 */
		var WebCenterAttributeUnit = function (id, name, type) {
			this._id = id;
			this._name = name;
			this._type = type;
		};

		/**
		 * Get the ID of the attribute unit
		 * @returns {String}
		 */
		WebCenterAttributeUnit.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Get the name of the attribute unit
		 * @returns {String}
		 */
		WebCenterAttributeUnit.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Get the type of the attribute unit
		 * @returns {String}
		 */
		WebCenterAttributeUnit.prototype.getType = function () {
			return this._type;
		};

		/**
		 * Is it a document reference unit
		 * @returns {Boolean} 
		 */
		WebCenterAttributeUnit.prototype.isDocRef = function () {
			return this._id === '00001_0000000022';
		};
		
		/**
		 * Is it a dateTime attibute
		 * @returns {Boolean} 
		 */
		WebCenterAttributeUnit.prototype.isDateTime = function () {
			return this._id === '00001_0000000020';
		};

		return WebCenterAttributeUnit;
	});

define('webcenterapi/data/WebCenterRestrictedSet',[],
	function () {
		"use strict";

		/**
		 * Represents a WebCenter restricted set
		 * @param {String} id - Id of the restricted set
		 * @param {String} name - Name of the restricted set e.g. Languages
		 * @param {WebCenterAttributeUnit} unit - Unit of the restricted set
		 * @param {Array.<WebCenterRestrictedSetValue>} values - Values of the restricted set                               
		 */
		var WebCenterRestrictedSet = function (id, name, unit, values) {
			this._id = id;
			this._name = name;
			this._unit = unit;
			this._values = values;
		};

		/**
		 * Get the ID of the restricted set
		 * @returns {String}
		 */
		WebCenterRestrictedSet.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Get the name of the restricted set
		 * @returns {String}
		 */
		WebCenterRestrictedSet.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Get the unit of the restricted set
		 * @returns {WebCenterAttributeUnit}
		 */
		WebCenterRestrictedSet.prototype.getType = function () {
			return this._unit;
		};

		/**
		 * Get the values of the restricted set
		 * @returns {Array.<WebCenterRestrictedSetValue>}
		 */
		WebCenterRestrictedSet.prototype.getValues = function () {
			return this._values;
		};


		return WebCenterRestrictedSet;
	});

define('webcenterapi/data/WebCenterRestrictedSetValue',[],
	function () {
		"use strict";

		/**
		 * Represents a WebCenter resticted set value
		 * @param {String} id - ID of the value
		 * @param {String} value - the value itself
		 */
		var WebCenterRestrictedSetValue = function (id, value) {
			this._id = id;
			this._value = value;
		};

		/**
		 * Get the ID of the restricted set value
		 * @returns {String}
		 */
		WebCenterRestrictedSetValue.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Get the value of the restricted set value
		 * @returns {String}
		 */
		WebCenterRestrictedSetValue.prototype.getValue = function () {
			return this._value;
		};

		return WebCenterRestrictedSetValue;
	});


define('webcenterapi/AttributesAPI',["./AbstractAPI",
		"./data/WebCenterAttribute",
		"./data/WebCenterAttributeUnit",
		"./data/WebCenterRestrictedSet",
		"./data/WebCenterRestrictedSetValue",
		"utils/Map",
		"utils/webi18n!WebCenterAPI",
		"underscore",
		"jquery"
	],
	function (
		AbstractAPI,
		WebCenterAttribute,
		WebCenterAttributeUnit,
		WebCenterRestrictedSet,
		WebCenterRestrictedSetValue,
		Map,
		i18n,
		_,
		$) {
		"use strict";

		/**
		 * Utility class used for all communication with WebCenter around attributes
		 * @exports AttributesAPI
		 * @class
		 * 
		 * @param {Object} wcrConfig - webcenter configuration object
		 * @param {ErrorHandler} errorHandler
		 */
		var AttributesAPI = function (wcrConfig, errorHandler) {
			AbstractAPI.call(this, wcrConfig, errorHandler);

			/**
			 * @member {Promise.<Map.<WebCenterAttribute>>} Promise for the WebCenterattributes
			 * @private
			 */
			this._deferredAttributesPromise = null;
			/**
			 * @member {Promise.<XMLDocument>} Promise for the call for fetching attributes
			 * @private
			 */
			this._attributesPromise = null;

			/**
			 * @member {Promise.<Map.<WebCenterAttribute>>} Promise for the WebCenterSpecifications
			 * @private
			 */
			this._deferredSpecificationsPromise = null;
			/**
			 * @member {Promise.<XMLDocument>} Promise for the call for fetching specifications
			 * @private
			 */
			this._specificationPromise = null;
		};

		AttributesAPI.prototype = Object.create(AbstractAPI.prototype);
		AttributesAPI.prototype.constructor = AttributesAPI;

		/**
		 * Get all WebCenter attribute definitions
		 * @returns {$.Deferred.<Map.<WebCenterAttribute>} Promise for the webcenter attributes
		 */
		AttributesAPI.prototype.getAttributes = function () {
			if (!this._deferredAttributesPromise) {
				var deferred = $.Deferred();
				this._deferredAttributesPromise = deferred.promise();

				if (!this._attributesPromise) {

					this._attributesPromise = this.callSDK("GetAttributeListXML.jsp", {}, {
						cache: true
					}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
						this._errorHandler.severe(i18n.t("LoadAttributesFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
						deferred.reject(jqXHR, textStatus, i18n.t("LoadAttributesFailed") + " (" + errorThrown + ")");
					}, this));
				}

				$.when(this._attributesPromise).done(_.partial(this._createAttributesMap, deferred));
			}
			return this._deferredAttributesPromise;
		};


		/**
		 * Get all WebCenter specification definitions
		 * @returns {$.Deferred.<Map.<WebCenterAttribute>} Promise for the webcenter specifiations
		 */
		AttributesAPI.prototype.getSpecifications = function () {
			if (!this._deferredSpecificationsPromise) {
				var deferred = $.Deferred();
				this._deferredSpecificationsPromise = deferred.promise();

				if (!this._specificationPromise) {

					this._specificationPromise = this.callSDK("GetAttributeListXML.jsp", {
						type: "task"
					}, {
						cache: true
					}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
						this._errorHandler.severe(i18n.t("LoadAttributesFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
						deferred.reject();
					}, this));
				}

				$.when(this._specificationPromise).done(_.partial(this._createAttributesMap, deferred));
			}
			return this._deferredSpecificationsPromise;
		};

		/**
		 * creates a map with all attributes that are present in the result XML and resolved the given deferred with this map
		 * @param {$.Deferred} deferred - the deferred to resolve when the map is finished
		 * @param {NodeList} result - XML containing all 
		 */
		AttributesAPI.prototype._createAttributesMap = function (deferred, result) {

			var attributeUnitsMap = new Map();
			var restrictedSetsMap = new Map();
			var attributesMap = new Map();

			if (result && result.documentElement) {
				var rootNode = result.documentElement;

				var attributeUnits = rootNode.getElementsByTagName("attribute_unit");

				var id, name, attrUnitId, unit;
				// attribute units
				for (var i = 0; i < attributeUnits.length; i++) {
					var attributeUnit = attributeUnits.item(i);

					id = attributeUnit.getAttribute("id");
					name = attributeUnit.getElementsByTagName("name")[0];
					var type = attributeUnit.getElementsByTagName("type")[0];
					if (id && name && type) {
						attributeUnitsMap.add(id, new WebCenterAttributeUnit(id, name.textContent, type.textContent));
					}
				}

				// restricted sets
				var rs;
				var restrictedSets = rootNode.getElementsByTagName("restricted_set");
				for (i = 0; i < restrictedSets.length; i++) {
					rs = restrictedSets.item(i);
					id = rs.getAttribute("id");
					name = rs.getElementsByTagName("name")[0];
					attrUnitId = rs.getElementsByTagName("attribute_unit_id")[0];

					if (id && name && attrUnitId) {
						unit = attributeUnitsMap.get(attrUnitId.textContent);

						// restricted set values
						var rsValues = rs.getElementsByTagName("restricted_set_value");
						var values = [];
						for (var j = 0; j < rsValues.length; j++) {
							var rsValue = rsValues.item(j);

							var rsValId = rsValue.getAttribute("id");
							var rsValVal = rsValue.getElementsByTagName("value")[0];

							if (rsValId && rsValVal) {
								values.push(new WebCenterRestrictedSetValue(rsValId, rsValVal.textContent));
							}
						}
						restrictedSetsMap.add(id, new WebCenterRestrictedSet(id, name.textContent, unit, values));
					}
				}

				// attributes
				var attributes = rootNode.getElementsByTagName("attribute");
				for (i = 0; i < attributes.length; i++) {
					var attr = attributes.item(i);

					id = attr.getAttribute("id");
					name = attr.getElementsByTagName("name")[0];
					attrUnitId = attr.getElementsByTagName("attribute_unit_id")[0];
					var rsId = attr.getElementsByTagName("restricted_set_id")[0];

					if (id && name && attrUnitId) {
						unit = attributeUnitsMap.get(attrUnitId.textContent);

						if (rsId) {
							rs = restrictedSetsMap.get(rsId.textContent);
						} else {
							rs = undefined;
						}

						attributesMap.add(id, new WebCenterAttribute(id, name.textContent, unit, rs));
					}
				}
			}

			deferred.resolve(attributesMap);

		};

		/**
		 * Get all document reference attributes
		 * @returns {$.Deferred.<Map.<id, WebCenterAttribute>>}
		 */
		AttributesAPI.prototype.getDocRefAttributes = function () {
			var deferred = $.Deferred();
			this.getAttributes().done(function (attributes) {
				var result = new Map();

				_.each(attributes.asObject(), function (attribute, id) {
					if (attribute.isDocRef()) {
						result.add(id, attribute);
					}
				});

				deferred.resolve(result);
			}).fail(function () {
				deferred.reject();
			});
			return deferred.promise();
		};

		return AttributesAPI;
	});

define('WFEditorWCR/core/WebCenterWFEditorConstants',{
	FeedbackUpdateInterval: 20 * 1000,
	WorkflowControlActions: {
		HOLD: 2,
		RESUME: 3,
		CANCEL: 1
	},
	WorkflowControlToolbar: "workflowControlActions",
	WorkflowControlActionButtons: {
		HOLD: "holdWorkflowAction",
		RESUME: "resumeWorkflowAction",
		CANCEL: "cancelWorkflowAction"
	},
	WorkflowStatusToolbar: "workflowStatusToolbar",
	WorkflowErrorStatus: "00001_0000000004",
	WorkflowErrorImage: "Canceled_pink.png",
	WorkflowCanceledStatus: "00001_0000000005",
	WorkflowCanceledImage: "wf_cancelled.gif",

	WorkflowActionsToolbar: "wcrActions",
	WorkflowSaveToolbarButton: "wcrSaveAction",
	WorkflowImportToolbarButton: "wcrImportAction",
	WorkflowExportToolbarButton: "wcrExportAction",
	WorkflowClearToolbarButton: "wcrClearAction",
	TaskNameToolbarComponent: "wcrTaskName",
	
	WorkflowWFModesToolbar: "wcrWFModesToolbar",
	WorkflowFullScreenToolbarButton: "wcrFullScreenAction",
	WorkflowHighlightActiveToolbarButton: "wcrHighlightActiveAction",

	WorkflowTaskDefaultIcon: "workflow/images/hand.png",
	
	WorkflowWorldBounds : {
		x: 0,
		y: 0,
		width: 16384,
		height: 16384
	}
});

define('webcenterapi/data/WebCenterTaskType',["utils/XMLUtils",
		"WFEditorWCR/core/WebCenterWFEditorConstants"
	],
	function (XMLUtils,
		WebCenterWFEditorConstants) {
		"use strict";

		/**
		 * Data structure for describing a WebCenter Task Type
		 * @class
		 * @exports WebCenterTaskType
		 */
		var WebCenterTaskType = function (taskTypeID, name, description, creatorID, leadDays, leadTime, hasWorkflow, icon, restrictedAssignee) {
			this._taskTypeID = taskTypeID;
			this._name = name;
			this._description = description;
			this._creatorID = creatorID;
			this._leadDays = leadDays;
			this._leadTime = leadTime;
			this._hasWorkflow = hasWorkflow;
			this._icon = icon;
			this._restrictedAssignee = restrictedAssignee;
		};

		/**
		 * Get the ID
		 * @returns {String}
		 */
		WebCenterTaskType.prototype.getId = function () {
			return this._taskTypeID;
		};

		/**
		 * Get the Name
		 * @returns {String} 
		 */
		WebCenterTaskType.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Get the Name
		 * @returns {String} 
		 */
		WebCenterTaskType.prototype.getDescription = function () {
			return this._description;
		};

		/**
		 * Get the Lead Days
		 * @returns {String} 
		 */
		WebCenterTaskType.prototype.getLeadDays = function () {
			return this._leadDays;
		};

		/**
		 * Get the Lead Time
		 * @returns {String} 
		 */
		WebCenterTaskType.prototype.getLeadTime = function () {
			return this._leadTime;
		};

		/**
		 * Get the Icon
		 * @returns {String} 
		 */
		WebCenterTaskType.prototype.getIcon = function () {
			return this._icon || WebCenterWFEditorConstants.WorkflowTaskDefaultIcon;
		};

		/**
		 * Gets data about assignee restriction
		 * 
		 * @returns {Object} 		 {
		 *   							restriction : "role" | "group" | "noUser",
		 *   							[id]: <String>
		 *  						}
		 */
		WebCenterTaskType.prototype.getRestrictedAssignee = function () {
			return this._restrictedAssignee;
		};

		/**
		 * Has the task type a workflow or not
		 * @returns {Boolean} True if it is a workflow, false for a manual task
		 */
		WebCenterTaskType.prototype.hasWorkflow = function () {
			return this._hasWorkflow == "1";
		};

		/**
		 * Create a new WebCenterTaskType based on a XML description of the task type
		 * @param   {XMLDocument}         xml - xml description of the task type (wctask_type element)
		 * @returns {WebCenterTaskType} 
		 */
		WebCenterTaskType.fromXML = function (xml) {
			var taskTypeID = xml.getAttribute("id");
			var name = xml.getElementsByTagName("task_type_name")[0];
			var description = xml.getElementsByTagName("description")[0];
			var creatorID = xml.getElementsByTagName("creator_id")[0];
			var leadDays = xml.getElementsByTagName("approximate_length")[0];
			var leadTime = xml.getElementsByTagName("approximate_time")[0];
			var hasWorkflow = xml.getElementsByTagName("has_workflow")[0];
			var icon = xml.getElementsByTagName("icon")[0];
			var restrictedAssignee;
			var restrictedAssigneeTag = xml.getElementsByTagName("restrict_assignees");
			if (restrictedAssigneeTag && restrictedAssigneeTag[0] && restrictedAssigneeTag[0].textContent === "1") {
				var restrictAssigneeID = xml.getElementsByTagName("main_assignee_id");
				if (restrictAssigneeID && restrictAssigneeID[0]) {
					restrictedAssignee = {
						restriction: "roleOrGroup",
						id: restrictAssigneeID[0].textContent
					};
				} else {
					restrictedAssignee = {
						restriction: "noUser"
					};
				}

			}
			if (taskTypeID && name && description && creatorID && leadDays && leadTime && hasWorkflow && icon) {
				return new WebCenterTaskType(taskTypeID, name.textContent, description.textContent, creatorID.textContent, leadDays.textContent, leadTime.textContent, hasWorkflow.textContent, icon.textContent, restrictedAssignee);
			}
		};

		return WebCenterTaskType;

	});

define('webcenterapi/TaskTypeAPI',["./AbstractAPI",
		"./data/WebCenterTaskType",
		"utils/Map",
		"utils/webi18n!WebCenterAPI",
		"underscore",
		"jquery"
	],
	function (
		AbstractAPI,
		WebCenterTaskType,
		Map,
		i18n,
		_,
		$) {
		"use strict";

		/**
		 * Utility class used for all communication with WebCenter around attributes
		 * @exports TaskTypeAPI
		 * @class
		 * 
		 * @param {Object} wcrConfig - webcenter configuration object
		 * @param {ErrorHandler} errorHandler
		 */
		var TaskTypeAPI = function (wcrConfig, errorHandler) {
			AbstractAPI.call(this, wcrConfig, errorHandler);

			/**
			 * @member {Promise.<XMLDocument>} Promise for the task type definition call
			 * @private
			 */
			this._tasktypeDefinitionPromise = null;

			/**
			 * @member {Promise.<Map.<WebCenterTaskType>>} Promise for the WebCenterTaskTypes
			 * @private
			 */
			this._deferredTaskTypesPromise = null;
		};

		TaskTypeAPI.prototype = Object.create(AbstractAPI.prototype);
		TaskTypeAPI.prototype.constructor = TaskTypeAPI;


		/**
		 * Get the TaskTypeDefinitions XML
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise for the TaskTypeDefinitions
		 */
		TaskTypeAPI.prototype.getTaskTypeDefinitions = function () {
			if (!this._tasktypeDefinitionPromise) {

				this._tasktypeDefinitionPromise = this.callSDK("GetTaskTypes.jsp", {
					verbose: "0",
					loadicons: "1",
					ajax: "1"
				}, {
					cache: true
				}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
					this._errorHandler.severe(i18n.t("LoadTaskTypesFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				}, this));
			}

			return this._tasktypeDefinitionPromise;
		};

		/**
		 * Get the task types as a Map
		 * @returns {$.Deferred.<Map.<id, WebCenterTaskType>>}} Promise for the TaskTypeDefinitions
		 */
		TaskTypeAPI.prototype.getTaskTypes = function () {
			if (!this._deferredTaskTypesPromise) {
				var deferred = $.Deferred();
				this._deferredTaskTypesPromise = deferred.promise();

				$.when(this.getTaskTypeDefinitions()).done(function (xml, textStatus, jqXHR) {
					var taskTypesMap = new Map();
					var taskTypeElements = xml.getElementsByTagName("wctask_type");

					for (var i = 0; i < taskTypeElements.length; i++) {
						var taskTypeElement = taskTypeElements.item(i);
						var taskType = WebCenterTaskType.fromXML(taskTypeElement);
						if (taskType) {
							taskTypesMap.add(taskType.getId(), taskType);
						}
					}
					deferred.resolve(taskTypesMap);
				}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
					deferred.reject(jqXHR, textStatus, i18n.t("LoadTaskTypesFailed") + " (" + errorThrown + ")");
				}, this));
			}
			return this._deferredTaskTypesPromise;
		};

		return TaskTypeAPI;
	});

define('WFEditorWCR/workflow/WebCenterWorkflowState',[],
	function () {
		"use strict";

		return {
			RUNNING: "running",
			PAUSED: "paused",
			STOPPING: "stopping",
			FINISHED: "finished"
		};
	});

define('WFEditorWCR/data/WebCenterAttributeCategory',[],
	function () {
		"use strict";

		/**
		 * Represents a WebCenter attribute category
		 * @param {String}                 id          - ID of the atttribute category
		 * @param {String}                 name          - Name of the atttribute category
		 * @param {String}                 description            - Description of the attribute category
		 */
		var WebCenterAttributeCategory = function (id, name, description) {
			this._id = id;
			this._name = name;
			this._description = description;
		};
		
		/**
		 * Get the ID of the attribute category
		 * @returns {String}
		 */
		WebCenterAttributeCategory.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Get the name of the attribute category
		 * @returns {String}
		 */
		WebCenterAttributeCategory.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Get the description of attribute category
		 * @returns {String}
		 */
		WebCenterAttributeCategory.prototype.getDescription = function () {
			return this._description;
		};

		return WebCenterAttributeCategory;
	});

define('webcenterapi/data/WebCenterGroup',["utils/FilterUtils", "underscore"],
	function (FilterUtils, _) {
		"use strict";

		/**
		 * Represents a WebCenter group
		 * @param {String} id - Id of the group
		 * @param {String} name - Name of the group
		 * @param {Array.<WebCenterUser>} members - members of the group
		 */
		var WebCenterGroup = function (id, name, members) {
			this._id = id;
			this._name = name;
			this._members = members;

		};

		/**
		 * Get the ID of the group
		 * @returns {String}
		 */
		WebCenterGroup.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Get the name of the group
		 * @returns {String}
		 */
		WebCenterGroup.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Get the members of a group
		 * @returns {Array.<WebCenterUser>} 
		 */
		WebCenterGroup.prototype.getMembers = function () {
			return this._members;
		};

		/**
		 * Does the group (or one of its members) match a query
		 * @param   {String}  query          
		 * @param   {Boolean} includeMembers - should members be checked as well?
		 * @returns {Boolean} 
		 */
		WebCenterGroup.prototype.match = function (query, includeMembers) {
			var regex = FilterUtils.generateRegex(query, true);

			return this._name.match(regex) || (includeMembers && _.some(this._members, function (member) {
				return member.match(query);
			})) || (this.getGroupAssignee() && this.getGroupAssignee().match(query));
		};

		/**
		 * Get the display icon classes for a group
		 * @returns {string} 
		 */
		WebCenterGroup.getDisplayIcon = function () {
			return "icon-green icon-WCR_Assignee-Group";
		};

		/**
		 * Get the display Element for a group
		 * @param {Boolean} includeIcon
		 * @returns {JQueryObj} 
		 */
		WebCenterGroup.prototype.getDisplayElement = function (includeIcon) {
			var displayElement = $("<span>", {
				text: this.getName()
			});
			if (includeIcon) {
				displayElement.prepend($("<i>", {
					'class': "icon icon-normal " + this.getDisplayIcon()
				}));
			}
			return displayElement;

		};

		/**
		 * Add extra markers for SAHI if needed (they are added after the default SHAI marker)
		 * @param {String} extraMarker - or null to reset extra markers
		 */
		WebCenterGroup.prototype.setExtraSAHIMarkers = function (extraMarker) {
			this._extraMarker = extraMarker;
		};

		/**
		 * Get the SAHI Marker that can be used for this group
		 * @returns {string} [[Description]]
		 */
		WebCenterGroup.prototype.getSAHIMarker = function () {
			var extra = (this._extraMarker ? (" " + this._extraMarker) : "");
			return "WCRGROUP " + this.getName() + extra;
		};

		/**
		 * Set the group assignee
		 * @param {WebCenterUser} groupAssignee 
		 */
		WebCenterGroup.prototype.setGroupAssignee = function (groupAssignee) {
			this._groupAssignee = groupAssignee;
			
			if (!_.find(this._members, function(user){
				return user.getId() === groupAssignee.getId();
			})){
				this.addMember(groupAssignee);	
			}
		};
	
		/**
		 * Get the group assignee
		 * @returns {WebCenterUser} undefined if not set	
		 */
		WebCenterGroup.prototype.getGroupAssignee = function () {
			return this._groupAssignee;
		};

		/**
		 * Add a member to the memberlist
		 * @param {WebCenterUser} member 
		 */
		WebCenterGroup.prototype.addMember = function (member) {
			this._members.push(member);
		};
		return WebCenterGroup;
	});

define('webcenterapi/data/WebCenterRole',["utils/FilterUtils", "underscore"],
	function (FilterUtils, _) {
		"use strict";

		/**
		 * Represents a WebCenter role
		 * @param {String} id - Id of the role
		 * @param {String} name - Name of the role
		 */
		var WebCenterRole = function (id, name, members) {
			this._id = id;
			this._name = name;
			this._members = members;
		};

		/**
		 * Get the ID of the role
		 * @returns {String}
		 */
		WebCenterRole.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Get the name of the role
		 * @returns {String}
		 */
		WebCenterRole.prototype.getName = function () {
			return this._name;
		};


		WebCenterRole.prototype.setMembers = function (members) {
			this._members = members;
		};

		WebCenterRole.prototype.getMembers = function () {
			return this._members;
		};

		WebCenterRole.prototype.match = function (query, includeMembers) {
			var regex = FilterUtils.generateRegex(query, true);
			return this._name.match(regex) || (includeMembers && _.some(this._members, function (member) {
				return member.match(query, false);
			}));
		};

		/**
		 * Add extra markers for SAHI if needed (they are added after the default SHAI marker)
		 * @param {String} extraMarker - or null to reset extra markers
		 */
		WebCenterRole.prototype.setExtraSAHIMarkers = function (extraMarker) {
			this._extraMarker = extraMarker;
		};

		/**
		 * Get the SAHI Marker that can be used for this group
		 * @returns {string} [[Description]]
		 */
		WebCenterRole.prototype.getSAHIMarker = function () {
			var extra = (this._extraMarker ? (" " + this._extraMarker) : "");
			return "WCRROLE " + this.getName() + extra;
		};

		return WebCenterRole;
	});

define('WFEditorWCR/data/WebCenterTextContentElementType',[],
	function () {
		"use strict";

		/**
		 * Represents a WebCenter Text Content Element type
		 * @param {String} id - Id of the text content element type
		 * @param {String} name - Name of the text content element type
		 * @param {String} Category - Category of the text content
		 */
		var WebCenterTextContentElementType = function (id, name, category) {
			this._id = id;
			this._name = name;
			this._category = category;
		};

		/**
		 * Get the ID of the element type
		 * @returns {String}
		 */
		WebCenterTextContentElementType.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Get the name of the element type
		 * @returns {String}
		 */
		WebCenterTextContentElementType.prototype.getName = function () {
			return this._name;
		};
		

		/**
		 * Get the category of the element type
		 * @returns {String}
		 */
		WebCenterTextContentElementType.prototype.getCategory = function () {
			return this._category;
		};

		return WebCenterTextContentElementType;
	});

define('WFEditorWCR/data/WebCenterTextContentLanguage',[],
	function () {
		"use strict";

		/**
		 * Represents a WebCenter Text Content Language
		 * @param {String} id - Id of the text content Language
		 * @param {String} name - Name of the text content Language
		 * @param {String} langFlag - the text content Language flag file name
		 * @param {String} langIndex - Index of the text content Language
		 */
		var WebCenterTextContentLanguage = function (id, name, langFlag, langIndex) {
			this._id = id;
			this._name = name;
			this._langFlag = langFlag;
			this._langIndex = langIndex;
		};

		/**
		 * Get the ID of the Language (in code format for e.g en-US)
		 * @returns {String}
		 */
		WebCenterTextContentLanguage.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Get the flag name of the Language
		 * @returns {String}
		 */
		WebCenterTextContentLanguage.prototype.getName = function () {
			return this._name;
		};
		
		/**
		 * Get the name of the Language flag file name
		 * @returns {String}
		 */
		WebCenterTextContentLanguage.prototype.getLangFlag = function () {
			return this._langFlag;
		};
		
		/**
		 * Get the index of the Language
		 * @returns {String}
		 */
		WebCenterTextContentLanguage.prototype.getLangIndex = function () {
			return this._langIndex;
		};
		
		return WebCenterTextContentLanguage;
	});

define('packaging-content-management/text-content-utils/LanguageUtils',["utils/Map",
        "underscore",
        "jquery"],
	function (Map,
			_,
			$) {
	
		"use strict";
		
		var LanguageUtils = function () {
			
		};
		
		
		LanguageUtils.prototype.removeFrom = function (langs, langsToRemove) {
			for (var i=0; i <langs.length; i++) {
				for (var j=0; j <langsToRemove.length; j++) {
					if (langs[i].langCode === langsToRemove[j].langCode) {
						langs.splice(i, 1); // remove the entry
					}
				}
			}
		};
		
		LanguageUtils.prototype.getLangForDescription = function (langs, langDescription) {
			for (var i=0; i <langs.length; i++) {
				if (langs[i].langDesc === langDescription) {
					return langs[i];
				}
			}
		};
		
		LanguageUtils.prototype.getLangForCode = function (langs, langCode) {
			for (var i=0; i <langs.length; i++) {
				if (langs[i].langCode === langCode) {
					return langs[i];
				}
			}
		};
		
		LanguageUtils.prototype.getLangsForCodes = function (langs, langCodes) {
			var matchedLangs = [];
			for (var i=0; i <langCodes.length; i++) {
				var lang = this.getLangForCode(langs, langCodes[i]);
				if (lang) {
					matchedLangs.push(lang);
				}
			}
			return matchedLangs;
		};
		
		LanguageUtils.prototype.isAProjectAttrLang = function(projectLangs, langCode) {
			if(projectLangs){
				for(var i=0; i<projectLangs.length; i++) {
					if(projectLangs[i].langCode === langCode) {
						return true;
						break;
					}
				}
			}
			return false;
		};
		
		/**
		 * Combines two array of language objects in single Map. <br>
		 * This method is being used basically for merging master and translation languages.
		 * @param masterLanguages {Array of objects}	- 	{langCode : "en-US", langDesc : "English - United States", langFlag : "US.png", langIndex : "1"}
		 * @param translationLanguages {Array of objects}	-	{langCode : "nl-BE", langDesc : "Dutch - Belgium", langFlag : "BE.png", langIndex : "2"}
		 * @returns {Map} - Map  with language code as key and language object as value
		 * 	E.g. key = "en-US" and value = {langDesc : "English - United States", langFlag : "US.png", langIndex : "1"}
		 */
		LanguageUtils.prototype.mergeBothLanguagesTogether = function (masterLanguages, translationLanguages) {
			var completeBothLanguage = new Map();
			
			$(masterLanguages).each(function(lang) {
				completeBothLanguage.add(masterLanguages[lang].langCode, {langCode : masterLanguages[lang].langCode, langDesc : masterLanguages[lang].langDesc, langFlag : masterLanguages[lang].langFlag, langIndex : masterLanguages[lang].langIndex});
			});
			
			$(translationLanguages).each(function(lang) {
				completeBothLanguage.add(translationLanguages[lang].langCode, {langCode : translationLanguages[lang].langCode, langDesc : translationLanguages[lang].langDesc, langFlag : translationLanguages[lang].langFlag, langIndex : translationLanguages[lang].langIndex});
			});
			return completeBothLanguage;
		};
		
		/**
		 * This method will search for language code in map of Languages and returns lang object 
		 * {langCode : "en-US", langDesc : "English - United States", langFlag : "US.png", langIndex : "1"} 
		 * @param langsMap - Map of languages where language code is key and language object is value
		 * @ returns LangObject
		 */
		LanguageUtils.prototype.getLangForCodeFromMap = function (langsMap, languageCode) {
			if(languageCode == "")
				return;
			var langObj ; 
			_.each(langsMap.entries(), _.bind(function(langCode){
				//if languageCode is found in map, set the langObj and return false to break the loop
				if(langCode == languageCode){
					langObj = langsMap.get(langCode);
					return false;
				}
			}));
			
			return langObj;
		};
		
		return new LanguageUtils();
		
});

define('packaging-content-management/text-content-services/languageService',["jquery",
        "utils/AttributeEventBus",
        "packaging-content-management/text-content-utils/LanguageUtils"], function ($, AttributeEventBus, LanguageUtils) {

	
	
	var LanguageService = function () {
		
		// These private variables will cache the returned languages and smartnames
        this._cachedLanguages = {};
        
        // These private variables take care of preventing duplicate requests for project languages.
        this._langsAreBeingRetrieved = {};
        this._onLangsRetrieved = {};
        
        // These private variables take care of preventing duplicate request for all languages.
        this._allLangsAreBeingRetrieved = false;
        this._onAllLangsRetrieved = [];
        
        // Add a listener to update our cache when a language was removed/added from the project.
        
        AttributeEventBus.getInstance().onChangeAttrValueListener(_.bind(function (srcID, attObj) {
        	
        	// Only listen to changes on the "MASTER_TEXT_LANGUAGE" and
			// "TRANSLATION_LANGUAGE" attributes.
			var masterLangChange = false;
			var transLangChange = false;
			if ('MASTER_TEXT_LANGUAGE' == attObj.name) {
				masterLangChange = true;
			} else if ('TRANSLATION_LANGUAGE' == attObj.name) {
				transLangChange = true;
			}

			// If the master or translation attribute changed ...
			if (!masterLangChange && !transLangChange) {
				return; // No master or translation language change.
			}
        	
			// Get the project ID from the srcID.
			var projectID = undefined;
        	var result = srcID.match(/projectAttrUID:(.*)_attr:/);
        	if (result && result.length > 1) {
        		projectID = result[1];
        	}
        	
        	// So only continue if this is an update to a project master or language attribute.
        	if (!projectID) {
        		return;
        	}
			
        	// Check if we have some cached languages for this project.
        	
			if(this._cachedLanguages[projectID]) {
				
				// Determine wether we need to work on Master/Translation text.
				var projLangs = [];
				var allLangs = [];
				if (masterLangChange) {
					projLangs = this._cachedLanguages[projectID].master;
					allLangs = this._cachedLanguages.all.master;
				} else if (transLangChange) {
					projLangs = this._cachedLanguages[projectID].translations;
					allLangs = this._cachedLanguages.all.translations;
				}
				
				// Get the descriptions of the new languages from the attribute.
				var newLangDesc = attObj.value.split(" | ");

				// Figure out which languages were removed.
				var removedLang = [];
				$.each(projLangs, function(index, lang) {
					for ( var i = 0; i < newLangDesc.length; i++) {
						if (lang.langDesc == newLangDesc[i]) {
							return;
						}
					}
					removedLang.push(lang);
				}.bind(this));

				// Figure out which language descriptions were added.
				var addedLangDesc = [];
				for ( var i = 0; i < newLangDesc.length; i++) {
					var langFound = false;
					$.each(projLangs, function(index, lang) {
						if (lang.langDesc == newLangDesc[i]) {
							langFound = true;
							return;
						}
					});
					if (!langFound) {
						addedLangDesc.push(newLangDesc[i]);
					}
				}
				// Let's start by mapping the passed language descriptions to real language objects.
				var addedLang = [];
				for (var i=0; i < addedLangDesc.length;i++) {
					var lang = LanguageUtils.getLangForDescription(allLangs, addedLangDesc[i]);
					if (lang) {
						addedLang.push(lang);
					}
				}
				
				// Now add or remove them to the project languages.
				if(addedLang.length > 0) {
					if (masterLangChange) {
						this._cachedLanguages[projectID].master = projLangs.concat(addedLang);
					} else if (transLangChange) {
						this._cachedLanguages[projectID].translations = projLangs.concat(addedLang);
					}
					
				}
				if(removedLang.length > 0) {
					if (masterLangChange) {
						LanguageUtils.removeFrom(this._cachedLanguages[projectID].master, removedLang);
					} else if (transLangChange) {
						LanguageUtils.removeFrom(this._cachedLanguages[projectID].translations, removedLang);
					}
				}
			}
        }, this));
        
	};
	

	/**
	 * Return the project master or translation languages for a certain text content document.
	 * 
	 * @param projectId
	 * @param isMaster
	 * @param onProjectLangsCallback
	 * @returns
	 */
	LanguageService.prototype.getProjectLanguages = function (projectId, isMaster, onProjectLangsCallback) {
		if (this._cachedLanguages[projectId]) {
			// We have a cache entry for that projectId, return from cache.
			this._returnCachedLanguagesFor(projectId, isMaster, true, onProjectLangsCallback);
		} else {
			// We don't have a cache entry, this means that they still need to be fetched.
			// Execute the "onProjectLangsCallback" when the're retrieved by adding it to the "this._onLangsRetrieved" array.
			if (!this._onLangsRetrieved[projectId]) {
				this._onLangsRetrieved[projectId] = [];
			}
			this._onLangsRetrieved[projectId].push(_.bind(function () {
				this._returnCachedLanguagesFor(projectId, isMaster, true, onProjectLangsCallback);
			}, this));
			
			// Fetch the languages.
			this._getLanguages(projectId);
		}
	};

	
	/**
	 * Return all the master or translation languages for whole WebCenter.
	 * 
	 * @param projectId
	 * @param isMaster
	 * @param onAllLangsCallback
	 * @returns
	 */
	LanguageService.prototype.getAllLanguages = function (projectId, isMaster, onAllLangsCallback) {
		if (this._cachedLanguages.all) {
			// We have a cache entry for that projectId, return from cache.
			this._returnCachedLanguagesFor(projectId, isMaster, false, onAllLangsCallback);
		} else {
			// We don't have a cache entry, this means that they still need to be fetched.
			// Execute the "onProjectLangsCallback" when the're retrieved by adding it to the "this._onLangsRetrieved" array.
			if (!this._onLangsRetrieved[projectId]) {
				this._onLangsRetrieved[projectId] = [];
			}
			this._onLangsRetrieved[projectId].push(_.bind(function () {
				this._returnCachedLanguagesFor(projectId, isMaster, false, onAllLangsCallback);
			}, this));
			
			// Fetch the languages.
			this._getLanguages(projectId);
		}
	};
	
	/**
	 * Get both the translation and master languages.
	 */
	LanguageService.prototype.getBothAllLanguages = function (onAllLangsCallback) {
		
		if (this._cachedLanguages.all){
			onAllLangsCallback(this._cachedLanguages.all.master, this._cachedLanguages.all.translations);
		} else {
			
			// We don't have a cache entry, this means that they still need to be fetched.
			// Execute the "onProjectLangsCallback" when the're retrieved by adding it to the "this._onLangsRetrieved" array.
			this._onAllLangsRetrieved.push(_.bind(function () {
				onAllLangsCallback(this._cachedLanguages.all.master, this._cachedLanguages.all.translations);
			}, this));
			
			// We need to fetch all the languages.
			this._getAllLanguages();
		}
	};
	
	/**
	 * A utility function to return some cached languages.
	 * 
	 * @param projectId
	 * @param isMaster
	 * @param onlyProject
	 * @param onLangsCallback
	 * @returns
	 */
	LanguageService.prototype._returnCachedLanguagesFor = function (projectId, isMaster, onlyProject, onLangsCallback) {
		
		// Check if need for all the languages or just the project languages.
		if (onlyProject) {
			// Get the cached languages for that project.
			var docVerLang =  this._cachedLanguages[projectId];
			if (isMaster) {
				onLangsCallback(docVerLang.master);
			} else {
				onLangsCallback(docVerLang.translations);
			}
		} else {
			// Get the cached languages for whole WebCenter.
			var allLang = this._cachedLanguages.all;
			if (isMaster) {
				onLangsCallback(allLang.master);
			} else {
				onLangsCallback(allLang.translations);
			}
		}
	};
	
	/**
	 * 
	 * @param projectId
	 * @param onLangsCallback
	 * @param textContentsLangs - In Viewer TextContentWidget when no Project language attributes are present then the languages in Textcontents are considered
	 */
	LanguageService.prototype.getLanguages = function(projectId, onLangsCallback, textContentsLangs) {
		
		if (this._cachedLanguages.all && this._cachedLanguages[projectId]) {
			// We have a cache entry for these languages.
			var allLang = this._cachedLanguages.all;
			var docVerLang =  this._cachedLanguages[projectId];
			onLangsCallback(docVerLang.master, docVerLang.translations, allLang.master, allLang.translations);
		} else {
			// We don't have a cache entry for the languages, this means that they still need to be fetched.
			// Execute the "onLangsCallback" when the're retrieved by adding it to the "this._onLangsRetrieved" array.
			if (!this._onLangsRetrieved[projectId]) {
				this._onLangsRetrieved[projectId] = [];
			}
			this._onLangsRetrieved[projectId].push(_.bind(function () {
				var allLang = this._cachedLanguages.all;
				var docVerLang =  this._cachedLanguages[projectId];
				onLangsCallback(docVerLang.master, docVerLang.translations, allLang.master, allLang.translations);
			}, this));
			
			// Fetch the languages.
			this._getLanguages(projectId, textContentsLangs);
		}
	};
	
	
	/**
	 * Make a language retrieval request to WebCenter.
	 */
	LanguageService.prototype._getLanguages = function (projectId, languages) {
    	
		// Check if we're not already retrieving the languages for this projectId.
		if (this._langsAreBeingRetrieved[projectId] === undefined || this._langsAreBeingRetrieved[projectId] === false) {
			
    		// Indicate that they're being fetched now for this projectId 
			this._langsAreBeingRetrieved[projectId] = true;
    		
			// The baseUrl of the service.
	        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
	        var url = jspBaseUrl+"GetProjectLanguageAttr.jsp?json=1";
	        
	        // Project id can null or undefined in case of attribute category setup and project creation page. 
	        if(projectId) {
	        	url += "&projectid=" + projectId;
	        }
	        
	        if(languages) {
	        	url += "&languages=" + languages;
	        }
	        
    		// Do the async call to the service
    		$.getJSON(url, _.bind(function( data ) {
    		
    			// Parse the json string.
    			this._cachedLanguages.all = {};
    			this._cachedLanguages.all.master = this._parseLangs(data.languages.complete_source_language.language);
    			this._cachedLanguages.all.translations = this._parseLangs(data.languages.complete_translation_language.language);
    			this._cachedLanguages[projectId] = {};
    			
    			if (data.languages.selected_source_language) {
        			this._cachedLanguages[projectId].master = this._parseLangs(data.languages.selected_source_language.language);
    			} else {
    				this._cachedLanguages[projectId].master = [];
    			}
    			
    			if (data.languages.selected_translation_language) {
    				this._cachedLanguages[projectId].translations = this._parseLangs(data.languages.selected_translation_language.language);
    			} else {
    				this._cachedLanguages[projectId].translations = [];
    			}
    			
    			// Execute the onLangsRetrieved callbacks.
    			var callbacks = this._onLangsRetrieved[projectId];
    			if (callbacks) {
    				for (var i=0; i < callbacks.length; i++) {
        				callbacks[i]();
        			}
    			}
    			
        	}, this)).fail(function(jqxhr, textStatus, error) {
        		
        		
        	});
        }
	};
	
	
	/**
	 * Make a language retrieval request to WebCenter without providing prjectId. <br>
	 * Useful for those functionality where projectId is not available like- Search.
	 */
	LanguageService.prototype._getAllLanguages = function () {
		
		if (this._allLangsAreBeingRetrieved === false) {
			
			// Indicate that they're being fetched now 
			this._allLangsAreBeingRetrieved = true;
    		
			// The baseUrl of the service.
	        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
			
    		// Do the async call to the service
    		$.getJSON( jspBaseUrl+"GetProjectLanguageAttr.jsp?json=1", _.bind(function( data ) {
    			
    			// Parse the json string.
    			this._cachedLanguages.all = {};
    			this._cachedLanguages.all.master = this._parseLangs(data.languages.complete_source_language.language);
    			this._cachedLanguages.all.translations = this._parseLangs(data.languages.complete_translation_language.language);
    			
    			// Execute the onLangsRetrieved callbacks.
    			for (var i=0; i < this._onAllLangsRetrieved.length; i++) {
    				this._onAllLangsRetrieved[i]();
        		}
  
    			
    			this._allLangsAreBeingRetrieved = false;
    			
    		}, this)).fail(function(jqxhr, textStatus, error) {
        		
        		
        	});
		}
	};
	
	/**
	 * Make a language retrieval request to WebCenter.
	 */
	LanguageService.prototype._parseLangs = function (langsFromXml) {
		var langs = [];
		
		if (langsFromXml instanceof Array) {
			for (var i=0; i <langsFromXml.length; i++) {
				var lang = {};
				lang.langCode = langsFromXml[i].language_code;
				lang.langDesc = langsFromXml[i].descriptor;
				lang.langFlag = langsFromXml[i].flag_file_name;
				lang.langIndex = langsFromXml[i].language_index;
				langs.push(lang);
			}
		} else if (langsFromXml instanceof Object) {
			var lang = {};
			lang.langCode = langsFromXml.language_code;
			lang.langDesc = langsFromXml.descriptor;
			lang.langFlag = langsFromXml.flag_file_name;
			lang.langIndex = langsFromXml.language_index;
			langs.push(lang);
		}
		
		return langs;
	};
	
    return new LanguageService();

});

define('WFEditorWCR/data/WebCenterAPI',["webcenterapi/AbstractAPI",
		"webcenterapi/AttributesAPI",
		"webcenterapi/TaskTypeAPI",
		"WFEditorWCR/core/WebCenterWFEditorConstants",
		"WFEditorWCR/workflow/WebCenterWorkflowState",
		"WFEditorWCR/data/WebCenterAttributeCategory",
		"webcenterapi/data/WebCenterUser",
		"webcenterapi/data/WebCenterGroup",
		"webcenterapi/data/WebCenterRole",
		"WFEditorWCR/data/WebCenterTextContentElementType",
		"WFEditorWCR/data/WebCenterTextContentLanguage",
		"packaging-content-management/text-content-services/languageService",
		"packaging-content-management/text-content-utils/LanguageUtils",
		"packaging-content-management/ContentConstants",
		"utils/Cache",
		"utils/HTTPUtils",
		"utils/XMLUtils",
		"utils/Map",
		"component/loadingindicator/LoadingIndicatorSingleton",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery"
	],
	function (
		AbstractAPI,
		AttributesAPI,
		TaskTypeAPI,
		WebCenterWFEditorConstants,
		WebCenterWorkflowState,
		WebCenterAttributeCategory,
		WebCenterUser,
		WebCenterGroup,
		WebCenterRole,
		WebCenterTextContentElementType,
		WebCenterTextContentLanguage,
		languageService,
		LanguageUtils,
		ContentConstants,
		Cache,
		HTTPUtils,
		XMLUtils,
		Map,
		LoadingIndicator,
		i18n, 
		_,
		$) {
		"use strict";

		/**
		 * Utility class used for all communication with WebCenter
		 * @exports WebCenterAPI
		 * @class
		 * 
		 * @param {Object} wcrConfig - webcenter configuration object
		 * @param {ErrorHandler} errorHandler
		 */
		var WebCenterAPI = function (wcrConfig, errorHandler) {
			AbstractAPI.call(this, wcrConfig, errorHandler);

			/**
			 * @member {Number} interval ID (value used by setInterval to pauze/stop polling
			 * @private
			 */
			this._keepSessionAlive = null;

			/**
			 * @member {Promise.<XMLDocument>} Promise for the WorkflowDefinition call
			 * @private
			 */
			this._workflowDefinitionsPromise = null;

			/**
			 * @member {Promise.<Map.<WebCenterAttributeCategory>>} Promise for the WebCenterAttributeCategories
			 * @private
			 */
			this._deferredAttributeCategoriesPromise = null;

			/**
			 * @member {Promise.<Map.<XMLDocument>>} Promise for the call for fetching attribute categories
			 * @private
			 */
			this._attributeCategoriesPromise = null;

			/**
			 * @member {AttributesAPI} API for fetching attributes stuff
			 * @private
			 */
			this._attributesAPI = new AttributesAPI(wcrConfig, errorHandler);

			/**
			 * @member {TaskTypeAPI} API for fetching task type stuff
			 * @private
			 */
			this._taskTypeAPI = new TaskTypeAPI(wcrConfig, errorHandler);

			/**
			 * @member {Promise.<Map.<WebCenterUser>, Map.<WebCenterGroup>, Map.<WebCenterRole>>} Promise for users, groups and roles
			 * @private
			 */
			this._deferredUsersGroupsRolesPromise = null;
			/**
			 * @member {Promise.<XMLDocument>} Promise for the call for fetching users, groups and roles
			 * @private
			 */
			this._usersGroupsRolesPromise = null;

			/**
			 * @member {Boolean} is a save operation ongoing?
			 * @private 
			 */
			this._saveOngoing = false;
			/**
			 * @member {DocVersionViewableCache} cache for checking whether a document is viewable
			 * @private
			 */
			this._docVersionViewableCache = null;
			/**
			 * @member {Promise.<Map.<WebCenterProjectStatus>>} promise for the list of WebCenter statuses
			 * @private 
			 */
			this._projectStatusListPromise = null;

			/**
			 * @member {Promise.<Map.<WebCenterWorkflowConnection>> promise for the list of WebCenter workflow connections
			 * @private
			 */
			this._workflowConnectionsPromise = null;
			/**
			 * @member {Promise.<Map.<WebCenterTextContentElementType>>} promise for the list of WebCenter text content element types
			 * @private 
			 */
			this._textContentElementTypesPromise = null;

			/**
			 * @member {Promise.<Map.<WebCenterTextContentElementType>>} promise for the list of WebCenter table content element types
			 * @private 
			 */
			this._tableContentElementTypesPromise = null;

			/**
			 * @member {Promise.<Map.<WebCenterTextContentLanguage>>} promise for the list of WebCenter text content languages
			 * @private 
			 */
			this._textContentLangListPromise = null;

			/**
			 * @member {Cache} promised cache for task type definitions
			 * @private
			 */
			this._taskTypeDefinitionCache = new Cache(30, false);
			this._taskTypeDefinitionCache._loadItem = _.bind(this._loadTaskTypeDefinition, this);

			/**
			 * @member {Cache} promised cache for task type specifications
			 * @private
			 */
			this._taskTypeSpecificationsCache = new Cache(30, false);
			this._taskTypeSpecificationsCache._loadItem = _.bind(this._loadTaskTypeSpecifications, this);
		};

		WebCenterAPI.prototype = Object.create(AbstractAPI.prototype);
		WebCenterAPI.prototype.constructor = WebCenterAPI;

		/**
		 * Get the xml description of the workflow of a task type
		 * @param {String} taskTypeID 
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise for the workflowXML
		 */
		WebCenterAPI.prototype.getTaskTypeWorkflowXML = function (taskTypeID) {
			var deferred = $.Deferred();
			this.callSDK("DownloadWorkflowXML.jsp", {
				tasktypeid: taskTypeID
			}).fail(_.bind(function (jqXHR, textStatus, errorThrown, codeString) {
				if (codeString === "151") { // empty task type -> resolve with an empty xml
					var xml = document.implementation.createDocument("", "", null);
					var root = xml.createElement("root");
					xml.appendChild(root);
					deferred.resolve(xml, textStatus, jqXHR);
				} else {
					deferred.reject(jqXHR, textStatus, errorThrown);
					this._errorHandler.severe(i18n.t("LoadWorkflowFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				}
			}, this)).done(function (xml, textStatus, jqXHR) {
				deferred.resolve(xml, textStatus, jqXHR);
			});
			return deferred.promise();
		};


		/**
		 * Get the xml description of the workflow of a task
		 * @param {String} taskID 
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR> Promise for the workflowXML
		 */
		WebCenterAPI.prototype.getTaskWorkflowXML = function (taskID) {
			return this.callSDK("DownloadWorkflowXML.jsp", {
				taskid: taskID
			}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._errorHandler.severe(i18n.t("LoadWorkflowFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
			}, this));
		};

		/**
		 * Get the WorkflowDefinitions XML
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise for the WorkflowDefinitions
		 */
		WebCenterAPI.prototype.getWorkflowDefinitions = function () {
			if (!this._workflowDefinitionsPromise) {

				this._workflowDefinitionsPromise = this.callSDK("GetWFActions.jsp", {}, {
					cache: true
				}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
					this._errorHandler.severe(i18n.t("LoadWorkflowDefinitionsFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				}, this));
			}

			return this._workflowDefinitionsPromise;
		};

		/**
		 * Get the TaskTypeDefinitions XML
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise for the TaskTypeDefinitions
		 */
		WebCenterAPI.prototype.getTaskTypeDefinitions = function () {
			return this._taskTypeAPI.getTaskTypeDefinitions();
		};

		/**
		 * Used by the TaskTypeDEfinition Cache to fecth information on a task type
		 * @param {String} taskTypeID - id of the TaskType
		 */
		WebCenterAPI.prototype._loadTaskTypeDefinition = function (taskTypeID) {

			var promise = this.callSDK("GetTaskTypes.jsp", {
				verbose: "1",
				loadicons: "1",
				tasktypeid: taskTypeID
			}, {
				cache: true
			});

			promise.fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._errorHandler.severe(i18n.t("LoadTaskTypesFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				this._taskTypeDefinitionCache.remove(taskTypeID);
			}, this));

			this._taskTypeDefinitionCache._registerItem(taskTypeID, promise);
		};

		/**
		 * Get the promise for fetching the data of the current task type, includes the task statuses
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise for tasktype details
		 */
		WebCenterAPI.prototype.getTaskTypeDefinition = function (taskTypeID) {
			return this._taskTypeDefinitionCache.get(taskTypeID);
		};

		/**
		 * Get the latest version of the WorkflowReport
		 * @param {String} taskID - id of the task for which the report should be fetched
		 *                        @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise for the workflow report
		 */
		WebCenterAPI.prototype.getWorkflowReport = function (taskID) {
			if (taskID) {
				var deferred = $.Deferred();
				this.callSDK("DownloadWorkflowReportXML.jsp", {
					taskid: taskID,
				}).fail(_.bind(function (jqXHR, textStatus, errorThrown, codeString, xml) {
					if (codeString === "154") {
						// there is just no report (e.g. still in Created status), let the feedback handle it
						deferred.resolve(xml, textStatus, jqXHR);
					} else {
						deferred.reject(jqXHR, textStatus, errorThrown);
						this._errorHandler.severe(i18n.t("LoadWorkflowReportFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
					}
				}, this)).done(function (xml, textStatus, jqXHR) {
					deferred.resolve(xml, textStatus, jqXHR);
				});
				return deferred.promise();
			} else {
				return null;
			}
		};

		/**
		 * Keep the session alive in WebCenter
		 * @param {Boolean} keepAlive - true: start keeping session alive, false: stop polling for session alive
		 */
		WebCenterAPI.prototype.keepSessionAlive = function (keepAlive) {
			if (!keepAlive && this._keepSessionAlive !== null) {
				clearInterval(this._keepSessionAlive);
				this._keepSessionAlive = null;
			} else if (keepAlive && this._keepSessionAlive === null) {
				var sessionAliveUrl = this.constructURL("sessionalive.jsp", {});


				this._keepSessionAlive = setInterval(_.bind(function () {
					$.ajax(sessionAliveUrl, {
						dataType: "xml",
						cache: true
					}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
						this._errorHandler.severe(i18n.t("WCSessionLostError") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
					}, this));
				}, this), 300000); // every 5 minutes
			}
		};


		/**
		 * Save a workflow
		 * @param {workflow} the workflow to save
		 * @returns {$.Deferred.<>} resolves when workflow is succesfully saved
		 */
		WebCenterAPI.prototype.saveWorkflow = function (workflow) {
			var deferred = $.Deferred();

			if (!this._saveOngoing) {
				this._saveOngoing = true;
				LoadingIndicator.start("WFEditorWCRSaveWorkflow", i18n.t("savingWorkflowDefinition"));

				var workflowXML = workflow.toXML();

				// generate workflow xml and submit it
				$.ajax(HTTPUtils.constructURL("UploadWorkflowXML.jsp", {}), {
					method: "POST",
					data: {
						tasktypeid: workflow.getTaskTypeID(),
						workflowxmlstring: workflowXML,
						dataType: "xml"
					}
				}).done(_.bind(function (xml) {
					var message;

					// request was succesful, check if the save was successful
					var result = XMLUtils.xml2obj(xml);
					if (result && result.task_type_id === workflow.getTaskTypeID()) {
						//workflow correctly saved
						deferred.resolve();
					} else {
						// error while saving... Concatenate all error information
						if (result.error) {
							var errors;
							if (_.isArray(result.error)) {
								errors = result.error;
							} else {
								errors = [result.error];
							}

							var errorCodes = [];
							var errorMessages = [];
							_.each(errors, function (error) {
								errorCodes.push(error.code);
								errorMessages.push(error.message);
							});

							message = "(" + errorCodes.join(", ") + ")";
							message += " " + errorMessages.join(", ");
						}

						// trigger error
						this._errorHandler.severe(i18n.t("workflowSaveError", message), result);

						deferred.reject();
					}
				}, this)).fail(_.bind(function (xml, textStatus, errorThrown) {
					// error while saving
					this._errorHandler.severe(i18n.t("workflowSaveError", textStatus), errorThrown);

					deferred.reject();
				}, this)).always(_.bind(function () {
					// update save button
					this._saveOngoing = false;
					LoadingIndicator.stop("WFEditorWCRSaveWorkflow");
				}, this));
			} else {
				deferred.reject();
			}

			return deferred.promise();
		};

		/**
		 * Is a save operation ongoing?
		 * @returns {Boolean} 
		 */
		WebCenterAPI.prototype.isSaveOngoing = function () {
			return this._saveOngoing;
		};

		/**
		 * Hold the workflow, this pauses the workflow. Only possible if the workflow is running.
		 * @returns {$.Deferred.<>} resolves when workflow is held
		 */
		WebCenterAPI.prototype.holdWorkflow = function (workflow) {
			var result = $.Deferred();

			var promise = this._getWFControlsPromise(WebCenterWFEditorConstants.WorkflowControlActions.HOLD, workflow.getTaskID());

			promise.fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._errorHandler.severe(i18n.t("holdWorkflowFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				result.reject();
			}, this)).done(_.bind(function (xmlResponse) {
				//first check if the hold call was successful
				var response = XMLUtils.xml2obj(xmlResponse);
				if (response && response.success == "1") {
					result.resolve();
				} else {
					this._errorHandler.severe(i18n.t("holdWorkflowFailed"));
					result.reject();
				}
			}, this));

			return result.promise();
		};

		/**
		 * Resume the workflow, only possible if the workflow is paused
		 * @returns {$.Deferred.<>} resolves when workflow is resumed
		 */
		WebCenterAPI.prototype.resumeWorkflow = function (workflow) {
			var result = $.Deferred();
			var promise = this._getWFControlsPromise(WebCenterWFEditorConstants.WorkflowControlActions.RESUME, workflow.getTaskID());

			promise.fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._errorHandler.severe(i18n.t("resumeWorkflowFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				result.reject();
			}, this)).done(_.bind(function (xmlResponse) {
				var response = XMLUtils.xml2obj(xmlResponse);
				if (response && response.success == "1") {
					result.resolve();
				} else {
					this._errorHandler.severe(i18n.t("resumeWorkflowFailed"));
					result.reject();
				}
			}, this));

			return result;
		};

		/**
		 * Cancel the workflow, only possible if the workflow is paused or in progress.
		 * @returns {$.Deferred.<>} resolves when workflow is cancelled
		 */
		WebCenterAPI.prototype.cancelWorkflow = function (workflow) {
			var result = $.Deferred();
			var promise = this._getWFControlsPromise(WebCenterWFEditorConstants.WorkflowControlActions.CANCEL, workflow.getTaskID());
			workflow.setWorkflowState(WebCenterWorkflowState.STOPPING);
			workflow.setTaskStatus({
				status: i18n.t("workflowStatusStopping")
			});

			promise.fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._errorHandler.severe(i18n.t("cancelWorkflowFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
				result.reject();
			}, this)).done(_.bind(function (xmlResponse) {
				var response = XMLUtils.xml2obj(xmlResponse);
				if (response && response.success == "1") {
					result.resolve();
				} else {
					this._errorHandler.severe(i18n.t("cancelWorkflowFailed"));
					result.reject();
				}
			}, this));

			return result.promise();
		};

		/**
		 * Get the WF control action promise for the given action
		 * @param   {WebCenterWFEditorConstants.WorkflowControlActions} action
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>}
		 */
		WebCenterAPI.prototype._getWFControlsPromise = function (action, taskID) {
			return this.callSDK("GetWFActions.jsp", {
				taskid: taskID,
				action: "updatewfstate",
				state: action
			});
		};

		/**
		 * Get the URL to open the viewer for a document
		 * @param   {String} documentVersionID - document version ID of the document to view
		 * @returns {String} URL to open viewer
		 */
		WebCenterAPI.prototype.getViewDocumentURL = function (documentVersionID) {
			return this.constructURL("viewdocument.jsp", {
				docVerID: documentVersionID
			});
		};

		/**
		 * Retry a certain node
		 * @param {Workflow} workflow - the workflow
		 * @param {WebCenterToken}    token    - token which should be retried
		 */
		WebCenterAPI.prototype.retryToken = function (workflow, token) {

			var retryPromise = this.callSDK("RelaunchWorkflow.jsp", {
				taskid: workflow.getTaskID(),
				token: token.getErrorHandlingId(),
				type: "retry"
			}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._errorHandler.severe(i18n.t("wfretry.failed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
			}, this));

			$.when(retryPromise).done(_.bind(function (result) {
				var resultObj = XMLUtils.xml2obj(result);

				if (resultObj && resultObj.taskid === "") {
					this._errorHandler.info(i18n.t("wfretry.success"));
				} else {
					this._errorHandler.warning("Unexpected returned result");
				}
			}, this));

		};


		/**
		 * Relaunch the owrkflow on a certain token
		 * @param   {Workflow}       workflow - workflow to relaunch
		 * @param   {WebCenterToken} token    - token on which the workflow should be relaunched
		 * @returns {$.Deferred.<String>}  resolved with new TaskID when workflow is relaunch
		 */
		WebCenterAPI.prototype.relaunchWorkflow = function (workflow, token) {
			var deferred = $.Deferred();


			if (token && _.size(token.getParents()) === 1 && token.getParents()[0].getId()) {
				var retryPromise = this.callSDK("RelaunchWorkflow.jsp", {
					taskid: workflow.getTaskID(),
					token: token.getParents()[0].getId(),
					type: "relaunch"
				}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
					this._errorHandler.severe(i18n.t("wfrelaunch.failed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
					deferred.reject();
				}, this));

				$.when(retryPromise).done(_.bind(function (result) {
					var resultObj = XMLUtils.xml2obj(result);

					if (resultObj && resultObj.taskid) {
						deferred.resolve(resultObj.taskid);
					} else {
						this._errorHandler.warning("Unexpected returned result");
						deferred.reject();
					}
				}, this));

			} else {
				this._errorHandler.severe(i18n.t("wfrelaunch.failed") + " Token not found.");
				deferred.reject();
			}

			return deferred.promise();
		};

		/**
		 * Check if a document is viewable
		 * @param {String}   documentVersionID - document version ID
		 * @param {Function} callback          - callback called when document viewable status is known
		 */
		WebCenterAPI.prototype.isDocumentViewable = function (documentVersionID, callback) {
			// only load the DocVersionViewableCache when needed
			requirejs(["WFEditorWCR/data/DocVersionViewableCache"], _.bind(function (DocVersionViewableCache) {
				if (!this._docVersionViewableCache) {
					this._docVersionViewableCache = new DocVersionViewableCache(this, this._errorHandler);
				}

				this._docVersionViewableCache.get(documentVersionID, callback);
			}, this));
		};

		/**
		 * Get all WebCenter attribute category definitions
		 * @returns {$.Deferred.<Map.<WebCenterAttributeCategory>} Promise for the webcenter attribute categories
		 */
		WebCenterAPI.prototype.getAttributeCategoryList = function () {
			if (!this._deferredAttributeCategoriesPromise) {
				var deferred = $.Deferred();
				this._deferredAttributeCategoriesPromise = deferred.promise();

				if (!this._attributeCategoriesPromise) {

					this._attributeCategoriesPromise = this.callSDK("GetAttributeCategoryList.jsp", {}, {
						cache: true
					}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
						this._errorHandler.severe(i18n.t("LoadAttributeCategoriesFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
						deferred.reject();
					}, this));
				}

				$.when(this._attributeCategoriesPromise).done(function (result) {
					var attributeCategoriesMap = new Map();
					//Add an option for "No Attribute Category" in the list
					attributeCategoriesMap.add("", new WebCenterAttributeCategory("", i18n.t('wfui.noattcat'), ""));

					if (result && result.documentElement) {
						var rootNode = result.documentElement;
						var id, name, description;

						var attCats = rootNode.getElementsByTagName("category");
						for (var i = 0; i < attCats.length; i++) {
							var attCat = attCats.item(i);

							id = attCat.getAttribute("id");
							name = attCat.getElementsByTagName("name")[0];
							description = attCat.getElementsByTagName("description")[0];

							if (id && name) {
								attributeCategoriesMap.add(id, new WebCenterAttributeCategory(id, name.textContent, description.textContent));
							}
						}
					}

					deferred.resolve(attributeCategoriesMap);
				});
			}
			return this._deferredAttributeCategoriesPromise;
		};

		/**
		 * Get all WebCenter attribute definitions
		 * @returns {$.Deferred.<Map.<WebCenterAttribute>} Promise for the webcenter attributes
		 */
		WebCenterAPI.prototype.getAttributes = function () {
			return this._attributesAPI.getAttributes();
		};


		/**
		 * Get all WebCenter specification definitions
		 * @returns {$.Deferred.<Map.<WebCenterAttribute>} Promise for the webcenter specifiations
		 */
		WebCenterAPI.prototype.getSpecifications = function () {
			return this._attributesAPI.getSpecifications();
		};

		/**
		 * Get all document reference attributes
		 * @returns {$.Deferred.<Map.<id, WebCenterAttribute>>}
		 */
		WebCenterAPI.prototype.getDocRefAttributes = function () {
			return this._attributesAPI.getDocRefAttributes();
		};

		/**
		 * Get task specifications
		 * @param {String} taskTypeID - The task type ID to get the specifications from
		 * @returns {$.Deferred.<Map.<WebCenterAttribute>>} Promise for the specifications
		 */
		WebCenterAPI.prototype.getTaskSpecifications = function (taskTypeID) {
			return this._taskTypeSpecificationsCache.get(taskTypeID);
		};

		/**
		 * Load specifications of a task type
		 * @param {String} taskTypeID - Id of the task type
		 */
		WebCenterAPI.prototype._loadTaskTypeSpecifications = function (taskTypeID) {
			var deferred = $.Deferred();

			$.when(this.getTaskTypeDefinition(taskTypeID), this.getSpecifications()).done(function (taskTypeDefinition, attributes) {
				var taskTypeDefinitionXML = taskTypeDefinition[0];
				var specifications = new Map();
				var specificationViews = taskTypeDefinitionXML.querySelectorAll("wctask_type_view[id='" + taskTypeID + "'] specification_list task_type_specification_view");
				for (var i = 0; i < specificationViews.length; i++) {
					var spec = specificationViews[i].querySelector("task_type_specification");
					var attrid = spec.querySelector("attr_id").textContent;
					var attribute = attributes.get(attrid);
					specifications.add(attrid, attribute);
				}
				deferred.resolve(specifications);
			});

			var promise = deferred.promise();

			this._taskTypeSpecificationsCache._registerItem(taskTypeID, promise);
		};

		/**
		 * Get the users groups and roles
		 * @returns {$.Deferred.<Map.<WebCenterUser>, Map.<WebCenterGroup>, Map.<WebCenterRole>>} Promise for users, groups and roles.
		 */
		WebCenterAPI.prototype.getUsersGroupsRoles = function () {
			if (!this._deferredUsersGroupsRolesPromise) {
				var deferred = $.Deferred();
				this._deferredUsersGroupsRolesPromise = deferred.promise();

				if (!this._usersGroupsRolesPromise) {

					this._usersGroupsRolesPromise = this.callSDK("GetPotentialProjectMembers.jsp", {
						fullname: 0
					}, {
						cache: true
					}).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
						this._errorHandler.severe(i18n.t("LoadUsersGroupsRolesFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
						deferred.reject();
					}, this));
				}


				$.when(this._usersGroupsRolesPromise).done(function (result) {
					var users = new Map();
					var groups = new Map();
					var roles = new Map();

					if (result && result.documentElement) {
						var rootNode = result.documentElement;

						var entries = rootNode.getElementsByTagName("entry");
						for (var i = 0; i < entries.length; i++) {
							var entry = entries.item(i);

							var sort = entry.getElementsByTagName("sort")[0];
							var id = entry.getElementsByTagName("id")[0];
							var name = entry.getElementsByTagName("name")[0];

							if (sort && id && name) {
								id = id.textContent.substring(2); // skip u_, g_
								sort = sort.textContent;
								name = name.textContent;

								if (sort.indexOf("1") === 0) {
									// users
									users.add(id, new WebCenterUser(id, name));
								} else if (sort.indexOf("2") === 0) {
									// roles
									roles.add(id, new WebCenterRole(id, name));
								} else if (sort.indexOf("3") === 0) {
									// groups
									groups.add(id, new WebCenterGroup(id, name));
								}
							}
						}

					}
					deferred.resolve(users, groups, roles);
				});
			}

			return this._deferredUsersGroupsRolesPromise;
		};

		/**
		 * Get the list of Project Statuses installed on the system
		 * @returns {$.Deferred.<Map.<WebCenterProjectStatus>>} promise for the project status list
		 */
		WebCenterAPI.prototype.getProjectStatusList = function () {
			var promise = this._projectStatusListPromise;
			if (!promise) {
				var statusListDeferred = $.Deferred();

				this.getWorkflowDefinitions().done(function (wfactions) {
					requirejs(["WFEditorWCR/data/WebCenterProjectStatus"], function (WebCenterProjectStatus) {
						// parse xml for statuses and resolve the result promise
						var projectStatusXML = wfactions.getElementsByTagName("ProjectStatuses");
						var statusList = new Map();
						if (projectStatusXML.length > 0) {
							_.each(projectStatusXML[0].getElementsByTagName("ProjectStatus"), function (statusXML) {
								var status = WebCenterProjectStatus.fromXML(statusXML);
								statusList.add(status.getId(), status);
							});

							statusListDeferred.resolve(statusList);
						}

						statusListDeferred.fail();
					});
				}).fail(function () {
					statusListDeferred.fail.apply(statusListDeferred, arguments);
				});

				promise = statusListDeferred.promise();
				this._projectStatusListPromise = promise;

				statusListDeferred.fail(_.bind(function () {
					this._projectStatusListPromise = null;
				}, this));
			}

			return promise;
		};


		/**
		 * Get the list of available Workflow Connections
		 * @returns {$.Deferred.<Map.<WebCenterWorkflowConnection>>} list of workflow connections
		 */
		WebCenterAPI.prototype.getWorkflowConnections = function () {
			var promise = this._workflowConnectionsPromise;
			if (!promise) {
				var workflowConnectionsDeferred = $.Deferred();

				this.getWorkflowDefinitions().done(function (wfactions) {
					requirejs(["WFEditorWCR/data/WebCenterWorkflowConnection"], function (WebCenterWorkflowConnection) {
						// parse xml for statuses and resolve the result promise
						var workflowsXML = wfactions.getElementsByTagName("Workflows");
						var connectionList = new Map();
						if (workflowsXML.length > 0) {
							_.each(workflowsXML[0].getElementsByTagName("Workflow"), function (workflowXML) {
								var connection = WebCenterWorkflowConnection.fromXML(workflowXML);
								connectionList.add(connection.getId(), connection);
							});

							workflowConnectionsDeferred.resolve(connectionList);
						}

						workflowConnectionsDeferred.fail();
					});
				}).fail(function () {
					workflowConnectionsDeferred.fail.apply(workflowConnectionsDeferred, arguments);
				});

				promise = workflowConnectionsDeferred.promise();
				this._workflowConnectionsPromise = promise;

				workflowConnectionsDeferred.fail(_.bind(function () {
					this._workflowConnectionsPromise = null;
				}, this));
			}

			return promise;
		};

		/**
		 * Get the list of JMF Workflow Connections
		 * @returns {$.Deferred.<Map.<WebcenterWorkflowConnection>>} 
		 */

		WebCenterAPI.prototype.getWorkflowConnectionsJMF = function () {
			var deferred = $.Deferred();

			this.getWorkflowConnections().done(function (list) {
				var map = new Map();
				_.each(list.asObject(), function (connection, id) {
					if (connection.isJMF()) {
						map.add(id, connection);
					}
				});

				deferred.resolve(map);

			}).fail(function () {
				deferred.reject();
			});

			return deferred.promise();
		};

		/**
		 * Get the list of status for a certain task type
		 * @param   {String} taskTypeID - task type ID for which statuses should be fetched
		 * @returns {$.Deferred.<Map.<WebCenterTaskStatus>>} 
		 */
		WebCenterAPI.prototype.getWorkflowTaskStatusList = function (taskTypeID) {
			var statusListDeferred = $.Deferred();
			this.getTaskTypeDefinition(taskTypeID).done(function (xml) {

				requirejs(["WFEditorWCR/data/WebCenterTaskStatus"], function (WebCenterTaskStatus) {
					var statusList = new Map();

					var statusListXML = xml.getElementsByTagName("status_list");
					if (statusListXML.length > 0) {
						_.each(statusListXML[0].getElementsByTagName("task_type_status_view"), function (statusXML) {
							var status = WebCenterTaskStatus.fromXML(statusXML);
							statusList.add(status.getId(), status);
						});
					}

					statusListDeferred.resolve(statusList);
				});
			}).fail(function () {
				statusListDeferred.fail();
			});

			return statusListDeferred.promise();

		};

		/**
		 * Clear the workflow with the given task type id
		 * @returns {$.Deferred.<XMLDocument,textStatus, jqXHR>} Promise of the call to clear workflow
		 */
		WebCenterAPI.prototype.clearWorkflow = function (taskTypeID) {
			return this.callSDK("UploadWorkflowXML.jsp", {
				tasktypeid: taskTypeID,
				clear: 1
			}).fail(_.bind(function (jqXHR, textStatus, errorThrown, codeString) {
				this._errorHandler.severe(i18n.t("workflowClearFailed") + " (" + errorThrown + ")", jqXHR, textStatus, errorThrown);
			}, this));
		};

		/**
		 * Get the text content element type list
		 * @returns {$.Deferred.<Map.<WebCenterTextContentElementType>} Promise for text content element types
		 */
		WebCenterAPI.prototype.getTextContentElementTypeList = function () {
			if (!this._textContentElementTypesPromise) {
				var deferredTCElementTypeRequest = $.Deferred();
				$.ajax(this.constructURL("GetRowsFromList.jsp", {
					listType: "WCR_DB_List",
					indexed: "1"
				}), {
					dataType: "text",
					type: "GET",
					data: {
						outputType: "JSON",
						list: "ELEMENT_TYPES",
						maxrows: 200,
						maxcolrows: 200
					}
				}).done(_.bind(function (jsonContent, textStatus, jqXHR) {
					this._parseContentElementResponse(jsonContent, deferredTCElementTypeRequest, false);
				}, this)).fail(function (jqXHR, textStatus, errorThrown) {
					deferredTCElementTypeRequest.reject(jqXHR, textStatus, errorThrown);
				});

				this._textContentElementTypesPromise = deferredTCElementTypeRequest.promise();
			}

			return this._textContentElementTypesPromise;
		};

		/**
		 * Get the table content element type list
		 * @returns {$.Deferred.<Map.<WebCenterTextContentElementType>} Promise for table content element types
		 */
		WebCenterAPI.prototype.getTableContentElementTypeList = function () {
			if (!this._tableContentElementTypesPromise) {
				var deferredTCElementTypeRequest = $.Deferred();
				$.ajax(this.constructURL("GetRowsFromList.jsp", {
					listType: "WCR_DB_List",
					indexed: "1"
				}), {
					dataType: "text",
					type: "GET",
					data: {
						outputType: "JSON",
						list: "TABLE_ELEMENT_TYPES",
						maxrows: 200,
						maxcolrows: 200
					}
				}).done(_.bind(function (jsonContent, textStatus, jqXHR) {
					this._parseContentElementResponse(jsonContent, deferredTCElementTypeRequest, true);
				}, this)).fail(function (jqXHR, textStatus, errorThrown) {
					deferredTCElementTypeRequest.reject(jqXHR, textStatus, errorThrown);
				});

				this._tableContentElementTypesPromise = deferredTCElementTypeRequest.promise();
			}

			return this._tableContentElementTypesPromise;
		};

		/**
		 * Parses the reponse received for content element type request and sets the result in a deffered Object
		 * 
		 * @param {JSON} jsonContent - JSON response received from element type list request
		 * @param {$.Deferred} deferredTCElementTypeRequest - deferred JQueryObj to resolve the result list
		 * @param {Boolean} isTableContent - Flag to specify whether the request is for TextContent or TableContent
		 */
		WebCenterAPI.prototype._parseContentElementResponse = function (jsonContent, deferredTCElementTypeRequest, isTableContent) {
			var cachedElementTypes = new Map();

			if (!isTableContent) {
				//Add the default element type (--- All Element Types --)
				var defElementType = new WebCenterTextContentElementType('ALL_ELEMENT_TYPES', i18n.t('wfui.multidynamiccontentmap.elementtypedefaultlabel'));
				cachedElementTypes.add('ALL_ELEMENT_TYPES', defElementType);
			}

			var jsonObj = JSON.parse(jsonContent);
			if (!jsonObj.success || jsonObj.success == "0") {
				deferredTCElementTypeRequest.fail();
			} else {
				var elementTypesValues = jsonObj.columns[0].values;
				var humanReadableValues = jsonObj.columns[1].values;
				var elementCategoryValues = jsonObj.columns[2].values;
				var combinations = jsonObj.combinations;

				_.each(combinations, _.bind(function (value) {
					var combinationsArr = value.split('~');
					var indexElemType = combinationsArr[0];
					var indexHumanElem = combinationsArr[1];
					var indexElemCat = combinationsArr[2];

					var elementType = new WebCenterTextContentElementType(elementTypesValues[indexElemType], humanReadableValues[indexHumanElem], elementCategoryValues[indexElemCat]);
					cachedElementTypes.add(elementTypesValues[indexElemType], elementType);
				}, this));
				deferredTCElementTypeRequest.resolve(cachedElementTypes);
			}
		};

		/**
		 * Get the text content language list
		 * @returns {$.Deferred.<Map.<WebCenterTextContentLanguage>} Promise for text content element types
		 */
		WebCenterAPI.prototype.getTextContentLanguageList = function () {

			if (!this._textContentLangListPromise) {
				var deferredTCLangListRequest = $.Deferred();

				//Need to fetch both master and translation languages
				languageService.getBothAllLanguages(_.bind(function (masterLangs, transLangs) {
					var masterLangMap = new Map();
					var transLangMap = new Map();
					var combinedLangMap = new Map();
					var langMap = new Map();

					//Master Languages
					_.each(masterLangs, function (langObj) {
						masterLangMap.add(langObj.langCode, new WebCenterTextContentLanguage(langObj.langCode, langObj.langDesc, langObj.langFlag, langObj.langIndex));
					});
					//Translation Languages
					_.each(transLangs, function (langObj) {
						transLangMap.add(langObj.langCode, new WebCenterTextContentLanguage(langObj.langCode, langObj.langDesc, langObj.langFlag, langObj.langIndex));
					});

					// Merge master and translation languages together.
					var combinedLangs = LanguageUtils.mergeBothLanguagesTogether(masterLangs, transLangs);
					_.each(combinedLangs.values(), function (langObj) {
						combinedLangMap.add(langObj.langCode, new WebCenterTextContentLanguage(langObj.langCode, langObj.langDesc, langObj.langFlag, langObj.langIndex));
					});

					langMap.add(ContentConstants.LanguageType.MASTER, masterLangMap);
					langMap.add(ContentConstants.LanguageType.TRANSLATION, transLangMap);
					langMap.add(ContentConstants.LanguageType.ALL, combinedLangMap);
					deferredTCLangListRequest.resolve(langMap);
				}, this));

				this._textContentLangListPromise = deferredTCLangListRequest.promise();
			}

			return this._textContentLangListPromise;

		};

		return WebCenterAPI;
	});

define('WFEditorWCR/workflow/pin/WebCenterPin',[
		"utils/ShapeUtils",
		"WFEditor/workflow/pin/Pin",
		"underscore"
	],
	function (
		ShapeUtils,
		Pin,
		_
	) {
		"use strict";

		/**
		 * WebCenterPin represents the model for a pin in the WebCenter Workflow Editor.
		 * The pin has a label
		 *
		 * @exports WebCenterPin
		 * @class
		 * @augments Pin
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {Node} node - node to which the pin belongs
		 * @param {String} id - pin
		 * @param {WebCenterPinConstants.Type} type
		 * @param {String} [label=""] - label for the pin
		 */
		var WebCenterPin = function (workflow, node, id, type, label) {
			Pin.call(this, workflow, node, id);

			/**
			 * @Member {WebCenterPinConstants.Type}
			 * @private
			 */
			this._pinType = type;

			/**
			 *  @member {String}
			 *  @private
			 */
			this._label = label || "";
		};

		WebCenterPin.prototype = Object.create(Pin.prototype);
		WebCenterPin.prototype.constructor = WebCenterPin;

		/**
		 * Returns the label of the pin
		 * @returns {String}
		 */
		WebCenterPin.prototype.getLabel = function () {
			return this._label;
		};

		/**
		 * Get the type of the pin
		 * @returns {WebCenterPinConstants.Type}
		 */
		WebCenterPin.prototype.getPinType = function () {
			return this._pinType;
		};


		/**
		 * Get the number that needs to be displayed in the pin to indicate the tokens.
		 * @returns {Number}
		 */
		WebCenterPin.prototype.getNumberForPin = function () {
			var number = 0;
			_.each(this._tokens, function (token) {
				if (!token.isCloned()) {
					number += token.getWeight();
				}
			}, this);
			return number;
		};

		/**
		 * Returns all the tokens in this webcenter pin. Removes the cloned tokens.
		 * @returns {Array.<Token>}
		 */
		WebCenterPin.prototype.getTokens = function () {
			return _.filter(this._tokens, function (token) {
				return !token.isCloned();
			}, this);
		};

		/**
		 * Returns all the active tokens in this webcenter pin. Active tokens are tokens that aren't processed yet.
		 * @returns {Array.<Token>}
		 */
		WebCenterPin.prototype.getActiveTokens = function () {
			return _.filter(this._tokens, function (token) {
				return !token.isCloned() && !token.isProcessed();
			});
		};

		return WebCenterPin;
	});

define('WFEditorWCR/workflow/pin/WebCenterPinConstants',{
	Type: {
		In: "IN",
		OutOK: "wfoutput.ok",
		OutWarning: "wfoutput.warning",
		OutError: "wfoutput.error",
		Terminus: "terminus",
		WorkflowConstructOK: "OK",
		WorkflowConstructError: "Error"
	},
	FeedbackTypes: {
		Input: "@Inp",
		Internal: "@Int",
		Terminus: "@Terminus"
	}
});

define('WFEditorWCR/workflow/pin/InputPin',[
		"WFEditorWCR/workflow/pin/WebCenterPin",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants"
	],
	function (
		WebCenterPin,
		WebCenterPinConstants
	) {
		"use strict";

		/**
		 * InputPin represents the model for an input pin in the WebCenter Workflow Editor.
		 * The pin has a label
		 *
		 * @exports InputPin
		 * @class
		 * @augments WebCenterPin
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {Node} node - node to which the pin belongs
		 */
		var InputPin = function InputPin(workflow, node) {
			WebCenterPin.call(this, workflow, node, node.getId() + WebCenterPinConstants.Type.In, WebCenterPinConstants.Type.In);
		};

		InputPin.prototype = Object.create(WebCenterPin.prototype);
		InputPin.prototype.constructor = InputPin;

		/**
		 * @override
		 */
		InputPin.prototype.acceptsIncomingEdges = function (fromPin) {
			// a WebCenter input pin only accepts edges from an output pin of another node (not the same one)
			return !fromPin || fromPin.getNode() !== this.getNode();
		};


		/**
		 * @override
		 */
		InputPin.prototype.acceptsOutgoingEdges = function (toPin) {
			// never accept outgoing edges
			return false;
		};


		return InputPin;
	});

define('WFEditorWCR/workflow/pin/OutputPin',[
		"WFEditorWCR/workflow/pin/WebCenterPin"
	],
	function (
		WebCenterPin
	) {
		"use strict";

		/**
		 * OutputPin represents the model for an output pin in the WebCenter Workflow Editor.
		 * The pin has a label
		 *
		 * @exports OutputPin
		 * @class
		 * @augments WebCenterPin
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {Node} node - node to which the pin belongs
		 * @param {String} ID - pin id
		 * @param {WebCenterPinConstants.Type} type
		 * @param {String} [label = ""] - label of the pin
		 */
		var OutputPin = function OutputPin(workflow, node, ID, type, label) {
			WebCenterPin.call(this, workflow, node, node.getId() + ID, type, label);
			
			/**
			 * @member {String} the id of the output status (eg. COMPLETED)
			 * @private
			 */
			this._outputID = ID;

		};

		OutputPin.prototype = Object.create(WebCenterPin.prototype);
		OutputPin.prototype.constructor = OutputPin;

		/**
		 * @override
		 */
		OutputPin.prototype.acceptsOutgoingEdges = function (toPin) {
			// a WebCenter output pin only accepts outgoing edges to an input pin of another node (not the same one)
			return !toPin || toPin.getNode() !== this.getNode();
		};


		/**
		 * @override
		 */
		OutputPin.prototype.acceptsIncomingEdges = function (fromPin) {
			// an output pin never accepts incoming edges
			return false;
		};
	
		/**
		 * returns the id of the output status of this pin
		 * @returns {String}
		 */
		OutputPin.prototype.getOutputId = function () {
			return this._outputID;	
		};


		return OutputPin;
	});

define('WFEditorWCR/workflow/pin/OutputPinOK',[
		"WFEditorWCR/workflow/pin/OutputPin",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"utils/webi18n!WFEditorWCR"
	],
	function (
		OutputPin,
		WebCenterPinConstants,
		i18n
	) {
		"use strict";

		/**
		 * WebCenterOutputPinOK represents the model for an ouput pin in the WebCenter Workflow Editor.
		 * The pin has a label
		 *
		 * @exports WebCenterOutputPinOK
		 * @class
		 * @augments WebCenterPin
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {Node} node - node to which the pin belongs
		 * @param {String} id - pin id
		 * @param {String} [label] - label
		 */
		var OutputPinOK = function OutputPinOK(workflow, node, ID, label) {
			if (!ID) {
				ID = WebCenterPinConstants.Type.OutOK;
			}

			if (typeof label === "undefined") {
				label = i18n.t(WebCenterPinConstants.Type.OutOK);
			}

			OutputPin.call(this, workflow, node, ID, WebCenterPinConstants.Type.OutOK, label);
		};

		OutputPinOK.prototype = Object.create(OutputPin.prototype);
		OutputPinOK.prototype.constructor = OutputPinOK;


		return OutputPinOK;
	});

define('WFEditorWCR/workflow/pin/OutputPinError',[
		"WFEditorWCR/workflow/pin/OutputPin",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"utils/webi18n!WFEditorWCR"

	],
	function (
		OutputPin,
		WebCenterPinConstants,
		i18n
	) {
		"use strict";

		/**
		 * OutputPinError represents the model for an ouput pin in the WebCenter Workflow Editor.
		 * The pin has a label
		 *
		 * @exports OutputPinError
		 * @class
		 * @augments WebCenterPin
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {Node} node - node to which the pin belongs
		 * @param {String} id - pin id
		 * @param {String} [label=Error] - error label
		 */
		var OutputPinError = function OutputPinError(workflow, node, ID, label) {
			if (!ID) {
				ID = WebCenterPinConstants.Type.OutError;
			}

			if (typeof label === "undefined") {
				label = i18n.t(WebCenterPinConstants.Type.OutError);
			}

			OutputPin.call(this, workflow, node, ID, WebCenterPinConstants.Type.OutError, label);

		};

		OutputPinError.prototype = Object.create(OutputPin.prototype);
		OutputPinError.prototype.constructor = OutputPinError;


		return OutputPinError;
	});

define('WFEditorWCR/workflow/pin/OutputPinWarning',[
		"WFEditorWCR/workflow/pin/OutputPin",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants"
	],
	function (
		OutputPin,
		WebCenterPinConstants
	) {
		"use strict";

		/**
		 * OutputPinWarning represents the model for an ouput pin in the WebCenter Workflow Editor.
		 * The pin has a label
		 *
		 * @exports OutputPinWarning
		 * @class
		 * @augments WebCenterPin
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {Node} node - node to which the pin belongs
		 * @param {String} id - pin
		 * @param {String} [label] - label
		 */
		var OutputPinWarning = function OutputPinWarning(workflow, node, ID, label) {
			if (!ID) {
				ID = WebCenterPinConstants.Type.OutWarning;
			}
			OutputPin.call(this, workflow, node, ID, WebCenterPinConstants.Type.OutWarning, label);

		};

		OutputPinWarning.prototype = Object.create(OutputPin.prototype);
		OutputPinWarning.prototype.constructor = OutputPinWarning;


		return OutputPinWarning;
	});

define('WFEditor/workflow/edge/Edge',[
		"WFEditor/workflow/WorkflowItem",
		"WFEditor/workflow/ModelConstants"
	],
	function (
		WorkflowItem,
		ModelConstants
	) {
		"use strict";

		/**
		 * Edge represents the model for an edge in the workflow. An edge connects two
		 * pins.
		 *
		 * @exports Edge
		 * @class
		 * @augments WorkflowItem
		 *
		 * @param {Workflow} workflow
		 * @param {String} id - edge id
		 * @param {Pin} from
		 * @param {Pin} to
		 */
		var Edge = function (workflow, id, from, to) {
			WorkflowItem.call(this, workflow, ModelConstants.WorkflowItemType.Edge, id);

			/**
			 *  @member {Pin}
			 *  @private
			 */
			this._from = from;

			/**
			 *  @member {Pin}
			 *  @private
			 */
			this._to = to;

		};

		Edge.prototype = Object.create(WorkflowItem.prototype);
		Edge.prototype.constructor = Edge;

		/**
		 * Return the Pin where the edge starts.
		 * @returns {Pin}
		 */
		Edge.prototype.getFromPin = function () {
			return this._from;
		};

		/**
		 * Return the Pin where the edge stops.
		 * @returns {Pin}
		 */
		Edge.prototype.getToPin = function () {
			return this._to;
		};

		return Edge;
	});


define('tpl!WFEditorWCR/workflow/edge/WebCenterEdge', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<transition name="'+
((__t=(d.getTransitionName()))==null?'':_.escape(__t))+
'" to="'+
((__t=(d.getDestinationNodeId()))==null?'':_.escape(__t))+
'" />';
return __p;
}});
define('WFEditorWCR/workflow/edge/WebCenterEdge',[
		"WFEditor/workflow/edge/Edge",
		"tpl!WFEditorWCR/workflow/edge/WebCenterEdge"
	],
	function (
		Edge,
		WebCenterEdgeXML
	) {
		"use strict";

		/**
		 * WebCenterEdge represents the model for an edge in the WebCenter workflow. An edge connects two
		 * pins.
		 *
		 * @exports WebCenterEdge
		 * @class
		 * @augments Edge
		 *
		 * @param {Workflow} workflow
		 * @param {String} id - edge id
		 */
		var WebCenterEdge = function (workflow, id, from, to) {
			Edge.call(this, workflow, id, from, to);
			/**
			 * @member {Boolean} if an edge is implicit, it is implicitly added to make sure the workflow works correctly, but shouldn't be shown to the user
			 */
			this._isImplicit = false;
		};

		WebCenterEdge.prototype = Object.create(Edge.prototype);
		WebCenterEdge.prototype.constructor = WebCenterEdge;

		/**
		 * Get the ID of the node to which this edge goes
		 * @returns {String} destinatio nnode id
		 */
		WebCenterEdge.prototype.getDestinationNodeId = function () {
			var to = this.getToPin();
			if (to) {
				var destinationNode = to.getNode();
				if (destinationNode) {
					return destinationNode.getId();
				}
			}
			return "";
		};

		/**
		 * Get the name of the transition (used for writing XML)
		 * @returns {String} name of the transition
		 */
		WebCenterEdge.prototype.getTransitionName = function () {
			var from = this.getFromPin();
			var name = "";

			if (from) {
				name = from.getId();
			}
			name += "/" + (from.getEdgeIndex(this) + 1);

			return name;
		};


		/**
		 * @Override
		 */
		WebCenterEdge.prototype.toXML = function () {
			return WebCenterEdgeXML(this);
		};

		/**
		 * Is the Edge implicit?
		 * @returns {Boolean}
		 */
		WebCenterEdge.prototype.isImplicit = function () {
			return this._isImplicit;
		};

		/**
		 * Make the Edge implicit (or not)
		 * @param {Boolean} isImplicit
		 */
		WebCenterEdge.prototype.setImplicit = function (implicit) {
			this._isImplicit = implicit;
		};

		/**
		 * Make sure implicit edges are not drawn
		 * @Override
		 */
		WebCenterEdge.prototype.isVisible = function () {
			return Edge.prototype.isVisible.call(this) && !this.isImplicit();
		};


		return WebCenterEdge;
	});

define('WFEditorWCR/workflow/node/WebCenterNodeConstants',{
	/**
	 * All the workflow constructs and task types are defined here
	 */
	Nodes: {
		DATACOLLECTOR: 'wfconstruct-datacollector',
		DATASPLITTER: 'wfconstruct-datasplitter',
		ERRORHANDLER: 'wfconstruct-errorhandler',
		TOKENSELECTOR: 'wfconstruct-markedtokenselector',
		TOKENMARKER: 'wfconstruct-tokenmarker',
		WCTASK: 'wctask',
		JOIN: 'join',
		FORK: 'fork',
		COMPLETED: 'completed',
		CANCEL: 'errorstop',
		START: 'start',
		ROUTER: 'bsroute',
		STARTAPPROVAL: "approvaltask",
		SUBMITWORKFLOW: "submitworkflow",
		ERRORHANDLING: "errorHandling",
		WAIT: "waittask",
		ERRORCONTINUE: "errorcontinue",
		ERRORCONTINUEIMPLICIT: "errorcontinue_implicit",
		SETTASKSPECIFICATION: "settaskspec",
		SETPROJECTATTRIBUTE: "setprojectattr",
		SETDOCUMENTATTRIBUTE: "setdocattr",
		IMPORTDYNAMICCONTENT: "importdynamiccontent"
	},
	Icons: {
		DATACOLLECTOR: "wf_construct_collect.png",
		DATASPLITTER: "wf_construct_split.png",
		ERRORHANDLER: "wf_construct_end_error.png",
		TOKENSELECTOR: "wf_construct_select.png",
		TOKENMARKER: "wf_construct_mark.png",
		JOIN: "wf_construct_join.png",
		FORK: "wf_construct_fork.png",
		COMPLETED: "wf_construct_end_ok.png",
		CANCEL: "wf_construct_cancel.png",
		ERROR: "wf_construct_end_error.png",
		START: "wf_construct_start.png",
		ERRORHANDLING: "start-point.png"
	},
	HitAreas: {
		LABEL: "NodeLabel",
		STICKYNOTE: {
			MOVE: "StickyNoteMove",
			RESIZE: "StickyNoteResize",
			TEXTAREA: "StickyNoteTextArea",
			EDITICON: "StickyNoteTextIcon",
			DELETE: "StickyNoteDelete"
		}
	},
	Privileges: {
		editLabels: "editLabels",
		editStickyNotes: "editStickyNotes",
		showWorkflowControls: "showWorkflowControls"
	},
	Status: {
		COMPLETED: "00001_0000000003"
	},
	SubmitWorkflow: {
		PUBLICPARAMETERS: "sw.publicparameters",
		PROJECTPROPERTIES: "sw.projectProperties",
		DOCUMENTPROPERTIES: "sw.documentProperties",
		PROJECTATTRIBUTES: "sw.projectAttributes",
		DOCUMENTATTRIBUTES: "sw.documentAttributes"
	},
	SetProjectAtrributes: {
		ATTRIBUTENAMELABEL: "router.projectattribute",
		ATTRIBUTENAME: "project.attname",
		ATTRIBUTEVALUE: "project.attvalue"
	},
	SetDocumentAttributes: {
		ATTRIBUTENAMELABEL: "router.documentattribute",
		ATTRIBUTENAME: "project.attname", // project is "correct" here
		ATTRIBUTEVALUE: "project.attvalue" // project is "correct" here
	},
	SetTaskSpecification: {
		SPECNAMELABEL: "router.workflowspec",
		SPECNAME: "task.specname",
		SPECVALUE: "task.specvalue"
	},
	DataCollector: {
		ERRORPOLICY: "errorPolicy",
		IGNOREERRORS: "ignoreerrors",
		FAILONERRORS: "failonerrors"
	},
	DefaultStartNode: {
		DefaultX: 2000,
		DefaultY: 2000
	},
	DefaultCompletedNode: {
		DefaultX: 3333,
		DefaultY: 2000
	},
	RouterNode: {
		Routers: {
			ProjectStatus: "WCWF_TProjectStatusRouter",
			ProjectField: "WCWF_TProjectFieldRouter",
			ProjectAttribute: "WCWF_TProjectAttributeRouter",
			DocumentAttribute: "WCWF_TDocumentAttributeRouter",
			DocumentName: "WCWF_TDocumentNameRouter",
			NumberOfDocuments: "WCWF_TNumberOfDocumentsRouter",
			TaskSpecification: "WCWF_TTaskSpecRouter",
			TextContentProperty: "WCWF_TTextContentPropertyRouter"
		},
		RuleTypes: {
			StringRule: "stringrule",
			NumericRule: "numericrule",
			EnumRule: "enumrule"
		}
	},
	WCTask: {
		TYPE: "task.type",
		NAME: "task.name",
		ASSIGNEE: "task.assignee",
		DESCRIPTION: "task.description",
		FAILIFEXISTS: "task.failifexists",
		LEADDAYS: "task.leaddays",
		LEADTIME: "task.leadtime",
		USERASSIGNEE: "task.Ctypeuser",
		GROUPASSIGNMENT: "task.Btypegroup",
		GROUPASSIGNEE: "task.group",
		ROLEASSIGNEE: "task.role",
		ROLEASSIGNMENT: "task.Atyperole"

	},
	StickyNote: {
		MIN_WIDTH: 200,
		DEFAULT_WIDTH: 300,
		MIN_HEIGHT: 60,
		DEFAULT_HEIGHT: 200
	},
	HelpTopics: {
		MARK: "common/wc/reference/re_wc_MarkandSelect",
		SELECT: "common/wc/reference/re_wc_MarkandSelect",
		DATACOLLECTOR: "common/wc/reference/re_Wc_Data_S_C",
		DATASPLITTER: "common/wc/reference/re_Wc_Data_S_C",
		FORK: "common/wc/reference/re_wc_ForkandJoin",
		JOIN: "common/wc/reference/re_wc_ForkandJoin",
		ERRORHANDLER: "common/wc/concept/co_wc_UsingErrorHandlerNode",
		CANCEL: "common/wc/reference/re_wc_StopNode",
		EDITOR: "common/wc/concept/co_wc_TheWorkflowEditor"
	},
	WarningPolicy: {
		WARNINGPOLICY: "warningpolicy",
		ERROR: "erroronwarning",
		OK: "okonwarning",
		DEDICATED: "dedicatedhandling"
	}

});


define('tpl!WFEditor/workflow/panel/Parameter', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<parameter>\r\n	<Name>'+
((__t=( d.getName() ))==null?'':_.escape(__t))+
'</Name>\r\n	<Value>';
 if(d._subparameters.length === 0){ 
__p+=''+
((__t=( d.getValue() ))==null?'':_.escape(__t))+
'';
 } else { 
__p+=''+
((__t=( d.getSubparameterXML() ))==null?'':__t)+
'';
 } 
__p+='</Value>\r\n</parameter>';
return __p;
}});
define('WFEditor/workflow/panel/Parameter',["tpl!./Parameter",
        "underscore"],
	function (parameter,
		_) {

		"use strict";

		/**
		 * This class represents a parameter which can be used for the AbstractPanelComponents
		 * @class
		 * @exports Parameter
		 * @param {String} name                - name of the parameter
		 * @param {String} [defaultValue = ""] - default value of the parameter
		 * @param {String} [value]
		 * @param {Array.<Parameter>} [subparameters]
		 */
		var Parameter = function (name, defaultValue, value, subparameters) {

			/**
			 * @member {String}
			 * @private
			 */
			this._name = name;

			/**
			 * @member {String}
			 * @private
			 */
			this._defaultValue = defaultValue || "";

			/**
			 * @member {String}
			 * @private
			 */
			this._value = value;

			/**
			 * @member {Array.<Parameter>}
			 * @private
			 */
			this._subparameters = subparameters || [];
		};

		/**
		 * gets the name of the parameter
		 * @returns {String}
		 */
		Parameter.prototype.getName = function () {
			return this._name;
		};

		/**
		 * gets the value of the parameter or the default value if the value is not defined
		 * @returns {String}
		 */
		Parameter.prototype.getValue = function () {
			return this._value || this._defaultValue;
		};

		/**
		 * gets the default value of the parameter
		 * @returns {String}
		 */
		Parameter.prototype.getDefaultValue = function () {
			return this._defaultValue;
		};

		/**
		 * Set the default value of the parameter
		 * @param {String} defaultValue - The new default value
		 */
		Parameter.prototype.setDefaultValue = function (defaultValue) {
			this._defaultValue = defaultValue;
		};

		/**
		 * gets the subparameters of the parameter
		 * @returns {Array.<Parameter>}
		 */
		Parameter.prototype.getSubparameters = function () {
			return this._subparameters;
		};

		/**
		 * returns the string representation of this parameter
		 * @returns {String}
		 */
		Parameter.prototype.toString = function () {
			return parameter(this);
		};

		/**
		 * returns a string representation of the sub parameters
		 * @returns {String}
		 */
		Parameter.prototype.getSubparameterXML = function () {
			var xml = "";
			_.each(this._subparameters, function (param) {
				xml += param.toString();
			});
			return xml;
		};

		/**
		 * sets the name of this parameter
		 * @param {String} name
		 */
		Parameter.prototype.setName = function (name) {
			this._name = name;
		};

		/**
		 * sets the value of this parameter
		 * @param {String} val
		 */
		Parameter.prototype.setValue = function (val) {
			this._value = val;
		};

		/**
		 * sets the sub parameters of this parameter
		 * @param {Array.<Parameter>} params 
		 */
		Parameter.prototype.setSubParameters = function (params) {
			this._subparameters = params;
		};

		/**
		 * creates a deep copy of this parameter
		 * @returns {Parameter}
		 */
		Parameter.prototype.copy = function () {
			var copiedSubparams = [];
			_.each(this._subparameters, function (subParam) {
				copiedSubparams.push(subParam.copy());
			});
			return new Parameter(this._name, this._defaultValue, this._value, copiedSubparams);
		};

		/**
		 * two params are equal if their names are equal
		 * @param   {Parameter} param
		 * @returns {Boolean}
		 */
		Parameter.prototype.equals = function (param) {
			var subparams1 = this.getSubparameters();
			var subparams2 = param.getSubparameters();
			if (subparams1.length != subparams2.length) {
				return false;
			} else {
				for (var i = 0; i < subparams1.length; i++) {
					if (!subparams1[i].equals(subparams2[i])) {
						return false;
					}
				}
			}

			return (this.getName() === param.getName() && this.getValue() === param.getValue());
		};



		return Parameter;

	});

define('WFEditorWCR/workflow/panel/WebCenterParameter',["underscore", "jquery", "WFEditor/workflow/panel/Parameter"], function (_, $, Parameter) {

	"use strict";

	/**
	 * An implementation of a parameter used in the WebCenterWorkflowEditor
	 * 
	 * @exports WebCenterParameter
	 * @class
	 * @augments Parameter
	 * 
	 * @param {String} name              
	 * @param {String} [defaultValue = ""]
	 * @param {String} [value]       
	 * @param {Array.<WebCenterParameter>} [subparameters]       
	 */
	var WebCenterParameter = function (name, defaultValue, value, subparameters) {
		Parameter.call(this, name, defaultValue, value, subparameters);
	};

	WebCenterParameter.prototype = Object.create(Parameter.prototype);
	WebCenterParameter.prototype.constructor = WebCenterParameter;

	/**
	 * updates a parameter from xml
	 * @param   {String} xml
	 * @returns {Parameter}
	 */
	WebCenterParameter.prototype.updateFromXML = function (xml) {
		_.each(xml.childNodes, function (child) {
			if (child.tagName === "Name") {
				this._name = child.textContent.trim();
			} else if (child.tagName === "Value") {
				//Take only direct children under this tag
				var potentialSubparams = $(child).children("parameter");
				if (potentialSubparams.length > 0) {
					_.each(potentialSubparams, function (subparamXML) {
						var subParam = new WebCenterParameter();
						subParam.updateFromXML(subparamXML);
						this._subparameters.push(subParam);
					}, this);
				} else {
					this._value = child.textContent.trim();
				}
			}
		}, this);
	};

	/**
	 * @override
	 */
	WebCenterParameter.prototype.copy = function () {
		var copiedSubparams = [];
		_.each(this._subparameters, function (subParam) {
			copiedSubparams.push(subParam.copy());
		});
		return new WebCenterParameter(this._name, this._defaultValue, this._value, copiedSubparams);
	};


	return WebCenterParameter;

});

define('WFEditorWCR/workflow/panel/PinParameter',["WFEditorWCR/workflow/panel/WebCenterParameter"], function (WebCenterParameter) {

	"use strict";

	/**
	 * This parameterclass can be used for parameters that enable/disable an outputpin if it is chosen in fe. a radio checkbox
	 * @class
	 * @augments WebCenterParameter
	 * @exports PinParameter
	 * @param {String} name - id of the parameter
	 * @param {String} defaultValue
	 * @param {String} [value]
	 * @param {Array.<Parameter>} subparameters
	 * @param {Function} pinFactory - function that returns a Pin
	 */
	var PinParameter = function (name, defaultValue, value, subparameters, pinFactory) {
		WebCenterParameter.call(this, name, defaultValue, value, subparameters);

		/**
		 * @member {Function.<Pin>}
		 * @private
		 */
		this._pinFactory = pinFactory;
		
		/**
		 * @member {Pin}
		 * @private
		 */	
		this.pins = pinFactory();

	};

	PinParameter.prototype = Object.create(WebCenterParameter.prototype);
	PinParameter.prototype.constructor = PinParameter;

	/**
	 * @override
	 */
	PinParameter.prototype.copy = function () {
		var copiedSubparams = [];
		_.each(this._subparameters, function (subParam) {
			copiedSubparams.push(subParam.copy());
		});		
		var param = new PinParameter(this._name, this._defaultValue, this._value, copiedSubparams, this._pinFactory);
		param.pins = this.pins;
		return param;
	};
	
	/**
	 * sets a new pin.
	 */
	PinParameter.prototype.setNewPin = function () {
		this.pins = this._pinFactory();
	};

	return PinParameter;

});


define('tpl!WFEditorWCR/workflow/node/WebCenterNode', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<node name="'+
((__t=(d.getId()))==null?'':_.escape(__t))+
'">\r\n	'+
((__t=(d.getActionXML()))==null?'':__t)+
'\r\n	'+
((__t=(d.getTransitionsXML()))==null?'':__t)+
'\r\n</node>';
return __p;
}});

define('tpl!WFEditorWCR/workflow/node/WebCenterNodeAction', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<action class="'+
((__t=(d.getNodeType()))==null?'':_.escape(__t))+
'">';
 if(d.canEditWarningPolicy()) {
print("<warningpolicy>"+d.getWarningPolicyString()+("</warningpolicy>"));
}
__p+='<parameters>';
 _.each(d.getConfiguration(), function(parameter) { print(parameter.toString()) }) 
__p+='</parameters>\r\n</action>';
return __p;
}});

define('tpl!WFEditorWCR/workflow/node/WebCenterNodeLayout', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<wfelement name="'+
((__t=(d.getId()))==null?'':_.escape(__t))+
'">\r\n	<preflocx>'+
((__t=(d.getLocation().round().x))==null?'':_.escape(__t))+
'</preflocx>\r\n	<preflocy>'+
((__t=(d.getLocation().round().y))==null?'':_.escape(__t))+
'</preflocy>\r\n	<label>'+
((__t=(d.getLabel()))==null?'':_.escape(__t))+
'</label>\r\n</wfelement>';
return __p;
}});
define('WFEditorWCR/workflow/panel/WebCenterPanelConstants',{
	PANELURL: "panelURL",
	MENUS: "menus",
	MENU: "menu",
	PARAMETERS: "parameters",
	PARAMETER: "parameter",
	OUTPUTS: "outputs",
	OUTPUT: "output",
	DATA: "data",
	SMARTNAMES: "smartnames",
	SMARTNAME: "smartname",
	STRING: "String",
	DATE: "Data",
	NUMBER: "Number", // currently not used
	EMAIL: "EMAIL",
	URL: "url",
	PM: "PM",
	TASKSPECVALUE: "TaskSpecValue",
	PROJECTATTRVALUE: "ProjectAttrValue",
	ATTRIBUTE: "Attribute",
	BOOLEAN: "Boolean",
	RADIO: "radio",
	SECRET: "secret",
	NAME: "name",
	VALUE: "value",
	TYPE: "type",
	DEFAULT: "default",
	SUBPARAMETERSTATE: {
		TAGNAME: "subparametersstate",
		INVISIBLE: "invisible",
		VISIBLE: "visible",
		ENABLED: "enabled",
		DISABLED: "disabled"
	},
	MULTIPROJATTRIBUTES: "MultiProjectAttributeSelection",
	ANYDOCUMNETREFERENCE: "AnyDocumentReference",
	MULTIPROJATTRIBUTESMAXROWS: 100,
	PROJECTSTATUS: "ProjectStatus",
	TASKSTATUS: "TaskStatus",
	TASKSPECIFICATIONS: "TaskTypeSpecifications",
	TASKSPECIFICATIONS_PARAMETERID: "task.specifications",
	WORKFLOWLIST: "WorkflowList",
	WORKFLOWLISTJMF: "WorkflowListJMF",
	ROLEASSIGNMENT: "RoleAssignment",
	ROLEASSIGNMENT_OPTIONS: ["task.role.oneOfRole", "task.role.allOfRoleOnePerGroup", "task.role.allOfRole"],
	GROUPASSIGNMENT: "GroupAssignment",
	GROUPASSIGNMENT_OPTIONS: ["task.group.oneOfGroup", "task.group.allOfGroup"],
	USERS: "Users",
	TASKSPECNAME: "TaskSpecName",
	PROJECTATTRIBUTENAME: "ProjectAttrName",
	ATTRIBUTECATEGORY: "AttributeCategory",
	DOCREFATTRIBUTE: "DocRefAttribute",
	DOCREFATTRIBUTEDEFAULT:"DocRefAttributeDefault",
	USERSANDGROUPS: "UsersAndGroups",
	HTML: "HTML",
	MULTIPARAMETERS: "MultiParameters",
	MULTIATTRIBUTES: "MultiAttribute",
	MULTIDYNAMICCONTENTIMPORTMAP: "MultiDynamicContentImportMap",
	MULTITEXTCONTENTLANGUAGEMAP: "MultiTextContentLanguageMap",
	MULTITEXTCONTENTFIELD: "MultiTextContentField",
	MULTITABLESETTINGS: "MultiTableContentSettings",
	MULTITEXTCONTENTPERMISSION: "MultiTextContentPermission",
	MULTITEXTCONTENTVALUES: "MultiTextContentValues",
	MULTITEXTCONTENTLANGUAGESTATUS: "MultiTextContentLanguageStatus",
	MULTITABLEELEMENTCONTENT: "MultiTableElementContent",
	MULTITABLEELEMENTADDREMOVE: "MultiTableElementAddRemove",
	MULTITABLEELEMENTAPPROVAL: "MultiTableElementApproval",
	GROUPS: "Groups",
	ROLES: "Roles",
	JAVASCRIPT: "JavaScript",
	ROLE: {
		PROJECTMANAGER: {
			VALUE: "invprojectmgr",
			NAME: "task.projectmanager"
		},
		SALESPERSON: {
			VALUE: "invsalesperson",
			NAME: "task.salesperson"
		},
		WORKFLOWMASTER: {
			VALUE: "invprojectmaster",
			NAME: "task.workflowmaster"
		}
	},
	ENUM: "Enum"
});

define('WFEditorWCR/workflow/panel/WorkflowSmartNameConstants',{
	SPECIFICATIONS: {
		CATEGORY: "specscat",
		PREFIX: "spec."
	},
	PROJECTATTRIBUTES: {
		CATEGORY: "projattrcat",
		PREFIX: "att."
	},
	DOCUMENTATTRIBUTES: {
		CATEGORY: "docattrcat",
		PREFIX: "docatt."
	},
	PROJECT: {
		CATEGORY: "projectcat",
		SMARTNAMES: {
			COMPANYNAME: "compname",
			COMPANYLEGALNAME: "compLegalName",
			COMPANYSHORTNAME: "compShortName",
			COMPANYLOCATION: "comploc",
			PROJECTNAME: "projname",
			PROJECTDESCRIPTION: "projdescr",
			PROJECTDUEDATE: "projduedate",
			PROJECTMANAGER: "projmanager",
			PROJECTSALESPERSON: "projsalesmgr"
		}
	},
	ITERATION: {
		CATEGORY: "itercat",
		SMARTNAMES: {
			ITERATOR: "iterator" // currently picked from back-end
		}
	},
	DOCUMENT: {
		CATEGORY: "filecat",
		SMARTNAMES: {
			FILENAME: "filename",
			FILETYPE: "filetype",
			DOCUMENTDESCRIPTION: "docdescription"
		}
	},
	TEXTCONTENT: {
		CATEGORY: "textcontentcat",
		SMARTNAMES: {
			ELEMENTTYPE: "tcelementtype",
			MASTERLANG: "tcmasterlang",
			TRANSLANG: "tctranslang"
		}
	},
	LINKS: {
		CATEGORY: "linkscat",
		SMARTNAMES: {
			DOCUMENTLINK: "doclink",
			PROJECTLINK: "projlink",
			TASKLINK: "tasklink",
		}
	},
	EMAIL: {
		CATEGORY: "emailcat",
		SMARTNAMES: {
			PROJECTMANAGER: "projmanagerid",
			SALESPERSON: "projsalesmgrid",
			WORKFLOWTASKASSIGNEE: "taskownerid",
		}
	},
	DATE: {
		CATEGORY: "dateCategory",
		SMARTNAMES: {
			NOW: "now",
			TASKDUEDATE: "tname.taskduedate",
			TASKSTARTDATE: "tname.taskstartdate",
			PROJECTDUEDATE: "tname.projectduedate",
		}
	},
	URL: {
		CATEGORY: "urlcat",
		SMARTNAMES: {
			WEBCENTERURL: "tname.webcenterurl"
		}
	},
	USER: {
		CATEGORY: "usercat",
		SMARTNAMES: {
			USERFIRSTNAME: "userfirstname",
			USERFAMILYNAME: "userfamilyname",
		}
	},
	TASKROLES: {
		CATEGORY: "task.assignee.roles",
		PREFIX: "role."
	},
	TASKGROUPS: {
		CATEGORY: "task.assignee.groups",
		PREFIX: "grp."
	},
	TASKUSERS: {
		CATEGORY: "task.assignee.users",
		PREFIX: "usr."
	},
	CALCULATED: {
		CATEGORY: "calculatedcat",
		SMARTNAMES: {
			LIST: "List"
		}
	}
});

define('WFEditorWCR/data/CustomSmartName',[],
	function () {
		"use strict";

		/**
		 * Represents a custom smartname
		 * @param {String}  smartName       - The smartname key
		 * @param {String}  category        - The smartname category
		 */
		var CustomSmartName = function (smartName, category) {
			this._smartName = smartName;
			this._category = category;
		};

		/**
		 * Get the smartname key
		 * @returns {String}
		 */
		CustomSmartName.prototype.getSmartName = function () {
			return this._smartName;
		};

		/**
		 * Get the category it belongs to
		 * @returns {String}
		 */
		CustomSmartName.prototype.getCategory = function () {
			return this._category;
		};

		/**
		 * Create CustomSmartName from xml
		 * @param   {String} xml
		 * @returns {CustomSmartName}
		 */
		CustomSmartName.fromXML = function (xml) {
			var name, category;
			_.each(xml.childNodes, function (child) {
				if (child.tagName === "name") {
					name = child.textContent;
				} else if (child.tagName === "category") {
					category = child.textContent;
				}
			}, this);
			return new CustomSmartName(name, category);
		};

		return CustomSmartName;
	});

define('WFEditorWCR/workflow/node/WebCenterNodeConfigurationPanel',["underscore",
		"WFEditor/workflow/node/NodeConfigurationPanel",
		"component/helpIcon/HelpIcon"
	],
	function (_,
		NodeConfigurationPanel,
		HelpIcon) {
		"use strict";

		/**
		 * Configuration Panel for a WebCenter Node
		 * @class
		 * @export WebCenterNodeConfigurationPanel
		 * @augments NodeConfigurationPanel
		 * @param {Node} node - node for which the configuration panel is created
		 */
		var WebCenterNodeConfigurationPanel = function (node) {
			NodeConfigurationPanel.call(this, node);

			this._element.addClass("wfeditor-node-configuration");

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._helpBalloon = null;

			/**
			 * @member {Tooltip} tooltip for the help icon, referenced to destroy it if panel is destroyed
			 * @private
			 */
			this._tooltip = null;

		};

		WebCenterNodeConfigurationPanel.prototype = Object.create(NodeConfigurationPanel.prototype);
		WebCenterNodeConfigurationPanel.prototype.constructor = WebCenterNodeConfigurationPanel;


		/**
		 * Get the title of the panel
		 * @override
		 * @returns {String} 
		 */
		WebCenterNodeConfigurationPanel.prototype.getTitle = function () {
			return this._node.getLabel();
		};

		/**
		 * Focus the first input element
		 * @override
		 */
		WebCenterNodeConfigurationPanel.prototype.focus = function () {
			this._element.find("input,textarea").first().focus();
		};

		/**
		 * Add a help balloon to the footer
		 * @override
		 */
		WebCenterNodeConfigurationPanel.prototype.getFooterContent = function () {
			if (!this._helpBalloon && this._node.getHelpTopic()) {
				var options = {
					cssClass: "wfeditor-node-help-tooltip tooltip inline_help_tooltip"
				};
				this._helpBalloon = HelpIcon.getInlineHelpTopicJQuery(this._node.getHelpTopic(),
					undefined, options);
				this._tooltip = options.tooltipReference;

			}
			return this._helpBalloon;
		};

		/**
		 * @override
		 */
		WebCenterNodeConfigurationPanel.prototype.destroy = function () {
			NodeConfigurationPanel.prototype.destroy.call(this);
			if(this._tooltip){
				this._tooltip.destroy();
			}
		};


		return WebCenterNodeConfigurationPanel;
	});

define('WFEditorWCR/data/WebCenterTextContentProperty',[],
	function () {
		"use strict";

		/**
		 * Represents a WebCenter Text Content Property
		 * @param {String} id - Id of the text content property
		 * @param {String} name - Name of the text content property
		 */
		var WebCenterTextContentProperty = function (id, name) {
			this._id = id;
			this._name = name;
		};

		/**
		 * Get the ID of the element type
		 * @returns {String}
		 */
		WebCenterTextContentProperty.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Get the name of the element type
		 * @returns {String}
		 */
		WebCenterTextContentProperty.prototype.getName = function () {
			return this._name;
		};

		return WebCenterTextContentProperty;
	});

define('WFEditorWCR/workflow/node/WebCenterNode',[
		"utils/ShapeUtils",
		"utils/XMLUtils",
		"utils/Map",
		"WFEditor/workflow/node/Node",
		"WFEditorWCR/workflow/pin/InputPin",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/OutputPinError",
		"WFEditorWCR/workflow/pin/OutputPinWarning",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/edge/WebCenterEdge",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"WFEditorWCR/workflow/panel/PinParameter",
		"webcenterapi/data/WebCenterAttribute",
		"tpl!WFEditorWCR/workflow/node/WebCenterNode",
		"tpl!WFEditorWCR/workflow/node/WebCenterNodeAction",
		"tpl!WFEditorWCR/workflow/node/WebCenterNodeLayout",
		"utils/webi18n!WFEditorWCR",
		"WFEditorWCR/workflow/panel/WebCenterPanelConstants",
		"WFEditorWCR/workflow/panel/WorkflowSmartNameConstants",
		"WFEditorWCR/data/CustomSmartName",
		"WFEditorWCR/workflow/node/WebCenterNodeConfigurationPanel",
		"packaging-content-management/ContentConstants",
		"WFEditorWCR/data/WebCenterTextContentProperty",
		"underscore"
	],
	function (
		ShapeUtils,
		XMLUtils,
		Map,
		Node,
		InputPin,
		OutputPinOK,
		OutputPinError,
		OutputPinWarning,
		WebCenterPinConstants,
		WebCenterEdge,
		WebCenterNodeConstants,
		WebCenterParameter,
		PinParameter,
		WebCenterAttribute,
		WebCenterNodeXML,
		WebCenterNodeActionXML,
		WebCenterNodeLayoutXML,
		i18n,
		WebCenterPanelConstants,
		WorkflowSmartNameConstants,
		CustomSmartName,
		WebCenterNodeConfigurationPanel,
		ContentConstants,
		WebCenterTextContentProperty,
		_
	) {
		"use strict";

		/**
		 * WebCenterNode represents the model for a node in the WebCenter Workflow Editor.
		 * The node has an icon and a specific type which are used in the rendering of the node.
		 *
		 * @exports WebCenterNode
		 * @class
		 * @augments Node
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {ShapeUtils.Point} location - location of the node
		 * @param {String} nodeType - specifies the type of the node
		 * @param {String} [icon] - gives the url to the icon for this node
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var WebCenterNode = function (workflow, id, nodeType, icon, label, webCenterAPI) {
			Node.call(this, workflow, id);

			/**
			 *  @member {String}
			 *  @private
			 */
			this._nodeType = nodeType;

			/**
			 *  @member {String} url relative to the webcenter instance
			 *  @private
			 */
			this._iconUrl = icon;

			/**
			 *  @member {String}
			 *  @private
			 */
			this._label = label;

			/**
			 * @member {InputPin} input pin if available
			 * @private
			 */
			this._inputPin = null;

			/**
			 * @member {OutputPinError} error pin for the node
			 * @private
			 */
			this._errorPin = null;

			/**
			 * @member {String} temporary storage of action xml
			 */
			this._actionXML = "";

			/**
			 * @member {String} temporary storage of resource xml
			 */
			this._resourceXML = "";

			/**
			 * @member {Boolean} is someone editing the label of this node?
			 */
			this._isEditingLabel = false;

			/**
			 * @member {WebCenterAPI}
			 * @private
			 */
			this._webCenterAPI = webCenterAPI;

			/**
			 * @member {String} topic of the help pages
			 * @private
			 */
			this._helpTopic = "";

			/**
			 * @member {$.Deferred} deferred used to check if the help topic has been fetched yet
			 * @private
			 */
			this._helpTopicDeferred = null;

			/** 
			 * @member {Parameter}
			 * @private
			 */
			this._warningPolicyParam = null;
			this._warningPolicyOk = null;
			this._warningPolicyError = null;
			this._warningPolicyDedicated = null;

			/**
			 * @member {Boolean}
			 * @private
			 */
			this._canEditWarningPolicy = false;

			/**
			 * @member {Array.<String>}
			 * @private 
			 */
			this._warningOutputNames = [];


			// Initialize the default parameters
			this._webCenterAPI.getWorkflowDefinitions().done(_.bind(function (workflowDefinitionXML) {
				var details = this._getConfigDetailsForNodeType(this.getNodeType(), workflowDefinitionXML);
				var parametersCopy = [];
				if (details.parameters) {
					_.each(details.parameters, function (param) {
						parametersCopy.push(param.copy());
					}, this);
				}
				this.setConfiguration(parametersCopy);

				this._helpTopic = details.helpTopic;
			}, this));

			this._initializePins();

			if (this._canEditWarningPolicy) {
				this._initWarningPolicyParams();
			}

		};

		WebCenterNode.prototype = Object.create(Node.prototype);
		WebCenterNode.prototype.constructor = WebCenterNode;


		/**
		 * Add the input pin
		 */
		WebCenterNode.prototype._addInputPin = function () {
			this._inputPin = new InputPin(this._workflow, this);
			this.addPin(this._inputPin);
		};

		/**
		 * Add an error pin to the node
		 * @param {WebCenterPinConstants.Type} [errorType] - type of the error pin, uses default one if undefined
		 */
		WebCenterNode.prototype._addErrorPin = function (errorType) {
			this._errorPin = new OutputPinError(this._workflow, this, errorType);
			this.addPin(this._errorPin);
		};

		/**
		 * Initialize the pins for the node
		 */
		WebCenterNode.prototype._initializePins = function () {
			// Add an input pin
			this._addInputPin();

			// Add custom output pins
			this._webCenterAPI.getWorkflowDefinitions().done(_.bind(function (workflowDefinitionXML) {
				var details = this._getConfigDetailsForNodeType(this.getNodeType(), workflowDefinitionXML);
				var outputPins = details.outputs;
				if (outputPins && _.size(outputPins) > 0) {
					_.each(outputPins, function (output) {
						if (output.type === "0") {
							this.addPin(new OutputPinOK(this._workflow, this, output.name, i18n.t(output.name)));
						} else if (output.type === "1") {
							this._warningOutputNames.push(output.name);
							this._canEditWarningPolicy = true;
						} else if (output.type === "2") {
							var errorPin = new OutputPinError(this._workflow, this, output.name, i18n.t(output.name));
							// this way the last one is the real error pin
							this._errorPin = errorPin;
							this.addPin(errorPin);
						}
					}, this);
				} else {
					// Add by default an error pin
					this._addErrorPin();
				}
			}, this));
		};

		/**
		 * Returns the type of this node
		 * @returns {String}
		 */
		WebCenterNode.prototype.getNodeType = function () {
			return this._nodeType;
		};

		/**
		 * Returns the path relative to the webcenter instance for the icon of this node
		 * @returns {String}
		 */
		WebCenterNode.prototype.getIconUrl = function () {
			return this._iconUrl;
		};

		/**
		 * Get visible parameters to render under the node
		 * @returns {Array.<Parameter>}
		 */
		WebCenterNode.prototype.getVisibleParameters = function () {
			return [];
		};

		/**
		 * Returns the label of the node
		 * @param {Boolean} [returnIdIfDebugMode = false] whether or not the id should be returned instead
		 *												  of the label when in debug mode
		 * @returns {String}
		 */
		WebCenterNode.prototype.getLabel = function (returnIdIfDebugMode) {
			return this._workflow.isDebugMode() && returnIdIfDebugMode ? this._id : this._label;
		};

		/**
		 * Set the label for the node.
		 * @param {String} label
		 */
		WebCenterNode.prototype.setLabel = function (label) {
			if (label !== this._label) {
				this._label = label;
				this._emit("changed.label");
			}
		};

		/**
		 * set the editing status of the node's label
		 * @param {Boolean} edit
		 */
		WebCenterNode.prototype.setEditingLabel = function (edit) {
			if (this._isEditingLabel !== edit) {
				this._isEditingLabel = edit;
				this._emit("changed.editingLabel");
			}
		};

		/**
		 * returns whether or not this label is being edited
		 * @returns {Boolean} 
		 */
		WebCenterNode.prototype.isEditingLabel = function () {
			return this._isEditingLabel;
		};

		/**
		 * Parse the node XML information
		 * @param {Element} nodeEl- XML element containing definition of the WorkflowNode
		 */
		WebCenterNode.prototype.setNodeXML = function (nodeEl) {
			// this is a temporary to save XML for using it when writing out node xml
			var actionEls = nodeEl.getElementsByTagName("action");
			if (actionEls[0]) {
				this._actionXML = new XMLSerializer().serializeToString(actionEls[0]);
			} else {
				this._actionXML = "";
			}
			var params = nodeEl.querySelectorAll("action > parameters > parameter > Name");
			_.each(params, function (nameNode) {
				var name = nameNode.textContent;
				var param = this.getConfigurationParameterByName(name);
				if (param) {
					param.updateFromXML(nameNode.parentNode);
				}
			}, this);
			if (this.canEditWarningPolicy()) {
				var warningPolicy = nodeEl.querySelector("action > warningpolicy");
				if (warningPolicy) {
					this._warningPolicyParam.setValue(warningPolicy.textContent);
				} else {
					this._warningPolicyParam.setValue(WebCenterNodeConstants.WarningPolicy.DEDICATED);
				}

				this._updateWarningPins();
			}
		};

		/**
		 * Handle the Layout XML
		 * --> read location and label
		 * @param {Element} layoutXML - xml element with layoutinformation (wfelement)
		 */
		WebCenterNode.prototype.setLayoutXML = function (layoutXML) {
			var labelEl = layoutXML.getElementsByTagName("label")[0];
			if (labelEl) {
				this.setLabel(labelEl.textContent);
			}

			var locXEl = layoutXML.getElementsByTagName("preflocx")[0];
			var locYEl = layoutXML.getElementsByTagName("preflocy")[0];

			if (locXEl && locYEl) {
				this.setLocation(new ShapeUtils.Point(parseFloat(locXEl.textContent), parseFloat(locYEl.textContent)));
			}
		};

		/** 
		 * Handle Resource XML
		 * --> handle extra resources for the node
		 * @parma {Element} resourceXML - xml element with resource information (resource)
		 */
		WebCenterNode.prototype.setResourceXML = function (resourceXML) {
			var outputs = resourceXML.getElementsByTagName("output");
			var i = 1;
			_.each(outputs, function (output) {
				var name = output.getElementsByTagName("name")[0].textContent;
				var id = output.getElementsByTagName("id")[0].textContent;
				this.addPin(new OutputPinOK(this._workflow, this, id, name), i);
				i++;
			}, this);
			// temporary hack to write out correct resource XML when generating XML for node
			this._resourceXML = new XMLSerializer().serializeToString(resourceXML);
		};

		/**
		 * Get the Sequence number of a pin
		 * For the input pin this is always 0, for other pins it depends on their position
		 * @param {WebCenterPin} pin
		 * @returns {Number}sequence number (-1 if pin is not found)
		 */
		WebCenterNode.prototype.getPinSequenceNumber = function (pin) {
			var sequenceNumber = this.getPinIndex(pin.getId());

			if (this.hasInputPin() && this._inputPin !== pin) {
				sequenceNumber--;
			}

			return sequenceNumber;
		};

		/**
		 * Get the number of output pins
		 * @returns {Number}
		 */
		WebCenterNode.prototype.getOutputPinCount = function () {
			var pinCount = this.getPinCount();
			if (this.hasInputPin()) {
				pinCount--;
			}
			return pinCount;
		};

		/**
		 * Get the input pin
		 * @returns {InputPin}
		 */
		WebCenterNode.prototype.getInputPin = function () {
			return this._inputPin;
		};

		/**
		 * Returns the set error pin, if no error pin is present it returns null
		 * @returns {OutputPinError|null}
		 */
		WebCenterNode.prototype.getErrorPin = function () {
			return this._errorPin;
		};

		/**
		 * Does this node has an input pin?
		 * @returns {Boolean}
		 */
		WebCenterNode.prototype.hasInputPin = function () {
			return this._inputPin != null;
		};

		/**
		 * Generate XML definition of the node
		 * @returns {String} XMLString
		 */
		WebCenterNode.prototype.toXML = function () {
			return WebCenterNodeXML(this);
		};

		/**
		 * Generate ActionXML
		 * @returns {String} XMLString
		 */
		WebCenterNode.prototype.getActionXML = function () {
			if (this._actionXML && !this._configuration) {
				return this._actionXML;
			} else {
				return WebCenterNodeActionXML(this);
			}
		};

		/**
		 * Generate the ResourceXML for the mode
		 * @returns {String} XMLString
		 */
		WebCenterNode.prototype.getResourceXML = function () {
			// when we start changing settings of the node, we need to make a decent implementation for this
			return this._resourceXML;
		};

		/**
		 * Generate the XML for defining the layout of the node
		 * @returns {String} XMLString
		 */
		WebCenterNode.prototype.getLayoutXML = function () {
			return WebCenterNodeLayoutXML(this);
		};

		/**
		 * Generate the XML that defines the connections leaving this node
		 * @returns {String} XMLString
		 */
		WebCenterNode.prototype.getTransitionsXML = function () {
			var inputPin = this.getInputPin();
			var errorPin = this._errorPin;
			var transitionsXML = "";
			var errorImplicitNode = this.getWorkflow().getNode(WebCenterNodeConstants.Nodes.ERRORCONTINUEIMPLICIT);
			var runOverEdges = function (edge) {
				transitionsXML += edge.toXML();
			};


			var edgeCounterForWarningOutput = 1;

			// This function is used to create the transitions out of the warning pin if a warning policy is set.
			var createWarningEdge = function (edge, warningOutputName) {
				var tempWarningPin = new OutputPinWarning(this._workflow, this, warningOutputName, i18n.t(warningOutputName));
				var tempEdge = new WebCenterEdge(this.getWorkflow(),
					tempWarningPin.getId() + "/" + edgeCounterForWarningOutput,
					tempWarningPin,
					edge.getToPin());
				tempWarningPin.addEdge(tempEdge);
				runOverEdges(tempEdge);
				edgeCounterForWarningOutput++;

			};

			_.each(this.getPins(), function (pin) {
				if (pin != inputPin && pin.getPinType() !== WebCenterPinConstants.Type.OutError) {
					_.each(pin.getEdges(), function (edge) {

						// If the warning policy is "OK", all edges going out of an OK pin should be copied for the warning pin. 
						if (this._canEditWarningPolicy && pin instanceof OutputPinOK && this._warningPolicyParam.getValue() === WebCenterNodeConstants.WarningPolicy.OK) {
							_.each(this._warningOutputNames, _.partial(_.bind(createWarningEdge, this), edge));
						}

						runOverEdges(edge);
					}, this);
				} else if (pin.getPinType() === WebCenterPinConstants.Type.OutError) {
					if (pin.getEdges().length === 0) {
						// There are no edges leaving the error pin -> set implicit error edge

						if (errorImplicitNode) {
							runOverEdges(new WebCenterEdge(this.getWorkflow(), pin.getId() + "/1", pin, errorImplicitNode.getInputPin()));
						}


					} else {
						// Run over the edges and add to xml
						_.each(pin.getEdges(), function (edge) {
							runOverEdges(edge);
						});

					}
					if (pin === errorPin && this._canEditWarningPolicy && this._warningPolicyParam.getValue() === WebCenterNodeConstants.WarningPolicy.ERROR) {
						// If the warning policy is "Error", all edges going out of the error pin should be copied for the warning pin. 
						// If the error pin has no outgoing edges, route the warning node to the implicit error node
						var errorEdges = [];
						if (pin.getEdges().length === 0) {
							if (errorImplicitNode) {
								errorEdges = [new WebCenterEdge(this.getWorkflow(), pin.getId() + "/1", pin, errorImplicitNode.getInputPin())];
							}
						} else {
							pin.getEdges();
						}
						_.each(errorEdges, function (edge) {
							_.each(this._warningOutputNames, _.partial(_.bind(createWarningEdge, this), edge));
						}, this);
					}
				}
			}, this);



			return transitionsXML;
		};

		/**
		 * All WebCenter nodes are configurable by default
		 * @override
		 * @returns {Boolean} 
		 */
		WebCenterNode.prototype.isConfigurable = function () {
			return this.getConfiguration().length > 0 && this.isEnabled();
		};

		/**
		 * @override
		 */
		WebCenterNode.prototype.getConfigurationPanel = function (wfEditorContext) {
			// probably custom node configuration panels need to be made based on the node type
			var panelPromise = $.Deferred();
			this._webCenterAPI.getWorkflowDefinitions().done(_.bind(function (workflowDefinitionXML) {
				var xmls = workflowDefinitionXML.querySelectorAll(
					"TaskExecuterFactories TaskExecuterFactory");
				var promise;
				var nodeConfigPanel;
				_.each(xmls, function (xml) {
					if (xml.getElementsByTagName("type")[0].textContent === this.getNodeType()) {
						if (!this._helpTopic) {
							var helpTopicNode = xml.querySelector("helptopic");
							if (helpTopicNode) {
								this._helpTopic = helpTopicNode.textContent;
							}
						}
						nodeConfigPanel = new WebCenterNodeConfigurationPanel(this);
						var config = xml.querySelector("config-inline");
						var smartNamesList = XMLUtils.getFirstChildElementByTagName(config, WebCenterPanelConstants.SMARTNAMES);
						var nodeCustomSmartNames = [];
						if (smartNamesList) {
							var smartNames = smartNamesList.getElementsByTagName(WebCenterPanelConstants.SMARTNAME);
							_.each(smartNames, function (smartName) {
								nodeCustomSmartNames.push(CustomSmartName.fromXML(smartName));
							});
						}
						promise = this._getSubPanels(xml.querySelector("config-inline"), nodeCustomSmartNames, wfEditorContext);
					}
				}, this);
				promise.done(_.bind(function (panels) {
					_.each(panels, function (panel) {
						nodeConfigPanel.add(panel);
					});

					panelPromise.resolve(nodeConfigPanel);
				}, this));
			}, this));

			return panelPromise.promise();
		};

		/**
		 * returns all panels that are specified in xml
		 * @param {Object}   xml - xml that contains the information for creating the panel
		 * @param {Array.<CustomSmartNames>} - Custom smartnames applicable for all node smartname enabled parameters
		 * @return {JQueryPromise.<Array.<Panel>>}                   
		 */
		WebCenterNode.prototype._getSubPanels = function (xml, nodeCustomSmartNames, wfEditorContext) {
			var deferreds = []; // list of deferreds the function should wait for before resolving its own deferred
			_.each(xml.childNodes, function (nodeXML) {
				if (!nodeXML.tagName) {
					return;
				}
				var taskTypeID = this.getWorkflow().getTaskTypeID();
				var subPromise;
				var myPromise = $.Deferred();
				var tagName = nodeXML.tagName.toLowerCase();
				switch (tagName) {
				case WebCenterPanelConstants.MENUS:
					// Get the promise of the sub panels
					subPromise = this._getSubPanels(nodeXML, nodeCustomSmartNames, wfEditorContext);
					// Add a promise for theses subpanels
					deferreds.push(myPromise);

					// Require tabbed panel
					requirejs(["WFEditorWCR/workflow/panel/TabbedPanel"], _.bind(function (TabbedPanel) {
						// if the tabbedpanel has been fetched and the subpanels have been created we can add those panels
						// to the tabbed panel and resolve our own promise.
						subPromise.done(function (panels) {
							var tabbedPanel = new TabbedPanel();
							_.each(panels, function (panel) {
								tabbedPanel.add(panel);
							});
							myPromise.resolve(tabbedPanel);
						});
					}, this));
					break;
				case WebCenterPanelConstants.PARAMETERS:
					// Add a promise for theses subpanels
					deferreds.push(myPromise);
					if (nodeXML.getAttribute(WebCenterPanelConstants.PANELURL)) {
						panelUrl = nodeXML.getAttribute(WebCenterPanelConstants.PANELURL);
						requirejs([panelUrl], _.bind(function (Panel) {
							myPromise.resolve(new Panel(this._webCenterAPI, taskTypeID, nodeXML, nodeCustomSmartNames, wfEditorContext));
						}, this));

					} else {

						// Get the promise of the sub panels
						subPromise = this._getSubPanels(nodeXML, nodeCustomSmartNames, wfEditorContext);


						requirejs(["WFEditorWCR/workflow/panel/WebCenterCompositePanelComponent"], _.bind(function (WebCenterCompositePanelComponent) {
							subPromise.done(function (panels) {
								var compositePanel = new WebCenterCompositePanelComponent([]);
								compositePanel.getContext().addClass('parameter-panel form-horizontal form-compact');
								_.each(panels, function (panel) {
									compositePanel.add(panel);
								});
								myPromise.resolve(compositePanel);
							});
						}, this));
					}
					break;
				case WebCenterPanelConstants.PARAMETER:
					var name, type, defaultValue, subparametersstate, paramId, options, panelUrl, data;
					var customSmartNames = nodeCustomSmartNames.slice(); // by default the node ones are added
					if (nodeXML.hasAttribute(WebCenterPanelConstants.PANELURL)) {
						panelUrl = nodeXML.getAttribute(WebCenterPanelConstants.PANELURL);
					}
					_.each(nodeXML.childNodes, function (child) {
						if (child.tagName === WebCenterPanelConstants.NAME) {
							paramId = child.textContent;
							name = i18n.tFallback(paramId, "");
						} else if (child.tagName === WebCenterPanelConstants.TYPE) {
							type = child.textContent;
						} else if (child.tagName === WebCenterPanelConstants.DEFAULT) {
							defaultValue = child.textContent;
						} else if (child.tagName === WebCenterPanelConstants.SUBPARAMETERSTATE.TAGNAME) {
							subparametersstate = child.textContent;
						} else if (child.tagName === WebCenterPanelConstants.SMARTNAMES) {
							var smartNames = child.getElementsByTagName(WebCenterPanelConstants.SMARTNAME);
							_.each(smartNames, function (smartName) {
								customSmartNames.push(CustomSmartName.fromXML(smartName));
							});
						} else if (child.tagName === WebCenterPanelConstants.DATA) {
							data = child;
						}
					}, this);
					if (paramId) {
						if (panelUrl) {
							deferreds.push(myPromise);
							requirejs([panelUrl], _.bind(function (Panel) {
								myPromise.resolve(new Panel(paramId, name, this._webCenterAPI, data, customSmartNames, taskTypeID));
							}, this));
						} else if (type) {
							deferreds.push(myPromise);
							// Check if subclasses have panels for this type, if not, try default implementation
							if (!this._getNodeSpecificSubPanel(myPromise, type, paramId, nodeXML, customSmartNames)) {
								switch (type) {
								case WebCenterPanelConstants.STRING:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameInputField, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new SmartNameInputField(paramId, name, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));

									}, this));
									break;
								case WebCenterPanelConstants.DATE:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/DateSmartNameDictionaryService"], _.bind(function (SmartNameInputField, DateSmartNameDictionaryService) {
										myPromise.resolve(new SmartNameInputField(paramId, name, new DateSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.URL:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/URLSmartNameDictionaryService"], _.bind(function (SmartNameInputField, URLSmartNameDictionaryService) {
										myPromise.resolve(new SmartNameInputField(paramId, name, new URLSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.EMAIL:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/EmailSmartNameDictionaryService"], _.bind(function (SmartNameInputField, EmailSmartNameDictionaryService) {
										myPromise.resolve(new SmartNameInputField(paramId, name, new EmailSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.NUMBER:
									// currently not used
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameInputField, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new SmartNameInputField(paramId, name, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.PM:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/PMSmartNameDictionaryService"], _.bind(function (SmartNameInputField, PMSmartNameDictionaryService) {
										myPromise.resolve(new SmartNameInputField(paramId, name, new PMSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.ATTRIBUTE:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/AttributeSmartNameDictionaryService"], _.bind(function (SmartNameInputField, AttributeSmartNameDictionaryService) {
										// ignore custom smartnames for attribute (iterate)
										myPromise.resolve(new SmartNameInputField(paramId, name, new AttributeSmartNameDictionaryService(this._webCenterAPI, taskTypeID)));
									}, this));
									break;
								case WebCenterPanelConstants.TASKSPECVALUE:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/AttributeValueSmartNameDictionaryService"], _.bind(function (SmartNameInputField, AttributeValueSmartNameDictionaryService) {
										// TODO: improve for restricted sets and dates.
										myPromise.resolve(new SmartNameInputField(paramId, name, new AttributeValueSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.PROJECTATTRVALUE:
									requirejs(["WFEditorWCR/workflow/panel/SmartnameInputField", "WFEditorWCR/workflow/panel/AttributeValueSmartNameDictionaryService"], _.bind(function (SmartNameInputField, AttributeValueSmartNameDictionaryService) {
										// TODO: improve for restricted sets and dates.
										myPromise.resolve(new SmartNameInputField(paramId, name, new AttributeValueSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.RADIO:
									subPromise = this._getSubPanels(nodeXML, nodeCustomSmartNames, wfEditorContext);
									requirejs(["WFEditorWCR/workflow/panel/RadioGroupPanel"], _.bind(function (RadioGroupPanel) {
										// find the name
										var legend = XMLUtils.getFirstChildElementByTagName(nodeXML, "name");
										if (!legend) {
											legend = "";
										} else {
											legend = legend.textContent;
										}

										var radioGroupPanel = new RadioGroupPanel(paramId, subparametersstate, legend);
										subPromise.done(function (panels) {
											_.each(panels, function (panel) {
												radioGroupPanel.add(panel);
											});
											myPromise.resolve(radioGroupPanel);
										});

									}, this));
									break;
								case WebCenterPanelConstants.BOOLEAN:
									subPromise = this._getSubPanels(nodeXML, nodeCustomSmartNames, wfEditorContext);
									requirejs(["WFEditorWCR/workflow/panel/BooleanPanel"], _.bind(function (BooleanPanel) {
										var booleanPanel = new BooleanPanel(paramId, name, subparametersstate);
										subPromise.done(function (panels) {
											_.each(panels, function (panel) {
												booleanPanel.add(panel);
											});
											myPromise.resolve(booleanPanel);
										});
									}, this));
									break;
								case WebCenterPanelConstants.SECRET:
									requirejs(["WFEditorWCR/workflow/panel/SecretPanel"], _.bind(function (SecretPanel) {
										myPromise.resolve(new SecretPanel(paramId, name));
									}, this));
									break;
								case WebCenterPanelConstants.HTML:
									requirejs(["WFEditorWCR/workflow/panel/HTMLPanel", "WFEditorWCR/workflow/panel/HTMLSmartNameDictionaryService"], _.bind(function (HTMLPanel, HTMLSmartNameDictionaryService) {
										myPromise.resolve(new HTMLPanel(paramId, name, new HTMLSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI));
									}, this));
									break;
								case WebCenterPanelConstants.MULTIPARAMETERS:
									requirejs(["WFEditorWCR/workflow/panel/MultiParametersPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiParametersPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiParametersPanel(paramId, i18n.t("wfui.multiparam.parameterName"), i18n.t("wfui.multiparam.parameterValue"), new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTIATTRIBUTES:
									requirejs(["WFEditorWCR/workflow/panel/MultiAttributePanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService", "WFEditorWCR/workflow/panel/DateSmartNameDictionaryService"], _.bind(function (MultiAttributePanel, WorkflowSmartNameDictionaryService, DateSmartNameDictionaryService) {
										myPromise.resolve(new MultiAttributePanel(paramId, i18n.t("wfui.multiprojattr.parameterName"), i18n.t("project.attvalue"), new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), new DateSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTIPROJATTRIBUTES:
									requirejs(["WFEditorWCR/workflow/panel/MultiProjectAttributePanel"], _.bind(function (MultiProjectAttributePanel) {
										myPromise.resolve(new MultiProjectAttributePanel(paramId, i18n.t("projattr.all"), i18n.t("projattr.selected"), this._webCenterAPI));
									}, this));
									break;
								case WebCenterPanelConstants.MULTIDYNAMICCONTENTIMPORTMAP:
									requirejs(["WFEditorWCR/workflow/panel/MultiDynamicContentImportPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiDynamicContentImportPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiDynamicContentImportPanel(paramId, i18n.t("wfui.multidynamiccontentmap.elementtype"), i18n.t("wfui.multidynamiccontentmap.docrefattr"), i18n.t("wfui.multidynamiccontentmap.doctempl"),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), true, this._webCenterAPI));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITEXTCONTENTLANGUAGEMAP:
									requirejs(["WFEditorWCR/workflow/panel/MultiTextContentLanguagePanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTextContentLanguagePanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTextContentLanguagePanel(paramId, i18n.t("wfui.importdc.langindex"), i18n.t("wfui.importdc.langdesc"),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITEXTCONTENTFIELD:
									requirejs(["WFEditorWCR/workflow/panel/MultiTextContentFieldPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTextContentFieldPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTextContentFieldPanel(paramId, i18n.t("wfui.settcfield.field"), i18n.t("wfui.settcfield.value"), this._getTextContentSettingsMap(false),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITEXTCONTENTPERMISSION:
									requirejs(["WFEditorWCR/workflow/panel/MultiTextContentPermissionPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTextContentPermissionPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTextContentPermissionPanel(paramId, i18n.t("wfui.settcfield.contentpermission"), i18n.t("wfui.settcfield.language"), i18n.t("wfui.settcfield.role"),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITEXTCONTENTVALUES:
									requirejs(["WFEditorWCR/workflow/panel/MultiTableParametersPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTableParametersPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTableParametersPanel(paramId, this._getTextContentFieldsMap(),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITEXTCONTENTLANGUAGESTATUS:
									requirejs(["WFEditorWCR/workflow/panel/MultiTextContentElementPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTextContentElementPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTextContentElementPanel(paramId, i18n.t("wfui.settcfield.language"), i18n.t("wfui.settcfield.langstatus"),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITABLESETTINGS:
									requirejs(["WFEditorWCR/workflow/panel/MultiTextContentFieldPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTextContentFieldPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTextContentFieldPanel(paramId, i18n.t("wfui.settcfield.field"), i18n.t("wfui.settcfield.value"), this._getTextContentSettingsMap(true),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITABLEELEMENTCONTENT:
									requirejs(["WFEditorWCR/workflow/panel/MultiTableParametersPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTableParametersPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTableParametersPanel(paramId, this._getTableContentFieldsMap(true),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.MULTITABLEELEMENTAPPROVAL:
									requirejs(["WFEditorWCR/workflow/panel/MultiTableParametersPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (MultiTableParametersPanel, WorkflowSmartNameDictionaryService) {
										myPromise.resolve(new MultiTableParametersPanel(paramId, this._getTableContentFieldsMap(false),
											new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, true));
									}, this));
									break;
								case WebCenterPanelConstants.PROJECTSTATUS:
									requirejs(["WFEditorWCR/workflow/panel/SmartNameDropdownPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameDropdownPanel, WorkflowSmartNameDictionaryService) {
										var smartNameDropdownPanel = new SmartNameDropdownPanel(paramId, name, undefined, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames));
										this._addProjectStatusesToDropdown(smartNameDropdownPanel);
										myPromise.resolve(smartNameDropdownPanel);
									}, this));
									break;
								case WebCenterPanelConstants.TASKSTATUS:
									requirejs(["WFEditorWCR/workflow/panel/SmartNameDropdownPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameDropdownPanel, WorkflowSmartNameDictionaryService) {
										var smartNameDropdownPanel = new SmartNameDropdownPanel(paramId, name, undefined, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames));
										this._addTaskStatusesToDropdown(smartNameDropdownPanel);
										myPromise.resolve(smartNameDropdownPanel);
									}, this));
									break;
								case WebCenterPanelConstants.ATTRIBUTECATEGORY:
									requirejs(["WFEditorWCR/workflow/panel/SmartNameDropdownPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameDropdownPanel, WorkflowSmartNameDictionaryService) {
										var smartNameDropdownPanel = new SmartNameDropdownPanel(paramId, name, undefined, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames));
										this._addAttributeCategoriesToDropdown(smartNameDropdownPanel);
										myPromise.resolve(smartNameDropdownPanel);
									}, this));
									break;
								case WebCenterPanelConstants.USERS:
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getUsersGroupsRoles(), null, WorkflowSmartNameConstants.TASKUSERS.PREFIX);
									break;
								case WebCenterPanelConstants.WORKFLOWLIST:
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getWorkflowConnections(), null);
									break;
								case WebCenterPanelConstants.WORKFLOWLISTJMF:
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getWorkflowConnectionsJMF(), null);
									break;
								case WebCenterPanelConstants.TASKSPECNAME:
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getTaskSpecifications(taskTypeID), null);
									break;
								case WebCenterPanelConstants.PROJECTATTRIBUTENAME:
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getAttributes(), null);
									break;
								case WebCenterPanelConstants.DOCREFATTRIBUTEDEFAULT:
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getDocRefAttributes(), null);
									break;
								case WebCenterPanelConstants.DOCREFATTRIBUTE:
									// Adding new option 'Any Document Reference' in Attribute List
									var addCustomKeyValueToSDrop = {
										key: WebCenterPanelConstants.ANYDOCUMNETREFERENCE,
										value: new WebCenterAttribute(WebCenterPanelConstants.ANYDOCUMNETREFERENCE, i18n.t("wfui.anydocrefattr"), null, null)
									};
									this._addStaticSDropPanel(myPromise, paramId, name, defaultValue, this._webCenterAPI.getDocRefAttributes(), null, null, addCustomKeyValueToSDrop);
									break;

								case WebCenterPanelConstants.GROUPASSIGNMENT:
								case WebCenterPanelConstants.ROLEASSIGNMENT:
									requirejs(["WFEditorWCR/workflow/panel/DropdownPanel"], _.bind(function (DropdownPanel) {
										var dropdown = new DropdownPanel(paramId, i18n.t(paramId));
										options = [];
										var optionValues = (type === WebCenterPanelConstants.GROUPASSIGNMENT) ? WebCenterPanelConstants.GROUPASSIGNMENT_OPTIONS : WebCenterPanelConstants.ROLEASSIGNMENT_OPTIONS;
										_.each(optionValues, function (option) {
											options.push({
												value: option,
												label: i18n.t(option)
											});
										}, this);
										dropdown.setOptions(options);
										myPromise.resolve(dropdown);
									}, this));
									break;
								case WebCenterPanelConstants.ROLES:
								case WebCenterPanelConstants.GROUPS:
									requirejs(["WFEditorWCR/workflow/panel/SmartNameDropdownPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameDropdownPanel, WorkflowSmartNameDictionaryService) {

										var smartNameDropdownPanel = new SmartNameDropdownPanel(paramId, name, undefined, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames));
										this._webCenterAPI.getUsersGroupsRoles().done(_.bind(function (u, g, r) {
											if (type === WebCenterPanelConstants.GROUPS) {
												this._addGroupsToDropdown(smartNameDropdownPanel, g);
											} else if (type === WebCenterPanelConstants.ROLES) {
												this._addRolesToDropdown(smartNameDropdownPanel, r);
											}
										}, this));
										myPromise.resolve(smartNameDropdownPanel);

									}, this));
									break;
								case WebCenterPanelConstants.ENUM:
									requirejs(["WFEditorWCR/workflow/panel/SmartNameDropdownPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService"], _.bind(function (SmartNameDropdownPanel, WorkflowSmartNameDictionaryService) {
										var enums = nodeXML.querySelectorAll("Enum");
										options = {};
										_.each(enums, function (enumNode) {
											var option = {};
											option.value = enumNode.textContent;
											option.text = i18n.t(option.value);
											options[option.value] = option;
										}, this);
										myPromise.resolve(new SmartNameDropdownPanel(paramId, name, options, new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames)));
									}, this));
									break;
								case WebCenterPanelConstants.JAVASCRIPT:
									requirejs(["WFEditorWCR/workflow/panel/JavaScriptPanel"], _.bind(function (JavaScriptPanel) {
										var options = {};
										var maxScriptLength = data.querySelector("maxScriptLength");
										if (maxScriptLength.textContent !== undefined) {
											maxScriptLength = parseInt(maxScriptLength.textContent, 10);
											options.maxScriptLength = maxScriptLength;
										}
										myPromise.resolve(new JavaScriptPanel(paramId, name, options));
									}, this));
									break;
								default:
									requirejs(["WFEditorWCR/workflow/panel/ReadOnlyStringPanel"], _.bind(function (ReadOnlyStringPanel) {
										myPromise.resolve(new ReadOnlyStringPanel(paramId, name));
									}, this));
									break;

								}
							}
						}
					}
					break;
				case WebCenterPanelConstants.MENU:
					// a menu indicates a tab in the tabbel panel
					deferreds.push(myPromise);
					subPromise = this._getSubPanels(nodeXML, nodeCustomSmartNames, wfEditorContext);
					requirejs(["WFEditorWCR/workflow/panel/TabPanel"], _.bind(function (TabPanel) {
						var tab = new TabPanel([], i18n.t(nodeXML.getAttribute(WebCenterPanelConstants.NAME)));
						subPromise.done(function (panels) {
							_.each(panels, function (panel) {
								tab.add(panel);
							});
							myPromise.resolve(tab);
						});
					}, this));
					break;
				case WebCenterPanelConstants.RADIO:
					deferreds.push(myPromise);
					subPromise = this._getSubPanels(nodeXML, nodeCustomSmartNames, wfEditorContext);
					requirejs(["WFEditorWCR/workflow/panel/RadioPanel"], _.bind(function (RadioPanel) {
						var name = nodeXML.getElementsByTagName(WebCenterPanelConstants.NAME)[0].textContent;
						var radioComponent = new RadioPanel(name, i18n.t(name));
						subPromise.done(function (panels) {
							_.each(panels, function (panel) {
								radioComponent.add(panel);
							});
							myPromise.resolve(radioComponent);
						});

					}, this));
					break;
				}

			}, this);

			var returnPromise = $.Deferred();

			if (deferreds.length > 0) {

				$.when.apply($, deferreds).done(_.bind(function () {
					var panels = [];
					_.each(arguments, function (arg) {
						this._panelAdded(arg);
						panels.push(arg);
					}, this);
					returnPromise.resolve(panels);
				}, this));
			} else {
				returnPromise.resolve([]);
			}

			return returnPromise.promise();
		};

		/**
		 * Add a panel with a static SDrop provider
		 * @param {$.Deferred}                  deferred     - resolved when the sdrop panel is initialized
		 * @param {String}                      paramId      - ID of the parameter
		 * @param {String}                      name         - name of the parameter
		 * @param {String}                      defaultValue - default value of the parameter
		 * @param {Promise.<Map.<String, Any>>} mapPromise   - promise for the data of the SDrop
		 * @param {Function}                    [orderMethod]  - optional order method 
		 * @param {String}                      [idPrefix]    - Prefix for the keys in the map  
		 * @param {Object}                      [addCustomKeyValueToSDrop] - Object contains Key and Value. Structure of this object is like {key:"key", value: Any}. The idPrefix if supplied is NOT applied to this object.                                            
		 */
		WebCenterNode.prototype._addStaticSDropPanel = function (deferred, paramId, name, defaultValue, mapPromise, orderMethod, idPrefix, addCustomKeyValueToSDrop) {
			requirejs(["WFEditorWCR/workflow/panel/StaticSDropPanel"], _.bind(function (StaticSDropPanel) {
				var promise = mapPromise;
				var newDeferredMap = $.Deferred();

				mapPromise.done(function (map) {
					var newMap = map.transformKeys(function (key) {
						if (idPrefix) {
							return idPrefix + key;
						} else {
							return key;
						}
					});

					if (addCustomKeyValueToSDrop) {
						newMap.add(addCustomKeyValueToSDrop.key, addCustomKeyValueToSDrop.value);
					}
					newDeferredMap.resolve(newMap);
				});
				promise = newDeferredMap.promise();

				var sdropPanel = new StaticSDropPanel(paramId, name, defaultValue, promise, orderMethod);
				deferred.resolve(sdropPanel);
			}, this));
		};


		/**
		 * returns the parameters and outputs for the given nodeType, the result is cached based on the nodeType
		 * @param {String} nodeType - the type of the node
		 * @param {NodeList} workflowDefinitionXML - the xml that should be searched in if it hasn't been cached yet                         
		 * @returns Object {
		 *				parameters: Array.<Parameter> // see _getParameters
		 *				outputs: Array.<Object> // see _getOutputPins
		 *			}
		 */
		WebCenterNode.prototype._getConfigDetailsForNodeType = _.memoize(function (nodeType, workflowDefinitionXML) {
			var parameters;
			var outputs;
			var helpTopic;
			var xmls = workflowDefinitionXML.querySelectorAll(
				"TaskExecuterFactories TaskExecuterFactory");
			_.each(xmls, function (xml) {
				if (xml.getElementsByTagName("type")[0].textContent === nodeType) {
					var config = xml.querySelector("config-inline");
					parameters = _getParameters(config); // _getParameters static function because non bound memoized function
					outputs = _getOutputPins(config); // _getOutputPins static function because non bound memoized function
					helpTopic = _getHelpTopic(config);
				}
			}, this);
			return {
				parameters: parameters,
				outputs: outputs,
				helpTopic: helpTopic
			};
		});

		/**
		 * Get the help topic of this node from the given XML
		 * @param   {NodeList} xml
		 * @returns {String}
		 */
		var _getHelpTopic = function (xml) {
			var helpTopicNode = xml.getElementsByTagName("helptopic")[0];
			if (helpTopicNode) {
				return helpTopicNode.textContent;
			}
			return "";
		};

		/**
		 * returns the default parameters as specified in xml
		 * @param {NodeList} xml
		 * @returns {Array.<Parameter>}
		 */
		var _getParameters = function (xml) {
			var parameters = [];
			_.each(xml.childNodes, function (node) {
				var tagName = node.tagName;
				switch (tagName) {
				case WebCenterPanelConstants.MENUS:
					parameters = parameters.concat(_getParameters(node));
					break;
				case WebCenterPanelConstants.PARAMETERS:
					parameters = parameters.concat(_getParameters(node));
					break;
				case WebCenterPanelConstants.PARAMETER:
					var paramId, defaultValue;
					_.each(node.childNodes, function (child) {
						if (child.tagName === WebCenterPanelConstants.NAME) {
							paramId = child.textContent;
						} else if (child.tagName === WebCenterPanelConstants.DEFAULT) {
							var enums = child.getElementsByTagName(WebCenterPanelConstants.ENUM);
							if (enums.length > 0) {
								defaultValue = enums[0].textContent;
							} else {
								defaultValue = child.textContent;
							}
						}
					}, this);
					if (paramId) {
						parameters = parameters.concat([new WebCenterParameter(paramId, defaultValue)]);
						parameters = parameters.concat(_getParameters(node));
					}
					break;
				case WebCenterPanelConstants.MENU:
					parameters = parameters.concat(_getParameters(node));
					break;
				case WebCenterPanelConstants.RADIO:
					parameters = parameters.concat(_getParameters(node));
					break;
				}
			}, this);
			return parameters;
		};

		/**
		 * returns the outputs as specified in the xml
		 * @param {NodeList} xml
		 * @returns {Array.<{
		 *		type: type of the pin (0, 1 or 2)
		 *		name: name of the pin
		 *	}>}
		 */
		var _getOutputPins = function (xml) {
			var outputPins = [];

			var outputs = xml.getElementsByTagName(WebCenterPanelConstants.OUTPUTS);

			if (outputs[0]) {
				var outputList = outputs[0].getElementsByTagName(WebCenterPanelConstants.OUTPUT);

				_.each(outputList, function (output) {
					var name = output.getElementsByTagName(WebCenterPanelConstants.NAME)[0];
					var type = output.getElementsByTagName(WebCenterPanelConstants.TYPE)[0];

					if (name && type) {
						type = type.textContent;
						name = name.textContent;
						outputPins.push({
							type: type,
							name: name
						});
					}
				});
			}

			return outputPins;
		};

		/**
		 * Add project statuses to task status dropdown
		 * @param {SmartNameDropdownPanel} smartNameDropdownPanel - panel to add options to
		 */
		WebCenterNode.prototype._addProjectStatusesToDropdown = function (smartNameDropdownPanel) {
			this._webCenterAPI.getProjectStatusList().done(_.bind(function (statusesMap) {
				this._addStatusesToDropdown(statusesMap, smartNameDropdownPanel);
			}, this));
		};

		/**
		 * Add task statuses to task status dropdown
		 * @param {SmartNameDropdownPanel} smartNameDropdownPanel - panel to add options to
		 */
		WebCenterNode.prototype._addTaskStatusesToDropdown = function (smartNameDropdownPanel) {
			this._webCenterAPI.getWorkflowTaskStatusList(this.getWorkflow().getTaskTypeID()).done(_.bind(function (statusesMap) {
				this._addStatusesToDropdown(statusesMap, smartNameDropdownPanel);
			}, this));
		};

		/**
		 * Add statuses to a status dropdown
		 * @param {SmartNameDropdownPanel} smartNameDropdownPanel - panel to add options to
		 */
		WebCenterNode.prototype._addStatusesToDropdown = function (statusesMap, smartNameDropdownPanel) {
			var statuses = statusesMap.values().sort(function (a, b) {
				return a.getDisplayOrder() - b.getDisplayOrder();
			});

			var options = [];
			_.each(statuses, function (status) {
				options.push({
					text: status.getName(),
					value: status.getId()
				});
			});
			smartNameDropdownPanel.setOptions(options);
		};

		/**
		 * Add Attribute Categories to smart name dropdown
		 * @param {SmartNameDropdownPanel} smartNameDropdownPanel - panel to add options to
		 */
		WebCenterNode.prototype._addAttributeCategoriesToDropdown = function (smartNameDropdownPanel) {
			this._webCenterAPI.getAttributeCategoryList().done(_.bind(function (attributeCategoriesMap) {
				var options = [];
				_.each(attributeCategoriesMap.values(), function (attCat) {
					options.push({
						text: attCat.getName(),
						value: attCat.getId()
					});
				});
				smartNameDropdownPanel.setOptions(options);
			}, this));
		};

		/**
		 * add correct groups to a groups dropdown
		 * @param   {SmartNameDropdownPanel}       smartNameDropdownPanel 
		 * @param   {Map.<String,WebCenterGroup>}  g
		 */
		WebCenterNode.prototype._addGroupsToDropdown = function (smartNameDropdownPanel, g) {
			var mappedGroups = _.mapObject(g.asObject(), function (group) {
				return {
					text: group.getName(),
					value: WorkflowSmartNameConstants.TASKGROUPS.PREFIX + group.getId()
				};
			});
			smartNameDropdownPanel.setOptions(mappedGroups);

		};


		/**
		 * add correct roles to a roles dropdown
		 * @param   {SmartNameDropdownPanel}       smartNameDropdownPanel 
		 * @param   {Map.<String,WebCenterRole>}   r
		 */
		WebCenterNode.prototype._addRolesToDropdown = function (smartNameDropdownPanel, r) {
			// Add default roles 
			var invprojectmgrName = i18n.t(WebCenterPanelConstants.ROLE.PROJECTMANAGER.NAME);
			var invsalespersonName = i18n.t(WebCenterPanelConstants.ROLE.SALESPERSON.NAME);
			var invworkflowmaster = i18n.t(WebCenterPanelConstants.ROLE.WORKFLOWMASTER.NAME);
			var options = {};
			options[WebCenterPanelConstants.ROLE.PROJECTMANAGER.VALUE] = {
				text: "[" + invprojectmgrName + "]",
				value: WebCenterPanelConstants.ROLE.PROJECTMANAGER.VALUE
			};
			options[WebCenterPanelConstants.ROLE.SALESPERSON.VALUE] = {
				text: "[" + invsalespersonName + "]",
				value: WebCenterPanelConstants.ROLE.SALESPERSON.VALUE
			};
			options[WebCenterPanelConstants.ROLE.WORKFLOWMASTER.VALUE] = {
				text: "[" + invworkflowmaster + "]",
				value: WebCenterPanelConstants.ROLE.WORKFLOWMASTER.VALUE
			};

			// Map roles to the right format and add prefix to the id
			var mappedRoles = _.mapObject(r.asObject(), function (role) {
				return {
					text: role.getName(),
					// Roles in this context also need the group prefix due to historic reasons
					value: WorkflowSmartNameConstants.TASKGROUPS.PREFIX + role.getId()
				};
			});

			smartNameDropdownPanel.setOptions(_.extend(options, mappedRoles));
		};

		/**
		 * Constructs a map for the fields required for Content Settings
		 * @param isTableContent
		 * @returns {Map.<String,WebCenterTextContentProperty>}
		 */
		WebCenterNode.prototype._getTextContentSettingsMap = function (isTableContent) {
			var settingsMap = new Map();

			if (!isTableContent) {
				settingsMap.add(ContentConstants.UpdateField.ELEMENT_TYPE, new WebCenterTextContentProperty(ContentConstants.UpdateField.ELEMENT_TYPE, i18n.t('settcfield.elementtype')));
			}
			settingsMap.add(ContentConstants.UpdateField.APPROVAL_TYPE, new WebCenterTextContentProperty(ContentConstants.UpdateField.APPROVAL_TYPE, i18n.t('settcfield.approvaltype')));
			settingsMap.add(ContentConstants.UpdateField.TEXTTYPE, new WebCenterTextContentProperty(ContentConstants.UpdateField.TEXTTYPE, i18n.t('settcfield.texttype')));
			settingsMap.add(ContentConstants.UpdateField.INVARIANT, new WebCenterTextContentProperty(ContentConstants.UpdateField.INVARIANT, i18n.t('settcfield.invariant')));

			return settingsMap;
		};

		/**
		 * Constructs a map for the fields required for Table Content tabs
		 * @param isValuesTab
		 * @returns {Map.<String,String>}
		 */
		WebCenterNode.prototype._getTableContentFieldsMap = function (isValuesTab) {
			var fieldMap = new Map();
			fieldMap.add("wfui.settc.element", "elementtype_smartdropdown");
			fieldMap.add("wfui.settc.instseq", "instanceseq_smartinput");
			fieldMap.add("wfui.settc.language", "language_smartdropdown");

			if (isValuesTab) {
				fieldMap.add("wfui.settc.action", "languageaction_smartdropdown");
				fieldMap.add("wfui.settc.value", "content_smartinput");
			} else {
				fieldMap.add("wfui.settc.status", "approval_smartdropdown");
			}

			return fieldMap;
		};

		/**
		 * Constructs a map for the fields required for text Content tabs
		 * @returns {Map.<String,String>}
		 */
		WebCenterNode.prototype._getTextContentFieldsMap = function () {
			var fieldMap = new Map();
			fieldMap.add("wfui.settc.language", "language_smartdropdown");
			fieldMap.add("wfui.settc.action", "languageaction_smartdropdown");
			fieldMap.add("wfui.settc.value", "content_smartinput");

			return fieldMap;
		};


		/**
		 * Check if there is a panel of type that is specific for a node. Should be overwritten by the specific implementation of the webcenternode
		 * @param   {$.Deferred}               deferred         - deferred object to be resolved with the subpanel
		 * @param   {String}                   type             - the parameter type
		 * @param   {String}                   paramId          - id of the parameter
		 * @param   {NodeList}                 nodeXML          - XML of the node
		 * @param   {Array.<CustomSmartNames>} customSmartNames - list of custom smartnames
		 * @returns {Boolean}                  is there a node specific panel with this type? 
		 *                                     return true if so and the deferred will be resolved, false if deferred won't be resolved 
		 */
		WebCenterNode.prototype._getNodeSpecificSubPanel = function (deferred, type, paramId, nodeXML, customSmartNames) {
			return false;
		};

		/**
		 * @override
		 * @param isHighlighted
		 */
		WebCenterNode.prototype.setHighlighted = function (isHighlighted) {
			_.each(this.getPins(), function (pin) {
				pin.setHighlighted(isHighlighted);
			});
			Node.prototype.setHighlighted.call(this, isHighlighted);
		};

		/**
		 * called when a panel is added to the configuration panel
		 * @param {AbstractPanelComponent} panel 
		 */
		WebCenterNode.prototype._panelAdded = function (panel) {
			// Override me	
		};

		/**
		 * returns the location of the help page for this node
		 * @return {String}
		 */
		WebCenterNode.prototype.getHelpTopic = function () {
			return this._helpTopic;
		};

		/** 
		 * This is used in WebCenterWorkflowFactory.createNode to create a copy of this node.
		 * @override
		 */
		WebCenterNode.prototype.getStateObjectForCopy = function () {
			return {
				nodeType: this.getNodeType(),
				configurationXML: this.toXML(),
				resourceXML: this.getResourceXML(),
				label: this.getLabel()
			};
		};

		/** 
		 * initializes the warning policy params
		 */
		WebCenterNode.prototype._initWarningPolicyParams = function () {
			this._warningPolicyParam = new WebCenterParameter(WebCenterNodeConstants.WarningPolicy.WARNINGPOLICY, "", WebCenterNodeConstants.WarningPolicy.DEDICATED);
			this._warningPolicyOk = new WebCenterParameter(WebCenterNodeConstants.WarningPolicy.OK, "");
			this._warningPolicyError = new WebCenterParameter(WebCenterNodeConstants.WarningPolicy.ERROR, "");
			this._warningPolicyDedicated = new PinParameter(WebCenterNodeConstants.WarningPolicy.DEDICATED, "", "", [], _.bind(function () {
				var pins = [];
				_.each(this._warningOutputNames, function (warningOutputName) {
					pins.push(new OutputPinWarning(this._workflow, this, warningOutputName, i18n.t(warningOutputName)));
				}, this);
				return pins;
			}, this));
			this._warningPolicyDedicated.setNewPin();

			this._updateWarningPins();

		};

		/**
		 * sets the warning policy parameters
		 * @param {Array.<WebCenterParameter>}
		 */
		WebCenterNode.prototype.setWarningPolicy = function (config) {
			_.each(config, _.bind(function (param) {
				if (param.getName() === WebCenterNodeConstants.WarningPolicy.WARNINGPOLICY) {
					this._warningPolicyParam = param;
				} else if (param.getName() === WebCenterNodeConstants.WarningPolicy.OK) {
					this._warningPolicyOk = param;
				} else if (param.getName() === WebCenterNodeConstants.WarningPolicy.DEDICATED) {
					this._warningPolicyDedicated = param;
				} else {
					this._warningPolicyError = param;
				}
			}, this));
			this._updateWarningPins();
		};

		/**
		 * updates the warning pin according to the warning policy params. If the dedicated policy is set, the warning output pin is added
		 */
		WebCenterNode.prototype._updateWarningPins = function () {
			var i;
			var pin;
			var pins;
			if (this._warningPolicyParam.getValue() === WebCenterNodeConstants.WarningPolicy.DEDICATED) {

				pins = this._warningPolicyDedicated.pins;

				// Check if the warning pins exist already
				for (var p in pins) {
					if (pins.hasOwnProperty(p)) {
						if (_.contains(this._pins.getItemsAsObject(), pins[p])) {
							// The warning pins exist already
							return;
						}
					}
				}

				// look for index of first error pin
				for (i = 0; i < this._pins.getLength(); i++) {
					if (this._pins.getItemByIndex(i).getPinType() === WebCenterPinConstants.Type.OutError) {
						break;
					}
				}
				_.each(pins, function (pin) {
					// Add all warning pins and the edges
					this.addPin(pin, i);
					_.each(pin.getEdges(), _.bind(function (edge) {
						edge.getToPin().addEdge(edge);
						this._workflow.addEdge(edge);
					}, this));
					i++;
				}, this);

			} else {
				pins = [];
				// Get all warning pins
				for (i = 0; i < this._pins.getLength(); i++) {
					pin = this._pins.getItemByIndex(i);
					if (pin.getPinType() === WebCenterPinConstants.Type.OutWarning) {
						pins.push(pin);
					}
				}

				// Remove them
				_.each(pins, function (pin) {
					this.removePin(pin);
					_.each(pin.getEdges(), _.bind(function (edge) {
						edge.getToPin().removeEdge(edge);
						this._workflow.removeEdge(edge);
					}, this));
				}, this);

			}
		};

		/**
		 * returns the warning policy params
		 * @return {Array.<WebCenterParameter>}
		 */
		WebCenterNode.prototype.getWarningPolicy = function () {
			var config = [];
			config.push(this._warningPolicyParam.copy());
			config.push(this._warningPolicyOk.copy());
			config.push(this._warningPolicyError.copy());
			config.push(this._warningPolicyDedicated.copy());
			return config;
		};

		/**
		 * returns the label key for the warning output
		 * @return {Array.<String>}
		 */
		WebCenterNode.prototype.getWarningLabels = function () {
			return this._warningOutputNames;
		};

		/**
		 * returns the selected warning policy as string
		 * @return {String}
		 */
		WebCenterNode.prototype.getWarningPolicyString = function () {
			return this._warningPolicyParam.getValue();
		};

		/**
		 * can the warning policy be edited?
		 * @return {Boolean}
		 */
		WebCenterNode.prototype.canEditWarningPolicy = function () {
			return this._canEditWarningPolicy;
		};


		return WebCenterNode;
	});


define('tpl!WFEditorWCR/workflow/node/endState/EndStateNode', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='';
 switch(d.getNodeType()){ 
	case "completed":
	/**** NORMAL COMPLETED NODE ****/

__p+='\r\n<end-state name="'+
((__t=(d.getId()))==null?'':_.escape(__t))+
'" outctxt="'+
((__t=(d.getId()))==null?'':_.escape(__t))+
'" />\r\n';

	break;
	case "errorcontinue":

__p+='\r\n<end-state name="'+
((__t=(d.getId()))==null?'':_.escape(__t))+
'" type="errorcontinue" outctxt="'+
((__t=(d.getId()))==null?'':_.escape(__t))+
'" ';
 if (d.isImplicit()){ 
__p+='implicit="1"';
 } 
__p+='/>\r\n';
 break;
	default:
	/* by default just have end-state node with its type, works correctly for cancel node */

__p+='\r\n<end-state name="'+
((__t=(d.getId()))==null?'':_.escape(__t))+
'" type="errorstop" />\r\n';

} 
__p+='';
return __p;
}});
define('WFEditorWCR/workflow/node/endState/EndStateNode',[
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"tpl!WFEditorWCR/workflow/node/endState/EndStateNode",
		"utils/webi18n!WFEditorWCR"
	],
	function (
		WebCenterNode,
		WebCenterNodeConstants,
		EndStateNodeXML,
		i18n
	) {
		"use strict";

		/**
		 * EndStateNode represents the model for an end node in the WebCenter Workflow Editor.
		 *
		 * @exports EndStateNode
		 * @class
		 * @augments WebCenterNode
		 *
		 * @param {Wofkflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node
		 * @param {String} pathToIcons - gives the url to the map where the icons can be found
		 * @param {String} [label] - the label of the node
		 * @param {String} nodeType - specifies the type of the node
		 */
		var EndStateNode = function (workflow, id, pathToIcons, label, nodeType, webCenterAPI) {
			var icon = null;
			if (!label) {
				label = i18n.t("Completed");
			}
			if (nodeType === WebCenterNodeConstants.Nodes.COMPLETED) {
				icon = WebCenterNodeConstants.Icons.COMPLETED;
			} else if (nodeType === WebCenterNodeConstants.Nodes.CANCEL) {
				icon = WebCenterNodeConstants.Icons.CANCEL;
			} else {
				icon = WebCenterNodeConstants.Icons.ERROR;
			}

			WebCenterNode.call(this, workflow, id, nodeType, pathToIcons + icon, label, webCenterAPI);

			/**
			 * @member {Boolean} for implicit/invisible endstate node this is true
			 * @private
			 */
			this._isImplicit = false;
			
			/**
			 * @member {String} the id of the end state. Not to be confused with the id of the node
			 * @private
			 */
			this._endStateId = id.substr(0, id.lastIndexOf("_"));
		};

		EndStateNode.prototype = Object.create(WebCenterNode.prototype);
		EndStateNode.prototype.constructor = EndStateNode;

		/**
		 * @Override
		 */
		EndStateNode.prototype.setNodeXML = function (nodeEl) {
			WebCenterNode.prototype.setNodeXML.apply(this, arguments);

			this._isImplicit = nodeEl.getAttribute("implicit");

		};

		/** 
		 * Is the node implicit?
		 * @Override
		 */
		EndStateNode.prototype.isImplicit = function () {
			return this._isImplicit;
		};

		/**
		 * sets whether or not the node is implicit
		 * @param {Boolean} isImplicit
		 */
		EndStateNode.prototype.setImplicit = function (isImplicit) {
			this._isImplicit = isImplicit;
		};

		/**
		 * @Override
		 */
		EndStateNode.prototype._initializePins = function () {
			this._addInputPin();
		};

		/**
		 * @Override
		 */
		EndStateNode.prototype.getLayoutXML = function () {
			if (this.isImplicit()) {
				return "";
			} else {
				return WebCenterNode.prototype.getLayoutXML.call(this);
			}
		};
		/**
		 * Make sure that an implicit end state node is not visible
		 * @Override
		 */
		EndStateNode.prototype.isVisible = function () {
			return WebCenterNode.prototype.isVisible.call(this) && !this.isImplicit();
		};

		/**
		 * Generate Node XML
		 * @Override
		 */
		EndStateNode.prototype.toXML = function () {
			return EndStateNodeXML(this);
		};

		/**
		 * @override
		 */
		EndStateNode.prototype.getHelpTopic = function () {
			if (this.getNodeType() === WebCenterNodeConstants.Nodes.CANCEL) {
				return WebCenterNodeConstants.HelpTopics.CANCEL;
			}
			return "";
		};
	
		/**
		 * @override
		 * Adds the endStateId to the state object.
		 */
		EndStateNode.prototype.getStateObjectForCopy = function () {
			var stateObject = WebCenterNode.prototype.getStateObjectForCopy.call(this);
			/* In this context the id is of the type of state, not the id of the instance of the end state,
			   as the id of the instance shouldn't be copied along */
			stateObject.id = this._endStateId;
			return stateObject;
		}
	
		return EndStateNode;
	});


define('tpl!WFEditorWCR/workflow/node/startNode/StartNode', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<start-state name="'+
((__t=(d.getId()))==null?'':_.escape(__t))+
'">\r\n'+
((__t=(d.getTransitionsXML()))==null?'':__t)+
'\r\n</start-state>';
return __p;
}});
define('WFEditorWCR/workflow/node/startNode/StartNode',[
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"tpl!WFEditorWCR/workflow/node/startNode/StartNode",
		"utils/webi18n!WFEditorWCR"
	],
	function (
		WebCenterNode,
		WebCenterNodeConstants,
		OutputPinOK,
		WebCenterPinConstants,
		StartNodeXML,
		i18n
	) {
		"use strict";

		/**
		 * StartNode represents the model for a  StartNode node in the WebCenter Workflow Editor.
		 *
		 * @exports StartNode
		 * @class
		 * @augments WebCenterNode
		 *
		 * @param {Wofkflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {String} pathToIcons - gives the url to the map where the icons can be found
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var StartNode = function (workflow, id, pathToIcons, label, webCenterAPI) {
			var icon = WebCenterNodeConstants.Icons.START;
			if (!label) {
				label = i18n.t("Start");
			}
			WebCenterNode.call(this, workflow, id, WebCenterNodeConstants.Nodes.START, pathToIcons + icon, label, webCenterAPI);

		};

		StartNode.prototype = Object.create(WebCenterNode.prototype);
		StartNode.prototype.constructor = StartNode;


		/**
		 * @Override
		 */
		StartNode.prototype._initializePins = function () {
			this.addPin(new OutputPinOK(this._workflow, this, WebCenterPinConstants.Type.WorkflowConstructOK));
		};

		/**
		 * @Override
		 */
		StartNode.prototype.toXML = function () {
			return StartNodeXML(this);
		};

		/**
		 * @override
		 */
		StartNode.prototype.isRemovable = function () {
			return false;
		};
	
		/**
		 * @override
		 */
		StartNode.prototype.isCopyable = function () {
			return false;
		};

		return StartNode;
	});


define('tpl!WFEditorWCR/workflow/WebCenterWorkflow', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<?xml version="1.0" encoding="UTF-8"?>\r\n<bswf:process-definition xmlns:bswf="urn:jbpm.org:jpdl-3.2">\r\n	';
 _.each(d.getSaveableNodes(), function(node){
		print(node.toXML());
	}); 
__p+='\r\n	<resources> \r\n		';
 _.each(d.getSaveableNodes(), function(node){
			print(node.getResourceXML());
		}); 
__p+='\r\n	</resources>\r\n	<layout>\r\n		';
 _.each(d.getSaveableNodes(), function(node){
			print(node.getLayoutXML());
		}); 
__p+='\r\n	</layout>\r\n	<extra>\r\n		<wfEditor type="HTML5" buildString="'+
((__t=( d.getWebCenterConfig().buildString ))==null?'':_.escape(__t))+
'" userAgent="'+
((__t=( d.getWebCenterConfig().userAgent ))==null?'':_.escape(__t))+
'"  />\r\n		<stickynotes>\r\n			';
 _.each(d.getStickyNotes(), function(stickyNote) {
			   print(stickyNote.toXML());
			   }); 
__p+='\r\n		</stickynotes>\r\n	</extra>	\r\n</bswf:process-definition>';
return __p;
}});

define('tpl!WFEditorWCR/workflow/helper/stickyNote/StickyNote', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<stickynote preflocx="'+
((__t=( Math.round(d.getLocation().x) ))==null?'':_.escape(__t))+
'" preflocy="'+
((__t=( Math.round(d.getLocation().y) ))==null?'':_.escape(__t))+
'" width="'+
((__t=( d.getSize().width ))==null?'':_.escape(__t))+
'" height="'+
((__t=( d.getSize().height ))==null?'':_.escape(__t))+
'">'+
((__t=( d.getText() ))==null?'':_.escape(__t))+
'</stickynote>';
return __p;
}});
define('WFEditorWCR/workflow/helper/stickyNote/StickyNoteHelper',["underscore",
		"WFEditor/workflow/helper/Helper",
		"utils/ShapeUtils",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"tpl!./StickyNote"
	],
	function (_,
		Helper,
		ShapeUtils,
		WebCenterNodeConstants,
		StickyNoteXML) {

		"use strict";

		// integer for generating a unique stickynote id - only used client side
		var stickyNoteIdGenerator = 0;

		/**
		 * @class
		 * @exports StickyNoteHelper
		 * @augments Helper
		 * @param {Workflow}            workflow 
		 * @param {ShapeUtils.Location} location 
		 * @param {ShapeUtils.Size}     size     
		 * @param {String}              text     
		 */
		var StickyNoteHelper = function (workflow, location, size, text) {
			Helper.call(this, workflow, "sticky_" + stickyNoteIdGenerator++);

			/**
			 * @param {ShapeUtils.Point}
			 * @private
			 */
			this._location = location;

			/**
			 * @param {ShapeUtils.Size}
			 * @private
			 */
			this._size = size || new ShapeUtils.Size(WebCenterNodeConstants.StickyNote.DEFAULT_WIDTH, WebCenterNodeConstants.StickyNote.DEFAULT_HEIGHT);

			/**
			 * @param {String}
			 * @private
			 */
			this._text = text;

			/**
			 * @member {Boolean}
			 * @private
			 */
			this._isBeingEdited = false;

		};

		StickyNoteHelper.prototype = Object.create(Helper.prototype);
		StickyNoteHelper.prototype.constructor = StickyNoteHelper;

		/**
		 * sets the location of the sticky note
		 * @param {ShapeUtils.Point} location 
		 */
		StickyNoteHelper.prototype.setLocation = function (location) {
			if (location.x !== this._location.x && location.y !== this._location.y) {
				this._location = location;
				this._emit("changed.location");
			}
		};

		/**
		 * moves the location of the sticky note horizontally by dx and vertically by dy
		 * @param {Number} dx 
		 * @param {Number} dy 
		 */
		StickyNoteHelper.prototype.offsetXY = function (dx, dy) {
			if (dx !== 0 || dy !== 0) {
				this._location.offsetXY(dx, dy);
				this._emit("changed.location");
			}
		};

		/**
		 * returns the location of the sticky note
		 * @returns {ShapeUtils.Point} 
		 */
		StickyNoteHelper.prototype.getLocation = function () {
			return this._location;
		};

		/**
		 * sets the size of the sticky note
		 * @param {Number} width
		 * @param {Number} height
		 */
		StickyNoteHelper.prototype.setSize = function (width, height) {
			if (this._size.width !== width ||
				this._size.height !== height
			) {
				this._size.width = Math.max(Math.round(width), WebCenterNodeConstants.StickyNote.MIN_WIDTH);
				this._size.height = Math.max(Math.round(height), WebCenterNodeConstants.StickyNote.MIN_HEIGHT);
				this._emit("changed.size");
			}
		};

		/**
		 * gets the size of the sticky note
		 * @returns {ShapeUtils.Size} 
		 */
		StickyNoteHelper.prototype.getSize = function () {
			return this._size;
		};

		/**
		 * sets the text of the sticky notez
		 * @param {String} text 	
		 */
		StickyNoteHelper.prototype.setText = function (text) {
			if (text !== this._text) {
				this._text = text;
				this._emit("changed.text");
			}
		};

		/**
		 * @returns {String} the text of the sticky note
		 */
		StickyNoteHelper.prototype.getText = function () {
			return this._text;
		};

		/**
		 * returns the xml for the sticky note
		 * @returns {String} 
		 */
		StickyNoteHelper.prototype.toXML = function () {
			return StickyNoteXML(this);
		};

		/**
		 * sets whether or not this sticky note is being edited
		 * @param {Boolean} isEditing 
		 */
		StickyNoteHelper.prototype.setEditingText = function (isEditing) {
			if (this._isBeingEdited !== isEditing) {
				this._isBeingEdited = isEditing;
				this._emit("changed.editing");
			}
		};

		/**
		 * @returns {Boolean} if the text is being edited
		 */
		StickyNoteHelper.prototype.isEditingText = function () {
			return this._isBeingEdited;
		};

		return StickyNoteHelper;

	});

define('WFEditorWCR/workflow/WebCenterWorkflow',[
		"WFEditor/workflow/Workflow",
		"utils/webi18n!WFEditorWCR",
		"WFEditorWCR/workflow/node/endState/EndStateNode",
		"WFEditorWCR/workflow/node/startNode/StartNode",
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"utils/ShapeUtils",
		"tpl!WFEditorWCR/workflow/WebCenterWorkflow",
		"WFEditorWCR/workflow/helper/stickyNote/StickyNoteHelper",
		"WFEditorWCR/workflow/WebCenterWorkflowState",

		"underscore"

	],
	function (
		Workflow,
		i18n,
		EndStateNode,
		StartNode,
		WebCenterNode,
		WebCenterNodeConstants,
		ShapeUtils,
		WebCenterWorkflowXML,
		StickyNoteHelper,
		WebCenterWorkflowState,
		_
	) {
		"use strict";

		/**
		 * WebCenterWorkflow contains specific model data for WebCenter
		 *
		 * @exports WebCenterWorkflow
		 * @class
		 * @augments Workflow
		 *
		 * @param {String} ID
		 * @param {String} taskTypeID - ID of the tasktype
		 * @param {String} taskID - WebCenter workflow task ID
		 * @param {String initialTaskStatusID - WebCenter Task Status ID of the workflow when the object was initialized
		 */
		var WebCenterWorkflow = function (ID, taskTypeID, taskID, initialTaskStatusID) {
			Workflow.call(this, ID);

			/**
			 * @member {String} config object with information of webcenter, loaction of the images, jsppages,...
			 * @private
			 */
			this._wcrConfig = null;

			/**
			 * @member {WebCenterWorkflowState} State of the workflow
			 * @private
			 */
			this._workflowState = null;

			/**
			 * @member {String} the current status of the workflow task
			 * @private
			 */
			this._taskStatus = null;

			/**
			 * @member {String} initial Task Status ID of the workflow task
			 * @private
			 */
			this._initialTaskStatusID = initialTaskStatusID;

			/**
			 * @member {String} WebCenter workflow task ID
			 * @private
			 */
			this._taskID = taskID;

			/**
			 * @member {String} TaskType ID
			 * @private
			 */
			this._taskTypeID = taskTypeID;
		};

		WebCenterWorkflow.prototype = Object.create(Workflow.prototype);
		WebCenterWorkflow.prototype.constructor = WebCenterWorkflow;

		/**
		 * Config object with information of webcenter, loaction of the images, jsppages,...
		 * @returns {String}
		 */
		WebCenterWorkflow.prototype.getWebCenterConfig = function () {
			return this._wcrConfig;
		};

		/**
		 * Set config object with information of webcenter, loaction of the images, jsppages,...
		 * @returns {String}
		 */
		WebCenterWorkflow.prototype.setWebCenterConfig = function (wcrConfig) {
			this._wcrConfig = wcrConfig;
		};

		/**
		 * Set the workflow state
		 * @param {WebCenterWorkflowState} workflowState - New state of the workflow
		 */
		WebCenterWorkflow.prototype.setWorkflowState = function (workflowState) {
			if (this._workflowState !== workflowState) {
				this._workflowState = workflowState;
				this._emit("workflowStateChanged");
			}
		};

		/**
		 * toggles the debug mode on the wf
		 */
		WebCenterWorkflow.prototype.toggleDebugMode = function () {
			this._debugMode = !this._debugMode;
			this._emit("changed.debugMode");
		};

		/**
		 * @return {Boolean} Whether or not the workflow is in debug mode
		 */
		WebCenterWorkflow.prototype.isDebugMode = function () {
			return this._debugMode;
		};

		/**
		 * Get the workflow state
		 * @returns {WebCenterWorkflowState} The current workflow state or null if not defined (yet)
		 */
		WebCenterWorkflow.prototype.getWorkflowState = function () {
			return this._workflowState;
		};

		/**
		 * Get the WebCenter workflow task ID
		 * @returns {String} The WebCenter workflow task ID
		 */
		WebCenterWorkflow.prototype.getTaskID = function () {
			return this._taskID;
		};

		/**
		 * Get the TaskType ID of te workflow
		 * @returns {String} the WebCenter Task Type ID
		 */
		WebCenterWorkflow.prototype.getTaskTypeID = function () {
			return this._taskTypeID;
		};


		/**
		 * Set the data for the current task status
		 * @param {object} taskStatus
		 * @example
		 * 	{
		 * 		icon: "url to the icon",
		 *   	status: "name of the status"
		 *	}
		 */
		WebCenterWorkflow.prototype.setTaskStatus = function (taskStatus) {
			this._taskStatus = taskStatus;
		};

		/**
		 * Get the current task status data
		 * @returns {Object}
		 * @example
		 * 	{
		 * 		icon: "url to the icon",
		 *   	status: "name of the status"
		 *	}
		 */
		WebCenterWorkflow.prototype.getTaskStatus = function () {
			return this._taskStatus;
		};

		/**
		 * Get the initial Task Status ID
		 * @returns {String}
		 */
		WebCenterWorkflow.prototype.getInitialTaskStatusID = function () {
			return this._initialTaskStatusID;
		};

		/**
		 * Generate XML for the Workflow
		 * @returns {String} XMLString
		 */
		WebCenterWorkflow.prototype.toXML = function () {
			return WebCenterWorkflowXML(this);
		};

		/**
		 * Returns the saveable nodes, ie. All webcenter nodes
		 * @returns {Array.<Node>}
		 */
		WebCenterWorkflow.prototype.getSaveableNodes = function () {
			var nodes = _.filter(this.getNodes(), function (node) {
				return node instanceof WebCenterNode;
			});
			return nodes;
		};

		/**
		 * Returns all sticky notes
		 * @return {Array.<StickyNoteHelper>}
		 */
		WebCenterWorkflow.prototype.getStickyNotes = function () {
			return _.filter(this.getHelpers(), function (helper) {
				return helper instanceof StickyNoteHelper;
			});
		};


		/**
		 * @override
		 * Adds the nodes with errors to the active nodes
		 * @returns {Array.<Node>}
		 */
		WebCenterWorkflow.prototype.getActiveNodes = function () {
			if (this.getWorkflowState() === WebCenterWorkflowState.FINISHED) {
				return [];
			}
			var activeNodes = Workflow.prototype.getActiveNodes.call(this);
			var errorNodes = _.uniq(
				_.map(
					_.filter(this._tokens, function (token) { // Map all processed tokens
						return token.getPin() && (token.hasErrors() && token.getPin().getNode().isVisible());
					}),
					function (token) { // To their nodes
						return token.getPin().getNode();
					}
				)
			);
			return _.union(activeNodes, errorNodes);
		};

		return WebCenterWorkflow;
	});

define('WFEditorWCR/workflow/node/MultiAttributeWebCenterNode',[
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/panel/WebCenterPanelConstants",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"WFEditorWCR/workflow/node/WebCenterNodeConfigurationPanel",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"utils/webi18n!WFEditorWCR",
		"underscore"
	],
	function (
		WebCenterNode,
		WebCenterPanelConstants,
		WebCenterParameter,
		WebCenterNodeConfigurationPanel,
		WebCenterNodeConstants,
		i18n,
		_
	) {
		"use strict";

		/**
		 * A type of node which handles parameters in a special way, to allow multiple attributes/specifications
		 *
		 * @exports MultiAttributeWebCenterNode
		 * @class
		 * @augments WebCenterNode
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {String} nodeType - specifies the type of the node
		 * @param {String} [icon] - gives the url to the icon for this node
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var MultiAttributeWebCenterNode = function (workflow, id, nodeType, icon, label, webCenterAPI) {
			WebCenterNode.call(this, workflow, id, nodeType, icon, label, webCenterAPI);
		};

		MultiAttributeWebCenterNode.prototype = Object.create(WebCenterNode.prototype);
		MultiAttributeWebCenterNode.prototype.constructor = MultiAttributeWebCenterNode;

		/**
		 * Return a WebCenterNodeConfiguration panel, containing only a MultiAttribute panel
		 * @override
		 */
		MultiAttributeWebCenterNode.prototype.getConfigurationPanel = function () {
			var panelPromise = $.Deferred();
			var nodeConfigPanel = new WebCenterNodeConfigurationPanel(this);
			requirejs(["WFEditorWCR/workflow/panel/MultiAttributePanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService", "WFEditorWCR/workflow/panel/DateSmartNameDictionaryService"], _.bind(function (MultiAttributePanel, WorkflowSmartNameDictionaryService, DateSmartNameDictionaryService) {
				var nameLabel, valueLabel, restriction;
				switch (this._nodeType) {
				case WebCenterNodeConstants.Nodes.SETTASKSPECIFICATION:
					nameLabel = WebCenterNodeConstants.SetTaskSpecification.SPECNAMELABEL;
					valueLabel = WebCenterNodeConstants.SetTaskSpecification.SPECVALUE;
					restriction = this._webCenterAPI.getTaskSpecifications(this._workflow.getTaskTypeID());
					break;
				case WebCenterNodeConstants.Nodes.SETPROJECTATTRIBUTE:
					nameLabel = WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTENAMELABEL;
					valueLabel = WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTEVALUE;
					break;
				case WebCenterNodeConstants.Nodes.SETDOCUMENTATTRIBUTE:
					nameLabel = WebCenterNodeConstants.SetDocumentAttributes.ATTRIBUTENAMELABEL;
					valueLabel = WebCenterNodeConstants.SetDocumentAttributes.ATTRIBUTEVALUE;
					break;
				}

				var panel = new MultiAttributePanel("", i18n.t(nameLabel), i18n.t(valueLabel), new WorkflowSmartNameDictionaryService(this._webCenterAPI, this._workflow.getTaskTypeID()), new DateSmartNameDictionaryService(this._webCenterAPI, this._workflow.getTaskTypeID()), this._webCenterAPI, false, restriction);
				panel.getContext().addClass('parameter-panel form-horizontal form-compact');

				nodeConfigPanel.add(panel);
				panelPromise.resolve(nodeConfigPanel);
			}, this));

			return panelPromise.promise();
		};

		/**
		 * sets the configuration of this node
		 * @param {List.<Parameter>} config
		 */
		MultiAttributeWebCenterNode.prototype.setConfiguration = function (config) {
			// when no values are defined yet, the empty default attributes are added
			// instead of that, we need the special value map parameters (name = paramvalue 1, value = paramvalue 2)
			// we'll ignore the default parameters
			var realConfig = config;
			var defaultNameFound = false;
			var defaultValueFound = false;
			if (config.length === 2) {
				switch (this._nodeType) {
				case WebCenterNodeConstants.Nodes.SETTASKSPECIFICATION:
					_.each(config, function (param) {
						if (param.getName() === WebCenterNodeConstants.SetTaskSpecification.SPECNAME) {
							defaultNameFound = true;
						} else if (param.getName() === WebCenterNodeConstants.SetTaskSpecification.SPECVALUE) {
							defaultValueFound = true;
						}
					});
					break;
				case WebCenterNodeConstants.Nodes.SETPROJECTATTRIBUTE:
				case WebCenterNodeConstants.Nodes.SETDOCUMENTATTRIBUTE: // this is not a mistake; documents aso have project.attX
					_.each(config, function (param) {
						if (param.getName() === WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTENAME) {
							defaultNameFound = true;
						} else if (param.getName() === WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTEVALUE) {
							defaultValueFound = true;
						}
					});
					break;
				}
			}

			if (defaultNameFound && defaultValueFound) {
				realConfig = [];
			}

			// call super
			WebCenterNode.prototype.setConfiguration.call(this, realConfig);
		};

		/**
		 * Parse the node XML information
		 * @param {Element} nodeEl- XML element containing definition of the WorkflowNode
		 */
		MultiAttributeWebCenterNode.prototype.setNodeXML = function (nodeEl) {
			var params = nodeEl.querySelectorAll("action > parameters > parameter > Name");

			var oldName, oldValue;
			_.each(params, function (nameNode) {
				var name = nameNode.textContent;

				// Migrate from single attribute to multi project attribute
				var param;
				// Old behavior
				if (name === WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTENAME || name === WebCenterNodeConstants.SetTaskSpecification.SPECNAME) {
					param = new WebCenterParameter(WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTENAME);
					param.updateFromXML(nameNode.parentNode);
					oldName = param.getValue();
				} else if (name === WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTEVALUE || name === WebCenterNodeConstants.SetTaskSpecification.SPECVALUE) {
					param = new WebCenterParameter(WebCenterNodeConstants.SetProjectAtrributes.ATTRIBUTEVALUE);
					param.updateFromXML(nameNode.parentNode);
					oldValue = param.getValue();
				} else {
					// Correct current behavior
					param = new WebCenterParameter(name);
					param.updateFromXML(nameNode.parentNode);
					this._configuration.push(param); // we still have to push the parameters because setConfiguration makes config empty because backend still returns "old layout"
				}
			}, this);

			if (oldName) {
				var migratedParam = new WebCenterParameter(oldName, oldValue);
				this._configuration.push(migratedParam);
			}
		};

		/**
		 * @Override 
		 */
		MultiAttributeWebCenterNode.prototype.isConfigurable = function () {
			return this.isEnabled();
		};

		return MultiAttributeWebCenterNode;
	});

define('WFEditorWCR/workflow/node/operation/OperationNode',[
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants"
	],
	function (
		WebCenterNode,
		WebCenterNodeConstants
	) {
		"use strict";

		/**
		 * OperationNode represents the model for a node that consists of an operation in the WebCenter Workflow Editor.
		 *
		 * @exports OperationNode
		 * @class
		 * @augments WebCenterNode
		 *
		 * @param {Workflow} workflow    - the workflow to which the node belongs
		 * @param {String}   id          - node id
		 * @param {String}   type        - the type of the node   
		 * @param {String}   pathToIcons - gives the url to the map where the icons can be found
		 * @param {String}   [label]     - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var OperationNode = function (workflow, id, type, pathToIcons, label, webCenterAPI) {
			WebCenterNode.call(this, workflow, id, type, pathToIcons, label, webCenterAPI);

			/**
			 * All operation tokens in this node
			 */
			this._operationTokens = {};

		};

		OperationNode.prototype = Object.create(WebCenterNode.prototype);
		OperationNode.prototype.constructor = OperationNode;

		/**
		 * Retrieve the operation tokens
		 * @returns {Map.<Sring,OperationToken>}
		 */
		OperationNode.prototype.getOperationTokens = function () {
			return this._operationTokens;
		};

		/**
		 * add an operation token to this node
		 * @param {OperationToken} opToken
		 */
		OperationNode.prototype.addOperationToken = function (opToken) {
			this._operationTokens[opToken.getId()] = opToken;
		};

		/**
		 * Removes an operation token
		 * @param {OperationToken} opToken
		 */
		OperationNode.prototype.removeOperationToken = function (opToken) {
			delete this._operationTokens[opToken.getId()];

		};


		return OperationNode;
	});


define('tpl!WFEditorWCR/workflow/node/dataCollector/DataCollectorNode', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<action class="wfconstruct-datacollector">\r\n	<warningpolicy>dedicatedhandling</warningpolicy>';
/* Doesn't seem to be used, but keep it here for compatibility */
__p+='\r\n	<errorpolicy>'+
((__t=(d.getErrorPolicy()))==null?'':_.escape(__t))+
'</errorpolicy>\r\n</action>';
return __p;
}});
define('WFEditorWCR/workflow/node/dataCollector/DataCollectorNode',[
		"WFEditorWCR/workflow/node/WebCenterNodeConfigurationPanel",
		"WFEditorWCR/workflow/node/operation/OperationNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"WFEditorWCR/workflow/panel/PinParameter",
		"tpl!WFEditorWCR/workflow/node/dataCollector/DataCollectorNode",
		"utils/XMLUtils",
		"utils/webi18n!WFEditorWCR",
		"WFEditorWCR/workflow/pin/OutputPinError",
		"WFEditorWCR/workflow/pin/InputPin"

	],
	function (
		WebCenterNodeConfigurationPanel,
		OperationNode,
		WebCenterNodeConstants,
		OutputPinOK,
		WebCenterPinConstants,
		WebCenterParameter,
		PinParameter,
		DataCollectorNodeXML,
		XMLUtils,
		i18n,
		OutputPinError,
		InputPin
	) {
		"use strict";

		/**
		 * DataCollectorNode represents the model for a  datacollector node in the WebCenter Workflow Editor.
		 *
		 * @exports DataCollectorNode
		 * @class
		 * @augments OperationNode
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {String} pathToIcons - gives the url to the map where the icons can be found
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var DataCollectorNode = function (workflow, id, pathToIcons, label, webCenterAPI) {
			this._icon = this._icon || WebCenterNodeConstants.Icons.DATACOLLECTOR;
			this._type = this._type || WebCenterNodeConstants.Nodes.DATACOLLECTOR;

			/**
			 * @member {String} error policy of the Data Collector Node
			 */
			this._errorPolicy = "";

			/**
			 * @member {RadioGroupPanel} - panel for error policy configuration
			 */
			this._radioGroupPanel = null;

			/**
			 * @member {Parameter} - parameters for the radio panel
			 */
			this._ErrorPolicyParam = null;
			this._FailParam = null;
			this._IgnoreParam = null;

			OperationNode.call(this, workflow, id, this._type, pathToIcons + this._icon, label, webCenterAPI);
		};

		DataCollectorNode.prototype = Object.create(OperationNode.prototype);
		DataCollectorNode.prototype.constructor = DataCollectorNode;

		/**
		 * @Override
		 */
		DataCollectorNode.prototype.setNodeXML = function (nodeXML) {
			// get error policy field
			this.readErrorPolicyFromXML(nodeXML);
			this._ErrorPolicyParam.setValue(this._errorPolicy);
		};

		/**
		 * @Override
		 */
		DataCollectorNode.prototype.setConfiguration = function (config) {
			if (config.length === 0) {
				// create fake parameters for error policy to be shown in a radio panel
				this._ErrorPolicyParam = new WebCenterParameter(WebCenterNodeConstants.DataCollector.ERRORPOLICY, WebCenterNodeConstants.DataCollector.FAILONERRORS);
				this._FailParam = new PinParameter(WebCenterNodeConstants.DataCollector.FAILONERRORS, "", WebCenterNodeConstants.DataCollector.FAILONERRORS, [], _.bind(function () {
					return [new OutputPinError(this._workflow, this, WebCenterPinConstants.Type.WorkflowConstructError)];
				}, this));

				this._IgnoreParam = new WebCenterParameter(WebCenterNodeConstants.DataCollector.IGNOREERRORS, "", WebCenterNodeConstants.DataCollector.IGNOREERRORS);
				config = [this._ErrorPolicyParam, this._FailParam, this._IgnoreParam];
			} else {
				_.each(config, _.bind(function (param) {
					if (param.getName() === WebCenterNodeConstants.DataCollector.ERRORPOLICY) {
						this._ErrorPolicyParam = param;
					} else if (param.getName() === WebCenterNodeConstants.DataCollector.FAILONERRORS) {
						this._FailParam = param;
					} else {
						this._IgnoreParam = param;
					}
				}, this));
			}
			this._updatePins();

			OperationNode.prototype.setConfiguration.call(this, config);
		};

		/**
		 * Get the error policy of the Join Node
		 * @returns {String}
		 * @override
		 */
		DataCollectorNode.prototype.getErrorPolicy = function () {
			this.updateErrorPolicy();
			return this._errorPolicy;
		};

		/**
		 * read the error policy from the given XML
		 * @param nodeXML
		 */
		DataCollectorNode.prototype.readErrorPolicyFromXML = function (nodeXML) {
			var nodeData = XMLUtils.xml2obj(nodeXML);
			this._errorPolicy = nodeData.action.errorpolicy;
			this._ErrorPolicyParam.setValue(this._errorPolicy);
			this._updatePins();
		};


		/**
		 * Update error policy value, based on the parameters that are passed to the radio panel
		 * @param nodeXML
		 */
		DataCollectorNode.prototype.updateErrorPolicy = function () {
			if (this._radioGroupPanel) {
				var parameters = this._radioGroupPanel.getParameters();
				this._errorPolicy = parameters[0].getValue();
			}
		};

		/**
		 * @Override
		 */
		DataCollectorNode.prototype._initializePins = function () {
			this._inputPin = new InputPin(this._workflow, this);
			this.addPin(this._inputPin, 0);
			this.addPin(new OutputPinOK(this._workflow, this, WebCenterPinConstants.Type.WorkflowConstructOK), 1);
		};

		/**
		 * @Override
		 */
		DataCollectorNode.prototype.getActionXML = function () {
			return DataCollectorNodeXML(this);
		};

		/**
		 * @override
		 */
		DataCollectorNode.prototype.getConfigurationPanel = function () {
			var panelPromise = $.Deferred();
			var nodeConfigPanel = new WebCenterNodeConfigurationPanel(this);
			nodeConfigPanel.getContext().addClass("parameter-panel form-horizontal form-compact");

			requirejs(["WFEditorWCR/workflow/panel/RadioPanel", "WFEditorWCR/workflow/panel/RadioGroupPanel"], _.bind(function (RadioPanel, RadioGroupPanel) {
				this._radioGroupPanel = new RadioGroupPanel(this._ErrorPolicyParam.getName(), undefined, WebCenterNodeConstants.DataCollector.ERRORPOLICY);
				nodeConfigPanel.add(this._radioGroupPanel);

				var ignorePanel = new RadioPanel(this._IgnoreParam.getName(), i18n.t(WebCenterNodeConstants.DataCollector.IGNOREERRORS));
				var failPanel = new RadioPanel(this._FailParam.getName(), i18n.t(WebCenterNodeConstants.DataCollector.FAILONERRORS));
				var self = this;

				// If the failPanel gets checked we wish to create a new error pin (there might be edges in the other one referring to nodes that have been deleted )
				failPanel.check = function (check, visibleWhenNotChecked) {
					RadioPanel.prototype.check.call(this, check, visibleWhenNotChecked);
					if (this._parameters[0]) {
						this._parameters[0].setNewPin();
					}
				};
				this._radioGroupPanel.add(failPanel);
				this._radioGroupPanel.add(ignorePanel);

				panelPromise.resolve(nodeConfigPanel);
			}, this));

			return panelPromise.promise();
		};

		/**
		 * updates the pins with respect to the error policy
		 */
		DataCollectorNode.prototype._updatePins = function () {
			if (this._ErrorPolicyParam.getValue() === WebCenterNodeConstants.DataCollector.FAILONERRORS) {
				if (!this._errorPin) {
					this._errorPin = this._FailParam.pins[0];
					this.addPin(this._errorPin);
					_.each(this._errorPin.getEdges(), _.bind(function (edge) {
						edge.getToPin().addEdge(edge);
						this._workflow.addEdge(edge);
					}, this));
				}
			} else {
				if (this._errorPin) {
					this.removePin(this._errorPin);
					_.each(this._errorPin.getEdges(), _.bind(function (edge) {
						edge.getToPin().removeEdge(edge);
						this._workflow.removeEdge(edge);
					}, this));
					this._errorPin = null;
				}
			}
		};

		/**
		 * @override
		 */
		DataCollectorNode.prototype.getHelpTopic = function () {
			return WebCenterNodeConstants.HelpTopics.DATACOLLECTOR;
		};
		return DataCollectorNode;
	});

define('WFEditorWCR/workflow/node/dataSplitter/DataSplitterNode',[
		"WFEditorWCR/workflow/node/operation/OperationNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants"
	],
	function (
		OperationNode,
		WebCenterNodeConstants,
		OutputPinOK,
		WebCenterPinConstants
	) {
		"use strict";

		/**
		 * DataSplitterNode represents the model for a  DataSplitterNode node in the WebCenter Workflow Editor
		 *
		 * @exports DataSplitterNode
		 * @class
		 * @augments OperationNode
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {String} pathToIcons - gives the url to the map where the icons can be found
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var DataSplitterNode = function (workflow, id, pathToIcons, label, webCenterAPI) {
			var icon = WebCenterNodeConstants.Icons.DATASPLITTER;
			OperationNode.call(this, workflow, id, WebCenterNodeConstants.Nodes.DATASPLITTER, pathToIcons + icon, label, webCenterAPI);
		};

		DataSplitterNode.prototype = Object.create(OperationNode.prototype);
		DataSplitterNode.prototype.constructor = DataSplitterNode;

		/**
		 * @Override
		 */
		DataSplitterNode.prototype._initializePins = function () {
			this._addInputPin();
			this.addPin(new OutputPinOK(this._workflow, this, WebCenterPinConstants.Type.WorkflowConstructOK));
		};

		/**
		 * @override
		 */
		DataSplitterNode.prototype.getHelpTopic = function () {
			return WebCenterNodeConstants.HelpTopics.DATASPLITTER;
		};

		return DataSplitterNode;
	});

define('WFEditorWCR/workflow/pin/ImplicitInputPin',[
		"WFEditorWCR/workflow/pin/InputPin"
	],
	function (
		InputPin
	) {
		"use strict";

		/**
		 * ImplicitInputPin represents an input pin that isn't visible. eg. for implicit edges from error pins to the error handler
		 *
		 * @exports ImplicitInputPin
		 * @class
		 * @augments InputPin
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {Node} node - node to which the pin belongs
		 */
		var ImplicitInputPin = function ImplicitInputPin(workflow, node) {
			InputPin.call(this, workflow, node);
			this._isVisible = false;
		};

		ImplicitInputPin.prototype = Object.create(InputPin.prototype);
		ImplicitInputPin.prototype.constructor = ImplicitInputPin;

		/**
		 * @override
		 */
		ImplicitInputPin.prototype.acceptsIncomingEdges = function (fromPin) {
			// an implicit input pin will never accept edges
			return false;
		};


		return ImplicitInputPin;
	});

define('WFEditorWCR/workflow/node/errorHandler/ErrorHandlerNode',[
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/ImplicitInputPin"
	],
	function (
		WebCenterNode,
		WebCenterNodeConstants,
		WebCenterPinConstants,
		OutputPinOK,
		ImplicitInputPin
	) {
		"use strict";

		/**
		 * ErrorHandlerNode represents the model for a  ErrorHandlerNode node in the WebCenter Workflow Editor.
		 *
		 * @exports ErrorHandlerNode
		 * @class
		 * @augments WebCenterNode
		 *
		 * @param {Wofkflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {String} pathToIcons - gives the url to the map where the icons can be found
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var ErrorHandlerNode = function (workflow, id, pathToIcons, label, webCenterAPI) {
			var icon = WebCenterNodeConstants.Icons.ERRORHANDLER;
			WebCenterNode.call(this, workflow, id, WebCenterNodeConstants.Nodes.ERRORHANDLER, pathToIcons + icon, label, webCenterAPI);

		};

		ErrorHandlerNode.prototype = Object.create(WebCenterNode.prototype);
		ErrorHandlerNode.prototype.constructor = ErrorHandlerNode;

		/**
		 * @Override
		 */
		ErrorHandlerNode.prototype._initializePins = function () {
			this._inputPin = new ImplicitInputPin(this._workflow, this);
			this.addPin(this._inputPin);
			this.addPin(new OutputPinOK(this._workflow, this, WebCenterPinConstants.Type.OutOK, ""));
		};

		/**
		 * @override
		 */
		ErrorHandlerNode.prototype.getHelpTopic = function () {
			return WebCenterNodeConstants.HelpTopics.ERRORHANDLER;
		};

		return ErrorHandlerNode;
	});


define('tpl!WFEditorWCR/workflow/node/fork/ForkNode', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<fork name="'+
((__t=(d.getId()))==null?'':_.escape(__t))+
'">\r\n	'+
((__t=(d.getTransitionsXML()))==null?'':__t)+
'\r\n</fork>';
return __p;
}});
define('WFEditorWCR/workflow/node/fork/ForkNode',[
		"WFEditorWCR/workflow/node/operation/OperationNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"tpl!WFEditorWCR/workflow/node/fork/ForkNode"
	],
	function (
		OperationNode,
		WebCenterNodeConstants,
		WebCenterPinConstants,
		OutputPinOK,
		ForkNodeXML
	) {
		"use strict";

		/**
		 * ForkNode represents the model for a  ForkNode node in the WebCenter Workflow Editor.
		 *
		 * @exports ForkNode
		 * @class
		 * @augments OperationNode
		 *
		 * @param {Wofkflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 *                    @param {String} pathToIcons - gives the url to the map where the icons can be found
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var ForkNode = function (workflow, id, pathToIcons, label, webCenterAPI) {
			var icon = WebCenterNodeConstants.Icons.FORK;
			OperationNode.call(this, workflow, id, WebCenterNodeConstants.Nodes.FORK, pathToIcons + icon, label, webCenterAPI);


		};

		ForkNode.prototype = Object.create(OperationNode.prototype);
		ForkNode.prototype.constructor = ForkNode;

		/**
		 * @Override
		 */
		ForkNode.prototype._initializePins = function () {
			this._addInputPin();
			this.addPin(new OutputPinOK(this._workflow, this, WebCenterPinConstants.Type.WorkflowConstructOK));
		};


		/**
		 * @Override
		 */
		ForkNode.prototype.toXML = function () {
			return ForkNodeXML(this);
		};

		/**
		 * @override
		 */
		ForkNode.prototype.getHelpTopic = function () {
			return WebCenterNodeConstants.HelpTopics.FORK;
		};

		return ForkNode;
	});


define('tpl!WFEditorWCR/workflow/node/join/JoinNode', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<join name="'+
((__t=(d.getId()))==null?'':_.escape(__t))+
'">\r\n	<errorpolicy>'+
((__t=(d.getErrorPolicy()))==null?'':_.escape(__t))+
'</errorpolicy>\r\n	'+
((__t=(d.getTransitionsXML()))==null?'':__t)+
'\r\n</join>';
return __p;
}});
define('WFEditorWCR/workflow/node/join/JoinNode',[
		"WFEditorWCR/workflow/node/dataCollector/DataCollectorNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"tpl!WFEditorWCR/workflow/node/join/JoinNode",
		"utils/XMLUtils"
	],
	function (
		DataCollectorNode,
		WebCenterNodeConstants,
		OutputPinOK,
		WebCenterPinConstants,
		JoinNodeXML,
		XMLUtils

	) {
		"use strict";

		/**
		 * JoinNode represents the model for a  JoinNode node in the WebCenter Workflow Editor.
		 *
		 * @exports JoinNode
		 * @class
		 * @augments DataCollectorNode
		 *
		 * @param {Wofkflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {String} pathToIcons - gives the url to the map where the icons can be found
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var JoinNode = function (workflow, id, pathToIcons, label, webCenterAPI) {
			this._icon = WebCenterNodeConstants.Icons.JOIN;
			this._type = WebCenterNodeConstants.Nodes.JOIN;
			DataCollectorNode.call(this, workflow, id, pathToIcons, label, webCenterAPI);
		};

		JoinNode.prototype = Object.create(DataCollectorNode.prototype);
		JoinNode.prototype.constructor = JoinNode;

		/**
		 * @override
		 */
		JoinNode.prototype.readErrorPolicyFromXML = function (nodeXML) {
			var nodeData = XMLUtils.xml2obj(nodeXML);
			this._errorPolicy = nodeData.errorpolicy;
			this._ErrorPolicyParam.setValue(this._errorPolicy);
			this._updatePins();
		};

		/**
		 * @Override
		 */
		JoinNode.prototype.toXML = function () {
			return JoinNodeXML(this);
		};

		/**
		 * @override
		 */
		JoinNode.prototype.getHelpTopic = function () {
			return WebCenterNodeConstants.HelpTopics.JOIN;
		};

		return JoinNode;
	});

define('WFEditorWCR/workflow/node/TokenMarkerSelectorNode',[
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConfigurationPanel",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"utils/Observer",
		"utils/Map",
		"utils/webi18n!WFEditorWCR"
	],
	function (
		WebCenterNode,
		OutputPinOK,
		WebCenterPinConstants,
		WebCenterNodeConfigurationPanel,
		WebCenterParameter,
		Observer,
		Map,
		i18n
	) {
		"use strict";

		/**
		 * TokenMarkerSelectorNode is a common super class for the TokenMarkerNode and TokenSelectorNode.
		 *
		 * @exports TokenMarkerSelectorNode
		 * @class
		 * @augments WebCenterNode
		 *
		 * @param {Wofkflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {WebCenterNodeConstants.Nodes} nodeType - The type of the node e.g. WebCenterNodeConstants.Nodes.TOKENSELECTOR
		 * @param {String} pathToIcons - gives the url to the map where the icons can be found e.g. WebCenterNodeConstants.Icons.TOKENSELECTOR
		 * @param {WebCenterNodeConstants.Icons} - icon - The icon of this node 
		 * @param {String} label - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data   
		 */
		var TokenMarkerSelectorNode = function (workflow, id, nodeType, pathToIcons, icon, label, webCenterAPI) {
			Observer.mixin(this);

			/*
			 * @member {WebCenterParameter} Mark
			 */
			this._markParameter = new WebCenterParameter("wfconstruct-tokenmarker.tag"); // needs to be constructed first

			/*
			 * @member {TagSelectPanel}
			 */
			this._tagSelectPanel = null;

			WebCenterNode.call(this, workflow, id, nodeType, pathToIcons + icon, label, webCenterAPI);


			// Now workflow is defined
			var allMarkers = this._getAllMarkers(this._getMarkersSourceNodeType());
			var allOwnMarkers = this._getAllMarkers(this.getNodeType());
			allMarkers = _.difference(allMarkers, allOwnMarkers);
			if (_.size(allMarkers) > 0) {
				this._markParameter.setValue(allMarkers[0]);
			} else {
				this._markParameter.setValue(this._getNextMark());
			}

		};

		TokenMarkerSelectorNode.prototype = Object.create(WebCenterNode.prototype);
		TokenMarkerSelectorNode.prototype.constructor = TokenMarkerSelectorNode;

		/**
		 * @override
		 */
		TokenMarkerSelectorNode.prototype.setConfiguration = function (config) {
			if (!config || _.size(config) === 0) {
				// configuration is defined client side
				config = [];
				config.push(this._markParameter);
			}

			this._markParameter = config[0];
			if (this._tagSelectPanel) {
				// update the options
				this._tagSelectPanel.setOptions(this._getAllMarkers(this._getMarkersSourceNodeType(), true));
			}

			WebCenterNode.prototype.setConfiguration.call(this, config);
			// if the configuration changed, a redraw is already triggered so the visible parameters will be updated
		};

		/**
		 * @Override
		 */
		TokenMarkerSelectorNode.prototype._initializePins = function () {
			this._addInputPin();
			this.addPin(new OutputPinOK(this._workflow, this, WebCenterPinConstants.Type.WorkflowConstructOK));
		};

		/**
		 * @override
		 */
		TokenMarkerSelectorNode.prototype.setNodeXML = function (nodeEl) {
			var mark = nodeEl.querySelector("action mark").textContent;
			this._markParameter.setValue(mark);
		};

		/**
		 * Get the mark value
		 * @returns {String}
		 */
		TokenMarkerSelectorNode.prototype.getMark = function () {
			return this._markParameter.getValue();
		};

		/**
		 * @override
		 */
		TokenMarkerSelectorNode.prototype.getConfigurationPanel = function () {
			var nodeConfigPanel = new WebCenterNodeConfigurationPanel(this);
			this._observe(nodeConfigPanel, "save", function () {
				this._emit("changed.visibleParameters");
			});

			var panelPromise = $.Deferred();

			requirejs(["WFEditorWCR/workflow/panel/TagSelectPanel"], _.bind(function (TagSelectPanel) {
				this._tagSelectPanel = new TagSelectPanel(this._markParameter.getName(), i18n.t(this.getNodeType() + ".markLabel"));

				// set the tag list
				this._tagSelectPanel.setOptions(this._getAllMarkers(this._getMarkersSourceNodeType(), true));

				nodeConfigPanel.add(this._tagSelectPanel);
				panelPromise.resolve(nodeConfigPanel);
			}, this));

			return panelPromise.promise();
		};


		/**
		 * @override
		 */
		TokenMarkerSelectorNode.prototype.getVisibleParameters = function () {
			var params = [];
			params.push(this._markParameter);
			return params;
		};

		/**
		 * Get the markers source node type.
		 * This will be used to find markers of nodes of this type for the tag dropdown selector.
		 * Currently this is the opposite type of this.getNodeType();
		 * @returns {WebCenterNodeConstants.Nodes} Currently either WebCenterNodeConstants.Nodes.TOKENSELECTOR or WebCenterNodeConstants.Nodes.TOKENMARKER
		 */
		TokenMarkerSelectorNode.prototype._getMarkersSourceNodeType = function () {
			// implement yourself
		};


		/**
		 * Get all markers of the nodes of the given type
		 * @param {WebCenterNodeConstants.Nodes} nodeType - This is either WebCenterNodeConstants.Nodes.TOKENSELECTOR or WebCenterNodeConstants.Nodes.TOKENMARKER
		 * @param {Boolean} includeSelf - True to include this node mark                                                
		 * @returns {Array.<String>} All tags of the nodes of the given type, sorted alphabetically. Empty array if none found.
		 */
		TokenMarkerSelectorNode.prototype._getAllMarkers = function (nodeType, includeSelf) {
			var allNodes = this.getWorkflow().getNodes();
			// filter + map
			var marks = _.reduce(allNodes, function (memo, node, key) {
				if (node.getNodeType() === nodeType) {
					memo.push(node.getMark());
				}
				return memo;
			}, []);
			if (includeSelf) {
				marks.push(this.getMark());
			}
			var sortedMarks = _.sortBy(marks, function (mark) {
				return mark.toLowerCase();
			});
			return _.uniq(sortedMarks, true);
		};

		/**
		 * Get the next mark for this type node
		 * @returns {String}
		 */
		TokenMarkerSelectorNode.prototype._getNextMark = function () {
			var allMarkersOfMyType = this._getAllMarkers(this.getNodeType());

			var i = 1;
			var next;
			do {
				next = i18n.t("wfconstruct-tokenmarker.tag") + " - " + i;
				i++;
			} while (_.contains(allMarkersOfMyType, next));
			return next;
		};

		return TokenMarkerSelectorNode;
	});


define('tpl!WFEditorWCR/workflow/node/tokenMarker/TokenMarkerNode', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<action class="wfconstruct-tokenmarker">\r\n<mark>'+
((__t=(d.getMark()))==null?'':_.escape(__t))+
'</mark>\r\n</action>';
return __p;
}});
define('WFEditorWCR/workflow/node/tokenMarker/TokenMarkerNode',[
		"WFEditorWCR/workflow/node/TokenMarkerSelectorNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"tpl!WFEditorWCR/workflow/node/tokenMarker/TokenMarkerNode"
	],
	function (
		TokenMarkerSelectorNode,
		WebCenterNodeConstants,
		OutputPinOK,
		WebCenterPinConstants,
		TokenMarkerNodeXML
	) {
		"use strict";

		/**
		 * TokenMarkerNode represents the model for a  TokenMarkerNode node in the WebCenter Workflow Editor.
		 *
		 * @exports TokenMarkerNode
		 * @class
		 * @augments TokenMarkerSelectorNode
		 *
		 * @param {Wofkflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {String} pathToIcons - gives the url to the map where the icons can be found
		 * @param {String} label - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var TokenMarkerNode = function (workflow, id, pathToIcons, label, webCenterAPI) {
			var icon = WebCenterNodeConstants.Icons.TOKENMARKER;
			var nodeType = WebCenterNodeConstants.Nodes.TOKENMARKER;
			TokenMarkerSelectorNode.call(this, workflow, id, nodeType, pathToIcons, icon, label, webCenterAPI);
		};

		TokenMarkerNode.prototype = Object.create(TokenMarkerSelectorNode.prototype);
		TokenMarkerNode.prototype.constructor = TokenMarkerNode;

		/**
		 * @Override
		 */
		TokenMarkerNode.prototype.getActionXML = function () {
			return TokenMarkerNodeXML(this);
		};

		/**
		 * @override
		 */
		TokenMarkerNode.prototype._getMarkersSourceNodeType = function () {
			return WebCenterNodeConstants.Nodes.TOKENSELECTOR;
		};
	
		/**
		 * @override
		 */
		TokenMarkerNode.prototype.getHelpTopic = function () {
			return WebCenterNodeConstants.HelpTopics.MARK;
		};

		return TokenMarkerNode;
	});


define('tpl!WFEditorWCR/workflow/node/tokenSelector/TokenSelectorNode', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<action class="wfconstruct-markedtokenselector">\r\n<mark>'+
((__t=(d.getMark()))==null?'':_.escape(__t))+
'</mark>\r\n</action>';
return __p;
}});
define('WFEditorWCR/workflow/node/tokenSelector/TokenSelectorNode',[
		"WFEditorWCR/workflow/node/TokenMarkerSelectorNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"tpl!WFEditorWCR/workflow/node/tokenSelector/TokenSelectorNode"
	],
	function (
		TokenMarkerSelectorNode,
		WebCenterNodeConstants,
		OutputPinOK,
		WebCenterPinConstants,
		TokenSelectorNodeXML
	) {
		"use strict";

		/**
		 * TokenSelectorNode represents the model for a  TokenSelectorNode node in the WebCenter Workflow Editor.
		 *
		 * @exports TokenSelectorNode
		 * @class
		 * @augments TokenMarkerSelectorNode
		 *
		 * @param {Wofkflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {String} pathToIcons - gives the url to the map where the icons can be found
		 * @param {String} label - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data   
		 */
		var TokenSelectorNode = function (workflow, id, pathToIcons, label, webCenterAPI) {
			var icon = WebCenterNodeConstants.Icons.TOKENSELECTOR;
			var nodeType = WebCenterNodeConstants.Nodes.TOKENSELECTOR;
			TokenMarkerSelectorNode.call(this, workflow, id, nodeType, pathToIcons, icon, label, webCenterAPI);
		};

		TokenSelectorNode.prototype = Object.create(TokenMarkerSelectorNode.prototype);
		TokenSelectorNode.prototype.constructor = TokenSelectorNode;

		/**
		 * @Override
		 */
		TokenSelectorNode.prototype.getActionXML = function () {
			return TokenSelectorNodeXML(this);
		};

		/**
		 * @override
		 */
		TokenSelectorNode.prototype._getMarkersSourceNodeType = function () {
			return WebCenterNodeConstants.Nodes.TOKENMARKER;
		};
	
		/**
		 * @override
		 */
		TokenSelectorNode.prototype.getHelpTopic = function () {
			return WebCenterNodeConstants.HelpTopics.SELECT;
		};

		return TokenSelectorNode;
	});

define('WFEditorWCR/workflow/panel/TaskSpecificationsParameter',["underscore", "jquery", "WFEditorWCR/workflow/panel/WebCenterParameter", "tpl!WFEditor/workflow/panel/Parameter"], function (_, $, WebCenterParameter, parameter) {

	"use strict";

	/**
	 * @class
	 * @augments WebCenterParameter
	 * @exports TaskSpecificationsParameter
	 * 
	 * a wrapper parameter for the task specifications
	 * 
	 * @param {String}                     name           - id of the parameter
	 * @param {Array.<WebCenterAttribute>} specifications - specifications
	 * @param {Array.<Parameter>}          subparameters  - subParameters, there is one subparameter for each specification that has a value
	 */
	var TaskSpecificationsParameter = function (name, specifications, subparameters) {
		WebCenterParameter.call(this, name, "", "", subparameters || []);

		/**
		 * @param {Array.<WebCenterAttribute>}
		 * @private
		 */
		this._specifications = specifications;

		/**
		 * @param {Boolean} should the specifications that are not selected be added to the toString method?
		 * @private
		 */
		this._shouldAddNotSelectedSpecifications = false;

	};

	TaskSpecificationsParameter.prototype = Object.create(WebCenterParameter.prototype);
	TaskSpecificationsParameter.prototype.constructor = TaskSpecificationsParameter;

	/**
	 * is @param {id} a specification?
	 * @param   {String}  id 
	 * @returns {Boolean} 
	 */
	TaskSpecificationsParameter.prototype.isSpecification = function (id) {
		return this._specifications.has(id);
	};

	/**
	 * @override
	 */
	TaskSpecificationsParameter.prototype.updateFromXML = function (xml) {
		var name, value;
		_.each(xml.childNodes, function (child) {
			if (child.tagName === "Name") {
				name = child.textContent.trim();
			} else if (child.tagName === "Value") {
				value = child.textContent;
				if(!value) {
					// If the value is empty we know that the not selected specifications are also added.
					// In a perfect world there could be a property for this, but to be compatible with the XMLs created by the late java applet this is the quicker solution
					this._shouldAddNotSelectedSpecifications = true;
				}
			}
		}, this);
		if (name && value) {
			this._subparameters.push(new WebCenterParameter(name, value, value));
		}
	};

	/**
	 * @override
	 */
	TaskSpecificationsParameter.prototype.toString = function () {
		var xml = "";
		_.each(this._specifications.values(), function (spec) {
			var subParam = _.find(this._subparameters, function (param) {
				return param.getName() === spec.getId();
			}, this);
			if (subParam) {
				xml += parameter(subParam);
			} else if(this._shouldAddNotSelectedSpecifications) {
				xml += parameter(new WebCenterParameter(spec.getId()));
			}
		}, this);
		return xml;
	};

	/**
	 * @override
	 */
	TaskSpecificationsParameter.prototype.copy = function () {
		var copiedSubparams = [];
		_.each(this._subparameters, function (subParam) {
			copiedSubparams.push(subParam.copy());
		});
		var copyParam =  new TaskSpecificationsParameter(this._name, this._specifications, copiedSubparams);
		copyParam.shouldAddNotSelectedSpecifications(this._shouldAddNotSelectedSpecifications);
		return copyParam;
	};

	/**
	 * gets or sets whether or not the not selected specifications should be added to the xml
	 * @param {Boolean}
	 * @return {Boolean | undefined}
	 */
	TaskSpecificationsParameter.prototype.shouldAddNotSelectedSpecifications = function (bool) {
		if (typeof bool !== "undefined") {
			this._shouldAddNotSelectedSpecifications = bool;
		} else {
			return this._shouldAddNotSelectedSpecifications;
		}
	};
	
	/**
	 * @override
	 * @param   {Parameter} param 
	 * @returns {Boolean}   
	 */
	TaskSpecificationsParameter.prototype.equals = function (param) {
		if(!(param instanceof TaskSpecificationsParameter)) {
			return false;
		}
		return WebCenterParameter.prototype.equals.call(this, param) && (this._shouldAddNotSelectedSpecifications === param.shouldAddNotSelectedSpecifications());
	};

	return TaskSpecificationsParameter;
});


define('tpl!WFEditorWCR/workflow/node/task/TaskNode', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<action class="wctask">\r\n	<wctask_type id="'+
((__t=(d.getTaskType().getId()))==null?'':_.escape(__t))+
'">\r\n		<task_type_name>\r\n			'+
((__t=(d.getTaskType().getName()))==null?'':_.escape(__t))+
'\r\n		</task_type_name>\r\n	</wctask_type>\r\n	<parameters>\r\n		';
 _.each(d.getConfiguration(), function(parameter) { if (d.shouldSaveParameter(parameter)) { print(parameter.toString()) } }) 
__p+='\r\n	</parameters>\r\n	<resourceref>'+
((__t=( d.getId() ))==null?'':_.escape(__t))+
'</resourceref>\r\n</action>';
return __p;
}});

define('tpl!WFEditorWCR/workflow/node/task/TaskNodeResource', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='';
 if (d.getOutputPinCount() > 1) { 
__p+='	<resource id="';
 print(d.getId()) 
__p+='">\r\n		<outputs>\r\n			';
 _.each(d.getOutputOkPins(), function(pin) { 
__p+='<output>\r\n					<name>'+
((__t=(pin.getLabel()))==null?'':_.escape(__t))+
'</name>\r\n					<id>'+
((__t=(pin.getOutputId()))==null?'':_.escape(__t))+
'</id>\r\n				</output>\r\n				';
 }); 
__p+='\r\n		</outputs>\r\n	</resource>';
 } 
__p+='';
return __p;
}});
 define('WFEditorWCR/workflow/node/task/TaskNode',[
 		"WFEditorWCR/workflow/node/WebCenterNode",
 		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
 		"WFEditorWCR/workflow/pin/OutputPinOK",
 		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
 		"WFEditorWCR/workflow/panel/WebCenterPanelConstants",
 		"WFEditorWCR/workflow/panel/TaskSpecificationsParameter",
 		"tpl!WFEditorWCR/workflow/node/task/TaskNode",
 		"tpl!WFEditorWCR/workflow/node/task/TaskNodeResource",
 		"utils/webi18n!WFEditorWCR",
 		"WFEditorWCR/workflow/panel/WorkflowSmartNameConstants",
 		"underscore"
 	],
 	function (
 		WebCenterNode,
 		WebCenterNodeConstants,
 		OutputPinOK,
 		WebCenterPinConstants,
 		WebCenterPanelConstants,
 		TaskSpecificationsParameter,
 		TaskNodeXML,
 		TaskNodeResourceXML,
 		i18n,
 		WorkflowSmartNameConstants,
 		_
 	) {
 		"use strict";

 		/**
 		 * Tasknode represents the model for a task node in the WebCenter Workflow Editor.
 		 *
 		 * @exports TaskNode
 		 * @class
 		 * @augments WebCenterNode
 		 *
 		 * @param {Workflow} workflow - the workflow to which the node belongs
 		 * @param {String} id - node id
 		 * @param {String} icon - gives the url for fetching the icon
 		 * @param {String} [label] - the label of the node
 		 * @param {WebCenterTaskType} taskType - The task type of the node                                                                                
 		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
 		 */
 		var TaskNode = function (workflow, id, icon, label, taskType, webCenterAPI) {
 			/**
 			 * @member {WebCenterTaskType} the task type of the node
 			 * @private
 			 */
 			this._taskType = taskType;

 			/**
 			 * @member {$.Deferred} the promise for the task specifications
 			 * @private
 			 */
 			this._taskSpecificationsDeferred = webCenterAPI.getTaskSpecifications(this._taskType.getId());

 			WebCenterNode.call(this, workflow, id, WebCenterNodeConstants.Nodes.WCTASK, icon, label, webCenterAPI);
 		};

 		TaskNode.prototype = Object.create(WebCenterNode.prototype);
 		TaskNode.prototype.constructor = TaskNode;

 		/**
 		 * @Override
 		 */
 		TaskNode.prototype._initializePins = function () {
 			this._addInputPin();
 			this._addErrorPin(WebCenterPinConstants.Type.WorkflowConstructError);
 			// the custom output pins are set via WebCenterWorkflowFactory
 		};

 		/*
 		 * @Override
 		 */
 		TaskNode.prototype.getActionXML = function () {
 			return TaskNodeXML(this);
 		};

 		/**
 		 * gets the task type of this node
 		 * @returns {WebCenterTaskType} 
 		 */
 		TaskNode.prototype.getTaskType = function () {
 			return this._taskType;
 		};

 		/*
 		 * @Override
 		 */
 		TaskNode.prototype.setConfiguration = function (config) {
 			_.each(config, function (param) {
 				var assigneeRestriction;
 				switch (param.getName()) {
 				case WebCenterNodeConstants.WCTask.TYPE:
 					param.setValue(this._taskType.getName());
 					param.setDefaultValue(this._taskType.getName());
 					break;
 				case WebCenterNodeConstants.WCTask.NAME:
 					param.setDefaultValue(this._taskType.getName());
 					break;
 				case WebCenterNodeConstants.WCTask.DESCRIPTION:
 					param.setDefaultValue(this._taskType.getDescription());
 					break;
 				case WebCenterNodeConstants.WCTask.LEADDAYS:
 					param.setDefaultValue(this._taskType.getLeadDays());
 					break;
 				case WebCenterNodeConstants.WCTask.LEADTIME:
 					param.setDefaultValue(this._taskType.getLeadTime());
 					break;
 				case WebCenterNodeConstants.WCTask.ASSIGNEE:
 					assigneeRestriction = this._taskType.getRestrictedAssignee();
 					if (assigneeRestriction && assigneeRestriction.restriction === "roleOrGroup") {
 						this._webCenterAPI.getUsersGroupsRoles().done(function (u, g, r) {
 							if (g.has(assigneeRestriction.id)) {
 								param.setDefaultValue(WebCenterNodeConstants.WCTask.GROUPASSIGNMENT);
 							} else if (r.has(assigneeRestriction.id)) {
 								param.setDefaultValue(WebCenterNodeConstants.WCTask.ROLEASSIGNMENT);
 							}
 						});
 					}
 					break;
 				case WebCenterNodeConstants.WCTask.GROUPASSIGNEE:
 					assigneeRestriction = this._taskType.getRestrictedAssignee();
 					if (assigneeRestriction && assigneeRestriction.restriction === "roleOrGroup") {
 						this._webCenterAPI.getUsersGroupsRoles().done(function (u, g, r) {
 							if (g.has(assigneeRestriction.id)) {
 								param.setDefaultValue("grp." + assigneeRestriction.id);
 							}
 						});
 					}
 					break;
 				case WebCenterNodeConstants.WCTask.ROLEASSIGNEE:
 					assigneeRestriction = this._taskType.getRestrictedAssignee();
 					if (assigneeRestriction && assigneeRestriction.restriction === "roleOrGroup") {
 						this._webCenterAPI.getUsersGroupsRoles().done(function (u, g, r) {
 							if (r.has(assigneeRestriction.id)) {
 								param.setDefaultValue("grp." + assigneeRestriction.id);
 							}
 						});
 					}
 					break;
 				default:
 					break;
 				}
 			}, this);
 			this._taskSpecificationsDeferred.done(_.bind(function (specs) {
 				var taskSpecParam = _.find(config, function (param) {
 					return param.getName() === WebCenterPanelConstants.TASKSPECIFICATIONS_PARAMETERID;
 				}, this);

 				// intercept the taskSpecPAram webcenter parameter and turn it into a TaskSpecificationParameter
 				if (!(taskSpecParam instanceof TaskSpecificationsParameter)) {
 					// If the param is not of type taskspecificationsparameter, convert it
 					config.splice(config.indexOf(taskSpecParam), 1);
 					config.push(new TaskSpecificationsParameter(WebCenterPanelConstants.TASKSPECIFICATIONS_PARAMETERID, specs));
 				}
 				WebCenterNode.prototype.setConfiguration.call(this, config);
 			}, this));
 		};

 		/**
 		 * Should the given parameter be saved
 		 * @param   {WebCenterParameter} parameter
 		 * @returns {Boolean} True to save it, false for not to
 		 */
 		TaskNode.prototype.shouldSaveParameter = function (parameter) {
 			return parameter.getName() !== WebCenterNodeConstants.WCTask.TYPE;
 		};

 		/**
 		 * @Override
 		 */
 		TaskNode.prototype.setNodeXML = function (nodeEl) {
 			this._taskSpecificationsDeferred.done(_.bind(function () {
 				WebCenterNode.prototype.setNodeXML.call(this, nodeEl);
 			}, this));
 			this._taskTypeID = nodeEl.querySelector("action wctask_type").getAttribute("id");
 		};

 		/**
 		 * @override
 		 */
 		TaskNode.prototype.getResourceXML = function () {
 			return TaskNodeResourceXML(this);
 		};

 		/**
 		 *	returns all output pins with type OutOk. These are used to save in the resources xml file
 		 * @returns {Map.<String, WebCenterPin>}
 		 */
 		TaskNode.prototype.getOutputOkPins = function () {
 			return _.filter(this.getPins(), function (pin) {
 				return pin.getPinType() === WebCenterPinConstants.Type.OutOK;
 			});
 		};

 		/**
 		 * @override
 		 */
 		TaskNode.prototype._addTaskStatusesToDropdown = function (smartNameDropdownPanel) {
 			this._webCenterAPI.getWorkflowTaskStatusList(this._taskType.getId()).done(_.bind(function (statusesMap) {
 				this._addStatusesToDropdown(statusesMap, smartNameDropdownPanel);
 			}, this));
 		};

 		/**
 		 * @override
 		 */
 		TaskNode.prototype._getNodeSpecificSubPanel = function (deferred, type, paramId, nodeXML, customSmartNames) {
 			if (type === WebCenterPanelConstants.TASKSPECIFICATIONS) {
 				requirejs(["WFEditorWCR/workflow/panel/TaskSpecificationsPanel", "WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService", "WFEditorWCR/workflow/panel/DateSmartNameDictionaryService"], _.bind(function (TaskSpecificationsPanel, WorkflowSmartNameDictionaryService, DateSmartNameDictionaryService) {
 					var nameLabel = WebCenterNodeConstants.SetTaskSpecification.SPECNAME;
 					var valueLabel = WebCenterNodeConstants.SetTaskSpecification.SPECVALUE;

 					var taskTypeID = this.getWorkflow().getTaskTypeID();
 					var panel = new TaskSpecificationsPanel(paramId, i18n.t(nameLabel), i18n.t(valueLabel), new WorkflowSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), new DateSmartNameDictionaryService(this._webCenterAPI, taskTypeID, customSmartNames), this._webCenterAPI, this._taskSpecificationsDeferred);
 					panel.getContext().addClass('parameter-panel form-horizontal form-compact');
 					deferred.resolve(panel);
 				}, this));
 				return true;

 			}
 			return false;
 		};

 		/**
 		 * @override
 		 * intercept task specification parameters
 		 */
 		TaskNode.prototype.getConfigurationParameterByName = function (name) {
 			var taskSpecParam = WebCenterNode.prototype.getConfigurationParameterByName.call(this, WebCenterPanelConstants.TASKSPECIFICATIONS_PARAMETERID);
 			if (taskSpecParam && taskSpecParam.isSpecification(name)) {
 				// if it is a specification parameter return the wrapper parameter
 				return taskSpecParam;
 			} else {
 				return WebCenterNode.prototype.getConfigurationParameterByName.call(this, name);
 			}
 		};

 		/**
 		 * @override
 		 */
 		TaskNode.prototype._addRolesToDropdown = function (dropdown, r) {
 			// If there is a role restriction, only add the specified role to the dropdown and disable it
 			var restrictedAssignee = this._taskType.getRestrictedAssignee();
 			if (restrictedAssignee && restrictedAssignee.restriction === "roleOrGroup" && r.has(restrictedAssignee.id)) {
 				var role = r.get(restrictedAssignee.id);
 				dropdown.setOptions([{
 					text: role.getName(),
 					// Roles in this context also need the group prefix due to historic reasons
 					value: WorkflowSmartNameConstants.TASKROLES.PREFIX + role.getId()
 				}]);
 				dropdown.disable();
 				dropdown.getContext().append($("<label/>").addClass("col-sm-12 col-lg-3")).append($("<span/>").addClass("col-sm-12 col-lg-9").text(i18n.t("task.assignee.restriction.role")));
 			} else {
 				WebCenterNode.prototype._addRolesToDropdown.call(this, dropdown, r);

 			}
 		};

 		/**
 		 * @override
 		 */
 		TaskNode.prototype._addGroupsToDropdown = function (dropdown, g) {
 			// If there is a group restriction, only add the specified group to the dropdown and disable it
 			var restrictedAssignee = this._taskType.getRestrictedAssignee();
 			if (restrictedAssignee && restrictedAssignee.restriction === "roleOrGroup" && g.has(restrictedAssignee.id)) {
 				var group = g.get(restrictedAssignee.id);
 				dropdown.setOptions([{
 					text: group.getName(),
 					// Roles in this context also need the group prefix due to historic reasons
 					value: WorkflowSmartNameConstants.TASKGROUPS.PREFIX + group.getId()
 				}]);
 				dropdown.disable();
 				dropdown.getContext().append($("<label/>").addClass("col-sm-12 col-lg-3")).append($("<span/>").addClass("col-sm-12 col-lg-9").text(i18n.t("task.assignee.restriction.group")));
 			} else {
 				WebCenterNode.prototype._addGroupsToDropdown.call(this, dropdown, g);
 			}

 		};

 		/**
 		 * @override
 		 */
 		TaskNode.prototype._panelAdded = function (panel) {
 			// If the user assignee option is added and there is a noUser restriction on the assignee, disable the option
 			var restrictedAssignee = this._taskType.getRestrictedAssignee();
 			if (restrictedAssignee &&
 				restrictedAssignee.restriction === "noUser" &&
 				_.find(panel.getOwnParameterWishList(), function (param) {
 					return param === WebCenterNodeConstants.WCTask.USERASSIGNEE;
 				})) {
 				panel.disable();
 				panel.getContext().append($("<span/>").text(i18n.t("task.assignee.restriction.noUser")).addClass("panel-information-text"));
 			}
 		};

 		/** 
 		 * @override
 		 */
 		TaskNode.prototype.getStateObjectForCopy = function () {
 			var stateObj = WebCenterNode.prototype.getStateObjectForCopy.call(this);
 			stateObj.taskTypeId = this._taskType.getId();
 			return stateObj;
 		};



 		return TaskNode;
 	});

define('WFEditorWCR/workflow/node/router/RouterParameter',["underscore",
		"jquery",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants"
	],
	function (
		_,
		$,
		WebCenterParameter,
		WebCenterPinConstants,
		WebCenterNodeConstants) {

		"use strict";

		/**
		 * Parameter used for router nodes
		 * @class
		 * @exports RouterParameter
		 * @augments WebCenterParameter
		 * @param {String} valueType  - When routing on a specific attribute this should be the type of the attribute - fe. TEXT or DATETIME
		 * @param {String} value      - When routing on an attribute this is the id of the attribute
		 * @param {String} routerType - the type of the router used - see WebCenterNodeConstants.RouterNode.Routers for the different types available
		 */
		var RouterParameter = function (valueType, value, routerType) {
			WebCenterParameter.call(this, valueType, null, value, []);

			/**
			 * @member {String}
			 * @private 
			 */
			this._routerType = routerType || WebCenterNodeConstants.RouterNode.Routers.ProjectField;

			/**
			 * @member {Array.<Object>} the different router entries, has a ruleType, ruleValues and an outputId
			 * @private 
			 */
			this._routerEntries = [];

			/**
			 * @member {Array.<WebCenterOutputPin>} list of outputs
			 * @private
			 */
			this._routerOutputs = [];

			/**
			 * @member {String} the id of the output used for the default route
			 * @private
			 */
			this._defaultOutputId = WebCenterPinConstants.Type.WorkflowConstructError;

			/**
			 * @member {Boolean} 
			 * @private 
			 */
			this._isCaseSensitive = false;

			/**
			 * @member {String}
			 * @private 
			 */
			this._shouldIncludeExtension = false;

			/**
			 * @member {WebCenterNodeConstants.RouterNode.RuleTypes} used for xml generation of the rules
			 * @private
			 */
			this._rulesType = WebCenterNodeConstants.RouterNode.RuleTypes.StringRule;

			/**
			 * @member {Array.<String>} the values of the restricteds et
			 * @private 
			 */
			this._rss = [];

		};

		RouterParameter.prototype = Object.create(WebCenterParameter.prototype);
		RouterParameter.prototype.constructor = RouterParameter;

		/**
		 * Reset the parameter
		 */
		RouterParameter.prototype.reset = function () {
			this._routerEntries = [];
			this._routerOutputs = [];
			this._routerType = WebCenterNodeConstants.RouterNode.Routers.ProjectField;
			this._name = "";
			this._value = "";
			this._defaultOutputId = WebCenterPinConstants.Type.WorkflowConstructError;
			this._isCaseSensitive = false;
			this._shouldIncludeExtension = false;
			this._rulesType = WebCenterNodeConstants.RouterNode.RuleTypes.StringRule;
			this._rss = [];
			this._subparameters = [];
		};

		/**
		 * Add a router entry
		 * @param {String}         ruleType   
		 * @param {Array.<String>} ruleValues 
		 * @param {String}         outputId   
		 */
		RouterParameter.prototype.addRouterEntry = function (ruleType, ruleValues, outputId) {
			this._routerEntries.push({
				"ruleType": ruleType,
				"ruleValues": ruleValues,
				"outputId": outputId
			});
		};

		/**
		 * Sets the outputs 
		 * @param {Array.<WebCenterOutputPin>} outputs
		 */
		RouterParameter.prototype.setRouterOutputs = function (outputs) {
			this._routerOutputs = outputs;
		};

		/**
		 * used for xml generation of the router parameter so that the backend know which type it is
		 * 
		 * @param {WebCenterNodeConstants.RouterNodes.RuleTypes} ruleType - can be any of "numericrule", "stringrule", "enumrule"
		 */
		RouterParameter.prototype.setRouterRuleType = function (rulesType) {
			if(rulesType) {
				this._rulesType = rulesType;
			}
		};

		/**
		 * gets the router rule type - used for xml generation
		 * @returns {WebCenterNodeConstants.RouterNode.RuleTypes} 
		 */
		RouterParameter.prototype.getRouterRuleType = function () {
			return this._rulesType;
		};

		/**
		 * gets the restricted set options for the parameter
		 * @returns {Array.<String>} 
		 */
		RouterParameter.prototype.getRestrictedSet = function () {
			return this._rss;
		};

		/**
		 * sets the restricted set options for the parameter
		 * @param {Array.<String>} rss 
		 */
		RouterParameter.prototype.setRestrictedSet = function (rss) {
			this._rss = rss;
		};

		/**
		 * @param {String}
		 */
		RouterParameter.prototype.setDefaultOutputId = function (id) {
			this._defaultOutputId = id;
		};

		/**
		 * @returns {String}
		 */
		RouterParameter.prototype.getDefaultOutputId = function () {
			return this._defaultOutputId;
		};

		/**
		 * @returns {Array.<WebCenterOutputPin>}
		 */
		RouterParameter.prototype.getRouterOutputs = function () {
			return this._routerOutputs;
		};

		/**
		 * @returns {Array.<Object>}
		 */
		RouterParameter.prototype.getRouterEntries = function () {
			return this._routerEntries;
		};

		/**
		 * returns a copy of this routerparameter
		 * @returns {RouterParameter} 
		 */
		RouterParameter.prototype.copy = function () {
			var routerParameter = new RouterParameter(this._name, this._value, this._routerType);

			_.each(this.getRouterEntries(), function (entry) {
				var valueCopy = [];
				_.each(entry.ruleValues, function (value) {
					valueCopy.push(value);
				});
				routerParameter.addRouterEntry(entry.ruleType, valueCopy, entry.outputId);
			}, this);

			var routerOutputsCopy = [];

			_.each(this._routerOutputs, function (output) {
				routerOutputsCopy.push(output);
			});

			routerParameter.isCaseSensitive(this._isCaseSensitive);
			routerParameter.shouldIncludeExtension(this._shouldIncludeExtension);
			routerParameter.setRouterOutputs(routerOutputsCopy);
			routerParameter.setDefaultOutputId(this._defaultOutputId);
			routerParameter.setRouterRuleType(this._rulesType);
			var rss = [];
			_.each(this._rss, function (rs) {
				rss.push(rs);
			});
			routerParameter.setRestrictedSet(rss);

			var subParams = [];
			_.each(this.getSubparameters(), function (subParam) {
				subParams.push(subParam.copy());
			});
			routerParameter.setSubParameters(subParams);

			return routerParameter;
		};

		/**
		 * @param {String}
		 */
		RouterParameter.prototype.setRouterType = function (routerType) {
			this._routerType = routerType;
		};

		/**
		 * sets the case sensitivity of the parameter, is no argument is defined, returns the case sensitivity.
		 * @param {Boolean} [isCaseSensitive] - if defined, sets the caseSensitivity to 
		 * @returns {Boolean | undefined}                                   
		 */
		RouterParameter.prototype.isCaseSensitive = function (isCaseSensitive) {
			if (typeof isCaseSensitive === "undefined") {
				return this._isCaseSensitive;
			} else {
				this._isCaseSensitive = isCaseSensitive;
			}
		};

		/**
		 * sets the whether the file extension should be included. If no argument is defined, returns whether the file extension should be included.
		 * @param {Boolean} [shouldIncludeExtension]
		 * @returns {Boolean | undefined}                                   
		 */
		RouterParameter.prototype.shouldIncludeExtension = function (shouldIncludeExtension) {
			if (typeof shouldIncludeExtension === "undefined") {
				return this._shouldIncludeExtension;
			} else {
				this._shouldIncludeExtension = shouldIncludeExtension;
			}
		};

		/**
		 * @returns {String}
		 */
		RouterParameter.prototype.getRouterType = function () {
			return this._routerType;
		};

		/**
		 * Is this routerParameter equal to the given parameter
		 * @param   {Parameter} param
		 * @returns {Boolean}   	
		 */
		RouterParameter.prototype.equals = function (param) {

			if (!WebCenterParameter.prototype.equals.call(this, param)) {
				return false;
			}

			// If param isn't a routerparameter or if it doens't have the same type, entries length or outputs length they're mos def not the same
			if (!(param instanceof RouterParameter) ||
				this.getRouterType() !== param.getRouterType() ||
				this.getRouterEntries().length !== param.getRouterEntries().length ||
				this.getRouterOutputs().length !== param.getRouterOutputs().length ||
				this._isCaseSensitive !== param.isCaseSensitive() ||
				this._shouldIncludeExtension !== param.shouldIncludeExtension() ||
				this._rss.length !== param.getRestrictedSet().length ||
				this._rulesType !== param.getRouterRuleType() ||
				this._defaultOutputId !== param.getDefaultOutputId()) {
				return false;
			}

			var i;
			var otherRSS = param.getRestrictedSet();
			for (i = 0; i < this._rss.length; i++) {
				if (this._rss[i] !== otherRSS[i]) {
					return false;
				}
			}

			// check if the router entries are different
			for (i = 0; i < this._routerEntries.length; i++) {
				var entry1 = this._routerEntries[i];
				var entry2 = param.getRouterEntries()[i];
				if (entry1.ruleType !== entry2.ruleType ||
					entry1.outputId !== entry2.outputId) {
					return false;
				}
				for (var j = 0; j < entry1.ruleValues.length; j++) {
					if (entry1.ruleValues[j] !== entry2.ruleValues[j]) {
						return false;
					}
				}
			}

			// check if the router outputs are different
			for (i = 0; i < this._routerOutputs.length; i++) {
				var output1 = this._routerOutputs[i];
				var output2 = param.getRouterOutputs()[i];
				if (output1 !== output2) {
					return false;
				}
			}

			// They are the same
			return true;

		};


		return RouterParameter;

	});


define('tpl!WFEditorWCR/workflow/node/router/RouterNodeResource', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<resource id="'+
((__t=(d.node.getId()))==null?'':_.escape(__t))+
'">\r\n	<filter class="'+
((__t=(d.param.getRouterType()))==null?'':_.escape(__t))+
'">\r\n		';
_.each(d.param.getRouterEntries(),function(entry){ 
__p+='<'+
((__t=(d.param.getRouterRuleType()))==null?'':_.escape(__t))+
'>\r\n			<op>'+
((__t=(entry.ruleType))==null?'':_.escape(__t))+
'</op>\r\n			';
if(d.param.getRouterRuleType() === "stringrule") {print("<casesensitive>");
__p+=''+
((__t=(d.param.isCaseSensitive()?"TRUE":"FALSE"))==null?'':_.escape(__t))+
'';
print("</casesensitive>");}
__p+='';
 if(entry.ruleValues.length===1){ 
__p+='\r\n			';
if(d.param.getRouterRuleType() === "numericrule") { print("<value_1>");
__p+=''+
((__t=(entry.ruleValues[0]))==null?'':_.escape(__t))+
'';
print("</value_1>"); } else { print("<value>");
__p+=''+
((__t=(entry.ruleValues[0]))==null?'':_.escape(__t))+
'';
print("</value>"); } 
__p+='';
 } else if(entry.ruleValues.length > 1) {_.each(entry.ruleValues,function(value,i){ 
__p+='<value_'+
((__t=(i+1))==null?'':_.escape(__t))+
'>'+
((__t=(value))==null?'':_.escape(__t))+
'</value_'+
((__t=(i+1))==null?'':_.escape(__t))+
'>';
})}
__p+='\r\n			<output>'+
((__t=(entry.outputId))==null?'':_.escape(__t))+
'</output>\r\n		</'+
((__t=(d.param.getRouterRuleType()))==null?'':_.escape(__t))+
'>';
})
__p+='\r\n		<unmatched>\r\n			<output>'+
((__t=(d.param.getDefaultOutputId()))==null?'':_.escape(__t))+
'</output>\r\n		</unmatched>\r\n		<outputs>\r\n			';
 _.each(d.param.getRouterOutputs(),function(output){ 
__p+='	<output>\r\n				<name>'+
((__t=(output.getLabel()))==null?'':_.escape(__t))+
'</name>\r\n				<id>'+
((__t=(output.getOutputId()))==null?'':_.escape(__t))+
'</id>\r\n				<type>'+
((__t=(output.getPinType()===d.OutError?"ERROR":"OK"))==null?'':_.escape(__t))+
'</type>\r\n			</output>\r\n		';
})
__p+='</outputs>\r\n		';
 if(d.param.getValue()) { 
__p+='<parameter>'+
((__t=(d.param.getValue()))==null?'':_.escape(__t))+
'</parameter>';
}
__p+='\r\n		';
 if(d.param.getSubparameters().length > 0){ 
__p+='<subparameters>'+
((__t=( d.param.getSubparameterXML() ))==null?'':__t)+
'</subparameters>';
 } 
__p+='\r\n		';
 if(d.param.getName()) { 
__p+='<type>'+
((__t=(d.param.getName()))==null?'':_.escape(__t))+
'</type>';
 } 
__p+='\r\n		';
 if(d.param.getRestrictedSet().length > 0) { 
__p+='<rss>\r\n			';
 _.each(d.param.getRestrictedSet(),function(value){ 
__p+='<rs>'+
((__t=(value))==null?'':_.escape(__t))+
'</rs>';
})
__p+='\r\n		</rss>\r\n		';
 } 
__p+='';
if(d.param.getRouterType() === "WCWF_TDocumentNameRouter"){print("<includeextension>");
__p+=''+
((__t=(d.param.shouldIncludeExtension()?"TRUE":"FALSE"))==null?'':_.escape(__t))+
'';
print("</includeextension>");}
__p+='\r\n	</filter>\r\n</resource>';
return __p;
}});

define('tpl!WFEditorWCR/workflow/node/router/RouterNodeAction', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<action class="bsroute">\r\n<warningpolicy>dedicatedhandling</warningpolicy>\r\n<parameters />\r\n<uiImpl>com.esko.wf.ui.plugin.model.WCWF_TWCRoute</uiImpl>\r\n<srvImpl>com.esko.wf.srv.plugin.bs.task.BS_TGenPropertyFilter</srvImpl>\r\n<resourceref>'+
((__t=( d.getId() ))==null?'':_.escape(__t))+
'</resourceref>\r\n</action>';
return __p;
}});
define('WFEditorWCR/workflow/node/router/RouterNode',[
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/node/router/RouterParameter",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"tpl!WFEditorWCR/workflow/node/router/RouterNodeResource",
		"tpl!WFEditorWCR/workflow/node/router/RouterNodeAction",
		"utils/XMLUtils",
		"underscore"
	],
	function (
		WebCenterNode,
		WebCenterNodeConstants,
		OutputPinOK,
		WebCenterPinConstants,
		RouterParameter,
		WebCenterParameter,
		RouterNodeResourceXML,
		RouterNodeActionXML,
		XMLUtils,
		_
	) {
		"use strict";

		/**
		 * RouterNode represents the model for a  router node in the WebCenter Workflow Editor.
		 *
		 * @exports RouterNode
		 * @class
		 * @augments WebCenterNode
		 *
		 * @param {Workflow}     workflow     - the workflow to which the node belongs
		 * @param {String}       id           - node id
		 * @param {String}       icon		  - icon url
		 * @param {String}       [label]      - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var RouterNode = function (workflow, id, icon, label, webCenterAPI) {
			WebCenterNode.call(this, workflow, id, WebCenterNodeConstants.Nodes.ROUTER, icon, label, webCenterAPI);

			// Set the default routernode configuration
			var routerParameter = new RouterParameter(null, "");
			routerParameter.setRouterType(WebCenterNodeConstants.RouterNode.Routers.ProjectField);
			this._configuration = [routerParameter];
			routerParameter.getRouterOutputs().push(this.getErrorPin());
		};

		RouterNode.prototype = Object.create(WebCenterNode.prototype);
		RouterNode.prototype.constructor = RouterNode;

		/**
		 * @Override
		 */
		RouterNode.prototype._initializePins = function () {
			this._addInputPin();
			this._addErrorPin(WebCenterPinConstants.Type.WorkflowConstructError);
		};

		/**
		 * Read the Resource XML
		 * @Override
		 */
		RouterNode.prototype.setResourceXML = function (resourceXML) {
			var resourceObj = XMLUtils.xml2obj(resourceXML);

			// save the resourceXML for generating correct XML when asked... 
			this._resourceXML = new XMLSerializer().serializeToString(resourceXML);



			if (resourceObj && resourceObj.filter) {
				var routerType = resourceObj.filter.attributes.class; // the type of the router
				var routerEntries = resourceObj.filter.numericrule || resourceObj.filter.stringrule || resourceObj.filter.enumrule;
				if (routerEntries && !_.isArray(routerEntries)) {
					routerEntries = [routerEntries];
				}
				this._configuration = [new RouterParameter(null, "")];
				this._configuration[0].setRouterType(routerType);
				this._configuration[0].setRouterRuleType(resourceObj.filter.numericrule ? WebCenterNodeConstants.RouterNode.RuleTypes.NumericRule :
					resourceObj.filter.enumrule ? WebCenterNodeConstants.RouterNode.RuleTypes.EnumRule : WebCenterNodeConstants.RouterNode.RuleTypes.StringRule);

				if (resourceObj.filter.outputs) {
					// At this point only the error node is added to the pins, we wish to have the error node at the back of all pins
					// Instead of inserting all the new pins before the errorPin, we first remove the error pin and then add it back.

					var errorPin = this.getErrorPin();
					this.removePin(errorPin);
					_.each(resourceObj.filter.outputs.output, function (output) {
						if (output && output.type == "OK") {
							this.addPin(new OutputPinOK(this._workflow, this, output.id, output.name));
						}

					}, this);
					this.addPin(errorPin);
					this._configuration[0].setRouterOutputs(_.filter(this.getPins(), _.bind(function (pin) {
						return pin !== this.getInputPin();
					}, this)));
				}

				var isCaseSensitive = false;
				_.each(routerEntries, function (entry) {
					var values = [];
					var i = 1;

					if (entry.casesensitive === "TRUE") {
						isCaseSensitive = true;
					}

					if (typeof entry.value !== "undefined") {
						values.push(entry.value);
					}

					if (entry.value_0) {
						values.push(entry.value_0);
					}
					while (entry["value_" + i]) {
						values.push(entry["value_" + i]);
						i++;
					}
					this._configuration[0].addRouterEntry(entry.op, values, entry.output);
				}, this);
				this._configuration[0].isCaseSensitive(isCaseSensitive);
				this._configuration[0].shouldIncludeExtension(resourceObj.filter.includeextension === "TRUE");
				if (resourceObj.filter.type) {
					this._configuration[0].setName(resourceObj.filter.type);
				}
				if (resourceObj.filter.parameter) {
					this._configuration[0].setValue(resourceObj.filter.parameter);
				}
				if (resourceObj.filter.subparameters) {
					var subParams = [];
					_.each(resourceObj.filter.subparameters, function (parameter) {
						if (parameter.Name) {
							subParams.push(new WebCenterParameter(parameter.Name, "", parameter.Value));
						}
					}, this);
					this._configuration[0].setSubParameters(subParams);
				}
				if (resourceObj.filter.unmatched) {
					if (resourceObj.filter.unmatched.output === "Error") {
						this._configuration[0].setDefaultOutputId(this.getErrorPin().getOutputId());
					} else {
						this._configuration[0].setDefaultOutputId(resourceObj.filter.unmatched.output);
					}
				}
				if (resourceObj.filter.rss) {
					var rss = [];
					_.each(resourceObj.filter.rss.rs, function (rs) {
						rss.push(rs);
					});
					this._configuration[0].setRestrictedSet(rss);
				}
			}
		};

		/**
		 * @override
		 */
		RouterNode.prototype.getResourceXML = function () {
			return RouterNodeResourceXML({
				node: this,
				param: this._configuration[0],
				OutError: WebCenterPinConstants.Type.OutError
			});
		};

		/**
		 * @override
		 */
		RouterNode.prototype.getActionXML = function () {
			return RouterNodeActionXML(this);
		};

		/**
		 * @override
		 */
		RouterNode.prototype.getStateObjectForCopy = function () {
			var stateObj = WebCenterNode.prototype.getStateObjectForCopy.call(this);
			stateObj.id = "genPropertyFilter";
			return stateObj;
		};

		/**
		 * @override
		 * @param   {Array.<RouterParameter>} configuration 
		 */
		RouterNode.prototype.setConfiguration = function (configuration) {
			if (configuration[0] instanceof RouterParameter) {
				WebCenterNode.prototype.setConfiguration.call(this, configuration);
				_.each(this.getPins(), function (pin) {
					// first remove all pins in the router
					if (pin !== this.getInputPin()) {
						this.removePin(pin);
						_.each(pin.getEdges(), function (edge) {
							edge.getToPin().removeEdge(edge);
							this.getWorkflow().removeEdge(edge);
						}, this);
					}
				}, this);
				_.each(configuration[0].getRouterOutputs(), function (output) {
					// then add all pins in the configuration and their edges
					this.addPin(output);
					_.each(output.getEdges(), function (edge) {
						edge.getToPin().addEdge(edge);
						this.getWorkflow().addEdge(edge);
					}, this);
				}, this);

			}
		};

		/**
		 * @override
		 * @returns {WebCenterNodeConfigurationPanel} 
		 */
		RouterNode.prototype.getConfigurationPanel = function () {
			var deferred = $.Deferred();
			requirejs([
				"WFEditorWCR/workflow/node/router/routerPanel/RouterPanel",
				"WFEditorWCR/workflow/panel/WorkflowSmartNameDictionaryService",
				"WFEditorWCR/workflow/panel/DateSmartNameDictionaryService"
			], _.bind(function (
				RouterPanel,
				WorkflowSmartNameDictionaryService,
				DateSmartNameDictionaryService
			) {
				if (!this._smartNameDictionary) {
					this._smartNameDictionary = new WorkflowSmartNameDictionaryService(this._webCenterAPI, this.getWorkflow().getTaskTypeID());
					this._dateSmartNameDictionary = new DateSmartNameDictionaryService(this._webCenterAPI, this.getWorkflow().getTaskTypeID());
				}
				deferred.resolve(new RouterPanel(this, this._webCenterAPI, this._smartNameDictionary, this._dateSmartNameDictionary));
			}, this));
			return deferred.promise();

		};



		return RouterNode;
	});

define('WFEditorWCR/workflow/node/submitWorkflow/SubmitWorkflowNode',[
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/OutputPinWarning",
		"WFEditorWCR/workflow/pin/OutputPinError",
		"WFEditorWCR/workflow/panel/WebCenterParameter",
		"utils/webi18n!WFEditorWCR"
	],
	function (
		WebCenterNode,
		WebCenterNodeConstants,
		OutputPinOK,
		OutputPinWarning,
		OutputPinError,
		WebCenterParameter,
		i18n
	) {
		"use strict";

		/**
		 * SubmitWorkflowNode represents the model for a SubmitWorkflow node in the WebCenter Workflow Editor.
		 *
		 * @exports SubmitWorkflowNode
		 * @class
		 * @augments WebCenterNode
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {ShapeUtils.Point} location - location of the node
		 * @param {String} icon - the url of the icon
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var SubmitWorkflowNode = function (workflow, id, icon, label, webCenterAPI) {
			WebCenterNode.call(this, workflow, id, WebCenterNodeConstants.Nodes.SUBMITWORKFLOW, icon, label, webCenterAPI);


		};

		SubmitWorkflowNode.prototype = Object.create(WebCenterNode.prototype);
		SubmitWorkflowNode.prototype.constructor = SubmitWorkflowNode;

		/**
		 * @override
		 */
		SubmitWorkflowNode.prototype.setNodeXML = function (nodeEl) {
			var params = nodeEl.querySelectorAll("action > parameters > parameter > Name");
			_.each(params, function (nameNode) {
				var name = nameNode.textContent;

				// Migrate public parameters to new checkboxes
				if (name === WebCenterNodeConstants.SubmitWorkflow.PUBLICPARAMETERS) {
					var publicParam = new WebCenterParameter(WebCenterNodeConstants.SubmitWorkflow.PUBLICPARAMETERS);
					publicParam.updateFromXML(nameNode.parentNode);

					if (publicParam.getValue() === "true") {
						// document properties
						this._migrateParameterToTrue(WebCenterNodeConstants.SubmitWorkflow.DOCUMENTPROPERTIES);
						// project properties
						this._migrateParameterToTrue(WebCenterNodeConstants.SubmitWorkflow.PROJECTPROPERTIES);
						// document attributes
						this._migrateParameterToTrue(WebCenterNodeConstants.SubmitWorkflow.DOCUMENTATTRIBUTES);
						// project attributes
						this._migrateParameterToTrue(WebCenterNodeConstants.SubmitWorkflow.PROJECTATTRIBUTES);
					}
				} else {
					var param = this.getConfigurationParameterByName(name);
					if (param) {
						param.updateFromXML(nameNode.parentNode);
					}
				}

			}, this);
		};

		/**
		 * Migrate one of the parameters with the given name to true
		 * @param {String} name - Name of the parameter to set to true
		 */
		SubmitWorkflowNode.prototype._migrateParameterToTrue = function (name) {
			var param = this.getConfigurationParameterByName(name);
			if (param) {
				param.setValue("true");
			}
		};

		return SubmitWorkflowNode;
	});

define('WFEditor/workflow/token/Token',[
		"WFEditor/workflow/WorkflowItem",
		"WFEditor/workflow/ModelConstants",
		"underscore"
	],
	function (
		WorkflowItem,
		ModelConstants,
		_
	) {
		"use strict";

		/**
		 * Token represents a collection of data that moves in the workflow and is shown in
		 * the feedback version of a workflow.
		 *
		 * @exports Token
		 * @class
		 * @augments WorkflowItem
		 *
		 * @param {Workflow} workflow
		 * @param {Pin} [Pin] - The pin where this token belongs to. Can be undefined.
		 * @param {String} id - token id
		 * @param {Boolean} isProcessed - has this string been processed yet?                   
		 */
		var Token = function (workflow, pin, id, isProcessed) {
			WorkflowItem.call(this, workflow, ModelConstants.WorkflowItemType.Token, id, pin);

			/**
			 *  @member {Array.<Token>}
			 *  @private
			 */
			this._parentTokens = [];

			/**
			 *  @member {Pin}
			 *  @private
			 */
			this._pin = pin;

			/**
			 *  @member {Boolean}
			 *  @private
			 */
			this._isSelected = false;

			/**
			 *  @member {Boolean}
			 *  @private
			 */
			this._isProcessed = isProcessed;

		};

		Token.prototype = Object.create(WorkflowItem.prototype);
		Token.prototype.constructor = Token;

		/**
		 * Returns the parent tokens
		 * @returns {Array.<Token>}
		 */
		Token.prototype.getParents = function () {
			return this._parentTokens;
		};

		/**
		 * Add a parent token to this token.
		 * @param {Token} newParent
		 */
		Token.prototype.addParent = function (newParent) {
			this._parentTokens.push(newParent);
		};

		/**
		 * Remove the parent tokens for this token
		 */
		Token.prototype.removeParentTokens = function () {
			this._parentTokens = [];
		};

		/**
		 * Check if a token with the given ID is a parent token of this token
		 * @param {String} parentId
		 * @returns {Boolean} True if the token with the given parent ID is a parent of this token, false otherwise
		 */
		Token.prototype.hasParent = function (parentId) {
			return _.some(this._parentTokens, function (parentToken) {
				return parentToken.getId() === parentId;
			}, this);
		};


		/**
		 * Check if this token has future tokens
		 * @returns {Boolean} True if this token is a parent of another token, false otherwise
		 */
		Token.prototype.hasFutureTokens = function () {
			return _.some(this.getWorkflow().getTokens(), function (token) {
				return token.hasParent(this.getId());
			}, this);
		};

		/**
		 * get the Pin this token is connected to or undefined if not attached to a pin
		 * @returns {Pin}
		 */
		Token.prototype.getPin = function () {
			return this._pin;
		};

		/**
		 * Get the number that is used for this token (weight of the token). by default this is 1.
		 * @returns {Number} [[Description]]
		 */
		Token.prototype.getWeight = function () {
			return 1;
		};

		/**
		 * Set the selected state of the token
		 * @param {Boolean} isSelected
		 */
		Token.prototype.setSelected = function (isSelected) {
			this._isSelected = isSelected;
		};

		/**
		 * Is the token selected or not.
		 * @returns {Boolean}
		 */
		Token.prototype.isSelected = function () {
			return this._isSelected;
		};

		/**
		 * has the token been processed yet
		 * @returns {Boolean}
		 */
		Token.prototype.isProcessed = function () {
			return this._isProcessed;
		};

		/**
		 * Sets the processed state of the Token
		 */
		Token.prototype.setProcessed = function (isProcessed) {
			if (isProcessed !== this._isProcessed) {
				this._isProcessed = isProcessed;
				this._emit("tokenChanged", this);
			}

		};

		return Token;
	});

define('WFEditorWCR/workflow/token/WebCenterToken',[
		"WFEditor/workflow/token/Token",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"underscore"
	],
	function (
		Token,
		WebCenterPinConstants,
		_
	) {
		"use strict";

		/**
		 * Represents the token used in the WebCenter workflow editor. The data of the token
		 * consists of several files.
		 *
		 * @exports WebCenterToken
		 * @class
		 * @augments Token
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {Pin} [pin] - node to which the token belongs. Undefined for the init token.
		 * @param {String} id
		 * @param {XMLObject} data - xml object with the data for this token
		 * @param {String|null} cloneId - id of the token this token has been cloned from, null if not cloned
		 * @param {Boolean} isProcessed - has this token been processed yet?
		 */
		var WebCenterToken = function (workflow, pin, id, data, cloneId, isProcessed) {
			Token.call(this, workflow, pin, id, isProcessed);

			/**
			 * @member {Map.<String, String>} maps file id to file name
			 * @private
			 */
			this._files = {};

			/**
			 * @member {Map.<String, String>} maps error id to the error message
			 * @private
			 */
			this._errors = {};

			/**
			 * @member {String|null} the id of the token this token is cloned from
			 * @private
			 */
			this._cloneId = cloneId;

			/**
			 * @member {String} Id of the document version (of this token) that is currently selected
			 * @private
			 */
			this._selectedDocumentVersionId = null;

			/**
			 * @member {String} id for the error handling part if the token sits in the error pin
			 * @private
			 */
			this._errorHandlingId = null;

			/**
			 * If this is a wait token, it has a timer.
			 * @member {Number} milliseconds until wait is over
			 * @private
			 */
			this._timer = 0;

			/**
			 * Object holding the subtasks of this token. Each subtask has a name,an owner and a status
			 */
			this._subTasks = {};

			if (data) { //parse provided XML token data
				this._processXMLData(data);
			}
		};

		WebCenterToken.prototype = Object.create(Token.prototype);
		WebCenterToken.prototype.constructor = WebCenterToken;

		/**
		 * Add a file to the token
		 * @param {String} fileName
		 * @param {String} fileId
		 */
		WebCenterToken.prototype.addFile = function (fileName, fileId) {
			this._files[fileId] = fileName;
		};

		/**
		 * Get all the files that are part of this token.
		 * @returns {Map.<String, String>}
		 */
		WebCenterToken.prototype.getFiles = function () {
			if (this.getPin().getPinType() === WebCenterPinConstants.Type.In && _.size(this._files) === 0) {
				this._forwardFilesFromParent();
			}
			return this._files;
		};

		/**
		 * Get the number of files in this token.
		 * @returns {Number}
		 */
		WebCenterToken.prototype.getFileCount = function () {
			if (this.getPin().getPinType() === WebCenterPinConstants.Type.In && _.size(this._files) === 0) {
				this._forwardFilesFromParent();
			}
			return _.size(this._files);
		};

		/**
		 * Creates the file for this token form its parents
		 */
		WebCenterToken.prototype._forwardFilesFromParent = function () {
			this._files = {};
			_.each(this._parentTokens, function (parent) {
				_.each(parent.getFiles(), function (file, fileId) {
					this._files[fileId] = file;
				}, this);
			}, this);
		};

		/**
		 * Adds an error message to this token.
		 * @param {String} errorId
		 * @param {String} errorMessage
		 */
		WebCenterToken.prototype.addErrorMessage = function (errorId, errorMessage) {
			this._errors[errorId] = errorMessage;
		};

		/**
		 * Get all the errors of this token.
		 * @returns {Array.<object>}
		 */
		WebCenterToken.prototype.getErrors = function () {
			return this._errors;
		};

		/**
		 * Does this token contain any errors.
		 * @returns {Boolean}
		 */
		WebCenterToken.prototype.hasErrors = function () {
			return _.size(this._errors) > 0;
		};

		/**
		 * Update the Token with new XML Data
		 * @param {XMLObject} XML data
		 */
		WebCenterToken.prototype.updateFromXMLData = function (data) {
			// clear out existing errors and files
			this._errors = {};
			this._files = {};

			if (data) {
				// process the xml
				this._processXMLData(data);
			}
		};

		/**
		 * Processes the XML data and adds the parsed data to the token
		 * @param {XMLObject} data
		 */
		WebCenterToken.prototype._processXMLData = function (data) {
			var filesEl = data.getElementsByTagName("files");
			if (filesEl && filesEl[0]) {
				_.each(filesEl[0].childNodes, function (fileEl) {
					if (!fileEl || fileEl.nodeType !== 1 /*Node.ELEMENT_NODE*/ ) {
						return;
					}
					var fileName = fileEl.getElementsByTagName("name");
					var fileId = fileEl.getElementsByTagName("id");
					var fileIsProcessedEl = fileEl.getElementsByTagName("processed");
					var fileIsProcessed = false;
					if (fileIsProcessedEl && fileIsProcessedEl[0]) {
						fileIsProcessed = fileIsProcessedEl[0].textContent === "1";
					}

					if (fileName && fileId && fileName[0] && fileId[0] && !fileIsProcessed) {
						this.addFile(fileName[0].textContent, fileId[0].textContent);
					}
				}, this);
			}
			var errorsEl = data.getElementsByTagName("errors");
			if (errorsEl && errorsEl[0]) {
				_.each(errorsEl[0].childNodes, function (errorEl) {
					if (!errorEl || errorEl.nodeType !== 1) {
						return;
					}
					var messageEl = errorEl.getElementsByTagName("message");
					var errorIdEl = errorEl.getElementsByTagName("id");
					if (messageEl && messageEl[0] && errorIdEl && errorIdEl[0]) {
						this.addErrorMessage(errorIdEl[0].textContent, messageEl[0].textContent);
					}
				}, this);
			}
			var timerEl = data.getElementsByTagName("timer");
			if (timerEl && timerEl[0]) {
				this._timer = parseInt(timerEl[0].textContent);
			}
			// Get token owner and token name
			var subtasksEl = data.getElementsByTagName("subtasks");
			if (subtasksEl && subtasksEl[0]) {
				_.each(subtasksEl[0].childNodes, function (subtaskEl) {
					if (!subtaskEl || subtaskEl.nodeType !== 1) {
						return;
					}
					var taskId = subtaskEl.getElementsByTagName("taskid");
					var owner = subtaskEl.getElementsByTagName("owner");
					var name = subtaskEl.getElementsByTagName("name");
					var status = subtaskEl.getElementsByTagName("status");
					if (taskId && taskId[0] && owner && owner[0] && name && name[0] && status && status[0]) {
						this._addSubtask(taskId[0].textContent, owner[0].textContent, name[0].textContent, status[0].textContent);
					}
				}, this);
			}
		};

		/**
		 * Get the weight of the token, this is the number shown on the pin
		 * @returns {Number}
		 */
		WebCenterToken.prototype.getWeight = function () {
			return _.size(this._files) || 1; //return 1 if the token contains no files
		};

		/**
		 * Is the token been cloned from another token
		 * @returns {Boolean}
		 */
		WebCenterToken.prototype.isCloned = function () {
			return this._cloneId !== null;
		};

		/**
		 * Returns the ID of the token this token was cloned from
		 * @returns {String|null}
		 */
		WebCenterToken.prototype.getClonedId = function () {
			return this._cloneId;
		};

		/**
		 * Set the selected state of the document, possible to set the selected document version id
		 * @param {Boolean} isSelected
		 * @param {String} versionId
		 */
		WebCenterToken.prototype.setSelected = function (isSelected, versionId) {
			if (this._isSelected !== isSelected ||
				versionId && this._selectedDocumentVersionId !== versionId ||
				!versionId && this._selectedDocumentVersionId !== null) {
				// selected state changed

				this._isSelected = isSelected;
				if (versionId) {
					this._selectedDocumentVersionId = versionId;
				} else {
					this._selectedDocumentVersionId = null;
				}

				this._emit("changed.select");
			}

		};

		/**
		 * Get the selected document version id.
		 * @returns {String|null}
		 */
		WebCenterToken.prototype.getSelectedDocumentVersionId = function () {
			return this._selectedDocumentVersionId;
		};

		/**
		 * Set the highlighted state of the token
		 * @param {Boolean} highlight
		 */
		WebCenterToken.prototype.setHighlighted = function (highlight) {
			var clonedToken = this._workflow.getToken(this._cloneId);
			//get the token this token was cloned from and if found set the highlighted state
			if (clonedToken) { //this is the token that is shown in the workflow feedback
				clonedToken.setHighlighted(highlight);
			}
			Token.prototype.setHighlighted.call(this, highlight);
		};

		/**
		 * Set the id for the error handling this is used for the retry action of the token
		 * @param {String} errorHandlingId
		 */
		WebCenterToken.prototype.setErrorHandlingId = function (errorHandlingId) {
			this._errorHandlingId = errorHandlingId;
		};

		/**
		 * Returns the id for error handling.
		 * @returns {String}
		 */
		WebCenterToken.prototype.getErrorHandlingId = function () {
			return this._errorHandlingId;
		};

		/**
		 * returns the timer of the token
		 * @returns {Number} timer
		 */
		WebCenterToken.prototype.getTimer = function () {
			return this._timer;
		};

		/**
		 * returns the subtasks of this token
		 * @returns {Object} - subtasks have a name and an owner
		 */
		WebCenterToken.prototype.getSubtasks = function () {
			return this._subTasks;
		};

		/**
		 * adds a subtask to this token
		 * @param {String} id
		 * @param {String} owner
		 * @param {String} name
		 * @param {String} status
		 */
		WebCenterToken.prototype._addSubtask = function (id, owner, name, status) {
			var subTaskObject = {};
			this._subTasks[id] = subTaskObject;
			subTaskObject.owner = owner;
			subTaskObject.name = name;
			subTaskObject.status = status;
		};

		/**
		 * Returns the amount of subtasks in this token
		 * @returns {Number}
		 */
		WebCenterToken.prototype.getSubtaskCount = function () {
			return _.size(this._subTasks);
		};




		return WebCenterToken;
	});

define('WFEditorWCR/workflow/token/WebCenterOperationToken',[
		"WFEditorWCR/workflow/token/WebCenterToken",
		"underscore"
	],
	function (
		WebCenterToken,
		_
	) {
		"use strict";

		/**
		 * Represents an operation, such as Join or DataCollector used in the WebCenter workflow editor. 
		 * The data of the this token consists of different Tokens
		 * 
		 * @exports WebCenterOperationToken
		 * @class
		 * @augments WebCenterToken
		 *
		 * @param {Workflow} workflow - the workflow to which the token belongs
		 * @param {Pin} pin - pin to which the token belongs
		 * @param {String} id
		 * @param {XMLObject} data - xml object with the data for this token
		 * @param {String|null} cloneId - id of the token this token has been cloned from, null if not cloned
		 * @param {Boolean} isProcessed - has this token been processed yet?                             
		 */
		var WebCenterOperationToken = function (workflow, pin, id, data, cloneId, isProcessed) {
			WebCenterToken.call(this, workflow, pin, id, data, cloneId, isProcessed);

			/**
			 * @member {Map.<String,WebCenterToken} 
			 * @private
			 */
			this._subTokens = {};

			if (data) { //parse provided XML token data
				this._processXMLData(data);
			}
		};

		WebCenterOperationToken.prototype = Object.create(WebCenterToken.prototype);
		WebCenterOperationToken.prototype.constructor = WebCenterOperationToken;


		/**
		 * The files of this token, are the files of its subtokens
		 * @returns {Map.<String, String>}
		 */
		WebCenterOperationToken.prototype.getFiles = function () {
			var files = {};
			var i = 0;
			_.each(this._subTokens, function (token) {
				var tokenFiles = token.getFiles();
				_.each(tokenFiles, function(file, key){
					files[key + "_" + i] = file;
				});
				i++;
			});
			return files;
		};

		/**
		 * Get the number of files in this token.
		 * @returns {Number}
		 */
		WebCenterOperationToken.prototype.getFileCount = function () {
			return _.size(this.getFiles());
		};

		/**
		 * Get the number of subtokens in this operation token
		 * @returns {WebCenterToken}
		 */
		WebCenterOperationToken.prototype.getSubTokens = function () {
			return this._subTokens;
		};

		/**
		 * Adds a subtoken to this operation token
		 * @param {Token} token
		 */
		WebCenterOperationToken.prototype.addSubToken = function (token) {
			this._subTokens[token.getId()] = token;
		};

		/**
		 * returns true if token is included in this operation token
		 * @param   {Token} token
		 * @returns {Boolean}
		 */
		WebCenterOperationToken.prototype.hasSubToken = function (token) {
			return this._subTokens[token.getId()] !== undefined;
		};





		return WebCenterOperationToken;
	});

define('WFEditorWCR/workflow/pin/OutputPinTerminus',[
		"WFEditorWCR/workflow/pin/OutputPin",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants"

	],
	function (
		OutputPin,
		WebCenterPinConstants
	) {
		"use strict";

		/**
		 * OutputPinTerminus represents the model for an ouput pin in the WebCenter Workflow Editor.
		 * This pin is only shown when there are tokens in the terminus node.
		 *
		 * @exports OutputPinTerminus
		 * @class
		 * @augments OutputPin
		 *
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {Node} node - node to which the pin belongs
		 * @param {String} [id] - pin id
		 * @param {String} [label]
		 */
		var OutputPinTerminus = function OutputPinTerminus(workflow, node, ID, label) {
			if (!ID) {
				ID = WebCenterPinConstants.Type.Terminus;
			}

			OutputPin.call(this, workflow, node, ID, WebCenterPinConstants.Type.Terminus, label);
		};

		OutputPinTerminus.prototype = Object.create(OutputPin.prototype);
		OutputPinTerminus.prototype.constructor = OutputPinTerminus;


		return OutputPinTerminus;
	});

define('WFEditorWCR/workflow/node/errorHandling/ErrorHandlingNode',[
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants"
	],
	function (
		WebCenterNode,
		WebCenterNodeConstants,
		OutputPinOK,
		WebCenterPinConstants
	) {
		"use strict";

		/**
		 * ErrorHandling represents the model for the invisible errorHandling nodes.
		 *
		 * @exports ErrorHandlingNode
		 * @class
		 * @augments WebCenterNode
		 *
		 * @param {Wofkflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {String} pathToIcons - gives the url to the map where the icons can be found
		 * @param {String} label - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data                        
		 */
		var ErrorHandlingNode = function (workflow, id, pathToIcons, label, webCenterAPI) {
			var icon = WebCenterNodeConstants.Icons.ERRORHANDLING;
			WebCenterNode.call(this, workflow, id, WebCenterNodeConstants.Nodes.ERRORHANDLING, pathToIcons + icon, label, webCenterAPI);

			this._isVisible = false;
		};

		ErrorHandlingNode.prototype = Object.create(WebCenterNode.prototype);
		ErrorHandlingNode.prototype.constructor = ErrorHandlingNode;

		/**
		 * @Override
		 */
		ErrorHandlingNode.prototype._initializePins = function () {
			this._addInputPin();
			this.addPin(new OutputPinOK(this._workflow, this, WebCenterPinConstants.Type.OutOK));
		};

		return ErrorHandlingNode;
	});

define('WFEditor/nodeGallery/model/GalleryNode',[
	],
	function (
	) {
		"use strict";

		/**
		 * GalleryNode represents the model for a node in the NodeGallery.
		 * 
		 * @exports GalleryNode
		 * @class
		 * 
		 * @param {String} name     - the name of the node
		 * @param {String} iconUrl  - path to the icon
		 * @param {String} category - category to which the node belongs
		 */
		var GalleryNode = function (name, iconUrl, category) {

            /**
			 *  @member {String}
			 *  @private
			 */
            this._iconUrl = iconUrl;
            
			/**
			 *  @member {String}
			 *  @private
			 */
			this._category = category;
            
            /**
			 *  @member {String}
			 *  @private
			 */
            this._name = name;
			
			/**
			 * @member {Object}  these options can be used for construction of nodes in the factory
			 * @private
			 */
			this._options = {};


		};
        
        /**
         * Get the full url to the icon of the GalleryNode. 
         * @returns {String} iconUrl
         */
        GalleryNode.prototype.getIconUrl = function () {
            return this._iconUrl;
        };
    
        /**
         * Get the category to which the GalleryNode belongs.
         * @returns {String} category
         */
        GalleryNode.prototype.getCategory = function () {
            return this._category;
        };
    
        /**
         * Get the label that should be shown for the node in the gallery. This is the name if there is one specified, else it's the nodeType.
         * @returns {String}
         */
        GalleryNode.prototype.getLabel = function () {
            return this._name;
        };
	
		/**
		 * sets options, these options can be used for construction of nodes in the factory
		 * @param {Object} options
		 */
		GalleryNode.prototype.setOptions = function (options) {
			this._options = options;
		};
	
		/**
		 * gets the construction options
		 * @returns {Object}
		 */
		GalleryNode.prototype.getOptions = function () {
			return this._options;
		};
        
		return GalleryNode;
	});

define('WFEditorWCR/nodeGallery/WebCenterNodeGalleryConstants',{
	ListSelections: {
		ALL: 'all',
		AUTOTASK: 'wcautoactions',
		STATUS: 'taskstatuses',
        TASKTYPE: 'wctasks',
		WFCONSTRUCT: 'wfconstructs'
	},
    FilterIcons: {
        SEARCH: 'icon-WCR_Search',
        CLEAR: 'icon-WCR_Reject'
    },
    WorkflowConstructNodeTypes : [
        'DATACOLLECTOR', 
        'DATASPLITTER', 
        'ERRORHANDLER', 
        'TOKENSELECTOR', 
        'TOKENMARKER', 
        'JOIN', 
        'FORK', 
        'ROUTER', 
        'CANCEL']
});

define('WFEditorWCR/workflow/WebCenterWorkflowFactory',[
		"WFEditor/workflow/AbstractWorkflowFactory",
		"utils/webi18n!WFEditorWCR",
		"utils/Cache",
		"utils/ShapeUtils",
		"utils/ExecutionQueue",
		"utils/HTTPUtils",
		"WFEditorWCR/workflow/WebCenterWorkflow",
		"WFEditorWCR/workflow/node/WebCenterNode",
		"WFEditorWCR/workflow/node/MultiAttributeWebCenterNode",
		"WFEditorWCR/workflow/node/dataCollector/DataCollectorNode",
		"WFEditorWCR/workflow/node/dataSplitter/DataSplitterNode",
		"WFEditorWCR/workflow/node/errorHandler/ErrorHandlerNode",
		"WFEditorWCR/workflow/node/fork/ForkNode",
		"WFEditorWCR/workflow/node/join/JoinNode",
		"WFEditorWCR/workflow/node/tokenMarker/TokenMarkerNode",
		"WFEditorWCR/workflow/node/tokenSelector/TokenSelectorNode",
		"WFEditorWCR/workflow/node/endState/EndStateNode",
		"WFEditorWCR/workflow/node/startNode/StartNode",
		"WFEditorWCR/workflow/node/task/TaskNode",
		"WFEditorWCR/workflow/node/router/RouterNode",
		"WFEditorWCR/workflow/node/submitWorkflow/SubmitWorkflowNode",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/edge/WebCenterEdge",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/pin/OutputPinOK",
		"WFEditorWCR/workflow/token/WebCenterToken",
		"WFEditorWCR/workflow/token/WebCenterOperationToken",
		"WFEditorWCR/workflow/pin/OutputPinTerminus",
		"WFEditorWCR/workflow/pin/OutputPinError",
		"WFEditorWCR/workflow/WebCenterWorkflowState",
		"WFEditorWCR/workflow/node/errorHandling/ErrorHandlingNode",
		"WFEditorWCR/core/WebCenterWFEditorConstants",
		"WFEditor/nodeGallery/model/GalleryNode",
		"WFEditorWCR/nodeGallery/WebCenterNodeGalleryConstants",
		"webcenterapi/data/WebCenterTaskType",
		"underscore",
		"jquery"
	],
	function (
		AbstractWorkflowFactory,
		i18n,
		Cache,
		ShapeUtils,
		ExecutionQueue,
		HTTPUtils,
		WebCenterWorkflow,
		WebCenterNode,
		MultiAttributeWebCenterNode,
		DataCollectorNode,
		DataSplitterNode,
		ErrorHandlerNode,
		ForkNode,
		JoinNode,
		TokenMarkerNode,
		TokenSelectorNode,
		EndStateNode,
		StartNode,
		TaskNode,
		RouterNode,
		SubmitWorkflowNode,
		WebCenterNodeConstants,
		WebCenterEdge,
		WebCenterPinConstants,
		OutputPinOK,
		WebCenterToken,
		WebCenterOperationToken,
		OutputPinTerminus,
		OutputPinError,
		WebCenterWorkflowState,
		ErrorHandlingNode,
		WebCenterWFEditorConstants,
		GalleryNode,
		WebCenterNodeGalleryConstants,
		WebCenterTaskType,
		_,
		$
	) {
		"use strict";

		/**
		 * Factory to construct the entire workflow model for the workflow editor. This factory
		 * is specific for WebCenter and fetches the correct information with calls to WebCenter.
		 *
		 * @exports WebCenterWorkflowFactory
		 * @class
		 * @augments AbstractWorkflowFactor
		 * @param {WebCenterAPI} webCenterAPI - all communication with WebCenter
		 */
		var WebCenterWorkflowFactory = function (webCenterAPI, errorHandler) {
			AbstractWorkflowFactory.call(this);

			/**
			 * @member {WebCenterAPI}
			 * @private
			 */
			this._webCenterAPI = webCenterAPI;


			/**
			 * @member {String} config object with information of webcenter, loaction of the images, jsppages,...
			 * @private
			 */
			this._wcrConfig = webCenterAPI.getWebCenterConfig();


			/**
			 * @member {Object} a map containing information on TaskExecutors
			 * @private
			 */
			this._taskExecutorData = null;

			this._errorHandler = errorHandler;

			/**
			 * @member {Map.<String, Function>} maps the node type id on the constructor of specific node model
			 * @private
			 */
			this._constructNodesMap = {};
			this._constructNodesMap[WebCenterNodeConstants.Nodes.DATACOLLECTOR] = DataCollectorNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.DATASPLITTER] = DataSplitterNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.ERRORHANDLER] = ErrorHandlerNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.TOKENSELECTOR] = TokenSelectorNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.TOKENMARKER] = TokenMarkerNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.WCTASK] = TaskNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.ROUTER] = RouterNode;
			this._constructNodesMap[WebCenterNodeConstants.Nodes.SUBMITWORKFLOW] = SubmitWorkflowNode;

			/**
			 * @member {Cache} - the cache for taskType information, currently only used for outputs
			 * @private
			 */
			this._taskTypeDefinitionsCache = new Cache();
			this._taskTypeDefinitionsCache._loadItem = _.partial(function (wcrWFFactory, taskTypeId) {

				var cacheItem = {};
				cacheItem.outputs = [];

				wcrWFFactory._webCenterAPI.getTaskTypeDefinition(taskTypeId)
					.done(_.bind(function (xml) {
						var taskType = xml.querySelector("wctask_type_view[id='" + taskTypeId + "'] wctask_type");
						if (taskType) {
							cacheItem.taskType = WebCenterTaskType.fromXML(taskType);

							var statusViews = xml.querySelectorAll("wctask_type_view[id='" + taskTypeId + "'] status_list task_type_status_view");
							var completedStatusNumber = wcrWFFactory.getCompletedStatusNumber(statusViews);
							_.each(statusViews, function (statusView) {
								var status = statusView.querySelector("wc_status");
								var displayOrder = parseInt(status.getElementsByTagName("display_order")[0].textContent);
								if (displayOrder >= completedStatusNumber) {
									var outputId = status.getAttribute("id");
									var outputStatusName = status.getElementsByTagName("status_name")[0].textContent;
									cacheItem.outputs.push({
										id: outputId,
										name: outputStatusName
									});
								}
							});
							this._registerItem(taskTypeId, cacheItem);
						} else {
							this._registerItem(taskTypeId);
						}
					}, this)).fail(_.bind(function () {
						this._registerItem(taskTypeId);
					}, this));

			}, this);

			// synchronous cache for quickly finding icons for tasktypes (only search them once)
			this._taskIconFinder = new Cache(50, false);
			this._taskIconFinder._loadItem = _.partial(function (wcrWfFactory, name) {
				var icon = wcrWfFactory._tasktypeDefinitionXML.querySelector("wctask_type_view[id='" + name + "'] wctask_type icon");

				if (icon) {
					this._registerItem(name, icon.textContent);
				}
			}, this);

			// synchronous cache for quickly finding labels for tasktypes (only search them once)
			this._taskNameFinder = new Cache(50, false);
			this._taskNameFinder._loadItem = _.partial(function (wcrWfFactory, id) {
				var name = wcrWfFactory._tasktypeDefinitionXML.querySelector("wctask_type_view[id='" + id + "'] wctask_type task_type_name");

				if (name) {
					this._registerItem(id, name.textContent);
				}
			}, this);
		};

		WebCenterWorkflowFactory.prototype = Object.create(AbstractWorkflowFactory.prototype);
		WebCenterWorkflowFactory.prototype.constructor = WebCenterWorkflowFactory;

		/**
		 * Creates the workflow from two xml files.
		 * @param {String} taskId - id of the task or task to be loaded.
		 * @param {String} taskTypeId - id of the task or task type to be loaded.
		 * @param {Map<String, Any>} privileges - The privileges for the workflows
		 * @param   {String} workflow - the description of the workflow in xml-format
		 * @param   {String} workflowDefinition - the definition of all the nodes the webcenter workflow can have.
		 * @param 	{String} taskTypeDefinitions - the definition of all the task types in the system
		 * @param 	{String} currentTaskTypeDefinition - the definition of current WF task type with more details (verbose)                                     
		 * @ param {String} [initialTaskStatusID - the initial TaskStatus ID of the workflow
		 * @returns {Workflow} Returns the model of the enitre workflow.
		 */
		WebCenterWorkflowFactory.prototype.createWorkflow = function (taskTypeId, taskId, privileges, workflowXml, workflowDefinition, taskTypeDefinitions, currentTaskTypeDefinition, initialTaskStatusID) {
			// reset task executor data
			this._taskExecutorData = null;

			this._taskTypeId = taskTypeId;
			var workflowID = taskTypeId;
			if (taskId) {
				workflowID += "_" + taskId;
			}

			this._workflowDefinitionXML = workflowDefinition;
			this._tasktypeDefinitionXML = taskTypeDefinitions;
			this._currentTasktypeDefinitionXML = currentTaskTypeDefinition;

			var workflow = new WebCenterWorkflow(workflowID, taskTypeId, taskId, initialTaskStatusID);
			workflow.setPrivileges(privileges);
			workflow.setWebCenterConfig(this._wcrConfig);

			//the location to find most icons (not for specific task types)
			var iconLocation = this._wcrConfig.requireBaseUrl + "WFEditorWCR/icons/";

			var resources;
			var layout;
			var stickyNotes;
			var icon;
			var newNode;
			var nodeID;
			var transitionsMap = {};

			// split up work in smalle chunks
			var queue = new ExecutionQueue();

			try {
				var rootNode = workflowXml.documentElement;

				// run over all child nodes
				queue.forEach(rootNode.childNodes, function (workflowEl, i) {
					newNode = null;
					// run through all children of the XML document, ignore text elements

					if (workflowEl.nodeType !== 1 /*Node.ELEMENT_NODE*/ ) {
						return;
					}

					nodeID = workflowEl.getAttribute("name");
					switch (workflowEl.tagName.toLowerCase()) {
					case 'node':
						// find the type of the node
						var actionEls = workflowEl.getElementsByTagName("action");
						if (actionEls.length >= 1) {
							var actionEl = actionEls[0];
							var nodeType = actionEl.getAttribute("class");
							var enabled = true;

							var taskExecutorData = this._getTaskExecutorData(nodeType, workflowDefinition);
							if (this._constructNodesMap[nodeType]) {
								// this Node has a specific implementation in WFEditorWCR

								icon = iconLocation;
								if (nodeType == WebCenterNodeConstants.Nodes.WCTASK) {
									// for WebCenter tasks we need to find the correct icon (using the taskIconFinder)
									var taskTypeEl = actionEl.getElementsByTagName("wctask_type");
									if (taskTypeEl[0]) {
										var taskTypeId = taskTypeEl[0].getAttribute("id");
										var taskTypeDef = taskTypeDefinitions.querySelector("wctask_type_view[id='" + taskTypeId + "'] wctask_type[id='" + taskTypeId + "']");

										var taskType = null;

										if (taskTypeDef) {
											taskType = WebCenterTaskType.fromXML(taskTypeDef);
										} else {
											taskType = new WebCenterTaskType(taskTypeId);
											enabled = false;
										}

										icon = this._wcrConfig.jspBaseUrl + taskType.getIcon();
										newNode = new this._constructNodesMap[nodeType](workflow, nodeID, icon, "", taskType, this._webCenterAPI);
									}
								} else {
									// Check if there is an icon for the node
									if (taskExecutorData && taskExecutorData.icon) {
										icon = iconLocation + taskExecutorData.icon;
									}
									// construct the node
									newNode = new this._constructNodesMap[nodeType](workflow, nodeID, icon, "", this._webCenterAPI);
								}
							} else {
								// no specific implementation exists for the Node (it is an ""AutoTask"")

								if (taskExecutorData) {
									if (taskExecutorData.icon) {
										icon = iconLocation + taskExecutorData.icon;
									} else {
										icon = undefined;
									}
									enabled = taskExecutorData.enabled;
								} else {
									icon = undefined;
									enabled = false;
								}

								// construct a WebCenter node with the correct type and icon
								newNode = this._getNodeOfCorrectType(workflow, nodeID, nodeType, icon, "", this._webCenterAPI);

							}
							// set it enabled or not
							newNode.setEnabled(enabled);
							//get all edges leaving the pins of this node
							this._getTransitions(workflowEl, nodeID, transitionsMap);

						}
						break;

					case WebCenterNodeConstants.Nodes.FORK:
						newNode = new ForkNode(workflow, nodeID, iconLocation, "", this._webCenterAPI);
						//get all edges leaving the pins of this node
						this._getTransitions(workflowEl, nodeID, transitionsMap);
						break;
					case WebCenterNodeConstants.Nodes.JOIN:
						newNode = new JoinNode(workflow, nodeID, iconLocation, "", this._webCenterAPI);
						//get all edges leaving the pins of this node
						this._getTransitions(workflowEl, nodeID, transitionsMap);
						break;
					case 'end-state':

						var type = workflowEl.getAttribute("type");
						if (!type) { //type is not defined in the XML, this means it is a completed end-state
							type = WebCenterNodeConstants.Nodes.COMPLETED;
						}

						newNode = new EndStateNode(workflow, nodeID, iconLocation, i18n.t(type), type, this._webCenterAPI);

						break;
					case 'start-state':
						newNode = new StartNode(workflow, nodeID, iconLocation, "", this._webCenterAPI);
						//get all edges leaving the pins of this node
						this._getTransitions(workflowEl, nodeID, transitionsMap);
						break;
					case 'resources':
						// store the resources XML for later usage
						resources = workflowEl;
						break;
					case 'layout':
						// store the layout XML for later usage
						layout = workflowEl;
						break;
					case 'extra':
						// check for sticky notes
						stickyNotes = workflowEl.getElementsByTagName("stickynotes")[0];
						break;

					}

					if (newNode) {
						// let the node process its XML and add it to the workflow
						newNode.setNodeXML(workflowEl);
						workflow.addNode(newNode);
					}



				}, this, "WebCenterWorkflowFactoryAnalyzeNodes", i18n.t("initializingNodes"));

				queue.run(function () {
					// process resource information
					if (resources) {
						queue.forEach(resources.childNodes, function (xmlElement) {
							this._handleResourceElement(xmlElement, workflow);
						}, this, "WebCenterWorkflowFactoryAnalyzeResources", i18n.t("initializingNodes"));
					}


					// process layout information (node locations and labels)
					if (layout) {
						queue.forEach(layout.childNodes, function (xmlElement) {
							this._handleLayoutElement(xmlElement, workflow);
						}, this, "WebCenterWorkflowFactoryAnalyzeLayout", i18n.t("initializingLayout"));
					}

					// Create sticky notes
					if (stickyNotes) {
						queue.forEach(stickyNotes.childNodes, function (xmlElement) {
							if (xmlElement.tagName === "stickynote") {
								this._handleStickyNoteElement(xmlElement, workflow);
							}
						}, this, "WebCenterWorkflowFactoryCreateStickyNotes");
					}


					queue.forEach(transitionsMap, function (transitionObj, from) {
						var fromNode = workflow.getNode(transitionObj.nodeID);
						var toNode = workflow.getNode(transitionObj.to);
						if (fromNode && toNode) {
							var fromPin = fromNode.getPin(_.first(from.split("/")));
							var toPin = toNode.getPin(toNode.getId() + WebCenterPinConstants.Type.In);
							if (fromPin && toPin) {
								var edge = this.createEdge(workflow, from, fromPin, toPin);
								fromPin.addEdge(edge);
								toPin.addEdge(edge);
								if (!fromPin.getEdgeTo(toPin)) {
									// the edge hasn't been added because the to or from pin doesn't accept the edge
									// This is a legacy issue, add it anyway. 
									fromPin.getEdges().push(edge);
									toPin.getEdges().push(edge);
								}
								workflow.addEdge(edge);
								if (toNode.isImplicit && toNode.isImplicit()) {
									// make the edge implicit
									edge.setImplicit(true);
								}

							}
						}
					}, this, "WebCenterWorkflowFactoryInitializeEdges", i18n.t("initializingConnections"));


					queue.run(function () {
						// If the workflow model doesn't have any nodes, create the default workflow items
						if (_.size(workflow.getNodes()) === 0) {

							var startNode = new StartNode(workflow, "InputCondition_1", iconLocation, null, this._webCenterAPI);
							var endNode = new EndStateNode(workflow, "00001_0000000003_3", iconLocation, i18n.t("Completed"), WebCenterNodeConstants.Nodes.COMPLETED, this._webCenterAPI);
							var errorImplicit = new EndStateNode(workflow, "errorcontinue_implicit", iconLocation, "", WebCenterNodeConstants.Nodes.ERRORCONTINUE, this._webCenterAPI);
							// Set locations of the nodes
							startNode.setLocation(new ShapeUtils.Point(WebCenterNodeConstants.DefaultStartNode.DefaultX, WebCenterNodeConstants.DefaultStartNode.DefaultY));
							endNode.setLocation(new ShapeUtils.Point(WebCenterNodeConstants.DefaultCompletedNode.DefaultX, WebCenterNodeConstants.DefaultCompletedNode.DefaultY));
							errorImplicit.setImplicit(true);

							workflow.addNode(errorImplicit);
							workflow.addNode(startNode);
							workflow.addNode(endNode);
						}
					}, this);
				}, this);

				var deferred = $.Deferred();

				// wait until operations are finished and call callback
				queue.whenFinished(function () {
					deferred.resolve(workflow);
				});

				return deferred.promise();
			} catch (e) {
				this._errorHandler.severe(i18n.t("CreateWorkflowFailed"), e, workflowXml, workflowDefinition, taskTypeDefinitions);

				var deferred = $.Deferred();
				deferred.reject(e);

				return deferred.promise();
			}

		};


		/**
		 * Get the data from the WorkflowDefinition XML for a specific executorType
		 * @param {String} executorType - type of the executer
		 * @param {Document} workflowDefinition - XML document with the workflowDefinitions
		 * @returns {Object} object containing task executor information (limited to icon and enabled for now) (or undefined if not available)
		 */
		WebCenterWorkflowFactory.prototype._getTaskExecutorData = function (executorType, workflowDefinition) {
			if (!this._taskExecutorData) {
				// if the taskExecuterData is not yet loaded, pars it for all TaskExecuters
				this._taskExecutorData = {};

				var taskExecuters = workflowDefinition.getElementsByTagName("TaskExecuterFactory");
				for (var j = 0; j < taskExecuters.length; j++) {
					var nameEl = taskExecuters[j].getElementsByTagName("type")[0];
					var icon = taskExecuters[j].querySelector("config-inline icon");

					var enabled = true;
					var enabledEl = taskExecuters[j].querySelector("config-inline enabled");
					if (enabledEl) {
						enabled = enabledEl.textContent == "1";
					}

					if (icon) {
						icon = _.last(icon.textContent.split("/"));
					}
					this._taskExecutorData[nameEl.textContent] = {
						icon: icon,
						enabled: enabled
					};
				}

			}

			return this._taskExecutorData[executorType];
		};

		/**
		 * Handle the Layout XML (set node locations and labels)
		 * @param {Element} wfElement - XML element with the layout information for a single workflowitem
		 * @param {Workflow} workflow - the workflow for which the layout is being handled
		 */
		WebCenterWorkflowFactory.prototype._handleLayoutElement = function (wfElement, workflow) {
			// ignore text nodes and only process wfelement tags
			if (wfElement.nodeType === 1 && wfElement.tagName.toLowerCase() == "wfelement") {
				var ID = wfElement.getAttribute("name");
				var node = workflow.getNode(ID);

				if (node) {
					// the node exists, it should handle its layoutXML
					node.setLayoutXML(wfElement);
				}
			}
		};


		/**
		 * Handle the Resources XML
		 * @param {Element} resources - XML element with the resource information for a single workflowitem
		 * @param {Workflow} workflow - the workflow for which the resources are being handled
		 */
		WebCenterWorkflowFactory.prototype._handleResourceElement = function (resource, workflow) {

			// ignore text node and only process resource elements
			if (resource.nodeType === 1 && resource.tagName.toLowerCase() == "resource") {
				var ID = resource.getAttribute("id");
				var node = workflow.getNode(ID);

				if (node) {
					// the node exists, it should handle its resource
					node.setResourceXML(resource);
				}
			}

		};

		/**
		 * Handle the Resources XML
		 * @param {Element} resources - XML element with the information for a single stickynote
		 * @param {Workflow} workflow
		 */
		WebCenterWorkflowFactory.prototype._handleStickyNoteElement = function (stickyNote, workflow) {
			requirejs(["WFEditorWCR/workflow/helper/stickyNote/StickyNoteHelper"], function (StickyNoteHelper) {
				workflow.addHelper(new StickyNoteHelper(workflow,
					new ShapeUtils.Point(parseInt(stickyNote.getAttribute("preflocx")), parseInt(stickyNote.getAttribute("preflocy"))),
					new ShapeUtils.Size(parseInt(stickyNote.getAttribute("width")), parseInt(stickyNote.getAttribute("height"))),
					stickyNote.textContent));
			});
		};

		/**
		 * Get all the transition out of the xml-object, workflowEl is the xml-object which represents a node.
		 * It looks for all the 'transition'-child nodes.
		 * @param {DOMObject} workflowEl
		 * @param {String} nodeID
		 * @param {Map.<String, Object>} transitionsMap - Map with all the transition, the method adds newly found transition to this collection.
		 */
		WebCenterWorkflowFactory.prototype._getTransitions = function (workflowEl, nodeID, transitionsMap) {
			var transitions = workflowEl.getElementsByTagName("transition");
			_.each(transitions, function (transition) {
				var from = transition.getAttribute("name");
				var to = transition.getAttribute("to");
				if (to !== WebCenterNodeConstants.Nodes.ERRORCONTINUEIMPLICIT) {
					// Don't add edges to errorcontinue_implicit, these will be added on save
					transitionsMap[from] = {
						'nodeID': nodeID,
						'to': to
					};
				}
			}, this);
		};

		/**
		 * Create workflow feedback, creates tokens and adds those tokens to the correct pins.
		 * @param {XMLObject} workflowReportEl
		 * @param {XMLObject} currentTaskTypeDefinitionEl
		 * @param {Workflow} workflow
		 */
		WebCenterWorkflowFactory.prototype.createWorkflowFeedback = function (workflowReportEl, currentTaskTypeDefinitionEl, workflow) {
			// Determine the workflow state
			var audit = workflowReportEl.getElementsByTagName("audit");
			var taskStatus = null;
			if (audit && audit.length > 0) {
				var wfFinished = audit.item(0).getAttribute("finished");

				var wf = audit.item(0).getElementsByTagName("wf");
				if (wf && wf.length > 0) {
					taskStatus = wf.item(0).getAttribute("taskstatus");
				}

				if (wfFinished == "1") { // == intentional
					workflow.setWorkflowState(WebCenterWorkflowState.FINISHED);
				} else if (wf && wf.length > 0 && workflow.getWorkflowState() !== WebCenterWorkflowState.STOPPING) {
					var paused = wf.item(0).getAttribute("paused");

					if (paused == "1") { // == intentional
						workflow.setWorkflowState(WebCenterWorkflowState.PAUSED);
					} else {
						workflow.setWorkflowState(WebCenterWorkflowState.RUNNING);
					}
				}

			} else {
				// there was no audit trail.. check if there was an sdk error
				var errorTag = _.first(workflowReportEl.getElementsByTagName("error"));
				if (errorTag) {
					var errorCode = _.first(errorTag.getElementsByTagName("code"));
					if (errorCode && errorCode.textContent == "154") {
						// no workflow report available, fall back on the initial task status id of the workflow
						taskStatus = workflow.getInitialTaskStatusID();
					}
				}
			}

			// reset all parent tokens
			_.invoke(workflow.getTokens(), "removeParentTokens");

			var tokens = workflowReportEl.getElementsByTagName("tokens"); //get the tokens-element
			var tokenMap = {}; //map to keep track of all created tokens, token ID mapped on Token object
			//maps parent token ID on the token that still needs a parent token
			var waitingForParent = {}; //map to keep track of all tokens that still need one of the parent tokens to be added

			if (tokens && tokens[0] && tokens[0].childNodes.length !== 0) {
				// create the init Token
				var initToken = new WebCenterToken(workflow, undefined, "init", null, null, true);
				tokenMap["init"] = initToken;
			}


			// a map to keep track of all processed states of the tokens, used to call setProcessed for each token after parents were added
			var processedMap = {};

			var errorHandlingWaitingForParent = {};
			try {
				//run over all the children of the tokens element, all place elements = each pin element with tokens
				for (var i = 0; tokens && tokens[0] && i < tokens[0].childNodes.length; i++) {
					// run through all children of the XML document, ignore text elements
					var placeEl = tokens[0].childNodes[i];
					if (placeEl.nodeType !== 1 /*Node.ELEMENT_NODE*/ ) {
						continue; //make sure to only handle element nodes
					}

					var nodeId = placeEl.getAttribute("node"); //ID of the node the pin belongs to
					var pinId = placeEl.getAttribute("channel"); //ID of the pin the token belongs to
					//Lets check if the token belongs to an error handling node
					var parentNodeId = nodeId.split("_");
					if (parentNodeId.pop() === "errorhandling") { //last part of the node id is equal to errorhandling
						var errorHandlingNode = workflow.getNode(nodeId); //get the error handling node
						var parentNode = workflow.getNode(parentNodeId.join("_"));
						if (!errorHandlingNode && parentNode) { //if the node is not yet present in the workflow, construct it, this way it only gets constructed when used
							errorHandlingNode = new ErrorHandlingNode(workflow, nodeId, this._wcrConfig.requireBaseUrl + "WFEditorWCR/icons/", null, this._webCenterAPI);
							workflow.addNode(errorHandlingNode);
							//now construct the transitions between the node and the error handling node
							//first from the node to the error handling node
							var fromPin = parentNode.getErrorPin();
							var toPin = errorHandlingNode.getInputPin();
							var edge = this.createEdge(workflow, fromPin.getId() + "_" + toPin.getId(), fromPin, toPin);
							fromPin.addEdge(edge);
							toPin.addEdge(edge);
							//now from the error handling node back to the main node
							fromPin = errorHandlingNode.getPin(errorHandlingNode.getId() + WebCenterPinConstants.Type.OutOK);
							toPin = parentNode.getInputPin();
							edge = this.createEdge(workflow, fromPin.getId() + "_" + toPin.getId(), fromPin, toPin);
							fromPin.addEdge(edge);
							toPin.addEdge(edge);
						}
					}
					var node = workflow.getNode(nodeId); //get the node from the workflow
					if (node) { // if the node can be found back
						var endPin = null;
						var pin = node.getPin(nodeId + pinId); //get the correct pin from the node
						//if it's an input pin or tokens that are generated internally add them to the input pin
						if (pinId === WebCenterPinConstants.FeedbackTypes.Input || pinId === WebCenterPinConstants.FeedbackTypes.Internal) {
							pin = node.getInputPin();
						} else if (pinId === WebCenterPinConstants.FeedbackTypes.Terminus) {
							//only when the end-state node has tokens in its 'output' pin add this pin to the node
							pin = node.getPin(nodeId + WebCenterPinConstants.Type.Terminus);
							if (!pin) {
								if (node.getNodeType() === WebCenterNodeConstants.Nodes.CANCEL) {
									endPin = new OutputPinError(workflow, node, WebCenterPinConstants.Type.Terminus, node.getLabel());
								} else {
									endPin = new OutputPinTerminus(workflow, node, WebCenterPinConstants.Type.Terminus, node.getLabel());
								}
								node.addPin(endPin);
								pin = node.getInputPin();
							}
						}

						// If the pin is actually a warning pin that isn't displayed, the token should be shown at the Error or OK pin
						if (!pin && node.canEditWarningPolicy() && _.contains(node.getWarningLabels(), pinId)) {

							if (node.getWarningPolicyString() === WebCenterNodeConstants.WarningPolicy.OK) {
								// Warning Policy === OK so show the token at the OK pin
								pin = _.find(node.getPins(), function (pin) {
									return pin instanceof OutputPinOK;
								});
							} else if (node.getWarningPolicyString() === WebCenterNodeConstants.WarningPolicy.ERROR) {
								// Warning Policy === ERROR so show the token at the ERROR pin
								pin = node.getErrorPin();
							}

						}

						if (pin) { //there is a pin found to add the token to
							_.each(placeEl.childNodes, function (tokenEL) { //run over all the tokens that are part of the pin
								if (!tokenEL || tokenEL.nodeType !== 1 /*Node.ELEMENT_NODE*/ ) {
									return;
								}
								var tokenId = tokenEL.getAttribute("id");
								var imported = tokenEL.getAttribute("import");

								// Has the token been processed yet?
								var processed = tokenEL.getAttribute("processed") !== "0" || wfFinished == "1"; // == intentional
								if (imported && imported === "1") {
									return;
								}
								//there can be multiple parents for one token (e.g. fork/join nodes)
								var parentIds = [tokenEL.getAttribute("parentid")];
								if (pinId === WebCenterPinConstants.FeedbackTypes.Internal) {
									//there are multiple parent possible for internally generated tokens
									parentIds = [];
									var parentsEL = tokenEL.getElementsByTagName("parents");
									_.each(parentsEL[0].childNodes, function (parentEL) {
										if (!parentEL || parentEL.nodeType !== 1 /*Node.ELEMENT_NODE*/ ) {
											return;
										}
										parentIds.push(parentEL.getAttribute("id"));
									}, this);
								}
								//a clone ID is present when there are multiple edges leaving the pin, tokens get cloned for each edge
								var cloneId = null;
								if (tokenEL.hasAttribute("cloneid")) {
									cloneId = tokenEL.getAttribute("cloneid");
								}


								var tokenData = tokenEL.getElementsByTagName("tokendata"); //docuemnts and extra information of the token


								var token = workflow.getToken(tokenId) || _.find(pin.getTokens(), function (token) {
									return token.getId() === tokenId;
								});
								if (!token) {
									// token doesn't exist yet, create it... (parent nodes will be set later on)
									token = new WebCenterToken(workflow, pin, tokenId, tokenData[0], cloneId, processed);
								} else {
									// update the token with the new token data
									token.updateFromXMLData(tokenData[0]);
								}

								// If there is an operation ID, a dedicated "OperationToken" is created. 
								// This OperationToken will aggregate all tokens in the operation
								var opId;
								if (tokenEL.hasAttribute("opid")) {
									opId = tokenEL.getAttribute("opid"); // Get the id of the operation
									// check if the operation is in the workflow or in the token map
									var opToken = workflow.getToken(opId) ? workflow.getToken(opId) : tokenMap[opId];


									// An 'R' opstate attribute indicates that the operation is in process
									if (tokenEL.getAttribute("opstate") !== 'R') {
										processed = true;
									} else {
										processed = false;
									}

									if (!opToken) {
										// Not yet created, create a new one
										opToken = new WebCenterOperationToken(workflow, pin, opId, tokenData[0], cloneId, processed);
										node.addOperationToken(opToken);
									}


									// If this operation doesn't contain the token yet, add it
									if (!opToken.hasSubToken(token)) {
										opToken.addSubToken(token);
									}

									processedMap[opToken.getId()] = processed;
									tokenMap[opId] = opToken; // add it the operation to the token map
									pin.addToken(opToken);
									// Tokens in OperationTokens are aggregated by the operationtoken and should be considered processed
									processed = true;

								} else if ((node.getNodeType() === WebCenterNodeConstants.Nodes.DATACOLLECTOR ||
										node.getNodeType() === WebCenterNodeConstants.Nodes.JOIN) &&
									pin.getPinType() === WebCenterPinConstants.Type.In) {
									// The node is an operation node but there is no opid, this happens when all operations in the node have been finished
									// -> set all operation tokens in the nodes to processed and remove them from the operation node
									_.each(node.getOperationTokens(), function (opToken) {
										processedMap[opToken.getId()] = true;
										node.removeOperationToken(opToken);
									});
								}

								// If the processed status hasn't been defined by now, it can be seen as processed
								if (processed === undefined || processed === null) {
									processed = true;
								}
								pin.addToken(token);
								processedMap[token.getId()] = processed;
								if (pinId === WebCenterPinConstants.FeedbackTypes.Terminus && endPin) {
									endPin.addToken(token);
								}
								//keep track of the created tokens by adding them to the tokenMap
								tokenMap[tokenId] = token;
								//run over all the parent id and check if they are already created
								_.each(parentIds, function (parentId) {
									if (parentId) {
										var parentToken = tokenMap[parentId];
										if (parentToken) {
											token.addParent(parentToken);
											if (nodeId && nodeId.split("_").pop() === "errorhandling") { //last part of the node id is equal to errorhandling
												parentToken.setErrorHandlingId(tokenId);
											}
										} else { //parent is not yet created add it to the map to be added later on
											if (!waitingForParent[parentId]) {
												waitingForParent[parentId] = [];
											}
											waitingForParent[parentId].push(token);
											if (nodeId && nodeId.split("_").pop() === "errorhandling") {
												errorHandlingWaitingForParent[parentId] = tokenId;
											}
										}
									}
								}, this);
								//check if there are tokens that have the newly created token as parent
								var tokenArray = waitingForParent[tokenId];
								_.each(tokenArray, function (childToken) {
									childToken.addParent(token);
								}, this);
								//each token can only go to one error handling node, so no need for foreach loop
								var errorHandlingId = errorHandlingWaitingForParent[tokenId];
								if (errorHandlingId) { //add error handling id if present
									token.setErrorHandlingId(errorHandlingId);
								}
							}, this);
						}
					}

				}

				_.each(processedMap, function (isProcessed, tokenId) {
					var token = tokenMap[tokenId] || workflow.getToken(tokenId);
					token.setProcessed(isProcessed);
				});
				workflow.setTokens(tokenMap);

				//start parsing the currentTaskTypeDefinitionEl to get the workflow status
				var statusListEl = currentTaskTypeDefinitionEl.getElementsByTagName("status_list");
				if (statusListEl && statusListEl[0] && taskStatus !== null && workflow.getWorkflowState() !== WebCenterWorkflowState.STOPPING) {
					var statusInfo = $(statusListEl).find("wc_status[id='" + taskStatus + "']");

					if (workflow.getWorkflowState() === WebCenterWorkflowState.PAUSED) {
						workflow.setTaskStatus({
							status: i18n.t("workflowStatusOnHold"),
							icon: null
						});
					} else if (statusInfo && statusInfo[0]) {
						var statusName = statusInfo[0].getElementsByTagName("status_name");
						var iconName = statusInfo[0].getElementsByTagName("icon_name");

						if (statusName && iconName && statusName[0] && iconName[0]) {
							workflow.setTaskStatus({
								status: statusName[0].textContent,
								icon: iconName[0].textContent && this._wcrConfig.jspBaseUrl + "images/statuses/" + iconName[0].textContent
							});
						}
					} else if (taskStatus === WebCenterWFEditorConstants.WorkflowErrorStatus) {
						workflow.setTaskStatus({
							status: i18n.t("workflowStatusError"),
							icon: this._wcrConfig.jspBaseUrl + "images/statuses/" + WebCenterWFEditorConstants.WorkflowErrorImage
						});
					} else if (taskStatus === WebCenterWFEditorConstants.WorkflowCanceledStatus) {
						workflow.setTaskStatus({
							status: i18n.t("workflowStatusCanceled"),
							icon: this._wcrConfig.jspBaseUrl + "images/statuses/" + WebCenterWFEditorConstants.WorkflowCanceledImage
						});
					}

				}

			} catch (e) {
				this._errorHandler.severe(i18n.t("CreateWorkflowFeedbackFailed"), e, workflowReportEl);
			}
		};

		/**
		 * Returns the definitions of the workflow construct nodes that should be shown in the NodeGallery
		 * @param {String} pathToIcons - the url to the folder where the icons can be found
		 * @return {Array.<GalleryNode>}
		 */
		WebCenterWorkflowFactory.prototype._getWFConstructNodeDefintions = function (pathToIcons) {
			var statusNodes = [];
			var newNode;

			_.each(WebCenterNodeGalleryConstants.WorkflowConstructNodeTypes, function (type) {
				var icon = pathToIcons + WebCenterNodeConstants.Icons[type];
				var options = {
					nodeType: WebCenterNodeConstants.Nodes[type]
				};
				if (type === "ROUTER") {
					options.taskTypeId = "genPropertyFilter";
					var taskExecutorData = this._getTaskExecutorData("bsroute", this._workflowDefinitionXML);
					if (taskExecutorData && taskExecutorData.icon) {
						icon = pathToIcons + taskExecutorData.icon;
					}
				}
				newNode = new GalleryNode(WebCenterNodeConstants.Nodes[type], icon, WebCenterNodeGalleryConstants.ListSelections.WFCONSTRUCT);
				newNode.setOptions(options);

				statusNodes.push(newNode);
			}, this);
			return statusNodes;
		};

		/**
		 * Returns the display order of the completed status
		 * @param   {NodeList} statuses - XML NodeList of status elements
		 * @returns {Number}
		 */
		WebCenterWorkflowFactory.prototype.getCompletedStatusNumber = function (statuses) {
			var completedOrderNr = 0;
			for (var i = 0; i < statuses.length; i++) {
				var wcstatus = statuses[i].querySelector("wc_status");
				var name = wcstatus.querySelector("status_name").textContent;
				if (name === "Completed") {
					completedOrderNr = parseInt(wcstatus.querySelector("display_order").textContent);
					break;
				}
			}
			return completedOrderNr;
		};

		/**
		 * Returns the definitions of the status nodes that should be shown in the NodeGallery.
		 * @param {String} pathToIcons - the url to the folder where the icons can be found
		 * @param {NodeList} statuses - XML NodeList of status elements
		 * @return {Array.<GalleryNode>}
		 */
		WebCenterWorkflowFactory.prototype._getStatusNodeDefintions = function (pathToIcons, statuses) {
			var statusNodes = [];
			var newNode;
			var wcstatus;
			var name;

			//only show statuses that have a display order >= 'Completed', so first need to find that display order number
			var completedOrderNr = this.getCompletedStatusNumber(statuses);
			newNode = new GalleryNode(WebCenterNodeConstants.Nodes.COMPLETED, pathToIcons + WebCenterNodeConstants.Icons.COMPLETED, WebCenterNodeGalleryConstants.ListSelections.STATUS);
			newNode.setOptions({
				id: WebCenterNodeConstants.Status.COMPLETED,
				nodeType: WebCenterNodeConstants.Nodes.COMPLETED
			});
			statusNodes.push(newNode);

			//make GalleryNodes if the display order is > completed display order
			for (var j = 0; j < statuses.length; j++) {
				wcstatus = statuses[j].querySelector("wc_status");
				var orderNr = parseInt(wcstatus.querySelector("display_order").textContent);
				if (orderNr <= completedOrderNr) {
					continue;
				}

				name = wcstatus.querySelector("status_name").textContent;
				var id = wcstatus.getAttribute("id");
				newNode = new GalleryNode(name, pathToIcons + WebCenterNodeConstants.Icons.COMPLETED, WebCenterNodeGalleryConstants.ListSelections.STATUS);
				newNode.setOptions({
					nodeType: WebCenterNodeConstants.Nodes.COMPLETED,
					id: id,
					name: name
				});
				statusNodes.push(newNode);
			}

			return statusNodes;
		};

		/**
		 * Returns the definitions of all the nodes that should be shown in the NodeGallery.
		 * @Override
		 * @return {Array.<GalleryNode>}
		 */
		WebCenterWorkflowFactory.prototype.getNodeDefinitions = function () {
			var nodes = [];
			var iconLocation = this._wcrConfig.requireBaseUrl + "WFEditorWCR/icons/";
			nodes = nodes.concat(this._getWFConstructNodeDefintions(iconLocation));

			if (this._workflowDefinitionXML && this._tasktypeDefinitionXML) {
				var newNode;

				var taskExecuters = this._workflowDefinitionXML.getElementsByTagName("TaskExecuterFactory");
				for (var j = 0; j < taskExecuters.length; j++) {
					var nodeType = (taskExecuters[j].getElementsByTagName("type")[0]).textContent;
					var visibility = taskExecuters[j].querySelector("config-inline visible");
					var enabled = taskExecuters[j].querySelector("config-inline enabled");
					if (visibility.textContent !== "1" || enabled.textContent !== "1") {
						continue;
					}
					var icon = taskExecuters[j].querySelector("config-inline icon");
					var path = null;
					if (icon) {
						path = this._wcrConfig.jspBaseUrl + icon.textContent;
					}
					newNode = new GalleryNode(nodeType, path, WebCenterNodeGalleryConstants.ListSelections.AUTOTASK);
					newNode.setOptions({
						nodeType: nodeType
					});
					nodes.push(newNode);
				}

				var taskTypes = this._tasktypeDefinitionXML.getElementsByTagName("wctask_type_view");
				_.each(taskTypes, function (task) {
					var id = task.getAttribute("id");
					//if taskId is the same as the current loaded workflow-id, add the statuses in the gallery, but don't add it
					if (id !== this._taskTypeId) {
						var icon = task.querySelector("wctask_type icon");
						var path = null;
						var name = task.querySelector("wctask_type task_type_name").textContent;
						if (icon) {
							path = this._wcrConfig.jspBaseUrl + icon.textContent;
						}
						newNode = new GalleryNode(name, path, WebCenterNodeGalleryConstants.ListSelections.TASKTYPE);
						newNode.setOptions({
							nodeType: WebCenterNodeConstants.Nodes.WCTASK,
							taskTypeId: id
						});
						nodes.push(newNode);
					}
				}, this);

				var currentTaskType = this._currentTasktypeDefinitionXML.getElementsByTagName("wctask_type_view");
				//if taskId is the same as the current loaded workflow-id, add the statuses in the gallery, but don't add it
				if (currentTaskType && currentTaskType[0]) {
					var statuses = currentTaskType[0].querySelectorAll("status_list task_type_status_view");
					nodes = nodes.concat(this._getStatusNodeDefintions(iconLocation, statuses));
				}
			}
			return nodes;
		};

		/**
		 * @override
		 */
		WebCenterWorkflowFactory.prototype.createEdge = function (workflow, id, fromPin, toPin) {
			return new WebCenterEdge(workflow, id, fromPin, toPin);
		};

		/**
		 * @override
		 * Possible options are:
		 * nodeType: the type of the node to be created
		 * [id]: sub id of the nodeType. eg. if nodeType is "Completed" the id is the correct type of completed status
		 * [taskTypeId]: id of the task type.
		 * [name]: used to generate a new label, a suffix will be added if a node with the same label exists
		 * [label]: if label is present, this will be the label of the new node, even if another node with the same label exists
		 * [configurationXML]: used to set the parameters of the newly created node
		 */
		WebCenterWorkflowFactory.prototype.createNode = function (workflow, options) {
			var deferred = $.Deferred();
			var promise = deferred.promise();
			var nodeTypeId = options.taskTypeId || options.id || "";
			var nodeType = options.nodeType;

			if (options.configurationXML) {
				promise.done(function (node) {
					var xmlDoc = $.parseXML(options.configurationXML);
					var rootNode = xmlDoc.firstChild;
					node.setNodeXML(rootNode);
				});
			}

			if (options.resourceXML) {
				promise.done(function (node) {
					var resourceDoc = $.parseXML(options.resourceXML);
					var rootNode = resourceDoc.firstChild;
					node.setResourceXML(rootNode);

				});
			}

			var i18nLabel = options.label || this._generateNewLabel(workflow, i18n.t(options.name || nodeType));

			var newNode;
			var nodeID = this._generateId(nodeType, nodeTypeId, workflow);
			var iconLocation = this._wcrConfig.requireBaseUrl + "WFEditorWCR/icons/";
			var taskExecutorData = this._getTaskExecutorData(nodeType, this._workflowDefinitionXML);
			switch (nodeType) {
			case WebCenterNodeConstants.Nodes.FORK:
				newNode = new ForkNode(workflow, nodeID, iconLocation, null, this._webCenterAPI);
				break;
			case WebCenterNodeConstants.Nodes.JOIN:
				newNode = new JoinNode(workflow, nodeID, iconLocation, null, this._webCenterAPI);
				break;
			case WebCenterNodeConstants.Nodes.COMPLETED:
				newNode = new EndStateNode(workflow, nodeID, iconLocation, i18n.t(nodeType), nodeType, this._webCenterAPI);
				break;
			case "errorstop":
				newNode = new EndStateNode(workflow, nodeID, iconLocation, i18n.t(nodeType), nodeType, this._webCenterAPI);
				break;
			case "end-state":
				newNode = new EndStateNode(workflow, nodeID, iconLocation, i18n.t(nodeType), nodeType, this._webCenterAPI);
				break;
			case WebCenterNodeConstants.Nodes.WCTASK:

				var resolveCacheFunction = _.bind(function (cacheItem) {
					if (cacheItem) {
						newNode = new TaskNode(workflow, nodeID, iconLocation, "", cacheItem.taskType, this._webCenterAPI);
						if (!options.resourceXML) {
							_.each(cacheItem.outputs, function (output) {
								newNode.addPin(new OutputPinOK(workflow, newNode, output.id, output.name), 1);
							});
						}
						newNode.setLabel(i18nLabel);
					} else {
						var taskType = new WebCenterTaskType(nodeTypeId);
						var icon = this._wcrConfig.jspBaseUrl + taskType.getIcon();
						newNode = new TaskNode(workflow, nodeID, icon, "", taskType, this._webCenterAPI);
						newNode.setEnabled(false);
						newNode.setLabel(i18nLabel);
					}
					deferred.resolve(newNode);
				}, this);

				iconLocation = this._taskIconFinder.get(nodeTypeId);
				var taskTypeName = this._taskNameFinder.get(nodeTypeId);
				i18nLabel = options.label || this._generateNewLabel(workflow, taskTypeName);
				// construct the node
				this._taskTypeDefinitionsCache.get(nodeTypeId, resolveCacheFunction);

				break;
			}

			if (newNode) {
				newNode.setLabel(i18nLabel);
				deferred.resolve(newNode);

			} else {
				// Check if a icon exists for the node
				var iconUrl = iconLocation;
				if (taskExecutorData && taskExecutorData.icon) {
					iconUrl = iconLocation + taskExecutorData.icon;
				}

				if (this._constructNodesMap[nodeType] && nodeType !== WebCenterNodeConstants.Nodes.WCTASK && nodeType !== WebCenterNodeConstants.Nodes.ERRORHANDLER) {
					newNode = new this._constructNodesMap[nodeType](workflow, nodeID, iconUrl, null, this._webCenterAPI);
					newNode.setLabel(i18nLabel);
					deferred.resolve(newNode);
				} else if (nodeType === WebCenterNodeConstants.Nodes.ERRORHANDLER) {
					// There can only be one error handler
					if (_.filter(workflow.getNodes(), function (node) {
							return node.getNodeType() === WebCenterNodeConstants.Nodes.ERRORHANDLER;
						}).length === 0) {
						newNode = new this._constructNodesMap[nodeType](workflow, nodeID, iconUrl, null, this._webCenterAPI);
						newNode.setLabel(i18nLabel);
						deferred.resolve(newNode);
					} else {
						deferred.reject(i18n.t("wferror.errorhandleralreadyexists"));
					}
				} else if (nodeType !== WebCenterNodeConstants.Nodes.WCTASK) {
					// construct a WebCenter node with the correct type and icon
					newNode = this._getNodeOfCorrectType(workflow, nodeID, nodeType, iconUrl, null, this._webCenterAPI);

					newNode.setLabel(i18nLabel);
					deferred.resolve(newNode);

				}
			}

			return promise;
		};


		// static number used for unique id generation
		var highestId = 0;

		/**
		 * generates the ID for a new node
		 * @param   {nodeType} nodeType 
		 * @param 	{nodeTypeId}                             
		 * @returns {String}
		 */
		WebCenterWorkflowFactory.prototype._generateId = function (nodeType, nodeTypeId, workflow) {
			if (highestId === 0) {
				_.each(workflow.getNodes(), function (node) {
					var idPostfix = parseInt(node.getId().substring(node.getId().lastIndexOf("_") + 1));
					if (idPostfix > highestId) {
						highestId = idPostfix;
					}
				});
			}
			highestId++;
			if (nodeTypeId) {
				return nodeTypeId + "_" + highestId;
			} else {
				return nodeType + "_" + highestId;
			}
		};

		/**
		 * Generates a label <prefix> - <counter> where counter is a unique number
		 * @param   {Workflow} workflow 
		 * @param   {String} prefix
		 * @returns {String}
		 */
		WebCenterWorkflowFactory.prototype._generateNewLabel = function (workflow, prefix) {
			var nodes = _.filter(workflow.getNodes(), function (node) {
				return node.getLabel().indexOf(prefix) === 0;
			});
			var counter = this._getNextPostfixCounter(nodes, prefix);
			return prefix + (counter === 1 ? "" : " - " + counter);

		};

		/**
		 * returns a unqiue postfix number
		 * @param   {Array.<Node>} nodes - the postfix should not be present in the labels of these nodes
		 * @param   {String} prefix 
		 * @returns {Number} 
		 */
		WebCenterWorkflowFactory.prototype._getNextPostfixCounter = function (nodes, prefix) {
			var postFixes = [];
			_.each(nodes, function (node) {
				var label = node.getLabel();
				var postFix = label.substring(prefix.length);
				if (postFix !== "" && (postFix.lastIndexOf(" - ") !== 0 || isNaN(parseInt(postFix.substring(3))))) {
					// Not a corrct node after all
					return;
				}
				var labelPostfix = label.substring(label.lastIndexOf(" - ") + 3);
				if (isNaN(parseInt(labelPostfix))) {
					labelPostfix = '1';
				}
				postFixes[labelPostfix] = true;
			});
			for (var i = 1; i < postFixes.length; i++) {
				if (!postFixes[i]) {
					return i;
				}
			}
			return i;
		};


		/**
		 * get correct node type
		 * @param {String} nodeType
		 * @param {Workflow} workflow - the workflow to which the node belongs
		 * @param {String} id - node id
		 * @param {ShapeUtils.Point} location - location of the node
		 * @param {String} nodeType - specifies the type of the node
		 * @param {String} [icon] - gives the url to the icon for this node
		 * @param {String} [label] - the label of the node
		 * @param {WebCenterAPI} webCenterAPI - used to access webcenter data  
		 * @returns {Node} taskTypeId                             
		 */
		WebCenterWorkflowFactory.prototype._getNodeOfCorrectType = function (workflow, id, nodeType, icon, label, webCenterAPI) {
			switch (nodeType) {
			case WebCenterNodeConstants.Nodes.SETTASKSPECIFICATION:
			case WebCenterNodeConstants.Nodes.SETPROJECTATTRIBUTE:
			case WebCenterNodeConstants.Nodes.SETDOCUMENTATTRIBUTE:
				return new MultiAttributeWebCenterNode(workflow, id, nodeType, icon, label, webCenterAPI);
			default:
				return new WebCenterNode(workflow, id, nodeType, icon, label, webCenterAPI);
			}
		};

		return WebCenterWorkflowFactory;
	});

define('WFEditorWCR/scene/WebCenterViewConstants',{
	/**
	 * Size of icons
	 */
	NodeIconSize: 40, // in pixels

	/**
	 * padding in icon
	 */
	NodeIconPadding: 4, // pixels

	/**
	 * Gap between node image and label
	 */
	NodeLabelGap: 15, // in pixels
	/**
	 * Max width of the node label
	 */
	NodeLabelMaxWidth: 100, // in pixels
	/**
	 * Line height of one line of label
	 */
	NodeLabelLineHeight: 15, // in pixels   

	/**
	 * Font for a node label
	 */
	NodeLabelFont: "12px 'open sans',sans-serif",

	/**
	 * Color for a node Label
	 */
	NodeLabelColor: "rgb(0,0,0)",

	NodeLabelBackgroundColor: "rgba(255,255,255, 0.7)",
	NodeLabelSelectedBackgroundColor: "rgb(51,153,255)",
	NodeLabelSelectedFontColor: "white",
	NodeLabelBackgroundPaddingWidth: 4,
	NodeLabelBackgroundPaddingHeight: 3,

	NodeOverlayColorSelected: "rgba(50,50,50,0.5)",
	NodeOverlayColorDisabled: "rgba(200,0,0,0.4)",

	NodeVisibleParameterColor: "rgb(87,134,190)",
	NodeVisibleParameterFont: "10px 'open sans',sans-serif",

	/**
	 * Constatns for the token in progress file icon and status
	 */
	TokenStatusTextColor: "rgb(87,134,190)",
	TokenSatusFont: "10px 'open sans',sans-serif",
	StatusToStatusGap: 20, // vertical gap between different token statuses in pixels
	NodeStatusGap: 30, // Gap between node image and token status in pixels
	TokenStatusFileIconWidth: 18, // Width of the file icon in pixels
	TokenStatusFileTextColor: "grey",

	PinRadius: 5,
	PinRadiusHovered: 7,
	PinFeedbackRadiusHovered: 7,
	PinFeedbackPadding: 2,
	PinToNodeTop: 10,
	PinInToNodeLeft: 8,
	PinOutToNodeRight: 8,
	PinToPinSpacing: 9,

	PinLabelFont: "10px 'open sans',sans-serif",
	PinLabelOffsetX: 1,
	PinLabelOffsetY: 1,
	PinLabelColor: "rgb(0,0,0)",
	PinCharacterFont: "9px 'open sans',sans-serif",
	PinCharacterFeedbackFontHovered: "11px 'open sans', sans-serif",
	PinLabelHeight: 8,

	/**
	 * Colors for OK pin
	 */
	PinOKBackground: "#B6C812",
	PinOKBorder: "#707B0B",
	PinOKCharacterColor: "#000",

	/**
	 * Colors for Error pin
	 */
	PinErrorBackground: "#F62F1A",
	PinErrorBorder: "#ce2c1c",
	PinErrorCharacterColor: "#ffffff",

	/**
	 * Colors for warning pin
	 */
	PinWarningBackground: "#FF9C15",
	PinWarningBorder: "#ef870c",
	PinWarningCharacterColor: "#ffffff",

	/**
	 * Colors for the edges
	 */
	EdgeLightestColor: "#DFF673",
	EdgeLightColor: "#B6C812",
	EdgeDarkColor: "#707B0B",

	/**
	 * Highlighted colors for the edges
	 */
	HighlightedEdgeLightestColor: "#E4E4E4",
	HighlightedEdgeLightColor: "#FFEA14",
	HighlightedEdgeDarkColor: "#FFA400",
	
	StickyNoteHeaderHeight: 20,
	StickyNoteHeaderPadding: 2,
	StickyNoteResizeHitAreaSize: 12,
	StickyNoteContentPadding: 3
});

define(
	'WFEditor/scene/hitarea/HitArea',[
		"utils/ShapeUtils"
	],
	function (
		ShapeUtils
	) {

		"use strict";

		/**
		 * A class defining a hit area for a view object. The hitarea should define a rectangle in world coordinates. 
		 * This way the view object can be detectable on the canvas.
		 * An optional class or array of classes can be given to distinguish between different hit areas of the same view.
		 * 
		 * @exports HitArea
		 * @class
		 * 
		 * @param {ShapeUtils.Rect} rect - the rectangle on the scene that should trigger this hit area
		 * @param {AbstractView} view - the view that defined this hit area                                 
		 * @param {String | Array.<String>} [classes = []] - list of classes used for this hitarea
		 * @param {Number} [zIndex = view.getZIndex()] - hit z index of the hit area, defaults to the zindex of the view, not used for rendering                                    
		 * @param {Object} [options] - optional options object
		 *                           {
		 *                             cursor: <class that will be added to scene on hover>
		 *                             tooltip: <tooltip to be displayed on hover>
		 *                           }
		 */
		var HitArea = function (rect, view, classes, zIndex, options) {

			var normalizedRect = rect.getNormalizedRectArray();

			/**
			 * @member {Number} x coordinate of top left most point of the hitarea
			 */
			this.minX = normalizedRect[0];

			/**
			 * @member {Number} y coordinate of top left most point of the hitarea
			 */
			this.minY = normalizedRect[1];

			/**
			 * @member {Number} x coordinate of bottom right most point of the hitarea
			 */
			this.maxX = normalizedRect[2];

			/**
			 * @member {Number} y coordinate of bottom right most point of the hitarea
			 */
			this.maxY = normalizedRect[3];

			/**
			 * @member {AbstractView}
			 * @private
			 */
			this._view = view;

			/**
			 * @member {Array.<String>} classes of this hit area
			 * @private
			 */
			this._classes = _.isString(classes) ? [classes] : classes || [];

			/**
			 * @member {Number} the hit Z-Index is used to specify which hit area should be selected when clicked by the mouse
			 *                  It is not used for rendering. Defaults to the Z-index of the view.
			 * @private
			 */
			this._hitZIndex = typeof zIndex === "undefined" ? view.getZIndex() : zIndex;

			/**
			 * @member {Object} options object
			 * @private 
			 */
			this._options = options || {};

		};

		/**
		 * Check if the hitareas are equal
		 * @param {HitArea} hitArea
		 */
		HitArea.prototype.equals = function (hitArea) {
			var rectArr = ["minX", "minY", "maxX", "maxY"];
			for (var i = 0; i < 4; i++) {
				if (this[rectArr[i]] !== hitArea[rectArr[i]]) {
					return false;
				}
			}
			if (this._classes.length !== hitArea.getClasses().length) {
				return false;
			}
			for (i = 0; i < this._classes.length; i++) {
				if (this._classes[i] !== hitArea.getClasses()[i]) {
					return false;
				}
			}

			if (this._hitZIndex !== hitArea.getZIndex()) {
				return false;
			}
			_.each(this._options, function(option, key){
				if(hitArea.getOption[key] !== option) {
					return false;
				}
			});
			return true;
		};

		/**
		 * returns the classes of the hit area
		 * @returns {Array.<String>}
		 */
		HitArea.prototype.getClasses = function () {
			return this._classes;
		};

		/**
		 * returns the view of the hitarea
		 * @returns {AbstractView}
		 */
		HitArea.prototype.getView = function () {
			return this._view;
		};

		/**
		 * returns true if the hit area has the given class
		 * @param   {String} clazz
		 * @returns {Boolean} 
		 */
		HitArea.prototype.hasClass = function (clazz) {
			return _.contains(this._classes, clazz);
		};

		/**
		 * returns the z index of the hit area
		 * @returns {Number}
		 */
		HitArea.prototype.getZIndex = function () {
			return this._hitZIndex;
		};

		/**
		 * returns an option of the hitArea by option name. Currently "cursor" and "tooltip"
		 * @returns {String} 
		 */
		HitArea.prototype.getOption = function (option) {
			return this._options[option];
		};

		return HitArea;

	});

define('WFEditorWCR/workflow/node/WebCenterNodeView',[
		"utils/ShapeUtils",
		"WFEditor/workflow/node/NodeView",
		"WFEditor/utils/IconDataStorage",
		"WFEditor/workflow/ModelConstants",
		"WFEditorWCR/scene/WebCenterViewConstants",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditor/scene/hitarea/HitArea",
		"utils/webi18n!WFEditorWCR",
		"underscore"
	],
	function (
		ShapeUtils,
		NodeView,
		IconDataStorage,
		ModelConstants,
		WebCenterViewConstants,
		WebCenterPinConstants,
		WebCenterNodeConstants,
		HitArea,
		i18n,
		_
	) {
		"use strict";

		/**
		 * The view for a webcenter node item. It renders the icon and the label of the node.
		 * It uses the IconDataStorage to fetch the icon.
		 *
		 * @exports WebCenterNodeView
		 * @class
		 * @augments NodeView
		 *
		 * @param {WebCenterNode}
		 * @param {Scene}
		 */
		var WebCenterNodeView = function (nodeModel, scene) {
			NodeView.call(this, nodeModel);

			/**
			 * @member {Number} the max text width of the labels
			 * @private
			 */
			this._maxTextWidth = 0;

			/**
			 * @member {String} the text align for the labels
			 * @private
			 */
			this._textAlign = "";

			/**
			 * @member {Object} cache the label lines so we don't have to recalculate all the time
			 * @private
			 */
			this._labelLines = null;

			/**
			 * @member {Boolean}
			 * @private
			 */
			this._labelLocationDirty = true;

			// Observer location changes
			this._observe(nodeModel, "changed.location", this._recalculateRenderInformation);
			this._observe(nodeModel, "changed.*", this.makeDirty);
			this._observe(nodeModel, "changed.label", this._recalculateRenderInformation);
			this._observe(nodeModel, "Pin.added", this._recalculateRenderInformation);
			this._observe(nodeModel.getWorkflow(), "changed.debugMode", this._recalculateRenderInformation);


		};

		WebCenterNodeView.prototype = Object.create(NodeView.prototype);
		WebCenterNodeView.prototype.constructor = WebCenterNodeView;

		/**
		 * Updates the bounding box, hit areas and label
		 */
		WebCenterNodeView.prototype._recalculateRenderInformation = function () {
			this._labelLocationDirty = true;
			this._updateBoundingRect();
			this._updateHitAreas();
			this.makeDirty();
		};

		/**
		 * @override
		 * Renders the node on the provided canvas context. Only renders if the icon is available.
		 * If the icon is not yet loaded, the iconstorage fetches the icon and the layer is marked dirty
		 * when the icon is correctly fetched.
		 *
		 * @param   {CanvasRenderingContext2D} context
		 * @param   {Layer} layer
		 * @param 	{Boolean} quickRender
		 */
		WebCenterNodeView.prototype.render = function (context, layer, quickRender) {
			var location = this._model.getLocation();
			var iconStorage = IconDataStorage.getInstance(); //get the singleton instance of the datastorage
			//request the icon from the datastorage, returns the icon if available, if not it starts loading the icon.
			if (this._model.getIconUrl()) {
				var icon = iconStorage.get(this._model.getIconUrl(), _.bind(this.makeDirty, this));
				if (icon) { //only render if the icon was already fetched

					var width = icon.naturalWidth;
					var height = icon.naturalHeight;


					// if the icon is too big, resize it proportionally
					if (width > WebCenterViewConstants.NodeIconSize || height > WebCenterViewConstants.NodeIconSize) {
						var ratio = width / height;
						if (ratio > 1) {
							width = WebCenterViewConstants.NodeIconSize;
							height = width / ratio;
						} else {
							height = WebCenterViewConstants.NodeIconSize;
							width = height * ratio;
						}
					}
					context.drawImage(icon, location.x, location.y, width, height);

					if (this._model.isSelected() || !this._model.isEnabled()) {
						// draw overlay
						context.save();

						if (!this._model.isEnabled()) {
							context.fillStyle = WebCenterViewConstants.NodeOverlayColorDisabled;
						} else {
							context.fillStyle = WebCenterViewConstants.NodeOverlayColorSelected;
						}
						var x = location.x + WebCenterViewConstants.NodeIconPadding;
						var y = location.y + WebCenterViewConstants.NodeIconPadding;
						var radius = 3;
						width = width - 2 * WebCenterViewConstants.NodeIconPadding;
						height = height - 2 * WebCenterViewConstants.NodeIconPadding;
						context.beginPath();
						context.moveTo(x + radius, y);
						context.lineTo(x + width - radius, y);
						context.quadraticCurveTo(x + width, y, x + width, y + radius);
						context.lineTo(x + width, y + height - radius);
						context.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
						context.lineTo(x + radius, y + height);
						context.quadraticCurveTo(x, y + height, x, y + height - radius);
						context.lineTo(x, y + radius);
						context.quadraticCurveTo(x, y, x + radius, y);
						context.closePath();
						context.fill();
						context.restore();

					}
				}
			}

			// only render labels in normal render mode and when the label isn't being edited
			if (!quickRender && !this._model.isEditingLabel()) {
				// Label rendering
				var labelLines = this._getLabelLines(context, this._model.getLabel(true), location, new ShapeUtils.Size(WebCenterViewConstants.NodeIconSize, WebCenterViewConstants.NodeIconSize));

				context.save();

				this._renderLabelBackground(context, labelLines);

				if (this._model.isSelected()) {
					context.fillStyle = WebCenterViewConstants.NodeLabelSelectedFontColor;
				} else {
					context.fillStyle = WebCenterViewConstants.NodeLabelColor;
				}
				context.font = WebCenterViewConstants.NodeLabelFont;

				_.each(labelLines, function (labelLine) {
					context.textAlign = this._textAlign;
					context.fillText(labelLine.text, labelLine.location.x, labelLine.location.y);
				}, this);

				context.restore();
			}

			// Render visible parameters under label
			if (!quickRender) {
				var visibleParams = this._model.getVisibleParameters();
				if (_.size(visibleParams) > 0) {
					var labelRect = this.getLabelRect();

					context.save();
					context.fillStyle = WebCenterViewConstants.NodeVisibleParameterColor;
					context.font = WebCenterViewConstants.NodeVisibleParameterFont;
					context.textAlign = this._textAlign;

					var xLoc = labelRect.center().x;
					var yLoc = labelRect.bottom() + WebCenterViewConstants.NodeLabelLineHeight - 3; // a bit less spacing

					_.each(visibleParams, function (param) {
						context.fillText(i18n.t(param.getName()) + ": " + param.getValue(), xLoc, yLoc);
						yLoc += WebCenterViewConstants.NodeLabelLineHeight;
					});
				}
			}
		};

		/**
		 * @Override
		 */
		WebCenterNodeView.prototype._updateHitAreas = function (context) {
			var nodeLocation = this._model.getLocation();
			var nodeRect = new ShapeUtils.Rect(nodeLocation, new ShapeUtils.Size(WebCenterViewConstants.NodeIconSize, WebCenterViewConstants.NodeIconSize));
			var labelRect = this.getLabelRect();
			if (labelRect.size.width === 0) {
				// There should be at least a little hit area for labels
				labelRect.size.width = WebCenterViewConstants.NodeIconSize;
				labelRect.size.height = WebCenterViewConstants.NodeLabelLineHeight;
			}
			var nodeSelectHitAreaCursor = this._model.getWorkflow().getPrivilege(ModelConstants.Privileges.viewNodeConfiguration) ? "pointer-cursor" : undefined;
			var editLabelHitAreaCursor = this._model.getWorkflow().getPrivilege(WebCenterNodeConstants.Privileges.editLabels) ? "text-cursor" : undefined;
			this._setHitAreas(
				[new HitArea(nodeRect, this, ModelConstants.HitAreaClass.NodeSelect, undefined, {
					cursor: nodeSelectHitAreaCursor
				}), new HitArea(labelRect, this, WebCenterNodeConstants.HitAreas.LABEL, 15, {
					cursor: editLabelHitAreaCursor
				})]);
		};

		/**
		 * Get the location for a certain pin
		 * @param {Pin} pin for which the location needs to be returned
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterNodeView.prototype.getLocationForPin = function (pin) {
			var pinLocation = this._model.getLocation().copy();
			pinLocation.y += WebCenterViewConstants.PinToNodeTop;

			var pinSeqNumber = this._model.getPinSequenceNumber(pin);

			if (pin === this._model.getInputPin()) {
				// input pin (we only have 1 location for this
				pinLocation.x -= WebCenterViewConstants.PinInToNodeLeft;
			} else {
				// output pin				
				pinLocation.x += WebCenterViewConstants.NodeIconSize + WebCenterViewConstants.PinOutToNodeRight;
			}

			pinLocation.y += pinSeqNumber * (WebCenterViewConstants.PinRadius * 2 + WebCenterViewConstants.PinToPinSpacing);

			return pinLocation;
		};

		/**
		 * Do the output pins come lower than the node icon?
		 * if so, the label should be right aligned
		 * @param {ShapeUtils.Size} nodeSize
		 * @returns {Boolean}
		 */
		WebCenterNodeView.prototype._shouldAdjustLabelPositionForPins = function (nodeSize) {
			var totalPinSize = (WebCenterViewConstants.PinRadius * 2) + WebCenterViewConstants.PinToPinSpacing;

			return (this._model.getOutputPinCount() * totalPinSize) >= nodeSize.height;
		};

		/**
		 * Get the given labelText wrapped in lines according to NodeLabelMaxWidth
		 * @param   {CanvasRenderingContext2D} context      - The canvas context
		 * @param   {String}                   labelText    - The label text
		 * @param   {ShapeUtils.Point}         nodeLocation - The top left location of the node (the icon)
		 * @param   {ShapeUtils.Size}          nodeSize     - The size of the node (the icon)
		 * @returns {Array.<Object>}           An array of lines with objects build like
		 *                                                  {	{String} text
		 *                                                      {ShapeUtils.Point} location
		 *                                                  	{Number} textWidth
		 *                                                  }
		 */
		WebCenterNodeView.prototype._getLabelLines = function (context, labelText, nodeLocation, nodeSize) {
			if (this._labelLocationDirty) {
				this._maxTextWidth = 0;
				var lines = [];

				context.fillStyle = WebCenterViewConstants.NodeLabelColor;
				context.font = WebCenterViewConstants.NodeLabelFont;

				var labelLocation = this.getLabelStartPoint(nodeLocation, nodeSize);

				var remainder = labelText;
				while (remainder.length > 0) {
					var line = this._getNextLineToken(context, remainder);
					var textWidth = context.measureText(line).width;
					if (textWidth > this._maxTextWidth) {
						this._maxTextWidth = textWidth;
					}
					lines.push({
						text: line.trim(),
						location: labelLocation.copy(),
						textWidth: textWidth
					});
					remainder = remainder.substring(line.length).trim();
					labelLocation.y += WebCenterViewConstants.NodeLabelLineHeight;
				}
				this._labelLines = lines;
				this._labelLocationDirty = false;
			}
			return this._labelLines;
		};

		/**
		 * returns the start point of the label
		 * @param   {ShapeUtils.Point} nodeLocation - the location of the node
		 * @param   {ShapeUtils.Size}   nodeSize - the size of the node
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterNodeView.prototype.getLabelStartPoint = function (nodeLocation, nodeSize) {
			if (typeof nodeLocation === "undefined") {
				nodeLocation = this._model.getLocation();
			}
			if (typeof nodeSize === "undefined") {
				nodeSize = new ShapeUtils.Size(WebCenterViewConstants.NodeIconSize, WebCenterViewConstants.NodeIconSize);
			}
			var labelLocation = nodeLocation.copy();

			if (this._shouldAdjustLabelPositionForPins(nodeSize)) {
				// label is aligned right because of pins
				this._textAlign = "right";
				labelLocation.y += nodeSize.height + WebCenterViewConstants.NodeLabelGap;
				labelLocation.x += nodeSize.width - WebCenterViewConstants.NodeIconPadding;
			} else {
				// label location is bottom-center of icon due to textAlign center
				this._textAlign = "center";
				labelLocation.y += nodeSize.height + WebCenterViewConstants.NodeLabelGap;
				labelLocation.x += nodeSize.width / 2;
			}

			return labelLocation;
		};

		/**
		 * Get next line token based on the maximum label width for the given text.
		 * It will try to split on a space if too big and if not present, to the point where it no longer fits the line.
		 * @param   {CanvasRenderingContext2D} context - The canvas context
		 * @param   {String} text - the text to get the first token from
		 * @returns {String} the next line token
		 */
		WebCenterNodeView.prototype._getNextLineToken = function (context, text) {
			var words = text.split(" ");
			var line = '';
			for (var i = 0; i < words.length; i++) {
				var word = words[i];
				line += i > 0 ? " " + word : word;
				var textMetrics = context.measureText(line);
				if (textMetrics.width > WebCenterViewConstants.NodeLabelMaxWidth) {
					if (i > 0) {
						// space found
						line = line.substring(0, line.length - word.length - 1);
						break;
					} else {
						// no space found -> find another split
						var newLine = "";
						for (var j = 0; j < line.length; j++) {
							newLine += line.charAt(j);
							textMetrics = context.measureText(newLine);
							if (textMetrics.width > WebCenterViewConstants.NodeLabelMaxWidth) {
								line = line.substring(0, newLine.length - 1);
								break;
							}
						}
					}
				}
			}
			return line;
		};

		/**
		 * @override
		 */
		WebCenterNodeView.prototype._updateBoundingRect = function () {
			var location = this._model.getLocation().copy();

			this._context.save();
			var labelLines = this._getLabelLines(this._context, this._model.getLabel(true), location, new ShapeUtils.Size(WebCenterViewConstants.NodeIconSize, WebCenterViewConstants.NodeIconSize));
			this._context.restore();

			// we assume WebCenterViewConstants.NodeLabelMaxWidth as label width
			if (this._textAlign === "center") {
				location.x += WebCenterViewConstants.NodeIconSize / 2 - WebCenterViewConstants.NodeLabelMaxWidth / 2 - WebCenterViewConstants.NodeLabelBackgroundPaddingWidth;
			} else {
				location.x += WebCenterViewConstants.NodeIconSize - WebCenterViewConstants.NodeLabelMaxWidth - WebCenterViewConstants.NodeLabelBackgroundPaddingWidth;

			}
			var width = Math.max(WebCenterViewConstants.NodeLabelMaxWidth + 2 * WebCenterViewConstants.NodeLabelBackgroundPaddingWidth, WebCenterViewConstants.NodeIconSize);
			// icon + gap + x lines: note that the first line is drawn on icon + gap y so, that's why the length - 1 is here
			var height = WebCenterViewConstants.NodeIconSize + WebCenterViewConstants.NodeLabelGap + (labelLines.length - 1) * WebCenterViewConstants.NodeLabelLineHeight + 2 * WebCenterViewConstants.NodeLabelBackgroundPaddingHeight + 1;


			this._setBoundingRect(new ShapeUtils.Rect(location, new ShapeUtils.Size(width, height)).getNormalizedRectArray());
		};


		/**
		 * render the label background for a selected node
		 * @param {CanvasRenderingContext2D}   context
		 * @param {Array.<Object>} labelLines info about the lines of the label
		 *
		 */
		WebCenterNodeView.prototype._renderLabelBackground = function (context, labelLines) {

			var labelRect = this.getLabelRect();
			var x = labelRect.origin.x;
			var y = labelRect.origin.y;
			var width = labelRect.size.width;
			var height = labelRect.size.height;
			if (width === 0 || height === 0) {
				return;
			}
			if (this._model.isSelected()) {
				context.fillStyle = WebCenterViewConstants.NodeLabelSelectedBackgroundColor;
			} else {
				context.fillStyle = WebCenterViewConstants.NodeLabelBackgroundColor;

			}

			var radius = 5;
			context.beginPath();
			context.moveTo(x + radius, y);
			context.lineTo(x + width - radius, y);
			context.quadraticCurveTo(x + width, y, x + width, y + radius);
			context.lineTo(x + width, y + height - radius);
			context.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
			context.lineTo(x + radius, y + height);
			context.quadraticCurveTo(x, y + height, x, y + height - radius);
			context.lineTo(x, y + radius);
			context.quadraticCurveTo(x, y, x + radius, y);
			context.closePath();
			context.fill();


		};

		/**
		 * returns the rect in which the label should be drawn
		 * @returns {ShapeUtils.Rect} 
		 */
		WebCenterNodeView.prototype.getLabelRect = function () {
			var labelLines = this._getLabelLines(this._context, this.getModel().getLabel(true), this.getModel().getLocation(), new ShapeUtils.Size(WebCenterViewConstants.NodeIconSize, WebCenterViewConstants.NodeIconSize));
			var maxTextWidth = this._maxTextWidth;
			var y = Infinity;
			var bottomY = -Infinity;
			var x = +Infinity;

			// get left most x and top most y value to start the rendering
			_.each(labelLines, function (labelLine) {
				if (labelLine.location.y < y) {
					y = labelLine.location.y;
				}
				if (labelLine.location.y > bottomY) {
					bottomY = labelLine.location.y;
				}
				if (labelLine.location.x < x) {
					x = labelLine.location.x;
				}
			});



			// the height of all text 
			var textHeight = bottomY - y + 12;

			// add horizontal padding
			var paddingHorizontal = WebCenterViewConstants.NodeLabelBackgroundPaddingWidth;
			var width = maxTextWidth + 2 * paddingHorizontal;

			// add vertical padding
			var paddingVertical = WebCenterViewConstants.NodeLabelBackgroundPaddingHeight;
			var height = textHeight + 2 * paddingVertical;

			// the starting x point depends on the alignment of the text
			if (this._textAlign === "center") {
				x = x - (maxTextWidth / 2) - paddingHorizontal;
			} else if (this._textAlign === "right") {
				x = x - (maxTextWidth) - paddingHorizontal;
			}
			y = y - 10 - paddingVertical;
			if (x === Infinity || y == Infinity) {

				var labelLocation = this.getLabelStartPoint(this._model.getLocation(), new ShapeUtils.Size(WebCenterViewConstants.NodeIconSize, WebCenterViewConstants.NodeIconSize));
				x = labelLocation.x;
				y = labelLocation.y -= WebCenterViewConstants.NodeLabelLineHeight;
				width = 0;
				height = 0;
			}
			return new ShapeUtils.Rect(new ShapeUtils.Point(x, y), new ShapeUtils.Size(width, height));

		};

		/**
		 * returns the text alignment of the node's label
		 * @returns {String} 
		 */
		WebCenterNodeView.prototype.getTextAlignment = function () {
			return this._textAlign;
		};

		/**
		 * @override
		 */
		WebCenterNodeView.prototype.getCenter = function () {
			var center = this._model.getLocation().copy();
			center.x += WebCenterViewConstants.NodeIconSize / 2;
			center.y += WebCenterViewConstants.NodeIconSize / 2;

			return center;
		};


		return WebCenterNodeView;

	});

define('WFEditor/components/DraggablePopup',[
		"utils/ShapeUtils",
		"utils/Observable",
		"underscore",
		"jquery",
		"jqueryUI"
	],
	function (
		ShapeUtils,
		Observable,
		_,
		$) {
		"use strict";

		/**
		 * Component for the WorkflowEditor, popup with a header, close button and a content
		 * area. The popup is draggable. Needs to be extended by a specific implementation.
		 * The popup is hidden by default.
		 *
		 * @exports DraggablePopup
		 * @class
		 *
		 * @param {JQueryObj} context - elements it adds itself to
		 */
		var DraggablePopup = function (context) {
			Observable.mixin(this, "draggablePopup");

			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._context = context;

			/**
			 * @member {Boolean} indicates if the popup is currently being hovered
			 * @private
			 */
			this._isHovered = false;

			/**
			 * @member {Boolean} indicates if the popup is being moved from its start position
			 * 						by JQuery UI draggable.
			 * @private
			 */
			this._isMoved = false;

			/**
			 * @member {Boolean} if a popup is sticky it doesn't hide on mouseout.
			 * The popup becomes sticky by clicking on it.
			 * @private
			 */
			this._isSticky = false;

			/**
			 * @member {Boolean} is the popup visible at this moment.
			 * @private
			 */
			this._isVisible = false;

			/**
			 * @member {JQueryObj} main element for the popup
			 * @private
			 */
			this._element = $("<div>").addClass("draggablePopup");

			/**
			 * @member {JQueryObj} header element, contains the title and the close button
			 * @private
			 */
			this._header = $("<div>").addClass("popupHeader");

			/**
			 * @member {JQueryObj} content of the popup
			 * @private
			 */
			this._content = $("<div>").addClass("popupContent");

			/**
			 * @member {JQueryObj} inline title element, is added to the header
			 * @private
			 */
			this._title = $("<span>");

			/**
			 * @member {JQueryObj} close button, used to close the popup
			 * @private
			 */
			this._closeBtn = $("<span>").text("X").addClass("closeBtn");
			this._closeBtn.hide(); //hide button by default, only visible when the popup has been moved

			//construct the correct DOM structure for the popup
			this._header.append(this._title);
			this._header.append(this._closeBtn);
			this._element.append(this._header);
			this._element.append(this._content);
			this._context.append(this._element);

			//make the element draggable, JQuery UI draggable is used
			this._element.draggable({
				handle: this._header,
				start: _.bind(this._draggableStart, this),
				stop: _.bind(this._draggableStop, this),
				containment: this._context
			});
			//hide the popup by default
			this.hide();
			//register the correct mouse listeners to the popup
			this._registerListeners();


			this._deferedCloseTimer = null;
		};

		/**
		 * Executed when dragging started
		 */
		DraggablePopup.prototype._draggableStart = function () {
			this._setMoved(true);
			this._closeBtn.show();
			this._startedMoving();
			this._emit("startMoving");
		};

		/**
		 * Executed when dragging stopped
		 */
		DraggablePopup.prototype._draggableStop = function () {
			this._stoppedMoving();
			this._emit("stopMoving");
		};

		/**
		 * Registers mouselisteners for the close button and the element
		 * @private
		 */
		DraggablePopup.prototype._registerListeners = function () {
			this._closeBtn.on("click", _.bind(function (e) {
				this._setHovered(false);
				this._setMoved(false);
				this._setSticky(false);
				this._closeBtn.hide();
				this._closed();
				this._emit("closed");
				this.hide(true);
				e.stopPropagation();
			}, this));

			this._element.on("click", _.bind(function (e) {
				this.setSticky(true);
				this._closeBtn.show();
			}, this));

			this._element.on("mouseenter", _.bind(function () {
				this._setHovered(true);
				this.cancelDeferedClose();
			}, this));
			this._element.on("mouseleave", _.bind(function () {
				this._setHovered(false);
			}, this));
			this._element.on("mousedown", function (e) {
				e.stopPropagation();
			});

		};
		/**
		 * Internal setter for hovered
		 * @param {Boolean} isHovered
		 */
		DraggablePopup.prototype._setHovered = function (isHovered) {
			if (this._isHovered !== isHovered) {
				this._isHovered = isHovered;
				this._emit("changed.hover", isHovered);
			}
		};

		/**
		 * Internal setter for moved
		 * @param {Boolean} isMoved
		 */
		DraggablePopup.prototype._setMoved = function (isMoved) {
			if (this._isMoved !== isMoved) {
				this._isMoved = isMoved;
				this._emit("changed.moved", isMoved);
			}
		};

		/**
		 * Internal setter for sticky
		 * @param {Boolean} isSticky
		 */
		DraggablePopup.prototype._setSticky = function (isSticky) {
			if (this._isSticky !== isSticky) {

				this._isSticky = isSticky;
				this._emit("changed.sticky", isSticky);
			}
		};

		/**
		 * Internal setter for visible
		 * @param {Boolean} isVisible
		 */
		DraggablePopup.prototype._setVisible = function (isVisible) {
			if (this._isVisible !== isVisible) {
				this._isVisible = isVisible;
				this._emit("changed.visible", isVisible);
			}
		};


		/**
		 * Is the popup sticky?
		 * @returns {Boolean}
		 */
		DraggablePopup.prototype.isSticky = function () {
			return this._isSticky;
		};

		/**
		 * Make the popup sticky or not
		 * @param {Boolean} sticky
		 */
		DraggablePopup.prototype.setSticky = function (sticky) {
			this._setSticky(sticky);
		};

		/**
		 * Is the popup beign hovered at this moment
		 * @returns {Boolean}
		 */
		DraggablePopup.prototype.isHovered = function () {
			return this._isHovered;
		};

		/**
		 * Show the popup.
		 */
		DraggablePopup.prototype.show = function () {
			this.cancelDeferedClose();
			this._element.show();
			this._setVisible(true);

		};

		/**
		 * Hide the popup, does not hide it if the popup is moved or hovered or sticky.
		 * Use parameter to force hide the popup.
		 * @param {Boolean} forceHide
		 */
		DraggablePopup.prototype.hide = function (forceHide) {
			if ((!this._isHovered && !this._isMoved && !this._isSticky) || forceHide) {
				this._element.hide();
				this._setVisible(false);
			}
		};

		/**
		 * Set the title in the popup.
		 * @param {String} title
		 */
		DraggablePopup.prototype.setTitle = function (title) {
			this._title.text(title);
		};

		/**
		 * Is the popup moved.
		 * @returns {Boolean}
		 */
		DraggablePopup.prototype.isMoved = function () {
			return this._isMoved;
		};

		/**
		 * Is the popup visible.
		 * @returns {Boolean}
		 */
		DraggablePopup.prototype.isVisible = function () {
			return this._isVisible;
		};

		/**
		 * Get the location of the popup, returns the distance from the top left corner of the context element.
		 * @returns {ShapeUtils.Point}
		 */
		DraggablePopup.prototype.getLocation = function () {
			var offset = this._element.offset(); //distance from the top left of the page
			var parentOffset = this._context.offset(); //distance of the context top left to the page top left corner
			var width = this._element.width();
			var height = this._element.height();
			return new ShapeUtils.Point(offset.left - parentOffset.left + width / 2, offset.top - parentOffset.top + height / 2);
		};

		/**
		 * The popup started moving around by user interaction.
		 * @private
		 */
		DraggablePopup.prototype._startedMoving = function () {
			//implement by specific popup to handle moving start	
		};

		/**
		 * The popup stopped to move around by user interaction.
		 * @private
		 */
		DraggablePopup.prototype._stoppedMoving = function () {
			//implement by specific popup to handle moving stop	
		};

		/**
		 * The popup was closed by using the close button on the popup.
		 * @private
		 */
		DraggablePopup.prototype._closed = function () {
			//implement by specific popup to hanl	
		};

		/**
		 * Destroys all resources of this popup
		 */
		DraggablePopup.prototype.destroyPopup = function () {
			this._element.remove();
		};

		/**
		 * Close the popup in after a delay
		 * cancelable by calling cancelDeferedClose()
		 */
		DraggablePopup.prototype.deferredClose = function () {
			this.cancelDeferedClose();
			this._deferedCloseTimer = setTimeout(_.bind(function () {
				this.hide();
			}, this), 500);
		};

		/**
		 * Cancel the ongoing defered close operation
		 */
		DraggablePopup.prototype.cancelDeferedClose = function () {
			clearTimeout(this._deferedCloseTimer);
			this._deferedCloseTimer = null;
		};


		return DraggablePopup;

	});

define('WFEditorWCR/documentContextMenu/WebCenterDocumentContextMenu',["component/dropdownmenu/DropdownMenu",
		"WFEditorWCR/workflow/WebCenterWorkflowState",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"utils/webi18n!WFEditorWCR",
		"utils/HTTPUtils",
		"utils/XMLUtils",
		"jquery",
		"underscore"
	],
	function (DropdownMenu,
		WebCenterWorkflowState,
		WebCenterPinConstants,
		WebCenterNodeConstants,
		i18n,
		HTTPUtils,
		XMLUtils,
		$,
		_) {
		"use strict";

		/**
		 * This class is a wrapper around a dropdown menu to show a custom context menu of a document in a token
		 *
		 * @class
		 * @exports WebCenterDocumentContextMenu
		 *
		 * @param {Scene} scene - used to get the controller and to get the context element to attach the anchor to
		 */
		var WebCenterDocumentContextMenu = function (scene) {
			// will be filled in by show
			this._documentVersionID = null;
			this._tokenID = null;
			this._pinModel = null;

			this._scene = scene;

			this._webCenterAPI = scene.getWebCenterAPI();

			this._anchorDiv = $("<div />").addClass("documentContextMenuAnchor"); // the context menu is attached to this element
			scene.getContext().append(this._anchorDiv);

			// note i18n only supports "t" here, not the loadSingleBundle like it should
			this._dropdown = new DropdownMenu(i18n, scene.getContext(), this._webCenterAPI.getWebCenterConfig().isMobile, "right", "wcr-dropdown");

			// View option
			this._dropdown.addAction({
				name: "view",
				content: i18n.t("wfview"),
				enabled: false,
				isVisible: false,
				action: _.bind(function () {
					var viewURL = this._webCenterAPI.getViewDocumentURL(this._documentVersionID);
					window.open(viewURL, "_blank");
				}, this)
			});

			// Retry option
			this._dropdown.addAction({
				name: "retry",
				content: i18n.t("wfretry"),
				enabled: false,
				isVisible: false,
				action: _.bind(function () {
					this._webCenterAPI.retryToken(this._pinModel.getWorkflow(), this._pinModel.getToken(this._tokenID));
				}, this)
			});

			// Relaunch option
			this._dropdown.addAction({
				name: "relaunch",
				content: i18n.t("wfrelaunch"),
				enabled: false,
				isVisible: false,
				action: _.bind(function () {
					var token = this._pinModel.getWorkflow().getToken(this._tokenID);
					this._webCenterAPI.relaunchWorkflow(this._pinModel.getWorkflow(), token).done(_.bind(function (taskID) {
						this._scene.requestNewWorkflow(taskID);
					}, this));
				}, this)
			});
		};

		/**
		 * Show the context menu
		 * @param {WebCenterPin} pinModel - The pin model
		 * @param {String} documentVersionId - The version ID of the document
		 * @param {String} tokenId - The token ID
		 * @param {ShapeUtils.Point} location - Location of the mouseclick
		 */
		WebCenterDocumentContextMenu.prototype.show = function (pinModel, documentVersionId, tokenId, location) {
			if (!tokenId) {
				return;
			}

			this._anchorDiv.css({
				left: location.x,
				top: location.y
			});

			this._documentVersionID = documentVersionId;
			this._tokenID = tokenId;
			this._pinModel = pinModel;

			// View
			if (this._documentVersionID) {
				this._webCenterAPI.isDocumentViewable(this._documentVersionID, _.bind(function (viewableObj) {
					if (viewableObj !== undefined && !viewableObj.supported && !viewableObj.viewable) {
						this._dropdown.hideElement("view");
					} else {
						this._dropdown.showElement("view");
						this._dropdown.setActionEnabled("view", viewableObj && viewableObj.viewable);
						this._dropdown.show(this._anchorDiv);
					}
				}, this));
			} else {
				this._dropdown.hideElement("view");
			}

			var workflow = this._pinModel.getWorkflow();
			var token = workflow.getToken(this._tokenID);

			// Retry
			if (token && (this._pinModel.getPinType() === WebCenterPinConstants.Type.OutError ||
					this._pinModel.getPinType() === WebCenterPinConstants.Type.WorkflowConstructError) && workflow.getPrivilege(WebCenterNodeConstants.Privileges.showWorkflowControls) && workflow.getWorkflowState() == WebCenterWorkflowState.RUNNING) {
				this._dropdown.showElement("retry");
				var enableRetry = false;
				var nextToken = workflow.getToken(token.getErrorHandlingId());
				if (nextToken && !nextToken.hasFutureTokens()) {
					enableRetry = true;
				}
				this._dropdown.setActionEnabled("retry", enableRetry); // retry only enabled if no future tokens 
				this._dropdown.show(this._anchorDiv);
			} else {
				this._dropdown.hideElement("retry");
			}


			// Relaunch
			if (token && workflow.getWorkflowState() == WebCenterWorkflowState.FINISHED && workflow.getPrivilege(WebCenterNodeConstants.Privileges.showWorkflowControls) && workflow.getPrivilege("isMasterTask")) {
				this._dropdown.showElement("relaunch");
				this._dropdown.setActionEnabled("relaunch", _.size(token.getParents()) === 1); // only if there is 1 parent token enable it
				this._dropdown.show(this._anchorDiv);
			} else {
				this._dropdown.hideElement("relaunch");
			}

			// If nothing to show, hide it
			if (!this._dropdown.isElementVisible("view") && !this._dropdown.isElementVisible("retry") && !this._dropdown.isElementVisible("relaunch")) {
				this._dropdown.hide();
			}
		};

		return WebCenterDocumentContextMenu;
	});


define('tpl!WFEditorWCR/webcenterInformationPopup/WebCenterInformationPopup', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div class="tokenList">\r\n	<ul>\r\n		';
 _.each(d.tokens, function(token, tokenId){ 
__p+='		\r\n			';
 if(token.getFileCount() === 0) { 
__p+='\r\n				<li class="document '+
((__t=(d.getSelectedClass(token)))==null?'':_.escape(__t))+
'" data-bind-tokenId="'+
((__t=(token.getId()))==null?'':_.escape(__t))+
'">\r\n					'+
((__t=(d.i18n("wfedit.output.nodocuments")))==null?'':_.escape(__t))+
'\r\n					\r\n					';
 _.each(token.getErrors(), function(error, errorId){ 
__p+='\r\n						<span class="tokenError" data-bind="'+
((__t=( errorId ))==null?'':_.escape(__t))+
'">\r\n							'+
((__t=(d.i18n(error) ))==null?'':_.escape(__t))+
'\r\n						</span>\r\n					';
 }) 
__p+='\r\n				</li>\r\n			';
 } else { 
__p+='\r\n				';
 _.each(token.getFiles(), function(document, versionId){ 
__p+='\r\n					<li class="document '+
((__t=(d.getSelectedClass(token,versionId)))==null?'':_.escape(__t))+
'" \r\n						data-bind-tokenId="'+
((__t=( token.getId() ))==null?'':_.escape(__t))+
'" data-bind-versionId="'+
((__t=( versionId ))==null?'':_.escape(__t))+
'">\r\n						'+
((__t=( document ))==null?'':_.escape(__t))+
'\r\n						';
 _.each(token.getErrors(), function(error, errorId){ 
__p+='\r\n							<span class="tokenError" data-bind="'+
((__t=( errorId ))==null?'':_.escape(__t))+
'">\r\n								'+
((__t=(d.i18n(error) ))==null?'':_.escape(__t))+
'\r\n							</span>\r\n						';
 }) 
__p+='\r\n					</li>\r\n				';
 }) 
__p+='\r\n			';
 } 
__p+='\r\n		';
 }); 
__p+='\r\n	</ul>\r\n</div>';
return __p;
}});
define('WFEditorWCR/webcenterInformationPopup/WebCenterInformationPopup',[
		"WFEditor/components/DraggablePopup",
		"WFEditorWCR/documentContextMenu/WebCenterDocumentContextMenu",
		"Template",
		"tpl!WFEditorWCR/webcenterInformationPopup/WebCenterInformationPopup",
		"utils/HTTPUtils",
		"utils/ShapeUtils",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery",
		"jqueryUI"
	],
	function (
		DraggablePopup,
		WebCenterDocumentContextMenu,
		Template,
		tpl,
		HTTPUtils,
		ShapeUtils,
		i18n,
		_,
		$) {
		"use strict";

		/**
		 * Token information is shown in a movable popup. It shows the tokens and their documents
		 * as a list in the popup. The popup extends DraggablePopup in the WFEditor framework.
		 *
		 * @exports WebCenterInformationPopup
		 * @class
		 * @augments DraggablePopup
		 *
		 * @param {Workflow} workflow
		 * @param {Scene} scene
		 * @param {String} backgroundColor
		 * @param {AbstractView} parentView - the view that is responsible for showing and hiding the popup
		 */
		var WebCenterInformationPopup = function (workflow, scene, backgroundColor, parentView) {
			DraggablePopup.call(this, scene.getContext());

			/**
			 * @member {Scene} general view object, popup is added to its context
			 * @private
			 */
			this._scene = scene;

			/**
			 * @member {Workflow}
			 * @private
			 */
			this._workflow = workflow;
			/**
			 * @member {String}
			 * @private
			 */
			this._color = backgroundColor;

			/**
			 * @member {AbstractView}
			 * @private
			 */
			this._parentView = parentView;

			/**
			 * @member {WebCenterDocumentContextMenu} context menu for a document in a token
			 * @private
			 */
			this._contextMenu = null;

			/**
			 * @member {ShapeUtils.Size} the amount of pixels the popup was moved by dragging
			 * @private
			 */
			this._movedSize = new ShapeUtils.Size();

			// initialize template
			this.t = new Template(i18n, tpl);
			//set the context for the template, this is the content of the popup
			this.t.setContext(this._content);
			//register click event an the token items
			this.t.bind("click", "li", this._tokenItemClicked, this);
			this.t.bind("contextmenu", "li", this._handleTokenItemContextMenu, this);
			//register a function to the template to fetch the correct class for selected items
			this.t.d.getSelectedClass = _.bind(this._getSelectedClass, this);
			
			// Containment gives weird effects if the popup is bigger than the scene
			this._element.draggable({
				containment: undefined
			});

		};

		WebCenterInformationPopup.prototype = Object.create(DraggablePopup.prototype);
		WebCenterInformationPopup.prototype.constructor = WebCenterInformationPopup;

		/**
		 * Renders the popup on the right position
		 */
		WebCenterInformationPopup.prototype.show = function () {
			if (!this.isVisible()) { //only show the popup if it not yet visible
				DraggablePopup.prototype.show.call(this);
				//set the title of the popup to the label of the pin
				this.setTitle(this.getTitle());
				var color = this._color;
				//get the location of the pin
				var location = this.getPopupLocation();
				//transform the point from world coordinates to pixels from scene div top left corner
				var t = this._scene.getTransformation();
				var loc = t.transformedPoint(location);
				this._element.css({
					left: loc.x, //draw to the right of the pin
					top: loc.y, //move some pixels upwards
					'background-color': color //set the correct background color
				});
				this.render();
			}
		};

		/**
		 * @override
		 */
		WebCenterInformationPopup.prototype.hide = function (forceHide) {
			if ((!this._isHovered && !this._isMoved && !this._isSticky) || forceHide) {
				DraggablePopup.prototype.hide.call(this);
				this._movedSize = new ShapeUtils.Size();
			}
		};

		/**
		 * recalculates the location of the popup
		 */
		WebCenterInformationPopup.prototype.relocate = function () {
			//get the location of the pin
			var location = this.getPopupLocation();
			//transform the point from world coordinates to pixels from scene div top left corner
			var t = this._scene.getTransformation();
			var loc = t.transformedPoint(location);
			this._element.css({
				left: loc.x + this._movedSize.width, //draw to the right of the pin
				top: loc.y + this._movedSize.height //move some pixels upwards
			});

		};

		/**
		 * Render the template of the popup.
		 */
		WebCenterInformationPopup.prototype.render = function () {
			//get the tokens that need to be rendered in the popup
			this.t.d.tokens = this.getTokens();
			//render the template
			this.t.render();
		};

		/**
		 * Get the correct class for the token items when they are selected. This method is
		 * used by the template.
		 * @param   {WebCenterToken} token
		 * @param   {String} versionId
		 * @returns {String}   class that is added to the list item or empty string if no class needs to be added.
		 */
		WebCenterInformationPopup.prototype._getSelectedClass = function (token, versionId) {
			if ((versionId && token.getSelectedDocumentVersionId() === versionId) || (token.getFileCount() === 0 && token.isSelected())) {
				return "docSelected";
			} else if (token.isSelected()) {
				return "tokenSelected";
			} else {
				var highlightedToken = this._workflow.getHightLightedToken();
				if (highlightedToken && this._parentView.getModel().isHighlighted() && token.isHighlighted()) {
					return "docPathSelected";
				} else {
					return "";
				}
			}
		};

		/**
		 * The popup is being moved around make sure the parentView is set to being interactive.
		 */
		WebCenterInformationPopup.prototype._startedMoving = function () {
			this._scene.setInteractive(this._parentView.getId(), true);
		};

		/**
		 * The popup has stopped moving around make sure the parentView is no longer marked as interactive.
		 */
		WebCenterInformationPopup.prototype._stoppedMoving = function () {
			this._scene.setInteractive(this._parentView.getId(), false);
			var position = this._element.position();
			var startPosition = this.getPopupLocation();
			//transform the point from world coordinates to pixels from scene div top left corner
			var loc = this._scene.getTransformation().transformedPoint(startPosition);
			this._movedSize = loc.sizeTo(new ShapeUtils.Point(position.left, position.top));
		};

		/**
		 * The popup has hidden itself (close button)
		 */
		WebCenterInformationPopup.prototype._closed = function () {
			var highlightedToken = this._workflow.getHightLightedToken();
			if (highlightedToken && _.contains(this.getTokens(), highlightedToken)) {
				this._scene.tokenItemSelected(null);
			}
		};

		/**
		 * Get the location of the popup in world coordinates
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterInformationPopup.prototype.getLocation = function () {
			var location = DraggablePopup.prototype.getLocation.call(this);
			var t = this._scene.getTransformation().createInverse();
			return t.transformedPoint(location);
		};

		/**
		 * Executed when a item of the token is clicked and needs to be highlighted.
		 * @param {DOMObject} el
		 * @param {Event} event
		 */
		WebCenterInformationPopup.prototype._tokenItemClicked = function (el, event) {
			var tokenId = $(el).attr("data-bind-tokenId");
			var versionId = $(el).attr("data-bind-versionId");
			var token = this._workflow.getToken(tokenId);

			this._scene.tokenItemSelected(token, versionId);
		};

		/**
		 * Handle the context menu click for the token. Initializes the view for the context menu.
		 * @param {JQueryObj} el
		 * @param {Object}   e
		 */
		WebCenterInformationPopup.prototype._handleTokenItemContextMenu = function (el, e) {
			// Make it stay open
			this._draggableStart();
			this._draggableStop();

			// Select the document
			this._tokenItemClicked(el, e);

			var versionId = $(el).attr("data-bind-versionId");
			var tokenId = $(el).attr("data-bind-tokenId");
			var token = this._workflow.getToken(tokenId);
			if (!this._contextMenu) {
				this._contextMenu = new WebCenterDocumentContextMenu(this._scene);
			}
			var sceneOffset = this._scene.getContext().offset();
			this._contextMenu.show(token.getPin(), versionId, tokenId, new ShapeUtils.Point(e.clientX - sceneOffset.left + window.pageXOffset, e.clientY - sceneOffset.top + window.pageYOffset));
			e.preventDefault();
			e.stopPropagation();
		};

		/**
		 * gets the title of the popup, should be implemented by specific popup
		 * @returns {String}
		 */
		WebCenterInformationPopup.prototype.getTitle = function () {};

		/**
		 * Gets the location of the popup, should be implemented by specific popup
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterInformationPopup.prototype.getPopupLocation = function () {};

		/**
		 * Gets the tokens that should be added to the popup, should be implemented by specific popup
		 * @returns {Array.<Token>}
		 */
		WebCenterInformationPopup.prototype.getTokens = function () {};

		return WebCenterInformationPopup;

	});

define('WFEditorWCR/webcenterInformationPopup/WebCenterPinInformationPopup',[
		"WFEditorWCR/webcenterInformationPopup/WebCenterInformationPopup"
	],
	function (
		WebCenterInformationPopup
	) {
		"use strict";

		/**
		 * Pin information is shown in a movable popup. It shows the tokens and their documents
		 * as a list in the popup. The color is the same as the pin color. The popup is part of
		 * the pin. The popup extends WebCenterInformationPopup in the WFEditor framework.
		 *
		 * @exports WebCenterPinInformationPopup
		 * @class
		 * @augments WebCenterInformationPopup
		 *
		 * @param {WebCenterPinView} pinView
		 * @param {WebCenterPinModel} pinModel
		 * @param {Scene} scene
		 */
		var WebCenterPinInformationPopup = function (pinView, pinModel, scene) {
			WebCenterInformationPopup.call(this, pinModel.getWorkflow(), scene, pinView.getPinColor(), pinView);

			/**
			 * @member {WebCenterPinView} view object of the pin
			 * @private
			 */
			this._pinView = pinView;

			/**
			 * @member {WebCenterPin} model of the pin, used to fetch the tokens from
			 * @private
			 */
			this._pinModel = pinModel;
		};

		WebCenterPinInformationPopup.prototype = Object.create(WebCenterInformationPopup.prototype);
		WebCenterPinInformationPopup.prototype.constructor = WebCenterPinInformationPopup;

		/**
		 * @override
		 * The title of the popup is the label of the pin
		 */
		WebCenterPinInformationPopup.prototype.getTitle = function () {
			return this._pinModel.getLabel();
		};

		/**
		 * @override
		 * the location of the popup is next to the pin
		 */
		WebCenterPinInformationPopup.prototype.getPopupLocation = function () {
			var location = this._pinView.getLocation();
			location.offsetXY(10, 4);
			return location;
		};

		/**
		 * @override
		 * the tokens in the popup are the tokens of the pin
		 */
		WebCenterPinInformationPopup.prototype.getTokens = function () {
			return this._pinModel.getTokens();
		};


		/**
		 * @Override
		 */
		WebCenterPinInformationPopup.prototype.show = function () {
			this._pinView.setPopupOpen(true);
			WebCenterInformationPopup.prototype.show.call(this);
		};

		/**
		 * @Override
		 */
		WebCenterPinInformationPopup.prototype._closed = function () {
			WebCenterInformationPopup.prototype._closed.call(this);

			this._pinView.setPopupOpen(false);
		};
		return WebCenterPinInformationPopup;

	});

define('WFEditorWCR/workflow/pin/WebCenterPinView',[
		"utils/ShapeUtils",
		"WFEditor/workflow/pin/PinView",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/scene/WebCenterViewConstants",
		"WFEditorWCR/webcenterInformationPopup/WebCenterPinInformationPopup",
		"utils/Animatable",
		"WFEditor/scene/hitarea/HitArea",
		"utils/webi18n!WFEditorWCR",
		"underscore"
	],
	function (
		ShapeUtils,
		PinView,
		WebCenterPinConstants,
		WebCenterViewConstants,
		WebCenterPinInformationPopup,
		Animatable,
		HitArea,
		webi18n,
		_
	) {
		"use strict";

		/**
		 * The view for a webcenter pin item.
		 *
		 * @exports WebCenterPinView
		 * @class
		 * @augments PinView
		 *
		 * @param {WebCenterPin} pinModel - model for the pin
		 * @param {WebCenterNodeView} nodeView - view for the node too which the pin belongs
		 * @param {Scene} scene
		 */
		var WebCenterPinView = function (pinModel, nodeView, scene) {
			PinView.call(this, pinModel, nodeView);

			this._scene = scene;

			/**
			 * @member {SceneModel}
			 * @private
			 */
			this._sceneModel = scene.getSceneModel();

			/**
			 * @member{Boolean} Is the popup opened
			 * @private
			 */
			this._isPopupOpen = false;


			/**
			 * @member {ShapeUtils.Rect} the rectangle of the lifeline between pin and popup (if it is being drawn)
			 * @private
			 */
			this._popupLifelineRect = null;

			/**
			 * @member {Boolean} has the location of the popup changed?
			 * @private
			 */
			this._isPopupLocationDirty = false;

			this.addAnimatableProperties({
				pinRadius: function () {
					if (pinModel.isHovered()) {
						return WebCenterViewConstants.PinRadiusHovered;
					} else {
						return WebCenterViewConstants.PinRadius;
					}
				}
			});

			this._observe(this._model, "changed.*", this.makeDirty);
			this._observe(this._model, "changed.hover", function () {
				if (this._model.getPinType() !== WebCenterPinConstants.Type.In) { // only animate output pins
					var previousPinRadius = !this._model.isHovered() ? WebCenterViewConstants.PinRadiusHovered : WebCenterViewConstants.PinRadius;
					this.animate("pinRadius", previousPinRadius, 350, "easeOutQuart");
				}
				this._hoverChanged();
			});
			this._observe(this._model, "Token.*.changed.select", this.makeDirty);
			this._observe(this._model, "Token.added", function () {
				this._updateBoundingRect();
				this._updateHitAreas();
				this.makeDirty();
			});
			this._observe(this._model.getNode(), "Pin.*", this._updateHitAreas);

			// observe node location changes
			this._observe(this._model.getNode(), "changed.location", this._locationChanged);
			// observe edges added
			_.each(this._model.getEdges(), function (edge) {
				this._edgeAdded(edge);
			}, this);
			this._observe(this._model, "Edge.added", this._edgeAdded);
			this._observe(this._model, "Edge.removed", this._edgeRemoved);

		};

		WebCenterPinView.prototype = Object.create(PinView.prototype);
		WebCenterPinView.prototype.constructor = WebCenterPinView;
		Animatable.mixin(WebCenterPinView.prototype);


		/**
		 * @override
		 * Renders the node on the provided canvas context. Only renders if the icon is available.
		 * If the icon is not yet loaded, the iconstorage fetches the icon and the layer is marked dirty
		 * when the icon is correctly fetched.
		 *
		 * @param   {CanvasRenderingContext2D} context
		 * @param   {DefaultLayer} layer
		 * @param	{Boolean} quickRender
		 */
		WebCenterPinView.prototype.render = function (context, layer, quickRender) {
			if (quickRender) {
				// Edges shouldn't be rendered in quickRender mode
				return;
			}

			// pin rendering
			context.save();
			context.translate(-0.5, -0.5);

			var location = this._nodeView.getLocationForPin(this._model);

			if (this._popup) {
				if (this._isPopupLocationDirty && this._popup.isVisible()) {
					this._popup.relocate();
					this._isPopupLocationDirty = false;
				}
				if (this._shouldShowPopup()) {

					if (this._popup.isMoved() && !this._isPopupLocationDirty) {
						context.save();
						context.strokeStyle = this.getPinColor();
						context.beginPath();
						context.moveTo(location.x, location.y);
						var popupLocation = this._popup.getLocation();
						context.lineTo(popupLocation.x, popupLocation.y);
						context.stroke();
						context.restore();

						this._popupLifelineRect = new ShapeUtils.Rect(location.copy());
						this._popupLifelineRect.size = location.sizeTo(popupLocation);
					} else {
						this._popupLifelineRect = null;
					}

					if (this._popup.isVisible()) {
						this._popup.render();
					}

					// make sure to update the bounding rect if there is a lifeline :)
					this._updateBoundingRect();
				} else if (this._popup) {
					this._popup.hide();
				}
			}


			// draw everything from location as origin
			context.translate(location.x, location.y);


			var pinType = this._model.getPinType();
			if (pinType === WebCenterPinConstants.Type.In) {

				//don't draw input pin

			} else {
				// output pin
				context.beginPath();
				context.lineWidth = 1;

				switch (pinType) {
				case WebCenterPinConstants.Type.OutError:
					context.fillStyle = WebCenterViewConstants.PinErrorBackground;
					context.strokeStyle = WebCenterViewConstants.PinErrorBorder;
					break;
				case WebCenterPinConstants.Type.OutWarning:
					context.fillStyle = WebCenterViewConstants.PinWarningBackground;
					context.strokeStyle = WebCenterViewConstants.PinWarningBorder;
					break;
				default:
					context.fillStyle = WebCenterViewConstants.PinOKBackground;
					context.strokeStyle = WebCenterViewConstants.PinOKBorder;
					break;
				}
				var radius = this.getValue("pinRadius");
				var pinTextWidth = 0;
				if (this._model.hasTokens()) {
					context.save();

					if (this._model.isHovered()) {
						context.font = WebCenterViewConstants.PinCharacterFeedbackFontHovered;
					} else {
						context.font = WebCenterViewConstants.PinCharacterFont;
					}



					context.textBaseline = "middle";
					context.textAlign = "left";
					pinTextWidth = context.measureText(this._model.getNumberForPin()).width;

					context.beginPath();
					context.moveTo(0, radius);
					context.arc(0, 0, radius, Math.PI / 2, -Math.PI / 2);
					context.lineTo(pinTextWidth, -radius);
					context.arc(pinTextWidth, 0, radius, -Math.PI / 2, Math.PI / 2);
					context.closePath();

					// first fill path with white opaque background
					this._renderOpaqueBackground(context);

					context.fill();
					context.stroke();
					if (pinType === WebCenterPinConstants.Type.OutError) {
						context.fillStyle = WebCenterViewConstants.PinErrorCharacterColor;
					} else {
						context.fillStyle = WebCenterViewConstants.PinOKCharacterColor;
					}

					context.fillText(this._model.getNumberForPin(), 0, 0);

					context.restore();
				} else {
					context.arc(0, 0, radius, 0, 2 * Math.PI, false);

					// first fill path with white opaque background
					this._renderOpaqueBackground(context);

					context.fill();
					context.stroke();

					if (pinType === WebCenterPinConstants.Type.OutError || pinType === WebCenterPinConstants.Type.OutWarning) {
						// render x or ! in the pin icon
						context.save();

						if (pinType === WebCenterPinConstants.Type.OutWarning) {
							// draw an exclamation mark
							var markPoint = Math.max(1, Math.floor(0.25 * radius));

							context.beginPath();
							context.moveTo(0, markPoint);
							context.lineTo(-0.5 * markPoint, -3.5 * markPoint);
							context.lineTo(0.5 * markPoint, -3.5 * markPoint);
							context.lineTo(0, markPoint);
							context.strokeStyle = WebCenterViewConstants.PinWarningCharacterColor;
							context.lineWidth = 1;
							context.fillStyle = WebCenterViewConstants.PinWarningCharacterColor;
							context.fill();
							context.stroke();

							context.beginPath();
							context.arc(0, 3.5 * markPoint, 0.75 * markPoint, 0, 2 * Math.PI, false);
							context.fill();


						} else {
							// draw a cross
							var crossSize = Math.floor(0.8 * radius);
							context.rotate(0.25 * Math.PI);
							context.beginPath();
							context.moveTo(-crossSize, 0);
							context.lineTo(crossSize, 0);
							context.moveTo(0, -crossSize);
							context.lineTo(0, crossSize);
							context.strokeStyle = WebCenterViewConstants.PinErrorCharacterColor;
							context.lineWidth = 1;
							context.stroke();
						}
						context.restore();
					}

				}

				if ((this._model.isHovered() || this._model.isConnectedEdgeHovered()) && this.getLabel() && !(this._popup && this._popup.isVisible())) {
					// draw label	
					context.fillStyle = WebCenterViewConstants.PinLabelColor;
					context.font = WebCenterViewConstants.PinLabelFont;
					context.textAlign = "left";

					context.fillText(this.getLabel(),
						radius + pinTextWidth + WebCenterViewConstants.PinLabelOffsetX, -radius + WebCenterViewConstants.PinLabelOffsetY);
				}
			}
			context.restore();
		};

		/**
		 * The location was changed of the pin view
		 * update hit areas, bounding rect and make the view dirty
		 */
		WebCenterPinView.prototype._locationChanged = function () {
			this._updateBoundingRect();
			this._updateHitAreas();
			this.makeDirty();
		};

		/**
		 * @Override
		 */
		WebCenterPinView.prototype._updateHitAreas = function () {
			var pinLocation = this._nodeView.getLocationForPin(this._model);
			var radius = WebCenterViewConstants.PinRadiusHovered + 1;
			var pinRect;

			var textWidth = 0;

			if (this._model.hasTokens() && this._model.getPinType() !== WebCenterPinConstants.Type.In) {

				// add size of token number text
				this._context.save();
				this._context.font = WebCenterViewConstants.PinCharacterFontHovered;
				this._context.textAlign = "left";
				textWidth = this._context.measureText(this._model.getNumberForPin()).width;

				this._context.restore();
			}

			pinRect = new ShapeUtils.Rect(pinLocation.copy(), new ShapeUtils.Size(2 * radius + textWidth, 2 * radius));
			pinRect.origin.offsetXY(-radius, -radius);

			this._setHitAreas([new HitArea(pinRect, this, [], undefined, {
				cursor: "pointer-cursor"
			})]);

		};

		/**
		 * Get the location of the pin.
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterPinView.prototype.getLocation = function () {
			return this._nodeView.getLocationForPin(this._model);
		};

		/**
		 * Get the location of the edge if all edges arriving in this pin are seperated.
		 * Returns the location of the endpoint of the edge, the location of the input pin does
		 * not change.
		 *
		 * @param {WebCenterEdgeView} the view of the edge the correct locatin needs to be returned for
		 * @param {Boolean} [isSplit=false] - The 'previous' value of isSplit in the model
		 * @returns {ShapeUtils.Point} the location of the end point of the provided edge
		 */
		WebCenterPinView.prototype.getEdgeLocation = function (edge, isSplit) {
			if ((isSplit !== undefined && !isSplit) || this._model.getNumberOfEdges() <= 1) {
				return this.getLocation();
			}

			var edgeViews = this._calculateEdgeOrder();

			var index = _.indexOf(edgeViews, edge);
			var location = this.getLocation();
			return new ShapeUtils.Point(location.x, location.y + (WebCenterViewConstants.PinRadius * 3.5 * index));
		};

		/**
		 * Get the view of the first edge in the pin.
		 *
		 * @returns {EdgeView}
		 */
		WebCenterPinView.prototype.getFirstEdge = function () {
			return this._calculateEdgeOrder()[0];
		};

		/**
		 * Calculate the order of the edges arriving in the pin.
		 * @returns {Array.<EdgeView>}
		 */
		WebCenterPinView.prototype._calculateEdgeOrder = function () {
			var edges = this._model.getEdges();
			var edgeViews = [];
			_.each(edges, function (edge) {
				var edgeView = this._scene.getView(edge.getId());
				if (edgeView) {
					edgeViews.push(edgeView);
				}
			}, this);
			var toLocation = this.getLocation();
			return _.sortBy(edgeViews, function (viewObj) {
				var fromPin = viewObj.getFromPin();
				var fromLocation = fromPin.getLocation();
				// the angle between the zero line (= through the toPoint parallel with the x-absces) and the toPoint
				var vector = new ShapeUtils.Vector(fromLocation.x - toLocation.x, -(fromLocation.y - toLocation.y));
				var angle = (vector.getAngleInDegrees() + 360) % 360;
				return angle;
			});
		};

		/**
		 * Returns the label if it should be shown (empty string if not)
		 * @returns {String}
		 */
		WebCenterPinView.prototype.getLabel = function () {
			return this._model.getLabel();
		};

		/**
		 * Returns the color used for this pin (stirng in #001122 format)
		 * @returns {String}
		 */
		WebCenterPinView.prototype.getPinColor = function () {
			var color = null;
			if (this._model.getPinType() === WebCenterPinConstants.Type.OutError ||
				this._model.getPinType() === WebCenterPinConstants.Type.WorkflowConstructError) {
				color = WebCenterViewConstants.PinErrorBackground;
			} else if (this._model.getPinType() === WebCenterPinConstants.Type.OutWarning) {
				color = WebCenterViewConstants.PinWarningBackground;
			} else {
				color = WebCenterViewConstants.PinOKBackground;
			}
			return color;
		};

		/**
		 * @override
		 */
		WebCenterPinView.prototype._updateBoundingRect = function () {
			var location = this._nodeView.getLocationForPin(this._model);

			var radius = WebCenterViewConstants.PinRadiusHovered + 1; // take in to account border (+1)
			// default bounding rect for Pin
			var boundingRect = new ShapeUtils.Rect(location.copy(), new ShapeUtils.Size(radius * 2, radius * 2));
			boundingRect.origin.x -= radius;
			boundingRect.origin.y -= radius;

			if (this._model.hasTokens()) {
				// add size of token number text
				this._context.save();
				this._context.font = WebCenterViewConstants.PinCharacterFontHovered;
				this._context.textAlign = "left";
				var width = this._context.measureText(this._model.getNumberForPin()).width;
				boundingRect.size.width += width;
				this._context.restore();
			}

			if (this.getLabel()) {
				//calculate label size and adapt boundingrect
				this._context.save();
				this._context.fillStyle = WebCenterViewConstants.PinLabelColor;
				this._context.font = WebCenterViewConstants.PinLabelFont;
				this._context.textAlign = "left";

				var textMetrics = this._context.measureText(this.getLabel());
				var labelYOffset = WebCenterViewConstants.PinLabelOffsetY + WebCenterViewConstants.PinLabelHeight;

				boundingRect.origin.y -= labelYOffset;
				boundingRect.size.height = boundingRect.size.height + labelYOffset;

				boundingRect.size.width = Math.max(boundingRect.size.width, textMetrics.width + WebCenterViewConstants.PinLabelOffsetX + 2 * WebCenterViewConstants.PinRadiusHovered);
				boundingRect.size.height = Math.max(boundingRect.size.height, WebCenterViewConstants.PinLabelHeight);

				this._context.restore();
			}

			if (this._popupLifelineRect && this._popup.isVisible()) {
				boundingRect = boundingRect.calculateArea(this._popupLifelineRect);
			}

			this._setBoundingRect(boundingRect.getNormalizedRectArray());
		};


		/**
		 * Get the location for a certain active token
		 * @param {Token} token for which the location needs to be returned
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterPinView.prototype.getLocationForToken = function (token) {
			var tokenLocation = this._nodeView.getLocationForPin(this._model).copy();
			var activeTokens = this._model.getActiveTokens();
			var i = 0;
			var found = false;
			// Find the sequence number of the token
			for (var j = 0; j < activeTokens.length && !found; j++) {
				if (token.getId() === activeTokens[j].getId()) {
					found = true;
				} else {
					i++;
				}
			}
			if (found) {
				tokenLocation.y -= WebCenterViewConstants.NodeStatusGap;
				tokenLocation.y -= i * WebCenterViewConstants.StatusToStatusGap;
				tokenLocation.x += WebCenterViewConstants.PinInToNodeLeft;
				return tokenLocation;
			}

		};

		/**
		 * An output pin has a higher z-index (default 30) than an input-pin (under the edge, so 5)
		 */
		WebCenterPinView.prototype.getZIndex = function () {
			if (this._model.getPinType() === WebCenterPinConstants.Type.In) {
				// input pin
				return 5;
			} else {
				// output pin
				return 30;
			}
		};

		/**
		 * Mark the popup open
		 */
		WebCenterPinView.prototype.setPopupOpen = function (isPopupOpen) {

			if (this._isPopupOpen !== isPopupOpen) {
				if (isPopupOpen) {
					this._setSceneModelListeners();
				} else {
					this._stopObserveObject(this._sceneModel);
				}
				this._isPopupOpen = isPopupOpen;
				this.makeDirty();
			}
		};

		/**
		 * React when an edge has been added to the model
		 * @param {Edge} edge - The added edge
		 */
		WebCenterPinView.prototype._edgeAdded = function (edge) {
			this._observe(edge, "changed.hover", this.makeDirty);
		};

		/**
		 * React when an edge has been removed from the model
		 * @param {Edge} edge - The removed edge
		 */
		WebCenterPinView.prototype._edgeRemoved = function (edge) {
			this._stopObserveObject(edge);
		};

		/**
		 * Hover statechanged
		 */
		WebCenterPinView.prototype._hoverChanged = function () {
			if (!this._popup && this._model.isHovered() && this._shouldShowPopup()) {
				this._popup = new WebCenterPinInformationPopup(this, this._model, this._scene);
				this._observe(this._popup, "changed.hover", this._hoverChanged);
			}

			if (this._popup) {
				this._popup.cancelDeferedClose();

				if (this._model.isHovered() && this._model.hasTokens() && this._model.getPinType() !== WebCenterPinConstants.Type.In) {
					this._popup.show();
				}

				if (!this._popup.isHovered() && !this._model.isHovered() && !this._popup.isSticky()) {
					this._popup.deferredClose();
				}
			}

		};

		/**
		 * Should the pin show a popup on hover?
		 * @returns {Boolean} 
		 */
		WebCenterPinView.prototype._shouldShowPopup = function () {
			return this._model.hasTokens() && this._model.getPinType() !== WebCenterPinConstants.Type.In;
		};

		/**
		 * Listen to sceneModel for panning and zoom changes
		 */
		WebCenterPinView.prototype._setSceneModelListeners = function () {
			this._observe(this._sceneModel, "panningOffsetChanged", _.bind(function () {
				if (this._popup && this._popup.isVisible()) {

					// defer to ensure transformation has been done. The relocation of the popup doesn't happen in the render
					// method because if the pin isn't in the visible rect anymore, the render method won't be called
					_.defer(_.bind(function () {
						this._popup.relocate();
						this.makeDirty();
					}, this));
				}
			}, this));

			this._observe(this._sceneModel, "zoomChanged", _.bind(function () {
				if (this._popup && this._popup.isVisible()) {
					// for zooming the change in location will be very big so we temporarily don't want to render the 
					// lifeline for popups as it will cause it to visibly jump from one position to another
					this._isPopupLocationDirty = true;

					// defer to ensure transformation has been done.
					_.defer(_.bind(function () {
						this._popup.relocate();
						this.makeDirty();
						this._isPopupLocationDirty = false;
					}, this));
				}
			}, this));
		};


		return WebCenterPinView;

	});

define('WFEditor/workflow/edge/EdgeView',[
		"WFEditor/scene/AbstractView"
	],
	function (
		AbstractView
	) {
		"use strict";

		/**
		 * EdgeView is a view object for rendering edges on a canvas. For specific edges,
		 * the render methods needs to be implemented.
		 *
		 * @exports EdgeView
		 * @class
		 * @augments AbstractView
		 *
		 * @param {Edge} edgeModel
		 * @param {PinView} fromPinView - view of the pin where the edge starts
		 * @param {PinView} toPinView - view of the pin where the edge stops
		 */
		var EdgeView = function (edgeModel, fromPinView, toPinView) {
			AbstractView.call(this, edgeModel);

			/**
			 * @member {PinView}
			 * @private
			 */
			this._fromPinView = fromPinView;

			/**
			 * @member {PinView}
			 * @private
			 */
			this._toPinView = toPinView;
		};

		EdgeView.prototype = Object.create(AbstractView.prototype);
		EdgeView.prototype.constructor = EdgeView;

		/**
		 * Eges are drawn at z-index 10 by default
		 * @Override
		 */
		EdgeView.prototype.getZIndex = function () {
			return 10;
		};

		/**
		 * Get the pin view of the pin where the edge is coming from
		 * @returns {PinView}
		 */
		EdgeView.prototype.getFromPinView = function () {
			return this._fromPinView;
		};

		/**
		 * Get the pin view of the pin where the edge is going to
		 * @returns {PinView}
		 */
		EdgeView.prototype.getToPinView = function () {
			return this._toPinView;
		};

		return EdgeView;

	});

define('WFEditorWCR/workflow/edge/WebCenterEdgeView',[
		"utils/ShapeUtils",
		"utils/Observer",
		"WFEditor/workflow/edge/EdgeView",
		"utils/Animatable",
		"WFEditor/tools/drawEdge/MouseNode",
		"WFEditorWCR/scene/WebCenterViewConstants",
		"WFEditor/scene/hitarea/HitArea",
		"lib/bezier",
		"underscore"
	],
	function (
		ShapeUtils,
		Observer,
		EdgeView,
		Animatable,
		MouseNode,
		WebCenterViewConstants,
		HitArea,
		Bezier,
		_
	) {
		"use strict";

		/**
		 * The view for a webcenter edge in the workflow. An edge connects two pins and has
		 * an arrow to indicate direction.
		 *
		 * @exports WebCenterEdgeView
		 * @class
		 * @augments EdgeView
		 *
		 * @parma {WebCenterEdge} edgeModel - model for the edge
		 * @param {WebCenterPinView} fromPinView - view of the pin where the edge starts
		 * @param {WebCenterPinView} toPinView - view of the pin where the edge stops
		 */
		var WebCenterEdgeView = function (edgeModel, fromPinView, toPinView) {
			EdgeView.call(this, edgeModel, fromPinView, toPinView);


			this.addAnimatableProperties({
				edgeLocationX: _.bind(function () {
					return this._toPinView.getEdgeLocation(this, this._toPinView.getModel().isSplitEdges()).x;
				}, this),
				edgeLocationY: _.bind(function () {
					return this._toPinView.getEdgeLocation(this, this._toPinView.getModel().isSplitEdges()).y;
				}, this)
			});

			/**
			 * @member {ShapeUtils.Point} startpoint for which _bezier is calculated
			 * @private
			 */
			this._startPoint = null;

			/**
			 * @member {ShapeUtils.Point} endpoint for which _bezier is calculated
			 * @private
			 */
			this._endPoint = null;

			/**
			 * @member {Bezier} the bezier curve for this edge
			 * @private
			 */
			this._bezier = null;

			this._updateBezier();




			// Listen to changes on the to pin
			this._observe(toPinView.getModel(), "changed.splitEdges", this._splitEdgesChanged);
			this._observe(this._model, "changed.*", this.makeDirty);
			this._observe(this._model.getFromPin().getNode(), "Pin.*", this.makeDirty);
			this._observe(this._model, "Pin.*.Token.*changed.select", this.makeDirty);

			// Listen to node location changes of the pins of the edge
			this._observe(this._model.getToPin().getNode(), "changed.location", this._endPointsChanged);
			this._observe(this._model.getToPin(), "Edge.*", function () {
				this._updateBoundingRect();
				this._updateHitAreas();
				this.makeDirty();
			});
			this._observe(this._model.getFromPin().getNode(), "changed.location", this._endPointsChanged);
		};

		WebCenterEdgeView.prototype = Object.create(EdgeView.prototype);
		WebCenterEdgeView.prototype.constructor = WebCenterEdgeView;

		// mixin Animatable in the prototype of WebCenterEdgeView
		Animatable.mixin(WebCenterEdgeView.prototype);

		/**
		 * @override
		 * Renders the edge on the provided canvas context.
		 *
		 * @param   {CanvasRenderingContext2D} context
		 * @param   {Layer} layer
		 * @param 	{Boolean} quickRender
		 */
		WebCenterEdgeView.prototype.render = function (context, layer, quickRender) {
			if (quickRender) {
				// Edges shouldn't be rendered in quickRender mode
				return;
			}
			var isHighlighted = this._model.isHighlighted();


			context.save();
			context.translate(-0.5, -0.5);


			this._updateBezier();


			//render main thick dark bezier curve
			context.beginPath();
			if (isHighlighted) {
				context.strokeStyle = WebCenterViewConstants.HighlightedEdgeDarkColor;
			} else {
				context.strokeStyle = WebCenterViewConstants.EdgeDarkColor;
			}
			context.lineWidth = 4;
			this._renderBezierCurve(context, this._bezier);
			context.stroke();
			context.closePath();

			//render main thin lighter bezier curve
			context.beginPath();
			if (isHighlighted) {
				context.strokeStyle = WebCenterViewConstants.HighLightedEdgeLightColor;
			} else {
				context.strokeStyle = WebCenterViewConstants.EdgeLightColor;
			}
			context.lineWidth = 2;
			this._renderBezierCurve(context, this._bezier);
			context.stroke();
			context.closePath();

			//draw arrow head
			context.beginPath();
			if (isHighlighted) {
				context.strokeStyle = WebCenterViewConstants.HighlightedEdgeDarkColor;
				context.fillStyle = WebCenterViewConstants.HighlightedEdgeLightColor;
			} else {
				context.strokeStyle = WebCenterViewConstants.EdgeDarkColor;
				context.fillStyle = WebCenterViewConstants.EdgeLightColor;
			}

			if (this.getModel().isHovered()) {
				context.lineWidth = 3;
			} else {
				context.lineWidth = 2;
			}

			var endLocation = new ShapeUtils.Point(this.getValue("edgeLocationX"), this.getValue("edgeLocationY"));
			context.moveTo(endLocation.x + 5, endLocation.y);
			context.lineTo(endLocation.x - 5, endLocation.y - 4);
			context.quadraticCurveTo(endLocation.x - 2, endLocation.y, endLocation.x - 5, endLocation.y + 4);
			context.closePath();

			context.stroke();
			this._renderOpaqueBackground(context);
			context.fill();


			context.restore();
		};

		/**
		 * The end points have changed
		 * update the hit areas and bounding rects
		 */
		WebCenterEdgeView.prototype._endPointsChanged = function () {
			// we want some new beziers
			this._updateBezier();
			this._updateBoundingRect();
			if (!this.isAnimating()) {
				this._updateHitAreas();
			}
			this.makeDirty();

		};

		/**
		 * @override
		 */
		WebCenterEdgeView.prototype._updateHitAreas = function () {
			var endLocation;
			if (!this._toPinView.getModel().isSplitEdges()) {
				// from the begin point
				endLocation = this._toPinView.getLocation();
			} else {
				// from the end point
				endLocation = this._toPinView.getEdgeLocation(this, true);
			}

			var pinRect = new ShapeUtils.Rect(new ShapeUtils.Point(endLocation.x - 10, endLocation.y - 9), new ShapeUtils.Size(20, 18));
			this._setHitAreas([new HitArea(pinRect, this, [], undefined, {cursor: "pointer-cursor"})]);
		};

		/**
		 * Generates a new bezier curve with the locations of the start and end pins.
		 * @private
		 * @returns {Bezier}
		 */
		WebCenterEdgeView.prototype._updateBezier = function () {
			var startLocation = this._fromPinView.getLocation();

			var endLocation = new ShapeUtils.Point(this.getValue("edgeLocationX"), this.getValue("edgeLocationY"));

			if (!this._bezier || !this._startPoint || !this._endPoint || !this._endPoint.equals(endLocation) || !this._startPoint.equals(startLocation)) {
				// bezier was never calculated, or start or end point has changed
				this._startPoint = startLocation;
				this._endPoint = endLocation;

				var bezierOffset = Math.max(50, Math.abs(endLocation.x - startLocation.x) / 2);

				this._bezier = new Bezier(startLocation.x, startLocation.y,
					startLocation.x + bezierOffset, startLocation.y,
					endLocation.x - bezierOffset, endLocation.y,
					endLocation.x, endLocation.y);
			}

		};

		/**
		 * Render the bezier curve on the canvas context.
		 * @param {CanvasRenderingContext2D} context
		 * @param {Bezier}   bezierCurve
		 */
		WebCenterEdgeView.prototype._renderBezierCurve = function (context, bezierCurve) {
			var p = bezierCurve.points;
			context.moveTo(p[0].x, p[0].y);
			if (p.length === 4) {
				context.bezierCurveTo(
					p[1].x, p[1].y,
					p[2].x, p[2].y,
					p[3].x, p[3].y
				);
			}
		};

		/**
		 * @override
		 */
		WebCenterEdgeView.prototype._updateBoundingRect = function () {
			var boundingBox = this._bezier.bbox();
			var rect = new ShapeUtils.Rect(new ShapeUtils.Point(boundingBox.x.min - 4, boundingBox.y.min - 4),
				new ShapeUtils.Size(boundingBox.x.size + 8, boundingBox.y.size + 8));

			// add arrow rect
			rect = rect.calculateArea(this.getArrowRect());

			this._setBoundingRect(rect.getNormalizedRectArray());
		};

		/**
		 * Get the rect of the arrow
		 */
		WebCenterEdgeView.prototype.getArrowRect = function () {
			if (!this._arrowRect) {
				this._arrowRect = new ShapeUtils.Rect();
			}

			this._arrowRect.origin.x = this.getValue("edgeLocationX", true) - 8;
			this._arrowRect.origin.y = this.getValue("edgeLocationY", true) - 7;
			this._arrowRect.size.width = 16;
			this._arrowRect.size.height = 14;

			return this._arrowRect;
		};

		/**
		 * Get the pin view the edge originates from.
		 * @returns {WebCenterPinView}
		 */
		WebCenterEdgeView.prototype.getFromPin = function () {
			return this._fromPinView;
		};

		/**
		 * Split edges changed -> animate
		 */
		WebCenterEdgeView.prototype._splitEdgesChanged = function () {
			var isSplitEdges = this._toPinView.getModel().isSplitEdges();
			if (this.isAnimating()) {
				// from the current point
				this.animate("edgeLocationY", this.getValue("edgeLocationY"), 500);
				// update the hit areas
				this._updateHitAreas();
			} else {
				if (isSplitEdges) {
					// from the begin point
					this.animate("edgeLocationY", this._toPinView.getLocation().y, 500);
					// update the hit areas
					this._updateHitAreas();
				} else {
					// from the end point
					this.animate("edgeLocationY", this._toPinView.getEdgeLocation(this, true).y, 500);
					this.once("animation.edgeLocationY.finished", _.bind(function () {
						this._updateHitAreas();
					}, this));
				}
			}



			this._updateBoundingRect();

		};

		return WebCenterEdgeView;

	});

define('WFEditor/workflow/token/TokenView',[
		"WFEditor/scene/AbstractView"
	],
	function (
		AbstractView
	) {
		"use strict";

		/**
		 * TokenView is a view object for rendering a view on a canvas. For specific Tokens,
		 * the render methods needs to be implemented.
		 *
		 * @exports TokenView
		 * @class
		 * @augments AbstractView
		 *
		 * @param {Token} tokenModel
		 */
		var TokenView = function (tokenModel) {
			AbstractView.call(this, tokenModel);
		};

		TokenView.prototype = Object.create(AbstractView.prototype);
		TokenView.prototype.constructor = TokenView;


		/**
		 * Tokens are drawn at z-index 40 by default
		 * @Override
		 */
		TokenView.prototype.getZIndex = function () {
			return 40;
		};


		return TokenView;

	});

define('WFEditorWCR/workflow/token/WebCenterTokenConstants',{
	/*
	 * Constants for tokens are defined here
	 */
	Icons: {
		WAIT: "task_file_hold_until.png",
		MULTIPLETASKS: "task_files.png",
		DEFAULT: "task_file.png"
	}

});

define('WFEditorWCR/webcenterInformationPopup/WebCenterTokenInformationPopup',[
		"WFEditorWCR/webcenterInformationPopup/WebCenterInformationPopup",
		"WFEditorWCR/scene/WebCenterViewConstants"
	],
	function (
		WebCenterInformationPopup,
		WebCenterViewConstants
	) {
		"use strict";

		/**
		 * The information of a token in progress is shown in a movable popup. It shows the tokens and their documents
		 * as a list in the popup. 
		 *
		 * @exports WebCenterTokenInformationPopup
		 * @class
		 * @augments WebCenterInformationPopup
		 *
		 * @param {WebCenterPinView} pinView
		 * @param {WebCenterToken} tokenModel
		 * @param {Scene} scene
		 * @param {TokenView} tokenView
		 */
		var WebCenterTokenInformationPopup = function (pinView, tokenModel, scene, tokenView) {
			WebCenterInformationPopup.call(this, tokenModel.getWorkflow(), scene, WebCenterViewConstants.TokenStatusTextColor, tokenView);

			/**
			 * @member {WebCenterPinView} the input 
			 * @private
			 */
			this._pinView = pinView;

			/**
			 * @member {WebCenterToken} model of the token
			 * @private
			 */
			this._tokenModel = tokenModel;

			/**
			 * @member {WebCenterTokenView} view object of the token
			 * @private
			 */
			this._tokenView = tokenView;


		};

		WebCenterTokenInformationPopup.prototype = Object.create(WebCenterInformationPopup.prototype);
		WebCenterTokenInformationPopup.prototype.constructor = WebCenterTokenInformationPopup;


		/**
		 * @override
		 * @returns {String} The title of the popup
		 */
		WebCenterTokenInformationPopup.prototype.getTitle = function () {
			return this._tokenView.getStatusText();
		};

		/**
		 * @override
		 * @returns {ShapeUtils.Point} The default location
		 */
		WebCenterTokenInformationPopup.prototype.getPopupLocation = function () {
			var point = this._pinView.getLocationForToken(this._tokenModel);
			point.offsetXY(16, 9);
			return point;
		};

		/**
		 * @override
		 * @returns {Array.Token}
		 */
		WebCenterTokenInformationPopup.prototype.getTokens = function () {
			if (this._tokenModel.getSubTokens) {
				return this._tokenModel.getSubTokens();
			} else {
				return [this._tokenModel];
			}
		};

		/**
		 * @Override
		 */
		WebCenterTokenInformationPopup.prototype.show = function () {
			this._tokenView.setPopupOpen(true);
			WebCenterInformationPopup.prototype.show.call(this);
		};

		/**
		 * @Override
		 */
		WebCenterTokenInformationPopup.prototype._closed = function () {
			WebCenterInformationPopup.prototype._closed.call(this);
			this._tokenView.setPopupOpen(false);
		};
		return WebCenterTokenInformationPopup;

	});

define('WFEditorWCR/workflow/token/WebCenterTokenView',[
		"WFEditor/workflow/token/TokenView",
		"WFEditor/utils/IconDataStorage",
		"utils/webi18n!WFEditorWCR",
		"utils/ShapeUtils",
		"utils/dateformatter",
		"WFEditorWCR/scene/WebCenterViewConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/workflow/token/WebCenterTokenConstants",
		"WFEditorWCR/webcenterInformationPopup/WebCenterTokenInformationPopup",
		"WFEditor/scene/hitarea/HitArea",
		"underscore"
	],
	function (
		TokenView,
		IconDataStorage,
		webi18n,
		ShapeUtils,
		DateFormatter,
		WebCenterViewConstants,
		WebCenterNodeConstants,
		WebCenterTokenConstants,
		WebCenterTokenInformationPopup,
		HitArea,
		_

	) {
		"use strict";

		/**
		 *
		 * @exports WebCenterTokenView
		 * @class
		 * @augments TokenView
		 *
		 * @param {WebCenterTokenModel} tokenModel
		 * @param {WebCenterPinView} pinView
		 * @param {Scene} scene
		 */
		var WebCenterTokenView = function (tokenModel, pinView, scene) {
			TokenView.call(this, tokenModel);

			/**
			 * @member {PinView}
			 * @private
			 */
			this._pinView = pinView;

			/**
			 * @member {WebCenterTokenInformationPopup}
			 * @private
			 */
			this._popup = null;

			/**
			 * @member{Boolean} Is the popup opened
			 * @private
			 */
			this._isPopupOpen = false;

			var wcrConfig = this._model.getWorkflow().getWebCenterConfig();

			/**
			 * @member {String}
			 * @private
			 */
			this._fileIconUrl = wcrConfig.requireBaseUrl + "WFEditorWCR/icons/";

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = scene;

			/**
			 * @member {SceneModel}
			 * @private
			 */
			this._sceneModel = scene.getSceneModel();

			/**
			 * @member {ShapeUtils.Rect} the rectangle of the lifeline between pin and popup (if it is being drawn)
			 * @private
			 */
			this._popupLifelineRect = null;

			/**
			 * @member {Boolean} has the location of the popup changed?
			 * @private
			 */
			this._isPopupLocationDirty = false;

			// location change of node
			this._observe(tokenModel.getPin().getNode(), "changed.location", this._locationChanged);
			this._observe(tokenModel, "changed.*", this.makeDirty);
			this._observe(tokenModel, "changed.hover", this._hoverChanged);

		};

		WebCenterTokenView.prototype = Object.create(TokenView.prototype);
		WebCenterTokenView.prototype.constructor = WebCenterTokenView;

		/**
		 * React on location changes of the node
		 */
		WebCenterTokenView.prototype._locationChanged = function () {
			this._updateBoundingRect();
			this._updateHitAreas();
			this.makeDirty();
		};

		/**
		 * @override
		 * Renders the a in progress token above its node, if hovered, also renders the file name
		 * @param {CanvasRenderingContext2D} context
		 * @param {Layer} layer
		 * @param {Boolean} quickRender - if true, only high priority render actions should be executed (i.e. during panning)
		 */
		WebCenterTokenView.prototype.render = function (context, layer, quickRender) {
			if (quickRender) {
				// Don't render tokens when quick rendering
				return;
			}
			var location = this._pinView.getLocationForToken(this._model);
			var iconStorage = IconDataStorage.getInstance(); //get the singleton instance of the datastorage
			//request the icon from the datastorage, returns the icon if available, if not it starts loading the icon.
			var icon = iconStorage.get(this._getTokenStatusFileIconUrl(), _.bind(function () {
				layer.makeDirty();
			}, this));



			// only create popup when needed
			if (this._popup) {

				if (this._popup.isMoved() && !this._isPopupLocationDirty) {
					context.save();
					context.strokeStyle = WebCenterViewConstants.TokenStatusTextColor;
					context.beginPath();
					context.moveTo(location.x + 9, location.y + 9);
					var popupLocation = this._popup.getLocation();
					context.lineTo(popupLocation.x, popupLocation.y);
					context.stroke();
					context.restore();

					this._popupLifelineRect = new ShapeUtils.Rect(location.copy());
					this._popupLifelineRect.size = location.sizeTo(popupLocation);
				} else {
					this._popupLifelineRect = null;
				}

				if (this._popup.isVisible()) {
					this._updateBoundingRect();
					this._popup.render();
				}
			}

			if (icon) { //only render if the icon was already fetched
				context.drawImage(icon, location.x, location.y);
			}

			context.save();

			var status = this.getStatusText(); // Which text should be displayed next to the file icon?
			context.font = WebCenterViewConstants.TokenSatusFont;
			location.offsetXY(WebCenterViewConstants.TokenStatusFileIconWidth, 12);
			this._renderTextBackground(context, status, location);
			context.fillStyle = WebCenterViewConstants.TokenStatusTextColor;
			context.globalAlpha = 1;
			context.fillText(status, location.x, location.y);
			location.offsetXY(-WebCenterViewConstants.TokenStatusFileIconWidth, 0);

			context.restore();
		};

		/**
		 * @override
		 */
		WebCenterTokenView.prototype._updateHitAreas = function () {
			// The hit area for a token is the location of the file icon
			var tokenLocation = this._pinView.getLocationForToken(this._model);
			var iconRect = new ShapeUtils.Rect(tokenLocation, new ShapeUtils.Size(WebCenterViewConstants.TokenStatusFileIconWidth, WebCenterViewConstants.TokenStatusFileIconWidth));
			this._setHitAreas([new HitArea(iconRect, this)]);
		};

		/**
		 * @override
		 */
		WebCenterTokenView.prototype._updateBoundingRect = function () {
			var origin = this._pinView.getLocationForToken(this._model);
			var size = new ShapeUtils.Size(WebCenterViewConstants.TokenStatusFileIconWidth, WebCenterViewConstants.TokenStatusFileIconWidth);
			var text = this.getStatusText();
			if (text) {
				this._context.save();
				this._context.font = WebCenterViewConstants.TokenSatusFont;
				var textWidth = this._context.measureText(text).width;

				size.width += textWidth;
				this._context.restore();
			}
			var boundingRect = new ShapeUtils.Rect(origin, size);

			if (this._popupLifelineRect && this._popup.isVisible()) {
				boundingRect = boundingRect.calculateArea(this._popupLifelineRect);
			}

			this._setBoundingRect(boundingRect.getNormalizedRectArray());
		};

		/**
		 * Renders the opaque white background for text
		 * @param {CanvasRenderingContext2D} context
		 * @param {String} text
		 * @param {ShapeUtils.Point} textLocation
		 */
		WebCenterTokenView.prototype._renderTextBackground = function (context, text, textLocation) {
			context.save();
			var width = context.measureText(text).width;
			context.fillStyle = "rgb(255,255,255)";
			context.globalAlpha = 0.5;
			context.fillRect(textLocation.x - 2, textLocation.y + 4, width + 4, -16);
			context.restore();
		};

		/**
		 * Returns the text that should be displayed for this token in progress
		 * @returns {String}
		 */
		WebCenterTokenView.prototype.getStatusText = function () {
			var nodeType = this._model.getPin().getNode().getNodeType();

			if (this._model.getSubtaskCount() > 1) {
				// If there are multiple subtasks, check how much are completed
				var subtasksCompleted = 0;
				_.each(this._model.getSubtasks(), function (subtask) {
					if (subtask.status === WebCenterNodeConstants.Status.COMPLETED) {
						subtasksCompleted++;
					}
				});
				return webi18n.t("TasksCompleted", subtasksCompleted, this._model.getSubtaskCount());
			}

			// One or Zero subtasks, the text is in function of the node
			switch (nodeType) {
			case WebCenterNodeConstants.Nodes.STARTAPPROVAL:
				return webi18n.t("WaitingApproval");
			case WebCenterNodeConstants.Nodes.DATACOLLECTOR:
				return webi18n.t("WaitingForTasks");
			case WebCenterNodeConstants.Nodes.JOIN:
				return webi18n.t("WaitingForTasks");
			case WebCenterNodeConstants.Nodes.WAIT:
				return webi18n.t("WaitingUntil", DateFormatter.formatDate(new Date(this._model.getTimer()), "NNN dd, yyyy HH:mm:ss"));
			case WebCenterNodeConstants.Nodes.SUBMITWORKFLOW:
				return webi18n.t("RunningOnAE");
			}
			return webi18n.t("InProgress"); // Defaults to "In Progress"   
		};

		/**
		 * Returns the url of the icon for this token
		 * @returns {String}
		 */
		WebCenterTokenView.prototype._getTokenStatusFileIconUrl = function () {
			var nodeType = this._model.getPin().getNode().getNodeType();
			if (nodeType === WebCenterNodeConstants.Nodes.WAIT) {
				return this._fileIconUrl + WebCenterTokenConstants.Icons.WAIT;
			}
			if (this._model.getSubtaskCount() > 1) {
				return this._fileIconUrl + WebCenterTokenConstants.Icons.MULTIPLETASKS;
			}
			return this._fileIconUrl + WebCenterTokenConstants.Icons.DEFAULT;

		};

		/**
		 * Mark the popup open
		 */
		WebCenterTokenView.prototype.setPopupOpen = function (isPopupOpen) {

			if (this._isPopupOpen !== isPopupOpen) {
				if (isPopupOpen) {
					this._setSceneModelListeners();
				} else {
					this._stopObserveObject(this._sceneModel);
				}
				this._isPopupOpen = isPopupOpen;
				this.makeDirty();
			}
		};

		/**
		 * @override
		 * clears the resources of the popup
		 */
		WebCenterTokenView.prototype.destroyView = function () {
			if (this._popup) {
				this._popup.destroyPopup();
				delete this._popup;
			}
		};


		/**
		 * Hover state of popup or token has changed
		 */
		WebCenterTokenView.prototype._hoverChanged = function () {
			if (this._popup) {
				this._popup.cancelDeferedClose();
			} else if (this._model.isHovered()) {
				this._popup = new WebCenterTokenInformationPopup(this._pinView, this._model, this._scene, this);
				this._popup.show();
				this._observe(this._popup, "changed.hover", this._hoverChanged);
				this._observe(this._popup, "changed.visible", function () {
					this.makeDirty();
					this._updateBoundingRect();
				});
				this.makeDirty();
			}

			if (this._popup) {
				if (!this._popup.isHovered() && !this._model.isHovered() && !this._popup.isSticky()) {
					this._popup.deferredClose();
				}

				if (this._popup.isHovered() || this._model.isHovered()) {
					this._popup.show();
				}
			}
		};


		/**
		 * Listen to sceneModel for panning and zoom changes
		 */
		WebCenterTokenView.prototype._setSceneModelListeners = function () {
			this._observe(this._sceneModel, "panningOffsetChanged", _.bind(function () {
				if (this._popup && this._popup.isVisible()) {

					// defer to ensure transformation has been done. The relocation of the popup doesn't happen in the render
					// method because if the pin isn't in the visible rect anymore, the render method won't be called
					_.defer(_.bind(function () {
						this._popup.relocate();
						this.makeDirty();
					}, this));
				}
			}, this));

			this._observe(this._sceneModel, "zoomChanged", _.bind(function () {
				if (this._popup && this._popup.isVisible()) {
					// for zooming the change in location will be very big so we temporarily don't want to render the 
					// lifeline for popups as it will cause it to visibly jump from one position to another
					this._isPopupLocationDirty = true;

					// defer to ensure transformation has been done.
					_.defer(_.bind(function () {
						this._popup.relocate();
						this.makeDirty();
						this._isPopupLocationDirty = false;
					}, this));
				}
			}, this));
		};



		return WebCenterTokenView;


	});

define('WFEditorWCR/workflow/helper/stickyNote/StickyNotePopup',["jquery",
	"underscore",
	"WFEditorWCR/workflow/node/WebCenterNodeConstants",
	"WFEditorWCR/scene/WebCenterViewConstants",
	"utils/ShapeUtils",
	"utils/Observable",
	"component/tooltip/Tooltip",
	"utils/webi18n!WFEditorWCR"
], function (
	$,
	_,
	WebCenterNodeConstants,
	WebCenterViewConstants,
	ShapeUtils,
	Observable,
	Tooltip,
	i18n) {

	"use strict";

	/**
	 * This popup is used for generating the html of a sticky note in the WFEditor
	 * @class
	 * @exports StickyNotePopup
	 * @mixes Observable
	 * @param {StickyNoteHelper} stickyNoteHelper 
	 * @param {Scene}            scene            
	 */
	var StickyNotePopup = function (stickyNoteHelper, scene) {
		Observable.mixin(this, ["Scene", "StickyNotePopup", stickyNoteHelper.getId()], "WFEditor." + scene.getWorkflowModel().getID());

		/**
		 * @member {StickyNoteHelper}
		 * @private
		 */
		this._stickyNote = stickyNoteHelper;

		/**
		 * @member {Scene}
		 * @private
		 */
		this._scene = scene;

		/**
		 * @member {JQueryObject} the encapsulating div
		 * @private
		 */
		this._stickyNoteDiv = $("<div/>").addClass("sticky-note");

		if (!stickyNoteHelper.getWorkflow().getPrivilege(WebCenterNodeConstants.Privileges.editStickyNotes)) {
			this._stickyNoteDiv.addClass("feedback-mode");
		}

		/**
		 * @member {JQueryObject} div for the header
		 * @private
		 */
		this._headerDiv = $("<div/>").addClass("sticky-note-header");


		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._editIcon = $("<span class='sticky-note-edit-icon'/>").append($("<i class='icon icon-WCR_Annotate'/>").css({
			"font-size": WebCenterViewConstants.StickyNoteHeaderHeight - 2 * WebCenterViewConstants.StickyNoteHeaderPadding
		})).css({
			padding: WebCenterViewConstants.StickyNoteHeaderPadding,
			"padding-left": WebCenterViewConstants.StickyNoteHeaderPadding
		});

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._deleteIcon = $("<span class='sticky-note-delete-icon'/>").append($("<i class='icon icon-WCR_Actions-Delete12'/>").css({
			"font-size": WebCenterViewConstants.StickyNoteHeaderHeight - 2 * WebCenterViewConstants.StickyNoteHeaderPadding
		})).css({
			padding: WebCenterViewConstants.StickyNoteHeaderPadding,
			"padding-right": WebCenterViewConstants.StickyNoteHeaderPadding
		});

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._confirmIcon = $("<span class='sticky-note-confirm-icon'/>").append(
			$("<i class='icon icon-WCR_Actions-Appr12'/>").css({
				"font-size": WebCenterViewConstants.StickyNoteHeaderHeight - 2 * WebCenterViewConstants.StickyNoteHeaderPadding
			})).css({
			padding: WebCenterViewConstants.StickyNoteHeaderPadding,
			"padding-left": WebCenterViewConstants.StickyNoteHeaderPadding
		});

		// init tooltip for confirm icon
		this._confirmationIconTooltip = new Tooltip(i18n, this._confirmIcon, this._scene.getContext(), this._scene.getContext(), false, i18n.t("confirmStickyNoteEditTooltip"));

		this._confirmIcon.on("click", _.bind(function () {
			this._emit("saved");
			this._scene.focus();
		}, this));

		this._headerDiv.append(this._editIcon).append(this._confirmIcon).append(this._deleteIcon);

		/**
		 * @member {JQueryObject} div for the  content
		 * @private
		 */
		this._contentDiv = $("<div/>").addClass("sticky-note-content");

		/**
		 * @member {JQueryObject} div for the text of the sticky note while not editing
		 * @private
		 */
		this._textDiv = $("<div/>").addClass("sticky-note-text-div");

		/**
		 * @member {JQueryObject} text area for the sticky note while editing
		 * @private
		 */
		this._textArea = $("<textarea/>").addClass("sticky-note-text-area");

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._resizeIcon = $("<span class='sticky-note-resize'/>").append($("<i class='icon icon-WCR_WF-Sticky-Resize'/>").css({
				"font-size": WebCenterViewConstants.StickyNoteResizeHitAreaSize
			}))
			.height(WebCenterViewConstants.StickyNoteResizeHitAreaSize)
			.width(WebCenterViewConstants.StickyNoteResizeHitAreaSize);

		this._stickyNoteDiv.append(this._headerDiv).append(this._contentDiv).append(this._resizeIcon);
	};

	/**
	 * shows the popup
	 */
	StickyNotePopup.prototype.show = function () {

		if (this._stickyNote.isEditingText()) {
			// Show text area while editing
			this._textArea.val(this._stickyNote.getText());
			this._contentDiv.empty().append(this._textArea);
			this._textArea.focus();
			this._stickyNoteDiv.addClass("is-editing");
		} else {
			// Show regular text while not editing
			this._confirmationIconTooltip.hide();
			this._textDiv.text(this._stickyNote.getText());
			this._contentDiv.empty().append(this._textDiv);
			this._stickyNoteDiv.removeClass("is-editing");
		}

		var location = this._stickyNote.getLocation().copy();
		var size = this._stickyNote.getSize();

		//transform the point from world coordinates to pixels from scene div top left corner
		var t = this._scene.getTransformation();
		var loc = t.transformedPoint(location);

		// the transformed size of the popup
		var transformedSize = t.transformedSize(new ShapeUtils.Size(size.width, size.height));

		// When editing show the popup with it's real width and height
		var height = transformedSize.height;
		var width = transformedSize.width;

		this._stickyNoteDiv.css({
			"height": height,
			"width": width,
			top: loc.y,
			left: loc.x,
		});

		var transformedHeaderConstantSizes = t.transformedSize(new ShapeUtils.Size(WebCenterViewConstants.StickyNoteHeaderPadding, WebCenterViewConstants.StickyNoteHeaderHeight));

		var headerHeight = transformedHeaderConstantSizes.height;

		// transform all elements in the popup to be the correct height and width  

		this._contentDiv.css({
			height: height - headerHeight
		});
		this._headerDiv.css({
			"height": headerHeight
		});

		this._editIcon.css({
			padding: transformedHeaderConstantSizes.width,
			"padding-left": transformedHeaderConstantSizes.width
		});
		this._editIcon.find("i").css({
			"font-size": (headerHeight - (2 * transformedHeaderConstantSizes.width))
		});

		this._deleteIcon.css({
			padding: transformedHeaderConstantSizes.width,
			"padding-right": transformedHeaderConstantSizes.width,
			width: headerHeight
		});
		this._deleteIcon.find("i").css({
			"font-size": (headerHeight - (2 * transformedHeaderConstantSizes.width))
		});

		this._confirmIcon.css({
			padding: transformedHeaderConstantSizes.width,
			"padding-right": transformedHeaderConstantSizes.width,
			width: headerHeight,
			height: headerHeight
		});
		this._confirmIcon.find("i").css({
			"font-size": (headerHeight - (2 * transformedHeaderConstantSizes.width))
		});


		var transformedResizeConstant = t.transformedSize(new ShapeUtils.Size(WebCenterViewConstants.StickyNoteResizeHitAreaSize, WebCenterViewConstants.StickyNoteResizeHitAreaSize));

		this._resizeIcon.find(".icon").css({
			"font-size": transformedResizeConstant.width
		});

		this._resizeIcon.css({
			"height": transformedResizeConstant.height,
			"width": transformedResizeConstant.width
		});
	};

	/**
	 * @returns {JQueryObject} the div of the popup
	 */
	StickyNotePopup.prototype.getElement = function () {
		return this._stickyNoteDiv;
	};

	/**
	 * returns the text of the text area
	 * @returns {String} 
	 */
	StickyNotePopup.prototype.getEditedText = function () {
		return this._textArea.val();
	};

	/**
	 * destroy the popup
	 */
	StickyNotePopup.prototype.destroy = function () {
		this._destroyObservable();
		this._stickyNoteDiv.remove();
	};

	return StickyNotePopup;

});


define('utils/webcss-text!WFEditorWCR/core/wfeditorwcr-style.css',[],function () { return '.wfeditor {\r\n\tcolor: #333;\r\n\t-webkit-box-sizing: content-box;\r\n\t-moz-box-sizing: content-box;\r\n\tbox-sizing: content-box\r\n}\r\n.wfeditor .sceneControls .btn i.icon:not(.subIcon) {\r\n\tmargin-top: 0px;\r\n\tmargin-bottom: 0px;\r\n}\r\n.minified-wfeditor-popup {\r\n\twidth: 300px;\r\n\theight: 300px;\r\n\t\r\n\tmin-width: 200px;\r\n\tmin-height: 150px;\r\n}\r\n.full-wfeditor-popup {\r\n\twidth: 635px;\r\n\theight:\t500px;\r\n\tmin-width: 635px;\r\n\tmin-height: 150px;\r\n} \r\n.preview-wfeditor-popup > .wfeditor-popup-header h3 {\r\n\topacity: 0.5;\r\n}\r\n.preview-wfeditor-popup.focused .wfeditor-popup-header h3 {\r\n\topacity: 1;\r\n}\r\n.preview-wfeditor-popup.wfeditor-popup.focused {\r\n\tz-index: 301;\r\n}\r\n.preview-wfeditor-popup.wfeditor-popup {\r\n\tposition: absolute;\r\n\tborder: 1px solid #ccc;\r\n\tbackground: #fff;\r\n\tz-index: 300;\r\n\t-webkit-box-shadow: 0px 0px 4px 0px rgba(0, 0, 0, 0.2);\r\n\t-moz-box-shadow: 0px 0px 4px 0px rgba(0, 0, 0, 0.2);\r\n\tbox-shadow: 0px 0px 4px 0px rgba(0, 0, 0, 0.2);\r\n}\r\n.preview-wfeditor-popup.wfeditor-popup.fullscreen {\r\n\tz-index: 9999;\r\n}\r\n.preview-wfeditor-popup .ui-resizable-handle {\r\n\tbackground-image: none;\r\n}\r\n.preview-wfeditor-popup .wfeditor-popup-header {\r\n\tborder-bottom: 1px solid #ccc;\r\n\tbackground: #F5F5F5;\r\n\tposition: absolute;\r\n\ttop: 0;\r\n\tleft: 0;\r\n\tright: 0;\r\n\theight: 26px;\r\n}\r\n.preview-wfeditor-popup a.wfeditor-popup-close {\r\n\tfloat: right;\r\n\tfont-size: 16px;\r\n\tcolor: #999;\r\n\tmargin: 2px 4px;\r\n\ttext-decoration: none;\r\n\tdisplay: block;\r\n\tcursor: pointer;\r\n}\r\n.minified-wfeditor-popup .enter-full-icon {\r\n\tposition: absolute;\r\n\ttop: 2px;\r\n\tright: 24px;\r\n\tcolor: #999;\r\n\tcursor: pointer;\r\n}\r\n.preview-wfeditor-popup a.wfeditor-popup-close:hover,\r\n.minified-wfeditor-popup .enter-full-icon:hover {\r\n\tcolor: #333;\r\n\ttext-decoration: none;\r\n}\r\n.preview-wfeditor-popup .wfeditor-popup-header h3 {\r\n\tpadding: 5px;\r\n\tmargin: 0;\r\n\tfont-size: 14px;\r\n\twhite-space: nowrap;\r\n\toverflow: hidden;\r\n\ttext-overflow: ellipsis;\r\n\tcursor: move;\r\n}\r\n.preview-wfeditor-popup .wfeditor-popup-content {\r\n\tposition: absolute;\r\n\ttop: 26px;\r\n\tbottom: 35px;\r\n\tleft: 0;\r\n\tright: 0;\r\n\toverflow: auto;\r\n\tpadding: 0;\r\n\tmargin: 0;\r\n}\r\n.preview-wfeditor-popup .wfeditor-popup-footer {\r\n\tposition: absolute;\r\n\tbottom: 0;\r\n\tright: 0;\r\n\tleft: 0;\r\n\tbackground: #fbfbfb;\r\n\tpadding: 3px;\r\n\tborder-top: 1px solid #ccc;\r\n\theight: 35px;\r\n}\r\n.minified-wfeditor-popup .wfeditor-popup-footer .minified-wfeditor-footer {\r\n    display: block;\r\n    margin: 0 auto;\r\n    position: relative;\r\n    top: auto;\r\n    left: auto;\r\n    float: none;\r\n\ttext-align: center;\r\n\theight:100%;\r\n}\r\n.minified-wfeditor-popup .wfeditor-popup-footer .minified-wfeditor-footer .btn-group {\r\n\tpadding: 3px 0px 2px 0px;\r\n}\r\n.minified-wfeditor-popup .wfeditor-popup-footer .minified-wfeditor-footer > div {\r\n\tdisplay: inline-block;\r\n}\r\n.minified-wfeditor-popup .wfeditor-popup-footer .no-active-nodes .go-to-active-button {\r\n\tdisplay: none;\r\n}\r\n.minified-wfeditor-popup .wfeditor-popup-footer .active-node-index {\r\n\tmargin: 0px 10px;\r\n\tpadding: 6px 0px;\r\n}\r\n.minified-wfeditor-popup .wfeditor-popup-footer .btn-toolbar > .btn-group {\r\n\tmargin-left: 0px;\r\n}\r\n.wfeditor .draggablePopup .popupContent {\r\n\tbackground-color: white;\r\n\tmargin: 1px;\r\n}\r\n.wfeditor .draggablePopup .popupContent li {\r\n\tpadding: 2px 4px;\r\n}\r\n.wfeditor .draggablePopup .popupContent li.tokenSelected,\r\n.wfeditor .draggablePopup .popupContent li.docPathSelected {\r\n\tbackground-color: #eee;\r\n}\r\n.wfeditor .draggablePopup .popupContent li.docSelected {\r\n\tbackground-color: #337ab7;\r\n\tcolor: white;\r\n}\r\n.wfeditor .draggablePopup .popupHeader {\r\n\tmargin: 2px 2px 2px 5px;\r\n\tcolor: white;\r\n\theight: 13px;\r\n}\r\n.wfeditor .draggablePopup .popupContent ul {\r\n\tlist-style: none;\r\n\tpadding: 0;\r\n\tmargin: 0;\r\n}\r\n.wfeditor .documentContextMenuAnchor {\r\n\tposition: absolute;\r\n\ttop: 0px;\r\n\tleft: 0px;\r\n}\r\n.wfeditor .popover {\r\n\t-webkit-box-shadow: none;\r\n\t-moz-box-shadow: none;\r\n\tbox-shadow: none;\r\n}\r\n.wfeditor .btn-toolbar {\r\n\tfont-size: 1em;\r\n\tposition: relative;\r\n\tdisplay: inline-block;\r\n\tvertical-align: middle;\r\n\tbackground: rgba(255, 255, 255, 0.9);\r\n\tpadding: 0 8px 5px 0;\r\n}\r\n.preview-wfeditor-popup .wfeditor .btn-toolbar {\r\n\tpadding: 5px;\r\n}\r\n.wfeditor .btn-toolbar.toolbarRight {\r\n\tfloat: right;\r\n\tright: 10px;\r\n}\r\n.wfeditor .btnGroupWithTitle {\r\n\theight: 100%;\r\n\tdisplay: inline-block;\r\n\tmargin-left: 5px;\r\n\ttext-align: center;\r\n\tposition: relative;\r\n}\r\n.wfeditor .btnGroupTitle {\r\n\tdisplay: block;\r\n\theight: auto;\r\n\twidth: auto;\r\n\tfont-size: 0.8em;\r\n\tpadding-top: 2px;\r\n\tcolor: #000;\r\n\tmargin: 0;\r\n\ttext-align: center;\r\n\tclear: both;\r\n}\r\n.wfeditor .taskNameToolbarComponent {\r\n\theight: 36px;\r\n\tmargin-right: 10px;\r\n\tline-height: 36px;\r\n\tfont-size: 16px;\r\n\tdisplay: inline-block;\r\n\tvertical-align: top;\r\n}\r\n.wfeditor .taskNameToolbarComponent span {\r\n\tfont-weight: 500;\r\n}\r\n.wfeditor .statusToolbarComponent {\r\n\theight: 100%;\r\n\tdisplay: inline-block;\r\n\tmargin-left: 20px;\r\n\ttext-align: center;\r\n\tposition: relative;\r\n\tvertical-align: top;\r\n}\r\n.wfeditor .statusToolbarContent {\r\n\theight: 36px;\r\n\tline-height: 36px;\r\n\tposition: relative;\r\n\tdisplay: inline-block;\r\n\tvertical-align: middle;\r\n\tcolor: #333333;\r\n\tbackground-color: #ffffff;\r\n\tmargin-bottom: 0;\r\n\tfont-weight: bold;\r\n\ttext-align: center;\r\n\twhite-space: nowrap;\r\n\tfont-size: 12px;\r\n\tborder-radius: 0px;\r\n}\r\n.wfeditor .statusToolbarContent img {\r\n\tdisplay: inline-block;\r\n\tmax-width: 22px;\r\n\tmax-height: 22px;\r\n\tmargin-right: 5px;\r\n}\r\n.wfeditor .statusToolbarContent span {\r\n\tvertical-align: middle;\r\n}\r\n.wfeditor .WFRichTextContainer {\r\n\tpadding: 0px;\r\n\theight: 100%;\r\n\twidth: 100%;\r\n\theight: 300px;\r\n\t\r\n}\r\n.wfeditor .WFRichTextContainer > div,\r\n.wfeditor .WFRichTextContainer > div iframe {\r\n\twidth: 100% !important;\r\n}\r\n.wfeditor .MultiParametersPanel {\r\n\tmargin-bottom: 15px;\r\n\tmargin-top: 5px;\r\n}\r\n.wfeditor .MultiParametersPanel table {\r\n\tmargin-bottom: 5px;\r\n\twidth: 100%;\r\n}\r\n.wfeditor .MultiParametersPanel .header {\r\n\ttext-overflow: ellipsis;\r\n\tfont-style: normal;\r\n\tfont-size: 12px;\r\n}\r\n.wfeditor .MultiParametersPanel .block,\r\n.wfeditor .MultiParametersPanel .sdrop {\r\n\tline-height: 12px;\r\n\tvertical-align: middle;\r\n\twidth: 100% !important;\r\n}\r\n.wfeditor .MultiParametersPanel .SmartField {\r\n\twidth: 100% !important;\r\n}\r\n.wfeditor .MultiParametersPanel .offsetRightDeleteButton {\r\n\tpadding-right: 40px;\r\n    float: left;\r\n    width: 100%;\r\n}\r\n.wfeditor .MultiColumnPanel .has-3-cols > div{\r\n\twidth: 33.33%;\r\n\tfloat: left;\r\n\tposition: relative;\r\n\tpadding: 2px;\r\n}\r\n.wfeditor .MultiColumnPanel .has-4-cols > div{\r\n\twidth: 25%;\r\n\tfloat: left;\r\n\tposition: relative;\r\n\tpadding: 2px;\r\n}\r\n.wfeditor .MultiColumnPanel .has-5-cols > div{\r\n\twidth: 20%;\r\n\tfloat: left;\r\n\tposition: relative;\r\n\tpadding: 2px;\r\n}\r\n.wfeditor .MultiParametersPanel .nameAndValue > div {\r\n\twidth: 50%;\r\n    float: left;\r\n    position: relative;\r\n    padding: 2px;\r\n}\r\n.wfeditor .MultiParametersPanel .deleteBtn {\r\n\tdisplay: inline-block;\r\n    margin-left: -38px;\r\n    position: relative;\r\n    padding: 2px;\r\n}\r\n.wfeditor .MultiParametersPanel .AddBtn {\r\n\tmargin-top: 3px;\r\n}\r\n.wfeditor .MultiParametersPanel input.block {\r\n\tborder: 1px solid lightgrey;\r\n}\r\n.wfeditor .panel-loading {\r\n\tpadding: 15px;\r\n}\r\n.wfeditor .wfeditor-node-configuration .panel-information-text {\r\n\tpadding-left: 35px;\r\n}\r\n.wfeditor .wfeditor-popup-footer .inline_help.btn {\r\n\tmin-width: 0px;\r\n\t}\r\n.w .wfeditor-node-help-tooltip.tooltip.inline_help_tooltip {\r\n\tz-index: 9000;\r\n}\r\n.wfeditor .form-horizontal.form-compact .form-group>div {\r\n\tmargin: 0;\r\n}\r\n.wfeditor .form-compact .form-group label,\r\n.wfeditor .form-compact .form-group .read-only-string {\r\n\tpadding-top: 5px;\r\n}\r\n.wfeditor .form-compact .form-group label {\r\n\ttext-align: right;\r\n}\r\n.wfeditor .form-compact .form-group > label {\r\n\tmax-width: 225px;\r\n}\r\n.wfeditor .container-sm .form-group label,\r\n.wfeditor .form-compact .form-group .checkbox label,\r\n.wfeditor .form-compact .form-group .radio label {\r\n\ttext-align: left;\r\n}\r\n.wfeditor .form-compact fieldset {\r\n\tborder: 1px solid #ddd;\r\n\tpadding: 0 15px;\r\n\tmargin: 10px 0 15px 0;\r\n}\r\n.wfeditor .form-compact legend {\r\n\tpadding: 0 5px 0 5px;\r\n\twidth: auto;\r\n\tmargin: 0;\r\n\tborder: 0;\r\n\tfont-size: 11px;\r\n}\r\n.wfeditor div.parameter-panel {\r\n\tpadding: 0 10px;\r\n}\r\n.wfeditor .sibling-panel .parameter-panel {\r\n\tpadding: 0;\r\n}\r\n.wfeditor .form-horizontal.form-compact .boolean-panel > .form-group > .sibling-extra-padding-top {\r\n\tmargin-top: 10px;\r\n}\r\n.wfeditor div.child-panel {\r\n\twidth: 100%;\r\n\tpadding: 0 0px 0 30px;\r\n}\r\n.wfeditor div.sibling-panel {\r\n\twidth: 100%;\r\n}\r\n.wfeditor .parameter-panel input[type="radio"],\r\n.wfeditor .parameter-panel input[type="checkbox"] {\r\n\tmargin-top: 0px;\r\n}\r\n.wfeditor .multi-project-attribute-panel .filter-list .list-group {\r\n\theight: 200px;\r\n}\r\n.wfeditor .nav-tabs {\r\n\tmargin-bottom: 8px;\r\n}\r\n.w .wfeditor .SmartField.SmartField-disabled .SmartNameInput select {\r\n\tbackground-color: #EBEBE4;\r\n}\r\n.wfeditor .onlyOffsetLabels > div.form-group > label.offsetLabel {\r\n\tdisplay: none;\r\n}\r\n.wfeditor .onlyOffsetLabels > .boolean-panel > label.offsetLabel {\r\n\tdisplay: none;\r\n}\r\n.wfeditor .container-lg .onlyOffsetLabels > .boolean-panel > .col-lg-9 {\r\n\twidth: 100%;\r\n}\r\n.wfeditor .notOnlyOffsetLabels > .boolean-panel > .sibling-panel > .parameter-panel > .boolean-panel > .form-group > label.offsetLabel {\r\n\tdisplay: block;\r\n}\r\n.wfeditor .radio-panel.has-children {\r\n\tmargin-bottom: 0px;\r\n}\r\n.wfeditor .form-horizontal.form-compact .form-group.radio-panel.has-children .radio {\r\n\tmargin-bottom: 5px;\r\n}\r\n.wfeditor .wfeditor-node-configuration {\r\n\tmargin-top: 5px;\r\n}\r\n.w .wfeditor-node-configuration .router-panel .router-panel-entry .router-entry-view,\r\n.w .wfeditor-node-configuration .router-panel-header .router-header-view {\r\n\tpadding-right: 35px;\r\n\tpadding-top: 1px;\r\n\tpadding-bottom: 2px;\r\n}\r\n.w .container-sm .router-panel .router-panel-entry .router-entry-view {\r\n\theight: 60px;\r\n}\r\n.wfeditor .wfeditor-node-configuration label {\r\n\tfont-weight: 500;\r\n}\r\n.wfeditor .wfeditor-node-configuration .checkbox label,\r\n.wfeditor .wfeditor-node-configuration .radio label{\r\n\tfont-weight: normal;\r\n}\r\n.w .wfeditor-node-configuration .router-panel .router-add-button {\r\n\tmargin-top: 5px;\r\n}\r\n.w .wfeditor-node-configuration .router-panel .between-smartname-label {\r\n\tdisplay: table-cell;\r\n\twidth: 1%;\r\n\twhite-space: nowrap;\r\n\tvertical-align: middle;\r\n\tpadding-left: 8px;\r\n\tpadding-right: 8px;\r\n\tpadding-bottom: 2px;\r\n}\r\n.w .wfeditor-node-configuration .router-panel .router-panel-entry .router-entry-delete {\r\n\tposition: relative;\r\n\tmargin-left: -33px;\r\n\tdisplay: none;\r\n}\r\n.w .wfeditor-node-configuration .router-panel .router-panel-entry {\r\n\tpadding: 4px 0;\r\n}\r\n.w .wfeditor-node-configuration .router-panel .router-panel-entry:hover .router-entry-delete {\r\n\tdisplay: inline-block;\r\n}\r\n.w .wfeditor-node-configuration .router-panel .router-entry-sortable {\r\n\tcursor: pointer;\r\n\tcursor: move;\r\n\tcursor: grab;\r\n}\r\n.w .wfeditor-node-configuration .router-panel .router-entry-placeholder {\r\n\theight: 20px;\r\n\tmargin: 4px 0px;\r\n}\r\n.w .wfeditor-node-configuration .router-panel.one-router-entry-left .router-entry-sortable {\r\n\tcursor: auto;\r\n}\r\n.w .wfeditor-node-configuration .router-panel.one-router-entry-left .router-panel-entry:hover .router-entry-delete {\r\n\tdisplay: none;\r\n}\r\n.w .wfeditor-node-configuration .multi-project-attribute-panel .filter-list .list-group {\r\n\theight: 200px;\r\n}\r\n.w .wfeditor-node-configuration .SmartField.SmartField-disabled .SmartNameInput select {\r\n\tbackground-color: #EBEBE4;\r\n}\r\n.w .wfeditor-node-configuration .tabbed-panel-tab {\r\n\tcursor: pointer;\r\n}\r\n.w .wfeditor-node-configuration .multi-project-attribute-panel {\r\n\tmargin-top: 15px;\r\n}\r\n.wfeditor .wcr-loadingIndicator.right {\r\n\ttop: 54px;\r\n}\r\n.minified-wfeditor-popup .wfeditor .wcr-loadingIndicator.right {\r\n\ttop: 5px;\r\n}\r\n.wfeditor .scene .sticky-note {\r\n\tposition: absolute;\r\n}\r\n.wfeditor .scene .sticky-note .sticky-note-header {\r\n\tbackground: #f8efbe;\r\n\tcursor: default;\r\n}\r\n.wfeditor .scene .sticky-note .sticky-note-content {\r\n\toverflow-y: hidden;\r\n\toverflow-x: hidden;\r\n\ttext-overflow: ellipsis;\r\n\twidth: 100%;\r\n\tbackground: #fcf8e3;\r\n}\r\n.wfeditor .scene .sticky-note .sticky-note-resize {\r\n\tposition: absolute;\r\n\tbottom: 1px;\r\n\tright: 1px;\r\n}\r\n.wfeditor .scene .sticky-note .sticky-note-resize i {\r\n\tcolor: #f1e28d;\r\n\tposition: absolute;\r\n}\r\n.wfeditor .scene .sticky-note .sticky-note-header .sticky-note-edit-icon {\r\n\tposition: absolute;\r\n\ttop: 0px;\r\n\tleft: 0px;\r\n}\r\n.wfeditor .scene .sticky-note .sticky-note-header .sticky-note-confirm-icon {\r\n\tcursor: pointer;\r\n\tposition: absolute;\r\n\ttop: 0px;\r\n\tleft: 0px;\r\n\tdisplay: none;\r\n}\r\n.wfeditor .scene .sticky-note.is-editing .sticky-note-header .sticky-note-confirm-icon {\r\n\tdisplay: block;\r\n}\r\n.wfeditor .scene .sticky-note .sticky-note-header .sticky-note-edit-icon i {\r\n\tposition: absolute;\r\n}\r\n.wfeditor .scene .sticky-note.is-editing .sticky-note-header .sticky-note-edit-icon,\r\n.wfeditor .scene .sticky-note.feedback-mode .sticky-note-header .sticky-note-edit-icon {\r\n\tdisplay: none;\r\n}\r\n.wfeditor .scene .sticky-note .sticky-note-header .sticky-note-delete-icon {\r\n\tposition: absolute;\r\n\ttop: 0px;\r\n\tright: 0px;\r\n}\r\n.wfeditor .scene .sticky-note .sticky-note-header .sticky-note-delete-icon i,\r\n.wfeditor .scene .sticky-note .sticky-note-header .sticky-note-confirm-icon i {\r\n\tposition: absolute;\r\n}\r\n.wfeditor .scene .sticky-note.is-editing .sticky-note-header .sticky-note-delete-icon,\r\n.wfeditor .scene .sticky-note.feedback-mode .sticky-note-header .sticky-note-delete-icon {\r\n\tdisplay: none;\r\n}\r\n.wfeditor .scene .sticky-note.is-editing .sticky-note-resize { \r\n\tdisplay: none;\r\n}\r\n.wfeditor .scene .sticky-note .sticky-note-content .sticky-note-text-area {\r\n\twidth: 100%;\r\n\theight: 100%;\r\n\tpadding: 4px;\r\n\tborder: none;\r\n\tbackground: rgba(0, 0, 0, 0);\r\n\tresize: none;\r\n}\r\n.wfeditor .scene .sticky-note .sticky-note-content .sticky-note-text-div {\r\n\twidth: 100%;\r\n\theight: 100%;\r\n\tpadding: 4px;\r\n\twhite-space: pre-wrap;\r\n}\r\n.wfeditor .nwse-resize-cursor {\r\n\tcursor: nwse-resize;\r\n}\r\n.wfeditor .text-cursor {\r\n\tcursor: text;\r\n}\r\n.w .wfeditor .wf-context-menu {\r\n\tborder-radius: 0px;\r\n\tpadding: 2px;\r\n}\r\n.w .wfeditor .scene .wf-context-menu .wcr-dropdown-menu li:hover:not(.group) {\r\n\tbackground-color: #a3b30f;\r\n\tcolor: #ffffff;\r\n}\r\n.w .wfeditor .scene .wf-context-menu .wcr-dropdown-menu a:hover {\r\n\tbackground: none;\r\n}\r\n.w .wfeditor .scene .wf-context-menu .wcr-dropdown-menu li {\r\n\tvertical-align: middle;\r\n}\r\n.w .wfeditor .scene .wf-context-menu.wcr-dropdown .wcr-dropdown-menu .group > ul{\r\n\tlist-style: none;\r\n\tpadding: 0px;\r\n\tmargin-left: -15px;\r\n}\r\n.w .wfeditor .scene .wf-context-menu.wcr-dropdown .wcr-dropdown-menu .group > ul li:not([style=\'display:none\']):last-child {\r\n\tborder-bottom: 1px solid #e5e5e5;\r\n}\r\n.w .wfeditor .scene .wf-context-menu.wcr-dropdown .wcr-dropdown-menu .group > ul li:not([style=\'display:none\']).last-element {\r\n\tborder-bottom: none;\r\n}\r\n.w .wfeditor .scene .wf-context-menu.wcr-dropdown .wcr-dropdown-menu .icon {\r\n    margin-left: -17px;\r\n    padding-right: 4px;\r\n\tpadding-left: 2px;\r\n\tvertical-align: middle;\r\n}\r\n.w .wfeditor .scene .wf-context-menu.wcr-dropdown .wcr-dropdown-menu li {\r\n\tpadding-left: 17px;\r\n\tpadding-right: 2px;\r\n}\r\n.w .wfeditor .scene .wf-context-menu.wcr-dropdown .wcr-dropdown-menu .group li {\r\n\tpadding-top: 2px;\r\n\tpadding-bottom: 2px;\r\n}\r\n.w .wfeditor .scene .wf-context-menu.wcr-dropdown .wcr-dropdown-menu .group > ul > li:first-child {\r\n\tdisplay: none;  \r\n}\r\n.wfeditor .sceneControls .btn-toolbar a.btn > div {\r\n\tmargin-bottom: 1px;\r\n}\r\nbody.wf-editor-is-fullscreen .ui-widget-overlay.ui-front,\r\nbody.wf-editor-is-fullscreen .ui-dialog,\r\nbody.wf-editor-is-fullscreen .modal,\r\nbody.wf-editor-is-fullscreen .modal-backdrop {\r\n\tz-index: 99999;\r\n}';});


define('WFEditorWCR/workflow/helper/stickyNote/StickyNoteView',["underscore",
		"jquery",
		"WFEditor/scene/AbstractView",
		"utils/ShapeUtils",
		"WFEditor/scene/hitarea/HitArea",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/scene/WebCenterViewConstants",
		"./StickyNotePopup",
		"utils/webi18n!WFEditorWCR",
		"utils/webcss!WFEditorWCR/core/wfeditorwcr-style.css"
	],
	function (_, $,
		AbstractView,
		ShapeUtils,
		HitArea,
		WebCenterNodeConstants,
		WebCenterViewConstants,
		StickyNotePopup,
		i18n) {


		"use strict";

		/**
		 * View for a sticky note - handles all events froom the sticky note and the scene. The actual rendering is done by StickyNotePopup
		 * 
		 * @class
		 * @exports StickyNoteView
		 * @augments AbstractView
		 * 
		 * @param {StickyNoteHelper} stickyNote 
		 * @param {Scene}            scene      Scene
		 */
		var StickyNoteView = function (stickyNote, scene) {
			AbstractView.call(this, stickyNote);

			/**
			 * @member {StickyNoteHelper}
			 * @private
			 */
			this._stickyNote = stickyNote;

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = scene;

			/**
			 * @member {StickyNotePopup}
			 * @private
			 */
			this._stickyNotePopup = new StickyNotePopup(stickyNote, scene);

			this._observe(stickyNote, "changed.location", this._updateRenderingInfo);
			this._observe(stickyNote, "changed.size", this._updateRenderingInfo);
			this._observe(stickyNote, "changed.text", this.makeDirty);
			this._observe(stickyNote, "changed.editing", this.makeDirty);

			this._setSceneModelListeners();

		};

		StickyNoteView.prototype = Object.create(AbstractView.prototype);
		StickyNoteView.prototype.constructor = StickyNoteView;

		/**
		 * update the rendering info
		 */
		StickyNoteView.prototype._updateRenderingInfo = function () {
			this._updateBoundingRect();
			this._updateHitAreas();
			this.makeDirty();
		};

		/**
		 * @override
		 */
		StickyNoteView.prototype.render = function (ctx, layer, quickRender) {
			var canvas = ctx.canvas;
			if (this._stickyNote.isEditingText()) {
				this._scene.getContext().append(this._stickyNotePopup.getElement());
			} else {
				$(canvas).nextAll("canvas:first").before(this._stickyNotePopup.getElement());

			}
			this._stickyNotePopup.show();

		};

		/**
		 * @override
		 */
		StickyNoteView.prototype._updateBoundingRect = function () {
			this._setBoundingRect(new ShapeUtils.Rect(new ShapeUtils.Point(this._stickyNote.getLocation().x, this._stickyNote.getLocation().y),
				new ShapeUtils.Size(this._stickyNote.getSize().width, this._stickyNote.getSize().height)).getNormalizedRectArray());
		};

		/**
		 * @override
		 */
		StickyNoteView.prototype._updateHitAreas = function (context) {
			var hasPrivilegeToEdit = this.getModel().getWorkflow().getPrivilege(WebCenterNodeConstants.Privileges.editStickyNotes);

			// The move hitArea - ie. the header minus the location for the edit icon and the delete icon
			var moveHitArea = new ShapeUtils.Rect(
				new ShapeUtils.Point(
					this._stickyNote.getLocation().x + (hasPrivilegeToEdit ? WebCenterViewConstants.StickyNoteHeaderPadding + WebCenterViewConstants.StickyNoteHeaderHeight : 0),
					this._stickyNote.getLocation().y
				), new ShapeUtils.Size(
					// If we have the privilege to edit leave some space for the edit and delete icons
					this._stickyNote.getSize().width - (hasPrivilegeToEdit ? (2 * (WebCenterViewConstants.StickyNoteHeaderPadding + WebCenterViewConstants.StickyNoteHeaderHeight)) : 0),
					WebCenterViewConstants.StickyNoteHeaderHeight)
			);

			// resizeHitArea - right bottom corner
			var resizeHitArea = new ShapeUtils.Rect(
				new ShapeUtils.Point(
					this._stickyNote.getLocation().x + this._stickyNote.getSize().width - WebCenterViewConstants.StickyNoteResizeHitAreaSize,
					this._stickyNote.getLocation().y + this._stickyNote.getSize().height - WebCenterViewConstants.StickyNoteResizeHitAreaSize),
				new ShapeUtils.Size(WebCenterViewConstants.StickyNoteResizeHitAreaSize, WebCenterViewConstants.StickyNoteResizeHitAreaSize));

			var hitAreas = [
				new HitArea(moveHitArea, this, WebCenterNodeConstants.HitAreas.STICKYNOTE.MOVE, this.getZIndex(), {
					cursor: "hand-cursor"
				}),
				new HitArea(resizeHitArea, this, WebCenterNodeConstants.HitAreas.STICKYNOTE.RESIZE, this.getZIndex(), {
					cursor: "nwse-resize-cursor"
				})
			];

			// add hit areas for privileged tools -> editing and deleting
			if (hasPrivilegeToEdit) {
				// Text area gets split in to as to not interfere with the resize hit area
				var textHitArea1 = new ShapeUtils.Rect(
					new ShapeUtils.Point(
						this._stickyNote.getLocation().x,
						this._stickyNote.getLocation().y + WebCenterViewConstants.StickyNoteHeaderHeight
					),
					new ShapeUtils.Size(
						this._stickyNote.getSize().width,
						this._stickyNote.getSize().height - WebCenterViewConstants.StickyNoteHeaderHeight - WebCenterViewConstants.StickyNoteResizeHitAreaSize
					)
				);

				hitAreas.push(new HitArea(textHitArea1, this, WebCenterNodeConstants.HitAreas.STICKYNOTE.TEXTAREA, this.getZIndex(), {
					cursor: "text-cursor"
				}));
				var textHitArea2 = new ShapeUtils.Rect(
					new ShapeUtils.Point(
						this._stickyNote.getLocation().x,
						this._stickyNote.getLocation().y + (this._stickyNote.getSize().height - WebCenterViewConstants.StickyNoteHeaderHeight - WebCenterViewConstants.StickyNoteResizeHitAreaSize)
					),
					new ShapeUtils.Size(
						this._stickyNote.getSize().width - WebCenterViewConstants.StickyNoteResizeHitAreaSize,
						WebCenterViewConstants.StickyNoteResizeHitAreaSize
					)
				);

				hitAreas.push(new HitArea(textHitArea2, this, WebCenterNodeConstants.HitAreas.STICKYNOTE.TEXTAREA, this.getZIndex(), {
					cursor: "text-cursor"
				}));
				// Edit icon floating left in the header
				var editTextIconArea = new ShapeUtils.Rect(
					new ShapeUtils.Point(
						this._stickyNote.getLocation().x + WebCenterViewConstants.StickyNoteHeaderPadding,
						this._stickyNote.getLocation().y + WebCenterViewConstants.StickyNoteHeaderPadding
					),
					new ShapeUtils.Size(
						WebCenterViewConstants.StickyNoteHeaderHeight,
						WebCenterViewConstants.StickyNoteHeaderHeight
					)
				);
				hitAreas.push(new HitArea(editTextIconArea, this, [WebCenterNodeConstants.HitAreas.STICKYNOTE.EDITICON], this.getZIndex(), {
					cursor: "pointer-cursor",
					tooltip: i18n.t("editStickyNoteIconTooltip")
				}));
				// delete icon floating right in the header
				var deleteIconArea = new ShapeUtils.Rect(
					new ShapeUtils.Point(
						this._stickyNote.getLocation().x + this._stickyNote.getSize().width - WebCenterViewConstants.StickyNoteHeaderPadding - WebCenterViewConstants.StickyNoteHeaderHeight,
						this._stickyNote.getLocation().y + WebCenterViewConstants.StickyNoteHeaderPadding
					),
					new ShapeUtils.Size(
						WebCenterViewConstants.StickyNoteHeaderHeight,
						WebCenterViewConstants.StickyNoteHeaderHeight
					)
				);
				hitAreas.push(new HitArea(deleteIconArea, this, [WebCenterNodeConstants.HitAreas.STICKYNOTE.DELETE], this.getZIndex(), {
					cursor: "pointer-cursor",
					tooltip: i18n.t("removeStickyNoteTooltip")
				}));

			}

			this._setHitAreas(hitAreas);
		};

		/**
		 * Listen to sceneModel for panning and zoom changes
		 */
		StickyNoteView.prototype._setSceneModelListeners = function () {
			this._observe(this._scene.getSceneModel(), "panningOffsetChanged", function () {

				// defer to ensure transformation has been done. The relocation of the note doesn't happen in the render
				// method because if the pin isn't in the visible rect anymore, the render method won't be called
				_.defer(_.bind(function () {
					this._stickyNotePopup.show();

				}, this));

			});

			this._observe(this._scene.getSceneModel(), "zoomChanged", _.bind(function () {
				// defer to ensure transformation has been done.
				_.defer(_.bind(function () {
					this._stickyNotePopup.show();
				}, this));

			}, this));
		};

		/**
		 * returns the sticky note popup for this view
		 * @returns {StickyNotePopup} 
		 */
		StickyNoteView.prototype.getStickyNotePopup = function () {
			return this._stickyNotePopup;
		};

		/**
		 * @override
		 */
		StickyNoteView.prototype.destroyView = function () {
			this._stickyNotePopup.destroy();
		};

		return StickyNoteView;
	});

define('WFEditorWCR/scene/WebCenterViewFactory',["WFEditor/scene/AbstractViewFactory",
		"WFEditor/workflow/ModelConstants",
		"utils/webi18n!WFEditorWCR",
		"WFEditorWCR/workflow/node/WebCenterNodeView",
		"WFEditorWCR/workflow/pin/WebCenterPinView",
		"WFEditorWCR/workflow/pin/WebCenterPinConstants",
		"WFEditorWCR/workflow/edge/WebCenterEdgeView",
		"WFEditorWCR/workflow/token/WebCenterTokenView",
		"WFEditorWCR/workflow/helper/stickyNote/StickyNoteHelper",
		"WFEditorWCR/workflow/helper/stickyNote/StickyNoteView",
		"utils/ExecutionQueue",
		"underscore",
		"jquery"
	],
	function (
		AbstractViewFactory,
		ModelConstants,
		i18n,
		WebCenterNodeView,
		WebCenterPinView,
		WebCenterPinConstants,
		WebCenterEdgeView,
		WebCenterTokenView,
		StickyNoteHelper,
		StickyNoteView,
		ExecutionQueue,
		_,
		$
	) {
		"use strict";

		/**
		 * Implementation of the view factory for WebCenter. It creates all view objects needed
		 * to render the WebCenter workflow editor.
		 *
		 * @exports WebCenterViewFactory
		 * @class
		 * @augments AbstractViewFactory
		 * @param {ErrorHandler} errorHandler
		 */
		var WebCenterViewFactory = function (errorHandler) {
			AbstractViewFactory.call(this);
			
			/**
			 * @member {ErrorHandler}
			 * @private
			 */
			this._errorHandler = errorHandler;
		};

		WebCenterViewFactory.prototype = Object.create(AbstractViewFactory.prototype);
		WebCenterViewFactory.prototype.constructor = WebCenterViewFactory;

		/**
		 * @Override
		 */
		WebCenterViewFactory.prototype.createViewsForWorkflow = function (workflow, scene) {
			var result = $.Deferred();
			var viewsDestination = {};


			var queue = new ExecutionQueue();

			queue.forEach(workflow.getNodes(), function (node) {
				this._createNode(node, viewsDestination, scene, viewsDestination);
			}, this, "ViewFactoryCreateNodes", i18n.t("initializingNodeViews"));

			queue.forEach(workflow.getEdges(), function (edge) {
				this._createEdge(edge, viewsDestination, scene, viewsDestination);
			}, this, "ViewFactoryCreateEdges", i18n.t("initializingConnectionViews"));

			queue.forEach(workflow.getTokens(), function (token) {
				this._createToken(token, viewsDestination, scene, viewsDestination);
			}, this, "ViewFactoryCreateTokens", i18n.t("initializingTokenViews"));

			queue.forEach(workflow.getHelpers(), function (helper) {
				this._createHelper(helper, viewsDestination, scene, viewsDestination);
			}, this, "ViewFactoryCreateHelper");

			queue.whenFinished(function () {
				result.resolve(viewsDestination);
			});

			return result;
		};

		/**
		 * @Override
		 */
		WebCenterViewFactory.prototype.createViewsForWorkflowItem = function (workflowItem, existingViews, scene, viewsDestination) {
			switch (workflowItem.getWorkflowItemType()) {
			case ModelConstants.WorkflowItemType.Edge:
				this._createEdge(workflowItem, existingViews, scene, viewsDestination);
				break;
			case ModelConstants.WorkflowItemType.Node:
				this._createNode(workflowItem, existingViews, scene, viewsDestination);
				break;
			case ModelConstants.WorkflowItemType.Pin:
				this._createPin(workflowItem, existingViews, scene, viewsDestination);
				break;
			case ModelConstants.WorkflowItemType.Token:
				this._createToken(workflowItem, existingViews, scene, viewsDestination);
				break;
			case ModelConstants.WorkflowItemType.Helper:
				this._createHelper(workflowItem, existingViews, scene, viewsDestination);
				break;
			}
		};

		/**
		 *  @Override
		 */
		WebCenterViewFactory.prototype._createNode = function (node, existingViews, scene, viewsDestination) {
			if (!AbstractViewFactory.prototype._createNode.call(this, node, existingViews, scene, viewsDestination)) { // first try super
				if (node.isVisible()) {
					var nodeView = new WebCenterNodeView(node, scene);
					viewsDestination[nodeView.getId()] = nodeView;
					existingViews[nodeView.getId()] = nodeView;

					_.each(node.getPins(), function (pin) {
						this._createPin(pin, existingViews, scene, viewsDestination);
					}, this);
					return true;
				}
				return false;
			}
			return true;
		};

		/**
		 * @Override
		 */
		WebCenterViewFactory.prototype._createPin = function (pin, existingViews, scene, viewsDestination) {
			if (!AbstractViewFactory.prototype._createPin.call(this, pin, existingViews, scene, viewsDestination)) { // first try super
				if (pin.isVisible()) {
					var nodeView = existingViews[pin.getNode().getId()];
					if (nodeView) {
						var pinView = new WebCenterPinView(pin, nodeView, scene);
						viewsDestination[pinView.getId()] = pinView;
						return true;
					}
				}
				return false;
			}
			return true;
		};

		/**
		 * @Override
		 */
		WebCenterViewFactory.prototype._createEdge = function (edge, existingViews, scene, viewsDestination) {
			// super always returns false
			if (edge.isVisible()) {
				var fromPinView = existingViews[edge.getFromPin().getId()];
				var toPinView = existingViews[edge.getToPin().getId()];

				if (fromPinView && toPinView) {
					var edgeView = new WebCenterEdgeView(edge, fromPinView, toPinView);

					viewsDestination[edgeView.getId()] = edgeView;
					return true;
				} else {
					this._errorHandler.severe(i18n.t("WorkflowEdgeViewError"));
					return false;
				}
			}
		};

		/**
		 * @Override
		 */
		WebCenterViewFactory.prototype._createToken = function (token, existingViews, scene, viewsDestination) {
			// super always returns false

			var pinView = existingViews[token.getPin().getId()];
			if (pinView && !token.isProcessed() && pinView.getModel().getPinType() === WebCenterPinConstants.Type.In) {
				var tokenView = new WebCenterTokenView(token, pinView, scene);
				viewsDestination[tokenView.getId()] = tokenView;
				return true;
			}
			return false;
		};

		/**
		 * @override
		 */
		WebCenterViewFactory.prototype._createHelper = function (helper, existingViews, scene, viewsDestination) {
			if (!AbstractViewFactory.prototype._createHelper.call(this, helper, existingViews, scene, viewsDestination)) {
				if (helper instanceof StickyNoteHelper) {
					var stickyNoteView = new StickyNoteView(helper, scene);
					viewsDestination[helper.getId()] = stickyNoteView;
					return true;
				}
				return false;
			}
			return true;
		};

		return WebCenterViewFactory;
	});


define('tpl!WFEditor/nodeGallery/view/NodeGalleryList', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<table>\r\n	';
 if(d.nodes.length > 0) { 
__p+='\r\n		';
 for(var i = 0; i<d.nodes.length; i++) { 
__p+='\r\n			<tr data-bind="'+
((__t=(i))==null?'':_.escape(__t))+
'" class="galleryItem">\r\n				<td class="galleryIconTd">\r\n					<img class="galleryIconImg" src="'+
((__t=(d.nodes[i].getIconUrl()))==null?'':_.escape(__t))+
'" alt="'+
((__t=(d.i18n(d.nodes[i].getLabel())))==null?'':_.escape(__t))+
'" />\r\n				</td>\r\n				<td class="galleryLabelTd">\r\n					'+
((__t=(d.i18n(d.nodes[i].getLabel())))==null?'':_.escape(__t))+
'\r\n				</td>\r\n			</tr>\r\n		';
 } 
__p+='\r\n	';
 } else { 
__p+='\r\n		<tr>\r\n			<td class="galleryNoMatches">\r\n				'+
((__t=(d.i18n("galleryNoMatches")))==null?'':_.escape(__t))+
'\r\n			</td>\r\n		</tr>\r\n	';
 } 
__p+='\r\n</table>';
return __p;
}});
define('WFEditor/nodeGallery/view/NodeGalleryList',[
		"Template",
		"tpl!WFEditor/nodeGallery/view/NodeGalleryList",
		"utils/FilterUtils",
		"jquery",
		"underscore",
		"jqueryUI",
		"utils/webcss!./NodeGalleryList-style.css"
	],
	function (
		Template,
		tpl,
		FilterUtils,
		$,
		_
	) {
		"use strict";

		/**
		 * NodeGalleryList is a view object for rendering the list part of a NodeGallery. 
		 * For specific NodeGalleryLists, the render and update methods need to be implemented.
		 * 
		 * @exports NodeGalleryList
		 * @class
		 * 
		 * @param {NodeGallery} nodegallery - the nodegallery to which the NodeGalleryList belongs.
		 * @param {I18n} i18n - instance of i18n
		 */
		var NodeGalleryList = function (nodegallery, i18n) {
			this._nodegallery = nodegallery;
			this._i18n = i18n;

			this._context = null;

			this.t = new Template(this._i18n, tpl);

			this.t.bind("mousedown", "tr", this._handleGalleryNodeActivation, this);
		};

		/**
		 * Render the gallery list
		 * @param   {JQueryObj} context
		 * @param   {Array.<GalleryNode>} nodes - array of nodes to display
		 */
		NodeGalleryList.prototype.render = function (context, nodes) {
			if (context) {
				this._context = context;
				this.t.setContext(this._context);
				this._context.disableSelection();
				this._context.css("cursor", "default");
			}

			if (this._context) {
				this.t.d.nodes = _.sortBy(nodes, _.bind(function (node) {
					return this._i18n.t(node.getLabel()).toLowerCase();
				}, this));

				this.t.render();
			}
		};

		/**
		 * Update the nodes displayed in the gallery
		 * @param   {Array.<GalleryNode>} nodes - array of nodes to display
		 * @param   {String} [filter] - show only nodes whose name match the filter
		 */
		NodeGalleryList.prototype.update = function (nodes, filter) {
			var filteredNodes;
			if (filter) {
				var filterRegExp = FilterUtils.generateRegex(filter, true);
				filteredNodes = _.filter(nodes, _.bind(function (node) {
					return filterRegExp.test(this._i18n.t(node.getLabel()));
				}, this));
			} else {
				filteredNodes = nodes;
			}

			this.render(null, filteredNodes);
		};

		/**
		 * Register listeners on the gallery list context
		 * @param {DOMElement} elem - selected element
		 * @param {Event} e - mouse event                         
		 */
		NodeGalleryList.prototype._handleGalleryNodeActivation = function (elem, e) {
			var idx = $(elem).attr("data-bind");
			if (e.button === 0 && idx) {
				var selectedNode = this.t.d.nodes[idx];
				this._nodegallery.selectNode(selectedNode, e);
			}
		};


		return NodeGalleryList;

	});


define('tpl!WFEditor/nodeGallery/view/NodeGallerySelection', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div class="dropdownGallery dropdown">\r\n    <button class="btn btn-default dropdown-toggle" type="button" id="dropdownMenuGallery" data-toggle="dropdown" aria-haspopup="true" aria-expanded="true">\r\n        <span class="dropdownTitle">'+
((__t=(d.dropdowntitle))==null?'':_.escape(__t))+
'</span>\r\n        <span class="caret"></span>\r\n    </button>\r\n    <ul class="dropdown-menu" aria-labelledby="dropdownMenuGallery">\r\n        ';
 _.each(d.dropdownlist, function(item){ 
__p+='\r\n            <li data-bind="'+
((__t=(item))==null?'':_.escape(__t))+
'">\r\n                <a href="#">'+
((__t=(d.i18n(item)))==null?'':_.escape(__t))+
'</a>\r\n            </li>\r\n        ';
 }); 
__p+='\r\n    </ul>\r\n</div>\r\n            \r\n\r\n<div class="input-group">\r\n  <input type="text" class="form-control filterInput">\r\n  <span class="input-group-btn">\r\n    <button class="btn btn-default" type="button"><i class="filterIcon icon '+
((__t=(d.filtericon))==null?'':_.escape(__t))+
'"></i></button>\r\n  </span>\r\n</div>';
return __p;
}});

define('utils/webcss-text!WFEditor/nodeGallery/view/NodeGallerySelection-style.css',[],function () { return '.dropdownGallery {\r\n    margin-bottom: 2px;\r\n}\r\n.dropdownGallery .dropdown-toggle {   \r\n    overflow: hidden;\r\n    padding-right: 24px ;\r\n    text-align: left;\r\n    text-overflow: ellipsis;    \r\n    width: 100%;\r\n}\r\n.dropdownGallery .dropdown-toggle .caret {\r\n    position: absolute;\r\n    right: 12px;\r\n    top: calc(50% - 2px);\r\n}\r\n#dropdownMenuGallery{\r\n    text-align: left;\r\n}\r\n.dropdownGallery .input-group {\r\n    width: 100%;\r\n}';});


define('WFEditor/nodeGallery/view/NodeGallerySelection',[
		"../NodeGallery",
        "Template",
		"tpl!WFEditor/nodeGallery/view/NodeGallerySelection",
		"jquery",
        "underscore",
        "utils/webcss!./NodeGallerySelection-style.css"
	],
	function (
        NodeGallery,
        Template,
        tpl,
        $,
        _
	) {
		"use strict";

		/**
		 * NodeGallerySelection is a view object for rendering the selection part of a NodeGallery. 
		 * For specific NodeGallerySelections, the render method needs to be implemented.
		 * 
		 * @exports NodeGallerySelection
		 * @class
		 * 
		 * @param {NodeGallery} nodegallery - the nodegallery to which the NodeGallerySelection belongs.
		 * @param {Map.<String,String>} filterIcons - maps css icon classes to icon label name
		 * @param {I18n} i18n - instance of i18n
		 */
		var NodeGallerySelection = function (nodegallery, filterIcons, i18n) {
            this._nodegallery = nodegallery;
            this._i18n = i18n;
            this._filterIcons = filterIcons;
            
            this._selected = "";
            this._filtertext = "";
            
            this._titleElem = undefined;
            this._filterIconElem = undefined;
            this._filterInputElem = undefined;
            
            this.t = new Template(this._i18n, tpl);
            this.t.bind("click", "li", this._selectList, this);
            this.t.bind("keyup", "input", this._filterList, this);
            this.t.bind("click", "button", this._clearFilter, this);
		};

        /**
         * Render the items displayed in the dropdownlist and initialize the filter input field.
         * @param   {JQueryObj} context
         * @param   {Array.<String>} dropdownlist - items for the dropdown
         */
        NodeGallerySelection.prototype.render = function(context, dropdownlist) {
            this.t.setContext(context);
            //init the dropdown
            this.t.d.dropdownlist = _.sortBy(dropdownlist, _.bind(function(item){
                return this._i18n.t(item);
            },this));
            this.t.d.dropdowntitle = this._i18n.t(this.t.d.dropdownlist[0]);
            this._selected = this.t.d.dropdownlist[0];
            
            //init the filter
            this.t.d.filtericon = this._filterIcons.SEARCH;
            this.t.render();
            
            this._titleElem = context.find('.dropdownTitle');
            this._filterIconElem = context.find(".filterIcon");
            this._filterInputElem = context.find(".filterInput");
        };
    
        /**
         * Called when an item is clicked in the dropdown. 
         * Updates the category of nodes that are shown in the gallery.
         * @param el         -     Element that was clicked on in the dropdown.
         * @param event      -     Event
         */
        NodeGallerySelection.prototype._selectList = function(el, event){
            this._selected = $(el).attr("data-bind");
            this._nodegallery.selectList(this._selected, this._filtertext);
            this._titleElem.text($(el).text());
			event.preventDefault();
        };
    
        /**
         * Called when something is typed in the inputfield.
         * Filters the nodes that are shown in the gallery.
         * @param el - The inputfield.
         * @param event - Event
         */
        NodeGallerySelection.prototype._filterList = function(el, event){
            this._filtertext = $(el).val();
            
            if(this._filtertext){
                this._filterIconElem.removeClass(this._filterIcons.SEARCH).addClass(this._filterIcons.CLEAR);
                this.t.d.filtericon = this._filterIcons.CLEAR;
            } else {
                this._filterIconElem.removeClass(this._filterIcons.CLEAR).addClass(this._filterIcons.SEARCH);
                this.t.d.filtericon = this._filterIcons.SEARCH;
            }
            
            this._nodegallery.selectList(this._selected, this._filtertext);
        };
    
        /**
         * Called when the icon next to the inputfield is clicked.
         * Clears the filter.
         * @param el - The button that was clicked
         * @param event - Event
         */
        NodeGallerySelection.prototype._clearFilter = function(el, event){
            //only clear the filter if something was in it, else do nothing
            if (this._filterIconElem.hasClass(this._filterIcons.CLEAR)){
                this._filterInputElem.val("");
                this._filtertext = "";
                this._nodegallery.selectList(this._selected);
                this._filterIconElem.removeClass(this._filterIcons.CLEAR).addClass(this._filterIcons.SEARCH);
            }
        };

		return NodeGallerySelection;

	});


define('utils/webcss-text!WFEditor/nodeGallery/NodeGallery-style.css',[],function () { return '.galleryBorder {\r\n\theight: 100%;\r\n}\r\n.galleryTable {\r\n\theight: 100%;\r\n\tposition: relative;\r\n}\r\n.gallerySelection {\r\n\twidth: 100%;\r\n\tposition: absolute;\r\n}\r\n.galleryListRow {\r\n\tposition: absolute;\r\n\ttop: 65px;\r\n\tbottom: 0px;\r\n\tleft: 0px;\r\n\tright: 0px;\r\n}\r\n.galleryList {\r\n\tpadding: 5px;\r\n\theight: 100%;\r\n\twidth: 100%;\r\n\toverflow: auto;\r\n}\r\n.dropdownGallery > .dropdown-menu {\r\n\twidth: 100%\r\n}\r\n.dropdownGallery .dropdownTitle {\r\n\tdisplay: block;\r\n\toverflow: hidden;\r\n\ttext-overflow: ellipsis;\r\n}';});


define('WFEditor/nodeGallery/NodeGallery',[
		"jquery",
		"underscore",
        "utils/Observable",
        "./view/NodeGalleryList",
        "./view/NodeGallerySelection",
        "utils/webcss!./NodeGallery-style.css"
    ],
	function (
        $,
        _,
        Observable,
        NodeGalleryList,
        NodeGallerySelection,
        NodeGalleryStyle
      ) {
		"use strict";

		/**
		 * NodeGallery is an overview of all the possible nodes for the workflow editor. The user
		 * can add nodes by dragging them from the gallery onto the scene.
		 *
		 * @exports NodeGallery
		 * @class
		 */
		var NodeGallery = function () {
            Observable.mixin(this, "nodeGallery");
            
            /**
			 *  @member {Array.<GalleryNode>} contains all nodes of the nodegallery
			 *  @private
			 */
            this._allNodes = [];
            
            /**
			 *  @member {Map.<String, Array.<GalleryNode>>} maps the node category on the list of nodes belonging to that category
			 *  @private
			 */
            this._nodesPerCategory = {};
            
            /**
			 *  @member {JQueryObj} context for the selection part of the gallery
			 *  @private
			 */
            this._selectionContext = $("<div>").addClass('gallerySelection');
            
            /**
			 *  @member {JQueryObj} context for the node list part of the gallery
			 *  @private
			 */
            this._listContext = $("<div>").addClass('galleryList');
            
            /**
			 *  @member {NodeGalleryList} the NodeGalleryList that belongs to this NodeGallery
			 *  @private
			 */
            this._gallerylist = null;
            
            /**
			 *  @member {NodeGallerySelection} the NodeGallerySelection that belongs to this NodeGallery
			 *  @private
			 */
            this._selection = null;
            
            // initialize NodeGalleryList and NodeGallerySelection
            this._initializeContent();
		};
    
        /**
		 * Set NodeGalleryList and NodeGallerySelection
		 * Override to have specific NodeGalleryList and NodeGallerySelection
		 */
		NodeGallery.prototype._initializeContent = function () {
            this._gallerylist = new NodeGalleryList(this, {}, {});
            this._selection = new NodeGallerySelection(this, {});
        };

		/**
		 * Set the context for the gallery and constructs the DOM structure.
		 * @param {JQueryObj} context
		 */
        NodeGallery.prototype.setContext = function (context) {
			this.context = context;
            
            var borderdiv = $("<div>").addClass("galleryBorder");
            var divtable = $("<div>").addClass("galleryTable");
			divtable.append(this._selectionContext);
            
            var divList = $("<div>").addClass("galleryListRow");
			divList.append(this._listContext);
            divtable.append(divList);
            
            borderdiv.append(divtable);
            this.context.append(borderdiv);
        };

		/**
		 * Sets the nodes to be shown in the gallery and initializes the category dropdown, 
		 * node filter and shown nodes.
		 * @param {Array.<GalleryNode>} nodes
		 */
		NodeGallery.prototype.setNodeDefinitions = function (nodes) {
            this._allNodes = nodes;
            this._nodesPerCategory = {};
            _.each(nodes, function(node){
                if (this._nodesPerCategory[node.getCategory()]){
                    this._nodesPerCategory[node.getCategory()].push(node);
                } else {
                    this._nodesPerCategory[node.getCategory()] = [node];
                }
            }, this);
            
            var dropdownitems = ['all'];
            $.each(this._nodesPerCategory, function(index, value){
                dropdownitems.push(index);
            });
            this._selection.render(this._selectionContext, dropdownitems);
            this._gallerylist.render(this._listContext, this._allNodes);
        };

        /**
         * Updates the galleryList to show nodes of the selected category.
         * @param {String} category - what category should be shown
         * @param {String} [filter] - show only items of the category matching the filter
         */
        NodeGallery.prototype.selectList = function (category, filter) {
            if (category == 'all'){
                this._gallerylist.update(this._allNodes, filter);
            }
            else {
                this._gallerylist.update(this._nodesPerCategory[category], filter);
            }
        };
    
        /**
         * Select a node from the gallery and emit event
         * @param {String} node - selected node
         * @param {Event} e - mouse event that triggeredt this selection                     
         */
        NodeGallery.prototype.selectNode = function (node, e) {
            this._emit("gallery.selectNode", node, e);
        };

		return NodeGallery;

	});

define('WFEditorWCR/nodeGallery/view/WebCenterNodeGalleryList',[
		"WFEditor/nodeGallery/view/NodeGalleryList",
		"utils/webi18n!WFEditorWCR",
	],
	function (
		NodeGalleryList,
		i18n
	) {
		"use strict";

		/**
		 * WebCenterNodeGalleryList is an implementation of the NodeGalleryList for WebCenter.
		 *
		 * @exports WebCenterNodeGalleryList
		 * @class
		 * @augments NodeGalleryList
		 *             
		 * @param {NodeGallery} nodegallery - the nodegallery to which the NodeGalleryList belongs.
		 */
		var WebCenterNodeGalleryList = function (nodegallery) {
			NodeGalleryList.call(this, nodegallery, i18n);
		};

		WebCenterNodeGalleryList.prototype = Object.create(NodeGalleryList.prototype);
		WebCenterNodeGalleryList.prototype.constructor = WebCenterNodeGalleryList;

		return WebCenterNodeGalleryList;

	});

define('WFEditorWCR/nodeGallery/view/WebCenterNodeGallerySelection',[
		"WFEditor/nodeGallery/view/NodeGallerySelection",
		"WFEditorWCR/nodeGallery/WebCenterNodeGalleryConstants",
		"utils/webi18n!WFEditorWCR"
	],
	function (
        NodeGallerySelection,
        WebCenterNodeGalleryConstants,
        i18n
	) {
		"use strict";
        
		/**
		 * WebCenterNodeGallerySelection is an implementation of the NodeGallerySelection for WebCenter.
		 * 
		 * @exports WebCenterNodeGallerySelection
		 * @class
		 * @augments NodeGallerySelection
		 * 
		 * @param {WebCenterNodeGallery} nodegallery
		 */
		var WebCenterNodeGallerySelection = function (nodegallery) {
            NodeGallerySelection.call(this, nodegallery, WebCenterNodeGalleryConstants.FilterIcons, i18n);
		};
    
        WebCenterNodeGallerySelection.prototype = Object.create(NodeGallerySelection.prototype);
		WebCenterNodeGallerySelection.prototype.constructor = WebCenterNodeGallerySelection;
    
		return WebCenterNodeGallerySelection;

	});

define('WFEditorWCR/nodeGallery/WebCenterNodeGallery',[
		"WFEditor/nodeGallery/NodeGallery",
		"WFEditorWCR/nodeGallery/view/WebCenterNodeGalleryList",
		"WFEditorWCR/nodeGallery/view/WebCenterNodeGallerySelection"
	],
	function (
		NodeGallery,
        WebCenterNodeGalleryList,
		WebCenterNodeGallerySelection
	) {
		"use strict";

		/**
		 * WebCenterNodeGallery is an implementation of the NodeGallery for WebCenter.
		 *
		 * @exports WebCenterNodeGallery
		 * @class
		 * @augments NodeGallery
		 */
		var WebCenterNodeGallery = function () {
			NodeGallery.call(this);
		};

		WebCenterNodeGallery.prototype = Object.create(NodeGallery.prototype);
		WebCenterNodeGallery.prototype.constructor = WebCenterNodeGallery;

        /**
		 * Override default NodeGalleryList and NodeGallerySelection
		 * @override
		 */
		WebCenterNodeGallery.prototype._initializeContent = function () {
            this._gallerylist = new WebCenterNodeGalleryList(this);
            this._selection = new WebCenterNodeGallerySelection(this);
        };
    
		return WebCenterNodeGallery;

	});

define('WFEditorWCR/tools/WebCenterToolConstants',{
	/**
	 * WebCenter specific tools can be set here
	 */
	Tools: {
		LABELEDITORTOOL: "LabelEditorTool",
		STICKYNOTE: {
			MOVE: "StickyNoteMoveTool",
			RESIZE: "StickyNoteResizeTool",
			CREATE: "CreateStickyNoteTool"
		}
	},
	State: {
		WAITINGFORDOUBLECLICK: "WaitingForDoubleClick",
		EDITINGLABELS: "EditingLabels",
		STICKYNOTE: {
			SHOULDMOVE: "StickyNoteShouldMoveState",
			MOVE: "MoveStickyNoteState",
			RESIZE: "ResizeStickyNoteState"
		}
	},
	ButtonGroups: {
		MISC: "miscellaneousGroup",
		WEBCENTERNODES: "WebCenterNodesGroup"
	},
	Buttons: {
		EDITCONFIGURATION: "EditConfiguration",
		EDITOUTPUTS: "EditOutputs",
		OPENNODEHELP: "OpenNodeHelp",
		OPENEDITORHELP: "OpenEditorHelp"
	}

});

define(
	'WFEditor/components/InputField',[
		"jquery",
		"underscore"
	],
	function (
		$,
		_
	) {


		"use strict";

		/**
		 * An inputfield that resizes itself according to its contents.
		 * @class
		 * @exports InputField
		 * 
		 * @param {JQueryObj} context - element to which to inputfield will be appended
		 * @param {Number} maxWidth - The maximum width of the input field
		 *                          @param {String} textAlign - text alignment of the inputfield
		 */
		var InputField = function (context, maxWidth, textAlign) {

			/**
			 * @member {String}
			 * @private
			 */
			this._textAlign = textAlign;

			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._context = context;

			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._element = $("<div></div>");

			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._inputElement = $("<textarea />");

			this._element.addClass("inputField");

			this._context.append(this._element);
			this._element.append(this._inputElement);

			/**
			 * @member {Number}
			 * @private
			 */
			this._maxWidth = maxWidth;

			this._registerListeners();

			// Make sure no scroll bar is shown
			this._inputElement.css({
				resize: "none",
				"overflow": "hidden",
				"text-align": textAlign
			});

			// resize the field on input
			this._inputElement.on("input", _.bind(function (e) {
				this._resize();
			}, this));


		};

		/**
		 * show the input
		 */
		InputField.prototype.show = function () {
			this._element.show();
			this._resize();
		};

		/**
		 * hide the input
		 */
		InputField.prototype.hide = function () {
			this._element.hide();
		};

		/**
		 * adds a function to the change event
		 * @param {Function} callback
		 */
		InputField.prototype.onChange = function (callback) {
			this._inputElement.on("change", callback);
		};

		/**
		 * returns the value of the input field
		 * @returns {String}
		 */
		InputField.prototype.getText = function () {
			return this._inputElement.val();
		};

		/**
		 * sets the value of the input field
		 * @param {String} text
		 */
		InputField.prototype.setText = function (text) {
			this._inputElement.val(text);
			this._resize();
		};

		/**
		 * register all listeners on the input element
		 */
		InputField.prototype._registerListeners = function () {
			this._inputElement.on("mousedown mouseup mousemove click dblclick mouseenter", _.bind(function (e) {
				e.stopPropagation();
			}, this));
		};

		/**
		 * resize the input field
		 */
		InputField.prototype._resize = function () {
			// To get the width of the text, we fill in a div with the text and get the width of the div
			var testDiv = $('<div></div>');
			testDiv.hide();
			testDiv.text(this._inputElement.val());
			$("body").append(testDiv);
			var elementWidth = Math.max(3, Math.min(testDiv.width(), this._maxWidth)) + 2;

			this._inputElement.width(elementWidth + "px");
			this._inputElement.height("1px");

			// set the height of the input field
			this._inputElement.height((this._inputElement.get(0).scrollHeight) + "px");

			var location = this._getLocation();

			if (this._textAlign === "center") {
				this._element.css({
					left: location.x - elementWidth / 2,
					top: location.y
				});
			} else if (this._textAlign === "right") {
				this._element.css({
					left: location.x - elementWidth,
					top: location.y
				});
			}

			testDiv.remove();
		};

		/**
		 * gets the location of the input field
		 */
		InputField.prototype._getLocation = function () {
			// should be implemented by a specific input field
		};

		/**
		 * destroys the element
		 */
		InputField.prototype.destroy = function () {
			this._element.remove();
		};

		/**
		 * gives focus to the input element
		 */
		InputField.prototype.focus = function () {
			this._inputElement.focus();
			this._inputElement.select();
		};

		/**
		 * adds event handler to the input element
		 * @param {String} eventName 
		 * @param {Function} callback  
		 */
		InputField.prototype.on = function (eventName, callback) {
			this._inputElement.on(eventName, callback);
		};

		/**
		 * removes event handler from the input element
		 * @param {String} eventName 
		 * @param {Function} callback  
		 */
		InputField.prototype.off = function (eventName, callback) {
			this._inputElement.off(eventName, callback);
		};



		return InputField;

	});

define('WFEditorWCR/tools/labelEditor/NodeLabelInputField',[
		"WFEditor/components/InputField",
		"utils/ShapeUtils",
		"WFEditorWCR/scene/WebCenterViewConstants",
		"utils/Command",
		"underscore"
	],
	function (
		InputField,
		ShapeUtils,
		WebCenterViewConstants,
		Command,
		_
	) {
		"use strict";

		/**
		 * an input field to edit the label of a node
		 * 
		 * @class
		 * @exports NodeLabelInputField
		 * 
		 * @param {Scene} scene
		 * @param {WebCenterNodeView} nodeView
		 * @param {Function} onEnter  what to do on enter?
		 * @param {Function} onEsc  what to do on escape?
		 * @param {UndoRedoController} undoManager
		 * @param {I18n} i18n
		 */
		var NodeLabelInputField = function (scene, nodeView, onEnter, onEsc, undoManager, i18n) {
			InputField.call(this, scene.getContext(), WebCenterViewConstants.NodeLabelMaxWidth, nodeView.getTextAlignment());

			/**
			 * @member {WebCenterNodeView}
			 * @private
			 */
			this._nodeView = nodeView;

			this._inputElement.val(nodeView.getModel().getLabel());

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = scene;

			/**
			 * @member {UndoRedoController}
			 * @private
			 */
			this._undoManager = undoManager;

			/**
			 * @member {I18n}
			 * @private
			 */
			this._i18n = i18n;

			// bind the enter key
			this._inputElement.keydown(_.bind(function (e) {
				if (e.keyCode == 13) {
					onEnter();
					e.preventDefault();
				} else if (e.keyCode == 27) {
					onEsc();
					e.preventDefault();
				}
			}, this));

		};

		NodeLabelInputField.prototype = Object.create(InputField.prototype);
		NodeLabelInputField.prototype.constructor = NodeLabelInputField;

		/**
		 * @override
		 */
		NodeLabelInputField.prototype._getLocation = function () {
			var t = this._scene.getTransformation();
			var location = this._nodeView.getLabelStartPoint();
			location.y -= WebCenterViewConstants.NodeLabelLineHeight;
			return t.transformedPoint(location);
		};

		/**
		 * @override
		 */
		NodeLabelInputField.prototype.show = function () {
			InputField.prototype.show.call(this);
			this._nodeView.getModel().setEditingLabel(true);
			this.focus();

		};

		/**
		 * @override
		 */
		NodeLabelInputField.prototype.hide = function () {
			this._nodeView.getModel().setEditingLabel(false);
			InputField.prototype.hide.call(this);
		};

		/**
		 * set the node's label to the content of the input field
		 */
		NodeLabelInputField.prototype.commit = function () {
			var currentLabel = this._nodeView.getModel().getLabel();
			var newLabel = this._inputElement.val().trim();
			if (currentLabel !== newLabel) {
				var command = new Command(
					_.bind(function () {
						this._nodeView.getModel().setLabel(newLabel);
					}, this),
					_.bind(function () {
						this._nodeView.getModel().setLabel(currentLabel);

					}, this),
					this._i18n.t("editLabelDescription")
				);
				this._nodeView.getModel().setLabel(this._inputElement.val());
				this._undoManager.execute(command);
			}
		};


		return NodeLabelInputField;
	}
);

define(
	'WFEditorWCR/tools/labelEditor/LabelEditorTool',[
		"WFEditor/tools/AbstractTool",
		"utils/Observer",
		"WFEditor/workflow/ModelConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/tools/WebCenterToolConstants",
		"WFEditorWCR/tools/labelEditor/NodeLabelInputField",
		"WFEditor/tools/ToolConstants",
		"underscore"
	],
	function (
		AbstractTool,
		Observer,
		ModelConstants,
		WebCenterNodeConstants,
		WebCenterToolConstants,
		NodeLabelInputField,
		ToolConstants,
		_
	) {

		"use strict";

		/**
		 * @exports LabelEditorTool
		 * @class
		 * @augments AbstractTool
		 * The tool for editing node labels
		 * @param {Observable} mouseInteractionTool 
		 * @param {WFEditorContext} WFEditorContext
		 */
		var LabelEditorTool = function (mouseInteractionTool, WFEditorContext) {
			AbstractTool.call(this, WFEditorContext.get("scene").getContext(), WebCenterToolConstants.Tools.LABELEDITORTOOL, WFEditorContext);
			Observer.mixin(this);

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = WFEditorContext.get("scene");

			/**
			 * @member {ToolManager}
			 * @private
			 */
			this._toolManager = WFEditorContext.get("toolManager");

			/**
			 * @member {ToolbarModel}
			 * @private
			 */
			this._toolbarModel = WFEditorContext.get("toolbarModel");

			/**
			 * @member {NodeLabelInputField}
			 * @private
			 */
			this._nodeLabelInputField = null;

			/**
			 * @member {UndoRedoController}
			 * @private
			 */
			this._undoRedo = WFEditorContext.get("undoRedo");

			/**
			 * @member {I18n}
			 * @private
			 */
			this._i18n = WFEditorContext.get("i18n");

			// Listen to mouse downs
			this._observe(mouseInteractionTool, "mousedown", this._checkMouseDownOnLabel);

			// rerender the input box if the zoom has changed
			this._observe(WFEditorContext.get("sceneModel"), "zoomChanged", _.bind(function () {
				if (this._nodeLabelInputField !== null) {
					// Defer to ensure the transformation has been done
					_.defer(_.bind(function () {
						if (this._nodeLabelInputField) {
							this._nodeLabelInputField.show();
						}
					}, this));
				}
			}, this));
		};

		LabelEditorTool.prototype = Object.create(AbstractTool.prototype);
		LabelEditorTool.prototype.constructor = LabelEditorTool;


		/**
		 * check if the mouse down event received from the mouse interaction tool is on a Label, if so, ask tool manager to become active
		 * @param {Object} options
		 */
		LabelEditorTool.prototype._checkMouseDownOnLabel = function (options) {
			var hitArea = options.hitArea;
			this._doneEditing();
			var isLabelEditingHitArea = hitArea && hitArea.hasClass(WebCenterNodeConstants.HitAreas.LABEL);

			if (isLabelEditingHitArea && hitArea.getView() &&
				hitArea.getView().getModel().getWorkflowItemType() === ModelConstants.WorkflowItemType.Node) {
				if (hitArea.getView().getModel().getWorkflow().getPrivilege(WebCenterNodeConstants.Privileges.editLabels)) {
					// There was a double click on a NodeLabel hitArea, the tool can be activated
					this._nodeLabelInputField = new NodeLabelInputField(this._scene, hitArea.getView(), _.bind(this._doneEditing, this), _.bind(this._stopWithoutSaving, this), this._undoRedo, this._i18n);
					this._nodeLabelInputField.on("blur", _.bind(this._doneEditing, this));
					this._nodeLabelInputField.show();
				}
			}
		};


		/**
		 * indicates that the editing has been finished, set the node's label if it isn't empty and hide the input field.
		 */
		LabelEditorTool.prototype._doneEditing = function () {
			if (this._nodeLabelInputField !== null) {
				this._nodeLabelInputField.hide();
				if (this._nodeLabelInputField.getText() !== "") {
					this._nodeLabelInputField.commit();
				}
				this._nodeLabelInputField.destroy();
				this._nodeLabelInputField = null;
			}

		};

		/**
		 * Stops the editing without saving it to the node's label
		 */
		LabelEditorTool.prototype._stopWithoutSaving = function () {
			if (this._nodeLabelInputField !== null) {
				this._nodeLabelInputField.hide();
				this._nodeLabelInputField.destroy();
				this._nodeLabelInputField = null;
			}
		};

		return LabelEditorTool;

	}
);

define(
	'WFEditorWCR/tools/stickyNoteTools/MoveStickyNoteTool',[
		"underscore",
		"WFEditor/tools/AbstractInteractiveTool",
		"WFEditorWCR/tools/WebCenterToolConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditor/tools/ToolConstants",
		"utils/Observer",
		"utils/Command"
	],
	function (_, AbstractInteractiveTool, WebCenterToolConstants, WebCenterNodeConstants, ToolConstants, Observer, Command) {

		"use strict";

		/**
		 * @class
		 * @exports MoveStickyNoteTool
		 * @augments AbstractInteractiveTool
		 * @mixes Observer
		 * 
		 * @param {SceneController}    sceneController        
		 * @param {Observable}         interactionEvents 
		 * @param {WFEditorContext}    WFEditorContext          
		 */
		var MoveStickyNoteTool = function (sceneController, interactionEvents, WFEditorContext) {
			AbstractInteractiveTool.call(this, WebCenterToolConstants.Tools.STICKYNOTE.MOVE, sceneController, WFEditorContext);
			Observer.mixin(this);

			this._observe(interactionEvents, "mousedown", this._handleMouseInteractionEvent);
			this._observe(interactionEvents, "mousemove", this._handleMouseInteractionEvent);
			this._observe(interactionEvents, "mouseup", this._handleMouseInteractionEvent);

			/**
			 * @member {StickyNoteView}
			 * @private
			 */ 
			this._stickyNoteView = null;

			/**
			 * @member {Scene}
			 * @private
			 */ 
			this._scene = WFEditorContext.get("scene");

			/**
			 * @member {ShapeUtils.Location}
			 * @private
			 */ 
			this._previousWorldPoint = null;

			/**
			 * @member {UndoRedoController}
			 * @private
			 */ 
			this._undoRedo = WFEditorContext.get("undoRedo");

			/**
			 * @member {Boolean}
			 * @private
			 */ 
			this._isMoving = false;

			/**
			 * @member {ToolbarModel}
			 * @private
			 */ 
			this._toolbarModel = WFEditorContext.get("toolbarModel");
			
			this.changeState(ToolConstants.State.DEFAULTSTATE);

			this.registerActiveEvents(["mousemove", "mouseup", "mousedown", "mouseleave", "mouseenter"]);

			this._registerStates();
		};

		MoveStickyNoteTool.prototype = Object.create(AbstractInteractiveTool.prototype);
		MoveStickyNoteTool.prototype.constructor = MoveStickyNoteTool;

		MoveStickyNoteTool.prototype._handleMouseInteractionEvent = function (options) {
			this.triggerEvent(options.eventName, options);
		};


		/**
		 * registers all states to handle the mouse interaction
		 */
		MoveStickyNoteTool.prototype._registerStates = function () {

			this.registerState(ToolConstants.State.DEFAULTSTATE, {
				"mousedown": _.bind(function (options) {
					var hitArea = options.hitArea;
					if (options.e.button !== 0 || !hitArea || (hitArea && !hitArea.hasClass(WebCenterNodeConstants.HitAreas.STICKYNOTE.MOVE))) {
						// Not a hit area we should react to
						return;
					}
					this.changeState(WebCenterToolConstants.State.STICKYNOTE.SHOULDMOVE, options);
				}, this),
				"mousemove": function () {},
				"mouseup": function () {},


			});

			this.registerState(WebCenterToolConstants.State.STICKYNOTE.SHOULDMOVE, {
				"enterState": _.bind(function (fromState, options) {
					this._stickyNoteView = options.hitArea.getView();
					this._startLocation = this._stickyNoteView.getModel().getLocation().copy();
				}, this),
				"mousemove": _.bind(function (options) {
					this.changeState(WebCenterToolConstants.State.STICKYNOTE.MOVE, options);
				}, this),
				"mouseup": _.bind(function (options) {
					this.changeState(ToolConstants.State.DEFAULTSTATE);
				}, this),
				"mousedown": function () {}
			});

			this.registerState(WebCenterToolConstants.State.STICKYNOTE.MOVE, {
				"enterState": _.bind(function (fromState, options) {
					this._scene.setInteractive(this._stickyNoteView.getId(), true);
					this._toolManager.activateTool(this);
					this._previousWorldPoint = this._scene.getWorldPointFromScenePoint(this._getMousePosition(options.e));
					this._isMoving = true;
				}, this),
				"leaveState": _.bind(function (fromState) {
					this._scene.setInteractive(this._stickyNoteView.getId(), false);
					var stickyNote = this._stickyNoteView.getModel();
					var oldLocation = this._startLocation;
					var newLocation = this._stickyNoteView.getModel().getLocation().copy();
					this._isMoving = false;
					this._undoRedo.execute(new Command(
						function () {
							stickyNote.setLocation(newLocation);
						},
						function () {
							stickyNote.setLocation(oldLocation);
						}, "moveStickyNoteDescription"
					), true);

				}, this),
				"mousemove": _.bind(function (e) {
					var currentMousePos = this._getMousePosition(e);
					e.preventDefault();
					e.stopPropagation();
					// Panning close to borders
					this._stopIntervalPanning();
					if (this._shouldPan(currentMousePos)) {
						this._calculatePanning(currentMousePos);
						this._startIntervalPanning(currentMousePos, this._moveStickyNote);
					}
					this._moveStickyNote(currentMousePos);
				}, this),
				"mouseup": _.bind(function (e) {
					this._toolManager.deactivateCurrentTool(true);
				}, this),
				"mouseleave": _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					this._continuePanningWhileMouseOutsideCanvas(this._moveStickyNote, _.bind(function (mousePosition) {
						this._moveStickyNote(mousePosition);
						this._toolManager.deactivateCurrentTool(true);
					}, this));
				}, this),
				"mouseenter": _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					this._stopPanningWhileMouseOutsideCanvas();
				}, this)
			});
		};

		/**
		 * @override
		 */
		MoveStickyNoteTool.prototype.activate = function (event, silent) {
			AbstractInteractiveTool.prototype.activate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, true);
		};

		/**
		 * moves the sticky note to current mouse position
		 * @param {ShapeUtils.Location} currentMousePos 
		 */
		MoveStickyNoteTool.prototype._moveStickyNote = function (currentMousePos) {
			var newWorldPoint = this._scene.getWorldPointFromScenePoint(currentMousePos);
			var offsetX = newWorldPoint.x - this._previousWorldPoint.x;
			var offsetY = newWorldPoint.y - this._previousWorldPoint.y;
			this._stickyNoteView.getModel().offsetXY(offsetX, offsetY);
			this._previousWorldPoint = newWorldPoint;

		};

		/**
		 * @override
		 */
		MoveStickyNoteTool.prototype.allowDeactivate = function () {
			return !this._isMoving;
		};

		/**
		 * @override
		 */
		MoveStickyNoteTool.prototype.getDefaultCursor = function () {
			return "grabbing-cursor";
		};

		return MoveStickyNoteTool;

	});

define(
	'WFEditorWCR/tools/stickyNoteTools/ResizeStickyNoteTool',[
		"underscore",
		"WFEditor/tools/AbstractTool",
		"WFEditorWCR/tools/WebCenterToolConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditor/tools/ToolConstants",
		"utils/Observer",
		"utils/Command"
	],
	function (_, AbstractTool, WebCenterToolConstants, WebCenterNodeConstants, ToolConstants, Observer, Command) {

		"use strict";

		/**
		 * Tool for resizing a sticky note
		 * @class
		 * @augments AbstractTool
		 * @mixes Observer
		 * @exports ResizeStickyNoteTool
		 * 
		 * @param {SceneController}      sceneController      
		 * @param {Observable} interactionEvents 
		 * @param {WFEditorContext}   WFEditorContext          
		 */
		var ResizeStickyNoteTool = function (sceneController, interactionEvents, WFEditorContext) {
			AbstractTool.call(this, WFEditorContext.get("scene").getContext(), WebCenterToolConstants.Tools.STICKYNOTE.RESIZE, WFEditorContext);
			Observer.mixin(this);

			this._observe(interactionEvents, "mousedown", this._handleMouseInteractionEvent);

			/**
			 * @member {StickyNoteView}
			 * @private
			 */
			this._stickyNoteView = null;

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = WFEditorContext.get("scene");

			/**
			 * @member {ToolManager}
			 * @private
			 */
			this._toolManager = WFEditorContext.get("toolManager");

			/**
			 * @member {ToolbarModel}
			 * @private
			 */
			this._toolbarModel = WFEditorContext.get("toolbarModel");

			this.changeState(ToolConstants.State.DEFAULTSTATE);

			this.registerActiveEvents(["mousemove", "mouseup"]);

			this._registerStates();

			/**
			 * @member {ShapeUtils.Location}
			 * @private
			 */
			this._previousWorldPoint = null;
			
			/**
			 * @member {ShapeUtils.Location}
			 * @private
			 */
			this._startSize = null;

			/**
			 * @member {UndoRedoController}
			 * @private
			 */
			this._undoRedo = WFEditorContext.get("undoRedo");

			/**
			 * @member {Boolean}
			 * @private
			 */
			this._isResizing = false;

		};

		ResizeStickyNoteTool.prototype = Object.create(AbstractTool.prototype);
		ResizeStickyNoteTool.prototype.constructor = ResizeStickyNoteTool;

		/**
		 * handle the events received from the mouse interaction tool
		 * @param {Object} options 
		 */
		ResizeStickyNoteTool.prototype._handleMouseInteractionEvent = function (options) {
			this.triggerEvent(options.eventName, options);
		};


		/**
		 * @override
		 */
		ResizeStickyNoteTool.prototype.activate = function (event, silent) {
			AbstractTool.prototype.activate.call(this, event, silent);
			this._toolbarModel.setButtonActive(ToolConstants.Buttons.SELECTION, true);
		};

		/**
		 * register states for sticky note resizing
		 */
		ResizeStickyNoteTool.prototype._registerStates = function () {

			this.registerState(ToolConstants.State.DEFAULTSTATE, {
				"mousedown": _.bind(function (options) {
					var hitArea = options.hitArea;
					if (!hitArea || (hitArea && !hitArea.hasClass(WebCenterNodeConstants.HitAreas.STICKYNOTE.RESIZE))) {
						// Not a hit area we should react to
						return;
					}
					this._toolManager.activateTool(this);
					this.changeState(WebCenterToolConstants.State.STICKYNOTE.RESIZE, options);
				}, this),
				"mousemove": function () {},
				"mouseup": function () {},


			});

			this.registerState(WebCenterToolConstants.State.STICKYNOTE.RESIZE, {
				"enterState": _.bind(function (fromState, options) {
					this._stickyNoteView = options.hitArea.getView();
					this._scene.setInteractive(this._stickyNoteView.getId(), true);
					this._previousWorldPoint = this._scene.getWorldPointFromScenePoint(this._getMousePosition(options.e));
					this._startSize = this._stickyNoteView.getModel().getSize().copy();
					this._isResizing = true;
				}, this),
				"mousemove": _.bind(function (e) {
					var currentMousePos = this._getMousePosition(e);
					e.preventDefault();
					e.stopPropagation();
					this._resizeStickyNote(currentMousePos);
				}, this),
				"mouseup": _.bind(function (e) {

					var oldSize = this._startSize;
					var newSize = this._stickyNoteView.getModel().getSize().copy();
					var stickyNote = this._stickyNoteView.getModel();
					this._undoRedo.execute(new Command(function () {
							stickyNote.setSize(newSize.width, newSize.height);
						},
						function () {
							stickyNote.setSize(oldSize.width, oldSize.height);
						},
						"resizeStickyNoteDescription"), true);
					this._isResizing = false;
					this._scene.setInteractive(this._stickyNoteView.getId(), false);
					this._toolManager.deactivateCurrentTool();
				}, this)
			});
		};

		/**
		 * resize a sticky note
		 * @param {ShapeUtils.Point} currentMousePos 
		 */
		ResizeStickyNoteTool.prototype._resizeStickyNote = function (currentMousePos) {
			var newWorldPoint = this._scene.getWorldPointFromScenePoint(currentMousePos);
			var offsetX = newWorldPoint.x - this._previousWorldPoint.x;
			var offsetY = newWorldPoint.y - this._previousWorldPoint.y;
			this._stickyNoteView.getModel().setSize(this._startSize.width + offsetX, this._startSize.height + offsetY);


		};

		/**
		 * @override
		 */
		ResizeStickyNoteTool.prototype.getDefaultCursor = function () {
			return "nwse-resize-cursor";
		};

		/**
		 * @override
		 */
		ResizeStickyNoteTool.prototype.allowDeactivate = function () {
			return !this._isResizing;
		};

		return ResizeStickyNoteTool;

	});

define(
	'WFEditorWCR/tools/stickyNoteTools/StickyNoteTextEditorTool',[
		"WFEditor/tools/AbstractTool",
		"utils/Observer",
		"WFEditor/workflow/ModelConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/tools/WebCenterToolConstants",
		"WFEditor/tools/ToolConstants",
		"utils/webi18n!WFEditorWCR",
		"utils/Command",
		"underscore"
	],
	function (
		AbstractTool,
		Observer,
		ModelConstants,
		WebCenterNodeConstants,
		WebCenterToolConstants,
		ToolConstants,
		i18n,
		Command,
		_
	) {

		"use strict";

		/**
		 * @exports StickyNoteTextEditdorTool
		 * @class
		 * @augments AbstractTool
		 * The tool for editing node labels      
		 * @param {Observable} mouseInteractionTool 
		 * @param {WFEditorContext} WFEditorContext
		 */
		var StickyNoteTextEditdorTool = function (mouseInteractionTool, WFEditorContext) {
			AbstractTool.call(this, WFEditorContext.get("scene").getContext(), WebCenterToolConstants.Tools.StickyNoteTextEditdorTool, WFEditorContext);
			Observer.mixin(this);

			/**
			 * @member {Scene}
			 * @private
			 */
			this._scene = WFEditorContext.get("scene");

			/**
			 * @member {ToolManager}
			 * @private
			 */
			this._toolManager = WFEditorContext.get("toolManager");

			/**
			 * @member {UndoRedoController}
			 * @private
			 */
			this._undoRedo = WFEditorContext.get("undoRedo");

			/**
			 * @member {I18n}
			 * @private
			 */
			this._i18n = WFEditorContext.get("i18n");

			/**
			 * @member {StickyNoteView}
			 * @private
			 */
			this._stickyNoteView = null;

			// If not active, listen to double clicks from the mouse interaction tool
			this._observe(mouseInteractionTool, "mousedown", this._checkClickOnEditIcon);
			this._observe(WFEditorContext.get("scene"), "StickyNotePopup.*.saved", this._doneEditing);

		};

		StickyNoteTextEditdorTool.prototype = Object.create(AbstractTool.prototype);
		StickyNoteTextEditdorTool.prototype.constructor = StickyNoteTextEditdorTool;


		/**
		 * check if the tool should start editing or delete the sticky note
		 * @param {Object} options 
		 */
		StickyNoteTextEditdorTool.prototype._checkClickOnEditIcon = function (options) {
			if (options.e.button !== 0) {
				return;
			}
			var hitArea = options.hitArea;

			var isLabelEditingHitArea = hitArea && (
				hitArea.hasClass(WebCenterNodeConstants.HitAreas.STICKYNOTE.EDITICON) ||
				hitArea.hasClass(WebCenterNodeConstants.HitAreas.STICKYNOTE.TEXTAREA));
			var isDeleteHitArea = hitArea && hitArea.hasClass(WebCenterNodeConstants.HitAreas.STICKYNOTE.DELETE);

			this._doneEditing();

			if (isLabelEditingHitArea) {
				this._stickyNoteView = hitArea.getView();
				this._stickyNoteView.getModel().setEditingText(true);

			} else if (isDeleteHitArea) {
				this._deleteStickyNote(hitArea.getView().getModel());
			}
		};


		/**
		 * indicates that the editing has been finished
		 */
		StickyNoteTextEditdorTool.prototype._doneEditing = function () {
			if (this._stickyNoteView && this._stickyNoteView.getModel().isEditingText()) {
				this._setNewStickyNoteText();
				this._stickyNoteView.getModel().setEditingText(false);
			}

		};

		/**
		 * set text of the sticky note
		 */
		StickyNoteTextEditdorTool.prototype._setNewStickyNoteText = function () {
			var currentText = this._stickyNoteView.getModel().getText();
			var newText = this._stickyNoteView.getStickyNotePopup().getEditedText().trim();
			if (currentText !== newText) {
				var command = new Command(
					_.bind(function () {
						this._stickyNoteView.getModel().setText(newText);
					}, this),
					_.bind(function () {
						this._stickyNoteView.getModel().setText(currentText);
					}, this),
					this._i18n.t("editStickyNoteTextDescription")
				);
				this._undoRedo.execute(command);
			}
		};

		/**
		 * Stops the editing without saving it to the node's label
		 */
		StickyNoteTextEditdorTool.prototype._stopWithoutSaving = function () {
			if (this._stickyNoteView && this._stickyNoteView.getModel().isEditingText()) {
				this._stickyNoteView.getModel().setEditingText(false);

			}
		};

		/**
		 * deletes a sticky note
		 * @param {StickyNoteHelper} stickyNote 
		 */
		StickyNoteTextEditdorTool.prototype._deleteStickyNote = function (stickyNote) {
			var command = new Command(_.bind(function () {
					this._scene.getWorkflowModel().removeHelper(stickyNote);
				}, this), _.bind(function () {
					this._scene.getWorkflowModel().addHelper(stickyNote);
				}, this),
				this._i18n.t("removeStickyNoteDescription"));
			this._undoRedo.execute(command);
		};

		/**
		 * @override
		 * @returns {String} 
		 */
		StickyNoteTextEditdorTool.prototype.getDefaultCursor = function () {
			return "default-cursor";
		};

		return StickyNoteTextEditdorTool;

	}
);

define('WFEditorWCR/scene/WebCenterSceneController',[
		"WFEditor/scene/SceneController",
		"WFEditorWCR/tools/labelEditor/LabelEditorTool",
		"WFEditorWCR/tools/stickyNoteTools/MoveStickyNoteTool",
		"WFEditorWCR/tools/stickyNoteTools/ResizeStickyNoteTool",
		"WFEditorWCR/tools/stickyNoteTools/StickyNoteTextEditorTool",
		"WFEditorWCR/workflow/helper/stickyNote/StickyNoteHelper",
		"WFEditorWCR/tools/WebCenterToolConstants",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditor/workflow/ModelConstants",
		"component/helpIcon/HelpIcon",
		"utils/Command",
		"utils/Observer",
		"underscore"
	],
	function (
		SceneController,
		LabelEditorTool,
		MoveStickyNoteTool,
		ResizeStickyNoteTool,
		StickyNoteTextEditorTool,
		StickyNoteHelper,
		WebCenterToolConstants,
		WebCenterNodeConstants,
		ModelConstants,
		HelpIcon,
		Command,
		Observer,
		_

	) {
		"use strict";

		/**
		 * Controller for the Scene of a webcenter workflow. Extends the SceneController
		 * and adds specific WebCenter functionality to the contoller.
		 *
		 * @exports WebCenterSceneController
		 * @class
		 * @augments SceneController
		 *
		 * @param {AbstractWorkflowFactory} workflowFactory - the workflow factory to create objects
		 * @param {SceneModel} sceneModel
		 * @param {Workflow} workflowModel
		 * @param {Scene} scene
		 * @param {WFEditorContext} WFEditorContext
		 */
		var WebCenterSceneController = function (workflowFactory, sceneModel, workflowModel, scene, WFEditorContext) {
			SceneController.call(this, workflowFactory, sceneModel, workflowModel, scene, WFEditorContext);
			Observer.mixin(this);

			// init label editor tool
			new LabelEditorTool(this._mouseInteractionTool, WFEditorContext);
			if (this._workflowModel.getPrivilege(WebCenterNodeConstants.Privileges.editStickyNotes)) {
				new StickyNoteTextEditorTool(this._mouseInteractionTool, WFEditorContext);
				this._contextMenuModel.addComponent(WebCenterToolConstants.ButtonGroups.MISC);
				this._contextMenuModel.addButton(WebCenterToolConstants.ButtonGroups.MISC,
					WebCenterToolConstants.Tools.STICKYNOTE.CREATE,
					WFEditorContext.get("i18n").t("createStickyNote"), "icon-WCR_WF-AddSticky12 icon-normal", _.bind(function (e) {
						this._createStickyNote(this._contextMenuModel.getContextMenuWorldLocation());
					}, this), true);
				this._contextMenuModel.addButton(WebCenterToolConstants.ButtonGroups.MISC,
					WebCenterToolConstants.Buttons.OPENEDITORHELP,
					WFEditorContext.get("i18n").t("openEditorHelp"), "icon-WCR_Info-Balloon icon-normal", _.bind(function (e) {
						var help = HelpIcon.getHelpURL(WebCenterNodeConstants.HelpTopics.EDITOR);
						help.onClickFunction();
					}, this), true);
			}
			new MoveStickyNoteTool(this, this._mouseInteractionTool, WFEditorContext);
			new ResizeStickyNoteTool(this, this._mouseInteractionTool, WFEditorContext);

			if (this._workflowModel.getPrivilege(ModelConstants.Privileges.editNodeConfiguration)) {

				this._initializeNodeConfigurationContextMenuButtons();
			}


			this._observe(this._workflowModel, "Node.*.Pin.*.Token.*.tokenChanged", this._tokenChanged);
		};

		WebCenterSceneController.prototype = Object.create(SceneController.prototype);
		WebCenterSceneController.prototype.constructor = WebCenterSceneController;

		/**
		 * A token got selected in the view, change the correct settings in the model.
		 *
		 * @param {Token} token - the token that got selected
		 * @param {String} documentId - the id of the document in the token that got selected
		 */
		WebCenterSceneController.prototype._tokenItemSelected = function (token, documentId) {
			if (token) {
				token.setSelected(true, documentId); //make sure to add the selected document to the selected state
			}
			SceneController.prototype._tokenItemSelected.call(this, token, documentId);
		};

		/**
		 * A token's status has changed, if it is now processed, remove the view
		 * @param {WebCenterToken} token
		 */
		WebCenterSceneController.prototype._tokenChanged = function (token) {
			if (token.isProcessed()) {
				// Check if this token was highlighted or if it is an operation token, one of its subtokens has been highlighted
				var highlightedToken = this._workflowModel.getHightLightedToken();
				if (highlightedToken && (token === highlightedToken || (token.hasSubToken && token.hasSubToken(highlightedToken)))) {
					// remove the highlights
					this._highlightTokenPath(highlightedToken, false);
					highlightedToken.setSelected(false);
				}
				this._scene.removeViewsForWorkflowItem(token);
			}
		};

		/**
		 * @override
		 * Same as super function, but checks for implicit edges
		 */
		WebCenterSceneController.prototype._doCreateEdge = function (workflow, fromPin, toPin) {
			// accepts the edge and the edge does not yet exist
			if (toPin.acceptsIncomingEdges(fromPin) && !fromPin.getEdgeTo(toPin)) {
				// find next possible id
				var i = 0;
				var checkExistingEdgeId = function (edge) {
					return edge.getId() === fromPin.getId() + "/" + i;
				};
				while (_.find(fromPin.getEdges(), checkExistingEdgeId)) {
					i++;
				}

				var edge = this._workflowFactory.createEdge(workflow, fromPin.getId() + "/" + i, fromPin, toPin);

				var toNode = toPin.getNode();
				if (toNode.isImplicit && toNode.isImplicit()) {
					edge.setImplicit(true);
				}
				fromPin.addEdge(edge);
				toPin.addEdge(edge);
				workflow.addEdge(edge);
			}
		};

		/**
		 * create a sticky note
		 * @param {ShapeUtils.Position} mousePosition 
		 */
		WebCenterSceneController.prototype._createStickyNote = function (mousePosition) {
			var location = mousePosition.copy();
			location.offsetXY(-WebCenterNodeConstants.StickyNote.DEFAULT_WIDTH / 2, 0);
			var stickyNote = new StickyNoteHelper(this._workflowModel, location);
			this._undoRedo.execute(new Command(_.bind(function () {
				this._workflowModel.addHelper(stickyNote);
			}, this), _.bind(function () {
				this._workflowModel.removeHelper(stickyNote);
			}, this), this._i18n.t("createStickyNote")));
		};

		/**
		 * @override
		 * add hostname to object to check for cross server pasting
		 */
		WebCenterSceneController.prototype._getObjectForCopy = function (selectedNodes) {
			var json = SceneController.prototype._getObjectForCopy.call(this, selectedNodes);
			json.wcrHost = location.hostname;
			return json;
		};

		/**
		 * @override
		 * don't allow cross server pasting, there are import/export tools for this
		 */
		WebCenterSceneController.prototype._createNodesFromPaste = function (nodesObject) {
			if (nodesObject.wcrHost === location.hostname) {
				SceneController.prototype._createNodesFromPaste.call(this, nodesObject);
			} else {
				this._errorHandler.warning(this._i18n.t("noCrossServerPaste"));
			}
		};

		/**
		 * opens a warning policy panel for node
		 * @param {WebCenterNode}
		 */
		WebCenterSceneController.prototype._openWarningPolicyPanel = function (node) {
			// Create a proxy that redirects the parameters configuration methods to the warning policy configuration
			var nodeProxy = Object.create(node);
			nodeProxy.getConfigurationPanel = function () {
				var panelPromise = $.Deferred();
				requirejs(["WFEditorWCR/workflow/panel/WarningPanel"], _.bind(function (WarningPanel) {
					var nodeConfigPanel = new WarningPanel(node);
					panelPromise.resolve(nodeConfigPanel);
				}, this));
				return panelPromise.promise();
			};
			nodeProxy.setConfiguration = function (configuration) {
				node.setWarningPolicy(configuration);
			};
			nodeProxy.getConfiguration = function () {
				return node.getWarningPolicy();
			};
			this._nodeConfigurationController.openNodeConfiguration(nodeProxy, "warningpolicy");
		};


		/**
		 * adds the node configuration buttons to the context menu
		 */
		WebCenterSceneController.prototype._initializeNodeConfigurationContextMenuButtons = function () {
			this._contextMenuModel.addComponent(WebCenterToolConstants.ButtonGroups.WEBCENTERNODES);

			// edit parameters
			this._contextMenuModel.addButton(WebCenterToolConstants.ButtonGroups.WEBCENTERNODES,
				WebCenterToolConstants.Buttons.EDITCONFIGURATION,
				this._i18n.t("editParametersContextMenuItem"), "icon-normal icon-WCR_Actions-Edit12",
				_.bind(function (e) {
					this.openNodeConfiguration(this._contextMenuModel.getHitArea().getView().getModel());
				}, this), false, false, false);

			// configure outputs
			this._contextMenuModel.addButton(WebCenterToolConstants.ButtonGroups.WEBCENTERNODES,
				WebCenterToolConstants.Buttons.EDITOUTPUTS,
				this._i18n.t("configureOutputsContextMenuItem"), "icon-normal icon-WCR_Configuration",
				_.bind(function (e) {
					this._openWarningPolicyPanel(this._contextMenuModel.getHitArea().getView().getModel());
				}, this), false, false, false);

			// open node help
			this._contextMenuModel.addButton(WebCenterToolConstants.ButtonGroups.WEBCENTERNODES,
				WebCenterToolConstants.Buttons.OPENNODEHELP,
				this._i18n.t("openNodeHelpContextMenuItem"), "icon-normal icon-WCR_Info-Balloon",
				_.bind(function (e) {
					var node = this._contextMenuModel.getHitArea().getView().getModel();
					var help = HelpIcon.getHelpURL(node.getHelpTopic());
					help.onClickFunction();
				}, this), false, false, false);

			this._observe(this._contextMenuModel, "changed.hitArea", function (hitArea) {
				if (hitArea) {
					var model = hitArea.getView().getModel();
					var isNode = model.getWorkflowItemType() === ModelConstants.WorkflowItemType.Node;
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.EDITCONFIGURATION, isNode);
					this._contextMenuModel.setButtonEnabled(WebCenterToolConstants.Buttons.EDITCONFIGURATION, isNode && model.isConfigurable());
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.EDITOUTPUTS, isNode);
					this._contextMenuModel.setButtonEnabled(WebCenterToolConstants.Buttons.EDITOUTPUTS, isNode && model.canEditWarningPolicy());
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.OPENNODEHELP, isNode);
					this._contextMenuModel.setButtonEnabled(WebCenterToolConstants.Buttons.OPENNODEHELP, isNode && model.getHelpTopic());
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.OPENEDITORHELP, !isNode); // only node help is enough
				} else {
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.EDITOUTPUTS, false);
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.EDITCONFIGURATION, false);
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.OPENNODEHELP, false);
					this._contextMenuModel.setButtonVisible(WebCenterToolConstants.Buttons.OPENEDITORHELP, true);
				}
			});
		};

		/** 
		 * @override
		 */
		WebCenterSceneController.prototype.removeEdges = function (workflow, edges, undoable, skipDo) {
			if (!undoable) {
				SceneController.prototype.removeEdges.call(this, workflow, edges, undoable, skipDo);
			} else {
				// It is possible that some edges cannot be redone, eg. an edge between an output pin and an input pin
				// of the same node. These types of edges can be made with the java editor but are no longer supported in the
				// HTML5 Version.
				var edgesThatCanBeUndone = _.filter(edges, function (edge) {
					return edge.getToPin().acceptsIncomingEdges(edge.getFromPin());
				});
				var edgesThatCannotBeUndone = _.without(edges, edgesThatCanBeUndone);

				if (edgesThatCanBeUndone.length > 0) {
					SceneController.prototype.removeEdges.call(this, workflow, edgesThatCanBeUndone, true, skipDo);
				}
				
				if (edgesThatCannotBeUndone.length > 0) {
					SceneController.prototype.removeEdges.call(this, workflow, edgesThatCannotBeUndone, false, skipDo);
					this._undoRedo.addForceModifiedFlag();
				}
			}

		};



		return WebCenterSceneController;

	});

define('WFEditorWCR/toolbar/StatusToolbarComponent',[
		"component/tooltip/Tooltip",
		"utils/webi18n!WFEditorWCR"
	],
	function (
		Tooltip,
		i18n //only t is supported never provide langBundle from here!
	) {
		"use strict";

		/**
		 * View for the showing the task status in the toolbar of the workflow editor
		 *
		 * @exports StatusToolbarComponent
		 * @class
		 *
		 * @param {String} url to the icon of the task status
		 * @param {String} the name of the task status
		 * @param {JQueryObj} globalContext
		 */
		var StatusToolbarComponent = function (icon, status, globalContext) {

			/**
			 * @member {String} the url to the icon for the task status
			 * @private
			 */
			this._icon = icon;

			/**
			 * @member {String} the name of the task status
			 * @private
			 */
			this._status = status;

			/**
			 * @member {JQueryObj} the element of the component
			 */
			this.element = $("<div>").addClass("statusToolbarComponent").addClass(this._name);
			this._taskStatus = $("<span>").text(this._status).addClass("enableTextSelection");
			this._content = $("<div>").addClass("statusToolbarContent");
			if (this._icon) {
				this._iconEl = $("<img>").attr("src", this._icon); //the image icon
				this._content.append(this._iconEl);
			}
			this._content.append(this._taskStatus);
			//append the content to the components element
			this.element.append(this._content);

			//add the correct tooltip
			this._tooltip = new Tooltip(i18n, this._content, globalContext, globalContext, false, i18n.t("workflowStatus"));
		};

		/**
		 * Update the status toolbar component in the workflow editor
		 * @param {Object} data - contains the icon and status properties, icon is not needed
		 */
		StatusToolbarComponent.prototype.updateView = function (data) {
			this._icon = data.icon;
			this._status = data.status;
			this._content.empty();

			if (this._iconEl) {
				if (this._icon) {
					this._iconEl.attr("src", this._icon);
				} else {
					this._iconEl = null;
				}
			} else {
				this._iconEl = $("<img>").attr("src", this._icon);
			}

			if (this._iconEl) {
				this._content.append(this._iconEl);
			}

			this._taskStatus.text(this._status);
			this._content.append(this._taskStatus);
		};

		return StatusToolbarComponent;
	});

define('WFEditorWCR/toolbar/WebCenterToolbar',[
		"WFEditor/toolbar/Toolbar",
		"component/toolbar/ButtonGroup",
		"component/toolbar/ToolbarButton",
		"component/toolbar/ToolbarComponent",
		"WFEditor/tools/ToolConstants",
		"WFEditorWCR/core/WebCenterWFEditorConstants",
		"WFEditorWCR/toolbar/StatusToolbarComponent",
		"utils/webi18n!WFEditorWCR"
	],
	function (
		Toolbar,
		ButtonGroup,
		ToolbarButton,
		ToolbarComponent,
		ToolConstants,
		WebCenterWFEditorConstants,
		StatusToolbarComponent,
		i18n //only t is supported never provide langBundle from here!
	) {
		"use strict";

		/**
		 * View for the toolbar with all its components for WebCenter.
		 *
		 * @exports WebCenterToolbar
		 * @class
		 * @augments Toolbar
		 *
		 * @param {WebCenterToolbarModel} toolbarModel
		 * @param {JQueryObj} context
		 * @param {JQueryObj} globalContext - used to create buttons in
		 */
		var WebCenterToolbar = function (toolbarModel, context, globalContext) {
			/**
			 * @member {ToolbarComponent} the toolbar compoment where all the button groupd with buttons are added to (left)
			 * @private
			 */
			this._leftToolBar = new ToolbarComponent("WFEditorToolbarLeft");

			/**
			 * @member {ToolbarComponent} the toolbar compoment where all the button groupd with buttons are added to (right)
			 * @private
			 */
			this._rightToolBar = new ToolbarComponent("WFEditorToolbarRight");

			/**
			 * @member {JQueryObj}
			 * @private
			 * The global context to which the toolbar button tooltips will be added
			 */
			this._globalContext = globalContext;

			/**
			 * @member {Array.<String>} order of toolbar groups that are added to the left toolbar
			 * @private
			 */
			this._leftOrder = [ToolConstants.ButtonGroups.ZOOM,
				ToolConstants.ButtonGroups.STEPTHROUGH,
				ToolConstants.ButtonGroups.NAVIGATION,
				WebCenterWFEditorConstants.WorkflowControlToolbar,
				ToolConstants.ButtonGroups.UNDOREDO,
				WebCenterWFEditorConstants.WorkflowStatusToolbar,
				ToolConstants.ButtonGroups.REMOVENODES
			];

			/**
			 * @member {Array.<String>} order of toolbar groups that are added to the right toolbar
			 * @private
			 */
			this._rightOrder = [
				WebCenterWFEditorConstants.TaskNameToolbarComponent,
				WebCenterWFEditorConstants.WorkflowActionsToolbar,
				WebCenterWFEditorConstants.WorkflowWFModesToolbar
			];



			// call the constructor of Toolbar
			Toolbar.call(this, toolbarModel, context);


			//append the toolbar to the toolbar context
			this._context.append(this._leftToolBar.element);
			this._context.append(this._rightToolBar.element.addClass("toolbarRight"));

		};

		WebCenterToolbar.prototype = Object.create(Toolbar.prototype);
		WebCenterToolbar.prototype.constructor = WebCenterToolbar;

		/**
		 * Add a toolbar component to the toolbar.
		 * @param {String} componentId -  id of the component
		 */
		WebCenterToolbar.prototype._addComponent = function (componentId) {
			if (this._toolbarComponents[componentId] === undefined) {

				var right = true;
				var order = _.indexOf(this._leftOrder, componentId);
				if (order < 0) {
					order = _.indexOf(this._rightOrder, componentId);
				} else {
					right = false;
				}
				var componentData;
				var component;
				if (componentId === WebCenterWFEditorConstants.WorkflowStatusToolbar) {
					componentData = this._model.getComponent(componentId);
					var statusData = componentData.data;
					if (statusData) {
						component = new StatusToolbarComponent(statusData.icon, statusData.status, this._globalContext);
						this._toolbarComponents[componentId] = component;
					}

				} else if (componentId === WebCenterWFEditorConstants.TaskNameToolbarComponent) {
					var element = $("<div/>").addClass("taskNameToolbarComponent enableTextSelection").append($("<span/>").text(this._model.getComponent(componentId).data));
					component = {
						element: element
					};
					
					this._toolbarComponents[componentId] = component;

				} else {
					componentData = this._model.getComponent(componentId);
					component = new ButtonGroup(componentId);
					if (componentData && componentData.title) { //only add title if it is defined in the component data
						component.setTitle(i18n, undefined, componentData.title);
					}

					this._toolbarComponents[componentId] = component;
				}
				if (right) {
					this._rightToolBar.addObject(component, order);
				} else {
					this._leftToolBar.addObject(component, order);
				}

			}
		};

		/**
		 * Add a button to the given toolbar component, the component is added if it doesn't exist yet.
		 * @param {String} componentId
		 * @param {String} buttonId
		 */
		WebCenterToolbar.prototype._addButton = function (componentId, buttonId) {
			if (this._toolbarButtons[buttonId] === undefined) {
				if (this._toolbarComponents[componentId] === undefined) {
					this._addComponent(componentId);
				}
				var buttonData = this._model.getButtonData(buttonId);
				if (buttonData) {
					var icon = buttonData.icon;
					if (icon && !_.isString(buttonData.icon) && buttonData.icon.icon && buttonData.icon.name) {
						icon = $("<div>").text(buttonData.icon.name);
					}
					if (buttonData.content) {
						// custom content for the button
						icon = buttonData.content;
					}
					var button = new ToolbarButton(i18n, this._globalContext, false, buttonData.id, icon, false, buttonData.action, buttonData.tooltipText);

					// Prevent the workflow editor to lose focus on mousedown
					button.element.on("mousedown", function (e) {
						e.preventDefault();
					});

					if (buttonData.cssClass) {
						button.addClass(buttonData.cssClass);
					}
					this._toolbarButtons[buttonId] = button;
					button.setEnabled(buttonData.enabled);
					this._toolbarComponents[componentId].addButton(button);

				}
			}
		};

		/**
		 * Update the button with given id.
		 * @param {String} buttonId
		 */
		WebCenterToolbar.prototype._updateButton = function (buttonId) {
			if (this._toolbarButtons[buttonId]) {
				var buttonModel = this._model.getButtonData(buttonId);
				this._toolbarButtons[buttonId].setActive(buttonModel.active);
				this._toolbarButtons[buttonId].setEnabled(buttonModel.enabled);
				this._toolbarButtons[buttonId].setTooltip(buttonModel.tooltipText);
				if (!buttonModel.content) {
					this._toolbarButtons[buttonId].setIcon(buttonModel.icon + " icon-normal");
				}
			}
		};

		/**
		 * Update a component of the toolbar
		 * @param {String} componentId
		 */
		WebCenterToolbar.prototype._updateComponent = function (componentId) {
			if (this._toolbarComponents[componentId]) {
				var componentData = this._model.getComponent(componentId);
				var data = componentData.data;
				if (data && this._toolbarComponents[componentId].updateView) {
					this._toolbarComponents[componentId].updateView(data);
				}

			}
		};

		return WebCenterToolbar;
	});

define('WFEditorWCR/scene/WebCenterScene',[
		"WFEditor/scene/Scene",
		"utils/ShapeUtils",
		"WFEditorWCR/core/WebCenterWFEditorConstants"
	],
	function (
		Scene,
		ShapeUtils,
		WebCenterWFEditorConstants
	) {
		"use strict";

		/**
		 * The WebCenter Scene is responsible for the layers and registers listeners on the model.
		 * It uses the viewFactory to create the correct view components and adds these to
		 * the correct layer. Specific webcenter functionality is added here to the general Scene.
		 *
		 * @exports WebCenterScene
		 * @class
		 *
		 * @param {JQueryObj}           context       - the context in which the workflow scene is drawn
		 * @param {SceneModel}	        sceneModel	  - the model of the scene
		 * @param {Workflow}            workflowModel - model of the workflow which contains all the workflow items
		 * @param {AbstractViewFactory} viewFactory   - factory used to generate all the view objects
		 * @param {WebCenterAPI} 		webCenterAPI 
		 * @param {WFEditorContext} 	WFEditorContext 
		 */
		var WebCenterScene = function (context, sceneModel, workflowModel, viewFactory, webCenterAPI, WFEditorContext) {
			Scene.call(this, context, sceneModel, workflowModel, viewFactory, WFEditorContext);

			this._sceneModel.setWorldBoundingBox(new ShapeUtils.Rect(
				new ShapeUtils.Point(WebCenterWFEditorConstants.WorkflowWorldBounds.x, WebCenterWFEditorConstants.WorkflowWorldBounds.y),
				new ShapeUtils.Size(WebCenterWFEditorConstants.WorkflowWorldBounds.width, WebCenterWFEditorConstants.WorkflowWorldBounds.height)));

			this._webCenterAPI = webCenterAPI;
			// set the insets correctly, we want 5px padding + scrollbars shouldn't overlap canvas content
			this._sceneModel.setInsets(55, this._verticalScroll.element.width() + 5, this._horizontalScroll.element.height() + 5, 5);

		};

		WebCenterScene.prototype = Object.create(Scene.prototype);
		WebCenterScene.prototype.constructor = WebCenterScene;

		/**
		 * @override
		 * A token got selected in the view, emit an event where the controller can listen to.
		 *
		 * @param {Token}  token      - the token that got selected
		 * @param {String} versionId - the id of the document version in the token that got selected
		 */
		WebCenterScene.prototype.tokenItemSelected = function (token, versionId) {
			this._emit("tokenItemSelected", token, versionId);
		};


		/**
		 * Get a reference to the WebCenter API
		 * @returns {WebCenterAPI} 
		 */
		WebCenterScene.prototype.getWebCenterAPI = function () {
			return this._webCenterAPI;
		};

		return WebCenterScene;

	});


define('tpl!WFEditorWCR/actionPopup/ImportWorkflowPopupContent', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<form enctype="MULTIPART/FORM-DATA" action="UploadWorkflowXML.jsp" method="post" name="wfimportform">\r\n	<input name="redirect" type="hidden" value="wctasktypedetailsworkflow.jsp?taskTypeID='+
((__t=(d.taskTypeID))==null?'':_.escape(__t))+
'&menu_file=wctasktypedetailsworkflow"></input>\r\n	<input name="isimport" type="hidden" value="1"></input>\r\n	<input name="tasktypeid" type="hidden" value="'+
((__t=(d.taskTypeID))==null?'':_.escape(__t))+
'"></input>\r\n	<input class="datafile" name="datafile" type="file"></input>\r\n</form>';
return __p;
}});
define('WFEditorWCR/actionPopup/ImportWorkflowPopup',["component/actionpopup/ImportPopup",
		"utils/webi18n!WFEditorWCR",
		"Template",
		"tpl!WFEditorWCR/actionPopup/ImportWorkflowPopupContent",
		"underscore",
		"jquery",

	],
	function (
		ImportPopup,
		i18n,
		Template,
		tpl,
		_,
		$) {

		"use strict";
	
		var languageStrings = {
				ImportConfirmText : i18n.t("workflowImportConfirm"), 
				ImportConfirmTooltip: i18n.t("workflowImportTooltip"),
				ImportConfirmCancel: i18n.t("configurationCancel"),
				ImportConfirmOk : i18n.t("configurationOK"),
				ImportTooltip: i18n.t("workflowImportTooltip"),
				ImportCancel: i18n.t("configurationCancel"),
				ImportOk: i18n.t("configurationOK")
		};

		/**
		 * Popup used to import a workflow
		 * @exports ImportWorkflowPopup
		 * @class
		 */
		var ImportWorkflowPopup = function (wfEditorWCR) {
			this._wfEditorWCR = wfEditorWCR;

			this._importPopup = new ImportPopup(languageStrings,new Template(i18n, tpl));
			
			//Make sure the workflow is imported when confirm is clicked
			this._importPopup.importConfirm = function(){
				wfEditorWCR.doImportWorkflow();
			};
		};

		/**
		 * Open the popup
		 */
		ImportWorkflowPopup.prototype.open = function () {
			this._importPopup.template.d.taskTypeID = this._wfEditorWCR.getTaskTypeID();
			this._importPopup.open();
		};

		return ImportWorkflowPopup;
	});

define('WFEditorWCR/actionPopup/ExportWorkflowPopup',["component/modal/Modal",
		"component/button/Button",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery",

	],
	function (
		Modal,
		Button,
		i18n,
		_,
		$) {

		"use strict";

		/**
		 * Popup used to export a workflow
		 * @exports ExportWorkflowPopup
		 * @class
		 */
		var ExportWorkflowPopup = function (wfEditorWCR) {
			this._wfEditorWCR = wfEditorWCR;

			// Popup content
			this._content = $("<div>").text(i18n.t("workflowExportSaveFirst"));
			this._exportWarningPopup = new Modal(this._content);
			this._exportWarningPopup.setTitle(i18n.t("workflowExportTooltip"));
			this._exportWarningPopup.onClose(_.bind(function () {
				this.destroy()
			}, this._exportWarningPopup));

			this._cancelButton = new Button(null, null, false, "Cancel", i18n.t("configurationCancel"), true, _.bind(function () {
				this._exportWarningPopup.close();
			}, this));
			this._cancelButton.addClass("btn btn-default btn-md");

			this._exportWithSaveButton = new Button(null, null, false, "OK", i18n.t("configurationOK"), true, _.partial(function (popup) {
				this.setEnabled(false);
				// Wait until save is done
				var savePromise = wfEditorWCR.doSaveCurrentWorkflow();
				savePromise.done(_.bind(function () {
					popup._wfEditorWCR.doExportWorkflow();
					this.setEnabled(true);
				}, this)).always(function () {
					// errors are already shown
					popup._exportWarningPopup.close();
				});
			}, this));
			this._exportWithSaveButton.addClass("btn btn-primary btn-md");

			this._buttonGroup = $("<div>").addClass("btn-group pull-right");
			this._buttonGroup.append(this._cancelButton.element).append(this._exportWithSaveButton.element);
			this._exportWarningPopup.setFooter(this._buttonGroup);
		};

		/**
		 * Open the popup
		 */
		ExportWorkflowPopup.prototype.open = function () {
			this._exportWarningPopup.open();
		};

		return ExportWorkflowPopup;
	});

define('WFEditorWCR/actionPopup/ResetToManualPopup',["component/modal/Modal",
		"component/button/Button",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery",

	],
	function (
		Modal,
		Button,
		i18n,
		_,
		$) {

		"use strict";

		/**
		 * Popup used to confirm resetting a workflow to a manual task
		 * @exports ResetToManualPopup
		 * @class
		 */
		var ResetToManualPopup = function (wfEditorWCR) {
			this._wfEditorWCR = wfEditorWCR;

			// Popup content
			this._content = $("<div />").text(i18n.t("workflowResetToManualConfirm"));
			this._confirmPopup = new Modal(this._content);
			this._confirmPopup.setTitle(i18n.t("workflowResetToManualTooltip"));

			this._confirmPopup.onClose(_.bind(function () {
				this.destroy();
			}, this._confirmPopup));

			this._yesButton = new Button(null, null, false, "Yes", i18n.t("yes"), true, _.partial(function (popup) {
				this.setEnabled(false);
				$.when(popup._wfEditorWCR.doResetToManualTask()).always(function () {
					// errors already logged
					popup._confirmPopup.close();
				});
			}, this));
			this._yesButton.addClass("btn btn-default btn-md");

			this._noButton = new Button(null, null, false, "No", i18n.t("no"), true, _.partial(function (popup) {
				popup._wfEditorWCR.doSaveCurrentWorkflow();
				popup._confirmPopup.close();
			}, this));
			this._noButton.addClass("btn btn-primary btn-md");

			this._buttonGroup = $("<div>").addClass("btn-group pull-right");
			this._buttonGroup.append(this._yesButton.element).append(this._noButton.element);
			this._confirmPopup.setFooter(this._buttonGroup);
		};

		/**
		 * Open the popup
		 */
		ResetToManualPopup.prototype.open = function () {
			this._confirmPopup.open();
			this._yesButton.setEnabled(true);
		};

		return ResetToManualPopup;
	});

define('WFEditorWCR/actionPopup/CancelWorkflowPopup',["component/modal/Modal",
		"component/button/Button",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery",

	],
	function (
		Modal,
		Button,
		i18n,
		_,
		$) {

		"use strict";

		/**
		 * Popup used to cancel a workflow
		 * @exports CancelWorkflowPopup
		 * @class
		 */
		var CancelWorkflowPopup = function (wfEditorWCR) {
			// Popup content
			this._content = $("<div />").text(i18n.t("wfedit.state.cancelmessage"));
			this._cancelConfirmPopup = new Modal(this._content);
			this._cancelConfirmPopup.setTitle(i18n.t("wfedit.state.canceltitle"));

			this._cancelConfirmPopup.onClose(_.bind(function(){
				this.destroy();
			}, this._cancelConfirmPopup));
			
			this._cancelButton = new Button(null, null, false, "Cancel", i18n.t("configurationCancel"), true, _.bind(function () {
				this._cancelConfirmPopup.close();
			}, this));
			this._cancelButton.addClass("btn btn-default btn-md");

			this._okButton = new Button(null, null, false, "OK", i18n.t("configurationOK"), true, _.bind(function () {
				this._cancelConfirmPopup.close();
				wfEditorWCR.doCancelWorkflow();
			}, this));
			this._okButton.addClass("btn btn-primary btn-md");

			this._buttonGroup = $("<div>").addClass("btn-group pull-right");
			this._buttonGroup.append(this._cancelButton.element).append(this._okButton.element);
			this._cancelConfirmPopup.setFooter(this._buttonGroup);
		};


		/**
		 * Open the popup
		 */
		CancelWorkflowPopup.prototype.open = function () {
			this._cancelConfirmPopup.open();
			this._okButton.setEnabled(true);
		};

		return CancelWorkflowPopup;
	});

define('WFEditorWCR/core/WFEditorWCR',["WFEditor/core/WFEditor",
		"WFEditor/workflow/ModelConstants",
		"WFEditorWCR/data/WebCenterAPI",
		"WFEditorWCR/core/WebCenterWFEditorConstants",
		"WFEditorWCR/workflow/WebCenterWorkflowState",
		"WFEditorWCR/workflow/WebCenterWorkflowFactory",
		"WFEditorWCR/scene/WebCenterViewFactory",
		"WFEditorWCR/nodeGallery/WebCenterNodeGallery",
		"WFEditorWCR/scene/WebCenterSceneController",
		"WFEditorWCR/toolbar/WebCenterToolbar",
		"WFEditorWCR/scene/WebCenterScene",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/actionPopup/ImportWorkflowPopup",
		"WFEditorWCR/actionPopup/ExportWorkflowPopup",
		"WFEditorWCR/actionPopup/ResetToManualPopup",
		"WFEditorWCR/actionPopup/CancelWorkflowPopup",
		"component/errorhandler/ErrorHandler",
		"component/loadingindicator/LoadingIndicatorSingleton",
		"component/button/Button",
		"utils/webi18n!WFEditorWCR",
		"utils/webcss!./wfeditorwcr-style.css",
		"jquery",
		"underscore"
	],
	function (WFEditor,
		ModelConstants,
		WebCenterAPI,
		WebCenterWFEditorConstants,
		WebCenterWorkflowState,
		WebCenterWorkflowFactory,
		WebCenterViewFactory,
		WebCenterNodeGallery,
		WebCenterSceneController,
		WebCenterToolbar,
		WebCenterScene,
		WebCenterNodeConstants,
		ImportWorkflowPopup,
		ExportWorkflowPopup,
		ResetToManualPopup,
		CancelWorkflowPopup,
		ErrorHandler,
		LoadingIndicator,
		Button,
		i18n,
		css,
		$,
		_
	) {
		"use strict";

		/**
		 * WFEditorWCR is an instance of the WebCenter HTML5 Workflow editor. To launch an instance of
		 * the WFEditor, use the constructor or use the data-init attribute in the context HTML-element
		 * on the page. WFEditor will initialize itself in this context. It needs the attribute tasktypeid
		 * set in the context-element to be able to start.
		 *
		 * @exports WFEditorWCR
		 * @class
		 * @augments {WFEditor}
		 *
		 * @param {HTMLObject} context    a reference to the div in which the workflow editor should be created
		 * @param {Map<String, Any>} privileges - The privileges for this workflow editor
		 * @param {String}     taskId - The id of the task to load in the Workflow Editor
		 * @param {String}     taskTypeId - The id of the task type to load in the Workflow Editor
		 * @param {String} [initialTaskStatus] - Initial Task Status ID of the workflow (needed when no report can be fetched)
		 * @param {String} [taskname] - Task name
		 */
		var WFEditorWCR = function (context, privileges, taskId, taskTypeId, initialTaskStatus, taskName) {
			/**
			 * @member {Map<String, Any>} privileges for the workflow
			 * @private
			 */
			this._privileges = privileges;

			/**
			 * @member {String} id of the task or task type to be loaded.
			 * @private
			 */
			this._taskId = taskId;

			/**
			 * @member {String} id of the task type to be loaded.
			 * @private
			 */
			this._taskTypeId = taskTypeId;

			/**
			 * @member {String | undefined} name of the task to be loaded.
			 * @private
			 */
			this._taskName = taskName;

			/**
			 * @member {String} id of the task status at the moment the WFEditor was launched
			 * @private
			 */
			this._initialTaskStatus = initialTaskStatus;

			/**
			 * @member {$.Deferred} xml of the definition of workflows in WebCenter
			 * @private
			 */
			this._workflowDefinitionPromise = null;

			/**
			 * @member {$.Deferred} xml of the definition of task types in WebCenter
			 * @private
			 */
			this._tasktypeDefinitionPromise = null;

			/**
			 * @member {ErrorHandler} error handler used in the wf
			 * @private
			 */
			var errorHandler = new ErrorHandler();

			/**
			 * @member {Boolean} is a save action already ongoing?
			 * @private
			 */
			this._saveOngoing = false;

			/**
			 * @member {String} config object with information of webcenter, loaction of the images, jsppages,...
			 * @private
			 */
			this._wcrConfig = window.webcenterConfig;

			// Add client browser information to wcrConfig
			this._wcrConfig.userAgent = typeof window.navigator !== "undefined" && typeof window.navigator.userAgent !== "undefined" ? window.navigator.userAgent : "";

			/**
			 * @member {WebCenterAPI} communication with WebCenter
			 */
			this._webCenterAPI = new WebCenterAPI(this._wcrConfig, errorHandler);

			//we create an instance of the WebCenterWorkflowFactory
			var workflowFactory = new WebCenterWorkflowFactory(this._webCenterAPI, errorHandler);

			//create an instance of the view factory
			var workflowViewFactory = new WebCenterViewFactory(errorHandler);

			var nodeGallery = null;
			if (this._privileges[ModelConstants.Privileges.addNodes]) {
				nodeGallery = new WebCenterNodeGallery();
			}

			// Empty the context
			$(context).empty();

			// we call the constructor of the framework worflow editor
			WFEditor.call(this, context, workflowFactory, workflowViewFactory, nodeGallery, i18n, errorHandler);

			// the WFEditor core is not initalized start loading the workflow (and feedback if available)
			this._loadAndInitalizeWorkflow();


			//>>excludeStart("PExclude", pragmas.productionExclude);
			// Make Development a bit more enjoyable... export WFEditorin non-optimized mode
			window.gWFEditor = this;
			//>>excludeEnd("PExclude");

			this._boundOnBeforeUnload = _.bind(this._onBeforeUnload, this);
			$(window).on("beforeunload", this._boundOnBeforeUnload);

			this._webCenterAPI.keepSessionAlive(true);
		};

		WFEditorWCR.prototype = Object.create(WFEditor.prototype);
		WFEditorWCR.prototype.constructor = WFEditorWCR;

		/**
		 * Initialize the the WFEditor with the data-init method of initialize static components
		 * @param {HTMLObject} context
		 */
		WFEditorWCR.wcrinit = function (context) {
			var tasktypeid = context.getAttribute("tasktypeid");
			var taskid = context.getAttribute("taskid");
			var initialTaskStatus = context.getAttribute("initialTaskStatus");
			var taskname = context.getAttribute("taskname");
			var privileges = {
				hasTaskEditRights: context.getAttribute("canedit") == "1" || context.getAttribute("canedit") == "true", // change task permissions in feedback mode or can edit in editor mode
				isMasterTask: context.getAttribute("mastertaskid") ? context.getAttribute("mastertaskid").length === 0 : true, // is a mastertask or not
				autorefresh: context.getAttribute("autorefresh") == "1" // should feedback be autorefreshed
			};

			var editableMode = !taskid && privileges.hasTaskEditRights; // editor mode if taskid is not defined and editable if canedit = 1

			privileges[ModelConstants.Privileges.zoomToActiveNode] = !!taskid; // only in feedback mode
			privileges[ModelConstants.Privileges.addEdges] = editableMode;
			privileges[ModelConstants.Privileges.removeEdges] = editableMode;
			privileges[ModelConstants.Privileges.moveEdges] = editableMode;
			privileges[ModelConstants.Privileges.canSave] = editableMode;
			privileges[ModelConstants.Privileges.addNodes] = editableMode;
			privileges[ModelConstants.Privileges.moveNodes] = true; // always allow nodes to be moved
			privileges[ModelConstants.Privileges.removeNodes] = editableMode;
			privileges[WebCenterNodeConstants.Privileges.editLabels] = editableMode;
			privileges[WebCenterNodeConstants.Privileges.editStickyNotes] = editableMode;
			privileges[WebCenterNodeConstants.Privileges.showWorkflowControls] = !!taskid && privileges.hasTaskEditRights;
			privileges[ModelConstants.Privileges.viewNodeConfiguration] = true;
			privileges[ModelConstants.Privileges.editNodeConfiguration] = editableMode;
			privileges[ModelConstants.Privileges.copyNodes] = editableMode;


			return new WFEditorWCR($(context), privileges, taskid, tasktypeid, initialTaskStatus, taskname);
		};

		/**
		 * Initializes the specific WebCenter controller for the workflow editor.
		 * @override
		 */
		WFEditorWCR.prototype._initializeSceneController = function (workflowFactory, sceneModel, workflowModel, scene, WFEditorContext) {
			return new WebCenterSceneController(workflowFactory, sceneModel, workflowModel, scene, WFEditorContext);
		};


		/**
		 * Fetch XML information from server and display the workflow
		 */
		WFEditorWCR.prototype._loadAndInitalizeWorkflow = function () {
			LoadingIndicator.start("loadAndInitializeWF", i18n.t("loadingWorkflowDefinition"));

			var workflowXMLPromise;

			if (this._taskId) {
				workflowXMLPromise = this._webCenterAPI.getTaskWorkflowXML(this._taskId);
			} else {
				workflowXMLPromise = this._webCenterAPI.getTaskTypeWorkflowXML(this._taskTypeId);
			}

			var workflowDefinitionPromise = this._webCenterAPI.getWorkflowDefinitions();
			var taskTypeDefinitionPromise = this._webCenterAPI.getTaskTypeDefinitions();
			var currentTaskTypeDefinition = this._webCenterAPI.getTaskTypeDefinition(this._taskTypeId);
			var workflowReportPromise = this._webCenterAPI.getWorkflowReport(this._taskId);

			// fetch WorkflowXML, WorkflowDefinitions and TaskTypeDefinitions in parallel
			$.when(workflowXMLPromise, workflowDefinitionPromise, taskTypeDefinitionPromise, currentTaskTypeDefinition)
				.done(_.bind(function (workflowXML, workflowDefinitionXML, tasktypeDefinitionXML, currentTaskTypeDefinitionXML) {
					var workflowPromise = this._workflowFactory.createWorkflow(this._taskTypeId,
						this._taskId,
						this._privileges,
						workflowXML[0],
						workflowDefinitionXML[0],
						tasktypeDefinitionXML[0],
						currentTaskTypeDefinitionXML[0],
						this._initialTaskStatus);
					workflowPromise.done(_.bind(function (workflow) {


						//>>excludeStart("PExclude", pragmas.productionExclude);
						// Make Development a bit more enjoyable... export workflow to window in non-optimized mode
						window.gWorkflow = workflow;
						//>>excludeEnd("PExclude");

						this.setWorkflow(workflow, _.bind(function () {
							// if there is a workflowReport, wait till it is loaded and start the feedbacktimer if needed
							if (workflowReportPromise) {
								$.when(workflowReportPromise, currentTaskTypeDefinition).done(_.bind(function (report, currentTaskDefinition) {
									this._workflowFactory.createWorkflowFeedback(report[0], currentTaskDefinition[0], workflow);

									// start feedback fetcher if needed
									this._startFeedbackTimer();
									//update the toolbar model with the created workflow feedback
									this._updateToolbarModel();
									// highlight active nodes (will only be visible in highlight mode)
									this._sceneController.resetHighlights();
								}, this));
							}
							this._wfEditorContext.get("keyboardHandler").bind("d e b u g enter", function () {
								workflow.toggleDebugMode();
							}, "keyup");

							LoadingIndicator.stop("loadAndInitializeWF");
						}, this));
					}, this));
				}, this));
		};

		/**
		 * If autorefresh is turned on, feedback should be fetched and added to the model at regular moments
		 * This method will start the feedbacktimer is autorefresh privilege is turned on
		 */
		WFEditorWCR.prototype._startFeedbackTimer = function () {
			if (!this._interval) {
				// Feedback should auto refresh, start feedback timer
				this._interval = setInterval(_.bind(function () {
					if (this._workflow && this._workflow.getPrivilege("autorefresh")) {
						this._reloadWorkflowFeedback();
					} else {
						clearInterval(this._interval);
						this._interval = null;
					}
				}, this), WebCenterWFEditorConstants.FeedbackUpdateInterval);
			}
		};

		/**
		 * Reload the feedback of the workflow and update the workflow model.
		 * @private
		 */
		WFEditorWCR.prototype._reloadWorkflowFeedback = function () {
			if (this._workflow) {
				var workflowReportPromise = this._webCenterAPI.getWorkflowReport(this._workflow.getTaskID());
				var currentTaskTypeDefinition = this._webCenterAPI.getTaskTypeDefinition(this._workflow.getTaskTypeID());

				$.when(workflowReportPromise, currentTaskTypeDefinition).done(_.bind(function (report, currentTaskDefinition) {
					// update feedback
					this._workflowFactory.createWorkflowFeedback(report[0], currentTaskDefinition[0], this._workflow);
					//update the toolbarmodel with the updated workflow feedback
					this._updateToolbarModel();
					// highlight active nodes (will only be visible in highlight mode)
					this._sceneController.resetHighlights();
				}, this));
			}
		};

		/**
		 * Reloads the feedback each interval ms until callback returns true
		 * @param {Number} interval   the interval between feedback calls
		 * @param {Function} callback The function to call to see if the reload of the workflow should stop
		 */
		WFEditorWCR.prototype._reloadWorkflowFeedbackUntil = function (interval, callback) {
			this._reloadWorkflowFeedback();
			var intervalId = setInterval(_.bind(function () {
				if (!callback()) {
					this._reloadWorkflowFeedback();
				} else {
					clearInterval(intervalId);
				}
			}, this), interval);
		};



		/**
		 * Request a new workflow to be loaded
		 * @param {String} id
		 */
		WFEditorWCR.prototype.loadNewWorkflow = function (id) {
			var currentUrl = document.location.href;
			var split = currentUrl.split("?");
			var mainUrl = split[0];
			var params = split[1];
			var newParams = [];
			_.each(params.split("&"), function (param) {
				var keyValue = param.split("=");
				if (keyValue[0] === "curProjTaskID") {
					newParams.push("curProjTaskID=" + id);
				} else {
					newParams.push(param);
				}
			}, this);

			document.location.href = mainUrl + "?" + newParams.join("&");
		};

		/**
		 * Initialize the toolbar for the workflow editor. Specific WebCenter implementation.
		 * @param   {ToolbarModel} toolbarModel
		 * @param   {JQueryObj} context
		 * @returns {WebCenterToolbar}
		 */
		WFEditorWCR.prototype._initializeToolbar = function (toolbarModel, context, globalContext) {
			var toolbar = new WebCenterToolbar(toolbarModel, context, globalContext);

			if (this._workflow && this._workflow.getPrivilege(WebCenterNodeConstants.Privileges.showWorkflowControls)) {
				toolbarModel.addComponent(WebCenterWFEditorConstants.WorkflowControlToolbar, "workflowStateActions");
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowControlToolbar, WebCenterWFEditorConstants.WorkflowControlActionButtons.HOLD, "icon-WCR_WF-Hold2", _.bind(this._holdWorkflow, this), i18n.t("HoldWorkflowButtonTooltipText"));
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowControlToolbar, WebCenterWFEditorConstants.WorkflowControlActionButtons.RESUME, "icon-WCR_WF-Resume2", _.bind(this._resumeWorkflow, this), i18n.t("ResumeWorkflowButtonTooltipText"));
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowControlToolbar, WebCenterWFEditorConstants.WorkflowControlActionButtons.CANCEL, "icon-WCR_WF-Cancel2", _.bind(this._cancelWorkflow, this), i18n.t("CancelWorkflowButtonTooltipText"));
			}

			toolbarModel.addComponent(WebCenterWFEditorConstants.WorkflowWFModesToolbar, "viewTitle");

			// add save button to the model
			if (this._workflow && this._workflow.getPrivilege(ModelConstants.Privileges.canSave)) {

				// Bind ctrl+s | command+s keys to saving the workflow
				this._wfEditorContext.get("keyboardHandler").bind("mod+s", _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					if (this._undoRedo.isWorkflowModified() && !this._saveOngoing) {
						this._saveCurrentWorkflow();
					}
				}, this));

				toolbarModel.addComponent(WebCenterWFEditorConstants.WorkflowActionsToolbar, "workflowTitle");
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowActionsToolbar, WebCenterWFEditorConstants.WorkflowSaveToolbarButton, "icon-WCR_Save", _.bind(this._saveCurrentWorkflow, this), i18n.t("workflowSaveTooltip"));
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowActionsToolbar, WebCenterWFEditorConstants.WorkflowImportToolbarButton, "icon-WCR_WF-Import", _.bind(this._importWorkflow, this), i18n.t("workflowImportTooltip"));
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowActionsToolbar, WebCenterWFEditorConstants.WorkflowExportToolbarButton, "icon-WCR_WF-Export", _.bind(this._exportWorkflow, this), i18n.t("workflowExportTooltip"));
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowActionsToolbar, WebCenterWFEditorConstants.WorkflowClearToolbarButton, "icon-WCR_WF-Clear", _.bind(this._clearWorkflow, this), i18n.t("workflowClearTooltip"));

				this._undoRedo.setWorkflowModifiedCallback(_.bind(this._updateSaveButton, this));
			} else if (this._workflow && this._workflow.getPrivilege(ModelConstants.Privileges.zoomToActiveNode)) {
				this._highlightIcon = $("<i/>").addClass("icon containerIcon icon-normal icon-grey icon-WCR_WF-Show-Active-Node-cont").append(
					$("<i/>").addClass("icon subIcon icon-normal icon-default icon-WCR_WF-Show-Active-Node-sub"));
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowWFModesToolbar, WebCenterWFEditorConstants.WorkflowWFModesToolbar, "", _.bind(this._toggleHighlightNodes, this), i18n.t("workflowToggleHighlightTooltipOn"), {
					content: this._highlightIcon
				});
			}
			if (this._taskName) {
				toolbarModel.addComponent(WebCenterWFEditorConstants.TaskNameToolbarComponent, "taskName", this._taskName);
			}
			toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowWFModesToolbar, WebCenterWFEditorConstants.WorkflowFullScreenToolbarButton, !this._isInFullScreen ? "icon-WCR_FullScreen" : "icon-WCR_NormalScreen", _.bind(this._toggleFullScreen, this), !this._isInFullScreen ? i18n.t("workflowToggleFullScreenTooltipOn") : i18n.t("workflowToggleFullScreenTooltipOff"));


			this._updateSaveButton();

			return toolbar;
		};

		/**
		 * Update the disabled state of the save button
		 * the button is disabled while a save is ongoing, or the workflow is not modified since the last save
		 */
		WFEditorWCR.prototype._updateSaveButton = function () {
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowSaveToolbarButton, this._undoRedo.isWorkflowModified() && !this._webCenterAPI.isSaveOngoing());
		};



		/**
		 * Update the toolbarmodel.
		 */
		WFEditorWCR.prototype._updateToolbarModel = function () {
			var workflowState = this._workflow.getWorkflowState();
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowControlActionButtons.HOLD, workflowState !== WebCenterWorkflowState.STOPPING && workflowState === WebCenterWorkflowState.RUNNING);
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowControlActionButtons.RESUME, workflowState !== WebCenterWorkflowState.STOPPING && workflowState === WebCenterWorkflowState.PAUSED);
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowControlActionButtons.CANCEL, workflowState !== WebCenterWorkflowState.STOPPING && (workflowState === WebCenterWorkflowState.RUNNING || workflowState === WebCenterWorkflowState.PAUSED));

			var workflowStatus = this._workflow.getTaskStatus();
			if (workflowStatus) {
				if (!this._toolbarModel.hasComponent(WebCenterWFEditorConstants.WorkflowStatusToolbar)) {
					this._toolbarModel.addComponent(WebCenterWFEditorConstants.WorkflowStatusToolbar, "workflowStateActions", workflowStatus);
				} else {
					this._toolbarModel.updateComponent(WebCenterWFEditorConstants.WorkflowStatusToolbar, workflowStatus);
				}

			}
		};

		/**
		 * @override
		 */
		WFEditorWCR.prototype._initializeScene = function (scene, sceneModel, workflow, workflowViewFactory, wfEditorContext) {
			return new WebCenterScene(scene, sceneModel, workflow, workflowViewFactory, this._webCenterAPI, wfEditorContext);
		};

		/**
		 * Save the current workflow
		 */
		WFEditorWCR.prototype._saveCurrentWorkflow = function () {
			// Check if this is a "default workflow"
			if (_.size(this._workflow.getEdges()) === 0) {
				// no edges, so check nodes
				var startNode = null;
				var completedNode = null;
				var errorContinueNode = null;
				if (_.every(this._workflow.getNodes(), function (node) {
						if (node.getNodeType() === WebCenterNodeConstants.Nodes.START) {
							if (startNode === null) {
								// Only 1 start node
								startNode = node;
								return true;
							} else {
								return false;
							}
						} else if (node.getNodeType() === WebCenterNodeConstants.Nodes.COMPLETED) {
							if (completedNode === null) {
								// Only 1 completed node
								completedNode = node;
								return true;
							} else {
								return false;
							}
						} else if (node.getNodeType() == WebCenterNodeConstants.Nodes.ERRORCONTINUE) {
							if (errorContinueNode === null) {
								// Only 1 error continue node
								errorContinueNode = node;
								return true;
							} else {
								return false;
							}
						}
					})) {
					// we have a case of default workflow
					new ResetToManualPopup(this).open();
					return;
				}
			}

			// Normal save
			this.doSaveCurrentWorkflow();
		};

		/**
		 * Do save the current workflow
		 * @return {$.Deferred.<>} Promise resolved when save is finished.
		 */
		WFEditorWCR.prototype.doSaveCurrentWorkflow = function () {
			var deferred = $.Deferred();
			this._webCenterAPI.saveWorkflow(this._workflow).done(_.bind(function () {
				// workflow saved succesfully
				this._messageController.sendSuccessMessage(this._i18n.t("workflowSaveSuccessful"));
				this._undoRedo.addSaveMarker();
			}, this)).always(_.bind(function () {
				// save is finsihed or might have failed
				this._updateSaveButton();
				deferred.resolve();
			}, this));

			this._updateSaveButton();
			return deferred.promise();
		};

		/**
		 * Hold the workflow, this pauses the workflow. Only possible if the workflow is running.
		 */
		WFEditorWCR.prototype._holdWorkflow = function () {
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowControlActionButtons.HOLD, false); //first disable the hold button to avoid double clicks

			this._webCenterAPI.holdWorkflow(this._workflow)
				.done(_.bind(function () {
					this._messageController.sendSuccessMessage(i18n.t("holdWorkflowSuccess"));
				}, this))
				.always(_.bind(function () {
					this._reloadWorkflowFeedback();
				}, this));
		};

		/**
		 * Resume the workflow, only possible if the workflow is paused
		 */
		WFEditorWCR.prototype._resumeWorkflow = function () {
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowControlActionButtons.RESUME, false);

			this._webCenterAPI.resumeWorkflow(this._workflow)
				.done(_.bind(function () {
					this._messageController.sendSuccessMessage(i18n.t("resumeWorkflowSuccess"));
				}, this))
				.always(_.bind(function () {
					this._reloadWorkflowFeedback();
				}, this));
		};

		/**
		 * Cancel the workflow, only possible if the workflow is paused or in progress.
		 */
		WFEditorWCR.prototype.doCancelWorkflow = function () {
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowControlActionButtons.CANCEL, false);

			this._updateToolbarModel();

			this._webCenterAPI.cancelWorkflow(this._workflow)
				.done(_.bind(function () {
					this._messageController.sendSuccessMessage(i18n.t("cancelWorkflowSuccess"));
				}, this))
				.always(_.bind(function () {
					this._reloadWorkflowFeedbackUntil(2000, _.bind(function () {
						return this._workflow.getWorkflowState() === WebCenterWorkflowState.FINISHED;
					}, this));
				}, this));
		};

		/**
		 * opens a popup to cancel the workflow
		 */
		WFEditorWCR.prototype._cancelWorkflow = function () {
			new CancelWorkflowPopup(this).open();
		};


		/**
		 * Import a new workflow
		 */
		WFEditorWCR.prototype._importWorkflow = function () {
			new ImportWorkflowPopup(this).open();
		};

		/**
		 * Do the import of the workflow
		 */
		WFEditorWCR.prototype.doImportWorkflow = function () {
			$(window).off("beforeunload", this._boundOnBeforeUnload); // we don't want to see this 
			document.wfimportform.submit();
		};

		/**
		 * Export the current workflow
		 */
		WFEditorWCR.prototype._exportWorkflow = function () {
			if (this._undoRedo.isWorkflowModified() && !this._saveOngoing) {
				new ExportWorkflowPopup(this).open();
			} else {
				this.doExportWorkflow();
			}
		};

		/**
		 * Do export workflow
		 */
		WFEditorWCR.prototype.doExportWorkflow = function () {
			// Check if there is a workflow first
			this._webCenterAPI.getTaskTypeWorkflowXML(this.getTaskTypeID()).done(_.bind(function (xml) {
				var root = xml.getElementsByTagName("root")[0];
				if (root && root.childNodes.length === 0) {
					// still a default workflow
					if (this._wfEditorContext.get("errorHandler")) {
						this._wfEditorContext.get("errorHandler").warning(i18n.t("workflowExportDefaultError"));
					}
				} else {
					// Download as file
					var url = "DownloadWorkflowXML.jsp?returnasfile=1&tasktypeid=" + this.getTaskTypeID();
					window.location.href = url;
				}
			}, this));
		};

		/**
		 * Clear the current workflow
		 */
		WFEditorWCR.prototype._clearWorkflow = function () {
			if (this._sceneController && this._workflow) {
				var startNode = null;
				var completedNode = null;
				var errorContinueNode = null;

				var nodesToDelete = _.filter(this._workflow.getNodes(), function (node) {
					var toDelete = true;
					if (node.getNodeType() === WebCenterNodeConstants.Nodes.START) {
						// Keep 1 start node
						if (startNode == null) {
							startNode = node;
							toDelete = false;
						}
					} else if (node.getNodeType() === WebCenterNodeConstants.Nodes.COMPLETED) {
						// Keep 1 completed node
						if (completedNode == null) {
							completedNode = node;
							toDelete = false;
						}
					} else if (node.getNodeType() === WebCenterNodeConstants.Nodes.ERRORCONTINUE) {
						// Keep 1 error continue node
						if (errorContinueNode == null) {
							errorContinueNode = node;
							toDelete = false;
						}
					}
					return toDelete;
				});

				// Remove them undoable
				if (_.size(nodesToDelete) > 0) {
					this._sceneController.removeNodes(this._workflow, nodesToDelete);
				}
			}
		};

		/**
		 * Resets a workflow back to a manual task
		 * @returns {$.Deferred} The promise resolved when clear is done
		 */
		WFEditorWCR.prototype.doResetToManualTask = function () {
			$(window).off("beforeunload", this._boundOnBeforeUnload); // we don't want to see this
			var deferred = $.Deferred();
			$.when(this._webCenterAPI.clearWorkflow(this.getTaskTypeID())).done(_.bind(function () {
				var context = this._context;
				context.empty();
				WFEditorWCR.wcrinit(context[0]);
			}, this)).always(function () {
				deferred.resolve();
			});
			return deferred.promise();
		};

		/**
		 * Enter Full Screen Mode (handle resizing)
		 * @override
		 */
		WFEditorWCR.prototype._enterFullScreen = function () {
			// make sure wcr menu doesn't pop pup over the fullscreen container
			$(".wcr .mainHeader, .wcr .footer").addClass('forcehide');

			WFEditor.prototype._enterFullScreen.call(this);

			// update toolbar
			if (this._toolbarModel) {
				this._toolbarModel.setButtonIcon(WebCenterWFEditorConstants.WorkflowFullScreenToolbarButton, "icon-WCR_NormalScreen");
			}
		};

		/**
		 * Exit Full Screen Mode (handle resizing)
		 * @override
		 */
		WFEditorWCR.prototype._exitFullScreen = function () {
			// recover menu
			$(".wcr .mainHeader, .wcr .footer").removeClass('forcehide');

			WFEditor.prototype._exitFullScreen.call(this);

			// update toolbar
			if (this._toolbarModel) {
				this._toolbarModel.setButtonIcon(WebCenterWFEditorConstants.WorkflowFullScreenToolbarButton, "icon-WCR_FullScreen");
			}
		};

		/**
		 * Get the current task type id
		 * @returns {String}
		 */
		WFEditorWCR.prototype.getTaskTypeID = function () {
			return this._taskTypeId;
		};

		/**
		 * Called before unloading of the page
		 */
		WFEditorWCR.prototype._onBeforeUnload = function () {
			if (this._workflow && this._workflow.getPrivilege(ModelConstants.Privileges.canSave) && this._undoRedo.isWorkflowModified()) {
				return i18n.t("workflowSaveWarning");
			}
		};

		/**
		 * Highlight active nodes or remove highlight
		 * @param {Boolean} [force] - can force the editor in or out of highlight mode without toggling
		 */
		WFEditorWCR.prototype._toggleHighlightNodes = function (force) {
			if (force === true || (force !== false && this._scene.getSceneModel().isInHighlightMode())) {
				this._scene.getSceneModel().setHighlightMode(false);
				this._toolbarModel.setButtonTooltip(WebCenterWFEditorConstants.WorkflowWFModesToolbar, i18n.t("workflowToggleHighlightTooltipOn"));
				if (this._highlightIcon && this._highlightIcon.hasClass("icon-default")) {
					this._highlightIcon.removeClass("icon-default");
					this._highlightIcon.addClass("icon-grey");
				}
			} else {
				this._scene.getSceneModel().setHighlightMode(true);
				this._toolbarModel.setButtonTooltip(WebCenterWFEditorConstants.WorkflowWFModesToolbar, i18n.t("workflowToggleHighlightTooltipOff"));
				if (this._highlightIcon && this._highlightIcon.hasClass("icon-grey")) {
					this._highlightIcon.removeClass("icon-grey");
					this._highlightIcon.addClass("icon-default");
				}
			}
		};

		/**
		 * @override
		 */
		WFEditorWCR.prototype._handleFullScreenEvent = function () {
			WFEditor.prototype._handleFullScreenEvent.call(this);
			this._toolbarModel.setButtonTooltip(WebCenterWFEditorConstants.WorkflowFullScreenToolbarButton, !this._isInFullScreen ? i18n.t("workflowToggleFullScreenTooltipOn") : i18n.t("workflowToggleFullScreenTooltipOff"));

		};


		/**
		 * check if the WFEditor can be closed 
		 * @return {Boolean}
		 */
		WFEditorWCR.prototype.canClose = function () {
			if (this._workflow && this._workflow.getPrivilege(ModelConstants.Privileges.canSave) && this._undoRedo && this._undoRedo.isWorkflowModified()) {
				return window.confirm(i18n.t("workflowSaveWarning"));
			}
			return true;
		};

		return WFEditorWCR;
	});

